// Copyright 2024 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// ** This file is automatically generated by gapic-generator-typescript. **
// ** https://github.com/googleapis/gapic-generator-typescript **
// ** All changes to this file may be overwritten. **

/* global window */
import type * as gax from 'google-gax';
import type {
  Callback,
  CallOptions,
  Descriptors,
  ClientOptions,
  GrpcClientOptions,
  LROperation,
  PaginationCallback,
  GaxCall,
} from 'google-gax';
import {Transform} from 'stream';
import * as protos from '../../protos/protos';
import jsonProtos = require('../../protos/protos.json');

/**
 * Client JSON configuration object, loaded from
 * `src/v2/bigtable_instance_admin_client_config.json`.
 * This file defines retry strategy and timeouts for all API methods in this library.
 */
import * as gapicConfig from './bigtable_instance_admin_client_config.json';
const version = require('../../../package.json').version;

/**
 *  Service for creating, configuring, and deleting Cloud Bigtable Instances and
 *  Clusters. Provides access to the Instance and Cluster schemas only, not the
 *  tables' metadata or data stored in those tables.
 * @class
 * @memberof v2
 */
export class BigtableInstanceAdminClient {
  private _terminated = false;
  private _opts: ClientOptions;
  private _providedCustomServicePath: boolean;
  private _gaxModule: typeof gax | typeof gax.fallback;
  private _gaxGrpc: gax.GrpcClient | gax.fallback.GrpcClient;
  private _protos: {};
  private _defaults: {[method: string]: gax.CallSettings};
  private _universeDomain: string;
  private _servicePath: string;
  auth: gax.GoogleAuth;
  descriptors: Descriptors = {
    page: {},
    stream: {},
    longrunning: {},
    batching: {},
  };
  warn: (code: string, message: string, warnType?: string) => void;
  innerApiCalls: {[name: string]: Function};
  pathTemplates: {[name: string]: gax.PathTemplate};
  operationsClient: gax.OperationsClient;
  bigtableInstanceAdminStub?: Promise<{[name: string]: Function}>;

  /**
   * Construct an instance of BigtableInstanceAdminClient.
   *
   * @param {object} [options] - The configuration object.
   * The options accepted by the constructor are described in detail
   * in [this document](https://github.com/googleapis/gax-nodejs/blob/main/client-libraries.md#creating-the-client-instance).
   * The common options are:
   * @param {object} [options.credentials] - Credentials object.
   * @param {string} [options.credentials.client_email]
   * @param {string} [options.credentials.private_key]
   * @param {string} [options.email] - Account email address. Required when
   *     using a .pem or .p12 keyFilename.
   * @param {string} [options.keyFilename] - Full path to the a .json, .pem, or
   *     .p12 key downloaded from the Google Developers Console. If you provide
   *     a path to a JSON file, the projectId option below is not necessary.
   *     NOTE: .pem and .p12 require you to specify options.email as well.
   * @param {number} [options.port] - The port on which to connect to
   *     the remote host.
   * @param {string} [options.projectId] - The project ID from the Google
   *     Developer's Console, e.g. 'grape-spaceship-123'. We will also check
   *     the environment variable GCLOUD_PROJECT for your project ID. If your
   *     app is running in an environment which supports
   *     {@link https://developers.google.com/identity/protocols/application-default-credentials Application Default Credentials},
   *     your project ID will be detected automatically.
   * @param {string} [options.apiEndpoint] - The domain name of the
   *     API remote host.
   * @param {gax.ClientConfig} [options.clientConfig] - Client configuration override.
   *     Follows the structure of {@link gapicConfig}.
   * @param {boolean} [options.fallback] - Use HTTP/1.1 REST mode.
   *     For more information, please check the
   *     {@link https://github.com/googleapis/gax-nodejs/blob/main/client-libraries.md#http11-rest-api-mode documentation}.
   * @param {gax} [gaxInstance]: loaded instance of `google-gax`. Useful if you
   *     need to avoid loading the default gRPC version and want to use the fallback
   *     HTTP implementation. Load only fallback version and pass it to the constructor:
   *     ```
   *     const gax = require('google-gax/build/src/fallback'); // avoids loading google-gax with gRPC
   *     const client = new BigtableInstanceAdminClient({fallback: true}, gax);
   *     ```
   */
  constructor(
    opts?: ClientOptions,
    gaxInstance?: typeof gax | typeof gax.fallback
  ) {
    // Ensure that options include all the required fields.
    const staticMembers = this
      .constructor as typeof BigtableInstanceAdminClient;
    if (
      opts?.universe_domain &&
      opts?.universeDomain &&
      opts?.universe_domain !== opts?.universeDomain
    ) {
      throw new Error(
        'Please set either universe_domain or universeDomain, but not both.'
      );
    }
    const universeDomainEnvVar =
      typeof process === 'object' && typeof process.env === 'object'
        ? process.env['GOOGLE_CLOUD_UNIVERSE_DOMAIN']
        : undefined;
    this._universeDomain =
      opts?.universeDomain ??
      opts?.universe_domain ??
      universeDomainEnvVar ??
      'googleapis.com';
    this._servicePath = 'bigtableadmin.' + this._universeDomain;
    const servicePath =
      opts?.servicePath || opts?.apiEndpoint || this._servicePath;
    this._providedCustomServicePath = !!(
      opts?.servicePath || opts?.apiEndpoint
    );
    const port = opts?.port || staticMembers.port;
    const clientConfig = opts?.clientConfig ?? {};
    const fallback =
      opts?.fallback ??
      (typeof window !== 'undefined' && typeof window?.fetch === 'function');
    opts = Object.assign({servicePath, port, clientConfig, fallback}, opts);

    // Request numeric enum values if REST transport is used.
    opts.numericEnums = true;

    // If scopes are unset in options and we're connecting to a non-default endpoint, set scopes just in case.
    if (servicePath !== this._servicePath && !('scopes' in opts)) {
      opts['scopes'] = staticMembers.scopes;
    }

    // Load google-gax module synchronously if needed
    if (!gaxInstance) {
      gaxInstance = require('google-gax') as typeof gax;
    }

    // Choose either gRPC or proto-over-HTTP implementation of google-gax.
    this._gaxModule = opts.fallback ? gaxInstance.fallback : gaxInstance;

    // Create a `gaxGrpc` object, with any grpc-specific options sent to the client.
    this._gaxGrpc = new this._gaxModule.GrpcClient(opts);

    // Save options to use in initialize() method.
    this._opts = opts;

    // Save the auth object to the client, for use by other methods.
    this.auth = this._gaxGrpc.auth as gax.GoogleAuth;

    // Set useJWTAccessWithScope on the auth object.
    this.auth.useJWTAccessWithScope = true;

    // Set defaultServicePath on the auth object.
    this.auth.defaultServicePath = this._servicePath;

    // Set the default scopes in auth client if needed.
    if (servicePath === this._servicePath) {
      this.auth.defaultScopes = staticMembers.scopes;
    }

    // Determine the client header string.
    const clientHeader = [`gax/${this._gaxModule.version}`, `gapic/${version}`];
    if (typeof process === 'object' && 'versions' in process) {
      clientHeader.push(`gl-node/${process.versions.node}`);
    } else {
      clientHeader.push(`gl-web/${this._gaxModule.version}`);
    }
    if (!opts.fallback) {
      clientHeader.push(`grpc/${this._gaxGrpc.grpcVersion}`);
    } else {
      clientHeader.push(`rest/${this._gaxGrpc.grpcVersion}`);
    }
    if (opts.libName && opts.libVersion) {
      clientHeader.push(`${opts.libName}/${opts.libVersion}`);
    }
    // Load the applicable protos.
    this._protos = this._gaxGrpc.loadProtoJSON(jsonProtos);

    // This API contains "path templates"; forward-slash-separated
    // identifiers to uniquely identify resources within the API.
    // Create useful helper objects for these.
    this.pathTemplates = {
      appProfilePathTemplate: new this._gaxModule.PathTemplate(
        'projects/{project}/instances/{instance}/appProfiles/{app_profile}'
      ),
      authorizedViewPathTemplate: new this._gaxModule.PathTemplate(
        'projects/{project}/instances/{instance}/tables/{table}/authorizedViews/{authorized_view}'
      ),
      backupPathTemplate: new this._gaxModule.PathTemplate(
        'projects/{project}/instances/{instance}/clusters/{cluster}/backups/{backup}'
      ),
      clusterPathTemplate: new this._gaxModule.PathTemplate(
        'projects/{project}/instances/{instance}/clusters/{cluster}'
      ),
      hotTabletPathTemplate: new this._gaxModule.PathTemplate(
        'projects/{project}/instances/{instance}/clusters/{cluster}/hotTablets/{hot_tablet}'
      ),
      instancePathTemplate: new this._gaxModule.PathTemplate(
        'projects/{project}/instances/{instance}'
      ),
      projectPathTemplate: new this._gaxModule.PathTemplate(
        'projects/{project}'
      ),
      snapshotPathTemplate: new this._gaxModule.PathTemplate(
        'projects/{project}/instances/{instance}/clusters/{cluster}/snapshots/{snapshot}'
      ),
      tablePathTemplate: new this._gaxModule.PathTemplate(
        'projects/{project}/instances/{instance}/tables/{table}'
      ),
    };

    // Some of the methods on this service return "paged" results,
    // (e.g. 50 results at a time, with tokens to get subsequent
    // pages). Denote the keys used for pagination and results.
    this.descriptors.page = {
      listAppProfiles: new this._gaxModule.PageDescriptor(
        'pageToken',
        'nextPageToken',
        'appProfiles'
      ),
      listHotTablets: new this._gaxModule.PageDescriptor(
        'pageToken',
        'nextPageToken',
        'hotTablets'
      ),
    };

    const protoFilesRoot = this._gaxModule.protobuf.Root.fromJSON(jsonProtos);
    // This API contains "long-running operations", which return a
    // an Operation object that allows for tracking of the operation,
    // rather than holding a request open.
    const lroOptions: GrpcClientOptions = {
      auth: this.auth,
      grpc: 'grpc' in this._gaxGrpc ? this._gaxGrpc.grpc : undefined,
    };
    if (opts.fallback) {
      lroOptions.protoJson = protoFilesRoot;
      lroOptions.httpRules = [
        {
          selector: 'google.longrunning.Operations.CancelOperation',
          post: '/v2/{name=operations/**}:cancel',
        },
        {
          selector: 'google.longrunning.Operations.DeleteOperation',
          delete: '/v2/{name=operations/**}',
        },
        {
          selector: 'google.longrunning.Operations.GetOperation',
          get: '/v2/{name=operations/**}',
        },
        {
          selector: 'google.longrunning.Operations.ListOperations',
          get: '/v2/{name=operations/projects/**}/operations',
        },
      ];
    }
    this.operationsClient = this._gaxModule
      .lro(lroOptions)
      .operationsClient(opts);
    const createInstanceResponse = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.Instance'
    ) as gax.protobuf.Type;
    const createInstanceMetadata = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.CreateInstanceMetadata'
    ) as gax.protobuf.Type;
    const partialUpdateInstanceResponse = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.Instance'
    ) as gax.protobuf.Type;
    const partialUpdateInstanceMetadata = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.UpdateInstanceMetadata'
    ) as gax.protobuf.Type;
    const createClusterResponse = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.Cluster'
    ) as gax.protobuf.Type;
    const createClusterMetadata = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.CreateClusterMetadata'
    ) as gax.protobuf.Type;
    const updateClusterResponse = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.Cluster'
    ) as gax.protobuf.Type;
    const updateClusterMetadata = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.UpdateClusterMetadata'
    ) as gax.protobuf.Type;
    const partialUpdateClusterResponse = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.Cluster'
    ) as gax.protobuf.Type;
    const partialUpdateClusterMetadata = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.PartialUpdateClusterMetadata'
    ) as gax.protobuf.Type;
    const updateAppProfileResponse = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.AppProfile'
    ) as gax.protobuf.Type;
    const updateAppProfileMetadata = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.UpdateAppProfileMetadata'
    ) as gax.protobuf.Type;

    this.descriptors.longrunning = {
      createInstance: new this._gaxModule.LongrunningDescriptor(
        this.operationsClient,
        createInstanceResponse.decode.bind(createInstanceResponse),
        createInstanceMetadata.decode.bind(createInstanceMetadata)
      ),
      partialUpdateInstance: new this._gaxModule.LongrunningDescriptor(
        this.operationsClient,
        partialUpdateInstanceResponse.decode.bind(
          partialUpdateInstanceResponse
        ),
        partialUpdateInstanceMetadata.decode.bind(partialUpdateInstanceMetadata)
      ),
      createCluster: new this._gaxModule.LongrunningDescriptor(
        this.operationsClient,
        createClusterResponse.decode.bind(createClusterResponse),
        createClusterMetadata.decode.bind(createClusterMetadata)
      ),
      updateCluster: new this._gaxModule.LongrunningDescriptor(
        this.operationsClient,
        updateClusterResponse.decode.bind(updateClusterResponse),
        updateClusterMetadata.decode.bind(updateClusterMetadata)
      ),
      partialUpdateCluster: new this._gaxModule.LongrunningDescriptor(
        this.operationsClient,
        partialUpdateClusterResponse.decode.bind(partialUpdateClusterResponse),
        partialUpdateClusterMetadata.decode.bind(partialUpdateClusterMetadata)
      ),
      updateAppProfile: new this._gaxModule.LongrunningDescriptor(
        this.operationsClient,
        updateAppProfileResponse.decode.bind(updateAppProfileResponse),
        updateAppProfileMetadata.decode.bind(updateAppProfileMetadata)
      ),
    };

    // Put together the default options sent with requests.
    this._defaults = this._gaxGrpc.constructSettings(
      'google.bigtable.admin.v2.BigtableInstanceAdmin',
      gapicConfig as gax.ClientConfig,
      opts.clientConfig || {},
      {'x-goog-api-client': clientHeader.join(' ')}
    );

    // Set up a dictionary of "inner API calls"; the core implementation
    // of calling the API is handled in `google-gax`, with this code
    // merely providing the destination and request information.
    this.innerApiCalls = {};

    // Add a warn function to the client constructor so it can be easily tested.
    this.warn = this._gaxModule.warn;
  }

  /**
   * Initialize the client.
   * Performs asynchronous operations (such as authentication) and prepares the client.
   * This function will be called automatically when any class method is called for the
   * first time, but if you need to initialize it before calling an actual method,
   * feel free to call initialize() directly.
   *
   * You can await on this method if you want to make sure the client is initialized.
   *
   * @returns {Promise} A promise that resolves to an authenticated service stub.
   */
  initialize() {
    // If the client stub promise is already initialized, return immediately.
    if (this.bigtableInstanceAdminStub) {
      return this.bigtableInstanceAdminStub;
    }

    // Put together the "service stub" for
    // google.bigtable.admin.v2.BigtableInstanceAdmin.
    this.bigtableInstanceAdminStub = this._gaxGrpc.createStub(
      this._opts.fallback
        ? (this._protos as protobuf.Root).lookupService(
            'google.bigtable.admin.v2.BigtableInstanceAdmin'
          )
        : // eslint-disable-next-line @typescript-eslint/no-explicit-any
          (this._protos as any).google.bigtable.admin.v2.BigtableInstanceAdmin,
      this._opts,
      this._providedCustomServicePath
    ) as Promise<{[method: string]: Function}>;

    // Iterate over each of the methods that the service provides
    // and create an API call method for each.
    const bigtableInstanceAdminStubMethods = [
      'createInstance',
      'getInstance',
      'listInstances',
      'updateInstance',
      'partialUpdateInstance',
      'deleteInstance',
      'createCluster',
      'getCluster',
      'listClusters',
      'updateCluster',
      'partialUpdateCluster',
      'deleteCluster',
      'createAppProfile',
      'getAppProfile',
      'listAppProfiles',
      'updateAppProfile',
      'deleteAppProfile',
      'getIamPolicy',
      'setIamPolicy',
      'testIamPermissions',
      'listHotTablets',
    ];
    for (const methodName of bigtableInstanceAdminStubMethods) {
      const callPromise = this.bigtableInstanceAdminStub.then(
        stub =>
          (...args: Array<{}>) => {
            if (this._terminated) {
              return Promise.reject('The client has already been closed.');
            }
            const func = stub[methodName];
            return func.apply(stub, args);
          },
        (err: Error | null | undefined) => () => {
          throw err;
        }
      );

      const descriptor =
        this.descriptors.page[methodName] ||
        this.descriptors.longrunning[methodName] ||
        undefined;
      const apiCall = this._gaxModule.createApiCall(
        callPromise,
        this._defaults[methodName],
        descriptor,
        this._opts.fallback
      );

      this.innerApiCalls[methodName] = apiCall;
    }

    return this.bigtableInstanceAdminStub;
  }

  /**
   * The DNS address for this API service.
   * @deprecated Use the apiEndpoint method of the client instance.
   * @returns {string} The DNS address for this service.
   */
  static get servicePath() {
    if (
      typeof process === 'object' &&
      typeof process.emitWarning === 'function'
    ) {
      process.emitWarning(
        'Static servicePath is deprecated, please use the instance method instead.',
        'DeprecationWarning'
      );
    }
    return 'bigtableadmin.googleapis.com';
  }

  /**
   * The DNS address for this API service - same as servicePath.
   * @deprecated Use the apiEndpoint method of the client instance.
   * @returns {string} The DNS address for this service.
   */
  static get apiEndpoint() {
    if (
      typeof process === 'object' &&
      typeof process.emitWarning === 'function'
    ) {
      process.emitWarning(
        'Static apiEndpoint is deprecated, please use the instance method instead.',
        'DeprecationWarning'
      );
    }
    return 'bigtableadmin.googleapis.com';
  }

  /**
   * The DNS address for this API service.
   * @returns {string} The DNS address for this service.
   */
  get apiEndpoint() {
    return this._servicePath;
  }

  get universeDomain() {
    return this._universeDomain;
  }

  /**
   * The port for this API service.
   * @returns {number} The default port for this service.
   */
  static get port() {
    return 443;
  }

  /**
   * The scopes needed to make gRPC calls for every method defined
   * in this service.
   * @returns {string[]} List of default scopes.
   */
  static get scopes() {
    return [
      'https://www.googleapis.com/auth/bigtable.admin',
      'https://www.googleapis.com/auth/bigtable.admin.cluster',
      'https://www.googleapis.com/auth/bigtable.admin.instance',
      'https://www.googleapis.com/auth/cloud-bigtable.admin',
      'https://www.googleapis.com/auth/cloud-bigtable.admin.cluster',
      'https://www.googleapis.com/auth/cloud-platform',
      'https://www.googleapis.com/auth/cloud-platform.read-only',
    ];
  }

  getProjectId(): Promise<string>;
  getProjectId(callback: Callback<string, undefined, undefined>): void;
  /**
   * Return the project ID used by this class.
   * @returns {Promise} A promise that resolves to string containing the project ID.
   */
  getProjectId(
    callback?: Callback<string, undefined, undefined>
  ): Promise<string> | void {
    if (callback) {
      this.auth.getProjectId(callback);
      return;
    }
    return this.auth.getProjectId();
  }

  // -------------------
  // -- Service calls --
  // -------------------
  /**
   * Gets information about an instance.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.name
   *   Required. The unique name of the requested instance. Values are of the form
   *   `projects/{project}/instances/{instance}`.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.bigtable.admin.v2.Instance|Instance}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_instance_admin.get_instance.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableInstanceAdmin_GetInstance_async
   */
  getInstance(
    request?: protos.google.bigtable.admin.v2.IGetInstanceRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.bigtable.admin.v2.IInstance,
      protos.google.bigtable.admin.v2.IGetInstanceRequest | undefined,
      {} | undefined,
    ]
  >;
  getInstance(
    request: protos.google.bigtable.admin.v2.IGetInstanceRequest,
    options: CallOptions,
    callback: Callback<
      protos.google.bigtable.admin.v2.IInstance,
      protos.google.bigtable.admin.v2.IGetInstanceRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  getInstance(
    request: protos.google.bigtable.admin.v2.IGetInstanceRequest,
    callback: Callback<
      protos.google.bigtable.admin.v2.IInstance,
      protos.google.bigtable.admin.v2.IGetInstanceRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  getInstance(
    request?: protos.google.bigtable.admin.v2.IGetInstanceRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.bigtable.admin.v2.IInstance,
          | protos.google.bigtable.admin.v2.IGetInstanceRequest
          | null
          | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.bigtable.admin.v2.IInstance,
      protos.google.bigtable.admin.v2.IGetInstanceRequest | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.bigtable.admin.v2.IInstance,
      protos.google.bigtable.admin.v2.IGetInstanceRequest | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        name: request.name ?? '',
      });
    this.initialize();
    return this.innerApiCalls.getInstance(request, options, callback);
  }
  /**
   * Lists information about instances in a project.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.parent
   *   Required. The unique name of the project for which a list of instances is
   *   requested. Values are of the form `projects/{project}`.
   * @param {string} request.pageToken
   *   DEPRECATED: This field is unused and ignored.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.bigtable.admin.v2.ListInstancesResponse|ListInstancesResponse}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_instance_admin.list_instances.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableInstanceAdmin_ListInstances_async
   */
  listInstances(
    request?: protos.google.bigtable.admin.v2.IListInstancesRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.bigtable.admin.v2.IListInstancesResponse,
      protos.google.bigtable.admin.v2.IListInstancesRequest | undefined,
      {} | undefined,
    ]
  >;
  listInstances(
    request: protos.google.bigtable.admin.v2.IListInstancesRequest,
    options: CallOptions,
    callback: Callback<
      protos.google.bigtable.admin.v2.IListInstancesResponse,
      protos.google.bigtable.admin.v2.IListInstancesRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  listInstances(
    request: protos.google.bigtable.admin.v2.IListInstancesRequest,
    callback: Callback<
      protos.google.bigtable.admin.v2.IListInstancesResponse,
      protos.google.bigtable.admin.v2.IListInstancesRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  listInstances(
    request?: protos.google.bigtable.admin.v2.IListInstancesRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.bigtable.admin.v2.IListInstancesResponse,
          | protos.google.bigtable.admin.v2.IListInstancesRequest
          | null
          | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.bigtable.admin.v2.IListInstancesResponse,
      protos.google.bigtable.admin.v2.IListInstancesRequest | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.bigtable.admin.v2.IListInstancesResponse,
      protos.google.bigtable.admin.v2.IListInstancesRequest | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        parent: request.parent ?? '',
      });
    this.initialize();
    return this.innerApiCalls.listInstances(request, options, callback);
  }
  /**
   * Updates an instance within a project. This method updates only the display
   * name and type for an Instance. To update other Instance properties, such as
   * labels, use PartialUpdateInstance.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.name
   *   The unique name of the instance. Values are of the form
   *   `projects/{project}/instances/{@link protos.a-z0-9\\-|a-z}+[a-z0-9]`.
   * @param {string} request.displayName
   *   Required. The descriptive name for this instance as it appears in UIs.
   *   Can be changed at any time, but should be kept globally unique
   *   to avoid confusion.
   * @param {google.bigtable.admin.v2.Instance.State} request.state
   *   (`OutputOnly`)
   *   The current state of the instance.
   * @param {google.bigtable.admin.v2.Instance.Type} request.type
   *   The type of the instance. Defaults to `PRODUCTION`.
   * @param {number[]} request.labels
   *   Labels are a flexible and lightweight mechanism for organizing cloud
   *   resources into groups that reflect a customer's organizational needs and
   *   deployment strategies. They can be used to filter resources and aggregate
   *   metrics.
   *
   *   * Label keys must be between 1 and 63 characters long and must conform to
   *     the regular expression: `{@link protos.\p{Ll}\p{Lo}\p{N}_-|\p{Ll}\p{Lo}}{0,62}`.
   *   * Label values must be between 0 and 63 characters long and must conform to
   *     the regular expression: `[\p{Ll}\p{Lo}\p{N}_-]{0,63}`.
   *   * No more than 64 labels can be associated with a given resource.
   *   * Keys and values must both be under 128 bytes.
   * @param {google.protobuf.Timestamp} request.createTime
   *   Output only. A server-assigned timestamp representing when this Instance
   *   was created. For instances created before this field was added (August
   *   2021), this value is `seconds: 0, nanos: 1`.
   * @param {boolean} request.satisfiesPzs
   *   Output only. Reserved for future use.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.bigtable.admin.v2.Instance|Instance}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_instance_admin.update_instance.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableInstanceAdmin_UpdateInstance_async
   */
  updateInstance(
    request?: protos.google.bigtable.admin.v2.IInstance,
    options?: CallOptions
  ): Promise<
    [
      protos.google.bigtable.admin.v2.IInstance,
      protos.google.bigtable.admin.v2.IInstance | undefined,
      {} | undefined,
    ]
  >;
  updateInstance(
    request: protos.google.bigtable.admin.v2.IInstance,
    options: CallOptions,
    callback: Callback<
      protos.google.bigtable.admin.v2.IInstance,
      protos.google.bigtable.admin.v2.IInstance | null | undefined,
      {} | null | undefined
    >
  ): void;
  updateInstance(
    request: protos.google.bigtable.admin.v2.IInstance,
    callback: Callback<
      protos.google.bigtable.admin.v2.IInstance,
      protos.google.bigtable.admin.v2.IInstance | null | undefined,
      {} | null | undefined
    >
  ): void;
  updateInstance(
    request?: protos.google.bigtable.admin.v2.IInstance,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.bigtable.admin.v2.IInstance,
          protos.google.bigtable.admin.v2.IInstance | null | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.bigtable.admin.v2.IInstance,
      protos.google.bigtable.admin.v2.IInstance | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.bigtable.admin.v2.IInstance,
      protos.google.bigtable.admin.v2.IInstance | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        name: request.name ?? '',
      });
    this.initialize();
    return this.innerApiCalls.updateInstance(request, options, callback);
  }
  /**
   * Delete an instance from a project.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.name
   *   Required. The unique name of the instance to be deleted.
   *   Values are of the form `projects/{project}/instances/{instance}`.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.protobuf.Empty|Empty}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_instance_admin.delete_instance.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableInstanceAdmin_DeleteInstance_async
   */
  deleteInstance(
    request?: protos.google.bigtable.admin.v2.IDeleteInstanceRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDeleteInstanceRequest | undefined,
      {} | undefined,
    ]
  >;
  deleteInstance(
    request: protos.google.bigtable.admin.v2.IDeleteInstanceRequest,
    options: CallOptions,
    callback: Callback<
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDeleteInstanceRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  deleteInstance(
    request: protos.google.bigtable.admin.v2.IDeleteInstanceRequest,
    callback: Callback<
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDeleteInstanceRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  deleteInstance(
    request?: protos.google.bigtable.admin.v2.IDeleteInstanceRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.protobuf.IEmpty,
          | protos.google.bigtable.admin.v2.IDeleteInstanceRequest
          | null
          | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDeleteInstanceRequest | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDeleteInstanceRequest | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        name: request.name ?? '',
      });
    this.initialize();
    return this.innerApiCalls.deleteInstance(request, options, callback);
  }
  /**
   * Gets information about a cluster.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.name
   *   Required. The unique name of the requested cluster. Values are of the form
   *   `projects/{project}/instances/{instance}/clusters/{cluster}`.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.bigtable.admin.v2.Cluster|Cluster}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_instance_admin.get_cluster.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableInstanceAdmin_GetCluster_async
   */
  getCluster(
    request?: protos.google.bigtable.admin.v2.IGetClusterRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.bigtable.admin.v2.ICluster,
      protos.google.bigtable.admin.v2.IGetClusterRequest | undefined,
      {} | undefined,
    ]
  >;
  getCluster(
    request: protos.google.bigtable.admin.v2.IGetClusterRequest,
    options: CallOptions,
    callback: Callback<
      protos.google.bigtable.admin.v2.ICluster,
      protos.google.bigtable.admin.v2.IGetClusterRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  getCluster(
    request: protos.google.bigtable.admin.v2.IGetClusterRequest,
    callback: Callback<
      protos.google.bigtable.admin.v2.ICluster,
      protos.google.bigtable.admin.v2.IGetClusterRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  getCluster(
    request?: protos.google.bigtable.admin.v2.IGetClusterRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.bigtable.admin.v2.ICluster,
          protos.google.bigtable.admin.v2.IGetClusterRequest | null | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.bigtable.admin.v2.ICluster,
      protos.google.bigtable.admin.v2.IGetClusterRequest | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.bigtable.admin.v2.ICluster,
      protos.google.bigtable.admin.v2.IGetClusterRequest | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        name: request.name ?? '',
      });
    this.initialize();
    return this.innerApiCalls.getCluster(request, options, callback);
  }
  /**
   * Lists information about clusters in an instance.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.parent
   *   Required. The unique name of the instance for which a list of clusters is
   *   requested. Values are of the form
   *   `projects/{project}/instances/{instance}`. Use `{instance} = '-'` to list
   *   Clusters for all Instances in a project, e.g.,
   *   `projects/myproject/instances/-`.
   * @param {string} request.pageToken
   *   DEPRECATED: This field is unused and ignored.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.bigtable.admin.v2.ListClustersResponse|ListClustersResponse}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_instance_admin.list_clusters.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableInstanceAdmin_ListClusters_async
   */
  listClusters(
    request?: protos.google.bigtable.admin.v2.IListClustersRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.bigtable.admin.v2.IListClustersResponse,
      protos.google.bigtable.admin.v2.IListClustersRequest | undefined,
      {} | undefined,
    ]
  >;
  listClusters(
    request: protos.google.bigtable.admin.v2.IListClustersRequest,
    options: CallOptions,
    callback: Callback<
      protos.google.bigtable.admin.v2.IListClustersResponse,
      protos.google.bigtable.admin.v2.IListClustersRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  listClusters(
    request: protos.google.bigtable.admin.v2.IListClustersRequest,
    callback: Callback<
      protos.google.bigtable.admin.v2.IListClustersResponse,
      protos.google.bigtable.admin.v2.IListClustersRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  listClusters(
    request?: protos.google.bigtable.admin.v2.IListClustersRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.bigtable.admin.v2.IListClustersResponse,
          | protos.google.bigtable.admin.v2.IListClustersRequest
          | null
          | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.bigtable.admin.v2.IListClustersResponse,
      protos.google.bigtable.admin.v2.IListClustersRequest | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.bigtable.admin.v2.IListClustersResponse,
      protos.google.bigtable.admin.v2.IListClustersRequest | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        parent: request.parent ?? '',
      });
    this.initialize();
    return this.innerApiCalls.listClusters(request, options, callback);
  }
  /**
   * Deletes a cluster from an instance.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.name
   *   Required. The unique name of the cluster to be deleted. Values are of the
   *   form `projects/{project}/instances/{instance}/clusters/{cluster}`.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.protobuf.Empty|Empty}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_instance_admin.delete_cluster.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableInstanceAdmin_DeleteCluster_async
   */
  deleteCluster(
    request?: protos.google.bigtable.admin.v2.IDeleteClusterRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDeleteClusterRequest | undefined,
      {} | undefined,
    ]
  >;
  deleteCluster(
    request: protos.google.bigtable.admin.v2.IDeleteClusterRequest,
    options: CallOptions,
    callback: Callback<
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDeleteClusterRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  deleteCluster(
    request: protos.google.bigtable.admin.v2.IDeleteClusterRequest,
    callback: Callback<
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDeleteClusterRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  deleteCluster(
    request?: protos.google.bigtable.admin.v2.IDeleteClusterRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.protobuf.IEmpty,
          | protos.google.bigtable.admin.v2.IDeleteClusterRequest
          | null
          | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDeleteClusterRequest | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDeleteClusterRequest | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        name: request.name ?? '',
      });
    this.initialize();
    return this.innerApiCalls.deleteCluster(request, options, callback);
  }
  /**
   * Creates an app profile within an instance.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.parent
   *   Required. The unique name of the instance in which to create the new app
   *   profile. Values are of the form `projects/{project}/instances/{instance}`.
   * @param {string} request.appProfileId
   *   Required. The ID to be used when referring to the new app profile within
   *   its instance, e.g., just `myprofile` rather than
   *   `projects/myproject/instances/myinstance/appProfiles/myprofile`.
   * @param {google.bigtable.admin.v2.AppProfile} request.appProfile
   *   Required. The app profile to be created.
   *   Fields marked `OutputOnly` will be ignored.
   * @param {boolean} request.ignoreWarnings
   *   If true, ignore safety checks when creating the app profile.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.bigtable.admin.v2.AppProfile|AppProfile}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_instance_admin.create_app_profile.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableInstanceAdmin_CreateAppProfile_async
   */
  createAppProfile(
    request?: protos.google.bigtable.admin.v2.ICreateAppProfileRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.bigtable.admin.v2.IAppProfile,
      protos.google.bigtable.admin.v2.ICreateAppProfileRequest | undefined,
      {} | undefined,
    ]
  >;
  createAppProfile(
    request: protos.google.bigtable.admin.v2.ICreateAppProfileRequest,
    options: CallOptions,
    callback: Callback<
      protos.google.bigtable.admin.v2.IAppProfile,
      | protos.google.bigtable.admin.v2.ICreateAppProfileRequest
      | null
      | undefined,
      {} | null | undefined
    >
  ): void;
  createAppProfile(
    request: protos.google.bigtable.admin.v2.ICreateAppProfileRequest,
    callback: Callback<
      protos.google.bigtable.admin.v2.IAppProfile,
      | protos.google.bigtable.admin.v2.ICreateAppProfileRequest
      | null
      | undefined,
      {} | null | undefined
    >
  ): void;
  createAppProfile(
    request?: protos.google.bigtable.admin.v2.ICreateAppProfileRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.bigtable.admin.v2.IAppProfile,
          | protos.google.bigtable.admin.v2.ICreateAppProfileRequest
          | null
          | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.bigtable.admin.v2.IAppProfile,
      | protos.google.bigtable.admin.v2.ICreateAppProfileRequest
      | null
      | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.bigtable.admin.v2.IAppProfile,
      protos.google.bigtable.admin.v2.ICreateAppProfileRequest | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        parent: request.parent ?? '',
      });
    this.initialize();
    return this.innerApiCalls.createAppProfile(request, options, callback);
  }
  /**
   * Gets information about an app profile.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.name
   *   Required. The unique name of the requested app profile. Values are of the
   *   form `projects/{project}/instances/{instance}/appProfiles/{app_profile}`.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.bigtable.admin.v2.AppProfile|AppProfile}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_instance_admin.get_app_profile.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableInstanceAdmin_GetAppProfile_async
   */
  getAppProfile(
    request?: protos.google.bigtable.admin.v2.IGetAppProfileRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.bigtable.admin.v2.IAppProfile,
      protos.google.bigtable.admin.v2.IGetAppProfileRequest | undefined,
      {} | undefined,
    ]
  >;
  getAppProfile(
    request: protos.google.bigtable.admin.v2.IGetAppProfileRequest,
    options: CallOptions,
    callback: Callback<
      protos.google.bigtable.admin.v2.IAppProfile,
      protos.google.bigtable.admin.v2.IGetAppProfileRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  getAppProfile(
    request: protos.google.bigtable.admin.v2.IGetAppProfileRequest,
    callback: Callback<
      protos.google.bigtable.admin.v2.IAppProfile,
      protos.google.bigtable.admin.v2.IGetAppProfileRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  getAppProfile(
    request?: protos.google.bigtable.admin.v2.IGetAppProfileRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.bigtable.admin.v2.IAppProfile,
          | protos.google.bigtable.admin.v2.IGetAppProfileRequest
          | null
          | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.bigtable.admin.v2.IAppProfile,
      protos.google.bigtable.admin.v2.IGetAppProfileRequest | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.bigtable.admin.v2.IAppProfile,
      protos.google.bigtable.admin.v2.IGetAppProfileRequest | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        name: request.name ?? '',
      });
    this.initialize();
    return this.innerApiCalls.getAppProfile(request, options, callback);
  }
  /**
   * Deletes an app profile from an instance.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.name
   *   Required. The unique name of the app profile to be deleted. Values are of
   *   the form
   *   `projects/{project}/instances/{instance}/appProfiles/{app_profile}`.
   * @param {boolean} request.ignoreWarnings
   *   Required. If true, ignore safety checks when deleting the app profile.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.protobuf.Empty|Empty}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_instance_admin.delete_app_profile.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableInstanceAdmin_DeleteAppProfile_async
   */
  deleteAppProfile(
    request?: protos.google.bigtable.admin.v2.IDeleteAppProfileRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDeleteAppProfileRequest | undefined,
      {} | undefined,
    ]
  >;
  deleteAppProfile(
    request: protos.google.bigtable.admin.v2.IDeleteAppProfileRequest,
    options: CallOptions,
    callback: Callback<
      protos.google.protobuf.IEmpty,
      | protos.google.bigtable.admin.v2.IDeleteAppProfileRequest
      | null
      | undefined,
      {} | null | undefined
    >
  ): void;
  deleteAppProfile(
    request: protos.google.bigtable.admin.v2.IDeleteAppProfileRequest,
    callback: Callback<
      protos.google.protobuf.IEmpty,
      | protos.google.bigtable.admin.v2.IDeleteAppProfileRequest
      | null
      | undefined,
      {} | null | undefined
    >
  ): void;
  deleteAppProfile(
    request?: protos.google.bigtable.admin.v2.IDeleteAppProfileRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.protobuf.IEmpty,
          | protos.google.bigtable.admin.v2.IDeleteAppProfileRequest
          | null
          | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.protobuf.IEmpty,
      | protos.google.bigtable.admin.v2.IDeleteAppProfileRequest
      | null
      | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDeleteAppProfileRequest | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        name: request.name ?? '',
      });
    this.initialize();
    return this.innerApiCalls.deleteAppProfile(request, options, callback);
  }
  /**
   * Gets the access control policy for an instance resource. Returns an empty
   * policy if an instance exists but does not have a policy set.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.resource
   *   REQUIRED: The resource for which the policy is being requested.
   *   See the operation documentation for the appropriate value for this field.
   * @param {google.iam.v1.GetPolicyOptions} request.options
   *   OPTIONAL: A `GetPolicyOptions` object for specifying options to
   *   `GetIamPolicy`.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.iam.v1.Policy|Policy}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_instance_admin.get_iam_policy.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableInstanceAdmin_GetIamPolicy_async
   */
  getIamPolicy(
    request?: protos.google.iam.v1.IGetIamPolicyRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.iam.v1.IPolicy,
      protos.google.iam.v1.IGetIamPolicyRequest | undefined,
      {} | undefined,
    ]
  >;
  getIamPolicy(
    request: protos.google.iam.v1.IGetIamPolicyRequest,
    options: CallOptions,
    callback: Callback<
      protos.google.iam.v1.IPolicy,
      protos.google.iam.v1.IGetIamPolicyRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  getIamPolicy(
    request: protos.google.iam.v1.IGetIamPolicyRequest,
    callback: Callback<
      protos.google.iam.v1.IPolicy,
      protos.google.iam.v1.IGetIamPolicyRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  getIamPolicy(
    request?: protos.google.iam.v1.IGetIamPolicyRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.iam.v1.IPolicy,
          protos.google.iam.v1.IGetIamPolicyRequest | null | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.iam.v1.IPolicy,
      protos.google.iam.v1.IGetIamPolicyRequest | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.iam.v1.IPolicy,
      protos.google.iam.v1.IGetIamPolicyRequest | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        resource: request.resource ?? '',
      });
    this.initialize();
    return this.innerApiCalls.getIamPolicy(request, options, callback);
  }
  /**
   * Sets the access control policy on an instance resource. Replaces any
   * existing policy.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.resource
   *   REQUIRED: The resource for which the policy is being specified.
   *   See the operation documentation for the appropriate value for this field.
   * @param {google.iam.v1.Policy} request.policy
   *   REQUIRED: The complete policy to be applied to the `resource`. The size of
   *   the policy is limited to a few 10s of KB. An empty policy is a
   *   valid policy but certain Cloud Platform services (such as Projects)
   *   might reject them.
   * @param {google.protobuf.FieldMask} request.updateMask
   *   OPTIONAL: A FieldMask specifying which fields of the policy to modify. Only
   *   the fields in the mask will be modified. If no mask is provided, the
   *   following default mask is used:
   *
   *   `paths: "bindings, etag"`
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.iam.v1.Policy|Policy}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_instance_admin.set_iam_policy.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableInstanceAdmin_SetIamPolicy_async
   */
  setIamPolicy(
    request?: protos.google.iam.v1.ISetIamPolicyRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.iam.v1.IPolicy,
      protos.google.iam.v1.ISetIamPolicyRequest | undefined,
      {} | undefined,
    ]
  >;
  setIamPolicy(
    request: protos.google.iam.v1.ISetIamPolicyRequest,
    options: CallOptions,
    callback: Callback<
      protos.google.iam.v1.IPolicy,
      protos.google.iam.v1.ISetIamPolicyRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  setIamPolicy(
    request: protos.google.iam.v1.ISetIamPolicyRequest,
    callback: Callback<
      protos.google.iam.v1.IPolicy,
      protos.google.iam.v1.ISetIamPolicyRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  setIamPolicy(
    request?: protos.google.iam.v1.ISetIamPolicyRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.iam.v1.IPolicy,
          protos.google.iam.v1.ISetIamPolicyRequest | null | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.iam.v1.IPolicy,
      protos.google.iam.v1.ISetIamPolicyRequest | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.iam.v1.IPolicy,
      protos.google.iam.v1.ISetIamPolicyRequest | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        resource: request.resource ?? '',
      });
    this.initialize();
    return this.innerApiCalls.setIamPolicy(request, options, callback);
  }
  /**
   * Returns permissions that the caller has on the specified instance resource.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.resource
   *   REQUIRED: The resource for which the policy detail is being requested.
   *   See the operation documentation for the appropriate value for this field.
   * @param {string[]} request.permissions
   *   The set of permissions to check for the `resource`. Permissions with
   *   wildcards (such as '*' or 'storage.*') are not allowed. For more
   *   information see
   *   [IAM Overview](https://cloud.google.com/iam/docs/overview#permissions).
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.iam.v1.TestIamPermissionsResponse|TestIamPermissionsResponse}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_instance_admin.test_iam_permissions.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableInstanceAdmin_TestIamPermissions_async
   */
  testIamPermissions(
    request?: protos.google.iam.v1.ITestIamPermissionsRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.iam.v1.ITestIamPermissionsResponse,
      protos.google.iam.v1.ITestIamPermissionsRequest | undefined,
      {} | undefined,
    ]
  >;
  testIamPermissions(
    request: protos.google.iam.v1.ITestIamPermissionsRequest,
    options: CallOptions,
    callback: Callback<
      protos.google.iam.v1.ITestIamPermissionsResponse,
      protos.google.iam.v1.ITestIamPermissionsRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  testIamPermissions(
    request: protos.google.iam.v1.ITestIamPermissionsRequest,
    callback: Callback<
      protos.google.iam.v1.ITestIamPermissionsResponse,
      protos.google.iam.v1.ITestIamPermissionsRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  testIamPermissions(
    request?: protos.google.iam.v1.ITestIamPermissionsRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.iam.v1.ITestIamPermissionsResponse,
          protos.google.iam.v1.ITestIamPermissionsRequest | null | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.iam.v1.ITestIamPermissionsResponse,
      protos.google.iam.v1.ITestIamPermissionsRequest | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.iam.v1.ITestIamPermissionsResponse,
      protos.google.iam.v1.ITestIamPermissionsRequest | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        resource: request.resource ?? '',
      });
    this.initialize();
    return this.innerApiCalls.testIamPermissions(request, options, callback);
  }

  /**
   * Create an instance within a project.
   *
   * Note that exactly one of Cluster.serve_nodes and
   * Cluster.cluster_config.cluster_autoscaling_config can be set. If
   * serve_nodes is set to non-zero, then the cluster is manually scaled. If
   * cluster_config.cluster_autoscaling_config is non-empty, then autoscaling is
   * enabled.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.parent
   *   Required. The unique name of the project in which to create the new
   *   instance. Values are of the form `projects/{project}`.
   * @param {string} request.instanceId
   *   Required. The ID to be used when referring to the new instance within its
   *   project, e.g., just `myinstance` rather than
   *   `projects/myproject/instances/myinstance`.
   * @param {google.bigtable.admin.v2.Instance} request.instance
   *   Required. The instance to create.
   *   Fields marked `OutputOnly` must be left blank.
   * @param {number[]} request.clusters
   *   Required. The clusters to be created within the instance, mapped by desired
   *   cluster ID, e.g., just `mycluster` rather than
   *   `projects/myproject/instances/myinstance/clusters/mycluster`.
   *   Fields marked `OutputOnly` must be left blank.
   *   Currently, at most four clusters can be specified.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing
   *   a long running operation. Its `promise()` method returns a promise
   *   you can `await` for.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_instance_admin.create_instance.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableInstanceAdmin_CreateInstance_async
   */
  createInstance(
    request?: protos.google.bigtable.admin.v2.ICreateInstanceRequest,
    options?: CallOptions
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.IInstance,
        protos.google.bigtable.admin.v2.ICreateInstanceMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  >;
  createInstance(
    request: protos.google.bigtable.admin.v2.ICreateInstanceRequest,
    options: CallOptions,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.IInstance,
        protos.google.bigtable.admin.v2.ICreateInstanceMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  createInstance(
    request: protos.google.bigtable.admin.v2.ICreateInstanceRequest,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.IInstance,
        protos.google.bigtable.admin.v2.ICreateInstanceMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  createInstance(
    request?: protos.google.bigtable.admin.v2.ICreateInstanceRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          LROperation<
            protos.google.bigtable.admin.v2.IInstance,
            protos.google.bigtable.admin.v2.ICreateInstanceMetadata
          >,
          protos.google.longrunning.IOperation | null | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.IInstance,
        protos.google.bigtable.admin.v2.ICreateInstanceMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.IInstance,
        protos.google.bigtable.admin.v2.ICreateInstanceMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        parent: request.parent ?? '',
      });
    this.initialize();
    return this.innerApiCalls.createInstance(request, options, callback);
  }
  /**
   * Check the status of the long running operation returned by `createInstance()`.
   * @param {String} name
   *   The operation name that will be passed.
   * @returns {Promise} - The promise which resolves to an object.
   *   The decoded operation object has result and metadata field to get information from.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_instance_admin.create_instance.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableInstanceAdmin_CreateInstance_async
   */
  async checkCreateInstanceProgress(
    name: string
  ): Promise<
    LROperation<
      protos.google.bigtable.admin.v2.Instance,
      protos.google.bigtable.admin.v2.CreateInstanceMetadata
    >
  > {
    const request =
      new this._gaxModule.operationsProtos.google.longrunning.GetOperationRequest(
        {name}
      );
    const [operation] = await this.operationsClient.getOperation(request);
    const decodeOperation = new this._gaxModule.Operation(
      operation,
      this.descriptors.longrunning.createInstance,
      this._gaxModule.createDefaultBackoffSettings()
    );
    return decodeOperation as LROperation<
      protos.google.bigtable.admin.v2.Instance,
      protos.google.bigtable.admin.v2.CreateInstanceMetadata
    >;
  }
  /**
   * Partially updates an instance within a project. This method can modify all
   * fields of an Instance and is the preferred way to update an Instance.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {google.bigtable.admin.v2.Instance} request.instance
   *   Required. The Instance which will (partially) replace the current value.
   * @param {google.protobuf.FieldMask} request.updateMask
   *   Required. The subset of Instance fields which should be replaced.
   *   Must be explicitly set.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing
   *   a long running operation. Its `promise()` method returns a promise
   *   you can `await` for.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_instance_admin.partial_update_instance.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableInstanceAdmin_PartialUpdateInstance_async
   */
  partialUpdateInstance(
    request?: protos.google.bigtable.admin.v2.IPartialUpdateInstanceRequest,
    options?: CallOptions
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.IInstance,
        protos.google.bigtable.admin.v2.IUpdateInstanceMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  >;
  partialUpdateInstance(
    request: protos.google.bigtable.admin.v2.IPartialUpdateInstanceRequest,
    options: CallOptions,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.IInstance,
        protos.google.bigtable.admin.v2.IUpdateInstanceMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  partialUpdateInstance(
    request: protos.google.bigtable.admin.v2.IPartialUpdateInstanceRequest,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.IInstance,
        protos.google.bigtable.admin.v2.IUpdateInstanceMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  partialUpdateInstance(
    request?: protos.google.bigtable.admin.v2.IPartialUpdateInstanceRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          LROperation<
            protos.google.bigtable.admin.v2.IInstance,
            protos.google.bigtable.admin.v2.IUpdateInstanceMetadata
          >,
          protos.google.longrunning.IOperation | null | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.IInstance,
        protos.google.bigtable.admin.v2.IUpdateInstanceMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.IInstance,
        protos.google.bigtable.admin.v2.IUpdateInstanceMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        'instance.name': request.instance!.name ?? '',
      });
    this.initialize();
    return this.innerApiCalls.partialUpdateInstance(request, options, callback);
  }
  /**
   * Check the status of the long running operation returned by `partialUpdateInstance()`.
   * @param {String} name
   *   The operation name that will be passed.
   * @returns {Promise} - The promise which resolves to an object.
   *   The decoded operation object has result and metadata field to get information from.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_instance_admin.partial_update_instance.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableInstanceAdmin_PartialUpdateInstance_async
   */
  async checkPartialUpdateInstanceProgress(
    name: string
  ): Promise<
    LROperation<
      protos.google.bigtable.admin.v2.Instance,
      protos.google.bigtable.admin.v2.UpdateInstanceMetadata
    >
  > {
    const request =
      new this._gaxModule.operationsProtos.google.longrunning.GetOperationRequest(
        {name}
      );
    const [operation] = await this.operationsClient.getOperation(request);
    const decodeOperation = new this._gaxModule.Operation(
      operation,
      this.descriptors.longrunning.partialUpdateInstance,
      this._gaxModule.createDefaultBackoffSettings()
    );
    return decodeOperation as LROperation<
      protos.google.bigtable.admin.v2.Instance,
      protos.google.bigtable.admin.v2.UpdateInstanceMetadata
    >;
  }
  /**
   * Creates a cluster within an instance.
   *
   * Note that exactly one of Cluster.serve_nodes and
   * Cluster.cluster_config.cluster_autoscaling_config can be set. If
   * serve_nodes is set to non-zero, then the cluster is manually scaled. If
   * cluster_config.cluster_autoscaling_config is non-empty, then autoscaling is
   * enabled.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.parent
   *   Required. The unique name of the instance in which to create the new
   *   cluster. Values are of the form `projects/{project}/instances/{instance}`.
   * @param {string} request.clusterId
   *   Required. The ID to be used when referring to the new cluster within its
   *   instance, e.g., just `mycluster` rather than
   *   `projects/myproject/instances/myinstance/clusters/mycluster`.
   * @param {google.bigtable.admin.v2.Cluster} request.cluster
   *   Required. The cluster to be created.
   *   Fields marked `OutputOnly` must be left blank.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing
   *   a long running operation. Its `promise()` method returns a promise
   *   you can `await` for.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_instance_admin.create_cluster.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableInstanceAdmin_CreateCluster_async
   */
  createCluster(
    request?: protos.google.bigtable.admin.v2.ICreateClusterRequest,
    options?: CallOptions
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.ICluster,
        protos.google.bigtable.admin.v2.ICreateClusterMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  >;
  createCluster(
    request: protos.google.bigtable.admin.v2.ICreateClusterRequest,
    options: CallOptions,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.ICluster,
        protos.google.bigtable.admin.v2.ICreateClusterMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  createCluster(
    request: protos.google.bigtable.admin.v2.ICreateClusterRequest,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.ICluster,
        protos.google.bigtable.admin.v2.ICreateClusterMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  createCluster(
    request?: protos.google.bigtable.admin.v2.ICreateClusterRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          LROperation<
            protos.google.bigtable.admin.v2.ICluster,
            protos.google.bigtable.admin.v2.ICreateClusterMetadata
          >,
          protos.google.longrunning.IOperation | null | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.ICluster,
        protos.google.bigtable.admin.v2.ICreateClusterMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.ICluster,
        protos.google.bigtable.admin.v2.ICreateClusterMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        parent: request.parent ?? '',
      });
    this.initialize();
    return this.innerApiCalls.createCluster(request, options, callback);
  }
  /**
   * Check the status of the long running operation returned by `createCluster()`.
   * @param {String} name
   *   The operation name that will be passed.
   * @returns {Promise} - The promise which resolves to an object.
   *   The decoded operation object has result and metadata field to get information from.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_instance_admin.create_cluster.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableInstanceAdmin_CreateCluster_async
   */
  async checkCreateClusterProgress(
    name: string
  ): Promise<
    LROperation<
      protos.google.bigtable.admin.v2.Cluster,
      protos.google.bigtable.admin.v2.CreateClusterMetadata
    >
  > {
    const request =
      new this._gaxModule.operationsProtos.google.longrunning.GetOperationRequest(
        {name}
      );
    const [operation] = await this.operationsClient.getOperation(request);
    const decodeOperation = new this._gaxModule.Operation(
      operation,
      this.descriptors.longrunning.createCluster,
      this._gaxModule.createDefaultBackoffSettings()
    );
    return decodeOperation as LROperation<
      protos.google.bigtable.admin.v2.Cluster,
      protos.google.bigtable.admin.v2.CreateClusterMetadata
    >;
  }
  /**
   * Updates a cluster within an instance.
   *
   * Note that UpdateCluster does not support updating
   * cluster_config.cluster_autoscaling_config. In order to update it, you
   * must use PartialUpdateCluster.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.name
   *   The unique name of the cluster. Values are of the form
   *   `projects/{project}/instances/{instance}/clusters/{@link protos.-a-z0-9|a-z}*`.
   * @param {string} request.location
   *   Immutable. The location where this cluster's nodes and storage reside. For
   *   best performance, clients should be located as close as possible to this
   *   cluster. Currently only zones are supported, so values should be of the
   *   form `projects/{project}/locations/{zone}`.
   * @param {google.bigtable.admin.v2.Cluster.State} request.state
   *   Output only. The current state of the cluster.
   * @param {number} request.serveNodes
   *   The number of nodes allocated to this cluster. More nodes enable higher
   *   throughput and more consistent performance.
   * @param {google.bigtable.admin.v2.Cluster.ClusterConfig} request.clusterConfig
   *   Configuration for this cluster.
   * @param {google.bigtable.admin.v2.StorageType} request.defaultStorageType
   *   Immutable. The type of storage used by this cluster to serve its
   *   parent instance's tables, unless explicitly overridden.
   * @param {google.bigtable.admin.v2.Cluster.EncryptionConfig} request.encryptionConfig
   *   Immutable. The encryption configuration for CMEK-protected clusters.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing
   *   a long running operation. Its `promise()` method returns a promise
   *   you can `await` for.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_instance_admin.update_cluster.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableInstanceAdmin_UpdateCluster_async
   */
  updateCluster(
    request?: protos.google.bigtable.admin.v2.ICluster,
    options?: CallOptions
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.ICluster,
        protos.google.bigtable.admin.v2.IUpdateClusterMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  >;
  updateCluster(
    request: protos.google.bigtable.admin.v2.ICluster,
    options: CallOptions,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.ICluster,
        protos.google.bigtable.admin.v2.IUpdateClusterMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  updateCluster(
    request: protos.google.bigtable.admin.v2.ICluster,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.ICluster,
        protos.google.bigtable.admin.v2.IUpdateClusterMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  updateCluster(
    request?: protos.google.bigtable.admin.v2.ICluster,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          LROperation<
            protos.google.bigtable.admin.v2.ICluster,
            protos.google.bigtable.admin.v2.IUpdateClusterMetadata
          >,
          protos.google.longrunning.IOperation | null | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.ICluster,
        protos.google.bigtable.admin.v2.IUpdateClusterMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.ICluster,
        protos.google.bigtable.admin.v2.IUpdateClusterMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        name: request.name ?? '',
      });
    this.initialize();
    return this.innerApiCalls.updateCluster(request, options, callback);
  }
  /**
   * Check the status of the long running operation returned by `updateCluster()`.
   * @param {String} name
   *   The operation name that will be passed.
   * @returns {Promise} - The promise which resolves to an object.
   *   The decoded operation object has result and metadata field to get information from.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_instance_admin.update_cluster.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableInstanceAdmin_UpdateCluster_async
   */
  async checkUpdateClusterProgress(
    name: string
  ): Promise<
    LROperation<
      protos.google.bigtable.admin.v2.Cluster,
      protos.google.bigtable.admin.v2.UpdateClusterMetadata
    >
  > {
    const request =
      new this._gaxModule.operationsProtos.google.longrunning.GetOperationRequest(
        {name}
      );
    const [operation] = await this.operationsClient.getOperation(request);
    const decodeOperation = new this._gaxModule.Operation(
      operation,
      this.descriptors.longrunning.updateCluster,
      this._gaxModule.createDefaultBackoffSettings()
    );
    return decodeOperation as LROperation<
      protos.google.bigtable.admin.v2.Cluster,
      protos.google.bigtable.admin.v2.UpdateClusterMetadata
    >;
  }
  /**
   * Partially updates a cluster within a project. This method is the preferred
   * way to update a Cluster.
   *
   * To enable and update autoscaling, set
   * cluster_config.cluster_autoscaling_config. When autoscaling is enabled,
   * serve_nodes is treated as an OUTPUT_ONLY field, meaning that updates to it
   * are ignored. Note that an update cannot simultaneously set serve_nodes to
   * non-zero and cluster_config.cluster_autoscaling_config to non-empty, and
   * also specify both in the update_mask.
   *
   * To disable autoscaling, clear cluster_config.cluster_autoscaling_config,
   * and explicitly set a serve_node count via the update_mask.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {google.bigtable.admin.v2.Cluster} request.cluster
   *   Required. The Cluster which contains the partial updates to be applied,
   *   subject to the update_mask.
   * @param {google.protobuf.FieldMask} request.updateMask
   *   Required. The subset of Cluster fields which should be replaced.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing
   *   a long running operation. Its `promise()` method returns a promise
   *   you can `await` for.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_instance_admin.partial_update_cluster.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableInstanceAdmin_PartialUpdateCluster_async
   */
  partialUpdateCluster(
    request?: protos.google.bigtable.admin.v2.IPartialUpdateClusterRequest,
    options?: CallOptions
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.ICluster,
        protos.google.bigtable.admin.v2.IPartialUpdateClusterMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  >;
  partialUpdateCluster(
    request: protos.google.bigtable.admin.v2.IPartialUpdateClusterRequest,
    options: CallOptions,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.ICluster,
        protos.google.bigtable.admin.v2.IPartialUpdateClusterMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  partialUpdateCluster(
    request: protos.google.bigtable.admin.v2.IPartialUpdateClusterRequest,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.ICluster,
        protos.google.bigtable.admin.v2.IPartialUpdateClusterMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  partialUpdateCluster(
    request?: protos.google.bigtable.admin.v2.IPartialUpdateClusterRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          LROperation<
            protos.google.bigtable.admin.v2.ICluster,
            protos.google.bigtable.admin.v2.IPartialUpdateClusterMetadata
          >,
          protos.google.longrunning.IOperation | null | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.ICluster,
        protos.google.bigtable.admin.v2.IPartialUpdateClusterMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.ICluster,
        protos.google.bigtable.admin.v2.IPartialUpdateClusterMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        'cluster.name': request.cluster!.name ?? '',
      });
    this.initialize();
    return this.innerApiCalls.partialUpdateCluster(request, options, callback);
  }
  /**
   * Check the status of the long running operation returned by `partialUpdateCluster()`.
   * @param {String} name
   *   The operation name that will be passed.
   * @returns {Promise} - The promise which resolves to an object.
   *   The decoded operation object has result and metadata field to get information from.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_instance_admin.partial_update_cluster.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableInstanceAdmin_PartialUpdateCluster_async
   */
  async checkPartialUpdateClusterProgress(
    name: string
  ): Promise<
    LROperation<
      protos.google.bigtable.admin.v2.Cluster,
      protos.google.bigtable.admin.v2.PartialUpdateClusterMetadata
    >
  > {
    const request =
      new this._gaxModule.operationsProtos.google.longrunning.GetOperationRequest(
        {name}
      );
    const [operation] = await this.operationsClient.getOperation(request);
    const decodeOperation = new this._gaxModule.Operation(
      operation,
      this.descriptors.longrunning.partialUpdateCluster,
      this._gaxModule.createDefaultBackoffSettings()
    );
    return decodeOperation as LROperation<
      protos.google.bigtable.admin.v2.Cluster,
      protos.google.bigtable.admin.v2.PartialUpdateClusterMetadata
    >;
  }
  /**
   * Updates an app profile within an instance.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {google.bigtable.admin.v2.AppProfile} request.appProfile
   *   Required. The app profile which will (partially) replace the current value.
   * @param {google.protobuf.FieldMask} request.updateMask
   *   Required. The subset of app profile fields which should be replaced.
   *   If unset, all fields will be replaced.
   * @param {boolean} request.ignoreWarnings
   *   If true, ignore safety checks when updating the app profile.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing
   *   a long running operation. Its `promise()` method returns a promise
   *   you can `await` for.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_instance_admin.update_app_profile.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableInstanceAdmin_UpdateAppProfile_async
   */
  updateAppProfile(
    request?: protos.google.bigtable.admin.v2.IUpdateAppProfileRequest,
    options?: CallOptions
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.IAppProfile,
        protos.google.bigtable.admin.v2.IUpdateAppProfileMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  >;
  updateAppProfile(
    request: protos.google.bigtable.admin.v2.IUpdateAppProfileRequest,
    options: CallOptions,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.IAppProfile,
        protos.google.bigtable.admin.v2.IUpdateAppProfileMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  updateAppProfile(
    request: protos.google.bigtable.admin.v2.IUpdateAppProfileRequest,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.IAppProfile,
        protos.google.bigtable.admin.v2.IUpdateAppProfileMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  updateAppProfile(
    request?: protos.google.bigtable.admin.v2.IUpdateAppProfileRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          LROperation<
            protos.google.bigtable.admin.v2.IAppProfile,
            protos.google.bigtable.admin.v2.IUpdateAppProfileMetadata
          >,
          protos.google.longrunning.IOperation | null | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.IAppProfile,
        protos.google.bigtable.admin.v2.IUpdateAppProfileMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.IAppProfile,
        protos.google.bigtable.admin.v2.IUpdateAppProfileMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        'app_profile.name': request.appProfile!.name ?? '',
      });
    this.initialize();
    return this.innerApiCalls.updateAppProfile(request, options, callback);
  }
  /**
   * Check the status of the long running operation returned by `updateAppProfile()`.
   * @param {String} name
   *   The operation name that will be passed.
   * @returns {Promise} - The promise which resolves to an object.
   *   The decoded operation object has result and metadata field to get information from.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_instance_admin.update_app_profile.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableInstanceAdmin_UpdateAppProfile_async
   */
  async checkUpdateAppProfileProgress(
    name: string
  ): Promise<
    LROperation<
      protos.google.bigtable.admin.v2.AppProfile,
      protos.google.bigtable.admin.v2.UpdateAppProfileMetadata
    >
  > {
    const request =
      new this._gaxModule.operationsProtos.google.longrunning.GetOperationRequest(
        {name}
      );
    const [operation] = await this.operationsClient.getOperation(request);
    const decodeOperation = new this._gaxModule.Operation(
      operation,
      this.descriptors.longrunning.updateAppProfile,
      this._gaxModule.createDefaultBackoffSettings()
    );
    return decodeOperation as LROperation<
      protos.google.bigtable.admin.v2.AppProfile,
      protos.google.bigtable.admin.v2.UpdateAppProfileMetadata
    >;
  }
  /**
   * Lists information about app profiles in an instance.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.parent
   *   Required. The unique name of the instance for which a list of app profiles
   *   is requested. Values are of the form
   *   `projects/{project}/instances/{instance}`.
   *   Use `{instance} = '-'` to list AppProfiles for all Instances in a project,
   *   e.g., `projects/myproject/instances/-`.
   * @param {number} request.pageSize
   *   Maximum number of results per page.
   *
   *   A page_size of zero lets the server choose the number of items to return.
   *   A page_size which is strictly positive will return at most that many items.
   *   A negative page_size will cause an error.
   *
   *   Following the first request, subsequent paginated calls are not required
   *   to pass a page_size. If a page_size is set in subsequent calls, it must
   *   match the page_size given in the first request.
   * @param {string} request.pageToken
   *   The value of `next_page_token` returned by a previous call.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is Array of {@link protos.google.bigtable.admin.v2.AppProfile|AppProfile}.
   *   The client library will perform auto-pagination by default: it will call the API as many
   *   times as needed and will merge results from all the pages into this array.
   *   Note that it can affect your quota.
   *   We recommend using `listAppProfilesAsync()`
   *   method described below for async iteration which you can stop as needed.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#auto-pagination | documentation }
   *   for more details and examples.
   */
  listAppProfiles(
    request?: protos.google.bigtable.admin.v2.IListAppProfilesRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.bigtable.admin.v2.IAppProfile[],
      protos.google.bigtable.admin.v2.IListAppProfilesRequest | null,
      protos.google.bigtable.admin.v2.IListAppProfilesResponse,
    ]
  >;
  listAppProfiles(
    request: protos.google.bigtable.admin.v2.IListAppProfilesRequest,
    options: CallOptions,
    callback: PaginationCallback<
      protos.google.bigtable.admin.v2.IListAppProfilesRequest,
      | protos.google.bigtable.admin.v2.IListAppProfilesResponse
      | null
      | undefined,
      protos.google.bigtable.admin.v2.IAppProfile
    >
  ): void;
  listAppProfiles(
    request: protos.google.bigtable.admin.v2.IListAppProfilesRequest,
    callback: PaginationCallback<
      protos.google.bigtable.admin.v2.IListAppProfilesRequest,
      | protos.google.bigtable.admin.v2.IListAppProfilesResponse
      | null
      | undefined,
      protos.google.bigtable.admin.v2.IAppProfile
    >
  ): void;
  listAppProfiles(
    request?: protos.google.bigtable.admin.v2.IListAppProfilesRequest,
    optionsOrCallback?:
      | CallOptions
      | PaginationCallback<
          protos.google.bigtable.admin.v2.IListAppProfilesRequest,
          | protos.google.bigtable.admin.v2.IListAppProfilesResponse
          | null
          | undefined,
          protos.google.bigtable.admin.v2.IAppProfile
        >,
    callback?: PaginationCallback<
      protos.google.bigtable.admin.v2.IListAppProfilesRequest,
      | protos.google.bigtable.admin.v2.IListAppProfilesResponse
      | null
      | undefined,
      protos.google.bigtable.admin.v2.IAppProfile
    >
  ): Promise<
    [
      protos.google.bigtable.admin.v2.IAppProfile[],
      protos.google.bigtable.admin.v2.IListAppProfilesRequest | null,
      protos.google.bigtable.admin.v2.IListAppProfilesResponse,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        parent: request.parent ?? '',
      });
    this.initialize();
    return this.innerApiCalls.listAppProfiles(request, options, callback);
  }

  /**
   * Equivalent to `method.name.toCamelCase()`, but returns a NodeJS Stream object.
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.parent
   *   Required. The unique name of the instance for which a list of app profiles
   *   is requested. Values are of the form
   *   `projects/{project}/instances/{instance}`.
   *   Use `{instance} = '-'` to list AppProfiles for all Instances in a project,
   *   e.g., `projects/myproject/instances/-`.
   * @param {number} request.pageSize
   *   Maximum number of results per page.
   *
   *   A page_size of zero lets the server choose the number of items to return.
   *   A page_size which is strictly positive will return at most that many items.
   *   A negative page_size will cause an error.
   *
   *   Following the first request, subsequent paginated calls are not required
   *   to pass a page_size. If a page_size is set in subsequent calls, it must
   *   match the page_size given in the first request.
   * @param {string} request.pageToken
   *   The value of `next_page_token` returned by a previous call.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Stream}
   *   An object stream which emits an object representing {@link protos.google.bigtable.admin.v2.AppProfile|AppProfile} on 'data' event.
   *   The client library will perform auto-pagination by default: it will call the API as many
   *   times as needed. Note that it can affect your quota.
   *   We recommend using `listAppProfilesAsync()`
   *   method described below for async iteration which you can stop as needed.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#auto-pagination | documentation }
   *   for more details and examples.
   */
  listAppProfilesStream(
    request?: protos.google.bigtable.admin.v2.IListAppProfilesRequest,
    options?: CallOptions
  ): Transform {
    request = request || {};
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        parent: request.parent ?? '',
      });
    const defaultCallSettings = this._defaults['listAppProfiles'];
    const callSettings = defaultCallSettings.merge(options);
    this.initialize();
    return this.descriptors.page.listAppProfiles.createStream(
      this.innerApiCalls.listAppProfiles as GaxCall,
      request,
      callSettings
    );
  }

  /**
   * Equivalent to `listAppProfiles`, but returns an iterable object.
   *
   * `for`-`await`-`of` syntax is used with the iterable to get response elements on-demand.
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.parent
   *   Required. The unique name of the instance for which a list of app profiles
   *   is requested. Values are of the form
   *   `projects/{project}/instances/{instance}`.
   *   Use `{instance} = '-'` to list AppProfiles for all Instances in a project,
   *   e.g., `projects/myproject/instances/-`.
   * @param {number} request.pageSize
   *   Maximum number of results per page.
   *
   *   A page_size of zero lets the server choose the number of items to return.
   *   A page_size which is strictly positive will return at most that many items.
   *   A negative page_size will cause an error.
   *
   *   Following the first request, subsequent paginated calls are not required
   *   to pass a page_size. If a page_size is set in subsequent calls, it must
   *   match the page_size given in the first request.
   * @param {string} request.pageToken
   *   The value of `next_page_token` returned by a previous call.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Object}
   *   An iterable Object that allows {@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Iteration_protocols | async iteration }.
   *   When you iterate the returned iterable, each element will be an object representing
   *   {@link protos.google.bigtable.admin.v2.AppProfile|AppProfile}. The API will be called under the hood as needed, once per the page,
   *   so you can stop the iteration when you don't need more results.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#auto-pagination | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_instance_admin.list_app_profiles.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableInstanceAdmin_ListAppProfiles_async
   */
  listAppProfilesAsync(
    request?: protos.google.bigtable.admin.v2.IListAppProfilesRequest,
    options?: CallOptions
  ): AsyncIterable<protos.google.bigtable.admin.v2.IAppProfile> {
    request = request || {};
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        parent: request.parent ?? '',
      });
    const defaultCallSettings = this._defaults['listAppProfiles'];
    const callSettings = defaultCallSettings.merge(options);
    this.initialize();
    return this.descriptors.page.listAppProfiles.asyncIterate(
      this.innerApiCalls['listAppProfiles'] as GaxCall,
      request as {},
      callSettings
    ) as AsyncIterable<protos.google.bigtable.admin.v2.IAppProfile>;
  }
  /**
   * Lists hot tablets in a cluster, within the time range provided. Hot
   * tablets are ordered based on CPU usage.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.parent
   *   Required. The cluster name to list hot tablets.
   *   Value is in the following form:
   *   `projects/{project}/instances/{instance}/clusters/{cluster}`.
   * @param {google.protobuf.Timestamp} request.startTime
   *   The start time to list hot tablets. The hot tablets in the response will
   *   have start times between the requested start time and end time. Start time
   *   defaults to Now if it is unset, and end time defaults to Now - 24 hours if
   *   it is unset. The start time should be less than the end time, and the
   *   maximum allowed time range between start time and end time is 48 hours.
   *   Start time and end time should have values between Now and Now - 14 days.
   * @param {google.protobuf.Timestamp} request.endTime
   *   The end time to list hot tablets.
   * @param {number} request.pageSize
   *   Maximum number of results per page.
   *
   *   A page_size that is empty or zero lets the server choose the number of
   *   items to return. A page_size which is strictly positive will return at most
   *   that many items. A negative page_size will cause an error.
   *
   *   Following the first request, subsequent paginated calls do not need a
   *   page_size field. If a page_size is set in subsequent calls, it must match
   *   the page_size given in the first request.
   * @param {string} request.pageToken
   *   The value of `next_page_token` returned by a previous call.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is Array of {@link protos.google.bigtable.admin.v2.HotTablet|HotTablet}.
   *   The client library will perform auto-pagination by default: it will call the API as many
   *   times as needed and will merge results from all the pages into this array.
   *   Note that it can affect your quota.
   *   We recommend using `listHotTabletsAsync()`
   *   method described below for async iteration which you can stop as needed.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#auto-pagination | documentation }
   *   for more details and examples.
   */
  listHotTablets(
    request?: protos.google.bigtable.admin.v2.IListHotTabletsRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.bigtable.admin.v2.IHotTablet[],
      protos.google.bigtable.admin.v2.IListHotTabletsRequest | null,
      protos.google.bigtable.admin.v2.IListHotTabletsResponse,
    ]
  >;
  listHotTablets(
    request: protos.google.bigtable.admin.v2.IListHotTabletsRequest,
    options: CallOptions,
    callback: PaginationCallback<
      protos.google.bigtable.admin.v2.IListHotTabletsRequest,
      | protos.google.bigtable.admin.v2.IListHotTabletsResponse
      | null
      | undefined,
      protos.google.bigtable.admin.v2.IHotTablet
    >
  ): void;
  listHotTablets(
    request: protos.google.bigtable.admin.v2.IListHotTabletsRequest,
    callback: PaginationCallback<
      protos.google.bigtable.admin.v2.IListHotTabletsRequest,
      | protos.google.bigtable.admin.v2.IListHotTabletsResponse
      | null
      | undefined,
      protos.google.bigtable.admin.v2.IHotTablet
    >
  ): void;
  listHotTablets(
    request?: protos.google.bigtable.admin.v2.IListHotTabletsRequest,
    optionsOrCallback?:
      | CallOptions
      | PaginationCallback<
          protos.google.bigtable.admin.v2.IListHotTabletsRequest,
          | protos.google.bigtable.admin.v2.IListHotTabletsResponse
          | null
          | undefined,
          protos.google.bigtable.admin.v2.IHotTablet
        >,
    callback?: PaginationCallback<
      protos.google.bigtable.admin.v2.IListHotTabletsRequest,
      | protos.google.bigtable.admin.v2.IListHotTabletsResponse
      | null
      | undefined,
      protos.google.bigtable.admin.v2.IHotTablet
    >
  ): Promise<
    [
      protos.google.bigtable.admin.v2.IHotTablet[],
      protos.google.bigtable.admin.v2.IListHotTabletsRequest | null,
      protos.google.bigtable.admin.v2.IListHotTabletsResponse,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        parent: request.parent ?? '',
      });
    this.initialize();
    return this.innerApiCalls.listHotTablets(request, options, callback);
  }

  /**
   * Equivalent to `method.name.toCamelCase()`, but returns a NodeJS Stream object.
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.parent
   *   Required. The cluster name to list hot tablets.
   *   Value is in the following form:
   *   `projects/{project}/instances/{instance}/clusters/{cluster}`.
   * @param {google.protobuf.Timestamp} request.startTime
   *   The start time to list hot tablets. The hot tablets in the response will
   *   have start times between the requested start time and end time. Start time
   *   defaults to Now if it is unset, and end time defaults to Now - 24 hours if
   *   it is unset. The start time should be less than the end time, and the
   *   maximum allowed time range between start time and end time is 48 hours.
   *   Start time and end time should have values between Now and Now - 14 days.
   * @param {google.protobuf.Timestamp} request.endTime
   *   The end time to list hot tablets.
   * @param {number} request.pageSize
   *   Maximum number of results per page.
   *
   *   A page_size that is empty or zero lets the server choose the number of
   *   items to return. A page_size which is strictly positive will return at most
   *   that many items. A negative page_size will cause an error.
   *
   *   Following the first request, subsequent paginated calls do not need a
   *   page_size field. If a page_size is set in subsequent calls, it must match
   *   the page_size given in the first request.
   * @param {string} request.pageToken
   *   The value of `next_page_token` returned by a previous call.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Stream}
   *   An object stream which emits an object representing {@link protos.google.bigtable.admin.v2.HotTablet|HotTablet} on 'data' event.
   *   The client library will perform auto-pagination by default: it will call the API as many
   *   times as needed. Note that it can affect your quota.
   *   We recommend using `listHotTabletsAsync()`
   *   method described below for async iteration which you can stop as needed.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#auto-pagination | documentation }
   *   for more details and examples.
   */
  listHotTabletsStream(
    request?: protos.google.bigtable.admin.v2.IListHotTabletsRequest,
    options?: CallOptions
  ): Transform {
    request = request || {};
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        parent: request.parent ?? '',
      });
    const defaultCallSettings = this._defaults['listHotTablets'];
    const callSettings = defaultCallSettings.merge(options);
    this.initialize();
    return this.descriptors.page.listHotTablets.createStream(
      this.innerApiCalls.listHotTablets as GaxCall,
      request,
      callSettings
    );
  }

  /**
   * Equivalent to `listHotTablets`, but returns an iterable object.
   *
   * `for`-`await`-`of` syntax is used with the iterable to get response elements on-demand.
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.parent
   *   Required. The cluster name to list hot tablets.
   *   Value is in the following form:
   *   `projects/{project}/instances/{instance}/clusters/{cluster}`.
   * @param {google.protobuf.Timestamp} request.startTime
   *   The start time to list hot tablets. The hot tablets in the response will
   *   have start times between the requested start time and end time. Start time
   *   defaults to Now if it is unset, and end time defaults to Now - 24 hours if
   *   it is unset. The start time should be less than the end time, and the
   *   maximum allowed time range between start time and end time is 48 hours.
   *   Start time and end time should have values between Now and Now - 14 days.
   * @param {google.protobuf.Timestamp} request.endTime
   *   The end time to list hot tablets.
   * @param {number} request.pageSize
   *   Maximum number of results per page.
   *
   *   A page_size that is empty or zero lets the server choose the number of
   *   items to return. A page_size which is strictly positive will return at most
   *   that many items. A negative page_size will cause an error.
   *
   *   Following the first request, subsequent paginated calls do not need a
   *   page_size field. If a page_size is set in subsequent calls, it must match
   *   the page_size given in the first request.
   * @param {string} request.pageToken
   *   The value of `next_page_token` returned by a previous call.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Object}
   *   An iterable Object that allows {@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Iteration_protocols | async iteration }.
   *   When you iterate the returned iterable, each element will be an object representing
   *   {@link protos.google.bigtable.admin.v2.HotTablet|HotTablet}. The API will be called under the hood as needed, once per the page,
   *   so you can stop the iteration when you don't need more results.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#auto-pagination | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_instance_admin.list_hot_tablets.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableInstanceAdmin_ListHotTablets_async
   */
  listHotTabletsAsync(
    request?: protos.google.bigtable.admin.v2.IListHotTabletsRequest,
    options?: CallOptions
  ): AsyncIterable<protos.google.bigtable.admin.v2.IHotTablet> {
    request = request || {};
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        parent: request.parent ?? '',
      });
    const defaultCallSettings = this._defaults['listHotTablets'];
    const callSettings = defaultCallSettings.merge(options);
    this.initialize();
    return this.descriptors.page.listHotTablets.asyncIterate(
      this.innerApiCalls['listHotTablets'] as GaxCall,
      request as {},
      callSettings
    ) as AsyncIterable<protos.google.bigtable.admin.v2.IHotTablet>;
  }
  // --------------------
  // -- Path templates --
  // --------------------

  /**
   * Return a fully-qualified appProfile resource name string.
   *
   * @param {string} project
   * @param {string} instance
   * @param {string} app_profile
   * @returns {string} Resource name string.
   */
  appProfilePath(project: string, instance: string, appProfile: string) {
    return this.pathTemplates.appProfilePathTemplate.render({
      project: project,
      instance: instance,
      app_profile: appProfile,
    });
  }

  /**
   * Parse the project from AppProfile resource.
   *
   * @param {string} appProfileName
   *   A fully-qualified path representing AppProfile resource.
   * @returns {string} A string representing the project.
   */
  matchProjectFromAppProfileName(appProfileName: string) {
    return this.pathTemplates.appProfilePathTemplate.match(appProfileName)
      .project;
  }

  /**
   * Parse the instance from AppProfile resource.
   *
   * @param {string} appProfileName
   *   A fully-qualified path representing AppProfile resource.
   * @returns {string} A string representing the instance.
   */
  matchInstanceFromAppProfileName(appProfileName: string) {
    return this.pathTemplates.appProfilePathTemplate.match(appProfileName)
      .instance;
  }

  /**
   * Parse the app_profile from AppProfile resource.
   *
   * @param {string} appProfileName
   *   A fully-qualified path representing AppProfile resource.
   * @returns {string} A string representing the app_profile.
   */
  matchAppProfileFromAppProfileName(appProfileName: string) {
    return this.pathTemplates.appProfilePathTemplate.match(appProfileName)
      .app_profile;
  }

  /**
   * Return a fully-qualified authorizedView resource name string.
   *
   * @param {string} project
   * @param {string} instance
   * @param {string} table
   * @param {string} authorized_view
   * @returns {string} Resource name string.
   */
  authorizedViewPath(
    project: string,
    instance: string,
    table: string,
    authorizedView: string
  ) {
    return this.pathTemplates.authorizedViewPathTemplate.render({
      project: project,
      instance: instance,
      table: table,
      authorized_view: authorizedView,
    });
  }

  /**
   * Parse the project from AuthorizedView resource.
   *
   * @param {string} authorizedViewName
   *   A fully-qualified path representing AuthorizedView resource.
   * @returns {string} A string representing the project.
   */
  matchProjectFromAuthorizedViewName(authorizedViewName: string) {
    return this.pathTemplates.authorizedViewPathTemplate.match(
      authorizedViewName
    ).project;
  }

  /**
   * Parse the instance from AuthorizedView resource.
   *
   * @param {string} authorizedViewName
   *   A fully-qualified path representing AuthorizedView resource.
   * @returns {string} A string representing the instance.
   */
  matchInstanceFromAuthorizedViewName(authorizedViewName: string) {
    return this.pathTemplates.authorizedViewPathTemplate.match(
      authorizedViewName
    ).instance;
  }

  /**
   * Parse the table from AuthorizedView resource.
   *
   * @param {string} authorizedViewName
   *   A fully-qualified path representing AuthorizedView resource.
   * @returns {string} A string representing the table.
   */
  matchTableFromAuthorizedViewName(authorizedViewName: string) {
    return this.pathTemplates.authorizedViewPathTemplate.match(
      authorizedViewName
    ).table;
  }

  /**
   * Parse the authorized_view from AuthorizedView resource.
   *
   * @param {string} authorizedViewName
   *   A fully-qualified path representing AuthorizedView resource.
   * @returns {string} A string representing the authorized_view.
   */
  matchAuthorizedViewFromAuthorizedViewName(authorizedViewName: string) {
    return this.pathTemplates.authorizedViewPathTemplate.match(
      authorizedViewName
    ).authorized_view;
  }

  /**
   * Return a fully-qualified backup resource name string.
   *
   * @param {string} project
   * @param {string} instance
   * @param {string} cluster
   * @param {string} backup
   * @returns {string} Resource name string.
   */
  backupPath(
    project: string,
    instance: string,
    cluster: string,
    backup: string
  ) {
    return this.pathTemplates.backupPathTemplate.render({
      project: project,
      instance: instance,
      cluster: cluster,
      backup: backup,
    });
  }

  /**
   * Parse the project from Backup resource.
   *
   * @param {string} backupName
   *   A fully-qualified path representing Backup resource.
   * @returns {string} A string representing the project.
   */
  matchProjectFromBackupName(backupName: string) {
    return this.pathTemplates.backupPathTemplate.match(backupName).project;
  }

  /**
   * Parse the instance from Backup resource.
   *
   * @param {string} backupName
   *   A fully-qualified path representing Backup resource.
   * @returns {string} A string representing the instance.
   */
  matchInstanceFromBackupName(backupName: string) {
    return this.pathTemplates.backupPathTemplate.match(backupName).instance;
  }

  /**
   * Parse the cluster from Backup resource.
   *
   * @param {string} backupName
   *   A fully-qualified path representing Backup resource.
   * @returns {string} A string representing the cluster.
   */
  matchClusterFromBackupName(backupName: string) {
    return this.pathTemplates.backupPathTemplate.match(backupName).cluster;
  }

  /**
   * Parse the backup from Backup resource.
   *
   * @param {string} backupName
   *   A fully-qualified path representing Backup resource.
   * @returns {string} A string representing the backup.
   */
  matchBackupFromBackupName(backupName: string) {
    return this.pathTemplates.backupPathTemplate.match(backupName).backup;
  }

  /**
   * Return a fully-qualified cluster resource name string.
   *
   * @param {string} project
   * @param {string} instance
   * @param {string} cluster
   * @returns {string} Resource name string.
   */
  clusterPath(project: string, instance: string, cluster: string) {
    return this.pathTemplates.clusterPathTemplate.render({
      project: project,
      instance: instance,
      cluster: cluster,
    });
  }

  /**
   * Parse the project from Cluster resource.
   *
   * @param {string} clusterName
   *   A fully-qualified path representing Cluster resource.
   * @returns {string} A string representing the project.
   */
  matchProjectFromClusterName(clusterName: string) {
    return this.pathTemplates.clusterPathTemplate.match(clusterName).project;
  }

  /**
   * Parse the instance from Cluster resource.
   *
   * @param {string} clusterName
   *   A fully-qualified path representing Cluster resource.
   * @returns {string} A string representing the instance.
   */
  matchInstanceFromClusterName(clusterName: string) {
    return this.pathTemplates.clusterPathTemplate.match(clusterName).instance;
  }

  /**
   * Parse the cluster from Cluster resource.
   *
   * @param {string} clusterName
   *   A fully-qualified path representing Cluster resource.
   * @returns {string} A string representing the cluster.
   */
  matchClusterFromClusterName(clusterName: string) {
    return this.pathTemplates.clusterPathTemplate.match(clusterName).cluster;
  }

  /**
   * Return a fully-qualified hotTablet resource name string.
   *
   * @param {string} project
   * @param {string} instance
   * @param {string} cluster
   * @param {string} hot_tablet
   * @returns {string} Resource name string.
   */
  hotTabletPath(
    project: string,
    instance: string,
    cluster: string,
    hotTablet: string
  ) {
    return this.pathTemplates.hotTabletPathTemplate.render({
      project: project,
      instance: instance,
      cluster: cluster,
      hot_tablet: hotTablet,
    });
  }

  /**
   * Parse the project from HotTablet resource.
   *
   * @param {string} hotTabletName
   *   A fully-qualified path representing HotTablet resource.
   * @returns {string} A string representing the project.
   */
  matchProjectFromHotTabletName(hotTabletName: string) {
    return this.pathTemplates.hotTabletPathTemplate.match(hotTabletName)
      .project;
  }

  /**
   * Parse the instance from HotTablet resource.
   *
   * @param {string} hotTabletName
   *   A fully-qualified path representing HotTablet resource.
   * @returns {string} A string representing the instance.
   */
  matchInstanceFromHotTabletName(hotTabletName: string) {
    return this.pathTemplates.hotTabletPathTemplate.match(hotTabletName)
      .instance;
  }

  /**
   * Parse the cluster from HotTablet resource.
   *
   * @param {string} hotTabletName
   *   A fully-qualified path representing HotTablet resource.
   * @returns {string} A string representing the cluster.
   */
  matchClusterFromHotTabletName(hotTabletName: string) {
    return this.pathTemplates.hotTabletPathTemplate.match(hotTabletName)
      .cluster;
  }

  /**
   * Parse the hot_tablet from HotTablet resource.
   *
   * @param {string} hotTabletName
   *   A fully-qualified path representing HotTablet resource.
   * @returns {string} A string representing the hot_tablet.
   */
  matchHotTabletFromHotTabletName(hotTabletName: string) {
    return this.pathTemplates.hotTabletPathTemplate.match(hotTabletName)
      .hot_tablet;
  }

  /**
   * Return a fully-qualified instance resource name string.
   *
   * @param {string} project
   * @param {string} instance
   * @returns {string} Resource name string.
   */
  instancePath(project: string, instance: string) {
    return this.pathTemplates.instancePathTemplate.render({
      project: project,
      instance: instance,
    });
  }

  /**
   * Parse the project from Instance resource.
   *
   * @param {string} instanceName
   *   A fully-qualified path representing Instance resource.
   * @returns {string} A string representing the project.
   */
  matchProjectFromInstanceName(instanceName: string) {
    return this.pathTemplates.instancePathTemplate.match(instanceName).project;
  }

  /**
   * Parse the instance from Instance resource.
   *
   * @param {string} instanceName
   *   A fully-qualified path representing Instance resource.
   * @returns {string} A string representing the instance.
   */
  matchInstanceFromInstanceName(instanceName: string) {
    return this.pathTemplates.instancePathTemplate.match(instanceName).instance;
  }

  /**
   * Return a fully-qualified project resource name string.
   *
   * @param {string} project
   * @returns {string} Resource name string.
   */
  projectPath(project: string) {
    return this.pathTemplates.projectPathTemplate.render({
      project: project,
    });
  }

  /**
   * Parse the project from Project resource.
   *
   * @param {string} projectName
   *   A fully-qualified path representing Project resource.
   * @returns {string} A string representing the project.
   */
  matchProjectFromProjectName(projectName: string) {
    return this.pathTemplates.projectPathTemplate.match(projectName).project;
  }

  /**
   * Return a fully-qualified snapshot resource name string.
   *
   * @param {string} project
   * @param {string} instance
   * @param {string} cluster
   * @param {string} snapshot
   * @returns {string} Resource name string.
   */
  snapshotPath(
    project: string,
    instance: string,
    cluster: string,
    snapshot: string
  ) {
    return this.pathTemplates.snapshotPathTemplate.render({
      project: project,
      instance: instance,
      cluster: cluster,
      snapshot: snapshot,
    });
  }

  /**
   * Parse the project from Snapshot resource.
   *
   * @param {string} snapshotName
   *   A fully-qualified path representing Snapshot resource.
   * @returns {string} A string representing the project.
   */
  matchProjectFromSnapshotName(snapshotName: string) {
    return this.pathTemplates.snapshotPathTemplate.match(snapshotName).project;
  }

  /**
   * Parse the instance from Snapshot resource.
   *
   * @param {string} snapshotName
   *   A fully-qualified path representing Snapshot resource.
   * @returns {string} A string representing the instance.
   */
  matchInstanceFromSnapshotName(snapshotName: string) {
    return this.pathTemplates.snapshotPathTemplate.match(snapshotName).instance;
  }

  /**
   * Parse the cluster from Snapshot resource.
   *
   * @param {string} snapshotName
   *   A fully-qualified path representing Snapshot resource.
   * @returns {string} A string representing the cluster.
   */
  matchClusterFromSnapshotName(snapshotName: string) {
    return this.pathTemplates.snapshotPathTemplate.match(snapshotName).cluster;
  }

  /**
   * Parse the snapshot from Snapshot resource.
   *
   * @param {string} snapshotName
   *   A fully-qualified path representing Snapshot resource.
   * @returns {string} A string representing the snapshot.
   */
  matchSnapshotFromSnapshotName(snapshotName: string) {
    return this.pathTemplates.snapshotPathTemplate.match(snapshotName).snapshot;
  }

  /**
   * Return a fully-qualified table resource name string.
   *
   * @param {string} project
   * @param {string} instance
   * @param {string} table
   * @returns {string} Resource name string.
   */
  tablePath(project: string, instance: string, table: string) {
    return this.pathTemplates.tablePathTemplate.render({
      project: project,
      instance: instance,
      table: table,
    });
  }

  /**
   * Parse the project from Table resource.
   *
   * @param {string} tableName
   *   A fully-qualified path representing Table resource.
   * @returns {string} A string representing the project.
   */
  matchProjectFromTableName(tableName: string) {
    return this.pathTemplates.tablePathTemplate.match(tableName).project;
  }

  /**
   * Parse the instance from Table resource.
   *
   * @param {string} tableName
   *   A fully-qualified path representing Table resource.
   * @returns {string} A string representing the instance.
   */
  matchInstanceFromTableName(tableName: string) {
    return this.pathTemplates.tablePathTemplate.match(tableName).instance;
  }

  /**
   * Parse the table from Table resource.
   *
   * @param {string} tableName
   *   A fully-qualified path representing Table resource.
   * @returns {string} A string representing the table.
   */
  matchTableFromTableName(tableName: string) {
    return this.pathTemplates.tablePathTemplate.match(tableName).table;
  }

  /**
   * Terminate the gRPC channel and close the client.
   *
   * The client will no longer be usable and all future behavior is undefined.
   * @returns {Promise} A promise that resolves when the client is closed.
   */
  close(): Promise<void> {
    if (this.bigtableInstanceAdminStub && !this._terminated) {
      return this.bigtableInstanceAdminStub.then(stub => {
        this._terminated = true;
        stub.close();
        this.operationsClient.close();
      });
    }
    return Promise.resolve();
  }
}
