// Copyright 2024 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// ** This file is automatically generated by gapic-generator-typescript. **
// ** https://github.com/googleapis/gapic-generator-typescript **
// ** All changes to this file may be overwritten. **

/* global window */
import type * as gax from 'google-gax';
import type {
  Callback,
  CallOptions,
  Descriptors,
  ClientOptions,
  GrpcClientOptions,
  LROperation,
  PaginationCallback,
  GaxCall,
} from 'google-gax';
import {Transform} from 'stream';
import * as protos from '../../protos/protos';
import jsonProtos = require('../../protos/protos.json');

/**
 * Client JSON configuration object, loaded from
 * `src/v2/bigtable_table_admin_client_config.json`.
 * This file defines retry strategy and timeouts for all API methods in this library.
 */
import * as gapicConfig from './bigtable_table_admin_client_config.json';
const version = require('../../../package.json').version;

/**
 *  Service for creating, configuring, and deleting Cloud Bigtable tables.
 *
 *
 *  Provides access to the table schemas only, not the data stored within
 *  the tables.
 * @class
 * @memberof v2
 */
export class BigtableTableAdminClient {
  private _terminated = false;
  private _opts: ClientOptions;
  private _providedCustomServicePath: boolean;
  private _gaxModule: typeof gax | typeof gax.fallback;
  private _gaxGrpc: gax.GrpcClient | gax.fallback.GrpcClient;
  private _protos: {};
  private _defaults: {[method: string]: gax.CallSettings};
  private _universeDomain: string;
  private _servicePath: string;
  auth: gax.GoogleAuth;
  descriptors: Descriptors = {
    page: {},
    stream: {},
    longrunning: {},
    batching: {},
  };
  warn: (code: string, message: string, warnType?: string) => void;
  innerApiCalls: {[name: string]: Function};
  pathTemplates: {[name: string]: gax.PathTemplate};
  operationsClient: gax.OperationsClient;
  bigtableTableAdminStub?: Promise<{[name: string]: Function}>;

  /**
   * Construct an instance of BigtableTableAdminClient.
   *
   * @param {object} [options] - The configuration object.
   * The options accepted by the constructor are described in detail
   * in [this document](https://github.com/googleapis/gax-nodejs/blob/main/client-libraries.md#creating-the-client-instance).
   * The common options are:
   * @param {object} [options.credentials] - Credentials object.
   * @param {string} [options.credentials.client_email]
   * @param {string} [options.credentials.private_key]
   * @param {string} [options.email] - Account email address. Required when
   *     using a .pem or .p12 keyFilename.
   * @param {string} [options.keyFilename] - Full path to the a .json, .pem, or
   *     .p12 key downloaded from the Google Developers Console. If you provide
   *     a path to a JSON file, the projectId option below is not necessary.
   *     NOTE: .pem and .p12 require you to specify options.email as well.
   * @param {number} [options.port] - The port on which to connect to
   *     the remote host.
   * @param {string} [options.projectId] - The project ID from the Google
   *     Developer's Console, e.g. 'grape-spaceship-123'. We will also check
   *     the environment variable GCLOUD_PROJECT for your project ID. If your
   *     app is running in an environment which supports
   *     {@link https://developers.google.com/identity/protocols/application-default-credentials Application Default Credentials},
   *     your project ID will be detected automatically.
   * @param {string} [options.apiEndpoint] - The domain name of the
   *     API remote host.
   * @param {gax.ClientConfig} [options.clientConfig] - Client configuration override.
   *     Follows the structure of {@link gapicConfig}.
   * @param {boolean} [options.fallback] - Use HTTP/1.1 REST mode.
   *     For more information, please check the
   *     {@link https://github.com/googleapis/gax-nodejs/blob/main/client-libraries.md#http11-rest-api-mode documentation}.
   * @param {gax} [gaxInstance]: loaded instance of `google-gax`. Useful if you
   *     need to avoid loading the default gRPC version and want to use the fallback
   *     HTTP implementation. Load only fallback version and pass it to the constructor:
   *     ```
   *     const gax = require('google-gax/build/src/fallback'); // avoids loading google-gax with gRPC
   *     const client = new BigtableTableAdminClient({fallback: true}, gax);
   *     ```
   */
  constructor(
    opts?: ClientOptions,
    gaxInstance?: typeof gax | typeof gax.fallback
  ) {
    // Ensure that options include all the required fields.
    const staticMembers = this.constructor as typeof BigtableTableAdminClient;
    if (
      opts?.universe_domain &&
      opts?.universeDomain &&
      opts?.universe_domain !== opts?.universeDomain
    ) {
      throw new Error(
        'Please set either universe_domain or universeDomain, but not both.'
      );
    }
    const universeDomainEnvVar =
      typeof process === 'object' && typeof process.env === 'object'
        ? process.env['GOOGLE_CLOUD_UNIVERSE_DOMAIN']
        : undefined;
    this._universeDomain =
      opts?.universeDomain ??
      opts?.universe_domain ??
      universeDomainEnvVar ??
      'googleapis.com';
    this._servicePath = 'bigtableadmin.' + this._universeDomain;
    const servicePath =
      opts?.servicePath || opts?.apiEndpoint || this._servicePath;
    this._providedCustomServicePath = !!(
      opts?.servicePath || opts?.apiEndpoint
    );
    const port = opts?.port || staticMembers.port;
    const clientConfig = opts?.clientConfig ?? {};
    const fallback =
      opts?.fallback ??
      (typeof window !== 'undefined' && typeof window?.fetch === 'function');
    opts = Object.assign({servicePath, port, clientConfig, fallback}, opts);

    // Request numeric enum values if REST transport is used.
    opts.numericEnums = true;

    // If scopes are unset in options and we're connecting to a non-default endpoint, set scopes just in case.
    if (servicePath !== this._servicePath && !('scopes' in opts)) {
      opts['scopes'] = staticMembers.scopes;
    }

    // Load google-gax module synchronously if needed
    if (!gaxInstance) {
      gaxInstance = require('google-gax') as typeof gax;
    }

    // Choose either gRPC or proto-over-HTTP implementation of google-gax.
    this._gaxModule = opts.fallback ? gaxInstance.fallback : gaxInstance;

    // Create a `gaxGrpc` object, with any grpc-specific options sent to the client.
    this._gaxGrpc = new this._gaxModule.GrpcClient(opts);

    // Save options to use in initialize() method.
    this._opts = opts;

    // Save the auth object to the client, for use by other methods.
    this.auth = this._gaxGrpc.auth as gax.GoogleAuth;

    // Set useJWTAccessWithScope on the auth object.
    this.auth.useJWTAccessWithScope = true;

    // Set defaultServicePath on the auth object.
    this.auth.defaultServicePath = this._servicePath;

    // Set the default scopes in auth client if needed.
    if (servicePath === this._servicePath) {
      this.auth.defaultScopes = staticMembers.scopes;
    }

    // Determine the client header string.
    const clientHeader = [`gax/${this._gaxModule.version}`, `gapic/${version}`];
    if (typeof process === 'object' && 'versions' in process) {
      clientHeader.push(`gl-node/${process.versions.node}`);
    } else {
      clientHeader.push(`gl-web/${this._gaxModule.version}`);
    }
    if (!opts.fallback) {
      clientHeader.push(`grpc/${this._gaxGrpc.grpcVersion}`);
    } else {
      clientHeader.push(`rest/${this._gaxGrpc.grpcVersion}`);
    }
    if (opts.libName && opts.libVersion) {
      clientHeader.push(`${opts.libName}/${opts.libVersion}`);
    }
    // Load the applicable protos.
    this._protos = this._gaxGrpc.loadProtoJSON(jsonProtos);

    // This API contains "path templates"; forward-slash-separated
    // identifiers to uniquely identify resources within the API.
    // Create useful helper objects for these.
    this.pathTemplates = {
      appProfilePathTemplate: new this._gaxModule.PathTemplate(
        'projects/{project}/instances/{instance}/appProfiles/{app_profile}'
      ),
      authorizedViewPathTemplate: new this._gaxModule.PathTemplate(
        'projects/{project}/instances/{instance}/tables/{table}/authorizedViews/{authorized_view}'
      ),
      backupPathTemplate: new this._gaxModule.PathTemplate(
        'projects/{project}/instances/{instance}/clusters/{cluster}/backups/{backup}'
      ),
      clusterPathTemplate: new this._gaxModule.PathTemplate(
        'projects/{project}/instances/{instance}/clusters/{cluster}'
      ),
      hotTabletPathTemplate: new this._gaxModule.PathTemplate(
        'projects/{project}/instances/{instance}/clusters/{cluster}/hotTablets/{hot_tablet}'
      ),
      instancePathTemplate: new this._gaxModule.PathTemplate(
        'projects/{project}/instances/{instance}'
      ),
      projectPathTemplate: new this._gaxModule.PathTemplate(
        'projects/{project}'
      ),
      snapshotPathTemplate: new this._gaxModule.PathTemplate(
        'projects/{project}/instances/{instance}/clusters/{cluster}/snapshots/{snapshot}'
      ),
      tablePathTemplate: new this._gaxModule.PathTemplate(
        'projects/{project}/instances/{instance}/tables/{table}'
      ),
    };

    // Some of the methods on this service return "paged" results,
    // (e.g. 50 results at a time, with tokens to get subsequent
    // pages). Denote the keys used for pagination and results.
    this.descriptors.page = {
      listTables: new this._gaxModule.PageDescriptor(
        'pageToken',
        'nextPageToken',
        'tables'
      ),
      listAuthorizedViews: new this._gaxModule.PageDescriptor(
        'pageToken',
        'nextPageToken',
        'authorizedViews'
      ),
      listSnapshots: new this._gaxModule.PageDescriptor(
        'pageToken',
        'nextPageToken',
        'snapshots'
      ),
      listBackups: new this._gaxModule.PageDescriptor(
        'pageToken',
        'nextPageToken',
        'backups'
      ),
    };

    const protoFilesRoot = this._gaxModule.protobuf.Root.fromJSON(jsonProtos);
    // This API contains "long-running operations", which return a
    // an Operation object that allows for tracking of the operation,
    // rather than holding a request open.
    const lroOptions: GrpcClientOptions = {
      auth: this.auth,
      grpc: 'grpc' in this._gaxGrpc ? this._gaxGrpc.grpc : undefined,
    };
    if (opts.fallback) {
      lroOptions.protoJson = protoFilesRoot;
      lroOptions.httpRules = [
        {
          selector: 'google.longrunning.Operations.CancelOperation',
          post: '/v2/{name=operations/**}:cancel',
        },
        {
          selector: 'google.longrunning.Operations.DeleteOperation',
          delete: '/v2/{name=operations/**}',
        },
        {
          selector: 'google.longrunning.Operations.GetOperation',
          get: '/v2/{name=operations/**}',
        },
        {
          selector: 'google.longrunning.Operations.ListOperations',
          get: '/v2/{name=operations/projects/**}/operations',
        },
      ];
    }
    this.operationsClient = this._gaxModule
      .lro(lroOptions)
      .operationsClient(opts);
    const createTableFromSnapshotResponse = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.Table'
    ) as gax.protobuf.Type;
    const createTableFromSnapshotMetadata = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.CreateTableFromSnapshotMetadata'
    ) as gax.protobuf.Type;
    const updateTableResponse = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.Table'
    ) as gax.protobuf.Type;
    const updateTableMetadata = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.UpdateTableMetadata'
    ) as gax.protobuf.Type;
    const undeleteTableResponse = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.Table'
    ) as gax.protobuf.Type;
    const undeleteTableMetadata = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.UndeleteTableMetadata'
    ) as gax.protobuf.Type;
    const createAuthorizedViewResponse = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.AuthorizedView'
    ) as gax.protobuf.Type;
    const createAuthorizedViewMetadata = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.CreateAuthorizedViewMetadata'
    ) as gax.protobuf.Type;
    const updateAuthorizedViewResponse = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.AuthorizedView'
    ) as gax.protobuf.Type;
    const updateAuthorizedViewMetadata = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.UpdateAuthorizedViewMetadata'
    ) as gax.protobuf.Type;
    const snapshotTableResponse = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.Snapshot'
    ) as gax.protobuf.Type;
    const snapshotTableMetadata = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.SnapshotTableMetadata'
    ) as gax.protobuf.Type;
    const createBackupResponse = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.Backup'
    ) as gax.protobuf.Type;
    const createBackupMetadata = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.CreateBackupMetadata'
    ) as gax.protobuf.Type;
    const restoreTableResponse = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.Table'
    ) as gax.protobuf.Type;
    const restoreTableMetadata = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.RestoreTableMetadata'
    ) as gax.protobuf.Type;
    const copyBackupResponse = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.Backup'
    ) as gax.protobuf.Type;
    const copyBackupMetadata = protoFilesRoot.lookup(
      '.google.bigtable.admin.v2.CopyBackupMetadata'
    ) as gax.protobuf.Type;

    this.descriptors.longrunning = {
      createTableFromSnapshot: new this._gaxModule.LongrunningDescriptor(
        this.operationsClient,
        createTableFromSnapshotResponse.decode.bind(
          createTableFromSnapshotResponse
        ),
        createTableFromSnapshotMetadata.decode.bind(
          createTableFromSnapshotMetadata
        )
      ),
      updateTable: new this._gaxModule.LongrunningDescriptor(
        this.operationsClient,
        updateTableResponse.decode.bind(updateTableResponse),
        updateTableMetadata.decode.bind(updateTableMetadata)
      ),
      undeleteTable: new this._gaxModule.LongrunningDescriptor(
        this.operationsClient,
        undeleteTableResponse.decode.bind(undeleteTableResponse),
        undeleteTableMetadata.decode.bind(undeleteTableMetadata)
      ),
      createAuthorizedView: new this._gaxModule.LongrunningDescriptor(
        this.operationsClient,
        createAuthorizedViewResponse.decode.bind(createAuthorizedViewResponse),
        createAuthorizedViewMetadata.decode.bind(createAuthorizedViewMetadata)
      ),
      updateAuthorizedView: new this._gaxModule.LongrunningDescriptor(
        this.operationsClient,
        updateAuthorizedViewResponse.decode.bind(updateAuthorizedViewResponse),
        updateAuthorizedViewMetadata.decode.bind(updateAuthorizedViewMetadata)
      ),
      snapshotTable: new this._gaxModule.LongrunningDescriptor(
        this.operationsClient,
        snapshotTableResponse.decode.bind(snapshotTableResponse),
        snapshotTableMetadata.decode.bind(snapshotTableMetadata)
      ),
      createBackup: new this._gaxModule.LongrunningDescriptor(
        this.operationsClient,
        createBackupResponse.decode.bind(createBackupResponse),
        createBackupMetadata.decode.bind(createBackupMetadata)
      ),
      restoreTable: new this._gaxModule.LongrunningDescriptor(
        this.operationsClient,
        restoreTableResponse.decode.bind(restoreTableResponse),
        restoreTableMetadata.decode.bind(restoreTableMetadata)
      ),
      copyBackup: new this._gaxModule.LongrunningDescriptor(
        this.operationsClient,
        copyBackupResponse.decode.bind(copyBackupResponse),
        copyBackupMetadata.decode.bind(copyBackupMetadata)
      ),
    };

    // Put together the default options sent with requests.
    this._defaults = this._gaxGrpc.constructSettings(
      'google.bigtable.admin.v2.BigtableTableAdmin',
      gapicConfig as gax.ClientConfig,
      opts.clientConfig || {},
      {'x-goog-api-client': clientHeader.join(' ')}
    );

    // Set up a dictionary of "inner API calls"; the core implementation
    // of calling the API is handled in `google-gax`, with this code
    // merely providing the destination and request information.
    this.innerApiCalls = {};

    // Add a warn function to the client constructor so it can be easily tested.
    this.warn = this._gaxModule.warn;
  }

  /**
   * Initialize the client.
   * Performs asynchronous operations (such as authentication) and prepares the client.
   * This function will be called automatically when any class method is called for the
   * first time, but if you need to initialize it before calling an actual method,
   * feel free to call initialize() directly.
   *
   * You can await on this method if you want to make sure the client is initialized.
   *
   * @returns {Promise} A promise that resolves to an authenticated service stub.
   */
  initialize() {
    // If the client stub promise is already initialized, return immediately.
    if (this.bigtableTableAdminStub) {
      return this.bigtableTableAdminStub;
    }

    // Put together the "service stub" for
    // google.bigtable.admin.v2.BigtableTableAdmin.
    this.bigtableTableAdminStub = this._gaxGrpc.createStub(
      this._opts.fallback
        ? (this._protos as protobuf.Root).lookupService(
            'google.bigtable.admin.v2.BigtableTableAdmin'
          )
        : // eslint-disable-next-line @typescript-eslint/no-explicit-any
          (this._protos as any).google.bigtable.admin.v2.BigtableTableAdmin,
      this._opts,
      this._providedCustomServicePath
    ) as Promise<{[method: string]: Function}>;

    // Iterate over each of the methods that the service provides
    // and create an API call method for each.
    const bigtableTableAdminStubMethods = [
      'createTable',
      'createTableFromSnapshot',
      'listTables',
      'getTable',
      'updateTable',
      'deleteTable',
      'undeleteTable',
      'createAuthorizedView',
      'listAuthorizedViews',
      'getAuthorizedView',
      'updateAuthorizedView',
      'deleteAuthorizedView',
      'modifyColumnFamilies',
      'dropRowRange',
      'generateConsistencyToken',
      'checkConsistency',
      'snapshotTable',
      'getSnapshot',
      'listSnapshots',
      'deleteSnapshot',
      'createBackup',
      'getBackup',
      'updateBackup',
      'deleteBackup',
      'listBackups',
      'restoreTable',
      'copyBackup',
      'getIamPolicy',
      'setIamPolicy',
      'testIamPermissions',
    ];
    for (const methodName of bigtableTableAdminStubMethods) {
      const callPromise = this.bigtableTableAdminStub.then(
        stub =>
          (...args: Array<{}>) => {
            if (this._terminated) {
              return Promise.reject('The client has already been closed.');
            }
            const func = stub[methodName];
            return func.apply(stub, args);
          },
        (err: Error | null | undefined) => () => {
          throw err;
        }
      );

      const descriptor =
        this.descriptors.page[methodName] ||
        this.descriptors.longrunning[methodName] ||
        undefined;
      const apiCall = this._gaxModule.createApiCall(
        callPromise,
        this._defaults[methodName],
        descriptor,
        this._opts.fallback
      );

      this.innerApiCalls[methodName] = apiCall;
    }

    return this.bigtableTableAdminStub;
  }

  /**
   * The DNS address for this API service.
   * @deprecated Use the apiEndpoint method of the client instance.
   * @returns {string} The DNS address for this service.
   */
  static get servicePath() {
    if (
      typeof process === 'object' &&
      typeof process.emitWarning === 'function'
    ) {
      process.emitWarning(
        'Static servicePath is deprecated, please use the instance method instead.',
        'DeprecationWarning'
      );
    }
    return 'bigtableadmin.googleapis.com';
  }

  /**
   * The DNS address for this API service - same as servicePath.
   * @deprecated Use the apiEndpoint method of the client instance.
   * @returns {string} The DNS address for this service.
   */
  static get apiEndpoint() {
    if (
      typeof process === 'object' &&
      typeof process.emitWarning === 'function'
    ) {
      process.emitWarning(
        'Static apiEndpoint is deprecated, please use the instance method instead.',
        'DeprecationWarning'
      );
    }
    return 'bigtableadmin.googleapis.com';
  }

  /**
   * The DNS address for this API service.
   * @returns {string} The DNS address for this service.
   */
  get apiEndpoint() {
    return this._servicePath;
  }

  get universeDomain() {
    return this._universeDomain;
  }

  /**
   * The port for this API service.
   * @returns {number} The default port for this service.
   */
  static get port() {
    return 443;
  }

  /**
   * The scopes needed to make gRPC calls for every method defined
   * in this service.
   * @returns {string[]} List of default scopes.
   */
  static get scopes() {
    return [
      'https://www.googleapis.com/auth/bigtable.admin',
      'https://www.googleapis.com/auth/bigtable.admin.table',
      'https://www.googleapis.com/auth/cloud-bigtable.admin',
      'https://www.googleapis.com/auth/cloud-bigtable.admin.table',
      'https://www.googleapis.com/auth/cloud-platform',
      'https://www.googleapis.com/auth/cloud-platform.read-only',
    ];
  }

  getProjectId(): Promise<string>;
  getProjectId(callback: Callback<string, undefined, undefined>): void;
  /**
   * Return the project ID used by this class.
   * @returns {Promise} A promise that resolves to string containing the project ID.
   */
  getProjectId(
    callback?: Callback<string, undefined, undefined>
  ): Promise<string> | void {
    if (callback) {
      this.auth.getProjectId(callback);
      return;
    }
    return this.auth.getProjectId();
  }

  // -------------------
  // -- Service calls --
  // -------------------
  /**
   * Creates a new table in the specified instance.
   * The table can be created with a full set of initial column families,
   * specified in the request.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.parent
   *   Required. The unique name of the instance in which to create the table.
   *   Values are of the form `projects/{project}/instances/{instance}`.
   * @param {string} request.tableId
   *   Required. The name by which the new table should be referred to within the
   *   parent instance, e.g., `foobar` rather than `{parent}/tables/foobar`.
   *   Maximum 50 characters.
   * @param {google.bigtable.admin.v2.Table} request.table
   *   Required. The Table to create.
   * @param {number[]} request.initialSplits
   *   The optional list of row keys that will be used to initially split the
   *   table into several tablets (tablets are similar to HBase regions).
   *   Given two split keys, `s1` and `s2`, three tablets will be created,
   *   spanning the key ranges: `[, s1), [s1, s2), [s2, )`.
   *
   *   Example:
   *
   *   * Row keys := `["a", "apple", "custom", "customer_1", "customer_2",`
   *                  `"other", "zz"]`
   *   * initial_split_keys := `["apple", "customer_1", "customer_2", "other"]`
   *   * Key assignment:
   *       - Tablet 1 `[, apple)                => {"a"}.`
   *       - Tablet 2 `[apple, customer_1)      => {"apple", "custom"}.`
   *       - Tablet 3 `[customer_1, customer_2) => {"customer_1"}.`
   *       - Tablet 4 `[customer_2, other)      => {"customer_2"}.`
   *       - Tablet 5 `[other, )                => {"other", "zz"}.`
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.bigtable.admin.v2.Table|Table}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.create_table.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_CreateTable_async
   */
  createTable(
    request?: protos.google.bigtable.admin.v2.ICreateTableRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.bigtable.admin.v2.ITable,
      protos.google.bigtable.admin.v2.ICreateTableRequest | undefined,
      {} | undefined,
    ]
  >;
  createTable(
    request: protos.google.bigtable.admin.v2.ICreateTableRequest,
    options: CallOptions,
    callback: Callback<
      protos.google.bigtable.admin.v2.ITable,
      protos.google.bigtable.admin.v2.ICreateTableRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  createTable(
    request: protos.google.bigtable.admin.v2.ICreateTableRequest,
    callback: Callback<
      protos.google.bigtable.admin.v2.ITable,
      protos.google.bigtable.admin.v2.ICreateTableRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  createTable(
    request?: protos.google.bigtable.admin.v2.ICreateTableRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.bigtable.admin.v2.ITable,
          | protos.google.bigtable.admin.v2.ICreateTableRequest
          | null
          | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.bigtable.admin.v2.ITable,
      protos.google.bigtable.admin.v2.ICreateTableRequest | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.bigtable.admin.v2.ITable,
      protos.google.bigtable.admin.v2.ICreateTableRequest | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        parent: request.parent ?? '',
      });
    this.initialize();
    return this.innerApiCalls.createTable(request, options, callback);
  }
  /**
   * Gets metadata information about the specified table.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.name
   *   Required. The unique name of the requested table.
   *   Values are of the form
   *   `projects/{project}/instances/{instance}/tables/{table}`.
   * @param {google.bigtable.admin.v2.Table.View} request.view
   *   The view to be applied to the returned table's fields.
   *   Defaults to `SCHEMA_VIEW` if unspecified.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.bigtable.admin.v2.Table|Table}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.get_table.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_GetTable_async
   */
  getTable(
    request?: protos.google.bigtable.admin.v2.IGetTableRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.bigtable.admin.v2.ITable,
      protos.google.bigtable.admin.v2.IGetTableRequest | undefined,
      {} | undefined,
    ]
  >;
  getTable(
    request: protos.google.bigtable.admin.v2.IGetTableRequest,
    options: CallOptions,
    callback: Callback<
      protos.google.bigtable.admin.v2.ITable,
      protos.google.bigtable.admin.v2.IGetTableRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  getTable(
    request: protos.google.bigtable.admin.v2.IGetTableRequest,
    callback: Callback<
      protos.google.bigtable.admin.v2.ITable,
      protos.google.bigtable.admin.v2.IGetTableRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  getTable(
    request?: protos.google.bigtable.admin.v2.IGetTableRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.bigtable.admin.v2.ITable,
          protos.google.bigtable.admin.v2.IGetTableRequest | null | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.bigtable.admin.v2.ITable,
      protos.google.bigtable.admin.v2.IGetTableRequest | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.bigtable.admin.v2.ITable,
      protos.google.bigtable.admin.v2.IGetTableRequest | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        name: request.name ?? '',
      });
    this.initialize();
    return this.innerApiCalls.getTable(request, options, callback);
  }
  /**
   * Permanently deletes a specified table and all of its data.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.name
   *   Required. The unique name of the table to be deleted.
   *   Values are of the form
   *   `projects/{project}/instances/{instance}/tables/{table}`.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.protobuf.Empty|Empty}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.delete_table.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_DeleteTable_async
   */
  deleteTable(
    request?: protos.google.bigtable.admin.v2.IDeleteTableRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDeleteTableRequest | undefined,
      {} | undefined,
    ]
  >;
  deleteTable(
    request: protos.google.bigtable.admin.v2.IDeleteTableRequest,
    options: CallOptions,
    callback: Callback<
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDeleteTableRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  deleteTable(
    request: protos.google.bigtable.admin.v2.IDeleteTableRequest,
    callback: Callback<
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDeleteTableRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  deleteTable(
    request?: protos.google.bigtable.admin.v2.IDeleteTableRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.protobuf.IEmpty,
          | protos.google.bigtable.admin.v2.IDeleteTableRequest
          | null
          | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDeleteTableRequest | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDeleteTableRequest | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        name: request.name ?? '',
      });
    this.initialize();
    return this.innerApiCalls.deleteTable(request, options, callback);
  }
  /**
   * Gets information from a specified AuthorizedView.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.name
   *   Required. The unique name of the requested AuthorizedView.
   *   Values are of the form
   *   `projects/{project}/instances/{instance}/tables/{table}/authorizedViews/{authorized_view}`.
   * @param {google.bigtable.admin.v2.AuthorizedView.ResponseView} [request.view]
   *   Optional. The resource_view to be applied to the returned AuthorizedView's
   *   fields. Default to BASIC.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.bigtable.admin.v2.AuthorizedView|AuthorizedView}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.get_authorized_view.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_GetAuthorizedView_async
   */
  getAuthorizedView(
    request?: protos.google.bigtable.admin.v2.IGetAuthorizedViewRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.bigtable.admin.v2.IAuthorizedView,
      protos.google.bigtable.admin.v2.IGetAuthorizedViewRequest | undefined,
      {} | undefined,
    ]
  >;
  getAuthorizedView(
    request: protos.google.bigtable.admin.v2.IGetAuthorizedViewRequest,
    options: CallOptions,
    callback: Callback<
      protos.google.bigtable.admin.v2.IAuthorizedView,
      | protos.google.bigtable.admin.v2.IGetAuthorizedViewRequest
      | null
      | undefined,
      {} | null | undefined
    >
  ): void;
  getAuthorizedView(
    request: protos.google.bigtable.admin.v2.IGetAuthorizedViewRequest,
    callback: Callback<
      protos.google.bigtable.admin.v2.IAuthorizedView,
      | protos.google.bigtable.admin.v2.IGetAuthorizedViewRequest
      | null
      | undefined,
      {} | null | undefined
    >
  ): void;
  getAuthorizedView(
    request?: protos.google.bigtable.admin.v2.IGetAuthorizedViewRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.bigtable.admin.v2.IAuthorizedView,
          | protos.google.bigtable.admin.v2.IGetAuthorizedViewRequest
          | null
          | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.bigtable.admin.v2.IAuthorizedView,
      | protos.google.bigtable.admin.v2.IGetAuthorizedViewRequest
      | null
      | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.bigtable.admin.v2.IAuthorizedView,
      protos.google.bigtable.admin.v2.IGetAuthorizedViewRequest | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        name: request.name ?? '',
      });
    this.initialize();
    return this.innerApiCalls.getAuthorizedView(request, options, callback);
  }
  /**
   * Permanently deletes a specified AuthorizedView.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.name
   *   Required. The unique name of the AuthorizedView to be deleted.
   *   Values are of the form
   *   `projects/{project}/instances/{instance}/tables/{table}/authorizedViews/{authorized_view}`.
   * @param {string} [request.etag]
   *   Optional. The current etag of the AuthorizedView.
   *   If an etag is provided and does not match the current etag of the
   *   AuthorizedView, deletion will be blocked and an ABORTED error will be
   *   returned.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.protobuf.Empty|Empty}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.delete_authorized_view.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_DeleteAuthorizedView_async
   */
  deleteAuthorizedView(
    request?: protos.google.bigtable.admin.v2.IDeleteAuthorizedViewRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDeleteAuthorizedViewRequest | undefined,
      {} | undefined,
    ]
  >;
  deleteAuthorizedView(
    request: protos.google.bigtable.admin.v2.IDeleteAuthorizedViewRequest,
    options: CallOptions,
    callback: Callback<
      protos.google.protobuf.IEmpty,
      | protos.google.bigtable.admin.v2.IDeleteAuthorizedViewRequest
      | null
      | undefined,
      {} | null | undefined
    >
  ): void;
  deleteAuthorizedView(
    request: protos.google.bigtable.admin.v2.IDeleteAuthorizedViewRequest,
    callback: Callback<
      protos.google.protobuf.IEmpty,
      | protos.google.bigtable.admin.v2.IDeleteAuthorizedViewRequest
      | null
      | undefined,
      {} | null | undefined
    >
  ): void;
  deleteAuthorizedView(
    request?: protos.google.bigtable.admin.v2.IDeleteAuthorizedViewRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.protobuf.IEmpty,
          | protos.google.bigtable.admin.v2.IDeleteAuthorizedViewRequest
          | null
          | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.protobuf.IEmpty,
      | protos.google.bigtable.admin.v2.IDeleteAuthorizedViewRequest
      | null
      | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDeleteAuthorizedViewRequest | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        name: request.name ?? '',
      });
    this.initialize();
    return this.innerApiCalls.deleteAuthorizedView(request, options, callback);
  }
  /**
   * Performs a series of column family modifications on the specified table.
   * Either all or none of the modifications will occur before this method
   * returns, but data requests received prior to that point may see a table
   * where only some modifications have taken effect.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.name
   *   Required. The unique name of the table whose families should be modified.
   *   Values are of the form
   *   `projects/{project}/instances/{instance}/tables/{table}`.
   * @param {number[]} request.modifications
   *   Required. Modifications to be atomically applied to the specified table's
   *   families. Entries are applied in order, meaning that earlier modifications
   *   can be masked by later ones (in the case of repeated updates to the same
   *   family, for example).
   * @param {boolean} [request.ignoreWarnings]
   *   Optional. If true, ignore safety checks when modifying the column families.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.bigtable.admin.v2.Table|Table}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.modify_column_families.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_ModifyColumnFamilies_async
   */
  modifyColumnFamilies(
    request?: protos.google.bigtable.admin.v2.IModifyColumnFamiliesRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.bigtable.admin.v2.ITable,
      protos.google.bigtable.admin.v2.IModifyColumnFamiliesRequest | undefined,
      {} | undefined,
    ]
  >;
  modifyColumnFamilies(
    request: protos.google.bigtable.admin.v2.IModifyColumnFamiliesRequest,
    options: CallOptions,
    callback: Callback<
      protos.google.bigtable.admin.v2.ITable,
      | protos.google.bigtable.admin.v2.IModifyColumnFamiliesRequest
      | null
      | undefined,
      {} | null | undefined
    >
  ): void;
  modifyColumnFamilies(
    request: protos.google.bigtable.admin.v2.IModifyColumnFamiliesRequest,
    callback: Callback<
      protos.google.bigtable.admin.v2.ITable,
      | protos.google.bigtable.admin.v2.IModifyColumnFamiliesRequest
      | null
      | undefined,
      {} | null | undefined
    >
  ): void;
  modifyColumnFamilies(
    request?: protos.google.bigtable.admin.v2.IModifyColumnFamiliesRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.bigtable.admin.v2.ITable,
          | protos.google.bigtable.admin.v2.IModifyColumnFamiliesRequest
          | null
          | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.bigtable.admin.v2.ITable,
      | protos.google.bigtable.admin.v2.IModifyColumnFamiliesRequest
      | null
      | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.bigtable.admin.v2.ITable,
      protos.google.bigtable.admin.v2.IModifyColumnFamiliesRequest | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        name: request.name ?? '',
      });
    this.initialize();
    return this.innerApiCalls.modifyColumnFamilies(request, options, callback);
  }
  /**
   * Permanently drop/delete a row range from a specified table. The request can
   * specify whether to delete all rows in a table, or only those that match a
   * particular prefix.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.name
   *   Required. The unique name of the table on which to drop a range of rows.
   *   Values are of the form
   *   `projects/{project}/instances/{instance}/tables/{table}`.
   * @param {Buffer} request.rowKeyPrefix
   *   Delete all rows that start with this row key prefix. Prefix cannot be
   *   zero length.
   * @param {boolean} request.deleteAllDataFromTable
   *   Delete all rows in the table. Setting this to false is a no-op.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.protobuf.Empty|Empty}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.drop_row_range.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_DropRowRange_async
   */
  dropRowRange(
    request?: protos.google.bigtable.admin.v2.IDropRowRangeRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDropRowRangeRequest | undefined,
      {} | undefined,
    ]
  >;
  dropRowRange(
    request: protos.google.bigtable.admin.v2.IDropRowRangeRequest,
    options: CallOptions,
    callback: Callback<
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDropRowRangeRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  dropRowRange(
    request: protos.google.bigtable.admin.v2.IDropRowRangeRequest,
    callback: Callback<
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDropRowRangeRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  dropRowRange(
    request?: protos.google.bigtable.admin.v2.IDropRowRangeRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.protobuf.IEmpty,
          | protos.google.bigtable.admin.v2.IDropRowRangeRequest
          | null
          | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDropRowRangeRequest | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDropRowRangeRequest | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        name: request.name ?? '',
      });
    this.initialize();
    return this.innerApiCalls.dropRowRange(request, options, callback);
  }
  /**
   * Generates a consistency token for a Table, which can be used in
   * CheckConsistency to check whether mutations to the table that finished
   * before this call started have been replicated. The tokens will be available
   * for 90 days.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.name
   *   Required. The unique name of the Table for which to create a consistency
   *   token. Values are of the form
   *   `projects/{project}/instances/{instance}/tables/{table}`.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.bigtable.admin.v2.GenerateConsistencyTokenResponse|GenerateConsistencyTokenResponse}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.generate_consistency_token.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_GenerateConsistencyToken_async
   */
  generateConsistencyToken(
    request?: protos.google.bigtable.admin.v2.IGenerateConsistencyTokenRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.bigtable.admin.v2.IGenerateConsistencyTokenResponse,
      (
        | protos.google.bigtable.admin.v2.IGenerateConsistencyTokenRequest
        | undefined
      ),
      {} | undefined,
    ]
  >;
  generateConsistencyToken(
    request: protos.google.bigtable.admin.v2.IGenerateConsistencyTokenRequest,
    options: CallOptions,
    callback: Callback<
      protos.google.bigtable.admin.v2.IGenerateConsistencyTokenResponse,
      | protos.google.bigtable.admin.v2.IGenerateConsistencyTokenRequest
      | null
      | undefined,
      {} | null | undefined
    >
  ): void;
  generateConsistencyToken(
    request: protos.google.bigtable.admin.v2.IGenerateConsistencyTokenRequest,
    callback: Callback<
      protos.google.bigtable.admin.v2.IGenerateConsistencyTokenResponse,
      | protos.google.bigtable.admin.v2.IGenerateConsistencyTokenRequest
      | null
      | undefined,
      {} | null | undefined
    >
  ): void;
  generateConsistencyToken(
    request?: protos.google.bigtable.admin.v2.IGenerateConsistencyTokenRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.bigtable.admin.v2.IGenerateConsistencyTokenResponse,
          | protos.google.bigtable.admin.v2.IGenerateConsistencyTokenRequest
          | null
          | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.bigtable.admin.v2.IGenerateConsistencyTokenResponse,
      | protos.google.bigtable.admin.v2.IGenerateConsistencyTokenRequest
      | null
      | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.bigtable.admin.v2.IGenerateConsistencyTokenResponse,
      (
        | protos.google.bigtable.admin.v2.IGenerateConsistencyTokenRequest
        | undefined
      ),
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        name: request.name ?? '',
      });
    this.initialize();
    return this.innerApiCalls.generateConsistencyToken(
      request,
      options,
      callback
    );
  }
  /**
   * Checks replication consistency based on a consistency token, that is, if
   * replication has caught up based on the conditions specified in the token
   * and the check request.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.name
   *   Required. The unique name of the Table for which to check replication
   *   consistency. Values are of the form
   *   `projects/{project}/instances/{instance}/tables/{table}`.
   * @param {string} request.consistencyToken
   *   Required. The token created using GenerateConsistencyToken for the Table.
   * @param {google.bigtable.admin.v2.StandardReadRemoteWrites} request.standardReadRemoteWrites
   *   Checks that reads using an app profile with `StandardIsolation` can
   *   see all writes committed before the token was created, even if the
   *   read and write target different clusters.
   * @param {google.bigtable.admin.v2.DataBoostReadLocalWrites} request.dataBoostReadLocalWrites
   *   Checks that reads using an app profile with `DataBoostIsolationReadOnly`
   *   can see all writes committed before the token was created, but only if
   *   the read and write target the same cluster.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.bigtable.admin.v2.CheckConsistencyResponse|CheckConsistencyResponse}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.check_consistency.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_CheckConsistency_async
   */
  checkConsistency(
    request?: protos.google.bigtable.admin.v2.ICheckConsistencyRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.bigtable.admin.v2.ICheckConsistencyResponse,
      protos.google.bigtable.admin.v2.ICheckConsistencyRequest | undefined,
      {} | undefined,
    ]
  >;
  checkConsistency(
    request: protos.google.bigtable.admin.v2.ICheckConsistencyRequest,
    options: CallOptions,
    callback: Callback<
      protos.google.bigtable.admin.v2.ICheckConsistencyResponse,
      | protos.google.bigtable.admin.v2.ICheckConsistencyRequest
      | null
      | undefined,
      {} | null | undefined
    >
  ): void;
  checkConsistency(
    request: protos.google.bigtable.admin.v2.ICheckConsistencyRequest,
    callback: Callback<
      protos.google.bigtable.admin.v2.ICheckConsistencyResponse,
      | protos.google.bigtable.admin.v2.ICheckConsistencyRequest
      | null
      | undefined,
      {} | null | undefined
    >
  ): void;
  checkConsistency(
    request?: protos.google.bigtable.admin.v2.ICheckConsistencyRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.bigtable.admin.v2.ICheckConsistencyResponse,
          | protos.google.bigtable.admin.v2.ICheckConsistencyRequest
          | null
          | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.bigtable.admin.v2.ICheckConsistencyResponse,
      | protos.google.bigtable.admin.v2.ICheckConsistencyRequest
      | null
      | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.bigtable.admin.v2.ICheckConsistencyResponse,
      protos.google.bigtable.admin.v2.ICheckConsistencyRequest | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        name: request.name ?? '',
      });
    this.initialize();
    return this.innerApiCalls.checkConsistency(request, options, callback);
  }
  /**
   * Gets metadata information about the specified snapshot.
   *
   * Note: This is a private alpha release of Cloud Bigtable snapshots. This
   * feature is not currently available to most Cloud Bigtable customers. This
   * feature might be changed in backward-incompatible ways and is not
   * recommended for production use. It is not subject to any SLA or deprecation
   * policy.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.name
   *   Required. The unique name of the requested snapshot.
   *   Values are of the form
   *   `projects/{project}/instances/{instance}/clusters/{cluster}/snapshots/{snapshot}`.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.bigtable.admin.v2.Snapshot|Snapshot}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.get_snapshot.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_GetSnapshot_async
   */
  getSnapshot(
    request?: protos.google.bigtable.admin.v2.IGetSnapshotRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.bigtable.admin.v2.ISnapshot,
      protos.google.bigtable.admin.v2.IGetSnapshotRequest | undefined,
      {} | undefined,
    ]
  >;
  getSnapshot(
    request: protos.google.bigtable.admin.v2.IGetSnapshotRequest,
    options: CallOptions,
    callback: Callback<
      protos.google.bigtable.admin.v2.ISnapshot,
      protos.google.bigtable.admin.v2.IGetSnapshotRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  getSnapshot(
    request: protos.google.bigtable.admin.v2.IGetSnapshotRequest,
    callback: Callback<
      protos.google.bigtable.admin.v2.ISnapshot,
      protos.google.bigtable.admin.v2.IGetSnapshotRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  getSnapshot(
    request?: protos.google.bigtable.admin.v2.IGetSnapshotRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.bigtable.admin.v2.ISnapshot,
          | protos.google.bigtable.admin.v2.IGetSnapshotRequest
          | null
          | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.bigtable.admin.v2.ISnapshot,
      protos.google.bigtable.admin.v2.IGetSnapshotRequest | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.bigtable.admin.v2.ISnapshot,
      protos.google.bigtable.admin.v2.IGetSnapshotRequest | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        name: request.name ?? '',
      });
    this.initialize();
    return this.innerApiCalls.getSnapshot(request, options, callback);
  }
  /**
   * Permanently deletes the specified snapshot.
   *
   * Note: This is a private alpha release of Cloud Bigtable snapshots. This
   * feature is not currently available to most Cloud Bigtable customers. This
   * feature might be changed in backward-incompatible ways and is not
   * recommended for production use. It is not subject to any SLA or deprecation
   * policy.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.name
   *   Required. The unique name of the snapshot to be deleted.
   *   Values are of the form
   *   `projects/{project}/instances/{instance}/clusters/{cluster}/snapshots/{snapshot}`.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.protobuf.Empty|Empty}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.delete_snapshot.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_DeleteSnapshot_async
   */
  deleteSnapshot(
    request?: protos.google.bigtable.admin.v2.IDeleteSnapshotRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDeleteSnapshotRequest | undefined,
      {} | undefined,
    ]
  >;
  deleteSnapshot(
    request: protos.google.bigtable.admin.v2.IDeleteSnapshotRequest,
    options: CallOptions,
    callback: Callback<
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDeleteSnapshotRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  deleteSnapshot(
    request: protos.google.bigtable.admin.v2.IDeleteSnapshotRequest,
    callback: Callback<
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDeleteSnapshotRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  deleteSnapshot(
    request?: protos.google.bigtable.admin.v2.IDeleteSnapshotRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.protobuf.IEmpty,
          | protos.google.bigtable.admin.v2.IDeleteSnapshotRequest
          | null
          | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDeleteSnapshotRequest | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDeleteSnapshotRequest | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        name: request.name ?? '',
      });
    this.initialize();
    return this.innerApiCalls.deleteSnapshot(request, options, callback);
  }
  /**
   * Gets metadata on a pending or completed Cloud Bigtable Backup.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.name
   *   Required. Name of the backup.
   *   Values are of the form
   *   `projects/{project}/instances/{instance}/clusters/{cluster}/backups/{backup}`.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.bigtable.admin.v2.Backup|Backup}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.get_backup.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_GetBackup_async
   */
  getBackup(
    request?: protos.google.bigtable.admin.v2.IGetBackupRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.bigtable.admin.v2.IBackup,
      protos.google.bigtable.admin.v2.IGetBackupRequest | undefined,
      {} | undefined,
    ]
  >;
  getBackup(
    request: protos.google.bigtable.admin.v2.IGetBackupRequest,
    options: CallOptions,
    callback: Callback<
      protos.google.bigtable.admin.v2.IBackup,
      protos.google.bigtable.admin.v2.IGetBackupRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  getBackup(
    request: protos.google.bigtable.admin.v2.IGetBackupRequest,
    callback: Callback<
      protos.google.bigtable.admin.v2.IBackup,
      protos.google.bigtable.admin.v2.IGetBackupRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  getBackup(
    request?: protos.google.bigtable.admin.v2.IGetBackupRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.bigtable.admin.v2.IBackup,
          protos.google.bigtable.admin.v2.IGetBackupRequest | null | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.bigtable.admin.v2.IBackup,
      protos.google.bigtable.admin.v2.IGetBackupRequest | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.bigtable.admin.v2.IBackup,
      protos.google.bigtable.admin.v2.IGetBackupRequest | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        name: request.name ?? '',
      });
    this.initialize();
    return this.innerApiCalls.getBackup(request, options, callback);
  }
  /**
   * Updates a pending or completed Cloud Bigtable Backup.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {google.bigtable.admin.v2.Backup} request.backup
   *   Required. The backup to update. `backup.name`, and the fields to be updated
   *   as specified by `update_mask` are required. Other fields are ignored.
   *   Update is only supported for the following fields:
   *
   *    * `backup.expire_time`.
   * @param {google.protobuf.FieldMask} request.updateMask
   *   Required. A mask specifying which fields (e.g. `expire_time`) in the
   *   Backup resource should be updated. This mask is relative to the Backup
   *   resource, not to the request message. The field mask must always be
   *   specified; this prevents any future fields from being erased accidentally
   *   by clients that do not know about them.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.bigtable.admin.v2.Backup|Backup}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.update_backup.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_UpdateBackup_async
   */
  updateBackup(
    request?: protos.google.bigtable.admin.v2.IUpdateBackupRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.bigtable.admin.v2.IBackup,
      protos.google.bigtable.admin.v2.IUpdateBackupRequest | undefined,
      {} | undefined,
    ]
  >;
  updateBackup(
    request: protos.google.bigtable.admin.v2.IUpdateBackupRequest,
    options: CallOptions,
    callback: Callback<
      protos.google.bigtable.admin.v2.IBackup,
      protos.google.bigtable.admin.v2.IUpdateBackupRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  updateBackup(
    request: protos.google.bigtable.admin.v2.IUpdateBackupRequest,
    callback: Callback<
      protos.google.bigtable.admin.v2.IBackup,
      protos.google.bigtable.admin.v2.IUpdateBackupRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  updateBackup(
    request?: protos.google.bigtable.admin.v2.IUpdateBackupRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.bigtable.admin.v2.IBackup,
          | protos.google.bigtable.admin.v2.IUpdateBackupRequest
          | null
          | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.bigtable.admin.v2.IBackup,
      protos.google.bigtable.admin.v2.IUpdateBackupRequest | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.bigtable.admin.v2.IBackup,
      protos.google.bigtable.admin.v2.IUpdateBackupRequest | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        'backup.name': request.backup!.name ?? '',
      });
    this.initialize();
    return this.innerApiCalls.updateBackup(request, options, callback);
  }
  /**
   * Deletes a pending or completed Cloud Bigtable backup.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.name
   *   Required. Name of the backup to delete.
   *   Values are of the form
   *   `projects/{project}/instances/{instance}/clusters/{cluster}/backups/{backup}`.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.protobuf.Empty|Empty}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.delete_backup.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_DeleteBackup_async
   */
  deleteBackup(
    request?: protos.google.bigtable.admin.v2.IDeleteBackupRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDeleteBackupRequest | undefined,
      {} | undefined,
    ]
  >;
  deleteBackup(
    request: protos.google.bigtable.admin.v2.IDeleteBackupRequest,
    options: CallOptions,
    callback: Callback<
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDeleteBackupRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  deleteBackup(
    request: protos.google.bigtable.admin.v2.IDeleteBackupRequest,
    callback: Callback<
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDeleteBackupRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  deleteBackup(
    request?: protos.google.bigtable.admin.v2.IDeleteBackupRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.protobuf.IEmpty,
          | protos.google.bigtable.admin.v2.IDeleteBackupRequest
          | null
          | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDeleteBackupRequest | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.protobuf.IEmpty,
      protos.google.bigtable.admin.v2.IDeleteBackupRequest | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        name: request.name ?? '',
      });
    this.initialize();
    return this.innerApiCalls.deleteBackup(request, options, callback);
  }
  /**
   * Gets the access control policy for a Table or Backup resource.
   * Returns an empty policy if the resource exists but does not have a policy
   * set.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.resource
   *   REQUIRED: The resource for which the policy is being requested.
   *   See the operation documentation for the appropriate value for this field.
   * @param {google.iam.v1.GetPolicyOptions} request.options
   *   OPTIONAL: A `GetPolicyOptions` object for specifying options to
   *   `GetIamPolicy`.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.iam.v1.Policy|Policy}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.get_iam_policy.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_GetIamPolicy_async
   */
  getIamPolicy(
    request?: protos.google.iam.v1.IGetIamPolicyRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.iam.v1.IPolicy,
      protos.google.iam.v1.IGetIamPolicyRequest | undefined,
      {} | undefined,
    ]
  >;
  getIamPolicy(
    request: protos.google.iam.v1.IGetIamPolicyRequest,
    options: CallOptions,
    callback: Callback<
      protos.google.iam.v1.IPolicy,
      protos.google.iam.v1.IGetIamPolicyRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  getIamPolicy(
    request: protos.google.iam.v1.IGetIamPolicyRequest,
    callback: Callback<
      protos.google.iam.v1.IPolicy,
      protos.google.iam.v1.IGetIamPolicyRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  getIamPolicy(
    request?: protos.google.iam.v1.IGetIamPolicyRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.iam.v1.IPolicy,
          protos.google.iam.v1.IGetIamPolicyRequest | null | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.iam.v1.IPolicy,
      protos.google.iam.v1.IGetIamPolicyRequest | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.iam.v1.IPolicy,
      protos.google.iam.v1.IGetIamPolicyRequest | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        resource: request.resource ?? '',
      });
    this.initialize();
    return this.innerApiCalls.getIamPolicy(request, options, callback);
  }
  /**
   * Sets the access control policy on a Table or Backup resource.
   * Replaces any existing policy.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.resource
   *   REQUIRED: The resource for which the policy is being specified.
   *   See the operation documentation for the appropriate value for this field.
   * @param {google.iam.v1.Policy} request.policy
   *   REQUIRED: The complete policy to be applied to the `resource`. The size of
   *   the policy is limited to a few 10s of KB. An empty policy is a
   *   valid policy but certain Cloud Platform services (such as Projects)
   *   might reject them.
   * @param {google.protobuf.FieldMask} request.updateMask
   *   OPTIONAL: A FieldMask specifying which fields of the policy to modify. Only
   *   the fields in the mask will be modified. If no mask is provided, the
   *   following default mask is used:
   *
   *   `paths: "bindings, etag"`
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.iam.v1.Policy|Policy}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.set_iam_policy.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_SetIamPolicy_async
   */
  setIamPolicy(
    request?: protos.google.iam.v1.ISetIamPolicyRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.iam.v1.IPolicy,
      protos.google.iam.v1.ISetIamPolicyRequest | undefined,
      {} | undefined,
    ]
  >;
  setIamPolicy(
    request: protos.google.iam.v1.ISetIamPolicyRequest,
    options: CallOptions,
    callback: Callback<
      protos.google.iam.v1.IPolicy,
      protos.google.iam.v1.ISetIamPolicyRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  setIamPolicy(
    request: protos.google.iam.v1.ISetIamPolicyRequest,
    callback: Callback<
      protos.google.iam.v1.IPolicy,
      protos.google.iam.v1.ISetIamPolicyRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  setIamPolicy(
    request?: protos.google.iam.v1.ISetIamPolicyRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.iam.v1.IPolicy,
          protos.google.iam.v1.ISetIamPolicyRequest | null | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.iam.v1.IPolicy,
      protos.google.iam.v1.ISetIamPolicyRequest | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.iam.v1.IPolicy,
      protos.google.iam.v1.ISetIamPolicyRequest | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        resource: request.resource ?? '',
      });
    this.initialize();
    return this.innerApiCalls.setIamPolicy(request, options, callback);
  }
  /**
   * Returns permissions that the caller has on the specified Table or Backup
   * resource.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.resource
   *   REQUIRED: The resource for which the policy detail is being requested.
   *   See the operation documentation for the appropriate value for this field.
   * @param {string[]} request.permissions
   *   The set of permissions to check for the `resource`. Permissions with
   *   wildcards (such as '*' or 'storage.*') are not allowed. For more
   *   information see
   *   [IAM Overview](https://cloud.google.com/iam/docs/overview#permissions).
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing {@link protos.google.iam.v1.TestIamPermissionsResponse|TestIamPermissionsResponse}.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#regular-methods | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.test_iam_permissions.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_TestIamPermissions_async
   */
  testIamPermissions(
    request?: protos.google.iam.v1.ITestIamPermissionsRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.iam.v1.ITestIamPermissionsResponse,
      protos.google.iam.v1.ITestIamPermissionsRequest | undefined,
      {} | undefined,
    ]
  >;
  testIamPermissions(
    request: protos.google.iam.v1.ITestIamPermissionsRequest,
    options: CallOptions,
    callback: Callback<
      protos.google.iam.v1.ITestIamPermissionsResponse,
      protos.google.iam.v1.ITestIamPermissionsRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  testIamPermissions(
    request: protos.google.iam.v1.ITestIamPermissionsRequest,
    callback: Callback<
      protos.google.iam.v1.ITestIamPermissionsResponse,
      protos.google.iam.v1.ITestIamPermissionsRequest | null | undefined,
      {} | null | undefined
    >
  ): void;
  testIamPermissions(
    request?: protos.google.iam.v1.ITestIamPermissionsRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          protos.google.iam.v1.ITestIamPermissionsResponse,
          protos.google.iam.v1.ITestIamPermissionsRequest | null | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      protos.google.iam.v1.ITestIamPermissionsResponse,
      protos.google.iam.v1.ITestIamPermissionsRequest | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      protos.google.iam.v1.ITestIamPermissionsResponse,
      protos.google.iam.v1.ITestIamPermissionsRequest | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        resource: request.resource ?? '',
      });
    this.initialize();
    return this.innerApiCalls.testIamPermissions(request, options, callback);
  }

  /**
   * Creates a new table from the specified snapshot. The target table must
   * not exist. The snapshot and the table must be in the same instance.
   *
   * Note: This is a private alpha release of Cloud Bigtable snapshots. This
   * feature is not currently available to most Cloud Bigtable customers. This
   * feature might be changed in backward-incompatible ways and is not
   * recommended for production use. It is not subject to any SLA or deprecation
   * policy.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.parent
   *   Required. The unique name of the instance in which to create the table.
   *   Values are of the form `projects/{project}/instances/{instance}`.
   * @param {string} request.tableId
   *   Required. The name by which the new table should be referred to within the
   *   parent instance, e.g., `foobar` rather than `{parent}/tables/foobar`.
   * @param {string} request.sourceSnapshot
   *   Required. The unique name of the snapshot from which to restore the table.
   *   The snapshot and the table must be in the same instance. Values are of the
   *   form
   *   `projects/{project}/instances/{instance}/clusters/{cluster}/snapshots/{snapshot}`.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing
   *   a long running operation. Its `promise()` method returns a promise
   *   you can `await` for.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.create_table_from_snapshot.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_CreateTableFromSnapshot_async
   */
  createTableFromSnapshot(
    request?: protos.google.bigtable.admin.v2.ICreateTableFromSnapshotRequest,
    options?: CallOptions
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.ITable,
        protos.google.bigtable.admin.v2.ICreateTableFromSnapshotMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  >;
  createTableFromSnapshot(
    request: protos.google.bigtable.admin.v2.ICreateTableFromSnapshotRequest,
    options: CallOptions,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.ITable,
        protos.google.bigtable.admin.v2.ICreateTableFromSnapshotMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  createTableFromSnapshot(
    request: protos.google.bigtable.admin.v2.ICreateTableFromSnapshotRequest,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.ITable,
        protos.google.bigtable.admin.v2.ICreateTableFromSnapshotMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  createTableFromSnapshot(
    request?: protos.google.bigtable.admin.v2.ICreateTableFromSnapshotRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          LROperation<
            protos.google.bigtable.admin.v2.ITable,
            protos.google.bigtable.admin.v2.ICreateTableFromSnapshotMetadata
          >,
          protos.google.longrunning.IOperation | null | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.ITable,
        protos.google.bigtable.admin.v2.ICreateTableFromSnapshotMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.ITable,
        protos.google.bigtable.admin.v2.ICreateTableFromSnapshotMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        parent: request.parent ?? '',
      });
    this.initialize();
    return this.innerApiCalls.createTableFromSnapshot(
      request,
      options,
      callback
    );
  }
  /**
   * Check the status of the long running operation returned by `createTableFromSnapshot()`.
   * @param {String} name
   *   The operation name that will be passed.
   * @returns {Promise} - The promise which resolves to an object.
   *   The decoded operation object has result and metadata field to get information from.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.create_table_from_snapshot.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_CreateTableFromSnapshot_async
   */
  async checkCreateTableFromSnapshotProgress(
    name: string
  ): Promise<
    LROperation<
      protos.google.bigtable.admin.v2.Table,
      protos.google.bigtable.admin.v2.CreateTableFromSnapshotMetadata
    >
  > {
    const request =
      new this._gaxModule.operationsProtos.google.longrunning.GetOperationRequest(
        {name}
      );
    const [operation] = await this.operationsClient.getOperation(request);
    const decodeOperation = new this._gaxModule.Operation(
      operation,
      this.descriptors.longrunning.createTableFromSnapshot,
      this._gaxModule.createDefaultBackoffSettings()
    );
    return decodeOperation as LROperation<
      protos.google.bigtable.admin.v2.Table,
      protos.google.bigtable.admin.v2.CreateTableFromSnapshotMetadata
    >;
  }
  /**
   * Updates a specified table.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {google.bigtable.admin.v2.Table} request.table
   *   Required. The table to update.
   *   The table's `name` field is used to identify the table to update.
   * @param {google.protobuf.FieldMask} request.updateMask
   *   Required. The list of fields to update.
   *   A mask specifying which fields (e.g. `change_stream_config`) in the `table`
   *   field should be updated. This mask is relative to the `table` field, not to
   *   the request message. The wildcard (*) path is currently not supported.
   *   Currently UpdateTable is only supported for the following fields:
   *
   *   * `change_stream_config`
   *   * `change_stream_config.retention_period`
   *   * `deletion_protection`
   *
   *   If `column_families` is set in `update_mask`, it will return an
   *   UNIMPLEMENTED error.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing
   *   a long running operation. Its `promise()` method returns a promise
   *   you can `await` for.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.update_table.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_UpdateTable_async
   */
  updateTable(
    request?: protos.google.bigtable.admin.v2.IUpdateTableRequest,
    options?: CallOptions
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.ITable,
        protos.google.bigtable.admin.v2.IUpdateTableMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  >;
  updateTable(
    request: protos.google.bigtable.admin.v2.IUpdateTableRequest,
    options: CallOptions,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.ITable,
        protos.google.bigtable.admin.v2.IUpdateTableMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  updateTable(
    request: protos.google.bigtable.admin.v2.IUpdateTableRequest,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.ITable,
        protos.google.bigtable.admin.v2.IUpdateTableMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  updateTable(
    request?: protos.google.bigtable.admin.v2.IUpdateTableRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          LROperation<
            protos.google.bigtable.admin.v2.ITable,
            protos.google.bigtable.admin.v2.IUpdateTableMetadata
          >,
          protos.google.longrunning.IOperation | null | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.ITable,
        protos.google.bigtable.admin.v2.IUpdateTableMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.ITable,
        protos.google.bigtable.admin.v2.IUpdateTableMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        'table.name': request.table!.name ?? '',
      });
    this.initialize();
    return this.innerApiCalls.updateTable(request, options, callback);
  }
  /**
   * Check the status of the long running operation returned by `updateTable()`.
   * @param {String} name
   *   The operation name that will be passed.
   * @returns {Promise} - The promise which resolves to an object.
   *   The decoded operation object has result and metadata field to get information from.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.update_table.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_UpdateTable_async
   */
  async checkUpdateTableProgress(
    name: string
  ): Promise<
    LROperation<
      protos.google.bigtable.admin.v2.Table,
      protos.google.bigtable.admin.v2.UpdateTableMetadata
    >
  > {
    const request =
      new this._gaxModule.operationsProtos.google.longrunning.GetOperationRequest(
        {name}
      );
    const [operation] = await this.operationsClient.getOperation(request);
    const decodeOperation = new this._gaxModule.Operation(
      operation,
      this.descriptors.longrunning.updateTable,
      this._gaxModule.createDefaultBackoffSettings()
    );
    return decodeOperation as LROperation<
      protos.google.bigtable.admin.v2.Table,
      protos.google.bigtable.admin.v2.UpdateTableMetadata
    >;
  }
  /**
   * Restores a specified table which was accidentally deleted.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.name
   *   Required. The unique name of the table to be restored.
   *   Values are of the form
   *   `projects/{project}/instances/{instance}/tables/{table}`.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing
   *   a long running operation. Its `promise()` method returns a promise
   *   you can `await` for.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.undelete_table.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_UndeleteTable_async
   */
  undeleteTable(
    request?: protos.google.bigtable.admin.v2.IUndeleteTableRequest,
    options?: CallOptions
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.ITable,
        protos.google.bigtable.admin.v2.IUndeleteTableMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  >;
  undeleteTable(
    request: protos.google.bigtable.admin.v2.IUndeleteTableRequest,
    options: CallOptions,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.ITable,
        protos.google.bigtable.admin.v2.IUndeleteTableMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  undeleteTable(
    request: protos.google.bigtable.admin.v2.IUndeleteTableRequest,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.ITable,
        protos.google.bigtable.admin.v2.IUndeleteTableMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  undeleteTable(
    request?: protos.google.bigtable.admin.v2.IUndeleteTableRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          LROperation<
            protos.google.bigtable.admin.v2.ITable,
            protos.google.bigtable.admin.v2.IUndeleteTableMetadata
          >,
          protos.google.longrunning.IOperation | null | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.ITable,
        protos.google.bigtable.admin.v2.IUndeleteTableMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.ITable,
        protos.google.bigtable.admin.v2.IUndeleteTableMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        name: request.name ?? '',
      });
    this.initialize();
    return this.innerApiCalls.undeleteTable(request, options, callback);
  }
  /**
   * Check the status of the long running operation returned by `undeleteTable()`.
   * @param {String} name
   *   The operation name that will be passed.
   * @returns {Promise} - The promise which resolves to an object.
   *   The decoded operation object has result and metadata field to get information from.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.undelete_table.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_UndeleteTable_async
   */
  async checkUndeleteTableProgress(
    name: string
  ): Promise<
    LROperation<
      protos.google.bigtable.admin.v2.Table,
      protos.google.bigtable.admin.v2.UndeleteTableMetadata
    >
  > {
    const request =
      new this._gaxModule.operationsProtos.google.longrunning.GetOperationRequest(
        {name}
      );
    const [operation] = await this.operationsClient.getOperation(request);
    const decodeOperation = new this._gaxModule.Operation(
      operation,
      this.descriptors.longrunning.undeleteTable,
      this._gaxModule.createDefaultBackoffSettings()
    );
    return decodeOperation as LROperation<
      protos.google.bigtable.admin.v2.Table,
      protos.google.bigtable.admin.v2.UndeleteTableMetadata
    >;
  }
  /**
   * Creates a new AuthorizedView in a table.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.parent
   *   Required. This is the name of the table the AuthorizedView belongs to.
   *   Values are of the form
   *   `projects/{project}/instances/{instance}/tables/{table}`.
   * @param {string} request.authorizedViewId
   *   Required. The id of the AuthorizedView to create. This AuthorizedView must
   *   not already exist. The `authorized_view_id` appended to `parent` forms the
   *   full AuthorizedView name of the form
   *   `projects/{project}/instances/{instance}/tables/{table}/authorizedView/{authorized_view}`.
   * @param {google.bigtable.admin.v2.AuthorizedView} request.authorizedView
   *   Required. The AuthorizedView to create.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing
   *   a long running operation. Its `promise()` method returns a promise
   *   you can `await` for.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.create_authorized_view.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_CreateAuthorizedView_async
   */
  createAuthorizedView(
    request?: protos.google.bigtable.admin.v2.ICreateAuthorizedViewRequest,
    options?: CallOptions
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.IAuthorizedView,
        protos.google.bigtable.admin.v2.ICreateAuthorizedViewMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  >;
  createAuthorizedView(
    request: protos.google.bigtable.admin.v2.ICreateAuthorizedViewRequest,
    options: CallOptions,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.IAuthorizedView,
        protos.google.bigtable.admin.v2.ICreateAuthorizedViewMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  createAuthorizedView(
    request: protos.google.bigtable.admin.v2.ICreateAuthorizedViewRequest,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.IAuthorizedView,
        protos.google.bigtable.admin.v2.ICreateAuthorizedViewMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  createAuthorizedView(
    request?: protos.google.bigtable.admin.v2.ICreateAuthorizedViewRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          LROperation<
            protos.google.bigtable.admin.v2.IAuthorizedView,
            protos.google.bigtable.admin.v2.ICreateAuthorizedViewMetadata
          >,
          protos.google.longrunning.IOperation | null | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.IAuthorizedView,
        protos.google.bigtable.admin.v2.ICreateAuthorizedViewMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.IAuthorizedView,
        protos.google.bigtable.admin.v2.ICreateAuthorizedViewMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        parent: request.parent ?? '',
      });
    this.initialize();
    return this.innerApiCalls.createAuthorizedView(request, options, callback);
  }
  /**
   * Check the status of the long running operation returned by `createAuthorizedView()`.
   * @param {String} name
   *   The operation name that will be passed.
   * @returns {Promise} - The promise which resolves to an object.
   *   The decoded operation object has result and metadata field to get information from.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.create_authorized_view.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_CreateAuthorizedView_async
   */
  async checkCreateAuthorizedViewProgress(
    name: string
  ): Promise<
    LROperation<
      protos.google.bigtable.admin.v2.AuthorizedView,
      protos.google.bigtable.admin.v2.CreateAuthorizedViewMetadata
    >
  > {
    const request =
      new this._gaxModule.operationsProtos.google.longrunning.GetOperationRequest(
        {name}
      );
    const [operation] = await this.operationsClient.getOperation(request);
    const decodeOperation = new this._gaxModule.Operation(
      operation,
      this.descriptors.longrunning.createAuthorizedView,
      this._gaxModule.createDefaultBackoffSettings()
    );
    return decodeOperation as LROperation<
      protos.google.bigtable.admin.v2.AuthorizedView,
      protos.google.bigtable.admin.v2.CreateAuthorizedViewMetadata
    >;
  }
  /**
   * Updates an AuthorizedView in a table.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {google.bigtable.admin.v2.AuthorizedView} request.authorizedView
   *   Required. The AuthorizedView to update. The `name` in `authorized_view` is
   *   used to identify the AuthorizedView. AuthorizedView name must in this
   *   format
   *   projects/<project>/instances/<instance>/tables/<table>/authorizedViews/<authorized_view>
   * @param {google.protobuf.FieldMask} [request.updateMask]
   *   Optional. The list of fields to update.
   *   A mask specifying which fields in the AuthorizedView resource should be
   *   updated. This mask is relative to the AuthorizedView resource, not to the
   *   request message. A field will be overwritten if it is in the mask. If
   *   empty, all fields set in the request will be overwritten. A special value
   *   `*` means to overwrite all fields (including fields not set in the
   *   request).
   * @param {boolean} [request.ignoreWarnings]
   *   Optional. If true, ignore the safety checks when updating the
   *   AuthorizedView.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing
   *   a long running operation. Its `promise()` method returns a promise
   *   you can `await` for.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.update_authorized_view.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_UpdateAuthorizedView_async
   */
  updateAuthorizedView(
    request?: protos.google.bigtable.admin.v2.IUpdateAuthorizedViewRequest,
    options?: CallOptions
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.IAuthorizedView,
        protos.google.bigtable.admin.v2.IUpdateAuthorizedViewMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  >;
  updateAuthorizedView(
    request: protos.google.bigtable.admin.v2.IUpdateAuthorizedViewRequest,
    options: CallOptions,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.IAuthorizedView,
        protos.google.bigtable.admin.v2.IUpdateAuthorizedViewMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  updateAuthorizedView(
    request: protos.google.bigtable.admin.v2.IUpdateAuthorizedViewRequest,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.IAuthorizedView,
        protos.google.bigtable.admin.v2.IUpdateAuthorizedViewMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  updateAuthorizedView(
    request?: protos.google.bigtable.admin.v2.IUpdateAuthorizedViewRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          LROperation<
            protos.google.bigtable.admin.v2.IAuthorizedView,
            protos.google.bigtable.admin.v2.IUpdateAuthorizedViewMetadata
          >,
          protos.google.longrunning.IOperation | null | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.IAuthorizedView,
        protos.google.bigtable.admin.v2.IUpdateAuthorizedViewMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.IAuthorizedView,
        protos.google.bigtable.admin.v2.IUpdateAuthorizedViewMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        'authorized_view.name': request.authorizedView!.name ?? '',
      });
    this.initialize();
    return this.innerApiCalls.updateAuthorizedView(request, options, callback);
  }
  /**
   * Check the status of the long running operation returned by `updateAuthorizedView()`.
   * @param {String} name
   *   The operation name that will be passed.
   * @returns {Promise} - The promise which resolves to an object.
   *   The decoded operation object has result and metadata field to get information from.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.update_authorized_view.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_UpdateAuthorizedView_async
   */
  async checkUpdateAuthorizedViewProgress(
    name: string
  ): Promise<
    LROperation<
      protos.google.bigtable.admin.v2.AuthorizedView,
      protos.google.bigtable.admin.v2.UpdateAuthorizedViewMetadata
    >
  > {
    const request =
      new this._gaxModule.operationsProtos.google.longrunning.GetOperationRequest(
        {name}
      );
    const [operation] = await this.operationsClient.getOperation(request);
    const decodeOperation = new this._gaxModule.Operation(
      operation,
      this.descriptors.longrunning.updateAuthorizedView,
      this._gaxModule.createDefaultBackoffSettings()
    );
    return decodeOperation as LROperation<
      protos.google.bigtable.admin.v2.AuthorizedView,
      protos.google.bigtable.admin.v2.UpdateAuthorizedViewMetadata
    >;
  }
  /**
   * Creates a new snapshot in the specified cluster from the specified
   * source table. The cluster and the table must be in the same instance.
   *
   * Note: This is a private alpha release of Cloud Bigtable snapshots. This
   * feature is not currently available to most Cloud Bigtable customers. This
   * feature might be changed in backward-incompatible ways and is not
   * recommended for production use. It is not subject to any SLA or deprecation
   * policy.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.name
   *   Required. The unique name of the table to have the snapshot taken.
   *   Values are of the form
   *   `projects/{project}/instances/{instance}/tables/{table}`.
   * @param {string} request.cluster
   *   Required. The name of the cluster where the snapshot will be created in.
   *   Values are of the form
   *   `projects/{project}/instances/{instance}/clusters/{cluster}`.
   * @param {string} request.snapshotId
   *   Required. The ID by which the new snapshot should be referred to within the
   *   parent cluster, e.g., `mysnapshot` of the form:
   *   `{@link protos.-_.a-zA-Z0-9|_a-zA-Z0-9}*` rather than
   *   `projects/{project}/instances/{instance}/clusters/{cluster}/snapshots/mysnapshot`.
   * @param {google.protobuf.Duration} request.ttl
   *   The amount of time that the new snapshot can stay active after it is
   *   created. Once 'ttl' expires, the snapshot will get deleted. The maximum
   *   amount of time a snapshot can stay active is 7 days. If 'ttl' is not
   *   specified, the default value of 24 hours will be used.
   * @param {string} request.description
   *   Description of the snapshot.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing
   *   a long running operation. Its `promise()` method returns a promise
   *   you can `await` for.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.snapshot_table.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_SnapshotTable_async
   */
  snapshotTable(
    request?: protos.google.bigtable.admin.v2.ISnapshotTableRequest,
    options?: CallOptions
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.ISnapshot,
        protos.google.bigtable.admin.v2.ISnapshotTableMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  >;
  snapshotTable(
    request: protos.google.bigtable.admin.v2.ISnapshotTableRequest,
    options: CallOptions,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.ISnapshot,
        protos.google.bigtable.admin.v2.ISnapshotTableMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  snapshotTable(
    request: protos.google.bigtable.admin.v2.ISnapshotTableRequest,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.ISnapshot,
        protos.google.bigtable.admin.v2.ISnapshotTableMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  snapshotTable(
    request?: protos.google.bigtable.admin.v2.ISnapshotTableRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          LROperation<
            protos.google.bigtable.admin.v2.ISnapshot,
            protos.google.bigtable.admin.v2.ISnapshotTableMetadata
          >,
          protos.google.longrunning.IOperation | null | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.ISnapshot,
        protos.google.bigtable.admin.v2.ISnapshotTableMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.ISnapshot,
        protos.google.bigtable.admin.v2.ISnapshotTableMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        name: request.name ?? '',
      });
    this.initialize();
    return this.innerApiCalls.snapshotTable(request, options, callback);
  }
  /**
   * Check the status of the long running operation returned by `snapshotTable()`.
   * @param {String} name
   *   The operation name that will be passed.
   * @returns {Promise} - The promise which resolves to an object.
   *   The decoded operation object has result and metadata field to get information from.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.snapshot_table.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_SnapshotTable_async
   */
  async checkSnapshotTableProgress(
    name: string
  ): Promise<
    LROperation<
      protos.google.bigtable.admin.v2.Snapshot,
      protos.google.bigtable.admin.v2.SnapshotTableMetadata
    >
  > {
    const request =
      new this._gaxModule.operationsProtos.google.longrunning.GetOperationRequest(
        {name}
      );
    const [operation] = await this.operationsClient.getOperation(request);
    const decodeOperation = new this._gaxModule.Operation(
      operation,
      this.descriptors.longrunning.snapshotTable,
      this._gaxModule.createDefaultBackoffSettings()
    );
    return decodeOperation as LROperation<
      protos.google.bigtable.admin.v2.Snapshot,
      protos.google.bigtable.admin.v2.SnapshotTableMetadata
    >;
  }
  /**
   * Starts creating a new Cloud Bigtable Backup.  The returned backup
   * {@link protos.google.longrunning.Operation|long-running operation} can be used to
   * track creation of the backup. The
   * {@link protos.google.longrunning.Operation.metadata|metadata} field type is
   * {@link protos.google.bigtable.admin.v2.CreateBackupMetadata|CreateBackupMetadata}. The
   * {@link protos.google.longrunning.Operation.response|response} field type is
   * {@link protos.google.bigtable.admin.v2.Backup|Backup}, if successful. Cancelling the
   * returned operation will stop the creation and delete the backup.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.parent
   *   Required. This must be one of the clusters in the instance in which this
   *   table is located. The backup will be stored in this cluster. Values are
   *   of the form `projects/{project}/instances/{instance}/clusters/{cluster}`.
   * @param {string} request.backupId
   *   Required. The id of the backup to be created. The `backup_id` along with
   *   the parent `parent` are combined as {parent}/backups/{backup_id} to create
   *   the full backup name, of the form:
   *   `projects/{project}/instances/{instance}/clusters/{cluster}/backups/{backup_id}`.
   *   This string must be between 1 and 50 characters in length and match the
   *   regex {@link protos.-_.a-zA-Z0-9|_a-zA-Z0-9}*.
   * @param {google.bigtable.admin.v2.Backup} request.backup
   *   Required. The backup to create.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing
   *   a long running operation. Its `promise()` method returns a promise
   *   you can `await` for.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.create_backup.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_CreateBackup_async
   */
  createBackup(
    request?: protos.google.bigtable.admin.v2.ICreateBackupRequest,
    options?: CallOptions
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.IBackup,
        protos.google.bigtable.admin.v2.ICreateBackupMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  >;
  createBackup(
    request: protos.google.bigtable.admin.v2.ICreateBackupRequest,
    options: CallOptions,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.IBackup,
        protos.google.bigtable.admin.v2.ICreateBackupMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  createBackup(
    request: protos.google.bigtable.admin.v2.ICreateBackupRequest,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.IBackup,
        protos.google.bigtable.admin.v2.ICreateBackupMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  createBackup(
    request?: protos.google.bigtable.admin.v2.ICreateBackupRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          LROperation<
            protos.google.bigtable.admin.v2.IBackup,
            protos.google.bigtable.admin.v2.ICreateBackupMetadata
          >,
          protos.google.longrunning.IOperation | null | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.IBackup,
        protos.google.bigtable.admin.v2.ICreateBackupMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.IBackup,
        protos.google.bigtable.admin.v2.ICreateBackupMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        parent: request.parent ?? '',
      });
    this.initialize();
    return this.innerApiCalls.createBackup(request, options, callback);
  }
  /**
   * Check the status of the long running operation returned by `createBackup()`.
   * @param {String} name
   *   The operation name that will be passed.
   * @returns {Promise} - The promise which resolves to an object.
   *   The decoded operation object has result and metadata field to get information from.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.create_backup.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_CreateBackup_async
   */
  async checkCreateBackupProgress(
    name: string
  ): Promise<
    LROperation<
      protos.google.bigtable.admin.v2.Backup,
      protos.google.bigtable.admin.v2.CreateBackupMetadata
    >
  > {
    const request =
      new this._gaxModule.operationsProtos.google.longrunning.GetOperationRequest(
        {name}
      );
    const [operation] = await this.operationsClient.getOperation(request);
    const decodeOperation = new this._gaxModule.Operation(
      operation,
      this.descriptors.longrunning.createBackup,
      this._gaxModule.createDefaultBackoffSettings()
    );
    return decodeOperation as LROperation<
      protos.google.bigtable.admin.v2.Backup,
      protos.google.bigtable.admin.v2.CreateBackupMetadata
    >;
  }
  /**
   * Create a new table by restoring from a completed backup.  The
   * returned table {@link protos.google.longrunning.Operation|long-running operation} can
   * be used to track the progress of the operation, and to cancel it.  The
   * {@link protos.google.longrunning.Operation.metadata|metadata} field type is
   * {@link protos.google.bigtable.admin.RestoreTableMetadata|RestoreTableMetadata}.  The
   * {@link protos.google.longrunning.Operation.response|response} type is
   * {@link protos.google.bigtable.admin.v2.Table|Table}, if successful.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.parent
   *   Required. The name of the instance in which to create the restored
   *   table. Values are of the form `projects/<project>/instances/<instance>`.
   * @param {string} request.tableId
   *   Required. The id of the table to create and restore to. This
   *   table must not already exist. The `table_id` appended to
   *   `parent` forms the full table name of the form
   *   `projects/<project>/instances/<instance>/tables/<table_id>`.
   * @param {string} request.backup
   *   Name of the backup from which to restore.  Values are of the form
   *   `projects/<project>/instances/<instance>/clusters/<cluster>/backups/<backup>`.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing
   *   a long running operation. Its `promise()` method returns a promise
   *   you can `await` for.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.restore_table.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_RestoreTable_async
   */
  restoreTable(
    request?: protos.google.bigtable.admin.v2.IRestoreTableRequest,
    options?: CallOptions
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.ITable,
        protos.google.bigtable.admin.v2.IRestoreTableMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  >;
  restoreTable(
    request: protos.google.bigtable.admin.v2.IRestoreTableRequest,
    options: CallOptions,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.ITable,
        protos.google.bigtable.admin.v2.IRestoreTableMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  restoreTable(
    request: protos.google.bigtable.admin.v2.IRestoreTableRequest,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.ITable,
        protos.google.bigtable.admin.v2.IRestoreTableMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  restoreTable(
    request?: protos.google.bigtable.admin.v2.IRestoreTableRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          LROperation<
            protos.google.bigtable.admin.v2.ITable,
            protos.google.bigtable.admin.v2.IRestoreTableMetadata
          >,
          protos.google.longrunning.IOperation | null | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.ITable,
        protos.google.bigtable.admin.v2.IRestoreTableMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.ITable,
        protos.google.bigtable.admin.v2.IRestoreTableMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        parent: request.parent ?? '',
      });
    this.initialize();
    return this.innerApiCalls.restoreTable(request, options, callback);
  }
  /**
   * Check the status of the long running operation returned by `restoreTable()`.
   * @param {String} name
   *   The operation name that will be passed.
   * @returns {Promise} - The promise which resolves to an object.
   *   The decoded operation object has result and metadata field to get information from.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.restore_table.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_RestoreTable_async
   */
  async checkRestoreTableProgress(
    name: string
  ): Promise<
    LROperation<
      protos.google.bigtable.admin.v2.Table,
      protos.google.bigtable.admin.v2.RestoreTableMetadata
    >
  > {
    const request =
      new this._gaxModule.operationsProtos.google.longrunning.GetOperationRequest(
        {name}
      );
    const [operation] = await this.operationsClient.getOperation(request);
    const decodeOperation = new this._gaxModule.Operation(
      operation,
      this.descriptors.longrunning.restoreTable,
      this._gaxModule.createDefaultBackoffSettings()
    );
    return decodeOperation as LROperation<
      protos.google.bigtable.admin.v2.Table,
      protos.google.bigtable.admin.v2.RestoreTableMetadata
    >;
  }
  /**
   * Copy a Cloud Bigtable backup to a new backup in the destination cluster
   * located in the destination instance and project.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.parent
   *   Required. The name of the destination cluster that will contain the backup
   *   copy. The cluster must already exists. Values are of the form:
   *   `projects/{project}/instances/{instance}/clusters/{cluster}`.
   * @param {string} request.backupId
   *   Required. The id of the new backup. The `backup_id` along with `parent`
   *   are combined as {parent}/backups/{backup_id} to create the full backup
   *   name, of the form:
   *   `projects/{project}/instances/{instance}/clusters/{cluster}/backups/{backup_id}`.
   *   This string must be between 1 and 50 characters in length and match the
   *   regex {@link protos.-_.a-zA-Z0-9|_a-zA-Z0-9}*.
   * @param {string} request.sourceBackup
   *   Required. The source backup to be copied from.
   *   The source backup needs to be in READY state for it to be copied.
   *   Copying a copied backup is not allowed.
   *   Once CopyBackup is in progress, the source backup cannot be deleted or
   *   cleaned up on expiration until CopyBackup is finished.
   *   Values are of the form:
   *   `projects/<project>/instances/<instance>/clusters/<cluster>/backups/<backup>`.
   * @param {google.protobuf.Timestamp} request.expireTime
   *   Required. Required. The expiration time of the copied backup with
   *   microsecond granularity that must be at least 6 hours and at most 30 days
   *   from the time the request is received. Once the `expire_time` has
   *   passed, Cloud Bigtable will delete the backup and free the resources used
   *   by the backup.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is an object representing
   *   a long running operation. Its `promise()` method returns a promise
   *   you can `await` for.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.copy_backup.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_CopyBackup_async
   */
  copyBackup(
    request?: protos.google.bigtable.admin.v2.ICopyBackupRequest,
    options?: CallOptions
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.IBackup,
        protos.google.bigtable.admin.v2.ICopyBackupMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  >;
  copyBackup(
    request: protos.google.bigtable.admin.v2.ICopyBackupRequest,
    options: CallOptions,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.IBackup,
        protos.google.bigtable.admin.v2.ICopyBackupMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  copyBackup(
    request: protos.google.bigtable.admin.v2.ICopyBackupRequest,
    callback: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.IBackup,
        protos.google.bigtable.admin.v2.ICopyBackupMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): void;
  copyBackup(
    request?: protos.google.bigtable.admin.v2.ICopyBackupRequest,
    optionsOrCallback?:
      | CallOptions
      | Callback<
          LROperation<
            protos.google.bigtable.admin.v2.IBackup,
            protos.google.bigtable.admin.v2.ICopyBackupMetadata
          >,
          protos.google.longrunning.IOperation | null | undefined,
          {} | null | undefined
        >,
    callback?: Callback<
      LROperation<
        protos.google.bigtable.admin.v2.IBackup,
        protos.google.bigtable.admin.v2.ICopyBackupMetadata
      >,
      protos.google.longrunning.IOperation | null | undefined,
      {} | null | undefined
    >
  ): Promise<
    [
      LROperation<
        protos.google.bigtable.admin.v2.IBackup,
        protos.google.bigtable.admin.v2.ICopyBackupMetadata
      >,
      protos.google.longrunning.IOperation | undefined,
      {} | undefined,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        parent: request.parent ?? '',
      });
    this.initialize();
    return this.innerApiCalls.copyBackup(request, options, callback);
  }
  /**
   * Check the status of the long running operation returned by `copyBackup()`.
   * @param {String} name
   *   The operation name that will be passed.
   * @returns {Promise} - The promise which resolves to an object.
   *   The decoded operation object has result and metadata field to get information from.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#long-running-operations | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.copy_backup.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_CopyBackup_async
   */
  async checkCopyBackupProgress(
    name: string
  ): Promise<
    LROperation<
      protos.google.bigtable.admin.v2.Backup,
      protos.google.bigtable.admin.v2.CopyBackupMetadata
    >
  > {
    const request =
      new this._gaxModule.operationsProtos.google.longrunning.GetOperationRequest(
        {name}
      );
    const [operation] = await this.operationsClient.getOperation(request);
    const decodeOperation = new this._gaxModule.Operation(
      operation,
      this.descriptors.longrunning.copyBackup,
      this._gaxModule.createDefaultBackoffSettings()
    );
    return decodeOperation as LROperation<
      protos.google.bigtable.admin.v2.Backup,
      protos.google.bigtable.admin.v2.CopyBackupMetadata
    >;
  }
  /**
   * Lists all tables served from a specified instance.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.parent
   *   Required. The unique name of the instance for which tables should be
   *   listed. Values are of the form `projects/{project}/instances/{instance}`.
   * @param {google.bigtable.admin.v2.Table.View} request.view
   *   The view to be applied to the returned tables' fields.
   *   NAME_ONLY view (default) and REPLICATION_VIEW are supported.
   * @param {number} request.pageSize
   *   Maximum number of results per page.
   *
   *   A page_size of zero lets the server choose the number of items to return.
   *   A page_size which is strictly positive will return at most that many items.
   *   A negative page_size will cause an error.
   *
   *   Following the first request, subsequent paginated calls are not required
   *   to pass a page_size. If a page_size is set in subsequent calls, it must
   *   match the page_size given in the first request.
   * @param {string} request.pageToken
   *   The value of `next_page_token` returned by a previous call.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is Array of {@link protos.google.bigtable.admin.v2.Table|Table}.
   *   The client library will perform auto-pagination by default: it will call the API as many
   *   times as needed and will merge results from all the pages into this array.
   *   Note that it can affect your quota.
   *   We recommend using `listTablesAsync()`
   *   method described below for async iteration which you can stop as needed.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#auto-pagination | documentation }
   *   for more details and examples.
   */
  listTables(
    request?: protos.google.bigtable.admin.v2.IListTablesRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.bigtable.admin.v2.ITable[],
      protos.google.bigtable.admin.v2.IListTablesRequest | null,
      protos.google.bigtable.admin.v2.IListTablesResponse,
    ]
  >;
  listTables(
    request: protos.google.bigtable.admin.v2.IListTablesRequest,
    options: CallOptions,
    callback: PaginationCallback<
      protos.google.bigtable.admin.v2.IListTablesRequest,
      protos.google.bigtable.admin.v2.IListTablesResponse | null | undefined,
      protos.google.bigtable.admin.v2.ITable
    >
  ): void;
  listTables(
    request: protos.google.bigtable.admin.v2.IListTablesRequest,
    callback: PaginationCallback<
      protos.google.bigtable.admin.v2.IListTablesRequest,
      protos.google.bigtable.admin.v2.IListTablesResponse | null | undefined,
      protos.google.bigtable.admin.v2.ITable
    >
  ): void;
  listTables(
    request?: protos.google.bigtable.admin.v2.IListTablesRequest,
    optionsOrCallback?:
      | CallOptions
      | PaginationCallback<
          protos.google.bigtable.admin.v2.IListTablesRequest,
          | protos.google.bigtable.admin.v2.IListTablesResponse
          | null
          | undefined,
          protos.google.bigtable.admin.v2.ITable
        >,
    callback?: PaginationCallback<
      protos.google.bigtable.admin.v2.IListTablesRequest,
      protos.google.bigtable.admin.v2.IListTablesResponse | null | undefined,
      protos.google.bigtable.admin.v2.ITable
    >
  ): Promise<
    [
      protos.google.bigtable.admin.v2.ITable[],
      protos.google.bigtable.admin.v2.IListTablesRequest | null,
      protos.google.bigtable.admin.v2.IListTablesResponse,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        parent: request.parent ?? '',
      });
    this.initialize();
    return this.innerApiCalls.listTables(request, options, callback);
  }

  /**
   * Equivalent to `method.name.toCamelCase()`, but returns a NodeJS Stream object.
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.parent
   *   Required. The unique name of the instance for which tables should be
   *   listed. Values are of the form `projects/{project}/instances/{instance}`.
   * @param {google.bigtable.admin.v2.Table.View} request.view
   *   The view to be applied to the returned tables' fields.
   *   NAME_ONLY view (default) and REPLICATION_VIEW are supported.
   * @param {number} request.pageSize
   *   Maximum number of results per page.
   *
   *   A page_size of zero lets the server choose the number of items to return.
   *   A page_size which is strictly positive will return at most that many items.
   *   A negative page_size will cause an error.
   *
   *   Following the first request, subsequent paginated calls are not required
   *   to pass a page_size. If a page_size is set in subsequent calls, it must
   *   match the page_size given in the first request.
   * @param {string} request.pageToken
   *   The value of `next_page_token` returned by a previous call.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Stream}
   *   An object stream which emits an object representing {@link protos.google.bigtable.admin.v2.Table|Table} on 'data' event.
   *   The client library will perform auto-pagination by default: it will call the API as many
   *   times as needed. Note that it can affect your quota.
   *   We recommend using `listTablesAsync()`
   *   method described below for async iteration which you can stop as needed.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#auto-pagination | documentation }
   *   for more details and examples.
   */
  listTablesStream(
    request?: protos.google.bigtable.admin.v2.IListTablesRequest,
    options?: CallOptions
  ): Transform {
    request = request || {};
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        parent: request.parent ?? '',
      });
    const defaultCallSettings = this._defaults['listTables'];
    const callSettings = defaultCallSettings.merge(options);
    this.initialize();
    return this.descriptors.page.listTables.createStream(
      this.innerApiCalls.listTables as GaxCall,
      request,
      callSettings
    );
  }

  /**
   * Equivalent to `listTables`, but returns an iterable object.
   *
   * `for`-`await`-`of` syntax is used with the iterable to get response elements on-demand.
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.parent
   *   Required. The unique name of the instance for which tables should be
   *   listed. Values are of the form `projects/{project}/instances/{instance}`.
   * @param {google.bigtable.admin.v2.Table.View} request.view
   *   The view to be applied to the returned tables' fields.
   *   NAME_ONLY view (default) and REPLICATION_VIEW are supported.
   * @param {number} request.pageSize
   *   Maximum number of results per page.
   *
   *   A page_size of zero lets the server choose the number of items to return.
   *   A page_size which is strictly positive will return at most that many items.
   *   A negative page_size will cause an error.
   *
   *   Following the first request, subsequent paginated calls are not required
   *   to pass a page_size. If a page_size is set in subsequent calls, it must
   *   match the page_size given in the first request.
   * @param {string} request.pageToken
   *   The value of `next_page_token` returned by a previous call.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Object}
   *   An iterable Object that allows {@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Iteration_protocols | async iteration }.
   *   When you iterate the returned iterable, each element will be an object representing
   *   {@link protos.google.bigtable.admin.v2.Table|Table}. The API will be called under the hood as needed, once per the page,
   *   so you can stop the iteration when you don't need more results.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#auto-pagination | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.list_tables.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_ListTables_async
   */
  listTablesAsync(
    request?: protos.google.bigtable.admin.v2.IListTablesRequest,
    options?: CallOptions
  ): AsyncIterable<protos.google.bigtable.admin.v2.ITable> {
    request = request || {};
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        parent: request.parent ?? '',
      });
    const defaultCallSettings = this._defaults['listTables'];
    const callSettings = defaultCallSettings.merge(options);
    this.initialize();
    return this.descriptors.page.listTables.asyncIterate(
      this.innerApiCalls['listTables'] as GaxCall,
      request as {},
      callSettings
    ) as AsyncIterable<protos.google.bigtable.admin.v2.ITable>;
  }
  /**
   * Lists all AuthorizedViews from a specific table.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.parent
   *   Required. The unique name of the table for which AuthorizedViews should be
   *   listed. Values are of the form
   *   `projects/{project}/instances/{instance}/tables/{table}`.
   * @param {number} [request.pageSize]
   *   Optional. Maximum number of results per page.
   *
   *   A page_size of zero lets the server choose the number of items to return.
   *   A page_size which is strictly positive will return at most that many items.
   *   A negative page_size will cause an error.
   *
   *   Following the first request, subsequent paginated calls are not required
   *   to pass a page_size. If a page_size is set in subsequent calls, it must
   *   match the page_size given in the first request.
   * @param {string} [request.pageToken]
   *   Optional. The value of `next_page_token` returned by a previous call.
   * @param {google.bigtable.admin.v2.AuthorizedView.ResponseView} [request.view]
   *   Optional. The resource_view to be applied to the returned views' fields.
   *   Default to NAME_ONLY.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is Array of {@link protos.google.bigtable.admin.v2.AuthorizedView|AuthorizedView}.
   *   The client library will perform auto-pagination by default: it will call the API as many
   *   times as needed and will merge results from all the pages into this array.
   *   Note that it can affect your quota.
   *   We recommend using `listAuthorizedViewsAsync()`
   *   method described below for async iteration which you can stop as needed.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#auto-pagination | documentation }
   *   for more details and examples.
   */
  listAuthorizedViews(
    request?: protos.google.bigtable.admin.v2.IListAuthorizedViewsRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.bigtable.admin.v2.IAuthorizedView[],
      protos.google.bigtable.admin.v2.IListAuthorizedViewsRequest | null,
      protos.google.bigtable.admin.v2.IListAuthorizedViewsResponse,
    ]
  >;
  listAuthorizedViews(
    request: protos.google.bigtable.admin.v2.IListAuthorizedViewsRequest,
    options: CallOptions,
    callback: PaginationCallback<
      protos.google.bigtable.admin.v2.IListAuthorizedViewsRequest,
      | protos.google.bigtable.admin.v2.IListAuthorizedViewsResponse
      | null
      | undefined,
      protos.google.bigtable.admin.v2.IAuthorizedView
    >
  ): void;
  listAuthorizedViews(
    request: protos.google.bigtable.admin.v2.IListAuthorizedViewsRequest,
    callback: PaginationCallback<
      protos.google.bigtable.admin.v2.IListAuthorizedViewsRequest,
      | protos.google.bigtable.admin.v2.IListAuthorizedViewsResponse
      | null
      | undefined,
      protos.google.bigtable.admin.v2.IAuthorizedView
    >
  ): void;
  listAuthorizedViews(
    request?: protos.google.bigtable.admin.v2.IListAuthorizedViewsRequest,
    optionsOrCallback?:
      | CallOptions
      | PaginationCallback<
          protos.google.bigtable.admin.v2.IListAuthorizedViewsRequest,
          | protos.google.bigtable.admin.v2.IListAuthorizedViewsResponse
          | null
          | undefined,
          protos.google.bigtable.admin.v2.IAuthorizedView
        >,
    callback?: PaginationCallback<
      protos.google.bigtable.admin.v2.IListAuthorizedViewsRequest,
      | protos.google.bigtable.admin.v2.IListAuthorizedViewsResponse
      | null
      | undefined,
      protos.google.bigtable.admin.v2.IAuthorizedView
    >
  ): Promise<
    [
      protos.google.bigtable.admin.v2.IAuthorizedView[],
      protos.google.bigtable.admin.v2.IListAuthorizedViewsRequest | null,
      protos.google.bigtable.admin.v2.IListAuthorizedViewsResponse,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        parent: request.parent ?? '',
      });
    this.initialize();
    return this.innerApiCalls.listAuthorizedViews(request, options, callback);
  }

  /**
   * Equivalent to `method.name.toCamelCase()`, but returns a NodeJS Stream object.
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.parent
   *   Required. The unique name of the table for which AuthorizedViews should be
   *   listed. Values are of the form
   *   `projects/{project}/instances/{instance}/tables/{table}`.
   * @param {number} [request.pageSize]
   *   Optional. Maximum number of results per page.
   *
   *   A page_size of zero lets the server choose the number of items to return.
   *   A page_size which is strictly positive will return at most that many items.
   *   A negative page_size will cause an error.
   *
   *   Following the first request, subsequent paginated calls are not required
   *   to pass a page_size. If a page_size is set in subsequent calls, it must
   *   match the page_size given in the first request.
   * @param {string} [request.pageToken]
   *   Optional. The value of `next_page_token` returned by a previous call.
   * @param {google.bigtable.admin.v2.AuthorizedView.ResponseView} [request.view]
   *   Optional. The resource_view to be applied to the returned views' fields.
   *   Default to NAME_ONLY.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Stream}
   *   An object stream which emits an object representing {@link protos.google.bigtable.admin.v2.AuthorizedView|AuthorizedView} on 'data' event.
   *   The client library will perform auto-pagination by default: it will call the API as many
   *   times as needed. Note that it can affect your quota.
   *   We recommend using `listAuthorizedViewsAsync()`
   *   method described below for async iteration which you can stop as needed.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#auto-pagination | documentation }
   *   for more details and examples.
   */
  listAuthorizedViewsStream(
    request?: protos.google.bigtable.admin.v2.IListAuthorizedViewsRequest,
    options?: CallOptions
  ): Transform {
    request = request || {};
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        parent: request.parent ?? '',
      });
    const defaultCallSettings = this._defaults['listAuthorizedViews'];
    const callSettings = defaultCallSettings.merge(options);
    this.initialize();
    return this.descriptors.page.listAuthorizedViews.createStream(
      this.innerApiCalls.listAuthorizedViews as GaxCall,
      request,
      callSettings
    );
  }

  /**
   * Equivalent to `listAuthorizedViews`, but returns an iterable object.
   *
   * `for`-`await`-`of` syntax is used with the iterable to get response elements on-demand.
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.parent
   *   Required. The unique name of the table for which AuthorizedViews should be
   *   listed. Values are of the form
   *   `projects/{project}/instances/{instance}/tables/{table}`.
   * @param {number} [request.pageSize]
   *   Optional. Maximum number of results per page.
   *
   *   A page_size of zero lets the server choose the number of items to return.
   *   A page_size which is strictly positive will return at most that many items.
   *   A negative page_size will cause an error.
   *
   *   Following the first request, subsequent paginated calls are not required
   *   to pass a page_size. If a page_size is set in subsequent calls, it must
   *   match the page_size given in the first request.
   * @param {string} [request.pageToken]
   *   Optional. The value of `next_page_token` returned by a previous call.
   * @param {google.bigtable.admin.v2.AuthorizedView.ResponseView} [request.view]
   *   Optional. The resource_view to be applied to the returned views' fields.
   *   Default to NAME_ONLY.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Object}
   *   An iterable Object that allows {@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Iteration_protocols | async iteration }.
   *   When you iterate the returned iterable, each element will be an object representing
   *   {@link protos.google.bigtable.admin.v2.AuthorizedView|AuthorizedView}. The API will be called under the hood as needed, once per the page,
   *   so you can stop the iteration when you don't need more results.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#auto-pagination | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.list_authorized_views.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_ListAuthorizedViews_async
   */
  listAuthorizedViewsAsync(
    request?: protos.google.bigtable.admin.v2.IListAuthorizedViewsRequest,
    options?: CallOptions
  ): AsyncIterable<protos.google.bigtable.admin.v2.IAuthorizedView> {
    request = request || {};
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        parent: request.parent ?? '',
      });
    const defaultCallSettings = this._defaults['listAuthorizedViews'];
    const callSettings = defaultCallSettings.merge(options);
    this.initialize();
    return this.descriptors.page.listAuthorizedViews.asyncIterate(
      this.innerApiCalls['listAuthorizedViews'] as GaxCall,
      request as {},
      callSettings
    ) as AsyncIterable<protos.google.bigtable.admin.v2.IAuthorizedView>;
  }
  /**
   * Lists all snapshots associated with the specified cluster.
   *
   * Note: This is a private alpha release of Cloud Bigtable snapshots. This
   * feature is not currently available to most Cloud Bigtable customers. This
   * feature might be changed in backward-incompatible ways and is not
   * recommended for production use. It is not subject to any SLA or deprecation
   * policy.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.parent
   *   Required. The unique name of the cluster for which snapshots should be
   *   listed. Values are of the form
   *   `projects/{project}/instances/{instance}/clusters/{cluster}`.
   *   Use `{cluster} = '-'` to list snapshots for all clusters in an instance,
   *   e.g., `projects/{project}/instances/{instance}/clusters/-`.
   * @param {number} request.pageSize
   *   The maximum number of snapshots to return per page.
   *   CURRENTLY UNIMPLEMENTED AND IGNORED.
   * @param {string} request.pageToken
   *   The value of `next_page_token` returned by a previous call.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is Array of {@link protos.google.bigtable.admin.v2.Snapshot|Snapshot}.
   *   The client library will perform auto-pagination by default: it will call the API as many
   *   times as needed and will merge results from all the pages into this array.
   *   Note that it can affect your quota.
   *   We recommend using `listSnapshotsAsync()`
   *   method described below for async iteration which you can stop as needed.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#auto-pagination | documentation }
   *   for more details and examples.
   */
  listSnapshots(
    request?: protos.google.bigtable.admin.v2.IListSnapshotsRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.bigtable.admin.v2.ISnapshot[],
      protos.google.bigtable.admin.v2.IListSnapshotsRequest | null,
      protos.google.bigtable.admin.v2.IListSnapshotsResponse,
    ]
  >;
  listSnapshots(
    request: protos.google.bigtable.admin.v2.IListSnapshotsRequest,
    options: CallOptions,
    callback: PaginationCallback<
      protos.google.bigtable.admin.v2.IListSnapshotsRequest,
      protos.google.bigtable.admin.v2.IListSnapshotsResponse | null | undefined,
      protos.google.bigtable.admin.v2.ISnapshot
    >
  ): void;
  listSnapshots(
    request: protos.google.bigtable.admin.v2.IListSnapshotsRequest,
    callback: PaginationCallback<
      protos.google.bigtable.admin.v2.IListSnapshotsRequest,
      protos.google.bigtable.admin.v2.IListSnapshotsResponse | null | undefined,
      protos.google.bigtable.admin.v2.ISnapshot
    >
  ): void;
  listSnapshots(
    request?: protos.google.bigtable.admin.v2.IListSnapshotsRequest,
    optionsOrCallback?:
      | CallOptions
      | PaginationCallback<
          protos.google.bigtable.admin.v2.IListSnapshotsRequest,
          | protos.google.bigtable.admin.v2.IListSnapshotsResponse
          | null
          | undefined,
          protos.google.bigtable.admin.v2.ISnapshot
        >,
    callback?: PaginationCallback<
      protos.google.bigtable.admin.v2.IListSnapshotsRequest,
      protos.google.bigtable.admin.v2.IListSnapshotsResponse | null | undefined,
      protos.google.bigtable.admin.v2.ISnapshot
    >
  ): Promise<
    [
      protos.google.bigtable.admin.v2.ISnapshot[],
      protos.google.bigtable.admin.v2.IListSnapshotsRequest | null,
      protos.google.bigtable.admin.v2.IListSnapshotsResponse,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        parent: request.parent ?? '',
      });
    this.initialize();
    return this.innerApiCalls.listSnapshots(request, options, callback);
  }

  /**
   * Equivalent to `method.name.toCamelCase()`, but returns a NodeJS Stream object.
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.parent
   *   Required. The unique name of the cluster for which snapshots should be
   *   listed. Values are of the form
   *   `projects/{project}/instances/{instance}/clusters/{cluster}`.
   *   Use `{cluster} = '-'` to list snapshots for all clusters in an instance,
   *   e.g., `projects/{project}/instances/{instance}/clusters/-`.
   * @param {number} request.pageSize
   *   The maximum number of snapshots to return per page.
   *   CURRENTLY UNIMPLEMENTED AND IGNORED.
   * @param {string} request.pageToken
   *   The value of `next_page_token` returned by a previous call.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Stream}
   *   An object stream which emits an object representing {@link protos.google.bigtable.admin.v2.Snapshot|Snapshot} on 'data' event.
   *   The client library will perform auto-pagination by default: it will call the API as many
   *   times as needed. Note that it can affect your quota.
   *   We recommend using `listSnapshotsAsync()`
   *   method described below for async iteration which you can stop as needed.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#auto-pagination | documentation }
   *   for more details and examples.
   */
  listSnapshotsStream(
    request?: protos.google.bigtable.admin.v2.IListSnapshotsRequest,
    options?: CallOptions
  ): Transform {
    request = request || {};
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        parent: request.parent ?? '',
      });
    const defaultCallSettings = this._defaults['listSnapshots'];
    const callSettings = defaultCallSettings.merge(options);
    this.initialize();
    return this.descriptors.page.listSnapshots.createStream(
      this.innerApiCalls.listSnapshots as GaxCall,
      request,
      callSettings
    );
  }

  /**
   * Equivalent to `listSnapshots`, but returns an iterable object.
   *
   * `for`-`await`-`of` syntax is used with the iterable to get response elements on-demand.
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.parent
   *   Required. The unique name of the cluster for which snapshots should be
   *   listed. Values are of the form
   *   `projects/{project}/instances/{instance}/clusters/{cluster}`.
   *   Use `{cluster} = '-'` to list snapshots for all clusters in an instance,
   *   e.g., `projects/{project}/instances/{instance}/clusters/-`.
   * @param {number} request.pageSize
   *   The maximum number of snapshots to return per page.
   *   CURRENTLY UNIMPLEMENTED AND IGNORED.
   * @param {string} request.pageToken
   *   The value of `next_page_token` returned by a previous call.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Object}
   *   An iterable Object that allows {@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Iteration_protocols | async iteration }.
   *   When you iterate the returned iterable, each element will be an object representing
   *   {@link protos.google.bigtable.admin.v2.Snapshot|Snapshot}. The API will be called under the hood as needed, once per the page,
   *   so you can stop the iteration when you don't need more results.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#auto-pagination | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.list_snapshots.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_ListSnapshots_async
   */
  listSnapshotsAsync(
    request?: protos.google.bigtable.admin.v2.IListSnapshotsRequest,
    options?: CallOptions
  ): AsyncIterable<protos.google.bigtable.admin.v2.ISnapshot> {
    request = request || {};
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        parent: request.parent ?? '',
      });
    const defaultCallSettings = this._defaults['listSnapshots'];
    const callSettings = defaultCallSettings.merge(options);
    this.initialize();
    return this.descriptors.page.listSnapshots.asyncIterate(
      this.innerApiCalls['listSnapshots'] as GaxCall,
      request as {},
      callSettings
    ) as AsyncIterable<protos.google.bigtable.admin.v2.ISnapshot>;
  }
  /**
   * Lists Cloud Bigtable backups. Returns both completed and pending
   * backups.
   *
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.parent
   *   Required. The cluster to list backups from.  Values are of the
   *   form `projects/{project}/instances/{instance}/clusters/{cluster}`.
   *   Use `{cluster} = '-'` to list backups for all clusters in an instance,
   *   e.g., `projects/{project}/instances/{instance}/clusters/-`.
   * @param {string} request.filter
   *   A filter expression that filters backups listed in the response.
   *   The expression must specify the field name, a comparison operator,
   *   and the value that you want to use for filtering. The value must be a
   *   string, a number, or a boolean. The comparison operator must be
   *   <, >, <=, >=, !=, =, or :. Colon ':' represents a HAS operator which is
   *   roughly synonymous with equality. Filter rules are case insensitive.
   *
   *   The fields eligible for filtering are:
   *
   *   * `name`
   *   * `source_table`
   *   * `state`
   *   * `start_time` (and values are of the format YYYY-MM-DDTHH:MM:SSZ)
   *   * `end_time` (and values are of the format YYYY-MM-DDTHH:MM:SSZ)
   *   * `expire_time` (and values are of the format YYYY-MM-DDTHH:MM:SSZ)
   *   * `size_bytes`
   *
   *   To filter on multiple expressions, provide each separate expression within
   *   parentheses. By default, each expression is an AND expression. However,
   *   you can include AND, OR, and NOT expressions explicitly.
   *
   *   Some examples of using filters are:
   *
   *   * `name:"exact"` --> The backup's name is the string "exact".
   *   * `name:howl` --> The backup's name contains the string "howl".
   *   * `source_table:prod`
   *          --> The source_table's name contains the string "prod".
   *   * `state:CREATING` --> The backup is pending creation.
   *   * `state:READY` --> The backup is fully created and ready for use.
   *   * `(name:howl) AND (start_time < \"2018-03-28T14:50:00Z\")`
   *          --> The backup name contains the string "howl" and start_time
   *              of the backup is before 2018-03-28T14:50:00Z.
   *   * `size_bytes > 10000000000` --> The backup's size is greater than 10GB
   * @param {string} request.orderBy
   *   An expression for specifying the sort order of the results of the request.
   *   The string value should specify one or more fields in
   *   {@link protos.google.bigtable.admin.v2.Backup|Backup}. The full syntax is described at
   *   https://aip.dev/132#ordering.
   *
   *   Fields supported are:
   *
   *   * name
   *   * source_table
   *   * expire_time
   *   * start_time
   *   * end_time
   *   * size_bytes
   *   * state
   *
   *   For example, "start_time". The default sorting order is ascending.
   *   To specify descending order for the field, a suffix " desc" should
   *   be appended to the field name. For example, "start_time desc".
   *   Redundant space characters in the syntax are insigificant.
   *
   *   If order_by is empty, results will be sorted by `start_time` in descending
   *   order starting from the most recently created backup.
   * @param {number} request.pageSize
   *   Number of backups to be returned in the response. If 0 or
   *   less, defaults to the server's maximum allowed page size.
   * @param {string} request.pageToken
   *   If non-empty, `page_token` should contain a
   *   {@link protos.google.bigtable.admin.v2.ListBackupsResponse.next_page_token|next_page_token}
   *   from a previous
   *   {@link protos.google.bigtable.admin.v2.ListBackupsResponse|ListBackupsResponse} to the
   *   same `parent` and with the same `filter`.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Promise} - The promise which resolves to an array.
   *   The first element of the array is Array of {@link protos.google.bigtable.admin.v2.Backup|Backup}.
   *   The client library will perform auto-pagination by default: it will call the API as many
   *   times as needed and will merge results from all the pages into this array.
   *   Note that it can affect your quota.
   *   We recommend using `listBackupsAsync()`
   *   method described below for async iteration which you can stop as needed.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#auto-pagination | documentation }
   *   for more details and examples.
   */
  listBackups(
    request?: protos.google.bigtable.admin.v2.IListBackupsRequest,
    options?: CallOptions
  ): Promise<
    [
      protos.google.bigtable.admin.v2.IBackup[],
      protos.google.bigtable.admin.v2.IListBackupsRequest | null,
      protos.google.bigtable.admin.v2.IListBackupsResponse,
    ]
  >;
  listBackups(
    request: protos.google.bigtable.admin.v2.IListBackupsRequest,
    options: CallOptions,
    callback: PaginationCallback<
      protos.google.bigtable.admin.v2.IListBackupsRequest,
      protos.google.bigtable.admin.v2.IListBackupsResponse | null | undefined,
      protos.google.bigtable.admin.v2.IBackup
    >
  ): void;
  listBackups(
    request: protos.google.bigtable.admin.v2.IListBackupsRequest,
    callback: PaginationCallback<
      protos.google.bigtable.admin.v2.IListBackupsRequest,
      protos.google.bigtable.admin.v2.IListBackupsResponse | null | undefined,
      protos.google.bigtable.admin.v2.IBackup
    >
  ): void;
  listBackups(
    request?: protos.google.bigtable.admin.v2.IListBackupsRequest,
    optionsOrCallback?:
      | CallOptions
      | PaginationCallback<
          protos.google.bigtable.admin.v2.IListBackupsRequest,
          | protos.google.bigtable.admin.v2.IListBackupsResponse
          | null
          | undefined,
          protos.google.bigtable.admin.v2.IBackup
        >,
    callback?: PaginationCallback<
      protos.google.bigtable.admin.v2.IListBackupsRequest,
      protos.google.bigtable.admin.v2.IListBackupsResponse | null | undefined,
      protos.google.bigtable.admin.v2.IBackup
    >
  ): Promise<
    [
      protos.google.bigtable.admin.v2.IBackup[],
      protos.google.bigtable.admin.v2.IListBackupsRequest | null,
      protos.google.bigtable.admin.v2.IListBackupsResponse,
    ]
  > | void {
    request = request || {};
    let options: CallOptions;
    if (typeof optionsOrCallback === 'function' && callback === undefined) {
      callback = optionsOrCallback;
      options = {};
    } else {
      options = optionsOrCallback as CallOptions;
    }
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        parent: request.parent ?? '',
      });
    this.initialize();
    return this.innerApiCalls.listBackups(request, options, callback);
  }

  /**
   * Equivalent to `method.name.toCamelCase()`, but returns a NodeJS Stream object.
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.parent
   *   Required. The cluster to list backups from.  Values are of the
   *   form `projects/{project}/instances/{instance}/clusters/{cluster}`.
   *   Use `{cluster} = '-'` to list backups for all clusters in an instance,
   *   e.g., `projects/{project}/instances/{instance}/clusters/-`.
   * @param {string} request.filter
   *   A filter expression that filters backups listed in the response.
   *   The expression must specify the field name, a comparison operator,
   *   and the value that you want to use for filtering. The value must be a
   *   string, a number, or a boolean. The comparison operator must be
   *   <, >, <=, >=, !=, =, or :. Colon ':' represents a HAS operator which is
   *   roughly synonymous with equality. Filter rules are case insensitive.
   *
   *   The fields eligible for filtering are:
   *
   *   * `name`
   *   * `source_table`
   *   * `state`
   *   * `start_time` (and values are of the format YYYY-MM-DDTHH:MM:SSZ)
   *   * `end_time` (and values are of the format YYYY-MM-DDTHH:MM:SSZ)
   *   * `expire_time` (and values are of the format YYYY-MM-DDTHH:MM:SSZ)
   *   * `size_bytes`
   *
   *   To filter on multiple expressions, provide each separate expression within
   *   parentheses. By default, each expression is an AND expression. However,
   *   you can include AND, OR, and NOT expressions explicitly.
   *
   *   Some examples of using filters are:
   *
   *   * `name:"exact"` --> The backup's name is the string "exact".
   *   * `name:howl` --> The backup's name contains the string "howl".
   *   * `source_table:prod`
   *          --> The source_table's name contains the string "prod".
   *   * `state:CREATING` --> The backup is pending creation.
   *   * `state:READY` --> The backup is fully created and ready for use.
   *   * `(name:howl) AND (start_time < \"2018-03-28T14:50:00Z\")`
   *          --> The backup name contains the string "howl" and start_time
   *              of the backup is before 2018-03-28T14:50:00Z.
   *   * `size_bytes > 10000000000` --> The backup's size is greater than 10GB
   * @param {string} request.orderBy
   *   An expression for specifying the sort order of the results of the request.
   *   The string value should specify one or more fields in
   *   {@link protos.google.bigtable.admin.v2.Backup|Backup}. The full syntax is described at
   *   https://aip.dev/132#ordering.
   *
   *   Fields supported are:
   *
   *   * name
   *   * source_table
   *   * expire_time
   *   * start_time
   *   * end_time
   *   * size_bytes
   *   * state
   *
   *   For example, "start_time". The default sorting order is ascending.
   *   To specify descending order for the field, a suffix " desc" should
   *   be appended to the field name. For example, "start_time desc".
   *   Redundant space characters in the syntax are insigificant.
   *
   *   If order_by is empty, results will be sorted by `start_time` in descending
   *   order starting from the most recently created backup.
   * @param {number} request.pageSize
   *   Number of backups to be returned in the response. If 0 or
   *   less, defaults to the server's maximum allowed page size.
   * @param {string} request.pageToken
   *   If non-empty, `page_token` should contain a
   *   {@link protos.google.bigtable.admin.v2.ListBackupsResponse.next_page_token|next_page_token}
   *   from a previous
   *   {@link protos.google.bigtable.admin.v2.ListBackupsResponse|ListBackupsResponse} to the
   *   same `parent` and with the same `filter`.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Stream}
   *   An object stream which emits an object representing {@link protos.google.bigtable.admin.v2.Backup|Backup} on 'data' event.
   *   The client library will perform auto-pagination by default: it will call the API as many
   *   times as needed. Note that it can affect your quota.
   *   We recommend using `listBackupsAsync()`
   *   method described below for async iteration which you can stop as needed.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#auto-pagination | documentation }
   *   for more details and examples.
   */
  listBackupsStream(
    request?: protos.google.bigtable.admin.v2.IListBackupsRequest,
    options?: CallOptions
  ): Transform {
    request = request || {};
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        parent: request.parent ?? '',
      });
    const defaultCallSettings = this._defaults['listBackups'];
    const callSettings = defaultCallSettings.merge(options);
    this.initialize();
    return this.descriptors.page.listBackups.createStream(
      this.innerApiCalls.listBackups as GaxCall,
      request,
      callSettings
    );
  }

  /**
   * Equivalent to `listBackups`, but returns an iterable object.
   *
   * `for`-`await`-`of` syntax is used with the iterable to get response elements on-demand.
   * @param {Object} request
   *   The request object that will be sent.
   * @param {string} request.parent
   *   Required. The cluster to list backups from.  Values are of the
   *   form `projects/{project}/instances/{instance}/clusters/{cluster}`.
   *   Use `{cluster} = '-'` to list backups for all clusters in an instance,
   *   e.g., `projects/{project}/instances/{instance}/clusters/-`.
   * @param {string} request.filter
   *   A filter expression that filters backups listed in the response.
   *   The expression must specify the field name, a comparison operator,
   *   and the value that you want to use for filtering. The value must be a
   *   string, a number, or a boolean. The comparison operator must be
   *   <, >, <=, >=, !=, =, or :. Colon ':' represents a HAS operator which is
   *   roughly synonymous with equality. Filter rules are case insensitive.
   *
   *   The fields eligible for filtering are:
   *
   *   * `name`
   *   * `source_table`
   *   * `state`
   *   * `start_time` (and values are of the format YYYY-MM-DDTHH:MM:SSZ)
   *   * `end_time` (and values are of the format YYYY-MM-DDTHH:MM:SSZ)
   *   * `expire_time` (and values are of the format YYYY-MM-DDTHH:MM:SSZ)
   *   * `size_bytes`
   *
   *   To filter on multiple expressions, provide each separate expression within
   *   parentheses. By default, each expression is an AND expression. However,
   *   you can include AND, OR, and NOT expressions explicitly.
   *
   *   Some examples of using filters are:
   *
   *   * `name:"exact"` --> The backup's name is the string "exact".
   *   * `name:howl` --> The backup's name contains the string "howl".
   *   * `source_table:prod`
   *          --> The source_table's name contains the string "prod".
   *   * `state:CREATING` --> The backup is pending creation.
   *   * `state:READY` --> The backup is fully created and ready for use.
   *   * `(name:howl) AND (start_time < \"2018-03-28T14:50:00Z\")`
   *          --> The backup name contains the string "howl" and start_time
   *              of the backup is before 2018-03-28T14:50:00Z.
   *   * `size_bytes > 10000000000` --> The backup's size is greater than 10GB
   * @param {string} request.orderBy
   *   An expression for specifying the sort order of the results of the request.
   *   The string value should specify one or more fields in
   *   {@link protos.google.bigtable.admin.v2.Backup|Backup}. The full syntax is described at
   *   https://aip.dev/132#ordering.
   *
   *   Fields supported are:
   *
   *   * name
   *   * source_table
   *   * expire_time
   *   * start_time
   *   * end_time
   *   * size_bytes
   *   * state
   *
   *   For example, "start_time". The default sorting order is ascending.
   *   To specify descending order for the field, a suffix " desc" should
   *   be appended to the field name. For example, "start_time desc".
   *   Redundant space characters in the syntax are insigificant.
   *
   *   If order_by is empty, results will be sorted by `start_time` in descending
   *   order starting from the most recently created backup.
   * @param {number} request.pageSize
   *   Number of backups to be returned in the response. If 0 or
   *   less, defaults to the server's maximum allowed page size.
   * @param {string} request.pageToken
   *   If non-empty, `page_token` should contain a
   *   {@link protos.google.bigtable.admin.v2.ListBackupsResponse.next_page_token|next_page_token}
   *   from a previous
   *   {@link protos.google.bigtable.admin.v2.ListBackupsResponse|ListBackupsResponse} to the
   *   same `parent` and with the same `filter`.
   * @param {object} [options]
   *   Call options. See {@link https://googleapis.dev/nodejs/google-gax/latest/interfaces/CallOptions.html|CallOptions} for more details.
   * @returns {Object}
   *   An iterable Object that allows {@link https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Iteration_protocols | async iteration }.
   *   When you iterate the returned iterable, each element will be an object representing
   *   {@link protos.google.bigtable.admin.v2.Backup|Backup}. The API will be called under the hood as needed, once per the page,
   *   so you can stop the iteration when you don't need more results.
   *   Please see the {@link https://github.com/googleapis/gax-nodejs/blob/master/client-libraries.md#auto-pagination | documentation }
   *   for more details and examples.
   * @example <caption>include:samples/generated/v2/bigtable_table_admin.list_backups.js</caption>
   * region_tag:bigtableadmin_v2_generated_BigtableTableAdmin_ListBackups_async
   */
  listBackupsAsync(
    request?: protos.google.bigtable.admin.v2.IListBackupsRequest,
    options?: CallOptions
  ): AsyncIterable<protos.google.bigtable.admin.v2.IBackup> {
    request = request || {};
    options = options || {};
    options.otherArgs = options.otherArgs || {};
    options.otherArgs.headers = options.otherArgs.headers || {};
    options.otherArgs.headers['x-goog-request-params'] =
      this._gaxModule.routingHeader.fromParams({
        parent: request.parent ?? '',
      });
    const defaultCallSettings = this._defaults['listBackups'];
    const callSettings = defaultCallSettings.merge(options);
    this.initialize();
    return this.descriptors.page.listBackups.asyncIterate(
      this.innerApiCalls['listBackups'] as GaxCall,
      request as {},
      callSettings
    ) as AsyncIterable<protos.google.bigtable.admin.v2.IBackup>;
  }
  // --------------------
  // -- Path templates --
  // --------------------

  /**
   * Return a fully-qualified appProfile resource name string.
   *
   * @param {string} project
   * @param {string} instance
   * @param {string} app_profile
   * @returns {string} Resource name string.
   */
  appProfilePath(project: string, instance: string, appProfile: string) {
    return this.pathTemplates.appProfilePathTemplate.render({
      project: project,
      instance: instance,
      app_profile: appProfile,
    });
  }

  /**
   * Parse the project from AppProfile resource.
   *
   * @param {string} appProfileName
   *   A fully-qualified path representing AppProfile resource.
   * @returns {string} A string representing the project.
   */
  matchProjectFromAppProfileName(appProfileName: string) {
    return this.pathTemplates.appProfilePathTemplate.match(appProfileName)
      .project;
  }

  /**
   * Parse the instance from AppProfile resource.
   *
   * @param {string} appProfileName
   *   A fully-qualified path representing AppProfile resource.
   * @returns {string} A string representing the instance.
   */
  matchInstanceFromAppProfileName(appProfileName: string) {
    return this.pathTemplates.appProfilePathTemplate.match(appProfileName)
      .instance;
  }

  /**
   * Parse the app_profile from AppProfile resource.
   *
   * @param {string} appProfileName
   *   A fully-qualified path representing AppProfile resource.
   * @returns {string} A string representing the app_profile.
   */
  matchAppProfileFromAppProfileName(appProfileName: string) {
    return this.pathTemplates.appProfilePathTemplate.match(appProfileName)
      .app_profile;
  }

  /**
   * Return a fully-qualified authorizedView resource name string.
   *
   * @param {string} project
   * @param {string} instance
   * @param {string} table
   * @param {string} authorized_view
   * @returns {string} Resource name string.
   */
  authorizedViewPath(
    project: string,
    instance: string,
    table: string,
    authorizedView: string
  ) {
    return this.pathTemplates.authorizedViewPathTemplate.render({
      project: project,
      instance: instance,
      table: table,
      authorized_view: authorizedView,
    });
  }

  /**
   * Parse the project from AuthorizedView resource.
   *
   * @param {string} authorizedViewName
   *   A fully-qualified path representing AuthorizedView resource.
   * @returns {string} A string representing the project.
   */
  matchProjectFromAuthorizedViewName(authorizedViewName: string) {
    return this.pathTemplates.authorizedViewPathTemplate.match(
      authorizedViewName
    ).project;
  }

  /**
   * Parse the instance from AuthorizedView resource.
   *
   * @param {string} authorizedViewName
   *   A fully-qualified path representing AuthorizedView resource.
   * @returns {string} A string representing the instance.
   */
  matchInstanceFromAuthorizedViewName(authorizedViewName: string) {
    return this.pathTemplates.authorizedViewPathTemplate.match(
      authorizedViewName
    ).instance;
  }

  /**
   * Parse the table from AuthorizedView resource.
   *
   * @param {string} authorizedViewName
   *   A fully-qualified path representing AuthorizedView resource.
   * @returns {string} A string representing the table.
   */
  matchTableFromAuthorizedViewName(authorizedViewName: string) {
    return this.pathTemplates.authorizedViewPathTemplate.match(
      authorizedViewName
    ).table;
  }

  /**
   * Parse the authorized_view from AuthorizedView resource.
   *
   * @param {string} authorizedViewName
   *   A fully-qualified path representing AuthorizedView resource.
   * @returns {string} A string representing the authorized_view.
   */
  matchAuthorizedViewFromAuthorizedViewName(authorizedViewName: string) {
    return this.pathTemplates.authorizedViewPathTemplate.match(
      authorizedViewName
    ).authorized_view;
  }

  /**
   * Return a fully-qualified backup resource name string.
   *
   * @param {string} project
   * @param {string} instance
   * @param {string} cluster
   * @param {string} backup
   * @returns {string} Resource name string.
   */
  backupPath(
    project: string,
    instance: string,
    cluster: string,
    backup: string
  ) {
    return this.pathTemplates.backupPathTemplate.render({
      project: project,
      instance: instance,
      cluster: cluster,
      backup: backup,
    });
  }

  /**
   * Parse the project from Backup resource.
   *
   * @param {string} backupName
   *   A fully-qualified path representing Backup resource.
   * @returns {string} A string representing the project.
   */
  matchProjectFromBackupName(backupName: string) {
    return this.pathTemplates.backupPathTemplate.match(backupName).project;
  }

  /**
   * Parse the instance from Backup resource.
   *
   * @param {string} backupName
   *   A fully-qualified path representing Backup resource.
   * @returns {string} A string representing the instance.
   */
  matchInstanceFromBackupName(backupName: string) {
    return this.pathTemplates.backupPathTemplate.match(backupName).instance;
  }

  /**
   * Parse the cluster from Backup resource.
   *
   * @param {string} backupName
   *   A fully-qualified path representing Backup resource.
   * @returns {string} A string representing the cluster.
   */
  matchClusterFromBackupName(backupName: string) {
    return this.pathTemplates.backupPathTemplate.match(backupName).cluster;
  }

  /**
   * Parse the backup from Backup resource.
   *
   * @param {string} backupName
   *   A fully-qualified path representing Backup resource.
   * @returns {string} A string representing the backup.
   */
  matchBackupFromBackupName(backupName: string) {
    return this.pathTemplates.backupPathTemplate.match(backupName).backup;
  }

  /**
   * Return a fully-qualified cluster resource name string.
   *
   * @param {string} project
   * @param {string} instance
   * @param {string} cluster
   * @returns {string} Resource name string.
   */
  clusterPath(project: string, instance: string, cluster: string) {
    return this.pathTemplates.clusterPathTemplate.render({
      project: project,
      instance: instance,
      cluster: cluster,
    });
  }

  /**
   * Parse the project from Cluster resource.
   *
   * @param {string} clusterName
   *   A fully-qualified path representing Cluster resource.
   * @returns {string} A string representing the project.
   */
  matchProjectFromClusterName(clusterName: string) {
    return this.pathTemplates.clusterPathTemplate.match(clusterName).project;
  }

  /**
   * Parse the instance from Cluster resource.
   *
   * @param {string} clusterName
   *   A fully-qualified path representing Cluster resource.
   * @returns {string} A string representing the instance.
   */
  matchInstanceFromClusterName(clusterName: string) {
    return this.pathTemplates.clusterPathTemplate.match(clusterName).instance;
  }

  /**
   * Parse the cluster from Cluster resource.
   *
   * @param {string} clusterName
   *   A fully-qualified path representing Cluster resource.
   * @returns {string} A string representing the cluster.
   */
  matchClusterFromClusterName(clusterName: string) {
    return this.pathTemplates.clusterPathTemplate.match(clusterName).cluster;
  }

  /**
   * Return a fully-qualified hotTablet resource name string.
   *
   * @param {string} project
   * @param {string} instance
   * @param {string} cluster
   * @param {string} hot_tablet
   * @returns {string} Resource name string.
   */
  hotTabletPath(
    project: string,
    instance: string,
    cluster: string,
    hotTablet: string
  ) {
    return this.pathTemplates.hotTabletPathTemplate.render({
      project: project,
      instance: instance,
      cluster: cluster,
      hot_tablet: hotTablet,
    });
  }

  /**
   * Parse the project from HotTablet resource.
   *
   * @param {string} hotTabletName
   *   A fully-qualified path representing HotTablet resource.
   * @returns {string} A string representing the project.
   */
  matchProjectFromHotTabletName(hotTabletName: string) {
    return this.pathTemplates.hotTabletPathTemplate.match(hotTabletName)
      .project;
  }

  /**
   * Parse the instance from HotTablet resource.
   *
   * @param {string} hotTabletName
   *   A fully-qualified path representing HotTablet resource.
   * @returns {string} A string representing the instance.
   */
  matchInstanceFromHotTabletName(hotTabletName: string) {
    return this.pathTemplates.hotTabletPathTemplate.match(hotTabletName)
      .instance;
  }

  /**
   * Parse the cluster from HotTablet resource.
   *
   * @param {string} hotTabletName
   *   A fully-qualified path representing HotTablet resource.
   * @returns {string} A string representing the cluster.
   */
  matchClusterFromHotTabletName(hotTabletName: string) {
    return this.pathTemplates.hotTabletPathTemplate.match(hotTabletName)
      .cluster;
  }

  /**
   * Parse the hot_tablet from HotTablet resource.
   *
   * @param {string} hotTabletName
   *   A fully-qualified path representing HotTablet resource.
   * @returns {string} A string representing the hot_tablet.
   */
  matchHotTabletFromHotTabletName(hotTabletName: string) {
    return this.pathTemplates.hotTabletPathTemplate.match(hotTabletName)
      .hot_tablet;
  }

  /**
   * Return a fully-qualified instance resource name string.
   *
   * @param {string} project
   * @param {string} instance
   * @returns {string} Resource name string.
   */
  instancePath(project: string, instance: string) {
    return this.pathTemplates.instancePathTemplate.render({
      project: project,
      instance: instance,
    });
  }

  /**
   * Parse the project from Instance resource.
   *
   * @param {string} instanceName
   *   A fully-qualified path representing Instance resource.
   * @returns {string} A string representing the project.
   */
  matchProjectFromInstanceName(instanceName: string) {
    return this.pathTemplates.instancePathTemplate.match(instanceName).project;
  }

  /**
   * Parse the instance from Instance resource.
   *
   * @param {string} instanceName
   *   A fully-qualified path representing Instance resource.
   * @returns {string} A string representing the instance.
   */
  matchInstanceFromInstanceName(instanceName: string) {
    return this.pathTemplates.instancePathTemplate.match(instanceName).instance;
  }

  /**
   * Return a fully-qualified project resource name string.
   *
   * @param {string} project
   * @returns {string} Resource name string.
   */
  projectPath(project: string) {
    return this.pathTemplates.projectPathTemplate.render({
      project: project,
    });
  }

  /**
   * Parse the project from Project resource.
   *
   * @param {string} projectName
   *   A fully-qualified path representing Project resource.
   * @returns {string} A string representing the project.
   */
  matchProjectFromProjectName(projectName: string) {
    return this.pathTemplates.projectPathTemplate.match(projectName).project;
  }

  /**
   * Return a fully-qualified snapshot resource name string.
   *
   * @param {string} project
   * @param {string} instance
   * @param {string} cluster
   * @param {string} snapshot
   * @returns {string} Resource name string.
   */
  snapshotPath(
    project: string,
    instance: string,
    cluster: string,
    snapshot: string
  ) {
    return this.pathTemplates.snapshotPathTemplate.render({
      project: project,
      instance: instance,
      cluster: cluster,
      snapshot: snapshot,
    });
  }

  /**
   * Parse the project from Snapshot resource.
   *
   * @param {string} snapshotName
   *   A fully-qualified path representing Snapshot resource.
   * @returns {string} A string representing the project.
   */
  matchProjectFromSnapshotName(snapshotName: string) {
    return this.pathTemplates.snapshotPathTemplate.match(snapshotName).project;
  }

  /**
   * Parse the instance from Snapshot resource.
   *
   * @param {string} snapshotName
   *   A fully-qualified path representing Snapshot resource.
   * @returns {string} A string representing the instance.
   */
  matchInstanceFromSnapshotName(snapshotName: string) {
    return this.pathTemplates.snapshotPathTemplate.match(snapshotName).instance;
  }

  /**
   * Parse the cluster from Snapshot resource.
   *
   * @param {string} snapshotName
   *   A fully-qualified path representing Snapshot resource.
   * @returns {string} A string representing the cluster.
   */
  matchClusterFromSnapshotName(snapshotName: string) {
    return this.pathTemplates.snapshotPathTemplate.match(snapshotName).cluster;
  }

  /**
   * Parse the snapshot from Snapshot resource.
   *
   * @param {string} snapshotName
   *   A fully-qualified path representing Snapshot resource.
   * @returns {string} A string representing the snapshot.
   */
  matchSnapshotFromSnapshotName(snapshotName: string) {
    return this.pathTemplates.snapshotPathTemplate.match(snapshotName).snapshot;
  }

  /**
   * Return a fully-qualified table resource name string.
   *
   * @param {string} project
   * @param {string} instance
   * @param {string} table
   * @returns {string} Resource name string.
   */
  tablePath(project: string, instance: string, table: string) {
    return this.pathTemplates.tablePathTemplate.render({
      project: project,
      instance: instance,
      table: table,
    });
  }

  /**
   * Parse the project from Table resource.
   *
   * @param {string} tableName
   *   A fully-qualified path representing Table resource.
   * @returns {string} A string representing the project.
   */
  matchProjectFromTableName(tableName: string) {
    return this.pathTemplates.tablePathTemplate.match(tableName).project;
  }

  /**
   * Parse the instance from Table resource.
   *
   * @param {string} tableName
   *   A fully-qualified path representing Table resource.
   * @returns {string} A string representing the instance.
   */
  matchInstanceFromTableName(tableName: string) {
    return this.pathTemplates.tablePathTemplate.match(tableName).instance;
  }

  /**
   * Parse the table from Table resource.
   *
   * @param {string} tableName
   *   A fully-qualified path representing Table resource.
   * @returns {string} A string representing the table.
   */
  matchTableFromTableName(tableName: string) {
    return this.pathTemplates.tablePathTemplate.match(tableName).table;
  }

  /**
   * Terminate the gRPC channel and close the client.
   *
   * The client will no longer be usable and all future behavior is undefined.
   * @returns {Promise} A promise that resolves when the client is closed.
   */
  close(): Promise<void> {
    if (this.bigtableTableAdminStub && !this._terminated) {
      return this.bigtableTableAdminStub.then(stub => {
        this._terminated = true;
        stub.close();
        this.operationsClient.close();
      });
    }
    return Promise.resolve();
  }
}
