// Copyright 2024 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/*eslint-disable block-scoped-var, id-length, no-control-regex, no-magic-numbers, no-prototype-builtins, no-redeclare, no-shadow, no-var, sort-vars*/
(function(global, factory) { /* global define, require, module */

    /* AMD */ if (typeof define === 'function' && define.amd)
        define(["protobufjs/minimal"], factory);

    /* CommonJS */ else if (typeof require === 'function' && typeof module === 'object' && module && module.exports)
        module.exports = factory(require("google-gax/build/src/protobuf").protobufMinimal);

})(this, function($protobuf) {
    "use strict";

    // Common aliases
    var $Reader = $protobuf.Reader, $Writer = $protobuf.Writer, $util = $protobuf.util;
    
    // Exported root namespace
    var $root = $protobuf.roots._google_cloud_bigtable_protos || ($protobuf.roots._google_cloud_bigtable_protos = {});
    
    $root.google = (function() {
    
        /**
         * Namespace google.
         * @exports google
         * @namespace
         */
        var google = {};
    
        google.bigtable = (function() {
    
            /**
             * Namespace bigtable.
             * @memberof google
             * @namespace
             */
            var bigtable = {};
    
            bigtable.admin = (function() {
    
                /**
                 * Namespace admin.
                 * @memberof google.bigtable
                 * @namespace
                 */
                var admin = {};
    
                admin.v2 = (function() {
    
                    /**
                     * Namespace v2.
                     * @memberof google.bigtable.admin
                     * @namespace
                     */
                    var v2 = {};
    
                    v2.BigtableInstanceAdmin = (function() {
    
                        /**
                         * Constructs a new BigtableInstanceAdmin service.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a BigtableInstanceAdmin
                         * @extends $protobuf.rpc.Service
                         * @constructor
                         * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                         * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                         * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                         */
                        function BigtableInstanceAdmin(rpcImpl, requestDelimited, responseDelimited) {
                            $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                        }
    
                        (BigtableInstanceAdmin.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = BigtableInstanceAdmin;
    
                        /**
                         * Creates new BigtableInstanceAdmin service using the specified rpc implementation.
                         * @function create
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @static
                         * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                         * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                         * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                         * @returns {BigtableInstanceAdmin} RPC service. Useful where requests and/or responses are streamed.
                         */
                        BigtableInstanceAdmin.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                            return new this(rpcImpl, requestDelimited, responseDelimited);
                        };
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableInstanceAdmin|createInstance}.
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @typedef CreateInstanceCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.longrunning.Operation} [response] Operation
                         */
    
                        /**
                         * Calls CreateInstance.
                         * @function createInstance
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.ICreateInstanceRequest} request CreateInstanceRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableInstanceAdmin.CreateInstanceCallback} callback Node-style callback called with the error, if any, and Operation
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableInstanceAdmin.prototype.createInstance = function createInstance(request, callback) {
                            return this.rpcCall(createInstance, $root.google.bigtable.admin.v2.CreateInstanceRequest, $root.google.longrunning.Operation, request, callback);
                        }, "name", { value: "CreateInstance" });
    
                        /**
                         * Calls CreateInstance.
                         * @function createInstance
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.ICreateInstanceRequest} request CreateInstanceRequest message or plain object
                         * @returns {Promise<google.longrunning.Operation>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableInstanceAdmin|getInstance}.
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @typedef GetInstanceCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.bigtable.admin.v2.Instance} [response] Instance
                         */
    
                        /**
                         * Calls GetInstance.
                         * @function getInstance
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IGetInstanceRequest} request GetInstanceRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableInstanceAdmin.GetInstanceCallback} callback Node-style callback called with the error, if any, and Instance
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableInstanceAdmin.prototype.getInstance = function getInstance(request, callback) {
                            return this.rpcCall(getInstance, $root.google.bigtable.admin.v2.GetInstanceRequest, $root.google.bigtable.admin.v2.Instance, request, callback);
                        }, "name", { value: "GetInstance" });
    
                        /**
                         * Calls GetInstance.
                         * @function getInstance
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IGetInstanceRequest} request GetInstanceRequest message or plain object
                         * @returns {Promise<google.bigtable.admin.v2.Instance>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableInstanceAdmin|listInstances}.
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @typedef ListInstancesCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.bigtable.admin.v2.ListInstancesResponse} [response] ListInstancesResponse
                         */
    
                        /**
                         * Calls ListInstances.
                         * @function listInstances
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IListInstancesRequest} request ListInstancesRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableInstanceAdmin.ListInstancesCallback} callback Node-style callback called with the error, if any, and ListInstancesResponse
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableInstanceAdmin.prototype.listInstances = function listInstances(request, callback) {
                            return this.rpcCall(listInstances, $root.google.bigtable.admin.v2.ListInstancesRequest, $root.google.bigtable.admin.v2.ListInstancesResponse, request, callback);
                        }, "name", { value: "ListInstances" });
    
                        /**
                         * Calls ListInstances.
                         * @function listInstances
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IListInstancesRequest} request ListInstancesRequest message or plain object
                         * @returns {Promise<google.bigtable.admin.v2.ListInstancesResponse>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableInstanceAdmin|updateInstance}.
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @typedef UpdateInstanceCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.bigtable.admin.v2.Instance} [response] Instance
                         */
    
                        /**
                         * Calls UpdateInstance.
                         * @function updateInstance
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IInstance} request Instance message or plain object
                         * @param {google.bigtable.admin.v2.BigtableInstanceAdmin.UpdateInstanceCallback} callback Node-style callback called with the error, if any, and Instance
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableInstanceAdmin.prototype.updateInstance = function updateInstance(request, callback) {
                            return this.rpcCall(updateInstance, $root.google.bigtable.admin.v2.Instance, $root.google.bigtable.admin.v2.Instance, request, callback);
                        }, "name", { value: "UpdateInstance" });
    
                        /**
                         * Calls UpdateInstance.
                         * @function updateInstance
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IInstance} request Instance message or plain object
                         * @returns {Promise<google.bigtable.admin.v2.Instance>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableInstanceAdmin|partialUpdateInstance}.
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @typedef PartialUpdateInstanceCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.longrunning.Operation} [response] Operation
                         */
    
                        /**
                         * Calls PartialUpdateInstance.
                         * @function partialUpdateInstance
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IPartialUpdateInstanceRequest} request PartialUpdateInstanceRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableInstanceAdmin.PartialUpdateInstanceCallback} callback Node-style callback called with the error, if any, and Operation
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableInstanceAdmin.prototype.partialUpdateInstance = function partialUpdateInstance(request, callback) {
                            return this.rpcCall(partialUpdateInstance, $root.google.bigtable.admin.v2.PartialUpdateInstanceRequest, $root.google.longrunning.Operation, request, callback);
                        }, "name", { value: "PartialUpdateInstance" });
    
                        /**
                         * Calls PartialUpdateInstance.
                         * @function partialUpdateInstance
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IPartialUpdateInstanceRequest} request PartialUpdateInstanceRequest message or plain object
                         * @returns {Promise<google.longrunning.Operation>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableInstanceAdmin|deleteInstance}.
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @typedef DeleteInstanceCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.protobuf.Empty} [response] Empty
                         */
    
                        /**
                         * Calls DeleteInstance.
                         * @function deleteInstance
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IDeleteInstanceRequest} request DeleteInstanceRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableInstanceAdmin.DeleteInstanceCallback} callback Node-style callback called with the error, if any, and Empty
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableInstanceAdmin.prototype.deleteInstance = function deleteInstance(request, callback) {
                            return this.rpcCall(deleteInstance, $root.google.bigtable.admin.v2.DeleteInstanceRequest, $root.google.protobuf.Empty, request, callback);
                        }, "name", { value: "DeleteInstance" });
    
                        /**
                         * Calls DeleteInstance.
                         * @function deleteInstance
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IDeleteInstanceRequest} request DeleteInstanceRequest message or plain object
                         * @returns {Promise<google.protobuf.Empty>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableInstanceAdmin|createCluster}.
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @typedef CreateClusterCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.longrunning.Operation} [response] Operation
                         */
    
                        /**
                         * Calls CreateCluster.
                         * @function createCluster
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.ICreateClusterRequest} request CreateClusterRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableInstanceAdmin.CreateClusterCallback} callback Node-style callback called with the error, if any, and Operation
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableInstanceAdmin.prototype.createCluster = function createCluster(request, callback) {
                            return this.rpcCall(createCluster, $root.google.bigtable.admin.v2.CreateClusterRequest, $root.google.longrunning.Operation, request, callback);
                        }, "name", { value: "CreateCluster" });
    
                        /**
                         * Calls CreateCluster.
                         * @function createCluster
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.ICreateClusterRequest} request CreateClusterRequest message or plain object
                         * @returns {Promise<google.longrunning.Operation>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableInstanceAdmin|getCluster}.
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @typedef GetClusterCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.bigtable.admin.v2.Cluster} [response] Cluster
                         */
    
                        /**
                         * Calls GetCluster.
                         * @function getCluster
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IGetClusterRequest} request GetClusterRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableInstanceAdmin.GetClusterCallback} callback Node-style callback called with the error, if any, and Cluster
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableInstanceAdmin.prototype.getCluster = function getCluster(request, callback) {
                            return this.rpcCall(getCluster, $root.google.bigtable.admin.v2.GetClusterRequest, $root.google.bigtable.admin.v2.Cluster, request, callback);
                        }, "name", { value: "GetCluster" });
    
                        /**
                         * Calls GetCluster.
                         * @function getCluster
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IGetClusterRequest} request GetClusterRequest message or plain object
                         * @returns {Promise<google.bigtable.admin.v2.Cluster>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableInstanceAdmin|listClusters}.
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @typedef ListClustersCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.bigtable.admin.v2.ListClustersResponse} [response] ListClustersResponse
                         */
    
                        /**
                         * Calls ListClusters.
                         * @function listClusters
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IListClustersRequest} request ListClustersRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableInstanceAdmin.ListClustersCallback} callback Node-style callback called with the error, if any, and ListClustersResponse
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableInstanceAdmin.prototype.listClusters = function listClusters(request, callback) {
                            return this.rpcCall(listClusters, $root.google.bigtable.admin.v2.ListClustersRequest, $root.google.bigtable.admin.v2.ListClustersResponse, request, callback);
                        }, "name", { value: "ListClusters" });
    
                        /**
                         * Calls ListClusters.
                         * @function listClusters
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IListClustersRequest} request ListClustersRequest message or plain object
                         * @returns {Promise<google.bigtable.admin.v2.ListClustersResponse>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableInstanceAdmin|updateCluster}.
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @typedef UpdateClusterCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.longrunning.Operation} [response] Operation
                         */
    
                        /**
                         * Calls UpdateCluster.
                         * @function updateCluster
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.ICluster} request Cluster message or plain object
                         * @param {google.bigtable.admin.v2.BigtableInstanceAdmin.UpdateClusterCallback} callback Node-style callback called with the error, if any, and Operation
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableInstanceAdmin.prototype.updateCluster = function updateCluster(request, callback) {
                            return this.rpcCall(updateCluster, $root.google.bigtable.admin.v2.Cluster, $root.google.longrunning.Operation, request, callback);
                        }, "name", { value: "UpdateCluster" });
    
                        /**
                         * Calls UpdateCluster.
                         * @function updateCluster
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.ICluster} request Cluster message or plain object
                         * @returns {Promise<google.longrunning.Operation>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableInstanceAdmin|partialUpdateCluster}.
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @typedef PartialUpdateClusterCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.longrunning.Operation} [response] Operation
                         */
    
                        /**
                         * Calls PartialUpdateCluster.
                         * @function partialUpdateCluster
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IPartialUpdateClusterRequest} request PartialUpdateClusterRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableInstanceAdmin.PartialUpdateClusterCallback} callback Node-style callback called with the error, if any, and Operation
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableInstanceAdmin.prototype.partialUpdateCluster = function partialUpdateCluster(request, callback) {
                            return this.rpcCall(partialUpdateCluster, $root.google.bigtable.admin.v2.PartialUpdateClusterRequest, $root.google.longrunning.Operation, request, callback);
                        }, "name", { value: "PartialUpdateCluster" });
    
                        /**
                         * Calls PartialUpdateCluster.
                         * @function partialUpdateCluster
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IPartialUpdateClusterRequest} request PartialUpdateClusterRequest message or plain object
                         * @returns {Promise<google.longrunning.Operation>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableInstanceAdmin|deleteCluster}.
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @typedef DeleteClusterCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.protobuf.Empty} [response] Empty
                         */
    
                        /**
                         * Calls DeleteCluster.
                         * @function deleteCluster
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IDeleteClusterRequest} request DeleteClusterRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableInstanceAdmin.DeleteClusterCallback} callback Node-style callback called with the error, if any, and Empty
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableInstanceAdmin.prototype.deleteCluster = function deleteCluster(request, callback) {
                            return this.rpcCall(deleteCluster, $root.google.bigtable.admin.v2.DeleteClusterRequest, $root.google.protobuf.Empty, request, callback);
                        }, "name", { value: "DeleteCluster" });
    
                        /**
                         * Calls DeleteCluster.
                         * @function deleteCluster
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IDeleteClusterRequest} request DeleteClusterRequest message or plain object
                         * @returns {Promise<google.protobuf.Empty>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableInstanceAdmin|createAppProfile}.
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @typedef CreateAppProfileCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.bigtable.admin.v2.AppProfile} [response] AppProfile
                         */
    
                        /**
                         * Calls CreateAppProfile.
                         * @function createAppProfile
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.ICreateAppProfileRequest} request CreateAppProfileRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableInstanceAdmin.CreateAppProfileCallback} callback Node-style callback called with the error, if any, and AppProfile
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableInstanceAdmin.prototype.createAppProfile = function createAppProfile(request, callback) {
                            return this.rpcCall(createAppProfile, $root.google.bigtable.admin.v2.CreateAppProfileRequest, $root.google.bigtable.admin.v2.AppProfile, request, callback);
                        }, "name", { value: "CreateAppProfile" });
    
                        /**
                         * Calls CreateAppProfile.
                         * @function createAppProfile
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.ICreateAppProfileRequest} request CreateAppProfileRequest message or plain object
                         * @returns {Promise<google.bigtable.admin.v2.AppProfile>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableInstanceAdmin|getAppProfile}.
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @typedef GetAppProfileCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.bigtable.admin.v2.AppProfile} [response] AppProfile
                         */
    
                        /**
                         * Calls GetAppProfile.
                         * @function getAppProfile
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IGetAppProfileRequest} request GetAppProfileRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableInstanceAdmin.GetAppProfileCallback} callback Node-style callback called with the error, if any, and AppProfile
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableInstanceAdmin.prototype.getAppProfile = function getAppProfile(request, callback) {
                            return this.rpcCall(getAppProfile, $root.google.bigtable.admin.v2.GetAppProfileRequest, $root.google.bigtable.admin.v2.AppProfile, request, callback);
                        }, "name", { value: "GetAppProfile" });
    
                        /**
                         * Calls GetAppProfile.
                         * @function getAppProfile
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IGetAppProfileRequest} request GetAppProfileRequest message or plain object
                         * @returns {Promise<google.bigtable.admin.v2.AppProfile>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableInstanceAdmin|listAppProfiles}.
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @typedef ListAppProfilesCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.bigtable.admin.v2.ListAppProfilesResponse} [response] ListAppProfilesResponse
                         */
    
                        /**
                         * Calls ListAppProfiles.
                         * @function listAppProfiles
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IListAppProfilesRequest} request ListAppProfilesRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableInstanceAdmin.ListAppProfilesCallback} callback Node-style callback called with the error, if any, and ListAppProfilesResponse
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableInstanceAdmin.prototype.listAppProfiles = function listAppProfiles(request, callback) {
                            return this.rpcCall(listAppProfiles, $root.google.bigtable.admin.v2.ListAppProfilesRequest, $root.google.bigtable.admin.v2.ListAppProfilesResponse, request, callback);
                        }, "name", { value: "ListAppProfiles" });
    
                        /**
                         * Calls ListAppProfiles.
                         * @function listAppProfiles
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IListAppProfilesRequest} request ListAppProfilesRequest message or plain object
                         * @returns {Promise<google.bigtable.admin.v2.ListAppProfilesResponse>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableInstanceAdmin|updateAppProfile}.
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @typedef UpdateAppProfileCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.longrunning.Operation} [response] Operation
                         */
    
                        /**
                         * Calls UpdateAppProfile.
                         * @function updateAppProfile
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IUpdateAppProfileRequest} request UpdateAppProfileRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableInstanceAdmin.UpdateAppProfileCallback} callback Node-style callback called with the error, if any, and Operation
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableInstanceAdmin.prototype.updateAppProfile = function updateAppProfile(request, callback) {
                            return this.rpcCall(updateAppProfile, $root.google.bigtable.admin.v2.UpdateAppProfileRequest, $root.google.longrunning.Operation, request, callback);
                        }, "name", { value: "UpdateAppProfile" });
    
                        /**
                         * Calls UpdateAppProfile.
                         * @function updateAppProfile
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IUpdateAppProfileRequest} request UpdateAppProfileRequest message or plain object
                         * @returns {Promise<google.longrunning.Operation>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableInstanceAdmin|deleteAppProfile}.
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @typedef DeleteAppProfileCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.protobuf.Empty} [response] Empty
                         */
    
                        /**
                         * Calls DeleteAppProfile.
                         * @function deleteAppProfile
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IDeleteAppProfileRequest} request DeleteAppProfileRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableInstanceAdmin.DeleteAppProfileCallback} callback Node-style callback called with the error, if any, and Empty
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableInstanceAdmin.prototype.deleteAppProfile = function deleteAppProfile(request, callback) {
                            return this.rpcCall(deleteAppProfile, $root.google.bigtable.admin.v2.DeleteAppProfileRequest, $root.google.protobuf.Empty, request, callback);
                        }, "name", { value: "DeleteAppProfile" });
    
                        /**
                         * Calls DeleteAppProfile.
                         * @function deleteAppProfile
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IDeleteAppProfileRequest} request DeleteAppProfileRequest message or plain object
                         * @returns {Promise<google.protobuf.Empty>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableInstanceAdmin|getIamPolicy}.
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @typedef GetIamPolicyCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.iam.v1.Policy} [response] Policy
                         */
    
                        /**
                         * Calls GetIamPolicy.
                         * @function getIamPolicy
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.iam.v1.IGetIamPolicyRequest} request GetIamPolicyRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableInstanceAdmin.GetIamPolicyCallback} callback Node-style callback called with the error, if any, and Policy
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableInstanceAdmin.prototype.getIamPolicy = function getIamPolicy(request, callback) {
                            return this.rpcCall(getIamPolicy, $root.google.iam.v1.GetIamPolicyRequest, $root.google.iam.v1.Policy, request, callback);
                        }, "name", { value: "GetIamPolicy" });
    
                        /**
                         * Calls GetIamPolicy.
                         * @function getIamPolicy
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.iam.v1.IGetIamPolicyRequest} request GetIamPolicyRequest message or plain object
                         * @returns {Promise<google.iam.v1.Policy>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableInstanceAdmin|setIamPolicy}.
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @typedef SetIamPolicyCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.iam.v1.Policy} [response] Policy
                         */
    
                        /**
                         * Calls SetIamPolicy.
                         * @function setIamPolicy
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.iam.v1.ISetIamPolicyRequest} request SetIamPolicyRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableInstanceAdmin.SetIamPolicyCallback} callback Node-style callback called with the error, if any, and Policy
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableInstanceAdmin.prototype.setIamPolicy = function setIamPolicy(request, callback) {
                            return this.rpcCall(setIamPolicy, $root.google.iam.v1.SetIamPolicyRequest, $root.google.iam.v1.Policy, request, callback);
                        }, "name", { value: "SetIamPolicy" });
    
                        /**
                         * Calls SetIamPolicy.
                         * @function setIamPolicy
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.iam.v1.ISetIamPolicyRequest} request SetIamPolicyRequest message or plain object
                         * @returns {Promise<google.iam.v1.Policy>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableInstanceAdmin|testIamPermissions}.
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @typedef TestIamPermissionsCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.iam.v1.TestIamPermissionsResponse} [response] TestIamPermissionsResponse
                         */
    
                        /**
                         * Calls TestIamPermissions.
                         * @function testIamPermissions
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.iam.v1.ITestIamPermissionsRequest} request TestIamPermissionsRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableInstanceAdmin.TestIamPermissionsCallback} callback Node-style callback called with the error, if any, and TestIamPermissionsResponse
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableInstanceAdmin.prototype.testIamPermissions = function testIamPermissions(request, callback) {
                            return this.rpcCall(testIamPermissions, $root.google.iam.v1.TestIamPermissionsRequest, $root.google.iam.v1.TestIamPermissionsResponse, request, callback);
                        }, "name", { value: "TestIamPermissions" });
    
                        /**
                         * Calls TestIamPermissions.
                         * @function testIamPermissions
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.iam.v1.ITestIamPermissionsRequest} request TestIamPermissionsRequest message or plain object
                         * @returns {Promise<google.iam.v1.TestIamPermissionsResponse>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableInstanceAdmin|listHotTablets}.
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @typedef ListHotTabletsCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.bigtable.admin.v2.ListHotTabletsResponse} [response] ListHotTabletsResponse
                         */
    
                        /**
                         * Calls ListHotTablets.
                         * @function listHotTablets
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IListHotTabletsRequest} request ListHotTabletsRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableInstanceAdmin.ListHotTabletsCallback} callback Node-style callback called with the error, if any, and ListHotTabletsResponse
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableInstanceAdmin.prototype.listHotTablets = function listHotTablets(request, callback) {
                            return this.rpcCall(listHotTablets, $root.google.bigtable.admin.v2.ListHotTabletsRequest, $root.google.bigtable.admin.v2.ListHotTabletsResponse, request, callback);
                        }, "name", { value: "ListHotTablets" });
    
                        /**
                         * Calls ListHotTablets.
                         * @function listHotTablets
                         * @memberof google.bigtable.admin.v2.BigtableInstanceAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IListHotTabletsRequest} request ListHotTabletsRequest message or plain object
                         * @returns {Promise<google.bigtable.admin.v2.ListHotTabletsResponse>} Promise
                         * @variation 2
                         */
    
                        return BigtableInstanceAdmin;
                    })();
    
                    v2.CreateInstanceRequest = (function() {
    
                        /**
                         * Properties of a CreateInstanceRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface ICreateInstanceRequest
                         * @property {string|null} [parent] CreateInstanceRequest parent
                         * @property {string|null} [instanceId] CreateInstanceRequest instanceId
                         * @property {google.bigtable.admin.v2.IInstance|null} [instance] CreateInstanceRequest instance
                         * @property {Object.<string,google.bigtable.admin.v2.ICluster>|null} [clusters] CreateInstanceRequest clusters
                         */
    
                        /**
                         * Constructs a new CreateInstanceRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a CreateInstanceRequest.
                         * @implements ICreateInstanceRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.ICreateInstanceRequest=} [properties] Properties to set
                         */
                        function CreateInstanceRequest(properties) {
                            this.clusters = {};
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * CreateInstanceRequest parent.
                         * @member {string} parent
                         * @memberof google.bigtable.admin.v2.CreateInstanceRequest
                         * @instance
                         */
                        CreateInstanceRequest.prototype.parent = "";
    
                        /**
                         * CreateInstanceRequest instanceId.
                         * @member {string} instanceId
                         * @memberof google.bigtable.admin.v2.CreateInstanceRequest
                         * @instance
                         */
                        CreateInstanceRequest.prototype.instanceId = "";
    
                        /**
                         * CreateInstanceRequest instance.
                         * @member {google.bigtable.admin.v2.IInstance|null|undefined} instance
                         * @memberof google.bigtable.admin.v2.CreateInstanceRequest
                         * @instance
                         */
                        CreateInstanceRequest.prototype.instance = null;
    
                        /**
                         * CreateInstanceRequest clusters.
                         * @member {Object.<string,google.bigtable.admin.v2.ICluster>} clusters
                         * @memberof google.bigtable.admin.v2.CreateInstanceRequest
                         * @instance
                         */
                        CreateInstanceRequest.prototype.clusters = $util.emptyObject;
    
                        /**
                         * Creates a new CreateInstanceRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.CreateInstanceRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateInstanceRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.CreateInstanceRequest} CreateInstanceRequest instance
                         */
                        CreateInstanceRequest.create = function create(properties) {
                            return new CreateInstanceRequest(properties);
                        };
    
                        /**
                         * Encodes the specified CreateInstanceRequest message. Does not implicitly {@link google.bigtable.admin.v2.CreateInstanceRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.CreateInstanceRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateInstanceRequest} message CreateInstanceRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CreateInstanceRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.parent != null && Object.hasOwnProperty.call(message, "parent"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.parent);
                            if (message.instanceId != null && Object.hasOwnProperty.call(message, "instanceId"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.instanceId);
                            if (message.instance != null && Object.hasOwnProperty.call(message, "instance"))
                                $root.google.bigtable.admin.v2.Instance.encode(message.instance, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            if (message.clusters != null && Object.hasOwnProperty.call(message, "clusters"))
                                for (var keys = Object.keys(message.clusters), i = 0; i < keys.length; ++i) {
                                    writer.uint32(/* id 4, wireType 2 =*/34).fork().uint32(/* id 1, wireType 2 =*/10).string(keys[i]);
                                    $root.google.bigtable.admin.v2.Cluster.encode(message.clusters[keys[i]], writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim().ldelim();
                                }
                            return writer;
                        };
    
                        /**
                         * Encodes the specified CreateInstanceRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.CreateInstanceRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.CreateInstanceRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateInstanceRequest} message CreateInstanceRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CreateInstanceRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a CreateInstanceRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.CreateInstanceRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.CreateInstanceRequest} CreateInstanceRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CreateInstanceRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.CreateInstanceRequest(), key, value;
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.parent = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.instanceId = reader.string();
                                        break;
                                    }
                                case 3: {
                                        message.instance = $root.google.bigtable.admin.v2.Instance.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 4: {
                                        if (message.clusters === $util.emptyObject)
                                            message.clusters = {};
                                        var end2 = reader.uint32() + reader.pos;
                                        key = "";
                                        value = null;
                                        while (reader.pos < end2) {
                                            var tag2 = reader.uint32();
                                            switch (tag2 >>> 3) {
                                            case 1:
                                                key = reader.string();
                                                break;
                                            case 2:
                                                value = $root.google.bigtable.admin.v2.Cluster.decode(reader, reader.uint32());
                                                break;
                                            default:
                                                reader.skipType(tag2 & 7);
                                                break;
                                            }
                                        }
                                        message.clusters[key] = value;
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a CreateInstanceRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.CreateInstanceRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.CreateInstanceRequest} CreateInstanceRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CreateInstanceRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a CreateInstanceRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.CreateInstanceRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        CreateInstanceRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                if (!$util.isString(message.parent))
                                    return "parent: string expected";
                            if (message.instanceId != null && message.hasOwnProperty("instanceId"))
                                if (!$util.isString(message.instanceId))
                                    return "instanceId: string expected";
                            if (message.instance != null && message.hasOwnProperty("instance")) {
                                var error = $root.google.bigtable.admin.v2.Instance.verify(message.instance);
                                if (error)
                                    return "instance." + error;
                            }
                            if (message.clusters != null && message.hasOwnProperty("clusters")) {
                                if (!$util.isObject(message.clusters))
                                    return "clusters: object expected";
                                var key = Object.keys(message.clusters);
                                for (var i = 0; i < key.length; ++i) {
                                    var error = $root.google.bigtable.admin.v2.Cluster.verify(message.clusters[key[i]]);
                                    if (error)
                                        return "clusters." + error;
                                }
                            }
                            return null;
                        };
    
                        /**
                         * Creates a CreateInstanceRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.CreateInstanceRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.CreateInstanceRequest} CreateInstanceRequest
                         */
                        CreateInstanceRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.CreateInstanceRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.CreateInstanceRequest();
                            if (object.parent != null)
                                message.parent = String(object.parent);
                            if (object.instanceId != null)
                                message.instanceId = String(object.instanceId);
                            if (object.instance != null) {
                                if (typeof object.instance !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.CreateInstanceRequest.instance: object expected");
                                message.instance = $root.google.bigtable.admin.v2.Instance.fromObject(object.instance);
                            }
                            if (object.clusters) {
                                if (typeof object.clusters !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.CreateInstanceRequest.clusters: object expected");
                                message.clusters = {};
                                for (var keys = Object.keys(object.clusters), i = 0; i < keys.length; ++i) {
                                    if (typeof object.clusters[keys[i]] !== "object")
                                        throw TypeError(".google.bigtable.admin.v2.CreateInstanceRequest.clusters: object expected");
                                    message.clusters[keys[i]] = $root.google.bigtable.admin.v2.Cluster.fromObject(object.clusters[keys[i]]);
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a CreateInstanceRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.CreateInstanceRequest
                         * @static
                         * @param {google.bigtable.admin.v2.CreateInstanceRequest} message CreateInstanceRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        CreateInstanceRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.objects || options.defaults)
                                object.clusters = {};
                            if (options.defaults) {
                                object.parent = "";
                                object.instanceId = "";
                                object.instance = null;
                            }
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                object.parent = message.parent;
                            if (message.instanceId != null && message.hasOwnProperty("instanceId"))
                                object.instanceId = message.instanceId;
                            if (message.instance != null && message.hasOwnProperty("instance"))
                                object.instance = $root.google.bigtable.admin.v2.Instance.toObject(message.instance, options);
                            var keys2;
                            if (message.clusters && (keys2 = Object.keys(message.clusters)).length) {
                                object.clusters = {};
                                for (var j = 0; j < keys2.length; ++j)
                                    object.clusters[keys2[j]] = $root.google.bigtable.admin.v2.Cluster.toObject(message.clusters[keys2[j]], options);
                            }
                            return object;
                        };
    
                        /**
                         * Converts this CreateInstanceRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.CreateInstanceRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        CreateInstanceRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for CreateInstanceRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.CreateInstanceRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        CreateInstanceRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.CreateInstanceRequest";
                        };
    
                        return CreateInstanceRequest;
                    })();
    
                    v2.GetInstanceRequest = (function() {
    
                        /**
                         * Properties of a GetInstanceRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IGetInstanceRequest
                         * @property {string|null} [name] GetInstanceRequest name
                         */
    
                        /**
                         * Constructs a new GetInstanceRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a GetInstanceRequest.
                         * @implements IGetInstanceRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IGetInstanceRequest=} [properties] Properties to set
                         */
                        function GetInstanceRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * GetInstanceRequest name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.GetInstanceRequest
                         * @instance
                         */
                        GetInstanceRequest.prototype.name = "";
    
                        /**
                         * Creates a new GetInstanceRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.GetInstanceRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IGetInstanceRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.GetInstanceRequest} GetInstanceRequest instance
                         */
                        GetInstanceRequest.create = function create(properties) {
                            return new GetInstanceRequest(properties);
                        };
    
                        /**
                         * Encodes the specified GetInstanceRequest message. Does not implicitly {@link google.bigtable.admin.v2.GetInstanceRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.GetInstanceRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IGetInstanceRequest} message GetInstanceRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        GetInstanceRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified GetInstanceRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.GetInstanceRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.GetInstanceRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IGetInstanceRequest} message GetInstanceRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        GetInstanceRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a GetInstanceRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.GetInstanceRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.GetInstanceRequest} GetInstanceRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        GetInstanceRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.GetInstanceRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a GetInstanceRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.GetInstanceRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.GetInstanceRequest} GetInstanceRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        GetInstanceRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a GetInstanceRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.GetInstanceRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        GetInstanceRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a GetInstanceRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.GetInstanceRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.GetInstanceRequest} GetInstanceRequest
                         */
                        GetInstanceRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.GetInstanceRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.GetInstanceRequest();
                            if (object.name != null)
                                message.name = String(object.name);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a GetInstanceRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.GetInstanceRequest
                         * @static
                         * @param {google.bigtable.admin.v2.GetInstanceRequest} message GetInstanceRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        GetInstanceRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults)
                                object.name = "";
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            return object;
                        };
    
                        /**
                         * Converts this GetInstanceRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.GetInstanceRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        GetInstanceRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for GetInstanceRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.GetInstanceRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        GetInstanceRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.GetInstanceRequest";
                        };
    
                        return GetInstanceRequest;
                    })();
    
                    v2.ListInstancesRequest = (function() {
    
                        /**
                         * Properties of a ListInstancesRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IListInstancesRequest
                         * @property {string|null} [parent] ListInstancesRequest parent
                         * @property {string|null} [pageToken] ListInstancesRequest pageToken
                         */
    
                        /**
                         * Constructs a new ListInstancesRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a ListInstancesRequest.
                         * @implements IListInstancesRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IListInstancesRequest=} [properties] Properties to set
                         */
                        function ListInstancesRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * ListInstancesRequest parent.
                         * @member {string} parent
                         * @memberof google.bigtable.admin.v2.ListInstancesRequest
                         * @instance
                         */
                        ListInstancesRequest.prototype.parent = "";
    
                        /**
                         * ListInstancesRequest pageToken.
                         * @member {string} pageToken
                         * @memberof google.bigtable.admin.v2.ListInstancesRequest
                         * @instance
                         */
                        ListInstancesRequest.prototype.pageToken = "";
    
                        /**
                         * Creates a new ListInstancesRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.ListInstancesRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IListInstancesRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.ListInstancesRequest} ListInstancesRequest instance
                         */
                        ListInstancesRequest.create = function create(properties) {
                            return new ListInstancesRequest(properties);
                        };
    
                        /**
                         * Encodes the specified ListInstancesRequest message. Does not implicitly {@link google.bigtable.admin.v2.ListInstancesRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.ListInstancesRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IListInstancesRequest} message ListInstancesRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListInstancesRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.parent != null && Object.hasOwnProperty.call(message, "parent"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.parent);
                            if (message.pageToken != null && Object.hasOwnProperty.call(message, "pageToken"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.pageToken);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified ListInstancesRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.ListInstancesRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.ListInstancesRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IListInstancesRequest} message ListInstancesRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListInstancesRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a ListInstancesRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.ListInstancesRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.ListInstancesRequest} ListInstancesRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListInstancesRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.ListInstancesRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.parent = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.pageToken = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a ListInstancesRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.ListInstancesRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.ListInstancesRequest} ListInstancesRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListInstancesRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a ListInstancesRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.ListInstancesRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        ListInstancesRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                if (!$util.isString(message.parent))
                                    return "parent: string expected";
                            if (message.pageToken != null && message.hasOwnProperty("pageToken"))
                                if (!$util.isString(message.pageToken))
                                    return "pageToken: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a ListInstancesRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.ListInstancesRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.ListInstancesRequest} ListInstancesRequest
                         */
                        ListInstancesRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.ListInstancesRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.ListInstancesRequest();
                            if (object.parent != null)
                                message.parent = String(object.parent);
                            if (object.pageToken != null)
                                message.pageToken = String(object.pageToken);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a ListInstancesRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.ListInstancesRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ListInstancesRequest} message ListInstancesRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        ListInstancesRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.parent = "";
                                object.pageToken = "";
                            }
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                object.parent = message.parent;
                            if (message.pageToken != null && message.hasOwnProperty("pageToken"))
                                object.pageToken = message.pageToken;
                            return object;
                        };
    
                        /**
                         * Converts this ListInstancesRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.ListInstancesRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        ListInstancesRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for ListInstancesRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.ListInstancesRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        ListInstancesRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.ListInstancesRequest";
                        };
    
                        return ListInstancesRequest;
                    })();
    
                    v2.ListInstancesResponse = (function() {
    
                        /**
                         * Properties of a ListInstancesResponse.
                         * @memberof google.bigtable.admin.v2
                         * @interface IListInstancesResponse
                         * @property {Array.<google.bigtable.admin.v2.IInstance>|null} [instances] ListInstancesResponse instances
                         * @property {Array.<string>|null} [failedLocations] ListInstancesResponse failedLocations
                         * @property {string|null} [nextPageToken] ListInstancesResponse nextPageToken
                         */
    
                        /**
                         * Constructs a new ListInstancesResponse.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a ListInstancesResponse.
                         * @implements IListInstancesResponse
                         * @constructor
                         * @param {google.bigtable.admin.v2.IListInstancesResponse=} [properties] Properties to set
                         */
                        function ListInstancesResponse(properties) {
                            this.instances = [];
                            this.failedLocations = [];
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * ListInstancesResponse instances.
                         * @member {Array.<google.bigtable.admin.v2.IInstance>} instances
                         * @memberof google.bigtable.admin.v2.ListInstancesResponse
                         * @instance
                         */
                        ListInstancesResponse.prototype.instances = $util.emptyArray;
    
                        /**
                         * ListInstancesResponse failedLocations.
                         * @member {Array.<string>} failedLocations
                         * @memberof google.bigtable.admin.v2.ListInstancesResponse
                         * @instance
                         */
                        ListInstancesResponse.prototype.failedLocations = $util.emptyArray;
    
                        /**
                         * ListInstancesResponse nextPageToken.
                         * @member {string} nextPageToken
                         * @memberof google.bigtable.admin.v2.ListInstancesResponse
                         * @instance
                         */
                        ListInstancesResponse.prototype.nextPageToken = "";
    
                        /**
                         * Creates a new ListInstancesResponse instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.ListInstancesResponse
                         * @static
                         * @param {google.bigtable.admin.v2.IListInstancesResponse=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.ListInstancesResponse} ListInstancesResponse instance
                         */
                        ListInstancesResponse.create = function create(properties) {
                            return new ListInstancesResponse(properties);
                        };
    
                        /**
                         * Encodes the specified ListInstancesResponse message. Does not implicitly {@link google.bigtable.admin.v2.ListInstancesResponse.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.ListInstancesResponse
                         * @static
                         * @param {google.bigtable.admin.v2.IListInstancesResponse} message ListInstancesResponse message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListInstancesResponse.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.instances != null && message.instances.length)
                                for (var i = 0; i < message.instances.length; ++i)
                                    $root.google.bigtable.admin.v2.Instance.encode(message.instances[i], writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            if (message.failedLocations != null && message.failedLocations.length)
                                for (var i = 0; i < message.failedLocations.length; ++i)
                                    writer.uint32(/* id 2, wireType 2 =*/18).string(message.failedLocations[i]);
                            if (message.nextPageToken != null && Object.hasOwnProperty.call(message, "nextPageToken"))
                                writer.uint32(/* id 3, wireType 2 =*/26).string(message.nextPageToken);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified ListInstancesResponse message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.ListInstancesResponse.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.ListInstancesResponse
                         * @static
                         * @param {google.bigtable.admin.v2.IListInstancesResponse} message ListInstancesResponse message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListInstancesResponse.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a ListInstancesResponse message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.ListInstancesResponse
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.ListInstancesResponse} ListInstancesResponse
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListInstancesResponse.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.ListInstancesResponse();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        if (!(message.instances && message.instances.length))
                                            message.instances = [];
                                        message.instances.push($root.google.bigtable.admin.v2.Instance.decode(reader, reader.uint32()));
                                        break;
                                    }
                                case 2: {
                                        if (!(message.failedLocations && message.failedLocations.length))
                                            message.failedLocations = [];
                                        message.failedLocations.push(reader.string());
                                        break;
                                    }
                                case 3: {
                                        message.nextPageToken = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a ListInstancesResponse message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.ListInstancesResponse
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.ListInstancesResponse} ListInstancesResponse
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListInstancesResponse.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a ListInstancesResponse message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.ListInstancesResponse
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        ListInstancesResponse.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.instances != null && message.hasOwnProperty("instances")) {
                                if (!Array.isArray(message.instances))
                                    return "instances: array expected";
                                for (var i = 0; i < message.instances.length; ++i) {
                                    var error = $root.google.bigtable.admin.v2.Instance.verify(message.instances[i]);
                                    if (error)
                                        return "instances." + error;
                                }
                            }
                            if (message.failedLocations != null && message.hasOwnProperty("failedLocations")) {
                                if (!Array.isArray(message.failedLocations))
                                    return "failedLocations: array expected";
                                for (var i = 0; i < message.failedLocations.length; ++i)
                                    if (!$util.isString(message.failedLocations[i]))
                                        return "failedLocations: string[] expected";
                            }
                            if (message.nextPageToken != null && message.hasOwnProperty("nextPageToken"))
                                if (!$util.isString(message.nextPageToken))
                                    return "nextPageToken: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a ListInstancesResponse message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.ListInstancesResponse
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.ListInstancesResponse} ListInstancesResponse
                         */
                        ListInstancesResponse.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.ListInstancesResponse)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.ListInstancesResponse();
                            if (object.instances) {
                                if (!Array.isArray(object.instances))
                                    throw TypeError(".google.bigtable.admin.v2.ListInstancesResponse.instances: array expected");
                                message.instances = [];
                                for (var i = 0; i < object.instances.length; ++i) {
                                    if (typeof object.instances[i] !== "object")
                                        throw TypeError(".google.bigtable.admin.v2.ListInstancesResponse.instances: object expected");
                                    message.instances[i] = $root.google.bigtable.admin.v2.Instance.fromObject(object.instances[i]);
                                }
                            }
                            if (object.failedLocations) {
                                if (!Array.isArray(object.failedLocations))
                                    throw TypeError(".google.bigtable.admin.v2.ListInstancesResponse.failedLocations: array expected");
                                message.failedLocations = [];
                                for (var i = 0; i < object.failedLocations.length; ++i)
                                    message.failedLocations[i] = String(object.failedLocations[i]);
                            }
                            if (object.nextPageToken != null)
                                message.nextPageToken = String(object.nextPageToken);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a ListInstancesResponse message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.ListInstancesResponse
                         * @static
                         * @param {google.bigtable.admin.v2.ListInstancesResponse} message ListInstancesResponse
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        ListInstancesResponse.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.arrays || options.defaults) {
                                object.instances = [];
                                object.failedLocations = [];
                            }
                            if (options.defaults)
                                object.nextPageToken = "";
                            if (message.instances && message.instances.length) {
                                object.instances = [];
                                for (var j = 0; j < message.instances.length; ++j)
                                    object.instances[j] = $root.google.bigtable.admin.v2.Instance.toObject(message.instances[j], options);
                            }
                            if (message.failedLocations && message.failedLocations.length) {
                                object.failedLocations = [];
                                for (var j = 0; j < message.failedLocations.length; ++j)
                                    object.failedLocations[j] = message.failedLocations[j];
                            }
                            if (message.nextPageToken != null && message.hasOwnProperty("nextPageToken"))
                                object.nextPageToken = message.nextPageToken;
                            return object;
                        };
    
                        /**
                         * Converts this ListInstancesResponse to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.ListInstancesResponse
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        ListInstancesResponse.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for ListInstancesResponse
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.ListInstancesResponse
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        ListInstancesResponse.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.ListInstancesResponse";
                        };
    
                        return ListInstancesResponse;
                    })();
    
                    v2.PartialUpdateInstanceRequest = (function() {
    
                        /**
                         * Properties of a PartialUpdateInstanceRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IPartialUpdateInstanceRequest
                         * @property {google.bigtable.admin.v2.IInstance|null} [instance] PartialUpdateInstanceRequest instance
                         * @property {google.protobuf.IFieldMask|null} [updateMask] PartialUpdateInstanceRequest updateMask
                         */
    
                        /**
                         * Constructs a new PartialUpdateInstanceRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a PartialUpdateInstanceRequest.
                         * @implements IPartialUpdateInstanceRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IPartialUpdateInstanceRequest=} [properties] Properties to set
                         */
                        function PartialUpdateInstanceRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * PartialUpdateInstanceRequest instance.
                         * @member {google.bigtable.admin.v2.IInstance|null|undefined} instance
                         * @memberof google.bigtable.admin.v2.PartialUpdateInstanceRequest
                         * @instance
                         */
                        PartialUpdateInstanceRequest.prototype.instance = null;
    
                        /**
                         * PartialUpdateInstanceRequest updateMask.
                         * @member {google.protobuf.IFieldMask|null|undefined} updateMask
                         * @memberof google.bigtable.admin.v2.PartialUpdateInstanceRequest
                         * @instance
                         */
                        PartialUpdateInstanceRequest.prototype.updateMask = null;
    
                        /**
                         * Creates a new PartialUpdateInstanceRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.PartialUpdateInstanceRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IPartialUpdateInstanceRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.PartialUpdateInstanceRequest} PartialUpdateInstanceRequest instance
                         */
                        PartialUpdateInstanceRequest.create = function create(properties) {
                            return new PartialUpdateInstanceRequest(properties);
                        };
    
                        /**
                         * Encodes the specified PartialUpdateInstanceRequest message. Does not implicitly {@link google.bigtable.admin.v2.PartialUpdateInstanceRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.PartialUpdateInstanceRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IPartialUpdateInstanceRequest} message PartialUpdateInstanceRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        PartialUpdateInstanceRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.instance != null && Object.hasOwnProperty.call(message, "instance"))
                                $root.google.bigtable.admin.v2.Instance.encode(message.instance, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            if (message.updateMask != null && Object.hasOwnProperty.call(message, "updateMask"))
                                $root.google.protobuf.FieldMask.encode(message.updateMask, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified PartialUpdateInstanceRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.PartialUpdateInstanceRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.PartialUpdateInstanceRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IPartialUpdateInstanceRequest} message PartialUpdateInstanceRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        PartialUpdateInstanceRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a PartialUpdateInstanceRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.PartialUpdateInstanceRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.PartialUpdateInstanceRequest} PartialUpdateInstanceRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        PartialUpdateInstanceRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.PartialUpdateInstanceRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.instance = $root.google.bigtable.admin.v2.Instance.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 2: {
                                        message.updateMask = $root.google.protobuf.FieldMask.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a PartialUpdateInstanceRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.PartialUpdateInstanceRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.PartialUpdateInstanceRequest} PartialUpdateInstanceRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        PartialUpdateInstanceRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a PartialUpdateInstanceRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.PartialUpdateInstanceRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        PartialUpdateInstanceRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.instance != null && message.hasOwnProperty("instance")) {
                                var error = $root.google.bigtable.admin.v2.Instance.verify(message.instance);
                                if (error)
                                    return "instance." + error;
                            }
                            if (message.updateMask != null && message.hasOwnProperty("updateMask")) {
                                var error = $root.google.protobuf.FieldMask.verify(message.updateMask);
                                if (error)
                                    return "updateMask." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates a PartialUpdateInstanceRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.PartialUpdateInstanceRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.PartialUpdateInstanceRequest} PartialUpdateInstanceRequest
                         */
                        PartialUpdateInstanceRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.PartialUpdateInstanceRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.PartialUpdateInstanceRequest();
                            if (object.instance != null) {
                                if (typeof object.instance !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.PartialUpdateInstanceRequest.instance: object expected");
                                message.instance = $root.google.bigtable.admin.v2.Instance.fromObject(object.instance);
                            }
                            if (object.updateMask != null) {
                                if (typeof object.updateMask !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.PartialUpdateInstanceRequest.updateMask: object expected");
                                message.updateMask = $root.google.protobuf.FieldMask.fromObject(object.updateMask);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a PartialUpdateInstanceRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.PartialUpdateInstanceRequest
                         * @static
                         * @param {google.bigtable.admin.v2.PartialUpdateInstanceRequest} message PartialUpdateInstanceRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        PartialUpdateInstanceRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.instance = null;
                                object.updateMask = null;
                            }
                            if (message.instance != null && message.hasOwnProperty("instance"))
                                object.instance = $root.google.bigtable.admin.v2.Instance.toObject(message.instance, options);
                            if (message.updateMask != null && message.hasOwnProperty("updateMask"))
                                object.updateMask = $root.google.protobuf.FieldMask.toObject(message.updateMask, options);
                            return object;
                        };
    
                        /**
                         * Converts this PartialUpdateInstanceRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.PartialUpdateInstanceRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        PartialUpdateInstanceRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for PartialUpdateInstanceRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.PartialUpdateInstanceRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        PartialUpdateInstanceRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.PartialUpdateInstanceRequest";
                        };
    
                        return PartialUpdateInstanceRequest;
                    })();
    
                    v2.DeleteInstanceRequest = (function() {
    
                        /**
                         * Properties of a DeleteInstanceRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IDeleteInstanceRequest
                         * @property {string|null} [name] DeleteInstanceRequest name
                         */
    
                        /**
                         * Constructs a new DeleteInstanceRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a DeleteInstanceRequest.
                         * @implements IDeleteInstanceRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IDeleteInstanceRequest=} [properties] Properties to set
                         */
                        function DeleteInstanceRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * DeleteInstanceRequest name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.DeleteInstanceRequest
                         * @instance
                         */
                        DeleteInstanceRequest.prototype.name = "";
    
                        /**
                         * Creates a new DeleteInstanceRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.DeleteInstanceRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IDeleteInstanceRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.DeleteInstanceRequest} DeleteInstanceRequest instance
                         */
                        DeleteInstanceRequest.create = function create(properties) {
                            return new DeleteInstanceRequest(properties);
                        };
    
                        /**
                         * Encodes the specified DeleteInstanceRequest message. Does not implicitly {@link google.bigtable.admin.v2.DeleteInstanceRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.DeleteInstanceRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IDeleteInstanceRequest} message DeleteInstanceRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        DeleteInstanceRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified DeleteInstanceRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.DeleteInstanceRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.DeleteInstanceRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IDeleteInstanceRequest} message DeleteInstanceRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        DeleteInstanceRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a DeleteInstanceRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.DeleteInstanceRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.DeleteInstanceRequest} DeleteInstanceRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        DeleteInstanceRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.DeleteInstanceRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a DeleteInstanceRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.DeleteInstanceRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.DeleteInstanceRequest} DeleteInstanceRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        DeleteInstanceRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a DeleteInstanceRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.DeleteInstanceRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        DeleteInstanceRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a DeleteInstanceRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.DeleteInstanceRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.DeleteInstanceRequest} DeleteInstanceRequest
                         */
                        DeleteInstanceRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.DeleteInstanceRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.DeleteInstanceRequest();
                            if (object.name != null)
                                message.name = String(object.name);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a DeleteInstanceRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.DeleteInstanceRequest
                         * @static
                         * @param {google.bigtable.admin.v2.DeleteInstanceRequest} message DeleteInstanceRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        DeleteInstanceRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults)
                                object.name = "";
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            return object;
                        };
    
                        /**
                         * Converts this DeleteInstanceRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.DeleteInstanceRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        DeleteInstanceRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for DeleteInstanceRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.DeleteInstanceRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        DeleteInstanceRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.DeleteInstanceRequest";
                        };
    
                        return DeleteInstanceRequest;
                    })();
    
                    v2.CreateClusterRequest = (function() {
    
                        /**
                         * Properties of a CreateClusterRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface ICreateClusterRequest
                         * @property {string|null} [parent] CreateClusterRequest parent
                         * @property {string|null} [clusterId] CreateClusterRequest clusterId
                         * @property {google.bigtable.admin.v2.ICluster|null} [cluster] CreateClusterRequest cluster
                         */
    
                        /**
                         * Constructs a new CreateClusterRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a CreateClusterRequest.
                         * @implements ICreateClusterRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.ICreateClusterRequest=} [properties] Properties to set
                         */
                        function CreateClusterRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * CreateClusterRequest parent.
                         * @member {string} parent
                         * @memberof google.bigtable.admin.v2.CreateClusterRequest
                         * @instance
                         */
                        CreateClusterRequest.prototype.parent = "";
    
                        /**
                         * CreateClusterRequest clusterId.
                         * @member {string} clusterId
                         * @memberof google.bigtable.admin.v2.CreateClusterRequest
                         * @instance
                         */
                        CreateClusterRequest.prototype.clusterId = "";
    
                        /**
                         * CreateClusterRequest cluster.
                         * @member {google.bigtable.admin.v2.ICluster|null|undefined} cluster
                         * @memberof google.bigtable.admin.v2.CreateClusterRequest
                         * @instance
                         */
                        CreateClusterRequest.prototype.cluster = null;
    
                        /**
                         * Creates a new CreateClusterRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.CreateClusterRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateClusterRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.CreateClusterRequest} CreateClusterRequest instance
                         */
                        CreateClusterRequest.create = function create(properties) {
                            return new CreateClusterRequest(properties);
                        };
    
                        /**
                         * Encodes the specified CreateClusterRequest message. Does not implicitly {@link google.bigtable.admin.v2.CreateClusterRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.CreateClusterRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateClusterRequest} message CreateClusterRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CreateClusterRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.parent != null && Object.hasOwnProperty.call(message, "parent"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.parent);
                            if (message.clusterId != null && Object.hasOwnProperty.call(message, "clusterId"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.clusterId);
                            if (message.cluster != null && Object.hasOwnProperty.call(message, "cluster"))
                                $root.google.bigtable.admin.v2.Cluster.encode(message.cluster, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified CreateClusterRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.CreateClusterRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.CreateClusterRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateClusterRequest} message CreateClusterRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CreateClusterRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a CreateClusterRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.CreateClusterRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.CreateClusterRequest} CreateClusterRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CreateClusterRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.CreateClusterRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.parent = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.clusterId = reader.string();
                                        break;
                                    }
                                case 3: {
                                        message.cluster = $root.google.bigtable.admin.v2.Cluster.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a CreateClusterRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.CreateClusterRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.CreateClusterRequest} CreateClusterRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CreateClusterRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a CreateClusterRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.CreateClusterRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        CreateClusterRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                if (!$util.isString(message.parent))
                                    return "parent: string expected";
                            if (message.clusterId != null && message.hasOwnProperty("clusterId"))
                                if (!$util.isString(message.clusterId))
                                    return "clusterId: string expected";
                            if (message.cluster != null && message.hasOwnProperty("cluster")) {
                                var error = $root.google.bigtable.admin.v2.Cluster.verify(message.cluster);
                                if (error)
                                    return "cluster." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates a CreateClusterRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.CreateClusterRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.CreateClusterRequest} CreateClusterRequest
                         */
                        CreateClusterRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.CreateClusterRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.CreateClusterRequest();
                            if (object.parent != null)
                                message.parent = String(object.parent);
                            if (object.clusterId != null)
                                message.clusterId = String(object.clusterId);
                            if (object.cluster != null) {
                                if (typeof object.cluster !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.CreateClusterRequest.cluster: object expected");
                                message.cluster = $root.google.bigtable.admin.v2.Cluster.fromObject(object.cluster);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a CreateClusterRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.CreateClusterRequest
                         * @static
                         * @param {google.bigtable.admin.v2.CreateClusterRequest} message CreateClusterRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        CreateClusterRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.parent = "";
                                object.clusterId = "";
                                object.cluster = null;
                            }
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                object.parent = message.parent;
                            if (message.clusterId != null && message.hasOwnProperty("clusterId"))
                                object.clusterId = message.clusterId;
                            if (message.cluster != null && message.hasOwnProperty("cluster"))
                                object.cluster = $root.google.bigtable.admin.v2.Cluster.toObject(message.cluster, options);
                            return object;
                        };
    
                        /**
                         * Converts this CreateClusterRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.CreateClusterRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        CreateClusterRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for CreateClusterRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.CreateClusterRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        CreateClusterRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.CreateClusterRequest";
                        };
    
                        return CreateClusterRequest;
                    })();
    
                    v2.GetClusterRequest = (function() {
    
                        /**
                         * Properties of a GetClusterRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IGetClusterRequest
                         * @property {string|null} [name] GetClusterRequest name
                         */
    
                        /**
                         * Constructs a new GetClusterRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a GetClusterRequest.
                         * @implements IGetClusterRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IGetClusterRequest=} [properties] Properties to set
                         */
                        function GetClusterRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * GetClusterRequest name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.GetClusterRequest
                         * @instance
                         */
                        GetClusterRequest.prototype.name = "";
    
                        /**
                         * Creates a new GetClusterRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.GetClusterRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IGetClusterRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.GetClusterRequest} GetClusterRequest instance
                         */
                        GetClusterRequest.create = function create(properties) {
                            return new GetClusterRequest(properties);
                        };
    
                        /**
                         * Encodes the specified GetClusterRequest message. Does not implicitly {@link google.bigtable.admin.v2.GetClusterRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.GetClusterRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IGetClusterRequest} message GetClusterRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        GetClusterRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified GetClusterRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.GetClusterRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.GetClusterRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IGetClusterRequest} message GetClusterRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        GetClusterRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a GetClusterRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.GetClusterRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.GetClusterRequest} GetClusterRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        GetClusterRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.GetClusterRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a GetClusterRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.GetClusterRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.GetClusterRequest} GetClusterRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        GetClusterRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a GetClusterRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.GetClusterRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        GetClusterRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a GetClusterRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.GetClusterRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.GetClusterRequest} GetClusterRequest
                         */
                        GetClusterRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.GetClusterRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.GetClusterRequest();
                            if (object.name != null)
                                message.name = String(object.name);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a GetClusterRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.GetClusterRequest
                         * @static
                         * @param {google.bigtable.admin.v2.GetClusterRequest} message GetClusterRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        GetClusterRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults)
                                object.name = "";
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            return object;
                        };
    
                        /**
                         * Converts this GetClusterRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.GetClusterRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        GetClusterRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for GetClusterRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.GetClusterRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        GetClusterRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.GetClusterRequest";
                        };
    
                        return GetClusterRequest;
                    })();
    
                    v2.ListClustersRequest = (function() {
    
                        /**
                         * Properties of a ListClustersRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IListClustersRequest
                         * @property {string|null} [parent] ListClustersRequest parent
                         * @property {string|null} [pageToken] ListClustersRequest pageToken
                         */
    
                        /**
                         * Constructs a new ListClustersRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a ListClustersRequest.
                         * @implements IListClustersRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IListClustersRequest=} [properties] Properties to set
                         */
                        function ListClustersRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * ListClustersRequest parent.
                         * @member {string} parent
                         * @memberof google.bigtable.admin.v2.ListClustersRequest
                         * @instance
                         */
                        ListClustersRequest.prototype.parent = "";
    
                        /**
                         * ListClustersRequest pageToken.
                         * @member {string} pageToken
                         * @memberof google.bigtable.admin.v2.ListClustersRequest
                         * @instance
                         */
                        ListClustersRequest.prototype.pageToken = "";
    
                        /**
                         * Creates a new ListClustersRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.ListClustersRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IListClustersRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.ListClustersRequest} ListClustersRequest instance
                         */
                        ListClustersRequest.create = function create(properties) {
                            return new ListClustersRequest(properties);
                        };
    
                        /**
                         * Encodes the specified ListClustersRequest message. Does not implicitly {@link google.bigtable.admin.v2.ListClustersRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.ListClustersRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IListClustersRequest} message ListClustersRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListClustersRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.parent != null && Object.hasOwnProperty.call(message, "parent"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.parent);
                            if (message.pageToken != null && Object.hasOwnProperty.call(message, "pageToken"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.pageToken);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified ListClustersRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.ListClustersRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.ListClustersRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IListClustersRequest} message ListClustersRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListClustersRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a ListClustersRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.ListClustersRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.ListClustersRequest} ListClustersRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListClustersRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.ListClustersRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.parent = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.pageToken = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a ListClustersRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.ListClustersRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.ListClustersRequest} ListClustersRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListClustersRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a ListClustersRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.ListClustersRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        ListClustersRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                if (!$util.isString(message.parent))
                                    return "parent: string expected";
                            if (message.pageToken != null && message.hasOwnProperty("pageToken"))
                                if (!$util.isString(message.pageToken))
                                    return "pageToken: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a ListClustersRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.ListClustersRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.ListClustersRequest} ListClustersRequest
                         */
                        ListClustersRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.ListClustersRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.ListClustersRequest();
                            if (object.parent != null)
                                message.parent = String(object.parent);
                            if (object.pageToken != null)
                                message.pageToken = String(object.pageToken);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a ListClustersRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.ListClustersRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ListClustersRequest} message ListClustersRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        ListClustersRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.parent = "";
                                object.pageToken = "";
                            }
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                object.parent = message.parent;
                            if (message.pageToken != null && message.hasOwnProperty("pageToken"))
                                object.pageToken = message.pageToken;
                            return object;
                        };
    
                        /**
                         * Converts this ListClustersRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.ListClustersRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        ListClustersRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for ListClustersRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.ListClustersRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        ListClustersRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.ListClustersRequest";
                        };
    
                        return ListClustersRequest;
                    })();
    
                    v2.ListClustersResponse = (function() {
    
                        /**
                         * Properties of a ListClustersResponse.
                         * @memberof google.bigtable.admin.v2
                         * @interface IListClustersResponse
                         * @property {Array.<google.bigtable.admin.v2.ICluster>|null} [clusters] ListClustersResponse clusters
                         * @property {Array.<string>|null} [failedLocations] ListClustersResponse failedLocations
                         * @property {string|null} [nextPageToken] ListClustersResponse nextPageToken
                         */
    
                        /**
                         * Constructs a new ListClustersResponse.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a ListClustersResponse.
                         * @implements IListClustersResponse
                         * @constructor
                         * @param {google.bigtable.admin.v2.IListClustersResponse=} [properties] Properties to set
                         */
                        function ListClustersResponse(properties) {
                            this.clusters = [];
                            this.failedLocations = [];
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * ListClustersResponse clusters.
                         * @member {Array.<google.bigtable.admin.v2.ICluster>} clusters
                         * @memberof google.bigtable.admin.v2.ListClustersResponse
                         * @instance
                         */
                        ListClustersResponse.prototype.clusters = $util.emptyArray;
    
                        /**
                         * ListClustersResponse failedLocations.
                         * @member {Array.<string>} failedLocations
                         * @memberof google.bigtable.admin.v2.ListClustersResponse
                         * @instance
                         */
                        ListClustersResponse.prototype.failedLocations = $util.emptyArray;
    
                        /**
                         * ListClustersResponse nextPageToken.
                         * @member {string} nextPageToken
                         * @memberof google.bigtable.admin.v2.ListClustersResponse
                         * @instance
                         */
                        ListClustersResponse.prototype.nextPageToken = "";
    
                        /**
                         * Creates a new ListClustersResponse instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.ListClustersResponse
                         * @static
                         * @param {google.bigtable.admin.v2.IListClustersResponse=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.ListClustersResponse} ListClustersResponse instance
                         */
                        ListClustersResponse.create = function create(properties) {
                            return new ListClustersResponse(properties);
                        };
    
                        /**
                         * Encodes the specified ListClustersResponse message. Does not implicitly {@link google.bigtable.admin.v2.ListClustersResponse.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.ListClustersResponse
                         * @static
                         * @param {google.bigtable.admin.v2.IListClustersResponse} message ListClustersResponse message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListClustersResponse.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.clusters != null && message.clusters.length)
                                for (var i = 0; i < message.clusters.length; ++i)
                                    $root.google.bigtable.admin.v2.Cluster.encode(message.clusters[i], writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            if (message.failedLocations != null && message.failedLocations.length)
                                for (var i = 0; i < message.failedLocations.length; ++i)
                                    writer.uint32(/* id 2, wireType 2 =*/18).string(message.failedLocations[i]);
                            if (message.nextPageToken != null && Object.hasOwnProperty.call(message, "nextPageToken"))
                                writer.uint32(/* id 3, wireType 2 =*/26).string(message.nextPageToken);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified ListClustersResponse message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.ListClustersResponse.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.ListClustersResponse
                         * @static
                         * @param {google.bigtable.admin.v2.IListClustersResponse} message ListClustersResponse message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListClustersResponse.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a ListClustersResponse message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.ListClustersResponse
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.ListClustersResponse} ListClustersResponse
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListClustersResponse.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.ListClustersResponse();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        if (!(message.clusters && message.clusters.length))
                                            message.clusters = [];
                                        message.clusters.push($root.google.bigtable.admin.v2.Cluster.decode(reader, reader.uint32()));
                                        break;
                                    }
                                case 2: {
                                        if (!(message.failedLocations && message.failedLocations.length))
                                            message.failedLocations = [];
                                        message.failedLocations.push(reader.string());
                                        break;
                                    }
                                case 3: {
                                        message.nextPageToken = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a ListClustersResponse message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.ListClustersResponse
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.ListClustersResponse} ListClustersResponse
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListClustersResponse.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a ListClustersResponse message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.ListClustersResponse
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        ListClustersResponse.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.clusters != null && message.hasOwnProperty("clusters")) {
                                if (!Array.isArray(message.clusters))
                                    return "clusters: array expected";
                                for (var i = 0; i < message.clusters.length; ++i) {
                                    var error = $root.google.bigtable.admin.v2.Cluster.verify(message.clusters[i]);
                                    if (error)
                                        return "clusters." + error;
                                }
                            }
                            if (message.failedLocations != null && message.hasOwnProperty("failedLocations")) {
                                if (!Array.isArray(message.failedLocations))
                                    return "failedLocations: array expected";
                                for (var i = 0; i < message.failedLocations.length; ++i)
                                    if (!$util.isString(message.failedLocations[i]))
                                        return "failedLocations: string[] expected";
                            }
                            if (message.nextPageToken != null && message.hasOwnProperty("nextPageToken"))
                                if (!$util.isString(message.nextPageToken))
                                    return "nextPageToken: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a ListClustersResponse message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.ListClustersResponse
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.ListClustersResponse} ListClustersResponse
                         */
                        ListClustersResponse.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.ListClustersResponse)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.ListClustersResponse();
                            if (object.clusters) {
                                if (!Array.isArray(object.clusters))
                                    throw TypeError(".google.bigtable.admin.v2.ListClustersResponse.clusters: array expected");
                                message.clusters = [];
                                for (var i = 0; i < object.clusters.length; ++i) {
                                    if (typeof object.clusters[i] !== "object")
                                        throw TypeError(".google.bigtable.admin.v2.ListClustersResponse.clusters: object expected");
                                    message.clusters[i] = $root.google.bigtable.admin.v2.Cluster.fromObject(object.clusters[i]);
                                }
                            }
                            if (object.failedLocations) {
                                if (!Array.isArray(object.failedLocations))
                                    throw TypeError(".google.bigtable.admin.v2.ListClustersResponse.failedLocations: array expected");
                                message.failedLocations = [];
                                for (var i = 0; i < object.failedLocations.length; ++i)
                                    message.failedLocations[i] = String(object.failedLocations[i]);
                            }
                            if (object.nextPageToken != null)
                                message.nextPageToken = String(object.nextPageToken);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a ListClustersResponse message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.ListClustersResponse
                         * @static
                         * @param {google.bigtable.admin.v2.ListClustersResponse} message ListClustersResponse
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        ListClustersResponse.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.arrays || options.defaults) {
                                object.clusters = [];
                                object.failedLocations = [];
                            }
                            if (options.defaults)
                                object.nextPageToken = "";
                            if (message.clusters && message.clusters.length) {
                                object.clusters = [];
                                for (var j = 0; j < message.clusters.length; ++j)
                                    object.clusters[j] = $root.google.bigtable.admin.v2.Cluster.toObject(message.clusters[j], options);
                            }
                            if (message.failedLocations && message.failedLocations.length) {
                                object.failedLocations = [];
                                for (var j = 0; j < message.failedLocations.length; ++j)
                                    object.failedLocations[j] = message.failedLocations[j];
                            }
                            if (message.nextPageToken != null && message.hasOwnProperty("nextPageToken"))
                                object.nextPageToken = message.nextPageToken;
                            return object;
                        };
    
                        /**
                         * Converts this ListClustersResponse to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.ListClustersResponse
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        ListClustersResponse.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for ListClustersResponse
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.ListClustersResponse
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        ListClustersResponse.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.ListClustersResponse";
                        };
    
                        return ListClustersResponse;
                    })();
    
                    v2.DeleteClusterRequest = (function() {
    
                        /**
                         * Properties of a DeleteClusterRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IDeleteClusterRequest
                         * @property {string|null} [name] DeleteClusterRequest name
                         */
    
                        /**
                         * Constructs a new DeleteClusterRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a DeleteClusterRequest.
                         * @implements IDeleteClusterRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IDeleteClusterRequest=} [properties] Properties to set
                         */
                        function DeleteClusterRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * DeleteClusterRequest name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.DeleteClusterRequest
                         * @instance
                         */
                        DeleteClusterRequest.prototype.name = "";
    
                        /**
                         * Creates a new DeleteClusterRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.DeleteClusterRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IDeleteClusterRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.DeleteClusterRequest} DeleteClusterRequest instance
                         */
                        DeleteClusterRequest.create = function create(properties) {
                            return new DeleteClusterRequest(properties);
                        };
    
                        /**
                         * Encodes the specified DeleteClusterRequest message. Does not implicitly {@link google.bigtable.admin.v2.DeleteClusterRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.DeleteClusterRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IDeleteClusterRequest} message DeleteClusterRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        DeleteClusterRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified DeleteClusterRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.DeleteClusterRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.DeleteClusterRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IDeleteClusterRequest} message DeleteClusterRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        DeleteClusterRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a DeleteClusterRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.DeleteClusterRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.DeleteClusterRequest} DeleteClusterRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        DeleteClusterRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.DeleteClusterRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a DeleteClusterRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.DeleteClusterRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.DeleteClusterRequest} DeleteClusterRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        DeleteClusterRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a DeleteClusterRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.DeleteClusterRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        DeleteClusterRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a DeleteClusterRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.DeleteClusterRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.DeleteClusterRequest} DeleteClusterRequest
                         */
                        DeleteClusterRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.DeleteClusterRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.DeleteClusterRequest();
                            if (object.name != null)
                                message.name = String(object.name);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a DeleteClusterRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.DeleteClusterRequest
                         * @static
                         * @param {google.bigtable.admin.v2.DeleteClusterRequest} message DeleteClusterRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        DeleteClusterRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults)
                                object.name = "";
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            return object;
                        };
    
                        /**
                         * Converts this DeleteClusterRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.DeleteClusterRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        DeleteClusterRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for DeleteClusterRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.DeleteClusterRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        DeleteClusterRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.DeleteClusterRequest";
                        };
    
                        return DeleteClusterRequest;
                    })();
    
                    v2.CreateInstanceMetadata = (function() {
    
                        /**
                         * Properties of a CreateInstanceMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @interface ICreateInstanceMetadata
                         * @property {google.bigtable.admin.v2.ICreateInstanceRequest|null} [originalRequest] CreateInstanceMetadata originalRequest
                         * @property {google.protobuf.ITimestamp|null} [requestTime] CreateInstanceMetadata requestTime
                         * @property {google.protobuf.ITimestamp|null} [finishTime] CreateInstanceMetadata finishTime
                         */
    
                        /**
                         * Constructs a new CreateInstanceMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a CreateInstanceMetadata.
                         * @implements ICreateInstanceMetadata
                         * @constructor
                         * @param {google.bigtable.admin.v2.ICreateInstanceMetadata=} [properties] Properties to set
                         */
                        function CreateInstanceMetadata(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * CreateInstanceMetadata originalRequest.
                         * @member {google.bigtable.admin.v2.ICreateInstanceRequest|null|undefined} originalRequest
                         * @memberof google.bigtable.admin.v2.CreateInstanceMetadata
                         * @instance
                         */
                        CreateInstanceMetadata.prototype.originalRequest = null;
    
                        /**
                         * CreateInstanceMetadata requestTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} requestTime
                         * @memberof google.bigtable.admin.v2.CreateInstanceMetadata
                         * @instance
                         */
                        CreateInstanceMetadata.prototype.requestTime = null;
    
                        /**
                         * CreateInstanceMetadata finishTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} finishTime
                         * @memberof google.bigtable.admin.v2.CreateInstanceMetadata
                         * @instance
                         */
                        CreateInstanceMetadata.prototype.finishTime = null;
    
                        /**
                         * Creates a new CreateInstanceMetadata instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.CreateInstanceMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateInstanceMetadata=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.CreateInstanceMetadata} CreateInstanceMetadata instance
                         */
                        CreateInstanceMetadata.create = function create(properties) {
                            return new CreateInstanceMetadata(properties);
                        };
    
                        /**
                         * Encodes the specified CreateInstanceMetadata message. Does not implicitly {@link google.bigtable.admin.v2.CreateInstanceMetadata.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.CreateInstanceMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateInstanceMetadata} message CreateInstanceMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CreateInstanceMetadata.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.originalRequest != null && Object.hasOwnProperty.call(message, "originalRequest"))
                                $root.google.bigtable.admin.v2.CreateInstanceRequest.encode(message.originalRequest, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            if (message.requestTime != null && Object.hasOwnProperty.call(message, "requestTime"))
                                $root.google.protobuf.Timestamp.encode(message.requestTime, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            if (message.finishTime != null && Object.hasOwnProperty.call(message, "finishTime"))
                                $root.google.protobuf.Timestamp.encode(message.finishTime, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified CreateInstanceMetadata message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.CreateInstanceMetadata.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.CreateInstanceMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateInstanceMetadata} message CreateInstanceMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CreateInstanceMetadata.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a CreateInstanceMetadata message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.CreateInstanceMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.CreateInstanceMetadata} CreateInstanceMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CreateInstanceMetadata.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.CreateInstanceMetadata();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.originalRequest = $root.google.bigtable.admin.v2.CreateInstanceRequest.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 2: {
                                        message.requestTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 3: {
                                        message.finishTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a CreateInstanceMetadata message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.CreateInstanceMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.CreateInstanceMetadata} CreateInstanceMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CreateInstanceMetadata.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a CreateInstanceMetadata message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.CreateInstanceMetadata
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        CreateInstanceMetadata.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.originalRequest != null && message.hasOwnProperty("originalRequest")) {
                                var error = $root.google.bigtable.admin.v2.CreateInstanceRequest.verify(message.originalRequest);
                                if (error)
                                    return "originalRequest." + error;
                            }
                            if (message.requestTime != null && message.hasOwnProperty("requestTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.requestTime);
                                if (error)
                                    return "requestTime." + error;
                            }
                            if (message.finishTime != null && message.hasOwnProperty("finishTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.finishTime);
                                if (error)
                                    return "finishTime." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates a CreateInstanceMetadata message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.CreateInstanceMetadata
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.CreateInstanceMetadata} CreateInstanceMetadata
                         */
                        CreateInstanceMetadata.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.CreateInstanceMetadata)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.CreateInstanceMetadata();
                            if (object.originalRequest != null) {
                                if (typeof object.originalRequest !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.CreateInstanceMetadata.originalRequest: object expected");
                                message.originalRequest = $root.google.bigtable.admin.v2.CreateInstanceRequest.fromObject(object.originalRequest);
                            }
                            if (object.requestTime != null) {
                                if (typeof object.requestTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.CreateInstanceMetadata.requestTime: object expected");
                                message.requestTime = $root.google.protobuf.Timestamp.fromObject(object.requestTime);
                            }
                            if (object.finishTime != null) {
                                if (typeof object.finishTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.CreateInstanceMetadata.finishTime: object expected");
                                message.finishTime = $root.google.protobuf.Timestamp.fromObject(object.finishTime);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a CreateInstanceMetadata message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.CreateInstanceMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.CreateInstanceMetadata} message CreateInstanceMetadata
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        CreateInstanceMetadata.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.originalRequest = null;
                                object.requestTime = null;
                                object.finishTime = null;
                            }
                            if (message.originalRequest != null && message.hasOwnProperty("originalRequest"))
                                object.originalRequest = $root.google.bigtable.admin.v2.CreateInstanceRequest.toObject(message.originalRequest, options);
                            if (message.requestTime != null && message.hasOwnProperty("requestTime"))
                                object.requestTime = $root.google.protobuf.Timestamp.toObject(message.requestTime, options);
                            if (message.finishTime != null && message.hasOwnProperty("finishTime"))
                                object.finishTime = $root.google.protobuf.Timestamp.toObject(message.finishTime, options);
                            return object;
                        };
    
                        /**
                         * Converts this CreateInstanceMetadata to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.CreateInstanceMetadata
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        CreateInstanceMetadata.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for CreateInstanceMetadata
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.CreateInstanceMetadata
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        CreateInstanceMetadata.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.CreateInstanceMetadata";
                        };
    
                        return CreateInstanceMetadata;
                    })();
    
                    v2.UpdateInstanceMetadata = (function() {
    
                        /**
                         * Properties of an UpdateInstanceMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @interface IUpdateInstanceMetadata
                         * @property {google.bigtable.admin.v2.IPartialUpdateInstanceRequest|null} [originalRequest] UpdateInstanceMetadata originalRequest
                         * @property {google.protobuf.ITimestamp|null} [requestTime] UpdateInstanceMetadata requestTime
                         * @property {google.protobuf.ITimestamp|null} [finishTime] UpdateInstanceMetadata finishTime
                         */
    
                        /**
                         * Constructs a new UpdateInstanceMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents an UpdateInstanceMetadata.
                         * @implements IUpdateInstanceMetadata
                         * @constructor
                         * @param {google.bigtable.admin.v2.IUpdateInstanceMetadata=} [properties] Properties to set
                         */
                        function UpdateInstanceMetadata(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * UpdateInstanceMetadata originalRequest.
                         * @member {google.bigtable.admin.v2.IPartialUpdateInstanceRequest|null|undefined} originalRequest
                         * @memberof google.bigtable.admin.v2.UpdateInstanceMetadata
                         * @instance
                         */
                        UpdateInstanceMetadata.prototype.originalRequest = null;
    
                        /**
                         * UpdateInstanceMetadata requestTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} requestTime
                         * @memberof google.bigtable.admin.v2.UpdateInstanceMetadata
                         * @instance
                         */
                        UpdateInstanceMetadata.prototype.requestTime = null;
    
                        /**
                         * UpdateInstanceMetadata finishTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} finishTime
                         * @memberof google.bigtable.admin.v2.UpdateInstanceMetadata
                         * @instance
                         */
                        UpdateInstanceMetadata.prototype.finishTime = null;
    
                        /**
                         * Creates a new UpdateInstanceMetadata instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.UpdateInstanceMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.IUpdateInstanceMetadata=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.UpdateInstanceMetadata} UpdateInstanceMetadata instance
                         */
                        UpdateInstanceMetadata.create = function create(properties) {
                            return new UpdateInstanceMetadata(properties);
                        };
    
                        /**
                         * Encodes the specified UpdateInstanceMetadata message. Does not implicitly {@link google.bigtable.admin.v2.UpdateInstanceMetadata.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.UpdateInstanceMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.IUpdateInstanceMetadata} message UpdateInstanceMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        UpdateInstanceMetadata.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.originalRequest != null && Object.hasOwnProperty.call(message, "originalRequest"))
                                $root.google.bigtable.admin.v2.PartialUpdateInstanceRequest.encode(message.originalRequest, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            if (message.requestTime != null && Object.hasOwnProperty.call(message, "requestTime"))
                                $root.google.protobuf.Timestamp.encode(message.requestTime, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            if (message.finishTime != null && Object.hasOwnProperty.call(message, "finishTime"))
                                $root.google.protobuf.Timestamp.encode(message.finishTime, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified UpdateInstanceMetadata message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.UpdateInstanceMetadata.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.UpdateInstanceMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.IUpdateInstanceMetadata} message UpdateInstanceMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        UpdateInstanceMetadata.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes an UpdateInstanceMetadata message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.UpdateInstanceMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.UpdateInstanceMetadata} UpdateInstanceMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        UpdateInstanceMetadata.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.UpdateInstanceMetadata();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.originalRequest = $root.google.bigtable.admin.v2.PartialUpdateInstanceRequest.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 2: {
                                        message.requestTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 3: {
                                        message.finishTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes an UpdateInstanceMetadata message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.UpdateInstanceMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.UpdateInstanceMetadata} UpdateInstanceMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        UpdateInstanceMetadata.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies an UpdateInstanceMetadata message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.UpdateInstanceMetadata
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        UpdateInstanceMetadata.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.originalRequest != null && message.hasOwnProperty("originalRequest")) {
                                var error = $root.google.bigtable.admin.v2.PartialUpdateInstanceRequest.verify(message.originalRequest);
                                if (error)
                                    return "originalRequest." + error;
                            }
                            if (message.requestTime != null && message.hasOwnProperty("requestTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.requestTime);
                                if (error)
                                    return "requestTime." + error;
                            }
                            if (message.finishTime != null && message.hasOwnProperty("finishTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.finishTime);
                                if (error)
                                    return "finishTime." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates an UpdateInstanceMetadata message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.UpdateInstanceMetadata
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.UpdateInstanceMetadata} UpdateInstanceMetadata
                         */
                        UpdateInstanceMetadata.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.UpdateInstanceMetadata)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.UpdateInstanceMetadata();
                            if (object.originalRequest != null) {
                                if (typeof object.originalRequest !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.UpdateInstanceMetadata.originalRequest: object expected");
                                message.originalRequest = $root.google.bigtable.admin.v2.PartialUpdateInstanceRequest.fromObject(object.originalRequest);
                            }
                            if (object.requestTime != null) {
                                if (typeof object.requestTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.UpdateInstanceMetadata.requestTime: object expected");
                                message.requestTime = $root.google.protobuf.Timestamp.fromObject(object.requestTime);
                            }
                            if (object.finishTime != null) {
                                if (typeof object.finishTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.UpdateInstanceMetadata.finishTime: object expected");
                                message.finishTime = $root.google.protobuf.Timestamp.fromObject(object.finishTime);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from an UpdateInstanceMetadata message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.UpdateInstanceMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.UpdateInstanceMetadata} message UpdateInstanceMetadata
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        UpdateInstanceMetadata.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.originalRequest = null;
                                object.requestTime = null;
                                object.finishTime = null;
                            }
                            if (message.originalRequest != null && message.hasOwnProperty("originalRequest"))
                                object.originalRequest = $root.google.bigtable.admin.v2.PartialUpdateInstanceRequest.toObject(message.originalRequest, options);
                            if (message.requestTime != null && message.hasOwnProperty("requestTime"))
                                object.requestTime = $root.google.protobuf.Timestamp.toObject(message.requestTime, options);
                            if (message.finishTime != null && message.hasOwnProperty("finishTime"))
                                object.finishTime = $root.google.protobuf.Timestamp.toObject(message.finishTime, options);
                            return object;
                        };
    
                        /**
                         * Converts this UpdateInstanceMetadata to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.UpdateInstanceMetadata
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        UpdateInstanceMetadata.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for UpdateInstanceMetadata
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.UpdateInstanceMetadata
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        UpdateInstanceMetadata.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.UpdateInstanceMetadata";
                        };
    
                        return UpdateInstanceMetadata;
                    })();
    
                    v2.CreateClusterMetadata = (function() {
    
                        /**
                         * Properties of a CreateClusterMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @interface ICreateClusterMetadata
                         * @property {google.bigtable.admin.v2.ICreateClusterRequest|null} [originalRequest] CreateClusterMetadata originalRequest
                         * @property {google.protobuf.ITimestamp|null} [requestTime] CreateClusterMetadata requestTime
                         * @property {google.protobuf.ITimestamp|null} [finishTime] CreateClusterMetadata finishTime
                         * @property {Object.<string,google.bigtable.admin.v2.CreateClusterMetadata.ITableProgress>|null} [tables] CreateClusterMetadata tables
                         */
    
                        /**
                         * Constructs a new CreateClusterMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a CreateClusterMetadata.
                         * @implements ICreateClusterMetadata
                         * @constructor
                         * @param {google.bigtable.admin.v2.ICreateClusterMetadata=} [properties] Properties to set
                         */
                        function CreateClusterMetadata(properties) {
                            this.tables = {};
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * CreateClusterMetadata originalRequest.
                         * @member {google.bigtable.admin.v2.ICreateClusterRequest|null|undefined} originalRequest
                         * @memberof google.bigtable.admin.v2.CreateClusterMetadata
                         * @instance
                         */
                        CreateClusterMetadata.prototype.originalRequest = null;
    
                        /**
                         * CreateClusterMetadata requestTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} requestTime
                         * @memberof google.bigtable.admin.v2.CreateClusterMetadata
                         * @instance
                         */
                        CreateClusterMetadata.prototype.requestTime = null;
    
                        /**
                         * CreateClusterMetadata finishTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} finishTime
                         * @memberof google.bigtable.admin.v2.CreateClusterMetadata
                         * @instance
                         */
                        CreateClusterMetadata.prototype.finishTime = null;
    
                        /**
                         * CreateClusterMetadata tables.
                         * @member {Object.<string,google.bigtable.admin.v2.CreateClusterMetadata.ITableProgress>} tables
                         * @memberof google.bigtable.admin.v2.CreateClusterMetadata
                         * @instance
                         */
                        CreateClusterMetadata.prototype.tables = $util.emptyObject;
    
                        /**
                         * Creates a new CreateClusterMetadata instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.CreateClusterMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateClusterMetadata=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.CreateClusterMetadata} CreateClusterMetadata instance
                         */
                        CreateClusterMetadata.create = function create(properties) {
                            return new CreateClusterMetadata(properties);
                        };
    
                        /**
                         * Encodes the specified CreateClusterMetadata message. Does not implicitly {@link google.bigtable.admin.v2.CreateClusterMetadata.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.CreateClusterMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateClusterMetadata} message CreateClusterMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CreateClusterMetadata.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.originalRequest != null && Object.hasOwnProperty.call(message, "originalRequest"))
                                $root.google.bigtable.admin.v2.CreateClusterRequest.encode(message.originalRequest, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            if (message.requestTime != null && Object.hasOwnProperty.call(message, "requestTime"))
                                $root.google.protobuf.Timestamp.encode(message.requestTime, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            if (message.finishTime != null && Object.hasOwnProperty.call(message, "finishTime"))
                                $root.google.protobuf.Timestamp.encode(message.finishTime, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            if (message.tables != null && Object.hasOwnProperty.call(message, "tables"))
                                for (var keys = Object.keys(message.tables), i = 0; i < keys.length; ++i) {
                                    writer.uint32(/* id 4, wireType 2 =*/34).fork().uint32(/* id 1, wireType 2 =*/10).string(keys[i]);
                                    $root.google.bigtable.admin.v2.CreateClusterMetadata.TableProgress.encode(message.tables[keys[i]], writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim().ldelim();
                                }
                            return writer;
                        };
    
                        /**
                         * Encodes the specified CreateClusterMetadata message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.CreateClusterMetadata.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.CreateClusterMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateClusterMetadata} message CreateClusterMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CreateClusterMetadata.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a CreateClusterMetadata message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.CreateClusterMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.CreateClusterMetadata} CreateClusterMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CreateClusterMetadata.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.CreateClusterMetadata(), key, value;
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.originalRequest = $root.google.bigtable.admin.v2.CreateClusterRequest.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 2: {
                                        message.requestTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 3: {
                                        message.finishTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 4: {
                                        if (message.tables === $util.emptyObject)
                                            message.tables = {};
                                        var end2 = reader.uint32() + reader.pos;
                                        key = "";
                                        value = null;
                                        while (reader.pos < end2) {
                                            var tag2 = reader.uint32();
                                            switch (tag2 >>> 3) {
                                            case 1:
                                                key = reader.string();
                                                break;
                                            case 2:
                                                value = $root.google.bigtable.admin.v2.CreateClusterMetadata.TableProgress.decode(reader, reader.uint32());
                                                break;
                                            default:
                                                reader.skipType(tag2 & 7);
                                                break;
                                            }
                                        }
                                        message.tables[key] = value;
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a CreateClusterMetadata message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.CreateClusterMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.CreateClusterMetadata} CreateClusterMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CreateClusterMetadata.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a CreateClusterMetadata message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.CreateClusterMetadata
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        CreateClusterMetadata.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.originalRequest != null && message.hasOwnProperty("originalRequest")) {
                                var error = $root.google.bigtable.admin.v2.CreateClusterRequest.verify(message.originalRequest);
                                if (error)
                                    return "originalRequest." + error;
                            }
                            if (message.requestTime != null && message.hasOwnProperty("requestTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.requestTime);
                                if (error)
                                    return "requestTime." + error;
                            }
                            if (message.finishTime != null && message.hasOwnProperty("finishTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.finishTime);
                                if (error)
                                    return "finishTime." + error;
                            }
                            if (message.tables != null && message.hasOwnProperty("tables")) {
                                if (!$util.isObject(message.tables))
                                    return "tables: object expected";
                                var key = Object.keys(message.tables);
                                for (var i = 0; i < key.length; ++i) {
                                    var error = $root.google.bigtable.admin.v2.CreateClusterMetadata.TableProgress.verify(message.tables[key[i]]);
                                    if (error)
                                        return "tables." + error;
                                }
                            }
                            return null;
                        };
    
                        /**
                         * Creates a CreateClusterMetadata message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.CreateClusterMetadata
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.CreateClusterMetadata} CreateClusterMetadata
                         */
                        CreateClusterMetadata.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.CreateClusterMetadata)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.CreateClusterMetadata();
                            if (object.originalRequest != null) {
                                if (typeof object.originalRequest !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.CreateClusterMetadata.originalRequest: object expected");
                                message.originalRequest = $root.google.bigtable.admin.v2.CreateClusterRequest.fromObject(object.originalRequest);
                            }
                            if (object.requestTime != null) {
                                if (typeof object.requestTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.CreateClusterMetadata.requestTime: object expected");
                                message.requestTime = $root.google.protobuf.Timestamp.fromObject(object.requestTime);
                            }
                            if (object.finishTime != null) {
                                if (typeof object.finishTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.CreateClusterMetadata.finishTime: object expected");
                                message.finishTime = $root.google.protobuf.Timestamp.fromObject(object.finishTime);
                            }
                            if (object.tables) {
                                if (typeof object.tables !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.CreateClusterMetadata.tables: object expected");
                                message.tables = {};
                                for (var keys = Object.keys(object.tables), i = 0; i < keys.length; ++i) {
                                    if (typeof object.tables[keys[i]] !== "object")
                                        throw TypeError(".google.bigtable.admin.v2.CreateClusterMetadata.tables: object expected");
                                    message.tables[keys[i]] = $root.google.bigtable.admin.v2.CreateClusterMetadata.TableProgress.fromObject(object.tables[keys[i]]);
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a CreateClusterMetadata message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.CreateClusterMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.CreateClusterMetadata} message CreateClusterMetadata
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        CreateClusterMetadata.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.objects || options.defaults)
                                object.tables = {};
                            if (options.defaults) {
                                object.originalRequest = null;
                                object.requestTime = null;
                                object.finishTime = null;
                            }
                            if (message.originalRequest != null && message.hasOwnProperty("originalRequest"))
                                object.originalRequest = $root.google.bigtable.admin.v2.CreateClusterRequest.toObject(message.originalRequest, options);
                            if (message.requestTime != null && message.hasOwnProperty("requestTime"))
                                object.requestTime = $root.google.protobuf.Timestamp.toObject(message.requestTime, options);
                            if (message.finishTime != null && message.hasOwnProperty("finishTime"))
                                object.finishTime = $root.google.protobuf.Timestamp.toObject(message.finishTime, options);
                            var keys2;
                            if (message.tables && (keys2 = Object.keys(message.tables)).length) {
                                object.tables = {};
                                for (var j = 0; j < keys2.length; ++j)
                                    object.tables[keys2[j]] = $root.google.bigtable.admin.v2.CreateClusterMetadata.TableProgress.toObject(message.tables[keys2[j]], options);
                            }
                            return object;
                        };
    
                        /**
                         * Converts this CreateClusterMetadata to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.CreateClusterMetadata
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        CreateClusterMetadata.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for CreateClusterMetadata
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.CreateClusterMetadata
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        CreateClusterMetadata.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.CreateClusterMetadata";
                        };
    
                        CreateClusterMetadata.TableProgress = (function() {
    
                            /**
                             * Properties of a TableProgress.
                             * @memberof google.bigtable.admin.v2.CreateClusterMetadata
                             * @interface ITableProgress
                             * @property {number|Long|null} [estimatedSizeBytes] TableProgress estimatedSizeBytes
                             * @property {number|Long|null} [estimatedCopiedBytes] TableProgress estimatedCopiedBytes
                             * @property {google.bigtable.admin.v2.CreateClusterMetadata.TableProgress.State|null} [state] TableProgress state
                             */
    
                            /**
                             * Constructs a new TableProgress.
                             * @memberof google.bigtable.admin.v2.CreateClusterMetadata
                             * @classdesc Represents a TableProgress.
                             * @implements ITableProgress
                             * @constructor
                             * @param {google.bigtable.admin.v2.CreateClusterMetadata.ITableProgress=} [properties] Properties to set
                             */
                            function TableProgress(properties) {
                                if (properties)
                                    for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                        if (properties[keys[i]] != null)
                                            this[keys[i]] = properties[keys[i]];
                            }
    
                            /**
                             * TableProgress estimatedSizeBytes.
                             * @member {number|Long} estimatedSizeBytes
                             * @memberof google.bigtable.admin.v2.CreateClusterMetadata.TableProgress
                             * @instance
                             */
                            TableProgress.prototype.estimatedSizeBytes = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * TableProgress estimatedCopiedBytes.
                             * @member {number|Long} estimatedCopiedBytes
                             * @memberof google.bigtable.admin.v2.CreateClusterMetadata.TableProgress
                             * @instance
                             */
                            TableProgress.prototype.estimatedCopiedBytes = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                            /**
                             * TableProgress state.
                             * @member {google.bigtable.admin.v2.CreateClusterMetadata.TableProgress.State} state
                             * @memberof google.bigtable.admin.v2.CreateClusterMetadata.TableProgress
                             * @instance
                             */
                            TableProgress.prototype.state = 0;
    
                            /**
                             * Creates a new TableProgress instance using the specified properties.
                             * @function create
                             * @memberof google.bigtable.admin.v2.CreateClusterMetadata.TableProgress
                             * @static
                             * @param {google.bigtable.admin.v2.CreateClusterMetadata.ITableProgress=} [properties] Properties to set
                             * @returns {google.bigtable.admin.v2.CreateClusterMetadata.TableProgress} TableProgress instance
                             */
                            TableProgress.create = function create(properties) {
                                return new TableProgress(properties);
                            };
    
                            /**
                             * Encodes the specified TableProgress message. Does not implicitly {@link google.bigtable.admin.v2.CreateClusterMetadata.TableProgress.verify|verify} messages.
                             * @function encode
                             * @memberof google.bigtable.admin.v2.CreateClusterMetadata.TableProgress
                             * @static
                             * @param {google.bigtable.admin.v2.CreateClusterMetadata.ITableProgress} message TableProgress message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TableProgress.encode = function encode(message, writer) {
                                if (!writer)
                                    writer = $Writer.create();
                                if (message.estimatedSizeBytes != null && Object.hasOwnProperty.call(message, "estimatedSizeBytes"))
                                    writer.uint32(/* id 2, wireType 0 =*/16).int64(message.estimatedSizeBytes);
                                if (message.estimatedCopiedBytes != null && Object.hasOwnProperty.call(message, "estimatedCopiedBytes"))
                                    writer.uint32(/* id 3, wireType 0 =*/24).int64(message.estimatedCopiedBytes);
                                if (message.state != null && Object.hasOwnProperty.call(message, "state"))
                                    writer.uint32(/* id 4, wireType 0 =*/32).int32(message.state);
                                return writer;
                            };
    
                            /**
                             * Encodes the specified TableProgress message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.CreateClusterMetadata.TableProgress.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.bigtable.admin.v2.CreateClusterMetadata.TableProgress
                             * @static
                             * @param {google.bigtable.admin.v2.CreateClusterMetadata.ITableProgress} message TableProgress message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            TableProgress.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a TableProgress message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.bigtable.admin.v2.CreateClusterMetadata.TableProgress
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @param {number} [length] Message length if known beforehand
                             * @returns {google.bigtable.admin.v2.CreateClusterMetadata.TableProgress} TableProgress
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TableProgress.decode = function decode(reader, length) {
                                if (!(reader instanceof $Reader))
                                    reader = $Reader.create(reader);
                                var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.CreateClusterMetadata.TableProgress();
                                while (reader.pos < end) {
                                    var tag = reader.uint32();
                                    switch (tag >>> 3) {
                                    case 2: {
                                            message.estimatedSizeBytes = reader.int64();
                                            break;
                                        }
                                    case 3: {
                                            message.estimatedCopiedBytes = reader.int64();
                                            break;
                                        }
                                    case 4: {
                                            message.state = reader.int32();
                                            break;
                                        }
                                    default:
                                        reader.skipType(tag & 7);
                                        break;
                                    }
                                }
                                return message;
                            };
    
                            /**
                             * Decodes a TableProgress message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.bigtable.admin.v2.CreateClusterMetadata.TableProgress
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.bigtable.admin.v2.CreateClusterMetadata.TableProgress} TableProgress
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            TableProgress.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a TableProgress message.
                             * @function verify
                             * @memberof google.bigtable.admin.v2.CreateClusterMetadata.TableProgress
                             * @static
                             * @param {Object.<string,*>} message Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            TableProgress.verify = function verify(message) {
                                if (typeof message !== "object" || message === null)
                                    return "object expected";
                                if (message.estimatedSizeBytes != null && message.hasOwnProperty("estimatedSizeBytes"))
                                    if (!$util.isInteger(message.estimatedSizeBytes) && !(message.estimatedSizeBytes && $util.isInteger(message.estimatedSizeBytes.low) && $util.isInteger(message.estimatedSizeBytes.high)))
                                        return "estimatedSizeBytes: integer|Long expected";
                                if (message.estimatedCopiedBytes != null && message.hasOwnProperty("estimatedCopiedBytes"))
                                    if (!$util.isInteger(message.estimatedCopiedBytes) && !(message.estimatedCopiedBytes && $util.isInteger(message.estimatedCopiedBytes.low) && $util.isInteger(message.estimatedCopiedBytes.high)))
                                        return "estimatedCopiedBytes: integer|Long expected";
                                if (message.state != null && message.hasOwnProperty("state"))
                                    switch (message.state) {
                                    default:
                                        return "state: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                        break;
                                    }
                                return null;
                            };
    
                            /**
                             * Creates a TableProgress message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.bigtable.admin.v2.CreateClusterMetadata.TableProgress
                             * @static
                             * @param {Object.<string,*>} object Plain object
                             * @returns {google.bigtable.admin.v2.CreateClusterMetadata.TableProgress} TableProgress
                             */
                            TableProgress.fromObject = function fromObject(object) {
                                if (object instanceof $root.google.bigtable.admin.v2.CreateClusterMetadata.TableProgress)
                                    return object;
                                var message = new $root.google.bigtable.admin.v2.CreateClusterMetadata.TableProgress();
                                if (object.estimatedSizeBytes != null)
                                    if ($util.Long)
                                        (message.estimatedSizeBytes = $util.Long.fromValue(object.estimatedSizeBytes)).unsigned = false;
                                    else if (typeof object.estimatedSizeBytes === "string")
                                        message.estimatedSizeBytes = parseInt(object.estimatedSizeBytes, 10);
                                    else if (typeof object.estimatedSizeBytes === "number")
                                        message.estimatedSizeBytes = object.estimatedSizeBytes;
                                    else if (typeof object.estimatedSizeBytes === "object")
                                        message.estimatedSizeBytes = new $util.LongBits(object.estimatedSizeBytes.low >>> 0, object.estimatedSizeBytes.high >>> 0).toNumber();
                                if (object.estimatedCopiedBytes != null)
                                    if ($util.Long)
                                        (message.estimatedCopiedBytes = $util.Long.fromValue(object.estimatedCopiedBytes)).unsigned = false;
                                    else if (typeof object.estimatedCopiedBytes === "string")
                                        message.estimatedCopiedBytes = parseInt(object.estimatedCopiedBytes, 10);
                                    else if (typeof object.estimatedCopiedBytes === "number")
                                        message.estimatedCopiedBytes = object.estimatedCopiedBytes;
                                    else if (typeof object.estimatedCopiedBytes === "object")
                                        message.estimatedCopiedBytes = new $util.LongBits(object.estimatedCopiedBytes.low >>> 0, object.estimatedCopiedBytes.high >>> 0).toNumber();
                                switch (object.state) {
                                default:
                                    if (typeof object.state === "number") {
                                        message.state = object.state;
                                        break;
                                    }
                                    break;
                                case "STATE_UNSPECIFIED":
                                case 0:
                                    message.state = 0;
                                    break;
                                case "PENDING":
                                case 1:
                                    message.state = 1;
                                    break;
                                case "COPYING":
                                case 2:
                                    message.state = 2;
                                    break;
                                case "COMPLETED":
                                case 3:
                                    message.state = 3;
                                    break;
                                case "CANCELLED":
                                case 4:
                                    message.state = 4;
                                    break;
                                }
                                return message;
                            };
    
                            /**
                             * Creates a plain object from a TableProgress message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.bigtable.admin.v2.CreateClusterMetadata.TableProgress
                             * @static
                             * @param {google.bigtable.admin.v2.CreateClusterMetadata.TableProgress} message TableProgress
                             * @param {$protobuf.IConversionOptions} [options] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            TableProgress.toObject = function toObject(message, options) {
                                if (!options)
                                    options = {};
                                var object = {};
                                if (options.defaults) {
                                    if ($util.Long) {
                                        var long = new $util.Long(0, 0, false);
                                        object.estimatedSizeBytes = options.longs === String ? long.toString() : options.longs === Number ? long.toNumber() : long;
                                    } else
                                        object.estimatedSizeBytes = options.longs === String ? "0" : 0;
                                    if ($util.Long) {
                                        var long = new $util.Long(0, 0, false);
                                        object.estimatedCopiedBytes = options.longs === String ? long.toString() : options.longs === Number ? long.toNumber() : long;
                                    } else
                                        object.estimatedCopiedBytes = options.longs === String ? "0" : 0;
                                    object.state = options.enums === String ? "STATE_UNSPECIFIED" : 0;
                                }
                                if (message.estimatedSizeBytes != null && message.hasOwnProperty("estimatedSizeBytes"))
                                    if (typeof message.estimatedSizeBytes === "number")
                                        object.estimatedSizeBytes = options.longs === String ? String(message.estimatedSizeBytes) : message.estimatedSizeBytes;
                                    else
                                        object.estimatedSizeBytes = options.longs === String ? $util.Long.prototype.toString.call(message.estimatedSizeBytes) : options.longs === Number ? new $util.LongBits(message.estimatedSizeBytes.low >>> 0, message.estimatedSizeBytes.high >>> 0).toNumber() : message.estimatedSizeBytes;
                                if (message.estimatedCopiedBytes != null && message.hasOwnProperty("estimatedCopiedBytes"))
                                    if (typeof message.estimatedCopiedBytes === "number")
                                        object.estimatedCopiedBytes = options.longs === String ? String(message.estimatedCopiedBytes) : message.estimatedCopiedBytes;
                                    else
                                        object.estimatedCopiedBytes = options.longs === String ? $util.Long.prototype.toString.call(message.estimatedCopiedBytes) : options.longs === Number ? new $util.LongBits(message.estimatedCopiedBytes.low >>> 0, message.estimatedCopiedBytes.high >>> 0).toNumber() : message.estimatedCopiedBytes;
                                if (message.state != null && message.hasOwnProperty("state"))
                                    object.state = options.enums === String ? $root.google.bigtable.admin.v2.CreateClusterMetadata.TableProgress.State[message.state] === undefined ? message.state : $root.google.bigtable.admin.v2.CreateClusterMetadata.TableProgress.State[message.state] : message.state;
                                return object;
                            };
    
                            /**
                             * Converts this TableProgress to JSON.
                             * @function toJSON
                             * @memberof google.bigtable.admin.v2.CreateClusterMetadata.TableProgress
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            TableProgress.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * Gets the default type url for TableProgress
                             * @function getTypeUrl
                             * @memberof google.bigtable.admin.v2.CreateClusterMetadata.TableProgress
                             * @static
                             * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                             * @returns {string} The default type url
                             */
                            TableProgress.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                                if (typeUrlPrefix === undefined) {
                                    typeUrlPrefix = "type.googleapis.com";
                                }
                                return typeUrlPrefix + "/google.bigtable.admin.v2.CreateClusterMetadata.TableProgress";
                            };
    
                            /**
                             * State enum.
                             * @name google.bigtable.admin.v2.CreateClusterMetadata.TableProgress.State
                             * @enum {number}
                             * @property {number} STATE_UNSPECIFIED=0 STATE_UNSPECIFIED value
                             * @property {number} PENDING=1 PENDING value
                             * @property {number} COPYING=2 COPYING value
                             * @property {number} COMPLETED=3 COMPLETED value
                             * @property {number} CANCELLED=4 CANCELLED value
                             */
                            TableProgress.State = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "STATE_UNSPECIFIED"] = 0;
                                values[valuesById[1] = "PENDING"] = 1;
                                values[valuesById[2] = "COPYING"] = 2;
                                values[valuesById[3] = "COMPLETED"] = 3;
                                values[valuesById[4] = "CANCELLED"] = 4;
                                return values;
                            })();
    
                            return TableProgress;
                        })();
    
                        return CreateClusterMetadata;
                    })();
    
                    v2.UpdateClusterMetadata = (function() {
    
                        /**
                         * Properties of an UpdateClusterMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @interface IUpdateClusterMetadata
                         * @property {google.bigtable.admin.v2.ICluster|null} [originalRequest] UpdateClusterMetadata originalRequest
                         * @property {google.protobuf.ITimestamp|null} [requestTime] UpdateClusterMetadata requestTime
                         * @property {google.protobuf.ITimestamp|null} [finishTime] UpdateClusterMetadata finishTime
                         */
    
                        /**
                         * Constructs a new UpdateClusterMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents an UpdateClusterMetadata.
                         * @implements IUpdateClusterMetadata
                         * @constructor
                         * @param {google.bigtable.admin.v2.IUpdateClusterMetadata=} [properties] Properties to set
                         */
                        function UpdateClusterMetadata(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * UpdateClusterMetadata originalRequest.
                         * @member {google.bigtable.admin.v2.ICluster|null|undefined} originalRequest
                         * @memberof google.bigtable.admin.v2.UpdateClusterMetadata
                         * @instance
                         */
                        UpdateClusterMetadata.prototype.originalRequest = null;
    
                        /**
                         * UpdateClusterMetadata requestTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} requestTime
                         * @memberof google.bigtable.admin.v2.UpdateClusterMetadata
                         * @instance
                         */
                        UpdateClusterMetadata.prototype.requestTime = null;
    
                        /**
                         * UpdateClusterMetadata finishTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} finishTime
                         * @memberof google.bigtable.admin.v2.UpdateClusterMetadata
                         * @instance
                         */
                        UpdateClusterMetadata.prototype.finishTime = null;
    
                        /**
                         * Creates a new UpdateClusterMetadata instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.UpdateClusterMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.IUpdateClusterMetadata=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.UpdateClusterMetadata} UpdateClusterMetadata instance
                         */
                        UpdateClusterMetadata.create = function create(properties) {
                            return new UpdateClusterMetadata(properties);
                        };
    
                        /**
                         * Encodes the specified UpdateClusterMetadata message. Does not implicitly {@link google.bigtable.admin.v2.UpdateClusterMetadata.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.UpdateClusterMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.IUpdateClusterMetadata} message UpdateClusterMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        UpdateClusterMetadata.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.originalRequest != null && Object.hasOwnProperty.call(message, "originalRequest"))
                                $root.google.bigtable.admin.v2.Cluster.encode(message.originalRequest, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            if (message.requestTime != null && Object.hasOwnProperty.call(message, "requestTime"))
                                $root.google.protobuf.Timestamp.encode(message.requestTime, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            if (message.finishTime != null && Object.hasOwnProperty.call(message, "finishTime"))
                                $root.google.protobuf.Timestamp.encode(message.finishTime, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified UpdateClusterMetadata message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.UpdateClusterMetadata.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.UpdateClusterMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.IUpdateClusterMetadata} message UpdateClusterMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        UpdateClusterMetadata.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes an UpdateClusterMetadata message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.UpdateClusterMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.UpdateClusterMetadata} UpdateClusterMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        UpdateClusterMetadata.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.UpdateClusterMetadata();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.originalRequest = $root.google.bigtable.admin.v2.Cluster.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 2: {
                                        message.requestTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 3: {
                                        message.finishTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes an UpdateClusterMetadata message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.UpdateClusterMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.UpdateClusterMetadata} UpdateClusterMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        UpdateClusterMetadata.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies an UpdateClusterMetadata message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.UpdateClusterMetadata
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        UpdateClusterMetadata.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.originalRequest != null && message.hasOwnProperty("originalRequest")) {
                                var error = $root.google.bigtable.admin.v2.Cluster.verify(message.originalRequest);
                                if (error)
                                    return "originalRequest." + error;
                            }
                            if (message.requestTime != null && message.hasOwnProperty("requestTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.requestTime);
                                if (error)
                                    return "requestTime." + error;
                            }
                            if (message.finishTime != null && message.hasOwnProperty("finishTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.finishTime);
                                if (error)
                                    return "finishTime." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates an UpdateClusterMetadata message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.UpdateClusterMetadata
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.UpdateClusterMetadata} UpdateClusterMetadata
                         */
                        UpdateClusterMetadata.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.UpdateClusterMetadata)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.UpdateClusterMetadata();
                            if (object.originalRequest != null) {
                                if (typeof object.originalRequest !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.UpdateClusterMetadata.originalRequest: object expected");
                                message.originalRequest = $root.google.bigtable.admin.v2.Cluster.fromObject(object.originalRequest);
                            }
                            if (object.requestTime != null) {
                                if (typeof object.requestTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.UpdateClusterMetadata.requestTime: object expected");
                                message.requestTime = $root.google.protobuf.Timestamp.fromObject(object.requestTime);
                            }
                            if (object.finishTime != null) {
                                if (typeof object.finishTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.UpdateClusterMetadata.finishTime: object expected");
                                message.finishTime = $root.google.protobuf.Timestamp.fromObject(object.finishTime);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from an UpdateClusterMetadata message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.UpdateClusterMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.UpdateClusterMetadata} message UpdateClusterMetadata
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        UpdateClusterMetadata.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.originalRequest = null;
                                object.requestTime = null;
                                object.finishTime = null;
                            }
                            if (message.originalRequest != null && message.hasOwnProperty("originalRequest"))
                                object.originalRequest = $root.google.bigtable.admin.v2.Cluster.toObject(message.originalRequest, options);
                            if (message.requestTime != null && message.hasOwnProperty("requestTime"))
                                object.requestTime = $root.google.protobuf.Timestamp.toObject(message.requestTime, options);
                            if (message.finishTime != null && message.hasOwnProperty("finishTime"))
                                object.finishTime = $root.google.protobuf.Timestamp.toObject(message.finishTime, options);
                            return object;
                        };
    
                        /**
                         * Converts this UpdateClusterMetadata to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.UpdateClusterMetadata
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        UpdateClusterMetadata.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for UpdateClusterMetadata
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.UpdateClusterMetadata
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        UpdateClusterMetadata.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.UpdateClusterMetadata";
                        };
    
                        return UpdateClusterMetadata;
                    })();
    
                    v2.PartialUpdateClusterMetadata = (function() {
    
                        /**
                         * Properties of a PartialUpdateClusterMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @interface IPartialUpdateClusterMetadata
                         * @property {google.protobuf.ITimestamp|null} [requestTime] PartialUpdateClusterMetadata requestTime
                         * @property {google.protobuf.ITimestamp|null} [finishTime] PartialUpdateClusterMetadata finishTime
                         * @property {google.bigtable.admin.v2.IPartialUpdateClusterRequest|null} [originalRequest] PartialUpdateClusterMetadata originalRequest
                         */
    
                        /**
                         * Constructs a new PartialUpdateClusterMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a PartialUpdateClusterMetadata.
                         * @implements IPartialUpdateClusterMetadata
                         * @constructor
                         * @param {google.bigtable.admin.v2.IPartialUpdateClusterMetadata=} [properties] Properties to set
                         */
                        function PartialUpdateClusterMetadata(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * PartialUpdateClusterMetadata requestTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} requestTime
                         * @memberof google.bigtable.admin.v2.PartialUpdateClusterMetadata
                         * @instance
                         */
                        PartialUpdateClusterMetadata.prototype.requestTime = null;
    
                        /**
                         * PartialUpdateClusterMetadata finishTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} finishTime
                         * @memberof google.bigtable.admin.v2.PartialUpdateClusterMetadata
                         * @instance
                         */
                        PartialUpdateClusterMetadata.prototype.finishTime = null;
    
                        /**
                         * PartialUpdateClusterMetadata originalRequest.
                         * @member {google.bigtable.admin.v2.IPartialUpdateClusterRequest|null|undefined} originalRequest
                         * @memberof google.bigtable.admin.v2.PartialUpdateClusterMetadata
                         * @instance
                         */
                        PartialUpdateClusterMetadata.prototype.originalRequest = null;
    
                        /**
                         * Creates a new PartialUpdateClusterMetadata instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.PartialUpdateClusterMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.IPartialUpdateClusterMetadata=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.PartialUpdateClusterMetadata} PartialUpdateClusterMetadata instance
                         */
                        PartialUpdateClusterMetadata.create = function create(properties) {
                            return new PartialUpdateClusterMetadata(properties);
                        };
    
                        /**
                         * Encodes the specified PartialUpdateClusterMetadata message. Does not implicitly {@link google.bigtable.admin.v2.PartialUpdateClusterMetadata.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.PartialUpdateClusterMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.IPartialUpdateClusterMetadata} message PartialUpdateClusterMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        PartialUpdateClusterMetadata.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.requestTime != null && Object.hasOwnProperty.call(message, "requestTime"))
                                $root.google.protobuf.Timestamp.encode(message.requestTime, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            if (message.finishTime != null && Object.hasOwnProperty.call(message, "finishTime"))
                                $root.google.protobuf.Timestamp.encode(message.finishTime, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            if (message.originalRequest != null && Object.hasOwnProperty.call(message, "originalRequest"))
                                $root.google.bigtable.admin.v2.PartialUpdateClusterRequest.encode(message.originalRequest, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified PartialUpdateClusterMetadata message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.PartialUpdateClusterMetadata.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.PartialUpdateClusterMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.IPartialUpdateClusterMetadata} message PartialUpdateClusterMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        PartialUpdateClusterMetadata.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a PartialUpdateClusterMetadata message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.PartialUpdateClusterMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.PartialUpdateClusterMetadata} PartialUpdateClusterMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        PartialUpdateClusterMetadata.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.PartialUpdateClusterMetadata();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.requestTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 2: {
                                        message.finishTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 3: {
                                        message.originalRequest = $root.google.bigtable.admin.v2.PartialUpdateClusterRequest.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a PartialUpdateClusterMetadata message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.PartialUpdateClusterMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.PartialUpdateClusterMetadata} PartialUpdateClusterMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        PartialUpdateClusterMetadata.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a PartialUpdateClusterMetadata message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.PartialUpdateClusterMetadata
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        PartialUpdateClusterMetadata.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.requestTime != null && message.hasOwnProperty("requestTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.requestTime);
                                if (error)
                                    return "requestTime." + error;
                            }
                            if (message.finishTime != null && message.hasOwnProperty("finishTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.finishTime);
                                if (error)
                                    return "finishTime." + error;
                            }
                            if (message.originalRequest != null && message.hasOwnProperty("originalRequest")) {
                                var error = $root.google.bigtable.admin.v2.PartialUpdateClusterRequest.verify(message.originalRequest);
                                if (error)
                                    return "originalRequest." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates a PartialUpdateClusterMetadata message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.PartialUpdateClusterMetadata
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.PartialUpdateClusterMetadata} PartialUpdateClusterMetadata
                         */
                        PartialUpdateClusterMetadata.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.PartialUpdateClusterMetadata)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.PartialUpdateClusterMetadata();
                            if (object.requestTime != null) {
                                if (typeof object.requestTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.PartialUpdateClusterMetadata.requestTime: object expected");
                                message.requestTime = $root.google.protobuf.Timestamp.fromObject(object.requestTime);
                            }
                            if (object.finishTime != null) {
                                if (typeof object.finishTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.PartialUpdateClusterMetadata.finishTime: object expected");
                                message.finishTime = $root.google.protobuf.Timestamp.fromObject(object.finishTime);
                            }
                            if (object.originalRequest != null) {
                                if (typeof object.originalRequest !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.PartialUpdateClusterMetadata.originalRequest: object expected");
                                message.originalRequest = $root.google.bigtable.admin.v2.PartialUpdateClusterRequest.fromObject(object.originalRequest);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a PartialUpdateClusterMetadata message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.PartialUpdateClusterMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.PartialUpdateClusterMetadata} message PartialUpdateClusterMetadata
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        PartialUpdateClusterMetadata.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.requestTime = null;
                                object.finishTime = null;
                                object.originalRequest = null;
                            }
                            if (message.requestTime != null && message.hasOwnProperty("requestTime"))
                                object.requestTime = $root.google.protobuf.Timestamp.toObject(message.requestTime, options);
                            if (message.finishTime != null && message.hasOwnProperty("finishTime"))
                                object.finishTime = $root.google.protobuf.Timestamp.toObject(message.finishTime, options);
                            if (message.originalRequest != null && message.hasOwnProperty("originalRequest"))
                                object.originalRequest = $root.google.bigtable.admin.v2.PartialUpdateClusterRequest.toObject(message.originalRequest, options);
                            return object;
                        };
    
                        /**
                         * Converts this PartialUpdateClusterMetadata to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.PartialUpdateClusterMetadata
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        PartialUpdateClusterMetadata.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for PartialUpdateClusterMetadata
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.PartialUpdateClusterMetadata
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        PartialUpdateClusterMetadata.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.PartialUpdateClusterMetadata";
                        };
    
                        return PartialUpdateClusterMetadata;
                    })();
    
                    v2.PartialUpdateClusterRequest = (function() {
    
                        /**
                         * Properties of a PartialUpdateClusterRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IPartialUpdateClusterRequest
                         * @property {google.bigtable.admin.v2.ICluster|null} [cluster] PartialUpdateClusterRequest cluster
                         * @property {google.protobuf.IFieldMask|null} [updateMask] PartialUpdateClusterRequest updateMask
                         */
    
                        /**
                         * Constructs a new PartialUpdateClusterRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a PartialUpdateClusterRequest.
                         * @implements IPartialUpdateClusterRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IPartialUpdateClusterRequest=} [properties] Properties to set
                         */
                        function PartialUpdateClusterRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * PartialUpdateClusterRequest cluster.
                         * @member {google.bigtable.admin.v2.ICluster|null|undefined} cluster
                         * @memberof google.bigtable.admin.v2.PartialUpdateClusterRequest
                         * @instance
                         */
                        PartialUpdateClusterRequest.prototype.cluster = null;
    
                        /**
                         * PartialUpdateClusterRequest updateMask.
                         * @member {google.protobuf.IFieldMask|null|undefined} updateMask
                         * @memberof google.bigtable.admin.v2.PartialUpdateClusterRequest
                         * @instance
                         */
                        PartialUpdateClusterRequest.prototype.updateMask = null;
    
                        /**
                         * Creates a new PartialUpdateClusterRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.PartialUpdateClusterRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IPartialUpdateClusterRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.PartialUpdateClusterRequest} PartialUpdateClusterRequest instance
                         */
                        PartialUpdateClusterRequest.create = function create(properties) {
                            return new PartialUpdateClusterRequest(properties);
                        };
    
                        /**
                         * Encodes the specified PartialUpdateClusterRequest message. Does not implicitly {@link google.bigtable.admin.v2.PartialUpdateClusterRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.PartialUpdateClusterRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IPartialUpdateClusterRequest} message PartialUpdateClusterRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        PartialUpdateClusterRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.cluster != null && Object.hasOwnProperty.call(message, "cluster"))
                                $root.google.bigtable.admin.v2.Cluster.encode(message.cluster, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            if (message.updateMask != null && Object.hasOwnProperty.call(message, "updateMask"))
                                $root.google.protobuf.FieldMask.encode(message.updateMask, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified PartialUpdateClusterRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.PartialUpdateClusterRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.PartialUpdateClusterRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IPartialUpdateClusterRequest} message PartialUpdateClusterRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        PartialUpdateClusterRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a PartialUpdateClusterRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.PartialUpdateClusterRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.PartialUpdateClusterRequest} PartialUpdateClusterRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        PartialUpdateClusterRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.PartialUpdateClusterRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.cluster = $root.google.bigtable.admin.v2.Cluster.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 2: {
                                        message.updateMask = $root.google.protobuf.FieldMask.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a PartialUpdateClusterRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.PartialUpdateClusterRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.PartialUpdateClusterRequest} PartialUpdateClusterRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        PartialUpdateClusterRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a PartialUpdateClusterRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.PartialUpdateClusterRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        PartialUpdateClusterRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.cluster != null && message.hasOwnProperty("cluster")) {
                                var error = $root.google.bigtable.admin.v2.Cluster.verify(message.cluster);
                                if (error)
                                    return "cluster." + error;
                            }
                            if (message.updateMask != null && message.hasOwnProperty("updateMask")) {
                                var error = $root.google.protobuf.FieldMask.verify(message.updateMask);
                                if (error)
                                    return "updateMask." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates a PartialUpdateClusterRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.PartialUpdateClusterRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.PartialUpdateClusterRequest} PartialUpdateClusterRequest
                         */
                        PartialUpdateClusterRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.PartialUpdateClusterRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.PartialUpdateClusterRequest();
                            if (object.cluster != null) {
                                if (typeof object.cluster !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.PartialUpdateClusterRequest.cluster: object expected");
                                message.cluster = $root.google.bigtable.admin.v2.Cluster.fromObject(object.cluster);
                            }
                            if (object.updateMask != null) {
                                if (typeof object.updateMask !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.PartialUpdateClusterRequest.updateMask: object expected");
                                message.updateMask = $root.google.protobuf.FieldMask.fromObject(object.updateMask);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a PartialUpdateClusterRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.PartialUpdateClusterRequest
                         * @static
                         * @param {google.bigtable.admin.v2.PartialUpdateClusterRequest} message PartialUpdateClusterRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        PartialUpdateClusterRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.cluster = null;
                                object.updateMask = null;
                            }
                            if (message.cluster != null && message.hasOwnProperty("cluster"))
                                object.cluster = $root.google.bigtable.admin.v2.Cluster.toObject(message.cluster, options);
                            if (message.updateMask != null && message.hasOwnProperty("updateMask"))
                                object.updateMask = $root.google.protobuf.FieldMask.toObject(message.updateMask, options);
                            return object;
                        };
    
                        /**
                         * Converts this PartialUpdateClusterRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.PartialUpdateClusterRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        PartialUpdateClusterRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for PartialUpdateClusterRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.PartialUpdateClusterRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        PartialUpdateClusterRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.PartialUpdateClusterRequest";
                        };
    
                        return PartialUpdateClusterRequest;
                    })();
    
                    v2.CreateAppProfileRequest = (function() {
    
                        /**
                         * Properties of a CreateAppProfileRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface ICreateAppProfileRequest
                         * @property {string|null} [parent] CreateAppProfileRequest parent
                         * @property {string|null} [appProfileId] CreateAppProfileRequest appProfileId
                         * @property {google.bigtable.admin.v2.IAppProfile|null} [appProfile] CreateAppProfileRequest appProfile
                         * @property {boolean|null} [ignoreWarnings] CreateAppProfileRequest ignoreWarnings
                         */
    
                        /**
                         * Constructs a new CreateAppProfileRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a CreateAppProfileRequest.
                         * @implements ICreateAppProfileRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.ICreateAppProfileRequest=} [properties] Properties to set
                         */
                        function CreateAppProfileRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * CreateAppProfileRequest parent.
                         * @member {string} parent
                         * @memberof google.bigtable.admin.v2.CreateAppProfileRequest
                         * @instance
                         */
                        CreateAppProfileRequest.prototype.parent = "";
    
                        /**
                         * CreateAppProfileRequest appProfileId.
                         * @member {string} appProfileId
                         * @memberof google.bigtable.admin.v2.CreateAppProfileRequest
                         * @instance
                         */
                        CreateAppProfileRequest.prototype.appProfileId = "";
    
                        /**
                         * CreateAppProfileRequest appProfile.
                         * @member {google.bigtable.admin.v2.IAppProfile|null|undefined} appProfile
                         * @memberof google.bigtable.admin.v2.CreateAppProfileRequest
                         * @instance
                         */
                        CreateAppProfileRequest.prototype.appProfile = null;
    
                        /**
                         * CreateAppProfileRequest ignoreWarnings.
                         * @member {boolean} ignoreWarnings
                         * @memberof google.bigtable.admin.v2.CreateAppProfileRequest
                         * @instance
                         */
                        CreateAppProfileRequest.prototype.ignoreWarnings = false;
    
                        /**
                         * Creates a new CreateAppProfileRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.CreateAppProfileRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateAppProfileRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.CreateAppProfileRequest} CreateAppProfileRequest instance
                         */
                        CreateAppProfileRequest.create = function create(properties) {
                            return new CreateAppProfileRequest(properties);
                        };
    
                        /**
                         * Encodes the specified CreateAppProfileRequest message. Does not implicitly {@link google.bigtable.admin.v2.CreateAppProfileRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.CreateAppProfileRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateAppProfileRequest} message CreateAppProfileRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CreateAppProfileRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.parent != null && Object.hasOwnProperty.call(message, "parent"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.parent);
                            if (message.appProfileId != null && Object.hasOwnProperty.call(message, "appProfileId"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.appProfileId);
                            if (message.appProfile != null && Object.hasOwnProperty.call(message, "appProfile"))
                                $root.google.bigtable.admin.v2.AppProfile.encode(message.appProfile, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            if (message.ignoreWarnings != null && Object.hasOwnProperty.call(message, "ignoreWarnings"))
                                writer.uint32(/* id 4, wireType 0 =*/32).bool(message.ignoreWarnings);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified CreateAppProfileRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.CreateAppProfileRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.CreateAppProfileRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateAppProfileRequest} message CreateAppProfileRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CreateAppProfileRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a CreateAppProfileRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.CreateAppProfileRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.CreateAppProfileRequest} CreateAppProfileRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CreateAppProfileRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.CreateAppProfileRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.parent = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.appProfileId = reader.string();
                                        break;
                                    }
                                case 3: {
                                        message.appProfile = $root.google.bigtable.admin.v2.AppProfile.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 4: {
                                        message.ignoreWarnings = reader.bool();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a CreateAppProfileRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.CreateAppProfileRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.CreateAppProfileRequest} CreateAppProfileRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CreateAppProfileRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a CreateAppProfileRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.CreateAppProfileRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        CreateAppProfileRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                if (!$util.isString(message.parent))
                                    return "parent: string expected";
                            if (message.appProfileId != null && message.hasOwnProperty("appProfileId"))
                                if (!$util.isString(message.appProfileId))
                                    return "appProfileId: string expected";
                            if (message.appProfile != null && message.hasOwnProperty("appProfile")) {
                                var error = $root.google.bigtable.admin.v2.AppProfile.verify(message.appProfile);
                                if (error)
                                    return "appProfile." + error;
                            }
                            if (message.ignoreWarnings != null && message.hasOwnProperty("ignoreWarnings"))
                                if (typeof message.ignoreWarnings !== "boolean")
                                    return "ignoreWarnings: boolean expected";
                            return null;
                        };
    
                        /**
                         * Creates a CreateAppProfileRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.CreateAppProfileRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.CreateAppProfileRequest} CreateAppProfileRequest
                         */
                        CreateAppProfileRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.CreateAppProfileRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.CreateAppProfileRequest();
                            if (object.parent != null)
                                message.parent = String(object.parent);
                            if (object.appProfileId != null)
                                message.appProfileId = String(object.appProfileId);
                            if (object.appProfile != null) {
                                if (typeof object.appProfile !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.CreateAppProfileRequest.appProfile: object expected");
                                message.appProfile = $root.google.bigtable.admin.v2.AppProfile.fromObject(object.appProfile);
                            }
                            if (object.ignoreWarnings != null)
                                message.ignoreWarnings = Boolean(object.ignoreWarnings);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a CreateAppProfileRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.CreateAppProfileRequest
                         * @static
                         * @param {google.bigtable.admin.v2.CreateAppProfileRequest} message CreateAppProfileRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        CreateAppProfileRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.parent = "";
                                object.appProfileId = "";
                                object.appProfile = null;
                                object.ignoreWarnings = false;
                            }
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                object.parent = message.parent;
                            if (message.appProfileId != null && message.hasOwnProperty("appProfileId"))
                                object.appProfileId = message.appProfileId;
                            if (message.appProfile != null && message.hasOwnProperty("appProfile"))
                                object.appProfile = $root.google.bigtable.admin.v2.AppProfile.toObject(message.appProfile, options);
                            if (message.ignoreWarnings != null && message.hasOwnProperty("ignoreWarnings"))
                                object.ignoreWarnings = message.ignoreWarnings;
                            return object;
                        };
    
                        /**
                         * Converts this CreateAppProfileRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.CreateAppProfileRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        CreateAppProfileRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for CreateAppProfileRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.CreateAppProfileRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        CreateAppProfileRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.CreateAppProfileRequest";
                        };
    
                        return CreateAppProfileRequest;
                    })();
    
                    v2.GetAppProfileRequest = (function() {
    
                        /**
                         * Properties of a GetAppProfileRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IGetAppProfileRequest
                         * @property {string|null} [name] GetAppProfileRequest name
                         */
    
                        /**
                         * Constructs a new GetAppProfileRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a GetAppProfileRequest.
                         * @implements IGetAppProfileRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IGetAppProfileRequest=} [properties] Properties to set
                         */
                        function GetAppProfileRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * GetAppProfileRequest name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.GetAppProfileRequest
                         * @instance
                         */
                        GetAppProfileRequest.prototype.name = "";
    
                        /**
                         * Creates a new GetAppProfileRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.GetAppProfileRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IGetAppProfileRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.GetAppProfileRequest} GetAppProfileRequest instance
                         */
                        GetAppProfileRequest.create = function create(properties) {
                            return new GetAppProfileRequest(properties);
                        };
    
                        /**
                         * Encodes the specified GetAppProfileRequest message. Does not implicitly {@link google.bigtable.admin.v2.GetAppProfileRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.GetAppProfileRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IGetAppProfileRequest} message GetAppProfileRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        GetAppProfileRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified GetAppProfileRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.GetAppProfileRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.GetAppProfileRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IGetAppProfileRequest} message GetAppProfileRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        GetAppProfileRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a GetAppProfileRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.GetAppProfileRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.GetAppProfileRequest} GetAppProfileRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        GetAppProfileRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.GetAppProfileRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a GetAppProfileRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.GetAppProfileRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.GetAppProfileRequest} GetAppProfileRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        GetAppProfileRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a GetAppProfileRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.GetAppProfileRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        GetAppProfileRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a GetAppProfileRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.GetAppProfileRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.GetAppProfileRequest} GetAppProfileRequest
                         */
                        GetAppProfileRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.GetAppProfileRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.GetAppProfileRequest();
                            if (object.name != null)
                                message.name = String(object.name);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a GetAppProfileRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.GetAppProfileRequest
                         * @static
                         * @param {google.bigtable.admin.v2.GetAppProfileRequest} message GetAppProfileRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        GetAppProfileRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults)
                                object.name = "";
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            return object;
                        };
    
                        /**
                         * Converts this GetAppProfileRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.GetAppProfileRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        GetAppProfileRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for GetAppProfileRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.GetAppProfileRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        GetAppProfileRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.GetAppProfileRequest";
                        };
    
                        return GetAppProfileRequest;
                    })();
    
                    v2.ListAppProfilesRequest = (function() {
    
                        /**
                         * Properties of a ListAppProfilesRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IListAppProfilesRequest
                         * @property {string|null} [parent] ListAppProfilesRequest parent
                         * @property {number|null} [pageSize] ListAppProfilesRequest pageSize
                         * @property {string|null} [pageToken] ListAppProfilesRequest pageToken
                         */
    
                        /**
                         * Constructs a new ListAppProfilesRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a ListAppProfilesRequest.
                         * @implements IListAppProfilesRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IListAppProfilesRequest=} [properties] Properties to set
                         */
                        function ListAppProfilesRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * ListAppProfilesRequest parent.
                         * @member {string} parent
                         * @memberof google.bigtable.admin.v2.ListAppProfilesRequest
                         * @instance
                         */
                        ListAppProfilesRequest.prototype.parent = "";
    
                        /**
                         * ListAppProfilesRequest pageSize.
                         * @member {number} pageSize
                         * @memberof google.bigtable.admin.v2.ListAppProfilesRequest
                         * @instance
                         */
                        ListAppProfilesRequest.prototype.pageSize = 0;
    
                        /**
                         * ListAppProfilesRequest pageToken.
                         * @member {string} pageToken
                         * @memberof google.bigtable.admin.v2.ListAppProfilesRequest
                         * @instance
                         */
                        ListAppProfilesRequest.prototype.pageToken = "";
    
                        /**
                         * Creates a new ListAppProfilesRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.ListAppProfilesRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IListAppProfilesRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.ListAppProfilesRequest} ListAppProfilesRequest instance
                         */
                        ListAppProfilesRequest.create = function create(properties) {
                            return new ListAppProfilesRequest(properties);
                        };
    
                        /**
                         * Encodes the specified ListAppProfilesRequest message. Does not implicitly {@link google.bigtable.admin.v2.ListAppProfilesRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.ListAppProfilesRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IListAppProfilesRequest} message ListAppProfilesRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListAppProfilesRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.parent != null && Object.hasOwnProperty.call(message, "parent"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.parent);
                            if (message.pageToken != null && Object.hasOwnProperty.call(message, "pageToken"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.pageToken);
                            if (message.pageSize != null && Object.hasOwnProperty.call(message, "pageSize"))
                                writer.uint32(/* id 3, wireType 0 =*/24).int32(message.pageSize);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified ListAppProfilesRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.ListAppProfilesRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.ListAppProfilesRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IListAppProfilesRequest} message ListAppProfilesRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListAppProfilesRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a ListAppProfilesRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.ListAppProfilesRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.ListAppProfilesRequest} ListAppProfilesRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListAppProfilesRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.ListAppProfilesRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.parent = reader.string();
                                        break;
                                    }
                                case 3: {
                                        message.pageSize = reader.int32();
                                        break;
                                    }
                                case 2: {
                                        message.pageToken = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a ListAppProfilesRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.ListAppProfilesRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.ListAppProfilesRequest} ListAppProfilesRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListAppProfilesRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a ListAppProfilesRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.ListAppProfilesRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        ListAppProfilesRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                if (!$util.isString(message.parent))
                                    return "parent: string expected";
                            if (message.pageSize != null && message.hasOwnProperty("pageSize"))
                                if (!$util.isInteger(message.pageSize))
                                    return "pageSize: integer expected";
                            if (message.pageToken != null && message.hasOwnProperty("pageToken"))
                                if (!$util.isString(message.pageToken))
                                    return "pageToken: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a ListAppProfilesRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.ListAppProfilesRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.ListAppProfilesRequest} ListAppProfilesRequest
                         */
                        ListAppProfilesRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.ListAppProfilesRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.ListAppProfilesRequest();
                            if (object.parent != null)
                                message.parent = String(object.parent);
                            if (object.pageSize != null)
                                message.pageSize = object.pageSize | 0;
                            if (object.pageToken != null)
                                message.pageToken = String(object.pageToken);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a ListAppProfilesRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.ListAppProfilesRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ListAppProfilesRequest} message ListAppProfilesRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        ListAppProfilesRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.parent = "";
                                object.pageToken = "";
                                object.pageSize = 0;
                            }
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                object.parent = message.parent;
                            if (message.pageToken != null && message.hasOwnProperty("pageToken"))
                                object.pageToken = message.pageToken;
                            if (message.pageSize != null && message.hasOwnProperty("pageSize"))
                                object.pageSize = message.pageSize;
                            return object;
                        };
    
                        /**
                         * Converts this ListAppProfilesRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.ListAppProfilesRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        ListAppProfilesRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for ListAppProfilesRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.ListAppProfilesRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        ListAppProfilesRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.ListAppProfilesRequest";
                        };
    
                        return ListAppProfilesRequest;
                    })();
    
                    v2.ListAppProfilesResponse = (function() {
    
                        /**
                         * Properties of a ListAppProfilesResponse.
                         * @memberof google.bigtable.admin.v2
                         * @interface IListAppProfilesResponse
                         * @property {Array.<google.bigtable.admin.v2.IAppProfile>|null} [appProfiles] ListAppProfilesResponse appProfiles
                         * @property {string|null} [nextPageToken] ListAppProfilesResponse nextPageToken
                         * @property {Array.<string>|null} [failedLocations] ListAppProfilesResponse failedLocations
                         */
    
                        /**
                         * Constructs a new ListAppProfilesResponse.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a ListAppProfilesResponse.
                         * @implements IListAppProfilesResponse
                         * @constructor
                         * @param {google.bigtable.admin.v2.IListAppProfilesResponse=} [properties] Properties to set
                         */
                        function ListAppProfilesResponse(properties) {
                            this.appProfiles = [];
                            this.failedLocations = [];
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * ListAppProfilesResponse appProfiles.
                         * @member {Array.<google.bigtable.admin.v2.IAppProfile>} appProfiles
                         * @memberof google.bigtable.admin.v2.ListAppProfilesResponse
                         * @instance
                         */
                        ListAppProfilesResponse.prototype.appProfiles = $util.emptyArray;
    
                        /**
                         * ListAppProfilesResponse nextPageToken.
                         * @member {string} nextPageToken
                         * @memberof google.bigtable.admin.v2.ListAppProfilesResponse
                         * @instance
                         */
                        ListAppProfilesResponse.prototype.nextPageToken = "";
    
                        /**
                         * ListAppProfilesResponse failedLocations.
                         * @member {Array.<string>} failedLocations
                         * @memberof google.bigtable.admin.v2.ListAppProfilesResponse
                         * @instance
                         */
                        ListAppProfilesResponse.prototype.failedLocations = $util.emptyArray;
    
                        /**
                         * Creates a new ListAppProfilesResponse instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.ListAppProfilesResponse
                         * @static
                         * @param {google.bigtable.admin.v2.IListAppProfilesResponse=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.ListAppProfilesResponse} ListAppProfilesResponse instance
                         */
                        ListAppProfilesResponse.create = function create(properties) {
                            return new ListAppProfilesResponse(properties);
                        };
    
                        /**
                         * Encodes the specified ListAppProfilesResponse message. Does not implicitly {@link google.bigtable.admin.v2.ListAppProfilesResponse.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.ListAppProfilesResponse
                         * @static
                         * @param {google.bigtable.admin.v2.IListAppProfilesResponse} message ListAppProfilesResponse message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListAppProfilesResponse.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.appProfiles != null && message.appProfiles.length)
                                for (var i = 0; i < message.appProfiles.length; ++i)
                                    $root.google.bigtable.admin.v2.AppProfile.encode(message.appProfiles[i], writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            if (message.nextPageToken != null && Object.hasOwnProperty.call(message, "nextPageToken"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.nextPageToken);
                            if (message.failedLocations != null && message.failedLocations.length)
                                for (var i = 0; i < message.failedLocations.length; ++i)
                                    writer.uint32(/* id 3, wireType 2 =*/26).string(message.failedLocations[i]);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified ListAppProfilesResponse message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.ListAppProfilesResponse.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.ListAppProfilesResponse
                         * @static
                         * @param {google.bigtable.admin.v2.IListAppProfilesResponse} message ListAppProfilesResponse message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListAppProfilesResponse.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a ListAppProfilesResponse message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.ListAppProfilesResponse
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.ListAppProfilesResponse} ListAppProfilesResponse
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListAppProfilesResponse.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.ListAppProfilesResponse();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        if (!(message.appProfiles && message.appProfiles.length))
                                            message.appProfiles = [];
                                        message.appProfiles.push($root.google.bigtable.admin.v2.AppProfile.decode(reader, reader.uint32()));
                                        break;
                                    }
                                case 2: {
                                        message.nextPageToken = reader.string();
                                        break;
                                    }
                                case 3: {
                                        if (!(message.failedLocations && message.failedLocations.length))
                                            message.failedLocations = [];
                                        message.failedLocations.push(reader.string());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a ListAppProfilesResponse message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.ListAppProfilesResponse
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.ListAppProfilesResponse} ListAppProfilesResponse
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListAppProfilesResponse.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a ListAppProfilesResponse message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.ListAppProfilesResponse
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        ListAppProfilesResponse.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.appProfiles != null && message.hasOwnProperty("appProfiles")) {
                                if (!Array.isArray(message.appProfiles))
                                    return "appProfiles: array expected";
                                for (var i = 0; i < message.appProfiles.length; ++i) {
                                    var error = $root.google.bigtable.admin.v2.AppProfile.verify(message.appProfiles[i]);
                                    if (error)
                                        return "appProfiles." + error;
                                }
                            }
                            if (message.nextPageToken != null && message.hasOwnProperty("nextPageToken"))
                                if (!$util.isString(message.nextPageToken))
                                    return "nextPageToken: string expected";
                            if (message.failedLocations != null && message.hasOwnProperty("failedLocations")) {
                                if (!Array.isArray(message.failedLocations))
                                    return "failedLocations: array expected";
                                for (var i = 0; i < message.failedLocations.length; ++i)
                                    if (!$util.isString(message.failedLocations[i]))
                                        return "failedLocations: string[] expected";
                            }
                            return null;
                        };
    
                        /**
                         * Creates a ListAppProfilesResponse message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.ListAppProfilesResponse
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.ListAppProfilesResponse} ListAppProfilesResponse
                         */
                        ListAppProfilesResponse.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.ListAppProfilesResponse)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.ListAppProfilesResponse();
                            if (object.appProfiles) {
                                if (!Array.isArray(object.appProfiles))
                                    throw TypeError(".google.bigtable.admin.v2.ListAppProfilesResponse.appProfiles: array expected");
                                message.appProfiles = [];
                                for (var i = 0; i < object.appProfiles.length; ++i) {
                                    if (typeof object.appProfiles[i] !== "object")
                                        throw TypeError(".google.bigtable.admin.v2.ListAppProfilesResponse.appProfiles: object expected");
                                    message.appProfiles[i] = $root.google.bigtable.admin.v2.AppProfile.fromObject(object.appProfiles[i]);
                                }
                            }
                            if (object.nextPageToken != null)
                                message.nextPageToken = String(object.nextPageToken);
                            if (object.failedLocations) {
                                if (!Array.isArray(object.failedLocations))
                                    throw TypeError(".google.bigtable.admin.v2.ListAppProfilesResponse.failedLocations: array expected");
                                message.failedLocations = [];
                                for (var i = 0; i < object.failedLocations.length; ++i)
                                    message.failedLocations[i] = String(object.failedLocations[i]);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a ListAppProfilesResponse message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.ListAppProfilesResponse
                         * @static
                         * @param {google.bigtable.admin.v2.ListAppProfilesResponse} message ListAppProfilesResponse
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        ListAppProfilesResponse.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.arrays || options.defaults) {
                                object.appProfiles = [];
                                object.failedLocations = [];
                            }
                            if (options.defaults)
                                object.nextPageToken = "";
                            if (message.appProfiles && message.appProfiles.length) {
                                object.appProfiles = [];
                                for (var j = 0; j < message.appProfiles.length; ++j)
                                    object.appProfiles[j] = $root.google.bigtable.admin.v2.AppProfile.toObject(message.appProfiles[j], options);
                            }
                            if (message.nextPageToken != null && message.hasOwnProperty("nextPageToken"))
                                object.nextPageToken = message.nextPageToken;
                            if (message.failedLocations && message.failedLocations.length) {
                                object.failedLocations = [];
                                for (var j = 0; j < message.failedLocations.length; ++j)
                                    object.failedLocations[j] = message.failedLocations[j];
                            }
                            return object;
                        };
    
                        /**
                         * Converts this ListAppProfilesResponse to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.ListAppProfilesResponse
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        ListAppProfilesResponse.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for ListAppProfilesResponse
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.ListAppProfilesResponse
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        ListAppProfilesResponse.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.ListAppProfilesResponse";
                        };
    
                        return ListAppProfilesResponse;
                    })();
    
                    v2.UpdateAppProfileRequest = (function() {
    
                        /**
                         * Properties of an UpdateAppProfileRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IUpdateAppProfileRequest
                         * @property {google.bigtable.admin.v2.IAppProfile|null} [appProfile] UpdateAppProfileRequest appProfile
                         * @property {google.protobuf.IFieldMask|null} [updateMask] UpdateAppProfileRequest updateMask
                         * @property {boolean|null} [ignoreWarnings] UpdateAppProfileRequest ignoreWarnings
                         */
    
                        /**
                         * Constructs a new UpdateAppProfileRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents an UpdateAppProfileRequest.
                         * @implements IUpdateAppProfileRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IUpdateAppProfileRequest=} [properties] Properties to set
                         */
                        function UpdateAppProfileRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * UpdateAppProfileRequest appProfile.
                         * @member {google.bigtable.admin.v2.IAppProfile|null|undefined} appProfile
                         * @memberof google.bigtable.admin.v2.UpdateAppProfileRequest
                         * @instance
                         */
                        UpdateAppProfileRequest.prototype.appProfile = null;
    
                        /**
                         * UpdateAppProfileRequest updateMask.
                         * @member {google.protobuf.IFieldMask|null|undefined} updateMask
                         * @memberof google.bigtable.admin.v2.UpdateAppProfileRequest
                         * @instance
                         */
                        UpdateAppProfileRequest.prototype.updateMask = null;
    
                        /**
                         * UpdateAppProfileRequest ignoreWarnings.
                         * @member {boolean} ignoreWarnings
                         * @memberof google.bigtable.admin.v2.UpdateAppProfileRequest
                         * @instance
                         */
                        UpdateAppProfileRequest.prototype.ignoreWarnings = false;
    
                        /**
                         * Creates a new UpdateAppProfileRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.UpdateAppProfileRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IUpdateAppProfileRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.UpdateAppProfileRequest} UpdateAppProfileRequest instance
                         */
                        UpdateAppProfileRequest.create = function create(properties) {
                            return new UpdateAppProfileRequest(properties);
                        };
    
                        /**
                         * Encodes the specified UpdateAppProfileRequest message. Does not implicitly {@link google.bigtable.admin.v2.UpdateAppProfileRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.UpdateAppProfileRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IUpdateAppProfileRequest} message UpdateAppProfileRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        UpdateAppProfileRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.appProfile != null && Object.hasOwnProperty.call(message, "appProfile"))
                                $root.google.bigtable.admin.v2.AppProfile.encode(message.appProfile, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            if (message.updateMask != null && Object.hasOwnProperty.call(message, "updateMask"))
                                $root.google.protobuf.FieldMask.encode(message.updateMask, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            if (message.ignoreWarnings != null && Object.hasOwnProperty.call(message, "ignoreWarnings"))
                                writer.uint32(/* id 3, wireType 0 =*/24).bool(message.ignoreWarnings);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified UpdateAppProfileRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.UpdateAppProfileRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.UpdateAppProfileRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IUpdateAppProfileRequest} message UpdateAppProfileRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        UpdateAppProfileRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes an UpdateAppProfileRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.UpdateAppProfileRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.UpdateAppProfileRequest} UpdateAppProfileRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        UpdateAppProfileRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.UpdateAppProfileRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.appProfile = $root.google.bigtable.admin.v2.AppProfile.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 2: {
                                        message.updateMask = $root.google.protobuf.FieldMask.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 3: {
                                        message.ignoreWarnings = reader.bool();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes an UpdateAppProfileRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.UpdateAppProfileRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.UpdateAppProfileRequest} UpdateAppProfileRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        UpdateAppProfileRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies an UpdateAppProfileRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.UpdateAppProfileRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        UpdateAppProfileRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.appProfile != null && message.hasOwnProperty("appProfile")) {
                                var error = $root.google.bigtable.admin.v2.AppProfile.verify(message.appProfile);
                                if (error)
                                    return "appProfile." + error;
                            }
                            if (message.updateMask != null && message.hasOwnProperty("updateMask")) {
                                var error = $root.google.protobuf.FieldMask.verify(message.updateMask);
                                if (error)
                                    return "updateMask." + error;
                            }
                            if (message.ignoreWarnings != null && message.hasOwnProperty("ignoreWarnings"))
                                if (typeof message.ignoreWarnings !== "boolean")
                                    return "ignoreWarnings: boolean expected";
                            return null;
                        };
    
                        /**
                         * Creates an UpdateAppProfileRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.UpdateAppProfileRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.UpdateAppProfileRequest} UpdateAppProfileRequest
                         */
                        UpdateAppProfileRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.UpdateAppProfileRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.UpdateAppProfileRequest();
                            if (object.appProfile != null) {
                                if (typeof object.appProfile !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.UpdateAppProfileRequest.appProfile: object expected");
                                message.appProfile = $root.google.bigtable.admin.v2.AppProfile.fromObject(object.appProfile);
                            }
                            if (object.updateMask != null) {
                                if (typeof object.updateMask !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.UpdateAppProfileRequest.updateMask: object expected");
                                message.updateMask = $root.google.protobuf.FieldMask.fromObject(object.updateMask);
                            }
                            if (object.ignoreWarnings != null)
                                message.ignoreWarnings = Boolean(object.ignoreWarnings);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from an UpdateAppProfileRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.UpdateAppProfileRequest
                         * @static
                         * @param {google.bigtable.admin.v2.UpdateAppProfileRequest} message UpdateAppProfileRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        UpdateAppProfileRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.appProfile = null;
                                object.updateMask = null;
                                object.ignoreWarnings = false;
                            }
                            if (message.appProfile != null && message.hasOwnProperty("appProfile"))
                                object.appProfile = $root.google.bigtable.admin.v2.AppProfile.toObject(message.appProfile, options);
                            if (message.updateMask != null && message.hasOwnProperty("updateMask"))
                                object.updateMask = $root.google.protobuf.FieldMask.toObject(message.updateMask, options);
                            if (message.ignoreWarnings != null && message.hasOwnProperty("ignoreWarnings"))
                                object.ignoreWarnings = message.ignoreWarnings;
                            return object;
                        };
    
                        /**
                         * Converts this UpdateAppProfileRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.UpdateAppProfileRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        UpdateAppProfileRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for UpdateAppProfileRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.UpdateAppProfileRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        UpdateAppProfileRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.UpdateAppProfileRequest";
                        };
    
                        return UpdateAppProfileRequest;
                    })();
    
                    v2.DeleteAppProfileRequest = (function() {
    
                        /**
                         * Properties of a DeleteAppProfileRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IDeleteAppProfileRequest
                         * @property {string|null} [name] DeleteAppProfileRequest name
                         * @property {boolean|null} [ignoreWarnings] DeleteAppProfileRequest ignoreWarnings
                         */
    
                        /**
                         * Constructs a new DeleteAppProfileRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a DeleteAppProfileRequest.
                         * @implements IDeleteAppProfileRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IDeleteAppProfileRequest=} [properties] Properties to set
                         */
                        function DeleteAppProfileRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * DeleteAppProfileRequest name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.DeleteAppProfileRequest
                         * @instance
                         */
                        DeleteAppProfileRequest.prototype.name = "";
    
                        /**
                         * DeleteAppProfileRequest ignoreWarnings.
                         * @member {boolean} ignoreWarnings
                         * @memberof google.bigtable.admin.v2.DeleteAppProfileRequest
                         * @instance
                         */
                        DeleteAppProfileRequest.prototype.ignoreWarnings = false;
    
                        /**
                         * Creates a new DeleteAppProfileRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.DeleteAppProfileRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IDeleteAppProfileRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.DeleteAppProfileRequest} DeleteAppProfileRequest instance
                         */
                        DeleteAppProfileRequest.create = function create(properties) {
                            return new DeleteAppProfileRequest(properties);
                        };
    
                        /**
                         * Encodes the specified DeleteAppProfileRequest message. Does not implicitly {@link google.bigtable.admin.v2.DeleteAppProfileRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.DeleteAppProfileRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IDeleteAppProfileRequest} message DeleteAppProfileRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        DeleteAppProfileRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            if (message.ignoreWarnings != null && Object.hasOwnProperty.call(message, "ignoreWarnings"))
                                writer.uint32(/* id 2, wireType 0 =*/16).bool(message.ignoreWarnings);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified DeleteAppProfileRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.DeleteAppProfileRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.DeleteAppProfileRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IDeleteAppProfileRequest} message DeleteAppProfileRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        DeleteAppProfileRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a DeleteAppProfileRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.DeleteAppProfileRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.DeleteAppProfileRequest} DeleteAppProfileRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        DeleteAppProfileRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.DeleteAppProfileRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.ignoreWarnings = reader.bool();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a DeleteAppProfileRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.DeleteAppProfileRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.DeleteAppProfileRequest} DeleteAppProfileRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        DeleteAppProfileRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a DeleteAppProfileRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.DeleteAppProfileRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        DeleteAppProfileRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            if (message.ignoreWarnings != null && message.hasOwnProperty("ignoreWarnings"))
                                if (typeof message.ignoreWarnings !== "boolean")
                                    return "ignoreWarnings: boolean expected";
                            return null;
                        };
    
                        /**
                         * Creates a DeleteAppProfileRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.DeleteAppProfileRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.DeleteAppProfileRequest} DeleteAppProfileRequest
                         */
                        DeleteAppProfileRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.DeleteAppProfileRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.DeleteAppProfileRequest();
                            if (object.name != null)
                                message.name = String(object.name);
                            if (object.ignoreWarnings != null)
                                message.ignoreWarnings = Boolean(object.ignoreWarnings);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a DeleteAppProfileRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.DeleteAppProfileRequest
                         * @static
                         * @param {google.bigtable.admin.v2.DeleteAppProfileRequest} message DeleteAppProfileRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        DeleteAppProfileRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.name = "";
                                object.ignoreWarnings = false;
                            }
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            if (message.ignoreWarnings != null && message.hasOwnProperty("ignoreWarnings"))
                                object.ignoreWarnings = message.ignoreWarnings;
                            return object;
                        };
    
                        /**
                         * Converts this DeleteAppProfileRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.DeleteAppProfileRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        DeleteAppProfileRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for DeleteAppProfileRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.DeleteAppProfileRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        DeleteAppProfileRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.DeleteAppProfileRequest";
                        };
    
                        return DeleteAppProfileRequest;
                    })();
    
                    v2.UpdateAppProfileMetadata = (function() {
    
                        /**
                         * Properties of an UpdateAppProfileMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @interface IUpdateAppProfileMetadata
                         */
    
                        /**
                         * Constructs a new UpdateAppProfileMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents an UpdateAppProfileMetadata.
                         * @implements IUpdateAppProfileMetadata
                         * @constructor
                         * @param {google.bigtable.admin.v2.IUpdateAppProfileMetadata=} [properties] Properties to set
                         */
                        function UpdateAppProfileMetadata(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * Creates a new UpdateAppProfileMetadata instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.UpdateAppProfileMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.IUpdateAppProfileMetadata=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.UpdateAppProfileMetadata} UpdateAppProfileMetadata instance
                         */
                        UpdateAppProfileMetadata.create = function create(properties) {
                            return new UpdateAppProfileMetadata(properties);
                        };
    
                        /**
                         * Encodes the specified UpdateAppProfileMetadata message. Does not implicitly {@link google.bigtable.admin.v2.UpdateAppProfileMetadata.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.UpdateAppProfileMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.IUpdateAppProfileMetadata} message UpdateAppProfileMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        UpdateAppProfileMetadata.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified UpdateAppProfileMetadata message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.UpdateAppProfileMetadata.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.UpdateAppProfileMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.IUpdateAppProfileMetadata} message UpdateAppProfileMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        UpdateAppProfileMetadata.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes an UpdateAppProfileMetadata message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.UpdateAppProfileMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.UpdateAppProfileMetadata} UpdateAppProfileMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        UpdateAppProfileMetadata.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.UpdateAppProfileMetadata();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes an UpdateAppProfileMetadata message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.UpdateAppProfileMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.UpdateAppProfileMetadata} UpdateAppProfileMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        UpdateAppProfileMetadata.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies an UpdateAppProfileMetadata message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.UpdateAppProfileMetadata
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        UpdateAppProfileMetadata.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            return null;
                        };
    
                        /**
                         * Creates an UpdateAppProfileMetadata message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.UpdateAppProfileMetadata
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.UpdateAppProfileMetadata} UpdateAppProfileMetadata
                         */
                        UpdateAppProfileMetadata.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.UpdateAppProfileMetadata)
                                return object;
                            return new $root.google.bigtable.admin.v2.UpdateAppProfileMetadata();
                        };
    
                        /**
                         * Creates a plain object from an UpdateAppProfileMetadata message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.UpdateAppProfileMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.UpdateAppProfileMetadata} message UpdateAppProfileMetadata
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        UpdateAppProfileMetadata.toObject = function toObject() {
                            return {};
                        };
    
                        /**
                         * Converts this UpdateAppProfileMetadata to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.UpdateAppProfileMetadata
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        UpdateAppProfileMetadata.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for UpdateAppProfileMetadata
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.UpdateAppProfileMetadata
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        UpdateAppProfileMetadata.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.UpdateAppProfileMetadata";
                        };
    
                        return UpdateAppProfileMetadata;
                    })();
    
                    v2.ListHotTabletsRequest = (function() {
    
                        /**
                         * Properties of a ListHotTabletsRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IListHotTabletsRequest
                         * @property {string|null} [parent] ListHotTabletsRequest parent
                         * @property {google.protobuf.ITimestamp|null} [startTime] ListHotTabletsRequest startTime
                         * @property {google.protobuf.ITimestamp|null} [endTime] ListHotTabletsRequest endTime
                         * @property {number|null} [pageSize] ListHotTabletsRequest pageSize
                         * @property {string|null} [pageToken] ListHotTabletsRequest pageToken
                         */
    
                        /**
                         * Constructs a new ListHotTabletsRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a ListHotTabletsRequest.
                         * @implements IListHotTabletsRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IListHotTabletsRequest=} [properties] Properties to set
                         */
                        function ListHotTabletsRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * ListHotTabletsRequest parent.
                         * @member {string} parent
                         * @memberof google.bigtable.admin.v2.ListHotTabletsRequest
                         * @instance
                         */
                        ListHotTabletsRequest.prototype.parent = "";
    
                        /**
                         * ListHotTabletsRequest startTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} startTime
                         * @memberof google.bigtable.admin.v2.ListHotTabletsRequest
                         * @instance
                         */
                        ListHotTabletsRequest.prototype.startTime = null;
    
                        /**
                         * ListHotTabletsRequest endTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} endTime
                         * @memberof google.bigtable.admin.v2.ListHotTabletsRequest
                         * @instance
                         */
                        ListHotTabletsRequest.prototype.endTime = null;
    
                        /**
                         * ListHotTabletsRequest pageSize.
                         * @member {number} pageSize
                         * @memberof google.bigtable.admin.v2.ListHotTabletsRequest
                         * @instance
                         */
                        ListHotTabletsRequest.prototype.pageSize = 0;
    
                        /**
                         * ListHotTabletsRequest pageToken.
                         * @member {string} pageToken
                         * @memberof google.bigtable.admin.v2.ListHotTabletsRequest
                         * @instance
                         */
                        ListHotTabletsRequest.prototype.pageToken = "";
    
                        /**
                         * Creates a new ListHotTabletsRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.ListHotTabletsRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IListHotTabletsRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.ListHotTabletsRequest} ListHotTabletsRequest instance
                         */
                        ListHotTabletsRequest.create = function create(properties) {
                            return new ListHotTabletsRequest(properties);
                        };
    
                        /**
                         * Encodes the specified ListHotTabletsRequest message. Does not implicitly {@link google.bigtable.admin.v2.ListHotTabletsRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.ListHotTabletsRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IListHotTabletsRequest} message ListHotTabletsRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListHotTabletsRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.parent != null && Object.hasOwnProperty.call(message, "parent"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.parent);
                            if (message.startTime != null && Object.hasOwnProperty.call(message, "startTime"))
                                $root.google.protobuf.Timestamp.encode(message.startTime, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            if (message.endTime != null && Object.hasOwnProperty.call(message, "endTime"))
                                $root.google.protobuf.Timestamp.encode(message.endTime, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            if (message.pageSize != null && Object.hasOwnProperty.call(message, "pageSize"))
                                writer.uint32(/* id 4, wireType 0 =*/32).int32(message.pageSize);
                            if (message.pageToken != null && Object.hasOwnProperty.call(message, "pageToken"))
                                writer.uint32(/* id 5, wireType 2 =*/42).string(message.pageToken);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified ListHotTabletsRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.ListHotTabletsRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.ListHotTabletsRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IListHotTabletsRequest} message ListHotTabletsRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListHotTabletsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a ListHotTabletsRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.ListHotTabletsRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.ListHotTabletsRequest} ListHotTabletsRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListHotTabletsRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.ListHotTabletsRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.parent = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.startTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 3: {
                                        message.endTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 4: {
                                        message.pageSize = reader.int32();
                                        break;
                                    }
                                case 5: {
                                        message.pageToken = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a ListHotTabletsRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.ListHotTabletsRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.ListHotTabletsRequest} ListHotTabletsRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListHotTabletsRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a ListHotTabletsRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.ListHotTabletsRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        ListHotTabletsRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                if (!$util.isString(message.parent))
                                    return "parent: string expected";
                            if (message.startTime != null && message.hasOwnProperty("startTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.startTime);
                                if (error)
                                    return "startTime." + error;
                            }
                            if (message.endTime != null && message.hasOwnProperty("endTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.endTime);
                                if (error)
                                    return "endTime." + error;
                            }
                            if (message.pageSize != null && message.hasOwnProperty("pageSize"))
                                if (!$util.isInteger(message.pageSize))
                                    return "pageSize: integer expected";
                            if (message.pageToken != null && message.hasOwnProperty("pageToken"))
                                if (!$util.isString(message.pageToken))
                                    return "pageToken: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a ListHotTabletsRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.ListHotTabletsRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.ListHotTabletsRequest} ListHotTabletsRequest
                         */
                        ListHotTabletsRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.ListHotTabletsRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.ListHotTabletsRequest();
                            if (object.parent != null)
                                message.parent = String(object.parent);
                            if (object.startTime != null) {
                                if (typeof object.startTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.ListHotTabletsRequest.startTime: object expected");
                                message.startTime = $root.google.protobuf.Timestamp.fromObject(object.startTime);
                            }
                            if (object.endTime != null) {
                                if (typeof object.endTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.ListHotTabletsRequest.endTime: object expected");
                                message.endTime = $root.google.protobuf.Timestamp.fromObject(object.endTime);
                            }
                            if (object.pageSize != null)
                                message.pageSize = object.pageSize | 0;
                            if (object.pageToken != null)
                                message.pageToken = String(object.pageToken);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a ListHotTabletsRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.ListHotTabletsRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ListHotTabletsRequest} message ListHotTabletsRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        ListHotTabletsRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.parent = "";
                                object.startTime = null;
                                object.endTime = null;
                                object.pageSize = 0;
                                object.pageToken = "";
                            }
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                object.parent = message.parent;
                            if (message.startTime != null && message.hasOwnProperty("startTime"))
                                object.startTime = $root.google.protobuf.Timestamp.toObject(message.startTime, options);
                            if (message.endTime != null && message.hasOwnProperty("endTime"))
                                object.endTime = $root.google.protobuf.Timestamp.toObject(message.endTime, options);
                            if (message.pageSize != null && message.hasOwnProperty("pageSize"))
                                object.pageSize = message.pageSize;
                            if (message.pageToken != null && message.hasOwnProperty("pageToken"))
                                object.pageToken = message.pageToken;
                            return object;
                        };
    
                        /**
                         * Converts this ListHotTabletsRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.ListHotTabletsRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        ListHotTabletsRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for ListHotTabletsRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.ListHotTabletsRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        ListHotTabletsRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.ListHotTabletsRequest";
                        };
    
                        return ListHotTabletsRequest;
                    })();
    
                    v2.ListHotTabletsResponse = (function() {
    
                        /**
                         * Properties of a ListHotTabletsResponse.
                         * @memberof google.bigtable.admin.v2
                         * @interface IListHotTabletsResponse
                         * @property {Array.<google.bigtable.admin.v2.IHotTablet>|null} [hotTablets] ListHotTabletsResponse hotTablets
                         * @property {string|null} [nextPageToken] ListHotTabletsResponse nextPageToken
                         */
    
                        /**
                         * Constructs a new ListHotTabletsResponse.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a ListHotTabletsResponse.
                         * @implements IListHotTabletsResponse
                         * @constructor
                         * @param {google.bigtable.admin.v2.IListHotTabletsResponse=} [properties] Properties to set
                         */
                        function ListHotTabletsResponse(properties) {
                            this.hotTablets = [];
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * ListHotTabletsResponse hotTablets.
                         * @member {Array.<google.bigtable.admin.v2.IHotTablet>} hotTablets
                         * @memberof google.bigtable.admin.v2.ListHotTabletsResponse
                         * @instance
                         */
                        ListHotTabletsResponse.prototype.hotTablets = $util.emptyArray;
    
                        /**
                         * ListHotTabletsResponse nextPageToken.
                         * @member {string} nextPageToken
                         * @memberof google.bigtable.admin.v2.ListHotTabletsResponse
                         * @instance
                         */
                        ListHotTabletsResponse.prototype.nextPageToken = "";
    
                        /**
                         * Creates a new ListHotTabletsResponse instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.ListHotTabletsResponse
                         * @static
                         * @param {google.bigtable.admin.v2.IListHotTabletsResponse=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.ListHotTabletsResponse} ListHotTabletsResponse instance
                         */
                        ListHotTabletsResponse.create = function create(properties) {
                            return new ListHotTabletsResponse(properties);
                        };
    
                        /**
                         * Encodes the specified ListHotTabletsResponse message. Does not implicitly {@link google.bigtable.admin.v2.ListHotTabletsResponse.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.ListHotTabletsResponse
                         * @static
                         * @param {google.bigtable.admin.v2.IListHotTabletsResponse} message ListHotTabletsResponse message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListHotTabletsResponse.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.hotTablets != null && message.hotTablets.length)
                                for (var i = 0; i < message.hotTablets.length; ++i)
                                    $root.google.bigtable.admin.v2.HotTablet.encode(message.hotTablets[i], writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            if (message.nextPageToken != null && Object.hasOwnProperty.call(message, "nextPageToken"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.nextPageToken);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified ListHotTabletsResponse message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.ListHotTabletsResponse.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.ListHotTabletsResponse
                         * @static
                         * @param {google.bigtable.admin.v2.IListHotTabletsResponse} message ListHotTabletsResponse message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListHotTabletsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a ListHotTabletsResponse message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.ListHotTabletsResponse
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.ListHotTabletsResponse} ListHotTabletsResponse
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListHotTabletsResponse.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.ListHotTabletsResponse();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        if (!(message.hotTablets && message.hotTablets.length))
                                            message.hotTablets = [];
                                        message.hotTablets.push($root.google.bigtable.admin.v2.HotTablet.decode(reader, reader.uint32()));
                                        break;
                                    }
                                case 2: {
                                        message.nextPageToken = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a ListHotTabletsResponse message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.ListHotTabletsResponse
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.ListHotTabletsResponse} ListHotTabletsResponse
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListHotTabletsResponse.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a ListHotTabletsResponse message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.ListHotTabletsResponse
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        ListHotTabletsResponse.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.hotTablets != null && message.hasOwnProperty("hotTablets")) {
                                if (!Array.isArray(message.hotTablets))
                                    return "hotTablets: array expected";
                                for (var i = 0; i < message.hotTablets.length; ++i) {
                                    var error = $root.google.bigtable.admin.v2.HotTablet.verify(message.hotTablets[i]);
                                    if (error)
                                        return "hotTablets." + error;
                                }
                            }
                            if (message.nextPageToken != null && message.hasOwnProperty("nextPageToken"))
                                if (!$util.isString(message.nextPageToken))
                                    return "nextPageToken: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a ListHotTabletsResponse message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.ListHotTabletsResponse
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.ListHotTabletsResponse} ListHotTabletsResponse
                         */
                        ListHotTabletsResponse.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.ListHotTabletsResponse)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.ListHotTabletsResponse();
                            if (object.hotTablets) {
                                if (!Array.isArray(object.hotTablets))
                                    throw TypeError(".google.bigtable.admin.v2.ListHotTabletsResponse.hotTablets: array expected");
                                message.hotTablets = [];
                                for (var i = 0; i < object.hotTablets.length; ++i) {
                                    if (typeof object.hotTablets[i] !== "object")
                                        throw TypeError(".google.bigtable.admin.v2.ListHotTabletsResponse.hotTablets: object expected");
                                    message.hotTablets[i] = $root.google.bigtable.admin.v2.HotTablet.fromObject(object.hotTablets[i]);
                                }
                            }
                            if (object.nextPageToken != null)
                                message.nextPageToken = String(object.nextPageToken);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a ListHotTabletsResponse message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.ListHotTabletsResponse
                         * @static
                         * @param {google.bigtable.admin.v2.ListHotTabletsResponse} message ListHotTabletsResponse
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        ListHotTabletsResponse.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.arrays || options.defaults)
                                object.hotTablets = [];
                            if (options.defaults)
                                object.nextPageToken = "";
                            if (message.hotTablets && message.hotTablets.length) {
                                object.hotTablets = [];
                                for (var j = 0; j < message.hotTablets.length; ++j)
                                    object.hotTablets[j] = $root.google.bigtable.admin.v2.HotTablet.toObject(message.hotTablets[j], options);
                            }
                            if (message.nextPageToken != null && message.hasOwnProperty("nextPageToken"))
                                object.nextPageToken = message.nextPageToken;
                            return object;
                        };
    
                        /**
                         * Converts this ListHotTabletsResponse to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.ListHotTabletsResponse
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        ListHotTabletsResponse.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for ListHotTabletsResponse
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.ListHotTabletsResponse
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        ListHotTabletsResponse.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.ListHotTabletsResponse";
                        };
    
                        return ListHotTabletsResponse;
                    })();
    
                    v2.Instance = (function() {
    
                        /**
                         * Properties of an Instance.
                         * @memberof google.bigtable.admin.v2
                         * @interface IInstance
                         * @property {string|null} [name] Instance name
                         * @property {string|null} [displayName] Instance displayName
                         * @property {google.bigtable.admin.v2.Instance.State|null} [state] Instance state
                         * @property {google.bigtable.admin.v2.Instance.Type|null} [type] Instance type
                         * @property {Object.<string,string>|null} [labels] Instance labels
                         * @property {google.protobuf.ITimestamp|null} [createTime] Instance createTime
                         * @property {boolean|null} [satisfiesPzs] Instance satisfiesPzs
                         */
    
                        /**
                         * Constructs a new Instance.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents an Instance.
                         * @implements IInstance
                         * @constructor
                         * @param {google.bigtable.admin.v2.IInstance=} [properties] Properties to set
                         */
                        function Instance(properties) {
                            this.labels = {};
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * Instance name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.Instance
                         * @instance
                         */
                        Instance.prototype.name = "";
    
                        /**
                         * Instance displayName.
                         * @member {string} displayName
                         * @memberof google.bigtable.admin.v2.Instance
                         * @instance
                         */
                        Instance.prototype.displayName = "";
    
                        /**
                         * Instance state.
                         * @member {google.bigtable.admin.v2.Instance.State} state
                         * @memberof google.bigtable.admin.v2.Instance
                         * @instance
                         */
                        Instance.prototype.state = 0;
    
                        /**
                         * Instance type.
                         * @member {google.bigtable.admin.v2.Instance.Type} type
                         * @memberof google.bigtable.admin.v2.Instance
                         * @instance
                         */
                        Instance.prototype.type = 0;
    
                        /**
                         * Instance labels.
                         * @member {Object.<string,string>} labels
                         * @memberof google.bigtable.admin.v2.Instance
                         * @instance
                         */
                        Instance.prototype.labels = $util.emptyObject;
    
                        /**
                         * Instance createTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} createTime
                         * @memberof google.bigtable.admin.v2.Instance
                         * @instance
                         */
                        Instance.prototype.createTime = null;
    
                        /**
                         * Instance satisfiesPzs.
                         * @member {boolean|null|undefined} satisfiesPzs
                         * @memberof google.bigtable.admin.v2.Instance
                         * @instance
                         */
                        Instance.prototype.satisfiesPzs = null;
    
                        // OneOf field names bound to virtual getters and setters
                        var $oneOfFields;
    
                        /**
                         * Instance _satisfiesPzs.
                         * @member {"satisfiesPzs"|undefined} _satisfiesPzs
                         * @memberof google.bigtable.admin.v2.Instance
                         * @instance
                         */
                        Object.defineProperty(Instance.prototype, "_satisfiesPzs", {
                            get: $util.oneOfGetter($oneOfFields = ["satisfiesPzs"]),
                            set: $util.oneOfSetter($oneOfFields)
                        });
    
                        /**
                         * Creates a new Instance instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.Instance
                         * @static
                         * @param {google.bigtable.admin.v2.IInstance=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.Instance} Instance instance
                         */
                        Instance.create = function create(properties) {
                            return new Instance(properties);
                        };
    
                        /**
                         * Encodes the specified Instance message. Does not implicitly {@link google.bigtable.admin.v2.Instance.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.Instance
                         * @static
                         * @param {google.bigtable.admin.v2.IInstance} message Instance message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        Instance.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            if (message.displayName != null && Object.hasOwnProperty.call(message, "displayName"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.displayName);
                            if (message.state != null && Object.hasOwnProperty.call(message, "state"))
                                writer.uint32(/* id 3, wireType 0 =*/24).int32(message.state);
                            if (message.type != null && Object.hasOwnProperty.call(message, "type"))
                                writer.uint32(/* id 4, wireType 0 =*/32).int32(message.type);
                            if (message.labels != null && Object.hasOwnProperty.call(message, "labels"))
                                for (var keys = Object.keys(message.labels), i = 0; i < keys.length; ++i)
                                    writer.uint32(/* id 5, wireType 2 =*/42).fork().uint32(/* id 1, wireType 2 =*/10).string(keys[i]).uint32(/* id 2, wireType 2 =*/18).string(message.labels[keys[i]]).ldelim();
                            if (message.createTime != null && Object.hasOwnProperty.call(message, "createTime"))
                                $root.google.protobuf.Timestamp.encode(message.createTime, writer.uint32(/* id 7, wireType 2 =*/58).fork()).ldelim();
                            if (message.satisfiesPzs != null && Object.hasOwnProperty.call(message, "satisfiesPzs"))
                                writer.uint32(/* id 8, wireType 0 =*/64).bool(message.satisfiesPzs);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified Instance message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.Instance.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.Instance
                         * @static
                         * @param {google.bigtable.admin.v2.IInstance} message Instance message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        Instance.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes an Instance message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.Instance
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.Instance} Instance
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        Instance.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.Instance(), key, value;
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.displayName = reader.string();
                                        break;
                                    }
                                case 3: {
                                        message.state = reader.int32();
                                        break;
                                    }
                                case 4: {
                                        message.type = reader.int32();
                                        break;
                                    }
                                case 5: {
                                        if (message.labels === $util.emptyObject)
                                            message.labels = {};
                                        var end2 = reader.uint32() + reader.pos;
                                        key = "";
                                        value = "";
                                        while (reader.pos < end2) {
                                            var tag2 = reader.uint32();
                                            switch (tag2 >>> 3) {
                                            case 1:
                                                key = reader.string();
                                                break;
                                            case 2:
                                                value = reader.string();
                                                break;
                                            default:
                                                reader.skipType(tag2 & 7);
                                                break;
                                            }
                                        }
                                        message.labels[key] = value;
                                        break;
                                    }
                                case 7: {
                                        message.createTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 8: {
                                        message.satisfiesPzs = reader.bool();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes an Instance message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.Instance
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.Instance} Instance
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        Instance.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies an Instance message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.Instance
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        Instance.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            var properties = {};
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            if (message.displayName != null && message.hasOwnProperty("displayName"))
                                if (!$util.isString(message.displayName))
                                    return "displayName: string expected";
                            if (message.state != null && message.hasOwnProperty("state"))
                                switch (message.state) {
                                default:
                                    return "state: enum value expected";
                                case 0:
                                case 1:
                                case 2:
                                    break;
                                }
                            if (message.type != null && message.hasOwnProperty("type"))
                                switch (message.type) {
                                default:
                                    return "type: enum value expected";
                                case 0:
                                case 1:
                                case 2:
                                    break;
                                }
                            if (message.labels != null && message.hasOwnProperty("labels")) {
                                if (!$util.isObject(message.labels))
                                    return "labels: object expected";
                                var key = Object.keys(message.labels);
                                for (var i = 0; i < key.length; ++i)
                                    if (!$util.isString(message.labels[key[i]]))
                                        return "labels: string{k:string} expected";
                            }
                            if (message.createTime != null && message.hasOwnProperty("createTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.createTime);
                                if (error)
                                    return "createTime." + error;
                            }
                            if (message.satisfiesPzs != null && message.hasOwnProperty("satisfiesPzs")) {
                                properties._satisfiesPzs = 1;
                                if (typeof message.satisfiesPzs !== "boolean")
                                    return "satisfiesPzs: boolean expected";
                            }
                            return null;
                        };
    
                        /**
                         * Creates an Instance message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.Instance
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.Instance} Instance
                         */
                        Instance.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.Instance)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.Instance();
                            if (object.name != null)
                                message.name = String(object.name);
                            if (object.displayName != null)
                                message.displayName = String(object.displayName);
                            switch (object.state) {
                            default:
                                if (typeof object.state === "number") {
                                    message.state = object.state;
                                    break;
                                }
                                break;
                            case "STATE_NOT_KNOWN":
                            case 0:
                                message.state = 0;
                                break;
                            case "READY":
                            case 1:
                                message.state = 1;
                                break;
                            case "CREATING":
                            case 2:
                                message.state = 2;
                                break;
                            }
                            switch (object.type) {
                            default:
                                if (typeof object.type === "number") {
                                    message.type = object.type;
                                    break;
                                }
                                break;
                            case "TYPE_UNSPECIFIED":
                            case 0:
                                message.type = 0;
                                break;
                            case "PRODUCTION":
                            case 1:
                                message.type = 1;
                                break;
                            case "DEVELOPMENT":
                            case 2:
                                message.type = 2;
                                break;
                            }
                            if (object.labels) {
                                if (typeof object.labels !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.Instance.labels: object expected");
                                message.labels = {};
                                for (var keys = Object.keys(object.labels), i = 0; i < keys.length; ++i)
                                    message.labels[keys[i]] = String(object.labels[keys[i]]);
                            }
                            if (object.createTime != null) {
                                if (typeof object.createTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.Instance.createTime: object expected");
                                message.createTime = $root.google.protobuf.Timestamp.fromObject(object.createTime);
                            }
                            if (object.satisfiesPzs != null)
                                message.satisfiesPzs = Boolean(object.satisfiesPzs);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from an Instance message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.Instance
                         * @static
                         * @param {google.bigtable.admin.v2.Instance} message Instance
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        Instance.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.objects || options.defaults)
                                object.labels = {};
                            if (options.defaults) {
                                object.name = "";
                                object.displayName = "";
                                object.state = options.enums === String ? "STATE_NOT_KNOWN" : 0;
                                object.type = options.enums === String ? "TYPE_UNSPECIFIED" : 0;
                                object.createTime = null;
                            }
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            if (message.displayName != null && message.hasOwnProperty("displayName"))
                                object.displayName = message.displayName;
                            if (message.state != null && message.hasOwnProperty("state"))
                                object.state = options.enums === String ? $root.google.bigtable.admin.v2.Instance.State[message.state] === undefined ? message.state : $root.google.bigtable.admin.v2.Instance.State[message.state] : message.state;
                            if (message.type != null && message.hasOwnProperty("type"))
                                object.type = options.enums === String ? $root.google.bigtable.admin.v2.Instance.Type[message.type] === undefined ? message.type : $root.google.bigtable.admin.v2.Instance.Type[message.type] : message.type;
                            var keys2;
                            if (message.labels && (keys2 = Object.keys(message.labels)).length) {
                                object.labels = {};
                                for (var j = 0; j < keys2.length; ++j)
                                    object.labels[keys2[j]] = message.labels[keys2[j]];
                            }
                            if (message.createTime != null && message.hasOwnProperty("createTime"))
                                object.createTime = $root.google.protobuf.Timestamp.toObject(message.createTime, options);
                            if (message.satisfiesPzs != null && message.hasOwnProperty("satisfiesPzs")) {
                                object.satisfiesPzs = message.satisfiesPzs;
                                if (options.oneofs)
                                    object._satisfiesPzs = "satisfiesPzs";
                            }
                            return object;
                        };
    
                        /**
                         * Converts this Instance to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.Instance
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        Instance.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for Instance
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.Instance
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        Instance.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.Instance";
                        };
    
                        /**
                         * State enum.
                         * @name google.bigtable.admin.v2.Instance.State
                         * @enum {number}
                         * @property {number} STATE_NOT_KNOWN=0 STATE_NOT_KNOWN value
                         * @property {number} READY=1 READY value
                         * @property {number} CREATING=2 CREATING value
                         */
                        Instance.State = (function() {
                            var valuesById = {}, values = Object.create(valuesById);
                            values[valuesById[0] = "STATE_NOT_KNOWN"] = 0;
                            values[valuesById[1] = "READY"] = 1;
                            values[valuesById[2] = "CREATING"] = 2;
                            return values;
                        })();
    
                        /**
                         * Type enum.
                         * @name google.bigtable.admin.v2.Instance.Type
                         * @enum {number}
                         * @property {number} TYPE_UNSPECIFIED=0 TYPE_UNSPECIFIED value
                         * @property {number} PRODUCTION=1 PRODUCTION value
                         * @property {number} DEVELOPMENT=2 DEVELOPMENT value
                         */
                        Instance.Type = (function() {
                            var valuesById = {}, values = Object.create(valuesById);
                            values[valuesById[0] = "TYPE_UNSPECIFIED"] = 0;
                            values[valuesById[1] = "PRODUCTION"] = 1;
                            values[valuesById[2] = "DEVELOPMENT"] = 2;
                            return values;
                        })();
    
                        return Instance;
                    })();
    
                    v2.AutoscalingTargets = (function() {
    
                        /**
                         * Properties of an AutoscalingTargets.
                         * @memberof google.bigtable.admin.v2
                         * @interface IAutoscalingTargets
                         * @property {number|null} [cpuUtilizationPercent] AutoscalingTargets cpuUtilizationPercent
                         * @property {number|null} [storageUtilizationGibPerNode] AutoscalingTargets storageUtilizationGibPerNode
                         */
    
                        /**
                         * Constructs a new AutoscalingTargets.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents an AutoscalingTargets.
                         * @implements IAutoscalingTargets
                         * @constructor
                         * @param {google.bigtable.admin.v2.IAutoscalingTargets=} [properties] Properties to set
                         */
                        function AutoscalingTargets(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * AutoscalingTargets cpuUtilizationPercent.
                         * @member {number} cpuUtilizationPercent
                         * @memberof google.bigtable.admin.v2.AutoscalingTargets
                         * @instance
                         */
                        AutoscalingTargets.prototype.cpuUtilizationPercent = 0;
    
                        /**
                         * AutoscalingTargets storageUtilizationGibPerNode.
                         * @member {number} storageUtilizationGibPerNode
                         * @memberof google.bigtable.admin.v2.AutoscalingTargets
                         * @instance
                         */
                        AutoscalingTargets.prototype.storageUtilizationGibPerNode = 0;
    
                        /**
                         * Creates a new AutoscalingTargets instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.AutoscalingTargets
                         * @static
                         * @param {google.bigtable.admin.v2.IAutoscalingTargets=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.AutoscalingTargets} AutoscalingTargets instance
                         */
                        AutoscalingTargets.create = function create(properties) {
                            return new AutoscalingTargets(properties);
                        };
    
                        /**
                         * Encodes the specified AutoscalingTargets message. Does not implicitly {@link google.bigtable.admin.v2.AutoscalingTargets.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.AutoscalingTargets
                         * @static
                         * @param {google.bigtable.admin.v2.IAutoscalingTargets} message AutoscalingTargets message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        AutoscalingTargets.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.cpuUtilizationPercent != null && Object.hasOwnProperty.call(message, "cpuUtilizationPercent"))
                                writer.uint32(/* id 2, wireType 0 =*/16).int32(message.cpuUtilizationPercent);
                            if (message.storageUtilizationGibPerNode != null && Object.hasOwnProperty.call(message, "storageUtilizationGibPerNode"))
                                writer.uint32(/* id 3, wireType 0 =*/24).int32(message.storageUtilizationGibPerNode);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified AutoscalingTargets message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.AutoscalingTargets.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.AutoscalingTargets
                         * @static
                         * @param {google.bigtable.admin.v2.IAutoscalingTargets} message AutoscalingTargets message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        AutoscalingTargets.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes an AutoscalingTargets message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.AutoscalingTargets
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.AutoscalingTargets} AutoscalingTargets
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        AutoscalingTargets.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.AutoscalingTargets();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 2: {
                                        message.cpuUtilizationPercent = reader.int32();
                                        break;
                                    }
                                case 3: {
                                        message.storageUtilizationGibPerNode = reader.int32();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes an AutoscalingTargets message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.AutoscalingTargets
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.AutoscalingTargets} AutoscalingTargets
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        AutoscalingTargets.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies an AutoscalingTargets message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.AutoscalingTargets
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        AutoscalingTargets.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.cpuUtilizationPercent != null && message.hasOwnProperty("cpuUtilizationPercent"))
                                if (!$util.isInteger(message.cpuUtilizationPercent))
                                    return "cpuUtilizationPercent: integer expected";
                            if (message.storageUtilizationGibPerNode != null && message.hasOwnProperty("storageUtilizationGibPerNode"))
                                if (!$util.isInteger(message.storageUtilizationGibPerNode))
                                    return "storageUtilizationGibPerNode: integer expected";
                            return null;
                        };
    
                        /**
                         * Creates an AutoscalingTargets message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.AutoscalingTargets
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.AutoscalingTargets} AutoscalingTargets
                         */
                        AutoscalingTargets.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.AutoscalingTargets)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.AutoscalingTargets();
                            if (object.cpuUtilizationPercent != null)
                                message.cpuUtilizationPercent = object.cpuUtilizationPercent | 0;
                            if (object.storageUtilizationGibPerNode != null)
                                message.storageUtilizationGibPerNode = object.storageUtilizationGibPerNode | 0;
                            return message;
                        };
    
                        /**
                         * Creates a plain object from an AutoscalingTargets message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.AutoscalingTargets
                         * @static
                         * @param {google.bigtable.admin.v2.AutoscalingTargets} message AutoscalingTargets
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        AutoscalingTargets.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.cpuUtilizationPercent = 0;
                                object.storageUtilizationGibPerNode = 0;
                            }
                            if (message.cpuUtilizationPercent != null && message.hasOwnProperty("cpuUtilizationPercent"))
                                object.cpuUtilizationPercent = message.cpuUtilizationPercent;
                            if (message.storageUtilizationGibPerNode != null && message.hasOwnProperty("storageUtilizationGibPerNode"))
                                object.storageUtilizationGibPerNode = message.storageUtilizationGibPerNode;
                            return object;
                        };
    
                        /**
                         * Converts this AutoscalingTargets to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.AutoscalingTargets
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        AutoscalingTargets.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for AutoscalingTargets
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.AutoscalingTargets
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        AutoscalingTargets.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.AutoscalingTargets";
                        };
    
                        return AutoscalingTargets;
                    })();
    
                    v2.AutoscalingLimits = (function() {
    
                        /**
                         * Properties of an AutoscalingLimits.
                         * @memberof google.bigtable.admin.v2
                         * @interface IAutoscalingLimits
                         * @property {number|null} [minServeNodes] AutoscalingLimits minServeNodes
                         * @property {number|null} [maxServeNodes] AutoscalingLimits maxServeNodes
                         */
    
                        /**
                         * Constructs a new AutoscalingLimits.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents an AutoscalingLimits.
                         * @implements IAutoscalingLimits
                         * @constructor
                         * @param {google.bigtable.admin.v2.IAutoscalingLimits=} [properties] Properties to set
                         */
                        function AutoscalingLimits(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * AutoscalingLimits minServeNodes.
                         * @member {number} minServeNodes
                         * @memberof google.bigtable.admin.v2.AutoscalingLimits
                         * @instance
                         */
                        AutoscalingLimits.prototype.minServeNodes = 0;
    
                        /**
                         * AutoscalingLimits maxServeNodes.
                         * @member {number} maxServeNodes
                         * @memberof google.bigtable.admin.v2.AutoscalingLimits
                         * @instance
                         */
                        AutoscalingLimits.prototype.maxServeNodes = 0;
    
                        /**
                         * Creates a new AutoscalingLimits instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.AutoscalingLimits
                         * @static
                         * @param {google.bigtable.admin.v2.IAutoscalingLimits=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.AutoscalingLimits} AutoscalingLimits instance
                         */
                        AutoscalingLimits.create = function create(properties) {
                            return new AutoscalingLimits(properties);
                        };
    
                        /**
                         * Encodes the specified AutoscalingLimits message. Does not implicitly {@link google.bigtable.admin.v2.AutoscalingLimits.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.AutoscalingLimits
                         * @static
                         * @param {google.bigtable.admin.v2.IAutoscalingLimits} message AutoscalingLimits message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        AutoscalingLimits.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.minServeNodes != null && Object.hasOwnProperty.call(message, "minServeNodes"))
                                writer.uint32(/* id 1, wireType 0 =*/8).int32(message.minServeNodes);
                            if (message.maxServeNodes != null && Object.hasOwnProperty.call(message, "maxServeNodes"))
                                writer.uint32(/* id 2, wireType 0 =*/16).int32(message.maxServeNodes);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified AutoscalingLimits message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.AutoscalingLimits.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.AutoscalingLimits
                         * @static
                         * @param {google.bigtable.admin.v2.IAutoscalingLimits} message AutoscalingLimits message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        AutoscalingLimits.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes an AutoscalingLimits message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.AutoscalingLimits
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.AutoscalingLimits} AutoscalingLimits
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        AutoscalingLimits.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.AutoscalingLimits();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.minServeNodes = reader.int32();
                                        break;
                                    }
                                case 2: {
                                        message.maxServeNodes = reader.int32();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes an AutoscalingLimits message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.AutoscalingLimits
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.AutoscalingLimits} AutoscalingLimits
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        AutoscalingLimits.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies an AutoscalingLimits message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.AutoscalingLimits
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        AutoscalingLimits.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.minServeNodes != null && message.hasOwnProperty("minServeNodes"))
                                if (!$util.isInteger(message.minServeNodes))
                                    return "minServeNodes: integer expected";
                            if (message.maxServeNodes != null && message.hasOwnProperty("maxServeNodes"))
                                if (!$util.isInteger(message.maxServeNodes))
                                    return "maxServeNodes: integer expected";
                            return null;
                        };
    
                        /**
                         * Creates an AutoscalingLimits message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.AutoscalingLimits
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.AutoscalingLimits} AutoscalingLimits
                         */
                        AutoscalingLimits.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.AutoscalingLimits)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.AutoscalingLimits();
                            if (object.minServeNodes != null)
                                message.minServeNodes = object.minServeNodes | 0;
                            if (object.maxServeNodes != null)
                                message.maxServeNodes = object.maxServeNodes | 0;
                            return message;
                        };
    
                        /**
                         * Creates a plain object from an AutoscalingLimits message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.AutoscalingLimits
                         * @static
                         * @param {google.bigtable.admin.v2.AutoscalingLimits} message AutoscalingLimits
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        AutoscalingLimits.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.minServeNodes = 0;
                                object.maxServeNodes = 0;
                            }
                            if (message.minServeNodes != null && message.hasOwnProperty("minServeNodes"))
                                object.minServeNodes = message.minServeNodes;
                            if (message.maxServeNodes != null && message.hasOwnProperty("maxServeNodes"))
                                object.maxServeNodes = message.maxServeNodes;
                            return object;
                        };
    
                        /**
                         * Converts this AutoscalingLimits to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.AutoscalingLimits
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        AutoscalingLimits.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for AutoscalingLimits
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.AutoscalingLimits
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        AutoscalingLimits.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.AutoscalingLimits";
                        };
    
                        return AutoscalingLimits;
                    })();
    
                    v2.Cluster = (function() {
    
                        /**
                         * Properties of a Cluster.
                         * @memberof google.bigtable.admin.v2
                         * @interface ICluster
                         * @property {string|null} [name] Cluster name
                         * @property {string|null} [location] Cluster location
                         * @property {google.bigtable.admin.v2.Cluster.State|null} [state] Cluster state
                         * @property {number|null} [serveNodes] Cluster serveNodes
                         * @property {google.bigtable.admin.v2.Cluster.IClusterConfig|null} [clusterConfig] Cluster clusterConfig
                         * @property {google.bigtable.admin.v2.StorageType|null} [defaultStorageType] Cluster defaultStorageType
                         * @property {google.bigtable.admin.v2.Cluster.IEncryptionConfig|null} [encryptionConfig] Cluster encryptionConfig
                         */
    
                        /**
                         * Constructs a new Cluster.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a Cluster.
                         * @implements ICluster
                         * @constructor
                         * @param {google.bigtable.admin.v2.ICluster=} [properties] Properties to set
                         */
                        function Cluster(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * Cluster name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.Cluster
                         * @instance
                         */
                        Cluster.prototype.name = "";
    
                        /**
                         * Cluster location.
                         * @member {string} location
                         * @memberof google.bigtable.admin.v2.Cluster
                         * @instance
                         */
                        Cluster.prototype.location = "";
    
                        /**
                         * Cluster state.
                         * @member {google.bigtable.admin.v2.Cluster.State} state
                         * @memberof google.bigtable.admin.v2.Cluster
                         * @instance
                         */
                        Cluster.prototype.state = 0;
    
                        /**
                         * Cluster serveNodes.
                         * @member {number} serveNodes
                         * @memberof google.bigtable.admin.v2.Cluster
                         * @instance
                         */
                        Cluster.prototype.serveNodes = 0;
    
                        /**
                         * Cluster clusterConfig.
                         * @member {google.bigtable.admin.v2.Cluster.IClusterConfig|null|undefined} clusterConfig
                         * @memberof google.bigtable.admin.v2.Cluster
                         * @instance
                         */
                        Cluster.prototype.clusterConfig = null;
    
                        /**
                         * Cluster defaultStorageType.
                         * @member {google.bigtable.admin.v2.StorageType} defaultStorageType
                         * @memberof google.bigtable.admin.v2.Cluster
                         * @instance
                         */
                        Cluster.prototype.defaultStorageType = 0;
    
                        /**
                         * Cluster encryptionConfig.
                         * @member {google.bigtable.admin.v2.Cluster.IEncryptionConfig|null|undefined} encryptionConfig
                         * @memberof google.bigtable.admin.v2.Cluster
                         * @instance
                         */
                        Cluster.prototype.encryptionConfig = null;
    
                        // OneOf field names bound to virtual getters and setters
                        var $oneOfFields;
    
                        /**
                         * Cluster config.
                         * @member {"clusterConfig"|undefined} config
                         * @memberof google.bigtable.admin.v2.Cluster
                         * @instance
                         */
                        Object.defineProperty(Cluster.prototype, "config", {
                            get: $util.oneOfGetter($oneOfFields = ["clusterConfig"]),
                            set: $util.oneOfSetter($oneOfFields)
                        });
    
                        /**
                         * Creates a new Cluster instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.Cluster
                         * @static
                         * @param {google.bigtable.admin.v2.ICluster=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.Cluster} Cluster instance
                         */
                        Cluster.create = function create(properties) {
                            return new Cluster(properties);
                        };
    
                        /**
                         * Encodes the specified Cluster message. Does not implicitly {@link google.bigtable.admin.v2.Cluster.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.Cluster
                         * @static
                         * @param {google.bigtable.admin.v2.ICluster} message Cluster message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        Cluster.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            if (message.location != null && Object.hasOwnProperty.call(message, "location"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.location);
                            if (message.state != null && Object.hasOwnProperty.call(message, "state"))
                                writer.uint32(/* id 3, wireType 0 =*/24).int32(message.state);
                            if (message.serveNodes != null && Object.hasOwnProperty.call(message, "serveNodes"))
                                writer.uint32(/* id 4, wireType 0 =*/32).int32(message.serveNodes);
                            if (message.defaultStorageType != null && Object.hasOwnProperty.call(message, "defaultStorageType"))
                                writer.uint32(/* id 5, wireType 0 =*/40).int32(message.defaultStorageType);
                            if (message.encryptionConfig != null && Object.hasOwnProperty.call(message, "encryptionConfig"))
                                $root.google.bigtable.admin.v2.Cluster.EncryptionConfig.encode(message.encryptionConfig, writer.uint32(/* id 6, wireType 2 =*/50).fork()).ldelim();
                            if (message.clusterConfig != null && Object.hasOwnProperty.call(message, "clusterConfig"))
                                $root.google.bigtable.admin.v2.Cluster.ClusterConfig.encode(message.clusterConfig, writer.uint32(/* id 7, wireType 2 =*/58).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified Cluster message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.Cluster.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.Cluster
                         * @static
                         * @param {google.bigtable.admin.v2.ICluster} message Cluster message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        Cluster.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a Cluster message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.Cluster
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.Cluster} Cluster
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        Cluster.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.Cluster();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.location = reader.string();
                                        break;
                                    }
                                case 3: {
                                        message.state = reader.int32();
                                        break;
                                    }
                                case 4: {
                                        message.serveNodes = reader.int32();
                                        break;
                                    }
                                case 7: {
                                        message.clusterConfig = $root.google.bigtable.admin.v2.Cluster.ClusterConfig.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 5: {
                                        message.defaultStorageType = reader.int32();
                                        break;
                                    }
                                case 6: {
                                        message.encryptionConfig = $root.google.bigtable.admin.v2.Cluster.EncryptionConfig.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a Cluster message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.Cluster
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.Cluster} Cluster
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        Cluster.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a Cluster message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.Cluster
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        Cluster.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            var properties = {};
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            if (message.location != null && message.hasOwnProperty("location"))
                                if (!$util.isString(message.location))
                                    return "location: string expected";
                            if (message.state != null && message.hasOwnProperty("state"))
                                switch (message.state) {
                                default:
                                    return "state: enum value expected";
                                case 0:
                                case 1:
                                case 2:
                                case 3:
                                case 4:
                                    break;
                                }
                            if (message.serveNodes != null && message.hasOwnProperty("serveNodes"))
                                if (!$util.isInteger(message.serveNodes))
                                    return "serveNodes: integer expected";
                            if (message.clusterConfig != null && message.hasOwnProperty("clusterConfig")) {
                                properties.config = 1;
                                {
                                    var error = $root.google.bigtable.admin.v2.Cluster.ClusterConfig.verify(message.clusterConfig);
                                    if (error)
                                        return "clusterConfig." + error;
                                }
                            }
                            if (message.defaultStorageType != null && message.hasOwnProperty("defaultStorageType"))
                                switch (message.defaultStorageType) {
                                default:
                                    return "defaultStorageType: enum value expected";
                                case 0:
                                case 1:
                                case 2:
                                    break;
                                }
                            if (message.encryptionConfig != null && message.hasOwnProperty("encryptionConfig")) {
                                var error = $root.google.bigtable.admin.v2.Cluster.EncryptionConfig.verify(message.encryptionConfig);
                                if (error)
                                    return "encryptionConfig." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates a Cluster message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.Cluster
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.Cluster} Cluster
                         */
                        Cluster.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.Cluster)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.Cluster();
                            if (object.name != null)
                                message.name = String(object.name);
                            if (object.location != null)
                                message.location = String(object.location);
                            switch (object.state) {
                            default:
                                if (typeof object.state === "number") {
                                    message.state = object.state;
                                    break;
                                }
                                break;
                            case "STATE_NOT_KNOWN":
                            case 0:
                                message.state = 0;
                                break;
                            case "READY":
                            case 1:
                                message.state = 1;
                                break;
                            case "CREATING":
                            case 2:
                                message.state = 2;
                                break;
                            case "RESIZING":
                            case 3:
                                message.state = 3;
                                break;
                            case "DISABLED":
                            case 4:
                                message.state = 4;
                                break;
                            }
                            if (object.serveNodes != null)
                                message.serveNodes = object.serveNodes | 0;
                            if (object.clusterConfig != null) {
                                if (typeof object.clusterConfig !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.Cluster.clusterConfig: object expected");
                                message.clusterConfig = $root.google.bigtable.admin.v2.Cluster.ClusterConfig.fromObject(object.clusterConfig);
                            }
                            switch (object.defaultStorageType) {
                            default:
                                if (typeof object.defaultStorageType === "number") {
                                    message.defaultStorageType = object.defaultStorageType;
                                    break;
                                }
                                break;
                            case "STORAGE_TYPE_UNSPECIFIED":
                            case 0:
                                message.defaultStorageType = 0;
                                break;
                            case "SSD":
                            case 1:
                                message.defaultStorageType = 1;
                                break;
                            case "HDD":
                            case 2:
                                message.defaultStorageType = 2;
                                break;
                            }
                            if (object.encryptionConfig != null) {
                                if (typeof object.encryptionConfig !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.Cluster.encryptionConfig: object expected");
                                message.encryptionConfig = $root.google.bigtable.admin.v2.Cluster.EncryptionConfig.fromObject(object.encryptionConfig);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a Cluster message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.Cluster
                         * @static
                         * @param {google.bigtable.admin.v2.Cluster} message Cluster
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        Cluster.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.name = "";
                                object.location = "";
                                object.state = options.enums === String ? "STATE_NOT_KNOWN" : 0;
                                object.serveNodes = 0;
                                object.defaultStorageType = options.enums === String ? "STORAGE_TYPE_UNSPECIFIED" : 0;
                                object.encryptionConfig = null;
                            }
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            if (message.location != null && message.hasOwnProperty("location"))
                                object.location = message.location;
                            if (message.state != null && message.hasOwnProperty("state"))
                                object.state = options.enums === String ? $root.google.bigtable.admin.v2.Cluster.State[message.state] === undefined ? message.state : $root.google.bigtable.admin.v2.Cluster.State[message.state] : message.state;
                            if (message.serveNodes != null && message.hasOwnProperty("serveNodes"))
                                object.serveNodes = message.serveNodes;
                            if (message.defaultStorageType != null && message.hasOwnProperty("defaultStorageType"))
                                object.defaultStorageType = options.enums === String ? $root.google.bigtable.admin.v2.StorageType[message.defaultStorageType] === undefined ? message.defaultStorageType : $root.google.bigtable.admin.v2.StorageType[message.defaultStorageType] : message.defaultStorageType;
                            if (message.encryptionConfig != null && message.hasOwnProperty("encryptionConfig"))
                                object.encryptionConfig = $root.google.bigtable.admin.v2.Cluster.EncryptionConfig.toObject(message.encryptionConfig, options);
                            if (message.clusterConfig != null && message.hasOwnProperty("clusterConfig")) {
                                object.clusterConfig = $root.google.bigtable.admin.v2.Cluster.ClusterConfig.toObject(message.clusterConfig, options);
                                if (options.oneofs)
                                    object.config = "clusterConfig";
                            }
                            return object;
                        };
    
                        /**
                         * Converts this Cluster to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.Cluster
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        Cluster.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for Cluster
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.Cluster
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        Cluster.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.Cluster";
                        };
    
                        /**
                         * State enum.
                         * @name google.bigtable.admin.v2.Cluster.State
                         * @enum {number}
                         * @property {number} STATE_NOT_KNOWN=0 STATE_NOT_KNOWN value
                         * @property {number} READY=1 READY value
                         * @property {number} CREATING=2 CREATING value
                         * @property {number} RESIZING=3 RESIZING value
                         * @property {number} DISABLED=4 DISABLED value
                         */
                        Cluster.State = (function() {
                            var valuesById = {}, values = Object.create(valuesById);
                            values[valuesById[0] = "STATE_NOT_KNOWN"] = 0;
                            values[valuesById[1] = "READY"] = 1;
                            values[valuesById[2] = "CREATING"] = 2;
                            values[valuesById[3] = "RESIZING"] = 3;
                            values[valuesById[4] = "DISABLED"] = 4;
                            return values;
                        })();
    
                        Cluster.ClusterAutoscalingConfig = (function() {
    
                            /**
                             * Properties of a ClusterAutoscalingConfig.
                             * @memberof google.bigtable.admin.v2.Cluster
                             * @interface IClusterAutoscalingConfig
                             * @property {google.bigtable.admin.v2.IAutoscalingLimits|null} [autoscalingLimits] ClusterAutoscalingConfig autoscalingLimits
                             * @property {google.bigtable.admin.v2.IAutoscalingTargets|null} [autoscalingTargets] ClusterAutoscalingConfig autoscalingTargets
                             */
    
                            /**
                             * Constructs a new ClusterAutoscalingConfig.
                             * @memberof google.bigtable.admin.v2.Cluster
                             * @classdesc Represents a ClusterAutoscalingConfig.
                             * @implements IClusterAutoscalingConfig
                             * @constructor
                             * @param {google.bigtable.admin.v2.Cluster.IClusterAutoscalingConfig=} [properties] Properties to set
                             */
                            function ClusterAutoscalingConfig(properties) {
                                if (properties)
                                    for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                        if (properties[keys[i]] != null)
                                            this[keys[i]] = properties[keys[i]];
                            }
    
                            /**
                             * ClusterAutoscalingConfig autoscalingLimits.
                             * @member {google.bigtable.admin.v2.IAutoscalingLimits|null|undefined} autoscalingLimits
                             * @memberof google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig
                             * @instance
                             */
                            ClusterAutoscalingConfig.prototype.autoscalingLimits = null;
    
                            /**
                             * ClusterAutoscalingConfig autoscalingTargets.
                             * @member {google.bigtable.admin.v2.IAutoscalingTargets|null|undefined} autoscalingTargets
                             * @memberof google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig
                             * @instance
                             */
                            ClusterAutoscalingConfig.prototype.autoscalingTargets = null;
    
                            /**
                             * Creates a new ClusterAutoscalingConfig instance using the specified properties.
                             * @function create
                             * @memberof google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig
                             * @static
                             * @param {google.bigtable.admin.v2.Cluster.IClusterAutoscalingConfig=} [properties] Properties to set
                             * @returns {google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig} ClusterAutoscalingConfig instance
                             */
                            ClusterAutoscalingConfig.create = function create(properties) {
                                return new ClusterAutoscalingConfig(properties);
                            };
    
                            /**
                             * Encodes the specified ClusterAutoscalingConfig message. Does not implicitly {@link google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig.verify|verify} messages.
                             * @function encode
                             * @memberof google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig
                             * @static
                             * @param {google.bigtable.admin.v2.Cluster.IClusterAutoscalingConfig} message ClusterAutoscalingConfig message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ClusterAutoscalingConfig.encode = function encode(message, writer) {
                                if (!writer)
                                    writer = $Writer.create();
                                if (message.autoscalingLimits != null && Object.hasOwnProperty.call(message, "autoscalingLimits"))
                                    $root.google.bigtable.admin.v2.AutoscalingLimits.encode(message.autoscalingLimits, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                                if (message.autoscalingTargets != null && Object.hasOwnProperty.call(message, "autoscalingTargets"))
                                    $root.google.bigtable.admin.v2.AutoscalingTargets.encode(message.autoscalingTargets, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                                return writer;
                            };
    
                            /**
                             * Encodes the specified ClusterAutoscalingConfig message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig
                             * @static
                             * @param {google.bigtable.admin.v2.Cluster.IClusterAutoscalingConfig} message ClusterAutoscalingConfig message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ClusterAutoscalingConfig.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ClusterAutoscalingConfig message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @param {number} [length] Message length if known beforehand
                             * @returns {google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig} ClusterAutoscalingConfig
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ClusterAutoscalingConfig.decode = function decode(reader, length) {
                                if (!(reader instanceof $Reader))
                                    reader = $Reader.create(reader);
                                var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig();
                                while (reader.pos < end) {
                                    var tag = reader.uint32();
                                    switch (tag >>> 3) {
                                    case 1: {
                                            message.autoscalingLimits = $root.google.bigtable.admin.v2.AutoscalingLimits.decode(reader, reader.uint32());
                                            break;
                                        }
                                    case 2: {
                                            message.autoscalingTargets = $root.google.bigtable.admin.v2.AutoscalingTargets.decode(reader, reader.uint32());
                                            break;
                                        }
                                    default:
                                        reader.skipType(tag & 7);
                                        break;
                                    }
                                }
                                return message;
                            };
    
                            /**
                             * Decodes a ClusterAutoscalingConfig message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig} ClusterAutoscalingConfig
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ClusterAutoscalingConfig.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ClusterAutoscalingConfig message.
                             * @function verify
                             * @memberof google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig
                             * @static
                             * @param {Object.<string,*>} message Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ClusterAutoscalingConfig.verify = function verify(message) {
                                if (typeof message !== "object" || message === null)
                                    return "object expected";
                                if (message.autoscalingLimits != null && message.hasOwnProperty("autoscalingLimits")) {
                                    var error = $root.google.bigtable.admin.v2.AutoscalingLimits.verify(message.autoscalingLimits);
                                    if (error)
                                        return "autoscalingLimits." + error;
                                }
                                if (message.autoscalingTargets != null && message.hasOwnProperty("autoscalingTargets")) {
                                    var error = $root.google.bigtable.admin.v2.AutoscalingTargets.verify(message.autoscalingTargets);
                                    if (error)
                                        return "autoscalingTargets." + error;
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ClusterAutoscalingConfig message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig
                             * @static
                             * @param {Object.<string,*>} object Plain object
                             * @returns {google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig} ClusterAutoscalingConfig
                             */
                            ClusterAutoscalingConfig.fromObject = function fromObject(object) {
                                if (object instanceof $root.google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig)
                                    return object;
                                var message = new $root.google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig();
                                if (object.autoscalingLimits != null) {
                                    if (typeof object.autoscalingLimits !== "object")
                                        throw TypeError(".google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig.autoscalingLimits: object expected");
                                    message.autoscalingLimits = $root.google.bigtable.admin.v2.AutoscalingLimits.fromObject(object.autoscalingLimits);
                                }
                                if (object.autoscalingTargets != null) {
                                    if (typeof object.autoscalingTargets !== "object")
                                        throw TypeError(".google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig.autoscalingTargets: object expected");
                                    message.autoscalingTargets = $root.google.bigtable.admin.v2.AutoscalingTargets.fromObject(object.autoscalingTargets);
                                }
                                return message;
                            };
    
                            /**
                             * Creates a plain object from a ClusterAutoscalingConfig message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig
                             * @static
                             * @param {google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig} message ClusterAutoscalingConfig
                             * @param {$protobuf.IConversionOptions} [options] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ClusterAutoscalingConfig.toObject = function toObject(message, options) {
                                if (!options)
                                    options = {};
                                var object = {};
                                if (options.defaults) {
                                    object.autoscalingLimits = null;
                                    object.autoscalingTargets = null;
                                }
                                if (message.autoscalingLimits != null && message.hasOwnProperty("autoscalingLimits"))
                                    object.autoscalingLimits = $root.google.bigtable.admin.v2.AutoscalingLimits.toObject(message.autoscalingLimits, options);
                                if (message.autoscalingTargets != null && message.hasOwnProperty("autoscalingTargets"))
                                    object.autoscalingTargets = $root.google.bigtable.admin.v2.AutoscalingTargets.toObject(message.autoscalingTargets, options);
                                return object;
                            };
    
                            /**
                             * Converts this ClusterAutoscalingConfig to JSON.
                             * @function toJSON
                             * @memberof google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ClusterAutoscalingConfig.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * Gets the default type url for ClusterAutoscalingConfig
                             * @function getTypeUrl
                             * @memberof google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig
                             * @static
                             * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                             * @returns {string} The default type url
                             */
                            ClusterAutoscalingConfig.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                                if (typeUrlPrefix === undefined) {
                                    typeUrlPrefix = "type.googleapis.com";
                                }
                                return typeUrlPrefix + "/google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig";
                            };
    
                            return ClusterAutoscalingConfig;
                        })();
    
                        Cluster.ClusterConfig = (function() {
    
                            /**
                             * Properties of a ClusterConfig.
                             * @memberof google.bigtable.admin.v2.Cluster
                             * @interface IClusterConfig
                             * @property {google.bigtable.admin.v2.Cluster.IClusterAutoscalingConfig|null} [clusterAutoscalingConfig] ClusterConfig clusterAutoscalingConfig
                             */
    
                            /**
                             * Constructs a new ClusterConfig.
                             * @memberof google.bigtable.admin.v2.Cluster
                             * @classdesc Represents a ClusterConfig.
                             * @implements IClusterConfig
                             * @constructor
                             * @param {google.bigtable.admin.v2.Cluster.IClusterConfig=} [properties] Properties to set
                             */
                            function ClusterConfig(properties) {
                                if (properties)
                                    for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                        if (properties[keys[i]] != null)
                                            this[keys[i]] = properties[keys[i]];
                            }
    
                            /**
                             * ClusterConfig clusterAutoscalingConfig.
                             * @member {google.bigtable.admin.v2.Cluster.IClusterAutoscalingConfig|null|undefined} clusterAutoscalingConfig
                             * @memberof google.bigtable.admin.v2.Cluster.ClusterConfig
                             * @instance
                             */
                            ClusterConfig.prototype.clusterAutoscalingConfig = null;
    
                            /**
                             * Creates a new ClusterConfig instance using the specified properties.
                             * @function create
                             * @memberof google.bigtable.admin.v2.Cluster.ClusterConfig
                             * @static
                             * @param {google.bigtable.admin.v2.Cluster.IClusterConfig=} [properties] Properties to set
                             * @returns {google.bigtable.admin.v2.Cluster.ClusterConfig} ClusterConfig instance
                             */
                            ClusterConfig.create = function create(properties) {
                                return new ClusterConfig(properties);
                            };
    
                            /**
                             * Encodes the specified ClusterConfig message. Does not implicitly {@link google.bigtable.admin.v2.Cluster.ClusterConfig.verify|verify} messages.
                             * @function encode
                             * @memberof google.bigtable.admin.v2.Cluster.ClusterConfig
                             * @static
                             * @param {google.bigtable.admin.v2.Cluster.IClusterConfig} message ClusterConfig message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ClusterConfig.encode = function encode(message, writer) {
                                if (!writer)
                                    writer = $Writer.create();
                                if (message.clusterAutoscalingConfig != null && Object.hasOwnProperty.call(message, "clusterAutoscalingConfig"))
                                    $root.google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig.encode(message.clusterAutoscalingConfig, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                                return writer;
                            };
    
                            /**
                             * Encodes the specified ClusterConfig message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.Cluster.ClusterConfig.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.bigtable.admin.v2.Cluster.ClusterConfig
                             * @static
                             * @param {google.bigtable.admin.v2.Cluster.IClusterConfig} message ClusterConfig message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ClusterConfig.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ClusterConfig message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.bigtable.admin.v2.Cluster.ClusterConfig
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @param {number} [length] Message length if known beforehand
                             * @returns {google.bigtable.admin.v2.Cluster.ClusterConfig} ClusterConfig
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ClusterConfig.decode = function decode(reader, length) {
                                if (!(reader instanceof $Reader))
                                    reader = $Reader.create(reader);
                                var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.Cluster.ClusterConfig();
                                while (reader.pos < end) {
                                    var tag = reader.uint32();
                                    switch (tag >>> 3) {
                                    case 1: {
                                            message.clusterAutoscalingConfig = $root.google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig.decode(reader, reader.uint32());
                                            break;
                                        }
                                    default:
                                        reader.skipType(tag & 7);
                                        break;
                                    }
                                }
                                return message;
                            };
    
                            /**
                             * Decodes a ClusterConfig message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.bigtable.admin.v2.Cluster.ClusterConfig
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.bigtable.admin.v2.Cluster.ClusterConfig} ClusterConfig
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ClusterConfig.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ClusterConfig message.
                             * @function verify
                             * @memberof google.bigtable.admin.v2.Cluster.ClusterConfig
                             * @static
                             * @param {Object.<string,*>} message Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ClusterConfig.verify = function verify(message) {
                                if (typeof message !== "object" || message === null)
                                    return "object expected";
                                if (message.clusterAutoscalingConfig != null && message.hasOwnProperty("clusterAutoscalingConfig")) {
                                    var error = $root.google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig.verify(message.clusterAutoscalingConfig);
                                    if (error)
                                        return "clusterAutoscalingConfig." + error;
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ClusterConfig message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.bigtable.admin.v2.Cluster.ClusterConfig
                             * @static
                             * @param {Object.<string,*>} object Plain object
                             * @returns {google.bigtable.admin.v2.Cluster.ClusterConfig} ClusterConfig
                             */
                            ClusterConfig.fromObject = function fromObject(object) {
                                if (object instanceof $root.google.bigtable.admin.v2.Cluster.ClusterConfig)
                                    return object;
                                var message = new $root.google.bigtable.admin.v2.Cluster.ClusterConfig();
                                if (object.clusterAutoscalingConfig != null) {
                                    if (typeof object.clusterAutoscalingConfig !== "object")
                                        throw TypeError(".google.bigtable.admin.v2.Cluster.ClusterConfig.clusterAutoscalingConfig: object expected");
                                    message.clusterAutoscalingConfig = $root.google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig.fromObject(object.clusterAutoscalingConfig);
                                }
                                return message;
                            };
    
                            /**
                             * Creates a plain object from a ClusterConfig message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.bigtable.admin.v2.Cluster.ClusterConfig
                             * @static
                             * @param {google.bigtable.admin.v2.Cluster.ClusterConfig} message ClusterConfig
                             * @param {$protobuf.IConversionOptions} [options] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ClusterConfig.toObject = function toObject(message, options) {
                                if (!options)
                                    options = {};
                                var object = {};
                                if (options.defaults)
                                    object.clusterAutoscalingConfig = null;
                                if (message.clusterAutoscalingConfig != null && message.hasOwnProperty("clusterAutoscalingConfig"))
                                    object.clusterAutoscalingConfig = $root.google.bigtable.admin.v2.Cluster.ClusterAutoscalingConfig.toObject(message.clusterAutoscalingConfig, options);
                                return object;
                            };
    
                            /**
                             * Converts this ClusterConfig to JSON.
                             * @function toJSON
                             * @memberof google.bigtable.admin.v2.Cluster.ClusterConfig
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ClusterConfig.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * Gets the default type url for ClusterConfig
                             * @function getTypeUrl
                             * @memberof google.bigtable.admin.v2.Cluster.ClusterConfig
                             * @static
                             * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                             * @returns {string} The default type url
                             */
                            ClusterConfig.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                                if (typeUrlPrefix === undefined) {
                                    typeUrlPrefix = "type.googleapis.com";
                                }
                                return typeUrlPrefix + "/google.bigtable.admin.v2.Cluster.ClusterConfig";
                            };
    
                            return ClusterConfig;
                        })();
    
                        Cluster.EncryptionConfig = (function() {
    
                            /**
                             * Properties of an EncryptionConfig.
                             * @memberof google.bigtable.admin.v2.Cluster
                             * @interface IEncryptionConfig
                             * @property {string|null} [kmsKeyName] EncryptionConfig kmsKeyName
                             */
    
                            /**
                             * Constructs a new EncryptionConfig.
                             * @memberof google.bigtable.admin.v2.Cluster
                             * @classdesc Represents an EncryptionConfig.
                             * @implements IEncryptionConfig
                             * @constructor
                             * @param {google.bigtable.admin.v2.Cluster.IEncryptionConfig=} [properties] Properties to set
                             */
                            function EncryptionConfig(properties) {
                                if (properties)
                                    for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                        if (properties[keys[i]] != null)
                                            this[keys[i]] = properties[keys[i]];
                            }
    
                            /**
                             * EncryptionConfig kmsKeyName.
                             * @member {string} kmsKeyName
                             * @memberof google.bigtable.admin.v2.Cluster.EncryptionConfig
                             * @instance
                             */
                            EncryptionConfig.prototype.kmsKeyName = "";
    
                            /**
                             * Creates a new EncryptionConfig instance using the specified properties.
                             * @function create
                             * @memberof google.bigtable.admin.v2.Cluster.EncryptionConfig
                             * @static
                             * @param {google.bigtable.admin.v2.Cluster.IEncryptionConfig=} [properties] Properties to set
                             * @returns {google.bigtable.admin.v2.Cluster.EncryptionConfig} EncryptionConfig instance
                             */
                            EncryptionConfig.create = function create(properties) {
                                return new EncryptionConfig(properties);
                            };
    
                            /**
                             * Encodes the specified EncryptionConfig message. Does not implicitly {@link google.bigtable.admin.v2.Cluster.EncryptionConfig.verify|verify} messages.
                             * @function encode
                             * @memberof google.bigtable.admin.v2.Cluster.EncryptionConfig
                             * @static
                             * @param {google.bigtable.admin.v2.Cluster.IEncryptionConfig} message EncryptionConfig message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            EncryptionConfig.encode = function encode(message, writer) {
                                if (!writer)
                                    writer = $Writer.create();
                                if (message.kmsKeyName != null && Object.hasOwnProperty.call(message, "kmsKeyName"))
                                    writer.uint32(/* id 1, wireType 2 =*/10).string(message.kmsKeyName);
                                return writer;
                            };
    
                            /**
                             * Encodes the specified EncryptionConfig message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.Cluster.EncryptionConfig.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.bigtable.admin.v2.Cluster.EncryptionConfig
                             * @static
                             * @param {google.bigtable.admin.v2.Cluster.IEncryptionConfig} message EncryptionConfig message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            EncryptionConfig.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an EncryptionConfig message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.bigtable.admin.v2.Cluster.EncryptionConfig
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @param {number} [length] Message length if known beforehand
                             * @returns {google.bigtable.admin.v2.Cluster.EncryptionConfig} EncryptionConfig
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            EncryptionConfig.decode = function decode(reader, length) {
                                if (!(reader instanceof $Reader))
                                    reader = $Reader.create(reader);
                                var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.Cluster.EncryptionConfig();
                                while (reader.pos < end) {
                                    var tag = reader.uint32();
                                    switch (tag >>> 3) {
                                    case 1: {
                                            message.kmsKeyName = reader.string();
                                            break;
                                        }
                                    default:
                                        reader.skipType(tag & 7);
                                        break;
                                    }
                                }
                                return message;
                            };
    
                            /**
                             * Decodes an EncryptionConfig message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.bigtable.admin.v2.Cluster.EncryptionConfig
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.bigtable.admin.v2.Cluster.EncryptionConfig} EncryptionConfig
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            EncryptionConfig.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an EncryptionConfig message.
                             * @function verify
                             * @memberof google.bigtable.admin.v2.Cluster.EncryptionConfig
                             * @static
                             * @param {Object.<string,*>} message Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            EncryptionConfig.verify = function verify(message) {
                                if (typeof message !== "object" || message === null)
                                    return "object expected";
                                if (message.kmsKeyName != null && message.hasOwnProperty("kmsKeyName"))
                                    if (!$util.isString(message.kmsKeyName))
                                        return "kmsKeyName: string expected";
                                return null;
                            };
    
                            /**
                             * Creates an EncryptionConfig message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.bigtable.admin.v2.Cluster.EncryptionConfig
                             * @static
                             * @param {Object.<string,*>} object Plain object
                             * @returns {google.bigtable.admin.v2.Cluster.EncryptionConfig} EncryptionConfig
                             */
                            EncryptionConfig.fromObject = function fromObject(object) {
                                if (object instanceof $root.google.bigtable.admin.v2.Cluster.EncryptionConfig)
                                    return object;
                                var message = new $root.google.bigtable.admin.v2.Cluster.EncryptionConfig();
                                if (object.kmsKeyName != null)
                                    message.kmsKeyName = String(object.kmsKeyName);
                                return message;
                            };
    
                            /**
                             * Creates a plain object from an EncryptionConfig message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.bigtable.admin.v2.Cluster.EncryptionConfig
                             * @static
                             * @param {google.bigtable.admin.v2.Cluster.EncryptionConfig} message EncryptionConfig
                             * @param {$protobuf.IConversionOptions} [options] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            EncryptionConfig.toObject = function toObject(message, options) {
                                if (!options)
                                    options = {};
                                var object = {};
                                if (options.defaults)
                                    object.kmsKeyName = "";
                                if (message.kmsKeyName != null && message.hasOwnProperty("kmsKeyName"))
                                    object.kmsKeyName = message.kmsKeyName;
                                return object;
                            };
    
                            /**
                             * Converts this EncryptionConfig to JSON.
                             * @function toJSON
                             * @memberof google.bigtable.admin.v2.Cluster.EncryptionConfig
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            EncryptionConfig.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * Gets the default type url for EncryptionConfig
                             * @function getTypeUrl
                             * @memberof google.bigtable.admin.v2.Cluster.EncryptionConfig
                             * @static
                             * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                             * @returns {string} The default type url
                             */
                            EncryptionConfig.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                                if (typeUrlPrefix === undefined) {
                                    typeUrlPrefix = "type.googleapis.com";
                                }
                                return typeUrlPrefix + "/google.bigtable.admin.v2.Cluster.EncryptionConfig";
                            };
    
                            return EncryptionConfig;
                        })();
    
                        return Cluster;
                    })();
    
                    v2.AppProfile = (function() {
    
                        /**
                         * Properties of an AppProfile.
                         * @memberof google.bigtable.admin.v2
                         * @interface IAppProfile
                         * @property {string|null} [name] AppProfile name
                         * @property {string|null} [etag] AppProfile etag
                         * @property {string|null} [description] AppProfile description
                         * @property {google.bigtable.admin.v2.AppProfile.IMultiClusterRoutingUseAny|null} [multiClusterRoutingUseAny] AppProfile multiClusterRoutingUseAny
                         * @property {google.bigtable.admin.v2.AppProfile.ISingleClusterRouting|null} [singleClusterRouting] AppProfile singleClusterRouting
                         * @property {google.bigtable.admin.v2.AppProfile.Priority|null} [priority] AppProfile priority
                         * @property {google.bigtable.admin.v2.AppProfile.IStandardIsolation|null} [standardIsolation] AppProfile standardIsolation
                         * @property {google.bigtable.admin.v2.AppProfile.IDataBoostIsolationReadOnly|null} [dataBoostIsolationReadOnly] AppProfile dataBoostIsolationReadOnly
                         */
    
                        /**
                         * Constructs a new AppProfile.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents an AppProfile.
                         * @implements IAppProfile
                         * @constructor
                         * @param {google.bigtable.admin.v2.IAppProfile=} [properties] Properties to set
                         */
                        function AppProfile(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * AppProfile name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.AppProfile
                         * @instance
                         */
                        AppProfile.prototype.name = "";
    
                        /**
                         * AppProfile etag.
                         * @member {string} etag
                         * @memberof google.bigtable.admin.v2.AppProfile
                         * @instance
                         */
                        AppProfile.prototype.etag = "";
    
                        /**
                         * AppProfile description.
                         * @member {string} description
                         * @memberof google.bigtable.admin.v2.AppProfile
                         * @instance
                         */
                        AppProfile.prototype.description = "";
    
                        /**
                         * AppProfile multiClusterRoutingUseAny.
                         * @member {google.bigtable.admin.v2.AppProfile.IMultiClusterRoutingUseAny|null|undefined} multiClusterRoutingUseAny
                         * @memberof google.bigtable.admin.v2.AppProfile
                         * @instance
                         */
                        AppProfile.prototype.multiClusterRoutingUseAny = null;
    
                        /**
                         * AppProfile singleClusterRouting.
                         * @member {google.bigtable.admin.v2.AppProfile.ISingleClusterRouting|null|undefined} singleClusterRouting
                         * @memberof google.bigtable.admin.v2.AppProfile
                         * @instance
                         */
                        AppProfile.prototype.singleClusterRouting = null;
    
                        /**
                         * AppProfile priority.
                         * @member {google.bigtable.admin.v2.AppProfile.Priority|null|undefined} priority
                         * @memberof google.bigtable.admin.v2.AppProfile
                         * @instance
                         */
                        AppProfile.prototype.priority = null;
    
                        /**
                         * AppProfile standardIsolation.
                         * @member {google.bigtable.admin.v2.AppProfile.IStandardIsolation|null|undefined} standardIsolation
                         * @memberof google.bigtable.admin.v2.AppProfile
                         * @instance
                         */
                        AppProfile.prototype.standardIsolation = null;
    
                        /**
                         * AppProfile dataBoostIsolationReadOnly.
                         * @member {google.bigtable.admin.v2.AppProfile.IDataBoostIsolationReadOnly|null|undefined} dataBoostIsolationReadOnly
                         * @memberof google.bigtable.admin.v2.AppProfile
                         * @instance
                         */
                        AppProfile.prototype.dataBoostIsolationReadOnly = null;
    
                        // OneOf field names bound to virtual getters and setters
                        var $oneOfFields;
    
                        /**
                         * AppProfile routingPolicy.
                         * @member {"multiClusterRoutingUseAny"|"singleClusterRouting"|undefined} routingPolicy
                         * @memberof google.bigtable.admin.v2.AppProfile
                         * @instance
                         */
                        Object.defineProperty(AppProfile.prototype, "routingPolicy", {
                            get: $util.oneOfGetter($oneOfFields = ["multiClusterRoutingUseAny", "singleClusterRouting"]),
                            set: $util.oneOfSetter($oneOfFields)
                        });
    
                        /**
                         * AppProfile isolation.
                         * @member {"priority"|"standardIsolation"|"dataBoostIsolationReadOnly"|undefined} isolation
                         * @memberof google.bigtable.admin.v2.AppProfile
                         * @instance
                         */
                        Object.defineProperty(AppProfile.prototype, "isolation", {
                            get: $util.oneOfGetter($oneOfFields = ["priority", "standardIsolation", "dataBoostIsolationReadOnly"]),
                            set: $util.oneOfSetter($oneOfFields)
                        });
    
                        /**
                         * Creates a new AppProfile instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.AppProfile
                         * @static
                         * @param {google.bigtable.admin.v2.IAppProfile=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.AppProfile} AppProfile instance
                         */
                        AppProfile.create = function create(properties) {
                            return new AppProfile(properties);
                        };
    
                        /**
                         * Encodes the specified AppProfile message. Does not implicitly {@link google.bigtable.admin.v2.AppProfile.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.AppProfile
                         * @static
                         * @param {google.bigtable.admin.v2.IAppProfile} message AppProfile message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        AppProfile.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            if (message.etag != null && Object.hasOwnProperty.call(message, "etag"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.etag);
                            if (message.description != null && Object.hasOwnProperty.call(message, "description"))
                                writer.uint32(/* id 3, wireType 2 =*/26).string(message.description);
                            if (message.multiClusterRoutingUseAny != null && Object.hasOwnProperty.call(message, "multiClusterRoutingUseAny"))
                                $root.google.bigtable.admin.v2.AppProfile.MultiClusterRoutingUseAny.encode(message.multiClusterRoutingUseAny, writer.uint32(/* id 5, wireType 2 =*/42).fork()).ldelim();
                            if (message.singleClusterRouting != null && Object.hasOwnProperty.call(message, "singleClusterRouting"))
                                $root.google.bigtable.admin.v2.AppProfile.SingleClusterRouting.encode(message.singleClusterRouting, writer.uint32(/* id 6, wireType 2 =*/50).fork()).ldelim();
                            if (message.priority != null && Object.hasOwnProperty.call(message, "priority"))
                                writer.uint32(/* id 7, wireType 0 =*/56).int32(message.priority);
                            if (message.dataBoostIsolationReadOnly != null && Object.hasOwnProperty.call(message, "dataBoostIsolationReadOnly"))
                                $root.google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly.encode(message.dataBoostIsolationReadOnly, writer.uint32(/* id 10, wireType 2 =*/82).fork()).ldelim();
                            if (message.standardIsolation != null && Object.hasOwnProperty.call(message, "standardIsolation"))
                                $root.google.bigtable.admin.v2.AppProfile.StandardIsolation.encode(message.standardIsolation, writer.uint32(/* id 11, wireType 2 =*/90).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified AppProfile message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.AppProfile.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.AppProfile
                         * @static
                         * @param {google.bigtable.admin.v2.IAppProfile} message AppProfile message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        AppProfile.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes an AppProfile message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.AppProfile
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.AppProfile} AppProfile
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        AppProfile.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.AppProfile();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.etag = reader.string();
                                        break;
                                    }
                                case 3: {
                                        message.description = reader.string();
                                        break;
                                    }
                                case 5: {
                                        message.multiClusterRoutingUseAny = $root.google.bigtable.admin.v2.AppProfile.MultiClusterRoutingUseAny.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 6: {
                                        message.singleClusterRouting = $root.google.bigtable.admin.v2.AppProfile.SingleClusterRouting.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 7: {
                                        message.priority = reader.int32();
                                        break;
                                    }
                                case 11: {
                                        message.standardIsolation = $root.google.bigtable.admin.v2.AppProfile.StandardIsolation.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 10: {
                                        message.dataBoostIsolationReadOnly = $root.google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes an AppProfile message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.AppProfile
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.AppProfile} AppProfile
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        AppProfile.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies an AppProfile message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.AppProfile
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        AppProfile.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            var properties = {};
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            if (message.etag != null && message.hasOwnProperty("etag"))
                                if (!$util.isString(message.etag))
                                    return "etag: string expected";
                            if (message.description != null && message.hasOwnProperty("description"))
                                if (!$util.isString(message.description))
                                    return "description: string expected";
                            if (message.multiClusterRoutingUseAny != null && message.hasOwnProperty("multiClusterRoutingUseAny")) {
                                properties.routingPolicy = 1;
                                {
                                    var error = $root.google.bigtable.admin.v2.AppProfile.MultiClusterRoutingUseAny.verify(message.multiClusterRoutingUseAny);
                                    if (error)
                                        return "multiClusterRoutingUseAny." + error;
                                }
                            }
                            if (message.singleClusterRouting != null && message.hasOwnProperty("singleClusterRouting")) {
                                if (properties.routingPolicy === 1)
                                    return "routingPolicy: multiple values";
                                properties.routingPolicy = 1;
                                {
                                    var error = $root.google.bigtable.admin.v2.AppProfile.SingleClusterRouting.verify(message.singleClusterRouting);
                                    if (error)
                                        return "singleClusterRouting." + error;
                                }
                            }
                            if (message.priority != null && message.hasOwnProperty("priority")) {
                                properties.isolation = 1;
                                switch (message.priority) {
                                default:
                                    return "priority: enum value expected";
                                case 0:
                                case 1:
                                case 2:
                                case 3:
                                    break;
                                }
                            }
                            if (message.standardIsolation != null && message.hasOwnProperty("standardIsolation")) {
                                if (properties.isolation === 1)
                                    return "isolation: multiple values";
                                properties.isolation = 1;
                                {
                                    var error = $root.google.bigtable.admin.v2.AppProfile.StandardIsolation.verify(message.standardIsolation);
                                    if (error)
                                        return "standardIsolation." + error;
                                }
                            }
                            if (message.dataBoostIsolationReadOnly != null && message.hasOwnProperty("dataBoostIsolationReadOnly")) {
                                if (properties.isolation === 1)
                                    return "isolation: multiple values";
                                properties.isolation = 1;
                                {
                                    var error = $root.google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly.verify(message.dataBoostIsolationReadOnly);
                                    if (error)
                                        return "dataBoostIsolationReadOnly." + error;
                                }
                            }
                            return null;
                        };
    
                        /**
                         * Creates an AppProfile message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.AppProfile
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.AppProfile} AppProfile
                         */
                        AppProfile.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.AppProfile)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.AppProfile();
                            if (object.name != null)
                                message.name = String(object.name);
                            if (object.etag != null)
                                message.etag = String(object.etag);
                            if (object.description != null)
                                message.description = String(object.description);
                            if (object.multiClusterRoutingUseAny != null) {
                                if (typeof object.multiClusterRoutingUseAny !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.AppProfile.multiClusterRoutingUseAny: object expected");
                                message.multiClusterRoutingUseAny = $root.google.bigtable.admin.v2.AppProfile.MultiClusterRoutingUseAny.fromObject(object.multiClusterRoutingUseAny);
                            }
                            if (object.singleClusterRouting != null) {
                                if (typeof object.singleClusterRouting !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.AppProfile.singleClusterRouting: object expected");
                                message.singleClusterRouting = $root.google.bigtable.admin.v2.AppProfile.SingleClusterRouting.fromObject(object.singleClusterRouting);
                            }
                            switch (object.priority) {
                            default:
                                if (typeof object.priority === "number") {
                                    message.priority = object.priority;
                                    break;
                                }
                                break;
                            case "PRIORITY_UNSPECIFIED":
                            case 0:
                                message.priority = 0;
                                break;
                            case "PRIORITY_LOW":
                            case 1:
                                message.priority = 1;
                                break;
                            case "PRIORITY_MEDIUM":
                            case 2:
                                message.priority = 2;
                                break;
                            case "PRIORITY_HIGH":
                            case 3:
                                message.priority = 3;
                                break;
                            }
                            if (object.standardIsolation != null) {
                                if (typeof object.standardIsolation !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.AppProfile.standardIsolation: object expected");
                                message.standardIsolation = $root.google.bigtable.admin.v2.AppProfile.StandardIsolation.fromObject(object.standardIsolation);
                            }
                            if (object.dataBoostIsolationReadOnly != null) {
                                if (typeof object.dataBoostIsolationReadOnly !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.AppProfile.dataBoostIsolationReadOnly: object expected");
                                message.dataBoostIsolationReadOnly = $root.google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly.fromObject(object.dataBoostIsolationReadOnly);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from an AppProfile message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.AppProfile
                         * @static
                         * @param {google.bigtable.admin.v2.AppProfile} message AppProfile
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        AppProfile.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.name = "";
                                object.etag = "";
                                object.description = "";
                            }
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            if (message.etag != null && message.hasOwnProperty("etag"))
                                object.etag = message.etag;
                            if (message.description != null && message.hasOwnProperty("description"))
                                object.description = message.description;
                            if (message.multiClusterRoutingUseAny != null && message.hasOwnProperty("multiClusterRoutingUseAny")) {
                                object.multiClusterRoutingUseAny = $root.google.bigtable.admin.v2.AppProfile.MultiClusterRoutingUseAny.toObject(message.multiClusterRoutingUseAny, options);
                                if (options.oneofs)
                                    object.routingPolicy = "multiClusterRoutingUseAny";
                            }
                            if (message.singleClusterRouting != null && message.hasOwnProperty("singleClusterRouting")) {
                                object.singleClusterRouting = $root.google.bigtable.admin.v2.AppProfile.SingleClusterRouting.toObject(message.singleClusterRouting, options);
                                if (options.oneofs)
                                    object.routingPolicy = "singleClusterRouting";
                            }
                            if (message.priority != null && message.hasOwnProperty("priority")) {
                                object.priority = options.enums === String ? $root.google.bigtable.admin.v2.AppProfile.Priority[message.priority] === undefined ? message.priority : $root.google.bigtable.admin.v2.AppProfile.Priority[message.priority] : message.priority;
                                if (options.oneofs)
                                    object.isolation = "priority";
                            }
                            if (message.dataBoostIsolationReadOnly != null && message.hasOwnProperty("dataBoostIsolationReadOnly")) {
                                object.dataBoostIsolationReadOnly = $root.google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly.toObject(message.dataBoostIsolationReadOnly, options);
                                if (options.oneofs)
                                    object.isolation = "dataBoostIsolationReadOnly";
                            }
                            if (message.standardIsolation != null && message.hasOwnProperty("standardIsolation")) {
                                object.standardIsolation = $root.google.bigtable.admin.v2.AppProfile.StandardIsolation.toObject(message.standardIsolation, options);
                                if (options.oneofs)
                                    object.isolation = "standardIsolation";
                            }
                            return object;
                        };
    
                        /**
                         * Converts this AppProfile to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.AppProfile
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        AppProfile.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for AppProfile
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.AppProfile
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        AppProfile.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.AppProfile";
                        };
    
                        AppProfile.MultiClusterRoutingUseAny = (function() {
    
                            /**
                             * Properties of a MultiClusterRoutingUseAny.
                             * @memberof google.bigtable.admin.v2.AppProfile
                             * @interface IMultiClusterRoutingUseAny
                             * @property {Array.<string>|null} [clusterIds] MultiClusterRoutingUseAny clusterIds
                             */
    
                            /**
                             * Constructs a new MultiClusterRoutingUseAny.
                             * @memberof google.bigtable.admin.v2.AppProfile
                             * @classdesc Represents a MultiClusterRoutingUseAny.
                             * @implements IMultiClusterRoutingUseAny
                             * @constructor
                             * @param {google.bigtable.admin.v2.AppProfile.IMultiClusterRoutingUseAny=} [properties] Properties to set
                             */
                            function MultiClusterRoutingUseAny(properties) {
                                this.clusterIds = [];
                                if (properties)
                                    for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                        if (properties[keys[i]] != null)
                                            this[keys[i]] = properties[keys[i]];
                            }
    
                            /**
                             * MultiClusterRoutingUseAny clusterIds.
                             * @member {Array.<string>} clusterIds
                             * @memberof google.bigtable.admin.v2.AppProfile.MultiClusterRoutingUseAny
                             * @instance
                             */
                            MultiClusterRoutingUseAny.prototype.clusterIds = $util.emptyArray;
    
                            /**
                             * Creates a new MultiClusterRoutingUseAny instance using the specified properties.
                             * @function create
                             * @memberof google.bigtable.admin.v2.AppProfile.MultiClusterRoutingUseAny
                             * @static
                             * @param {google.bigtable.admin.v2.AppProfile.IMultiClusterRoutingUseAny=} [properties] Properties to set
                             * @returns {google.bigtable.admin.v2.AppProfile.MultiClusterRoutingUseAny} MultiClusterRoutingUseAny instance
                             */
                            MultiClusterRoutingUseAny.create = function create(properties) {
                                return new MultiClusterRoutingUseAny(properties);
                            };
    
                            /**
                             * Encodes the specified MultiClusterRoutingUseAny message. Does not implicitly {@link google.bigtable.admin.v2.AppProfile.MultiClusterRoutingUseAny.verify|verify} messages.
                             * @function encode
                             * @memberof google.bigtable.admin.v2.AppProfile.MultiClusterRoutingUseAny
                             * @static
                             * @param {google.bigtable.admin.v2.AppProfile.IMultiClusterRoutingUseAny} message MultiClusterRoutingUseAny message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MultiClusterRoutingUseAny.encode = function encode(message, writer) {
                                if (!writer)
                                    writer = $Writer.create();
                                if (message.clusterIds != null && message.clusterIds.length)
                                    for (var i = 0; i < message.clusterIds.length; ++i)
                                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.clusterIds[i]);
                                return writer;
                            };
    
                            /**
                             * Encodes the specified MultiClusterRoutingUseAny message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.AppProfile.MultiClusterRoutingUseAny.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.bigtable.admin.v2.AppProfile.MultiClusterRoutingUseAny
                             * @static
                             * @param {google.bigtable.admin.v2.AppProfile.IMultiClusterRoutingUseAny} message MultiClusterRoutingUseAny message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            MultiClusterRoutingUseAny.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a MultiClusterRoutingUseAny message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.bigtable.admin.v2.AppProfile.MultiClusterRoutingUseAny
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @param {number} [length] Message length if known beforehand
                             * @returns {google.bigtable.admin.v2.AppProfile.MultiClusterRoutingUseAny} MultiClusterRoutingUseAny
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MultiClusterRoutingUseAny.decode = function decode(reader, length) {
                                if (!(reader instanceof $Reader))
                                    reader = $Reader.create(reader);
                                var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.AppProfile.MultiClusterRoutingUseAny();
                                while (reader.pos < end) {
                                    var tag = reader.uint32();
                                    switch (tag >>> 3) {
                                    case 1: {
                                            if (!(message.clusterIds && message.clusterIds.length))
                                                message.clusterIds = [];
                                            message.clusterIds.push(reader.string());
                                            break;
                                        }
                                    default:
                                        reader.skipType(tag & 7);
                                        break;
                                    }
                                }
                                return message;
                            };
    
                            /**
                             * Decodes a MultiClusterRoutingUseAny message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.bigtable.admin.v2.AppProfile.MultiClusterRoutingUseAny
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.bigtable.admin.v2.AppProfile.MultiClusterRoutingUseAny} MultiClusterRoutingUseAny
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            MultiClusterRoutingUseAny.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a MultiClusterRoutingUseAny message.
                             * @function verify
                             * @memberof google.bigtable.admin.v2.AppProfile.MultiClusterRoutingUseAny
                             * @static
                             * @param {Object.<string,*>} message Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            MultiClusterRoutingUseAny.verify = function verify(message) {
                                if (typeof message !== "object" || message === null)
                                    return "object expected";
                                if (message.clusterIds != null && message.hasOwnProperty("clusterIds")) {
                                    if (!Array.isArray(message.clusterIds))
                                        return "clusterIds: array expected";
                                    for (var i = 0; i < message.clusterIds.length; ++i)
                                        if (!$util.isString(message.clusterIds[i]))
                                            return "clusterIds: string[] expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a MultiClusterRoutingUseAny message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.bigtable.admin.v2.AppProfile.MultiClusterRoutingUseAny
                             * @static
                             * @param {Object.<string,*>} object Plain object
                             * @returns {google.bigtable.admin.v2.AppProfile.MultiClusterRoutingUseAny} MultiClusterRoutingUseAny
                             */
                            MultiClusterRoutingUseAny.fromObject = function fromObject(object) {
                                if (object instanceof $root.google.bigtable.admin.v2.AppProfile.MultiClusterRoutingUseAny)
                                    return object;
                                var message = new $root.google.bigtable.admin.v2.AppProfile.MultiClusterRoutingUseAny();
                                if (object.clusterIds) {
                                    if (!Array.isArray(object.clusterIds))
                                        throw TypeError(".google.bigtable.admin.v2.AppProfile.MultiClusterRoutingUseAny.clusterIds: array expected");
                                    message.clusterIds = [];
                                    for (var i = 0; i < object.clusterIds.length; ++i)
                                        message.clusterIds[i] = String(object.clusterIds[i]);
                                }
                                return message;
                            };
    
                            /**
                             * Creates a plain object from a MultiClusterRoutingUseAny message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.bigtable.admin.v2.AppProfile.MultiClusterRoutingUseAny
                             * @static
                             * @param {google.bigtable.admin.v2.AppProfile.MultiClusterRoutingUseAny} message MultiClusterRoutingUseAny
                             * @param {$protobuf.IConversionOptions} [options] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            MultiClusterRoutingUseAny.toObject = function toObject(message, options) {
                                if (!options)
                                    options = {};
                                var object = {};
                                if (options.arrays || options.defaults)
                                    object.clusterIds = [];
                                if (message.clusterIds && message.clusterIds.length) {
                                    object.clusterIds = [];
                                    for (var j = 0; j < message.clusterIds.length; ++j)
                                        object.clusterIds[j] = message.clusterIds[j];
                                }
                                return object;
                            };
    
                            /**
                             * Converts this MultiClusterRoutingUseAny to JSON.
                             * @function toJSON
                             * @memberof google.bigtable.admin.v2.AppProfile.MultiClusterRoutingUseAny
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            MultiClusterRoutingUseAny.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * Gets the default type url for MultiClusterRoutingUseAny
                             * @function getTypeUrl
                             * @memberof google.bigtable.admin.v2.AppProfile.MultiClusterRoutingUseAny
                             * @static
                             * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                             * @returns {string} The default type url
                             */
                            MultiClusterRoutingUseAny.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                                if (typeUrlPrefix === undefined) {
                                    typeUrlPrefix = "type.googleapis.com";
                                }
                                return typeUrlPrefix + "/google.bigtable.admin.v2.AppProfile.MultiClusterRoutingUseAny";
                            };
    
                            return MultiClusterRoutingUseAny;
                        })();
    
                        AppProfile.SingleClusterRouting = (function() {
    
                            /**
                             * Properties of a SingleClusterRouting.
                             * @memberof google.bigtable.admin.v2.AppProfile
                             * @interface ISingleClusterRouting
                             * @property {string|null} [clusterId] SingleClusterRouting clusterId
                             * @property {boolean|null} [allowTransactionalWrites] SingleClusterRouting allowTransactionalWrites
                             */
    
                            /**
                             * Constructs a new SingleClusterRouting.
                             * @memberof google.bigtable.admin.v2.AppProfile
                             * @classdesc Represents a SingleClusterRouting.
                             * @implements ISingleClusterRouting
                             * @constructor
                             * @param {google.bigtable.admin.v2.AppProfile.ISingleClusterRouting=} [properties] Properties to set
                             */
                            function SingleClusterRouting(properties) {
                                if (properties)
                                    for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                        if (properties[keys[i]] != null)
                                            this[keys[i]] = properties[keys[i]];
                            }
    
                            /**
                             * SingleClusterRouting clusterId.
                             * @member {string} clusterId
                             * @memberof google.bigtable.admin.v2.AppProfile.SingleClusterRouting
                             * @instance
                             */
                            SingleClusterRouting.prototype.clusterId = "";
    
                            /**
                             * SingleClusterRouting allowTransactionalWrites.
                             * @member {boolean} allowTransactionalWrites
                             * @memberof google.bigtable.admin.v2.AppProfile.SingleClusterRouting
                             * @instance
                             */
                            SingleClusterRouting.prototype.allowTransactionalWrites = false;
    
                            /**
                             * Creates a new SingleClusterRouting instance using the specified properties.
                             * @function create
                             * @memberof google.bigtable.admin.v2.AppProfile.SingleClusterRouting
                             * @static
                             * @param {google.bigtable.admin.v2.AppProfile.ISingleClusterRouting=} [properties] Properties to set
                             * @returns {google.bigtable.admin.v2.AppProfile.SingleClusterRouting} SingleClusterRouting instance
                             */
                            SingleClusterRouting.create = function create(properties) {
                                return new SingleClusterRouting(properties);
                            };
    
                            /**
                             * Encodes the specified SingleClusterRouting message. Does not implicitly {@link google.bigtable.admin.v2.AppProfile.SingleClusterRouting.verify|verify} messages.
                             * @function encode
                             * @memberof google.bigtable.admin.v2.AppProfile.SingleClusterRouting
                             * @static
                             * @param {google.bigtable.admin.v2.AppProfile.ISingleClusterRouting} message SingleClusterRouting message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SingleClusterRouting.encode = function encode(message, writer) {
                                if (!writer)
                                    writer = $Writer.create();
                                if (message.clusterId != null && Object.hasOwnProperty.call(message, "clusterId"))
                                    writer.uint32(/* id 1, wireType 2 =*/10).string(message.clusterId);
                                if (message.allowTransactionalWrites != null && Object.hasOwnProperty.call(message, "allowTransactionalWrites"))
                                    writer.uint32(/* id 2, wireType 0 =*/16).bool(message.allowTransactionalWrites);
                                return writer;
                            };
    
                            /**
                             * Encodes the specified SingleClusterRouting message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.AppProfile.SingleClusterRouting.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.bigtable.admin.v2.AppProfile.SingleClusterRouting
                             * @static
                             * @param {google.bigtable.admin.v2.AppProfile.ISingleClusterRouting} message SingleClusterRouting message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SingleClusterRouting.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SingleClusterRouting message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.bigtable.admin.v2.AppProfile.SingleClusterRouting
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @param {number} [length] Message length if known beforehand
                             * @returns {google.bigtable.admin.v2.AppProfile.SingleClusterRouting} SingleClusterRouting
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SingleClusterRouting.decode = function decode(reader, length) {
                                if (!(reader instanceof $Reader))
                                    reader = $Reader.create(reader);
                                var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.AppProfile.SingleClusterRouting();
                                while (reader.pos < end) {
                                    var tag = reader.uint32();
                                    switch (tag >>> 3) {
                                    case 1: {
                                            message.clusterId = reader.string();
                                            break;
                                        }
                                    case 2: {
                                            message.allowTransactionalWrites = reader.bool();
                                            break;
                                        }
                                    default:
                                        reader.skipType(tag & 7);
                                        break;
                                    }
                                }
                                return message;
                            };
    
                            /**
                             * Decodes a SingleClusterRouting message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.bigtable.admin.v2.AppProfile.SingleClusterRouting
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.bigtable.admin.v2.AppProfile.SingleClusterRouting} SingleClusterRouting
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SingleClusterRouting.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SingleClusterRouting message.
                             * @function verify
                             * @memberof google.bigtable.admin.v2.AppProfile.SingleClusterRouting
                             * @static
                             * @param {Object.<string,*>} message Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SingleClusterRouting.verify = function verify(message) {
                                if (typeof message !== "object" || message === null)
                                    return "object expected";
                                if (message.clusterId != null && message.hasOwnProperty("clusterId"))
                                    if (!$util.isString(message.clusterId))
                                        return "clusterId: string expected";
                                if (message.allowTransactionalWrites != null && message.hasOwnProperty("allowTransactionalWrites"))
                                    if (typeof message.allowTransactionalWrites !== "boolean")
                                        return "allowTransactionalWrites: boolean expected";
                                return null;
                            };
    
                            /**
                             * Creates a SingleClusterRouting message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.bigtable.admin.v2.AppProfile.SingleClusterRouting
                             * @static
                             * @param {Object.<string,*>} object Plain object
                             * @returns {google.bigtable.admin.v2.AppProfile.SingleClusterRouting} SingleClusterRouting
                             */
                            SingleClusterRouting.fromObject = function fromObject(object) {
                                if (object instanceof $root.google.bigtable.admin.v2.AppProfile.SingleClusterRouting)
                                    return object;
                                var message = new $root.google.bigtable.admin.v2.AppProfile.SingleClusterRouting();
                                if (object.clusterId != null)
                                    message.clusterId = String(object.clusterId);
                                if (object.allowTransactionalWrites != null)
                                    message.allowTransactionalWrites = Boolean(object.allowTransactionalWrites);
                                return message;
                            };
    
                            /**
                             * Creates a plain object from a SingleClusterRouting message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.bigtable.admin.v2.AppProfile.SingleClusterRouting
                             * @static
                             * @param {google.bigtable.admin.v2.AppProfile.SingleClusterRouting} message SingleClusterRouting
                             * @param {$protobuf.IConversionOptions} [options] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SingleClusterRouting.toObject = function toObject(message, options) {
                                if (!options)
                                    options = {};
                                var object = {};
                                if (options.defaults) {
                                    object.clusterId = "";
                                    object.allowTransactionalWrites = false;
                                }
                                if (message.clusterId != null && message.hasOwnProperty("clusterId"))
                                    object.clusterId = message.clusterId;
                                if (message.allowTransactionalWrites != null && message.hasOwnProperty("allowTransactionalWrites"))
                                    object.allowTransactionalWrites = message.allowTransactionalWrites;
                                return object;
                            };
    
                            /**
                             * Converts this SingleClusterRouting to JSON.
                             * @function toJSON
                             * @memberof google.bigtable.admin.v2.AppProfile.SingleClusterRouting
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SingleClusterRouting.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * Gets the default type url for SingleClusterRouting
                             * @function getTypeUrl
                             * @memberof google.bigtable.admin.v2.AppProfile.SingleClusterRouting
                             * @static
                             * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                             * @returns {string} The default type url
                             */
                            SingleClusterRouting.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                                if (typeUrlPrefix === undefined) {
                                    typeUrlPrefix = "type.googleapis.com";
                                }
                                return typeUrlPrefix + "/google.bigtable.admin.v2.AppProfile.SingleClusterRouting";
                            };
    
                            return SingleClusterRouting;
                        })();
    
                        /**
                         * Priority enum.
                         * @name google.bigtable.admin.v2.AppProfile.Priority
                         * @enum {number}
                         * @property {number} PRIORITY_UNSPECIFIED=0 PRIORITY_UNSPECIFIED value
                         * @property {number} PRIORITY_LOW=1 PRIORITY_LOW value
                         * @property {number} PRIORITY_MEDIUM=2 PRIORITY_MEDIUM value
                         * @property {number} PRIORITY_HIGH=3 PRIORITY_HIGH value
                         */
                        AppProfile.Priority = (function() {
                            var valuesById = {}, values = Object.create(valuesById);
                            values[valuesById[0] = "PRIORITY_UNSPECIFIED"] = 0;
                            values[valuesById[1] = "PRIORITY_LOW"] = 1;
                            values[valuesById[2] = "PRIORITY_MEDIUM"] = 2;
                            values[valuesById[3] = "PRIORITY_HIGH"] = 3;
                            return values;
                        })();
    
                        AppProfile.StandardIsolation = (function() {
    
                            /**
                             * Properties of a StandardIsolation.
                             * @memberof google.bigtable.admin.v2.AppProfile
                             * @interface IStandardIsolation
                             * @property {google.bigtable.admin.v2.AppProfile.Priority|null} [priority] StandardIsolation priority
                             */
    
                            /**
                             * Constructs a new StandardIsolation.
                             * @memberof google.bigtable.admin.v2.AppProfile
                             * @classdesc Represents a StandardIsolation.
                             * @implements IStandardIsolation
                             * @constructor
                             * @param {google.bigtable.admin.v2.AppProfile.IStandardIsolation=} [properties] Properties to set
                             */
                            function StandardIsolation(properties) {
                                if (properties)
                                    for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                        if (properties[keys[i]] != null)
                                            this[keys[i]] = properties[keys[i]];
                            }
    
                            /**
                             * StandardIsolation priority.
                             * @member {google.bigtable.admin.v2.AppProfile.Priority} priority
                             * @memberof google.bigtable.admin.v2.AppProfile.StandardIsolation
                             * @instance
                             */
                            StandardIsolation.prototype.priority = 0;
    
                            /**
                             * Creates a new StandardIsolation instance using the specified properties.
                             * @function create
                             * @memberof google.bigtable.admin.v2.AppProfile.StandardIsolation
                             * @static
                             * @param {google.bigtable.admin.v2.AppProfile.IStandardIsolation=} [properties] Properties to set
                             * @returns {google.bigtable.admin.v2.AppProfile.StandardIsolation} StandardIsolation instance
                             */
                            StandardIsolation.create = function create(properties) {
                                return new StandardIsolation(properties);
                            };
    
                            /**
                             * Encodes the specified StandardIsolation message. Does not implicitly {@link google.bigtable.admin.v2.AppProfile.StandardIsolation.verify|verify} messages.
                             * @function encode
                             * @memberof google.bigtable.admin.v2.AppProfile.StandardIsolation
                             * @static
                             * @param {google.bigtable.admin.v2.AppProfile.IStandardIsolation} message StandardIsolation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            StandardIsolation.encode = function encode(message, writer) {
                                if (!writer)
                                    writer = $Writer.create();
                                if (message.priority != null && Object.hasOwnProperty.call(message, "priority"))
                                    writer.uint32(/* id 1, wireType 0 =*/8).int32(message.priority);
                                return writer;
                            };
    
                            /**
                             * Encodes the specified StandardIsolation message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.AppProfile.StandardIsolation.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.bigtable.admin.v2.AppProfile.StandardIsolation
                             * @static
                             * @param {google.bigtable.admin.v2.AppProfile.IStandardIsolation} message StandardIsolation message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            StandardIsolation.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a StandardIsolation message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.bigtable.admin.v2.AppProfile.StandardIsolation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @param {number} [length] Message length if known beforehand
                             * @returns {google.bigtable.admin.v2.AppProfile.StandardIsolation} StandardIsolation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            StandardIsolation.decode = function decode(reader, length) {
                                if (!(reader instanceof $Reader))
                                    reader = $Reader.create(reader);
                                var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.AppProfile.StandardIsolation();
                                while (reader.pos < end) {
                                    var tag = reader.uint32();
                                    switch (tag >>> 3) {
                                    case 1: {
                                            message.priority = reader.int32();
                                            break;
                                        }
                                    default:
                                        reader.skipType(tag & 7);
                                        break;
                                    }
                                }
                                return message;
                            };
    
                            /**
                             * Decodes a StandardIsolation message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.bigtable.admin.v2.AppProfile.StandardIsolation
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.bigtable.admin.v2.AppProfile.StandardIsolation} StandardIsolation
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            StandardIsolation.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a StandardIsolation message.
                             * @function verify
                             * @memberof google.bigtable.admin.v2.AppProfile.StandardIsolation
                             * @static
                             * @param {Object.<string,*>} message Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            StandardIsolation.verify = function verify(message) {
                                if (typeof message !== "object" || message === null)
                                    return "object expected";
                                if (message.priority != null && message.hasOwnProperty("priority"))
                                    switch (message.priority) {
                                    default:
                                        return "priority: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                        break;
                                    }
                                return null;
                            };
    
                            /**
                             * Creates a StandardIsolation message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.bigtable.admin.v2.AppProfile.StandardIsolation
                             * @static
                             * @param {Object.<string,*>} object Plain object
                             * @returns {google.bigtable.admin.v2.AppProfile.StandardIsolation} StandardIsolation
                             */
                            StandardIsolation.fromObject = function fromObject(object) {
                                if (object instanceof $root.google.bigtable.admin.v2.AppProfile.StandardIsolation)
                                    return object;
                                var message = new $root.google.bigtable.admin.v2.AppProfile.StandardIsolation();
                                switch (object.priority) {
                                default:
                                    if (typeof object.priority === "number") {
                                        message.priority = object.priority;
                                        break;
                                    }
                                    break;
                                case "PRIORITY_UNSPECIFIED":
                                case 0:
                                    message.priority = 0;
                                    break;
                                case "PRIORITY_LOW":
                                case 1:
                                    message.priority = 1;
                                    break;
                                case "PRIORITY_MEDIUM":
                                case 2:
                                    message.priority = 2;
                                    break;
                                case "PRIORITY_HIGH":
                                case 3:
                                    message.priority = 3;
                                    break;
                                }
                                return message;
                            };
    
                            /**
                             * Creates a plain object from a StandardIsolation message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.bigtable.admin.v2.AppProfile.StandardIsolation
                             * @static
                             * @param {google.bigtable.admin.v2.AppProfile.StandardIsolation} message StandardIsolation
                             * @param {$protobuf.IConversionOptions} [options] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            StandardIsolation.toObject = function toObject(message, options) {
                                if (!options)
                                    options = {};
                                var object = {};
                                if (options.defaults)
                                    object.priority = options.enums === String ? "PRIORITY_UNSPECIFIED" : 0;
                                if (message.priority != null && message.hasOwnProperty("priority"))
                                    object.priority = options.enums === String ? $root.google.bigtable.admin.v2.AppProfile.Priority[message.priority] === undefined ? message.priority : $root.google.bigtable.admin.v2.AppProfile.Priority[message.priority] : message.priority;
                                return object;
                            };
    
                            /**
                             * Converts this StandardIsolation to JSON.
                             * @function toJSON
                             * @memberof google.bigtable.admin.v2.AppProfile.StandardIsolation
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            StandardIsolation.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * Gets the default type url for StandardIsolation
                             * @function getTypeUrl
                             * @memberof google.bigtable.admin.v2.AppProfile.StandardIsolation
                             * @static
                             * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                             * @returns {string} The default type url
                             */
                            StandardIsolation.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                                if (typeUrlPrefix === undefined) {
                                    typeUrlPrefix = "type.googleapis.com";
                                }
                                return typeUrlPrefix + "/google.bigtable.admin.v2.AppProfile.StandardIsolation";
                            };
    
                            return StandardIsolation;
                        })();
    
                        AppProfile.DataBoostIsolationReadOnly = (function() {
    
                            /**
                             * Properties of a DataBoostIsolationReadOnly.
                             * @memberof google.bigtable.admin.v2.AppProfile
                             * @interface IDataBoostIsolationReadOnly
                             * @property {google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly.ComputeBillingOwner|null} [computeBillingOwner] DataBoostIsolationReadOnly computeBillingOwner
                             */
    
                            /**
                             * Constructs a new DataBoostIsolationReadOnly.
                             * @memberof google.bigtable.admin.v2.AppProfile
                             * @classdesc Represents a DataBoostIsolationReadOnly.
                             * @implements IDataBoostIsolationReadOnly
                             * @constructor
                             * @param {google.bigtable.admin.v2.AppProfile.IDataBoostIsolationReadOnly=} [properties] Properties to set
                             */
                            function DataBoostIsolationReadOnly(properties) {
                                if (properties)
                                    for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                        if (properties[keys[i]] != null)
                                            this[keys[i]] = properties[keys[i]];
                            }
    
                            /**
                             * DataBoostIsolationReadOnly computeBillingOwner.
                             * @member {google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly.ComputeBillingOwner|null|undefined} computeBillingOwner
                             * @memberof google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly
                             * @instance
                             */
                            DataBoostIsolationReadOnly.prototype.computeBillingOwner = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * DataBoostIsolationReadOnly _computeBillingOwner.
                             * @member {"computeBillingOwner"|undefined} _computeBillingOwner
                             * @memberof google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly
                             * @instance
                             */
                            Object.defineProperty(DataBoostIsolationReadOnly.prototype, "_computeBillingOwner", {
                                get: $util.oneOfGetter($oneOfFields = ["computeBillingOwner"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new DataBoostIsolationReadOnly instance using the specified properties.
                             * @function create
                             * @memberof google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly
                             * @static
                             * @param {google.bigtable.admin.v2.AppProfile.IDataBoostIsolationReadOnly=} [properties] Properties to set
                             * @returns {google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly} DataBoostIsolationReadOnly instance
                             */
                            DataBoostIsolationReadOnly.create = function create(properties) {
                                return new DataBoostIsolationReadOnly(properties);
                            };
    
                            /**
                             * Encodes the specified DataBoostIsolationReadOnly message. Does not implicitly {@link google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly.verify|verify} messages.
                             * @function encode
                             * @memberof google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly
                             * @static
                             * @param {google.bigtable.admin.v2.AppProfile.IDataBoostIsolationReadOnly} message DataBoostIsolationReadOnly message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DataBoostIsolationReadOnly.encode = function encode(message, writer) {
                                if (!writer)
                                    writer = $Writer.create();
                                if (message.computeBillingOwner != null && Object.hasOwnProperty.call(message, "computeBillingOwner"))
                                    writer.uint32(/* id 1, wireType 0 =*/8).int32(message.computeBillingOwner);
                                return writer;
                            };
    
                            /**
                             * Encodes the specified DataBoostIsolationReadOnly message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly
                             * @static
                             * @param {google.bigtable.admin.v2.AppProfile.IDataBoostIsolationReadOnly} message DataBoostIsolationReadOnly message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            DataBoostIsolationReadOnly.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a DataBoostIsolationReadOnly message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @param {number} [length] Message length if known beforehand
                             * @returns {google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly} DataBoostIsolationReadOnly
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DataBoostIsolationReadOnly.decode = function decode(reader, length) {
                                if (!(reader instanceof $Reader))
                                    reader = $Reader.create(reader);
                                var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly();
                                while (reader.pos < end) {
                                    var tag = reader.uint32();
                                    switch (tag >>> 3) {
                                    case 1: {
                                            message.computeBillingOwner = reader.int32();
                                            break;
                                        }
                                    default:
                                        reader.skipType(tag & 7);
                                        break;
                                    }
                                }
                                return message;
                            };
    
                            /**
                             * Decodes a DataBoostIsolationReadOnly message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly} DataBoostIsolationReadOnly
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            DataBoostIsolationReadOnly.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a DataBoostIsolationReadOnly message.
                             * @function verify
                             * @memberof google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly
                             * @static
                             * @param {Object.<string,*>} message Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            DataBoostIsolationReadOnly.verify = function verify(message) {
                                if (typeof message !== "object" || message === null)
                                    return "object expected";
                                var properties = {};
                                if (message.computeBillingOwner != null && message.hasOwnProperty("computeBillingOwner")) {
                                    properties._computeBillingOwner = 1;
                                    switch (message.computeBillingOwner) {
                                    default:
                                        return "computeBillingOwner: enum value expected";
                                    case 0:
                                    case 1:
                                        break;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a DataBoostIsolationReadOnly message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly
                             * @static
                             * @param {Object.<string,*>} object Plain object
                             * @returns {google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly} DataBoostIsolationReadOnly
                             */
                            DataBoostIsolationReadOnly.fromObject = function fromObject(object) {
                                if (object instanceof $root.google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly)
                                    return object;
                                var message = new $root.google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly();
                                switch (object.computeBillingOwner) {
                                default:
                                    if (typeof object.computeBillingOwner === "number") {
                                        message.computeBillingOwner = object.computeBillingOwner;
                                        break;
                                    }
                                    break;
                                case "COMPUTE_BILLING_OWNER_UNSPECIFIED":
                                case 0:
                                    message.computeBillingOwner = 0;
                                    break;
                                case "HOST_PAYS":
                                case 1:
                                    message.computeBillingOwner = 1;
                                    break;
                                }
                                return message;
                            };
    
                            /**
                             * Creates a plain object from a DataBoostIsolationReadOnly message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly
                             * @static
                             * @param {google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly} message DataBoostIsolationReadOnly
                             * @param {$protobuf.IConversionOptions} [options] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            DataBoostIsolationReadOnly.toObject = function toObject(message, options) {
                                if (!options)
                                    options = {};
                                var object = {};
                                if (message.computeBillingOwner != null && message.hasOwnProperty("computeBillingOwner")) {
                                    object.computeBillingOwner = options.enums === String ? $root.google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly.ComputeBillingOwner[message.computeBillingOwner] === undefined ? message.computeBillingOwner : $root.google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly.ComputeBillingOwner[message.computeBillingOwner] : message.computeBillingOwner;
                                    if (options.oneofs)
                                        object._computeBillingOwner = "computeBillingOwner";
                                }
                                return object;
                            };
    
                            /**
                             * Converts this DataBoostIsolationReadOnly to JSON.
                             * @function toJSON
                             * @memberof google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            DataBoostIsolationReadOnly.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * Gets the default type url for DataBoostIsolationReadOnly
                             * @function getTypeUrl
                             * @memberof google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly
                             * @static
                             * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                             * @returns {string} The default type url
                             */
                            DataBoostIsolationReadOnly.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                                if (typeUrlPrefix === undefined) {
                                    typeUrlPrefix = "type.googleapis.com";
                                }
                                return typeUrlPrefix + "/google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly";
                            };
    
                            /**
                             * ComputeBillingOwner enum.
                             * @name google.bigtable.admin.v2.AppProfile.DataBoostIsolationReadOnly.ComputeBillingOwner
                             * @enum {number}
                             * @property {number} COMPUTE_BILLING_OWNER_UNSPECIFIED=0 COMPUTE_BILLING_OWNER_UNSPECIFIED value
                             * @property {number} HOST_PAYS=1 HOST_PAYS value
                             */
                            DataBoostIsolationReadOnly.ComputeBillingOwner = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "COMPUTE_BILLING_OWNER_UNSPECIFIED"] = 0;
                                values[valuesById[1] = "HOST_PAYS"] = 1;
                                return values;
                            })();
    
                            return DataBoostIsolationReadOnly;
                        })();
    
                        return AppProfile;
                    })();
    
                    v2.HotTablet = (function() {
    
                        /**
                         * Properties of a HotTablet.
                         * @memberof google.bigtable.admin.v2
                         * @interface IHotTablet
                         * @property {string|null} [name] HotTablet name
                         * @property {string|null} [tableName] HotTablet tableName
                         * @property {google.protobuf.ITimestamp|null} [startTime] HotTablet startTime
                         * @property {google.protobuf.ITimestamp|null} [endTime] HotTablet endTime
                         * @property {string|null} [startKey] HotTablet startKey
                         * @property {string|null} [endKey] HotTablet endKey
                         * @property {number|null} [nodeCpuUsagePercent] HotTablet nodeCpuUsagePercent
                         */
    
                        /**
                         * Constructs a new HotTablet.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a HotTablet.
                         * @implements IHotTablet
                         * @constructor
                         * @param {google.bigtable.admin.v2.IHotTablet=} [properties] Properties to set
                         */
                        function HotTablet(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * HotTablet name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.HotTablet
                         * @instance
                         */
                        HotTablet.prototype.name = "";
    
                        /**
                         * HotTablet tableName.
                         * @member {string} tableName
                         * @memberof google.bigtable.admin.v2.HotTablet
                         * @instance
                         */
                        HotTablet.prototype.tableName = "";
    
                        /**
                         * HotTablet startTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} startTime
                         * @memberof google.bigtable.admin.v2.HotTablet
                         * @instance
                         */
                        HotTablet.prototype.startTime = null;
    
                        /**
                         * HotTablet endTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} endTime
                         * @memberof google.bigtable.admin.v2.HotTablet
                         * @instance
                         */
                        HotTablet.prototype.endTime = null;
    
                        /**
                         * HotTablet startKey.
                         * @member {string} startKey
                         * @memberof google.bigtable.admin.v2.HotTablet
                         * @instance
                         */
                        HotTablet.prototype.startKey = "";
    
                        /**
                         * HotTablet endKey.
                         * @member {string} endKey
                         * @memberof google.bigtable.admin.v2.HotTablet
                         * @instance
                         */
                        HotTablet.prototype.endKey = "";
    
                        /**
                         * HotTablet nodeCpuUsagePercent.
                         * @member {number} nodeCpuUsagePercent
                         * @memberof google.bigtable.admin.v2.HotTablet
                         * @instance
                         */
                        HotTablet.prototype.nodeCpuUsagePercent = 0;
    
                        /**
                         * Creates a new HotTablet instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.HotTablet
                         * @static
                         * @param {google.bigtable.admin.v2.IHotTablet=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.HotTablet} HotTablet instance
                         */
                        HotTablet.create = function create(properties) {
                            return new HotTablet(properties);
                        };
    
                        /**
                         * Encodes the specified HotTablet message. Does not implicitly {@link google.bigtable.admin.v2.HotTablet.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.HotTablet
                         * @static
                         * @param {google.bigtable.admin.v2.IHotTablet} message HotTablet message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        HotTablet.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            if (message.tableName != null && Object.hasOwnProperty.call(message, "tableName"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.tableName);
                            if (message.startTime != null && Object.hasOwnProperty.call(message, "startTime"))
                                $root.google.protobuf.Timestamp.encode(message.startTime, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            if (message.endTime != null && Object.hasOwnProperty.call(message, "endTime"))
                                $root.google.protobuf.Timestamp.encode(message.endTime, writer.uint32(/* id 4, wireType 2 =*/34).fork()).ldelim();
                            if (message.startKey != null && Object.hasOwnProperty.call(message, "startKey"))
                                writer.uint32(/* id 5, wireType 2 =*/42).string(message.startKey);
                            if (message.endKey != null && Object.hasOwnProperty.call(message, "endKey"))
                                writer.uint32(/* id 6, wireType 2 =*/50).string(message.endKey);
                            if (message.nodeCpuUsagePercent != null && Object.hasOwnProperty.call(message, "nodeCpuUsagePercent"))
                                writer.uint32(/* id 7, wireType 5 =*/61).float(message.nodeCpuUsagePercent);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified HotTablet message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.HotTablet.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.HotTablet
                         * @static
                         * @param {google.bigtable.admin.v2.IHotTablet} message HotTablet message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        HotTablet.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a HotTablet message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.HotTablet
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.HotTablet} HotTablet
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        HotTablet.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.HotTablet();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.tableName = reader.string();
                                        break;
                                    }
                                case 3: {
                                        message.startTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 4: {
                                        message.endTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 5: {
                                        message.startKey = reader.string();
                                        break;
                                    }
                                case 6: {
                                        message.endKey = reader.string();
                                        break;
                                    }
                                case 7: {
                                        message.nodeCpuUsagePercent = reader.float();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a HotTablet message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.HotTablet
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.HotTablet} HotTablet
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        HotTablet.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a HotTablet message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.HotTablet
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        HotTablet.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            if (message.tableName != null && message.hasOwnProperty("tableName"))
                                if (!$util.isString(message.tableName))
                                    return "tableName: string expected";
                            if (message.startTime != null && message.hasOwnProperty("startTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.startTime);
                                if (error)
                                    return "startTime." + error;
                            }
                            if (message.endTime != null && message.hasOwnProperty("endTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.endTime);
                                if (error)
                                    return "endTime." + error;
                            }
                            if (message.startKey != null && message.hasOwnProperty("startKey"))
                                if (!$util.isString(message.startKey))
                                    return "startKey: string expected";
                            if (message.endKey != null && message.hasOwnProperty("endKey"))
                                if (!$util.isString(message.endKey))
                                    return "endKey: string expected";
                            if (message.nodeCpuUsagePercent != null && message.hasOwnProperty("nodeCpuUsagePercent"))
                                if (typeof message.nodeCpuUsagePercent !== "number")
                                    return "nodeCpuUsagePercent: number expected";
                            return null;
                        };
    
                        /**
                         * Creates a HotTablet message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.HotTablet
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.HotTablet} HotTablet
                         */
                        HotTablet.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.HotTablet)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.HotTablet();
                            if (object.name != null)
                                message.name = String(object.name);
                            if (object.tableName != null)
                                message.tableName = String(object.tableName);
                            if (object.startTime != null) {
                                if (typeof object.startTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.HotTablet.startTime: object expected");
                                message.startTime = $root.google.protobuf.Timestamp.fromObject(object.startTime);
                            }
                            if (object.endTime != null) {
                                if (typeof object.endTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.HotTablet.endTime: object expected");
                                message.endTime = $root.google.protobuf.Timestamp.fromObject(object.endTime);
                            }
                            if (object.startKey != null)
                                message.startKey = String(object.startKey);
                            if (object.endKey != null)
                                message.endKey = String(object.endKey);
                            if (object.nodeCpuUsagePercent != null)
                                message.nodeCpuUsagePercent = Number(object.nodeCpuUsagePercent);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a HotTablet message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.HotTablet
                         * @static
                         * @param {google.bigtable.admin.v2.HotTablet} message HotTablet
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        HotTablet.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.name = "";
                                object.tableName = "";
                                object.startTime = null;
                                object.endTime = null;
                                object.startKey = "";
                                object.endKey = "";
                                object.nodeCpuUsagePercent = 0;
                            }
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            if (message.tableName != null && message.hasOwnProperty("tableName"))
                                object.tableName = message.tableName;
                            if (message.startTime != null && message.hasOwnProperty("startTime"))
                                object.startTime = $root.google.protobuf.Timestamp.toObject(message.startTime, options);
                            if (message.endTime != null && message.hasOwnProperty("endTime"))
                                object.endTime = $root.google.protobuf.Timestamp.toObject(message.endTime, options);
                            if (message.startKey != null && message.hasOwnProperty("startKey"))
                                object.startKey = message.startKey;
                            if (message.endKey != null && message.hasOwnProperty("endKey"))
                                object.endKey = message.endKey;
                            if (message.nodeCpuUsagePercent != null && message.hasOwnProperty("nodeCpuUsagePercent"))
                                object.nodeCpuUsagePercent = options.json && !isFinite(message.nodeCpuUsagePercent) ? String(message.nodeCpuUsagePercent) : message.nodeCpuUsagePercent;
                            return object;
                        };
    
                        /**
                         * Converts this HotTablet to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.HotTablet
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        HotTablet.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for HotTablet
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.HotTablet
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        HotTablet.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.HotTablet";
                        };
    
                        return HotTablet;
                    })();
    
                    /**
                     * StorageType enum.
                     * @name google.bigtable.admin.v2.StorageType
                     * @enum {number}
                     * @property {number} STORAGE_TYPE_UNSPECIFIED=0 STORAGE_TYPE_UNSPECIFIED value
                     * @property {number} SSD=1 SSD value
                     * @property {number} HDD=2 HDD value
                     */
                    v2.StorageType = (function() {
                        var valuesById = {}, values = Object.create(valuesById);
                        values[valuesById[0] = "STORAGE_TYPE_UNSPECIFIED"] = 0;
                        values[valuesById[1] = "SSD"] = 1;
                        values[valuesById[2] = "HDD"] = 2;
                        return values;
                    })();
    
                    v2.OperationProgress = (function() {
    
                        /**
                         * Properties of an OperationProgress.
                         * @memberof google.bigtable.admin.v2
                         * @interface IOperationProgress
                         * @property {number|null} [progressPercent] OperationProgress progressPercent
                         * @property {google.protobuf.ITimestamp|null} [startTime] OperationProgress startTime
                         * @property {google.protobuf.ITimestamp|null} [endTime] OperationProgress endTime
                         */
    
                        /**
                         * Constructs a new OperationProgress.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents an OperationProgress.
                         * @implements IOperationProgress
                         * @constructor
                         * @param {google.bigtable.admin.v2.IOperationProgress=} [properties] Properties to set
                         */
                        function OperationProgress(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * OperationProgress progressPercent.
                         * @member {number} progressPercent
                         * @memberof google.bigtable.admin.v2.OperationProgress
                         * @instance
                         */
                        OperationProgress.prototype.progressPercent = 0;
    
                        /**
                         * OperationProgress startTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} startTime
                         * @memberof google.bigtable.admin.v2.OperationProgress
                         * @instance
                         */
                        OperationProgress.prototype.startTime = null;
    
                        /**
                         * OperationProgress endTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} endTime
                         * @memberof google.bigtable.admin.v2.OperationProgress
                         * @instance
                         */
                        OperationProgress.prototype.endTime = null;
    
                        /**
                         * Creates a new OperationProgress instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.OperationProgress
                         * @static
                         * @param {google.bigtable.admin.v2.IOperationProgress=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.OperationProgress} OperationProgress instance
                         */
                        OperationProgress.create = function create(properties) {
                            return new OperationProgress(properties);
                        };
    
                        /**
                         * Encodes the specified OperationProgress message. Does not implicitly {@link google.bigtable.admin.v2.OperationProgress.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.OperationProgress
                         * @static
                         * @param {google.bigtable.admin.v2.IOperationProgress} message OperationProgress message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        OperationProgress.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.progressPercent != null && Object.hasOwnProperty.call(message, "progressPercent"))
                                writer.uint32(/* id 1, wireType 0 =*/8).int32(message.progressPercent);
                            if (message.startTime != null && Object.hasOwnProperty.call(message, "startTime"))
                                $root.google.protobuf.Timestamp.encode(message.startTime, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            if (message.endTime != null && Object.hasOwnProperty.call(message, "endTime"))
                                $root.google.protobuf.Timestamp.encode(message.endTime, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified OperationProgress message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.OperationProgress.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.OperationProgress
                         * @static
                         * @param {google.bigtable.admin.v2.IOperationProgress} message OperationProgress message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        OperationProgress.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes an OperationProgress message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.OperationProgress
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.OperationProgress} OperationProgress
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        OperationProgress.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.OperationProgress();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.progressPercent = reader.int32();
                                        break;
                                    }
                                case 2: {
                                        message.startTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 3: {
                                        message.endTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes an OperationProgress message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.OperationProgress
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.OperationProgress} OperationProgress
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        OperationProgress.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies an OperationProgress message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.OperationProgress
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        OperationProgress.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.progressPercent != null && message.hasOwnProperty("progressPercent"))
                                if (!$util.isInteger(message.progressPercent))
                                    return "progressPercent: integer expected";
                            if (message.startTime != null && message.hasOwnProperty("startTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.startTime);
                                if (error)
                                    return "startTime." + error;
                            }
                            if (message.endTime != null && message.hasOwnProperty("endTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.endTime);
                                if (error)
                                    return "endTime." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates an OperationProgress message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.OperationProgress
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.OperationProgress} OperationProgress
                         */
                        OperationProgress.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.OperationProgress)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.OperationProgress();
                            if (object.progressPercent != null)
                                message.progressPercent = object.progressPercent | 0;
                            if (object.startTime != null) {
                                if (typeof object.startTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.OperationProgress.startTime: object expected");
                                message.startTime = $root.google.protobuf.Timestamp.fromObject(object.startTime);
                            }
                            if (object.endTime != null) {
                                if (typeof object.endTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.OperationProgress.endTime: object expected");
                                message.endTime = $root.google.protobuf.Timestamp.fromObject(object.endTime);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from an OperationProgress message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.OperationProgress
                         * @static
                         * @param {google.bigtable.admin.v2.OperationProgress} message OperationProgress
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        OperationProgress.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.progressPercent = 0;
                                object.startTime = null;
                                object.endTime = null;
                            }
                            if (message.progressPercent != null && message.hasOwnProperty("progressPercent"))
                                object.progressPercent = message.progressPercent;
                            if (message.startTime != null && message.hasOwnProperty("startTime"))
                                object.startTime = $root.google.protobuf.Timestamp.toObject(message.startTime, options);
                            if (message.endTime != null && message.hasOwnProperty("endTime"))
                                object.endTime = $root.google.protobuf.Timestamp.toObject(message.endTime, options);
                            return object;
                        };
    
                        /**
                         * Converts this OperationProgress to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.OperationProgress
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        OperationProgress.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for OperationProgress
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.OperationProgress
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        OperationProgress.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.OperationProgress";
                        };
    
                        return OperationProgress;
                    })();
    
                    v2.BigtableTableAdmin = (function() {
    
                        /**
                         * Constructs a new BigtableTableAdmin service.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a BigtableTableAdmin
                         * @extends $protobuf.rpc.Service
                         * @constructor
                         * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                         * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                         * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                         */
                        function BigtableTableAdmin(rpcImpl, requestDelimited, responseDelimited) {
                            $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                        }
    
                        (BigtableTableAdmin.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = BigtableTableAdmin;
    
                        /**
                         * Creates new BigtableTableAdmin service using the specified rpc implementation.
                         * @function create
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @static
                         * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                         * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                         * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                         * @returns {BigtableTableAdmin} RPC service. Useful where requests and/or responses are streamed.
                         */
                        BigtableTableAdmin.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                            return new this(rpcImpl, requestDelimited, responseDelimited);
                        };
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|createTable}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef CreateTableCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.bigtable.admin.v2.Table} [response] Table
                         */
    
                        /**
                         * Calls CreateTable.
                         * @function createTable
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.ICreateTableRequest} request CreateTableRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.CreateTableCallback} callback Node-style callback called with the error, if any, and Table
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.createTable = function createTable(request, callback) {
                            return this.rpcCall(createTable, $root.google.bigtable.admin.v2.CreateTableRequest, $root.google.bigtable.admin.v2.Table, request, callback);
                        }, "name", { value: "CreateTable" });
    
                        /**
                         * Calls CreateTable.
                         * @function createTable
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.ICreateTableRequest} request CreateTableRequest message or plain object
                         * @returns {Promise<google.bigtable.admin.v2.Table>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|createTableFromSnapshot}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef CreateTableFromSnapshotCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.longrunning.Operation} [response] Operation
                         */
    
                        /**
                         * Calls CreateTableFromSnapshot.
                         * @function createTableFromSnapshot
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.ICreateTableFromSnapshotRequest} request CreateTableFromSnapshotRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.CreateTableFromSnapshotCallback} callback Node-style callback called with the error, if any, and Operation
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.createTableFromSnapshot = function createTableFromSnapshot(request, callback) {
                            return this.rpcCall(createTableFromSnapshot, $root.google.bigtable.admin.v2.CreateTableFromSnapshotRequest, $root.google.longrunning.Operation, request, callback);
                        }, "name", { value: "CreateTableFromSnapshot" });
    
                        /**
                         * Calls CreateTableFromSnapshot.
                         * @function createTableFromSnapshot
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.ICreateTableFromSnapshotRequest} request CreateTableFromSnapshotRequest message or plain object
                         * @returns {Promise<google.longrunning.Operation>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|listTables}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef ListTablesCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.bigtable.admin.v2.ListTablesResponse} [response] ListTablesResponse
                         */
    
                        /**
                         * Calls ListTables.
                         * @function listTables
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IListTablesRequest} request ListTablesRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.ListTablesCallback} callback Node-style callback called with the error, if any, and ListTablesResponse
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.listTables = function listTables(request, callback) {
                            return this.rpcCall(listTables, $root.google.bigtable.admin.v2.ListTablesRequest, $root.google.bigtable.admin.v2.ListTablesResponse, request, callback);
                        }, "name", { value: "ListTables" });
    
                        /**
                         * Calls ListTables.
                         * @function listTables
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IListTablesRequest} request ListTablesRequest message or plain object
                         * @returns {Promise<google.bigtable.admin.v2.ListTablesResponse>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|getTable}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef GetTableCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.bigtable.admin.v2.Table} [response] Table
                         */
    
                        /**
                         * Calls GetTable.
                         * @function getTable
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IGetTableRequest} request GetTableRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.GetTableCallback} callback Node-style callback called with the error, if any, and Table
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.getTable = function getTable(request, callback) {
                            return this.rpcCall(getTable, $root.google.bigtable.admin.v2.GetTableRequest, $root.google.bigtable.admin.v2.Table, request, callback);
                        }, "name", { value: "GetTable" });
    
                        /**
                         * Calls GetTable.
                         * @function getTable
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IGetTableRequest} request GetTableRequest message or plain object
                         * @returns {Promise<google.bigtable.admin.v2.Table>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|updateTable}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef UpdateTableCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.longrunning.Operation} [response] Operation
                         */
    
                        /**
                         * Calls UpdateTable.
                         * @function updateTable
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IUpdateTableRequest} request UpdateTableRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.UpdateTableCallback} callback Node-style callback called with the error, if any, and Operation
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.updateTable = function updateTable(request, callback) {
                            return this.rpcCall(updateTable, $root.google.bigtable.admin.v2.UpdateTableRequest, $root.google.longrunning.Operation, request, callback);
                        }, "name", { value: "UpdateTable" });
    
                        /**
                         * Calls UpdateTable.
                         * @function updateTable
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IUpdateTableRequest} request UpdateTableRequest message or plain object
                         * @returns {Promise<google.longrunning.Operation>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|deleteTable}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef DeleteTableCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.protobuf.Empty} [response] Empty
                         */
    
                        /**
                         * Calls DeleteTable.
                         * @function deleteTable
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IDeleteTableRequest} request DeleteTableRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.DeleteTableCallback} callback Node-style callback called with the error, if any, and Empty
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.deleteTable = function deleteTable(request, callback) {
                            return this.rpcCall(deleteTable, $root.google.bigtable.admin.v2.DeleteTableRequest, $root.google.protobuf.Empty, request, callback);
                        }, "name", { value: "DeleteTable" });
    
                        /**
                         * Calls DeleteTable.
                         * @function deleteTable
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IDeleteTableRequest} request DeleteTableRequest message or plain object
                         * @returns {Promise<google.protobuf.Empty>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|undeleteTable}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef UndeleteTableCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.longrunning.Operation} [response] Operation
                         */
    
                        /**
                         * Calls UndeleteTable.
                         * @function undeleteTable
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IUndeleteTableRequest} request UndeleteTableRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.UndeleteTableCallback} callback Node-style callback called with the error, if any, and Operation
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.undeleteTable = function undeleteTable(request, callback) {
                            return this.rpcCall(undeleteTable, $root.google.bigtable.admin.v2.UndeleteTableRequest, $root.google.longrunning.Operation, request, callback);
                        }, "name", { value: "UndeleteTable" });
    
                        /**
                         * Calls UndeleteTable.
                         * @function undeleteTable
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IUndeleteTableRequest} request UndeleteTableRequest message or plain object
                         * @returns {Promise<google.longrunning.Operation>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|createAuthorizedView}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef CreateAuthorizedViewCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.longrunning.Operation} [response] Operation
                         */
    
                        /**
                         * Calls CreateAuthorizedView.
                         * @function createAuthorizedView
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.ICreateAuthorizedViewRequest} request CreateAuthorizedViewRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.CreateAuthorizedViewCallback} callback Node-style callback called with the error, if any, and Operation
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.createAuthorizedView = function createAuthorizedView(request, callback) {
                            return this.rpcCall(createAuthorizedView, $root.google.bigtable.admin.v2.CreateAuthorizedViewRequest, $root.google.longrunning.Operation, request, callback);
                        }, "name", { value: "CreateAuthorizedView" });
    
                        /**
                         * Calls CreateAuthorizedView.
                         * @function createAuthorizedView
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.ICreateAuthorizedViewRequest} request CreateAuthorizedViewRequest message or plain object
                         * @returns {Promise<google.longrunning.Operation>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|listAuthorizedViews}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef ListAuthorizedViewsCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.bigtable.admin.v2.ListAuthorizedViewsResponse} [response] ListAuthorizedViewsResponse
                         */
    
                        /**
                         * Calls ListAuthorizedViews.
                         * @function listAuthorizedViews
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IListAuthorizedViewsRequest} request ListAuthorizedViewsRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.ListAuthorizedViewsCallback} callback Node-style callback called with the error, if any, and ListAuthorizedViewsResponse
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.listAuthorizedViews = function listAuthorizedViews(request, callback) {
                            return this.rpcCall(listAuthorizedViews, $root.google.bigtable.admin.v2.ListAuthorizedViewsRequest, $root.google.bigtable.admin.v2.ListAuthorizedViewsResponse, request, callback);
                        }, "name", { value: "ListAuthorizedViews" });
    
                        /**
                         * Calls ListAuthorizedViews.
                         * @function listAuthorizedViews
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IListAuthorizedViewsRequest} request ListAuthorizedViewsRequest message or plain object
                         * @returns {Promise<google.bigtable.admin.v2.ListAuthorizedViewsResponse>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|getAuthorizedView}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef GetAuthorizedViewCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.bigtable.admin.v2.AuthorizedView} [response] AuthorizedView
                         */
    
                        /**
                         * Calls GetAuthorizedView.
                         * @function getAuthorizedView
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IGetAuthorizedViewRequest} request GetAuthorizedViewRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.GetAuthorizedViewCallback} callback Node-style callback called with the error, if any, and AuthorizedView
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.getAuthorizedView = function getAuthorizedView(request, callback) {
                            return this.rpcCall(getAuthorizedView, $root.google.bigtable.admin.v2.GetAuthorizedViewRequest, $root.google.bigtable.admin.v2.AuthorizedView, request, callback);
                        }, "name", { value: "GetAuthorizedView" });
    
                        /**
                         * Calls GetAuthorizedView.
                         * @function getAuthorizedView
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IGetAuthorizedViewRequest} request GetAuthorizedViewRequest message or plain object
                         * @returns {Promise<google.bigtable.admin.v2.AuthorizedView>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|updateAuthorizedView}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef UpdateAuthorizedViewCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.longrunning.Operation} [response] Operation
                         */
    
                        /**
                         * Calls UpdateAuthorizedView.
                         * @function updateAuthorizedView
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IUpdateAuthorizedViewRequest} request UpdateAuthorizedViewRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.UpdateAuthorizedViewCallback} callback Node-style callback called with the error, if any, and Operation
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.updateAuthorizedView = function updateAuthorizedView(request, callback) {
                            return this.rpcCall(updateAuthorizedView, $root.google.bigtable.admin.v2.UpdateAuthorizedViewRequest, $root.google.longrunning.Operation, request, callback);
                        }, "name", { value: "UpdateAuthorizedView" });
    
                        /**
                         * Calls UpdateAuthorizedView.
                         * @function updateAuthorizedView
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IUpdateAuthorizedViewRequest} request UpdateAuthorizedViewRequest message or plain object
                         * @returns {Promise<google.longrunning.Operation>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|deleteAuthorizedView}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef DeleteAuthorizedViewCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.protobuf.Empty} [response] Empty
                         */
    
                        /**
                         * Calls DeleteAuthorizedView.
                         * @function deleteAuthorizedView
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IDeleteAuthorizedViewRequest} request DeleteAuthorizedViewRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.DeleteAuthorizedViewCallback} callback Node-style callback called with the error, if any, and Empty
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.deleteAuthorizedView = function deleteAuthorizedView(request, callback) {
                            return this.rpcCall(deleteAuthorizedView, $root.google.bigtable.admin.v2.DeleteAuthorizedViewRequest, $root.google.protobuf.Empty, request, callback);
                        }, "name", { value: "DeleteAuthorizedView" });
    
                        /**
                         * Calls DeleteAuthorizedView.
                         * @function deleteAuthorizedView
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IDeleteAuthorizedViewRequest} request DeleteAuthorizedViewRequest message or plain object
                         * @returns {Promise<google.protobuf.Empty>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|modifyColumnFamilies}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef ModifyColumnFamiliesCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.bigtable.admin.v2.Table} [response] Table
                         */
    
                        /**
                         * Calls ModifyColumnFamilies.
                         * @function modifyColumnFamilies
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IModifyColumnFamiliesRequest} request ModifyColumnFamiliesRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.ModifyColumnFamiliesCallback} callback Node-style callback called with the error, if any, and Table
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.modifyColumnFamilies = function modifyColumnFamilies(request, callback) {
                            return this.rpcCall(modifyColumnFamilies, $root.google.bigtable.admin.v2.ModifyColumnFamiliesRequest, $root.google.bigtable.admin.v2.Table, request, callback);
                        }, "name", { value: "ModifyColumnFamilies" });
    
                        /**
                         * Calls ModifyColumnFamilies.
                         * @function modifyColumnFamilies
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IModifyColumnFamiliesRequest} request ModifyColumnFamiliesRequest message or plain object
                         * @returns {Promise<google.bigtable.admin.v2.Table>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|dropRowRange}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef DropRowRangeCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.protobuf.Empty} [response] Empty
                         */
    
                        /**
                         * Calls DropRowRange.
                         * @function dropRowRange
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IDropRowRangeRequest} request DropRowRangeRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.DropRowRangeCallback} callback Node-style callback called with the error, if any, and Empty
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.dropRowRange = function dropRowRange(request, callback) {
                            return this.rpcCall(dropRowRange, $root.google.bigtable.admin.v2.DropRowRangeRequest, $root.google.protobuf.Empty, request, callback);
                        }, "name", { value: "DropRowRange" });
    
                        /**
                         * Calls DropRowRange.
                         * @function dropRowRange
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IDropRowRangeRequest} request DropRowRangeRequest message or plain object
                         * @returns {Promise<google.protobuf.Empty>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|generateConsistencyToken}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef GenerateConsistencyTokenCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.bigtable.admin.v2.GenerateConsistencyTokenResponse} [response] GenerateConsistencyTokenResponse
                         */
    
                        /**
                         * Calls GenerateConsistencyToken.
                         * @function generateConsistencyToken
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IGenerateConsistencyTokenRequest} request GenerateConsistencyTokenRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.GenerateConsistencyTokenCallback} callback Node-style callback called with the error, if any, and GenerateConsistencyTokenResponse
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.generateConsistencyToken = function generateConsistencyToken(request, callback) {
                            return this.rpcCall(generateConsistencyToken, $root.google.bigtable.admin.v2.GenerateConsistencyTokenRequest, $root.google.bigtable.admin.v2.GenerateConsistencyTokenResponse, request, callback);
                        }, "name", { value: "GenerateConsistencyToken" });
    
                        /**
                         * Calls GenerateConsistencyToken.
                         * @function generateConsistencyToken
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IGenerateConsistencyTokenRequest} request GenerateConsistencyTokenRequest message or plain object
                         * @returns {Promise<google.bigtable.admin.v2.GenerateConsistencyTokenResponse>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|checkConsistency}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef CheckConsistencyCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.bigtable.admin.v2.CheckConsistencyResponse} [response] CheckConsistencyResponse
                         */
    
                        /**
                         * Calls CheckConsistency.
                         * @function checkConsistency
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.ICheckConsistencyRequest} request CheckConsistencyRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.CheckConsistencyCallback} callback Node-style callback called with the error, if any, and CheckConsistencyResponse
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.checkConsistency = function checkConsistency(request, callback) {
                            return this.rpcCall(checkConsistency, $root.google.bigtable.admin.v2.CheckConsistencyRequest, $root.google.bigtable.admin.v2.CheckConsistencyResponse, request, callback);
                        }, "name", { value: "CheckConsistency" });
    
                        /**
                         * Calls CheckConsistency.
                         * @function checkConsistency
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.ICheckConsistencyRequest} request CheckConsistencyRequest message or plain object
                         * @returns {Promise<google.bigtable.admin.v2.CheckConsistencyResponse>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|snapshotTable}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef SnapshotTableCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.longrunning.Operation} [response] Operation
                         */
    
                        /**
                         * Calls SnapshotTable.
                         * @function snapshotTable
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.ISnapshotTableRequest} request SnapshotTableRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.SnapshotTableCallback} callback Node-style callback called with the error, if any, and Operation
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.snapshotTable = function snapshotTable(request, callback) {
                            return this.rpcCall(snapshotTable, $root.google.bigtable.admin.v2.SnapshotTableRequest, $root.google.longrunning.Operation, request, callback);
                        }, "name", { value: "SnapshotTable" });
    
                        /**
                         * Calls SnapshotTable.
                         * @function snapshotTable
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.ISnapshotTableRequest} request SnapshotTableRequest message or plain object
                         * @returns {Promise<google.longrunning.Operation>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|getSnapshot}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef GetSnapshotCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.bigtable.admin.v2.Snapshot} [response] Snapshot
                         */
    
                        /**
                         * Calls GetSnapshot.
                         * @function getSnapshot
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IGetSnapshotRequest} request GetSnapshotRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.GetSnapshotCallback} callback Node-style callback called with the error, if any, and Snapshot
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.getSnapshot = function getSnapshot(request, callback) {
                            return this.rpcCall(getSnapshot, $root.google.bigtable.admin.v2.GetSnapshotRequest, $root.google.bigtable.admin.v2.Snapshot, request, callback);
                        }, "name", { value: "GetSnapshot" });
    
                        /**
                         * Calls GetSnapshot.
                         * @function getSnapshot
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IGetSnapshotRequest} request GetSnapshotRequest message or plain object
                         * @returns {Promise<google.bigtable.admin.v2.Snapshot>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|listSnapshots}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef ListSnapshotsCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.bigtable.admin.v2.ListSnapshotsResponse} [response] ListSnapshotsResponse
                         */
    
                        /**
                         * Calls ListSnapshots.
                         * @function listSnapshots
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IListSnapshotsRequest} request ListSnapshotsRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.ListSnapshotsCallback} callback Node-style callback called with the error, if any, and ListSnapshotsResponse
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.listSnapshots = function listSnapshots(request, callback) {
                            return this.rpcCall(listSnapshots, $root.google.bigtable.admin.v2.ListSnapshotsRequest, $root.google.bigtable.admin.v2.ListSnapshotsResponse, request, callback);
                        }, "name", { value: "ListSnapshots" });
    
                        /**
                         * Calls ListSnapshots.
                         * @function listSnapshots
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IListSnapshotsRequest} request ListSnapshotsRequest message or plain object
                         * @returns {Promise<google.bigtable.admin.v2.ListSnapshotsResponse>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|deleteSnapshot}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef DeleteSnapshotCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.protobuf.Empty} [response] Empty
                         */
    
                        /**
                         * Calls DeleteSnapshot.
                         * @function deleteSnapshot
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IDeleteSnapshotRequest} request DeleteSnapshotRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.DeleteSnapshotCallback} callback Node-style callback called with the error, if any, and Empty
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.deleteSnapshot = function deleteSnapshot(request, callback) {
                            return this.rpcCall(deleteSnapshot, $root.google.bigtable.admin.v2.DeleteSnapshotRequest, $root.google.protobuf.Empty, request, callback);
                        }, "name", { value: "DeleteSnapshot" });
    
                        /**
                         * Calls DeleteSnapshot.
                         * @function deleteSnapshot
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IDeleteSnapshotRequest} request DeleteSnapshotRequest message or plain object
                         * @returns {Promise<google.protobuf.Empty>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|createBackup}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef CreateBackupCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.longrunning.Operation} [response] Operation
                         */
    
                        /**
                         * Calls CreateBackup.
                         * @function createBackup
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.ICreateBackupRequest} request CreateBackupRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.CreateBackupCallback} callback Node-style callback called with the error, if any, and Operation
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.createBackup = function createBackup(request, callback) {
                            return this.rpcCall(createBackup, $root.google.bigtable.admin.v2.CreateBackupRequest, $root.google.longrunning.Operation, request, callback);
                        }, "name", { value: "CreateBackup" });
    
                        /**
                         * Calls CreateBackup.
                         * @function createBackup
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.ICreateBackupRequest} request CreateBackupRequest message or plain object
                         * @returns {Promise<google.longrunning.Operation>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|getBackup}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef GetBackupCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.bigtable.admin.v2.Backup} [response] Backup
                         */
    
                        /**
                         * Calls GetBackup.
                         * @function getBackup
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IGetBackupRequest} request GetBackupRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.GetBackupCallback} callback Node-style callback called with the error, if any, and Backup
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.getBackup = function getBackup(request, callback) {
                            return this.rpcCall(getBackup, $root.google.bigtable.admin.v2.GetBackupRequest, $root.google.bigtable.admin.v2.Backup, request, callback);
                        }, "name", { value: "GetBackup" });
    
                        /**
                         * Calls GetBackup.
                         * @function getBackup
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IGetBackupRequest} request GetBackupRequest message or plain object
                         * @returns {Promise<google.bigtable.admin.v2.Backup>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|updateBackup}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef UpdateBackupCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.bigtable.admin.v2.Backup} [response] Backup
                         */
    
                        /**
                         * Calls UpdateBackup.
                         * @function updateBackup
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IUpdateBackupRequest} request UpdateBackupRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.UpdateBackupCallback} callback Node-style callback called with the error, if any, and Backup
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.updateBackup = function updateBackup(request, callback) {
                            return this.rpcCall(updateBackup, $root.google.bigtable.admin.v2.UpdateBackupRequest, $root.google.bigtable.admin.v2.Backup, request, callback);
                        }, "name", { value: "UpdateBackup" });
    
                        /**
                         * Calls UpdateBackup.
                         * @function updateBackup
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IUpdateBackupRequest} request UpdateBackupRequest message or plain object
                         * @returns {Promise<google.bigtable.admin.v2.Backup>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|deleteBackup}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef DeleteBackupCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.protobuf.Empty} [response] Empty
                         */
    
                        /**
                         * Calls DeleteBackup.
                         * @function deleteBackup
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IDeleteBackupRequest} request DeleteBackupRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.DeleteBackupCallback} callback Node-style callback called with the error, if any, and Empty
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.deleteBackup = function deleteBackup(request, callback) {
                            return this.rpcCall(deleteBackup, $root.google.bigtable.admin.v2.DeleteBackupRequest, $root.google.protobuf.Empty, request, callback);
                        }, "name", { value: "DeleteBackup" });
    
                        /**
                         * Calls DeleteBackup.
                         * @function deleteBackup
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IDeleteBackupRequest} request DeleteBackupRequest message or plain object
                         * @returns {Promise<google.protobuf.Empty>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|listBackups}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef ListBackupsCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.bigtable.admin.v2.ListBackupsResponse} [response] ListBackupsResponse
                         */
    
                        /**
                         * Calls ListBackups.
                         * @function listBackups
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IListBackupsRequest} request ListBackupsRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.ListBackupsCallback} callback Node-style callback called with the error, if any, and ListBackupsResponse
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.listBackups = function listBackups(request, callback) {
                            return this.rpcCall(listBackups, $root.google.bigtable.admin.v2.ListBackupsRequest, $root.google.bigtable.admin.v2.ListBackupsResponse, request, callback);
                        }, "name", { value: "ListBackups" });
    
                        /**
                         * Calls ListBackups.
                         * @function listBackups
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IListBackupsRequest} request ListBackupsRequest message or plain object
                         * @returns {Promise<google.bigtable.admin.v2.ListBackupsResponse>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|restoreTable}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef RestoreTableCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.longrunning.Operation} [response] Operation
                         */
    
                        /**
                         * Calls RestoreTable.
                         * @function restoreTable
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IRestoreTableRequest} request RestoreTableRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.RestoreTableCallback} callback Node-style callback called with the error, if any, and Operation
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.restoreTable = function restoreTable(request, callback) {
                            return this.rpcCall(restoreTable, $root.google.bigtable.admin.v2.RestoreTableRequest, $root.google.longrunning.Operation, request, callback);
                        }, "name", { value: "RestoreTable" });
    
                        /**
                         * Calls RestoreTable.
                         * @function restoreTable
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.IRestoreTableRequest} request RestoreTableRequest message or plain object
                         * @returns {Promise<google.longrunning.Operation>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|copyBackup}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef CopyBackupCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.longrunning.Operation} [response] Operation
                         */
    
                        /**
                         * Calls CopyBackup.
                         * @function copyBackup
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.ICopyBackupRequest} request CopyBackupRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.CopyBackupCallback} callback Node-style callback called with the error, if any, and Operation
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.copyBackup = function copyBackup(request, callback) {
                            return this.rpcCall(copyBackup, $root.google.bigtable.admin.v2.CopyBackupRequest, $root.google.longrunning.Operation, request, callback);
                        }, "name", { value: "CopyBackup" });
    
                        /**
                         * Calls CopyBackup.
                         * @function copyBackup
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.bigtable.admin.v2.ICopyBackupRequest} request CopyBackupRequest message or plain object
                         * @returns {Promise<google.longrunning.Operation>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|getIamPolicy}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef GetIamPolicyCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.iam.v1.Policy} [response] Policy
                         */
    
                        /**
                         * Calls GetIamPolicy.
                         * @function getIamPolicy
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.iam.v1.IGetIamPolicyRequest} request GetIamPolicyRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.GetIamPolicyCallback} callback Node-style callback called with the error, if any, and Policy
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.getIamPolicy = function getIamPolicy(request, callback) {
                            return this.rpcCall(getIamPolicy, $root.google.iam.v1.GetIamPolicyRequest, $root.google.iam.v1.Policy, request, callback);
                        }, "name", { value: "GetIamPolicy" });
    
                        /**
                         * Calls GetIamPolicy.
                         * @function getIamPolicy
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.iam.v1.IGetIamPolicyRequest} request GetIamPolicyRequest message or plain object
                         * @returns {Promise<google.iam.v1.Policy>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|setIamPolicy}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef SetIamPolicyCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.iam.v1.Policy} [response] Policy
                         */
    
                        /**
                         * Calls SetIamPolicy.
                         * @function setIamPolicy
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.iam.v1.ISetIamPolicyRequest} request SetIamPolicyRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.SetIamPolicyCallback} callback Node-style callback called with the error, if any, and Policy
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.setIamPolicy = function setIamPolicy(request, callback) {
                            return this.rpcCall(setIamPolicy, $root.google.iam.v1.SetIamPolicyRequest, $root.google.iam.v1.Policy, request, callback);
                        }, "name", { value: "SetIamPolicy" });
    
                        /**
                         * Calls SetIamPolicy.
                         * @function setIamPolicy
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.iam.v1.ISetIamPolicyRequest} request SetIamPolicyRequest message or plain object
                         * @returns {Promise<google.iam.v1.Policy>} Promise
                         * @variation 2
                         */
    
                        /**
                         * Callback as used by {@link google.bigtable.admin.v2.BigtableTableAdmin|testIamPermissions}.
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @typedef TestIamPermissionsCallback
                         * @type {function}
                         * @param {Error|null} error Error, if any
                         * @param {google.iam.v1.TestIamPermissionsResponse} [response] TestIamPermissionsResponse
                         */
    
                        /**
                         * Calls TestIamPermissions.
                         * @function testIamPermissions
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.iam.v1.ITestIamPermissionsRequest} request TestIamPermissionsRequest message or plain object
                         * @param {google.bigtable.admin.v2.BigtableTableAdmin.TestIamPermissionsCallback} callback Node-style callback called with the error, if any, and TestIamPermissionsResponse
                         * @returns {undefined}
                         * @variation 1
                         */
                        Object.defineProperty(BigtableTableAdmin.prototype.testIamPermissions = function testIamPermissions(request, callback) {
                            return this.rpcCall(testIamPermissions, $root.google.iam.v1.TestIamPermissionsRequest, $root.google.iam.v1.TestIamPermissionsResponse, request, callback);
                        }, "name", { value: "TestIamPermissions" });
    
                        /**
                         * Calls TestIamPermissions.
                         * @function testIamPermissions
                         * @memberof google.bigtable.admin.v2.BigtableTableAdmin
                         * @instance
                         * @param {google.iam.v1.ITestIamPermissionsRequest} request TestIamPermissionsRequest message or plain object
                         * @returns {Promise<google.iam.v1.TestIamPermissionsResponse>} Promise
                         * @variation 2
                         */
    
                        return BigtableTableAdmin;
                    })();
    
                    v2.RestoreTableRequest = (function() {
    
                        /**
                         * Properties of a RestoreTableRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IRestoreTableRequest
                         * @property {string|null} [parent] RestoreTableRequest parent
                         * @property {string|null} [tableId] RestoreTableRequest tableId
                         * @property {string|null} [backup] RestoreTableRequest backup
                         */
    
                        /**
                         * Constructs a new RestoreTableRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a RestoreTableRequest.
                         * @implements IRestoreTableRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IRestoreTableRequest=} [properties] Properties to set
                         */
                        function RestoreTableRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * RestoreTableRequest parent.
                         * @member {string} parent
                         * @memberof google.bigtable.admin.v2.RestoreTableRequest
                         * @instance
                         */
                        RestoreTableRequest.prototype.parent = "";
    
                        /**
                         * RestoreTableRequest tableId.
                         * @member {string} tableId
                         * @memberof google.bigtable.admin.v2.RestoreTableRequest
                         * @instance
                         */
                        RestoreTableRequest.prototype.tableId = "";
    
                        /**
                         * RestoreTableRequest backup.
                         * @member {string|null|undefined} backup
                         * @memberof google.bigtable.admin.v2.RestoreTableRequest
                         * @instance
                         */
                        RestoreTableRequest.prototype.backup = null;
    
                        // OneOf field names bound to virtual getters and setters
                        var $oneOfFields;
    
                        /**
                         * RestoreTableRequest source.
                         * @member {"backup"|undefined} source
                         * @memberof google.bigtable.admin.v2.RestoreTableRequest
                         * @instance
                         */
                        Object.defineProperty(RestoreTableRequest.prototype, "source", {
                            get: $util.oneOfGetter($oneOfFields = ["backup"]),
                            set: $util.oneOfSetter($oneOfFields)
                        });
    
                        /**
                         * Creates a new RestoreTableRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.RestoreTableRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IRestoreTableRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.RestoreTableRequest} RestoreTableRequest instance
                         */
                        RestoreTableRequest.create = function create(properties) {
                            return new RestoreTableRequest(properties);
                        };
    
                        /**
                         * Encodes the specified RestoreTableRequest message. Does not implicitly {@link google.bigtable.admin.v2.RestoreTableRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.RestoreTableRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IRestoreTableRequest} message RestoreTableRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        RestoreTableRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.parent != null && Object.hasOwnProperty.call(message, "parent"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.parent);
                            if (message.tableId != null && Object.hasOwnProperty.call(message, "tableId"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.tableId);
                            if (message.backup != null && Object.hasOwnProperty.call(message, "backup"))
                                writer.uint32(/* id 3, wireType 2 =*/26).string(message.backup);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified RestoreTableRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.RestoreTableRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.RestoreTableRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IRestoreTableRequest} message RestoreTableRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        RestoreTableRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a RestoreTableRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.RestoreTableRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.RestoreTableRequest} RestoreTableRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        RestoreTableRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.RestoreTableRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.parent = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.tableId = reader.string();
                                        break;
                                    }
                                case 3: {
                                        message.backup = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a RestoreTableRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.RestoreTableRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.RestoreTableRequest} RestoreTableRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        RestoreTableRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a RestoreTableRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.RestoreTableRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        RestoreTableRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            var properties = {};
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                if (!$util.isString(message.parent))
                                    return "parent: string expected";
                            if (message.tableId != null && message.hasOwnProperty("tableId"))
                                if (!$util.isString(message.tableId))
                                    return "tableId: string expected";
                            if (message.backup != null && message.hasOwnProperty("backup")) {
                                properties.source = 1;
                                if (!$util.isString(message.backup))
                                    return "backup: string expected";
                            }
                            return null;
                        };
    
                        /**
                         * Creates a RestoreTableRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.RestoreTableRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.RestoreTableRequest} RestoreTableRequest
                         */
                        RestoreTableRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.RestoreTableRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.RestoreTableRequest();
                            if (object.parent != null)
                                message.parent = String(object.parent);
                            if (object.tableId != null)
                                message.tableId = String(object.tableId);
                            if (object.backup != null)
                                message.backup = String(object.backup);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a RestoreTableRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.RestoreTableRequest
                         * @static
                         * @param {google.bigtable.admin.v2.RestoreTableRequest} message RestoreTableRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        RestoreTableRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.parent = "";
                                object.tableId = "";
                            }
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                object.parent = message.parent;
                            if (message.tableId != null && message.hasOwnProperty("tableId"))
                                object.tableId = message.tableId;
                            if (message.backup != null && message.hasOwnProperty("backup")) {
                                object.backup = message.backup;
                                if (options.oneofs)
                                    object.source = "backup";
                            }
                            return object;
                        };
    
                        /**
                         * Converts this RestoreTableRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.RestoreTableRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        RestoreTableRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for RestoreTableRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.RestoreTableRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        RestoreTableRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.RestoreTableRequest";
                        };
    
                        return RestoreTableRequest;
                    })();
    
                    v2.RestoreTableMetadata = (function() {
    
                        /**
                         * Properties of a RestoreTableMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @interface IRestoreTableMetadata
                         * @property {string|null} [name] RestoreTableMetadata name
                         * @property {google.bigtable.admin.v2.RestoreSourceType|null} [sourceType] RestoreTableMetadata sourceType
                         * @property {google.bigtable.admin.v2.IBackupInfo|null} [backupInfo] RestoreTableMetadata backupInfo
                         * @property {string|null} [optimizeTableOperationName] RestoreTableMetadata optimizeTableOperationName
                         * @property {google.bigtable.admin.v2.IOperationProgress|null} [progress] RestoreTableMetadata progress
                         */
    
                        /**
                         * Constructs a new RestoreTableMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a RestoreTableMetadata.
                         * @implements IRestoreTableMetadata
                         * @constructor
                         * @param {google.bigtable.admin.v2.IRestoreTableMetadata=} [properties] Properties to set
                         */
                        function RestoreTableMetadata(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * RestoreTableMetadata name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.RestoreTableMetadata
                         * @instance
                         */
                        RestoreTableMetadata.prototype.name = "";
    
                        /**
                         * RestoreTableMetadata sourceType.
                         * @member {google.bigtable.admin.v2.RestoreSourceType} sourceType
                         * @memberof google.bigtable.admin.v2.RestoreTableMetadata
                         * @instance
                         */
                        RestoreTableMetadata.prototype.sourceType = 0;
    
                        /**
                         * RestoreTableMetadata backupInfo.
                         * @member {google.bigtable.admin.v2.IBackupInfo|null|undefined} backupInfo
                         * @memberof google.bigtable.admin.v2.RestoreTableMetadata
                         * @instance
                         */
                        RestoreTableMetadata.prototype.backupInfo = null;
    
                        /**
                         * RestoreTableMetadata optimizeTableOperationName.
                         * @member {string} optimizeTableOperationName
                         * @memberof google.bigtable.admin.v2.RestoreTableMetadata
                         * @instance
                         */
                        RestoreTableMetadata.prototype.optimizeTableOperationName = "";
    
                        /**
                         * RestoreTableMetadata progress.
                         * @member {google.bigtable.admin.v2.IOperationProgress|null|undefined} progress
                         * @memberof google.bigtable.admin.v2.RestoreTableMetadata
                         * @instance
                         */
                        RestoreTableMetadata.prototype.progress = null;
    
                        // OneOf field names bound to virtual getters and setters
                        var $oneOfFields;
    
                        /**
                         * RestoreTableMetadata sourceInfo.
                         * @member {"backupInfo"|undefined} sourceInfo
                         * @memberof google.bigtable.admin.v2.RestoreTableMetadata
                         * @instance
                         */
                        Object.defineProperty(RestoreTableMetadata.prototype, "sourceInfo", {
                            get: $util.oneOfGetter($oneOfFields = ["backupInfo"]),
                            set: $util.oneOfSetter($oneOfFields)
                        });
    
                        /**
                         * Creates a new RestoreTableMetadata instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.RestoreTableMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.IRestoreTableMetadata=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.RestoreTableMetadata} RestoreTableMetadata instance
                         */
                        RestoreTableMetadata.create = function create(properties) {
                            return new RestoreTableMetadata(properties);
                        };
    
                        /**
                         * Encodes the specified RestoreTableMetadata message. Does not implicitly {@link google.bigtable.admin.v2.RestoreTableMetadata.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.RestoreTableMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.IRestoreTableMetadata} message RestoreTableMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        RestoreTableMetadata.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            if (message.sourceType != null && Object.hasOwnProperty.call(message, "sourceType"))
                                writer.uint32(/* id 2, wireType 0 =*/16).int32(message.sourceType);
                            if (message.backupInfo != null && Object.hasOwnProperty.call(message, "backupInfo"))
                                $root.google.bigtable.admin.v2.BackupInfo.encode(message.backupInfo, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            if (message.optimizeTableOperationName != null && Object.hasOwnProperty.call(message, "optimizeTableOperationName"))
                                writer.uint32(/* id 4, wireType 2 =*/34).string(message.optimizeTableOperationName);
                            if (message.progress != null && Object.hasOwnProperty.call(message, "progress"))
                                $root.google.bigtable.admin.v2.OperationProgress.encode(message.progress, writer.uint32(/* id 5, wireType 2 =*/42).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified RestoreTableMetadata message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.RestoreTableMetadata.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.RestoreTableMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.IRestoreTableMetadata} message RestoreTableMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        RestoreTableMetadata.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a RestoreTableMetadata message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.RestoreTableMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.RestoreTableMetadata} RestoreTableMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        RestoreTableMetadata.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.RestoreTableMetadata();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.sourceType = reader.int32();
                                        break;
                                    }
                                case 3: {
                                        message.backupInfo = $root.google.bigtable.admin.v2.BackupInfo.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 4: {
                                        message.optimizeTableOperationName = reader.string();
                                        break;
                                    }
                                case 5: {
                                        message.progress = $root.google.bigtable.admin.v2.OperationProgress.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a RestoreTableMetadata message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.RestoreTableMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.RestoreTableMetadata} RestoreTableMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        RestoreTableMetadata.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a RestoreTableMetadata message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.RestoreTableMetadata
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        RestoreTableMetadata.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            var properties = {};
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            if (message.sourceType != null && message.hasOwnProperty("sourceType"))
                                switch (message.sourceType) {
                                default:
                                    return "sourceType: enum value expected";
                                case 0:
                                case 1:
                                    break;
                                }
                            if (message.backupInfo != null && message.hasOwnProperty("backupInfo")) {
                                properties.sourceInfo = 1;
                                {
                                    var error = $root.google.bigtable.admin.v2.BackupInfo.verify(message.backupInfo);
                                    if (error)
                                        return "backupInfo." + error;
                                }
                            }
                            if (message.optimizeTableOperationName != null && message.hasOwnProperty("optimizeTableOperationName"))
                                if (!$util.isString(message.optimizeTableOperationName))
                                    return "optimizeTableOperationName: string expected";
                            if (message.progress != null && message.hasOwnProperty("progress")) {
                                var error = $root.google.bigtable.admin.v2.OperationProgress.verify(message.progress);
                                if (error)
                                    return "progress." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates a RestoreTableMetadata message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.RestoreTableMetadata
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.RestoreTableMetadata} RestoreTableMetadata
                         */
                        RestoreTableMetadata.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.RestoreTableMetadata)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.RestoreTableMetadata();
                            if (object.name != null)
                                message.name = String(object.name);
                            switch (object.sourceType) {
                            default:
                                if (typeof object.sourceType === "number") {
                                    message.sourceType = object.sourceType;
                                    break;
                                }
                                break;
                            case "RESTORE_SOURCE_TYPE_UNSPECIFIED":
                            case 0:
                                message.sourceType = 0;
                                break;
                            case "BACKUP":
                            case 1:
                                message.sourceType = 1;
                                break;
                            }
                            if (object.backupInfo != null) {
                                if (typeof object.backupInfo !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.RestoreTableMetadata.backupInfo: object expected");
                                message.backupInfo = $root.google.bigtable.admin.v2.BackupInfo.fromObject(object.backupInfo);
                            }
                            if (object.optimizeTableOperationName != null)
                                message.optimizeTableOperationName = String(object.optimizeTableOperationName);
                            if (object.progress != null) {
                                if (typeof object.progress !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.RestoreTableMetadata.progress: object expected");
                                message.progress = $root.google.bigtable.admin.v2.OperationProgress.fromObject(object.progress);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a RestoreTableMetadata message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.RestoreTableMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.RestoreTableMetadata} message RestoreTableMetadata
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        RestoreTableMetadata.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.name = "";
                                object.sourceType = options.enums === String ? "RESTORE_SOURCE_TYPE_UNSPECIFIED" : 0;
                                object.optimizeTableOperationName = "";
                                object.progress = null;
                            }
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            if (message.sourceType != null && message.hasOwnProperty("sourceType"))
                                object.sourceType = options.enums === String ? $root.google.bigtable.admin.v2.RestoreSourceType[message.sourceType] === undefined ? message.sourceType : $root.google.bigtable.admin.v2.RestoreSourceType[message.sourceType] : message.sourceType;
                            if (message.backupInfo != null && message.hasOwnProperty("backupInfo")) {
                                object.backupInfo = $root.google.bigtable.admin.v2.BackupInfo.toObject(message.backupInfo, options);
                                if (options.oneofs)
                                    object.sourceInfo = "backupInfo";
                            }
                            if (message.optimizeTableOperationName != null && message.hasOwnProperty("optimizeTableOperationName"))
                                object.optimizeTableOperationName = message.optimizeTableOperationName;
                            if (message.progress != null && message.hasOwnProperty("progress"))
                                object.progress = $root.google.bigtable.admin.v2.OperationProgress.toObject(message.progress, options);
                            return object;
                        };
    
                        /**
                         * Converts this RestoreTableMetadata to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.RestoreTableMetadata
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        RestoreTableMetadata.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for RestoreTableMetadata
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.RestoreTableMetadata
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        RestoreTableMetadata.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.RestoreTableMetadata";
                        };
    
                        return RestoreTableMetadata;
                    })();
    
                    v2.OptimizeRestoredTableMetadata = (function() {
    
                        /**
                         * Properties of an OptimizeRestoredTableMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @interface IOptimizeRestoredTableMetadata
                         * @property {string|null} [name] OptimizeRestoredTableMetadata name
                         * @property {google.bigtable.admin.v2.IOperationProgress|null} [progress] OptimizeRestoredTableMetadata progress
                         */
    
                        /**
                         * Constructs a new OptimizeRestoredTableMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents an OptimizeRestoredTableMetadata.
                         * @implements IOptimizeRestoredTableMetadata
                         * @constructor
                         * @param {google.bigtable.admin.v2.IOptimizeRestoredTableMetadata=} [properties] Properties to set
                         */
                        function OptimizeRestoredTableMetadata(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * OptimizeRestoredTableMetadata name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.OptimizeRestoredTableMetadata
                         * @instance
                         */
                        OptimizeRestoredTableMetadata.prototype.name = "";
    
                        /**
                         * OptimizeRestoredTableMetadata progress.
                         * @member {google.bigtable.admin.v2.IOperationProgress|null|undefined} progress
                         * @memberof google.bigtable.admin.v2.OptimizeRestoredTableMetadata
                         * @instance
                         */
                        OptimizeRestoredTableMetadata.prototype.progress = null;
    
                        /**
                         * Creates a new OptimizeRestoredTableMetadata instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.OptimizeRestoredTableMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.IOptimizeRestoredTableMetadata=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.OptimizeRestoredTableMetadata} OptimizeRestoredTableMetadata instance
                         */
                        OptimizeRestoredTableMetadata.create = function create(properties) {
                            return new OptimizeRestoredTableMetadata(properties);
                        };
    
                        /**
                         * Encodes the specified OptimizeRestoredTableMetadata message. Does not implicitly {@link google.bigtable.admin.v2.OptimizeRestoredTableMetadata.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.OptimizeRestoredTableMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.IOptimizeRestoredTableMetadata} message OptimizeRestoredTableMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        OptimizeRestoredTableMetadata.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            if (message.progress != null && Object.hasOwnProperty.call(message, "progress"))
                                $root.google.bigtable.admin.v2.OperationProgress.encode(message.progress, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified OptimizeRestoredTableMetadata message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.OptimizeRestoredTableMetadata.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.OptimizeRestoredTableMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.IOptimizeRestoredTableMetadata} message OptimizeRestoredTableMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        OptimizeRestoredTableMetadata.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes an OptimizeRestoredTableMetadata message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.OptimizeRestoredTableMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.OptimizeRestoredTableMetadata} OptimizeRestoredTableMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        OptimizeRestoredTableMetadata.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.OptimizeRestoredTableMetadata();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.progress = $root.google.bigtable.admin.v2.OperationProgress.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes an OptimizeRestoredTableMetadata message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.OptimizeRestoredTableMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.OptimizeRestoredTableMetadata} OptimizeRestoredTableMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        OptimizeRestoredTableMetadata.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies an OptimizeRestoredTableMetadata message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.OptimizeRestoredTableMetadata
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        OptimizeRestoredTableMetadata.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            if (message.progress != null && message.hasOwnProperty("progress")) {
                                var error = $root.google.bigtable.admin.v2.OperationProgress.verify(message.progress);
                                if (error)
                                    return "progress." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates an OptimizeRestoredTableMetadata message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.OptimizeRestoredTableMetadata
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.OptimizeRestoredTableMetadata} OptimizeRestoredTableMetadata
                         */
                        OptimizeRestoredTableMetadata.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.OptimizeRestoredTableMetadata)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.OptimizeRestoredTableMetadata();
                            if (object.name != null)
                                message.name = String(object.name);
                            if (object.progress != null) {
                                if (typeof object.progress !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.OptimizeRestoredTableMetadata.progress: object expected");
                                message.progress = $root.google.bigtable.admin.v2.OperationProgress.fromObject(object.progress);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from an OptimizeRestoredTableMetadata message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.OptimizeRestoredTableMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.OptimizeRestoredTableMetadata} message OptimizeRestoredTableMetadata
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        OptimizeRestoredTableMetadata.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.name = "";
                                object.progress = null;
                            }
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            if (message.progress != null && message.hasOwnProperty("progress"))
                                object.progress = $root.google.bigtable.admin.v2.OperationProgress.toObject(message.progress, options);
                            return object;
                        };
    
                        /**
                         * Converts this OptimizeRestoredTableMetadata to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.OptimizeRestoredTableMetadata
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        OptimizeRestoredTableMetadata.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for OptimizeRestoredTableMetadata
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.OptimizeRestoredTableMetadata
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        OptimizeRestoredTableMetadata.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.OptimizeRestoredTableMetadata";
                        };
    
                        return OptimizeRestoredTableMetadata;
                    })();
    
                    v2.CreateTableRequest = (function() {
    
                        /**
                         * Properties of a CreateTableRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface ICreateTableRequest
                         * @property {string|null} [parent] CreateTableRequest parent
                         * @property {string|null} [tableId] CreateTableRequest tableId
                         * @property {google.bigtable.admin.v2.ITable|null} [table] CreateTableRequest table
                         * @property {Array.<google.bigtable.admin.v2.CreateTableRequest.ISplit>|null} [initialSplits] CreateTableRequest initialSplits
                         */
    
                        /**
                         * Constructs a new CreateTableRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a CreateTableRequest.
                         * @implements ICreateTableRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.ICreateTableRequest=} [properties] Properties to set
                         */
                        function CreateTableRequest(properties) {
                            this.initialSplits = [];
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * CreateTableRequest parent.
                         * @member {string} parent
                         * @memberof google.bigtable.admin.v2.CreateTableRequest
                         * @instance
                         */
                        CreateTableRequest.prototype.parent = "";
    
                        /**
                         * CreateTableRequest tableId.
                         * @member {string} tableId
                         * @memberof google.bigtable.admin.v2.CreateTableRequest
                         * @instance
                         */
                        CreateTableRequest.prototype.tableId = "";
    
                        /**
                         * CreateTableRequest table.
                         * @member {google.bigtable.admin.v2.ITable|null|undefined} table
                         * @memberof google.bigtable.admin.v2.CreateTableRequest
                         * @instance
                         */
                        CreateTableRequest.prototype.table = null;
    
                        /**
                         * CreateTableRequest initialSplits.
                         * @member {Array.<google.bigtable.admin.v2.CreateTableRequest.ISplit>} initialSplits
                         * @memberof google.bigtable.admin.v2.CreateTableRequest
                         * @instance
                         */
                        CreateTableRequest.prototype.initialSplits = $util.emptyArray;
    
                        /**
                         * Creates a new CreateTableRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.CreateTableRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateTableRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.CreateTableRequest} CreateTableRequest instance
                         */
                        CreateTableRequest.create = function create(properties) {
                            return new CreateTableRequest(properties);
                        };
    
                        /**
                         * Encodes the specified CreateTableRequest message. Does not implicitly {@link google.bigtable.admin.v2.CreateTableRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.CreateTableRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateTableRequest} message CreateTableRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CreateTableRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.parent != null && Object.hasOwnProperty.call(message, "parent"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.parent);
                            if (message.tableId != null && Object.hasOwnProperty.call(message, "tableId"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.tableId);
                            if (message.table != null && Object.hasOwnProperty.call(message, "table"))
                                $root.google.bigtable.admin.v2.Table.encode(message.table, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            if (message.initialSplits != null && message.initialSplits.length)
                                for (var i = 0; i < message.initialSplits.length; ++i)
                                    $root.google.bigtable.admin.v2.CreateTableRequest.Split.encode(message.initialSplits[i], writer.uint32(/* id 4, wireType 2 =*/34).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified CreateTableRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.CreateTableRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.CreateTableRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateTableRequest} message CreateTableRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CreateTableRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a CreateTableRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.CreateTableRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.CreateTableRequest} CreateTableRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CreateTableRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.CreateTableRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.parent = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.tableId = reader.string();
                                        break;
                                    }
                                case 3: {
                                        message.table = $root.google.bigtable.admin.v2.Table.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 4: {
                                        if (!(message.initialSplits && message.initialSplits.length))
                                            message.initialSplits = [];
                                        message.initialSplits.push($root.google.bigtable.admin.v2.CreateTableRequest.Split.decode(reader, reader.uint32()));
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a CreateTableRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.CreateTableRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.CreateTableRequest} CreateTableRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CreateTableRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a CreateTableRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.CreateTableRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        CreateTableRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                if (!$util.isString(message.parent))
                                    return "parent: string expected";
                            if (message.tableId != null && message.hasOwnProperty("tableId"))
                                if (!$util.isString(message.tableId))
                                    return "tableId: string expected";
                            if (message.table != null && message.hasOwnProperty("table")) {
                                var error = $root.google.bigtable.admin.v2.Table.verify(message.table);
                                if (error)
                                    return "table." + error;
                            }
                            if (message.initialSplits != null && message.hasOwnProperty("initialSplits")) {
                                if (!Array.isArray(message.initialSplits))
                                    return "initialSplits: array expected";
                                for (var i = 0; i < message.initialSplits.length; ++i) {
                                    var error = $root.google.bigtable.admin.v2.CreateTableRequest.Split.verify(message.initialSplits[i]);
                                    if (error)
                                        return "initialSplits." + error;
                                }
                            }
                            return null;
                        };
    
                        /**
                         * Creates a CreateTableRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.CreateTableRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.CreateTableRequest} CreateTableRequest
                         */
                        CreateTableRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.CreateTableRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.CreateTableRequest();
                            if (object.parent != null)
                                message.parent = String(object.parent);
                            if (object.tableId != null)
                                message.tableId = String(object.tableId);
                            if (object.table != null) {
                                if (typeof object.table !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.CreateTableRequest.table: object expected");
                                message.table = $root.google.bigtable.admin.v2.Table.fromObject(object.table);
                            }
                            if (object.initialSplits) {
                                if (!Array.isArray(object.initialSplits))
                                    throw TypeError(".google.bigtable.admin.v2.CreateTableRequest.initialSplits: array expected");
                                message.initialSplits = [];
                                for (var i = 0; i < object.initialSplits.length; ++i) {
                                    if (typeof object.initialSplits[i] !== "object")
                                        throw TypeError(".google.bigtable.admin.v2.CreateTableRequest.initialSplits: object expected");
                                    message.initialSplits[i] = $root.google.bigtable.admin.v2.CreateTableRequest.Split.fromObject(object.initialSplits[i]);
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a CreateTableRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.CreateTableRequest
                         * @static
                         * @param {google.bigtable.admin.v2.CreateTableRequest} message CreateTableRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        CreateTableRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.arrays || options.defaults)
                                object.initialSplits = [];
                            if (options.defaults) {
                                object.parent = "";
                                object.tableId = "";
                                object.table = null;
                            }
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                object.parent = message.parent;
                            if (message.tableId != null && message.hasOwnProperty("tableId"))
                                object.tableId = message.tableId;
                            if (message.table != null && message.hasOwnProperty("table"))
                                object.table = $root.google.bigtable.admin.v2.Table.toObject(message.table, options);
                            if (message.initialSplits && message.initialSplits.length) {
                                object.initialSplits = [];
                                for (var j = 0; j < message.initialSplits.length; ++j)
                                    object.initialSplits[j] = $root.google.bigtable.admin.v2.CreateTableRequest.Split.toObject(message.initialSplits[j], options);
                            }
                            return object;
                        };
    
                        /**
                         * Converts this CreateTableRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.CreateTableRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        CreateTableRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for CreateTableRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.CreateTableRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        CreateTableRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.CreateTableRequest";
                        };
    
                        CreateTableRequest.Split = (function() {
    
                            /**
                             * Properties of a Split.
                             * @memberof google.bigtable.admin.v2.CreateTableRequest
                             * @interface ISplit
                             * @property {Uint8Array|null} [key] Split key
                             */
    
                            /**
                             * Constructs a new Split.
                             * @memberof google.bigtable.admin.v2.CreateTableRequest
                             * @classdesc Represents a Split.
                             * @implements ISplit
                             * @constructor
                             * @param {google.bigtable.admin.v2.CreateTableRequest.ISplit=} [properties] Properties to set
                             */
                            function Split(properties) {
                                if (properties)
                                    for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                        if (properties[keys[i]] != null)
                                            this[keys[i]] = properties[keys[i]];
                            }
    
                            /**
                             * Split key.
                             * @member {Uint8Array} key
                             * @memberof google.bigtable.admin.v2.CreateTableRequest.Split
                             * @instance
                             */
                            Split.prototype.key = $util.newBuffer([]);
    
                            /**
                             * Creates a new Split instance using the specified properties.
                             * @function create
                             * @memberof google.bigtable.admin.v2.CreateTableRequest.Split
                             * @static
                             * @param {google.bigtable.admin.v2.CreateTableRequest.ISplit=} [properties] Properties to set
                             * @returns {google.bigtable.admin.v2.CreateTableRequest.Split} Split instance
                             */
                            Split.create = function create(properties) {
                                return new Split(properties);
                            };
    
                            /**
                             * Encodes the specified Split message. Does not implicitly {@link google.bigtable.admin.v2.CreateTableRequest.Split.verify|verify} messages.
                             * @function encode
                             * @memberof google.bigtable.admin.v2.CreateTableRequest.Split
                             * @static
                             * @param {google.bigtable.admin.v2.CreateTableRequest.ISplit} message Split message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Split.encode = function encode(message, writer) {
                                if (!writer)
                                    writer = $Writer.create();
                                if (message.key != null && Object.hasOwnProperty.call(message, "key"))
                                    writer.uint32(/* id 1, wireType 2 =*/10).bytes(message.key);
                                return writer;
                            };
    
                            /**
                             * Encodes the specified Split message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.CreateTableRequest.Split.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.bigtable.admin.v2.CreateTableRequest.Split
                             * @static
                             * @param {google.bigtable.admin.v2.CreateTableRequest.ISplit} message Split message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Split.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a Split message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.bigtable.admin.v2.CreateTableRequest.Split
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @param {number} [length] Message length if known beforehand
                             * @returns {google.bigtable.admin.v2.CreateTableRequest.Split} Split
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Split.decode = function decode(reader, length) {
                                if (!(reader instanceof $Reader))
                                    reader = $Reader.create(reader);
                                var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.CreateTableRequest.Split();
                                while (reader.pos < end) {
                                    var tag = reader.uint32();
                                    switch (tag >>> 3) {
                                    case 1: {
                                            message.key = reader.bytes();
                                            break;
                                        }
                                    default:
                                        reader.skipType(tag & 7);
                                        break;
                                    }
                                }
                                return message;
                            };
    
                            /**
                             * Decodes a Split message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.bigtable.admin.v2.CreateTableRequest.Split
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.bigtable.admin.v2.CreateTableRequest.Split} Split
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Split.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a Split message.
                             * @function verify
                             * @memberof google.bigtable.admin.v2.CreateTableRequest.Split
                             * @static
                             * @param {Object.<string,*>} message Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            Split.verify = function verify(message) {
                                if (typeof message !== "object" || message === null)
                                    return "object expected";
                                if (message.key != null && message.hasOwnProperty("key"))
                                    if (!(message.key && typeof message.key.length === "number" || $util.isString(message.key)))
                                        return "key: buffer expected";
                                return null;
                            };
    
                            /**
                             * Creates a Split message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.bigtable.admin.v2.CreateTableRequest.Split
                             * @static
                             * @param {Object.<string,*>} object Plain object
                             * @returns {google.bigtable.admin.v2.CreateTableRequest.Split} Split
                             */
                            Split.fromObject = function fromObject(object) {
                                if (object instanceof $root.google.bigtable.admin.v2.CreateTableRequest.Split)
                                    return object;
                                var message = new $root.google.bigtable.admin.v2.CreateTableRequest.Split();
                                if (object.key != null)
                                    if (typeof object.key === "string")
                                        $util.base64.decode(object.key, message.key = $util.newBuffer($util.base64.length(object.key)), 0);
                                    else if (object.key.length >= 0)
                                        message.key = object.key;
                                return message;
                            };
    
                            /**
                             * Creates a plain object from a Split message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.bigtable.admin.v2.CreateTableRequest.Split
                             * @static
                             * @param {google.bigtable.admin.v2.CreateTableRequest.Split} message Split
                             * @param {$protobuf.IConversionOptions} [options] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            Split.toObject = function toObject(message, options) {
                                if (!options)
                                    options = {};
                                var object = {};
                                if (options.defaults)
                                    if (options.bytes === String)
                                        object.key = "";
                                    else {
                                        object.key = [];
                                        if (options.bytes !== Array)
                                            object.key = $util.newBuffer(object.key);
                                    }
                                if (message.key != null && message.hasOwnProperty("key"))
                                    object.key = options.bytes === String ? $util.base64.encode(message.key, 0, message.key.length) : options.bytes === Array ? Array.prototype.slice.call(message.key) : message.key;
                                return object;
                            };
    
                            /**
                             * Converts this Split to JSON.
                             * @function toJSON
                             * @memberof google.bigtable.admin.v2.CreateTableRequest.Split
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            Split.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * Gets the default type url for Split
                             * @function getTypeUrl
                             * @memberof google.bigtable.admin.v2.CreateTableRequest.Split
                             * @static
                             * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                             * @returns {string} The default type url
                             */
                            Split.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                                if (typeUrlPrefix === undefined) {
                                    typeUrlPrefix = "type.googleapis.com";
                                }
                                return typeUrlPrefix + "/google.bigtable.admin.v2.CreateTableRequest.Split";
                            };
    
                            return Split;
                        })();
    
                        return CreateTableRequest;
                    })();
    
                    v2.CreateTableFromSnapshotRequest = (function() {
    
                        /**
                         * Properties of a CreateTableFromSnapshotRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface ICreateTableFromSnapshotRequest
                         * @property {string|null} [parent] CreateTableFromSnapshotRequest parent
                         * @property {string|null} [tableId] CreateTableFromSnapshotRequest tableId
                         * @property {string|null} [sourceSnapshot] CreateTableFromSnapshotRequest sourceSnapshot
                         */
    
                        /**
                         * Constructs a new CreateTableFromSnapshotRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a CreateTableFromSnapshotRequest.
                         * @implements ICreateTableFromSnapshotRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.ICreateTableFromSnapshotRequest=} [properties] Properties to set
                         */
                        function CreateTableFromSnapshotRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * CreateTableFromSnapshotRequest parent.
                         * @member {string} parent
                         * @memberof google.bigtable.admin.v2.CreateTableFromSnapshotRequest
                         * @instance
                         */
                        CreateTableFromSnapshotRequest.prototype.parent = "";
    
                        /**
                         * CreateTableFromSnapshotRequest tableId.
                         * @member {string} tableId
                         * @memberof google.bigtable.admin.v2.CreateTableFromSnapshotRequest
                         * @instance
                         */
                        CreateTableFromSnapshotRequest.prototype.tableId = "";
    
                        /**
                         * CreateTableFromSnapshotRequest sourceSnapshot.
                         * @member {string} sourceSnapshot
                         * @memberof google.bigtable.admin.v2.CreateTableFromSnapshotRequest
                         * @instance
                         */
                        CreateTableFromSnapshotRequest.prototype.sourceSnapshot = "";
    
                        /**
                         * Creates a new CreateTableFromSnapshotRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.CreateTableFromSnapshotRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateTableFromSnapshotRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.CreateTableFromSnapshotRequest} CreateTableFromSnapshotRequest instance
                         */
                        CreateTableFromSnapshotRequest.create = function create(properties) {
                            return new CreateTableFromSnapshotRequest(properties);
                        };
    
                        /**
                         * Encodes the specified CreateTableFromSnapshotRequest message. Does not implicitly {@link google.bigtable.admin.v2.CreateTableFromSnapshotRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.CreateTableFromSnapshotRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateTableFromSnapshotRequest} message CreateTableFromSnapshotRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CreateTableFromSnapshotRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.parent != null && Object.hasOwnProperty.call(message, "parent"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.parent);
                            if (message.tableId != null && Object.hasOwnProperty.call(message, "tableId"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.tableId);
                            if (message.sourceSnapshot != null && Object.hasOwnProperty.call(message, "sourceSnapshot"))
                                writer.uint32(/* id 3, wireType 2 =*/26).string(message.sourceSnapshot);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified CreateTableFromSnapshotRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.CreateTableFromSnapshotRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.CreateTableFromSnapshotRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateTableFromSnapshotRequest} message CreateTableFromSnapshotRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CreateTableFromSnapshotRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a CreateTableFromSnapshotRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.CreateTableFromSnapshotRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.CreateTableFromSnapshotRequest} CreateTableFromSnapshotRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CreateTableFromSnapshotRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.CreateTableFromSnapshotRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.parent = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.tableId = reader.string();
                                        break;
                                    }
                                case 3: {
                                        message.sourceSnapshot = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a CreateTableFromSnapshotRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.CreateTableFromSnapshotRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.CreateTableFromSnapshotRequest} CreateTableFromSnapshotRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CreateTableFromSnapshotRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a CreateTableFromSnapshotRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.CreateTableFromSnapshotRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        CreateTableFromSnapshotRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                if (!$util.isString(message.parent))
                                    return "parent: string expected";
                            if (message.tableId != null && message.hasOwnProperty("tableId"))
                                if (!$util.isString(message.tableId))
                                    return "tableId: string expected";
                            if (message.sourceSnapshot != null && message.hasOwnProperty("sourceSnapshot"))
                                if (!$util.isString(message.sourceSnapshot))
                                    return "sourceSnapshot: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a CreateTableFromSnapshotRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.CreateTableFromSnapshotRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.CreateTableFromSnapshotRequest} CreateTableFromSnapshotRequest
                         */
                        CreateTableFromSnapshotRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.CreateTableFromSnapshotRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.CreateTableFromSnapshotRequest();
                            if (object.parent != null)
                                message.parent = String(object.parent);
                            if (object.tableId != null)
                                message.tableId = String(object.tableId);
                            if (object.sourceSnapshot != null)
                                message.sourceSnapshot = String(object.sourceSnapshot);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a CreateTableFromSnapshotRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.CreateTableFromSnapshotRequest
                         * @static
                         * @param {google.bigtable.admin.v2.CreateTableFromSnapshotRequest} message CreateTableFromSnapshotRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        CreateTableFromSnapshotRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.parent = "";
                                object.tableId = "";
                                object.sourceSnapshot = "";
                            }
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                object.parent = message.parent;
                            if (message.tableId != null && message.hasOwnProperty("tableId"))
                                object.tableId = message.tableId;
                            if (message.sourceSnapshot != null && message.hasOwnProperty("sourceSnapshot"))
                                object.sourceSnapshot = message.sourceSnapshot;
                            return object;
                        };
    
                        /**
                         * Converts this CreateTableFromSnapshotRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.CreateTableFromSnapshotRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        CreateTableFromSnapshotRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for CreateTableFromSnapshotRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.CreateTableFromSnapshotRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        CreateTableFromSnapshotRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.CreateTableFromSnapshotRequest";
                        };
    
                        return CreateTableFromSnapshotRequest;
                    })();
    
                    v2.DropRowRangeRequest = (function() {
    
                        /**
                         * Properties of a DropRowRangeRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IDropRowRangeRequest
                         * @property {string|null} [name] DropRowRangeRequest name
                         * @property {Uint8Array|null} [rowKeyPrefix] DropRowRangeRequest rowKeyPrefix
                         * @property {boolean|null} [deleteAllDataFromTable] DropRowRangeRequest deleteAllDataFromTable
                         */
    
                        /**
                         * Constructs a new DropRowRangeRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a DropRowRangeRequest.
                         * @implements IDropRowRangeRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IDropRowRangeRequest=} [properties] Properties to set
                         */
                        function DropRowRangeRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * DropRowRangeRequest name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.DropRowRangeRequest
                         * @instance
                         */
                        DropRowRangeRequest.prototype.name = "";
    
                        /**
                         * DropRowRangeRequest rowKeyPrefix.
                         * @member {Uint8Array|null|undefined} rowKeyPrefix
                         * @memberof google.bigtable.admin.v2.DropRowRangeRequest
                         * @instance
                         */
                        DropRowRangeRequest.prototype.rowKeyPrefix = null;
    
                        /**
                         * DropRowRangeRequest deleteAllDataFromTable.
                         * @member {boolean|null|undefined} deleteAllDataFromTable
                         * @memberof google.bigtable.admin.v2.DropRowRangeRequest
                         * @instance
                         */
                        DropRowRangeRequest.prototype.deleteAllDataFromTable = null;
    
                        // OneOf field names bound to virtual getters and setters
                        var $oneOfFields;
    
                        /**
                         * DropRowRangeRequest target.
                         * @member {"rowKeyPrefix"|"deleteAllDataFromTable"|undefined} target
                         * @memberof google.bigtable.admin.v2.DropRowRangeRequest
                         * @instance
                         */
                        Object.defineProperty(DropRowRangeRequest.prototype, "target", {
                            get: $util.oneOfGetter($oneOfFields = ["rowKeyPrefix", "deleteAllDataFromTable"]),
                            set: $util.oneOfSetter($oneOfFields)
                        });
    
                        /**
                         * Creates a new DropRowRangeRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.DropRowRangeRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IDropRowRangeRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.DropRowRangeRequest} DropRowRangeRequest instance
                         */
                        DropRowRangeRequest.create = function create(properties) {
                            return new DropRowRangeRequest(properties);
                        };
    
                        /**
                         * Encodes the specified DropRowRangeRequest message. Does not implicitly {@link google.bigtable.admin.v2.DropRowRangeRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.DropRowRangeRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IDropRowRangeRequest} message DropRowRangeRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        DropRowRangeRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            if (message.rowKeyPrefix != null && Object.hasOwnProperty.call(message, "rowKeyPrefix"))
                                writer.uint32(/* id 2, wireType 2 =*/18).bytes(message.rowKeyPrefix);
                            if (message.deleteAllDataFromTable != null && Object.hasOwnProperty.call(message, "deleteAllDataFromTable"))
                                writer.uint32(/* id 3, wireType 0 =*/24).bool(message.deleteAllDataFromTable);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified DropRowRangeRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.DropRowRangeRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.DropRowRangeRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IDropRowRangeRequest} message DropRowRangeRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        DropRowRangeRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a DropRowRangeRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.DropRowRangeRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.DropRowRangeRequest} DropRowRangeRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        DropRowRangeRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.DropRowRangeRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.rowKeyPrefix = reader.bytes();
                                        break;
                                    }
                                case 3: {
                                        message.deleteAllDataFromTable = reader.bool();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a DropRowRangeRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.DropRowRangeRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.DropRowRangeRequest} DropRowRangeRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        DropRowRangeRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a DropRowRangeRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.DropRowRangeRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        DropRowRangeRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            var properties = {};
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            if (message.rowKeyPrefix != null && message.hasOwnProperty("rowKeyPrefix")) {
                                properties.target = 1;
                                if (!(message.rowKeyPrefix && typeof message.rowKeyPrefix.length === "number" || $util.isString(message.rowKeyPrefix)))
                                    return "rowKeyPrefix: buffer expected";
                            }
                            if (message.deleteAllDataFromTable != null && message.hasOwnProperty("deleteAllDataFromTable")) {
                                if (properties.target === 1)
                                    return "target: multiple values";
                                properties.target = 1;
                                if (typeof message.deleteAllDataFromTable !== "boolean")
                                    return "deleteAllDataFromTable: boolean expected";
                            }
                            return null;
                        };
    
                        /**
                         * Creates a DropRowRangeRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.DropRowRangeRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.DropRowRangeRequest} DropRowRangeRequest
                         */
                        DropRowRangeRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.DropRowRangeRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.DropRowRangeRequest();
                            if (object.name != null)
                                message.name = String(object.name);
                            if (object.rowKeyPrefix != null)
                                if (typeof object.rowKeyPrefix === "string")
                                    $util.base64.decode(object.rowKeyPrefix, message.rowKeyPrefix = $util.newBuffer($util.base64.length(object.rowKeyPrefix)), 0);
                                else if (object.rowKeyPrefix.length >= 0)
                                    message.rowKeyPrefix = object.rowKeyPrefix;
                            if (object.deleteAllDataFromTable != null)
                                message.deleteAllDataFromTable = Boolean(object.deleteAllDataFromTable);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a DropRowRangeRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.DropRowRangeRequest
                         * @static
                         * @param {google.bigtable.admin.v2.DropRowRangeRequest} message DropRowRangeRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        DropRowRangeRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults)
                                object.name = "";
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            if (message.rowKeyPrefix != null && message.hasOwnProperty("rowKeyPrefix")) {
                                object.rowKeyPrefix = options.bytes === String ? $util.base64.encode(message.rowKeyPrefix, 0, message.rowKeyPrefix.length) : options.bytes === Array ? Array.prototype.slice.call(message.rowKeyPrefix) : message.rowKeyPrefix;
                                if (options.oneofs)
                                    object.target = "rowKeyPrefix";
                            }
                            if (message.deleteAllDataFromTable != null && message.hasOwnProperty("deleteAllDataFromTable")) {
                                object.deleteAllDataFromTable = message.deleteAllDataFromTable;
                                if (options.oneofs)
                                    object.target = "deleteAllDataFromTable";
                            }
                            return object;
                        };
    
                        /**
                         * Converts this DropRowRangeRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.DropRowRangeRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        DropRowRangeRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for DropRowRangeRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.DropRowRangeRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        DropRowRangeRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.DropRowRangeRequest";
                        };
    
                        return DropRowRangeRequest;
                    })();
    
                    v2.ListTablesRequest = (function() {
    
                        /**
                         * Properties of a ListTablesRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IListTablesRequest
                         * @property {string|null} [parent] ListTablesRequest parent
                         * @property {google.bigtable.admin.v2.Table.View|null} [view] ListTablesRequest view
                         * @property {number|null} [pageSize] ListTablesRequest pageSize
                         * @property {string|null} [pageToken] ListTablesRequest pageToken
                         */
    
                        /**
                         * Constructs a new ListTablesRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a ListTablesRequest.
                         * @implements IListTablesRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IListTablesRequest=} [properties] Properties to set
                         */
                        function ListTablesRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * ListTablesRequest parent.
                         * @member {string} parent
                         * @memberof google.bigtable.admin.v2.ListTablesRequest
                         * @instance
                         */
                        ListTablesRequest.prototype.parent = "";
    
                        /**
                         * ListTablesRequest view.
                         * @member {google.bigtable.admin.v2.Table.View} view
                         * @memberof google.bigtable.admin.v2.ListTablesRequest
                         * @instance
                         */
                        ListTablesRequest.prototype.view = 0;
    
                        /**
                         * ListTablesRequest pageSize.
                         * @member {number} pageSize
                         * @memberof google.bigtable.admin.v2.ListTablesRequest
                         * @instance
                         */
                        ListTablesRequest.prototype.pageSize = 0;
    
                        /**
                         * ListTablesRequest pageToken.
                         * @member {string} pageToken
                         * @memberof google.bigtable.admin.v2.ListTablesRequest
                         * @instance
                         */
                        ListTablesRequest.prototype.pageToken = "";
    
                        /**
                         * Creates a new ListTablesRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.ListTablesRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IListTablesRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.ListTablesRequest} ListTablesRequest instance
                         */
                        ListTablesRequest.create = function create(properties) {
                            return new ListTablesRequest(properties);
                        };
    
                        /**
                         * Encodes the specified ListTablesRequest message. Does not implicitly {@link google.bigtable.admin.v2.ListTablesRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.ListTablesRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IListTablesRequest} message ListTablesRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListTablesRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.parent != null && Object.hasOwnProperty.call(message, "parent"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.parent);
                            if (message.view != null && Object.hasOwnProperty.call(message, "view"))
                                writer.uint32(/* id 2, wireType 0 =*/16).int32(message.view);
                            if (message.pageToken != null && Object.hasOwnProperty.call(message, "pageToken"))
                                writer.uint32(/* id 3, wireType 2 =*/26).string(message.pageToken);
                            if (message.pageSize != null && Object.hasOwnProperty.call(message, "pageSize"))
                                writer.uint32(/* id 4, wireType 0 =*/32).int32(message.pageSize);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified ListTablesRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.ListTablesRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.ListTablesRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IListTablesRequest} message ListTablesRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListTablesRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a ListTablesRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.ListTablesRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.ListTablesRequest} ListTablesRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListTablesRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.ListTablesRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.parent = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.view = reader.int32();
                                        break;
                                    }
                                case 4: {
                                        message.pageSize = reader.int32();
                                        break;
                                    }
                                case 3: {
                                        message.pageToken = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a ListTablesRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.ListTablesRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.ListTablesRequest} ListTablesRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListTablesRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a ListTablesRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.ListTablesRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        ListTablesRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                if (!$util.isString(message.parent))
                                    return "parent: string expected";
                            if (message.view != null && message.hasOwnProperty("view"))
                                switch (message.view) {
                                default:
                                    return "view: enum value expected";
                                case 0:
                                case 1:
                                case 2:
                                case 3:
                                case 5:
                                case 4:
                                    break;
                                }
                            if (message.pageSize != null && message.hasOwnProperty("pageSize"))
                                if (!$util.isInteger(message.pageSize))
                                    return "pageSize: integer expected";
                            if (message.pageToken != null && message.hasOwnProperty("pageToken"))
                                if (!$util.isString(message.pageToken))
                                    return "pageToken: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a ListTablesRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.ListTablesRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.ListTablesRequest} ListTablesRequest
                         */
                        ListTablesRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.ListTablesRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.ListTablesRequest();
                            if (object.parent != null)
                                message.parent = String(object.parent);
                            switch (object.view) {
                            default:
                                if (typeof object.view === "number") {
                                    message.view = object.view;
                                    break;
                                }
                                break;
                            case "VIEW_UNSPECIFIED":
                            case 0:
                                message.view = 0;
                                break;
                            case "NAME_ONLY":
                            case 1:
                                message.view = 1;
                                break;
                            case "SCHEMA_VIEW":
                            case 2:
                                message.view = 2;
                                break;
                            case "REPLICATION_VIEW":
                            case 3:
                                message.view = 3;
                                break;
                            case "ENCRYPTION_VIEW":
                            case 5:
                                message.view = 5;
                                break;
                            case "FULL":
                            case 4:
                                message.view = 4;
                                break;
                            }
                            if (object.pageSize != null)
                                message.pageSize = object.pageSize | 0;
                            if (object.pageToken != null)
                                message.pageToken = String(object.pageToken);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a ListTablesRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.ListTablesRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ListTablesRequest} message ListTablesRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        ListTablesRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.parent = "";
                                object.view = options.enums === String ? "VIEW_UNSPECIFIED" : 0;
                                object.pageToken = "";
                                object.pageSize = 0;
                            }
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                object.parent = message.parent;
                            if (message.view != null && message.hasOwnProperty("view"))
                                object.view = options.enums === String ? $root.google.bigtable.admin.v2.Table.View[message.view] === undefined ? message.view : $root.google.bigtable.admin.v2.Table.View[message.view] : message.view;
                            if (message.pageToken != null && message.hasOwnProperty("pageToken"))
                                object.pageToken = message.pageToken;
                            if (message.pageSize != null && message.hasOwnProperty("pageSize"))
                                object.pageSize = message.pageSize;
                            return object;
                        };
    
                        /**
                         * Converts this ListTablesRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.ListTablesRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        ListTablesRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for ListTablesRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.ListTablesRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        ListTablesRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.ListTablesRequest";
                        };
    
                        return ListTablesRequest;
                    })();
    
                    v2.ListTablesResponse = (function() {
    
                        /**
                         * Properties of a ListTablesResponse.
                         * @memberof google.bigtable.admin.v2
                         * @interface IListTablesResponse
                         * @property {Array.<google.bigtable.admin.v2.ITable>|null} [tables] ListTablesResponse tables
                         * @property {string|null} [nextPageToken] ListTablesResponse nextPageToken
                         */
    
                        /**
                         * Constructs a new ListTablesResponse.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a ListTablesResponse.
                         * @implements IListTablesResponse
                         * @constructor
                         * @param {google.bigtable.admin.v2.IListTablesResponse=} [properties] Properties to set
                         */
                        function ListTablesResponse(properties) {
                            this.tables = [];
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * ListTablesResponse tables.
                         * @member {Array.<google.bigtable.admin.v2.ITable>} tables
                         * @memberof google.bigtable.admin.v2.ListTablesResponse
                         * @instance
                         */
                        ListTablesResponse.prototype.tables = $util.emptyArray;
    
                        /**
                         * ListTablesResponse nextPageToken.
                         * @member {string} nextPageToken
                         * @memberof google.bigtable.admin.v2.ListTablesResponse
                         * @instance
                         */
                        ListTablesResponse.prototype.nextPageToken = "";
    
                        /**
                         * Creates a new ListTablesResponse instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.ListTablesResponse
                         * @static
                         * @param {google.bigtable.admin.v2.IListTablesResponse=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.ListTablesResponse} ListTablesResponse instance
                         */
                        ListTablesResponse.create = function create(properties) {
                            return new ListTablesResponse(properties);
                        };
    
                        /**
                         * Encodes the specified ListTablesResponse message. Does not implicitly {@link google.bigtable.admin.v2.ListTablesResponse.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.ListTablesResponse
                         * @static
                         * @param {google.bigtable.admin.v2.IListTablesResponse} message ListTablesResponse message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListTablesResponse.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.tables != null && message.tables.length)
                                for (var i = 0; i < message.tables.length; ++i)
                                    $root.google.bigtable.admin.v2.Table.encode(message.tables[i], writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            if (message.nextPageToken != null && Object.hasOwnProperty.call(message, "nextPageToken"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.nextPageToken);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified ListTablesResponse message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.ListTablesResponse.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.ListTablesResponse
                         * @static
                         * @param {google.bigtable.admin.v2.IListTablesResponse} message ListTablesResponse message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListTablesResponse.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a ListTablesResponse message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.ListTablesResponse
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.ListTablesResponse} ListTablesResponse
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListTablesResponse.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.ListTablesResponse();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        if (!(message.tables && message.tables.length))
                                            message.tables = [];
                                        message.tables.push($root.google.bigtable.admin.v2.Table.decode(reader, reader.uint32()));
                                        break;
                                    }
                                case 2: {
                                        message.nextPageToken = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a ListTablesResponse message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.ListTablesResponse
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.ListTablesResponse} ListTablesResponse
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListTablesResponse.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a ListTablesResponse message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.ListTablesResponse
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        ListTablesResponse.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.tables != null && message.hasOwnProperty("tables")) {
                                if (!Array.isArray(message.tables))
                                    return "tables: array expected";
                                for (var i = 0; i < message.tables.length; ++i) {
                                    var error = $root.google.bigtable.admin.v2.Table.verify(message.tables[i]);
                                    if (error)
                                        return "tables." + error;
                                }
                            }
                            if (message.nextPageToken != null && message.hasOwnProperty("nextPageToken"))
                                if (!$util.isString(message.nextPageToken))
                                    return "nextPageToken: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a ListTablesResponse message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.ListTablesResponse
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.ListTablesResponse} ListTablesResponse
                         */
                        ListTablesResponse.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.ListTablesResponse)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.ListTablesResponse();
                            if (object.tables) {
                                if (!Array.isArray(object.tables))
                                    throw TypeError(".google.bigtable.admin.v2.ListTablesResponse.tables: array expected");
                                message.tables = [];
                                for (var i = 0; i < object.tables.length; ++i) {
                                    if (typeof object.tables[i] !== "object")
                                        throw TypeError(".google.bigtable.admin.v2.ListTablesResponse.tables: object expected");
                                    message.tables[i] = $root.google.bigtable.admin.v2.Table.fromObject(object.tables[i]);
                                }
                            }
                            if (object.nextPageToken != null)
                                message.nextPageToken = String(object.nextPageToken);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a ListTablesResponse message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.ListTablesResponse
                         * @static
                         * @param {google.bigtable.admin.v2.ListTablesResponse} message ListTablesResponse
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        ListTablesResponse.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.arrays || options.defaults)
                                object.tables = [];
                            if (options.defaults)
                                object.nextPageToken = "";
                            if (message.tables && message.tables.length) {
                                object.tables = [];
                                for (var j = 0; j < message.tables.length; ++j)
                                    object.tables[j] = $root.google.bigtable.admin.v2.Table.toObject(message.tables[j], options);
                            }
                            if (message.nextPageToken != null && message.hasOwnProperty("nextPageToken"))
                                object.nextPageToken = message.nextPageToken;
                            return object;
                        };
    
                        /**
                         * Converts this ListTablesResponse to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.ListTablesResponse
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        ListTablesResponse.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for ListTablesResponse
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.ListTablesResponse
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        ListTablesResponse.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.ListTablesResponse";
                        };
    
                        return ListTablesResponse;
                    })();
    
                    v2.GetTableRequest = (function() {
    
                        /**
                         * Properties of a GetTableRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IGetTableRequest
                         * @property {string|null} [name] GetTableRequest name
                         * @property {google.bigtable.admin.v2.Table.View|null} [view] GetTableRequest view
                         */
    
                        /**
                         * Constructs a new GetTableRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a GetTableRequest.
                         * @implements IGetTableRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IGetTableRequest=} [properties] Properties to set
                         */
                        function GetTableRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * GetTableRequest name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.GetTableRequest
                         * @instance
                         */
                        GetTableRequest.prototype.name = "";
    
                        /**
                         * GetTableRequest view.
                         * @member {google.bigtable.admin.v2.Table.View} view
                         * @memberof google.bigtable.admin.v2.GetTableRequest
                         * @instance
                         */
                        GetTableRequest.prototype.view = 0;
    
                        /**
                         * Creates a new GetTableRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.GetTableRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IGetTableRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.GetTableRequest} GetTableRequest instance
                         */
                        GetTableRequest.create = function create(properties) {
                            return new GetTableRequest(properties);
                        };
    
                        /**
                         * Encodes the specified GetTableRequest message. Does not implicitly {@link google.bigtable.admin.v2.GetTableRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.GetTableRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IGetTableRequest} message GetTableRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        GetTableRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            if (message.view != null && Object.hasOwnProperty.call(message, "view"))
                                writer.uint32(/* id 2, wireType 0 =*/16).int32(message.view);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified GetTableRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.GetTableRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.GetTableRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IGetTableRequest} message GetTableRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        GetTableRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a GetTableRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.GetTableRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.GetTableRequest} GetTableRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        GetTableRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.GetTableRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.view = reader.int32();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a GetTableRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.GetTableRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.GetTableRequest} GetTableRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        GetTableRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a GetTableRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.GetTableRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        GetTableRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            if (message.view != null && message.hasOwnProperty("view"))
                                switch (message.view) {
                                default:
                                    return "view: enum value expected";
                                case 0:
                                case 1:
                                case 2:
                                case 3:
                                case 5:
                                case 4:
                                    break;
                                }
                            return null;
                        };
    
                        /**
                         * Creates a GetTableRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.GetTableRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.GetTableRequest} GetTableRequest
                         */
                        GetTableRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.GetTableRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.GetTableRequest();
                            if (object.name != null)
                                message.name = String(object.name);
                            switch (object.view) {
                            default:
                                if (typeof object.view === "number") {
                                    message.view = object.view;
                                    break;
                                }
                                break;
                            case "VIEW_UNSPECIFIED":
                            case 0:
                                message.view = 0;
                                break;
                            case "NAME_ONLY":
                            case 1:
                                message.view = 1;
                                break;
                            case "SCHEMA_VIEW":
                            case 2:
                                message.view = 2;
                                break;
                            case "REPLICATION_VIEW":
                            case 3:
                                message.view = 3;
                                break;
                            case "ENCRYPTION_VIEW":
                            case 5:
                                message.view = 5;
                                break;
                            case "FULL":
                            case 4:
                                message.view = 4;
                                break;
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a GetTableRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.GetTableRequest
                         * @static
                         * @param {google.bigtable.admin.v2.GetTableRequest} message GetTableRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        GetTableRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.name = "";
                                object.view = options.enums === String ? "VIEW_UNSPECIFIED" : 0;
                            }
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            if (message.view != null && message.hasOwnProperty("view"))
                                object.view = options.enums === String ? $root.google.bigtable.admin.v2.Table.View[message.view] === undefined ? message.view : $root.google.bigtable.admin.v2.Table.View[message.view] : message.view;
                            return object;
                        };
    
                        /**
                         * Converts this GetTableRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.GetTableRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        GetTableRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for GetTableRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.GetTableRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        GetTableRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.GetTableRequest";
                        };
    
                        return GetTableRequest;
                    })();
    
                    v2.UpdateTableRequest = (function() {
    
                        /**
                         * Properties of an UpdateTableRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IUpdateTableRequest
                         * @property {google.bigtable.admin.v2.ITable|null} [table] UpdateTableRequest table
                         * @property {google.protobuf.IFieldMask|null} [updateMask] UpdateTableRequest updateMask
                         */
    
                        /**
                         * Constructs a new UpdateTableRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents an UpdateTableRequest.
                         * @implements IUpdateTableRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IUpdateTableRequest=} [properties] Properties to set
                         */
                        function UpdateTableRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * UpdateTableRequest table.
                         * @member {google.bigtable.admin.v2.ITable|null|undefined} table
                         * @memberof google.bigtable.admin.v2.UpdateTableRequest
                         * @instance
                         */
                        UpdateTableRequest.prototype.table = null;
    
                        /**
                         * UpdateTableRequest updateMask.
                         * @member {google.protobuf.IFieldMask|null|undefined} updateMask
                         * @memberof google.bigtable.admin.v2.UpdateTableRequest
                         * @instance
                         */
                        UpdateTableRequest.prototype.updateMask = null;
    
                        /**
                         * Creates a new UpdateTableRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.UpdateTableRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IUpdateTableRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.UpdateTableRequest} UpdateTableRequest instance
                         */
                        UpdateTableRequest.create = function create(properties) {
                            return new UpdateTableRequest(properties);
                        };
    
                        /**
                         * Encodes the specified UpdateTableRequest message. Does not implicitly {@link google.bigtable.admin.v2.UpdateTableRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.UpdateTableRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IUpdateTableRequest} message UpdateTableRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        UpdateTableRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.table != null && Object.hasOwnProperty.call(message, "table"))
                                $root.google.bigtable.admin.v2.Table.encode(message.table, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            if (message.updateMask != null && Object.hasOwnProperty.call(message, "updateMask"))
                                $root.google.protobuf.FieldMask.encode(message.updateMask, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified UpdateTableRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.UpdateTableRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.UpdateTableRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IUpdateTableRequest} message UpdateTableRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        UpdateTableRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes an UpdateTableRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.UpdateTableRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.UpdateTableRequest} UpdateTableRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        UpdateTableRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.UpdateTableRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.table = $root.google.bigtable.admin.v2.Table.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 2: {
                                        message.updateMask = $root.google.protobuf.FieldMask.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes an UpdateTableRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.UpdateTableRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.UpdateTableRequest} UpdateTableRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        UpdateTableRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies an UpdateTableRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.UpdateTableRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        UpdateTableRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.table != null && message.hasOwnProperty("table")) {
                                var error = $root.google.bigtable.admin.v2.Table.verify(message.table);
                                if (error)
                                    return "table." + error;
                            }
                            if (message.updateMask != null && message.hasOwnProperty("updateMask")) {
                                var error = $root.google.protobuf.FieldMask.verify(message.updateMask);
                                if (error)
                                    return "updateMask." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates an UpdateTableRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.UpdateTableRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.UpdateTableRequest} UpdateTableRequest
                         */
                        UpdateTableRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.UpdateTableRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.UpdateTableRequest();
                            if (object.table != null) {
                                if (typeof object.table !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.UpdateTableRequest.table: object expected");
                                message.table = $root.google.bigtable.admin.v2.Table.fromObject(object.table);
                            }
                            if (object.updateMask != null) {
                                if (typeof object.updateMask !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.UpdateTableRequest.updateMask: object expected");
                                message.updateMask = $root.google.protobuf.FieldMask.fromObject(object.updateMask);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from an UpdateTableRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.UpdateTableRequest
                         * @static
                         * @param {google.bigtable.admin.v2.UpdateTableRequest} message UpdateTableRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        UpdateTableRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.table = null;
                                object.updateMask = null;
                            }
                            if (message.table != null && message.hasOwnProperty("table"))
                                object.table = $root.google.bigtable.admin.v2.Table.toObject(message.table, options);
                            if (message.updateMask != null && message.hasOwnProperty("updateMask"))
                                object.updateMask = $root.google.protobuf.FieldMask.toObject(message.updateMask, options);
                            return object;
                        };
    
                        /**
                         * Converts this UpdateTableRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.UpdateTableRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        UpdateTableRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for UpdateTableRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.UpdateTableRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        UpdateTableRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.UpdateTableRequest";
                        };
    
                        return UpdateTableRequest;
                    })();
    
                    v2.UpdateTableMetadata = (function() {
    
                        /**
                         * Properties of an UpdateTableMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @interface IUpdateTableMetadata
                         * @property {string|null} [name] UpdateTableMetadata name
                         * @property {google.protobuf.ITimestamp|null} [startTime] UpdateTableMetadata startTime
                         * @property {google.protobuf.ITimestamp|null} [endTime] UpdateTableMetadata endTime
                         */
    
                        /**
                         * Constructs a new UpdateTableMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents an UpdateTableMetadata.
                         * @implements IUpdateTableMetadata
                         * @constructor
                         * @param {google.bigtable.admin.v2.IUpdateTableMetadata=} [properties] Properties to set
                         */
                        function UpdateTableMetadata(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * UpdateTableMetadata name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.UpdateTableMetadata
                         * @instance
                         */
                        UpdateTableMetadata.prototype.name = "";
    
                        /**
                         * UpdateTableMetadata startTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} startTime
                         * @memberof google.bigtable.admin.v2.UpdateTableMetadata
                         * @instance
                         */
                        UpdateTableMetadata.prototype.startTime = null;
    
                        /**
                         * UpdateTableMetadata endTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} endTime
                         * @memberof google.bigtable.admin.v2.UpdateTableMetadata
                         * @instance
                         */
                        UpdateTableMetadata.prototype.endTime = null;
    
                        /**
                         * Creates a new UpdateTableMetadata instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.UpdateTableMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.IUpdateTableMetadata=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.UpdateTableMetadata} UpdateTableMetadata instance
                         */
                        UpdateTableMetadata.create = function create(properties) {
                            return new UpdateTableMetadata(properties);
                        };
    
                        /**
                         * Encodes the specified UpdateTableMetadata message. Does not implicitly {@link google.bigtable.admin.v2.UpdateTableMetadata.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.UpdateTableMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.IUpdateTableMetadata} message UpdateTableMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        UpdateTableMetadata.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            if (message.startTime != null && Object.hasOwnProperty.call(message, "startTime"))
                                $root.google.protobuf.Timestamp.encode(message.startTime, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            if (message.endTime != null && Object.hasOwnProperty.call(message, "endTime"))
                                $root.google.protobuf.Timestamp.encode(message.endTime, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified UpdateTableMetadata message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.UpdateTableMetadata.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.UpdateTableMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.IUpdateTableMetadata} message UpdateTableMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        UpdateTableMetadata.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes an UpdateTableMetadata message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.UpdateTableMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.UpdateTableMetadata} UpdateTableMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        UpdateTableMetadata.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.UpdateTableMetadata();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.startTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 3: {
                                        message.endTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes an UpdateTableMetadata message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.UpdateTableMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.UpdateTableMetadata} UpdateTableMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        UpdateTableMetadata.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies an UpdateTableMetadata message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.UpdateTableMetadata
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        UpdateTableMetadata.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            if (message.startTime != null && message.hasOwnProperty("startTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.startTime);
                                if (error)
                                    return "startTime." + error;
                            }
                            if (message.endTime != null && message.hasOwnProperty("endTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.endTime);
                                if (error)
                                    return "endTime." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates an UpdateTableMetadata message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.UpdateTableMetadata
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.UpdateTableMetadata} UpdateTableMetadata
                         */
                        UpdateTableMetadata.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.UpdateTableMetadata)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.UpdateTableMetadata();
                            if (object.name != null)
                                message.name = String(object.name);
                            if (object.startTime != null) {
                                if (typeof object.startTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.UpdateTableMetadata.startTime: object expected");
                                message.startTime = $root.google.protobuf.Timestamp.fromObject(object.startTime);
                            }
                            if (object.endTime != null) {
                                if (typeof object.endTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.UpdateTableMetadata.endTime: object expected");
                                message.endTime = $root.google.protobuf.Timestamp.fromObject(object.endTime);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from an UpdateTableMetadata message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.UpdateTableMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.UpdateTableMetadata} message UpdateTableMetadata
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        UpdateTableMetadata.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.name = "";
                                object.startTime = null;
                                object.endTime = null;
                            }
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            if (message.startTime != null && message.hasOwnProperty("startTime"))
                                object.startTime = $root.google.protobuf.Timestamp.toObject(message.startTime, options);
                            if (message.endTime != null && message.hasOwnProperty("endTime"))
                                object.endTime = $root.google.protobuf.Timestamp.toObject(message.endTime, options);
                            return object;
                        };
    
                        /**
                         * Converts this UpdateTableMetadata to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.UpdateTableMetadata
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        UpdateTableMetadata.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for UpdateTableMetadata
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.UpdateTableMetadata
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        UpdateTableMetadata.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.UpdateTableMetadata";
                        };
    
                        return UpdateTableMetadata;
                    })();
    
                    v2.DeleteTableRequest = (function() {
    
                        /**
                         * Properties of a DeleteTableRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IDeleteTableRequest
                         * @property {string|null} [name] DeleteTableRequest name
                         */
    
                        /**
                         * Constructs a new DeleteTableRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a DeleteTableRequest.
                         * @implements IDeleteTableRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IDeleteTableRequest=} [properties] Properties to set
                         */
                        function DeleteTableRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * DeleteTableRequest name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.DeleteTableRequest
                         * @instance
                         */
                        DeleteTableRequest.prototype.name = "";
    
                        /**
                         * Creates a new DeleteTableRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.DeleteTableRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IDeleteTableRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.DeleteTableRequest} DeleteTableRequest instance
                         */
                        DeleteTableRequest.create = function create(properties) {
                            return new DeleteTableRequest(properties);
                        };
    
                        /**
                         * Encodes the specified DeleteTableRequest message. Does not implicitly {@link google.bigtable.admin.v2.DeleteTableRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.DeleteTableRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IDeleteTableRequest} message DeleteTableRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        DeleteTableRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified DeleteTableRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.DeleteTableRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.DeleteTableRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IDeleteTableRequest} message DeleteTableRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        DeleteTableRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a DeleteTableRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.DeleteTableRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.DeleteTableRequest} DeleteTableRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        DeleteTableRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.DeleteTableRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a DeleteTableRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.DeleteTableRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.DeleteTableRequest} DeleteTableRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        DeleteTableRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a DeleteTableRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.DeleteTableRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        DeleteTableRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a DeleteTableRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.DeleteTableRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.DeleteTableRequest} DeleteTableRequest
                         */
                        DeleteTableRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.DeleteTableRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.DeleteTableRequest();
                            if (object.name != null)
                                message.name = String(object.name);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a DeleteTableRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.DeleteTableRequest
                         * @static
                         * @param {google.bigtable.admin.v2.DeleteTableRequest} message DeleteTableRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        DeleteTableRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults)
                                object.name = "";
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            return object;
                        };
    
                        /**
                         * Converts this DeleteTableRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.DeleteTableRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        DeleteTableRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for DeleteTableRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.DeleteTableRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        DeleteTableRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.DeleteTableRequest";
                        };
    
                        return DeleteTableRequest;
                    })();
    
                    v2.UndeleteTableRequest = (function() {
    
                        /**
                         * Properties of an UndeleteTableRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IUndeleteTableRequest
                         * @property {string|null} [name] UndeleteTableRequest name
                         */
    
                        /**
                         * Constructs a new UndeleteTableRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents an UndeleteTableRequest.
                         * @implements IUndeleteTableRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IUndeleteTableRequest=} [properties] Properties to set
                         */
                        function UndeleteTableRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * UndeleteTableRequest name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.UndeleteTableRequest
                         * @instance
                         */
                        UndeleteTableRequest.prototype.name = "";
    
                        /**
                         * Creates a new UndeleteTableRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.UndeleteTableRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IUndeleteTableRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.UndeleteTableRequest} UndeleteTableRequest instance
                         */
                        UndeleteTableRequest.create = function create(properties) {
                            return new UndeleteTableRequest(properties);
                        };
    
                        /**
                         * Encodes the specified UndeleteTableRequest message. Does not implicitly {@link google.bigtable.admin.v2.UndeleteTableRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.UndeleteTableRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IUndeleteTableRequest} message UndeleteTableRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        UndeleteTableRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified UndeleteTableRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.UndeleteTableRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.UndeleteTableRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IUndeleteTableRequest} message UndeleteTableRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        UndeleteTableRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes an UndeleteTableRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.UndeleteTableRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.UndeleteTableRequest} UndeleteTableRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        UndeleteTableRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.UndeleteTableRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes an UndeleteTableRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.UndeleteTableRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.UndeleteTableRequest} UndeleteTableRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        UndeleteTableRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies an UndeleteTableRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.UndeleteTableRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        UndeleteTableRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            return null;
                        };
    
                        /**
                         * Creates an UndeleteTableRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.UndeleteTableRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.UndeleteTableRequest} UndeleteTableRequest
                         */
                        UndeleteTableRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.UndeleteTableRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.UndeleteTableRequest();
                            if (object.name != null)
                                message.name = String(object.name);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from an UndeleteTableRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.UndeleteTableRequest
                         * @static
                         * @param {google.bigtable.admin.v2.UndeleteTableRequest} message UndeleteTableRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        UndeleteTableRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults)
                                object.name = "";
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            return object;
                        };
    
                        /**
                         * Converts this UndeleteTableRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.UndeleteTableRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        UndeleteTableRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for UndeleteTableRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.UndeleteTableRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        UndeleteTableRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.UndeleteTableRequest";
                        };
    
                        return UndeleteTableRequest;
                    })();
    
                    v2.UndeleteTableMetadata = (function() {
    
                        /**
                         * Properties of an UndeleteTableMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @interface IUndeleteTableMetadata
                         * @property {string|null} [name] UndeleteTableMetadata name
                         * @property {google.protobuf.ITimestamp|null} [startTime] UndeleteTableMetadata startTime
                         * @property {google.protobuf.ITimestamp|null} [endTime] UndeleteTableMetadata endTime
                         */
    
                        /**
                         * Constructs a new UndeleteTableMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents an UndeleteTableMetadata.
                         * @implements IUndeleteTableMetadata
                         * @constructor
                         * @param {google.bigtable.admin.v2.IUndeleteTableMetadata=} [properties] Properties to set
                         */
                        function UndeleteTableMetadata(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * UndeleteTableMetadata name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.UndeleteTableMetadata
                         * @instance
                         */
                        UndeleteTableMetadata.prototype.name = "";
    
                        /**
                         * UndeleteTableMetadata startTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} startTime
                         * @memberof google.bigtable.admin.v2.UndeleteTableMetadata
                         * @instance
                         */
                        UndeleteTableMetadata.prototype.startTime = null;
    
                        /**
                         * UndeleteTableMetadata endTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} endTime
                         * @memberof google.bigtable.admin.v2.UndeleteTableMetadata
                         * @instance
                         */
                        UndeleteTableMetadata.prototype.endTime = null;
    
                        /**
                         * Creates a new UndeleteTableMetadata instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.UndeleteTableMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.IUndeleteTableMetadata=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.UndeleteTableMetadata} UndeleteTableMetadata instance
                         */
                        UndeleteTableMetadata.create = function create(properties) {
                            return new UndeleteTableMetadata(properties);
                        };
    
                        /**
                         * Encodes the specified UndeleteTableMetadata message. Does not implicitly {@link google.bigtable.admin.v2.UndeleteTableMetadata.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.UndeleteTableMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.IUndeleteTableMetadata} message UndeleteTableMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        UndeleteTableMetadata.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            if (message.startTime != null && Object.hasOwnProperty.call(message, "startTime"))
                                $root.google.protobuf.Timestamp.encode(message.startTime, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            if (message.endTime != null && Object.hasOwnProperty.call(message, "endTime"))
                                $root.google.protobuf.Timestamp.encode(message.endTime, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified UndeleteTableMetadata message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.UndeleteTableMetadata.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.UndeleteTableMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.IUndeleteTableMetadata} message UndeleteTableMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        UndeleteTableMetadata.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes an UndeleteTableMetadata message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.UndeleteTableMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.UndeleteTableMetadata} UndeleteTableMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        UndeleteTableMetadata.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.UndeleteTableMetadata();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.startTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 3: {
                                        message.endTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes an UndeleteTableMetadata message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.UndeleteTableMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.UndeleteTableMetadata} UndeleteTableMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        UndeleteTableMetadata.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies an UndeleteTableMetadata message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.UndeleteTableMetadata
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        UndeleteTableMetadata.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            if (message.startTime != null && message.hasOwnProperty("startTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.startTime);
                                if (error)
                                    return "startTime." + error;
                            }
                            if (message.endTime != null && message.hasOwnProperty("endTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.endTime);
                                if (error)
                                    return "endTime." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates an UndeleteTableMetadata message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.UndeleteTableMetadata
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.UndeleteTableMetadata} UndeleteTableMetadata
                         */
                        UndeleteTableMetadata.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.UndeleteTableMetadata)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.UndeleteTableMetadata();
                            if (object.name != null)
                                message.name = String(object.name);
                            if (object.startTime != null) {
                                if (typeof object.startTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.UndeleteTableMetadata.startTime: object expected");
                                message.startTime = $root.google.protobuf.Timestamp.fromObject(object.startTime);
                            }
                            if (object.endTime != null) {
                                if (typeof object.endTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.UndeleteTableMetadata.endTime: object expected");
                                message.endTime = $root.google.protobuf.Timestamp.fromObject(object.endTime);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from an UndeleteTableMetadata message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.UndeleteTableMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.UndeleteTableMetadata} message UndeleteTableMetadata
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        UndeleteTableMetadata.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.name = "";
                                object.startTime = null;
                                object.endTime = null;
                            }
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            if (message.startTime != null && message.hasOwnProperty("startTime"))
                                object.startTime = $root.google.protobuf.Timestamp.toObject(message.startTime, options);
                            if (message.endTime != null && message.hasOwnProperty("endTime"))
                                object.endTime = $root.google.protobuf.Timestamp.toObject(message.endTime, options);
                            return object;
                        };
    
                        /**
                         * Converts this UndeleteTableMetadata to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.UndeleteTableMetadata
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        UndeleteTableMetadata.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for UndeleteTableMetadata
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.UndeleteTableMetadata
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        UndeleteTableMetadata.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.UndeleteTableMetadata";
                        };
    
                        return UndeleteTableMetadata;
                    })();
    
                    v2.ModifyColumnFamiliesRequest = (function() {
    
                        /**
                         * Properties of a ModifyColumnFamiliesRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IModifyColumnFamiliesRequest
                         * @property {string|null} [name] ModifyColumnFamiliesRequest name
                         * @property {Array.<google.bigtable.admin.v2.ModifyColumnFamiliesRequest.IModification>|null} [modifications] ModifyColumnFamiliesRequest modifications
                         * @property {boolean|null} [ignoreWarnings] ModifyColumnFamiliesRequest ignoreWarnings
                         */
    
                        /**
                         * Constructs a new ModifyColumnFamiliesRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a ModifyColumnFamiliesRequest.
                         * @implements IModifyColumnFamiliesRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IModifyColumnFamiliesRequest=} [properties] Properties to set
                         */
                        function ModifyColumnFamiliesRequest(properties) {
                            this.modifications = [];
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * ModifyColumnFamiliesRequest name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest
                         * @instance
                         */
                        ModifyColumnFamiliesRequest.prototype.name = "";
    
                        /**
                         * ModifyColumnFamiliesRequest modifications.
                         * @member {Array.<google.bigtable.admin.v2.ModifyColumnFamiliesRequest.IModification>} modifications
                         * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest
                         * @instance
                         */
                        ModifyColumnFamiliesRequest.prototype.modifications = $util.emptyArray;
    
                        /**
                         * ModifyColumnFamiliesRequest ignoreWarnings.
                         * @member {boolean} ignoreWarnings
                         * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest
                         * @instance
                         */
                        ModifyColumnFamiliesRequest.prototype.ignoreWarnings = false;
    
                        /**
                         * Creates a new ModifyColumnFamiliesRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IModifyColumnFamiliesRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.ModifyColumnFamiliesRequest} ModifyColumnFamiliesRequest instance
                         */
                        ModifyColumnFamiliesRequest.create = function create(properties) {
                            return new ModifyColumnFamiliesRequest(properties);
                        };
    
                        /**
                         * Encodes the specified ModifyColumnFamiliesRequest message. Does not implicitly {@link google.bigtable.admin.v2.ModifyColumnFamiliesRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IModifyColumnFamiliesRequest} message ModifyColumnFamiliesRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ModifyColumnFamiliesRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            if (message.modifications != null && message.modifications.length)
                                for (var i = 0; i < message.modifications.length; ++i)
                                    $root.google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification.encode(message.modifications[i], writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            if (message.ignoreWarnings != null && Object.hasOwnProperty.call(message, "ignoreWarnings"))
                                writer.uint32(/* id 3, wireType 0 =*/24).bool(message.ignoreWarnings);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified ModifyColumnFamiliesRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.ModifyColumnFamiliesRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IModifyColumnFamiliesRequest} message ModifyColumnFamiliesRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ModifyColumnFamiliesRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a ModifyColumnFamiliesRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.ModifyColumnFamiliesRequest} ModifyColumnFamiliesRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ModifyColumnFamiliesRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.ModifyColumnFamiliesRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                case 2: {
                                        if (!(message.modifications && message.modifications.length))
                                            message.modifications = [];
                                        message.modifications.push($root.google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification.decode(reader, reader.uint32()));
                                        break;
                                    }
                                case 3: {
                                        message.ignoreWarnings = reader.bool();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a ModifyColumnFamiliesRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.ModifyColumnFamiliesRequest} ModifyColumnFamiliesRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ModifyColumnFamiliesRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a ModifyColumnFamiliesRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        ModifyColumnFamiliesRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            if (message.modifications != null && message.hasOwnProperty("modifications")) {
                                if (!Array.isArray(message.modifications))
                                    return "modifications: array expected";
                                for (var i = 0; i < message.modifications.length; ++i) {
                                    var error = $root.google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification.verify(message.modifications[i]);
                                    if (error)
                                        return "modifications." + error;
                                }
                            }
                            if (message.ignoreWarnings != null && message.hasOwnProperty("ignoreWarnings"))
                                if (typeof message.ignoreWarnings !== "boolean")
                                    return "ignoreWarnings: boolean expected";
                            return null;
                        };
    
                        /**
                         * Creates a ModifyColumnFamiliesRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.ModifyColumnFamiliesRequest} ModifyColumnFamiliesRequest
                         */
                        ModifyColumnFamiliesRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.ModifyColumnFamiliesRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.ModifyColumnFamiliesRequest();
                            if (object.name != null)
                                message.name = String(object.name);
                            if (object.modifications) {
                                if (!Array.isArray(object.modifications))
                                    throw TypeError(".google.bigtable.admin.v2.ModifyColumnFamiliesRequest.modifications: array expected");
                                message.modifications = [];
                                for (var i = 0; i < object.modifications.length; ++i) {
                                    if (typeof object.modifications[i] !== "object")
                                        throw TypeError(".google.bigtable.admin.v2.ModifyColumnFamiliesRequest.modifications: object expected");
                                    message.modifications[i] = $root.google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification.fromObject(object.modifications[i]);
                                }
                            }
                            if (object.ignoreWarnings != null)
                                message.ignoreWarnings = Boolean(object.ignoreWarnings);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a ModifyColumnFamiliesRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ModifyColumnFamiliesRequest} message ModifyColumnFamiliesRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        ModifyColumnFamiliesRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.arrays || options.defaults)
                                object.modifications = [];
                            if (options.defaults) {
                                object.name = "";
                                object.ignoreWarnings = false;
                            }
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            if (message.modifications && message.modifications.length) {
                                object.modifications = [];
                                for (var j = 0; j < message.modifications.length; ++j)
                                    object.modifications[j] = $root.google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification.toObject(message.modifications[j], options);
                            }
                            if (message.ignoreWarnings != null && message.hasOwnProperty("ignoreWarnings"))
                                object.ignoreWarnings = message.ignoreWarnings;
                            return object;
                        };
    
                        /**
                         * Converts this ModifyColumnFamiliesRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        ModifyColumnFamiliesRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for ModifyColumnFamiliesRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        ModifyColumnFamiliesRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.ModifyColumnFamiliesRequest";
                        };
    
                        ModifyColumnFamiliesRequest.Modification = (function() {
    
                            /**
                             * Properties of a Modification.
                             * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest
                             * @interface IModification
                             * @property {string|null} [id] Modification id
                             * @property {google.bigtable.admin.v2.IColumnFamily|null} [create] Modification create
                             * @property {google.bigtable.admin.v2.IColumnFamily|null} [update] Modification update
                             * @property {boolean|null} [drop] Modification drop
                             * @property {google.protobuf.IFieldMask|null} [updateMask] Modification updateMask
                             */
    
                            /**
                             * Constructs a new Modification.
                             * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest
                             * @classdesc Represents a Modification.
                             * @implements IModification
                             * @constructor
                             * @param {google.bigtable.admin.v2.ModifyColumnFamiliesRequest.IModification=} [properties] Properties to set
                             */
                            function Modification(properties) {
                                if (properties)
                                    for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                        if (properties[keys[i]] != null)
                                            this[keys[i]] = properties[keys[i]];
                            }
    
                            /**
                             * Modification id.
                             * @member {string} id
                             * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification
                             * @instance
                             */
                            Modification.prototype.id = "";
    
                            /**
                             * Modification create.
                             * @member {google.bigtable.admin.v2.IColumnFamily|null|undefined} create
                             * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification
                             * @instance
                             */
                            Modification.prototype.create = null;
    
                            /**
                             * Modification update.
                             * @member {google.bigtable.admin.v2.IColumnFamily|null|undefined} update
                             * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification
                             * @instance
                             */
                            Modification.prototype.update = null;
    
                            /**
                             * Modification drop.
                             * @member {boolean|null|undefined} drop
                             * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification
                             * @instance
                             */
                            Modification.prototype.drop = null;
    
                            /**
                             * Modification updateMask.
                             * @member {google.protobuf.IFieldMask|null|undefined} updateMask
                             * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification
                             * @instance
                             */
                            Modification.prototype.updateMask = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * Modification mod.
                             * @member {"create"|"update"|"drop"|undefined} mod
                             * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification
                             * @instance
                             */
                            Object.defineProperty(Modification.prototype, "mod", {
                                get: $util.oneOfGetter($oneOfFields = ["create", "update", "drop"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new Modification instance using the specified properties.
                             * @function create
                             * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification
                             * @static
                             * @param {google.bigtable.admin.v2.ModifyColumnFamiliesRequest.IModification=} [properties] Properties to set
                             * @returns {google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification} Modification instance
                             */
                            Modification.create = function create(properties) {
                                return new Modification(properties);
                            };
    
                            /**
                             * Encodes the specified Modification message. Does not implicitly {@link google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification.verify|verify} messages.
                             * @function encode
                             * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification
                             * @static
                             * @param {google.bigtable.admin.v2.ModifyColumnFamiliesRequest.IModification} message Modification message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Modification.encode = function encode(message, writer) {
                                if (!writer)
                                    writer = $Writer.create();
                                if (message.id != null && Object.hasOwnProperty.call(message, "id"))
                                    writer.uint32(/* id 1, wireType 2 =*/10).string(message.id);
                                if (message.create != null && Object.hasOwnProperty.call(message, "create"))
                                    $root.google.bigtable.admin.v2.ColumnFamily.encode(message.create, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                                if (message.update != null && Object.hasOwnProperty.call(message, "update"))
                                    $root.google.bigtable.admin.v2.ColumnFamily.encode(message.update, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                                if (message.drop != null && Object.hasOwnProperty.call(message, "drop"))
                                    writer.uint32(/* id 4, wireType 0 =*/32).bool(message.drop);
                                if (message.updateMask != null && Object.hasOwnProperty.call(message, "updateMask"))
                                    $root.google.protobuf.FieldMask.encode(message.updateMask, writer.uint32(/* id 6, wireType 2 =*/50).fork()).ldelim();
                                return writer;
                            };
    
                            /**
                             * Encodes the specified Modification message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification
                             * @static
                             * @param {google.bigtable.admin.v2.ModifyColumnFamiliesRequest.IModification} message Modification message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Modification.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a Modification message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @param {number} [length] Message length if known beforehand
                             * @returns {google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification} Modification
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Modification.decode = function decode(reader, length) {
                                if (!(reader instanceof $Reader))
                                    reader = $Reader.create(reader);
                                var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification();
                                while (reader.pos < end) {
                                    var tag = reader.uint32();
                                    switch (tag >>> 3) {
                                    case 1: {
                                            message.id = reader.string();
                                            break;
                                        }
                                    case 2: {
                                            message.create = $root.google.bigtable.admin.v2.ColumnFamily.decode(reader, reader.uint32());
                                            break;
                                        }
                                    case 3: {
                                            message.update = $root.google.bigtable.admin.v2.ColumnFamily.decode(reader, reader.uint32());
                                            break;
                                        }
                                    case 4: {
                                            message.drop = reader.bool();
                                            break;
                                        }
                                    case 6: {
                                            message.updateMask = $root.google.protobuf.FieldMask.decode(reader, reader.uint32());
                                            break;
                                        }
                                    default:
                                        reader.skipType(tag & 7);
                                        break;
                                    }
                                }
                                return message;
                            };
    
                            /**
                             * Decodes a Modification message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification} Modification
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Modification.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a Modification message.
                             * @function verify
                             * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification
                             * @static
                             * @param {Object.<string,*>} message Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            Modification.verify = function verify(message) {
                                if (typeof message !== "object" || message === null)
                                    return "object expected";
                                var properties = {};
                                if (message.id != null && message.hasOwnProperty("id"))
                                    if (!$util.isString(message.id))
                                        return "id: string expected";
                                if (message.create != null && message.hasOwnProperty("create")) {
                                    properties.mod = 1;
                                    {
                                        var error = $root.google.bigtable.admin.v2.ColumnFamily.verify(message.create);
                                        if (error)
                                            return "create." + error;
                                    }
                                }
                                if (message.update != null && message.hasOwnProperty("update")) {
                                    if (properties.mod === 1)
                                        return "mod: multiple values";
                                    properties.mod = 1;
                                    {
                                        var error = $root.google.bigtable.admin.v2.ColumnFamily.verify(message.update);
                                        if (error)
                                            return "update." + error;
                                    }
                                }
                                if (message.drop != null && message.hasOwnProperty("drop")) {
                                    if (properties.mod === 1)
                                        return "mod: multiple values";
                                    properties.mod = 1;
                                    if (typeof message.drop !== "boolean")
                                        return "drop: boolean expected";
                                }
                                if (message.updateMask != null && message.hasOwnProperty("updateMask")) {
                                    var error = $root.google.protobuf.FieldMask.verify(message.updateMask);
                                    if (error)
                                        return "updateMask." + error;
                                }
                                return null;
                            };
    
                            /**
                             * Creates a Modification message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification
                             * @static
                             * @param {Object.<string,*>} object Plain object
                             * @returns {google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification} Modification
                             */
                            Modification.fromObject = function fromObject(object) {
                                if (object instanceof $root.google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification)
                                    return object;
                                var message = new $root.google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification();
                                if (object.id != null)
                                    message.id = String(object.id);
                                if (object.create != null) {
                                    if (typeof object.create !== "object")
                                        throw TypeError(".google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification.create: object expected");
                                    message.create = $root.google.bigtable.admin.v2.ColumnFamily.fromObject(object.create);
                                }
                                if (object.update != null) {
                                    if (typeof object.update !== "object")
                                        throw TypeError(".google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification.update: object expected");
                                    message.update = $root.google.bigtable.admin.v2.ColumnFamily.fromObject(object.update);
                                }
                                if (object.drop != null)
                                    message.drop = Boolean(object.drop);
                                if (object.updateMask != null) {
                                    if (typeof object.updateMask !== "object")
                                        throw TypeError(".google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification.updateMask: object expected");
                                    message.updateMask = $root.google.protobuf.FieldMask.fromObject(object.updateMask);
                                }
                                return message;
                            };
    
                            /**
                             * Creates a plain object from a Modification message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification
                             * @static
                             * @param {google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification} message Modification
                             * @param {$protobuf.IConversionOptions} [options] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            Modification.toObject = function toObject(message, options) {
                                if (!options)
                                    options = {};
                                var object = {};
                                if (options.defaults) {
                                    object.id = "";
                                    object.updateMask = null;
                                }
                                if (message.id != null && message.hasOwnProperty("id"))
                                    object.id = message.id;
                                if (message.create != null && message.hasOwnProperty("create")) {
                                    object.create = $root.google.bigtable.admin.v2.ColumnFamily.toObject(message.create, options);
                                    if (options.oneofs)
                                        object.mod = "create";
                                }
                                if (message.update != null && message.hasOwnProperty("update")) {
                                    object.update = $root.google.bigtable.admin.v2.ColumnFamily.toObject(message.update, options);
                                    if (options.oneofs)
                                        object.mod = "update";
                                }
                                if (message.drop != null && message.hasOwnProperty("drop")) {
                                    object.drop = message.drop;
                                    if (options.oneofs)
                                        object.mod = "drop";
                                }
                                if (message.updateMask != null && message.hasOwnProperty("updateMask"))
                                    object.updateMask = $root.google.protobuf.FieldMask.toObject(message.updateMask, options);
                                return object;
                            };
    
                            /**
                             * Converts this Modification to JSON.
                             * @function toJSON
                             * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            Modification.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * Gets the default type url for Modification
                             * @function getTypeUrl
                             * @memberof google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification
                             * @static
                             * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                             * @returns {string} The default type url
                             */
                            Modification.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                                if (typeUrlPrefix === undefined) {
                                    typeUrlPrefix = "type.googleapis.com";
                                }
                                return typeUrlPrefix + "/google.bigtable.admin.v2.ModifyColumnFamiliesRequest.Modification";
                            };
    
                            return Modification;
                        })();
    
                        return ModifyColumnFamiliesRequest;
                    })();
    
                    v2.GenerateConsistencyTokenRequest = (function() {
    
                        /**
                         * Properties of a GenerateConsistencyTokenRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IGenerateConsistencyTokenRequest
                         * @property {string|null} [name] GenerateConsistencyTokenRequest name
                         */
    
                        /**
                         * Constructs a new GenerateConsistencyTokenRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a GenerateConsistencyTokenRequest.
                         * @implements IGenerateConsistencyTokenRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IGenerateConsistencyTokenRequest=} [properties] Properties to set
                         */
                        function GenerateConsistencyTokenRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * GenerateConsistencyTokenRequest name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.GenerateConsistencyTokenRequest
                         * @instance
                         */
                        GenerateConsistencyTokenRequest.prototype.name = "";
    
                        /**
                         * Creates a new GenerateConsistencyTokenRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.GenerateConsistencyTokenRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IGenerateConsistencyTokenRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.GenerateConsistencyTokenRequest} GenerateConsistencyTokenRequest instance
                         */
                        GenerateConsistencyTokenRequest.create = function create(properties) {
                            return new GenerateConsistencyTokenRequest(properties);
                        };
    
                        /**
                         * Encodes the specified GenerateConsistencyTokenRequest message. Does not implicitly {@link google.bigtable.admin.v2.GenerateConsistencyTokenRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.GenerateConsistencyTokenRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IGenerateConsistencyTokenRequest} message GenerateConsistencyTokenRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        GenerateConsistencyTokenRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified GenerateConsistencyTokenRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.GenerateConsistencyTokenRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.GenerateConsistencyTokenRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IGenerateConsistencyTokenRequest} message GenerateConsistencyTokenRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        GenerateConsistencyTokenRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a GenerateConsistencyTokenRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.GenerateConsistencyTokenRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.GenerateConsistencyTokenRequest} GenerateConsistencyTokenRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        GenerateConsistencyTokenRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.GenerateConsistencyTokenRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a GenerateConsistencyTokenRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.GenerateConsistencyTokenRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.GenerateConsistencyTokenRequest} GenerateConsistencyTokenRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        GenerateConsistencyTokenRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a GenerateConsistencyTokenRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.GenerateConsistencyTokenRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        GenerateConsistencyTokenRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a GenerateConsistencyTokenRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.GenerateConsistencyTokenRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.GenerateConsistencyTokenRequest} GenerateConsistencyTokenRequest
                         */
                        GenerateConsistencyTokenRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.GenerateConsistencyTokenRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.GenerateConsistencyTokenRequest();
                            if (object.name != null)
                                message.name = String(object.name);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a GenerateConsistencyTokenRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.GenerateConsistencyTokenRequest
                         * @static
                         * @param {google.bigtable.admin.v2.GenerateConsistencyTokenRequest} message GenerateConsistencyTokenRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        GenerateConsistencyTokenRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults)
                                object.name = "";
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            return object;
                        };
    
                        /**
                         * Converts this GenerateConsistencyTokenRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.GenerateConsistencyTokenRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        GenerateConsistencyTokenRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for GenerateConsistencyTokenRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.GenerateConsistencyTokenRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        GenerateConsistencyTokenRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.GenerateConsistencyTokenRequest";
                        };
    
                        return GenerateConsistencyTokenRequest;
                    })();
    
                    v2.GenerateConsistencyTokenResponse = (function() {
    
                        /**
                         * Properties of a GenerateConsistencyTokenResponse.
                         * @memberof google.bigtable.admin.v2
                         * @interface IGenerateConsistencyTokenResponse
                         * @property {string|null} [consistencyToken] GenerateConsistencyTokenResponse consistencyToken
                         */
    
                        /**
                         * Constructs a new GenerateConsistencyTokenResponse.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a GenerateConsistencyTokenResponse.
                         * @implements IGenerateConsistencyTokenResponse
                         * @constructor
                         * @param {google.bigtable.admin.v2.IGenerateConsistencyTokenResponse=} [properties] Properties to set
                         */
                        function GenerateConsistencyTokenResponse(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * GenerateConsistencyTokenResponse consistencyToken.
                         * @member {string} consistencyToken
                         * @memberof google.bigtable.admin.v2.GenerateConsistencyTokenResponse
                         * @instance
                         */
                        GenerateConsistencyTokenResponse.prototype.consistencyToken = "";
    
                        /**
                         * Creates a new GenerateConsistencyTokenResponse instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.GenerateConsistencyTokenResponse
                         * @static
                         * @param {google.bigtable.admin.v2.IGenerateConsistencyTokenResponse=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.GenerateConsistencyTokenResponse} GenerateConsistencyTokenResponse instance
                         */
                        GenerateConsistencyTokenResponse.create = function create(properties) {
                            return new GenerateConsistencyTokenResponse(properties);
                        };
    
                        /**
                         * Encodes the specified GenerateConsistencyTokenResponse message. Does not implicitly {@link google.bigtable.admin.v2.GenerateConsistencyTokenResponse.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.GenerateConsistencyTokenResponse
                         * @static
                         * @param {google.bigtable.admin.v2.IGenerateConsistencyTokenResponse} message GenerateConsistencyTokenResponse message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        GenerateConsistencyTokenResponse.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.consistencyToken != null && Object.hasOwnProperty.call(message, "consistencyToken"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.consistencyToken);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified GenerateConsistencyTokenResponse message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.GenerateConsistencyTokenResponse.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.GenerateConsistencyTokenResponse
                         * @static
                         * @param {google.bigtable.admin.v2.IGenerateConsistencyTokenResponse} message GenerateConsistencyTokenResponse message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        GenerateConsistencyTokenResponse.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a GenerateConsistencyTokenResponse message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.GenerateConsistencyTokenResponse
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.GenerateConsistencyTokenResponse} GenerateConsistencyTokenResponse
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        GenerateConsistencyTokenResponse.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.GenerateConsistencyTokenResponse();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.consistencyToken = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a GenerateConsistencyTokenResponse message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.GenerateConsistencyTokenResponse
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.GenerateConsistencyTokenResponse} GenerateConsistencyTokenResponse
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        GenerateConsistencyTokenResponse.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a GenerateConsistencyTokenResponse message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.GenerateConsistencyTokenResponse
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        GenerateConsistencyTokenResponse.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.consistencyToken != null && message.hasOwnProperty("consistencyToken"))
                                if (!$util.isString(message.consistencyToken))
                                    return "consistencyToken: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a GenerateConsistencyTokenResponse message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.GenerateConsistencyTokenResponse
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.GenerateConsistencyTokenResponse} GenerateConsistencyTokenResponse
                         */
                        GenerateConsistencyTokenResponse.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.GenerateConsistencyTokenResponse)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.GenerateConsistencyTokenResponse();
                            if (object.consistencyToken != null)
                                message.consistencyToken = String(object.consistencyToken);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a GenerateConsistencyTokenResponse message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.GenerateConsistencyTokenResponse
                         * @static
                         * @param {google.bigtable.admin.v2.GenerateConsistencyTokenResponse} message GenerateConsistencyTokenResponse
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        GenerateConsistencyTokenResponse.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults)
                                object.consistencyToken = "";
                            if (message.consistencyToken != null && message.hasOwnProperty("consistencyToken"))
                                object.consistencyToken = message.consistencyToken;
                            return object;
                        };
    
                        /**
                         * Converts this GenerateConsistencyTokenResponse to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.GenerateConsistencyTokenResponse
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        GenerateConsistencyTokenResponse.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for GenerateConsistencyTokenResponse
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.GenerateConsistencyTokenResponse
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        GenerateConsistencyTokenResponse.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.GenerateConsistencyTokenResponse";
                        };
    
                        return GenerateConsistencyTokenResponse;
                    })();
    
                    v2.CheckConsistencyRequest = (function() {
    
                        /**
                         * Properties of a CheckConsistencyRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface ICheckConsistencyRequest
                         * @property {string|null} [name] CheckConsistencyRequest name
                         * @property {string|null} [consistencyToken] CheckConsistencyRequest consistencyToken
                         * @property {google.bigtable.admin.v2.IStandardReadRemoteWrites|null} [standardReadRemoteWrites] CheckConsistencyRequest standardReadRemoteWrites
                         * @property {google.bigtable.admin.v2.IDataBoostReadLocalWrites|null} [dataBoostReadLocalWrites] CheckConsistencyRequest dataBoostReadLocalWrites
                         */
    
                        /**
                         * Constructs a new CheckConsistencyRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a CheckConsistencyRequest.
                         * @implements ICheckConsistencyRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.ICheckConsistencyRequest=} [properties] Properties to set
                         */
                        function CheckConsistencyRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * CheckConsistencyRequest name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.CheckConsistencyRequest
                         * @instance
                         */
                        CheckConsistencyRequest.prototype.name = "";
    
                        /**
                         * CheckConsistencyRequest consistencyToken.
                         * @member {string} consistencyToken
                         * @memberof google.bigtable.admin.v2.CheckConsistencyRequest
                         * @instance
                         */
                        CheckConsistencyRequest.prototype.consistencyToken = "";
    
                        /**
                         * CheckConsistencyRequest standardReadRemoteWrites.
                         * @member {google.bigtable.admin.v2.IStandardReadRemoteWrites|null|undefined} standardReadRemoteWrites
                         * @memberof google.bigtable.admin.v2.CheckConsistencyRequest
                         * @instance
                         */
                        CheckConsistencyRequest.prototype.standardReadRemoteWrites = null;
    
                        /**
                         * CheckConsistencyRequest dataBoostReadLocalWrites.
                         * @member {google.bigtable.admin.v2.IDataBoostReadLocalWrites|null|undefined} dataBoostReadLocalWrites
                         * @memberof google.bigtable.admin.v2.CheckConsistencyRequest
                         * @instance
                         */
                        CheckConsistencyRequest.prototype.dataBoostReadLocalWrites = null;
    
                        // OneOf field names bound to virtual getters and setters
                        var $oneOfFields;
    
                        /**
                         * CheckConsistencyRequest mode.
                         * @member {"standardReadRemoteWrites"|"dataBoostReadLocalWrites"|undefined} mode
                         * @memberof google.bigtable.admin.v2.CheckConsistencyRequest
                         * @instance
                         */
                        Object.defineProperty(CheckConsistencyRequest.prototype, "mode", {
                            get: $util.oneOfGetter($oneOfFields = ["standardReadRemoteWrites", "dataBoostReadLocalWrites"]),
                            set: $util.oneOfSetter($oneOfFields)
                        });
    
                        /**
                         * Creates a new CheckConsistencyRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.CheckConsistencyRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ICheckConsistencyRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.CheckConsistencyRequest} CheckConsistencyRequest instance
                         */
                        CheckConsistencyRequest.create = function create(properties) {
                            return new CheckConsistencyRequest(properties);
                        };
    
                        /**
                         * Encodes the specified CheckConsistencyRequest message. Does not implicitly {@link google.bigtable.admin.v2.CheckConsistencyRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.CheckConsistencyRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ICheckConsistencyRequest} message CheckConsistencyRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CheckConsistencyRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            if (message.consistencyToken != null && Object.hasOwnProperty.call(message, "consistencyToken"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.consistencyToken);
                            if (message.standardReadRemoteWrites != null && Object.hasOwnProperty.call(message, "standardReadRemoteWrites"))
                                $root.google.bigtable.admin.v2.StandardReadRemoteWrites.encode(message.standardReadRemoteWrites, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            if (message.dataBoostReadLocalWrites != null && Object.hasOwnProperty.call(message, "dataBoostReadLocalWrites"))
                                $root.google.bigtable.admin.v2.DataBoostReadLocalWrites.encode(message.dataBoostReadLocalWrites, writer.uint32(/* id 4, wireType 2 =*/34).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified CheckConsistencyRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.CheckConsistencyRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.CheckConsistencyRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ICheckConsistencyRequest} message CheckConsistencyRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CheckConsistencyRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a CheckConsistencyRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.CheckConsistencyRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.CheckConsistencyRequest} CheckConsistencyRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CheckConsistencyRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.CheckConsistencyRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.consistencyToken = reader.string();
                                        break;
                                    }
                                case 3: {
                                        message.standardReadRemoteWrites = $root.google.bigtable.admin.v2.StandardReadRemoteWrites.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 4: {
                                        message.dataBoostReadLocalWrites = $root.google.bigtable.admin.v2.DataBoostReadLocalWrites.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a CheckConsistencyRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.CheckConsistencyRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.CheckConsistencyRequest} CheckConsistencyRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CheckConsistencyRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a CheckConsistencyRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.CheckConsistencyRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        CheckConsistencyRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            var properties = {};
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            if (message.consistencyToken != null && message.hasOwnProperty("consistencyToken"))
                                if (!$util.isString(message.consistencyToken))
                                    return "consistencyToken: string expected";
                            if (message.standardReadRemoteWrites != null && message.hasOwnProperty("standardReadRemoteWrites")) {
                                properties.mode = 1;
                                {
                                    var error = $root.google.bigtable.admin.v2.StandardReadRemoteWrites.verify(message.standardReadRemoteWrites);
                                    if (error)
                                        return "standardReadRemoteWrites." + error;
                                }
                            }
                            if (message.dataBoostReadLocalWrites != null && message.hasOwnProperty("dataBoostReadLocalWrites")) {
                                if (properties.mode === 1)
                                    return "mode: multiple values";
                                properties.mode = 1;
                                {
                                    var error = $root.google.bigtable.admin.v2.DataBoostReadLocalWrites.verify(message.dataBoostReadLocalWrites);
                                    if (error)
                                        return "dataBoostReadLocalWrites." + error;
                                }
                            }
                            return null;
                        };
    
                        /**
                         * Creates a CheckConsistencyRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.CheckConsistencyRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.CheckConsistencyRequest} CheckConsistencyRequest
                         */
                        CheckConsistencyRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.CheckConsistencyRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.CheckConsistencyRequest();
                            if (object.name != null)
                                message.name = String(object.name);
                            if (object.consistencyToken != null)
                                message.consistencyToken = String(object.consistencyToken);
                            if (object.standardReadRemoteWrites != null) {
                                if (typeof object.standardReadRemoteWrites !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.CheckConsistencyRequest.standardReadRemoteWrites: object expected");
                                message.standardReadRemoteWrites = $root.google.bigtable.admin.v2.StandardReadRemoteWrites.fromObject(object.standardReadRemoteWrites);
                            }
                            if (object.dataBoostReadLocalWrites != null) {
                                if (typeof object.dataBoostReadLocalWrites !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.CheckConsistencyRequest.dataBoostReadLocalWrites: object expected");
                                message.dataBoostReadLocalWrites = $root.google.bigtable.admin.v2.DataBoostReadLocalWrites.fromObject(object.dataBoostReadLocalWrites);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a CheckConsistencyRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.CheckConsistencyRequest
                         * @static
                         * @param {google.bigtable.admin.v2.CheckConsistencyRequest} message CheckConsistencyRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        CheckConsistencyRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.name = "";
                                object.consistencyToken = "";
                            }
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            if (message.consistencyToken != null && message.hasOwnProperty("consistencyToken"))
                                object.consistencyToken = message.consistencyToken;
                            if (message.standardReadRemoteWrites != null && message.hasOwnProperty("standardReadRemoteWrites")) {
                                object.standardReadRemoteWrites = $root.google.bigtable.admin.v2.StandardReadRemoteWrites.toObject(message.standardReadRemoteWrites, options);
                                if (options.oneofs)
                                    object.mode = "standardReadRemoteWrites";
                            }
                            if (message.dataBoostReadLocalWrites != null && message.hasOwnProperty("dataBoostReadLocalWrites")) {
                                object.dataBoostReadLocalWrites = $root.google.bigtable.admin.v2.DataBoostReadLocalWrites.toObject(message.dataBoostReadLocalWrites, options);
                                if (options.oneofs)
                                    object.mode = "dataBoostReadLocalWrites";
                            }
                            return object;
                        };
    
                        /**
                         * Converts this CheckConsistencyRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.CheckConsistencyRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        CheckConsistencyRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for CheckConsistencyRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.CheckConsistencyRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        CheckConsistencyRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.CheckConsistencyRequest";
                        };
    
                        return CheckConsistencyRequest;
                    })();
    
                    v2.StandardReadRemoteWrites = (function() {
    
                        /**
                         * Properties of a StandardReadRemoteWrites.
                         * @memberof google.bigtable.admin.v2
                         * @interface IStandardReadRemoteWrites
                         */
    
                        /**
                         * Constructs a new StandardReadRemoteWrites.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a StandardReadRemoteWrites.
                         * @implements IStandardReadRemoteWrites
                         * @constructor
                         * @param {google.bigtable.admin.v2.IStandardReadRemoteWrites=} [properties] Properties to set
                         */
                        function StandardReadRemoteWrites(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * Creates a new StandardReadRemoteWrites instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.StandardReadRemoteWrites
                         * @static
                         * @param {google.bigtable.admin.v2.IStandardReadRemoteWrites=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.StandardReadRemoteWrites} StandardReadRemoteWrites instance
                         */
                        StandardReadRemoteWrites.create = function create(properties) {
                            return new StandardReadRemoteWrites(properties);
                        };
    
                        /**
                         * Encodes the specified StandardReadRemoteWrites message. Does not implicitly {@link google.bigtable.admin.v2.StandardReadRemoteWrites.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.StandardReadRemoteWrites
                         * @static
                         * @param {google.bigtable.admin.v2.IStandardReadRemoteWrites} message StandardReadRemoteWrites message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        StandardReadRemoteWrites.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified StandardReadRemoteWrites message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.StandardReadRemoteWrites.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.StandardReadRemoteWrites
                         * @static
                         * @param {google.bigtable.admin.v2.IStandardReadRemoteWrites} message StandardReadRemoteWrites message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        StandardReadRemoteWrites.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a StandardReadRemoteWrites message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.StandardReadRemoteWrites
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.StandardReadRemoteWrites} StandardReadRemoteWrites
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        StandardReadRemoteWrites.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.StandardReadRemoteWrites();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a StandardReadRemoteWrites message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.StandardReadRemoteWrites
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.StandardReadRemoteWrites} StandardReadRemoteWrites
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        StandardReadRemoteWrites.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a StandardReadRemoteWrites message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.StandardReadRemoteWrites
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        StandardReadRemoteWrites.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            return null;
                        };
    
                        /**
                         * Creates a StandardReadRemoteWrites message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.StandardReadRemoteWrites
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.StandardReadRemoteWrites} StandardReadRemoteWrites
                         */
                        StandardReadRemoteWrites.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.StandardReadRemoteWrites)
                                return object;
                            return new $root.google.bigtable.admin.v2.StandardReadRemoteWrites();
                        };
    
                        /**
                         * Creates a plain object from a StandardReadRemoteWrites message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.StandardReadRemoteWrites
                         * @static
                         * @param {google.bigtable.admin.v2.StandardReadRemoteWrites} message StandardReadRemoteWrites
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        StandardReadRemoteWrites.toObject = function toObject() {
                            return {};
                        };
    
                        /**
                         * Converts this StandardReadRemoteWrites to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.StandardReadRemoteWrites
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        StandardReadRemoteWrites.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for StandardReadRemoteWrites
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.StandardReadRemoteWrites
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        StandardReadRemoteWrites.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.StandardReadRemoteWrites";
                        };
    
                        return StandardReadRemoteWrites;
                    })();
    
                    v2.DataBoostReadLocalWrites = (function() {
    
                        /**
                         * Properties of a DataBoostReadLocalWrites.
                         * @memberof google.bigtable.admin.v2
                         * @interface IDataBoostReadLocalWrites
                         */
    
                        /**
                         * Constructs a new DataBoostReadLocalWrites.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a DataBoostReadLocalWrites.
                         * @implements IDataBoostReadLocalWrites
                         * @constructor
                         * @param {google.bigtable.admin.v2.IDataBoostReadLocalWrites=} [properties] Properties to set
                         */
                        function DataBoostReadLocalWrites(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * Creates a new DataBoostReadLocalWrites instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.DataBoostReadLocalWrites
                         * @static
                         * @param {google.bigtable.admin.v2.IDataBoostReadLocalWrites=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.DataBoostReadLocalWrites} DataBoostReadLocalWrites instance
                         */
                        DataBoostReadLocalWrites.create = function create(properties) {
                            return new DataBoostReadLocalWrites(properties);
                        };
    
                        /**
                         * Encodes the specified DataBoostReadLocalWrites message. Does not implicitly {@link google.bigtable.admin.v2.DataBoostReadLocalWrites.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.DataBoostReadLocalWrites
                         * @static
                         * @param {google.bigtable.admin.v2.IDataBoostReadLocalWrites} message DataBoostReadLocalWrites message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        DataBoostReadLocalWrites.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified DataBoostReadLocalWrites message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.DataBoostReadLocalWrites.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.DataBoostReadLocalWrites
                         * @static
                         * @param {google.bigtable.admin.v2.IDataBoostReadLocalWrites} message DataBoostReadLocalWrites message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        DataBoostReadLocalWrites.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a DataBoostReadLocalWrites message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.DataBoostReadLocalWrites
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.DataBoostReadLocalWrites} DataBoostReadLocalWrites
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        DataBoostReadLocalWrites.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.DataBoostReadLocalWrites();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a DataBoostReadLocalWrites message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.DataBoostReadLocalWrites
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.DataBoostReadLocalWrites} DataBoostReadLocalWrites
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        DataBoostReadLocalWrites.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a DataBoostReadLocalWrites message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.DataBoostReadLocalWrites
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        DataBoostReadLocalWrites.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            return null;
                        };
    
                        /**
                         * Creates a DataBoostReadLocalWrites message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.DataBoostReadLocalWrites
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.DataBoostReadLocalWrites} DataBoostReadLocalWrites
                         */
                        DataBoostReadLocalWrites.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.DataBoostReadLocalWrites)
                                return object;
                            return new $root.google.bigtable.admin.v2.DataBoostReadLocalWrites();
                        };
    
                        /**
                         * Creates a plain object from a DataBoostReadLocalWrites message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.DataBoostReadLocalWrites
                         * @static
                         * @param {google.bigtable.admin.v2.DataBoostReadLocalWrites} message DataBoostReadLocalWrites
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        DataBoostReadLocalWrites.toObject = function toObject() {
                            return {};
                        };
    
                        /**
                         * Converts this DataBoostReadLocalWrites to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.DataBoostReadLocalWrites
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        DataBoostReadLocalWrites.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for DataBoostReadLocalWrites
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.DataBoostReadLocalWrites
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        DataBoostReadLocalWrites.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.DataBoostReadLocalWrites";
                        };
    
                        return DataBoostReadLocalWrites;
                    })();
    
                    v2.CheckConsistencyResponse = (function() {
    
                        /**
                         * Properties of a CheckConsistencyResponse.
                         * @memberof google.bigtable.admin.v2
                         * @interface ICheckConsistencyResponse
                         * @property {boolean|null} [consistent] CheckConsistencyResponse consistent
                         */
    
                        /**
                         * Constructs a new CheckConsistencyResponse.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a CheckConsistencyResponse.
                         * @implements ICheckConsistencyResponse
                         * @constructor
                         * @param {google.bigtable.admin.v2.ICheckConsistencyResponse=} [properties] Properties to set
                         */
                        function CheckConsistencyResponse(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * CheckConsistencyResponse consistent.
                         * @member {boolean} consistent
                         * @memberof google.bigtable.admin.v2.CheckConsistencyResponse
                         * @instance
                         */
                        CheckConsistencyResponse.prototype.consistent = false;
    
                        /**
                         * Creates a new CheckConsistencyResponse instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.CheckConsistencyResponse
                         * @static
                         * @param {google.bigtable.admin.v2.ICheckConsistencyResponse=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.CheckConsistencyResponse} CheckConsistencyResponse instance
                         */
                        CheckConsistencyResponse.create = function create(properties) {
                            return new CheckConsistencyResponse(properties);
                        };
    
                        /**
                         * Encodes the specified CheckConsistencyResponse message. Does not implicitly {@link google.bigtable.admin.v2.CheckConsistencyResponse.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.CheckConsistencyResponse
                         * @static
                         * @param {google.bigtable.admin.v2.ICheckConsistencyResponse} message CheckConsistencyResponse message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CheckConsistencyResponse.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.consistent != null && Object.hasOwnProperty.call(message, "consistent"))
                                writer.uint32(/* id 1, wireType 0 =*/8).bool(message.consistent);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified CheckConsistencyResponse message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.CheckConsistencyResponse.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.CheckConsistencyResponse
                         * @static
                         * @param {google.bigtable.admin.v2.ICheckConsistencyResponse} message CheckConsistencyResponse message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CheckConsistencyResponse.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a CheckConsistencyResponse message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.CheckConsistencyResponse
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.CheckConsistencyResponse} CheckConsistencyResponse
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CheckConsistencyResponse.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.CheckConsistencyResponse();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.consistent = reader.bool();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a CheckConsistencyResponse message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.CheckConsistencyResponse
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.CheckConsistencyResponse} CheckConsistencyResponse
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CheckConsistencyResponse.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a CheckConsistencyResponse message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.CheckConsistencyResponse
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        CheckConsistencyResponse.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.consistent != null && message.hasOwnProperty("consistent"))
                                if (typeof message.consistent !== "boolean")
                                    return "consistent: boolean expected";
                            return null;
                        };
    
                        /**
                         * Creates a CheckConsistencyResponse message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.CheckConsistencyResponse
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.CheckConsistencyResponse} CheckConsistencyResponse
                         */
                        CheckConsistencyResponse.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.CheckConsistencyResponse)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.CheckConsistencyResponse();
                            if (object.consistent != null)
                                message.consistent = Boolean(object.consistent);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a CheckConsistencyResponse message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.CheckConsistencyResponse
                         * @static
                         * @param {google.bigtable.admin.v2.CheckConsistencyResponse} message CheckConsistencyResponse
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        CheckConsistencyResponse.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults)
                                object.consistent = false;
                            if (message.consistent != null && message.hasOwnProperty("consistent"))
                                object.consistent = message.consistent;
                            return object;
                        };
    
                        /**
                         * Converts this CheckConsistencyResponse to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.CheckConsistencyResponse
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        CheckConsistencyResponse.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for CheckConsistencyResponse
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.CheckConsistencyResponse
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        CheckConsistencyResponse.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.CheckConsistencyResponse";
                        };
    
                        return CheckConsistencyResponse;
                    })();
    
                    v2.SnapshotTableRequest = (function() {
    
                        /**
                         * Properties of a SnapshotTableRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface ISnapshotTableRequest
                         * @property {string|null} [name] SnapshotTableRequest name
                         * @property {string|null} [cluster] SnapshotTableRequest cluster
                         * @property {string|null} [snapshotId] SnapshotTableRequest snapshotId
                         * @property {google.protobuf.IDuration|null} [ttl] SnapshotTableRequest ttl
                         * @property {string|null} [description] SnapshotTableRequest description
                         */
    
                        /**
                         * Constructs a new SnapshotTableRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a SnapshotTableRequest.
                         * @implements ISnapshotTableRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.ISnapshotTableRequest=} [properties] Properties to set
                         */
                        function SnapshotTableRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * SnapshotTableRequest name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.SnapshotTableRequest
                         * @instance
                         */
                        SnapshotTableRequest.prototype.name = "";
    
                        /**
                         * SnapshotTableRequest cluster.
                         * @member {string} cluster
                         * @memberof google.bigtable.admin.v2.SnapshotTableRequest
                         * @instance
                         */
                        SnapshotTableRequest.prototype.cluster = "";
    
                        /**
                         * SnapshotTableRequest snapshotId.
                         * @member {string} snapshotId
                         * @memberof google.bigtable.admin.v2.SnapshotTableRequest
                         * @instance
                         */
                        SnapshotTableRequest.prototype.snapshotId = "";
    
                        /**
                         * SnapshotTableRequest ttl.
                         * @member {google.protobuf.IDuration|null|undefined} ttl
                         * @memberof google.bigtable.admin.v2.SnapshotTableRequest
                         * @instance
                         */
                        SnapshotTableRequest.prototype.ttl = null;
    
                        /**
                         * SnapshotTableRequest description.
                         * @member {string} description
                         * @memberof google.bigtable.admin.v2.SnapshotTableRequest
                         * @instance
                         */
                        SnapshotTableRequest.prototype.description = "";
    
                        /**
                         * Creates a new SnapshotTableRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.SnapshotTableRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ISnapshotTableRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.SnapshotTableRequest} SnapshotTableRequest instance
                         */
                        SnapshotTableRequest.create = function create(properties) {
                            return new SnapshotTableRequest(properties);
                        };
    
                        /**
                         * Encodes the specified SnapshotTableRequest message. Does not implicitly {@link google.bigtable.admin.v2.SnapshotTableRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.SnapshotTableRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ISnapshotTableRequest} message SnapshotTableRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        SnapshotTableRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            if (message.cluster != null && Object.hasOwnProperty.call(message, "cluster"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.cluster);
                            if (message.snapshotId != null && Object.hasOwnProperty.call(message, "snapshotId"))
                                writer.uint32(/* id 3, wireType 2 =*/26).string(message.snapshotId);
                            if (message.ttl != null && Object.hasOwnProperty.call(message, "ttl"))
                                $root.google.protobuf.Duration.encode(message.ttl, writer.uint32(/* id 4, wireType 2 =*/34).fork()).ldelim();
                            if (message.description != null && Object.hasOwnProperty.call(message, "description"))
                                writer.uint32(/* id 5, wireType 2 =*/42).string(message.description);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified SnapshotTableRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.SnapshotTableRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.SnapshotTableRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ISnapshotTableRequest} message SnapshotTableRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        SnapshotTableRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a SnapshotTableRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.SnapshotTableRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.SnapshotTableRequest} SnapshotTableRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        SnapshotTableRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.SnapshotTableRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.cluster = reader.string();
                                        break;
                                    }
                                case 3: {
                                        message.snapshotId = reader.string();
                                        break;
                                    }
                                case 4: {
                                        message.ttl = $root.google.protobuf.Duration.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 5: {
                                        message.description = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a SnapshotTableRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.SnapshotTableRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.SnapshotTableRequest} SnapshotTableRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        SnapshotTableRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a SnapshotTableRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.SnapshotTableRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        SnapshotTableRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            if (message.cluster != null && message.hasOwnProperty("cluster"))
                                if (!$util.isString(message.cluster))
                                    return "cluster: string expected";
                            if (message.snapshotId != null && message.hasOwnProperty("snapshotId"))
                                if (!$util.isString(message.snapshotId))
                                    return "snapshotId: string expected";
                            if (message.ttl != null && message.hasOwnProperty("ttl")) {
                                var error = $root.google.protobuf.Duration.verify(message.ttl);
                                if (error)
                                    return "ttl." + error;
                            }
                            if (message.description != null && message.hasOwnProperty("description"))
                                if (!$util.isString(message.description))
                                    return "description: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a SnapshotTableRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.SnapshotTableRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.SnapshotTableRequest} SnapshotTableRequest
                         */
                        SnapshotTableRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.SnapshotTableRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.SnapshotTableRequest();
                            if (object.name != null)
                                message.name = String(object.name);
                            if (object.cluster != null)
                                message.cluster = String(object.cluster);
                            if (object.snapshotId != null)
                                message.snapshotId = String(object.snapshotId);
                            if (object.ttl != null) {
                                if (typeof object.ttl !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.SnapshotTableRequest.ttl: object expected");
                                message.ttl = $root.google.protobuf.Duration.fromObject(object.ttl);
                            }
                            if (object.description != null)
                                message.description = String(object.description);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a SnapshotTableRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.SnapshotTableRequest
                         * @static
                         * @param {google.bigtable.admin.v2.SnapshotTableRequest} message SnapshotTableRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        SnapshotTableRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.name = "";
                                object.cluster = "";
                                object.snapshotId = "";
                                object.ttl = null;
                                object.description = "";
                            }
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            if (message.cluster != null && message.hasOwnProperty("cluster"))
                                object.cluster = message.cluster;
                            if (message.snapshotId != null && message.hasOwnProperty("snapshotId"))
                                object.snapshotId = message.snapshotId;
                            if (message.ttl != null && message.hasOwnProperty("ttl"))
                                object.ttl = $root.google.protobuf.Duration.toObject(message.ttl, options);
                            if (message.description != null && message.hasOwnProperty("description"))
                                object.description = message.description;
                            return object;
                        };
    
                        /**
                         * Converts this SnapshotTableRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.SnapshotTableRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        SnapshotTableRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for SnapshotTableRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.SnapshotTableRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        SnapshotTableRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.SnapshotTableRequest";
                        };
    
                        return SnapshotTableRequest;
                    })();
    
                    v2.GetSnapshotRequest = (function() {
    
                        /**
                         * Properties of a GetSnapshotRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IGetSnapshotRequest
                         * @property {string|null} [name] GetSnapshotRequest name
                         */
    
                        /**
                         * Constructs a new GetSnapshotRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a GetSnapshotRequest.
                         * @implements IGetSnapshotRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IGetSnapshotRequest=} [properties] Properties to set
                         */
                        function GetSnapshotRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * GetSnapshotRequest name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.GetSnapshotRequest
                         * @instance
                         */
                        GetSnapshotRequest.prototype.name = "";
    
                        /**
                         * Creates a new GetSnapshotRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.GetSnapshotRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IGetSnapshotRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.GetSnapshotRequest} GetSnapshotRequest instance
                         */
                        GetSnapshotRequest.create = function create(properties) {
                            return new GetSnapshotRequest(properties);
                        };
    
                        /**
                         * Encodes the specified GetSnapshotRequest message. Does not implicitly {@link google.bigtable.admin.v2.GetSnapshotRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.GetSnapshotRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IGetSnapshotRequest} message GetSnapshotRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        GetSnapshotRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified GetSnapshotRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.GetSnapshotRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.GetSnapshotRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IGetSnapshotRequest} message GetSnapshotRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        GetSnapshotRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a GetSnapshotRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.GetSnapshotRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.GetSnapshotRequest} GetSnapshotRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        GetSnapshotRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.GetSnapshotRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a GetSnapshotRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.GetSnapshotRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.GetSnapshotRequest} GetSnapshotRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        GetSnapshotRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a GetSnapshotRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.GetSnapshotRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        GetSnapshotRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a GetSnapshotRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.GetSnapshotRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.GetSnapshotRequest} GetSnapshotRequest
                         */
                        GetSnapshotRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.GetSnapshotRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.GetSnapshotRequest();
                            if (object.name != null)
                                message.name = String(object.name);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a GetSnapshotRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.GetSnapshotRequest
                         * @static
                         * @param {google.bigtable.admin.v2.GetSnapshotRequest} message GetSnapshotRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        GetSnapshotRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults)
                                object.name = "";
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            return object;
                        };
    
                        /**
                         * Converts this GetSnapshotRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.GetSnapshotRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        GetSnapshotRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for GetSnapshotRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.GetSnapshotRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        GetSnapshotRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.GetSnapshotRequest";
                        };
    
                        return GetSnapshotRequest;
                    })();
    
                    v2.ListSnapshotsRequest = (function() {
    
                        /**
                         * Properties of a ListSnapshotsRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IListSnapshotsRequest
                         * @property {string|null} [parent] ListSnapshotsRequest parent
                         * @property {number|null} [pageSize] ListSnapshotsRequest pageSize
                         * @property {string|null} [pageToken] ListSnapshotsRequest pageToken
                         */
    
                        /**
                         * Constructs a new ListSnapshotsRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a ListSnapshotsRequest.
                         * @implements IListSnapshotsRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IListSnapshotsRequest=} [properties] Properties to set
                         */
                        function ListSnapshotsRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * ListSnapshotsRequest parent.
                         * @member {string} parent
                         * @memberof google.bigtable.admin.v2.ListSnapshotsRequest
                         * @instance
                         */
                        ListSnapshotsRequest.prototype.parent = "";
    
                        /**
                         * ListSnapshotsRequest pageSize.
                         * @member {number} pageSize
                         * @memberof google.bigtable.admin.v2.ListSnapshotsRequest
                         * @instance
                         */
                        ListSnapshotsRequest.prototype.pageSize = 0;
    
                        /**
                         * ListSnapshotsRequest pageToken.
                         * @member {string} pageToken
                         * @memberof google.bigtable.admin.v2.ListSnapshotsRequest
                         * @instance
                         */
                        ListSnapshotsRequest.prototype.pageToken = "";
    
                        /**
                         * Creates a new ListSnapshotsRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.ListSnapshotsRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IListSnapshotsRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.ListSnapshotsRequest} ListSnapshotsRequest instance
                         */
                        ListSnapshotsRequest.create = function create(properties) {
                            return new ListSnapshotsRequest(properties);
                        };
    
                        /**
                         * Encodes the specified ListSnapshotsRequest message. Does not implicitly {@link google.bigtable.admin.v2.ListSnapshotsRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.ListSnapshotsRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IListSnapshotsRequest} message ListSnapshotsRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListSnapshotsRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.parent != null && Object.hasOwnProperty.call(message, "parent"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.parent);
                            if (message.pageSize != null && Object.hasOwnProperty.call(message, "pageSize"))
                                writer.uint32(/* id 2, wireType 0 =*/16).int32(message.pageSize);
                            if (message.pageToken != null && Object.hasOwnProperty.call(message, "pageToken"))
                                writer.uint32(/* id 3, wireType 2 =*/26).string(message.pageToken);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified ListSnapshotsRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.ListSnapshotsRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.ListSnapshotsRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IListSnapshotsRequest} message ListSnapshotsRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListSnapshotsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a ListSnapshotsRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.ListSnapshotsRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.ListSnapshotsRequest} ListSnapshotsRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListSnapshotsRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.ListSnapshotsRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.parent = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.pageSize = reader.int32();
                                        break;
                                    }
                                case 3: {
                                        message.pageToken = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a ListSnapshotsRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.ListSnapshotsRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.ListSnapshotsRequest} ListSnapshotsRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListSnapshotsRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a ListSnapshotsRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.ListSnapshotsRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        ListSnapshotsRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                if (!$util.isString(message.parent))
                                    return "parent: string expected";
                            if (message.pageSize != null && message.hasOwnProperty("pageSize"))
                                if (!$util.isInteger(message.pageSize))
                                    return "pageSize: integer expected";
                            if (message.pageToken != null && message.hasOwnProperty("pageToken"))
                                if (!$util.isString(message.pageToken))
                                    return "pageToken: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a ListSnapshotsRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.ListSnapshotsRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.ListSnapshotsRequest} ListSnapshotsRequest
                         */
                        ListSnapshotsRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.ListSnapshotsRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.ListSnapshotsRequest();
                            if (object.parent != null)
                                message.parent = String(object.parent);
                            if (object.pageSize != null)
                                message.pageSize = object.pageSize | 0;
                            if (object.pageToken != null)
                                message.pageToken = String(object.pageToken);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a ListSnapshotsRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.ListSnapshotsRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ListSnapshotsRequest} message ListSnapshotsRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        ListSnapshotsRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.parent = "";
                                object.pageSize = 0;
                                object.pageToken = "";
                            }
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                object.parent = message.parent;
                            if (message.pageSize != null && message.hasOwnProperty("pageSize"))
                                object.pageSize = message.pageSize;
                            if (message.pageToken != null && message.hasOwnProperty("pageToken"))
                                object.pageToken = message.pageToken;
                            return object;
                        };
    
                        /**
                         * Converts this ListSnapshotsRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.ListSnapshotsRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        ListSnapshotsRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for ListSnapshotsRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.ListSnapshotsRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        ListSnapshotsRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.ListSnapshotsRequest";
                        };
    
                        return ListSnapshotsRequest;
                    })();
    
                    v2.ListSnapshotsResponse = (function() {
    
                        /**
                         * Properties of a ListSnapshotsResponse.
                         * @memberof google.bigtable.admin.v2
                         * @interface IListSnapshotsResponse
                         * @property {Array.<google.bigtable.admin.v2.ISnapshot>|null} [snapshots] ListSnapshotsResponse snapshots
                         * @property {string|null} [nextPageToken] ListSnapshotsResponse nextPageToken
                         */
    
                        /**
                         * Constructs a new ListSnapshotsResponse.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a ListSnapshotsResponse.
                         * @implements IListSnapshotsResponse
                         * @constructor
                         * @param {google.bigtable.admin.v2.IListSnapshotsResponse=} [properties] Properties to set
                         */
                        function ListSnapshotsResponse(properties) {
                            this.snapshots = [];
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * ListSnapshotsResponse snapshots.
                         * @member {Array.<google.bigtable.admin.v2.ISnapshot>} snapshots
                         * @memberof google.bigtable.admin.v2.ListSnapshotsResponse
                         * @instance
                         */
                        ListSnapshotsResponse.prototype.snapshots = $util.emptyArray;
    
                        /**
                         * ListSnapshotsResponse nextPageToken.
                         * @member {string} nextPageToken
                         * @memberof google.bigtable.admin.v2.ListSnapshotsResponse
                         * @instance
                         */
                        ListSnapshotsResponse.prototype.nextPageToken = "";
    
                        /**
                         * Creates a new ListSnapshotsResponse instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.ListSnapshotsResponse
                         * @static
                         * @param {google.bigtable.admin.v2.IListSnapshotsResponse=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.ListSnapshotsResponse} ListSnapshotsResponse instance
                         */
                        ListSnapshotsResponse.create = function create(properties) {
                            return new ListSnapshotsResponse(properties);
                        };
    
                        /**
                         * Encodes the specified ListSnapshotsResponse message. Does not implicitly {@link google.bigtable.admin.v2.ListSnapshotsResponse.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.ListSnapshotsResponse
                         * @static
                         * @param {google.bigtable.admin.v2.IListSnapshotsResponse} message ListSnapshotsResponse message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListSnapshotsResponse.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.snapshots != null && message.snapshots.length)
                                for (var i = 0; i < message.snapshots.length; ++i)
                                    $root.google.bigtable.admin.v2.Snapshot.encode(message.snapshots[i], writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            if (message.nextPageToken != null && Object.hasOwnProperty.call(message, "nextPageToken"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.nextPageToken);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified ListSnapshotsResponse message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.ListSnapshotsResponse.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.ListSnapshotsResponse
                         * @static
                         * @param {google.bigtable.admin.v2.IListSnapshotsResponse} message ListSnapshotsResponse message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListSnapshotsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a ListSnapshotsResponse message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.ListSnapshotsResponse
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.ListSnapshotsResponse} ListSnapshotsResponse
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListSnapshotsResponse.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.ListSnapshotsResponse();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        if (!(message.snapshots && message.snapshots.length))
                                            message.snapshots = [];
                                        message.snapshots.push($root.google.bigtable.admin.v2.Snapshot.decode(reader, reader.uint32()));
                                        break;
                                    }
                                case 2: {
                                        message.nextPageToken = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a ListSnapshotsResponse message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.ListSnapshotsResponse
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.ListSnapshotsResponse} ListSnapshotsResponse
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListSnapshotsResponse.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a ListSnapshotsResponse message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.ListSnapshotsResponse
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        ListSnapshotsResponse.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.snapshots != null && message.hasOwnProperty("snapshots")) {
                                if (!Array.isArray(message.snapshots))
                                    return "snapshots: array expected";
                                for (var i = 0; i < message.snapshots.length; ++i) {
                                    var error = $root.google.bigtable.admin.v2.Snapshot.verify(message.snapshots[i]);
                                    if (error)
                                        return "snapshots." + error;
                                }
                            }
                            if (message.nextPageToken != null && message.hasOwnProperty("nextPageToken"))
                                if (!$util.isString(message.nextPageToken))
                                    return "nextPageToken: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a ListSnapshotsResponse message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.ListSnapshotsResponse
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.ListSnapshotsResponse} ListSnapshotsResponse
                         */
                        ListSnapshotsResponse.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.ListSnapshotsResponse)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.ListSnapshotsResponse();
                            if (object.snapshots) {
                                if (!Array.isArray(object.snapshots))
                                    throw TypeError(".google.bigtable.admin.v2.ListSnapshotsResponse.snapshots: array expected");
                                message.snapshots = [];
                                for (var i = 0; i < object.snapshots.length; ++i) {
                                    if (typeof object.snapshots[i] !== "object")
                                        throw TypeError(".google.bigtable.admin.v2.ListSnapshotsResponse.snapshots: object expected");
                                    message.snapshots[i] = $root.google.bigtable.admin.v2.Snapshot.fromObject(object.snapshots[i]);
                                }
                            }
                            if (object.nextPageToken != null)
                                message.nextPageToken = String(object.nextPageToken);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a ListSnapshotsResponse message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.ListSnapshotsResponse
                         * @static
                         * @param {google.bigtable.admin.v2.ListSnapshotsResponse} message ListSnapshotsResponse
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        ListSnapshotsResponse.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.arrays || options.defaults)
                                object.snapshots = [];
                            if (options.defaults)
                                object.nextPageToken = "";
                            if (message.snapshots && message.snapshots.length) {
                                object.snapshots = [];
                                for (var j = 0; j < message.snapshots.length; ++j)
                                    object.snapshots[j] = $root.google.bigtable.admin.v2.Snapshot.toObject(message.snapshots[j], options);
                            }
                            if (message.nextPageToken != null && message.hasOwnProperty("nextPageToken"))
                                object.nextPageToken = message.nextPageToken;
                            return object;
                        };
    
                        /**
                         * Converts this ListSnapshotsResponse to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.ListSnapshotsResponse
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        ListSnapshotsResponse.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for ListSnapshotsResponse
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.ListSnapshotsResponse
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        ListSnapshotsResponse.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.ListSnapshotsResponse";
                        };
    
                        return ListSnapshotsResponse;
                    })();
    
                    v2.DeleteSnapshotRequest = (function() {
    
                        /**
                         * Properties of a DeleteSnapshotRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IDeleteSnapshotRequest
                         * @property {string|null} [name] DeleteSnapshotRequest name
                         */
    
                        /**
                         * Constructs a new DeleteSnapshotRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a DeleteSnapshotRequest.
                         * @implements IDeleteSnapshotRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IDeleteSnapshotRequest=} [properties] Properties to set
                         */
                        function DeleteSnapshotRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * DeleteSnapshotRequest name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.DeleteSnapshotRequest
                         * @instance
                         */
                        DeleteSnapshotRequest.prototype.name = "";
    
                        /**
                         * Creates a new DeleteSnapshotRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.DeleteSnapshotRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IDeleteSnapshotRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.DeleteSnapshotRequest} DeleteSnapshotRequest instance
                         */
                        DeleteSnapshotRequest.create = function create(properties) {
                            return new DeleteSnapshotRequest(properties);
                        };
    
                        /**
                         * Encodes the specified DeleteSnapshotRequest message. Does not implicitly {@link google.bigtable.admin.v2.DeleteSnapshotRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.DeleteSnapshotRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IDeleteSnapshotRequest} message DeleteSnapshotRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        DeleteSnapshotRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified DeleteSnapshotRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.DeleteSnapshotRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.DeleteSnapshotRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IDeleteSnapshotRequest} message DeleteSnapshotRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        DeleteSnapshotRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a DeleteSnapshotRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.DeleteSnapshotRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.DeleteSnapshotRequest} DeleteSnapshotRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        DeleteSnapshotRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.DeleteSnapshotRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a DeleteSnapshotRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.DeleteSnapshotRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.DeleteSnapshotRequest} DeleteSnapshotRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        DeleteSnapshotRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a DeleteSnapshotRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.DeleteSnapshotRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        DeleteSnapshotRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a DeleteSnapshotRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.DeleteSnapshotRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.DeleteSnapshotRequest} DeleteSnapshotRequest
                         */
                        DeleteSnapshotRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.DeleteSnapshotRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.DeleteSnapshotRequest();
                            if (object.name != null)
                                message.name = String(object.name);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a DeleteSnapshotRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.DeleteSnapshotRequest
                         * @static
                         * @param {google.bigtable.admin.v2.DeleteSnapshotRequest} message DeleteSnapshotRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        DeleteSnapshotRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults)
                                object.name = "";
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            return object;
                        };
    
                        /**
                         * Converts this DeleteSnapshotRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.DeleteSnapshotRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        DeleteSnapshotRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for DeleteSnapshotRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.DeleteSnapshotRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        DeleteSnapshotRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.DeleteSnapshotRequest";
                        };
    
                        return DeleteSnapshotRequest;
                    })();
    
                    v2.SnapshotTableMetadata = (function() {
    
                        /**
                         * Properties of a SnapshotTableMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @interface ISnapshotTableMetadata
                         * @property {google.bigtable.admin.v2.ISnapshotTableRequest|null} [originalRequest] SnapshotTableMetadata originalRequest
                         * @property {google.protobuf.ITimestamp|null} [requestTime] SnapshotTableMetadata requestTime
                         * @property {google.protobuf.ITimestamp|null} [finishTime] SnapshotTableMetadata finishTime
                         */
    
                        /**
                         * Constructs a new SnapshotTableMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a SnapshotTableMetadata.
                         * @implements ISnapshotTableMetadata
                         * @constructor
                         * @param {google.bigtable.admin.v2.ISnapshotTableMetadata=} [properties] Properties to set
                         */
                        function SnapshotTableMetadata(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * SnapshotTableMetadata originalRequest.
                         * @member {google.bigtable.admin.v2.ISnapshotTableRequest|null|undefined} originalRequest
                         * @memberof google.bigtable.admin.v2.SnapshotTableMetadata
                         * @instance
                         */
                        SnapshotTableMetadata.prototype.originalRequest = null;
    
                        /**
                         * SnapshotTableMetadata requestTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} requestTime
                         * @memberof google.bigtable.admin.v2.SnapshotTableMetadata
                         * @instance
                         */
                        SnapshotTableMetadata.prototype.requestTime = null;
    
                        /**
                         * SnapshotTableMetadata finishTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} finishTime
                         * @memberof google.bigtable.admin.v2.SnapshotTableMetadata
                         * @instance
                         */
                        SnapshotTableMetadata.prototype.finishTime = null;
    
                        /**
                         * Creates a new SnapshotTableMetadata instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.SnapshotTableMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.ISnapshotTableMetadata=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.SnapshotTableMetadata} SnapshotTableMetadata instance
                         */
                        SnapshotTableMetadata.create = function create(properties) {
                            return new SnapshotTableMetadata(properties);
                        };
    
                        /**
                         * Encodes the specified SnapshotTableMetadata message. Does not implicitly {@link google.bigtable.admin.v2.SnapshotTableMetadata.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.SnapshotTableMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.ISnapshotTableMetadata} message SnapshotTableMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        SnapshotTableMetadata.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.originalRequest != null && Object.hasOwnProperty.call(message, "originalRequest"))
                                $root.google.bigtable.admin.v2.SnapshotTableRequest.encode(message.originalRequest, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            if (message.requestTime != null && Object.hasOwnProperty.call(message, "requestTime"))
                                $root.google.protobuf.Timestamp.encode(message.requestTime, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            if (message.finishTime != null && Object.hasOwnProperty.call(message, "finishTime"))
                                $root.google.protobuf.Timestamp.encode(message.finishTime, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified SnapshotTableMetadata message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.SnapshotTableMetadata.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.SnapshotTableMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.ISnapshotTableMetadata} message SnapshotTableMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        SnapshotTableMetadata.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a SnapshotTableMetadata message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.SnapshotTableMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.SnapshotTableMetadata} SnapshotTableMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        SnapshotTableMetadata.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.SnapshotTableMetadata();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.originalRequest = $root.google.bigtable.admin.v2.SnapshotTableRequest.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 2: {
                                        message.requestTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 3: {
                                        message.finishTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a SnapshotTableMetadata message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.SnapshotTableMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.SnapshotTableMetadata} SnapshotTableMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        SnapshotTableMetadata.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a SnapshotTableMetadata message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.SnapshotTableMetadata
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        SnapshotTableMetadata.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.originalRequest != null && message.hasOwnProperty("originalRequest")) {
                                var error = $root.google.bigtable.admin.v2.SnapshotTableRequest.verify(message.originalRequest);
                                if (error)
                                    return "originalRequest." + error;
                            }
                            if (message.requestTime != null && message.hasOwnProperty("requestTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.requestTime);
                                if (error)
                                    return "requestTime." + error;
                            }
                            if (message.finishTime != null && message.hasOwnProperty("finishTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.finishTime);
                                if (error)
                                    return "finishTime." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates a SnapshotTableMetadata message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.SnapshotTableMetadata
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.SnapshotTableMetadata} SnapshotTableMetadata
                         */
                        SnapshotTableMetadata.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.SnapshotTableMetadata)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.SnapshotTableMetadata();
                            if (object.originalRequest != null) {
                                if (typeof object.originalRequest !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.SnapshotTableMetadata.originalRequest: object expected");
                                message.originalRequest = $root.google.bigtable.admin.v2.SnapshotTableRequest.fromObject(object.originalRequest);
                            }
                            if (object.requestTime != null) {
                                if (typeof object.requestTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.SnapshotTableMetadata.requestTime: object expected");
                                message.requestTime = $root.google.protobuf.Timestamp.fromObject(object.requestTime);
                            }
                            if (object.finishTime != null) {
                                if (typeof object.finishTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.SnapshotTableMetadata.finishTime: object expected");
                                message.finishTime = $root.google.protobuf.Timestamp.fromObject(object.finishTime);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a SnapshotTableMetadata message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.SnapshotTableMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.SnapshotTableMetadata} message SnapshotTableMetadata
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        SnapshotTableMetadata.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.originalRequest = null;
                                object.requestTime = null;
                                object.finishTime = null;
                            }
                            if (message.originalRequest != null && message.hasOwnProperty("originalRequest"))
                                object.originalRequest = $root.google.bigtable.admin.v2.SnapshotTableRequest.toObject(message.originalRequest, options);
                            if (message.requestTime != null && message.hasOwnProperty("requestTime"))
                                object.requestTime = $root.google.protobuf.Timestamp.toObject(message.requestTime, options);
                            if (message.finishTime != null && message.hasOwnProperty("finishTime"))
                                object.finishTime = $root.google.protobuf.Timestamp.toObject(message.finishTime, options);
                            return object;
                        };
    
                        /**
                         * Converts this SnapshotTableMetadata to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.SnapshotTableMetadata
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        SnapshotTableMetadata.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for SnapshotTableMetadata
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.SnapshotTableMetadata
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        SnapshotTableMetadata.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.SnapshotTableMetadata";
                        };
    
                        return SnapshotTableMetadata;
                    })();
    
                    v2.CreateTableFromSnapshotMetadata = (function() {
    
                        /**
                         * Properties of a CreateTableFromSnapshotMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @interface ICreateTableFromSnapshotMetadata
                         * @property {google.bigtable.admin.v2.ICreateTableFromSnapshotRequest|null} [originalRequest] CreateTableFromSnapshotMetadata originalRequest
                         * @property {google.protobuf.ITimestamp|null} [requestTime] CreateTableFromSnapshotMetadata requestTime
                         * @property {google.protobuf.ITimestamp|null} [finishTime] CreateTableFromSnapshotMetadata finishTime
                         */
    
                        /**
                         * Constructs a new CreateTableFromSnapshotMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a CreateTableFromSnapshotMetadata.
                         * @implements ICreateTableFromSnapshotMetadata
                         * @constructor
                         * @param {google.bigtable.admin.v2.ICreateTableFromSnapshotMetadata=} [properties] Properties to set
                         */
                        function CreateTableFromSnapshotMetadata(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * CreateTableFromSnapshotMetadata originalRequest.
                         * @member {google.bigtable.admin.v2.ICreateTableFromSnapshotRequest|null|undefined} originalRequest
                         * @memberof google.bigtable.admin.v2.CreateTableFromSnapshotMetadata
                         * @instance
                         */
                        CreateTableFromSnapshotMetadata.prototype.originalRequest = null;
    
                        /**
                         * CreateTableFromSnapshotMetadata requestTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} requestTime
                         * @memberof google.bigtable.admin.v2.CreateTableFromSnapshotMetadata
                         * @instance
                         */
                        CreateTableFromSnapshotMetadata.prototype.requestTime = null;
    
                        /**
                         * CreateTableFromSnapshotMetadata finishTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} finishTime
                         * @memberof google.bigtable.admin.v2.CreateTableFromSnapshotMetadata
                         * @instance
                         */
                        CreateTableFromSnapshotMetadata.prototype.finishTime = null;
    
                        /**
                         * Creates a new CreateTableFromSnapshotMetadata instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.CreateTableFromSnapshotMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateTableFromSnapshotMetadata=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.CreateTableFromSnapshotMetadata} CreateTableFromSnapshotMetadata instance
                         */
                        CreateTableFromSnapshotMetadata.create = function create(properties) {
                            return new CreateTableFromSnapshotMetadata(properties);
                        };
    
                        /**
                         * Encodes the specified CreateTableFromSnapshotMetadata message. Does not implicitly {@link google.bigtable.admin.v2.CreateTableFromSnapshotMetadata.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.CreateTableFromSnapshotMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateTableFromSnapshotMetadata} message CreateTableFromSnapshotMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CreateTableFromSnapshotMetadata.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.originalRequest != null && Object.hasOwnProperty.call(message, "originalRequest"))
                                $root.google.bigtable.admin.v2.CreateTableFromSnapshotRequest.encode(message.originalRequest, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            if (message.requestTime != null && Object.hasOwnProperty.call(message, "requestTime"))
                                $root.google.protobuf.Timestamp.encode(message.requestTime, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            if (message.finishTime != null && Object.hasOwnProperty.call(message, "finishTime"))
                                $root.google.protobuf.Timestamp.encode(message.finishTime, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified CreateTableFromSnapshotMetadata message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.CreateTableFromSnapshotMetadata.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.CreateTableFromSnapshotMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateTableFromSnapshotMetadata} message CreateTableFromSnapshotMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CreateTableFromSnapshotMetadata.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a CreateTableFromSnapshotMetadata message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.CreateTableFromSnapshotMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.CreateTableFromSnapshotMetadata} CreateTableFromSnapshotMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CreateTableFromSnapshotMetadata.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.CreateTableFromSnapshotMetadata();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.originalRequest = $root.google.bigtable.admin.v2.CreateTableFromSnapshotRequest.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 2: {
                                        message.requestTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 3: {
                                        message.finishTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a CreateTableFromSnapshotMetadata message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.CreateTableFromSnapshotMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.CreateTableFromSnapshotMetadata} CreateTableFromSnapshotMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CreateTableFromSnapshotMetadata.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a CreateTableFromSnapshotMetadata message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.CreateTableFromSnapshotMetadata
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        CreateTableFromSnapshotMetadata.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.originalRequest != null && message.hasOwnProperty("originalRequest")) {
                                var error = $root.google.bigtable.admin.v2.CreateTableFromSnapshotRequest.verify(message.originalRequest);
                                if (error)
                                    return "originalRequest." + error;
                            }
                            if (message.requestTime != null && message.hasOwnProperty("requestTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.requestTime);
                                if (error)
                                    return "requestTime." + error;
                            }
                            if (message.finishTime != null && message.hasOwnProperty("finishTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.finishTime);
                                if (error)
                                    return "finishTime." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates a CreateTableFromSnapshotMetadata message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.CreateTableFromSnapshotMetadata
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.CreateTableFromSnapshotMetadata} CreateTableFromSnapshotMetadata
                         */
                        CreateTableFromSnapshotMetadata.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.CreateTableFromSnapshotMetadata)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.CreateTableFromSnapshotMetadata();
                            if (object.originalRequest != null) {
                                if (typeof object.originalRequest !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.CreateTableFromSnapshotMetadata.originalRequest: object expected");
                                message.originalRequest = $root.google.bigtable.admin.v2.CreateTableFromSnapshotRequest.fromObject(object.originalRequest);
                            }
                            if (object.requestTime != null) {
                                if (typeof object.requestTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.CreateTableFromSnapshotMetadata.requestTime: object expected");
                                message.requestTime = $root.google.protobuf.Timestamp.fromObject(object.requestTime);
                            }
                            if (object.finishTime != null) {
                                if (typeof object.finishTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.CreateTableFromSnapshotMetadata.finishTime: object expected");
                                message.finishTime = $root.google.protobuf.Timestamp.fromObject(object.finishTime);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a CreateTableFromSnapshotMetadata message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.CreateTableFromSnapshotMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.CreateTableFromSnapshotMetadata} message CreateTableFromSnapshotMetadata
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        CreateTableFromSnapshotMetadata.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.originalRequest = null;
                                object.requestTime = null;
                                object.finishTime = null;
                            }
                            if (message.originalRequest != null && message.hasOwnProperty("originalRequest"))
                                object.originalRequest = $root.google.bigtable.admin.v2.CreateTableFromSnapshotRequest.toObject(message.originalRequest, options);
                            if (message.requestTime != null && message.hasOwnProperty("requestTime"))
                                object.requestTime = $root.google.protobuf.Timestamp.toObject(message.requestTime, options);
                            if (message.finishTime != null && message.hasOwnProperty("finishTime"))
                                object.finishTime = $root.google.protobuf.Timestamp.toObject(message.finishTime, options);
                            return object;
                        };
    
                        /**
                         * Converts this CreateTableFromSnapshotMetadata to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.CreateTableFromSnapshotMetadata
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        CreateTableFromSnapshotMetadata.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for CreateTableFromSnapshotMetadata
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.CreateTableFromSnapshotMetadata
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        CreateTableFromSnapshotMetadata.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.CreateTableFromSnapshotMetadata";
                        };
    
                        return CreateTableFromSnapshotMetadata;
                    })();
    
                    v2.CreateBackupRequest = (function() {
    
                        /**
                         * Properties of a CreateBackupRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface ICreateBackupRequest
                         * @property {string|null} [parent] CreateBackupRequest parent
                         * @property {string|null} [backupId] CreateBackupRequest backupId
                         * @property {google.bigtable.admin.v2.IBackup|null} [backup] CreateBackupRequest backup
                         */
    
                        /**
                         * Constructs a new CreateBackupRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a CreateBackupRequest.
                         * @implements ICreateBackupRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.ICreateBackupRequest=} [properties] Properties to set
                         */
                        function CreateBackupRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * CreateBackupRequest parent.
                         * @member {string} parent
                         * @memberof google.bigtable.admin.v2.CreateBackupRequest
                         * @instance
                         */
                        CreateBackupRequest.prototype.parent = "";
    
                        /**
                         * CreateBackupRequest backupId.
                         * @member {string} backupId
                         * @memberof google.bigtable.admin.v2.CreateBackupRequest
                         * @instance
                         */
                        CreateBackupRequest.prototype.backupId = "";
    
                        /**
                         * CreateBackupRequest backup.
                         * @member {google.bigtable.admin.v2.IBackup|null|undefined} backup
                         * @memberof google.bigtable.admin.v2.CreateBackupRequest
                         * @instance
                         */
                        CreateBackupRequest.prototype.backup = null;
    
                        /**
                         * Creates a new CreateBackupRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.CreateBackupRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateBackupRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.CreateBackupRequest} CreateBackupRequest instance
                         */
                        CreateBackupRequest.create = function create(properties) {
                            return new CreateBackupRequest(properties);
                        };
    
                        /**
                         * Encodes the specified CreateBackupRequest message. Does not implicitly {@link google.bigtable.admin.v2.CreateBackupRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.CreateBackupRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateBackupRequest} message CreateBackupRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CreateBackupRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.parent != null && Object.hasOwnProperty.call(message, "parent"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.parent);
                            if (message.backupId != null && Object.hasOwnProperty.call(message, "backupId"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.backupId);
                            if (message.backup != null && Object.hasOwnProperty.call(message, "backup"))
                                $root.google.bigtable.admin.v2.Backup.encode(message.backup, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified CreateBackupRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.CreateBackupRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.CreateBackupRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateBackupRequest} message CreateBackupRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CreateBackupRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a CreateBackupRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.CreateBackupRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.CreateBackupRequest} CreateBackupRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CreateBackupRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.CreateBackupRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.parent = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.backupId = reader.string();
                                        break;
                                    }
                                case 3: {
                                        message.backup = $root.google.bigtable.admin.v2.Backup.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a CreateBackupRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.CreateBackupRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.CreateBackupRequest} CreateBackupRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CreateBackupRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a CreateBackupRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.CreateBackupRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        CreateBackupRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                if (!$util.isString(message.parent))
                                    return "parent: string expected";
                            if (message.backupId != null && message.hasOwnProperty("backupId"))
                                if (!$util.isString(message.backupId))
                                    return "backupId: string expected";
                            if (message.backup != null && message.hasOwnProperty("backup")) {
                                var error = $root.google.bigtable.admin.v2.Backup.verify(message.backup);
                                if (error)
                                    return "backup." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates a CreateBackupRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.CreateBackupRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.CreateBackupRequest} CreateBackupRequest
                         */
                        CreateBackupRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.CreateBackupRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.CreateBackupRequest();
                            if (object.parent != null)
                                message.parent = String(object.parent);
                            if (object.backupId != null)
                                message.backupId = String(object.backupId);
                            if (object.backup != null) {
                                if (typeof object.backup !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.CreateBackupRequest.backup: object expected");
                                message.backup = $root.google.bigtable.admin.v2.Backup.fromObject(object.backup);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a CreateBackupRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.CreateBackupRequest
                         * @static
                         * @param {google.bigtable.admin.v2.CreateBackupRequest} message CreateBackupRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        CreateBackupRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.parent = "";
                                object.backupId = "";
                                object.backup = null;
                            }
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                object.parent = message.parent;
                            if (message.backupId != null && message.hasOwnProperty("backupId"))
                                object.backupId = message.backupId;
                            if (message.backup != null && message.hasOwnProperty("backup"))
                                object.backup = $root.google.bigtable.admin.v2.Backup.toObject(message.backup, options);
                            return object;
                        };
    
                        /**
                         * Converts this CreateBackupRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.CreateBackupRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        CreateBackupRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for CreateBackupRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.CreateBackupRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        CreateBackupRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.CreateBackupRequest";
                        };
    
                        return CreateBackupRequest;
                    })();
    
                    v2.CreateBackupMetadata = (function() {
    
                        /**
                         * Properties of a CreateBackupMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @interface ICreateBackupMetadata
                         * @property {string|null} [name] CreateBackupMetadata name
                         * @property {string|null} [sourceTable] CreateBackupMetadata sourceTable
                         * @property {google.protobuf.ITimestamp|null} [startTime] CreateBackupMetadata startTime
                         * @property {google.protobuf.ITimestamp|null} [endTime] CreateBackupMetadata endTime
                         */
    
                        /**
                         * Constructs a new CreateBackupMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a CreateBackupMetadata.
                         * @implements ICreateBackupMetadata
                         * @constructor
                         * @param {google.bigtable.admin.v2.ICreateBackupMetadata=} [properties] Properties to set
                         */
                        function CreateBackupMetadata(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * CreateBackupMetadata name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.CreateBackupMetadata
                         * @instance
                         */
                        CreateBackupMetadata.prototype.name = "";
    
                        /**
                         * CreateBackupMetadata sourceTable.
                         * @member {string} sourceTable
                         * @memberof google.bigtable.admin.v2.CreateBackupMetadata
                         * @instance
                         */
                        CreateBackupMetadata.prototype.sourceTable = "";
    
                        /**
                         * CreateBackupMetadata startTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} startTime
                         * @memberof google.bigtable.admin.v2.CreateBackupMetadata
                         * @instance
                         */
                        CreateBackupMetadata.prototype.startTime = null;
    
                        /**
                         * CreateBackupMetadata endTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} endTime
                         * @memberof google.bigtable.admin.v2.CreateBackupMetadata
                         * @instance
                         */
                        CreateBackupMetadata.prototype.endTime = null;
    
                        /**
                         * Creates a new CreateBackupMetadata instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.CreateBackupMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateBackupMetadata=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.CreateBackupMetadata} CreateBackupMetadata instance
                         */
                        CreateBackupMetadata.create = function create(properties) {
                            return new CreateBackupMetadata(properties);
                        };
    
                        /**
                         * Encodes the specified CreateBackupMetadata message. Does not implicitly {@link google.bigtable.admin.v2.CreateBackupMetadata.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.CreateBackupMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateBackupMetadata} message CreateBackupMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CreateBackupMetadata.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            if (message.sourceTable != null && Object.hasOwnProperty.call(message, "sourceTable"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.sourceTable);
                            if (message.startTime != null && Object.hasOwnProperty.call(message, "startTime"))
                                $root.google.protobuf.Timestamp.encode(message.startTime, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            if (message.endTime != null && Object.hasOwnProperty.call(message, "endTime"))
                                $root.google.protobuf.Timestamp.encode(message.endTime, writer.uint32(/* id 4, wireType 2 =*/34).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified CreateBackupMetadata message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.CreateBackupMetadata.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.CreateBackupMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateBackupMetadata} message CreateBackupMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CreateBackupMetadata.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a CreateBackupMetadata message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.CreateBackupMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.CreateBackupMetadata} CreateBackupMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CreateBackupMetadata.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.CreateBackupMetadata();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.sourceTable = reader.string();
                                        break;
                                    }
                                case 3: {
                                        message.startTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 4: {
                                        message.endTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a CreateBackupMetadata message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.CreateBackupMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.CreateBackupMetadata} CreateBackupMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CreateBackupMetadata.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a CreateBackupMetadata message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.CreateBackupMetadata
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        CreateBackupMetadata.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            if (message.sourceTable != null && message.hasOwnProperty("sourceTable"))
                                if (!$util.isString(message.sourceTable))
                                    return "sourceTable: string expected";
                            if (message.startTime != null && message.hasOwnProperty("startTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.startTime);
                                if (error)
                                    return "startTime." + error;
                            }
                            if (message.endTime != null && message.hasOwnProperty("endTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.endTime);
                                if (error)
                                    return "endTime." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates a CreateBackupMetadata message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.CreateBackupMetadata
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.CreateBackupMetadata} CreateBackupMetadata
                         */
                        CreateBackupMetadata.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.CreateBackupMetadata)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.CreateBackupMetadata();
                            if (object.name != null)
                                message.name = String(object.name);
                            if (object.sourceTable != null)
                                message.sourceTable = String(object.sourceTable);
                            if (object.startTime != null) {
                                if (typeof object.startTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.CreateBackupMetadata.startTime: object expected");
                                message.startTime = $root.google.protobuf.Timestamp.fromObject(object.startTime);
                            }
                            if (object.endTime != null) {
                                if (typeof object.endTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.CreateBackupMetadata.endTime: object expected");
                                message.endTime = $root.google.protobuf.Timestamp.fromObject(object.endTime);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a CreateBackupMetadata message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.CreateBackupMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.CreateBackupMetadata} message CreateBackupMetadata
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        CreateBackupMetadata.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.name = "";
                                object.sourceTable = "";
                                object.startTime = null;
                                object.endTime = null;
                            }
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            if (message.sourceTable != null && message.hasOwnProperty("sourceTable"))
                                object.sourceTable = message.sourceTable;
                            if (message.startTime != null && message.hasOwnProperty("startTime"))
                                object.startTime = $root.google.protobuf.Timestamp.toObject(message.startTime, options);
                            if (message.endTime != null && message.hasOwnProperty("endTime"))
                                object.endTime = $root.google.protobuf.Timestamp.toObject(message.endTime, options);
                            return object;
                        };
    
                        /**
                         * Converts this CreateBackupMetadata to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.CreateBackupMetadata
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        CreateBackupMetadata.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for CreateBackupMetadata
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.CreateBackupMetadata
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        CreateBackupMetadata.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.CreateBackupMetadata";
                        };
    
                        return CreateBackupMetadata;
                    })();
    
                    v2.UpdateBackupRequest = (function() {
    
                        /**
                         * Properties of an UpdateBackupRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IUpdateBackupRequest
                         * @property {google.bigtable.admin.v2.IBackup|null} [backup] UpdateBackupRequest backup
                         * @property {google.protobuf.IFieldMask|null} [updateMask] UpdateBackupRequest updateMask
                         */
    
                        /**
                         * Constructs a new UpdateBackupRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents an UpdateBackupRequest.
                         * @implements IUpdateBackupRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IUpdateBackupRequest=} [properties] Properties to set
                         */
                        function UpdateBackupRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * UpdateBackupRequest backup.
                         * @member {google.bigtable.admin.v2.IBackup|null|undefined} backup
                         * @memberof google.bigtable.admin.v2.UpdateBackupRequest
                         * @instance
                         */
                        UpdateBackupRequest.prototype.backup = null;
    
                        /**
                         * UpdateBackupRequest updateMask.
                         * @member {google.protobuf.IFieldMask|null|undefined} updateMask
                         * @memberof google.bigtable.admin.v2.UpdateBackupRequest
                         * @instance
                         */
                        UpdateBackupRequest.prototype.updateMask = null;
    
                        /**
                         * Creates a new UpdateBackupRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.UpdateBackupRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IUpdateBackupRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.UpdateBackupRequest} UpdateBackupRequest instance
                         */
                        UpdateBackupRequest.create = function create(properties) {
                            return new UpdateBackupRequest(properties);
                        };
    
                        /**
                         * Encodes the specified UpdateBackupRequest message. Does not implicitly {@link google.bigtable.admin.v2.UpdateBackupRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.UpdateBackupRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IUpdateBackupRequest} message UpdateBackupRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        UpdateBackupRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.backup != null && Object.hasOwnProperty.call(message, "backup"))
                                $root.google.bigtable.admin.v2.Backup.encode(message.backup, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            if (message.updateMask != null && Object.hasOwnProperty.call(message, "updateMask"))
                                $root.google.protobuf.FieldMask.encode(message.updateMask, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified UpdateBackupRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.UpdateBackupRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.UpdateBackupRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IUpdateBackupRequest} message UpdateBackupRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        UpdateBackupRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes an UpdateBackupRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.UpdateBackupRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.UpdateBackupRequest} UpdateBackupRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        UpdateBackupRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.UpdateBackupRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.backup = $root.google.bigtable.admin.v2.Backup.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 2: {
                                        message.updateMask = $root.google.protobuf.FieldMask.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes an UpdateBackupRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.UpdateBackupRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.UpdateBackupRequest} UpdateBackupRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        UpdateBackupRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies an UpdateBackupRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.UpdateBackupRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        UpdateBackupRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.backup != null && message.hasOwnProperty("backup")) {
                                var error = $root.google.bigtable.admin.v2.Backup.verify(message.backup);
                                if (error)
                                    return "backup." + error;
                            }
                            if (message.updateMask != null && message.hasOwnProperty("updateMask")) {
                                var error = $root.google.protobuf.FieldMask.verify(message.updateMask);
                                if (error)
                                    return "updateMask." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates an UpdateBackupRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.UpdateBackupRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.UpdateBackupRequest} UpdateBackupRequest
                         */
                        UpdateBackupRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.UpdateBackupRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.UpdateBackupRequest();
                            if (object.backup != null) {
                                if (typeof object.backup !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.UpdateBackupRequest.backup: object expected");
                                message.backup = $root.google.bigtable.admin.v2.Backup.fromObject(object.backup);
                            }
                            if (object.updateMask != null) {
                                if (typeof object.updateMask !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.UpdateBackupRequest.updateMask: object expected");
                                message.updateMask = $root.google.protobuf.FieldMask.fromObject(object.updateMask);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from an UpdateBackupRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.UpdateBackupRequest
                         * @static
                         * @param {google.bigtable.admin.v2.UpdateBackupRequest} message UpdateBackupRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        UpdateBackupRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.backup = null;
                                object.updateMask = null;
                            }
                            if (message.backup != null && message.hasOwnProperty("backup"))
                                object.backup = $root.google.bigtable.admin.v2.Backup.toObject(message.backup, options);
                            if (message.updateMask != null && message.hasOwnProperty("updateMask"))
                                object.updateMask = $root.google.protobuf.FieldMask.toObject(message.updateMask, options);
                            return object;
                        };
    
                        /**
                         * Converts this UpdateBackupRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.UpdateBackupRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        UpdateBackupRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for UpdateBackupRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.UpdateBackupRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        UpdateBackupRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.UpdateBackupRequest";
                        };
    
                        return UpdateBackupRequest;
                    })();
    
                    v2.GetBackupRequest = (function() {
    
                        /**
                         * Properties of a GetBackupRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IGetBackupRequest
                         * @property {string|null} [name] GetBackupRequest name
                         */
    
                        /**
                         * Constructs a new GetBackupRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a GetBackupRequest.
                         * @implements IGetBackupRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IGetBackupRequest=} [properties] Properties to set
                         */
                        function GetBackupRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * GetBackupRequest name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.GetBackupRequest
                         * @instance
                         */
                        GetBackupRequest.prototype.name = "";
    
                        /**
                         * Creates a new GetBackupRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.GetBackupRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IGetBackupRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.GetBackupRequest} GetBackupRequest instance
                         */
                        GetBackupRequest.create = function create(properties) {
                            return new GetBackupRequest(properties);
                        };
    
                        /**
                         * Encodes the specified GetBackupRequest message. Does not implicitly {@link google.bigtable.admin.v2.GetBackupRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.GetBackupRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IGetBackupRequest} message GetBackupRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        GetBackupRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified GetBackupRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.GetBackupRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.GetBackupRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IGetBackupRequest} message GetBackupRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        GetBackupRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a GetBackupRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.GetBackupRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.GetBackupRequest} GetBackupRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        GetBackupRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.GetBackupRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a GetBackupRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.GetBackupRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.GetBackupRequest} GetBackupRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        GetBackupRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a GetBackupRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.GetBackupRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        GetBackupRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a GetBackupRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.GetBackupRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.GetBackupRequest} GetBackupRequest
                         */
                        GetBackupRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.GetBackupRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.GetBackupRequest();
                            if (object.name != null)
                                message.name = String(object.name);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a GetBackupRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.GetBackupRequest
                         * @static
                         * @param {google.bigtable.admin.v2.GetBackupRequest} message GetBackupRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        GetBackupRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults)
                                object.name = "";
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            return object;
                        };
    
                        /**
                         * Converts this GetBackupRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.GetBackupRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        GetBackupRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for GetBackupRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.GetBackupRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        GetBackupRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.GetBackupRequest";
                        };
    
                        return GetBackupRequest;
                    })();
    
                    v2.DeleteBackupRequest = (function() {
    
                        /**
                         * Properties of a DeleteBackupRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IDeleteBackupRequest
                         * @property {string|null} [name] DeleteBackupRequest name
                         */
    
                        /**
                         * Constructs a new DeleteBackupRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a DeleteBackupRequest.
                         * @implements IDeleteBackupRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IDeleteBackupRequest=} [properties] Properties to set
                         */
                        function DeleteBackupRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * DeleteBackupRequest name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.DeleteBackupRequest
                         * @instance
                         */
                        DeleteBackupRequest.prototype.name = "";
    
                        /**
                         * Creates a new DeleteBackupRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.DeleteBackupRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IDeleteBackupRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.DeleteBackupRequest} DeleteBackupRequest instance
                         */
                        DeleteBackupRequest.create = function create(properties) {
                            return new DeleteBackupRequest(properties);
                        };
    
                        /**
                         * Encodes the specified DeleteBackupRequest message. Does not implicitly {@link google.bigtable.admin.v2.DeleteBackupRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.DeleteBackupRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IDeleteBackupRequest} message DeleteBackupRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        DeleteBackupRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified DeleteBackupRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.DeleteBackupRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.DeleteBackupRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IDeleteBackupRequest} message DeleteBackupRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        DeleteBackupRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a DeleteBackupRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.DeleteBackupRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.DeleteBackupRequest} DeleteBackupRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        DeleteBackupRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.DeleteBackupRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a DeleteBackupRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.DeleteBackupRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.DeleteBackupRequest} DeleteBackupRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        DeleteBackupRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a DeleteBackupRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.DeleteBackupRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        DeleteBackupRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a DeleteBackupRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.DeleteBackupRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.DeleteBackupRequest} DeleteBackupRequest
                         */
                        DeleteBackupRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.DeleteBackupRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.DeleteBackupRequest();
                            if (object.name != null)
                                message.name = String(object.name);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a DeleteBackupRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.DeleteBackupRequest
                         * @static
                         * @param {google.bigtable.admin.v2.DeleteBackupRequest} message DeleteBackupRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        DeleteBackupRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults)
                                object.name = "";
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            return object;
                        };
    
                        /**
                         * Converts this DeleteBackupRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.DeleteBackupRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        DeleteBackupRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for DeleteBackupRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.DeleteBackupRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        DeleteBackupRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.DeleteBackupRequest";
                        };
    
                        return DeleteBackupRequest;
                    })();
    
                    v2.ListBackupsRequest = (function() {
    
                        /**
                         * Properties of a ListBackupsRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IListBackupsRequest
                         * @property {string|null} [parent] ListBackupsRequest parent
                         * @property {string|null} [filter] ListBackupsRequest filter
                         * @property {string|null} [orderBy] ListBackupsRequest orderBy
                         * @property {number|null} [pageSize] ListBackupsRequest pageSize
                         * @property {string|null} [pageToken] ListBackupsRequest pageToken
                         */
    
                        /**
                         * Constructs a new ListBackupsRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a ListBackupsRequest.
                         * @implements IListBackupsRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IListBackupsRequest=} [properties] Properties to set
                         */
                        function ListBackupsRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * ListBackupsRequest parent.
                         * @member {string} parent
                         * @memberof google.bigtable.admin.v2.ListBackupsRequest
                         * @instance
                         */
                        ListBackupsRequest.prototype.parent = "";
    
                        /**
                         * ListBackupsRequest filter.
                         * @member {string} filter
                         * @memberof google.bigtable.admin.v2.ListBackupsRequest
                         * @instance
                         */
                        ListBackupsRequest.prototype.filter = "";
    
                        /**
                         * ListBackupsRequest orderBy.
                         * @member {string} orderBy
                         * @memberof google.bigtable.admin.v2.ListBackupsRequest
                         * @instance
                         */
                        ListBackupsRequest.prototype.orderBy = "";
    
                        /**
                         * ListBackupsRequest pageSize.
                         * @member {number} pageSize
                         * @memberof google.bigtable.admin.v2.ListBackupsRequest
                         * @instance
                         */
                        ListBackupsRequest.prototype.pageSize = 0;
    
                        /**
                         * ListBackupsRequest pageToken.
                         * @member {string} pageToken
                         * @memberof google.bigtable.admin.v2.ListBackupsRequest
                         * @instance
                         */
                        ListBackupsRequest.prototype.pageToken = "";
    
                        /**
                         * Creates a new ListBackupsRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.ListBackupsRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IListBackupsRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.ListBackupsRequest} ListBackupsRequest instance
                         */
                        ListBackupsRequest.create = function create(properties) {
                            return new ListBackupsRequest(properties);
                        };
    
                        /**
                         * Encodes the specified ListBackupsRequest message. Does not implicitly {@link google.bigtable.admin.v2.ListBackupsRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.ListBackupsRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IListBackupsRequest} message ListBackupsRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListBackupsRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.parent != null && Object.hasOwnProperty.call(message, "parent"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.parent);
                            if (message.filter != null && Object.hasOwnProperty.call(message, "filter"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.filter);
                            if (message.orderBy != null && Object.hasOwnProperty.call(message, "orderBy"))
                                writer.uint32(/* id 3, wireType 2 =*/26).string(message.orderBy);
                            if (message.pageSize != null && Object.hasOwnProperty.call(message, "pageSize"))
                                writer.uint32(/* id 4, wireType 0 =*/32).int32(message.pageSize);
                            if (message.pageToken != null && Object.hasOwnProperty.call(message, "pageToken"))
                                writer.uint32(/* id 5, wireType 2 =*/42).string(message.pageToken);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified ListBackupsRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.ListBackupsRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.ListBackupsRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IListBackupsRequest} message ListBackupsRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListBackupsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a ListBackupsRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.ListBackupsRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.ListBackupsRequest} ListBackupsRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListBackupsRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.ListBackupsRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.parent = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.filter = reader.string();
                                        break;
                                    }
                                case 3: {
                                        message.orderBy = reader.string();
                                        break;
                                    }
                                case 4: {
                                        message.pageSize = reader.int32();
                                        break;
                                    }
                                case 5: {
                                        message.pageToken = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a ListBackupsRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.ListBackupsRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.ListBackupsRequest} ListBackupsRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListBackupsRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a ListBackupsRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.ListBackupsRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        ListBackupsRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                if (!$util.isString(message.parent))
                                    return "parent: string expected";
                            if (message.filter != null && message.hasOwnProperty("filter"))
                                if (!$util.isString(message.filter))
                                    return "filter: string expected";
                            if (message.orderBy != null && message.hasOwnProperty("orderBy"))
                                if (!$util.isString(message.orderBy))
                                    return "orderBy: string expected";
                            if (message.pageSize != null && message.hasOwnProperty("pageSize"))
                                if (!$util.isInteger(message.pageSize))
                                    return "pageSize: integer expected";
                            if (message.pageToken != null && message.hasOwnProperty("pageToken"))
                                if (!$util.isString(message.pageToken))
                                    return "pageToken: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a ListBackupsRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.ListBackupsRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.ListBackupsRequest} ListBackupsRequest
                         */
                        ListBackupsRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.ListBackupsRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.ListBackupsRequest();
                            if (object.parent != null)
                                message.parent = String(object.parent);
                            if (object.filter != null)
                                message.filter = String(object.filter);
                            if (object.orderBy != null)
                                message.orderBy = String(object.orderBy);
                            if (object.pageSize != null)
                                message.pageSize = object.pageSize | 0;
                            if (object.pageToken != null)
                                message.pageToken = String(object.pageToken);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a ListBackupsRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.ListBackupsRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ListBackupsRequest} message ListBackupsRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        ListBackupsRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.parent = "";
                                object.filter = "";
                                object.orderBy = "";
                                object.pageSize = 0;
                                object.pageToken = "";
                            }
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                object.parent = message.parent;
                            if (message.filter != null && message.hasOwnProperty("filter"))
                                object.filter = message.filter;
                            if (message.orderBy != null && message.hasOwnProperty("orderBy"))
                                object.orderBy = message.orderBy;
                            if (message.pageSize != null && message.hasOwnProperty("pageSize"))
                                object.pageSize = message.pageSize;
                            if (message.pageToken != null && message.hasOwnProperty("pageToken"))
                                object.pageToken = message.pageToken;
                            return object;
                        };
    
                        /**
                         * Converts this ListBackupsRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.ListBackupsRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        ListBackupsRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for ListBackupsRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.ListBackupsRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        ListBackupsRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.ListBackupsRequest";
                        };
    
                        return ListBackupsRequest;
                    })();
    
                    v2.ListBackupsResponse = (function() {
    
                        /**
                         * Properties of a ListBackupsResponse.
                         * @memberof google.bigtable.admin.v2
                         * @interface IListBackupsResponse
                         * @property {Array.<google.bigtable.admin.v2.IBackup>|null} [backups] ListBackupsResponse backups
                         * @property {string|null} [nextPageToken] ListBackupsResponse nextPageToken
                         */
    
                        /**
                         * Constructs a new ListBackupsResponse.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a ListBackupsResponse.
                         * @implements IListBackupsResponse
                         * @constructor
                         * @param {google.bigtable.admin.v2.IListBackupsResponse=} [properties] Properties to set
                         */
                        function ListBackupsResponse(properties) {
                            this.backups = [];
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * ListBackupsResponse backups.
                         * @member {Array.<google.bigtable.admin.v2.IBackup>} backups
                         * @memberof google.bigtable.admin.v2.ListBackupsResponse
                         * @instance
                         */
                        ListBackupsResponse.prototype.backups = $util.emptyArray;
    
                        /**
                         * ListBackupsResponse nextPageToken.
                         * @member {string} nextPageToken
                         * @memberof google.bigtable.admin.v2.ListBackupsResponse
                         * @instance
                         */
                        ListBackupsResponse.prototype.nextPageToken = "";
    
                        /**
                         * Creates a new ListBackupsResponse instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.ListBackupsResponse
                         * @static
                         * @param {google.bigtable.admin.v2.IListBackupsResponse=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.ListBackupsResponse} ListBackupsResponse instance
                         */
                        ListBackupsResponse.create = function create(properties) {
                            return new ListBackupsResponse(properties);
                        };
    
                        /**
                         * Encodes the specified ListBackupsResponse message. Does not implicitly {@link google.bigtable.admin.v2.ListBackupsResponse.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.ListBackupsResponse
                         * @static
                         * @param {google.bigtable.admin.v2.IListBackupsResponse} message ListBackupsResponse message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListBackupsResponse.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.backups != null && message.backups.length)
                                for (var i = 0; i < message.backups.length; ++i)
                                    $root.google.bigtable.admin.v2.Backup.encode(message.backups[i], writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            if (message.nextPageToken != null && Object.hasOwnProperty.call(message, "nextPageToken"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.nextPageToken);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified ListBackupsResponse message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.ListBackupsResponse.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.ListBackupsResponse
                         * @static
                         * @param {google.bigtable.admin.v2.IListBackupsResponse} message ListBackupsResponse message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListBackupsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a ListBackupsResponse message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.ListBackupsResponse
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.ListBackupsResponse} ListBackupsResponse
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListBackupsResponse.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.ListBackupsResponse();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        if (!(message.backups && message.backups.length))
                                            message.backups = [];
                                        message.backups.push($root.google.bigtable.admin.v2.Backup.decode(reader, reader.uint32()));
                                        break;
                                    }
                                case 2: {
                                        message.nextPageToken = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a ListBackupsResponse message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.ListBackupsResponse
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.ListBackupsResponse} ListBackupsResponse
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListBackupsResponse.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a ListBackupsResponse message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.ListBackupsResponse
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        ListBackupsResponse.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.backups != null && message.hasOwnProperty("backups")) {
                                if (!Array.isArray(message.backups))
                                    return "backups: array expected";
                                for (var i = 0; i < message.backups.length; ++i) {
                                    var error = $root.google.bigtable.admin.v2.Backup.verify(message.backups[i]);
                                    if (error)
                                        return "backups." + error;
                                }
                            }
                            if (message.nextPageToken != null && message.hasOwnProperty("nextPageToken"))
                                if (!$util.isString(message.nextPageToken))
                                    return "nextPageToken: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a ListBackupsResponse message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.ListBackupsResponse
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.ListBackupsResponse} ListBackupsResponse
                         */
                        ListBackupsResponse.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.ListBackupsResponse)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.ListBackupsResponse();
                            if (object.backups) {
                                if (!Array.isArray(object.backups))
                                    throw TypeError(".google.bigtable.admin.v2.ListBackupsResponse.backups: array expected");
                                message.backups = [];
                                for (var i = 0; i < object.backups.length; ++i) {
                                    if (typeof object.backups[i] !== "object")
                                        throw TypeError(".google.bigtable.admin.v2.ListBackupsResponse.backups: object expected");
                                    message.backups[i] = $root.google.bigtable.admin.v2.Backup.fromObject(object.backups[i]);
                                }
                            }
                            if (object.nextPageToken != null)
                                message.nextPageToken = String(object.nextPageToken);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a ListBackupsResponse message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.ListBackupsResponse
                         * @static
                         * @param {google.bigtable.admin.v2.ListBackupsResponse} message ListBackupsResponse
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        ListBackupsResponse.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.arrays || options.defaults)
                                object.backups = [];
                            if (options.defaults)
                                object.nextPageToken = "";
                            if (message.backups && message.backups.length) {
                                object.backups = [];
                                for (var j = 0; j < message.backups.length; ++j)
                                    object.backups[j] = $root.google.bigtable.admin.v2.Backup.toObject(message.backups[j], options);
                            }
                            if (message.nextPageToken != null && message.hasOwnProperty("nextPageToken"))
                                object.nextPageToken = message.nextPageToken;
                            return object;
                        };
    
                        /**
                         * Converts this ListBackupsResponse to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.ListBackupsResponse
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        ListBackupsResponse.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for ListBackupsResponse
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.ListBackupsResponse
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        ListBackupsResponse.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.ListBackupsResponse";
                        };
    
                        return ListBackupsResponse;
                    })();
    
                    v2.CopyBackupRequest = (function() {
    
                        /**
                         * Properties of a CopyBackupRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface ICopyBackupRequest
                         * @property {string|null} [parent] CopyBackupRequest parent
                         * @property {string|null} [backupId] CopyBackupRequest backupId
                         * @property {string|null} [sourceBackup] CopyBackupRequest sourceBackup
                         * @property {google.protobuf.ITimestamp|null} [expireTime] CopyBackupRequest expireTime
                         */
    
                        /**
                         * Constructs a new CopyBackupRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a CopyBackupRequest.
                         * @implements ICopyBackupRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.ICopyBackupRequest=} [properties] Properties to set
                         */
                        function CopyBackupRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * CopyBackupRequest parent.
                         * @member {string} parent
                         * @memberof google.bigtable.admin.v2.CopyBackupRequest
                         * @instance
                         */
                        CopyBackupRequest.prototype.parent = "";
    
                        /**
                         * CopyBackupRequest backupId.
                         * @member {string} backupId
                         * @memberof google.bigtable.admin.v2.CopyBackupRequest
                         * @instance
                         */
                        CopyBackupRequest.prototype.backupId = "";
    
                        /**
                         * CopyBackupRequest sourceBackup.
                         * @member {string} sourceBackup
                         * @memberof google.bigtable.admin.v2.CopyBackupRequest
                         * @instance
                         */
                        CopyBackupRequest.prototype.sourceBackup = "";
    
                        /**
                         * CopyBackupRequest expireTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} expireTime
                         * @memberof google.bigtable.admin.v2.CopyBackupRequest
                         * @instance
                         */
                        CopyBackupRequest.prototype.expireTime = null;
    
                        /**
                         * Creates a new CopyBackupRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.CopyBackupRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ICopyBackupRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.CopyBackupRequest} CopyBackupRequest instance
                         */
                        CopyBackupRequest.create = function create(properties) {
                            return new CopyBackupRequest(properties);
                        };
    
                        /**
                         * Encodes the specified CopyBackupRequest message. Does not implicitly {@link google.bigtable.admin.v2.CopyBackupRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.CopyBackupRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ICopyBackupRequest} message CopyBackupRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CopyBackupRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.parent != null && Object.hasOwnProperty.call(message, "parent"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.parent);
                            if (message.backupId != null && Object.hasOwnProperty.call(message, "backupId"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.backupId);
                            if (message.sourceBackup != null && Object.hasOwnProperty.call(message, "sourceBackup"))
                                writer.uint32(/* id 3, wireType 2 =*/26).string(message.sourceBackup);
                            if (message.expireTime != null && Object.hasOwnProperty.call(message, "expireTime"))
                                $root.google.protobuf.Timestamp.encode(message.expireTime, writer.uint32(/* id 4, wireType 2 =*/34).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified CopyBackupRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.CopyBackupRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.CopyBackupRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ICopyBackupRequest} message CopyBackupRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CopyBackupRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a CopyBackupRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.CopyBackupRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.CopyBackupRequest} CopyBackupRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CopyBackupRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.CopyBackupRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.parent = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.backupId = reader.string();
                                        break;
                                    }
                                case 3: {
                                        message.sourceBackup = reader.string();
                                        break;
                                    }
                                case 4: {
                                        message.expireTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a CopyBackupRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.CopyBackupRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.CopyBackupRequest} CopyBackupRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CopyBackupRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a CopyBackupRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.CopyBackupRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        CopyBackupRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                if (!$util.isString(message.parent))
                                    return "parent: string expected";
                            if (message.backupId != null && message.hasOwnProperty("backupId"))
                                if (!$util.isString(message.backupId))
                                    return "backupId: string expected";
                            if (message.sourceBackup != null && message.hasOwnProperty("sourceBackup"))
                                if (!$util.isString(message.sourceBackup))
                                    return "sourceBackup: string expected";
                            if (message.expireTime != null && message.hasOwnProperty("expireTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.expireTime);
                                if (error)
                                    return "expireTime." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates a CopyBackupRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.CopyBackupRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.CopyBackupRequest} CopyBackupRequest
                         */
                        CopyBackupRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.CopyBackupRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.CopyBackupRequest();
                            if (object.parent != null)
                                message.parent = String(object.parent);
                            if (object.backupId != null)
                                message.backupId = String(object.backupId);
                            if (object.sourceBackup != null)
                                message.sourceBackup = String(object.sourceBackup);
                            if (object.expireTime != null) {
                                if (typeof object.expireTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.CopyBackupRequest.expireTime: object expected");
                                message.expireTime = $root.google.protobuf.Timestamp.fromObject(object.expireTime);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a CopyBackupRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.CopyBackupRequest
                         * @static
                         * @param {google.bigtable.admin.v2.CopyBackupRequest} message CopyBackupRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        CopyBackupRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.parent = "";
                                object.backupId = "";
                                object.sourceBackup = "";
                                object.expireTime = null;
                            }
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                object.parent = message.parent;
                            if (message.backupId != null && message.hasOwnProperty("backupId"))
                                object.backupId = message.backupId;
                            if (message.sourceBackup != null && message.hasOwnProperty("sourceBackup"))
                                object.sourceBackup = message.sourceBackup;
                            if (message.expireTime != null && message.hasOwnProperty("expireTime"))
                                object.expireTime = $root.google.protobuf.Timestamp.toObject(message.expireTime, options);
                            return object;
                        };
    
                        /**
                         * Converts this CopyBackupRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.CopyBackupRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        CopyBackupRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for CopyBackupRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.CopyBackupRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        CopyBackupRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.CopyBackupRequest";
                        };
    
                        return CopyBackupRequest;
                    })();
    
                    v2.CopyBackupMetadata = (function() {
    
                        /**
                         * Properties of a CopyBackupMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @interface ICopyBackupMetadata
                         * @property {string|null} [name] CopyBackupMetadata name
                         * @property {google.bigtable.admin.v2.IBackupInfo|null} [sourceBackupInfo] CopyBackupMetadata sourceBackupInfo
                         * @property {google.bigtable.admin.v2.IOperationProgress|null} [progress] CopyBackupMetadata progress
                         */
    
                        /**
                         * Constructs a new CopyBackupMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a CopyBackupMetadata.
                         * @implements ICopyBackupMetadata
                         * @constructor
                         * @param {google.bigtable.admin.v2.ICopyBackupMetadata=} [properties] Properties to set
                         */
                        function CopyBackupMetadata(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * CopyBackupMetadata name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.CopyBackupMetadata
                         * @instance
                         */
                        CopyBackupMetadata.prototype.name = "";
    
                        /**
                         * CopyBackupMetadata sourceBackupInfo.
                         * @member {google.bigtable.admin.v2.IBackupInfo|null|undefined} sourceBackupInfo
                         * @memberof google.bigtable.admin.v2.CopyBackupMetadata
                         * @instance
                         */
                        CopyBackupMetadata.prototype.sourceBackupInfo = null;
    
                        /**
                         * CopyBackupMetadata progress.
                         * @member {google.bigtable.admin.v2.IOperationProgress|null|undefined} progress
                         * @memberof google.bigtable.admin.v2.CopyBackupMetadata
                         * @instance
                         */
                        CopyBackupMetadata.prototype.progress = null;
    
                        /**
                         * Creates a new CopyBackupMetadata instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.CopyBackupMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.ICopyBackupMetadata=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.CopyBackupMetadata} CopyBackupMetadata instance
                         */
                        CopyBackupMetadata.create = function create(properties) {
                            return new CopyBackupMetadata(properties);
                        };
    
                        /**
                         * Encodes the specified CopyBackupMetadata message. Does not implicitly {@link google.bigtable.admin.v2.CopyBackupMetadata.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.CopyBackupMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.ICopyBackupMetadata} message CopyBackupMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CopyBackupMetadata.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            if (message.sourceBackupInfo != null && Object.hasOwnProperty.call(message, "sourceBackupInfo"))
                                $root.google.bigtable.admin.v2.BackupInfo.encode(message.sourceBackupInfo, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            if (message.progress != null && Object.hasOwnProperty.call(message, "progress"))
                                $root.google.bigtable.admin.v2.OperationProgress.encode(message.progress, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified CopyBackupMetadata message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.CopyBackupMetadata.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.CopyBackupMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.ICopyBackupMetadata} message CopyBackupMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CopyBackupMetadata.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a CopyBackupMetadata message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.CopyBackupMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.CopyBackupMetadata} CopyBackupMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CopyBackupMetadata.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.CopyBackupMetadata();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.sourceBackupInfo = $root.google.bigtable.admin.v2.BackupInfo.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 3: {
                                        message.progress = $root.google.bigtable.admin.v2.OperationProgress.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a CopyBackupMetadata message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.CopyBackupMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.CopyBackupMetadata} CopyBackupMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CopyBackupMetadata.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a CopyBackupMetadata message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.CopyBackupMetadata
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        CopyBackupMetadata.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            if (message.sourceBackupInfo != null && message.hasOwnProperty("sourceBackupInfo")) {
                                var error = $root.google.bigtable.admin.v2.BackupInfo.verify(message.sourceBackupInfo);
                                if (error)
                                    return "sourceBackupInfo." + error;
                            }
                            if (message.progress != null && message.hasOwnProperty("progress")) {
                                var error = $root.google.bigtable.admin.v2.OperationProgress.verify(message.progress);
                                if (error)
                                    return "progress." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates a CopyBackupMetadata message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.CopyBackupMetadata
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.CopyBackupMetadata} CopyBackupMetadata
                         */
                        CopyBackupMetadata.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.CopyBackupMetadata)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.CopyBackupMetadata();
                            if (object.name != null)
                                message.name = String(object.name);
                            if (object.sourceBackupInfo != null) {
                                if (typeof object.sourceBackupInfo !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.CopyBackupMetadata.sourceBackupInfo: object expected");
                                message.sourceBackupInfo = $root.google.bigtable.admin.v2.BackupInfo.fromObject(object.sourceBackupInfo);
                            }
                            if (object.progress != null) {
                                if (typeof object.progress !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.CopyBackupMetadata.progress: object expected");
                                message.progress = $root.google.bigtable.admin.v2.OperationProgress.fromObject(object.progress);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a CopyBackupMetadata message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.CopyBackupMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.CopyBackupMetadata} message CopyBackupMetadata
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        CopyBackupMetadata.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.name = "";
                                object.sourceBackupInfo = null;
                                object.progress = null;
                            }
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            if (message.sourceBackupInfo != null && message.hasOwnProperty("sourceBackupInfo"))
                                object.sourceBackupInfo = $root.google.bigtable.admin.v2.BackupInfo.toObject(message.sourceBackupInfo, options);
                            if (message.progress != null && message.hasOwnProperty("progress"))
                                object.progress = $root.google.bigtable.admin.v2.OperationProgress.toObject(message.progress, options);
                            return object;
                        };
    
                        /**
                         * Converts this CopyBackupMetadata to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.CopyBackupMetadata
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        CopyBackupMetadata.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for CopyBackupMetadata
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.CopyBackupMetadata
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        CopyBackupMetadata.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.CopyBackupMetadata";
                        };
    
                        return CopyBackupMetadata;
                    })();
    
                    v2.CreateAuthorizedViewRequest = (function() {
    
                        /**
                         * Properties of a CreateAuthorizedViewRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface ICreateAuthorizedViewRequest
                         * @property {string|null} [parent] CreateAuthorizedViewRequest parent
                         * @property {string|null} [authorizedViewId] CreateAuthorizedViewRequest authorizedViewId
                         * @property {google.bigtable.admin.v2.IAuthorizedView|null} [authorizedView] CreateAuthorizedViewRequest authorizedView
                         */
    
                        /**
                         * Constructs a new CreateAuthorizedViewRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a CreateAuthorizedViewRequest.
                         * @implements ICreateAuthorizedViewRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.ICreateAuthorizedViewRequest=} [properties] Properties to set
                         */
                        function CreateAuthorizedViewRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * CreateAuthorizedViewRequest parent.
                         * @member {string} parent
                         * @memberof google.bigtable.admin.v2.CreateAuthorizedViewRequest
                         * @instance
                         */
                        CreateAuthorizedViewRequest.prototype.parent = "";
    
                        /**
                         * CreateAuthorizedViewRequest authorizedViewId.
                         * @member {string} authorizedViewId
                         * @memberof google.bigtable.admin.v2.CreateAuthorizedViewRequest
                         * @instance
                         */
                        CreateAuthorizedViewRequest.prototype.authorizedViewId = "";
    
                        /**
                         * CreateAuthorizedViewRequest authorizedView.
                         * @member {google.bigtable.admin.v2.IAuthorizedView|null|undefined} authorizedView
                         * @memberof google.bigtable.admin.v2.CreateAuthorizedViewRequest
                         * @instance
                         */
                        CreateAuthorizedViewRequest.prototype.authorizedView = null;
    
                        /**
                         * Creates a new CreateAuthorizedViewRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.CreateAuthorizedViewRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateAuthorizedViewRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.CreateAuthorizedViewRequest} CreateAuthorizedViewRequest instance
                         */
                        CreateAuthorizedViewRequest.create = function create(properties) {
                            return new CreateAuthorizedViewRequest(properties);
                        };
    
                        /**
                         * Encodes the specified CreateAuthorizedViewRequest message. Does not implicitly {@link google.bigtable.admin.v2.CreateAuthorizedViewRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.CreateAuthorizedViewRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateAuthorizedViewRequest} message CreateAuthorizedViewRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CreateAuthorizedViewRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.parent != null && Object.hasOwnProperty.call(message, "parent"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.parent);
                            if (message.authorizedViewId != null && Object.hasOwnProperty.call(message, "authorizedViewId"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.authorizedViewId);
                            if (message.authorizedView != null && Object.hasOwnProperty.call(message, "authorizedView"))
                                $root.google.bigtable.admin.v2.AuthorizedView.encode(message.authorizedView, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified CreateAuthorizedViewRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.CreateAuthorizedViewRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.CreateAuthorizedViewRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateAuthorizedViewRequest} message CreateAuthorizedViewRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CreateAuthorizedViewRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a CreateAuthorizedViewRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.CreateAuthorizedViewRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.CreateAuthorizedViewRequest} CreateAuthorizedViewRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CreateAuthorizedViewRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.CreateAuthorizedViewRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.parent = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.authorizedViewId = reader.string();
                                        break;
                                    }
                                case 3: {
                                        message.authorizedView = $root.google.bigtable.admin.v2.AuthorizedView.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a CreateAuthorizedViewRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.CreateAuthorizedViewRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.CreateAuthorizedViewRequest} CreateAuthorizedViewRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CreateAuthorizedViewRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a CreateAuthorizedViewRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.CreateAuthorizedViewRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        CreateAuthorizedViewRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                if (!$util.isString(message.parent))
                                    return "parent: string expected";
                            if (message.authorizedViewId != null && message.hasOwnProperty("authorizedViewId"))
                                if (!$util.isString(message.authorizedViewId))
                                    return "authorizedViewId: string expected";
                            if (message.authorizedView != null && message.hasOwnProperty("authorizedView")) {
                                var error = $root.google.bigtable.admin.v2.AuthorizedView.verify(message.authorizedView);
                                if (error)
                                    return "authorizedView." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates a CreateAuthorizedViewRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.CreateAuthorizedViewRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.CreateAuthorizedViewRequest} CreateAuthorizedViewRequest
                         */
                        CreateAuthorizedViewRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.CreateAuthorizedViewRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.CreateAuthorizedViewRequest();
                            if (object.parent != null)
                                message.parent = String(object.parent);
                            if (object.authorizedViewId != null)
                                message.authorizedViewId = String(object.authorizedViewId);
                            if (object.authorizedView != null) {
                                if (typeof object.authorizedView !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.CreateAuthorizedViewRequest.authorizedView: object expected");
                                message.authorizedView = $root.google.bigtable.admin.v2.AuthorizedView.fromObject(object.authorizedView);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a CreateAuthorizedViewRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.CreateAuthorizedViewRequest
                         * @static
                         * @param {google.bigtable.admin.v2.CreateAuthorizedViewRequest} message CreateAuthorizedViewRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        CreateAuthorizedViewRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.parent = "";
                                object.authorizedViewId = "";
                                object.authorizedView = null;
                            }
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                object.parent = message.parent;
                            if (message.authorizedViewId != null && message.hasOwnProperty("authorizedViewId"))
                                object.authorizedViewId = message.authorizedViewId;
                            if (message.authorizedView != null && message.hasOwnProperty("authorizedView"))
                                object.authorizedView = $root.google.bigtable.admin.v2.AuthorizedView.toObject(message.authorizedView, options);
                            return object;
                        };
    
                        /**
                         * Converts this CreateAuthorizedViewRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.CreateAuthorizedViewRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        CreateAuthorizedViewRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for CreateAuthorizedViewRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.CreateAuthorizedViewRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        CreateAuthorizedViewRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.CreateAuthorizedViewRequest";
                        };
    
                        return CreateAuthorizedViewRequest;
                    })();
    
                    v2.CreateAuthorizedViewMetadata = (function() {
    
                        /**
                         * Properties of a CreateAuthorizedViewMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @interface ICreateAuthorizedViewMetadata
                         * @property {google.bigtable.admin.v2.ICreateAuthorizedViewRequest|null} [originalRequest] CreateAuthorizedViewMetadata originalRequest
                         * @property {google.protobuf.ITimestamp|null} [requestTime] CreateAuthorizedViewMetadata requestTime
                         * @property {google.protobuf.ITimestamp|null} [finishTime] CreateAuthorizedViewMetadata finishTime
                         */
    
                        /**
                         * Constructs a new CreateAuthorizedViewMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a CreateAuthorizedViewMetadata.
                         * @implements ICreateAuthorizedViewMetadata
                         * @constructor
                         * @param {google.bigtable.admin.v2.ICreateAuthorizedViewMetadata=} [properties] Properties to set
                         */
                        function CreateAuthorizedViewMetadata(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * CreateAuthorizedViewMetadata originalRequest.
                         * @member {google.bigtable.admin.v2.ICreateAuthorizedViewRequest|null|undefined} originalRequest
                         * @memberof google.bigtable.admin.v2.CreateAuthorizedViewMetadata
                         * @instance
                         */
                        CreateAuthorizedViewMetadata.prototype.originalRequest = null;
    
                        /**
                         * CreateAuthorizedViewMetadata requestTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} requestTime
                         * @memberof google.bigtable.admin.v2.CreateAuthorizedViewMetadata
                         * @instance
                         */
                        CreateAuthorizedViewMetadata.prototype.requestTime = null;
    
                        /**
                         * CreateAuthorizedViewMetadata finishTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} finishTime
                         * @memberof google.bigtable.admin.v2.CreateAuthorizedViewMetadata
                         * @instance
                         */
                        CreateAuthorizedViewMetadata.prototype.finishTime = null;
    
                        /**
                         * Creates a new CreateAuthorizedViewMetadata instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.CreateAuthorizedViewMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateAuthorizedViewMetadata=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.CreateAuthorizedViewMetadata} CreateAuthorizedViewMetadata instance
                         */
                        CreateAuthorizedViewMetadata.create = function create(properties) {
                            return new CreateAuthorizedViewMetadata(properties);
                        };
    
                        /**
                         * Encodes the specified CreateAuthorizedViewMetadata message. Does not implicitly {@link google.bigtable.admin.v2.CreateAuthorizedViewMetadata.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.CreateAuthorizedViewMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateAuthorizedViewMetadata} message CreateAuthorizedViewMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CreateAuthorizedViewMetadata.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.originalRequest != null && Object.hasOwnProperty.call(message, "originalRequest"))
                                $root.google.bigtable.admin.v2.CreateAuthorizedViewRequest.encode(message.originalRequest, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            if (message.requestTime != null && Object.hasOwnProperty.call(message, "requestTime"))
                                $root.google.protobuf.Timestamp.encode(message.requestTime, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            if (message.finishTime != null && Object.hasOwnProperty.call(message, "finishTime"))
                                $root.google.protobuf.Timestamp.encode(message.finishTime, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified CreateAuthorizedViewMetadata message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.CreateAuthorizedViewMetadata.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.CreateAuthorizedViewMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.ICreateAuthorizedViewMetadata} message CreateAuthorizedViewMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CreateAuthorizedViewMetadata.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a CreateAuthorizedViewMetadata message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.CreateAuthorizedViewMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.CreateAuthorizedViewMetadata} CreateAuthorizedViewMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CreateAuthorizedViewMetadata.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.CreateAuthorizedViewMetadata();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.originalRequest = $root.google.bigtable.admin.v2.CreateAuthorizedViewRequest.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 2: {
                                        message.requestTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 3: {
                                        message.finishTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a CreateAuthorizedViewMetadata message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.CreateAuthorizedViewMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.CreateAuthorizedViewMetadata} CreateAuthorizedViewMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CreateAuthorizedViewMetadata.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a CreateAuthorizedViewMetadata message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.CreateAuthorizedViewMetadata
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        CreateAuthorizedViewMetadata.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.originalRequest != null && message.hasOwnProperty("originalRequest")) {
                                var error = $root.google.bigtable.admin.v2.CreateAuthorizedViewRequest.verify(message.originalRequest);
                                if (error)
                                    return "originalRequest." + error;
                            }
                            if (message.requestTime != null && message.hasOwnProperty("requestTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.requestTime);
                                if (error)
                                    return "requestTime." + error;
                            }
                            if (message.finishTime != null && message.hasOwnProperty("finishTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.finishTime);
                                if (error)
                                    return "finishTime." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates a CreateAuthorizedViewMetadata message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.CreateAuthorizedViewMetadata
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.CreateAuthorizedViewMetadata} CreateAuthorizedViewMetadata
                         */
                        CreateAuthorizedViewMetadata.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.CreateAuthorizedViewMetadata)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.CreateAuthorizedViewMetadata();
                            if (object.originalRequest != null) {
                                if (typeof object.originalRequest !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.CreateAuthorizedViewMetadata.originalRequest: object expected");
                                message.originalRequest = $root.google.bigtable.admin.v2.CreateAuthorizedViewRequest.fromObject(object.originalRequest);
                            }
                            if (object.requestTime != null) {
                                if (typeof object.requestTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.CreateAuthorizedViewMetadata.requestTime: object expected");
                                message.requestTime = $root.google.protobuf.Timestamp.fromObject(object.requestTime);
                            }
                            if (object.finishTime != null) {
                                if (typeof object.finishTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.CreateAuthorizedViewMetadata.finishTime: object expected");
                                message.finishTime = $root.google.protobuf.Timestamp.fromObject(object.finishTime);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a CreateAuthorizedViewMetadata message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.CreateAuthorizedViewMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.CreateAuthorizedViewMetadata} message CreateAuthorizedViewMetadata
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        CreateAuthorizedViewMetadata.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.originalRequest = null;
                                object.requestTime = null;
                                object.finishTime = null;
                            }
                            if (message.originalRequest != null && message.hasOwnProperty("originalRequest"))
                                object.originalRequest = $root.google.bigtable.admin.v2.CreateAuthorizedViewRequest.toObject(message.originalRequest, options);
                            if (message.requestTime != null && message.hasOwnProperty("requestTime"))
                                object.requestTime = $root.google.protobuf.Timestamp.toObject(message.requestTime, options);
                            if (message.finishTime != null && message.hasOwnProperty("finishTime"))
                                object.finishTime = $root.google.protobuf.Timestamp.toObject(message.finishTime, options);
                            return object;
                        };
    
                        /**
                         * Converts this CreateAuthorizedViewMetadata to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.CreateAuthorizedViewMetadata
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        CreateAuthorizedViewMetadata.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for CreateAuthorizedViewMetadata
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.CreateAuthorizedViewMetadata
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        CreateAuthorizedViewMetadata.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.CreateAuthorizedViewMetadata";
                        };
    
                        return CreateAuthorizedViewMetadata;
                    })();
    
                    v2.ListAuthorizedViewsRequest = (function() {
    
                        /**
                         * Properties of a ListAuthorizedViewsRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IListAuthorizedViewsRequest
                         * @property {string|null} [parent] ListAuthorizedViewsRequest parent
                         * @property {number|null} [pageSize] ListAuthorizedViewsRequest pageSize
                         * @property {string|null} [pageToken] ListAuthorizedViewsRequest pageToken
                         * @property {google.bigtable.admin.v2.AuthorizedView.ResponseView|null} [view] ListAuthorizedViewsRequest view
                         */
    
                        /**
                         * Constructs a new ListAuthorizedViewsRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a ListAuthorizedViewsRequest.
                         * @implements IListAuthorizedViewsRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IListAuthorizedViewsRequest=} [properties] Properties to set
                         */
                        function ListAuthorizedViewsRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * ListAuthorizedViewsRequest parent.
                         * @member {string} parent
                         * @memberof google.bigtable.admin.v2.ListAuthorizedViewsRequest
                         * @instance
                         */
                        ListAuthorizedViewsRequest.prototype.parent = "";
    
                        /**
                         * ListAuthorizedViewsRequest pageSize.
                         * @member {number} pageSize
                         * @memberof google.bigtable.admin.v2.ListAuthorizedViewsRequest
                         * @instance
                         */
                        ListAuthorizedViewsRequest.prototype.pageSize = 0;
    
                        /**
                         * ListAuthorizedViewsRequest pageToken.
                         * @member {string} pageToken
                         * @memberof google.bigtable.admin.v2.ListAuthorizedViewsRequest
                         * @instance
                         */
                        ListAuthorizedViewsRequest.prototype.pageToken = "";
    
                        /**
                         * ListAuthorizedViewsRequest view.
                         * @member {google.bigtable.admin.v2.AuthorizedView.ResponseView} view
                         * @memberof google.bigtable.admin.v2.ListAuthorizedViewsRequest
                         * @instance
                         */
                        ListAuthorizedViewsRequest.prototype.view = 0;
    
                        /**
                         * Creates a new ListAuthorizedViewsRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.ListAuthorizedViewsRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IListAuthorizedViewsRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.ListAuthorizedViewsRequest} ListAuthorizedViewsRequest instance
                         */
                        ListAuthorizedViewsRequest.create = function create(properties) {
                            return new ListAuthorizedViewsRequest(properties);
                        };
    
                        /**
                         * Encodes the specified ListAuthorizedViewsRequest message. Does not implicitly {@link google.bigtable.admin.v2.ListAuthorizedViewsRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.ListAuthorizedViewsRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IListAuthorizedViewsRequest} message ListAuthorizedViewsRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListAuthorizedViewsRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.parent != null && Object.hasOwnProperty.call(message, "parent"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.parent);
                            if (message.pageSize != null && Object.hasOwnProperty.call(message, "pageSize"))
                                writer.uint32(/* id 2, wireType 0 =*/16).int32(message.pageSize);
                            if (message.pageToken != null && Object.hasOwnProperty.call(message, "pageToken"))
                                writer.uint32(/* id 3, wireType 2 =*/26).string(message.pageToken);
                            if (message.view != null && Object.hasOwnProperty.call(message, "view"))
                                writer.uint32(/* id 4, wireType 0 =*/32).int32(message.view);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified ListAuthorizedViewsRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.ListAuthorizedViewsRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.ListAuthorizedViewsRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IListAuthorizedViewsRequest} message ListAuthorizedViewsRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListAuthorizedViewsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a ListAuthorizedViewsRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.ListAuthorizedViewsRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.ListAuthorizedViewsRequest} ListAuthorizedViewsRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListAuthorizedViewsRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.ListAuthorizedViewsRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.parent = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.pageSize = reader.int32();
                                        break;
                                    }
                                case 3: {
                                        message.pageToken = reader.string();
                                        break;
                                    }
                                case 4: {
                                        message.view = reader.int32();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a ListAuthorizedViewsRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.ListAuthorizedViewsRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.ListAuthorizedViewsRequest} ListAuthorizedViewsRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListAuthorizedViewsRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a ListAuthorizedViewsRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.ListAuthorizedViewsRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        ListAuthorizedViewsRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                if (!$util.isString(message.parent))
                                    return "parent: string expected";
                            if (message.pageSize != null && message.hasOwnProperty("pageSize"))
                                if (!$util.isInteger(message.pageSize))
                                    return "pageSize: integer expected";
                            if (message.pageToken != null && message.hasOwnProperty("pageToken"))
                                if (!$util.isString(message.pageToken))
                                    return "pageToken: string expected";
                            if (message.view != null && message.hasOwnProperty("view"))
                                switch (message.view) {
                                default:
                                    return "view: enum value expected";
                                case 0:
                                case 1:
                                case 2:
                                case 3:
                                    break;
                                }
                            return null;
                        };
    
                        /**
                         * Creates a ListAuthorizedViewsRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.ListAuthorizedViewsRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.ListAuthorizedViewsRequest} ListAuthorizedViewsRequest
                         */
                        ListAuthorizedViewsRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.ListAuthorizedViewsRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.ListAuthorizedViewsRequest();
                            if (object.parent != null)
                                message.parent = String(object.parent);
                            if (object.pageSize != null)
                                message.pageSize = object.pageSize | 0;
                            if (object.pageToken != null)
                                message.pageToken = String(object.pageToken);
                            switch (object.view) {
                            default:
                                if (typeof object.view === "number") {
                                    message.view = object.view;
                                    break;
                                }
                                break;
                            case "RESPONSE_VIEW_UNSPECIFIED":
                            case 0:
                                message.view = 0;
                                break;
                            case "NAME_ONLY":
                            case 1:
                                message.view = 1;
                                break;
                            case "BASIC":
                            case 2:
                                message.view = 2;
                                break;
                            case "FULL":
                            case 3:
                                message.view = 3;
                                break;
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a ListAuthorizedViewsRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.ListAuthorizedViewsRequest
                         * @static
                         * @param {google.bigtable.admin.v2.ListAuthorizedViewsRequest} message ListAuthorizedViewsRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        ListAuthorizedViewsRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.parent = "";
                                object.pageSize = 0;
                                object.pageToken = "";
                                object.view = options.enums === String ? "RESPONSE_VIEW_UNSPECIFIED" : 0;
                            }
                            if (message.parent != null && message.hasOwnProperty("parent"))
                                object.parent = message.parent;
                            if (message.pageSize != null && message.hasOwnProperty("pageSize"))
                                object.pageSize = message.pageSize;
                            if (message.pageToken != null && message.hasOwnProperty("pageToken"))
                                object.pageToken = message.pageToken;
                            if (message.view != null && message.hasOwnProperty("view"))
                                object.view = options.enums === String ? $root.google.bigtable.admin.v2.AuthorizedView.ResponseView[message.view] === undefined ? message.view : $root.google.bigtable.admin.v2.AuthorizedView.ResponseView[message.view] : message.view;
                            return object;
                        };
    
                        /**
                         * Converts this ListAuthorizedViewsRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.ListAuthorizedViewsRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        ListAuthorizedViewsRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for ListAuthorizedViewsRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.ListAuthorizedViewsRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        ListAuthorizedViewsRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.ListAuthorizedViewsRequest";
                        };
    
                        return ListAuthorizedViewsRequest;
                    })();
    
                    v2.ListAuthorizedViewsResponse = (function() {
    
                        /**
                         * Properties of a ListAuthorizedViewsResponse.
                         * @memberof google.bigtable.admin.v2
                         * @interface IListAuthorizedViewsResponse
                         * @property {Array.<google.bigtable.admin.v2.IAuthorizedView>|null} [authorizedViews] ListAuthorizedViewsResponse authorizedViews
                         * @property {string|null} [nextPageToken] ListAuthorizedViewsResponse nextPageToken
                         */
    
                        /**
                         * Constructs a new ListAuthorizedViewsResponse.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a ListAuthorizedViewsResponse.
                         * @implements IListAuthorizedViewsResponse
                         * @constructor
                         * @param {google.bigtable.admin.v2.IListAuthorizedViewsResponse=} [properties] Properties to set
                         */
                        function ListAuthorizedViewsResponse(properties) {
                            this.authorizedViews = [];
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * ListAuthorizedViewsResponse authorizedViews.
                         * @member {Array.<google.bigtable.admin.v2.IAuthorizedView>} authorizedViews
                         * @memberof google.bigtable.admin.v2.ListAuthorizedViewsResponse
                         * @instance
                         */
                        ListAuthorizedViewsResponse.prototype.authorizedViews = $util.emptyArray;
    
                        /**
                         * ListAuthorizedViewsResponse nextPageToken.
                         * @member {string} nextPageToken
                         * @memberof google.bigtable.admin.v2.ListAuthorizedViewsResponse
                         * @instance
                         */
                        ListAuthorizedViewsResponse.prototype.nextPageToken = "";
    
                        /**
                         * Creates a new ListAuthorizedViewsResponse instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.ListAuthorizedViewsResponse
                         * @static
                         * @param {google.bigtable.admin.v2.IListAuthorizedViewsResponse=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.ListAuthorizedViewsResponse} ListAuthorizedViewsResponse instance
                         */
                        ListAuthorizedViewsResponse.create = function create(properties) {
                            return new ListAuthorizedViewsResponse(properties);
                        };
    
                        /**
                         * Encodes the specified ListAuthorizedViewsResponse message. Does not implicitly {@link google.bigtable.admin.v2.ListAuthorizedViewsResponse.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.ListAuthorizedViewsResponse
                         * @static
                         * @param {google.bigtable.admin.v2.IListAuthorizedViewsResponse} message ListAuthorizedViewsResponse message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListAuthorizedViewsResponse.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.authorizedViews != null && message.authorizedViews.length)
                                for (var i = 0; i < message.authorizedViews.length; ++i)
                                    $root.google.bigtable.admin.v2.AuthorizedView.encode(message.authorizedViews[i], writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            if (message.nextPageToken != null && Object.hasOwnProperty.call(message, "nextPageToken"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.nextPageToken);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified ListAuthorizedViewsResponse message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.ListAuthorizedViewsResponse.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.ListAuthorizedViewsResponse
                         * @static
                         * @param {google.bigtable.admin.v2.IListAuthorizedViewsResponse} message ListAuthorizedViewsResponse message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ListAuthorizedViewsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a ListAuthorizedViewsResponse message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.ListAuthorizedViewsResponse
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.ListAuthorizedViewsResponse} ListAuthorizedViewsResponse
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListAuthorizedViewsResponse.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.ListAuthorizedViewsResponse();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        if (!(message.authorizedViews && message.authorizedViews.length))
                                            message.authorizedViews = [];
                                        message.authorizedViews.push($root.google.bigtable.admin.v2.AuthorizedView.decode(reader, reader.uint32()));
                                        break;
                                    }
                                case 2: {
                                        message.nextPageToken = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a ListAuthorizedViewsResponse message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.ListAuthorizedViewsResponse
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.ListAuthorizedViewsResponse} ListAuthorizedViewsResponse
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ListAuthorizedViewsResponse.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a ListAuthorizedViewsResponse message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.ListAuthorizedViewsResponse
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        ListAuthorizedViewsResponse.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.authorizedViews != null && message.hasOwnProperty("authorizedViews")) {
                                if (!Array.isArray(message.authorizedViews))
                                    return "authorizedViews: array expected";
                                for (var i = 0; i < message.authorizedViews.length; ++i) {
                                    var error = $root.google.bigtable.admin.v2.AuthorizedView.verify(message.authorizedViews[i]);
                                    if (error)
                                        return "authorizedViews." + error;
                                }
                            }
                            if (message.nextPageToken != null && message.hasOwnProperty("nextPageToken"))
                                if (!$util.isString(message.nextPageToken))
                                    return "nextPageToken: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a ListAuthorizedViewsResponse message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.ListAuthorizedViewsResponse
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.ListAuthorizedViewsResponse} ListAuthorizedViewsResponse
                         */
                        ListAuthorizedViewsResponse.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.ListAuthorizedViewsResponse)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.ListAuthorizedViewsResponse();
                            if (object.authorizedViews) {
                                if (!Array.isArray(object.authorizedViews))
                                    throw TypeError(".google.bigtable.admin.v2.ListAuthorizedViewsResponse.authorizedViews: array expected");
                                message.authorizedViews = [];
                                for (var i = 0; i < object.authorizedViews.length; ++i) {
                                    if (typeof object.authorizedViews[i] !== "object")
                                        throw TypeError(".google.bigtable.admin.v2.ListAuthorizedViewsResponse.authorizedViews: object expected");
                                    message.authorizedViews[i] = $root.google.bigtable.admin.v2.AuthorizedView.fromObject(object.authorizedViews[i]);
                                }
                            }
                            if (object.nextPageToken != null)
                                message.nextPageToken = String(object.nextPageToken);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a ListAuthorizedViewsResponse message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.ListAuthorizedViewsResponse
                         * @static
                         * @param {google.bigtable.admin.v2.ListAuthorizedViewsResponse} message ListAuthorizedViewsResponse
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        ListAuthorizedViewsResponse.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.arrays || options.defaults)
                                object.authorizedViews = [];
                            if (options.defaults)
                                object.nextPageToken = "";
                            if (message.authorizedViews && message.authorizedViews.length) {
                                object.authorizedViews = [];
                                for (var j = 0; j < message.authorizedViews.length; ++j)
                                    object.authorizedViews[j] = $root.google.bigtable.admin.v2.AuthorizedView.toObject(message.authorizedViews[j], options);
                            }
                            if (message.nextPageToken != null && message.hasOwnProperty("nextPageToken"))
                                object.nextPageToken = message.nextPageToken;
                            return object;
                        };
    
                        /**
                         * Converts this ListAuthorizedViewsResponse to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.ListAuthorizedViewsResponse
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        ListAuthorizedViewsResponse.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for ListAuthorizedViewsResponse
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.ListAuthorizedViewsResponse
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        ListAuthorizedViewsResponse.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.ListAuthorizedViewsResponse";
                        };
    
                        return ListAuthorizedViewsResponse;
                    })();
    
                    v2.GetAuthorizedViewRequest = (function() {
    
                        /**
                         * Properties of a GetAuthorizedViewRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IGetAuthorizedViewRequest
                         * @property {string|null} [name] GetAuthorizedViewRequest name
                         * @property {google.bigtable.admin.v2.AuthorizedView.ResponseView|null} [view] GetAuthorizedViewRequest view
                         */
    
                        /**
                         * Constructs a new GetAuthorizedViewRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a GetAuthorizedViewRequest.
                         * @implements IGetAuthorizedViewRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IGetAuthorizedViewRequest=} [properties] Properties to set
                         */
                        function GetAuthorizedViewRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * GetAuthorizedViewRequest name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.GetAuthorizedViewRequest
                         * @instance
                         */
                        GetAuthorizedViewRequest.prototype.name = "";
    
                        /**
                         * GetAuthorizedViewRequest view.
                         * @member {google.bigtable.admin.v2.AuthorizedView.ResponseView} view
                         * @memberof google.bigtable.admin.v2.GetAuthorizedViewRequest
                         * @instance
                         */
                        GetAuthorizedViewRequest.prototype.view = 0;
    
                        /**
                         * Creates a new GetAuthorizedViewRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.GetAuthorizedViewRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IGetAuthorizedViewRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.GetAuthorizedViewRequest} GetAuthorizedViewRequest instance
                         */
                        GetAuthorizedViewRequest.create = function create(properties) {
                            return new GetAuthorizedViewRequest(properties);
                        };
    
                        /**
                         * Encodes the specified GetAuthorizedViewRequest message. Does not implicitly {@link google.bigtable.admin.v2.GetAuthorizedViewRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.GetAuthorizedViewRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IGetAuthorizedViewRequest} message GetAuthorizedViewRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        GetAuthorizedViewRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            if (message.view != null && Object.hasOwnProperty.call(message, "view"))
                                writer.uint32(/* id 2, wireType 0 =*/16).int32(message.view);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified GetAuthorizedViewRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.GetAuthorizedViewRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.GetAuthorizedViewRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IGetAuthorizedViewRequest} message GetAuthorizedViewRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        GetAuthorizedViewRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a GetAuthorizedViewRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.GetAuthorizedViewRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.GetAuthorizedViewRequest} GetAuthorizedViewRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        GetAuthorizedViewRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.GetAuthorizedViewRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.view = reader.int32();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a GetAuthorizedViewRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.GetAuthorizedViewRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.GetAuthorizedViewRequest} GetAuthorizedViewRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        GetAuthorizedViewRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a GetAuthorizedViewRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.GetAuthorizedViewRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        GetAuthorizedViewRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            if (message.view != null && message.hasOwnProperty("view"))
                                switch (message.view) {
                                default:
                                    return "view: enum value expected";
                                case 0:
                                case 1:
                                case 2:
                                case 3:
                                    break;
                                }
                            return null;
                        };
    
                        /**
                         * Creates a GetAuthorizedViewRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.GetAuthorizedViewRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.GetAuthorizedViewRequest} GetAuthorizedViewRequest
                         */
                        GetAuthorizedViewRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.GetAuthorizedViewRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.GetAuthorizedViewRequest();
                            if (object.name != null)
                                message.name = String(object.name);
                            switch (object.view) {
                            default:
                                if (typeof object.view === "number") {
                                    message.view = object.view;
                                    break;
                                }
                                break;
                            case "RESPONSE_VIEW_UNSPECIFIED":
                            case 0:
                                message.view = 0;
                                break;
                            case "NAME_ONLY":
                            case 1:
                                message.view = 1;
                                break;
                            case "BASIC":
                            case 2:
                                message.view = 2;
                                break;
                            case "FULL":
                            case 3:
                                message.view = 3;
                                break;
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a GetAuthorizedViewRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.GetAuthorizedViewRequest
                         * @static
                         * @param {google.bigtable.admin.v2.GetAuthorizedViewRequest} message GetAuthorizedViewRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        GetAuthorizedViewRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.name = "";
                                object.view = options.enums === String ? "RESPONSE_VIEW_UNSPECIFIED" : 0;
                            }
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            if (message.view != null && message.hasOwnProperty("view"))
                                object.view = options.enums === String ? $root.google.bigtable.admin.v2.AuthorizedView.ResponseView[message.view] === undefined ? message.view : $root.google.bigtable.admin.v2.AuthorizedView.ResponseView[message.view] : message.view;
                            return object;
                        };
    
                        /**
                         * Converts this GetAuthorizedViewRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.GetAuthorizedViewRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        GetAuthorizedViewRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for GetAuthorizedViewRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.GetAuthorizedViewRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        GetAuthorizedViewRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.GetAuthorizedViewRequest";
                        };
    
                        return GetAuthorizedViewRequest;
                    })();
    
                    v2.UpdateAuthorizedViewRequest = (function() {
    
                        /**
                         * Properties of an UpdateAuthorizedViewRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IUpdateAuthorizedViewRequest
                         * @property {google.bigtable.admin.v2.IAuthorizedView|null} [authorizedView] UpdateAuthorizedViewRequest authorizedView
                         * @property {google.protobuf.IFieldMask|null} [updateMask] UpdateAuthorizedViewRequest updateMask
                         * @property {boolean|null} [ignoreWarnings] UpdateAuthorizedViewRequest ignoreWarnings
                         */
    
                        /**
                         * Constructs a new UpdateAuthorizedViewRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents an UpdateAuthorizedViewRequest.
                         * @implements IUpdateAuthorizedViewRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IUpdateAuthorizedViewRequest=} [properties] Properties to set
                         */
                        function UpdateAuthorizedViewRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * UpdateAuthorizedViewRequest authorizedView.
                         * @member {google.bigtable.admin.v2.IAuthorizedView|null|undefined} authorizedView
                         * @memberof google.bigtable.admin.v2.UpdateAuthorizedViewRequest
                         * @instance
                         */
                        UpdateAuthorizedViewRequest.prototype.authorizedView = null;
    
                        /**
                         * UpdateAuthorizedViewRequest updateMask.
                         * @member {google.protobuf.IFieldMask|null|undefined} updateMask
                         * @memberof google.bigtable.admin.v2.UpdateAuthorizedViewRequest
                         * @instance
                         */
                        UpdateAuthorizedViewRequest.prototype.updateMask = null;
    
                        /**
                         * UpdateAuthorizedViewRequest ignoreWarnings.
                         * @member {boolean} ignoreWarnings
                         * @memberof google.bigtable.admin.v2.UpdateAuthorizedViewRequest
                         * @instance
                         */
                        UpdateAuthorizedViewRequest.prototype.ignoreWarnings = false;
    
                        /**
                         * Creates a new UpdateAuthorizedViewRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.UpdateAuthorizedViewRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IUpdateAuthorizedViewRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.UpdateAuthorizedViewRequest} UpdateAuthorizedViewRequest instance
                         */
                        UpdateAuthorizedViewRequest.create = function create(properties) {
                            return new UpdateAuthorizedViewRequest(properties);
                        };
    
                        /**
                         * Encodes the specified UpdateAuthorizedViewRequest message. Does not implicitly {@link google.bigtable.admin.v2.UpdateAuthorizedViewRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.UpdateAuthorizedViewRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IUpdateAuthorizedViewRequest} message UpdateAuthorizedViewRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        UpdateAuthorizedViewRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.authorizedView != null && Object.hasOwnProperty.call(message, "authorizedView"))
                                $root.google.bigtable.admin.v2.AuthorizedView.encode(message.authorizedView, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            if (message.updateMask != null && Object.hasOwnProperty.call(message, "updateMask"))
                                $root.google.protobuf.FieldMask.encode(message.updateMask, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            if (message.ignoreWarnings != null && Object.hasOwnProperty.call(message, "ignoreWarnings"))
                                writer.uint32(/* id 3, wireType 0 =*/24).bool(message.ignoreWarnings);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified UpdateAuthorizedViewRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.UpdateAuthorizedViewRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.UpdateAuthorizedViewRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IUpdateAuthorizedViewRequest} message UpdateAuthorizedViewRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        UpdateAuthorizedViewRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes an UpdateAuthorizedViewRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.UpdateAuthorizedViewRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.UpdateAuthorizedViewRequest} UpdateAuthorizedViewRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        UpdateAuthorizedViewRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.UpdateAuthorizedViewRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.authorizedView = $root.google.bigtable.admin.v2.AuthorizedView.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 2: {
                                        message.updateMask = $root.google.protobuf.FieldMask.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 3: {
                                        message.ignoreWarnings = reader.bool();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes an UpdateAuthorizedViewRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.UpdateAuthorizedViewRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.UpdateAuthorizedViewRequest} UpdateAuthorizedViewRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        UpdateAuthorizedViewRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies an UpdateAuthorizedViewRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.UpdateAuthorizedViewRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        UpdateAuthorizedViewRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.authorizedView != null && message.hasOwnProperty("authorizedView")) {
                                var error = $root.google.bigtable.admin.v2.AuthorizedView.verify(message.authorizedView);
                                if (error)
                                    return "authorizedView." + error;
                            }
                            if (message.updateMask != null && message.hasOwnProperty("updateMask")) {
                                var error = $root.google.protobuf.FieldMask.verify(message.updateMask);
                                if (error)
                                    return "updateMask." + error;
                            }
                            if (message.ignoreWarnings != null && message.hasOwnProperty("ignoreWarnings"))
                                if (typeof message.ignoreWarnings !== "boolean")
                                    return "ignoreWarnings: boolean expected";
                            return null;
                        };
    
                        /**
                         * Creates an UpdateAuthorizedViewRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.UpdateAuthorizedViewRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.UpdateAuthorizedViewRequest} UpdateAuthorizedViewRequest
                         */
                        UpdateAuthorizedViewRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.UpdateAuthorizedViewRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.UpdateAuthorizedViewRequest();
                            if (object.authorizedView != null) {
                                if (typeof object.authorizedView !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.UpdateAuthorizedViewRequest.authorizedView: object expected");
                                message.authorizedView = $root.google.bigtable.admin.v2.AuthorizedView.fromObject(object.authorizedView);
                            }
                            if (object.updateMask != null) {
                                if (typeof object.updateMask !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.UpdateAuthorizedViewRequest.updateMask: object expected");
                                message.updateMask = $root.google.protobuf.FieldMask.fromObject(object.updateMask);
                            }
                            if (object.ignoreWarnings != null)
                                message.ignoreWarnings = Boolean(object.ignoreWarnings);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from an UpdateAuthorizedViewRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.UpdateAuthorizedViewRequest
                         * @static
                         * @param {google.bigtable.admin.v2.UpdateAuthorizedViewRequest} message UpdateAuthorizedViewRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        UpdateAuthorizedViewRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.authorizedView = null;
                                object.updateMask = null;
                                object.ignoreWarnings = false;
                            }
                            if (message.authorizedView != null && message.hasOwnProperty("authorizedView"))
                                object.authorizedView = $root.google.bigtable.admin.v2.AuthorizedView.toObject(message.authorizedView, options);
                            if (message.updateMask != null && message.hasOwnProperty("updateMask"))
                                object.updateMask = $root.google.protobuf.FieldMask.toObject(message.updateMask, options);
                            if (message.ignoreWarnings != null && message.hasOwnProperty("ignoreWarnings"))
                                object.ignoreWarnings = message.ignoreWarnings;
                            return object;
                        };
    
                        /**
                         * Converts this UpdateAuthorizedViewRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.UpdateAuthorizedViewRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        UpdateAuthorizedViewRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for UpdateAuthorizedViewRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.UpdateAuthorizedViewRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        UpdateAuthorizedViewRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.UpdateAuthorizedViewRequest";
                        };
    
                        return UpdateAuthorizedViewRequest;
                    })();
    
                    v2.UpdateAuthorizedViewMetadata = (function() {
    
                        /**
                         * Properties of an UpdateAuthorizedViewMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @interface IUpdateAuthorizedViewMetadata
                         * @property {google.bigtable.admin.v2.IUpdateAuthorizedViewRequest|null} [originalRequest] UpdateAuthorizedViewMetadata originalRequest
                         * @property {google.protobuf.ITimestamp|null} [requestTime] UpdateAuthorizedViewMetadata requestTime
                         * @property {google.protobuf.ITimestamp|null} [finishTime] UpdateAuthorizedViewMetadata finishTime
                         */
    
                        /**
                         * Constructs a new UpdateAuthorizedViewMetadata.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents an UpdateAuthorizedViewMetadata.
                         * @implements IUpdateAuthorizedViewMetadata
                         * @constructor
                         * @param {google.bigtable.admin.v2.IUpdateAuthorizedViewMetadata=} [properties] Properties to set
                         */
                        function UpdateAuthorizedViewMetadata(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * UpdateAuthorizedViewMetadata originalRequest.
                         * @member {google.bigtable.admin.v2.IUpdateAuthorizedViewRequest|null|undefined} originalRequest
                         * @memberof google.bigtable.admin.v2.UpdateAuthorizedViewMetadata
                         * @instance
                         */
                        UpdateAuthorizedViewMetadata.prototype.originalRequest = null;
    
                        /**
                         * UpdateAuthorizedViewMetadata requestTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} requestTime
                         * @memberof google.bigtable.admin.v2.UpdateAuthorizedViewMetadata
                         * @instance
                         */
                        UpdateAuthorizedViewMetadata.prototype.requestTime = null;
    
                        /**
                         * UpdateAuthorizedViewMetadata finishTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} finishTime
                         * @memberof google.bigtable.admin.v2.UpdateAuthorizedViewMetadata
                         * @instance
                         */
                        UpdateAuthorizedViewMetadata.prototype.finishTime = null;
    
                        /**
                         * Creates a new UpdateAuthorizedViewMetadata instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.UpdateAuthorizedViewMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.IUpdateAuthorizedViewMetadata=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.UpdateAuthorizedViewMetadata} UpdateAuthorizedViewMetadata instance
                         */
                        UpdateAuthorizedViewMetadata.create = function create(properties) {
                            return new UpdateAuthorizedViewMetadata(properties);
                        };
    
                        /**
                         * Encodes the specified UpdateAuthorizedViewMetadata message. Does not implicitly {@link google.bigtable.admin.v2.UpdateAuthorizedViewMetadata.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.UpdateAuthorizedViewMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.IUpdateAuthorizedViewMetadata} message UpdateAuthorizedViewMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        UpdateAuthorizedViewMetadata.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.originalRequest != null && Object.hasOwnProperty.call(message, "originalRequest"))
                                $root.google.bigtable.admin.v2.UpdateAuthorizedViewRequest.encode(message.originalRequest, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            if (message.requestTime != null && Object.hasOwnProperty.call(message, "requestTime"))
                                $root.google.protobuf.Timestamp.encode(message.requestTime, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            if (message.finishTime != null && Object.hasOwnProperty.call(message, "finishTime"))
                                $root.google.protobuf.Timestamp.encode(message.finishTime, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified UpdateAuthorizedViewMetadata message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.UpdateAuthorizedViewMetadata.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.UpdateAuthorizedViewMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.IUpdateAuthorizedViewMetadata} message UpdateAuthorizedViewMetadata message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        UpdateAuthorizedViewMetadata.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes an UpdateAuthorizedViewMetadata message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.UpdateAuthorizedViewMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.UpdateAuthorizedViewMetadata} UpdateAuthorizedViewMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        UpdateAuthorizedViewMetadata.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.UpdateAuthorizedViewMetadata();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.originalRequest = $root.google.bigtable.admin.v2.UpdateAuthorizedViewRequest.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 2: {
                                        message.requestTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 3: {
                                        message.finishTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes an UpdateAuthorizedViewMetadata message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.UpdateAuthorizedViewMetadata
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.UpdateAuthorizedViewMetadata} UpdateAuthorizedViewMetadata
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        UpdateAuthorizedViewMetadata.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies an UpdateAuthorizedViewMetadata message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.UpdateAuthorizedViewMetadata
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        UpdateAuthorizedViewMetadata.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.originalRequest != null && message.hasOwnProperty("originalRequest")) {
                                var error = $root.google.bigtable.admin.v2.UpdateAuthorizedViewRequest.verify(message.originalRequest);
                                if (error)
                                    return "originalRequest." + error;
                            }
                            if (message.requestTime != null && message.hasOwnProperty("requestTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.requestTime);
                                if (error)
                                    return "requestTime." + error;
                            }
                            if (message.finishTime != null && message.hasOwnProperty("finishTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.finishTime);
                                if (error)
                                    return "finishTime." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates an UpdateAuthorizedViewMetadata message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.UpdateAuthorizedViewMetadata
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.UpdateAuthorizedViewMetadata} UpdateAuthorizedViewMetadata
                         */
                        UpdateAuthorizedViewMetadata.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.UpdateAuthorizedViewMetadata)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.UpdateAuthorizedViewMetadata();
                            if (object.originalRequest != null) {
                                if (typeof object.originalRequest !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.UpdateAuthorizedViewMetadata.originalRequest: object expected");
                                message.originalRequest = $root.google.bigtable.admin.v2.UpdateAuthorizedViewRequest.fromObject(object.originalRequest);
                            }
                            if (object.requestTime != null) {
                                if (typeof object.requestTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.UpdateAuthorizedViewMetadata.requestTime: object expected");
                                message.requestTime = $root.google.protobuf.Timestamp.fromObject(object.requestTime);
                            }
                            if (object.finishTime != null) {
                                if (typeof object.finishTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.UpdateAuthorizedViewMetadata.finishTime: object expected");
                                message.finishTime = $root.google.protobuf.Timestamp.fromObject(object.finishTime);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from an UpdateAuthorizedViewMetadata message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.UpdateAuthorizedViewMetadata
                         * @static
                         * @param {google.bigtable.admin.v2.UpdateAuthorizedViewMetadata} message UpdateAuthorizedViewMetadata
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        UpdateAuthorizedViewMetadata.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.originalRequest = null;
                                object.requestTime = null;
                                object.finishTime = null;
                            }
                            if (message.originalRequest != null && message.hasOwnProperty("originalRequest"))
                                object.originalRequest = $root.google.bigtable.admin.v2.UpdateAuthorizedViewRequest.toObject(message.originalRequest, options);
                            if (message.requestTime != null && message.hasOwnProperty("requestTime"))
                                object.requestTime = $root.google.protobuf.Timestamp.toObject(message.requestTime, options);
                            if (message.finishTime != null && message.hasOwnProperty("finishTime"))
                                object.finishTime = $root.google.protobuf.Timestamp.toObject(message.finishTime, options);
                            return object;
                        };
    
                        /**
                         * Converts this UpdateAuthorizedViewMetadata to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.UpdateAuthorizedViewMetadata
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        UpdateAuthorizedViewMetadata.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for UpdateAuthorizedViewMetadata
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.UpdateAuthorizedViewMetadata
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        UpdateAuthorizedViewMetadata.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.UpdateAuthorizedViewMetadata";
                        };
    
                        return UpdateAuthorizedViewMetadata;
                    })();
    
                    v2.DeleteAuthorizedViewRequest = (function() {
    
                        /**
                         * Properties of a DeleteAuthorizedViewRequest.
                         * @memberof google.bigtable.admin.v2
                         * @interface IDeleteAuthorizedViewRequest
                         * @property {string|null} [name] DeleteAuthorizedViewRequest name
                         * @property {string|null} [etag] DeleteAuthorizedViewRequest etag
                         */
    
                        /**
                         * Constructs a new DeleteAuthorizedViewRequest.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a DeleteAuthorizedViewRequest.
                         * @implements IDeleteAuthorizedViewRequest
                         * @constructor
                         * @param {google.bigtable.admin.v2.IDeleteAuthorizedViewRequest=} [properties] Properties to set
                         */
                        function DeleteAuthorizedViewRequest(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * DeleteAuthorizedViewRequest name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.DeleteAuthorizedViewRequest
                         * @instance
                         */
                        DeleteAuthorizedViewRequest.prototype.name = "";
    
                        /**
                         * DeleteAuthorizedViewRequest etag.
                         * @member {string} etag
                         * @memberof google.bigtable.admin.v2.DeleteAuthorizedViewRequest
                         * @instance
                         */
                        DeleteAuthorizedViewRequest.prototype.etag = "";
    
                        /**
                         * Creates a new DeleteAuthorizedViewRequest instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.DeleteAuthorizedViewRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IDeleteAuthorizedViewRequest=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.DeleteAuthorizedViewRequest} DeleteAuthorizedViewRequest instance
                         */
                        DeleteAuthorizedViewRequest.create = function create(properties) {
                            return new DeleteAuthorizedViewRequest(properties);
                        };
    
                        /**
                         * Encodes the specified DeleteAuthorizedViewRequest message. Does not implicitly {@link google.bigtable.admin.v2.DeleteAuthorizedViewRequest.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.DeleteAuthorizedViewRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IDeleteAuthorizedViewRequest} message DeleteAuthorizedViewRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        DeleteAuthorizedViewRequest.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            if (message.etag != null && Object.hasOwnProperty.call(message, "etag"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.etag);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified DeleteAuthorizedViewRequest message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.DeleteAuthorizedViewRequest.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.DeleteAuthorizedViewRequest
                         * @static
                         * @param {google.bigtable.admin.v2.IDeleteAuthorizedViewRequest} message DeleteAuthorizedViewRequest message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        DeleteAuthorizedViewRequest.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a DeleteAuthorizedViewRequest message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.DeleteAuthorizedViewRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.DeleteAuthorizedViewRequest} DeleteAuthorizedViewRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        DeleteAuthorizedViewRequest.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.DeleteAuthorizedViewRequest();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.etag = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a DeleteAuthorizedViewRequest message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.DeleteAuthorizedViewRequest
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.DeleteAuthorizedViewRequest} DeleteAuthorizedViewRequest
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        DeleteAuthorizedViewRequest.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a DeleteAuthorizedViewRequest message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.DeleteAuthorizedViewRequest
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        DeleteAuthorizedViewRequest.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            if (message.etag != null && message.hasOwnProperty("etag"))
                                if (!$util.isString(message.etag))
                                    return "etag: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a DeleteAuthorizedViewRequest message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.DeleteAuthorizedViewRequest
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.DeleteAuthorizedViewRequest} DeleteAuthorizedViewRequest
                         */
                        DeleteAuthorizedViewRequest.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.DeleteAuthorizedViewRequest)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.DeleteAuthorizedViewRequest();
                            if (object.name != null)
                                message.name = String(object.name);
                            if (object.etag != null)
                                message.etag = String(object.etag);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a DeleteAuthorizedViewRequest message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.DeleteAuthorizedViewRequest
                         * @static
                         * @param {google.bigtable.admin.v2.DeleteAuthorizedViewRequest} message DeleteAuthorizedViewRequest
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        DeleteAuthorizedViewRequest.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.name = "";
                                object.etag = "";
                            }
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            if (message.etag != null && message.hasOwnProperty("etag"))
                                object.etag = message.etag;
                            return object;
                        };
    
                        /**
                         * Converts this DeleteAuthorizedViewRequest to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.DeleteAuthorizedViewRequest
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        DeleteAuthorizedViewRequest.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for DeleteAuthorizedViewRequest
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.DeleteAuthorizedViewRequest
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        DeleteAuthorizedViewRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.DeleteAuthorizedViewRequest";
                        };
    
                        return DeleteAuthorizedViewRequest;
                    })();
    
                    v2.RestoreInfo = (function() {
    
                        /**
                         * Properties of a RestoreInfo.
                         * @memberof google.bigtable.admin.v2
                         * @interface IRestoreInfo
                         * @property {google.bigtable.admin.v2.RestoreSourceType|null} [sourceType] RestoreInfo sourceType
                         * @property {google.bigtable.admin.v2.IBackupInfo|null} [backupInfo] RestoreInfo backupInfo
                         */
    
                        /**
                         * Constructs a new RestoreInfo.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a RestoreInfo.
                         * @implements IRestoreInfo
                         * @constructor
                         * @param {google.bigtable.admin.v2.IRestoreInfo=} [properties] Properties to set
                         */
                        function RestoreInfo(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * RestoreInfo sourceType.
                         * @member {google.bigtable.admin.v2.RestoreSourceType} sourceType
                         * @memberof google.bigtable.admin.v2.RestoreInfo
                         * @instance
                         */
                        RestoreInfo.prototype.sourceType = 0;
    
                        /**
                         * RestoreInfo backupInfo.
                         * @member {google.bigtable.admin.v2.IBackupInfo|null|undefined} backupInfo
                         * @memberof google.bigtable.admin.v2.RestoreInfo
                         * @instance
                         */
                        RestoreInfo.prototype.backupInfo = null;
    
                        // OneOf field names bound to virtual getters and setters
                        var $oneOfFields;
    
                        /**
                         * RestoreInfo sourceInfo.
                         * @member {"backupInfo"|undefined} sourceInfo
                         * @memberof google.bigtable.admin.v2.RestoreInfo
                         * @instance
                         */
                        Object.defineProperty(RestoreInfo.prototype, "sourceInfo", {
                            get: $util.oneOfGetter($oneOfFields = ["backupInfo"]),
                            set: $util.oneOfSetter($oneOfFields)
                        });
    
                        /**
                         * Creates a new RestoreInfo instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.RestoreInfo
                         * @static
                         * @param {google.bigtable.admin.v2.IRestoreInfo=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.RestoreInfo} RestoreInfo instance
                         */
                        RestoreInfo.create = function create(properties) {
                            return new RestoreInfo(properties);
                        };
    
                        /**
                         * Encodes the specified RestoreInfo message. Does not implicitly {@link google.bigtable.admin.v2.RestoreInfo.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.RestoreInfo
                         * @static
                         * @param {google.bigtable.admin.v2.IRestoreInfo} message RestoreInfo message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        RestoreInfo.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.sourceType != null && Object.hasOwnProperty.call(message, "sourceType"))
                                writer.uint32(/* id 1, wireType 0 =*/8).int32(message.sourceType);
                            if (message.backupInfo != null && Object.hasOwnProperty.call(message, "backupInfo"))
                                $root.google.bigtable.admin.v2.BackupInfo.encode(message.backupInfo, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified RestoreInfo message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.RestoreInfo.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.RestoreInfo
                         * @static
                         * @param {google.bigtable.admin.v2.IRestoreInfo} message RestoreInfo message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        RestoreInfo.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a RestoreInfo message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.RestoreInfo
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.RestoreInfo} RestoreInfo
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        RestoreInfo.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.RestoreInfo();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.sourceType = reader.int32();
                                        break;
                                    }
                                case 2: {
                                        message.backupInfo = $root.google.bigtable.admin.v2.BackupInfo.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a RestoreInfo message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.RestoreInfo
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.RestoreInfo} RestoreInfo
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        RestoreInfo.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a RestoreInfo message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.RestoreInfo
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        RestoreInfo.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            var properties = {};
                            if (message.sourceType != null && message.hasOwnProperty("sourceType"))
                                switch (message.sourceType) {
                                default:
                                    return "sourceType: enum value expected";
                                case 0:
                                case 1:
                                    break;
                                }
                            if (message.backupInfo != null && message.hasOwnProperty("backupInfo")) {
                                properties.sourceInfo = 1;
                                {
                                    var error = $root.google.bigtable.admin.v2.BackupInfo.verify(message.backupInfo);
                                    if (error)
                                        return "backupInfo." + error;
                                }
                            }
                            return null;
                        };
    
                        /**
                         * Creates a RestoreInfo message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.RestoreInfo
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.RestoreInfo} RestoreInfo
                         */
                        RestoreInfo.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.RestoreInfo)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.RestoreInfo();
                            switch (object.sourceType) {
                            default:
                                if (typeof object.sourceType === "number") {
                                    message.sourceType = object.sourceType;
                                    break;
                                }
                                break;
                            case "RESTORE_SOURCE_TYPE_UNSPECIFIED":
                            case 0:
                                message.sourceType = 0;
                                break;
                            case "BACKUP":
                            case 1:
                                message.sourceType = 1;
                                break;
                            }
                            if (object.backupInfo != null) {
                                if (typeof object.backupInfo !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.RestoreInfo.backupInfo: object expected");
                                message.backupInfo = $root.google.bigtable.admin.v2.BackupInfo.fromObject(object.backupInfo);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a RestoreInfo message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.RestoreInfo
                         * @static
                         * @param {google.bigtable.admin.v2.RestoreInfo} message RestoreInfo
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        RestoreInfo.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults)
                                object.sourceType = options.enums === String ? "RESTORE_SOURCE_TYPE_UNSPECIFIED" : 0;
                            if (message.sourceType != null && message.hasOwnProperty("sourceType"))
                                object.sourceType = options.enums === String ? $root.google.bigtable.admin.v2.RestoreSourceType[message.sourceType] === undefined ? message.sourceType : $root.google.bigtable.admin.v2.RestoreSourceType[message.sourceType] : message.sourceType;
                            if (message.backupInfo != null && message.hasOwnProperty("backupInfo")) {
                                object.backupInfo = $root.google.bigtable.admin.v2.BackupInfo.toObject(message.backupInfo, options);
                                if (options.oneofs)
                                    object.sourceInfo = "backupInfo";
                            }
                            return object;
                        };
    
                        /**
                         * Converts this RestoreInfo to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.RestoreInfo
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        RestoreInfo.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for RestoreInfo
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.RestoreInfo
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        RestoreInfo.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.RestoreInfo";
                        };
    
                        return RestoreInfo;
                    })();
    
                    v2.ChangeStreamConfig = (function() {
    
                        /**
                         * Properties of a ChangeStreamConfig.
                         * @memberof google.bigtable.admin.v2
                         * @interface IChangeStreamConfig
                         * @property {google.protobuf.IDuration|null} [retentionPeriod] ChangeStreamConfig retentionPeriod
                         */
    
                        /**
                         * Constructs a new ChangeStreamConfig.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a ChangeStreamConfig.
                         * @implements IChangeStreamConfig
                         * @constructor
                         * @param {google.bigtable.admin.v2.IChangeStreamConfig=} [properties] Properties to set
                         */
                        function ChangeStreamConfig(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * ChangeStreamConfig retentionPeriod.
                         * @member {google.protobuf.IDuration|null|undefined} retentionPeriod
                         * @memberof google.bigtable.admin.v2.ChangeStreamConfig
                         * @instance
                         */
                        ChangeStreamConfig.prototype.retentionPeriod = null;
    
                        /**
                         * Creates a new ChangeStreamConfig instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.ChangeStreamConfig
                         * @static
                         * @param {google.bigtable.admin.v2.IChangeStreamConfig=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.ChangeStreamConfig} ChangeStreamConfig instance
                         */
                        ChangeStreamConfig.create = function create(properties) {
                            return new ChangeStreamConfig(properties);
                        };
    
                        /**
                         * Encodes the specified ChangeStreamConfig message. Does not implicitly {@link google.bigtable.admin.v2.ChangeStreamConfig.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.ChangeStreamConfig
                         * @static
                         * @param {google.bigtable.admin.v2.IChangeStreamConfig} message ChangeStreamConfig message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ChangeStreamConfig.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.retentionPeriod != null && Object.hasOwnProperty.call(message, "retentionPeriod"))
                                $root.google.protobuf.Duration.encode(message.retentionPeriod, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified ChangeStreamConfig message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.ChangeStreamConfig.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.ChangeStreamConfig
                         * @static
                         * @param {google.bigtable.admin.v2.IChangeStreamConfig} message ChangeStreamConfig message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ChangeStreamConfig.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a ChangeStreamConfig message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.ChangeStreamConfig
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.ChangeStreamConfig} ChangeStreamConfig
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ChangeStreamConfig.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.ChangeStreamConfig();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.retentionPeriod = $root.google.protobuf.Duration.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a ChangeStreamConfig message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.ChangeStreamConfig
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.ChangeStreamConfig} ChangeStreamConfig
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ChangeStreamConfig.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a ChangeStreamConfig message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.ChangeStreamConfig
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        ChangeStreamConfig.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.retentionPeriod != null && message.hasOwnProperty("retentionPeriod")) {
                                var error = $root.google.protobuf.Duration.verify(message.retentionPeriod);
                                if (error)
                                    return "retentionPeriod." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates a ChangeStreamConfig message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.ChangeStreamConfig
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.ChangeStreamConfig} ChangeStreamConfig
                         */
                        ChangeStreamConfig.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.ChangeStreamConfig)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.ChangeStreamConfig();
                            if (object.retentionPeriod != null) {
                                if (typeof object.retentionPeriod !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.ChangeStreamConfig.retentionPeriod: object expected");
                                message.retentionPeriod = $root.google.protobuf.Duration.fromObject(object.retentionPeriod);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a ChangeStreamConfig message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.ChangeStreamConfig
                         * @static
                         * @param {google.bigtable.admin.v2.ChangeStreamConfig} message ChangeStreamConfig
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        ChangeStreamConfig.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults)
                                object.retentionPeriod = null;
                            if (message.retentionPeriod != null && message.hasOwnProperty("retentionPeriod"))
                                object.retentionPeriod = $root.google.protobuf.Duration.toObject(message.retentionPeriod, options);
                            return object;
                        };
    
                        /**
                         * Converts this ChangeStreamConfig to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.ChangeStreamConfig
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        ChangeStreamConfig.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for ChangeStreamConfig
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.ChangeStreamConfig
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        ChangeStreamConfig.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.ChangeStreamConfig";
                        };
    
                        return ChangeStreamConfig;
                    })();
    
                    v2.Table = (function() {
    
                        /**
                         * Properties of a Table.
                         * @memberof google.bigtable.admin.v2
                         * @interface ITable
                         * @property {string|null} [name] Table name
                         * @property {Object.<string,google.bigtable.admin.v2.Table.IClusterState>|null} [clusterStates] Table clusterStates
                         * @property {Object.<string,google.bigtable.admin.v2.IColumnFamily>|null} [columnFamilies] Table columnFamilies
                         * @property {google.bigtable.admin.v2.Table.TimestampGranularity|null} [granularity] Table granularity
                         * @property {google.bigtable.admin.v2.IRestoreInfo|null} [restoreInfo] Table restoreInfo
                         * @property {google.bigtable.admin.v2.IChangeStreamConfig|null} [changeStreamConfig] Table changeStreamConfig
                         * @property {boolean|null} [deletionProtection] Table deletionProtection
                         * @property {google.bigtable.admin.v2.Table.IAutomatedBackupPolicy|null} [automatedBackupPolicy] Table automatedBackupPolicy
                         */
    
                        /**
                         * Constructs a new Table.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a Table.
                         * @implements ITable
                         * @constructor
                         * @param {google.bigtable.admin.v2.ITable=} [properties] Properties to set
                         */
                        function Table(properties) {
                            this.clusterStates = {};
                            this.columnFamilies = {};
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * Table name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.Table
                         * @instance
                         */
                        Table.prototype.name = "";
    
                        /**
                         * Table clusterStates.
                         * @member {Object.<string,google.bigtable.admin.v2.Table.IClusterState>} clusterStates
                         * @memberof google.bigtable.admin.v2.Table
                         * @instance
                         */
                        Table.prototype.clusterStates = $util.emptyObject;
    
                        /**
                         * Table columnFamilies.
                         * @member {Object.<string,google.bigtable.admin.v2.IColumnFamily>} columnFamilies
                         * @memberof google.bigtable.admin.v2.Table
                         * @instance
                         */
                        Table.prototype.columnFamilies = $util.emptyObject;
    
                        /**
                         * Table granularity.
                         * @member {google.bigtable.admin.v2.Table.TimestampGranularity} granularity
                         * @memberof google.bigtable.admin.v2.Table
                         * @instance
                         */
                        Table.prototype.granularity = 0;
    
                        /**
                         * Table restoreInfo.
                         * @member {google.bigtable.admin.v2.IRestoreInfo|null|undefined} restoreInfo
                         * @memberof google.bigtable.admin.v2.Table
                         * @instance
                         */
                        Table.prototype.restoreInfo = null;
    
                        /**
                         * Table changeStreamConfig.
                         * @member {google.bigtable.admin.v2.IChangeStreamConfig|null|undefined} changeStreamConfig
                         * @memberof google.bigtable.admin.v2.Table
                         * @instance
                         */
                        Table.prototype.changeStreamConfig = null;
    
                        /**
                         * Table deletionProtection.
                         * @member {boolean} deletionProtection
                         * @memberof google.bigtable.admin.v2.Table
                         * @instance
                         */
                        Table.prototype.deletionProtection = false;
    
                        /**
                         * Table automatedBackupPolicy.
                         * @member {google.bigtable.admin.v2.Table.IAutomatedBackupPolicy|null|undefined} automatedBackupPolicy
                         * @memberof google.bigtable.admin.v2.Table
                         * @instance
                         */
                        Table.prototype.automatedBackupPolicy = null;
    
                        // OneOf field names bound to virtual getters and setters
                        var $oneOfFields;
    
                        /**
                         * Table automatedBackupConfig.
                         * @member {"automatedBackupPolicy"|undefined} automatedBackupConfig
                         * @memberof google.bigtable.admin.v2.Table
                         * @instance
                         */
                        Object.defineProperty(Table.prototype, "automatedBackupConfig", {
                            get: $util.oneOfGetter($oneOfFields = ["automatedBackupPolicy"]),
                            set: $util.oneOfSetter($oneOfFields)
                        });
    
                        /**
                         * Creates a new Table instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.Table
                         * @static
                         * @param {google.bigtable.admin.v2.ITable=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.Table} Table instance
                         */
                        Table.create = function create(properties) {
                            return new Table(properties);
                        };
    
                        /**
                         * Encodes the specified Table message. Does not implicitly {@link google.bigtable.admin.v2.Table.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.Table
                         * @static
                         * @param {google.bigtable.admin.v2.ITable} message Table message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        Table.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            if (message.clusterStates != null && Object.hasOwnProperty.call(message, "clusterStates"))
                                for (var keys = Object.keys(message.clusterStates), i = 0; i < keys.length; ++i) {
                                    writer.uint32(/* id 2, wireType 2 =*/18).fork().uint32(/* id 1, wireType 2 =*/10).string(keys[i]);
                                    $root.google.bigtable.admin.v2.Table.ClusterState.encode(message.clusterStates[keys[i]], writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim().ldelim();
                                }
                            if (message.columnFamilies != null && Object.hasOwnProperty.call(message, "columnFamilies"))
                                for (var keys = Object.keys(message.columnFamilies), i = 0; i < keys.length; ++i) {
                                    writer.uint32(/* id 3, wireType 2 =*/26).fork().uint32(/* id 1, wireType 2 =*/10).string(keys[i]);
                                    $root.google.bigtable.admin.v2.ColumnFamily.encode(message.columnFamilies[keys[i]], writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim().ldelim();
                                }
                            if (message.granularity != null && Object.hasOwnProperty.call(message, "granularity"))
                                writer.uint32(/* id 4, wireType 0 =*/32).int32(message.granularity);
                            if (message.restoreInfo != null && Object.hasOwnProperty.call(message, "restoreInfo"))
                                $root.google.bigtable.admin.v2.RestoreInfo.encode(message.restoreInfo, writer.uint32(/* id 6, wireType 2 =*/50).fork()).ldelim();
                            if (message.changeStreamConfig != null && Object.hasOwnProperty.call(message, "changeStreamConfig"))
                                $root.google.bigtable.admin.v2.ChangeStreamConfig.encode(message.changeStreamConfig, writer.uint32(/* id 8, wireType 2 =*/66).fork()).ldelim();
                            if (message.deletionProtection != null && Object.hasOwnProperty.call(message, "deletionProtection"))
                                writer.uint32(/* id 9, wireType 0 =*/72).bool(message.deletionProtection);
                            if (message.automatedBackupPolicy != null && Object.hasOwnProperty.call(message, "automatedBackupPolicy"))
                                $root.google.bigtable.admin.v2.Table.AutomatedBackupPolicy.encode(message.automatedBackupPolicy, writer.uint32(/* id 13, wireType 2 =*/106).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified Table message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.Table.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.Table
                         * @static
                         * @param {google.bigtable.admin.v2.ITable} message Table message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        Table.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a Table message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.Table
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.Table} Table
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        Table.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.Table(), key, value;
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                case 2: {
                                        if (message.clusterStates === $util.emptyObject)
                                            message.clusterStates = {};
                                        var end2 = reader.uint32() + reader.pos;
                                        key = "";
                                        value = null;
                                        while (reader.pos < end2) {
                                            var tag2 = reader.uint32();
                                            switch (tag2 >>> 3) {
                                            case 1:
                                                key = reader.string();
                                                break;
                                            case 2:
                                                value = $root.google.bigtable.admin.v2.Table.ClusterState.decode(reader, reader.uint32());
                                                break;
                                            default:
                                                reader.skipType(tag2 & 7);
                                                break;
                                            }
                                        }
                                        message.clusterStates[key] = value;
                                        break;
                                    }
                                case 3: {
                                        if (message.columnFamilies === $util.emptyObject)
                                            message.columnFamilies = {};
                                        var end2 = reader.uint32() + reader.pos;
                                        key = "";
                                        value = null;
                                        while (reader.pos < end2) {
                                            var tag2 = reader.uint32();
                                            switch (tag2 >>> 3) {
                                            case 1:
                                                key = reader.string();
                                                break;
                                            case 2:
                                                value = $root.google.bigtable.admin.v2.ColumnFamily.decode(reader, reader.uint32());
                                                break;
                                            default:
                                                reader.skipType(tag2 & 7);
                                                break;
                                            }
                                        }
                                        message.columnFamilies[key] = value;
                                        break;
                                    }
                                case 4: {
                                        message.granularity = reader.int32();
                                        break;
                                    }
                                case 6: {
                                        message.restoreInfo = $root.google.bigtable.admin.v2.RestoreInfo.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 8: {
                                        message.changeStreamConfig = $root.google.bigtable.admin.v2.ChangeStreamConfig.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 9: {
                                        message.deletionProtection = reader.bool();
                                        break;
                                    }
                                case 13: {
                                        message.automatedBackupPolicy = $root.google.bigtable.admin.v2.Table.AutomatedBackupPolicy.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a Table message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.Table
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.Table} Table
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        Table.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a Table message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.Table
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        Table.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            var properties = {};
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            if (message.clusterStates != null && message.hasOwnProperty("clusterStates")) {
                                if (!$util.isObject(message.clusterStates))
                                    return "clusterStates: object expected";
                                var key = Object.keys(message.clusterStates);
                                for (var i = 0; i < key.length; ++i) {
                                    var error = $root.google.bigtable.admin.v2.Table.ClusterState.verify(message.clusterStates[key[i]]);
                                    if (error)
                                        return "clusterStates." + error;
                                }
                            }
                            if (message.columnFamilies != null && message.hasOwnProperty("columnFamilies")) {
                                if (!$util.isObject(message.columnFamilies))
                                    return "columnFamilies: object expected";
                                var key = Object.keys(message.columnFamilies);
                                for (var i = 0; i < key.length; ++i) {
                                    var error = $root.google.bigtable.admin.v2.ColumnFamily.verify(message.columnFamilies[key[i]]);
                                    if (error)
                                        return "columnFamilies." + error;
                                }
                            }
                            if (message.granularity != null && message.hasOwnProperty("granularity"))
                                switch (message.granularity) {
                                default:
                                    return "granularity: enum value expected";
                                case 0:
                                case 1:
                                    break;
                                }
                            if (message.restoreInfo != null && message.hasOwnProperty("restoreInfo")) {
                                var error = $root.google.bigtable.admin.v2.RestoreInfo.verify(message.restoreInfo);
                                if (error)
                                    return "restoreInfo." + error;
                            }
                            if (message.changeStreamConfig != null && message.hasOwnProperty("changeStreamConfig")) {
                                var error = $root.google.bigtable.admin.v2.ChangeStreamConfig.verify(message.changeStreamConfig);
                                if (error)
                                    return "changeStreamConfig." + error;
                            }
                            if (message.deletionProtection != null && message.hasOwnProperty("deletionProtection"))
                                if (typeof message.deletionProtection !== "boolean")
                                    return "deletionProtection: boolean expected";
                            if (message.automatedBackupPolicy != null && message.hasOwnProperty("automatedBackupPolicy")) {
                                properties.automatedBackupConfig = 1;
                                {
                                    var error = $root.google.bigtable.admin.v2.Table.AutomatedBackupPolicy.verify(message.automatedBackupPolicy);
                                    if (error)
                                        return "automatedBackupPolicy." + error;
                                }
                            }
                            return null;
                        };
    
                        /**
                         * Creates a Table message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.Table
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.Table} Table
                         */
                        Table.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.Table)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.Table();
                            if (object.name != null)
                                message.name = String(object.name);
                            if (object.clusterStates) {
                                if (typeof object.clusterStates !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.Table.clusterStates: object expected");
                                message.clusterStates = {};
                                for (var keys = Object.keys(object.clusterStates), i = 0; i < keys.length; ++i) {
                                    if (typeof object.clusterStates[keys[i]] !== "object")
                                        throw TypeError(".google.bigtable.admin.v2.Table.clusterStates: object expected");
                                    message.clusterStates[keys[i]] = $root.google.bigtable.admin.v2.Table.ClusterState.fromObject(object.clusterStates[keys[i]]);
                                }
                            }
                            if (object.columnFamilies) {
                                if (typeof object.columnFamilies !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.Table.columnFamilies: object expected");
                                message.columnFamilies = {};
                                for (var keys = Object.keys(object.columnFamilies), i = 0; i < keys.length; ++i) {
                                    if (typeof object.columnFamilies[keys[i]] !== "object")
                                        throw TypeError(".google.bigtable.admin.v2.Table.columnFamilies: object expected");
                                    message.columnFamilies[keys[i]] = $root.google.bigtable.admin.v2.ColumnFamily.fromObject(object.columnFamilies[keys[i]]);
                                }
                            }
                            switch (object.granularity) {
                            default:
                                if (typeof object.granularity === "number") {
                                    message.granularity = object.granularity;
                                    break;
                                }
                                break;
                            case "TIMESTAMP_GRANULARITY_UNSPECIFIED":
                            case 0:
                                message.granularity = 0;
                                break;
                            case "MILLIS":
                            case 1:
                                message.granularity = 1;
                                break;
                            }
                            if (object.restoreInfo != null) {
                                if (typeof object.restoreInfo !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.Table.restoreInfo: object expected");
                                message.restoreInfo = $root.google.bigtable.admin.v2.RestoreInfo.fromObject(object.restoreInfo);
                            }
                            if (object.changeStreamConfig != null) {
                                if (typeof object.changeStreamConfig !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.Table.changeStreamConfig: object expected");
                                message.changeStreamConfig = $root.google.bigtable.admin.v2.ChangeStreamConfig.fromObject(object.changeStreamConfig);
                            }
                            if (object.deletionProtection != null)
                                message.deletionProtection = Boolean(object.deletionProtection);
                            if (object.automatedBackupPolicy != null) {
                                if (typeof object.automatedBackupPolicy !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.Table.automatedBackupPolicy: object expected");
                                message.automatedBackupPolicy = $root.google.bigtable.admin.v2.Table.AutomatedBackupPolicy.fromObject(object.automatedBackupPolicy);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a Table message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.Table
                         * @static
                         * @param {google.bigtable.admin.v2.Table} message Table
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        Table.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.objects || options.defaults) {
                                object.clusterStates = {};
                                object.columnFamilies = {};
                            }
                            if (options.defaults) {
                                object.name = "";
                                object.granularity = options.enums === String ? "TIMESTAMP_GRANULARITY_UNSPECIFIED" : 0;
                                object.restoreInfo = null;
                                object.changeStreamConfig = null;
                                object.deletionProtection = false;
                            }
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            var keys2;
                            if (message.clusterStates && (keys2 = Object.keys(message.clusterStates)).length) {
                                object.clusterStates = {};
                                for (var j = 0; j < keys2.length; ++j)
                                    object.clusterStates[keys2[j]] = $root.google.bigtable.admin.v2.Table.ClusterState.toObject(message.clusterStates[keys2[j]], options);
                            }
                            if (message.columnFamilies && (keys2 = Object.keys(message.columnFamilies)).length) {
                                object.columnFamilies = {};
                                for (var j = 0; j < keys2.length; ++j)
                                    object.columnFamilies[keys2[j]] = $root.google.bigtable.admin.v2.ColumnFamily.toObject(message.columnFamilies[keys2[j]], options);
                            }
                            if (message.granularity != null && message.hasOwnProperty("granularity"))
                                object.granularity = options.enums === String ? $root.google.bigtable.admin.v2.Table.TimestampGranularity[message.granularity] === undefined ? message.granularity : $root.google.bigtable.admin.v2.Table.TimestampGranularity[message.granularity] : message.granularity;
                            if (message.restoreInfo != null && message.hasOwnProperty("restoreInfo"))
                                object.restoreInfo = $root.google.bigtable.admin.v2.RestoreInfo.toObject(message.restoreInfo, options);
                            if (message.changeStreamConfig != null && message.hasOwnProperty("changeStreamConfig"))
                                object.changeStreamConfig = $root.google.bigtable.admin.v2.ChangeStreamConfig.toObject(message.changeStreamConfig, options);
                            if (message.deletionProtection != null && message.hasOwnProperty("deletionProtection"))
                                object.deletionProtection = message.deletionProtection;
                            if (message.automatedBackupPolicy != null && message.hasOwnProperty("automatedBackupPolicy")) {
                                object.automatedBackupPolicy = $root.google.bigtable.admin.v2.Table.AutomatedBackupPolicy.toObject(message.automatedBackupPolicy, options);
                                if (options.oneofs)
                                    object.automatedBackupConfig = "automatedBackupPolicy";
                            }
                            return object;
                        };
    
                        /**
                         * Converts this Table to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.Table
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        Table.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for Table
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.Table
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        Table.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.Table";
                        };
    
                        Table.ClusterState = (function() {
    
                            /**
                             * Properties of a ClusterState.
                             * @memberof google.bigtable.admin.v2.Table
                             * @interface IClusterState
                             * @property {google.bigtable.admin.v2.Table.ClusterState.ReplicationState|null} [replicationState] ClusterState replicationState
                             * @property {Array.<google.bigtable.admin.v2.IEncryptionInfo>|null} [encryptionInfo] ClusterState encryptionInfo
                             */
    
                            /**
                             * Constructs a new ClusterState.
                             * @memberof google.bigtable.admin.v2.Table
                             * @classdesc Represents a ClusterState.
                             * @implements IClusterState
                             * @constructor
                             * @param {google.bigtable.admin.v2.Table.IClusterState=} [properties] Properties to set
                             */
                            function ClusterState(properties) {
                                this.encryptionInfo = [];
                                if (properties)
                                    for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                        if (properties[keys[i]] != null)
                                            this[keys[i]] = properties[keys[i]];
                            }
    
                            /**
                             * ClusterState replicationState.
                             * @member {google.bigtable.admin.v2.Table.ClusterState.ReplicationState} replicationState
                             * @memberof google.bigtable.admin.v2.Table.ClusterState
                             * @instance
                             */
                            ClusterState.prototype.replicationState = 0;
    
                            /**
                             * ClusterState encryptionInfo.
                             * @member {Array.<google.bigtable.admin.v2.IEncryptionInfo>} encryptionInfo
                             * @memberof google.bigtable.admin.v2.Table.ClusterState
                             * @instance
                             */
                            ClusterState.prototype.encryptionInfo = $util.emptyArray;
    
                            /**
                             * Creates a new ClusterState instance using the specified properties.
                             * @function create
                             * @memberof google.bigtable.admin.v2.Table.ClusterState
                             * @static
                             * @param {google.bigtable.admin.v2.Table.IClusterState=} [properties] Properties to set
                             * @returns {google.bigtable.admin.v2.Table.ClusterState} ClusterState instance
                             */
                            ClusterState.create = function create(properties) {
                                return new ClusterState(properties);
                            };
    
                            /**
                             * Encodes the specified ClusterState message. Does not implicitly {@link google.bigtable.admin.v2.Table.ClusterState.verify|verify} messages.
                             * @function encode
                             * @memberof google.bigtable.admin.v2.Table.ClusterState
                             * @static
                             * @param {google.bigtable.admin.v2.Table.IClusterState} message ClusterState message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ClusterState.encode = function encode(message, writer) {
                                if (!writer)
                                    writer = $Writer.create();
                                if (message.replicationState != null && Object.hasOwnProperty.call(message, "replicationState"))
                                    writer.uint32(/* id 1, wireType 0 =*/8).int32(message.replicationState);
                                if (message.encryptionInfo != null && message.encryptionInfo.length)
                                    for (var i = 0; i < message.encryptionInfo.length; ++i)
                                        $root.google.bigtable.admin.v2.EncryptionInfo.encode(message.encryptionInfo[i], writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                                return writer;
                            };
    
                            /**
                             * Encodes the specified ClusterState message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.Table.ClusterState.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.bigtable.admin.v2.Table.ClusterState
                             * @static
                             * @param {google.bigtable.admin.v2.Table.IClusterState} message ClusterState message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ClusterState.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ClusterState message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.bigtable.admin.v2.Table.ClusterState
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @param {number} [length] Message length if known beforehand
                             * @returns {google.bigtable.admin.v2.Table.ClusterState} ClusterState
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ClusterState.decode = function decode(reader, length) {
                                if (!(reader instanceof $Reader))
                                    reader = $Reader.create(reader);
                                var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.Table.ClusterState();
                                while (reader.pos < end) {
                                    var tag = reader.uint32();
                                    switch (tag >>> 3) {
                                    case 1: {
                                            message.replicationState = reader.int32();
                                            break;
                                        }
                                    case 2: {
                                            if (!(message.encryptionInfo && message.encryptionInfo.length))
                                                message.encryptionInfo = [];
                                            message.encryptionInfo.push($root.google.bigtable.admin.v2.EncryptionInfo.decode(reader, reader.uint32()));
                                            break;
                                        }
                                    default:
                                        reader.skipType(tag & 7);
                                        break;
                                    }
                                }
                                return message;
                            };
    
                            /**
                             * Decodes a ClusterState message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.bigtable.admin.v2.Table.ClusterState
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.bigtable.admin.v2.Table.ClusterState} ClusterState
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ClusterState.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ClusterState message.
                             * @function verify
                             * @memberof google.bigtable.admin.v2.Table.ClusterState
                             * @static
                             * @param {Object.<string,*>} message Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ClusterState.verify = function verify(message) {
                                if (typeof message !== "object" || message === null)
                                    return "object expected";
                                if (message.replicationState != null && message.hasOwnProperty("replicationState"))
                                    switch (message.replicationState) {
                                    default:
                                        return "replicationState: enum value expected";
                                    case 0:
                                    case 1:
                                    case 2:
                                    case 3:
                                    case 4:
                                    case 5:
                                        break;
                                    }
                                if (message.encryptionInfo != null && message.hasOwnProperty("encryptionInfo")) {
                                    if (!Array.isArray(message.encryptionInfo))
                                        return "encryptionInfo: array expected";
                                    for (var i = 0; i < message.encryptionInfo.length; ++i) {
                                        var error = $root.google.bigtable.admin.v2.EncryptionInfo.verify(message.encryptionInfo[i]);
                                        if (error)
                                            return "encryptionInfo." + error;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a ClusterState message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.bigtable.admin.v2.Table.ClusterState
                             * @static
                             * @param {Object.<string,*>} object Plain object
                             * @returns {google.bigtable.admin.v2.Table.ClusterState} ClusterState
                             */
                            ClusterState.fromObject = function fromObject(object) {
                                if (object instanceof $root.google.bigtable.admin.v2.Table.ClusterState)
                                    return object;
                                var message = new $root.google.bigtable.admin.v2.Table.ClusterState();
                                switch (object.replicationState) {
                                default:
                                    if (typeof object.replicationState === "number") {
                                        message.replicationState = object.replicationState;
                                        break;
                                    }
                                    break;
                                case "STATE_NOT_KNOWN":
                                case 0:
                                    message.replicationState = 0;
                                    break;
                                case "INITIALIZING":
                                case 1:
                                    message.replicationState = 1;
                                    break;
                                case "PLANNED_MAINTENANCE":
                                case 2:
                                    message.replicationState = 2;
                                    break;
                                case "UNPLANNED_MAINTENANCE":
                                case 3:
                                    message.replicationState = 3;
                                    break;
                                case "READY":
                                case 4:
                                    message.replicationState = 4;
                                    break;
                                case "READY_OPTIMIZING":
                                case 5:
                                    message.replicationState = 5;
                                    break;
                                }
                                if (object.encryptionInfo) {
                                    if (!Array.isArray(object.encryptionInfo))
                                        throw TypeError(".google.bigtable.admin.v2.Table.ClusterState.encryptionInfo: array expected");
                                    message.encryptionInfo = [];
                                    for (var i = 0; i < object.encryptionInfo.length; ++i) {
                                        if (typeof object.encryptionInfo[i] !== "object")
                                            throw TypeError(".google.bigtable.admin.v2.Table.ClusterState.encryptionInfo: object expected");
                                        message.encryptionInfo[i] = $root.google.bigtable.admin.v2.EncryptionInfo.fromObject(object.encryptionInfo[i]);
                                    }
                                }
                                return message;
                            };
    
                            /**
                             * Creates a plain object from a ClusterState message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.bigtable.admin.v2.Table.ClusterState
                             * @static
                             * @param {google.bigtable.admin.v2.Table.ClusterState} message ClusterState
                             * @param {$protobuf.IConversionOptions} [options] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ClusterState.toObject = function toObject(message, options) {
                                if (!options)
                                    options = {};
                                var object = {};
                                if (options.arrays || options.defaults)
                                    object.encryptionInfo = [];
                                if (options.defaults)
                                    object.replicationState = options.enums === String ? "STATE_NOT_KNOWN" : 0;
                                if (message.replicationState != null && message.hasOwnProperty("replicationState"))
                                    object.replicationState = options.enums === String ? $root.google.bigtable.admin.v2.Table.ClusterState.ReplicationState[message.replicationState] === undefined ? message.replicationState : $root.google.bigtable.admin.v2.Table.ClusterState.ReplicationState[message.replicationState] : message.replicationState;
                                if (message.encryptionInfo && message.encryptionInfo.length) {
                                    object.encryptionInfo = [];
                                    for (var j = 0; j < message.encryptionInfo.length; ++j)
                                        object.encryptionInfo[j] = $root.google.bigtable.admin.v2.EncryptionInfo.toObject(message.encryptionInfo[j], options);
                                }
                                return object;
                            };
    
                            /**
                             * Converts this ClusterState to JSON.
                             * @function toJSON
                             * @memberof google.bigtable.admin.v2.Table.ClusterState
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ClusterState.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * Gets the default type url for ClusterState
                             * @function getTypeUrl
                             * @memberof google.bigtable.admin.v2.Table.ClusterState
                             * @static
                             * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                             * @returns {string} The default type url
                             */
                            ClusterState.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                                if (typeUrlPrefix === undefined) {
                                    typeUrlPrefix = "type.googleapis.com";
                                }
                                return typeUrlPrefix + "/google.bigtable.admin.v2.Table.ClusterState";
                            };
    
                            /**
                             * ReplicationState enum.
                             * @name google.bigtable.admin.v2.Table.ClusterState.ReplicationState
                             * @enum {number}
                             * @property {number} STATE_NOT_KNOWN=0 STATE_NOT_KNOWN value
                             * @property {number} INITIALIZING=1 INITIALIZING value
                             * @property {number} PLANNED_MAINTENANCE=2 PLANNED_MAINTENANCE value
                             * @property {number} UNPLANNED_MAINTENANCE=3 UNPLANNED_MAINTENANCE value
                             * @property {number} READY=4 READY value
                             * @property {number} READY_OPTIMIZING=5 READY_OPTIMIZING value
                             */
                            ClusterState.ReplicationState = (function() {
                                var valuesById = {}, values = Object.create(valuesById);
                                values[valuesById[0] = "STATE_NOT_KNOWN"] = 0;
                                values[valuesById[1] = "INITIALIZING"] = 1;
                                values[valuesById[2] = "PLANNED_MAINTENANCE"] = 2;
                                values[valuesById[3] = "UNPLANNED_MAINTENANCE"] = 3;
                                values[valuesById[4] = "READY"] = 4;
                                values[valuesById[5] = "READY_OPTIMIZING"] = 5;
                                return values;
                            })();
    
                            return ClusterState;
                        })();
    
                        /**
                         * TimestampGranularity enum.
                         * @name google.bigtable.admin.v2.Table.TimestampGranularity
                         * @enum {number}
                         * @property {number} TIMESTAMP_GRANULARITY_UNSPECIFIED=0 TIMESTAMP_GRANULARITY_UNSPECIFIED value
                         * @property {number} MILLIS=1 MILLIS value
                         */
                        Table.TimestampGranularity = (function() {
                            var valuesById = {}, values = Object.create(valuesById);
                            values[valuesById[0] = "TIMESTAMP_GRANULARITY_UNSPECIFIED"] = 0;
                            values[valuesById[1] = "MILLIS"] = 1;
                            return values;
                        })();
    
                        /**
                         * View enum.
                         * @name google.bigtable.admin.v2.Table.View
                         * @enum {number}
                         * @property {number} VIEW_UNSPECIFIED=0 VIEW_UNSPECIFIED value
                         * @property {number} NAME_ONLY=1 NAME_ONLY value
                         * @property {number} SCHEMA_VIEW=2 SCHEMA_VIEW value
                         * @property {number} REPLICATION_VIEW=3 REPLICATION_VIEW value
                         * @property {number} ENCRYPTION_VIEW=5 ENCRYPTION_VIEW value
                         * @property {number} FULL=4 FULL value
                         */
                        Table.View = (function() {
                            var valuesById = {}, values = Object.create(valuesById);
                            values[valuesById[0] = "VIEW_UNSPECIFIED"] = 0;
                            values[valuesById[1] = "NAME_ONLY"] = 1;
                            values[valuesById[2] = "SCHEMA_VIEW"] = 2;
                            values[valuesById[3] = "REPLICATION_VIEW"] = 3;
                            values[valuesById[5] = "ENCRYPTION_VIEW"] = 5;
                            values[valuesById[4] = "FULL"] = 4;
                            return values;
                        })();
    
                        Table.AutomatedBackupPolicy = (function() {
    
                            /**
                             * Properties of an AutomatedBackupPolicy.
                             * @memberof google.bigtable.admin.v2.Table
                             * @interface IAutomatedBackupPolicy
                             * @property {google.protobuf.IDuration|null} [retentionPeriod] AutomatedBackupPolicy retentionPeriod
                             * @property {google.protobuf.IDuration|null} [frequency] AutomatedBackupPolicy frequency
                             */
    
                            /**
                             * Constructs a new AutomatedBackupPolicy.
                             * @memberof google.bigtable.admin.v2.Table
                             * @classdesc Represents an AutomatedBackupPolicy.
                             * @implements IAutomatedBackupPolicy
                             * @constructor
                             * @param {google.bigtable.admin.v2.Table.IAutomatedBackupPolicy=} [properties] Properties to set
                             */
                            function AutomatedBackupPolicy(properties) {
                                if (properties)
                                    for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                        if (properties[keys[i]] != null)
                                            this[keys[i]] = properties[keys[i]];
                            }
    
                            /**
                             * AutomatedBackupPolicy retentionPeriod.
                             * @member {google.protobuf.IDuration|null|undefined} retentionPeriod
                             * @memberof google.bigtable.admin.v2.Table.AutomatedBackupPolicy
                             * @instance
                             */
                            AutomatedBackupPolicy.prototype.retentionPeriod = null;
    
                            /**
                             * AutomatedBackupPolicy frequency.
                             * @member {google.protobuf.IDuration|null|undefined} frequency
                             * @memberof google.bigtable.admin.v2.Table.AutomatedBackupPolicy
                             * @instance
                             */
                            AutomatedBackupPolicy.prototype.frequency = null;
    
                            /**
                             * Creates a new AutomatedBackupPolicy instance using the specified properties.
                             * @function create
                             * @memberof google.bigtable.admin.v2.Table.AutomatedBackupPolicy
                             * @static
                             * @param {google.bigtable.admin.v2.Table.IAutomatedBackupPolicy=} [properties] Properties to set
                             * @returns {google.bigtable.admin.v2.Table.AutomatedBackupPolicy} AutomatedBackupPolicy instance
                             */
                            AutomatedBackupPolicy.create = function create(properties) {
                                return new AutomatedBackupPolicy(properties);
                            };
    
                            /**
                             * Encodes the specified AutomatedBackupPolicy message. Does not implicitly {@link google.bigtable.admin.v2.Table.AutomatedBackupPolicy.verify|verify} messages.
                             * @function encode
                             * @memberof google.bigtable.admin.v2.Table.AutomatedBackupPolicy
                             * @static
                             * @param {google.bigtable.admin.v2.Table.IAutomatedBackupPolicy} message AutomatedBackupPolicy message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AutomatedBackupPolicy.encode = function encode(message, writer) {
                                if (!writer)
                                    writer = $Writer.create();
                                if (message.retentionPeriod != null && Object.hasOwnProperty.call(message, "retentionPeriod"))
                                    $root.google.protobuf.Duration.encode(message.retentionPeriod, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                                if (message.frequency != null && Object.hasOwnProperty.call(message, "frequency"))
                                    $root.google.protobuf.Duration.encode(message.frequency, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                                return writer;
                            };
    
                            /**
                             * Encodes the specified AutomatedBackupPolicy message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.Table.AutomatedBackupPolicy.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.bigtable.admin.v2.Table.AutomatedBackupPolicy
                             * @static
                             * @param {google.bigtable.admin.v2.Table.IAutomatedBackupPolicy} message AutomatedBackupPolicy message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            AutomatedBackupPolicy.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an AutomatedBackupPolicy message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.bigtable.admin.v2.Table.AutomatedBackupPolicy
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @param {number} [length] Message length if known beforehand
                             * @returns {google.bigtable.admin.v2.Table.AutomatedBackupPolicy} AutomatedBackupPolicy
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AutomatedBackupPolicy.decode = function decode(reader, length) {
                                if (!(reader instanceof $Reader))
                                    reader = $Reader.create(reader);
                                var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.Table.AutomatedBackupPolicy();
                                while (reader.pos < end) {
                                    var tag = reader.uint32();
                                    switch (tag >>> 3) {
                                    case 1: {
                                            message.retentionPeriod = $root.google.protobuf.Duration.decode(reader, reader.uint32());
                                            break;
                                        }
                                    case 2: {
                                            message.frequency = $root.google.protobuf.Duration.decode(reader, reader.uint32());
                                            break;
                                        }
                                    default:
                                        reader.skipType(tag & 7);
                                        break;
                                    }
                                }
                                return message;
                            };
    
                            /**
                             * Decodes an AutomatedBackupPolicy message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.bigtable.admin.v2.Table.AutomatedBackupPolicy
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.bigtable.admin.v2.Table.AutomatedBackupPolicy} AutomatedBackupPolicy
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            AutomatedBackupPolicy.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an AutomatedBackupPolicy message.
                             * @function verify
                             * @memberof google.bigtable.admin.v2.Table.AutomatedBackupPolicy
                             * @static
                             * @param {Object.<string,*>} message Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            AutomatedBackupPolicy.verify = function verify(message) {
                                if (typeof message !== "object" || message === null)
                                    return "object expected";
                                if (message.retentionPeriod != null && message.hasOwnProperty("retentionPeriod")) {
                                    var error = $root.google.protobuf.Duration.verify(message.retentionPeriod);
                                    if (error)
                                        return "retentionPeriod." + error;
                                }
                                if (message.frequency != null && message.hasOwnProperty("frequency")) {
                                    var error = $root.google.protobuf.Duration.verify(message.frequency);
                                    if (error)
                                        return "frequency." + error;
                                }
                                return null;
                            };
    
                            /**
                             * Creates an AutomatedBackupPolicy message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.bigtable.admin.v2.Table.AutomatedBackupPolicy
                             * @static
                             * @param {Object.<string,*>} object Plain object
                             * @returns {google.bigtable.admin.v2.Table.AutomatedBackupPolicy} AutomatedBackupPolicy
                             */
                            AutomatedBackupPolicy.fromObject = function fromObject(object) {
                                if (object instanceof $root.google.bigtable.admin.v2.Table.AutomatedBackupPolicy)
                                    return object;
                                var message = new $root.google.bigtable.admin.v2.Table.AutomatedBackupPolicy();
                                if (object.retentionPeriod != null) {
                                    if (typeof object.retentionPeriod !== "object")
                                        throw TypeError(".google.bigtable.admin.v2.Table.AutomatedBackupPolicy.retentionPeriod: object expected");
                                    message.retentionPeriod = $root.google.protobuf.Duration.fromObject(object.retentionPeriod);
                                }
                                if (object.frequency != null) {
                                    if (typeof object.frequency !== "object")
                                        throw TypeError(".google.bigtable.admin.v2.Table.AutomatedBackupPolicy.frequency: object expected");
                                    message.frequency = $root.google.protobuf.Duration.fromObject(object.frequency);
                                }
                                return message;
                            };
    
                            /**
                             * Creates a plain object from an AutomatedBackupPolicy message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.bigtable.admin.v2.Table.AutomatedBackupPolicy
                             * @static
                             * @param {google.bigtable.admin.v2.Table.AutomatedBackupPolicy} message AutomatedBackupPolicy
                             * @param {$protobuf.IConversionOptions} [options] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            AutomatedBackupPolicy.toObject = function toObject(message, options) {
                                if (!options)
                                    options = {};
                                var object = {};
                                if (options.defaults) {
                                    object.retentionPeriod = null;
                                    object.frequency = null;
                                }
                                if (message.retentionPeriod != null && message.hasOwnProperty("retentionPeriod"))
                                    object.retentionPeriod = $root.google.protobuf.Duration.toObject(message.retentionPeriod, options);
                                if (message.frequency != null && message.hasOwnProperty("frequency"))
                                    object.frequency = $root.google.protobuf.Duration.toObject(message.frequency, options);
                                return object;
                            };
    
                            /**
                             * Converts this AutomatedBackupPolicy to JSON.
                             * @function toJSON
                             * @memberof google.bigtable.admin.v2.Table.AutomatedBackupPolicy
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            AutomatedBackupPolicy.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * Gets the default type url for AutomatedBackupPolicy
                             * @function getTypeUrl
                             * @memberof google.bigtable.admin.v2.Table.AutomatedBackupPolicy
                             * @static
                             * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                             * @returns {string} The default type url
                             */
                            AutomatedBackupPolicy.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                                if (typeUrlPrefix === undefined) {
                                    typeUrlPrefix = "type.googleapis.com";
                                }
                                return typeUrlPrefix + "/google.bigtable.admin.v2.Table.AutomatedBackupPolicy";
                            };
    
                            return AutomatedBackupPolicy;
                        })();
    
                        return Table;
                    })();
    
                    v2.AuthorizedView = (function() {
    
                        /**
                         * Properties of an AuthorizedView.
                         * @memberof google.bigtable.admin.v2
                         * @interface IAuthorizedView
                         * @property {string|null} [name] AuthorizedView name
                         * @property {google.bigtable.admin.v2.AuthorizedView.ISubsetView|null} [subsetView] AuthorizedView subsetView
                         * @property {string|null} [etag] AuthorizedView etag
                         * @property {boolean|null} [deletionProtection] AuthorizedView deletionProtection
                         */
    
                        /**
                         * Constructs a new AuthorizedView.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents an AuthorizedView.
                         * @implements IAuthorizedView
                         * @constructor
                         * @param {google.bigtable.admin.v2.IAuthorizedView=} [properties] Properties to set
                         */
                        function AuthorizedView(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * AuthorizedView name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.AuthorizedView
                         * @instance
                         */
                        AuthorizedView.prototype.name = "";
    
                        /**
                         * AuthorizedView subsetView.
                         * @member {google.bigtable.admin.v2.AuthorizedView.ISubsetView|null|undefined} subsetView
                         * @memberof google.bigtable.admin.v2.AuthorizedView
                         * @instance
                         */
                        AuthorizedView.prototype.subsetView = null;
    
                        /**
                         * AuthorizedView etag.
                         * @member {string} etag
                         * @memberof google.bigtable.admin.v2.AuthorizedView
                         * @instance
                         */
                        AuthorizedView.prototype.etag = "";
    
                        /**
                         * AuthorizedView deletionProtection.
                         * @member {boolean} deletionProtection
                         * @memberof google.bigtable.admin.v2.AuthorizedView
                         * @instance
                         */
                        AuthorizedView.prototype.deletionProtection = false;
    
                        // OneOf field names bound to virtual getters and setters
                        var $oneOfFields;
    
                        /**
                         * AuthorizedView authorizedView.
                         * @member {"subsetView"|undefined} authorizedView
                         * @memberof google.bigtable.admin.v2.AuthorizedView
                         * @instance
                         */
                        Object.defineProperty(AuthorizedView.prototype, "authorizedView", {
                            get: $util.oneOfGetter($oneOfFields = ["subsetView"]),
                            set: $util.oneOfSetter($oneOfFields)
                        });
    
                        /**
                         * Creates a new AuthorizedView instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.AuthorizedView
                         * @static
                         * @param {google.bigtable.admin.v2.IAuthorizedView=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.AuthorizedView} AuthorizedView instance
                         */
                        AuthorizedView.create = function create(properties) {
                            return new AuthorizedView(properties);
                        };
    
                        /**
                         * Encodes the specified AuthorizedView message. Does not implicitly {@link google.bigtable.admin.v2.AuthorizedView.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.AuthorizedView
                         * @static
                         * @param {google.bigtable.admin.v2.IAuthorizedView} message AuthorizedView message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        AuthorizedView.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            if (message.subsetView != null && Object.hasOwnProperty.call(message, "subsetView"))
                                $root.google.bigtable.admin.v2.AuthorizedView.SubsetView.encode(message.subsetView, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            if (message.etag != null && Object.hasOwnProperty.call(message, "etag"))
                                writer.uint32(/* id 3, wireType 2 =*/26).string(message.etag);
                            if (message.deletionProtection != null && Object.hasOwnProperty.call(message, "deletionProtection"))
                                writer.uint32(/* id 4, wireType 0 =*/32).bool(message.deletionProtection);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified AuthorizedView message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.AuthorizedView.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.AuthorizedView
                         * @static
                         * @param {google.bigtable.admin.v2.IAuthorizedView} message AuthorizedView message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        AuthorizedView.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes an AuthorizedView message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.AuthorizedView
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.AuthorizedView} AuthorizedView
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        AuthorizedView.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.AuthorizedView();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.subsetView = $root.google.bigtable.admin.v2.AuthorizedView.SubsetView.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 3: {
                                        message.etag = reader.string();
                                        break;
                                    }
                                case 4: {
                                        message.deletionProtection = reader.bool();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes an AuthorizedView message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.AuthorizedView
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.AuthorizedView} AuthorizedView
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        AuthorizedView.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies an AuthorizedView message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.AuthorizedView
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        AuthorizedView.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            var properties = {};
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            if (message.subsetView != null && message.hasOwnProperty("subsetView")) {
                                properties.authorizedView = 1;
                                {
                                    var error = $root.google.bigtable.admin.v2.AuthorizedView.SubsetView.verify(message.subsetView);
                                    if (error)
                                        return "subsetView." + error;
                                }
                            }
                            if (message.etag != null && message.hasOwnProperty("etag"))
                                if (!$util.isString(message.etag))
                                    return "etag: string expected";
                            if (message.deletionProtection != null && message.hasOwnProperty("deletionProtection"))
                                if (typeof message.deletionProtection !== "boolean")
                                    return "deletionProtection: boolean expected";
                            return null;
                        };
    
                        /**
                         * Creates an AuthorizedView message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.AuthorizedView
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.AuthorizedView} AuthorizedView
                         */
                        AuthorizedView.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.AuthorizedView)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.AuthorizedView();
                            if (object.name != null)
                                message.name = String(object.name);
                            if (object.subsetView != null) {
                                if (typeof object.subsetView !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.AuthorizedView.subsetView: object expected");
                                message.subsetView = $root.google.bigtable.admin.v2.AuthorizedView.SubsetView.fromObject(object.subsetView);
                            }
                            if (object.etag != null)
                                message.etag = String(object.etag);
                            if (object.deletionProtection != null)
                                message.deletionProtection = Boolean(object.deletionProtection);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from an AuthorizedView message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.AuthorizedView
                         * @static
                         * @param {google.bigtable.admin.v2.AuthorizedView} message AuthorizedView
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        AuthorizedView.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.name = "";
                                object.etag = "";
                                object.deletionProtection = false;
                            }
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            if (message.subsetView != null && message.hasOwnProperty("subsetView")) {
                                object.subsetView = $root.google.bigtable.admin.v2.AuthorizedView.SubsetView.toObject(message.subsetView, options);
                                if (options.oneofs)
                                    object.authorizedView = "subsetView";
                            }
                            if (message.etag != null && message.hasOwnProperty("etag"))
                                object.etag = message.etag;
                            if (message.deletionProtection != null && message.hasOwnProperty("deletionProtection"))
                                object.deletionProtection = message.deletionProtection;
                            return object;
                        };
    
                        /**
                         * Converts this AuthorizedView to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.AuthorizedView
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        AuthorizedView.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for AuthorizedView
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.AuthorizedView
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        AuthorizedView.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.AuthorizedView";
                        };
    
                        AuthorizedView.FamilySubsets = (function() {
    
                            /**
                             * Properties of a FamilySubsets.
                             * @memberof google.bigtable.admin.v2.AuthorizedView
                             * @interface IFamilySubsets
                             * @property {Array.<Uint8Array>|null} [qualifiers] FamilySubsets qualifiers
                             * @property {Array.<Uint8Array>|null} [qualifierPrefixes] FamilySubsets qualifierPrefixes
                             */
    
                            /**
                             * Constructs a new FamilySubsets.
                             * @memberof google.bigtable.admin.v2.AuthorizedView
                             * @classdesc Represents a FamilySubsets.
                             * @implements IFamilySubsets
                             * @constructor
                             * @param {google.bigtable.admin.v2.AuthorizedView.IFamilySubsets=} [properties] Properties to set
                             */
                            function FamilySubsets(properties) {
                                this.qualifiers = [];
                                this.qualifierPrefixes = [];
                                if (properties)
                                    for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                        if (properties[keys[i]] != null)
                                            this[keys[i]] = properties[keys[i]];
                            }
    
                            /**
                             * FamilySubsets qualifiers.
                             * @member {Array.<Uint8Array>} qualifiers
                             * @memberof google.bigtable.admin.v2.AuthorizedView.FamilySubsets
                             * @instance
                             */
                            FamilySubsets.prototype.qualifiers = $util.emptyArray;
    
                            /**
                             * FamilySubsets qualifierPrefixes.
                             * @member {Array.<Uint8Array>} qualifierPrefixes
                             * @memberof google.bigtable.admin.v2.AuthorizedView.FamilySubsets
                             * @instance
                             */
                            FamilySubsets.prototype.qualifierPrefixes = $util.emptyArray;
    
                            /**
                             * Creates a new FamilySubsets instance using the specified properties.
                             * @function create
                             * @memberof google.bigtable.admin.v2.AuthorizedView.FamilySubsets
                             * @static
                             * @param {google.bigtable.admin.v2.AuthorizedView.IFamilySubsets=} [properties] Properties to set
                             * @returns {google.bigtable.admin.v2.AuthorizedView.FamilySubsets} FamilySubsets instance
                             */
                            FamilySubsets.create = function create(properties) {
                                return new FamilySubsets(properties);
                            };
    
                            /**
                             * Encodes the specified FamilySubsets message. Does not implicitly {@link google.bigtable.admin.v2.AuthorizedView.FamilySubsets.verify|verify} messages.
                             * @function encode
                             * @memberof google.bigtable.admin.v2.AuthorizedView.FamilySubsets
                             * @static
                             * @param {google.bigtable.admin.v2.AuthorizedView.IFamilySubsets} message FamilySubsets message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FamilySubsets.encode = function encode(message, writer) {
                                if (!writer)
                                    writer = $Writer.create();
                                if (message.qualifiers != null && message.qualifiers.length)
                                    for (var i = 0; i < message.qualifiers.length; ++i)
                                        writer.uint32(/* id 1, wireType 2 =*/10).bytes(message.qualifiers[i]);
                                if (message.qualifierPrefixes != null && message.qualifierPrefixes.length)
                                    for (var i = 0; i < message.qualifierPrefixes.length; ++i)
                                        writer.uint32(/* id 2, wireType 2 =*/18).bytes(message.qualifierPrefixes[i]);
                                return writer;
                            };
    
                            /**
                             * Encodes the specified FamilySubsets message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.AuthorizedView.FamilySubsets.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.bigtable.admin.v2.AuthorizedView.FamilySubsets
                             * @static
                             * @param {google.bigtable.admin.v2.AuthorizedView.IFamilySubsets} message FamilySubsets message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            FamilySubsets.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a FamilySubsets message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.bigtable.admin.v2.AuthorizedView.FamilySubsets
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @param {number} [length] Message length if known beforehand
                             * @returns {google.bigtable.admin.v2.AuthorizedView.FamilySubsets} FamilySubsets
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FamilySubsets.decode = function decode(reader, length) {
                                if (!(reader instanceof $Reader))
                                    reader = $Reader.create(reader);
                                var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.AuthorizedView.FamilySubsets();
                                while (reader.pos < end) {
                                    var tag = reader.uint32();
                                    switch (tag >>> 3) {
                                    case 1: {
                                            if (!(message.qualifiers && message.qualifiers.length))
                                                message.qualifiers = [];
                                            message.qualifiers.push(reader.bytes());
                                            break;
                                        }
                                    case 2: {
                                            if (!(message.qualifierPrefixes && message.qualifierPrefixes.length))
                                                message.qualifierPrefixes = [];
                                            message.qualifierPrefixes.push(reader.bytes());
                                            break;
                                        }
                                    default:
                                        reader.skipType(tag & 7);
                                        break;
                                    }
                                }
                                return message;
                            };
    
                            /**
                             * Decodes a FamilySubsets message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.bigtable.admin.v2.AuthorizedView.FamilySubsets
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.bigtable.admin.v2.AuthorizedView.FamilySubsets} FamilySubsets
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            FamilySubsets.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a FamilySubsets message.
                             * @function verify
                             * @memberof google.bigtable.admin.v2.AuthorizedView.FamilySubsets
                             * @static
                             * @param {Object.<string,*>} message Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            FamilySubsets.verify = function verify(message) {
                                if (typeof message !== "object" || message === null)
                                    return "object expected";
                                if (message.qualifiers != null && message.hasOwnProperty("qualifiers")) {
                                    if (!Array.isArray(message.qualifiers))
                                        return "qualifiers: array expected";
                                    for (var i = 0; i < message.qualifiers.length; ++i)
                                        if (!(message.qualifiers[i] && typeof message.qualifiers[i].length === "number" || $util.isString(message.qualifiers[i])))
                                            return "qualifiers: buffer[] expected";
                                }
                                if (message.qualifierPrefixes != null && message.hasOwnProperty("qualifierPrefixes")) {
                                    if (!Array.isArray(message.qualifierPrefixes))
                                        return "qualifierPrefixes: array expected";
                                    for (var i = 0; i < message.qualifierPrefixes.length; ++i)
                                        if (!(message.qualifierPrefixes[i] && typeof message.qualifierPrefixes[i].length === "number" || $util.isString(message.qualifierPrefixes[i])))
                                            return "qualifierPrefixes: buffer[] expected";
                                }
                                return null;
                            };
    
                            /**
                             * Creates a FamilySubsets message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.bigtable.admin.v2.AuthorizedView.FamilySubsets
                             * @static
                             * @param {Object.<string,*>} object Plain object
                             * @returns {google.bigtable.admin.v2.AuthorizedView.FamilySubsets} FamilySubsets
                             */
                            FamilySubsets.fromObject = function fromObject(object) {
                                if (object instanceof $root.google.bigtable.admin.v2.AuthorizedView.FamilySubsets)
                                    return object;
                                var message = new $root.google.bigtable.admin.v2.AuthorizedView.FamilySubsets();
                                if (object.qualifiers) {
                                    if (!Array.isArray(object.qualifiers))
                                        throw TypeError(".google.bigtable.admin.v2.AuthorizedView.FamilySubsets.qualifiers: array expected");
                                    message.qualifiers = [];
                                    for (var i = 0; i < object.qualifiers.length; ++i)
                                        if (typeof object.qualifiers[i] === "string")
                                            $util.base64.decode(object.qualifiers[i], message.qualifiers[i] = $util.newBuffer($util.base64.length(object.qualifiers[i])), 0);
                                        else if (object.qualifiers[i].length >= 0)
                                            message.qualifiers[i] = object.qualifiers[i];
                                }
                                if (object.qualifierPrefixes) {
                                    if (!Array.isArray(object.qualifierPrefixes))
                                        throw TypeError(".google.bigtable.admin.v2.AuthorizedView.FamilySubsets.qualifierPrefixes: array expected");
                                    message.qualifierPrefixes = [];
                                    for (var i = 0; i < object.qualifierPrefixes.length; ++i)
                                        if (typeof object.qualifierPrefixes[i] === "string")
                                            $util.base64.decode(object.qualifierPrefixes[i], message.qualifierPrefixes[i] = $util.newBuffer($util.base64.length(object.qualifierPrefixes[i])), 0);
                                        else if (object.qualifierPrefixes[i].length >= 0)
                                            message.qualifierPrefixes[i] = object.qualifierPrefixes[i];
                                }
                                return message;
                            };
    
                            /**
                             * Creates a plain object from a FamilySubsets message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.bigtable.admin.v2.AuthorizedView.FamilySubsets
                             * @static
                             * @param {google.bigtable.admin.v2.AuthorizedView.FamilySubsets} message FamilySubsets
                             * @param {$protobuf.IConversionOptions} [options] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            FamilySubsets.toObject = function toObject(message, options) {
                                if (!options)
                                    options = {};
                                var object = {};
                                if (options.arrays || options.defaults) {
                                    object.qualifiers = [];
                                    object.qualifierPrefixes = [];
                                }
                                if (message.qualifiers && message.qualifiers.length) {
                                    object.qualifiers = [];
                                    for (var j = 0; j < message.qualifiers.length; ++j)
                                        object.qualifiers[j] = options.bytes === String ? $util.base64.encode(message.qualifiers[j], 0, message.qualifiers[j].length) : options.bytes === Array ? Array.prototype.slice.call(message.qualifiers[j]) : message.qualifiers[j];
                                }
                                if (message.qualifierPrefixes && message.qualifierPrefixes.length) {
                                    object.qualifierPrefixes = [];
                                    for (var j = 0; j < message.qualifierPrefixes.length; ++j)
                                        object.qualifierPrefixes[j] = options.bytes === String ? $util.base64.encode(message.qualifierPrefixes[j], 0, message.qualifierPrefixes[j].length) : options.bytes === Array ? Array.prototype.slice.call(message.qualifierPrefixes[j]) : message.qualifierPrefixes[j];
                                }
                                return object;
                            };
    
                            /**
                             * Converts this FamilySubsets to JSON.
                             * @function toJSON
                             * @memberof google.bigtable.admin.v2.AuthorizedView.FamilySubsets
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            FamilySubsets.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * Gets the default type url for FamilySubsets
                             * @function getTypeUrl
                             * @memberof google.bigtable.admin.v2.AuthorizedView.FamilySubsets
                             * @static
                             * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                             * @returns {string} The default type url
                             */
                            FamilySubsets.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                                if (typeUrlPrefix === undefined) {
                                    typeUrlPrefix = "type.googleapis.com";
                                }
                                return typeUrlPrefix + "/google.bigtable.admin.v2.AuthorizedView.FamilySubsets";
                            };
    
                            return FamilySubsets;
                        })();
    
                        AuthorizedView.SubsetView = (function() {
    
                            /**
                             * Properties of a SubsetView.
                             * @memberof google.bigtable.admin.v2.AuthorizedView
                             * @interface ISubsetView
                             * @property {Array.<Uint8Array>|null} [rowPrefixes] SubsetView rowPrefixes
                             * @property {Object.<string,google.bigtable.admin.v2.AuthorizedView.IFamilySubsets>|null} [familySubsets] SubsetView familySubsets
                             */
    
                            /**
                             * Constructs a new SubsetView.
                             * @memberof google.bigtable.admin.v2.AuthorizedView
                             * @classdesc Represents a SubsetView.
                             * @implements ISubsetView
                             * @constructor
                             * @param {google.bigtable.admin.v2.AuthorizedView.ISubsetView=} [properties] Properties to set
                             */
                            function SubsetView(properties) {
                                this.rowPrefixes = [];
                                this.familySubsets = {};
                                if (properties)
                                    for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                        if (properties[keys[i]] != null)
                                            this[keys[i]] = properties[keys[i]];
                            }
    
                            /**
                             * SubsetView rowPrefixes.
                             * @member {Array.<Uint8Array>} rowPrefixes
                             * @memberof google.bigtable.admin.v2.AuthorizedView.SubsetView
                             * @instance
                             */
                            SubsetView.prototype.rowPrefixes = $util.emptyArray;
    
                            /**
                             * SubsetView familySubsets.
                             * @member {Object.<string,google.bigtable.admin.v2.AuthorizedView.IFamilySubsets>} familySubsets
                             * @memberof google.bigtable.admin.v2.AuthorizedView.SubsetView
                             * @instance
                             */
                            SubsetView.prototype.familySubsets = $util.emptyObject;
    
                            /**
                             * Creates a new SubsetView instance using the specified properties.
                             * @function create
                             * @memberof google.bigtable.admin.v2.AuthorizedView.SubsetView
                             * @static
                             * @param {google.bigtable.admin.v2.AuthorizedView.ISubsetView=} [properties] Properties to set
                             * @returns {google.bigtable.admin.v2.AuthorizedView.SubsetView} SubsetView instance
                             */
                            SubsetView.create = function create(properties) {
                                return new SubsetView(properties);
                            };
    
                            /**
                             * Encodes the specified SubsetView message. Does not implicitly {@link google.bigtable.admin.v2.AuthorizedView.SubsetView.verify|verify} messages.
                             * @function encode
                             * @memberof google.bigtable.admin.v2.AuthorizedView.SubsetView
                             * @static
                             * @param {google.bigtable.admin.v2.AuthorizedView.ISubsetView} message SubsetView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SubsetView.encode = function encode(message, writer) {
                                if (!writer)
                                    writer = $Writer.create();
                                if (message.rowPrefixes != null && message.rowPrefixes.length)
                                    for (var i = 0; i < message.rowPrefixes.length; ++i)
                                        writer.uint32(/* id 1, wireType 2 =*/10).bytes(message.rowPrefixes[i]);
                                if (message.familySubsets != null && Object.hasOwnProperty.call(message, "familySubsets"))
                                    for (var keys = Object.keys(message.familySubsets), i = 0; i < keys.length; ++i) {
                                        writer.uint32(/* id 2, wireType 2 =*/18).fork().uint32(/* id 1, wireType 2 =*/10).string(keys[i]);
                                        $root.google.bigtable.admin.v2.AuthorizedView.FamilySubsets.encode(message.familySubsets[keys[i]], writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim().ldelim();
                                    }
                                return writer;
                            };
    
                            /**
                             * Encodes the specified SubsetView message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.AuthorizedView.SubsetView.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.bigtable.admin.v2.AuthorizedView.SubsetView
                             * @static
                             * @param {google.bigtable.admin.v2.AuthorizedView.ISubsetView} message SubsetView message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            SubsetView.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a SubsetView message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.bigtable.admin.v2.AuthorizedView.SubsetView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @param {number} [length] Message length if known beforehand
                             * @returns {google.bigtable.admin.v2.AuthorizedView.SubsetView} SubsetView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SubsetView.decode = function decode(reader, length) {
                                if (!(reader instanceof $Reader))
                                    reader = $Reader.create(reader);
                                var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.AuthorizedView.SubsetView(), key, value;
                                while (reader.pos < end) {
                                    var tag = reader.uint32();
                                    switch (tag >>> 3) {
                                    case 1: {
                                            if (!(message.rowPrefixes && message.rowPrefixes.length))
                                                message.rowPrefixes = [];
                                            message.rowPrefixes.push(reader.bytes());
                                            break;
                                        }
                                    case 2: {
                                            if (message.familySubsets === $util.emptyObject)
                                                message.familySubsets = {};
                                            var end2 = reader.uint32() + reader.pos;
                                            key = "";
                                            value = null;
                                            while (reader.pos < end2) {
                                                var tag2 = reader.uint32();
                                                switch (tag2 >>> 3) {
                                                case 1:
                                                    key = reader.string();
                                                    break;
                                                case 2:
                                                    value = $root.google.bigtable.admin.v2.AuthorizedView.FamilySubsets.decode(reader, reader.uint32());
                                                    break;
                                                default:
                                                    reader.skipType(tag2 & 7);
                                                    break;
                                                }
                                            }
                                            message.familySubsets[key] = value;
                                            break;
                                        }
                                    default:
                                        reader.skipType(tag & 7);
                                        break;
                                    }
                                }
                                return message;
                            };
    
                            /**
                             * Decodes a SubsetView message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.bigtable.admin.v2.AuthorizedView.SubsetView
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.bigtable.admin.v2.AuthorizedView.SubsetView} SubsetView
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            SubsetView.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a SubsetView message.
                             * @function verify
                             * @memberof google.bigtable.admin.v2.AuthorizedView.SubsetView
                             * @static
                             * @param {Object.<string,*>} message Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            SubsetView.verify = function verify(message) {
                                if (typeof message !== "object" || message === null)
                                    return "object expected";
                                if (message.rowPrefixes != null && message.hasOwnProperty("rowPrefixes")) {
                                    if (!Array.isArray(message.rowPrefixes))
                                        return "rowPrefixes: array expected";
                                    for (var i = 0; i < message.rowPrefixes.length; ++i)
                                        if (!(message.rowPrefixes[i] && typeof message.rowPrefixes[i].length === "number" || $util.isString(message.rowPrefixes[i])))
                                            return "rowPrefixes: buffer[] expected";
                                }
                                if (message.familySubsets != null && message.hasOwnProperty("familySubsets")) {
                                    if (!$util.isObject(message.familySubsets))
                                        return "familySubsets: object expected";
                                    var key = Object.keys(message.familySubsets);
                                    for (var i = 0; i < key.length; ++i) {
                                        var error = $root.google.bigtable.admin.v2.AuthorizedView.FamilySubsets.verify(message.familySubsets[key[i]]);
                                        if (error)
                                            return "familySubsets." + error;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates a SubsetView message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.bigtable.admin.v2.AuthorizedView.SubsetView
                             * @static
                             * @param {Object.<string,*>} object Plain object
                             * @returns {google.bigtable.admin.v2.AuthorizedView.SubsetView} SubsetView
                             */
                            SubsetView.fromObject = function fromObject(object) {
                                if (object instanceof $root.google.bigtable.admin.v2.AuthorizedView.SubsetView)
                                    return object;
                                var message = new $root.google.bigtable.admin.v2.AuthorizedView.SubsetView();
                                if (object.rowPrefixes) {
                                    if (!Array.isArray(object.rowPrefixes))
                                        throw TypeError(".google.bigtable.admin.v2.AuthorizedView.SubsetView.rowPrefixes: array expected");
                                    message.rowPrefixes = [];
                                    for (var i = 0; i < object.rowPrefixes.length; ++i)
                                        if (typeof object.rowPrefixes[i] === "string")
                                            $util.base64.decode(object.rowPrefixes[i], message.rowPrefixes[i] = $util.newBuffer($util.base64.length(object.rowPrefixes[i])), 0);
                                        else if (object.rowPrefixes[i].length >= 0)
                                            message.rowPrefixes[i] = object.rowPrefixes[i];
                                }
                                if (object.familySubsets) {
                                    if (typeof object.familySubsets !== "object")
                                        throw TypeError(".google.bigtable.admin.v2.AuthorizedView.SubsetView.familySubsets: object expected");
                                    message.familySubsets = {};
                                    for (var keys = Object.keys(object.familySubsets), i = 0; i < keys.length; ++i) {
                                        if (typeof object.familySubsets[keys[i]] !== "object")
                                            throw TypeError(".google.bigtable.admin.v2.AuthorizedView.SubsetView.familySubsets: object expected");
                                        message.familySubsets[keys[i]] = $root.google.bigtable.admin.v2.AuthorizedView.FamilySubsets.fromObject(object.familySubsets[keys[i]]);
                                    }
                                }
                                return message;
                            };
    
                            /**
                             * Creates a plain object from a SubsetView message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.bigtable.admin.v2.AuthorizedView.SubsetView
                             * @static
                             * @param {google.bigtable.admin.v2.AuthorizedView.SubsetView} message SubsetView
                             * @param {$protobuf.IConversionOptions} [options] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            SubsetView.toObject = function toObject(message, options) {
                                if (!options)
                                    options = {};
                                var object = {};
                                if (options.arrays || options.defaults)
                                    object.rowPrefixes = [];
                                if (options.objects || options.defaults)
                                    object.familySubsets = {};
                                if (message.rowPrefixes && message.rowPrefixes.length) {
                                    object.rowPrefixes = [];
                                    for (var j = 0; j < message.rowPrefixes.length; ++j)
                                        object.rowPrefixes[j] = options.bytes === String ? $util.base64.encode(message.rowPrefixes[j], 0, message.rowPrefixes[j].length) : options.bytes === Array ? Array.prototype.slice.call(message.rowPrefixes[j]) : message.rowPrefixes[j];
                                }
                                var keys2;
                                if (message.familySubsets && (keys2 = Object.keys(message.familySubsets)).length) {
                                    object.familySubsets = {};
                                    for (var j = 0; j < keys2.length; ++j)
                                        object.familySubsets[keys2[j]] = $root.google.bigtable.admin.v2.AuthorizedView.FamilySubsets.toObject(message.familySubsets[keys2[j]], options);
                                }
                                return object;
                            };
    
                            /**
                             * Converts this SubsetView to JSON.
                             * @function toJSON
                             * @memberof google.bigtable.admin.v2.AuthorizedView.SubsetView
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            SubsetView.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * Gets the default type url for SubsetView
                             * @function getTypeUrl
                             * @memberof google.bigtable.admin.v2.AuthorizedView.SubsetView
                             * @static
                             * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                             * @returns {string} The default type url
                             */
                            SubsetView.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                                if (typeUrlPrefix === undefined) {
                                    typeUrlPrefix = "type.googleapis.com";
                                }
                                return typeUrlPrefix + "/google.bigtable.admin.v2.AuthorizedView.SubsetView";
                            };
    
                            return SubsetView;
                        })();
    
                        /**
                         * ResponseView enum.
                         * @name google.bigtable.admin.v2.AuthorizedView.ResponseView
                         * @enum {number}
                         * @property {number} RESPONSE_VIEW_UNSPECIFIED=0 RESPONSE_VIEW_UNSPECIFIED value
                         * @property {number} NAME_ONLY=1 NAME_ONLY value
                         * @property {number} BASIC=2 BASIC value
                         * @property {number} FULL=3 FULL value
                         */
                        AuthorizedView.ResponseView = (function() {
                            var valuesById = {}, values = Object.create(valuesById);
                            values[valuesById[0] = "RESPONSE_VIEW_UNSPECIFIED"] = 0;
                            values[valuesById[1] = "NAME_ONLY"] = 1;
                            values[valuesById[2] = "BASIC"] = 2;
                            values[valuesById[3] = "FULL"] = 3;
                            return values;
                        })();
    
                        return AuthorizedView;
                    })();
    
                    v2.ColumnFamily = (function() {
    
                        /**
                         * Properties of a ColumnFamily.
                         * @memberof google.bigtable.admin.v2
                         * @interface IColumnFamily
                         * @property {google.bigtable.admin.v2.IGcRule|null} [gcRule] ColumnFamily gcRule
                         * @property {google.bigtable.admin.v2.IType|null} [valueType] ColumnFamily valueType
                         */
    
                        /**
                         * Constructs a new ColumnFamily.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a ColumnFamily.
                         * @implements IColumnFamily
                         * @constructor
                         * @param {google.bigtable.admin.v2.IColumnFamily=} [properties] Properties to set
                         */
                        function ColumnFamily(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * ColumnFamily gcRule.
                         * @member {google.bigtable.admin.v2.IGcRule|null|undefined} gcRule
                         * @memberof google.bigtable.admin.v2.ColumnFamily
                         * @instance
                         */
                        ColumnFamily.prototype.gcRule = null;
    
                        /**
                         * ColumnFamily valueType.
                         * @member {google.bigtable.admin.v2.IType|null|undefined} valueType
                         * @memberof google.bigtable.admin.v2.ColumnFamily
                         * @instance
                         */
                        ColumnFamily.prototype.valueType = null;
    
                        /**
                         * Creates a new ColumnFamily instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.ColumnFamily
                         * @static
                         * @param {google.bigtable.admin.v2.IColumnFamily=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.ColumnFamily} ColumnFamily instance
                         */
                        ColumnFamily.create = function create(properties) {
                            return new ColumnFamily(properties);
                        };
    
                        /**
                         * Encodes the specified ColumnFamily message. Does not implicitly {@link google.bigtable.admin.v2.ColumnFamily.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.ColumnFamily
                         * @static
                         * @param {google.bigtable.admin.v2.IColumnFamily} message ColumnFamily message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ColumnFamily.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.gcRule != null && Object.hasOwnProperty.call(message, "gcRule"))
                                $root.google.bigtable.admin.v2.GcRule.encode(message.gcRule, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            if (message.valueType != null && Object.hasOwnProperty.call(message, "valueType"))
                                $root.google.bigtable.admin.v2.Type.encode(message.valueType, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified ColumnFamily message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.ColumnFamily.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.ColumnFamily
                         * @static
                         * @param {google.bigtable.admin.v2.IColumnFamily} message ColumnFamily message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        ColumnFamily.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a ColumnFamily message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.ColumnFamily
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.ColumnFamily} ColumnFamily
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ColumnFamily.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.ColumnFamily();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.gcRule = $root.google.bigtable.admin.v2.GcRule.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 3: {
                                        message.valueType = $root.google.bigtable.admin.v2.Type.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a ColumnFamily message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.ColumnFamily
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.ColumnFamily} ColumnFamily
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        ColumnFamily.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a ColumnFamily message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.ColumnFamily
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        ColumnFamily.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.gcRule != null && message.hasOwnProperty("gcRule")) {
                                var error = $root.google.bigtable.admin.v2.GcRule.verify(message.gcRule);
                                if (error)
                                    return "gcRule." + error;
                            }
                            if (message.valueType != null && message.hasOwnProperty("valueType")) {
                                var error = $root.google.bigtable.admin.v2.Type.verify(message.valueType);
                                if (error)
                                    return "valueType." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates a ColumnFamily message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.ColumnFamily
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.ColumnFamily} ColumnFamily
                         */
                        ColumnFamily.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.ColumnFamily)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.ColumnFamily();
                            if (object.gcRule != null) {
                                if (typeof object.gcRule !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.ColumnFamily.gcRule: object expected");
                                message.gcRule = $root.google.bigtable.admin.v2.GcRule.fromObject(object.gcRule);
                            }
                            if (object.valueType != null) {
                                if (typeof object.valueType !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.ColumnFamily.valueType: object expected");
                                message.valueType = $root.google.bigtable.admin.v2.Type.fromObject(object.valueType);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a ColumnFamily message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.ColumnFamily
                         * @static
                         * @param {google.bigtable.admin.v2.ColumnFamily} message ColumnFamily
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        ColumnFamily.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.gcRule = null;
                                object.valueType = null;
                            }
                            if (message.gcRule != null && message.hasOwnProperty("gcRule"))
                                object.gcRule = $root.google.bigtable.admin.v2.GcRule.toObject(message.gcRule, options);
                            if (message.valueType != null && message.hasOwnProperty("valueType"))
                                object.valueType = $root.google.bigtable.admin.v2.Type.toObject(message.valueType, options);
                            return object;
                        };
    
                        /**
                         * Converts this ColumnFamily to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.ColumnFamily
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        ColumnFamily.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for ColumnFamily
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.ColumnFamily
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        ColumnFamily.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.ColumnFamily";
                        };
    
                        return ColumnFamily;
                    })();
    
                    v2.GcRule = (function() {
    
                        /**
                         * Properties of a GcRule.
                         * @memberof google.bigtable.admin.v2
                         * @interface IGcRule
                         * @property {number|null} [maxNumVersions] GcRule maxNumVersions
                         * @property {google.protobuf.IDuration|null} [maxAge] GcRule maxAge
                         * @property {google.bigtable.admin.v2.GcRule.IIntersection|null} [intersection] GcRule intersection
                         * @property {google.bigtable.admin.v2.GcRule.IUnion|null} [union] GcRule union
                         */
    
                        /**
                         * Constructs a new GcRule.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a GcRule.
                         * @implements IGcRule
                         * @constructor
                         * @param {google.bigtable.admin.v2.IGcRule=} [properties] Properties to set
                         */
                        function GcRule(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * GcRule maxNumVersions.
                         * @member {number|null|undefined} maxNumVersions
                         * @memberof google.bigtable.admin.v2.GcRule
                         * @instance
                         */
                        GcRule.prototype.maxNumVersions = null;
    
                        /**
                         * GcRule maxAge.
                         * @member {google.protobuf.IDuration|null|undefined} maxAge
                         * @memberof google.bigtable.admin.v2.GcRule
                         * @instance
                         */
                        GcRule.prototype.maxAge = null;
    
                        /**
                         * GcRule intersection.
                         * @member {google.bigtable.admin.v2.GcRule.IIntersection|null|undefined} intersection
                         * @memberof google.bigtable.admin.v2.GcRule
                         * @instance
                         */
                        GcRule.prototype.intersection = null;
    
                        /**
                         * GcRule union.
                         * @member {google.bigtable.admin.v2.GcRule.IUnion|null|undefined} union
                         * @memberof google.bigtable.admin.v2.GcRule
                         * @instance
                         */
                        GcRule.prototype.union = null;
    
                        // OneOf field names bound to virtual getters and setters
                        var $oneOfFields;
    
                        /**
                         * GcRule rule.
                         * @member {"maxNumVersions"|"maxAge"|"intersection"|"union"|undefined} rule
                         * @memberof google.bigtable.admin.v2.GcRule
                         * @instance
                         */
                        Object.defineProperty(GcRule.prototype, "rule", {
                            get: $util.oneOfGetter($oneOfFields = ["maxNumVersions", "maxAge", "intersection", "union"]),
                            set: $util.oneOfSetter($oneOfFields)
                        });
    
                        /**
                         * Creates a new GcRule instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.GcRule
                         * @static
                         * @param {google.bigtable.admin.v2.IGcRule=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.GcRule} GcRule instance
                         */
                        GcRule.create = function create(properties) {
                            return new GcRule(properties);
                        };
    
                        /**
                         * Encodes the specified GcRule message. Does not implicitly {@link google.bigtable.admin.v2.GcRule.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.GcRule
                         * @static
                         * @param {google.bigtable.admin.v2.IGcRule} message GcRule message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        GcRule.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.maxNumVersions != null && Object.hasOwnProperty.call(message, "maxNumVersions"))
                                writer.uint32(/* id 1, wireType 0 =*/8).int32(message.maxNumVersions);
                            if (message.maxAge != null && Object.hasOwnProperty.call(message, "maxAge"))
                                $root.google.protobuf.Duration.encode(message.maxAge, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            if (message.intersection != null && Object.hasOwnProperty.call(message, "intersection"))
                                $root.google.bigtable.admin.v2.GcRule.Intersection.encode(message.intersection, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            if (message.union != null && Object.hasOwnProperty.call(message, "union"))
                                $root.google.bigtable.admin.v2.GcRule.Union.encode(message.union, writer.uint32(/* id 4, wireType 2 =*/34).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified GcRule message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.GcRule.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.GcRule
                         * @static
                         * @param {google.bigtable.admin.v2.IGcRule} message GcRule message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        GcRule.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a GcRule message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.GcRule
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.GcRule} GcRule
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        GcRule.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.GcRule();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.maxNumVersions = reader.int32();
                                        break;
                                    }
                                case 2: {
                                        message.maxAge = $root.google.protobuf.Duration.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 3: {
                                        message.intersection = $root.google.bigtable.admin.v2.GcRule.Intersection.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 4: {
                                        message.union = $root.google.bigtable.admin.v2.GcRule.Union.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a GcRule message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.GcRule
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.GcRule} GcRule
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        GcRule.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a GcRule message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.GcRule
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        GcRule.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            var properties = {};
                            if (message.maxNumVersions != null && message.hasOwnProperty("maxNumVersions")) {
                                properties.rule = 1;
                                if (!$util.isInteger(message.maxNumVersions))
                                    return "maxNumVersions: integer expected";
                            }
                            if (message.maxAge != null && message.hasOwnProperty("maxAge")) {
                                if (properties.rule === 1)
                                    return "rule: multiple values";
                                properties.rule = 1;
                                {
                                    var error = $root.google.protobuf.Duration.verify(message.maxAge);
                                    if (error)
                                        return "maxAge." + error;
                                }
                            }
                            if (message.intersection != null && message.hasOwnProperty("intersection")) {
                                if (properties.rule === 1)
                                    return "rule: multiple values";
                                properties.rule = 1;
                                {
                                    var error = $root.google.bigtable.admin.v2.GcRule.Intersection.verify(message.intersection);
                                    if (error)
                                        return "intersection." + error;
                                }
                            }
                            if (message.union != null && message.hasOwnProperty("union")) {
                                if (properties.rule === 1)
                                    return "rule: multiple values";
                                properties.rule = 1;
                                {
                                    var error = $root.google.bigtable.admin.v2.GcRule.Union.verify(message.union);
                                    if (error)
                                        return "union." + error;
                                }
                            }
                            return null;
                        };
    
                        /**
                         * Creates a GcRule message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.GcRule
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.GcRule} GcRule
                         */
                        GcRule.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.GcRule)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.GcRule();
                            if (object.maxNumVersions != null)
                                message.maxNumVersions = object.maxNumVersions | 0;
                            if (object.maxAge != null) {
                                if (typeof object.maxAge !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.GcRule.maxAge: object expected");
                                message.maxAge = $root.google.protobuf.Duration.fromObject(object.maxAge);
                            }
                            if (object.intersection != null) {
                                if (typeof object.intersection !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.GcRule.intersection: object expected");
                                message.intersection = $root.google.bigtable.admin.v2.GcRule.Intersection.fromObject(object.intersection);
                            }
                            if (object.union != null) {
                                if (typeof object.union !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.GcRule.union: object expected");
                                message.union = $root.google.bigtable.admin.v2.GcRule.Union.fromObject(object.union);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a GcRule message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.GcRule
                         * @static
                         * @param {google.bigtable.admin.v2.GcRule} message GcRule
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        GcRule.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (message.maxNumVersions != null && message.hasOwnProperty("maxNumVersions")) {
                                object.maxNumVersions = message.maxNumVersions;
                                if (options.oneofs)
                                    object.rule = "maxNumVersions";
                            }
                            if (message.maxAge != null && message.hasOwnProperty("maxAge")) {
                                object.maxAge = $root.google.protobuf.Duration.toObject(message.maxAge, options);
                                if (options.oneofs)
                                    object.rule = "maxAge";
                            }
                            if (message.intersection != null && message.hasOwnProperty("intersection")) {
                                object.intersection = $root.google.bigtable.admin.v2.GcRule.Intersection.toObject(message.intersection, options);
                                if (options.oneofs)
                                    object.rule = "intersection";
                            }
                            if (message.union != null && message.hasOwnProperty("union")) {
                                object.union = $root.google.bigtable.admin.v2.GcRule.Union.toObject(message.union, options);
                                if (options.oneofs)
                                    object.rule = "union";
                            }
                            return object;
                        };
    
                        /**
                         * Converts this GcRule to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.GcRule
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        GcRule.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for GcRule
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.GcRule
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        GcRule.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.GcRule";
                        };
    
                        GcRule.Intersection = (function() {
    
                            /**
                             * Properties of an Intersection.
                             * @memberof google.bigtable.admin.v2.GcRule
                             * @interface IIntersection
                             * @property {Array.<google.bigtable.admin.v2.IGcRule>|null} [rules] Intersection rules
                             */
    
                            /**
                             * Constructs a new Intersection.
                             * @memberof google.bigtable.admin.v2.GcRule
                             * @classdesc Represents an Intersection.
                             * @implements IIntersection
                             * @constructor
                             * @param {google.bigtable.admin.v2.GcRule.IIntersection=} [properties] Properties to set
                             */
                            function Intersection(properties) {
                                this.rules = [];
                                if (properties)
                                    for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                        if (properties[keys[i]] != null)
                                            this[keys[i]] = properties[keys[i]];
                            }
    
                            /**
                             * Intersection rules.
                             * @member {Array.<google.bigtable.admin.v2.IGcRule>} rules
                             * @memberof google.bigtable.admin.v2.GcRule.Intersection
                             * @instance
                             */
                            Intersection.prototype.rules = $util.emptyArray;
    
                            /**
                             * Creates a new Intersection instance using the specified properties.
                             * @function create
                             * @memberof google.bigtable.admin.v2.GcRule.Intersection
                             * @static
                             * @param {google.bigtable.admin.v2.GcRule.IIntersection=} [properties] Properties to set
                             * @returns {google.bigtable.admin.v2.GcRule.Intersection} Intersection instance
                             */
                            Intersection.create = function create(properties) {
                                return new Intersection(properties);
                            };
    
                            /**
                             * Encodes the specified Intersection message. Does not implicitly {@link google.bigtable.admin.v2.GcRule.Intersection.verify|verify} messages.
                             * @function encode
                             * @memberof google.bigtable.admin.v2.GcRule.Intersection
                             * @static
                             * @param {google.bigtable.admin.v2.GcRule.IIntersection} message Intersection message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Intersection.encode = function encode(message, writer) {
                                if (!writer)
                                    writer = $Writer.create();
                                if (message.rules != null && message.rules.length)
                                    for (var i = 0; i < message.rules.length; ++i)
                                        $root.google.bigtable.admin.v2.GcRule.encode(message.rules[i], writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                                return writer;
                            };
    
                            /**
                             * Encodes the specified Intersection message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.GcRule.Intersection.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.bigtable.admin.v2.GcRule.Intersection
                             * @static
                             * @param {google.bigtable.admin.v2.GcRule.IIntersection} message Intersection message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Intersection.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an Intersection message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.bigtable.admin.v2.GcRule.Intersection
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @param {number} [length] Message length if known beforehand
                             * @returns {google.bigtable.admin.v2.GcRule.Intersection} Intersection
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Intersection.decode = function decode(reader, length) {
                                if (!(reader instanceof $Reader))
                                    reader = $Reader.create(reader);
                                var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.GcRule.Intersection();
                                while (reader.pos < end) {
                                    var tag = reader.uint32();
                                    switch (tag >>> 3) {
                                    case 1: {
                                            if (!(message.rules && message.rules.length))
                                                message.rules = [];
                                            message.rules.push($root.google.bigtable.admin.v2.GcRule.decode(reader, reader.uint32()));
                                            break;
                                        }
                                    default:
                                        reader.skipType(tag & 7);
                                        break;
                                    }
                                }
                                return message;
                            };
    
                            /**
                             * Decodes an Intersection message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.bigtable.admin.v2.GcRule.Intersection
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.bigtable.admin.v2.GcRule.Intersection} Intersection
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Intersection.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an Intersection message.
                             * @function verify
                             * @memberof google.bigtable.admin.v2.GcRule.Intersection
                             * @static
                             * @param {Object.<string,*>} message Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            Intersection.verify = function verify(message) {
                                if (typeof message !== "object" || message === null)
                                    return "object expected";
                                if (message.rules != null && message.hasOwnProperty("rules")) {
                                    if (!Array.isArray(message.rules))
                                        return "rules: array expected";
                                    for (var i = 0; i < message.rules.length; ++i) {
                                        var error = $root.google.bigtable.admin.v2.GcRule.verify(message.rules[i]);
                                        if (error)
                                            return "rules." + error;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an Intersection message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.bigtable.admin.v2.GcRule.Intersection
                             * @static
                             * @param {Object.<string,*>} object Plain object
                             * @returns {google.bigtable.admin.v2.GcRule.Intersection} Intersection
                             */
                            Intersection.fromObject = function fromObject(object) {
                                if (object instanceof $root.google.bigtable.admin.v2.GcRule.Intersection)
                                    return object;
                                var message = new $root.google.bigtable.admin.v2.GcRule.Intersection();
                                if (object.rules) {
                                    if (!Array.isArray(object.rules))
                                        throw TypeError(".google.bigtable.admin.v2.GcRule.Intersection.rules: array expected");
                                    message.rules = [];
                                    for (var i = 0; i < object.rules.length; ++i) {
                                        if (typeof object.rules[i] !== "object")
                                            throw TypeError(".google.bigtable.admin.v2.GcRule.Intersection.rules: object expected");
                                        message.rules[i] = $root.google.bigtable.admin.v2.GcRule.fromObject(object.rules[i]);
                                    }
                                }
                                return message;
                            };
    
                            /**
                             * Creates a plain object from an Intersection message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.bigtable.admin.v2.GcRule.Intersection
                             * @static
                             * @param {google.bigtable.admin.v2.GcRule.Intersection} message Intersection
                             * @param {$protobuf.IConversionOptions} [options] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            Intersection.toObject = function toObject(message, options) {
                                if (!options)
                                    options = {};
                                var object = {};
                                if (options.arrays || options.defaults)
                                    object.rules = [];
                                if (message.rules && message.rules.length) {
                                    object.rules = [];
                                    for (var j = 0; j < message.rules.length; ++j)
                                        object.rules[j] = $root.google.bigtable.admin.v2.GcRule.toObject(message.rules[j], options);
                                }
                                return object;
                            };
    
                            /**
                             * Converts this Intersection to JSON.
                             * @function toJSON
                             * @memberof google.bigtable.admin.v2.GcRule.Intersection
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            Intersection.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * Gets the default type url for Intersection
                             * @function getTypeUrl
                             * @memberof google.bigtable.admin.v2.GcRule.Intersection
                             * @static
                             * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                             * @returns {string} The default type url
                             */
                            Intersection.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                                if (typeUrlPrefix === undefined) {
                                    typeUrlPrefix = "type.googleapis.com";
                                }
                                return typeUrlPrefix + "/google.bigtable.admin.v2.GcRule.Intersection";
                            };
    
                            return Intersection;
                        })();
    
                        GcRule.Union = (function() {
    
                            /**
                             * Properties of an Union.
                             * @memberof google.bigtable.admin.v2.GcRule
                             * @interface IUnion
                             * @property {Array.<google.bigtable.admin.v2.IGcRule>|null} [rules] Union rules
                             */
    
                            /**
                             * Constructs a new Union.
                             * @memberof google.bigtable.admin.v2.GcRule
                             * @classdesc Represents an Union.
                             * @implements IUnion
                             * @constructor
                             * @param {google.bigtable.admin.v2.GcRule.IUnion=} [properties] Properties to set
                             */
                            function Union(properties) {
                                this.rules = [];
                                if (properties)
                                    for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                        if (properties[keys[i]] != null)
                                            this[keys[i]] = properties[keys[i]];
                            }
    
                            /**
                             * Union rules.
                             * @member {Array.<google.bigtable.admin.v2.IGcRule>} rules
                             * @memberof google.bigtable.admin.v2.GcRule.Union
                             * @instance
                             */
                            Union.prototype.rules = $util.emptyArray;
    
                            /**
                             * Creates a new Union instance using the specified properties.
                             * @function create
                             * @memberof google.bigtable.admin.v2.GcRule.Union
                             * @static
                             * @param {google.bigtable.admin.v2.GcRule.IUnion=} [properties] Properties to set
                             * @returns {google.bigtable.admin.v2.GcRule.Union} Union instance
                             */
                            Union.create = function create(properties) {
                                return new Union(properties);
                            };
    
                            /**
                             * Encodes the specified Union message. Does not implicitly {@link google.bigtable.admin.v2.GcRule.Union.verify|verify} messages.
                             * @function encode
                             * @memberof google.bigtable.admin.v2.GcRule.Union
                             * @static
                             * @param {google.bigtable.admin.v2.GcRule.IUnion} message Union message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Union.encode = function encode(message, writer) {
                                if (!writer)
                                    writer = $Writer.create();
                                if (message.rules != null && message.rules.length)
                                    for (var i = 0; i < message.rules.length; ++i)
                                        $root.google.bigtable.admin.v2.GcRule.encode(message.rules[i], writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                                return writer;
                            };
    
                            /**
                             * Encodes the specified Union message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.GcRule.Union.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.bigtable.admin.v2.GcRule.Union
                             * @static
                             * @param {google.bigtable.admin.v2.GcRule.IUnion} message Union message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Union.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an Union message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.bigtable.admin.v2.GcRule.Union
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @param {number} [length] Message length if known beforehand
                             * @returns {google.bigtable.admin.v2.GcRule.Union} Union
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Union.decode = function decode(reader, length) {
                                if (!(reader instanceof $Reader))
                                    reader = $Reader.create(reader);
                                var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.GcRule.Union();
                                while (reader.pos < end) {
                                    var tag = reader.uint32();
                                    switch (tag >>> 3) {
                                    case 1: {
                                            if (!(message.rules && message.rules.length))
                                                message.rules = [];
                                            message.rules.push($root.google.bigtable.admin.v2.GcRule.decode(reader, reader.uint32()));
                                            break;
                                        }
                                    default:
                                        reader.skipType(tag & 7);
                                        break;
                                    }
                                }
                                return message;
                            };
    
                            /**
                             * Decodes an Union message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.bigtable.admin.v2.GcRule.Union
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.bigtable.admin.v2.GcRule.Union} Union
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Union.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an Union message.
                             * @function verify
                             * @memberof google.bigtable.admin.v2.GcRule.Union
                             * @static
                             * @param {Object.<string,*>} message Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            Union.verify = function verify(message) {
                                if (typeof message !== "object" || message === null)
                                    return "object expected";
                                if (message.rules != null && message.hasOwnProperty("rules")) {
                                    if (!Array.isArray(message.rules))
                                        return "rules: array expected";
                                    for (var i = 0; i < message.rules.length; ++i) {
                                        var error = $root.google.bigtable.admin.v2.GcRule.verify(message.rules[i]);
                                        if (error)
                                            return "rules." + error;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an Union message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.bigtable.admin.v2.GcRule.Union
                             * @static
                             * @param {Object.<string,*>} object Plain object
                             * @returns {google.bigtable.admin.v2.GcRule.Union} Union
                             */
                            Union.fromObject = function fromObject(object) {
                                if (object instanceof $root.google.bigtable.admin.v2.GcRule.Union)
                                    return object;
                                var message = new $root.google.bigtable.admin.v2.GcRule.Union();
                                if (object.rules) {
                                    if (!Array.isArray(object.rules))
                                        throw TypeError(".google.bigtable.admin.v2.GcRule.Union.rules: array expected");
                                    message.rules = [];
                                    for (var i = 0; i < object.rules.length; ++i) {
                                        if (typeof object.rules[i] !== "object")
                                            throw TypeError(".google.bigtable.admin.v2.GcRule.Union.rules: object expected");
                                        message.rules[i] = $root.google.bigtable.admin.v2.GcRule.fromObject(object.rules[i]);
                                    }
                                }
                                return message;
                            };
    
                            /**
                             * Creates a plain object from an Union message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.bigtable.admin.v2.GcRule.Union
                             * @static
                             * @param {google.bigtable.admin.v2.GcRule.Union} message Union
                             * @param {$protobuf.IConversionOptions} [options] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            Union.toObject = function toObject(message, options) {
                                if (!options)
                                    options = {};
                                var object = {};
                                if (options.arrays || options.defaults)
                                    object.rules = [];
                                if (message.rules && message.rules.length) {
                                    object.rules = [];
                                    for (var j = 0; j < message.rules.length; ++j)
                                        object.rules[j] = $root.google.bigtable.admin.v2.GcRule.toObject(message.rules[j], options);
                                }
                                return object;
                            };
    
                            /**
                             * Converts this Union to JSON.
                             * @function toJSON
                             * @memberof google.bigtable.admin.v2.GcRule.Union
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            Union.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * Gets the default type url for Union
                             * @function getTypeUrl
                             * @memberof google.bigtable.admin.v2.GcRule.Union
                             * @static
                             * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                             * @returns {string} The default type url
                             */
                            Union.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                                if (typeUrlPrefix === undefined) {
                                    typeUrlPrefix = "type.googleapis.com";
                                }
                                return typeUrlPrefix + "/google.bigtable.admin.v2.GcRule.Union";
                            };
    
                            return Union;
                        })();
    
                        return GcRule;
                    })();
    
                    v2.EncryptionInfo = (function() {
    
                        /**
                         * Properties of an EncryptionInfo.
                         * @memberof google.bigtable.admin.v2
                         * @interface IEncryptionInfo
                         * @property {google.bigtable.admin.v2.EncryptionInfo.EncryptionType|null} [encryptionType] EncryptionInfo encryptionType
                         * @property {google.rpc.IStatus|null} [encryptionStatus] EncryptionInfo encryptionStatus
                         * @property {string|null} [kmsKeyVersion] EncryptionInfo kmsKeyVersion
                         */
    
                        /**
                         * Constructs a new EncryptionInfo.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents an EncryptionInfo.
                         * @implements IEncryptionInfo
                         * @constructor
                         * @param {google.bigtable.admin.v2.IEncryptionInfo=} [properties] Properties to set
                         */
                        function EncryptionInfo(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * EncryptionInfo encryptionType.
                         * @member {google.bigtable.admin.v2.EncryptionInfo.EncryptionType} encryptionType
                         * @memberof google.bigtable.admin.v2.EncryptionInfo
                         * @instance
                         */
                        EncryptionInfo.prototype.encryptionType = 0;
    
                        /**
                         * EncryptionInfo encryptionStatus.
                         * @member {google.rpc.IStatus|null|undefined} encryptionStatus
                         * @memberof google.bigtable.admin.v2.EncryptionInfo
                         * @instance
                         */
                        EncryptionInfo.prototype.encryptionStatus = null;
    
                        /**
                         * EncryptionInfo kmsKeyVersion.
                         * @member {string} kmsKeyVersion
                         * @memberof google.bigtable.admin.v2.EncryptionInfo
                         * @instance
                         */
                        EncryptionInfo.prototype.kmsKeyVersion = "";
    
                        /**
                         * Creates a new EncryptionInfo instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.EncryptionInfo
                         * @static
                         * @param {google.bigtable.admin.v2.IEncryptionInfo=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.EncryptionInfo} EncryptionInfo instance
                         */
                        EncryptionInfo.create = function create(properties) {
                            return new EncryptionInfo(properties);
                        };
    
                        /**
                         * Encodes the specified EncryptionInfo message. Does not implicitly {@link google.bigtable.admin.v2.EncryptionInfo.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.EncryptionInfo
                         * @static
                         * @param {google.bigtable.admin.v2.IEncryptionInfo} message EncryptionInfo message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        EncryptionInfo.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.kmsKeyVersion != null && Object.hasOwnProperty.call(message, "kmsKeyVersion"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.kmsKeyVersion);
                            if (message.encryptionType != null && Object.hasOwnProperty.call(message, "encryptionType"))
                                writer.uint32(/* id 3, wireType 0 =*/24).int32(message.encryptionType);
                            if (message.encryptionStatus != null && Object.hasOwnProperty.call(message, "encryptionStatus"))
                                $root.google.rpc.Status.encode(message.encryptionStatus, writer.uint32(/* id 4, wireType 2 =*/34).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified EncryptionInfo message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.EncryptionInfo.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.EncryptionInfo
                         * @static
                         * @param {google.bigtable.admin.v2.IEncryptionInfo} message EncryptionInfo message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        EncryptionInfo.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes an EncryptionInfo message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.EncryptionInfo
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.EncryptionInfo} EncryptionInfo
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        EncryptionInfo.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.EncryptionInfo();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 3: {
                                        message.encryptionType = reader.int32();
                                        break;
                                    }
                                case 4: {
                                        message.encryptionStatus = $root.google.rpc.Status.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 2: {
                                        message.kmsKeyVersion = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes an EncryptionInfo message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.EncryptionInfo
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.EncryptionInfo} EncryptionInfo
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        EncryptionInfo.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies an EncryptionInfo message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.EncryptionInfo
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        EncryptionInfo.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.encryptionType != null && message.hasOwnProperty("encryptionType"))
                                switch (message.encryptionType) {
                                default:
                                    return "encryptionType: enum value expected";
                                case 0:
                                case 1:
                                case 2:
                                    break;
                                }
                            if (message.encryptionStatus != null && message.hasOwnProperty("encryptionStatus")) {
                                var error = $root.google.rpc.Status.verify(message.encryptionStatus);
                                if (error)
                                    return "encryptionStatus." + error;
                            }
                            if (message.kmsKeyVersion != null && message.hasOwnProperty("kmsKeyVersion"))
                                if (!$util.isString(message.kmsKeyVersion))
                                    return "kmsKeyVersion: string expected";
                            return null;
                        };
    
                        /**
                         * Creates an EncryptionInfo message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.EncryptionInfo
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.EncryptionInfo} EncryptionInfo
                         */
                        EncryptionInfo.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.EncryptionInfo)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.EncryptionInfo();
                            switch (object.encryptionType) {
                            default:
                                if (typeof object.encryptionType === "number") {
                                    message.encryptionType = object.encryptionType;
                                    break;
                                }
                                break;
                            case "ENCRYPTION_TYPE_UNSPECIFIED":
                            case 0:
                                message.encryptionType = 0;
                                break;
                            case "GOOGLE_DEFAULT_ENCRYPTION":
                            case 1:
                                message.encryptionType = 1;
                                break;
                            case "CUSTOMER_MANAGED_ENCRYPTION":
                            case 2:
                                message.encryptionType = 2;
                                break;
                            }
                            if (object.encryptionStatus != null) {
                                if (typeof object.encryptionStatus !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.EncryptionInfo.encryptionStatus: object expected");
                                message.encryptionStatus = $root.google.rpc.Status.fromObject(object.encryptionStatus);
                            }
                            if (object.kmsKeyVersion != null)
                                message.kmsKeyVersion = String(object.kmsKeyVersion);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from an EncryptionInfo message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.EncryptionInfo
                         * @static
                         * @param {google.bigtable.admin.v2.EncryptionInfo} message EncryptionInfo
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        EncryptionInfo.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.kmsKeyVersion = "";
                                object.encryptionType = options.enums === String ? "ENCRYPTION_TYPE_UNSPECIFIED" : 0;
                                object.encryptionStatus = null;
                            }
                            if (message.kmsKeyVersion != null && message.hasOwnProperty("kmsKeyVersion"))
                                object.kmsKeyVersion = message.kmsKeyVersion;
                            if (message.encryptionType != null && message.hasOwnProperty("encryptionType"))
                                object.encryptionType = options.enums === String ? $root.google.bigtable.admin.v2.EncryptionInfo.EncryptionType[message.encryptionType] === undefined ? message.encryptionType : $root.google.bigtable.admin.v2.EncryptionInfo.EncryptionType[message.encryptionType] : message.encryptionType;
                            if (message.encryptionStatus != null && message.hasOwnProperty("encryptionStatus"))
                                object.encryptionStatus = $root.google.rpc.Status.toObject(message.encryptionStatus, options);
                            return object;
                        };
    
                        /**
                         * Converts this EncryptionInfo to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.EncryptionInfo
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        EncryptionInfo.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for EncryptionInfo
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.EncryptionInfo
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        EncryptionInfo.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.EncryptionInfo";
                        };
    
                        /**
                         * EncryptionType enum.
                         * @name google.bigtable.admin.v2.EncryptionInfo.EncryptionType
                         * @enum {number}
                         * @property {number} ENCRYPTION_TYPE_UNSPECIFIED=0 ENCRYPTION_TYPE_UNSPECIFIED value
                         * @property {number} GOOGLE_DEFAULT_ENCRYPTION=1 GOOGLE_DEFAULT_ENCRYPTION value
                         * @property {number} CUSTOMER_MANAGED_ENCRYPTION=2 CUSTOMER_MANAGED_ENCRYPTION value
                         */
                        EncryptionInfo.EncryptionType = (function() {
                            var valuesById = {}, values = Object.create(valuesById);
                            values[valuesById[0] = "ENCRYPTION_TYPE_UNSPECIFIED"] = 0;
                            values[valuesById[1] = "GOOGLE_DEFAULT_ENCRYPTION"] = 1;
                            values[valuesById[2] = "CUSTOMER_MANAGED_ENCRYPTION"] = 2;
                            return values;
                        })();
    
                        return EncryptionInfo;
                    })();
    
                    v2.Snapshot = (function() {
    
                        /**
                         * Properties of a Snapshot.
                         * @memberof google.bigtable.admin.v2
                         * @interface ISnapshot
                         * @property {string|null} [name] Snapshot name
                         * @property {google.bigtable.admin.v2.ITable|null} [sourceTable] Snapshot sourceTable
                         * @property {number|Long|null} [dataSizeBytes] Snapshot dataSizeBytes
                         * @property {google.protobuf.ITimestamp|null} [createTime] Snapshot createTime
                         * @property {google.protobuf.ITimestamp|null} [deleteTime] Snapshot deleteTime
                         * @property {google.bigtable.admin.v2.Snapshot.State|null} [state] Snapshot state
                         * @property {string|null} [description] Snapshot description
                         */
    
                        /**
                         * Constructs a new Snapshot.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a Snapshot.
                         * @implements ISnapshot
                         * @constructor
                         * @param {google.bigtable.admin.v2.ISnapshot=} [properties] Properties to set
                         */
                        function Snapshot(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * Snapshot name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.Snapshot
                         * @instance
                         */
                        Snapshot.prototype.name = "";
    
                        /**
                         * Snapshot sourceTable.
                         * @member {google.bigtable.admin.v2.ITable|null|undefined} sourceTable
                         * @memberof google.bigtable.admin.v2.Snapshot
                         * @instance
                         */
                        Snapshot.prototype.sourceTable = null;
    
                        /**
                         * Snapshot dataSizeBytes.
                         * @member {number|Long} dataSizeBytes
                         * @memberof google.bigtable.admin.v2.Snapshot
                         * @instance
                         */
                        Snapshot.prototype.dataSizeBytes = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                        /**
                         * Snapshot createTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} createTime
                         * @memberof google.bigtable.admin.v2.Snapshot
                         * @instance
                         */
                        Snapshot.prototype.createTime = null;
    
                        /**
                         * Snapshot deleteTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} deleteTime
                         * @memberof google.bigtable.admin.v2.Snapshot
                         * @instance
                         */
                        Snapshot.prototype.deleteTime = null;
    
                        /**
                         * Snapshot state.
                         * @member {google.bigtable.admin.v2.Snapshot.State} state
                         * @memberof google.bigtable.admin.v2.Snapshot
                         * @instance
                         */
                        Snapshot.prototype.state = 0;
    
                        /**
                         * Snapshot description.
                         * @member {string} description
                         * @memberof google.bigtable.admin.v2.Snapshot
                         * @instance
                         */
                        Snapshot.prototype.description = "";
    
                        /**
                         * Creates a new Snapshot instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.Snapshot
                         * @static
                         * @param {google.bigtable.admin.v2.ISnapshot=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.Snapshot} Snapshot instance
                         */
                        Snapshot.create = function create(properties) {
                            return new Snapshot(properties);
                        };
    
                        /**
                         * Encodes the specified Snapshot message. Does not implicitly {@link google.bigtable.admin.v2.Snapshot.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.Snapshot
                         * @static
                         * @param {google.bigtable.admin.v2.ISnapshot} message Snapshot message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        Snapshot.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            if (message.sourceTable != null && Object.hasOwnProperty.call(message, "sourceTable"))
                                $root.google.bigtable.admin.v2.Table.encode(message.sourceTable, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            if (message.dataSizeBytes != null && Object.hasOwnProperty.call(message, "dataSizeBytes"))
                                writer.uint32(/* id 3, wireType 0 =*/24).int64(message.dataSizeBytes);
                            if (message.createTime != null && Object.hasOwnProperty.call(message, "createTime"))
                                $root.google.protobuf.Timestamp.encode(message.createTime, writer.uint32(/* id 4, wireType 2 =*/34).fork()).ldelim();
                            if (message.deleteTime != null && Object.hasOwnProperty.call(message, "deleteTime"))
                                $root.google.protobuf.Timestamp.encode(message.deleteTime, writer.uint32(/* id 5, wireType 2 =*/42).fork()).ldelim();
                            if (message.state != null && Object.hasOwnProperty.call(message, "state"))
                                writer.uint32(/* id 6, wireType 0 =*/48).int32(message.state);
                            if (message.description != null && Object.hasOwnProperty.call(message, "description"))
                                writer.uint32(/* id 7, wireType 2 =*/58).string(message.description);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified Snapshot message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.Snapshot.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.Snapshot
                         * @static
                         * @param {google.bigtable.admin.v2.ISnapshot} message Snapshot message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        Snapshot.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a Snapshot message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.Snapshot
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.Snapshot} Snapshot
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        Snapshot.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.Snapshot();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.sourceTable = $root.google.bigtable.admin.v2.Table.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 3: {
                                        message.dataSizeBytes = reader.int64();
                                        break;
                                    }
                                case 4: {
                                        message.createTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 5: {
                                        message.deleteTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 6: {
                                        message.state = reader.int32();
                                        break;
                                    }
                                case 7: {
                                        message.description = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a Snapshot message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.Snapshot
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.Snapshot} Snapshot
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        Snapshot.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a Snapshot message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.Snapshot
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        Snapshot.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            if (message.sourceTable != null && message.hasOwnProperty("sourceTable")) {
                                var error = $root.google.bigtable.admin.v2.Table.verify(message.sourceTable);
                                if (error)
                                    return "sourceTable." + error;
                            }
                            if (message.dataSizeBytes != null && message.hasOwnProperty("dataSizeBytes"))
                                if (!$util.isInteger(message.dataSizeBytes) && !(message.dataSizeBytes && $util.isInteger(message.dataSizeBytes.low) && $util.isInteger(message.dataSizeBytes.high)))
                                    return "dataSizeBytes: integer|Long expected";
                            if (message.createTime != null && message.hasOwnProperty("createTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.createTime);
                                if (error)
                                    return "createTime." + error;
                            }
                            if (message.deleteTime != null && message.hasOwnProperty("deleteTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.deleteTime);
                                if (error)
                                    return "deleteTime." + error;
                            }
                            if (message.state != null && message.hasOwnProperty("state"))
                                switch (message.state) {
                                default:
                                    return "state: enum value expected";
                                case 0:
                                case 1:
                                case 2:
                                    break;
                                }
                            if (message.description != null && message.hasOwnProperty("description"))
                                if (!$util.isString(message.description))
                                    return "description: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a Snapshot message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.Snapshot
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.Snapshot} Snapshot
                         */
                        Snapshot.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.Snapshot)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.Snapshot();
                            if (object.name != null)
                                message.name = String(object.name);
                            if (object.sourceTable != null) {
                                if (typeof object.sourceTable !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.Snapshot.sourceTable: object expected");
                                message.sourceTable = $root.google.bigtable.admin.v2.Table.fromObject(object.sourceTable);
                            }
                            if (object.dataSizeBytes != null)
                                if ($util.Long)
                                    (message.dataSizeBytes = $util.Long.fromValue(object.dataSizeBytes)).unsigned = false;
                                else if (typeof object.dataSizeBytes === "string")
                                    message.dataSizeBytes = parseInt(object.dataSizeBytes, 10);
                                else if (typeof object.dataSizeBytes === "number")
                                    message.dataSizeBytes = object.dataSizeBytes;
                                else if (typeof object.dataSizeBytes === "object")
                                    message.dataSizeBytes = new $util.LongBits(object.dataSizeBytes.low >>> 0, object.dataSizeBytes.high >>> 0).toNumber();
                            if (object.createTime != null) {
                                if (typeof object.createTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.Snapshot.createTime: object expected");
                                message.createTime = $root.google.protobuf.Timestamp.fromObject(object.createTime);
                            }
                            if (object.deleteTime != null) {
                                if (typeof object.deleteTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.Snapshot.deleteTime: object expected");
                                message.deleteTime = $root.google.protobuf.Timestamp.fromObject(object.deleteTime);
                            }
                            switch (object.state) {
                            default:
                                if (typeof object.state === "number") {
                                    message.state = object.state;
                                    break;
                                }
                                break;
                            case "STATE_NOT_KNOWN":
                            case 0:
                                message.state = 0;
                                break;
                            case "READY":
                            case 1:
                                message.state = 1;
                                break;
                            case "CREATING":
                            case 2:
                                message.state = 2;
                                break;
                            }
                            if (object.description != null)
                                message.description = String(object.description);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a Snapshot message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.Snapshot
                         * @static
                         * @param {google.bigtable.admin.v2.Snapshot} message Snapshot
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        Snapshot.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.name = "";
                                object.sourceTable = null;
                                if ($util.Long) {
                                    var long = new $util.Long(0, 0, false);
                                    object.dataSizeBytes = options.longs === String ? long.toString() : options.longs === Number ? long.toNumber() : long;
                                } else
                                    object.dataSizeBytes = options.longs === String ? "0" : 0;
                                object.createTime = null;
                                object.deleteTime = null;
                                object.state = options.enums === String ? "STATE_NOT_KNOWN" : 0;
                                object.description = "";
                            }
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            if (message.sourceTable != null && message.hasOwnProperty("sourceTable"))
                                object.sourceTable = $root.google.bigtable.admin.v2.Table.toObject(message.sourceTable, options);
                            if (message.dataSizeBytes != null && message.hasOwnProperty("dataSizeBytes"))
                                if (typeof message.dataSizeBytes === "number")
                                    object.dataSizeBytes = options.longs === String ? String(message.dataSizeBytes) : message.dataSizeBytes;
                                else
                                    object.dataSizeBytes = options.longs === String ? $util.Long.prototype.toString.call(message.dataSizeBytes) : options.longs === Number ? new $util.LongBits(message.dataSizeBytes.low >>> 0, message.dataSizeBytes.high >>> 0).toNumber() : message.dataSizeBytes;
                            if (message.createTime != null && message.hasOwnProperty("createTime"))
                                object.createTime = $root.google.protobuf.Timestamp.toObject(message.createTime, options);
                            if (message.deleteTime != null && message.hasOwnProperty("deleteTime"))
                                object.deleteTime = $root.google.protobuf.Timestamp.toObject(message.deleteTime, options);
                            if (message.state != null && message.hasOwnProperty("state"))
                                object.state = options.enums === String ? $root.google.bigtable.admin.v2.Snapshot.State[message.state] === undefined ? message.state : $root.google.bigtable.admin.v2.Snapshot.State[message.state] : message.state;
                            if (message.description != null && message.hasOwnProperty("description"))
                                object.description = message.description;
                            return object;
                        };
    
                        /**
                         * Converts this Snapshot to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.Snapshot
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        Snapshot.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for Snapshot
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.Snapshot
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        Snapshot.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.Snapshot";
                        };
    
                        /**
                         * State enum.
                         * @name google.bigtable.admin.v2.Snapshot.State
                         * @enum {number}
                         * @property {number} STATE_NOT_KNOWN=0 STATE_NOT_KNOWN value
                         * @property {number} READY=1 READY value
                         * @property {number} CREATING=2 CREATING value
                         */
                        Snapshot.State = (function() {
                            var valuesById = {}, values = Object.create(valuesById);
                            values[valuesById[0] = "STATE_NOT_KNOWN"] = 0;
                            values[valuesById[1] = "READY"] = 1;
                            values[valuesById[2] = "CREATING"] = 2;
                            return values;
                        })();
    
                        return Snapshot;
                    })();
    
                    v2.Backup = (function() {
    
                        /**
                         * Properties of a Backup.
                         * @memberof google.bigtable.admin.v2
                         * @interface IBackup
                         * @property {string|null} [name] Backup name
                         * @property {string|null} [sourceTable] Backup sourceTable
                         * @property {string|null} [sourceBackup] Backup sourceBackup
                         * @property {google.protobuf.ITimestamp|null} [expireTime] Backup expireTime
                         * @property {google.protobuf.ITimestamp|null} [startTime] Backup startTime
                         * @property {google.protobuf.ITimestamp|null} [endTime] Backup endTime
                         * @property {number|Long|null} [sizeBytes] Backup sizeBytes
                         * @property {google.bigtable.admin.v2.Backup.State|null} [state] Backup state
                         * @property {google.bigtable.admin.v2.IEncryptionInfo|null} [encryptionInfo] Backup encryptionInfo
                         */
    
                        /**
                         * Constructs a new Backup.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a Backup.
                         * @implements IBackup
                         * @constructor
                         * @param {google.bigtable.admin.v2.IBackup=} [properties] Properties to set
                         */
                        function Backup(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * Backup name.
                         * @member {string} name
                         * @memberof google.bigtable.admin.v2.Backup
                         * @instance
                         */
                        Backup.prototype.name = "";
    
                        /**
                         * Backup sourceTable.
                         * @member {string} sourceTable
                         * @memberof google.bigtable.admin.v2.Backup
                         * @instance
                         */
                        Backup.prototype.sourceTable = "";
    
                        /**
                         * Backup sourceBackup.
                         * @member {string} sourceBackup
                         * @memberof google.bigtable.admin.v2.Backup
                         * @instance
                         */
                        Backup.prototype.sourceBackup = "";
    
                        /**
                         * Backup expireTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} expireTime
                         * @memberof google.bigtable.admin.v2.Backup
                         * @instance
                         */
                        Backup.prototype.expireTime = null;
    
                        /**
                         * Backup startTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} startTime
                         * @memberof google.bigtable.admin.v2.Backup
                         * @instance
                         */
                        Backup.prototype.startTime = null;
    
                        /**
                         * Backup endTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} endTime
                         * @memberof google.bigtable.admin.v2.Backup
                         * @instance
                         */
                        Backup.prototype.endTime = null;
    
                        /**
                         * Backup sizeBytes.
                         * @member {number|Long} sizeBytes
                         * @memberof google.bigtable.admin.v2.Backup
                         * @instance
                         */
                        Backup.prototype.sizeBytes = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                        /**
                         * Backup state.
                         * @member {google.bigtable.admin.v2.Backup.State} state
                         * @memberof google.bigtable.admin.v2.Backup
                         * @instance
                         */
                        Backup.prototype.state = 0;
    
                        /**
                         * Backup encryptionInfo.
                         * @member {google.bigtable.admin.v2.IEncryptionInfo|null|undefined} encryptionInfo
                         * @memberof google.bigtable.admin.v2.Backup
                         * @instance
                         */
                        Backup.prototype.encryptionInfo = null;
    
                        /**
                         * Creates a new Backup instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.Backup
                         * @static
                         * @param {google.bigtable.admin.v2.IBackup=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.Backup} Backup instance
                         */
                        Backup.create = function create(properties) {
                            return new Backup(properties);
                        };
    
                        /**
                         * Encodes the specified Backup message. Does not implicitly {@link google.bigtable.admin.v2.Backup.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.Backup
                         * @static
                         * @param {google.bigtable.admin.v2.IBackup} message Backup message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        Backup.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                            if (message.sourceTable != null && Object.hasOwnProperty.call(message, "sourceTable"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.sourceTable);
                            if (message.expireTime != null && Object.hasOwnProperty.call(message, "expireTime"))
                                $root.google.protobuf.Timestamp.encode(message.expireTime, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            if (message.startTime != null && Object.hasOwnProperty.call(message, "startTime"))
                                $root.google.protobuf.Timestamp.encode(message.startTime, writer.uint32(/* id 4, wireType 2 =*/34).fork()).ldelim();
                            if (message.endTime != null && Object.hasOwnProperty.call(message, "endTime"))
                                $root.google.protobuf.Timestamp.encode(message.endTime, writer.uint32(/* id 5, wireType 2 =*/42).fork()).ldelim();
                            if (message.sizeBytes != null && Object.hasOwnProperty.call(message, "sizeBytes"))
                                writer.uint32(/* id 6, wireType 0 =*/48).int64(message.sizeBytes);
                            if (message.state != null && Object.hasOwnProperty.call(message, "state"))
                                writer.uint32(/* id 7, wireType 0 =*/56).int32(message.state);
                            if (message.encryptionInfo != null && Object.hasOwnProperty.call(message, "encryptionInfo"))
                                $root.google.bigtable.admin.v2.EncryptionInfo.encode(message.encryptionInfo, writer.uint32(/* id 9, wireType 2 =*/74).fork()).ldelim();
                            if (message.sourceBackup != null && Object.hasOwnProperty.call(message, "sourceBackup"))
                                writer.uint32(/* id 10, wireType 2 =*/82).string(message.sourceBackup);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified Backup message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.Backup.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.Backup
                         * @static
                         * @param {google.bigtable.admin.v2.IBackup} message Backup message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        Backup.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a Backup message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.Backup
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.Backup} Backup
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        Backup.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.Backup();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.name = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.sourceTable = reader.string();
                                        break;
                                    }
                                case 10: {
                                        message.sourceBackup = reader.string();
                                        break;
                                    }
                                case 3: {
                                        message.expireTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 4: {
                                        message.startTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 5: {
                                        message.endTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 6: {
                                        message.sizeBytes = reader.int64();
                                        break;
                                    }
                                case 7: {
                                        message.state = reader.int32();
                                        break;
                                    }
                                case 9: {
                                        message.encryptionInfo = $root.google.bigtable.admin.v2.EncryptionInfo.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a Backup message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.Backup
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.Backup} Backup
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        Backup.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a Backup message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.Backup
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        Backup.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.name != null && message.hasOwnProperty("name"))
                                if (!$util.isString(message.name))
                                    return "name: string expected";
                            if (message.sourceTable != null && message.hasOwnProperty("sourceTable"))
                                if (!$util.isString(message.sourceTable))
                                    return "sourceTable: string expected";
                            if (message.sourceBackup != null && message.hasOwnProperty("sourceBackup"))
                                if (!$util.isString(message.sourceBackup))
                                    return "sourceBackup: string expected";
                            if (message.expireTime != null && message.hasOwnProperty("expireTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.expireTime);
                                if (error)
                                    return "expireTime." + error;
                            }
                            if (message.startTime != null && message.hasOwnProperty("startTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.startTime);
                                if (error)
                                    return "startTime." + error;
                            }
                            if (message.endTime != null && message.hasOwnProperty("endTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.endTime);
                                if (error)
                                    return "endTime." + error;
                            }
                            if (message.sizeBytes != null && message.hasOwnProperty("sizeBytes"))
                                if (!$util.isInteger(message.sizeBytes) && !(message.sizeBytes && $util.isInteger(message.sizeBytes.low) && $util.isInteger(message.sizeBytes.high)))
                                    return "sizeBytes: integer|Long expected";
                            if (message.state != null && message.hasOwnProperty("state"))
                                switch (message.state) {
                                default:
                                    return "state: enum value expected";
                                case 0:
                                case 1:
                                case 2:
                                    break;
                                }
                            if (message.encryptionInfo != null && message.hasOwnProperty("encryptionInfo")) {
                                var error = $root.google.bigtable.admin.v2.EncryptionInfo.verify(message.encryptionInfo);
                                if (error)
                                    return "encryptionInfo." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates a Backup message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.Backup
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.Backup} Backup
                         */
                        Backup.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.Backup)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.Backup();
                            if (object.name != null)
                                message.name = String(object.name);
                            if (object.sourceTable != null)
                                message.sourceTable = String(object.sourceTable);
                            if (object.sourceBackup != null)
                                message.sourceBackup = String(object.sourceBackup);
                            if (object.expireTime != null) {
                                if (typeof object.expireTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.Backup.expireTime: object expected");
                                message.expireTime = $root.google.protobuf.Timestamp.fromObject(object.expireTime);
                            }
                            if (object.startTime != null) {
                                if (typeof object.startTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.Backup.startTime: object expected");
                                message.startTime = $root.google.protobuf.Timestamp.fromObject(object.startTime);
                            }
                            if (object.endTime != null) {
                                if (typeof object.endTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.Backup.endTime: object expected");
                                message.endTime = $root.google.protobuf.Timestamp.fromObject(object.endTime);
                            }
                            if (object.sizeBytes != null)
                                if ($util.Long)
                                    (message.sizeBytes = $util.Long.fromValue(object.sizeBytes)).unsigned = false;
                                else if (typeof object.sizeBytes === "string")
                                    message.sizeBytes = parseInt(object.sizeBytes, 10);
                                else if (typeof object.sizeBytes === "number")
                                    message.sizeBytes = object.sizeBytes;
                                else if (typeof object.sizeBytes === "object")
                                    message.sizeBytes = new $util.LongBits(object.sizeBytes.low >>> 0, object.sizeBytes.high >>> 0).toNumber();
                            switch (object.state) {
                            default:
                                if (typeof object.state === "number") {
                                    message.state = object.state;
                                    break;
                                }
                                break;
                            case "STATE_UNSPECIFIED":
                            case 0:
                                message.state = 0;
                                break;
                            case "CREATING":
                            case 1:
                                message.state = 1;
                                break;
                            case "READY":
                            case 2:
                                message.state = 2;
                                break;
                            }
                            if (object.encryptionInfo != null) {
                                if (typeof object.encryptionInfo !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.Backup.encryptionInfo: object expected");
                                message.encryptionInfo = $root.google.bigtable.admin.v2.EncryptionInfo.fromObject(object.encryptionInfo);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a Backup message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.Backup
                         * @static
                         * @param {google.bigtable.admin.v2.Backup} message Backup
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        Backup.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.name = "";
                                object.sourceTable = "";
                                object.expireTime = null;
                                object.startTime = null;
                                object.endTime = null;
                                if ($util.Long) {
                                    var long = new $util.Long(0, 0, false);
                                    object.sizeBytes = options.longs === String ? long.toString() : options.longs === Number ? long.toNumber() : long;
                                } else
                                    object.sizeBytes = options.longs === String ? "0" : 0;
                                object.state = options.enums === String ? "STATE_UNSPECIFIED" : 0;
                                object.encryptionInfo = null;
                                object.sourceBackup = "";
                            }
                            if (message.name != null && message.hasOwnProperty("name"))
                                object.name = message.name;
                            if (message.sourceTable != null && message.hasOwnProperty("sourceTable"))
                                object.sourceTable = message.sourceTable;
                            if (message.expireTime != null && message.hasOwnProperty("expireTime"))
                                object.expireTime = $root.google.protobuf.Timestamp.toObject(message.expireTime, options);
                            if (message.startTime != null && message.hasOwnProperty("startTime"))
                                object.startTime = $root.google.protobuf.Timestamp.toObject(message.startTime, options);
                            if (message.endTime != null && message.hasOwnProperty("endTime"))
                                object.endTime = $root.google.protobuf.Timestamp.toObject(message.endTime, options);
                            if (message.sizeBytes != null && message.hasOwnProperty("sizeBytes"))
                                if (typeof message.sizeBytes === "number")
                                    object.sizeBytes = options.longs === String ? String(message.sizeBytes) : message.sizeBytes;
                                else
                                    object.sizeBytes = options.longs === String ? $util.Long.prototype.toString.call(message.sizeBytes) : options.longs === Number ? new $util.LongBits(message.sizeBytes.low >>> 0, message.sizeBytes.high >>> 0).toNumber() : message.sizeBytes;
                            if (message.state != null && message.hasOwnProperty("state"))
                                object.state = options.enums === String ? $root.google.bigtable.admin.v2.Backup.State[message.state] === undefined ? message.state : $root.google.bigtable.admin.v2.Backup.State[message.state] : message.state;
                            if (message.encryptionInfo != null && message.hasOwnProperty("encryptionInfo"))
                                object.encryptionInfo = $root.google.bigtable.admin.v2.EncryptionInfo.toObject(message.encryptionInfo, options);
                            if (message.sourceBackup != null && message.hasOwnProperty("sourceBackup"))
                                object.sourceBackup = message.sourceBackup;
                            return object;
                        };
    
                        /**
                         * Converts this Backup to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.Backup
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        Backup.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for Backup
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.Backup
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        Backup.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.Backup";
                        };
    
                        /**
                         * State enum.
                         * @name google.bigtable.admin.v2.Backup.State
                         * @enum {number}
                         * @property {number} STATE_UNSPECIFIED=0 STATE_UNSPECIFIED value
                         * @property {number} CREATING=1 CREATING value
                         * @property {number} READY=2 READY value
                         */
                        Backup.State = (function() {
                            var valuesById = {}, values = Object.create(valuesById);
                            values[valuesById[0] = "STATE_UNSPECIFIED"] = 0;
                            values[valuesById[1] = "CREATING"] = 1;
                            values[valuesById[2] = "READY"] = 2;
                            return values;
                        })();
    
                        return Backup;
                    })();
    
                    v2.BackupInfo = (function() {
    
                        /**
                         * Properties of a BackupInfo.
                         * @memberof google.bigtable.admin.v2
                         * @interface IBackupInfo
                         * @property {string|null} [backup] BackupInfo backup
                         * @property {google.protobuf.ITimestamp|null} [startTime] BackupInfo startTime
                         * @property {google.protobuf.ITimestamp|null} [endTime] BackupInfo endTime
                         * @property {string|null} [sourceTable] BackupInfo sourceTable
                         * @property {string|null} [sourceBackup] BackupInfo sourceBackup
                         */
    
                        /**
                         * Constructs a new BackupInfo.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a BackupInfo.
                         * @implements IBackupInfo
                         * @constructor
                         * @param {google.bigtable.admin.v2.IBackupInfo=} [properties] Properties to set
                         */
                        function BackupInfo(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * BackupInfo backup.
                         * @member {string} backup
                         * @memberof google.bigtable.admin.v2.BackupInfo
                         * @instance
                         */
                        BackupInfo.prototype.backup = "";
    
                        /**
                         * BackupInfo startTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} startTime
                         * @memberof google.bigtable.admin.v2.BackupInfo
                         * @instance
                         */
                        BackupInfo.prototype.startTime = null;
    
                        /**
                         * BackupInfo endTime.
                         * @member {google.protobuf.ITimestamp|null|undefined} endTime
                         * @memberof google.bigtable.admin.v2.BackupInfo
                         * @instance
                         */
                        BackupInfo.prototype.endTime = null;
    
                        /**
                         * BackupInfo sourceTable.
                         * @member {string} sourceTable
                         * @memberof google.bigtable.admin.v2.BackupInfo
                         * @instance
                         */
                        BackupInfo.prototype.sourceTable = "";
    
                        /**
                         * BackupInfo sourceBackup.
                         * @member {string} sourceBackup
                         * @memberof google.bigtable.admin.v2.BackupInfo
                         * @instance
                         */
                        BackupInfo.prototype.sourceBackup = "";
    
                        /**
                         * Creates a new BackupInfo instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.BackupInfo
                         * @static
                         * @param {google.bigtable.admin.v2.IBackupInfo=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.BackupInfo} BackupInfo instance
                         */
                        BackupInfo.create = function create(properties) {
                            return new BackupInfo(properties);
                        };
    
                        /**
                         * Encodes the specified BackupInfo message. Does not implicitly {@link google.bigtable.admin.v2.BackupInfo.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.BackupInfo
                         * @static
                         * @param {google.bigtable.admin.v2.IBackupInfo} message BackupInfo message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        BackupInfo.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.backup != null && Object.hasOwnProperty.call(message, "backup"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.backup);
                            if (message.startTime != null && Object.hasOwnProperty.call(message, "startTime"))
                                $root.google.protobuf.Timestamp.encode(message.startTime, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            if (message.endTime != null && Object.hasOwnProperty.call(message, "endTime"))
                                $root.google.protobuf.Timestamp.encode(message.endTime, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            if (message.sourceTable != null && Object.hasOwnProperty.call(message, "sourceTable"))
                                writer.uint32(/* id 4, wireType 2 =*/34).string(message.sourceTable);
                            if (message.sourceBackup != null && Object.hasOwnProperty.call(message, "sourceBackup"))
                                writer.uint32(/* id 10, wireType 2 =*/82).string(message.sourceBackup);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified BackupInfo message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.BackupInfo.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.BackupInfo
                         * @static
                         * @param {google.bigtable.admin.v2.IBackupInfo} message BackupInfo message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        BackupInfo.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a BackupInfo message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.BackupInfo
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.BackupInfo} BackupInfo
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        BackupInfo.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.BackupInfo();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.backup = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.startTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 3: {
                                        message.endTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 4: {
                                        message.sourceTable = reader.string();
                                        break;
                                    }
                                case 10: {
                                        message.sourceBackup = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a BackupInfo message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.BackupInfo
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.BackupInfo} BackupInfo
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        BackupInfo.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a BackupInfo message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.BackupInfo
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        BackupInfo.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.backup != null && message.hasOwnProperty("backup"))
                                if (!$util.isString(message.backup))
                                    return "backup: string expected";
                            if (message.startTime != null && message.hasOwnProperty("startTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.startTime);
                                if (error)
                                    return "startTime." + error;
                            }
                            if (message.endTime != null && message.hasOwnProperty("endTime")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.endTime);
                                if (error)
                                    return "endTime." + error;
                            }
                            if (message.sourceTable != null && message.hasOwnProperty("sourceTable"))
                                if (!$util.isString(message.sourceTable))
                                    return "sourceTable: string expected";
                            if (message.sourceBackup != null && message.hasOwnProperty("sourceBackup"))
                                if (!$util.isString(message.sourceBackup))
                                    return "sourceBackup: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a BackupInfo message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.BackupInfo
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.BackupInfo} BackupInfo
                         */
                        BackupInfo.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.BackupInfo)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.BackupInfo();
                            if (object.backup != null)
                                message.backup = String(object.backup);
                            if (object.startTime != null) {
                                if (typeof object.startTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.BackupInfo.startTime: object expected");
                                message.startTime = $root.google.protobuf.Timestamp.fromObject(object.startTime);
                            }
                            if (object.endTime != null) {
                                if (typeof object.endTime !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.BackupInfo.endTime: object expected");
                                message.endTime = $root.google.protobuf.Timestamp.fromObject(object.endTime);
                            }
                            if (object.sourceTable != null)
                                message.sourceTable = String(object.sourceTable);
                            if (object.sourceBackup != null)
                                message.sourceBackup = String(object.sourceBackup);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a BackupInfo message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.BackupInfo
                         * @static
                         * @param {google.bigtable.admin.v2.BackupInfo} message BackupInfo
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        BackupInfo.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.backup = "";
                                object.startTime = null;
                                object.endTime = null;
                                object.sourceTable = "";
                                object.sourceBackup = "";
                            }
                            if (message.backup != null && message.hasOwnProperty("backup"))
                                object.backup = message.backup;
                            if (message.startTime != null && message.hasOwnProperty("startTime"))
                                object.startTime = $root.google.protobuf.Timestamp.toObject(message.startTime, options);
                            if (message.endTime != null && message.hasOwnProperty("endTime"))
                                object.endTime = $root.google.protobuf.Timestamp.toObject(message.endTime, options);
                            if (message.sourceTable != null && message.hasOwnProperty("sourceTable"))
                                object.sourceTable = message.sourceTable;
                            if (message.sourceBackup != null && message.hasOwnProperty("sourceBackup"))
                                object.sourceBackup = message.sourceBackup;
                            return object;
                        };
    
                        /**
                         * Converts this BackupInfo to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.BackupInfo
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        BackupInfo.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for BackupInfo
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.BackupInfo
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        BackupInfo.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.BackupInfo";
                        };
    
                        return BackupInfo;
                    })();
    
                    /**
                     * RestoreSourceType enum.
                     * @name google.bigtable.admin.v2.RestoreSourceType
                     * @enum {number}
                     * @property {number} RESTORE_SOURCE_TYPE_UNSPECIFIED=0 RESTORE_SOURCE_TYPE_UNSPECIFIED value
                     * @property {number} BACKUP=1 BACKUP value
                     */
                    v2.RestoreSourceType = (function() {
                        var valuesById = {}, values = Object.create(valuesById);
                        values[valuesById[0] = "RESTORE_SOURCE_TYPE_UNSPECIFIED"] = 0;
                        values[valuesById[1] = "BACKUP"] = 1;
                        return values;
                    })();
    
                    v2.Type = (function() {
    
                        /**
                         * Properties of a Type.
                         * @memberof google.bigtable.admin.v2
                         * @interface IType
                         * @property {google.bigtable.admin.v2.Type.IBytes|null} [bytesType] Type bytesType
                         * @property {google.bigtable.admin.v2.Type.IInt64|null} [int64Type] Type int64Type
                         * @property {google.bigtable.admin.v2.Type.IAggregate|null} [aggregateType] Type aggregateType
                         */
    
                        /**
                         * Constructs a new Type.
                         * @memberof google.bigtable.admin.v2
                         * @classdesc Represents a Type.
                         * @implements IType
                         * @constructor
                         * @param {google.bigtable.admin.v2.IType=} [properties] Properties to set
                         */
                        function Type(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * Type bytesType.
                         * @member {google.bigtable.admin.v2.Type.IBytes|null|undefined} bytesType
                         * @memberof google.bigtable.admin.v2.Type
                         * @instance
                         */
                        Type.prototype.bytesType = null;
    
                        /**
                         * Type int64Type.
                         * @member {google.bigtable.admin.v2.Type.IInt64|null|undefined} int64Type
                         * @memberof google.bigtable.admin.v2.Type
                         * @instance
                         */
                        Type.prototype.int64Type = null;
    
                        /**
                         * Type aggregateType.
                         * @member {google.bigtable.admin.v2.Type.IAggregate|null|undefined} aggregateType
                         * @memberof google.bigtable.admin.v2.Type
                         * @instance
                         */
                        Type.prototype.aggregateType = null;
    
                        // OneOf field names bound to virtual getters and setters
                        var $oneOfFields;
    
                        /**
                         * Type kind.
                         * @member {"bytesType"|"int64Type"|"aggregateType"|undefined} kind
                         * @memberof google.bigtable.admin.v2.Type
                         * @instance
                         */
                        Object.defineProperty(Type.prototype, "kind", {
                            get: $util.oneOfGetter($oneOfFields = ["bytesType", "int64Type", "aggregateType"]),
                            set: $util.oneOfSetter($oneOfFields)
                        });
    
                        /**
                         * Creates a new Type instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.admin.v2.Type
                         * @static
                         * @param {google.bigtable.admin.v2.IType=} [properties] Properties to set
                         * @returns {google.bigtable.admin.v2.Type} Type instance
                         */
                        Type.create = function create(properties) {
                            return new Type(properties);
                        };
    
                        /**
                         * Encodes the specified Type message. Does not implicitly {@link google.bigtable.admin.v2.Type.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.admin.v2.Type
                         * @static
                         * @param {google.bigtable.admin.v2.IType} message Type message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        Type.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.bytesType != null && Object.hasOwnProperty.call(message, "bytesType"))
                                $root.google.bigtable.admin.v2.Type.Bytes.encode(message.bytesType, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            if (message.int64Type != null && Object.hasOwnProperty.call(message, "int64Type"))
                                $root.google.bigtable.admin.v2.Type.Int64.encode(message.int64Type, writer.uint32(/* id 5, wireType 2 =*/42).fork()).ldelim();
                            if (message.aggregateType != null && Object.hasOwnProperty.call(message, "aggregateType"))
                                $root.google.bigtable.admin.v2.Type.Aggregate.encode(message.aggregateType, writer.uint32(/* id 6, wireType 2 =*/50).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified Type message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.Type.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.admin.v2.Type
                         * @static
                         * @param {google.bigtable.admin.v2.IType} message Type message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        Type.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a Type message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.admin.v2.Type
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.admin.v2.Type} Type
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        Type.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.Type();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.bytesType = $root.google.bigtable.admin.v2.Type.Bytes.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 5: {
                                        message.int64Type = $root.google.bigtable.admin.v2.Type.Int64.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 6: {
                                        message.aggregateType = $root.google.bigtable.admin.v2.Type.Aggregate.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a Type message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.admin.v2.Type
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.admin.v2.Type} Type
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        Type.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a Type message.
                         * @function verify
                         * @memberof google.bigtable.admin.v2.Type
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        Type.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            var properties = {};
                            if (message.bytesType != null && message.hasOwnProperty("bytesType")) {
                                properties.kind = 1;
                                {
                                    var error = $root.google.bigtable.admin.v2.Type.Bytes.verify(message.bytesType);
                                    if (error)
                                        return "bytesType." + error;
                                }
                            }
                            if (message.int64Type != null && message.hasOwnProperty("int64Type")) {
                                if (properties.kind === 1)
                                    return "kind: multiple values";
                                properties.kind = 1;
                                {
                                    var error = $root.google.bigtable.admin.v2.Type.Int64.verify(message.int64Type);
                                    if (error)
                                        return "int64Type." + error;
                                }
                            }
                            if (message.aggregateType != null && message.hasOwnProperty("aggregateType")) {
                                if (properties.kind === 1)
                                    return "kind: multiple values";
                                properties.kind = 1;
                                {
                                    var error = $root.google.bigtable.admin.v2.Type.Aggregate.verify(message.aggregateType);
                                    if (error)
                                        return "aggregateType." + error;
                                }
                            }
                            return null;
                        };
    
                        /**
                         * Creates a Type message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.admin.v2.Type
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.admin.v2.Type} Type
                         */
                        Type.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.admin.v2.Type)
                                return object;
                            var message = new $root.google.bigtable.admin.v2.Type();
                            if (object.bytesType != null) {
                                if (typeof object.bytesType !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.Type.bytesType: object expected");
                                message.bytesType = $root.google.bigtable.admin.v2.Type.Bytes.fromObject(object.bytesType);
                            }
                            if (object.int64Type != null) {
                                if (typeof object.int64Type !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.Type.int64Type: object expected");
                                message.int64Type = $root.google.bigtable.admin.v2.Type.Int64.fromObject(object.int64Type);
                            }
                            if (object.aggregateType != null) {
                                if (typeof object.aggregateType !== "object")
                                    throw TypeError(".google.bigtable.admin.v2.Type.aggregateType: object expected");
                                message.aggregateType = $root.google.bigtable.admin.v2.Type.Aggregate.fromObject(object.aggregateType);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a Type message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.admin.v2.Type
                         * @static
                         * @param {google.bigtable.admin.v2.Type} message Type
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        Type.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (message.bytesType != null && message.hasOwnProperty("bytesType")) {
                                object.bytesType = $root.google.bigtable.admin.v2.Type.Bytes.toObject(message.bytesType, options);
                                if (options.oneofs)
                                    object.kind = "bytesType";
                            }
                            if (message.int64Type != null && message.hasOwnProperty("int64Type")) {
                                object.int64Type = $root.google.bigtable.admin.v2.Type.Int64.toObject(message.int64Type, options);
                                if (options.oneofs)
                                    object.kind = "int64Type";
                            }
                            if (message.aggregateType != null && message.hasOwnProperty("aggregateType")) {
                                object.aggregateType = $root.google.bigtable.admin.v2.Type.Aggregate.toObject(message.aggregateType, options);
                                if (options.oneofs)
                                    object.kind = "aggregateType";
                            }
                            return object;
                        };
    
                        /**
                         * Converts this Type to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.admin.v2.Type
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        Type.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for Type
                         * @function getTypeUrl
                         * @memberof google.bigtable.admin.v2.Type
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        Type.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.admin.v2.Type";
                        };
    
                        Type.Bytes = (function() {
    
                            /**
                             * Properties of a Bytes.
                             * @memberof google.bigtable.admin.v2.Type
                             * @interface IBytes
                             * @property {google.bigtable.admin.v2.Type.Bytes.IEncoding|null} [encoding] Bytes encoding
                             */
    
                            /**
                             * Constructs a new Bytes.
                             * @memberof google.bigtable.admin.v2.Type
                             * @classdesc Represents a Bytes.
                             * @implements IBytes
                             * @constructor
                             * @param {google.bigtable.admin.v2.Type.IBytes=} [properties] Properties to set
                             */
                            function Bytes(properties) {
                                if (properties)
                                    for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                        if (properties[keys[i]] != null)
                                            this[keys[i]] = properties[keys[i]];
                            }
    
                            /**
                             * Bytes encoding.
                             * @member {google.bigtable.admin.v2.Type.Bytes.IEncoding|null|undefined} encoding
                             * @memberof google.bigtable.admin.v2.Type.Bytes
                             * @instance
                             */
                            Bytes.prototype.encoding = null;
    
                            /**
                             * Creates a new Bytes instance using the specified properties.
                             * @function create
                             * @memberof google.bigtable.admin.v2.Type.Bytes
                             * @static
                             * @param {google.bigtable.admin.v2.Type.IBytes=} [properties] Properties to set
                             * @returns {google.bigtable.admin.v2.Type.Bytes} Bytes instance
                             */
                            Bytes.create = function create(properties) {
                                return new Bytes(properties);
                            };
    
                            /**
                             * Encodes the specified Bytes message. Does not implicitly {@link google.bigtable.admin.v2.Type.Bytes.verify|verify} messages.
                             * @function encode
                             * @memberof google.bigtable.admin.v2.Type.Bytes
                             * @static
                             * @param {google.bigtable.admin.v2.Type.IBytes} message Bytes message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Bytes.encode = function encode(message, writer) {
                                if (!writer)
                                    writer = $Writer.create();
                                if (message.encoding != null && Object.hasOwnProperty.call(message, "encoding"))
                                    $root.google.bigtable.admin.v2.Type.Bytes.Encoding.encode(message.encoding, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                                return writer;
                            };
    
                            /**
                             * Encodes the specified Bytes message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.Type.Bytes.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.bigtable.admin.v2.Type.Bytes
                             * @static
                             * @param {google.bigtable.admin.v2.Type.IBytes} message Bytes message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Bytes.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a Bytes message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.bigtable.admin.v2.Type.Bytes
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @param {number} [length] Message length if known beforehand
                             * @returns {google.bigtable.admin.v2.Type.Bytes} Bytes
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Bytes.decode = function decode(reader, length) {
                                if (!(reader instanceof $Reader))
                                    reader = $Reader.create(reader);
                                var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.Type.Bytes();
                                while (reader.pos < end) {
                                    var tag = reader.uint32();
                                    switch (tag >>> 3) {
                                    case 1: {
                                            message.encoding = $root.google.bigtable.admin.v2.Type.Bytes.Encoding.decode(reader, reader.uint32());
                                            break;
                                        }
                                    default:
                                        reader.skipType(tag & 7);
                                        break;
                                    }
                                }
                                return message;
                            };
    
                            /**
                             * Decodes a Bytes message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.bigtable.admin.v2.Type.Bytes
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.bigtable.admin.v2.Type.Bytes} Bytes
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Bytes.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a Bytes message.
                             * @function verify
                             * @memberof google.bigtable.admin.v2.Type.Bytes
                             * @static
                             * @param {Object.<string,*>} message Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            Bytes.verify = function verify(message) {
                                if (typeof message !== "object" || message === null)
                                    return "object expected";
                                if (message.encoding != null && message.hasOwnProperty("encoding")) {
                                    var error = $root.google.bigtable.admin.v2.Type.Bytes.Encoding.verify(message.encoding);
                                    if (error)
                                        return "encoding." + error;
                                }
                                return null;
                            };
    
                            /**
                             * Creates a Bytes message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.bigtable.admin.v2.Type.Bytes
                             * @static
                             * @param {Object.<string,*>} object Plain object
                             * @returns {google.bigtable.admin.v2.Type.Bytes} Bytes
                             */
                            Bytes.fromObject = function fromObject(object) {
                                if (object instanceof $root.google.bigtable.admin.v2.Type.Bytes)
                                    return object;
                                var message = new $root.google.bigtable.admin.v2.Type.Bytes();
                                if (object.encoding != null) {
                                    if (typeof object.encoding !== "object")
                                        throw TypeError(".google.bigtable.admin.v2.Type.Bytes.encoding: object expected");
                                    message.encoding = $root.google.bigtable.admin.v2.Type.Bytes.Encoding.fromObject(object.encoding);
                                }
                                return message;
                            };
    
                            /**
                             * Creates a plain object from a Bytes message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.bigtable.admin.v2.Type.Bytes
                             * @static
                             * @param {google.bigtable.admin.v2.Type.Bytes} message Bytes
                             * @param {$protobuf.IConversionOptions} [options] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            Bytes.toObject = function toObject(message, options) {
                                if (!options)
                                    options = {};
                                var object = {};
                                if (options.defaults)
                                    object.encoding = null;
                                if (message.encoding != null && message.hasOwnProperty("encoding"))
                                    object.encoding = $root.google.bigtable.admin.v2.Type.Bytes.Encoding.toObject(message.encoding, options);
                                return object;
                            };
    
                            /**
                             * Converts this Bytes to JSON.
                             * @function toJSON
                             * @memberof google.bigtable.admin.v2.Type.Bytes
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            Bytes.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * Gets the default type url for Bytes
                             * @function getTypeUrl
                             * @memberof google.bigtable.admin.v2.Type.Bytes
                             * @static
                             * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                             * @returns {string} The default type url
                             */
                            Bytes.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                                if (typeUrlPrefix === undefined) {
                                    typeUrlPrefix = "type.googleapis.com";
                                }
                                return typeUrlPrefix + "/google.bigtable.admin.v2.Type.Bytes";
                            };
    
                            Bytes.Encoding = (function() {
    
                                /**
                                 * Properties of an Encoding.
                                 * @memberof google.bigtable.admin.v2.Type.Bytes
                                 * @interface IEncoding
                                 * @property {google.bigtable.admin.v2.Type.Bytes.Encoding.IRaw|null} [raw] Encoding raw
                                 */
    
                                /**
                                 * Constructs a new Encoding.
                                 * @memberof google.bigtable.admin.v2.Type.Bytes
                                 * @classdesc Represents an Encoding.
                                 * @implements IEncoding
                                 * @constructor
                                 * @param {google.bigtable.admin.v2.Type.Bytes.IEncoding=} [properties] Properties to set
                                 */
                                function Encoding(properties) {
                                    if (properties)
                                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                            if (properties[keys[i]] != null)
                                                this[keys[i]] = properties[keys[i]];
                                }
    
                                /**
                                 * Encoding raw.
                                 * @member {google.bigtable.admin.v2.Type.Bytes.Encoding.IRaw|null|undefined} raw
                                 * @memberof google.bigtable.admin.v2.Type.Bytes.Encoding
                                 * @instance
                                 */
                                Encoding.prototype.raw = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * Encoding encoding.
                                 * @member {"raw"|undefined} encoding
                                 * @memberof google.bigtable.admin.v2.Type.Bytes.Encoding
                                 * @instance
                                 */
                                Object.defineProperty(Encoding.prototype, "encoding", {
                                    get: $util.oneOfGetter($oneOfFields = ["raw"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new Encoding instance using the specified properties.
                                 * @function create
                                 * @memberof google.bigtable.admin.v2.Type.Bytes.Encoding
                                 * @static
                                 * @param {google.bigtable.admin.v2.Type.Bytes.IEncoding=} [properties] Properties to set
                                 * @returns {google.bigtable.admin.v2.Type.Bytes.Encoding} Encoding instance
                                 */
                                Encoding.create = function create(properties) {
                                    return new Encoding(properties);
                                };
    
                                /**
                                 * Encodes the specified Encoding message. Does not implicitly {@link google.bigtable.admin.v2.Type.Bytes.Encoding.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.bigtable.admin.v2.Type.Bytes.Encoding
                                 * @static
                                 * @param {google.bigtable.admin.v2.Type.Bytes.IEncoding} message Encoding message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                Encoding.encode = function encode(message, writer) {
                                    if (!writer)
                                        writer = $Writer.create();
                                    if (message.raw != null && Object.hasOwnProperty.call(message, "raw"))
                                        $root.google.bigtable.admin.v2.Type.Bytes.Encoding.Raw.encode(message.raw, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                                    return writer;
                                };
    
                                /**
                                 * Encodes the specified Encoding message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.Type.Bytes.Encoding.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.bigtable.admin.v2.Type.Bytes.Encoding
                                 * @static
                                 * @param {google.bigtable.admin.v2.Type.Bytes.IEncoding} message Encoding message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                Encoding.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes an Encoding message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.bigtable.admin.v2.Type.Bytes.Encoding
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @param {number} [length] Message length if known beforehand
                                 * @returns {google.bigtable.admin.v2.Type.Bytes.Encoding} Encoding
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                Encoding.decode = function decode(reader, length) {
                                    if (!(reader instanceof $Reader))
                                        reader = $Reader.create(reader);
                                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.Type.Bytes.Encoding();
                                    while (reader.pos < end) {
                                        var tag = reader.uint32();
                                        switch (tag >>> 3) {
                                        case 1: {
                                                message.raw = $root.google.bigtable.admin.v2.Type.Bytes.Encoding.Raw.decode(reader, reader.uint32());
                                                break;
                                            }
                                        default:
                                            reader.skipType(tag & 7);
                                            break;
                                        }
                                    }
                                    return message;
                                };
    
                                /**
                                 * Decodes an Encoding message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.bigtable.admin.v2.Type.Bytes.Encoding
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.bigtable.admin.v2.Type.Bytes.Encoding} Encoding
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                Encoding.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies an Encoding message.
                                 * @function verify
                                 * @memberof google.bigtable.admin.v2.Type.Bytes.Encoding
                                 * @static
                                 * @param {Object.<string,*>} message Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                Encoding.verify = function verify(message) {
                                    if (typeof message !== "object" || message === null)
                                        return "object expected";
                                    var properties = {};
                                    if (message.raw != null && message.hasOwnProperty("raw")) {
                                        properties.encoding = 1;
                                        {
                                            var error = $root.google.bigtable.admin.v2.Type.Bytes.Encoding.Raw.verify(message.raw);
                                            if (error)
                                                return "raw." + error;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates an Encoding message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.bigtable.admin.v2.Type.Bytes.Encoding
                                 * @static
                                 * @param {Object.<string,*>} object Plain object
                                 * @returns {google.bigtable.admin.v2.Type.Bytes.Encoding} Encoding
                                 */
                                Encoding.fromObject = function fromObject(object) {
                                    if (object instanceof $root.google.bigtable.admin.v2.Type.Bytes.Encoding)
                                        return object;
                                    var message = new $root.google.bigtable.admin.v2.Type.Bytes.Encoding();
                                    if (object.raw != null) {
                                        if (typeof object.raw !== "object")
                                            throw TypeError(".google.bigtable.admin.v2.Type.Bytes.Encoding.raw: object expected");
                                        message.raw = $root.google.bigtable.admin.v2.Type.Bytes.Encoding.Raw.fromObject(object.raw);
                                    }
                                    return message;
                                };
    
                                /**
                                 * Creates a plain object from an Encoding message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.bigtable.admin.v2.Type.Bytes.Encoding
                                 * @static
                                 * @param {google.bigtable.admin.v2.Type.Bytes.Encoding} message Encoding
                                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                Encoding.toObject = function toObject(message, options) {
                                    if (!options)
                                        options = {};
                                    var object = {};
                                    if (message.raw != null && message.hasOwnProperty("raw")) {
                                        object.raw = $root.google.bigtable.admin.v2.Type.Bytes.Encoding.Raw.toObject(message.raw, options);
                                        if (options.oneofs)
                                            object.encoding = "raw";
                                    }
                                    return object;
                                };
    
                                /**
                                 * Converts this Encoding to JSON.
                                 * @function toJSON
                                 * @memberof google.bigtable.admin.v2.Type.Bytes.Encoding
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                Encoding.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                /**
                                 * Gets the default type url for Encoding
                                 * @function getTypeUrl
                                 * @memberof google.bigtable.admin.v2.Type.Bytes.Encoding
                                 * @static
                                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                                 * @returns {string} The default type url
                                 */
                                Encoding.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                                    if (typeUrlPrefix === undefined) {
                                        typeUrlPrefix = "type.googleapis.com";
                                    }
                                    return typeUrlPrefix + "/google.bigtable.admin.v2.Type.Bytes.Encoding";
                                };
    
                                Encoding.Raw = (function() {
    
                                    /**
                                     * Properties of a Raw.
                                     * @memberof google.bigtable.admin.v2.Type.Bytes.Encoding
                                     * @interface IRaw
                                     */
    
                                    /**
                                     * Constructs a new Raw.
                                     * @memberof google.bigtable.admin.v2.Type.Bytes.Encoding
                                     * @classdesc Represents a Raw.
                                     * @implements IRaw
                                     * @constructor
                                     * @param {google.bigtable.admin.v2.Type.Bytes.Encoding.IRaw=} [properties] Properties to set
                                     */
                                    function Raw(properties) {
                                        if (properties)
                                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                                if (properties[keys[i]] != null)
                                                    this[keys[i]] = properties[keys[i]];
                                    }
    
                                    /**
                                     * Creates a new Raw instance using the specified properties.
                                     * @function create
                                     * @memberof google.bigtable.admin.v2.Type.Bytes.Encoding.Raw
                                     * @static
                                     * @param {google.bigtable.admin.v2.Type.Bytes.Encoding.IRaw=} [properties] Properties to set
                                     * @returns {google.bigtable.admin.v2.Type.Bytes.Encoding.Raw} Raw instance
                                     */
                                    Raw.create = function create(properties) {
                                        return new Raw(properties);
                                    };
    
                                    /**
                                     * Encodes the specified Raw message. Does not implicitly {@link google.bigtable.admin.v2.Type.Bytes.Encoding.Raw.verify|verify} messages.
                                     * @function encode
                                     * @memberof google.bigtable.admin.v2.Type.Bytes.Encoding.Raw
                                     * @static
                                     * @param {google.bigtable.admin.v2.Type.Bytes.Encoding.IRaw} message Raw message or plain object to encode
                                     * @param {$protobuf.Writer} [writer] Writer to encode to
                                     * @returns {$protobuf.Writer} Writer
                                     */
                                    Raw.encode = function encode(message, writer) {
                                        if (!writer)
                                            writer = $Writer.create();
                                        return writer;
                                    };
    
                                    /**
                                     * Encodes the specified Raw message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.Type.Bytes.Encoding.Raw.verify|verify} messages.
                                     * @function encodeDelimited
                                     * @memberof google.bigtable.admin.v2.Type.Bytes.Encoding.Raw
                                     * @static
                                     * @param {google.bigtable.admin.v2.Type.Bytes.Encoding.IRaw} message Raw message or plain object to encode
                                     * @param {$protobuf.Writer} [writer] Writer to encode to
                                     * @returns {$protobuf.Writer} Writer
                                     */
                                    Raw.encodeDelimited = function encodeDelimited(message, writer) {
                                        return this.encode(message, writer).ldelim();
                                    };
    
                                    /**
                                     * Decodes a Raw message from the specified reader or buffer.
                                     * @function decode
                                     * @memberof google.bigtable.admin.v2.Type.Bytes.Encoding.Raw
                                     * @static
                                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                     * @param {number} [length] Message length if known beforehand
                                     * @returns {google.bigtable.admin.v2.Type.Bytes.Encoding.Raw} Raw
                                     * @throws {Error} If the payload is not a reader or valid buffer
                                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                     */
                                    Raw.decode = function decode(reader, length) {
                                        if (!(reader instanceof $Reader))
                                            reader = $Reader.create(reader);
                                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.Type.Bytes.Encoding.Raw();
                                        while (reader.pos < end) {
                                            var tag = reader.uint32();
                                            switch (tag >>> 3) {
                                            default:
                                                reader.skipType(tag & 7);
                                                break;
                                            }
                                        }
                                        return message;
                                    };
    
                                    /**
                                     * Decodes a Raw message from the specified reader or buffer, length delimited.
                                     * @function decodeDelimited
                                     * @memberof google.bigtable.admin.v2.Type.Bytes.Encoding.Raw
                                     * @static
                                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                     * @returns {google.bigtable.admin.v2.Type.Bytes.Encoding.Raw} Raw
                                     * @throws {Error} If the payload is not a reader or valid buffer
                                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                     */
                                    Raw.decodeDelimited = function decodeDelimited(reader) {
                                        if (!(reader instanceof $Reader))
                                            reader = new $Reader(reader);
                                        return this.decode(reader, reader.uint32());
                                    };
    
                                    /**
                                     * Verifies a Raw message.
                                     * @function verify
                                     * @memberof google.bigtable.admin.v2.Type.Bytes.Encoding.Raw
                                     * @static
                                     * @param {Object.<string,*>} message Plain object to verify
                                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                     */
                                    Raw.verify = function verify(message) {
                                        if (typeof message !== "object" || message === null)
                                            return "object expected";
                                        return null;
                                    };
    
                                    /**
                                     * Creates a Raw message from a plain object. Also converts values to their respective internal types.
                                     * @function fromObject
                                     * @memberof google.bigtable.admin.v2.Type.Bytes.Encoding.Raw
                                     * @static
                                     * @param {Object.<string,*>} object Plain object
                                     * @returns {google.bigtable.admin.v2.Type.Bytes.Encoding.Raw} Raw
                                     */
                                    Raw.fromObject = function fromObject(object) {
                                        if (object instanceof $root.google.bigtable.admin.v2.Type.Bytes.Encoding.Raw)
                                            return object;
                                        return new $root.google.bigtable.admin.v2.Type.Bytes.Encoding.Raw();
                                    };
    
                                    /**
                                     * Creates a plain object from a Raw message. Also converts values to other types if specified.
                                     * @function toObject
                                     * @memberof google.bigtable.admin.v2.Type.Bytes.Encoding.Raw
                                     * @static
                                     * @param {google.bigtable.admin.v2.Type.Bytes.Encoding.Raw} message Raw
                                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                                     * @returns {Object.<string,*>} Plain object
                                     */
                                    Raw.toObject = function toObject() {
                                        return {};
                                    };
    
                                    /**
                                     * Converts this Raw to JSON.
                                     * @function toJSON
                                     * @memberof google.bigtable.admin.v2.Type.Bytes.Encoding.Raw
                                     * @instance
                                     * @returns {Object.<string,*>} JSON object
                                     */
                                    Raw.prototype.toJSON = function toJSON() {
                                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                    };
    
                                    /**
                                     * Gets the default type url for Raw
                                     * @function getTypeUrl
                                     * @memberof google.bigtable.admin.v2.Type.Bytes.Encoding.Raw
                                     * @static
                                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                                     * @returns {string} The default type url
                                     */
                                    Raw.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                                        if (typeUrlPrefix === undefined) {
                                            typeUrlPrefix = "type.googleapis.com";
                                        }
                                        return typeUrlPrefix + "/google.bigtable.admin.v2.Type.Bytes.Encoding.Raw";
                                    };
    
                                    return Raw;
                                })();
    
                                return Encoding;
                            })();
    
                            return Bytes;
                        })();
    
                        Type.Int64 = (function() {
    
                            /**
                             * Properties of an Int64.
                             * @memberof google.bigtable.admin.v2.Type
                             * @interface IInt64
                             * @property {google.bigtable.admin.v2.Type.Int64.IEncoding|null} [encoding] Int64 encoding
                             */
    
                            /**
                             * Constructs a new Int64.
                             * @memberof google.bigtable.admin.v2.Type
                             * @classdesc Represents an Int64.
                             * @implements IInt64
                             * @constructor
                             * @param {google.bigtable.admin.v2.Type.IInt64=} [properties] Properties to set
                             */
                            function Int64(properties) {
                                if (properties)
                                    for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                        if (properties[keys[i]] != null)
                                            this[keys[i]] = properties[keys[i]];
                            }
    
                            /**
                             * Int64 encoding.
                             * @member {google.bigtable.admin.v2.Type.Int64.IEncoding|null|undefined} encoding
                             * @memberof google.bigtable.admin.v2.Type.Int64
                             * @instance
                             */
                            Int64.prototype.encoding = null;
    
                            /**
                             * Creates a new Int64 instance using the specified properties.
                             * @function create
                             * @memberof google.bigtable.admin.v2.Type.Int64
                             * @static
                             * @param {google.bigtable.admin.v2.Type.IInt64=} [properties] Properties to set
                             * @returns {google.bigtable.admin.v2.Type.Int64} Int64 instance
                             */
                            Int64.create = function create(properties) {
                                return new Int64(properties);
                            };
    
                            /**
                             * Encodes the specified Int64 message. Does not implicitly {@link google.bigtable.admin.v2.Type.Int64.verify|verify} messages.
                             * @function encode
                             * @memberof google.bigtable.admin.v2.Type.Int64
                             * @static
                             * @param {google.bigtable.admin.v2.Type.IInt64} message Int64 message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Int64.encode = function encode(message, writer) {
                                if (!writer)
                                    writer = $Writer.create();
                                if (message.encoding != null && Object.hasOwnProperty.call(message, "encoding"))
                                    $root.google.bigtable.admin.v2.Type.Int64.Encoding.encode(message.encoding, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                                return writer;
                            };
    
                            /**
                             * Encodes the specified Int64 message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.Type.Int64.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.bigtable.admin.v2.Type.Int64
                             * @static
                             * @param {google.bigtable.admin.v2.Type.IInt64} message Int64 message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Int64.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an Int64 message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.bigtable.admin.v2.Type.Int64
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @param {number} [length] Message length if known beforehand
                             * @returns {google.bigtable.admin.v2.Type.Int64} Int64
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Int64.decode = function decode(reader, length) {
                                if (!(reader instanceof $Reader))
                                    reader = $Reader.create(reader);
                                var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.Type.Int64();
                                while (reader.pos < end) {
                                    var tag = reader.uint32();
                                    switch (tag >>> 3) {
                                    case 1: {
                                            message.encoding = $root.google.bigtable.admin.v2.Type.Int64.Encoding.decode(reader, reader.uint32());
                                            break;
                                        }
                                    default:
                                        reader.skipType(tag & 7);
                                        break;
                                    }
                                }
                                return message;
                            };
    
                            /**
                             * Decodes an Int64 message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.bigtable.admin.v2.Type.Int64
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.bigtable.admin.v2.Type.Int64} Int64
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Int64.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an Int64 message.
                             * @function verify
                             * @memberof google.bigtable.admin.v2.Type.Int64
                             * @static
                             * @param {Object.<string,*>} message Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            Int64.verify = function verify(message) {
                                if (typeof message !== "object" || message === null)
                                    return "object expected";
                                if (message.encoding != null && message.hasOwnProperty("encoding")) {
                                    var error = $root.google.bigtable.admin.v2.Type.Int64.Encoding.verify(message.encoding);
                                    if (error)
                                        return "encoding." + error;
                                }
                                return null;
                            };
    
                            /**
                             * Creates an Int64 message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.bigtable.admin.v2.Type.Int64
                             * @static
                             * @param {Object.<string,*>} object Plain object
                             * @returns {google.bigtable.admin.v2.Type.Int64} Int64
                             */
                            Int64.fromObject = function fromObject(object) {
                                if (object instanceof $root.google.bigtable.admin.v2.Type.Int64)
                                    return object;
                                var message = new $root.google.bigtable.admin.v2.Type.Int64();
                                if (object.encoding != null) {
                                    if (typeof object.encoding !== "object")
                                        throw TypeError(".google.bigtable.admin.v2.Type.Int64.encoding: object expected");
                                    message.encoding = $root.google.bigtable.admin.v2.Type.Int64.Encoding.fromObject(object.encoding);
                                }
                                return message;
                            };
    
                            /**
                             * Creates a plain object from an Int64 message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.bigtable.admin.v2.Type.Int64
                             * @static
                             * @param {google.bigtable.admin.v2.Type.Int64} message Int64
                             * @param {$protobuf.IConversionOptions} [options] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            Int64.toObject = function toObject(message, options) {
                                if (!options)
                                    options = {};
                                var object = {};
                                if (options.defaults)
                                    object.encoding = null;
                                if (message.encoding != null && message.hasOwnProperty("encoding"))
                                    object.encoding = $root.google.bigtable.admin.v2.Type.Int64.Encoding.toObject(message.encoding, options);
                                return object;
                            };
    
                            /**
                             * Converts this Int64 to JSON.
                             * @function toJSON
                             * @memberof google.bigtable.admin.v2.Type.Int64
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            Int64.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * Gets the default type url for Int64
                             * @function getTypeUrl
                             * @memberof google.bigtable.admin.v2.Type.Int64
                             * @static
                             * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                             * @returns {string} The default type url
                             */
                            Int64.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                                if (typeUrlPrefix === undefined) {
                                    typeUrlPrefix = "type.googleapis.com";
                                }
                                return typeUrlPrefix + "/google.bigtable.admin.v2.Type.Int64";
                            };
    
                            Int64.Encoding = (function() {
    
                                /**
                                 * Properties of an Encoding.
                                 * @memberof google.bigtable.admin.v2.Type.Int64
                                 * @interface IEncoding
                                 * @property {google.bigtable.admin.v2.Type.Int64.Encoding.IBigEndianBytes|null} [bigEndianBytes] Encoding bigEndianBytes
                                 */
    
                                /**
                                 * Constructs a new Encoding.
                                 * @memberof google.bigtable.admin.v2.Type.Int64
                                 * @classdesc Represents an Encoding.
                                 * @implements IEncoding
                                 * @constructor
                                 * @param {google.bigtable.admin.v2.Type.Int64.IEncoding=} [properties] Properties to set
                                 */
                                function Encoding(properties) {
                                    if (properties)
                                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                            if (properties[keys[i]] != null)
                                                this[keys[i]] = properties[keys[i]];
                                }
    
                                /**
                                 * Encoding bigEndianBytes.
                                 * @member {google.bigtable.admin.v2.Type.Int64.Encoding.IBigEndianBytes|null|undefined} bigEndianBytes
                                 * @memberof google.bigtable.admin.v2.Type.Int64.Encoding
                                 * @instance
                                 */
                                Encoding.prototype.bigEndianBytes = null;
    
                                // OneOf field names bound to virtual getters and setters
                                var $oneOfFields;
    
                                /**
                                 * Encoding encoding.
                                 * @member {"bigEndianBytes"|undefined} encoding
                                 * @memberof google.bigtable.admin.v2.Type.Int64.Encoding
                                 * @instance
                                 */
                                Object.defineProperty(Encoding.prototype, "encoding", {
                                    get: $util.oneOfGetter($oneOfFields = ["bigEndianBytes"]),
                                    set: $util.oneOfSetter($oneOfFields)
                                });
    
                                /**
                                 * Creates a new Encoding instance using the specified properties.
                                 * @function create
                                 * @memberof google.bigtable.admin.v2.Type.Int64.Encoding
                                 * @static
                                 * @param {google.bigtable.admin.v2.Type.Int64.IEncoding=} [properties] Properties to set
                                 * @returns {google.bigtable.admin.v2.Type.Int64.Encoding} Encoding instance
                                 */
                                Encoding.create = function create(properties) {
                                    return new Encoding(properties);
                                };
    
                                /**
                                 * Encodes the specified Encoding message. Does not implicitly {@link google.bigtable.admin.v2.Type.Int64.Encoding.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.bigtable.admin.v2.Type.Int64.Encoding
                                 * @static
                                 * @param {google.bigtable.admin.v2.Type.Int64.IEncoding} message Encoding message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                Encoding.encode = function encode(message, writer) {
                                    if (!writer)
                                        writer = $Writer.create();
                                    if (message.bigEndianBytes != null && Object.hasOwnProperty.call(message, "bigEndianBytes"))
                                        $root.google.bigtable.admin.v2.Type.Int64.Encoding.BigEndianBytes.encode(message.bigEndianBytes, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                                    return writer;
                                };
    
                                /**
                                 * Encodes the specified Encoding message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.Type.Int64.Encoding.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.bigtable.admin.v2.Type.Int64.Encoding
                                 * @static
                                 * @param {google.bigtable.admin.v2.Type.Int64.IEncoding} message Encoding message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                Encoding.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes an Encoding message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.bigtable.admin.v2.Type.Int64.Encoding
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @param {number} [length] Message length if known beforehand
                                 * @returns {google.bigtable.admin.v2.Type.Int64.Encoding} Encoding
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                Encoding.decode = function decode(reader, length) {
                                    if (!(reader instanceof $Reader))
                                        reader = $Reader.create(reader);
                                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.Type.Int64.Encoding();
                                    while (reader.pos < end) {
                                        var tag = reader.uint32();
                                        switch (tag >>> 3) {
                                        case 1: {
                                                message.bigEndianBytes = $root.google.bigtable.admin.v2.Type.Int64.Encoding.BigEndianBytes.decode(reader, reader.uint32());
                                                break;
                                            }
                                        default:
                                            reader.skipType(tag & 7);
                                            break;
                                        }
                                    }
                                    return message;
                                };
    
                                /**
                                 * Decodes an Encoding message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.bigtable.admin.v2.Type.Int64.Encoding
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.bigtable.admin.v2.Type.Int64.Encoding} Encoding
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                Encoding.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies an Encoding message.
                                 * @function verify
                                 * @memberof google.bigtable.admin.v2.Type.Int64.Encoding
                                 * @static
                                 * @param {Object.<string,*>} message Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                Encoding.verify = function verify(message) {
                                    if (typeof message !== "object" || message === null)
                                        return "object expected";
                                    var properties = {};
                                    if (message.bigEndianBytes != null && message.hasOwnProperty("bigEndianBytes")) {
                                        properties.encoding = 1;
                                        {
                                            var error = $root.google.bigtable.admin.v2.Type.Int64.Encoding.BigEndianBytes.verify(message.bigEndianBytes);
                                            if (error)
                                                return "bigEndianBytes." + error;
                                        }
                                    }
                                    return null;
                                };
    
                                /**
                                 * Creates an Encoding message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.bigtable.admin.v2.Type.Int64.Encoding
                                 * @static
                                 * @param {Object.<string,*>} object Plain object
                                 * @returns {google.bigtable.admin.v2.Type.Int64.Encoding} Encoding
                                 */
                                Encoding.fromObject = function fromObject(object) {
                                    if (object instanceof $root.google.bigtable.admin.v2.Type.Int64.Encoding)
                                        return object;
                                    var message = new $root.google.bigtable.admin.v2.Type.Int64.Encoding();
                                    if (object.bigEndianBytes != null) {
                                        if (typeof object.bigEndianBytes !== "object")
                                            throw TypeError(".google.bigtable.admin.v2.Type.Int64.Encoding.bigEndianBytes: object expected");
                                        message.bigEndianBytes = $root.google.bigtable.admin.v2.Type.Int64.Encoding.BigEndianBytes.fromObject(object.bigEndianBytes);
                                    }
                                    return message;
                                };
    
                                /**
                                 * Creates a plain object from an Encoding message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.bigtable.admin.v2.Type.Int64.Encoding
                                 * @static
                                 * @param {google.bigtable.admin.v2.Type.Int64.Encoding} message Encoding
                                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                Encoding.toObject = function toObject(message, options) {
                                    if (!options)
                                        options = {};
                                    var object = {};
                                    if (message.bigEndianBytes != null && message.hasOwnProperty("bigEndianBytes")) {
                                        object.bigEndianBytes = $root.google.bigtable.admin.v2.Type.Int64.Encoding.BigEndianBytes.toObject(message.bigEndianBytes, options);
                                        if (options.oneofs)
                                            object.encoding = "bigEndianBytes";
                                    }
                                    return object;
                                };
    
                                /**
                                 * Converts this Encoding to JSON.
                                 * @function toJSON
                                 * @memberof google.bigtable.admin.v2.Type.Int64.Encoding
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                Encoding.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                /**
                                 * Gets the default type url for Encoding
                                 * @function getTypeUrl
                                 * @memberof google.bigtable.admin.v2.Type.Int64.Encoding
                                 * @static
                                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                                 * @returns {string} The default type url
                                 */
                                Encoding.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                                    if (typeUrlPrefix === undefined) {
                                        typeUrlPrefix = "type.googleapis.com";
                                    }
                                    return typeUrlPrefix + "/google.bigtable.admin.v2.Type.Int64.Encoding";
                                };
    
                                Encoding.BigEndianBytes = (function() {
    
                                    /**
                                     * Properties of a BigEndianBytes.
                                     * @memberof google.bigtable.admin.v2.Type.Int64.Encoding
                                     * @interface IBigEndianBytes
                                     * @property {google.bigtable.admin.v2.Type.IBytes|null} [bytesType] BigEndianBytes bytesType
                                     */
    
                                    /**
                                     * Constructs a new BigEndianBytes.
                                     * @memberof google.bigtable.admin.v2.Type.Int64.Encoding
                                     * @classdesc Represents a BigEndianBytes.
                                     * @implements IBigEndianBytes
                                     * @constructor
                                     * @param {google.bigtable.admin.v2.Type.Int64.Encoding.IBigEndianBytes=} [properties] Properties to set
                                     */
                                    function BigEndianBytes(properties) {
                                        if (properties)
                                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                                if (properties[keys[i]] != null)
                                                    this[keys[i]] = properties[keys[i]];
                                    }
    
                                    /**
                                     * BigEndianBytes bytesType.
                                     * @member {google.bigtable.admin.v2.Type.IBytes|null|undefined} bytesType
                                     * @memberof google.bigtable.admin.v2.Type.Int64.Encoding.BigEndianBytes
                                     * @instance
                                     */
                                    BigEndianBytes.prototype.bytesType = null;
    
                                    /**
                                     * Creates a new BigEndianBytes instance using the specified properties.
                                     * @function create
                                     * @memberof google.bigtable.admin.v2.Type.Int64.Encoding.BigEndianBytes
                                     * @static
                                     * @param {google.bigtable.admin.v2.Type.Int64.Encoding.IBigEndianBytes=} [properties] Properties to set
                                     * @returns {google.bigtable.admin.v2.Type.Int64.Encoding.BigEndianBytes} BigEndianBytes instance
                                     */
                                    BigEndianBytes.create = function create(properties) {
                                        return new BigEndianBytes(properties);
                                    };
    
                                    /**
                                     * Encodes the specified BigEndianBytes message. Does not implicitly {@link google.bigtable.admin.v2.Type.Int64.Encoding.BigEndianBytes.verify|verify} messages.
                                     * @function encode
                                     * @memberof google.bigtable.admin.v2.Type.Int64.Encoding.BigEndianBytes
                                     * @static
                                     * @param {google.bigtable.admin.v2.Type.Int64.Encoding.IBigEndianBytes} message BigEndianBytes message or plain object to encode
                                     * @param {$protobuf.Writer} [writer] Writer to encode to
                                     * @returns {$protobuf.Writer} Writer
                                     */
                                    BigEndianBytes.encode = function encode(message, writer) {
                                        if (!writer)
                                            writer = $Writer.create();
                                        if (message.bytesType != null && Object.hasOwnProperty.call(message, "bytesType"))
                                            $root.google.bigtable.admin.v2.Type.Bytes.encode(message.bytesType, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                                        return writer;
                                    };
    
                                    /**
                                     * Encodes the specified BigEndianBytes message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.Type.Int64.Encoding.BigEndianBytes.verify|verify} messages.
                                     * @function encodeDelimited
                                     * @memberof google.bigtable.admin.v2.Type.Int64.Encoding.BigEndianBytes
                                     * @static
                                     * @param {google.bigtable.admin.v2.Type.Int64.Encoding.IBigEndianBytes} message BigEndianBytes message or plain object to encode
                                     * @param {$protobuf.Writer} [writer] Writer to encode to
                                     * @returns {$protobuf.Writer} Writer
                                     */
                                    BigEndianBytes.encodeDelimited = function encodeDelimited(message, writer) {
                                        return this.encode(message, writer).ldelim();
                                    };
    
                                    /**
                                     * Decodes a BigEndianBytes message from the specified reader or buffer.
                                     * @function decode
                                     * @memberof google.bigtable.admin.v2.Type.Int64.Encoding.BigEndianBytes
                                     * @static
                                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                     * @param {number} [length] Message length if known beforehand
                                     * @returns {google.bigtable.admin.v2.Type.Int64.Encoding.BigEndianBytes} BigEndianBytes
                                     * @throws {Error} If the payload is not a reader or valid buffer
                                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                     */
                                    BigEndianBytes.decode = function decode(reader, length) {
                                        if (!(reader instanceof $Reader))
                                            reader = $Reader.create(reader);
                                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.Type.Int64.Encoding.BigEndianBytes();
                                        while (reader.pos < end) {
                                            var tag = reader.uint32();
                                            switch (tag >>> 3) {
                                            case 1: {
                                                    message.bytesType = $root.google.bigtable.admin.v2.Type.Bytes.decode(reader, reader.uint32());
                                                    break;
                                                }
                                            default:
                                                reader.skipType(tag & 7);
                                                break;
                                            }
                                        }
                                        return message;
                                    };
    
                                    /**
                                     * Decodes a BigEndianBytes message from the specified reader or buffer, length delimited.
                                     * @function decodeDelimited
                                     * @memberof google.bigtable.admin.v2.Type.Int64.Encoding.BigEndianBytes
                                     * @static
                                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                     * @returns {google.bigtable.admin.v2.Type.Int64.Encoding.BigEndianBytes} BigEndianBytes
                                     * @throws {Error} If the payload is not a reader or valid buffer
                                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                     */
                                    BigEndianBytes.decodeDelimited = function decodeDelimited(reader) {
                                        if (!(reader instanceof $Reader))
                                            reader = new $Reader(reader);
                                        return this.decode(reader, reader.uint32());
                                    };
    
                                    /**
                                     * Verifies a BigEndianBytes message.
                                     * @function verify
                                     * @memberof google.bigtable.admin.v2.Type.Int64.Encoding.BigEndianBytes
                                     * @static
                                     * @param {Object.<string,*>} message Plain object to verify
                                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                     */
                                    BigEndianBytes.verify = function verify(message) {
                                        if (typeof message !== "object" || message === null)
                                            return "object expected";
                                        if (message.bytesType != null && message.hasOwnProperty("bytesType")) {
                                            var error = $root.google.bigtable.admin.v2.Type.Bytes.verify(message.bytesType);
                                            if (error)
                                                return "bytesType." + error;
                                        }
                                        return null;
                                    };
    
                                    /**
                                     * Creates a BigEndianBytes message from a plain object. Also converts values to their respective internal types.
                                     * @function fromObject
                                     * @memberof google.bigtable.admin.v2.Type.Int64.Encoding.BigEndianBytes
                                     * @static
                                     * @param {Object.<string,*>} object Plain object
                                     * @returns {google.bigtable.admin.v2.Type.Int64.Encoding.BigEndianBytes} BigEndianBytes
                                     */
                                    BigEndianBytes.fromObject = function fromObject(object) {
                                        if (object instanceof $root.google.bigtable.admin.v2.Type.Int64.Encoding.BigEndianBytes)
                                            return object;
                                        var message = new $root.google.bigtable.admin.v2.Type.Int64.Encoding.BigEndianBytes();
                                        if (object.bytesType != null) {
                                            if (typeof object.bytesType !== "object")
                                                throw TypeError(".google.bigtable.admin.v2.Type.Int64.Encoding.BigEndianBytes.bytesType: object expected");
                                            message.bytesType = $root.google.bigtable.admin.v2.Type.Bytes.fromObject(object.bytesType);
                                        }
                                        return message;
                                    };
    
                                    /**
                                     * Creates a plain object from a BigEndianBytes message. Also converts values to other types if specified.
                                     * @function toObject
                                     * @memberof google.bigtable.admin.v2.Type.Int64.Encoding.BigEndianBytes
                                     * @static
                                     * @param {google.bigtable.admin.v2.Type.Int64.Encoding.BigEndianBytes} message BigEndianBytes
                                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                                     * @returns {Object.<string,*>} Plain object
                                     */
                                    BigEndianBytes.toObject = function toObject(message, options) {
                                        if (!options)
                                            options = {};
                                        var object = {};
                                        if (options.defaults)
                                            object.bytesType = null;
                                        if (message.bytesType != null && message.hasOwnProperty("bytesType"))
                                            object.bytesType = $root.google.bigtable.admin.v2.Type.Bytes.toObject(message.bytesType, options);
                                        return object;
                                    };
    
                                    /**
                                     * Converts this BigEndianBytes to JSON.
                                     * @function toJSON
                                     * @memberof google.bigtable.admin.v2.Type.Int64.Encoding.BigEndianBytes
                                     * @instance
                                     * @returns {Object.<string,*>} JSON object
                                     */
                                    BigEndianBytes.prototype.toJSON = function toJSON() {
                                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                    };
    
                                    /**
                                     * Gets the default type url for BigEndianBytes
                                     * @function getTypeUrl
                                     * @memberof google.bigtable.admin.v2.Type.Int64.Encoding.BigEndianBytes
                                     * @static
                                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                                     * @returns {string} The default type url
                                     */
                                    BigEndianBytes.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                                        if (typeUrlPrefix === undefined) {
                                            typeUrlPrefix = "type.googleapis.com";
                                        }
                                        return typeUrlPrefix + "/google.bigtable.admin.v2.Type.Int64.Encoding.BigEndianBytes";
                                    };
    
                                    return BigEndianBytes;
                                })();
    
                                return Encoding;
                            })();
    
                            return Int64;
                        })();
    
                        Type.Aggregate = (function() {
    
                            /**
                             * Properties of an Aggregate.
                             * @memberof google.bigtable.admin.v2.Type
                             * @interface IAggregate
                             * @property {google.bigtable.admin.v2.IType|null} [inputType] Aggregate inputType
                             * @property {google.bigtable.admin.v2.IType|null} [stateType] Aggregate stateType
                             * @property {google.bigtable.admin.v2.Type.Aggregate.ISum|null} [sum] Aggregate sum
                             */
    
                            /**
                             * Constructs a new Aggregate.
                             * @memberof google.bigtable.admin.v2.Type
                             * @classdesc Represents an Aggregate.
                             * @implements IAggregate
                             * @constructor
                             * @param {google.bigtable.admin.v2.Type.IAggregate=} [properties] Properties to set
                             */
                            function Aggregate(properties) {
                                if (properties)
                                    for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                        if (properties[keys[i]] != null)
                                            this[keys[i]] = properties[keys[i]];
                            }
    
                            /**
                             * Aggregate inputType.
                             * @member {google.bigtable.admin.v2.IType|null|undefined} inputType
                             * @memberof google.bigtable.admin.v2.Type.Aggregate
                             * @instance
                             */
                            Aggregate.prototype.inputType = null;
    
                            /**
                             * Aggregate stateType.
                             * @member {google.bigtable.admin.v2.IType|null|undefined} stateType
                             * @memberof google.bigtable.admin.v2.Type.Aggregate
                             * @instance
                             */
                            Aggregate.prototype.stateType = null;
    
                            /**
                             * Aggregate sum.
                             * @member {google.bigtable.admin.v2.Type.Aggregate.ISum|null|undefined} sum
                             * @memberof google.bigtable.admin.v2.Type.Aggregate
                             * @instance
                             */
                            Aggregate.prototype.sum = null;
    
                            // OneOf field names bound to virtual getters and setters
                            var $oneOfFields;
    
                            /**
                             * Aggregate aggregator.
                             * @member {"sum"|undefined} aggregator
                             * @memberof google.bigtable.admin.v2.Type.Aggregate
                             * @instance
                             */
                            Object.defineProperty(Aggregate.prototype, "aggregator", {
                                get: $util.oneOfGetter($oneOfFields = ["sum"]),
                                set: $util.oneOfSetter($oneOfFields)
                            });
    
                            /**
                             * Creates a new Aggregate instance using the specified properties.
                             * @function create
                             * @memberof google.bigtable.admin.v2.Type.Aggregate
                             * @static
                             * @param {google.bigtable.admin.v2.Type.IAggregate=} [properties] Properties to set
                             * @returns {google.bigtable.admin.v2.Type.Aggregate} Aggregate instance
                             */
                            Aggregate.create = function create(properties) {
                                return new Aggregate(properties);
                            };
    
                            /**
                             * Encodes the specified Aggregate message. Does not implicitly {@link google.bigtable.admin.v2.Type.Aggregate.verify|verify} messages.
                             * @function encode
                             * @memberof google.bigtable.admin.v2.Type.Aggregate
                             * @static
                             * @param {google.bigtable.admin.v2.Type.IAggregate} message Aggregate message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Aggregate.encode = function encode(message, writer) {
                                if (!writer)
                                    writer = $Writer.create();
                                if (message.inputType != null && Object.hasOwnProperty.call(message, "inputType"))
                                    $root.google.bigtable.admin.v2.Type.encode(message.inputType, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                                if (message.stateType != null && Object.hasOwnProperty.call(message, "stateType"))
                                    $root.google.bigtable.admin.v2.Type.encode(message.stateType, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                                if (message.sum != null && Object.hasOwnProperty.call(message, "sum"))
                                    $root.google.bigtable.admin.v2.Type.Aggregate.Sum.encode(message.sum, writer.uint32(/* id 4, wireType 2 =*/34).fork()).ldelim();
                                return writer;
                            };
    
                            /**
                             * Encodes the specified Aggregate message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.Type.Aggregate.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.bigtable.admin.v2.Type.Aggregate
                             * @static
                             * @param {google.bigtable.admin.v2.Type.IAggregate} message Aggregate message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            Aggregate.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes an Aggregate message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.bigtable.admin.v2.Type.Aggregate
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @param {number} [length] Message length if known beforehand
                             * @returns {google.bigtable.admin.v2.Type.Aggregate} Aggregate
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Aggregate.decode = function decode(reader, length) {
                                if (!(reader instanceof $Reader))
                                    reader = $Reader.create(reader);
                                var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.Type.Aggregate();
                                while (reader.pos < end) {
                                    var tag = reader.uint32();
                                    switch (tag >>> 3) {
                                    case 1: {
                                            message.inputType = $root.google.bigtable.admin.v2.Type.decode(reader, reader.uint32());
                                            break;
                                        }
                                    case 2: {
                                            message.stateType = $root.google.bigtable.admin.v2.Type.decode(reader, reader.uint32());
                                            break;
                                        }
                                    case 4: {
                                            message.sum = $root.google.bigtable.admin.v2.Type.Aggregate.Sum.decode(reader, reader.uint32());
                                            break;
                                        }
                                    default:
                                        reader.skipType(tag & 7);
                                        break;
                                    }
                                }
                                return message;
                            };
    
                            /**
                             * Decodes an Aggregate message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.bigtable.admin.v2.Type.Aggregate
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.bigtable.admin.v2.Type.Aggregate} Aggregate
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            Aggregate.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies an Aggregate message.
                             * @function verify
                             * @memberof google.bigtable.admin.v2.Type.Aggregate
                             * @static
                             * @param {Object.<string,*>} message Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            Aggregate.verify = function verify(message) {
                                if (typeof message !== "object" || message === null)
                                    return "object expected";
                                var properties = {};
                                if (message.inputType != null && message.hasOwnProperty("inputType")) {
                                    var error = $root.google.bigtable.admin.v2.Type.verify(message.inputType);
                                    if (error)
                                        return "inputType." + error;
                                }
                                if (message.stateType != null && message.hasOwnProperty("stateType")) {
                                    var error = $root.google.bigtable.admin.v2.Type.verify(message.stateType);
                                    if (error)
                                        return "stateType." + error;
                                }
                                if (message.sum != null && message.hasOwnProperty("sum")) {
                                    properties.aggregator = 1;
                                    {
                                        var error = $root.google.bigtable.admin.v2.Type.Aggregate.Sum.verify(message.sum);
                                        if (error)
                                            return "sum." + error;
                                    }
                                }
                                return null;
                            };
    
                            /**
                             * Creates an Aggregate message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.bigtable.admin.v2.Type.Aggregate
                             * @static
                             * @param {Object.<string,*>} object Plain object
                             * @returns {google.bigtable.admin.v2.Type.Aggregate} Aggregate
                             */
                            Aggregate.fromObject = function fromObject(object) {
                                if (object instanceof $root.google.bigtable.admin.v2.Type.Aggregate)
                                    return object;
                                var message = new $root.google.bigtable.admin.v2.Type.Aggregate();
                                if (object.inputType != null) {
                                    if (typeof object.inputType !== "object")
                                        throw TypeError(".google.bigtable.admin.v2.Type.Aggregate.inputType: object expected");
                                    message.inputType = $root.google.bigtable.admin.v2.Type.fromObject(object.inputType);
                                }
                                if (object.stateType != null) {
                                    if (typeof object.stateType !== "object")
                                        throw TypeError(".google.bigtable.admin.v2.Type.Aggregate.stateType: object expected");
                                    message.stateType = $root.google.bigtable.admin.v2.Type.fromObject(object.stateType);
                                }
                                if (object.sum != null) {
                                    if (typeof object.sum !== "object")
                                        throw TypeError(".google.bigtable.admin.v2.Type.Aggregate.sum: object expected");
                                    message.sum = $root.google.bigtable.admin.v2.Type.Aggregate.Sum.fromObject(object.sum);
                                }
                                return message;
                            };
    
                            /**
                             * Creates a plain object from an Aggregate message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.bigtable.admin.v2.Type.Aggregate
                             * @static
                             * @param {google.bigtable.admin.v2.Type.Aggregate} message Aggregate
                             * @param {$protobuf.IConversionOptions} [options] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            Aggregate.toObject = function toObject(message, options) {
                                if (!options)
                                    options = {};
                                var object = {};
                                if (options.defaults) {
                                    object.inputType = null;
                                    object.stateType = null;
                                }
                                if (message.inputType != null && message.hasOwnProperty("inputType"))
                                    object.inputType = $root.google.bigtable.admin.v2.Type.toObject(message.inputType, options);
                                if (message.stateType != null && message.hasOwnProperty("stateType"))
                                    object.stateType = $root.google.bigtable.admin.v2.Type.toObject(message.stateType, options);
                                if (message.sum != null && message.hasOwnProperty("sum")) {
                                    object.sum = $root.google.bigtable.admin.v2.Type.Aggregate.Sum.toObject(message.sum, options);
                                    if (options.oneofs)
                                        object.aggregator = "sum";
                                }
                                return object;
                            };
    
                            /**
                             * Converts this Aggregate to JSON.
                             * @function toJSON
                             * @memberof google.bigtable.admin.v2.Type.Aggregate
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            Aggregate.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * Gets the default type url for Aggregate
                             * @function getTypeUrl
                             * @memberof google.bigtable.admin.v2.Type.Aggregate
                             * @static
                             * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                             * @returns {string} The default type url
                             */
                            Aggregate.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                                if (typeUrlPrefix === undefined) {
                                    typeUrlPrefix = "type.googleapis.com";
                                }
                                return typeUrlPrefix + "/google.bigtable.admin.v2.Type.Aggregate";
                            };
    
                            Aggregate.Sum = (function() {
    
                                /**
                                 * Properties of a Sum.
                                 * @memberof google.bigtable.admin.v2.Type.Aggregate
                                 * @interface ISum
                                 */
    
                                /**
                                 * Constructs a new Sum.
                                 * @memberof google.bigtable.admin.v2.Type.Aggregate
                                 * @classdesc Represents a Sum.
                                 * @implements ISum
                                 * @constructor
                                 * @param {google.bigtable.admin.v2.Type.Aggregate.ISum=} [properties] Properties to set
                                 */
                                function Sum(properties) {
                                    if (properties)
                                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                            if (properties[keys[i]] != null)
                                                this[keys[i]] = properties[keys[i]];
                                }
    
                                /**
                                 * Creates a new Sum instance using the specified properties.
                                 * @function create
                                 * @memberof google.bigtable.admin.v2.Type.Aggregate.Sum
                                 * @static
                                 * @param {google.bigtable.admin.v2.Type.Aggregate.ISum=} [properties] Properties to set
                                 * @returns {google.bigtable.admin.v2.Type.Aggregate.Sum} Sum instance
                                 */
                                Sum.create = function create(properties) {
                                    return new Sum(properties);
                                };
    
                                /**
                                 * Encodes the specified Sum message. Does not implicitly {@link google.bigtable.admin.v2.Type.Aggregate.Sum.verify|verify} messages.
                                 * @function encode
                                 * @memberof google.bigtable.admin.v2.Type.Aggregate.Sum
                                 * @static
                                 * @param {google.bigtable.admin.v2.Type.Aggregate.ISum} message Sum message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                Sum.encode = function encode(message, writer) {
                                    if (!writer)
                                        writer = $Writer.create();
                                    return writer;
                                };
    
                                /**
                                 * Encodes the specified Sum message, length delimited. Does not implicitly {@link google.bigtable.admin.v2.Type.Aggregate.Sum.verify|verify} messages.
                                 * @function encodeDelimited
                                 * @memberof google.bigtable.admin.v2.Type.Aggregate.Sum
                                 * @static
                                 * @param {google.bigtable.admin.v2.Type.Aggregate.ISum} message Sum message or plain object to encode
                                 * @param {$protobuf.Writer} [writer] Writer to encode to
                                 * @returns {$protobuf.Writer} Writer
                                 */
                                Sum.encodeDelimited = function encodeDelimited(message, writer) {
                                    return this.encode(message, writer).ldelim();
                                };
    
                                /**
                                 * Decodes a Sum message from the specified reader or buffer.
                                 * @function decode
                                 * @memberof google.bigtable.admin.v2.Type.Aggregate.Sum
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @param {number} [length] Message length if known beforehand
                                 * @returns {google.bigtable.admin.v2.Type.Aggregate.Sum} Sum
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                Sum.decode = function decode(reader, length) {
                                    if (!(reader instanceof $Reader))
                                        reader = $Reader.create(reader);
                                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.admin.v2.Type.Aggregate.Sum();
                                    while (reader.pos < end) {
                                        var tag = reader.uint32();
                                        switch (tag >>> 3) {
                                        default:
                                            reader.skipType(tag & 7);
                                            break;
                                        }
                                    }
                                    return message;
                                };
    
                                /**
                                 * Decodes a Sum message from the specified reader or buffer, length delimited.
                                 * @function decodeDelimited
                                 * @memberof google.bigtable.admin.v2.Type.Aggregate.Sum
                                 * @static
                                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                                 * @returns {google.bigtable.admin.v2.Type.Aggregate.Sum} Sum
                                 * @throws {Error} If the payload is not a reader or valid buffer
                                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                                 */
                                Sum.decodeDelimited = function decodeDelimited(reader) {
                                    if (!(reader instanceof $Reader))
                                        reader = new $Reader(reader);
                                    return this.decode(reader, reader.uint32());
                                };
    
                                /**
                                 * Verifies a Sum message.
                                 * @function verify
                                 * @memberof google.bigtable.admin.v2.Type.Aggregate.Sum
                                 * @static
                                 * @param {Object.<string,*>} message Plain object to verify
                                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                                 */
                                Sum.verify = function verify(message) {
                                    if (typeof message !== "object" || message === null)
                                        return "object expected";
                                    return null;
                                };
    
                                /**
                                 * Creates a Sum message from a plain object. Also converts values to their respective internal types.
                                 * @function fromObject
                                 * @memberof google.bigtable.admin.v2.Type.Aggregate.Sum
                                 * @static
                                 * @param {Object.<string,*>} object Plain object
                                 * @returns {google.bigtable.admin.v2.Type.Aggregate.Sum} Sum
                                 */
                                Sum.fromObject = function fromObject(object) {
                                    if (object instanceof $root.google.bigtable.admin.v2.Type.Aggregate.Sum)
                                        return object;
                                    return new $root.google.bigtable.admin.v2.Type.Aggregate.Sum();
                                };
    
                                /**
                                 * Creates a plain object from a Sum message. Also converts values to other types if specified.
                                 * @function toObject
                                 * @memberof google.bigtable.admin.v2.Type.Aggregate.Sum
                                 * @static
                                 * @param {google.bigtable.admin.v2.Type.Aggregate.Sum} message Sum
                                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                                 * @returns {Object.<string,*>} Plain object
                                 */
                                Sum.toObject = function toObject() {
                                    return {};
                                };
    
                                /**
                                 * Converts this Sum to JSON.
                                 * @function toJSON
                                 * @memberof google.bigtable.admin.v2.Type.Aggregate.Sum
                                 * @instance
                                 * @returns {Object.<string,*>} JSON object
                                 */
                                Sum.prototype.toJSON = function toJSON() {
                                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                                };
    
                                /**
                                 * Gets the default type url for Sum
                                 * @function getTypeUrl
                                 * @memberof google.bigtable.admin.v2.Type.Aggregate.Sum
                                 * @static
                                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                                 * @returns {string} The default type url
                                 */
                                Sum.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                                    if (typeUrlPrefix === undefined) {
                                        typeUrlPrefix = "type.googleapis.com";
                                    }
                                    return typeUrlPrefix + "/google.bigtable.admin.v2.Type.Aggregate.Sum";
                                };
    
                                return Sum;
                            })();
    
                            return Aggregate;
                        })();
    
                        return Type;
                    })();
    
                    return v2;
                })();
    
                return admin;
            })();
    
            bigtable.v2 = (function() {
    
                /**
                 * Namespace v2.
                 * @memberof google.bigtable
                 * @namespace
                 */
                var v2 = {};
    
                v2.Bigtable = (function() {
    
                    /**
                     * Constructs a new Bigtable service.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a Bigtable
                     * @extends $protobuf.rpc.Service
                     * @constructor
                     * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                     * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                     * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                     */
                    function Bigtable(rpcImpl, requestDelimited, responseDelimited) {
                        $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                    }
    
                    (Bigtable.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = Bigtable;
    
                    /**
                     * Creates new Bigtable service using the specified rpc implementation.
                     * @function create
                     * @memberof google.bigtable.v2.Bigtable
                     * @static
                     * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                     * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                     * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                     * @returns {Bigtable} RPC service. Useful where requests and/or responses are streamed.
                     */
                    Bigtable.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                        return new this(rpcImpl, requestDelimited, responseDelimited);
                    };
    
                    /**
                     * Callback as used by {@link google.bigtable.v2.Bigtable|readRows}.
                     * @memberof google.bigtable.v2.Bigtable
                     * @typedef ReadRowsCallback
                     * @type {function}
                     * @param {Error|null} error Error, if any
                     * @param {google.bigtable.v2.ReadRowsResponse} [response] ReadRowsResponse
                     */
    
                    /**
                     * Calls ReadRows.
                     * @function readRows
                     * @memberof google.bigtable.v2.Bigtable
                     * @instance
                     * @param {google.bigtable.v2.IReadRowsRequest} request ReadRowsRequest message or plain object
                     * @param {google.bigtable.v2.Bigtable.ReadRowsCallback} callback Node-style callback called with the error, if any, and ReadRowsResponse
                     * @returns {undefined}
                     * @variation 1
                     */
                    Object.defineProperty(Bigtable.prototype.readRows = function readRows(request, callback) {
                        return this.rpcCall(readRows, $root.google.bigtable.v2.ReadRowsRequest, $root.google.bigtable.v2.ReadRowsResponse, request, callback);
                    }, "name", { value: "ReadRows" });
    
                    /**
                     * Calls ReadRows.
                     * @function readRows
                     * @memberof google.bigtable.v2.Bigtable
                     * @instance
                     * @param {google.bigtable.v2.IReadRowsRequest} request ReadRowsRequest message or plain object
                     * @returns {Promise<google.bigtable.v2.ReadRowsResponse>} Promise
                     * @variation 2
                     */
    
                    /**
                     * Callback as used by {@link google.bigtable.v2.Bigtable|sampleRowKeys}.
                     * @memberof google.bigtable.v2.Bigtable
                     * @typedef SampleRowKeysCallback
                     * @type {function}
                     * @param {Error|null} error Error, if any
                     * @param {google.bigtable.v2.SampleRowKeysResponse} [response] SampleRowKeysResponse
                     */
    
                    /**
                     * Calls SampleRowKeys.
                     * @function sampleRowKeys
                     * @memberof google.bigtable.v2.Bigtable
                     * @instance
                     * @param {google.bigtable.v2.ISampleRowKeysRequest} request SampleRowKeysRequest message or plain object
                     * @param {google.bigtable.v2.Bigtable.SampleRowKeysCallback} callback Node-style callback called with the error, if any, and SampleRowKeysResponse
                     * @returns {undefined}
                     * @variation 1
                     */
                    Object.defineProperty(Bigtable.prototype.sampleRowKeys = function sampleRowKeys(request, callback) {
                        return this.rpcCall(sampleRowKeys, $root.google.bigtable.v2.SampleRowKeysRequest, $root.google.bigtable.v2.SampleRowKeysResponse, request, callback);
                    }, "name", { value: "SampleRowKeys" });
    
                    /**
                     * Calls SampleRowKeys.
                     * @function sampleRowKeys
                     * @memberof google.bigtable.v2.Bigtable
                     * @instance
                     * @param {google.bigtable.v2.ISampleRowKeysRequest} request SampleRowKeysRequest message or plain object
                     * @returns {Promise<google.bigtable.v2.SampleRowKeysResponse>} Promise
                     * @variation 2
                     */
    
                    /**
                     * Callback as used by {@link google.bigtable.v2.Bigtable|mutateRow}.
                     * @memberof google.bigtable.v2.Bigtable
                     * @typedef MutateRowCallback
                     * @type {function}
                     * @param {Error|null} error Error, if any
                     * @param {google.bigtable.v2.MutateRowResponse} [response] MutateRowResponse
                     */
    
                    /**
                     * Calls MutateRow.
                     * @function mutateRow
                     * @memberof google.bigtable.v2.Bigtable
                     * @instance
                     * @param {google.bigtable.v2.IMutateRowRequest} request MutateRowRequest message or plain object
                     * @param {google.bigtable.v2.Bigtable.MutateRowCallback} callback Node-style callback called with the error, if any, and MutateRowResponse
                     * @returns {undefined}
                     * @variation 1
                     */
                    Object.defineProperty(Bigtable.prototype.mutateRow = function mutateRow(request, callback) {
                        return this.rpcCall(mutateRow, $root.google.bigtable.v2.MutateRowRequest, $root.google.bigtable.v2.MutateRowResponse, request, callback);
                    }, "name", { value: "MutateRow" });
    
                    /**
                     * Calls MutateRow.
                     * @function mutateRow
                     * @memberof google.bigtable.v2.Bigtable
                     * @instance
                     * @param {google.bigtable.v2.IMutateRowRequest} request MutateRowRequest message or plain object
                     * @returns {Promise<google.bigtable.v2.MutateRowResponse>} Promise
                     * @variation 2
                     */
    
                    /**
                     * Callback as used by {@link google.bigtable.v2.Bigtable|mutateRows}.
                     * @memberof google.bigtable.v2.Bigtable
                     * @typedef MutateRowsCallback
                     * @type {function}
                     * @param {Error|null} error Error, if any
                     * @param {google.bigtable.v2.MutateRowsResponse} [response] MutateRowsResponse
                     */
    
                    /**
                     * Calls MutateRows.
                     * @function mutateRows
                     * @memberof google.bigtable.v2.Bigtable
                     * @instance
                     * @param {google.bigtable.v2.IMutateRowsRequest} request MutateRowsRequest message or plain object
                     * @param {google.bigtable.v2.Bigtable.MutateRowsCallback} callback Node-style callback called with the error, if any, and MutateRowsResponse
                     * @returns {undefined}
                     * @variation 1
                     */
                    Object.defineProperty(Bigtable.prototype.mutateRows = function mutateRows(request, callback) {
                        return this.rpcCall(mutateRows, $root.google.bigtable.v2.MutateRowsRequest, $root.google.bigtable.v2.MutateRowsResponse, request, callback);
                    }, "name", { value: "MutateRows" });
    
                    /**
                     * Calls MutateRows.
                     * @function mutateRows
                     * @memberof google.bigtable.v2.Bigtable
                     * @instance
                     * @param {google.bigtable.v2.IMutateRowsRequest} request MutateRowsRequest message or plain object
                     * @returns {Promise<google.bigtable.v2.MutateRowsResponse>} Promise
                     * @variation 2
                     */
    
                    /**
                     * Callback as used by {@link google.bigtable.v2.Bigtable|checkAndMutateRow}.
                     * @memberof google.bigtable.v2.Bigtable
                     * @typedef CheckAndMutateRowCallback
                     * @type {function}
                     * @param {Error|null} error Error, if any
                     * @param {google.bigtable.v2.CheckAndMutateRowResponse} [response] CheckAndMutateRowResponse
                     */
    
                    /**
                     * Calls CheckAndMutateRow.
                     * @function checkAndMutateRow
                     * @memberof google.bigtable.v2.Bigtable
                     * @instance
                     * @param {google.bigtable.v2.ICheckAndMutateRowRequest} request CheckAndMutateRowRequest message or plain object
                     * @param {google.bigtable.v2.Bigtable.CheckAndMutateRowCallback} callback Node-style callback called with the error, if any, and CheckAndMutateRowResponse
                     * @returns {undefined}
                     * @variation 1
                     */
                    Object.defineProperty(Bigtable.prototype.checkAndMutateRow = function checkAndMutateRow(request, callback) {
                        return this.rpcCall(checkAndMutateRow, $root.google.bigtable.v2.CheckAndMutateRowRequest, $root.google.bigtable.v2.CheckAndMutateRowResponse, request, callback);
                    }, "name", { value: "CheckAndMutateRow" });
    
                    /**
                     * Calls CheckAndMutateRow.
                     * @function checkAndMutateRow
                     * @memberof google.bigtable.v2.Bigtable
                     * @instance
                     * @param {google.bigtable.v2.ICheckAndMutateRowRequest} request CheckAndMutateRowRequest message or plain object
                     * @returns {Promise<google.bigtable.v2.CheckAndMutateRowResponse>} Promise
                     * @variation 2
                     */
    
                    /**
                     * Callback as used by {@link google.bigtable.v2.Bigtable|pingAndWarm}.
                     * @memberof google.bigtable.v2.Bigtable
                     * @typedef PingAndWarmCallback
                     * @type {function}
                     * @param {Error|null} error Error, if any
                     * @param {google.bigtable.v2.PingAndWarmResponse} [response] PingAndWarmResponse
                     */
    
                    /**
                     * Calls PingAndWarm.
                     * @function pingAndWarm
                     * @memberof google.bigtable.v2.Bigtable
                     * @instance
                     * @param {google.bigtable.v2.IPingAndWarmRequest} request PingAndWarmRequest message or plain object
                     * @param {google.bigtable.v2.Bigtable.PingAndWarmCallback} callback Node-style callback called with the error, if any, and PingAndWarmResponse
                     * @returns {undefined}
                     * @variation 1
                     */
                    Object.defineProperty(Bigtable.prototype.pingAndWarm = function pingAndWarm(request, callback) {
                        return this.rpcCall(pingAndWarm, $root.google.bigtable.v2.PingAndWarmRequest, $root.google.bigtable.v2.PingAndWarmResponse, request, callback);
                    }, "name", { value: "PingAndWarm" });
    
                    /**
                     * Calls PingAndWarm.
                     * @function pingAndWarm
                     * @memberof google.bigtable.v2.Bigtable
                     * @instance
                     * @param {google.bigtable.v2.IPingAndWarmRequest} request PingAndWarmRequest message or plain object
                     * @returns {Promise<google.bigtable.v2.PingAndWarmResponse>} Promise
                     * @variation 2
                     */
    
                    /**
                     * Callback as used by {@link google.bigtable.v2.Bigtable|readModifyWriteRow}.
                     * @memberof google.bigtable.v2.Bigtable
                     * @typedef ReadModifyWriteRowCallback
                     * @type {function}
                     * @param {Error|null} error Error, if any
                     * @param {google.bigtable.v2.ReadModifyWriteRowResponse} [response] ReadModifyWriteRowResponse
                     */
    
                    /**
                     * Calls ReadModifyWriteRow.
                     * @function readModifyWriteRow
                     * @memberof google.bigtable.v2.Bigtable
                     * @instance
                     * @param {google.bigtable.v2.IReadModifyWriteRowRequest} request ReadModifyWriteRowRequest message or plain object
                     * @param {google.bigtable.v2.Bigtable.ReadModifyWriteRowCallback} callback Node-style callback called with the error, if any, and ReadModifyWriteRowResponse
                     * @returns {undefined}
                     * @variation 1
                     */
                    Object.defineProperty(Bigtable.prototype.readModifyWriteRow = function readModifyWriteRow(request, callback) {
                        return this.rpcCall(readModifyWriteRow, $root.google.bigtable.v2.ReadModifyWriteRowRequest, $root.google.bigtable.v2.ReadModifyWriteRowResponse, request, callback);
                    }, "name", { value: "ReadModifyWriteRow" });
    
                    /**
                     * Calls ReadModifyWriteRow.
                     * @function readModifyWriteRow
                     * @memberof google.bigtable.v2.Bigtable
                     * @instance
                     * @param {google.bigtable.v2.IReadModifyWriteRowRequest} request ReadModifyWriteRowRequest message or plain object
                     * @returns {Promise<google.bigtable.v2.ReadModifyWriteRowResponse>} Promise
                     * @variation 2
                     */
    
                    /**
                     * Callback as used by {@link google.bigtable.v2.Bigtable|generateInitialChangeStreamPartitions}.
                     * @memberof google.bigtable.v2.Bigtable
                     * @typedef GenerateInitialChangeStreamPartitionsCallback
                     * @type {function}
                     * @param {Error|null} error Error, if any
                     * @param {google.bigtable.v2.GenerateInitialChangeStreamPartitionsResponse} [response] GenerateInitialChangeStreamPartitionsResponse
                     */
    
                    /**
                     * Calls GenerateInitialChangeStreamPartitions.
                     * @function generateInitialChangeStreamPartitions
                     * @memberof google.bigtable.v2.Bigtable
                     * @instance
                     * @param {google.bigtable.v2.IGenerateInitialChangeStreamPartitionsRequest} request GenerateInitialChangeStreamPartitionsRequest message or plain object
                     * @param {google.bigtable.v2.Bigtable.GenerateInitialChangeStreamPartitionsCallback} callback Node-style callback called with the error, if any, and GenerateInitialChangeStreamPartitionsResponse
                     * @returns {undefined}
                     * @variation 1
                     */
                    Object.defineProperty(Bigtable.prototype.generateInitialChangeStreamPartitions = function generateInitialChangeStreamPartitions(request, callback) {
                        return this.rpcCall(generateInitialChangeStreamPartitions, $root.google.bigtable.v2.GenerateInitialChangeStreamPartitionsRequest, $root.google.bigtable.v2.GenerateInitialChangeStreamPartitionsResponse, request, callback);
                    }, "name", { value: "GenerateInitialChangeStreamPartitions" });
    
                    /**
                     * Calls GenerateInitialChangeStreamPartitions.
                     * @function generateInitialChangeStreamPartitions
                     * @memberof google.bigtable.v2.Bigtable
                     * @instance
                     * @param {google.bigtable.v2.IGenerateInitialChangeStreamPartitionsRequest} request GenerateInitialChangeStreamPartitionsRequest message or plain object
                     * @returns {Promise<google.bigtable.v2.GenerateInitialChangeStreamPartitionsResponse>} Promise
                     * @variation 2
                     */
    
                    /**
                     * Callback as used by {@link google.bigtable.v2.Bigtable|readChangeStream}.
                     * @memberof google.bigtable.v2.Bigtable
                     * @typedef ReadChangeStreamCallback
                     * @type {function}
                     * @param {Error|null} error Error, if any
                     * @param {google.bigtable.v2.ReadChangeStreamResponse} [response] ReadChangeStreamResponse
                     */
    
                    /**
                     * Calls ReadChangeStream.
                     * @function readChangeStream
                     * @memberof google.bigtable.v2.Bigtable
                     * @instance
                     * @param {google.bigtable.v2.IReadChangeStreamRequest} request ReadChangeStreamRequest message or plain object
                     * @param {google.bigtable.v2.Bigtable.ReadChangeStreamCallback} callback Node-style callback called with the error, if any, and ReadChangeStreamResponse
                     * @returns {undefined}
                     * @variation 1
                     */
                    Object.defineProperty(Bigtable.prototype.readChangeStream = function readChangeStream(request, callback) {
                        return this.rpcCall(readChangeStream, $root.google.bigtable.v2.ReadChangeStreamRequest, $root.google.bigtable.v2.ReadChangeStreamResponse, request, callback);
                    }, "name", { value: "ReadChangeStream" });
    
                    /**
                     * Calls ReadChangeStream.
                     * @function readChangeStream
                     * @memberof google.bigtable.v2.Bigtable
                     * @instance
                     * @param {google.bigtable.v2.IReadChangeStreamRequest} request ReadChangeStreamRequest message or plain object
                     * @returns {Promise<google.bigtable.v2.ReadChangeStreamResponse>} Promise
                     * @variation 2
                     */
    
                    return Bigtable;
                })();
    
                v2.ReadRowsRequest = (function() {
    
                    /**
                     * Properties of a ReadRowsRequest.
                     * @memberof google.bigtable.v2
                     * @interface IReadRowsRequest
                     * @property {string|null} [tableName] ReadRowsRequest tableName
                     * @property {string|null} [authorizedViewName] ReadRowsRequest authorizedViewName
                     * @property {string|null} [appProfileId] ReadRowsRequest appProfileId
                     * @property {google.bigtable.v2.IRowSet|null} [rows] ReadRowsRequest rows
                     * @property {google.bigtable.v2.IRowFilter|null} [filter] ReadRowsRequest filter
                     * @property {number|Long|null} [rowsLimit] ReadRowsRequest rowsLimit
                     * @property {google.bigtable.v2.ReadRowsRequest.RequestStatsView|null} [requestStatsView] ReadRowsRequest requestStatsView
                     * @property {boolean|null} [reversed] ReadRowsRequest reversed
                     */
    
                    /**
                     * Constructs a new ReadRowsRequest.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a ReadRowsRequest.
                     * @implements IReadRowsRequest
                     * @constructor
                     * @param {google.bigtable.v2.IReadRowsRequest=} [properties] Properties to set
                     */
                    function ReadRowsRequest(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * ReadRowsRequest tableName.
                     * @member {string} tableName
                     * @memberof google.bigtable.v2.ReadRowsRequest
                     * @instance
                     */
                    ReadRowsRequest.prototype.tableName = "";
    
                    /**
                     * ReadRowsRequest authorizedViewName.
                     * @member {string} authorizedViewName
                     * @memberof google.bigtable.v2.ReadRowsRequest
                     * @instance
                     */
                    ReadRowsRequest.prototype.authorizedViewName = "";
    
                    /**
                     * ReadRowsRequest appProfileId.
                     * @member {string} appProfileId
                     * @memberof google.bigtable.v2.ReadRowsRequest
                     * @instance
                     */
                    ReadRowsRequest.prototype.appProfileId = "";
    
                    /**
                     * ReadRowsRequest rows.
                     * @member {google.bigtable.v2.IRowSet|null|undefined} rows
                     * @memberof google.bigtable.v2.ReadRowsRequest
                     * @instance
                     */
                    ReadRowsRequest.prototype.rows = null;
    
                    /**
                     * ReadRowsRequest filter.
                     * @member {google.bigtable.v2.IRowFilter|null|undefined} filter
                     * @memberof google.bigtable.v2.ReadRowsRequest
                     * @instance
                     */
                    ReadRowsRequest.prototype.filter = null;
    
                    /**
                     * ReadRowsRequest rowsLimit.
                     * @member {number|Long} rowsLimit
                     * @memberof google.bigtable.v2.ReadRowsRequest
                     * @instance
                     */
                    ReadRowsRequest.prototype.rowsLimit = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                    /**
                     * ReadRowsRequest requestStatsView.
                     * @member {google.bigtable.v2.ReadRowsRequest.RequestStatsView} requestStatsView
                     * @memberof google.bigtable.v2.ReadRowsRequest
                     * @instance
                     */
                    ReadRowsRequest.prototype.requestStatsView = 0;
    
                    /**
                     * ReadRowsRequest reversed.
                     * @member {boolean} reversed
                     * @memberof google.bigtable.v2.ReadRowsRequest
                     * @instance
                     */
                    ReadRowsRequest.prototype.reversed = false;
    
                    /**
                     * Creates a new ReadRowsRequest instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.ReadRowsRequest
                     * @static
                     * @param {google.bigtable.v2.IReadRowsRequest=} [properties] Properties to set
                     * @returns {google.bigtable.v2.ReadRowsRequest} ReadRowsRequest instance
                     */
                    ReadRowsRequest.create = function create(properties) {
                        return new ReadRowsRequest(properties);
                    };
    
                    /**
                     * Encodes the specified ReadRowsRequest message. Does not implicitly {@link google.bigtable.v2.ReadRowsRequest.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.ReadRowsRequest
                     * @static
                     * @param {google.bigtable.v2.IReadRowsRequest} message ReadRowsRequest message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ReadRowsRequest.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.tableName != null && Object.hasOwnProperty.call(message, "tableName"))
                            writer.uint32(/* id 1, wireType 2 =*/10).string(message.tableName);
                        if (message.rows != null && Object.hasOwnProperty.call(message, "rows"))
                            $root.google.bigtable.v2.RowSet.encode(message.rows, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                        if (message.filter != null && Object.hasOwnProperty.call(message, "filter"))
                            $root.google.bigtable.v2.RowFilter.encode(message.filter, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                        if (message.rowsLimit != null && Object.hasOwnProperty.call(message, "rowsLimit"))
                            writer.uint32(/* id 4, wireType 0 =*/32).int64(message.rowsLimit);
                        if (message.appProfileId != null && Object.hasOwnProperty.call(message, "appProfileId"))
                            writer.uint32(/* id 5, wireType 2 =*/42).string(message.appProfileId);
                        if (message.requestStatsView != null && Object.hasOwnProperty.call(message, "requestStatsView"))
                            writer.uint32(/* id 6, wireType 0 =*/48).int32(message.requestStatsView);
                        if (message.reversed != null && Object.hasOwnProperty.call(message, "reversed"))
                            writer.uint32(/* id 7, wireType 0 =*/56).bool(message.reversed);
                        if (message.authorizedViewName != null && Object.hasOwnProperty.call(message, "authorizedViewName"))
                            writer.uint32(/* id 9, wireType 2 =*/74).string(message.authorizedViewName);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified ReadRowsRequest message, length delimited. Does not implicitly {@link google.bigtable.v2.ReadRowsRequest.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.ReadRowsRequest
                     * @static
                     * @param {google.bigtable.v2.IReadRowsRequest} message ReadRowsRequest message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ReadRowsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a ReadRowsRequest message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.ReadRowsRequest
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.ReadRowsRequest} ReadRowsRequest
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ReadRowsRequest.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.ReadRowsRequest();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.tableName = reader.string();
                                    break;
                                }
                            case 9: {
                                    message.authorizedViewName = reader.string();
                                    break;
                                }
                            case 5: {
                                    message.appProfileId = reader.string();
                                    break;
                                }
                            case 2: {
                                    message.rows = $root.google.bigtable.v2.RowSet.decode(reader, reader.uint32());
                                    break;
                                }
                            case 3: {
                                    message.filter = $root.google.bigtable.v2.RowFilter.decode(reader, reader.uint32());
                                    break;
                                }
                            case 4: {
                                    message.rowsLimit = reader.int64();
                                    break;
                                }
                            case 6: {
                                    message.requestStatsView = reader.int32();
                                    break;
                                }
                            case 7: {
                                    message.reversed = reader.bool();
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a ReadRowsRequest message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.ReadRowsRequest
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.ReadRowsRequest} ReadRowsRequest
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ReadRowsRequest.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a ReadRowsRequest message.
                     * @function verify
                     * @memberof google.bigtable.v2.ReadRowsRequest
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    ReadRowsRequest.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.tableName != null && message.hasOwnProperty("tableName"))
                            if (!$util.isString(message.tableName))
                                return "tableName: string expected";
                        if (message.authorizedViewName != null && message.hasOwnProperty("authorizedViewName"))
                            if (!$util.isString(message.authorizedViewName))
                                return "authorizedViewName: string expected";
                        if (message.appProfileId != null && message.hasOwnProperty("appProfileId"))
                            if (!$util.isString(message.appProfileId))
                                return "appProfileId: string expected";
                        if (message.rows != null && message.hasOwnProperty("rows")) {
                            var error = $root.google.bigtable.v2.RowSet.verify(message.rows);
                            if (error)
                                return "rows." + error;
                        }
                        if (message.filter != null && message.hasOwnProperty("filter")) {
                            var error = $root.google.bigtable.v2.RowFilter.verify(message.filter);
                            if (error)
                                return "filter." + error;
                        }
                        if (message.rowsLimit != null && message.hasOwnProperty("rowsLimit"))
                            if (!$util.isInteger(message.rowsLimit) && !(message.rowsLimit && $util.isInteger(message.rowsLimit.low) && $util.isInteger(message.rowsLimit.high)))
                                return "rowsLimit: integer|Long expected";
                        if (message.requestStatsView != null && message.hasOwnProperty("requestStatsView"))
                            switch (message.requestStatsView) {
                            default:
                                return "requestStatsView: enum value expected";
                            case 0:
                            case 1:
                            case 2:
                                break;
                            }
                        if (message.reversed != null && message.hasOwnProperty("reversed"))
                            if (typeof message.reversed !== "boolean")
                                return "reversed: boolean expected";
                        return null;
                    };
    
                    /**
                     * Creates a ReadRowsRequest message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.ReadRowsRequest
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.ReadRowsRequest} ReadRowsRequest
                     */
                    ReadRowsRequest.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.ReadRowsRequest)
                            return object;
                        var message = new $root.google.bigtable.v2.ReadRowsRequest();
                        if (object.tableName != null)
                            message.tableName = String(object.tableName);
                        if (object.authorizedViewName != null)
                            message.authorizedViewName = String(object.authorizedViewName);
                        if (object.appProfileId != null)
                            message.appProfileId = String(object.appProfileId);
                        if (object.rows != null) {
                            if (typeof object.rows !== "object")
                                throw TypeError(".google.bigtable.v2.ReadRowsRequest.rows: object expected");
                            message.rows = $root.google.bigtable.v2.RowSet.fromObject(object.rows);
                        }
                        if (object.filter != null) {
                            if (typeof object.filter !== "object")
                                throw TypeError(".google.bigtable.v2.ReadRowsRequest.filter: object expected");
                            message.filter = $root.google.bigtable.v2.RowFilter.fromObject(object.filter);
                        }
                        if (object.rowsLimit != null)
                            if ($util.Long)
                                (message.rowsLimit = $util.Long.fromValue(object.rowsLimit)).unsigned = false;
                            else if (typeof object.rowsLimit === "string")
                                message.rowsLimit = parseInt(object.rowsLimit, 10);
                            else if (typeof object.rowsLimit === "number")
                                message.rowsLimit = object.rowsLimit;
                            else if (typeof object.rowsLimit === "object")
                                message.rowsLimit = new $util.LongBits(object.rowsLimit.low >>> 0, object.rowsLimit.high >>> 0).toNumber();
                        switch (object.requestStatsView) {
                        default:
                            if (typeof object.requestStatsView === "number") {
                                message.requestStatsView = object.requestStatsView;
                                break;
                            }
                            break;
                        case "REQUEST_STATS_VIEW_UNSPECIFIED":
                        case 0:
                            message.requestStatsView = 0;
                            break;
                        case "REQUEST_STATS_NONE":
                        case 1:
                            message.requestStatsView = 1;
                            break;
                        case "REQUEST_STATS_FULL":
                        case 2:
                            message.requestStatsView = 2;
                            break;
                        }
                        if (object.reversed != null)
                            message.reversed = Boolean(object.reversed);
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a ReadRowsRequest message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.ReadRowsRequest
                     * @static
                     * @param {google.bigtable.v2.ReadRowsRequest} message ReadRowsRequest
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    ReadRowsRequest.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults) {
                            object.tableName = "";
                            object.rows = null;
                            object.filter = null;
                            if ($util.Long) {
                                var long = new $util.Long(0, 0, false);
                                object.rowsLimit = options.longs === String ? long.toString() : options.longs === Number ? long.toNumber() : long;
                            } else
                                object.rowsLimit = options.longs === String ? "0" : 0;
                            object.appProfileId = "";
                            object.requestStatsView = options.enums === String ? "REQUEST_STATS_VIEW_UNSPECIFIED" : 0;
                            object.reversed = false;
                            object.authorizedViewName = "";
                        }
                        if (message.tableName != null && message.hasOwnProperty("tableName"))
                            object.tableName = message.tableName;
                        if (message.rows != null && message.hasOwnProperty("rows"))
                            object.rows = $root.google.bigtable.v2.RowSet.toObject(message.rows, options);
                        if (message.filter != null && message.hasOwnProperty("filter"))
                            object.filter = $root.google.bigtable.v2.RowFilter.toObject(message.filter, options);
                        if (message.rowsLimit != null && message.hasOwnProperty("rowsLimit"))
                            if (typeof message.rowsLimit === "number")
                                object.rowsLimit = options.longs === String ? String(message.rowsLimit) : message.rowsLimit;
                            else
                                object.rowsLimit = options.longs === String ? $util.Long.prototype.toString.call(message.rowsLimit) : options.longs === Number ? new $util.LongBits(message.rowsLimit.low >>> 0, message.rowsLimit.high >>> 0).toNumber() : message.rowsLimit;
                        if (message.appProfileId != null && message.hasOwnProperty("appProfileId"))
                            object.appProfileId = message.appProfileId;
                        if (message.requestStatsView != null && message.hasOwnProperty("requestStatsView"))
                            object.requestStatsView = options.enums === String ? $root.google.bigtable.v2.ReadRowsRequest.RequestStatsView[message.requestStatsView] === undefined ? message.requestStatsView : $root.google.bigtable.v2.ReadRowsRequest.RequestStatsView[message.requestStatsView] : message.requestStatsView;
                        if (message.reversed != null && message.hasOwnProperty("reversed"))
                            object.reversed = message.reversed;
                        if (message.authorizedViewName != null && message.hasOwnProperty("authorizedViewName"))
                            object.authorizedViewName = message.authorizedViewName;
                        return object;
                    };
    
                    /**
                     * Converts this ReadRowsRequest to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.ReadRowsRequest
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    ReadRowsRequest.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for ReadRowsRequest
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.ReadRowsRequest
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    ReadRowsRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.ReadRowsRequest";
                    };
    
                    /**
                     * RequestStatsView enum.
                     * @name google.bigtable.v2.ReadRowsRequest.RequestStatsView
                     * @enum {number}
                     * @property {number} REQUEST_STATS_VIEW_UNSPECIFIED=0 REQUEST_STATS_VIEW_UNSPECIFIED value
                     * @property {number} REQUEST_STATS_NONE=1 REQUEST_STATS_NONE value
                     * @property {number} REQUEST_STATS_FULL=2 REQUEST_STATS_FULL value
                     */
                    ReadRowsRequest.RequestStatsView = (function() {
                        var valuesById = {}, values = Object.create(valuesById);
                        values[valuesById[0] = "REQUEST_STATS_VIEW_UNSPECIFIED"] = 0;
                        values[valuesById[1] = "REQUEST_STATS_NONE"] = 1;
                        values[valuesById[2] = "REQUEST_STATS_FULL"] = 2;
                        return values;
                    })();
    
                    return ReadRowsRequest;
                })();
    
                v2.ReadRowsResponse = (function() {
    
                    /**
                     * Properties of a ReadRowsResponse.
                     * @memberof google.bigtable.v2
                     * @interface IReadRowsResponse
                     * @property {Array.<google.bigtable.v2.ReadRowsResponse.ICellChunk>|null} [chunks] ReadRowsResponse chunks
                     * @property {Uint8Array|null} [lastScannedRowKey] ReadRowsResponse lastScannedRowKey
                     * @property {google.bigtable.v2.IRequestStats|null} [requestStats] ReadRowsResponse requestStats
                     */
    
                    /**
                     * Constructs a new ReadRowsResponse.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a ReadRowsResponse.
                     * @implements IReadRowsResponse
                     * @constructor
                     * @param {google.bigtable.v2.IReadRowsResponse=} [properties] Properties to set
                     */
                    function ReadRowsResponse(properties) {
                        this.chunks = [];
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * ReadRowsResponse chunks.
                     * @member {Array.<google.bigtable.v2.ReadRowsResponse.ICellChunk>} chunks
                     * @memberof google.bigtable.v2.ReadRowsResponse
                     * @instance
                     */
                    ReadRowsResponse.prototype.chunks = $util.emptyArray;
    
                    /**
                     * ReadRowsResponse lastScannedRowKey.
                     * @member {Uint8Array} lastScannedRowKey
                     * @memberof google.bigtable.v2.ReadRowsResponse
                     * @instance
                     */
                    ReadRowsResponse.prototype.lastScannedRowKey = $util.newBuffer([]);
    
                    /**
                     * ReadRowsResponse requestStats.
                     * @member {google.bigtable.v2.IRequestStats|null|undefined} requestStats
                     * @memberof google.bigtable.v2.ReadRowsResponse
                     * @instance
                     */
                    ReadRowsResponse.prototype.requestStats = null;
    
                    /**
                     * Creates a new ReadRowsResponse instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.ReadRowsResponse
                     * @static
                     * @param {google.bigtable.v2.IReadRowsResponse=} [properties] Properties to set
                     * @returns {google.bigtable.v2.ReadRowsResponse} ReadRowsResponse instance
                     */
                    ReadRowsResponse.create = function create(properties) {
                        return new ReadRowsResponse(properties);
                    };
    
                    /**
                     * Encodes the specified ReadRowsResponse message. Does not implicitly {@link google.bigtable.v2.ReadRowsResponse.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.ReadRowsResponse
                     * @static
                     * @param {google.bigtable.v2.IReadRowsResponse} message ReadRowsResponse message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ReadRowsResponse.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.chunks != null && message.chunks.length)
                            for (var i = 0; i < message.chunks.length; ++i)
                                $root.google.bigtable.v2.ReadRowsResponse.CellChunk.encode(message.chunks[i], writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                        if (message.lastScannedRowKey != null && Object.hasOwnProperty.call(message, "lastScannedRowKey"))
                            writer.uint32(/* id 2, wireType 2 =*/18).bytes(message.lastScannedRowKey);
                        if (message.requestStats != null && Object.hasOwnProperty.call(message, "requestStats"))
                            $root.google.bigtable.v2.RequestStats.encode(message.requestStats, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                        return writer;
                    };
    
                    /**
                     * Encodes the specified ReadRowsResponse message, length delimited. Does not implicitly {@link google.bigtable.v2.ReadRowsResponse.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.ReadRowsResponse
                     * @static
                     * @param {google.bigtable.v2.IReadRowsResponse} message ReadRowsResponse message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ReadRowsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a ReadRowsResponse message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.ReadRowsResponse
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.ReadRowsResponse} ReadRowsResponse
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ReadRowsResponse.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.ReadRowsResponse();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    if (!(message.chunks && message.chunks.length))
                                        message.chunks = [];
                                    message.chunks.push($root.google.bigtable.v2.ReadRowsResponse.CellChunk.decode(reader, reader.uint32()));
                                    break;
                                }
                            case 2: {
                                    message.lastScannedRowKey = reader.bytes();
                                    break;
                                }
                            case 3: {
                                    message.requestStats = $root.google.bigtable.v2.RequestStats.decode(reader, reader.uint32());
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a ReadRowsResponse message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.ReadRowsResponse
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.ReadRowsResponse} ReadRowsResponse
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ReadRowsResponse.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a ReadRowsResponse message.
                     * @function verify
                     * @memberof google.bigtable.v2.ReadRowsResponse
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    ReadRowsResponse.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.chunks != null && message.hasOwnProperty("chunks")) {
                            if (!Array.isArray(message.chunks))
                                return "chunks: array expected";
                            for (var i = 0; i < message.chunks.length; ++i) {
                                var error = $root.google.bigtable.v2.ReadRowsResponse.CellChunk.verify(message.chunks[i]);
                                if (error)
                                    return "chunks." + error;
                            }
                        }
                        if (message.lastScannedRowKey != null && message.hasOwnProperty("lastScannedRowKey"))
                            if (!(message.lastScannedRowKey && typeof message.lastScannedRowKey.length === "number" || $util.isString(message.lastScannedRowKey)))
                                return "lastScannedRowKey: buffer expected";
                        if (message.requestStats != null && message.hasOwnProperty("requestStats")) {
                            var error = $root.google.bigtable.v2.RequestStats.verify(message.requestStats);
                            if (error)
                                return "requestStats." + error;
                        }
                        return null;
                    };
    
                    /**
                     * Creates a ReadRowsResponse message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.ReadRowsResponse
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.ReadRowsResponse} ReadRowsResponse
                     */
                    ReadRowsResponse.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.ReadRowsResponse)
                            return object;
                        var message = new $root.google.bigtable.v2.ReadRowsResponse();
                        if (object.chunks) {
                            if (!Array.isArray(object.chunks))
                                throw TypeError(".google.bigtable.v2.ReadRowsResponse.chunks: array expected");
                            message.chunks = [];
                            for (var i = 0; i < object.chunks.length; ++i) {
                                if (typeof object.chunks[i] !== "object")
                                    throw TypeError(".google.bigtable.v2.ReadRowsResponse.chunks: object expected");
                                message.chunks[i] = $root.google.bigtable.v2.ReadRowsResponse.CellChunk.fromObject(object.chunks[i]);
                            }
                        }
                        if (object.lastScannedRowKey != null)
                            if (typeof object.lastScannedRowKey === "string")
                                $util.base64.decode(object.lastScannedRowKey, message.lastScannedRowKey = $util.newBuffer($util.base64.length(object.lastScannedRowKey)), 0);
                            else if (object.lastScannedRowKey.length >= 0)
                                message.lastScannedRowKey = object.lastScannedRowKey;
                        if (object.requestStats != null) {
                            if (typeof object.requestStats !== "object")
                                throw TypeError(".google.bigtable.v2.ReadRowsResponse.requestStats: object expected");
                            message.requestStats = $root.google.bigtable.v2.RequestStats.fromObject(object.requestStats);
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a ReadRowsResponse message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.ReadRowsResponse
                     * @static
                     * @param {google.bigtable.v2.ReadRowsResponse} message ReadRowsResponse
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    ReadRowsResponse.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.arrays || options.defaults)
                            object.chunks = [];
                        if (options.defaults) {
                            if (options.bytes === String)
                                object.lastScannedRowKey = "";
                            else {
                                object.lastScannedRowKey = [];
                                if (options.bytes !== Array)
                                    object.lastScannedRowKey = $util.newBuffer(object.lastScannedRowKey);
                            }
                            object.requestStats = null;
                        }
                        if (message.chunks && message.chunks.length) {
                            object.chunks = [];
                            for (var j = 0; j < message.chunks.length; ++j)
                                object.chunks[j] = $root.google.bigtable.v2.ReadRowsResponse.CellChunk.toObject(message.chunks[j], options);
                        }
                        if (message.lastScannedRowKey != null && message.hasOwnProperty("lastScannedRowKey"))
                            object.lastScannedRowKey = options.bytes === String ? $util.base64.encode(message.lastScannedRowKey, 0, message.lastScannedRowKey.length) : options.bytes === Array ? Array.prototype.slice.call(message.lastScannedRowKey) : message.lastScannedRowKey;
                        if (message.requestStats != null && message.hasOwnProperty("requestStats"))
                            object.requestStats = $root.google.bigtable.v2.RequestStats.toObject(message.requestStats, options);
                        return object;
                    };
    
                    /**
                     * Converts this ReadRowsResponse to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.ReadRowsResponse
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    ReadRowsResponse.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for ReadRowsResponse
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.ReadRowsResponse
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    ReadRowsResponse.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.ReadRowsResponse";
                    };
    
                    ReadRowsResponse.CellChunk = (function() {
    
                        /**
                         * Properties of a CellChunk.
                         * @memberof google.bigtable.v2.ReadRowsResponse
                         * @interface ICellChunk
                         * @property {Uint8Array|null} [rowKey] CellChunk rowKey
                         * @property {google.protobuf.IStringValue|null} [familyName] CellChunk familyName
                         * @property {google.protobuf.IBytesValue|null} [qualifier] CellChunk qualifier
                         * @property {number|Long|null} [timestampMicros] CellChunk timestampMicros
                         * @property {Array.<string>|null} [labels] CellChunk labels
                         * @property {Uint8Array|null} [value] CellChunk value
                         * @property {number|null} [valueSize] CellChunk valueSize
                         * @property {boolean|null} [resetRow] CellChunk resetRow
                         * @property {boolean|null} [commitRow] CellChunk commitRow
                         */
    
                        /**
                         * Constructs a new CellChunk.
                         * @memberof google.bigtable.v2.ReadRowsResponse
                         * @classdesc Represents a CellChunk.
                         * @implements ICellChunk
                         * @constructor
                         * @param {google.bigtable.v2.ReadRowsResponse.ICellChunk=} [properties] Properties to set
                         */
                        function CellChunk(properties) {
                            this.labels = [];
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * CellChunk rowKey.
                         * @member {Uint8Array} rowKey
                         * @memberof google.bigtable.v2.ReadRowsResponse.CellChunk
                         * @instance
                         */
                        CellChunk.prototype.rowKey = $util.newBuffer([]);
    
                        /**
                         * CellChunk familyName.
                         * @member {google.protobuf.IStringValue|null|undefined} familyName
                         * @memberof google.bigtable.v2.ReadRowsResponse.CellChunk
                         * @instance
                         */
                        CellChunk.prototype.familyName = null;
    
                        /**
                         * CellChunk qualifier.
                         * @member {google.protobuf.IBytesValue|null|undefined} qualifier
                         * @memberof google.bigtable.v2.ReadRowsResponse.CellChunk
                         * @instance
                         */
                        CellChunk.prototype.qualifier = null;
    
                        /**
                         * CellChunk timestampMicros.
                         * @member {number|Long} timestampMicros
                         * @memberof google.bigtable.v2.ReadRowsResponse.CellChunk
                         * @instance
                         */
                        CellChunk.prototype.timestampMicros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                        /**
                         * CellChunk labels.
                         * @member {Array.<string>} labels
                         * @memberof google.bigtable.v2.ReadRowsResponse.CellChunk
                         * @instance
                         */
                        CellChunk.prototype.labels = $util.emptyArray;
    
                        /**
                         * CellChunk value.
                         * @member {Uint8Array} value
                         * @memberof google.bigtable.v2.ReadRowsResponse.CellChunk
                         * @instance
                         */
                        CellChunk.prototype.value = $util.newBuffer([]);
    
                        /**
                         * CellChunk valueSize.
                         * @member {number} valueSize
                         * @memberof google.bigtable.v2.ReadRowsResponse.CellChunk
                         * @instance
                         */
                        CellChunk.prototype.valueSize = 0;
    
                        /**
                         * CellChunk resetRow.
                         * @member {boolean|null|undefined} resetRow
                         * @memberof google.bigtable.v2.ReadRowsResponse.CellChunk
                         * @instance
                         */
                        CellChunk.prototype.resetRow = null;
    
                        /**
                         * CellChunk commitRow.
                         * @member {boolean|null|undefined} commitRow
                         * @memberof google.bigtable.v2.ReadRowsResponse.CellChunk
                         * @instance
                         */
                        CellChunk.prototype.commitRow = null;
    
                        // OneOf field names bound to virtual getters and setters
                        var $oneOfFields;
    
                        /**
                         * CellChunk rowStatus.
                         * @member {"resetRow"|"commitRow"|undefined} rowStatus
                         * @memberof google.bigtable.v2.ReadRowsResponse.CellChunk
                         * @instance
                         */
                        Object.defineProperty(CellChunk.prototype, "rowStatus", {
                            get: $util.oneOfGetter($oneOfFields = ["resetRow", "commitRow"]),
                            set: $util.oneOfSetter($oneOfFields)
                        });
    
                        /**
                         * Creates a new CellChunk instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.v2.ReadRowsResponse.CellChunk
                         * @static
                         * @param {google.bigtable.v2.ReadRowsResponse.ICellChunk=} [properties] Properties to set
                         * @returns {google.bigtable.v2.ReadRowsResponse.CellChunk} CellChunk instance
                         */
                        CellChunk.create = function create(properties) {
                            return new CellChunk(properties);
                        };
    
                        /**
                         * Encodes the specified CellChunk message. Does not implicitly {@link google.bigtable.v2.ReadRowsResponse.CellChunk.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.v2.ReadRowsResponse.CellChunk
                         * @static
                         * @param {google.bigtable.v2.ReadRowsResponse.ICellChunk} message CellChunk message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CellChunk.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.rowKey != null && Object.hasOwnProperty.call(message, "rowKey"))
                                writer.uint32(/* id 1, wireType 2 =*/10).bytes(message.rowKey);
                            if (message.familyName != null && Object.hasOwnProperty.call(message, "familyName"))
                                $root.google.protobuf.StringValue.encode(message.familyName, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            if (message.qualifier != null && Object.hasOwnProperty.call(message, "qualifier"))
                                $root.google.protobuf.BytesValue.encode(message.qualifier, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            if (message.timestampMicros != null && Object.hasOwnProperty.call(message, "timestampMicros"))
                                writer.uint32(/* id 4, wireType 0 =*/32).int64(message.timestampMicros);
                            if (message.labels != null && message.labels.length)
                                for (var i = 0; i < message.labels.length; ++i)
                                    writer.uint32(/* id 5, wireType 2 =*/42).string(message.labels[i]);
                            if (message.value != null && Object.hasOwnProperty.call(message, "value"))
                                writer.uint32(/* id 6, wireType 2 =*/50).bytes(message.value);
                            if (message.valueSize != null && Object.hasOwnProperty.call(message, "valueSize"))
                                writer.uint32(/* id 7, wireType 0 =*/56).int32(message.valueSize);
                            if (message.resetRow != null && Object.hasOwnProperty.call(message, "resetRow"))
                                writer.uint32(/* id 8, wireType 0 =*/64).bool(message.resetRow);
                            if (message.commitRow != null && Object.hasOwnProperty.call(message, "commitRow"))
                                writer.uint32(/* id 9, wireType 0 =*/72).bool(message.commitRow);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified CellChunk message, length delimited. Does not implicitly {@link google.bigtable.v2.ReadRowsResponse.CellChunk.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.v2.ReadRowsResponse.CellChunk
                         * @static
                         * @param {google.bigtable.v2.ReadRowsResponse.ICellChunk} message CellChunk message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CellChunk.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a CellChunk message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.v2.ReadRowsResponse.CellChunk
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.v2.ReadRowsResponse.CellChunk} CellChunk
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CellChunk.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.ReadRowsResponse.CellChunk();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.rowKey = reader.bytes();
                                        break;
                                    }
                                case 2: {
                                        message.familyName = $root.google.protobuf.StringValue.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 3: {
                                        message.qualifier = $root.google.protobuf.BytesValue.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 4: {
                                        message.timestampMicros = reader.int64();
                                        break;
                                    }
                                case 5: {
                                        if (!(message.labels && message.labels.length))
                                            message.labels = [];
                                        message.labels.push(reader.string());
                                        break;
                                    }
                                case 6: {
                                        message.value = reader.bytes();
                                        break;
                                    }
                                case 7: {
                                        message.valueSize = reader.int32();
                                        break;
                                    }
                                case 8: {
                                        message.resetRow = reader.bool();
                                        break;
                                    }
                                case 9: {
                                        message.commitRow = reader.bool();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a CellChunk message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.v2.ReadRowsResponse.CellChunk
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.v2.ReadRowsResponse.CellChunk} CellChunk
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CellChunk.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a CellChunk message.
                         * @function verify
                         * @memberof google.bigtable.v2.ReadRowsResponse.CellChunk
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        CellChunk.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            var properties = {};
                            if (message.rowKey != null && message.hasOwnProperty("rowKey"))
                                if (!(message.rowKey && typeof message.rowKey.length === "number" || $util.isString(message.rowKey)))
                                    return "rowKey: buffer expected";
                            if (message.familyName != null && message.hasOwnProperty("familyName")) {
                                var error = $root.google.protobuf.StringValue.verify(message.familyName);
                                if (error)
                                    return "familyName." + error;
                            }
                            if (message.qualifier != null && message.hasOwnProperty("qualifier")) {
                                var error = $root.google.protobuf.BytesValue.verify(message.qualifier);
                                if (error)
                                    return "qualifier." + error;
                            }
                            if (message.timestampMicros != null && message.hasOwnProperty("timestampMicros"))
                                if (!$util.isInteger(message.timestampMicros) && !(message.timestampMicros && $util.isInteger(message.timestampMicros.low) && $util.isInteger(message.timestampMicros.high)))
                                    return "timestampMicros: integer|Long expected";
                            if (message.labels != null && message.hasOwnProperty("labels")) {
                                if (!Array.isArray(message.labels))
                                    return "labels: array expected";
                                for (var i = 0; i < message.labels.length; ++i)
                                    if (!$util.isString(message.labels[i]))
                                        return "labels: string[] expected";
                            }
                            if (message.value != null && message.hasOwnProperty("value"))
                                if (!(message.value && typeof message.value.length === "number" || $util.isString(message.value)))
                                    return "value: buffer expected";
                            if (message.valueSize != null && message.hasOwnProperty("valueSize"))
                                if (!$util.isInteger(message.valueSize))
                                    return "valueSize: integer expected";
                            if (message.resetRow != null && message.hasOwnProperty("resetRow")) {
                                properties.rowStatus = 1;
                                if (typeof message.resetRow !== "boolean")
                                    return "resetRow: boolean expected";
                            }
                            if (message.commitRow != null && message.hasOwnProperty("commitRow")) {
                                if (properties.rowStatus === 1)
                                    return "rowStatus: multiple values";
                                properties.rowStatus = 1;
                                if (typeof message.commitRow !== "boolean")
                                    return "commitRow: boolean expected";
                            }
                            return null;
                        };
    
                        /**
                         * Creates a CellChunk message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.v2.ReadRowsResponse.CellChunk
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.v2.ReadRowsResponse.CellChunk} CellChunk
                         */
                        CellChunk.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.v2.ReadRowsResponse.CellChunk)
                                return object;
                            var message = new $root.google.bigtable.v2.ReadRowsResponse.CellChunk();
                            if (object.rowKey != null)
                                if (typeof object.rowKey === "string")
                                    $util.base64.decode(object.rowKey, message.rowKey = $util.newBuffer($util.base64.length(object.rowKey)), 0);
                                else if (object.rowKey.length >= 0)
                                    message.rowKey = object.rowKey;
                            if (object.familyName != null) {
                                if (typeof object.familyName !== "object")
                                    throw TypeError(".google.bigtable.v2.ReadRowsResponse.CellChunk.familyName: object expected");
                                message.familyName = $root.google.protobuf.StringValue.fromObject(object.familyName);
                            }
                            if (object.qualifier != null) {
                                if (typeof object.qualifier !== "object")
                                    throw TypeError(".google.bigtable.v2.ReadRowsResponse.CellChunk.qualifier: object expected");
                                message.qualifier = $root.google.protobuf.BytesValue.fromObject(object.qualifier);
                            }
                            if (object.timestampMicros != null)
                                if ($util.Long)
                                    (message.timestampMicros = $util.Long.fromValue(object.timestampMicros)).unsigned = false;
                                else if (typeof object.timestampMicros === "string")
                                    message.timestampMicros = parseInt(object.timestampMicros, 10);
                                else if (typeof object.timestampMicros === "number")
                                    message.timestampMicros = object.timestampMicros;
                                else if (typeof object.timestampMicros === "object")
                                    message.timestampMicros = new $util.LongBits(object.timestampMicros.low >>> 0, object.timestampMicros.high >>> 0).toNumber();
                            if (object.labels) {
                                if (!Array.isArray(object.labels))
                                    throw TypeError(".google.bigtable.v2.ReadRowsResponse.CellChunk.labels: array expected");
                                message.labels = [];
                                for (var i = 0; i < object.labels.length; ++i)
                                    message.labels[i] = String(object.labels[i]);
                            }
                            if (object.value != null)
                                if (typeof object.value === "string")
                                    $util.base64.decode(object.value, message.value = $util.newBuffer($util.base64.length(object.value)), 0);
                                else if (object.value.length >= 0)
                                    message.value = object.value;
                            if (object.valueSize != null)
                                message.valueSize = object.valueSize | 0;
                            if (object.resetRow != null)
                                message.resetRow = Boolean(object.resetRow);
                            if (object.commitRow != null)
                                message.commitRow = Boolean(object.commitRow);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a CellChunk message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.v2.ReadRowsResponse.CellChunk
                         * @static
                         * @param {google.bigtable.v2.ReadRowsResponse.CellChunk} message CellChunk
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        CellChunk.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.arrays || options.defaults)
                                object.labels = [];
                            if (options.defaults) {
                                if (options.bytes === String)
                                    object.rowKey = "";
                                else {
                                    object.rowKey = [];
                                    if (options.bytes !== Array)
                                        object.rowKey = $util.newBuffer(object.rowKey);
                                }
                                object.familyName = null;
                                object.qualifier = null;
                                if ($util.Long) {
                                    var long = new $util.Long(0, 0, false);
                                    object.timestampMicros = options.longs === String ? long.toString() : options.longs === Number ? long.toNumber() : long;
                                } else
                                    object.timestampMicros = options.longs === String ? "0" : 0;
                                if (options.bytes === String)
                                    object.value = "";
                                else {
                                    object.value = [];
                                    if (options.bytes !== Array)
                                        object.value = $util.newBuffer(object.value);
                                }
                                object.valueSize = 0;
                            }
                            if (message.rowKey != null && message.hasOwnProperty("rowKey"))
                                object.rowKey = options.bytes === String ? $util.base64.encode(message.rowKey, 0, message.rowKey.length) : options.bytes === Array ? Array.prototype.slice.call(message.rowKey) : message.rowKey;
                            if (message.familyName != null && message.hasOwnProperty("familyName"))
                                object.familyName = $root.google.protobuf.StringValue.toObject(message.familyName, options);
                            if (message.qualifier != null && message.hasOwnProperty("qualifier"))
                                object.qualifier = $root.google.protobuf.BytesValue.toObject(message.qualifier, options);
                            if (message.timestampMicros != null && message.hasOwnProperty("timestampMicros"))
                                if (typeof message.timestampMicros === "number")
                                    object.timestampMicros = options.longs === String ? String(message.timestampMicros) : message.timestampMicros;
                                else
                                    object.timestampMicros = options.longs === String ? $util.Long.prototype.toString.call(message.timestampMicros) : options.longs === Number ? new $util.LongBits(message.timestampMicros.low >>> 0, message.timestampMicros.high >>> 0).toNumber() : message.timestampMicros;
                            if (message.labels && message.labels.length) {
                                object.labels = [];
                                for (var j = 0; j < message.labels.length; ++j)
                                    object.labels[j] = message.labels[j];
                            }
                            if (message.value != null && message.hasOwnProperty("value"))
                                object.value = options.bytes === String ? $util.base64.encode(message.value, 0, message.value.length) : options.bytes === Array ? Array.prototype.slice.call(message.value) : message.value;
                            if (message.valueSize != null && message.hasOwnProperty("valueSize"))
                                object.valueSize = message.valueSize;
                            if (message.resetRow != null && message.hasOwnProperty("resetRow")) {
                                object.resetRow = message.resetRow;
                                if (options.oneofs)
                                    object.rowStatus = "resetRow";
                            }
                            if (message.commitRow != null && message.hasOwnProperty("commitRow")) {
                                object.commitRow = message.commitRow;
                                if (options.oneofs)
                                    object.rowStatus = "commitRow";
                            }
                            return object;
                        };
    
                        /**
                         * Converts this CellChunk to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.v2.ReadRowsResponse.CellChunk
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        CellChunk.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for CellChunk
                         * @function getTypeUrl
                         * @memberof google.bigtable.v2.ReadRowsResponse.CellChunk
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        CellChunk.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.v2.ReadRowsResponse.CellChunk";
                        };
    
                        return CellChunk;
                    })();
    
                    return ReadRowsResponse;
                })();
    
                v2.SampleRowKeysRequest = (function() {
    
                    /**
                     * Properties of a SampleRowKeysRequest.
                     * @memberof google.bigtable.v2
                     * @interface ISampleRowKeysRequest
                     * @property {string|null} [tableName] SampleRowKeysRequest tableName
                     * @property {string|null} [authorizedViewName] SampleRowKeysRequest authorizedViewName
                     * @property {string|null} [appProfileId] SampleRowKeysRequest appProfileId
                     */
    
                    /**
                     * Constructs a new SampleRowKeysRequest.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a SampleRowKeysRequest.
                     * @implements ISampleRowKeysRequest
                     * @constructor
                     * @param {google.bigtable.v2.ISampleRowKeysRequest=} [properties] Properties to set
                     */
                    function SampleRowKeysRequest(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * SampleRowKeysRequest tableName.
                     * @member {string} tableName
                     * @memberof google.bigtable.v2.SampleRowKeysRequest
                     * @instance
                     */
                    SampleRowKeysRequest.prototype.tableName = "";
    
                    /**
                     * SampleRowKeysRequest authorizedViewName.
                     * @member {string} authorizedViewName
                     * @memberof google.bigtable.v2.SampleRowKeysRequest
                     * @instance
                     */
                    SampleRowKeysRequest.prototype.authorizedViewName = "";
    
                    /**
                     * SampleRowKeysRequest appProfileId.
                     * @member {string} appProfileId
                     * @memberof google.bigtable.v2.SampleRowKeysRequest
                     * @instance
                     */
                    SampleRowKeysRequest.prototype.appProfileId = "";
    
                    /**
                     * Creates a new SampleRowKeysRequest instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.SampleRowKeysRequest
                     * @static
                     * @param {google.bigtable.v2.ISampleRowKeysRequest=} [properties] Properties to set
                     * @returns {google.bigtable.v2.SampleRowKeysRequest} SampleRowKeysRequest instance
                     */
                    SampleRowKeysRequest.create = function create(properties) {
                        return new SampleRowKeysRequest(properties);
                    };
    
                    /**
                     * Encodes the specified SampleRowKeysRequest message. Does not implicitly {@link google.bigtable.v2.SampleRowKeysRequest.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.SampleRowKeysRequest
                     * @static
                     * @param {google.bigtable.v2.ISampleRowKeysRequest} message SampleRowKeysRequest message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    SampleRowKeysRequest.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.tableName != null && Object.hasOwnProperty.call(message, "tableName"))
                            writer.uint32(/* id 1, wireType 2 =*/10).string(message.tableName);
                        if (message.appProfileId != null && Object.hasOwnProperty.call(message, "appProfileId"))
                            writer.uint32(/* id 2, wireType 2 =*/18).string(message.appProfileId);
                        if (message.authorizedViewName != null && Object.hasOwnProperty.call(message, "authorizedViewName"))
                            writer.uint32(/* id 4, wireType 2 =*/34).string(message.authorizedViewName);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified SampleRowKeysRequest message, length delimited. Does not implicitly {@link google.bigtable.v2.SampleRowKeysRequest.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.SampleRowKeysRequest
                     * @static
                     * @param {google.bigtable.v2.ISampleRowKeysRequest} message SampleRowKeysRequest message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    SampleRowKeysRequest.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a SampleRowKeysRequest message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.SampleRowKeysRequest
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.SampleRowKeysRequest} SampleRowKeysRequest
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    SampleRowKeysRequest.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.SampleRowKeysRequest();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.tableName = reader.string();
                                    break;
                                }
                            case 4: {
                                    message.authorizedViewName = reader.string();
                                    break;
                                }
                            case 2: {
                                    message.appProfileId = reader.string();
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a SampleRowKeysRequest message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.SampleRowKeysRequest
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.SampleRowKeysRequest} SampleRowKeysRequest
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    SampleRowKeysRequest.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a SampleRowKeysRequest message.
                     * @function verify
                     * @memberof google.bigtable.v2.SampleRowKeysRequest
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    SampleRowKeysRequest.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.tableName != null && message.hasOwnProperty("tableName"))
                            if (!$util.isString(message.tableName))
                                return "tableName: string expected";
                        if (message.authorizedViewName != null && message.hasOwnProperty("authorizedViewName"))
                            if (!$util.isString(message.authorizedViewName))
                                return "authorizedViewName: string expected";
                        if (message.appProfileId != null && message.hasOwnProperty("appProfileId"))
                            if (!$util.isString(message.appProfileId))
                                return "appProfileId: string expected";
                        return null;
                    };
    
                    /**
                     * Creates a SampleRowKeysRequest message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.SampleRowKeysRequest
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.SampleRowKeysRequest} SampleRowKeysRequest
                     */
                    SampleRowKeysRequest.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.SampleRowKeysRequest)
                            return object;
                        var message = new $root.google.bigtable.v2.SampleRowKeysRequest();
                        if (object.tableName != null)
                            message.tableName = String(object.tableName);
                        if (object.authorizedViewName != null)
                            message.authorizedViewName = String(object.authorizedViewName);
                        if (object.appProfileId != null)
                            message.appProfileId = String(object.appProfileId);
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a SampleRowKeysRequest message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.SampleRowKeysRequest
                     * @static
                     * @param {google.bigtable.v2.SampleRowKeysRequest} message SampleRowKeysRequest
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    SampleRowKeysRequest.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults) {
                            object.tableName = "";
                            object.appProfileId = "";
                            object.authorizedViewName = "";
                        }
                        if (message.tableName != null && message.hasOwnProperty("tableName"))
                            object.tableName = message.tableName;
                        if (message.appProfileId != null && message.hasOwnProperty("appProfileId"))
                            object.appProfileId = message.appProfileId;
                        if (message.authorizedViewName != null && message.hasOwnProperty("authorizedViewName"))
                            object.authorizedViewName = message.authorizedViewName;
                        return object;
                    };
    
                    /**
                     * Converts this SampleRowKeysRequest to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.SampleRowKeysRequest
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    SampleRowKeysRequest.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for SampleRowKeysRequest
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.SampleRowKeysRequest
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    SampleRowKeysRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.SampleRowKeysRequest";
                    };
    
                    return SampleRowKeysRequest;
                })();
    
                v2.SampleRowKeysResponse = (function() {
    
                    /**
                     * Properties of a SampleRowKeysResponse.
                     * @memberof google.bigtable.v2
                     * @interface ISampleRowKeysResponse
                     * @property {Uint8Array|null} [rowKey] SampleRowKeysResponse rowKey
                     * @property {number|Long|null} [offsetBytes] SampleRowKeysResponse offsetBytes
                     */
    
                    /**
                     * Constructs a new SampleRowKeysResponse.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a SampleRowKeysResponse.
                     * @implements ISampleRowKeysResponse
                     * @constructor
                     * @param {google.bigtable.v2.ISampleRowKeysResponse=} [properties] Properties to set
                     */
                    function SampleRowKeysResponse(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * SampleRowKeysResponse rowKey.
                     * @member {Uint8Array} rowKey
                     * @memberof google.bigtable.v2.SampleRowKeysResponse
                     * @instance
                     */
                    SampleRowKeysResponse.prototype.rowKey = $util.newBuffer([]);
    
                    /**
                     * SampleRowKeysResponse offsetBytes.
                     * @member {number|Long} offsetBytes
                     * @memberof google.bigtable.v2.SampleRowKeysResponse
                     * @instance
                     */
                    SampleRowKeysResponse.prototype.offsetBytes = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                    /**
                     * Creates a new SampleRowKeysResponse instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.SampleRowKeysResponse
                     * @static
                     * @param {google.bigtable.v2.ISampleRowKeysResponse=} [properties] Properties to set
                     * @returns {google.bigtable.v2.SampleRowKeysResponse} SampleRowKeysResponse instance
                     */
                    SampleRowKeysResponse.create = function create(properties) {
                        return new SampleRowKeysResponse(properties);
                    };
    
                    /**
                     * Encodes the specified SampleRowKeysResponse message. Does not implicitly {@link google.bigtable.v2.SampleRowKeysResponse.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.SampleRowKeysResponse
                     * @static
                     * @param {google.bigtable.v2.ISampleRowKeysResponse} message SampleRowKeysResponse message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    SampleRowKeysResponse.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.rowKey != null && Object.hasOwnProperty.call(message, "rowKey"))
                            writer.uint32(/* id 1, wireType 2 =*/10).bytes(message.rowKey);
                        if (message.offsetBytes != null && Object.hasOwnProperty.call(message, "offsetBytes"))
                            writer.uint32(/* id 2, wireType 0 =*/16).int64(message.offsetBytes);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified SampleRowKeysResponse message, length delimited. Does not implicitly {@link google.bigtable.v2.SampleRowKeysResponse.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.SampleRowKeysResponse
                     * @static
                     * @param {google.bigtable.v2.ISampleRowKeysResponse} message SampleRowKeysResponse message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    SampleRowKeysResponse.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a SampleRowKeysResponse message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.SampleRowKeysResponse
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.SampleRowKeysResponse} SampleRowKeysResponse
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    SampleRowKeysResponse.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.SampleRowKeysResponse();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.rowKey = reader.bytes();
                                    break;
                                }
                            case 2: {
                                    message.offsetBytes = reader.int64();
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a SampleRowKeysResponse message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.SampleRowKeysResponse
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.SampleRowKeysResponse} SampleRowKeysResponse
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    SampleRowKeysResponse.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a SampleRowKeysResponse message.
                     * @function verify
                     * @memberof google.bigtable.v2.SampleRowKeysResponse
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    SampleRowKeysResponse.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.rowKey != null && message.hasOwnProperty("rowKey"))
                            if (!(message.rowKey && typeof message.rowKey.length === "number" || $util.isString(message.rowKey)))
                                return "rowKey: buffer expected";
                        if (message.offsetBytes != null && message.hasOwnProperty("offsetBytes"))
                            if (!$util.isInteger(message.offsetBytes) && !(message.offsetBytes && $util.isInteger(message.offsetBytes.low) && $util.isInteger(message.offsetBytes.high)))
                                return "offsetBytes: integer|Long expected";
                        return null;
                    };
    
                    /**
                     * Creates a SampleRowKeysResponse message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.SampleRowKeysResponse
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.SampleRowKeysResponse} SampleRowKeysResponse
                     */
                    SampleRowKeysResponse.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.SampleRowKeysResponse)
                            return object;
                        var message = new $root.google.bigtable.v2.SampleRowKeysResponse();
                        if (object.rowKey != null)
                            if (typeof object.rowKey === "string")
                                $util.base64.decode(object.rowKey, message.rowKey = $util.newBuffer($util.base64.length(object.rowKey)), 0);
                            else if (object.rowKey.length >= 0)
                                message.rowKey = object.rowKey;
                        if (object.offsetBytes != null)
                            if ($util.Long)
                                (message.offsetBytes = $util.Long.fromValue(object.offsetBytes)).unsigned = false;
                            else if (typeof object.offsetBytes === "string")
                                message.offsetBytes = parseInt(object.offsetBytes, 10);
                            else if (typeof object.offsetBytes === "number")
                                message.offsetBytes = object.offsetBytes;
                            else if (typeof object.offsetBytes === "object")
                                message.offsetBytes = new $util.LongBits(object.offsetBytes.low >>> 0, object.offsetBytes.high >>> 0).toNumber();
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a SampleRowKeysResponse message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.SampleRowKeysResponse
                     * @static
                     * @param {google.bigtable.v2.SampleRowKeysResponse} message SampleRowKeysResponse
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    SampleRowKeysResponse.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults) {
                            if (options.bytes === String)
                                object.rowKey = "";
                            else {
                                object.rowKey = [];
                                if (options.bytes !== Array)
                                    object.rowKey = $util.newBuffer(object.rowKey);
                            }
                            if ($util.Long) {
                                var long = new $util.Long(0, 0, false);
                                object.offsetBytes = options.longs === String ? long.toString() : options.longs === Number ? long.toNumber() : long;
                            } else
                                object.offsetBytes = options.longs === String ? "0" : 0;
                        }
                        if (message.rowKey != null && message.hasOwnProperty("rowKey"))
                            object.rowKey = options.bytes === String ? $util.base64.encode(message.rowKey, 0, message.rowKey.length) : options.bytes === Array ? Array.prototype.slice.call(message.rowKey) : message.rowKey;
                        if (message.offsetBytes != null && message.hasOwnProperty("offsetBytes"))
                            if (typeof message.offsetBytes === "number")
                                object.offsetBytes = options.longs === String ? String(message.offsetBytes) : message.offsetBytes;
                            else
                                object.offsetBytes = options.longs === String ? $util.Long.prototype.toString.call(message.offsetBytes) : options.longs === Number ? new $util.LongBits(message.offsetBytes.low >>> 0, message.offsetBytes.high >>> 0).toNumber() : message.offsetBytes;
                        return object;
                    };
    
                    /**
                     * Converts this SampleRowKeysResponse to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.SampleRowKeysResponse
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    SampleRowKeysResponse.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for SampleRowKeysResponse
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.SampleRowKeysResponse
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    SampleRowKeysResponse.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.SampleRowKeysResponse";
                    };
    
                    return SampleRowKeysResponse;
                })();
    
                v2.MutateRowRequest = (function() {
    
                    /**
                     * Properties of a MutateRowRequest.
                     * @memberof google.bigtable.v2
                     * @interface IMutateRowRequest
                     * @property {string|null} [tableName] MutateRowRequest tableName
                     * @property {string|null} [authorizedViewName] MutateRowRequest authorizedViewName
                     * @property {string|null} [appProfileId] MutateRowRequest appProfileId
                     * @property {Uint8Array|null} [rowKey] MutateRowRequest rowKey
                     * @property {Array.<google.bigtable.v2.IMutation>|null} [mutations] MutateRowRequest mutations
                     */
    
                    /**
                     * Constructs a new MutateRowRequest.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a MutateRowRequest.
                     * @implements IMutateRowRequest
                     * @constructor
                     * @param {google.bigtable.v2.IMutateRowRequest=} [properties] Properties to set
                     */
                    function MutateRowRequest(properties) {
                        this.mutations = [];
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * MutateRowRequest tableName.
                     * @member {string} tableName
                     * @memberof google.bigtable.v2.MutateRowRequest
                     * @instance
                     */
                    MutateRowRequest.prototype.tableName = "";
    
                    /**
                     * MutateRowRequest authorizedViewName.
                     * @member {string} authorizedViewName
                     * @memberof google.bigtable.v2.MutateRowRequest
                     * @instance
                     */
                    MutateRowRequest.prototype.authorizedViewName = "";
    
                    /**
                     * MutateRowRequest appProfileId.
                     * @member {string} appProfileId
                     * @memberof google.bigtable.v2.MutateRowRequest
                     * @instance
                     */
                    MutateRowRequest.prototype.appProfileId = "";
    
                    /**
                     * MutateRowRequest rowKey.
                     * @member {Uint8Array} rowKey
                     * @memberof google.bigtable.v2.MutateRowRequest
                     * @instance
                     */
                    MutateRowRequest.prototype.rowKey = $util.newBuffer([]);
    
                    /**
                     * MutateRowRequest mutations.
                     * @member {Array.<google.bigtable.v2.IMutation>} mutations
                     * @memberof google.bigtable.v2.MutateRowRequest
                     * @instance
                     */
                    MutateRowRequest.prototype.mutations = $util.emptyArray;
    
                    /**
                     * Creates a new MutateRowRequest instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.MutateRowRequest
                     * @static
                     * @param {google.bigtable.v2.IMutateRowRequest=} [properties] Properties to set
                     * @returns {google.bigtable.v2.MutateRowRequest} MutateRowRequest instance
                     */
                    MutateRowRequest.create = function create(properties) {
                        return new MutateRowRequest(properties);
                    };
    
                    /**
                     * Encodes the specified MutateRowRequest message. Does not implicitly {@link google.bigtable.v2.MutateRowRequest.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.MutateRowRequest
                     * @static
                     * @param {google.bigtable.v2.IMutateRowRequest} message MutateRowRequest message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    MutateRowRequest.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.tableName != null && Object.hasOwnProperty.call(message, "tableName"))
                            writer.uint32(/* id 1, wireType 2 =*/10).string(message.tableName);
                        if (message.rowKey != null && Object.hasOwnProperty.call(message, "rowKey"))
                            writer.uint32(/* id 2, wireType 2 =*/18).bytes(message.rowKey);
                        if (message.mutations != null && message.mutations.length)
                            for (var i = 0; i < message.mutations.length; ++i)
                                $root.google.bigtable.v2.Mutation.encode(message.mutations[i], writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                        if (message.appProfileId != null && Object.hasOwnProperty.call(message, "appProfileId"))
                            writer.uint32(/* id 4, wireType 2 =*/34).string(message.appProfileId);
                        if (message.authorizedViewName != null && Object.hasOwnProperty.call(message, "authorizedViewName"))
                            writer.uint32(/* id 6, wireType 2 =*/50).string(message.authorizedViewName);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified MutateRowRequest message, length delimited. Does not implicitly {@link google.bigtable.v2.MutateRowRequest.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.MutateRowRequest
                     * @static
                     * @param {google.bigtable.v2.IMutateRowRequest} message MutateRowRequest message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    MutateRowRequest.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a MutateRowRequest message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.MutateRowRequest
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.MutateRowRequest} MutateRowRequest
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    MutateRowRequest.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.MutateRowRequest();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.tableName = reader.string();
                                    break;
                                }
                            case 6: {
                                    message.authorizedViewName = reader.string();
                                    break;
                                }
                            case 4: {
                                    message.appProfileId = reader.string();
                                    break;
                                }
                            case 2: {
                                    message.rowKey = reader.bytes();
                                    break;
                                }
                            case 3: {
                                    if (!(message.mutations && message.mutations.length))
                                        message.mutations = [];
                                    message.mutations.push($root.google.bigtable.v2.Mutation.decode(reader, reader.uint32()));
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a MutateRowRequest message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.MutateRowRequest
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.MutateRowRequest} MutateRowRequest
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    MutateRowRequest.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a MutateRowRequest message.
                     * @function verify
                     * @memberof google.bigtable.v2.MutateRowRequest
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    MutateRowRequest.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.tableName != null && message.hasOwnProperty("tableName"))
                            if (!$util.isString(message.tableName))
                                return "tableName: string expected";
                        if (message.authorizedViewName != null && message.hasOwnProperty("authorizedViewName"))
                            if (!$util.isString(message.authorizedViewName))
                                return "authorizedViewName: string expected";
                        if (message.appProfileId != null && message.hasOwnProperty("appProfileId"))
                            if (!$util.isString(message.appProfileId))
                                return "appProfileId: string expected";
                        if (message.rowKey != null && message.hasOwnProperty("rowKey"))
                            if (!(message.rowKey && typeof message.rowKey.length === "number" || $util.isString(message.rowKey)))
                                return "rowKey: buffer expected";
                        if (message.mutations != null && message.hasOwnProperty("mutations")) {
                            if (!Array.isArray(message.mutations))
                                return "mutations: array expected";
                            for (var i = 0; i < message.mutations.length; ++i) {
                                var error = $root.google.bigtable.v2.Mutation.verify(message.mutations[i]);
                                if (error)
                                    return "mutations." + error;
                            }
                        }
                        return null;
                    };
    
                    /**
                     * Creates a MutateRowRequest message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.MutateRowRequest
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.MutateRowRequest} MutateRowRequest
                     */
                    MutateRowRequest.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.MutateRowRequest)
                            return object;
                        var message = new $root.google.bigtable.v2.MutateRowRequest();
                        if (object.tableName != null)
                            message.tableName = String(object.tableName);
                        if (object.authorizedViewName != null)
                            message.authorizedViewName = String(object.authorizedViewName);
                        if (object.appProfileId != null)
                            message.appProfileId = String(object.appProfileId);
                        if (object.rowKey != null)
                            if (typeof object.rowKey === "string")
                                $util.base64.decode(object.rowKey, message.rowKey = $util.newBuffer($util.base64.length(object.rowKey)), 0);
                            else if (object.rowKey.length >= 0)
                                message.rowKey = object.rowKey;
                        if (object.mutations) {
                            if (!Array.isArray(object.mutations))
                                throw TypeError(".google.bigtable.v2.MutateRowRequest.mutations: array expected");
                            message.mutations = [];
                            for (var i = 0; i < object.mutations.length; ++i) {
                                if (typeof object.mutations[i] !== "object")
                                    throw TypeError(".google.bigtable.v2.MutateRowRequest.mutations: object expected");
                                message.mutations[i] = $root.google.bigtable.v2.Mutation.fromObject(object.mutations[i]);
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a MutateRowRequest message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.MutateRowRequest
                     * @static
                     * @param {google.bigtable.v2.MutateRowRequest} message MutateRowRequest
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    MutateRowRequest.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.arrays || options.defaults)
                            object.mutations = [];
                        if (options.defaults) {
                            object.tableName = "";
                            if (options.bytes === String)
                                object.rowKey = "";
                            else {
                                object.rowKey = [];
                                if (options.bytes !== Array)
                                    object.rowKey = $util.newBuffer(object.rowKey);
                            }
                            object.appProfileId = "";
                            object.authorizedViewName = "";
                        }
                        if (message.tableName != null && message.hasOwnProperty("tableName"))
                            object.tableName = message.tableName;
                        if (message.rowKey != null && message.hasOwnProperty("rowKey"))
                            object.rowKey = options.bytes === String ? $util.base64.encode(message.rowKey, 0, message.rowKey.length) : options.bytes === Array ? Array.prototype.slice.call(message.rowKey) : message.rowKey;
                        if (message.mutations && message.mutations.length) {
                            object.mutations = [];
                            for (var j = 0; j < message.mutations.length; ++j)
                                object.mutations[j] = $root.google.bigtable.v2.Mutation.toObject(message.mutations[j], options);
                        }
                        if (message.appProfileId != null && message.hasOwnProperty("appProfileId"))
                            object.appProfileId = message.appProfileId;
                        if (message.authorizedViewName != null && message.hasOwnProperty("authorizedViewName"))
                            object.authorizedViewName = message.authorizedViewName;
                        return object;
                    };
    
                    /**
                     * Converts this MutateRowRequest to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.MutateRowRequest
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    MutateRowRequest.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for MutateRowRequest
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.MutateRowRequest
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    MutateRowRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.MutateRowRequest";
                    };
    
                    return MutateRowRequest;
                })();
    
                v2.MutateRowResponse = (function() {
    
                    /**
                     * Properties of a MutateRowResponse.
                     * @memberof google.bigtable.v2
                     * @interface IMutateRowResponse
                     */
    
                    /**
                     * Constructs a new MutateRowResponse.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a MutateRowResponse.
                     * @implements IMutateRowResponse
                     * @constructor
                     * @param {google.bigtable.v2.IMutateRowResponse=} [properties] Properties to set
                     */
                    function MutateRowResponse(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * Creates a new MutateRowResponse instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.MutateRowResponse
                     * @static
                     * @param {google.bigtable.v2.IMutateRowResponse=} [properties] Properties to set
                     * @returns {google.bigtable.v2.MutateRowResponse} MutateRowResponse instance
                     */
                    MutateRowResponse.create = function create(properties) {
                        return new MutateRowResponse(properties);
                    };
    
                    /**
                     * Encodes the specified MutateRowResponse message. Does not implicitly {@link google.bigtable.v2.MutateRowResponse.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.MutateRowResponse
                     * @static
                     * @param {google.bigtable.v2.IMutateRowResponse} message MutateRowResponse message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    MutateRowResponse.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        return writer;
                    };
    
                    /**
                     * Encodes the specified MutateRowResponse message, length delimited. Does not implicitly {@link google.bigtable.v2.MutateRowResponse.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.MutateRowResponse
                     * @static
                     * @param {google.bigtable.v2.IMutateRowResponse} message MutateRowResponse message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    MutateRowResponse.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a MutateRowResponse message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.MutateRowResponse
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.MutateRowResponse} MutateRowResponse
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    MutateRowResponse.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.MutateRowResponse();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a MutateRowResponse message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.MutateRowResponse
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.MutateRowResponse} MutateRowResponse
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    MutateRowResponse.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a MutateRowResponse message.
                     * @function verify
                     * @memberof google.bigtable.v2.MutateRowResponse
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    MutateRowResponse.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        return null;
                    };
    
                    /**
                     * Creates a MutateRowResponse message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.MutateRowResponse
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.MutateRowResponse} MutateRowResponse
                     */
                    MutateRowResponse.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.MutateRowResponse)
                            return object;
                        return new $root.google.bigtable.v2.MutateRowResponse();
                    };
    
                    /**
                     * Creates a plain object from a MutateRowResponse message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.MutateRowResponse
                     * @static
                     * @param {google.bigtable.v2.MutateRowResponse} message MutateRowResponse
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    MutateRowResponse.toObject = function toObject() {
                        return {};
                    };
    
                    /**
                     * Converts this MutateRowResponse to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.MutateRowResponse
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    MutateRowResponse.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for MutateRowResponse
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.MutateRowResponse
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    MutateRowResponse.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.MutateRowResponse";
                    };
    
                    return MutateRowResponse;
                })();
    
                v2.MutateRowsRequest = (function() {
    
                    /**
                     * Properties of a MutateRowsRequest.
                     * @memberof google.bigtable.v2
                     * @interface IMutateRowsRequest
                     * @property {string|null} [tableName] MutateRowsRequest tableName
                     * @property {string|null} [authorizedViewName] MutateRowsRequest authorizedViewName
                     * @property {string|null} [appProfileId] MutateRowsRequest appProfileId
                     * @property {Array.<google.bigtable.v2.MutateRowsRequest.IEntry>|null} [entries] MutateRowsRequest entries
                     */
    
                    /**
                     * Constructs a new MutateRowsRequest.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a MutateRowsRequest.
                     * @implements IMutateRowsRequest
                     * @constructor
                     * @param {google.bigtable.v2.IMutateRowsRequest=} [properties] Properties to set
                     */
                    function MutateRowsRequest(properties) {
                        this.entries = [];
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * MutateRowsRequest tableName.
                     * @member {string} tableName
                     * @memberof google.bigtable.v2.MutateRowsRequest
                     * @instance
                     */
                    MutateRowsRequest.prototype.tableName = "";
    
                    /**
                     * MutateRowsRequest authorizedViewName.
                     * @member {string} authorizedViewName
                     * @memberof google.bigtable.v2.MutateRowsRequest
                     * @instance
                     */
                    MutateRowsRequest.prototype.authorizedViewName = "";
    
                    /**
                     * MutateRowsRequest appProfileId.
                     * @member {string} appProfileId
                     * @memberof google.bigtable.v2.MutateRowsRequest
                     * @instance
                     */
                    MutateRowsRequest.prototype.appProfileId = "";
    
                    /**
                     * MutateRowsRequest entries.
                     * @member {Array.<google.bigtable.v2.MutateRowsRequest.IEntry>} entries
                     * @memberof google.bigtable.v2.MutateRowsRequest
                     * @instance
                     */
                    MutateRowsRequest.prototype.entries = $util.emptyArray;
    
                    /**
                     * Creates a new MutateRowsRequest instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.MutateRowsRequest
                     * @static
                     * @param {google.bigtable.v2.IMutateRowsRequest=} [properties] Properties to set
                     * @returns {google.bigtable.v2.MutateRowsRequest} MutateRowsRequest instance
                     */
                    MutateRowsRequest.create = function create(properties) {
                        return new MutateRowsRequest(properties);
                    };
    
                    /**
                     * Encodes the specified MutateRowsRequest message. Does not implicitly {@link google.bigtable.v2.MutateRowsRequest.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.MutateRowsRequest
                     * @static
                     * @param {google.bigtable.v2.IMutateRowsRequest} message MutateRowsRequest message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    MutateRowsRequest.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.tableName != null && Object.hasOwnProperty.call(message, "tableName"))
                            writer.uint32(/* id 1, wireType 2 =*/10).string(message.tableName);
                        if (message.entries != null && message.entries.length)
                            for (var i = 0; i < message.entries.length; ++i)
                                $root.google.bigtable.v2.MutateRowsRequest.Entry.encode(message.entries[i], writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                        if (message.appProfileId != null && Object.hasOwnProperty.call(message, "appProfileId"))
                            writer.uint32(/* id 3, wireType 2 =*/26).string(message.appProfileId);
                        if (message.authorizedViewName != null && Object.hasOwnProperty.call(message, "authorizedViewName"))
                            writer.uint32(/* id 5, wireType 2 =*/42).string(message.authorizedViewName);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified MutateRowsRequest message, length delimited. Does not implicitly {@link google.bigtable.v2.MutateRowsRequest.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.MutateRowsRequest
                     * @static
                     * @param {google.bigtable.v2.IMutateRowsRequest} message MutateRowsRequest message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    MutateRowsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a MutateRowsRequest message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.MutateRowsRequest
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.MutateRowsRequest} MutateRowsRequest
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    MutateRowsRequest.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.MutateRowsRequest();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.tableName = reader.string();
                                    break;
                                }
                            case 5: {
                                    message.authorizedViewName = reader.string();
                                    break;
                                }
                            case 3: {
                                    message.appProfileId = reader.string();
                                    break;
                                }
                            case 2: {
                                    if (!(message.entries && message.entries.length))
                                        message.entries = [];
                                    message.entries.push($root.google.bigtable.v2.MutateRowsRequest.Entry.decode(reader, reader.uint32()));
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a MutateRowsRequest message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.MutateRowsRequest
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.MutateRowsRequest} MutateRowsRequest
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    MutateRowsRequest.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a MutateRowsRequest message.
                     * @function verify
                     * @memberof google.bigtable.v2.MutateRowsRequest
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    MutateRowsRequest.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.tableName != null && message.hasOwnProperty("tableName"))
                            if (!$util.isString(message.tableName))
                                return "tableName: string expected";
                        if (message.authorizedViewName != null && message.hasOwnProperty("authorizedViewName"))
                            if (!$util.isString(message.authorizedViewName))
                                return "authorizedViewName: string expected";
                        if (message.appProfileId != null && message.hasOwnProperty("appProfileId"))
                            if (!$util.isString(message.appProfileId))
                                return "appProfileId: string expected";
                        if (message.entries != null && message.hasOwnProperty("entries")) {
                            if (!Array.isArray(message.entries))
                                return "entries: array expected";
                            for (var i = 0; i < message.entries.length; ++i) {
                                var error = $root.google.bigtable.v2.MutateRowsRequest.Entry.verify(message.entries[i]);
                                if (error)
                                    return "entries." + error;
                            }
                        }
                        return null;
                    };
    
                    /**
                     * Creates a MutateRowsRequest message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.MutateRowsRequest
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.MutateRowsRequest} MutateRowsRequest
                     */
                    MutateRowsRequest.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.MutateRowsRequest)
                            return object;
                        var message = new $root.google.bigtable.v2.MutateRowsRequest();
                        if (object.tableName != null)
                            message.tableName = String(object.tableName);
                        if (object.authorizedViewName != null)
                            message.authorizedViewName = String(object.authorizedViewName);
                        if (object.appProfileId != null)
                            message.appProfileId = String(object.appProfileId);
                        if (object.entries) {
                            if (!Array.isArray(object.entries))
                                throw TypeError(".google.bigtable.v2.MutateRowsRequest.entries: array expected");
                            message.entries = [];
                            for (var i = 0; i < object.entries.length; ++i) {
                                if (typeof object.entries[i] !== "object")
                                    throw TypeError(".google.bigtable.v2.MutateRowsRequest.entries: object expected");
                                message.entries[i] = $root.google.bigtable.v2.MutateRowsRequest.Entry.fromObject(object.entries[i]);
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a MutateRowsRequest message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.MutateRowsRequest
                     * @static
                     * @param {google.bigtable.v2.MutateRowsRequest} message MutateRowsRequest
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    MutateRowsRequest.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.arrays || options.defaults)
                            object.entries = [];
                        if (options.defaults) {
                            object.tableName = "";
                            object.appProfileId = "";
                            object.authorizedViewName = "";
                        }
                        if (message.tableName != null && message.hasOwnProperty("tableName"))
                            object.tableName = message.tableName;
                        if (message.entries && message.entries.length) {
                            object.entries = [];
                            for (var j = 0; j < message.entries.length; ++j)
                                object.entries[j] = $root.google.bigtable.v2.MutateRowsRequest.Entry.toObject(message.entries[j], options);
                        }
                        if (message.appProfileId != null && message.hasOwnProperty("appProfileId"))
                            object.appProfileId = message.appProfileId;
                        if (message.authorizedViewName != null && message.hasOwnProperty("authorizedViewName"))
                            object.authorizedViewName = message.authorizedViewName;
                        return object;
                    };
    
                    /**
                     * Converts this MutateRowsRequest to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.MutateRowsRequest
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    MutateRowsRequest.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for MutateRowsRequest
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.MutateRowsRequest
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    MutateRowsRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.MutateRowsRequest";
                    };
    
                    MutateRowsRequest.Entry = (function() {
    
                        /**
                         * Properties of an Entry.
                         * @memberof google.bigtable.v2.MutateRowsRequest
                         * @interface IEntry
                         * @property {Uint8Array|null} [rowKey] Entry rowKey
                         * @property {Array.<google.bigtable.v2.IMutation>|null} [mutations] Entry mutations
                         */
    
                        /**
                         * Constructs a new Entry.
                         * @memberof google.bigtable.v2.MutateRowsRequest
                         * @classdesc Represents an Entry.
                         * @implements IEntry
                         * @constructor
                         * @param {google.bigtable.v2.MutateRowsRequest.IEntry=} [properties] Properties to set
                         */
                        function Entry(properties) {
                            this.mutations = [];
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * Entry rowKey.
                         * @member {Uint8Array} rowKey
                         * @memberof google.bigtable.v2.MutateRowsRequest.Entry
                         * @instance
                         */
                        Entry.prototype.rowKey = $util.newBuffer([]);
    
                        /**
                         * Entry mutations.
                         * @member {Array.<google.bigtable.v2.IMutation>} mutations
                         * @memberof google.bigtable.v2.MutateRowsRequest.Entry
                         * @instance
                         */
                        Entry.prototype.mutations = $util.emptyArray;
    
                        /**
                         * Creates a new Entry instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.v2.MutateRowsRequest.Entry
                         * @static
                         * @param {google.bigtable.v2.MutateRowsRequest.IEntry=} [properties] Properties to set
                         * @returns {google.bigtable.v2.MutateRowsRequest.Entry} Entry instance
                         */
                        Entry.create = function create(properties) {
                            return new Entry(properties);
                        };
    
                        /**
                         * Encodes the specified Entry message. Does not implicitly {@link google.bigtable.v2.MutateRowsRequest.Entry.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.v2.MutateRowsRequest.Entry
                         * @static
                         * @param {google.bigtable.v2.MutateRowsRequest.IEntry} message Entry message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        Entry.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.rowKey != null && Object.hasOwnProperty.call(message, "rowKey"))
                                writer.uint32(/* id 1, wireType 2 =*/10).bytes(message.rowKey);
                            if (message.mutations != null && message.mutations.length)
                                for (var i = 0; i < message.mutations.length; ++i)
                                    $root.google.bigtable.v2.Mutation.encode(message.mutations[i], writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified Entry message, length delimited. Does not implicitly {@link google.bigtable.v2.MutateRowsRequest.Entry.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.v2.MutateRowsRequest.Entry
                         * @static
                         * @param {google.bigtable.v2.MutateRowsRequest.IEntry} message Entry message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        Entry.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes an Entry message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.v2.MutateRowsRequest.Entry
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.v2.MutateRowsRequest.Entry} Entry
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        Entry.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.MutateRowsRequest.Entry();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.rowKey = reader.bytes();
                                        break;
                                    }
                                case 2: {
                                        if (!(message.mutations && message.mutations.length))
                                            message.mutations = [];
                                        message.mutations.push($root.google.bigtable.v2.Mutation.decode(reader, reader.uint32()));
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes an Entry message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.v2.MutateRowsRequest.Entry
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.v2.MutateRowsRequest.Entry} Entry
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        Entry.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies an Entry message.
                         * @function verify
                         * @memberof google.bigtable.v2.MutateRowsRequest.Entry
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        Entry.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.rowKey != null && message.hasOwnProperty("rowKey"))
                                if (!(message.rowKey && typeof message.rowKey.length === "number" || $util.isString(message.rowKey)))
                                    return "rowKey: buffer expected";
                            if (message.mutations != null && message.hasOwnProperty("mutations")) {
                                if (!Array.isArray(message.mutations))
                                    return "mutations: array expected";
                                for (var i = 0; i < message.mutations.length; ++i) {
                                    var error = $root.google.bigtable.v2.Mutation.verify(message.mutations[i]);
                                    if (error)
                                        return "mutations." + error;
                                }
                            }
                            return null;
                        };
    
                        /**
                         * Creates an Entry message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.v2.MutateRowsRequest.Entry
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.v2.MutateRowsRequest.Entry} Entry
                         */
                        Entry.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.v2.MutateRowsRequest.Entry)
                                return object;
                            var message = new $root.google.bigtable.v2.MutateRowsRequest.Entry();
                            if (object.rowKey != null)
                                if (typeof object.rowKey === "string")
                                    $util.base64.decode(object.rowKey, message.rowKey = $util.newBuffer($util.base64.length(object.rowKey)), 0);
                                else if (object.rowKey.length >= 0)
                                    message.rowKey = object.rowKey;
                            if (object.mutations) {
                                if (!Array.isArray(object.mutations))
                                    throw TypeError(".google.bigtable.v2.MutateRowsRequest.Entry.mutations: array expected");
                                message.mutations = [];
                                for (var i = 0; i < object.mutations.length; ++i) {
                                    if (typeof object.mutations[i] !== "object")
                                        throw TypeError(".google.bigtable.v2.MutateRowsRequest.Entry.mutations: object expected");
                                    message.mutations[i] = $root.google.bigtable.v2.Mutation.fromObject(object.mutations[i]);
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from an Entry message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.v2.MutateRowsRequest.Entry
                         * @static
                         * @param {google.bigtable.v2.MutateRowsRequest.Entry} message Entry
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        Entry.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.arrays || options.defaults)
                                object.mutations = [];
                            if (options.defaults)
                                if (options.bytes === String)
                                    object.rowKey = "";
                                else {
                                    object.rowKey = [];
                                    if (options.bytes !== Array)
                                        object.rowKey = $util.newBuffer(object.rowKey);
                                }
                            if (message.rowKey != null && message.hasOwnProperty("rowKey"))
                                object.rowKey = options.bytes === String ? $util.base64.encode(message.rowKey, 0, message.rowKey.length) : options.bytes === Array ? Array.prototype.slice.call(message.rowKey) : message.rowKey;
                            if (message.mutations && message.mutations.length) {
                                object.mutations = [];
                                for (var j = 0; j < message.mutations.length; ++j)
                                    object.mutations[j] = $root.google.bigtable.v2.Mutation.toObject(message.mutations[j], options);
                            }
                            return object;
                        };
    
                        /**
                         * Converts this Entry to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.v2.MutateRowsRequest.Entry
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        Entry.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for Entry
                         * @function getTypeUrl
                         * @memberof google.bigtable.v2.MutateRowsRequest.Entry
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        Entry.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.v2.MutateRowsRequest.Entry";
                        };
    
                        return Entry;
                    })();
    
                    return MutateRowsRequest;
                })();
    
                v2.MutateRowsResponse = (function() {
    
                    /**
                     * Properties of a MutateRowsResponse.
                     * @memberof google.bigtable.v2
                     * @interface IMutateRowsResponse
                     * @property {Array.<google.bigtable.v2.MutateRowsResponse.IEntry>|null} [entries] MutateRowsResponse entries
                     * @property {google.bigtable.v2.IRateLimitInfo|null} [rateLimitInfo] MutateRowsResponse rateLimitInfo
                     */
    
                    /**
                     * Constructs a new MutateRowsResponse.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a MutateRowsResponse.
                     * @implements IMutateRowsResponse
                     * @constructor
                     * @param {google.bigtable.v2.IMutateRowsResponse=} [properties] Properties to set
                     */
                    function MutateRowsResponse(properties) {
                        this.entries = [];
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * MutateRowsResponse entries.
                     * @member {Array.<google.bigtable.v2.MutateRowsResponse.IEntry>} entries
                     * @memberof google.bigtable.v2.MutateRowsResponse
                     * @instance
                     */
                    MutateRowsResponse.prototype.entries = $util.emptyArray;
    
                    /**
                     * MutateRowsResponse rateLimitInfo.
                     * @member {google.bigtable.v2.IRateLimitInfo|null|undefined} rateLimitInfo
                     * @memberof google.bigtable.v2.MutateRowsResponse
                     * @instance
                     */
                    MutateRowsResponse.prototype.rateLimitInfo = null;
    
                    // OneOf field names bound to virtual getters and setters
                    var $oneOfFields;
    
                    /**
                     * MutateRowsResponse _rateLimitInfo.
                     * @member {"rateLimitInfo"|undefined} _rateLimitInfo
                     * @memberof google.bigtable.v2.MutateRowsResponse
                     * @instance
                     */
                    Object.defineProperty(MutateRowsResponse.prototype, "_rateLimitInfo", {
                        get: $util.oneOfGetter($oneOfFields = ["rateLimitInfo"]),
                        set: $util.oneOfSetter($oneOfFields)
                    });
    
                    /**
                     * Creates a new MutateRowsResponse instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.MutateRowsResponse
                     * @static
                     * @param {google.bigtable.v2.IMutateRowsResponse=} [properties] Properties to set
                     * @returns {google.bigtable.v2.MutateRowsResponse} MutateRowsResponse instance
                     */
                    MutateRowsResponse.create = function create(properties) {
                        return new MutateRowsResponse(properties);
                    };
    
                    /**
                     * Encodes the specified MutateRowsResponse message. Does not implicitly {@link google.bigtable.v2.MutateRowsResponse.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.MutateRowsResponse
                     * @static
                     * @param {google.bigtable.v2.IMutateRowsResponse} message MutateRowsResponse message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    MutateRowsResponse.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.entries != null && message.entries.length)
                            for (var i = 0; i < message.entries.length; ++i)
                                $root.google.bigtable.v2.MutateRowsResponse.Entry.encode(message.entries[i], writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                        if (message.rateLimitInfo != null && Object.hasOwnProperty.call(message, "rateLimitInfo"))
                            $root.google.bigtable.v2.RateLimitInfo.encode(message.rateLimitInfo, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                        return writer;
                    };
    
                    /**
                     * Encodes the specified MutateRowsResponse message, length delimited. Does not implicitly {@link google.bigtable.v2.MutateRowsResponse.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.MutateRowsResponse
                     * @static
                     * @param {google.bigtable.v2.IMutateRowsResponse} message MutateRowsResponse message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    MutateRowsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a MutateRowsResponse message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.MutateRowsResponse
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.MutateRowsResponse} MutateRowsResponse
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    MutateRowsResponse.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.MutateRowsResponse();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    if (!(message.entries && message.entries.length))
                                        message.entries = [];
                                    message.entries.push($root.google.bigtable.v2.MutateRowsResponse.Entry.decode(reader, reader.uint32()));
                                    break;
                                }
                            case 3: {
                                    message.rateLimitInfo = $root.google.bigtable.v2.RateLimitInfo.decode(reader, reader.uint32());
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a MutateRowsResponse message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.MutateRowsResponse
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.MutateRowsResponse} MutateRowsResponse
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    MutateRowsResponse.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a MutateRowsResponse message.
                     * @function verify
                     * @memberof google.bigtable.v2.MutateRowsResponse
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    MutateRowsResponse.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        var properties = {};
                        if (message.entries != null && message.hasOwnProperty("entries")) {
                            if (!Array.isArray(message.entries))
                                return "entries: array expected";
                            for (var i = 0; i < message.entries.length; ++i) {
                                var error = $root.google.bigtable.v2.MutateRowsResponse.Entry.verify(message.entries[i]);
                                if (error)
                                    return "entries." + error;
                            }
                        }
                        if (message.rateLimitInfo != null && message.hasOwnProperty("rateLimitInfo")) {
                            properties._rateLimitInfo = 1;
                            {
                                var error = $root.google.bigtable.v2.RateLimitInfo.verify(message.rateLimitInfo);
                                if (error)
                                    return "rateLimitInfo." + error;
                            }
                        }
                        return null;
                    };
    
                    /**
                     * Creates a MutateRowsResponse message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.MutateRowsResponse
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.MutateRowsResponse} MutateRowsResponse
                     */
                    MutateRowsResponse.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.MutateRowsResponse)
                            return object;
                        var message = new $root.google.bigtable.v2.MutateRowsResponse();
                        if (object.entries) {
                            if (!Array.isArray(object.entries))
                                throw TypeError(".google.bigtable.v2.MutateRowsResponse.entries: array expected");
                            message.entries = [];
                            for (var i = 0; i < object.entries.length; ++i) {
                                if (typeof object.entries[i] !== "object")
                                    throw TypeError(".google.bigtable.v2.MutateRowsResponse.entries: object expected");
                                message.entries[i] = $root.google.bigtable.v2.MutateRowsResponse.Entry.fromObject(object.entries[i]);
                            }
                        }
                        if (object.rateLimitInfo != null) {
                            if (typeof object.rateLimitInfo !== "object")
                                throw TypeError(".google.bigtable.v2.MutateRowsResponse.rateLimitInfo: object expected");
                            message.rateLimitInfo = $root.google.bigtable.v2.RateLimitInfo.fromObject(object.rateLimitInfo);
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a MutateRowsResponse message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.MutateRowsResponse
                     * @static
                     * @param {google.bigtable.v2.MutateRowsResponse} message MutateRowsResponse
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    MutateRowsResponse.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.arrays || options.defaults)
                            object.entries = [];
                        if (message.entries && message.entries.length) {
                            object.entries = [];
                            for (var j = 0; j < message.entries.length; ++j)
                                object.entries[j] = $root.google.bigtable.v2.MutateRowsResponse.Entry.toObject(message.entries[j], options);
                        }
                        if (message.rateLimitInfo != null && message.hasOwnProperty("rateLimitInfo")) {
                            object.rateLimitInfo = $root.google.bigtable.v2.RateLimitInfo.toObject(message.rateLimitInfo, options);
                            if (options.oneofs)
                                object._rateLimitInfo = "rateLimitInfo";
                        }
                        return object;
                    };
    
                    /**
                     * Converts this MutateRowsResponse to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.MutateRowsResponse
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    MutateRowsResponse.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for MutateRowsResponse
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.MutateRowsResponse
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    MutateRowsResponse.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.MutateRowsResponse";
                    };
    
                    MutateRowsResponse.Entry = (function() {
    
                        /**
                         * Properties of an Entry.
                         * @memberof google.bigtable.v2.MutateRowsResponse
                         * @interface IEntry
                         * @property {number|Long|null} [index] Entry index
                         * @property {google.rpc.IStatus|null} [status] Entry status
                         */
    
                        /**
                         * Constructs a new Entry.
                         * @memberof google.bigtable.v2.MutateRowsResponse
                         * @classdesc Represents an Entry.
                         * @implements IEntry
                         * @constructor
                         * @param {google.bigtable.v2.MutateRowsResponse.IEntry=} [properties] Properties to set
                         */
                        function Entry(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * Entry index.
                         * @member {number|Long} index
                         * @memberof google.bigtable.v2.MutateRowsResponse.Entry
                         * @instance
                         */
                        Entry.prototype.index = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                        /**
                         * Entry status.
                         * @member {google.rpc.IStatus|null|undefined} status
                         * @memberof google.bigtable.v2.MutateRowsResponse.Entry
                         * @instance
                         */
                        Entry.prototype.status = null;
    
                        /**
                         * Creates a new Entry instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.v2.MutateRowsResponse.Entry
                         * @static
                         * @param {google.bigtable.v2.MutateRowsResponse.IEntry=} [properties] Properties to set
                         * @returns {google.bigtable.v2.MutateRowsResponse.Entry} Entry instance
                         */
                        Entry.create = function create(properties) {
                            return new Entry(properties);
                        };
    
                        /**
                         * Encodes the specified Entry message. Does not implicitly {@link google.bigtable.v2.MutateRowsResponse.Entry.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.v2.MutateRowsResponse.Entry
                         * @static
                         * @param {google.bigtable.v2.MutateRowsResponse.IEntry} message Entry message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        Entry.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.index != null && Object.hasOwnProperty.call(message, "index"))
                                writer.uint32(/* id 1, wireType 0 =*/8).int64(message.index);
                            if (message.status != null && Object.hasOwnProperty.call(message, "status"))
                                $root.google.rpc.Status.encode(message.status, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified Entry message, length delimited. Does not implicitly {@link google.bigtable.v2.MutateRowsResponse.Entry.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.v2.MutateRowsResponse.Entry
                         * @static
                         * @param {google.bigtable.v2.MutateRowsResponse.IEntry} message Entry message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        Entry.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes an Entry message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.v2.MutateRowsResponse.Entry
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.v2.MutateRowsResponse.Entry} Entry
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        Entry.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.MutateRowsResponse.Entry();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.index = reader.int64();
                                        break;
                                    }
                                case 2: {
                                        message.status = $root.google.rpc.Status.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes an Entry message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.v2.MutateRowsResponse.Entry
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.v2.MutateRowsResponse.Entry} Entry
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        Entry.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies an Entry message.
                         * @function verify
                         * @memberof google.bigtable.v2.MutateRowsResponse.Entry
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        Entry.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.index != null && message.hasOwnProperty("index"))
                                if (!$util.isInteger(message.index) && !(message.index && $util.isInteger(message.index.low) && $util.isInteger(message.index.high)))
                                    return "index: integer|Long expected";
                            if (message.status != null && message.hasOwnProperty("status")) {
                                var error = $root.google.rpc.Status.verify(message.status);
                                if (error)
                                    return "status." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates an Entry message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.v2.MutateRowsResponse.Entry
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.v2.MutateRowsResponse.Entry} Entry
                         */
                        Entry.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.v2.MutateRowsResponse.Entry)
                                return object;
                            var message = new $root.google.bigtable.v2.MutateRowsResponse.Entry();
                            if (object.index != null)
                                if ($util.Long)
                                    (message.index = $util.Long.fromValue(object.index)).unsigned = false;
                                else if (typeof object.index === "string")
                                    message.index = parseInt(object.index, 10);
                                else if (typeof object.index === "number")
                                    message.index = object.index;
                                else if (typeof object.index === "object")
                                    message.index = new $util.LongBits(object.index.low >>> 0, object.index.high >>> 0).toNumber();
                            if (object.status != null) {
                                if (typeof object.status !== "object")
                                    throw TypeError(".google.bigtable.v2.MutateRowsResponse.Entry.status: object expected");
                                message.status = $root.google.rpc.Status.fromObject(object.status);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from an Entry message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.v2.MutateRowsResponse.Entry
                         * @static
                         * @param {google.bigtable.v2.MutateRowsResponse.Entry} message Entry
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        Entry.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                if ($util.Long) {
                                    var long = new $util.Long(0, 0, false);
                                    object.index = options.longs === String ? long.toString() : options.longs === Number ? long.toNumber() : long;
                                } else
                                    object.index = options.longs === String ? "0" : 0;
                                object.status = null;
                            }
                            if (message.index != null && message.hasOwnProperty("index"))
                                if (typeof message.index === "number")
                                    object.index = options.longs === String ? String(message.index) : message.index;
                                else
                                    object.index = options.longs === String ? $util.Long.prototype.toString.call(message.index) : options.longs === Number ? new $util.LongBits(message.index.low >>> 0, message.index.high >>> 0).toNumber() : message.index;
                            if (message.status != null && message.hasOwnProperty("status"))
                                object.status = $root.google.rpc.Status.toObject(message.status, options);
                            return object;
                        };
    
                        /**
                         * Converts this Entry to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.v2.MutateRowsResponse.Entry
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        Entry.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for Entry
                         * @function getTypeUrl
                         * @memberof google.bigtable.v2.MutateRowsResponse.Entry
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        Entry.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.v2.MutateRowsResponse.Entry";
                        };
    
                        return Entry;
                    })();
    
                    return MutateRowsResponse;
                })();
    
                v2.RateLimitInfo = (function() {
    
                    /**
                     * Properties of a RateLimitInfo.
                     * @memberof google.bigtable.v2
                     * @interface IRateLimitInfo
                     * @property {google.protobuf.IDuration|null} [period] RateLimitInfo period
                     * @property {number|null} [factor] RateLimitInfo factor
                     */
    
                    /**
                     * Constructs a new RateLimitInfo.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a RateLimitInfo.
                     * @implements IRateLimitInfo
                     * @constructor
                     * @param {google.bigtable.v2.IRateLimitInfo=} [properties] Properties to set
                     */
                    function RateLimitInfo(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * RateLimitInfo period.
                     * @member {google.protobuf.IDuration|null|undefined} period
                     * @memberof google.bigtable.v2.RateLimitInfo
                     * @instance
                     */
                    RateLimitInfo.prototype.period = null;
    
                    /**
                     * RateLimitInfo factor.
                     * @member {number} factor
                     * @memberof google.bigtable.v2.RateLimitInfo
                     * @instance
                     */
                    RateLimitInfo.prototype.factor = 0;
    
                    /**
                     * Creates a new RateLimitInfo instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.RateLimitInfo
                     * @static
                     * @param {google.bigtable.v2.IRateLimitInfo=} [properties] Properties to set
                     * @returns {google.bigtable.v2.RateLimitInfo} RateLimitInfo instance
                     */
                    RateLimitInfo.create = function create(properties) {
                        return new RateLimitInfo(properties);
                    };
    
                    /**
                     * Encodes the specified RateLimitInfo message. Does not implicitly {@link google.bigtable.v2.RateLimitInfo.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.RateLimitInfo
                     * @static
                     * @param {google.bigtable.v2.IRateLimitInfo} message RateLimitInfo message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    RateLimitInfo.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.period != null && Object.hasOwnProperty.call(message, "period"))
                            $root.google.protobuf.Duration.encode(message.period, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                        if (message.factor != null && Object.hasOwnProperty.call(message, "factor"))
                            writer.uint32(/* id 2, wireType 1 =*/17).double(message.factor);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified RateLimitInfo message, length delimited. Does not implicitly {@link google.bigtable.v2.RateLimitInfo.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.RateLimitInfo
                     * @static
                     * @param {google.bigtable.v2.IRateLimitInfo} message RateLimitInfo message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    RateLimitInfo.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a RateLimitInfo message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.RateLimitInfo
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.RateLimitInfo} RateLimitInfo
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    RateLimitInfo.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.RateLimitInfo();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.period = $root.google.protobuf.Duration.decode(reader, reader.uint32());
                                    break;
                                }
                            case 2: {
                                    message.factor = reader.double();
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a RateLimitInfo message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.RateLimitInfo
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.RateLimitInfo} RateLimitInfo
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    RateLimitInfo.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a RateLimitInfo message.
                     * @function verify
                     * @memberof google.bigtable.v2.RateLimitInfo
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    RateLimitInfo.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.period != null && message.hasOwnProperty("period")) {
                            var error = $root.google.protobuf.Duration.verify(message.period);
                            if (error)
                                return "period." + error;
                        }
                        if (message.factor != null && message.hasOwnProperty("factor"))
                            if (typeof message.factor !== "number")
                                return "factor: number expected";
                        return null;
                    };
    
                    /**
                     * Creates a RateLimitInfo message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.RateLimitInfo
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.RateLimitInfo} RateLimitInfo
                     */
                    RateLimitInfo.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.RateLimitInfo)
                            return object;
                        var message = new $root.google.bigtable.v2.RateLimitInfo();
                        if (object.period != null) {
                            if (typeof object.period !== "object")
                                throw TypeError(".google.bigtable.v2.RateLimitInfo.period: object expected");
                            message.period = $root.google.protobuf.Duration.fromObject(object.period);
                        }
                        if (object.factor != null)
                            message.factor = Number(object.factor);
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a RateLimitInfo message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.RateLimitInfo
                     * @static
                     * @param {google.bigtable.v2.RateLimitInfo} message RateLimitInfo
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    RateLimitInfo.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults) {
                            object.period = null;
                            object.factor = 0;
                        }
                        if (message.period != null && message.hasOwnProperty("period"))
                            object.period = $root.google.protobuf.Duration.toObject(message.period, options);
                        if (message.factor != null && message.hasOwnProperty("factor"))
                            object.factor = options.json && !isFinite(message.factor) ? String(message.factor) : message.factor;
                        return object;
                    };
    
                    /**
                     * Converts this RateLimitInfo to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.RateLimitInfo
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    RateLimitInfo.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for RateLimitInfo
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.RateLimitInfo
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    RateLimitInfo.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.RateLimitInfo";
                    };
    
                    return RateLimitInfo;
                })();
    
                v2.CheckAndMutateRowRequest = (function() {
    
                    /**
                     * Properties of a CheckAndMutateRowRequest.
                     * @memberof google.bigtable.v2
                     * @interface ICheckAndMutateRowRequest
                     * @property {string|null} [tableName] CheckAndMutateRowRequest tableName
                     * @property {string|null} [authorizedViewName] CheckAndMutateRowRequest authorizedViewName
                     * @property {string|null} [appProfileId] CheckAndMutateRowRequest appProfileId
                     * @property {Uint8Array|null} [rowKey] CheckAndMutateRowRequest rowKey
                     * @property {google.bigtable.v2.IRowFilter|null} [predicateFilter] CheckAndMutateRowRequest predicateFilter
                     * @property {Array.<google.bigtable.v2.IMutation>|null} [trueMutations] CheckAndMutateRowRequest trueMutations
                     * @property {Array.<google.bigtable.v2.IMutation>|null} [falseMutations] CheckAndMutateRowRequest falseMutations
                     */
    
                    /**
                     * Constructs a new CheckAndMutateRowRequest.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a CheckAndMutateRowRequest.
                     * @implements ICheckAndMutateRowRequest
                     * @constructor
                     * @param {google.bigtable.v2.ICheckAndMutateRowRequest=} [properties] Properties to set
                     */
                    function CheckAndMutateRowRequest(properties) {
                        this.trueMutations = [];
                        this.falseMutations = [];
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * CheckAndMutateRowRequest tableName.
                     * @member {string} tableName
                     * @memberof google.bigtable.v2.CheckAndMutateRowRequest
                     * @instance
                     */
                    CheckAndMutateRowRequest.prototype.tableName = "";
    
                    /**
                     * CheckAndMutateRowRequest authorizedViewName.
                     * @member {string} authorizedViewName
                     * @memberof google.bigtable.v2.CheckAndMutateRowRequest
                     * @instance
                     */
                    CheckAndMutateRowRequest.prototype.authorizedViewName = "";
    
                    /**
                     * CheckAndMutateRowRequest appProfileId.
                     * @member {string} appProfileId
                     * @memberof google.bigtable.v2.CheckAndMutateRowRequest
                     * @instance
                     */
                    CheckAndMutateRowRequest.prototype.appProfileId = "";
    
                    /**
                     * CheckAndMutateRowRequest rowKey.
                     * @member {Uint8Array} rowKey
                     * @memberof google.bigtable.v2.CheckAndMutateRowRequest
                     * @instance
                     */
                    CheckAndMutateRowRequest.prototype.rowKey = $util.newBuffer([]);
    
                    /**
                     * CheckAndMutateRowRequest predicateFilter.
                     * @member {google.bigtable.v2.IRowFilter|null|undefined} predicateFilter
                     * @memberof google.bigtable.v2.CheckAndMutateRowRequest
                     * @instance
                     */
                    CheckAndMutateRowRequest.prototype.predicateFilter = null;
    
                    /**
                     * CheckAndMutateRowRequest trueMutations.
                     * @member {Array.<google.bigtable.v2.IMutation>} trueMutations
                     * @memberof google.bigtable.v2.CheckAndMutateRowRequest
                     * @instance
                     */
                    CheckAndMutateRowRequest.prototype.trueMutations = $util.emptyArray;
    
                    /**
                     * CheckAndMutateRowRequest falseMutations.
                     * @member {Array.<google.bigtable.v2.IMutation>} falseMutations
                     * @memberof google.bigtable.v2.CheckAndMutateRowRequest
                     * @instance
                     */
                    CheckAndMutateRowRequest.prototype.falseMutations = $util.emptyArray;
    
                    /**
                     * Creates a new CheckAndMutateRowRequest instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.CheckAndMutateRowRequest
                     * @static
                     * @param {google.bigtable.v2.ICheckAndMutateRowRequest=} [properties] Properties to set
                     * @returns {google.bigtable.v2.CheckAndMutateRowRequest} CheckAndMutateRowRequest instance
                     */
                    CheckAndMutateRowRequest.create = function create(properties) {
                        return new CheckAndMutateRowRequest(properties);
                    };
    
                    /**
                     * Encodes the specified CheckAndMutateRowRequest message. Does not implicitly {@link google.bigtable.v2.CheckAndMutateRowRequest.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.CheckAndMutateRowRequest
                     * @static
                     * @param {google.bigtable.v2.ICheckAndMutateRowRequest} message CheckAndMutateRowRequest message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    CheckAndMutateRowRequest.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.tableName != null && Object.hasOwnProperty.call(message, "tableName"))
                            writer.uint32(/* id 1, wireType 2 =*/10).string(message.tableName);
                        if (message.rowKey != null && Object.hasOwnProperty.call(message, "rowKey"))
                            writer.uint32(/* id 2, wireType 2 =*/18).bytes(message.rowKey);
                        if (message.trueMutations != null && message.trueMutations.length)
                            for (var i = 0; i < message.trueMutations.length; ++i)
                                $root.google.bigtable.v2.Mutation.encode(message.trueMutations[i], writer.uint32(/* id 4, wireType 2 =*/34).fork()).ldelim();
                        if (message.falseMutations != null && message.falseMutations.length)
                            for (var i = 0; i < message.falseMutations.length; ++i)
                                $root.google.bigtable.v2.Mutation.encode(message.falseMutations[i], writer.uint32(/* id 5, wireType 2 =*/42).fork()).ldelim();
                        if (message.predicateFilter != null && Object.hasOwnProperty.call(message, "predicateFilter"))
                            $root.google.bigtable.v2.RowFilter.encode(message.predicateFilter, writer.uint32(/* id 6, wireType 2 =*/50).fork()).ldelim();
                        if (message.appProfileId != null && Object.hasOwnProperty.call(message, "appProfileId"))
                            writer.uint32(/* id 7, wireType 2 =*/58).string(message.appProfileId);
                        if (message.authorizedViewName != null && Object.hasOwnProperty.call(message, "authorizedViewName"))
                            writer.uint32(/* id 9, wireType 2 =*/74).string(message.authorizedViewName);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified CheckAndMutateRowRequest message, length delimited. Does not implicitly {@link google.bigtable.v2.CheckAndMutateRowRequest.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.CheckAndMutateRowRequest
                     * @static
                     * @param {google.bigtable.v2.ICheckAndMutateRowRequest} message CheckAndMutateRowRequest message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    CheckAndMutateRowRequest.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a CheckAndMutateRowRequest message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.CheckAndMutateRowRequest
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.CheckAndMutateRowRequest} CheckAndMutateRowRequest
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    CheckAndMutateRowRequest.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.CheckAndMutateRowRequest();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.tableName = reader.string();
                                    break;
                                }
                            case 9: {
                                    message.authorizedViewName = reader.string();
                                    break;
                                }
                            case 7: {
                                    message.appProfileId = reader.string();
                                    break;
                                }
                            case 2: {
                                    message.rowKey = reader.bytes();
                                    break;
                                }
                            case 6: {
                                    message.predicateFilter = $root.google.bigtable.v2.RowFilter.decode(reader, reader.uint32());
                                    break;
                                }
                            case 4: {
                                    if (!(message.trueMutations && message.trueMutations.length))
                                        message.trueMutations = [];
                                    message.trueMutations.push($root.google.bigtable.v2.Mutation.decode(reader, reader.uint32()));
                                    break;
                                }
                            case 5: {
                                    if (!(message.falseMutations && message.falseMutations.length))
                                        message.falseMutations = [];
                                    message.falseMutations.push($root.google.bigtable.v2.Mutation.decode(reader, reader.uint32()));
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a CheckAndMutateRowRequest message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.CheckAndMutateRowRequest
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.CheckAndMutateRowRequest} CheckAndMutateRowRequest
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    CheckAndMutateRowRequest.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a CheckAndMutateRowRequest message.
                     * @function verify
                     * @memberof google.bigtable.v2.CheckAndMutateRowRequest
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    CheckAndMutateRowRequest.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.tableName != null && message.hasOwnProperty("tableName"))
                            if (!$util.isString(message.tableName))
                                return "tableName: string expected";
                        if (message.authorizedViewName != null && message.hasOwnProperty("authorizedViewName"))
                            if (!$util.isString(message.authorizedViewName))
                                return "authorizedViewName: string expected";
                        if (message.appProfileId != null && message.hasOwnProperty("appProfileId"))
                            if (!$util.isString(message.appProfileId))
                                return "appProfileId: string expected";
                        if (message.rowKey != null && message.hasOwnProperty("rowKey"))
                            if (!(message.rowKey && typeof message.rowKey.length === "number" || $util.isString(message.rowKey)))
                                return "rowKey: buffer expected";
                        if (message.predicateFilter != null && message.hasOwnProperty("predicateFilter")) {
                            var error = $root.google.bigtable.v2.RowFilter.verify(message.predicateFilter);
                            if (error)
                                return "predicateFilter." + error;
                        }
                        if (message.trueMutations != null && message.hasOwnProperty("trueMutations")) {
                            if (!Array.isArray(message.trueMutations))
                                return "trueMutations: array expected";
                            for (var i = 0; i < message.trueMutations.length; ++i) {
                                var error = $root.google.bigtable.v2.Mutation.verify(message.trueMutations[i]);
                                if (error)
                                    return "trueMutations." + error;
                            }
                        }
                        if (message.falseMutations != null && message.hasOwnProperty("falseMutations")) {
                            if (!Array.isArray(message.falseMutations))
                                return "falseMutations: array expected";
                            for (var i = 0; i < message.falseMutations.length; ++i) {
                                var error = $root.google.bigtable.v2.Mutation.verify(message.falseMutations[i]);
                                if (error)
                                    return "falseMutations." + error;
                            }
                        }
                        return null;
                    };
    
                    /**
                     * Creates a CheckAndMutateRowRequest message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.CheckAndMutateRowRequest
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.CheckAndMutateRowRequest} CheckAndMutateRowRequest
                     */
                    CheckAndMutateRowRequest.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.CheckAndMutateRowRequest)
                            return object;
                        var message = new $root.google.bigtable.v2.CheckAndMutateRowRequest();
                        if (object.tableName != null)
                            message.tableName = String(object.tableName);
                        if (object.authorizedViewName != null)
                            message.authorizedViewName = String(object.authorizedViewName);
                        if (object.appProfileId != null)
                            message.appProfileId = String(object.appProfileId);
                        if (object.rowKey != null)
                            if (typeof object.rowKey === "string")
                                $util.base64.decode(object.rowKey, message.rowKey = $util.newBuffer($util.base64.length(object.rowKey)), 0);
                            else if (object.rowKey.length >= 0)
                                message.rowKey = object.rowKey;
                        if (object.predicateFilter != null) {
                            if (typeof object.predicateFilter !== "object")
                                throw TypeError(".google.bigtable.v2.CheckAndMutateRowRequest.predicateFilter: object expected");
                            message.predicateFilter = $root.google.bigtable.v2.RowFilter.fromObject(object.predicateFilter);
                        }
                        if (object.trueMutations) {
                            if (!Array.isArray(object.trueMutations))
                                throw TypeError(".google.bigtable.v2.CheckAndMutateRowRequest.trueMutations: array expected");
                            message.trueMutations = [];
                            for (var i = 0; i < object.trueMutations.length; ++i) {
                                if (typeof object.trueMutations[i] !== "object")
                                    throw TypeError(".google.bigtable.v2.CheckAndMutateRowRequest.trueMutations: object expected");
                                message.trueMutations[i] = $root.google.bigtable.v2.Mutation.fromObject(object.trueMutations[i]);
                            }
                        }
                        if (object.falseMutations) {
                            if (!Array.isArray(object.falseMutations))
                                throw TypeError(".google.bigtable.v2.CheckAndMutateRowRequest.falseMutations: array expected");
                            message.falseMutations = [];
                            for (var i = 0; i < object.falseMutations.length; ++i) {
                                if (typeof object.falseMutations[i] !== "object")
                                    throw TypeError(".google.bigtable.v2.CheckAndMutateRowRequest.falseMutations: object expected");
                                message.falseMutations[i] = $root.google.bigtable.v2.Mutation.fromObject(object.falseMutations[i]);
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a CheckAndMutateRowRequest message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.CheckAndMutateRowRequest
                     * @static
                     * @param {google.bigtable.v2.CheckAndMutateRowRequest} message CheckAndMutateRowRequest
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    CheckAndMutateRowRequest.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.arrays || options.defaults) {
                            object.trueMutations = [];
                            object.falseMutations = [];
                        }
                        if (options.defaults) {
                            object.tableName = "";
                            if (options.bytes === String)
                                object.rowKey = "";
                            else {
                                object.rowKey = [];
                                if (options.bytes !== Array)
                                    object.rowKey = $util.newBuffer(object.rowKey);
                            }
                            object.predicateFilter = null;
                            object.appProfileId = "";
                            object.authorizedViewName = "";
                        }
                        if (message.tableName != null && message.hasOwnProperty("tableName"))
                            object.tableName = message.tableName;
                        if (message.rowKey != null && message.hasOwnProperty("rowKey"))
                            object.rowKey = options.bytes === String ? $util.base64.encode(message.rowKey, 0, message.rowKey.length) : options.bytes === Array ? Array.prototype.slice.call(message.rowKey) : message.rowKey;
                        if (message.trueMutations && message.trueMutations.length) {
                            object.trueMutations = [];
                            for (var j = 0; j < message.trueMutations.length; ++j)
                                object.trueMutations[j] = $root.google.bigtable.v2.Mutation.toObject(message.trueMutations[j], options);
                        }
                        if (message.falseMutations && message.falseMutations.length) {
                            object.falseMutations = [];
                            for (var j = 0; j < message.falseMutations.length; ++j)
                                object.falseMutations[j] = $root.google.bigtable.v2.Mutation.toObject(message.falseMutations[j], options);
                        }
                        if (message.predicateFilter != null && message.hasOwnProperty("predicateFilter"))
                            object.predicateFilter = $root.google.bigtable.v2.RowFilter.toObject(message.predicateFilter, options);
                        if (message.appProfileId != null && message.hasOwnProperty("appProfileId"))
                            object.appProfileId = message.appProfileId;
                        if (message.authorizedViewName != null && message.hasOwnProperty("authorizedViewName"))
                            object.authorizedViewName = message.authorizedViewName;
                        return object;
                    };
    
                    /**
                     * Converts this CheckAndMutateRowRequest to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.CheckAndMutateRowRequest
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    CheckAndMutateRowRequest.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for CheckAndMutateRowRequest
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.CheckAndMutateRowRequest
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    CheckAndMutateRowRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.CheckAndMutateRowRequest";
                    };
    
                    return CheckAndMutateRowRequest;
                })();
    
                v2.CheckAndMutateRowResponse = (function() {
    
                    /**
                     * Properties of a CheckAndMutateRowResponse.
                     * @memberof google.bigtable.v2
                     * @interface ICheckAndMutateRowResponse
                     * @property {boolean|null} [predicateMatched] CheckAndMutateRowResponse predicateMatched
                     */
    
                    /**
                     * Constructs a new CheckAndMutateRowResponse.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a CheckAndMutateRowResponse.
                     * @implements ICheckAndMutateRowResponse
                     * @constructor
                     * @param {google.bigtable.v2.ICheckAndMutateRowResponse=} [properties] Properties to set
                     */
                    function CheckAndMutateRowResponse(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * CheckAndMutateRowResponse predicateMatched.
                     * @member {boolean} predicateMatched
                     * @memberof google.bigtable.v2.CheckAndMutateRowResponse
                     * @instance
                     */
                    CheckAndMutateRowResponse.prototype.predicateMatched = false;
    
                    /**
                     * Creates a new CheckAndMutateRowResponse instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.CheckAndMutateRowResponse
                     * @static
                     * @param {google.bigtable.v2.ICheckAndMutateRowResponse=} [properties] Properties to set
                     * @returns {google.bigtable.v2.CheckAndMutateRowResponse} CheckAndMutateRowResponse instance
                     */
                    CheckAndMutateRowResponse.create = function create(properties) {
                        return new CheckAndMutateRowResponse(properties);
                    };
    
                    /**
                     * Encodes the specified CheckAndMutateRowResponse message. Does not implicitly {@link google.bigtable.v2.CheckAndMutateRowResponse.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.CheckAndMutateRowResponse
                     * @static
                     * @param {google.bigtable.v2.ICheckAndMutateRowResponse} message CheckAndMutateRowResponse message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    CheckAndMutateRowResponse.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.predicateMatched != null && Object.hasOwnProperty.call(message, "predicateMatched"))
                            writer.uint32(/* id 1, wireType 0 =*/8).bool(message.predicateMatched);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified CheckAndMutateRowResponse message, length delimited. Does not implicitly {@link google.bigtable.v2.CheckAndMutateRowResponse.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.CheckAndMutateRowResponse
                     * @static
                     * @param {google.bigtable.v2.ICheckAndMutateRowResponse} message CheckAndMutateRowResponse message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    CheckAndMutateRowResponse.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a CheckAndMutateRowResponse message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.CheckAndMutateRowResponse
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.CheckAndMutateRowResponse} CheckAndMutateRowResponse
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    CheckAndMutateRowResponse.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.CheckAndMutateRowResponse();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.predicateMatched = reader.bool();
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a CheckAndMutateRowResponse message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.CheckAndMutateRowResponse
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.CheckAndMutateRowResponse} CheckAndMutateRowResponse
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    CheckAndMutateRowResponse.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a CheckAndMutateRowResponse message.
                     * @function verify
                     * @memberof google.bigtable.v2.CheckAndMutateRowResponse
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    CheckAndMutateRowResponse.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.predicateMatched != null && message.hasOwnProperty("predicateMatched"))
                            if (typeof message.predicateMatched !== "boolean")
                                return "predicateMatched: boolean expected";
                        return null;
                    };
    
                    /**
                     * Creates a CheckAndMutateRowResponse message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.CheckAndMutateRowResponse
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.CheckAndMutateRowResponse} CheckAndMutateRowResponse
                     */
                    CheckAndMutateRowResponse.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.CheckAndMutateRowResponse)
                            return object;
                        var message = new $root.google.bigtable.v2.CheckAndMutateRowResponse();
                        if (object.predicateMatched != null)
                            message.predicateMatched = Boolean(object.predicateMatched);
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a CheckAndMutateRowResponse message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.CheckAndMutateRowResponse
                     * @static
                     * @param {google.bigtable.v2.CheckAndMutateRowResponse} message CheckAndMutateRowResponse
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    CheckAndMutateRowResponse.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults)
                            object.predicateMatched = false;
                        if (message.predicateMatched != null && message.hasOwnProperty("predicateMatched"))
                            object.predicateMatched = message.predicateMatched;
                        return object;
                    };
    
                    /**
                     * Converts this CheckAndMutateRowResponse to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.CheckAndMutateRowResponse
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    CheckAndMutateRowResponse.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for CheckAndMutateRowResponse
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.CheckAndMutateRowResponse
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    CheckAndMutateRowResponse.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.CheckAndMutateRowResponse";
                    };
    
                    return CheckAndMutateRowResponse;
                })();
    
                v2.PingAndWarmRequest = (function() {
    
                    /**
                     * Properties of a PingAndWarmRequest.
                     * @memberof google.bigtable.v2
                     * @interface IPingAndWarmRequest
                     * @property {string|null} [name] PingAndWarmRequest name
                     * @property {string|null} [appProfileId] PingAndWarmRequest appProfileId
                     */
    
                    /**
                     * Constructs a new PingAndWarmRequest.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a PingAndWarmRequest.
                     * @implements IPingAndWarmRequest
                     * @constructor
                     * @param {google.bigtable.v2.IPingAndWarmRequest=} [properties] Properties to set
                     */
                    function PingAndWarmRequest(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * PingAndWarmRequest name.
                     * @member {string} name
                     * @memberof google.bigtable.v2.PingAndWarmRequest
                     * @instance
                     */
                    PingAndWarmRequest.prototype.name = "";
    
                    /**
                     * PingAndWarmRequest appProfileId.
                     * @member {string} appProfileId
                     * @memberof google.bigtable.v2.PingAndWarmRequest
                     * @instance
                     */
                    PingAndWarmRequest.prototype.appProfileId = "";
    
                    /**
                     * Creates a new PingAndWarmRequest instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.PingAndWarmRequest
                     * @static
                     * @param {google.bigtable.v2.IPingAndWarmRequest=} [properties] Properties to set
                     * @returns {google.bigtable.v2.PingAndWarmRequest} PingAndWarmRequest instance
                     */
                    PingAndWarmRequest.create = function create(properties) {
                        return new PingAndWarmRequest(properties);
                    };
    
                    /**
                     * Encodes the specified PingAndWarmRequest message. Does not implicitly {@link google.bigtable.v2.PingAndWarmRequest.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.PingAndWarmRequest
                     * @static
                     * @param {google.bigtable.v2.IPingAndWarmRequest} message PingAndWarmRequest message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    PingAndWarmRequest.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                            writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                        if (message.appProfileId != null && Object.hasOwnProperty.call(message, "appProfileId"))
                            writer.uint32(/* id 2, wireType 2 =*/18).string(message.appProfileId);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified PingAndWarmRequest message, length delimited. Does not implicitly {@link google.bigtable.v2.PingAndWarmRequest.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.PingAndWarmRequest
                     * @static
                     * @param {google.bigtable.v2.IPingAndWarmRequest} message PingAndWarmRequest message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    PingAndWarmRequest.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a PingAndWarmRequest message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.PingAndWarmRequest
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.PingAndWarmRequest} PingAndWarmRequest
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    PingAndWarmRequest.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.PingAndWarmRequest();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.name = reader.string();
                                    break;
                                }
                            case 2: {
                                    message.appProfileId = reader.string();
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a PingAndWarmRequest message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.PingAndWarmRequest
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.PingAndWarmRequest} PingAndWarmRequest
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    PingAndWarmRequest.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a PingAndWarmRequest message.
                     * @function verify
                     * @memberof google.bigtable.v2.PingAndWarmRequest
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    PingAndWarmRequest.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.name != null && message.hasOwnProperty("name"))
                            if (!$util.isString(message.name))
                                return "name: string expected";
                        if (message.appProfileId != null && message.hasOwnProperty("appProfileId"))
                            if (!$util.isString(message.appProfileId))
                                return "appProfileId: string expected";
                        return null;
                    };
    
                    /**
                     * Creates a PingAndWarmRequest message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.PingAndWarmRequest
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.PingAndWarmRequest} PingAndWarmRequest
                     */
                    PingAndWarmRequest.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.PingAndWarmRequest)
                            return object;
                        var message = new $root.google.bigtable.v2.PingAndWarmRequest();
                        if (object.name != null)
                            message.name = String(object.name);
                        if (object.appProfileId != null)
                            message.appProfileId = String(object.appProfileId);
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a PingAndWarmRequest message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.PingAndWarmRequest
                     * @static
                     * @param {google.bigtable.v2.PingAndWarmRequest} message PingAndWarmRequest
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    PingAndWarmRequest.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults) {
                            object.name = "";
                            object.appProfileId = "";
                        }
                        if (message.name != null && message.hasOwnProperty("name"))
                            object.name = message.name;
                        if (message.appProfileId != null && message.hasOwnProperty("appProfileId"))
                            object.appProfileId = message.appProfileId;
                        return object;
                    };
    
                    /**
                     * Converts this PingAndWarmRequest to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.PingAndWarmRequest
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    PingAndWarmRequest.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for PingAndWarmRequest
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.PingAndWarmRequest
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    PingAndWarmRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.PingAndWarmRequest";
                    };
    
                    return PingAndWarmRequest;
                })();
    
                v2.PingAndWarmResponse = (function() {
    
                    /**
                     * Properties of a PingAndWarmResponse.
                     * @memberof google.bigtable.v2
                     * @interface IPingAndWarmResponse
                     */
    
                    /**
                     * Constructs a new PingAndWarmResponse.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a PingAndWarmResponse.
                     * @implements IPingAndWarmResponse
                     * @constructor
                     * @param {google.bigtable.v2.IPingAndWarmResponse=} [properties] Properties to set
                     */
                    function PingAndWarmResponse(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * Creates a new PingAndWarmResponse instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.PingAndWarmResponse
                     * @static
                     * @param {google.bigtable.v2.IPingAndWarmResponse=} [properties] Properties to set
                     * @returns {google.bigtable.v2.PingAndWarmResponse} PingAndWarmResponse instance
                     */
                    PingAndWarmResponse.create = function create(properties) {
                        return new PingAndWarmResponse(properties);
                    };
    
                    /**
                     * Encodes the specified PingAndWarmResponse message. Does not implicitly {@link google.bigtable.v2.PingAndWarmResponse.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.PingAndWarmResponse
                     * @static
                     * @param {google.bigtable.v2.IPingAndWarmResponse} message PingAndWarmResponse message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    PingAndWarmResponse.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        return writer;
                    };
    
                    /**
                     * Encodes the specified PingAndWarmResponse message, length delimited. Does not implicitly {@link google.bigtable.v2.PingAndWarmResponse.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.PingAndWarmResponse
                     * @static
                     * @param {google.bigtable.v2.IPingAndWarmResponse} message PingAndWarmResponse message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    PingAndWarmResponse.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a PingAndWarmResponse message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.PingAndWarmResponse
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.PingAndWarmResponse} PingAndWarmResponse
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    PingAndWarmResponse.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.PingAndWarmResponse();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a PingAndWarmResponse message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.PingAndWarmResponse
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.PingAndWarmResponse} PingAndWarmResponse
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    PingAndWarmResponse.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a PingAndWarmResponse message.
                     * @function verify
                     * @memberof google.bigtable.v2.PingAndWarmResponse
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    PingAndWarmResponse.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        return null;
                    };
    
                    /**
                     * Creates a PingAndWarmResponse message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.PingAndWarmResponse
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.PingAndWarmResponse} PingAndWarmResponse
                     */
                    PingAndWarmResponse.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.PingAndWarmResponse)
                            return object;
                        return new $root.google.bigtable.v2.PingAndWarmResponse();
                    };
    
                    /**
                     * Creates a plain object from a PingAndWarmResponse message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.PingAndWarmResponse
                     * @static
                     * @param {google.bigtable.v2.PingAndWarmResponse} message PingAndWarmResponse
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    PingAndWarmResponse.toObject = function toObject() {
                        return {};
                    };
    
                    /**
                     * Converts this PingAndWarmResponse to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.PingAndWarmResponse
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    PingAndWarmResponse.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for PingAndWarmResponse
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.PingAndWarmResponse
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    PingAndWarmResponse.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.PingAndWarmResponse";
                    };
    
                    return PingAndWarmResponse;
                })();
    
                v2.ReadModifyWriteRowRequest = (function() {
    
                    /**
                     * Properties of a ReadModifyWriteRowRequest.
                     * @memberof google.bigtable.v2
                     * @interface IReadModifyWriteRowRequest
                     * @property {string|null} [tableName] ReadModifyWriteRowRequest tableName
                     * @property {string|null} [authorizedViewName] ReadModifyWriteRowRequest authorizedViewName
                     * @property {string|null} [appProfileId] ReadModifyWriteRowRequest appProfileId
                     * @property {Uint8Array|null} [rowKey] ReadModifyWriteRowRequest rowKey
                     * @property {Array.<google.bigtable.v2.IReadModifyWriteRule>|null} [rules] ReadModifyWriteRowRequest rules
                     */
    
                    /**
                     * Constructs a new ReadModifyWriteRowRequest.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a ReadModifyWriteRowRequest.
                     * @implements IReadModifyWriteRowRequest
                     * @constructor
                     * @param {google.bigtable.v2.IReadModifyWriteRowRequest=} [properties] Properties to set
                     */
                    function ReadModifyWriteRowRequest(properties) {
                        this.rules = [];
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * ReadModifyWriteRowRequest tableName.
                     * @member {string} tableName
                     * @memberof google.bigtable.v2.ReadModifyWriteRowRequest
                     * @instance
                     */
                    ReadModifyWriteRowRequest.prototype.tableName = "";
    
                    /**
                     * ReadModifyWriteRowRequest authorizedViewName.
                     * @member {string} authorizedViewName
                     * @memberof google.bigtable.v2.ReadModifyWriteRowRequest
                     * @instance
                     */
                    ReadModifyWriteRowRequest.prototype.authorizedViewName = "";
    
                    /**
                     * ReadModifyWriteRowRequest appProfileId.
                     * @member {string} appProfileId
                     * @memberof google.bigtable.v2.ReadModifyWriteRowRequest
                     * @instance
                     */
                    ReadModifyWriteRowRequest.prototype.appProfileId = "";
    
                    /**
                     * ReadModifyWriteRowRequest rowKey.
                     * @member {Uint8Array} rowKey
                     * @memberof google.bigtable.v2.ReadModifyWriteRowRequest
                     * @instance
                     */
                    ReadModifyWriteRowRequest.prototype.rowKey = $util.newBuffer([]);
    
                    /**
                     * ReadModifyWriteRowRequest rules.
                     * @member {Array.<google.bigtable.v2.IReadModifyWriteRule>} rules
                     * @memberof google.bigtable.v2.ReadModifyWriteRowRequest
                     * @instance
                     */
                    ReadModifyWriteRowRequest.prototype.rules = $util.emptyArray;
    
                    /**
                     * Creates a new ReadModifyWriteRowRequest instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.ReadModifyWriteRowRequest
                     * @static
                     * @param {google.bigtable.v2.IReadModifyWriteRowRequest=} [properties] Properties to set
                     * @returns {google.bigtable.v2.ReadModifyWriteRowRequest} ReadModifyWriteRowRequest instance
                     */
                    ReadModifyWriteRowRequest.create = function create(properties) {
                        return new ReadModifyWriteRowRequest(properties);
                    };
    
                    /**
                     * Encodes the specified ReadModifyWriteRowRequest message. Does not implicitly {@link google.bigtable.v2.ReadModifyWriteRowRequest.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.ReadModifyWriteRowRequest
                     * @static
                     * @param {google.bigtable.v2.IReadModifyWriteRowRequest} message ReadModifyWriteRowRequest message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ReadModifyWriteRowRequest.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.tableName != null && Object.hasOwnProperty.call(message, "tableName"))
                            writer.uint32(/* id 1, wireType 2 =*/10).string(message.tableName);
                        if (message.rowKey != null && Object.hasOwnProperty.call(message, "rowKey"))
                            writer.uint32(/* id 2, wireType 2 =*/18).bytes(message.rowKey);
                        if (message.rules != null && message.rules.length)
                            for (var i = 0; i < message.rules.length; ++i)
                                $root.google.bigtable.v2.ReadModifyWriteRule.encode(message.rules[i], writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                        if (message.appProfileId != null && Object.hasOwnProperty.call(message, "appProfileId"))
                            writer.uint32(/* id 4, wireType 2 =*/34).string(message.appProfileId);
                        if (message.authorizedViewName != null && Object.hasOwnProperty.call(message, "authorizedViewName"))
                            writer.uint32(/* id 6, wireType 2 =*/50).string(message.authorizedViewName);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified ReadModifyWriteRowRequest message, length delimited. Does not implicitly {@link google.bigtable.v2.ReadModifyWriteRowRequest.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.ReadModifyWriteRowRequest
                     * @static
                     * @param {google.bigtable.v2.IReadModifyWriteRowRequest} message ReadModifyWriteRowRequest message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ReadModifyWriteRowRequest.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a ReadModifyWriteRowRequest message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.ReadModifyWriteRowRequest
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.ReadModifyWriteRowRequest} ReadModifyWriteRowRequest
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ReadModifyWriteRowRequest.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.ReadModifyWriteRowRequest();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.tableName = reader.string();
                                    break;
                                }
                            case 6: {
                                    message.authorizedViewName = reader.string();
                                    break;
                                }
                            case 4: {
                                    message.appProfileId = reader.string();
                                    break;
                                }
                            case 2: {
                                    message.rowKey = reader.bytes();
                                    break;
                                }
                            case 3: {
                                    if (!(message.rules && message.rules.length))
                                        message.rules = [];
                                    message.rules.push($root.google.bigtable.v2.ReadModifyWriteRule.decode(reader, reader.uint32()));
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a ReadModifyWriteRowRequest message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.ReadModifyWriteRowRequest
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.ReadModifyWriteRowRequest} ReadModifyWriteRowRequest
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ReadModifyWriteRowRequest.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a ReadModifyWriteRowRequest message.
                     * @function verify
                     * @memberof google.bigtable.v2.ReadModifyWriteRowRequest
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    ReadModifyWriteRowRequest.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.tableName != null && message.hasOwnProperty("tableName"))
                            if (!$util.isString(message.tableName))
                                return "tableName: string expected";
                        if (message.authorizedViewName != null && message.hasOwnProperty("authorizedViewName"))
                            if (!$util.isString(message.authorizedViewName))
                                return "authorizedViewName: string expected";
                        if (message.appProfileId != null && message.hasOwnProperty("appProfileId"))
                            if (!$util.isString(message.appProfileId))
                                return "appProfileId: string expected";
                        if (message.rowKey != null && message.hasOwnProperty("rowKey"))
                            if (!(message.rowKey && typeof message.rowKey.length === "number" || $util.isString(message.rowKey)))
                                return "rowKey: buffer expected";
                        if (message.rules != null && message.hasOwnProperty("rules")) {
                            if (!Array.isArray(message.rules))
                                return "rules: array expected";
                            for (var i = 0; i < message.rules.length; ++i) {
                                var error = $root.google.bigtable.v2.ReadModifyWriteRule.verify(message.rules[i]);
                                if (error)
                                    return "rules." + error;
                            }
                        }
                        return null;
                    };
    
                    /**
                     * Creates a ReadModifyWriteRowRequest message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.ReadModifyWriteRowRequest
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.ReadModifyWriteRowRequest} ReadModifyWriteRowRequest
                     */
                    ReadModifyWriteRowRequest.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.ReadModifyWriteRowRequest)
                            return object;
                        var message = new $root.google.bigtable.v2.ReadModifyWriteRowRequest();
                        if (object.tableName != null)
                            message.tableName = String(object.tableName);
                        if (object.authorizedViewName != null)
                            message.authorizedViewName = String(object.authorizedViewName);
                        if (object.appProfileId != null)
                            message.appProfileId = String(object.appProfileId);
                        if (object.rowKey != null)
                            if (typeof object.rowKey === "string")
                                $util.base64.decode(object.rowKey, message.rowKey = $util.newBuffer($util.base64.length(object.rowKey)), 0);
                            else if (object.rowKey.length >= 0)
                                message.rowKey = object.rowKey;
                        if (object.rules) {
                            if (!Array.isArray(object.rules))
                                throw TypeError(".google.bigtable.v2.ReadModifyWriteRowRequest.rules: array expected");
                            message.rules = [];
                            for (var i = 0; i < object.rules.length; ++i) {
                                if (typeof object.rules[i] !== "object")
                                    throw TypeError(".google.bigtable.v2.ReadModifyWriteRowRequest.rules: object expected");
                                message.rules[i] = $root.google.bigtable.v2.ReadModifyWriteRule.fromObject(object.rules[i]);
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a ReadModifyWriteRowRequest message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.ReadModifyWriteRowRequest
                     * @static
                     * @param {google.bigtable.v2.ReadModifyWriteRowRequest} message ReadModifyWriteRowRequest
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    ReadModifyWriteRowRequest.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.arrays || options.defaults)
                            object.rules = [];
                        if (options.defaults) {
                            object.tableName = "";
                            if (options.bytes === String)
                                object.rowKey = "";
                            else {
                                object.rowKey = [];
                                if (options.bytes !== Array)
                                    object.rowKey = $util.newBuffer(object.rowKey);
                            }
                            object.appProfileId = "";
                            object.authorizedViewName = "";
                        }
                        if (message.tableName != null && message.hasOwnProperty("tableName"))
                            object.tableName = message.tableName;
                        if (message.rowKey != null && message.hasOwnProperty("rowKey"))
                            object.rowKey = options.bytes === String ? $util.base64.encode(message.rowKey, 0, message.rowKey.length) : options.bytes === Array ? Array.prototype.slice.call(message.rowKey) : message.rowKey;
                        if (message.rules && message.rules.length) {
                            object.rules = [];
                            for (var j = 0; j < message.rules.length; ++j)
                                object.rules[j] = $root.google.bigtable.v2.ReadModifyWriteRule.toObject(message.rules[j], options);
                        }
                        if (message.appProfileId != null && message.hasOwnProperty("appProfileId"))
                            object.appProfileId = message.appProfileId;
                        if (message.authorizedViewName != null && message.hasOwnProperty("authorizedViewName"))
                            object.authorizedViewName = message.authorizedViewName;
                        return object;
                    };
    
                    /**
                     * Converts this ReadModifyWriteRowRequest to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.ReadModifyWriteRowRequest
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    ReadModifyWriteRowRequest.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for ReadModifyWriteRowRequest
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.ReadModifyWriteRowRequest
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    ReadModifyWriteRowRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.ReadModifyWriteRowRequest";
                    };
    
                    return ReadModifyWriteRowRequest;
                })();
    
                v2.ReadModifyWriteRowResponse = (function() {
    
                    /**
                     * Properties of a ReadModifyWriteRowResponse.
                     * @memberof google.bigtable.v2
                     * @interface IReadModifyWriteRowResponse
                     * @property {google.bigtable.v2.IRow|null} [row] ReadModifyWriteRowResponse row
                     */
    
                    /**
                     * Constructs a new ReadModifyWriteRowResponse.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a ReadModifyWriteRowResponse.
                     * @implements IReadModifyWriteRowResponse
                     * @constructor
                     * @param {google.bigtable.v2.IReadModifyWriteRowResponse=} [properties] Properties to set
                     */
                    function ReadModifyWriteRowResponse(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * ReadModifyWriteRowResponse row.
                     * @member {google.bigtable.v2.IRow|null|undefined} row
                     * @memberof google.bigtable.v2.ReadModifyWriteRowResponse
                     * @instance
                     */
                    ReadModifyWriteRowResponse.prototype.row = null;
    
                    /**
                     * Creates a new ReadModifyWriteRowResponse instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.ReadModifyWriteRowResponse
                     * @static
                     * @param {google.bigtable.v2.IReadModifyWriteRowResponse=} [properties] Properties to set
                     * @returns {google.bigtable.v2.ReadModifyWriteRowResponse} ReadModifyWriteRowResponse instance
                     */
                    ReadModifyWriteRowResponse.create = function create(properties) {
                        return new ReadModifyWriteRowResponse(properties);
                    };
    
                    /**
                     * Encodes the specified ReadModifyWriteRowResponse message. Does not implicitly {@link google.bigtable.v2.ReadModifyWriteRowResponse.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.ReadModifyWriteRowResponse
                     * @static
                     * @param {google.bigtable.v2.IReadModifyWriteRowResponse} message ReadModifyWriteRowResponse message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ReadModifyWriteRowResponse.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.row != null && Object.hasOwnProperty.call(message, "row"))
                            $root.google.bigtable.v2.Row.encode(message.row, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                        return writer;
                    };
    
                    /**
                     * Encodes the specified ReadModifyWriteRowResponse message, length delimited. Does not implicitly {@link google.bigtable.v2.ReadModifyWriteRowResponse.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.ReadModifyWriteRowResponse
                     * @static
                     * @param {google.bigtable.v2.IReadModifyWriteRowResponse} message ReadModifyWriteRowResponse message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ReadModifyWriteRowResponse.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a ReadModifyWriteRowResponse message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.ReadModifyWriteRowResponse
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.ReadModifyWriteRowResponse} ReadModifyWriteRowResponse
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ReadModifyWriteRowResponse.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.ReadModifyWriteRowResponse();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.row = $root.google.bigtable.v2.Row.decode(reader, reader.uint32());
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a ReadModifyWriteRowResponse message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.ReadModifyWriteRowResponse
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.ReadModifyWriteRowResponse} ReadModifyWriteRowResponse
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ReadModifyWriteRowResponse.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a ReadModifyWriteRowResponse message.
                     * @function verify
                     * @memberof google.bigtable.v2.ReadModifyWriteRowResponse
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    ReadModifyWriteRowResponse.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.row != null && message.hasOwnProperty("row")) {
                            var error = $root.google.bigtable.v2.Row.verify(message.row);
                            if (error)
                                return "row." + error;
                        }
                        return null;
                    };
    
                    /**
                     * Creates a ReadModifyWriteRowResponse message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.ReadModifyWriteRowResponse
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.ReadModifyWriteRowResponse} ReadModifyWriteRowResponse
                     */
                    ReadModifyWriteRowResponse.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.ReadModifyWriteRowResponse)
                            return object;
                        var message = new $root.google.bigtable.v2.ReadModifyWriteRowResponse();
                        if (object.row != null) {
                            if (typeof object.row !== "object")
                                throw TypeError(".google.bigtable.v2.ReadModifyWriteRowResponse.row: object expected");
                            message.row = $root.google.bigtable.v2.Row.fromObject(object.row);
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a ReadModifyWriteRowResponse message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.ReadModifyWriteRowResponse
                     * @static
                     * @param {google.bigtable.v2.ReadModifyWriteRowResponse} message ReadModifyWriteRowResponse
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    ReadModifyWriteRowResponse.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults)
                            object.row = null;
                        if (message.row != null && message.hasOwnProperty("row"))
                            object.row = $root.google.bigtable.v2.Row.toObject(message.row, options);
                        return object;
                    };
    
                    /**
                     * Converts this ReadModifyWriteRowResponse to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.ReadModifyWriteRowResponse
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    ReadModifyWriteRowResponse.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for ReadModifyWriteRowResponse
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.ReadModifyWriteRowResponse
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    ReadModifyWriteRowResponse.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.ReadModifyWriteRowResponse";
                    };
    
                    return ReadModifyWriteRowResponse;
                })();
    
                v2.GenerateInitialChangeStreamPartitionsRequest = (function() {
    
                    /**
                     * Properties of a GenerateInitialChangeStreamPartitionsRequest.
                     * @memberof google.bigtable.v2
                     * @interface IGenerateInitialChangeStreamPartitionsRequest
                     * @property {string|null} [tableName] GenerateInitialChangeStreamPartitionsRequest tableName
                     * @property {string|null} [appProfileId] GenerateInitialChangeStreamPartitionsRequest appProfileId
                     */
    
                    /**
                     * Constructs a new GenerateInitialChangeStreamPartitionsRequest.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a GenerateInitialChangeStreamPartitionsRequest.
                     * @implements IGenerateInitialChangeStreamPartitionsRequest
                     * @constructor
                     * @param {google.bigtable.v2.IGenerateInitialChangeStreamPartitionsRequest=} [properties] Properties to set
                     */
                    function GenerateInitialChangeStreamPartitionsRequest(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * GenerateInitialChangeStreamPartitionsRequest tableName.
                     * @member {string} tableName
                     * @memberof google.bigtable.v2.GenerateInitialChangeStreamPartitionsRequest
                     * @instance
                     */
                    GenerateInitialChangeStreamPartitionsRequest.prototype.tableName = "";
    
                    /**
                     * GenerateInitialChangeStreamPartitionsRequest appProfileId.
                     * @member {string} appProfileId
                     * @memberof google.bigtable.v2.GenerateInitialChangeStreamPartitionsRequest
                     * @instance
                     */
                    GenerateInitialChangeStreamPartitionsRequest.prototype.appProfileId = "";
    
                    /**
                     * Creates a new GenerateInitialChangeStreamPartitionsRequest instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.GenerateInitialChangeStreamPartitionsRequest
                     * @static
                     * @param {google.bigtable.v2.IGenerateInitialChangeStreamPartitionsRequest=} [properties] Properties to set
                     * @returns {google.bigtable.v2.GenerateInitialChangeStreamPartitionsRequest} GenerateInitialChangeStreamPartitionsRequest instance
                     */
                    GenerateInitialChangeStreamPartitionsRequest.create = function create(properties) {
                        return new GenerateInitialChangeStreamPartitionsRequest(properties);
                    };
    
                    /**
                     * Encodes the specified GenerateInitialChangeStreamPartitionsRequest message. Does not implicitly {@link google.bigtable.v2.GenerateInitialChangeStreamPartitionsRequest.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.GenerateInitialChangeStreamPartitionsRequest
                     * @static
                     * @param {google.bigtable.v2.IGenerateInitialChangeStreamPartitionsRequest} message GenerateInitialChangeStreamPartitionsRequest message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    GenerateInitialChangeStreamPartitionsRequest.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.tableName != null && Object.hasOwnProperty.call(message, "tableName"))
                            writer.uint32(/* id 1, wireType 2 =*/10).string(message.tableName);
                        if (message.appProfileId != null && Object.hasOwnProperty.call(message, "appProfileId"))
                            writer.uint32(/* id 2, wireType 2 =*/18).string(message.appProfileId);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified GenerateInitialChangeStreamPartitionsRequest message, length delimited. Does not implicitly {@link google.bigtable.v2.GenerateInitialChangeStreamPartitionsRequest.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.GenerateInitialChangeStreamPartitionsRequest
                     * @static
                     * @param {google.bigtable.v2.IGenerateInitialChangeStreamPartitionsRequest} message GenerateInitialChangeStreamPartitionsRequest message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    GenerateInitialChangeStreamPartitionsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a GenerateInitialChangeStreamPartitionsRequest message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.GenerateInitialChangeStreamPartitionsRequest
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.GenerateInitialChangeStreamPartitionsRequest} GenerateInitialChangeStreamPartitionsRequest
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    GenerateInitialChangeStreamPartitionsRequest.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.GenerateInitialChangeStreamPartitionsRequest();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.tableName = reader.string();
                                    break;
                                }
                            case 2: {
                                    message.appProfileId = reader.string();
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a GenerateInitialChangeStreamPartitionsRequest message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.GenerateInitialChangeStreamPartitionsRequest
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.GenerateInitialChangeStreamPartitionsRequest} GenerateInitialChangeStreamPartitionsRequest
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    GenerateInitialChangeStreamPartitionsRequest.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a GenerateInitialChangeStreamPartitionsRequest message.
                     * @function verify
                     * @memberof google.bigtable.v2.GenerateInitialChangeStreamPartitionsRequest
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    GenerateInitialChangeStreamPartitionsRequest.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.tableName != null && message.hasOwnProperty("tableName"))
                            if (!$util.isString(message.tableName))
                                return "tableName: string expected";
                        if (message.appProfileId != null && message.hasOwnProperty("appProfileId"))
                            if (!$util.isString(message.appProfileId))
                                return "appProfileId: string expected";
                        return null;
                    };
    
                    /**
                     * Creates a GenerateInitialChangeStreamPartitionsRequest message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.GenerateInitialChangeStreamPartitionsRequest
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.GenerateInitialChangeStreamPartitionsRequest} GenerateInitialChangeStreamPartitionsRequest
                     */
                    GenerateInitialChangeStreamPartitionsRequest.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.GenerateInitialChangeStreamPartitionsRequest)
                            return object;
                        var message = new $root.google.bigtable.v2.GenerateInitialChangeStreamPartitionsRequest();
                        if (object.tableName != null)
                            message.tableName = String(object.tableName);
                        if (object.appProfileId != null)
                            message.appProfileId = String(object.appProfileId);
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a GenerateInitialChangeStreamPartitionsRequest message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.GenerateInitialChangeStreamPartitionsRequest
                     * @static
                     * @param {google.bigtable.v2.GenerateInitialChangeStreamPartitionsRequest} message GenerateInitialChangeStreamPartitionsRequest
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    GenerateInitialChangeStreamPartitionsRequest.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults) {
                            object.tableName = "";
                            object.appProfileId = "";
                        }
                        if (message.tableName != null && message.hasOwnProperty("tableName"))
                            object.tableName = message.tableName;
                        if (message.appProfileId != null && message.hasOwnProperty("appProfileId"))
                            object.appProfileId = message.appProfileId;
                        return object;
                    };
    
                    /**
                     * Converts this GenerateInitialChangeStreamPartitionsRequest to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.GenerateInitialChangeStreamPartitionsRequest
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    GenerateInitialChangeStreamPartitionsRequest.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for GenerateInitialChangeStreamPartitionsRequest
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.GenerateInitialChangeStreamPartitionsRequest
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    GenerateInitialChangeStreamPartitionsRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.GenerateInitialChangeStreamPartitionsRequest";
                    };
    
                    return GenerateInitialChangeStreamPartitionsRequest;
                })();
    
                v2.GenerateInitialChangeStreamPartitionsResponse = (function() {
    
                    /**
                     * Properties of a GenerateInitialChangeStreamPartitionsResponse.
                     * @memberof google.bigtable.v2
                     * @interface IGenerateInitialChangeStreamPartitionsResponse
                     * @property {google.bigtable.v2.IStreamPartition|null} [partition] GenerateInitialChangeStreamPartitionsResponse partition
                     */
    
                    /**
                     * Constructs a new GenerateInitialChangeStreamPartitionsResponse.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a GenerateInitialChangeStreamPartitionsResponse.
                     * @implements IGenerateInitialChangeStreamPartitionsResponse
                     * @constructor
                     * @param {google.bigtable.v2.IGenerateInitialChangeStreamPartitionsResponse=} [properties] Properties to set
                     */
                    function GenerateInitialChangeStreamPartitionsResponse(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * GenerateInitialChangeStreamPartitionsResponse partition.
                     * @member {google.bigtable.v2.IStreamPartition|null|undefined} partition
                     * @memberof google.bigtable.v2.GenerateInitialChangeStreamPartitionsResponse
                     * @instance
                     */
                    GenerateInitialChangeStreamPartitionsResponse.prototype.partition = null;
    
                    /**
                     * Creates a new GenerateInitialChangeStreamPartitionsResponse instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.GenerateInitialChangeStreamPartitionsResponse
                     * @static
                     * @param {google.bigtable.v2.IGenerateInitialChangeStreamPartitionsResponse=} [properties] Properties to set
                     * @returns {google.bigtable.v2.GenerateInitialChangeStreamPartitionsResponse} GenerateInitialChangeStreamPartitionsResponse instance
                     */
                    GenerateInitialChangeStreamPartitionsResponse.create = function create(properties) {
                        return new GenerateInitialChangeStreamPartitionsResponse(properties);
                    };
    
                    /**
                     * Encodes the specified GenerateInitialChangeStreamPartitionsResponse message. Does not implicitly {@link google.bigtable.v2.GenerateInitialChangeStreamPartitionsResponse.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.GenerateInitialChangeStreamPartitionsResponse
                     * @static
                     * @param {google.bigtable.v2.IGenerateInitialChangeStreamPartitionsResponse} message GenerateInitialChangeStreamPartitionsResponse message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    GenerateInitialChangeStreamPartitionsResponse.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.partition != null && Object.hasOwnProperty.call(message, "partition"))
                            $root.google.bigtable.v2.StreamPartition.encode(message.partition, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                        return writer;
                    };
    
                    /**
                     * Encodes the specified GenerateInitialChangeStreamPartitionsResponse message, length delimited. Does not implicitly {@link google.bigtable.v2.GenerateInitialChangeStreamPartitionsResponse.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.GenerateInitialChangeStreamPartitionsResponse
                     * @static
                     * @param {google.bigtable.v2.IGenerateInitialChangeStreamPartitionsResponse} message GenerateInitialChangeStreamPartitionsResponse message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    GenerateInitialChangeStreamPartitionsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a GenerateInitialChangeStreamPartitionsResponse message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.GenerateInitialChangeStreamPartitionsResponse
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.GenerateInitialChangeStreamPartitionsResponse} GenerateInitialChangeStreamPartitionsResponse
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    GenerateInitialChangeStreamPartitionsResponse.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.GenerateInitialChangeStreamPartitionsResponse();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.partition = $root.google.bigtable.v2.StreamPartition.decode(reader, reader.uint32());
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a GenerateInitialChangeStreamPartitionsResponse message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.GenerateInitialChangeStreamPartitionsResponse
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.GenerateInitialChangeStreamPartitionsResponse} GenerateInitialChangeStreamPartitionsResponse
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    GenerateInitialChangeStreamPartitionsResponse.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a GenerateInitialChangeStreamPartitionsResponse message.
                     * @function verify
                     * @memberof google.bigtable.v2.GenerateInitialChangeStreamPartitionsResponse
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    GenerateInitialChangeStreamPartitionsResponse.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.partition != null && message.hasOwnProperty("partition")) {
                            var error = $root.google.bigtable.v2.StreamPartition.verify(message.partition);
                            if (error)
                                return "partition." + error;
                        }
                        return null;
                    };
    
                    /**
                     * Creates a GenerateInitialChangeStreamPartitionsResponse message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.GenerateInitialChangeStreamPartitionsResponse
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.GenerateInitialChangeStreamPartitionsResponse} GenerateInitialChangeStreamPartitionsResponse
                     */
                    GenerateInitialChangeStreamPartitionsResponse.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.GenerateInitialChangeStreamPartitionsResponse)
                            return object;
                        var message = new $root.google.bigtable.v2.GenerateInitialChangeStreamPartitionsResponse();
                        if (object.partition != null) {
                            if (typeof object.partition !== "object")
                                throw TypeError(".google.bigtable.v2.GenerateInitialChangeStreamPartitionsResponse.partition: object expected");
                            message.partition = $root.google.bigtable.v2.StreamPartition.fromObject(object.partition);
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a GenerateInitialChangeStreamPartitionsResponse message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.GenerateInitialChangeStreamPartitionsResponse
                     * @static
                     * @param {google.bigtable.v2.GenerateInitialChangeStreamPartitionsResponse} message GenerateInitialChangeStreamPartitionsResponse
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    GenerateInitialChangeStreamPartitionsResponse.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults)
                            object.partition = null;
                        if (message.partition != null && message.hasOwnProperty("partition"))
                            object.partition = $root.google.bigtable.v2.StreamPartition.toObject(message.partition, options);
                        return object;
                    };
    
                    /**
                     * Converts this GenerateInitialChangeStreamPartitionsResponse to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.GenerateInitialChangeStreamPartitionsResponse
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    GenerateInitialChangeStreamPartitionsResponse.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for GenerateInitialChangeStreamPartitionsResponse
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.GenerateInitialChangeStreamPartitionsResponse
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    GenerateInitialChangeStreamPartitionsResponse.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.GenerateInitialChangeStreamPartitionsResponse";
                    };
    
                    return GenerateInitialChangeStreamPartitionsResponse;
                })();
    
                v2.ReadChangeStreamRequest = (function() {
    
                    /**
                     * Properties of a ReadChangeStreamRequest.
                     * @memberof google.bigtable.v2
                     * @interface IReadChangeStreamRequest
                     * @property {string|null} [tableName] ReadChangeStreamRequest tableName
                     * @property {string|null} [appProfileId] ReadChangeStreamRequest appProfileId
                     * @property {google.bigtable.v2.IStreamPartition|null} [partition] ReadChangeStreamRequest partition
                     * @property {google.protobuf.ITimestamp|null} [startTime] ReadChangeStreamRequest startTime
                     * @property {google.bigtable.v2.IStreamContinuationTokens|null} [continuationTokens] ReadChangeStreamRequest continuationTokens
                     * @property {google.protobuf.ITimestamp|null} [endTime] ReadChangeStreamRequest endTime
                     * @property {google.protobuf.IDuration|null} [heartbeatDuration] ReadChangeStreamRequest heartbeatDuration
                     */
    
                    /**
                     * Constructs a new ReadChangeStreamRequest.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a ReadChangeStreamRequest.
                     * @implements IReadChangeStreamRequest
                     * @constructor
                     * @param {google.bigtable.v2.IReadChangeStreamRequest=} [properties] Properties to set
                     */
                    function ReadChangeStreamRequest(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * ReadChangeStreamRequest tableName.
                     * @member {string} tableName
                     * @memberof google.bigtable.v2.ReadChangeStreamRequest
                     * @instance
                     */
                    ReadChangeStreamRequest.prototype.tableName = "";
    
                    /**
                     * ReadChangeStreamRequest appProfileId.
                     * @member {string} appProfileId
                     * @memberof google.bigtable.v2.ReadChangeStreamRequest
                     * @instance
                     */
                    ReadChangeStreamRequest.prototype.appProfileId = "";
    
                    /**
                     * ReadChangeStreamRequest partition.
                     * @member {google.bigtable.v2.IStreamPartition|null|undefined} partition
                     * @memberof google.bigtable.v2.ReadChangeStreamRequest
                     * @instance
                     */
                    ReadChangeStreamRequest.prototype.partition = null;
    
                    /**
                     * ReadChangeStreamRequest startTime.
                     * @member {google.protobuf.ITimestamp|null|undefined} startTime
                     * @memberof google.bigtable.v2.ReadChangeStreamRequest
                     * @instance
                     */
                    ReadChangeStreamRequest.prototype.startTime = null;
    
                    /**
                     * ReadChangeStreamRequest continuationTokens.
                     * @member {google.bigtable.v2.IStreamContinuationTokens|null|undefined} continuationTokens
                     * @memberof google.bigtable.v2.ReadChangeStreamRequest
                     * @instance
                     */
                    ReadChangeStreamRequest.prototype.continuationTokens = null;
    
                    /**
                     * ReadChangeStreamRequest endTime.
                     * @member {google.protobuf.ITimestamp|null|undefined} endTime
                     * @memberof google.bigtable.v2.ReadChangeStreamRequest
                     * @instance
                     */
                    ReadChangeStreamRequest.prototype.endTime = null;
    
                    /**
                     * ReadChangeStreamRequest heartbeatDuration.
                     * @member {google.protobuf.IDuration|null|undefined} heartbeatDuration
                     * @memberof google.bigtable.v2.ReadChangeStreamRequest
                     * @instance
                     */
                    ReadChangeStreamRequest.prototype.heartbeatDuration = null;
    
                    // OneOf field names bound to virtual getters and setters
                    var $oneOfFields;
    
                    /**
                     * ReadChangeStreamRequest startFrom.
                     * @member {"startTime"|"continuationTokens"|undefined} startFrom
                     * @memberof google.bigtable.v2.ReadChangeStreamRequest
                     * @instance
                     */
                    Object.defineProperty(ReadChangeStreamRequest.prototype, "startFrom", {
                        get: $util.oneOfGetter($oneOfFields = ["startTime", "continuationTokens"]),
                        set: $util.oneOfSetter($oneOfFields)
                    });
    
                    /**
                     * Creates a new ReadChangeStreamRequest instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.ReadChangeStreamRequest
                     * @static
                     * @param {google.bigtable.v2.IReadChangeStreamRequest=} [properties] Properties to set
                     * @returns {google.bigtable.v2.ReadChangeStreamRequest} ReadChangeStreamRequest instance
                     */
                    ReadChangeStreamRequest.create = function create(properties) {
                        return new ReadChangeStreamRequest(properties);
                    };
    
                    /**
                     * Encodes the specified ReadChangeStreamRequest message. Does not implicitly {@link google.bigtable.v2.ReadChangeStreamRequest.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.ReadChangeStreamRequest
                     * @static
                     * @param {google.bigtable.v2.IReadChangeStreamRequest} message ReadChangeStreamRequest message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ReadChangeStreamRequest.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.tableName != null && Object.hasOwnProperty.call(message, "tableName"))
                            writer.uint32(/* id 1, wireType 2 =*/10).string(message.tableName);
                        if (message.appProfileId != null && Object.hasOwnProperty.call(message, "appProfileId"))
                            writer.uint32(/* id 2, wireType 2 =*/18).string(message.appProfileId);
                        if (message.partition != null && Object.hasOwnProperty.call(message, "partition"))
                            $root.google.bigtable.v2.StreamPartition.encode(message.partition, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                        if (message.startTime != null && Object.hasOwnProperty.call(message, "startTime"))
                            $root.google.protobuf.Timestamp.encode(message.startTime, writer.uint32(/* id 4, wireType 2 =*/34).fork()).ldelim();
                        if (message.endTime != null && Object.hasOwnProperty.call(message, "endTime"))
                            $root.google.protobuf.Timestamp.encode(message.endTime, writer.uint32(/* id 5, wireType 2 =*/42).fork()).ldelim();
                        if (message.continuationTokens != null && Object.hasOwnProperty.call(message, "continuationTokens"))
                            $root.google.bigtable.v2.StreamContinuationTokens.encode(message.continuationTokens, writer.uint32(/* id 6, wireType 2 =*/50).fork()).ldelim();
                        if (message.heartbeatDuration != null && Object.hasOwnProperty.call(message, "heartbeatDuration"))
                            $root.google.protobuf.Duration.encode(message.heartbeatDuration, writer.uint32(/* id 7, wireType 2 =*/58).fork()).ldelim();
                        return writer;
                    };
    
                    /**
                     * Encodes the specified ReadChangeStreamRequest message, length delimited. Does not implicitly {@link google.bigtable.v2.ReadChangeStreamRequest.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.ReadChangeStreamRequest
                     * @static
                     * @param {google.bigtable.v2.IReadChangeStreamRequest} message ReadChangeStreamRequest message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ReadChangeStreamRequest.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a ReadChangeStreamRequest message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.ReadChangeStreamRequest
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.ReadChangeStreamRequest} ReadChangeStreamRequest
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ReadChangeStreamRequest.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.ReadChangeStreamRequest();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.tableName = reader.string();
                                    break;
                                }
                            case 2: {
                                    message.appProfileId = reader.string();
                                    break;
                                }
                            case 3: {
                                    message.partition = $root.google.bigtable.v2.StreamPartition.decode(reader, reader.uint32());
                                    break;
                                }
                            case 4: {
                                    message.startTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                    break;
                                }
                            case 6: {
                                    message.continuationTokens = $root.google.bigtable.v2.StreamContinuationTokens.decode(reader, reader.uint32());
                                    break;
                                }
                            case 5: {
                                    message.endTime = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                    break;
                                }
                            case 7: {
                                    message.heartbeatDuration = $root.google.protobuf.Duration.decode(reader, reader.uint32());
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a ReadChangeStreamRequest message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.ReadChangeStreamRequest
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.ReadChangeStreamRequest} ReadChangeStreamRequest
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ReadChangeStreamRequest.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a ReadChangeStreamRequest message.
                     * @function verify
                     * @memberof google.bigtable.v2.ReadChangeStreamRequest
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    ReadChangeStreamRequest.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        var properties = {};
                        if (message.tableName != null && message.hasOwnProperty("tableName"))
                            if (!$util.isString(message.tableName))
                                return "tableName: string expected";
                        if (message.appProfileId != null && message.hasOwnProperty("appProfileId"))
                            if (!$util.isString(message.appProfileId))
                                return "appProfileId: string expected";
                        if (message.partition != null && message.hasOwnProperty("partition")) {
                            var error = $root.google.bigtable.v2.StreamPartition.verify(message.partition);
                            if (error)
                                return "partition." + error;
                        }
                        if (message.startTime != null && message.hasOwnProperty("startTime")) {
                            properties.startFrom = 1;
                            {
                                var error = $root.google.protobuf.Timestamp.verify(message.startTime);
                                if (error)
                                    return "startTime." + error;
                            }
                        }
                        if (message.continuationTokens != null && message.hasOwnProperty("continuationTokens")) {
                            if (properties.startFrom === 1)
                                return "startFrom: multiple values";
                            properties.startFrom = 1;
                            {
                                var error = $root.google.bigtable.v2.StreamContinuationTokens.verify(message.continuationTokens);
                                if (error)
                                    return "continuationTokens." + error;
                            }
                        }
                        if (message.endTime != null && message.hasOwnProperty("endTime")) {
                            var error = $root.google.protobuf.Timestamp.verify(message.endTime);
                            if (error)
                                return "endTime." + error;
                        }
                        if (message.heartbeatDuration != null && message.hasOwnProperty("heartbeatDuration")) {
                            var error = $root.google.protobuf.Duration.verify(message.heartbeatDuration);
                            if (error)
                                return "heartbeatDuration." + error;
                        }
                        return null;
                    };
    
                    /**
                     * Creates a ReadChangeStreamRequest message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.ReadChangeStreamRequest
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.ReadChangeStreamRequest} ReadChangeStreamRequest
                     */
                    ReadChangeStreamRequest.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.ReadChangeStreamRequest)
                            return object;
                        var message = new $root.google.bigtable.v2.ReadChangeStreamRequest();
                        if (object.tableName != null)
                            message.tableName = String(object.tableName);
                        if (object.appProfileId != null)
                            message.appProfileId = String(object.appProfileId);
                        if (object.partition != null) {
                            if (typeof object.partition !== "object")
                                throw TypeError(".google.bigtable.v2.ReadChangeStreamRequest.partition: object expected");
                            message.partition = $root.google.bigtable.v2.StreamPartition.fromObject(object.partition);
                        }
                        if (object.startTime != null) {
                            if (typeof object.startTime !== "object")
                                throw TypeError(".google.bigtable.v2.ReadChangeStreamRequest.startTime: object expected");
                            message.startTime = $root.google.protobuf.Timestamp.fromObject(object.startTime);
                        }
                        if (object.continuationTokens != null) {
                            if (typeof object.continuationTokens !== "object")
                                throw TypeError(".google.bigtable.v2.ReadChangeStreamRequest.continuationTokens: object expected");
                            message.continuationTokens = $root.google.bigtable.v2.StreamContinuationTokens.fromObject(object.continuationTokens);
                        }
                        if (object.endTime != null) {
                            if (typeof object.endTime !== "object")
                                throw TypeError(".google.bigtable.v2.ReadChangeStreamRequest.endTime: object expected");
                            message.endTime = $root.google.protobuf.Timestamp.fromObject(object.endTime);
                        }
                        if (object.heartbeatDuration != null) {
                            if (typeof object.heartbeatDuration !== "object")
                                throw TypeError(".google.bigtable.v2.ReadChangeStreamRequest.heartbeatDuration: object expected");
                            message.heartbeatDuration = $root.google.protobuf.Duration.fromObject(object.heartbeatDuration);
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a ReadChangeStreamRequest message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.ReadChangeStreamRequest
                     * @static
                     * @param {google.bigtable.v2.ReadChangeStreamRequest} message ReadChangeStreamRequest
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    ReadChangeStreamRequest.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults) {
                            object.tableName = "";
                            object.appProfileId = "";
                            object.partition = null;
                            object.endTime = null;
                            object.heartbeatDuration = null;
                        }
                        if (message.tableName != null && message.hasOwnProperty("tableName"))
                            object.tableName = message.tableName;
                        if (message.appProfileId != null && message.hasOwnProperty("appProfileId"))
                            object.appProfileId = message.appProfileId;
                        if (message.partition != null && message.hasOwnProperty("partition"))
                            object.partition = $root.google.bigtable.v2.StreamPartition.toObject(message.partition, options);
                        if (message.startTime != null && message.hasOwnProperty("startTime")) {
                            object.startTime = $root.google.protobuf.Timestamp.toObject(message.startTime, options);
                            if (options.oneofs)
                                object.startFrom = "startTime";
                        }
                        if (message.endTime != null && message.hasOwnProperty("endTime"))
                            object.endTime = $root.google.protobuf.Timestamp.toObject(message.endTime, options);
                        if (message.continuationTokens != null && message.hasOwnProperty("continuationTokens")) {
                            object.continuationTokens = $root.google.bigtable.v2.StreamContinuationTokens.toObject(message.continuationTokens, options);
                            if (options.oneofs)
                                object.startFrom = "continuationTokens";
                        }
                        if (message.heartbeatDuration != null && message.hasOwnProperty("heartbeatDuration"))
                            object.heartbeatDuration = $root.google.protobuf.Duration.toObject(message.heartbeatDuration, options);
                        return object;
                    };
    
                    /**
                     * Converts this ReadChangeStreamRequest to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.ReadChangeStreamRequest
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    ReadChangeStreamRequest.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for ReadChangeStreamRequest
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.ReadChangeStreamRequest
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    ReadChangeStreamRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.ReadChangeStreamRequest";
                    };
    
                    return ReadChangeStreamRequest;
                })();
    
                v2.ReadChangeStreamResponse = (function() {
    
                    /**
                     * Properties of a ReadChangeStreamResponse.
                     * @memberof google.bigtable.v2
                     * @interface IReadChangeStreamResponse
                     * @property {google.bigtable.v2.ReadChangeStreamResponse.IDataChange|null} [dataChange] ReadChangeStreamResponse dataChange
                     * @property {google.bigtable.v2.ReadChangeStreamResponse.IHeartbeat|null} [heartbeat] ReadChangeStreamResponse heartbeat
                     * @property {google.bigtable.v2.ReadChangeStreamResponse.ICloseStream|null} [closeStream] ReadChangeStreamResponse closeStream
                     */
    
                    /**
                     * Constructs a new ReadChangeStreamResponse.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a ReadChangeStreamResponse.
                     * @implements IReadChangeStreamResponse
                     * @constructor
                     * @param {google.bigtable.v2.IReadChangeStreamResponse=} [properties] Properties to set
                     */
                    function ReadChangeStreamResponse(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * ReadChangeStreamResponse dataChange.
                     * @member {google.bigtable.v2.ReadChangeStreamResponse.IDataChange|null|undefined} dataChange
                     * @memberof google.bigtable.v2.ReadChangeStreamResponse
                     * @instance
                     */
                    ReadChangeStreamResponse.prototype.dataChange = null;
    
                    /**
                     * ReadChangeStreamResponse heartbeat.
                     * @member {google.bigtable.v2.ReadChangeStreamResponse.IHeartbeat|null|undefined} heartbeat
                     * @memberof google.bigtable.v2.ReadChangeStreamResponse
                     * @instance
                     */
                    ReadChangeStreamResponse.prototype.heartbeat = null;
    
                    /**
                     * ReadChangeStreamResponse closeStream.
                     * @member {google.bigtable.v2.ReadChangeStreamResponse.ICloseStream|null|undefined} closeStream
                     * @memberof google.bigtable.v2.ReadChangeStreamResponse
                     * @instance
                     */
                    ReadChangeStreamResponse.prototype.closeStream = null;
    
                    // OneOf field names bound to virtual getters and setters
                    var $oneOfFields;
    
                    /**
                     * ReadChangeStreamResponse streamRecord.
                     * @member {"dataChange"|"heartbeat"|"closeStream"|undefined} streamRecord
                     * @memberof google.bigtable.v2.ReadChangeStreamResponse
                     * @instance
                     */
                    Object.defineProperty(ReadChangeStreamResponse.prototype, "streamRecord", {
                        get: $util.oneOfGetter($oneOfFields = ["dataChange", "heartbeat", "closeStream"]),
                        set: $util.oneOfSetter($oneOfFields)
                    });
    
                    /**
                     * Creates a new ReadChangeStreamResponse instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.ReadChangeStreamResponse
                     * @static
                     * @param {google.bigtable.v2.IReadChangeStreamResponse=} [properties] Properties to set
                     * @returns {google.bigtable.v2.ReadChangeStreamResponse} ReadChangeStreamResponse instance
                     */
                    ReadChangeStreamResponse.create = function create(properties) {
                        return new ReadChangeStreamResponse(properties);
                    };
    
                    /**
                     * Encodes the specified ReadChangeStreamResponse message. Does not implicitly {@link google.bigtable.v2.ReadChangeStreamResponse.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.ReadChangeStreamResponse
                     * @static
                     * @param {google.bigtable.v2.IReadChangeStreamResponse} message ReadChangeStreamResponse message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ReadChangeStreamResponse.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.dataChange != null && Object.hasOwnProperty.call(message, "dataChange"))
                            $root.google.bigtable.v2.ReadChangeStreamResponse.DataChange.encode(message.dataChange, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                        if (message.heartbeat != null && Object.hasOwnProperty.call(message, "heartbeat"))
                            $root.google.bigtable.v2.ReadChangeStreamResponse.Heartbeat.encode(message.heartbeat, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                        if (message.closeStream != null && Object.hasOwnProperty.call(message, "closeStream"))
                            $root.google.bigtable.v2.ReadChangeStreamResponse.CloseStream.encode(message.closeStream, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                        return writer;
                    };
    
                    /**
                     * Encodes the specified ReadChangeStreamResponse message, length delimited. Does not implicitly {@link google.bigtable.v2.ReadChangeStreamResponse.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.ReadChangeStreamResponse
                     * @static
                     * @param {google.bigtable.v2.IReadChangeStreamResponse} message ReadChangeStreamResponse message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ReadChangeStreamResponse.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a ReadChangeStreamResponse message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.ReadChangeStreamResponse
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.ReadChangeStreamResponse} ReadChangeStreamResponse
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ReadChangeStreamResponse.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.ReadChangeStreamResponse();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.dataChange = $root.google.bigtable.v2.ReadChangeStreamResponse.DataChange.decode(reader, reader.uint32());
                                    break;
                                }
                            case 2: {
                                    message.heartbeat = $root.google.bigtable.v2.ReadChangeStreamResponse.Heartbeat.decode(reader, reader.uint32());
                                    break;
                                }
                            case 3: {
                                    message.closeStream = $root.google.bigtable.v2.ReadChangeStreamResponse.CloseStream.decode(reader, reader.uint32());
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a ReadChangeStreamResponse message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.ReadChangeStreamResponse
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.ReadChangeStreamResponse} ReadChangeStreamResponse
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ReadChangeStreamResponse.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a ReadChangeStreamResponse message.
                     * @function verify
                     * @memberof google.bigtable.v2.ReadChangeStreamResponse
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    ReadChangeStreamResponse.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        var properties = {};
                        if (message.dataChange != null && message.hasOwnProperty("dataChange")) {
                            properties.streamRecord = 1;
                            {
                                var error = $root.google.bigtable.v2.ReadChangeStreamResponse.DataChange.verify(message.dataChange);
                                if (error)
                                    return "dataChange." + error;
                            }
                        }
                        if (message.heartbeat != null && message.hasOwnProperty("heartbeat")) {
                            if (properties.streamRecord === 1)
                                return "streamRecord: multiple values";
                            properties.streamRecord = 1;
                            {
                                var error = $root.google.bigtable.v2.ReadChangeStreamResponse.Heartbeat.verify(message.heartbeat);
                                if (error)
                                    return "heartbeat." + error;
                            }
                        }
                        if (message.closeStream != null && message.hasOwnProperty("closeStream")) {
                            if (properties.streamRecord === 1)
                                return "streamRecord: multiple values";
                            properties.streamRecord = 1;
                            {
                                var error = $root.google.bigtable.v2.ReadChangeStreamResponse.CloseStream.verify(message.closeStream);
                                if (error)
                                    return "closeStream." + error;
                            }
                        }
                        return null;
                    };
    
                    /**
                     * Creates a ReadChangeStreamResponse message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.ReadChangeStreamResponse
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.ReadChangeStreamResponse} ReadChangeStreamResponse
                     */
                    ReadChangeStreamResponse.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.ReadChangeStreamResponse)
                            return object;
                        var message = new $root.google.bigtable.v2.ReadChangeStreamResponse();
                        if (object.dataChange != null) {
                            if (typeof object.dataChange !== "object")
                                throw TypeError(".google.bigtable.v2.ReadChangeStreamResponse.dataChange: object expected");
                            message.dataChange = $root.google.bigtable.v2.ReadChangeStreamResponse.DataChange.fromObject(object.dataChange);
                        }
                        if (object.heartbeat != null) {
                            if (typeof object.heartbeat !== "object")
                                throw TypeError(".google.bigtable.v2.ReadChangeStreamResponse.heartbeat: object expected");
                            message.heartbeat = $root.google.bigtable.v2.ReadChangeStreamResponse.Heartbeat.fromObject(object.heartbeat);
                        }
                        if (object.closeStream != null) {
                            if (typeof object.closeStream !== "object")
                                throw TypeError(".google.bigtable.v2.ReadChangeStreamResponse.closeStream: object expected");
                            message.closeStream = $root.google.bigtable.v2.ReadChangeStreamResponse.CloseStream.fromObject(object.closeStream);
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a ReadChangeStreamResponse message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.ReadChangeStreamResponse
                     * @static
                     * @param {google.bigtable.v2.ReadChangeStreamResponse} message ReadChangeStreamResponse
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    ReadChangeStreamResponse.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (message.dataChange != null && message.hasOwnProperty("dataChange")) {
                            object.dataChange = $root.google.bigtable.v2.ReadChangeStreamResponse.DataChange.toObject(message.dataChange, options);
                            if (options.oneofs)
                                object.streamRecord = "dataChange";
                        }
                        if (message.heartbeat != null && message.hasOwnProperty("heartbeat")) {
                            object.heartbeat = $root.google.bigtable.v2.ReadChangeStreamResponse.Heartbeat.toObject(message.heartbeat, options);
                            if (options.oneofs)
                                object.streamRecord = "heartbeat";
                        }
                        if (message.closeStream != null && message.hasOwnProperty("closeStream")) {
                            object.closeStream = $root.google.bigtable.v2.ReadChangeStreamResponse.CloseStream.toObject(message.closeStream, options);
                            if (options.oneofs)
                                object.streamRecord = "closeStream";
                        }
                        return object;
                    };
    
                    /**
                     * Converts this ReadChangeStreamResponse to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.ReadChangeStreamResponse
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    ReadChangeStreamResponse.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for ReadChangeStreamResponse
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.ReadChangeStreamResponse
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    ReadChangeStreamResponse.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.ReadChangeStreamResponse";
                    };
    
                    ReadChangeStreamResponse.MutationChunk = (function() {
    
                        /**
                         * Properties of a MutationChunk.
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse
                         * @interface IMutationChunk
                         * @property {google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.IChunkInfo|null} [chunkInfo] MutationChunk chunkInfo
                         * @property {google.bigtable.v2.IMutation|null} [mutation] MutationChunk mutation
                         */
    
                        /**
                         * Constructs a new MutationChunk.
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse
                         * @classdesc Represents a MutationChunk.
                         * @implements IMutationChunk
                         * @constructor
                         * @param {google.bigtable.v2.ReadChangeStreamResponse.IMutationChunk=} [properties] Properties to set
                         */
                        function MutationChunk(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * MutationChunk chunkInfo.
                         * @member {google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.IChunkInfo|null|undefined} chunkInfo
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.MutationChunk
                         * @instance
                         */
                        MutationChunk.prototype.chunkInfo = null;
    
                        /**
                         * MutationChunk mutation.
                         * @member {google.bigtable.v2.IMutation|null|undefined} mutation
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.MutationChunk
                         * @instance
                         */
                        MutationChunk.prototype.mutation = null;
    
                        /**
                         * Creates a new MutationChunk instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.MutationChunk
                         * @static
                         * @param {google.bigtable.v2.ReadChangeStreamResponse.IMutationChunk=} [properties] Properties to set
                         * @returns {google.bigtable.v2.ReadChangeStreamResponse.MutationChunk} MutationChunk instance
                         */
                        MutationChunk.create = function create(properties) {
                            return new MutationChunk(properties);
                        };
    
                        /**
                         * Encodes the specified MutationChunk message. Does not implicitly {@link google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.MutationChunk
                         * @static
                         * @param {google.bigtable.v2.ReadChangeStreamResponse.IMutationChunk} message MutationChunk message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        MutationChunk.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.chunkInfo != null && Object.hasOwnProperty.call(message, "chunkInfo"))
                                $root.google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.ChunkInfo.encode(message.chunkInfo, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            if (message.mutation != null && Object.hasOwnProperty.call(message, "mutation"))
                                $root.google.bigtable.v2.Mutation.encode(message.mutation, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified MutationChunk message, length delimited. Does not implicitly {@link google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.MutationChunk
                         * @static
                         * @param {google.bigtable.v2.ReadChangeStreamResponse.IMutationChunk} message MutationChunk message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        MutationChunk.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a MutationChunk message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.MutationChunk
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.v2.ReadChangeStreamResponse.MutationChunk} MutationChunk
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        MutationChunk.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.ReadChangeStreamResponse.MutationChunk();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.chunkInfo = $root.google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.ChunkInfo.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 2: {
                                        message.mutation = $root.google.bigtable.v2.Mutation.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a MutationChunk message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.MutationChunk
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.v2.ReadChangeStreamResponse.MutationChunk} MutationChunk
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        MutationChunk.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a MutationChunk message.
                         * @function verify
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.MutationChunk
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        MutationChunk.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.chunkInfo != null && message.hasOwnProperty("chunkInfo")) {
                                var error = $root.google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.ChunkInfo.verify(message.chunkInfo);
                                if (error)
                                    return "chunkInfo." + error;
                            }
                            if (message.mutation != null && message.hasOwnProperty("mutation")) {
                                var error = $root.google.bigtable.v2.Mutation.verify(message.mutation);
                                if (error)
                                    return "mutation." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates a MutationChunk message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.MutationChunk
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.v2.ReadChangeStreamResponse.MutationChunk} MutationChunk
                         */
                        MutationChunk.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.v2.ReadChangeStreamResponse.MutationChunk)
                                return object;
                            var message = new $root.google.bigtable.v2.ReadChangeStreamResponse.MutationChunk();
                            if (object.chunkInfo != null) {
                                if (typeof object.chunkInfo !== "object")
                                    throw TypeError(".google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.chunkInfo: object expected");
                                message.chunkInfo = $root.google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.ChunkInfo.fromObject(object.chunkInfo);
                            }
                            if (object.mutation != null) {
                                if (typeof object.mutation !== "object")
                                    throw TypeError(".google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.mutation: object expected");
                                message.mutation = $root.google.bigtable.v2.Mutation.fromObject(object.mutation);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a MutationChunk message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.MutationChunk
                         * @static
                         * @param {google.bigtable.v2.ReadChangeStreamResponse.MutationChunk} message MutationChunk
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        MutationChunk.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.chunkInfo = null;
                                object.mutation = null;
                            }
                            if (message.chunkInfo != null && message.hasOwnProperty("chunkInfo"))
                                object.chunkInfo = $root.google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.ChunkInfo.toObject(message.chunkInfo, options);
                            if (message.mutation != null && message.hasOwnProperty("mutation"))
                                object.mutation = $root.google.bigtable.v2.Mutation.toObject(message.mutation, options);
                            return object;
                        };
    
                        /**
                         * Converts this MutationChunk to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.MutationChunk
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        MutationChunk.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for MutationChunk
                         * @function getTypeUrl
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.MutationChunk
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        MutationChunk.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.v2.ReadChangeStreamResponse.MutationChunk";
                        };
    
                        MutationChunk.ChunkInfo = (function() {
    
                            /**
                             * Properties of a ChunkInfo.
                             * @memberof google.bigtable.v2.ReadChangeStreamResponse.MutationChunk
                             * @interface IChunkInfo
                             * @property {number|null} [chunkedValueSize] ChunkInfo chunkedValueSize
                             * @property {number|null} [chunkedValueOffset] ChunkInfo chunkedValueOffset
                             * @property {boolean|null} [lastChunk] ChunkInfo lastChunk
                             */
    
                            /**
                             * Constructs a new ChunkInfo.
                             * @memberof google.bigtable.v2.ReadChangeStreamResponse.MutationChunk
                             * @classdesc Represents a ChunkInfo.
                             * @implements IChunkInfo
                             * @constructor
                             * @param {google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.IChunkInfo=} [properties] Properties to set
                             */
                            function ChunkInfo(properties) {
                                if (properties)
                                    for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                        if (properties[keys[i]] != null)
                                            this[keys[i]] = properties[keys[i]];
                            }
    
                            /**
                             * ChunkInfo chunkedValueSize.
                             * @member {number} chunkedValueSize
                             * @memberof google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.ChunkInfo
                             * @instance
                             */
                            ChunkInfo.prototype.chunkedValueSize = 0;
    
                            /**
                             * ChunkInfo chunkedValueOffset.
                             * @member {number} chunkedValueOffset
                             * @memberof google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.ChunkInfo
                             * @instance
                             */
                            ChunkInfo.prototype.chunkedValueOffset = 0;
    
                            /**
                             * ChunkInfo lastChunk.
                             * @member {boolean} lastChunk
                             * @memberof google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.ChunkInfo
                             * @instance
                             */
                            ChunkInfo.prototype.lastChunk = false;
    
                            /**
                             * Creates a new ChunkInfo instance using the specified properties.
                             * @function create
                             * @memberof google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.ChunkInfo
                             * @static
                             * @param {google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.IChunkInfo=} [properties] Properties to set
                             * @returns {google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.ChunkInfo} ChunkInfo instance
                             */
                            ChunkInfo.create = function create(properties) {
                                return new ChunkInfo(properties);
                            };
    
                            /**
                             * Encodes the specified ChunkInfo message. Does not implicitly {@link google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.ChunkInfo.verify|verify} messages.
                             * @function encode
                             * @memberof google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.ChunkInfo
                             * @static
                             * @param {google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.IChunkInfo} message ChunkInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ChunkInfo.encode = function encode(message, writer) {
                                if (!writer)
                                    writer = $Writer.create();
                                if (message.chunkedValueSize != null && Object.hasOwnProperty.call(message, "chunkedValueSize"))
                                    writer.uint32(/* id 1, wireType 0 =*/8).int32(message.chunkedValueSize);
                                if (message.chunkedValueOffset != null && Object.hasOwnProperty.call(message, "chunkedValueOffset"))
                                    writer.uint32(/* id 2, wireType 0 =*/16).int32(message.chunkedValueOffset);
                                if (message.lastChunk != null && Object.hasOwnProperty.call(message, "lastChunk"))
                                    writer.uint32(/* id 3, wireType 0 =*/24).bool(message.lastChunk);
                                return writer;
                            };
    
                            /**
                             * Encodes the specified ChunkInfo message, length delimited. Does not implicitly {@link google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.ChunkInfo.verify|verify} messages.
                             * @function encodeDelimited
                             * @memberof google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.ChunkInfo
                             * @static
                             * @param {google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.IChunkInfo} message ChunkInfo message or plain object to encode
                             * @param {$protobuf.Writer} [writer] Writer to encode to
                             * @returns {$protobuf.Writer} Writer
                             */
                            ChunkInfo.encodeDelimited = function encodeDelimited(message, writer) {
                                return this.encode(message, writer).ldelim();
                            };
    
                            /**
                             * Decodes a ChunkInfo message from the specified reader or buffer.
                             * @function decode
                             * @memberof google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.ChunkInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @param {number} [length] Message length if known beforehand
                             * @returns {google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.ChunkInfo} ChunkInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ChunkInfo.decode = function decode(reader, length) {
                                if (!(reader instanceof $Reader))
                                    reader = $Reader.create(reader);
                                var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.ChunkInfo();
                                while (reader.pos < end) {
                                    var tag = reader.uint32();
                                    switch (tag >>> 3) {
                                    case 1: {
                                            message.chunkedValueSize = reader.int32();
                                            break;
                                        }
                                    case 2: {
                                            message.chunkedValueOffset = reader.int32();
                                            break;
                                        }
                                    case 3: {
                                            message.lastChunk = reader.bool();
                                            break;
                                        }
                                    default:
                                        reader.skipType(tag & 7);
                                        break;
                                    }
                                }
                                return message;
                            };
    
                            /**
                             * Decodes a ChunkInfo message from the specified reader or buffer, length delimited.
                             * @function decodeDelimited
                             * @memberof google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.ChunkInfo
                             * @static
                             * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                             * @returns {google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.ChunkInfo} ChunkInfo
                             * @throws {Error} If the payload is not a reader or valid buffer
                             * @throws {$protobuf.util.ProtocolError} If required fields are missing
                             */
                            ChunkInfo.decodeDelimited = function decodeDelimited(reader) {
                                if (!(reader instanceof $Reader))
                                    reader = new $Reader(reader);
                                return this.decode(reader, reader.uint32());
                            };
    
                            /**
                             * Verifies a ChunkInfo message.
                             * @function verify
                             * @memberof google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.ChunkInfo
                             * @static
                             * @param {Object.<string,*>} message Plain object to verify
                             * @returns {string|null} `null` if valid, otherwise the reason why it is not
                             */
                            ChunkInfo.verify = function verify(message) {
                                if (typeof message !== "object" || message === null)
                                    return "object expected";
                                if (message.chunkedValueSize != null && message.hasOwnProperty("chunkedValueSize"))
                                    if (!$util.isInteger(message.chunkedValueSize))
                                        return "chunkedValueSize: integer expected";
                                if (message.chunkedValueOffset != null && message.hasOwnProperty("chunkedValueOffset"))
                                    if (!$util.isInteger(message.chunkedValueOffset))
                                        return "chunkedValueOffset: integer expected";
                                if (message.lastChunk != null && message.hasOwnProperty("lastChunk"))
                                    if (typeof message.lastChunk !== "boolean")
                                        return "lastChunk: boolean expected";
                                return null;
                            };
    
                            /**
                             * Creates a ChunkInfo message from a plain object. Also converts values to their respective internal types.
                             * @function fromObject
                             * @memberof google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.ChunkInfo
                             * @static
                             * @param {Object.<string,*>} object Plain object
                             * @returns {google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.ChunkInfo} ChunkInfo
                             */
                            ChunkInfo.fromObject = function fromObject(object) {
                                if (object instanceof $root.google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.ChunkInfo)
                                    return object;
                                var message = new $root.google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.ChunkInfo();
                                if (object.chunkedValueSize != null)
                                    message.chunkedValueSize = object.chunkedValueSize | 0;
                                if (object.chunkedValueOffset != null)
                                    message.chunkedValueOffset = object.chunkedValueOffset | 0;
                                if (object.lastChunk != null)
                                    message.lastChunk = Boolean(object.lastChunk);
                                return message;
                            };
    
                            /**
                             * Creates a plain object from a ChunkInfo message. Also converts values to other types if specified.
                             * @function toObject
                             * @memberof google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.ChunkInfo
                             * @static
                             * @param {google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.ChunkInfo} message ChunkInfo
                             * @param {$protobuf.IConversionOptions} [options] Conversion options
                             * @returns {Object.<string,*>} Plain object
                             */
                            ChunkInfo.toObject = function toObject(message, options) {
                                if (!options)
                                    options = {};
                                var object = {};
                                if (options.defaults) {
                                    object.chunkedValueSize = 0;
                                    object.chunkedValueOffset = 0;
                                    object.lastChunk = false;
                                }
                                if (message.chunkedValueSize != null && message.hasOwnProperty("chunkedValueSize"))
                                    object.chunkedValueSize = message.chunkedValueSize;
                                if (message.chunkedValueOffset != null && message.hasOwnProperty("chunkedValueOffset"))
                                    object.chunkedValueOffset = message.chunkedValueOffset;
                                if (message.lastChunk != null && message.hasOwnProperty("lastChunk"))
                                    object.lastChunk = message.lastChunk;
                                return object;
                            };
    
                            /**
                             * Converts this ChunkInfo to JSON.
                             * @function toJSON
                             * @memberof google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.ChunkInfo
                             * @instance
                             * @returns {Object.<string,*>} JSON object
                             */
                            ChunkInfo.prototype.toJSON = function toJSON() {
                                return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                            };
    
                            /**
                             * Gets the default type url for ChunkInfo
                             * @function getTypeUrl
                             * @memberof google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.ChunkInfo
                             * @static
                             * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                             * @returns {string} The default type url
                             */
                            ChunkInfo.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                                if (typeUrlPrefix === undefined) {
                                    typeUrlPrefix = "type.googleapis.com";
                                }
                                return typeUrlPrefix + "/google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.ChunkInfo";
                            };
    
                            return ChunkInfo;
                        })();
    
                        return MutationChunk;
                    })();
    
                    ReadChangeStreamResponse.DataChange = (function() {
    
                        /**
                         * Properties of a DataChange.
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse
                         * @interface IDataChange
                         * @property {google.bigtable.v2.ReadChangeStreamResponse.DataChange.Type|null} [type] DataChange type
                         * @property {string|null} [sourceClusterId] DataChange sourceClusterId
                         * @property {Uint8Array|null} [rowKey] DataChange rowKey
                         * @property {google.protobuf.ITimestamp|null} [commitTimestamp] DataChange commitTimestamp
                         * @property {number|null} [tiebreaker] DataChange tiebreaker
                         * @property {Array.<google.bigtable.v2.ReadChangeStreamResponse.IMutationChunk>|null} [chunks] DataChange chunks
                         * @property {boolean|null} [done] DataChange done
                         * @property {string|null} [token] DataChange token
                         * @property {google.protobuf.ITimestamp|null} [estimatedLowWatermark] DataChange estimatedLowWatermark
                         */
    
                        /**
                         * Constructs a new DataChange.
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse
                         * @classdesc Represents a DataChange.
                         * @implements IDataChange
                         * @constructor
                         * @param {google.bigtable.v2.ReadChangeStreamResponse.IDataChange=} [properties] Properties to set
                         */
                        function DataChange(properties) {
                            this.chunks = [];
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * DataChange type.
                         * @member {google.bigtable.v2.ReadChangeStreamResponse.DataChange.Type} type
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.DataChange
                         * @instance
                         */
                        DataChange.prototype.type = 0;
    
                        /**
                         * DataChange sourceClusterId.
                         * @member {string} sourceClusterId
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.DataChange
                         * @instance
                         */
                        DataChange.prototype.sourceClusterId = "";
    
                        /**
                         * DataChange rowKey.
                         * @member {Uint8Array} rowKey
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.DataChange
                         * @instance
                         */
                        DataChange.prototype.rowKey = $util.newBuffer([]);
    
                        /**
                         * DataChange commitTimestamp.
                         * @member {google.protobuf.ITimestamp|null|undefined} commitTimestamp
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.DataChange
                         * @instance
                         */
                        DataChange.prototype.commitTimestamp = null;
    
                        /**
                         * DataChange tiebreaker.
                         * @member {number} tiebreaker
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.DataChange
                         * @instance
                         */
                        DataChange.prototype.tiebreaker = 0;
    
                        /**
                         * DataChange chunks.
                         * @member {Array.<google.bigtable.v2.ReadChangeStreamResponse.IMutationChunk>} chunks
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.DataChange
                         * @instance
                         */
                        DataChange.prototype.chunks = $util.emptyArray;
    
                        /**
                         * DataChange done.
                         * @member {boolean} done
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.DataChange
                         * @instance
                         */
                        DataChange.prototype.done = false;
    
                        /**
                         * DataChange token.
                         * @member {string} token
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.DataChange
                         * @instance
                         */
                        DataChange.prototype.token = "";
    
                        /**
                         * DataChange estimatedLowWatermark.
                         * @member {google.protobuf.ITimestamp|null|undefined} estimatedLowWatermark
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.DataChange
                         * @instance
                         */
                        DataChange.prototype.estimatedLowWatermark = null;
    
                        /**
                         * Creates a new DataChange instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.DataChange
                         * @static
                         * @param {google.bigtable.v2.ReadChangeStreamResponse.IDataChange=} [properties] Properties to set
                         * @returns {google.bigtable.v2.ReadChangeStreamResponse.DataChange} DataChange instance
                         */
                        DataChange.create = function create(properties) {
                            return new DataChange(properties);
                        };
    
                        /**
                         * Encodes the specified DataChange message. Does not implicitly {@link google.bigtable.v2.ReadChangeStreamResponse.DataChange.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.DataChange
                         * @static
                         * @param {google.bigtable.v2.ReadChangeStreamResponse.IDataChange} message DataChange message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        DataChange.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.type != null && Object.hasOwnProperty.call(message, "type"))
                                writer.uint32(/* id 1, wireType 0 =*/8).int32(message.type);
                            if (message.sourceClusterId != null && Object.hasOwnProperty.call(message, "sourceClusterId"))
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.sourceClusterId);
                            if (message.rowKey != null && Object.hasOwnProperty.call(message, "rowKey"))
                                writer.uint32(/* id 3, wireType 2 =*/26).bytes(message.rowKey);
                            if (message.commitTimestamp != null && Object.hasOwnProperty.call(message, "commitTimestamp"))
                                $root.google.protobuf.Timestamp.encode(message.commitTimestamp, writer.uint32(/* id 4, wireType 2 =*/34).fork()).ldelim();
                            if (message.tiebreaker != null && Object.hasOwnProperty.call(message, "tiebreaker"))
                                writer.uint32(/* id 5, wireType 0 =*/40).int32(message.tiebreaker);
                            if (message.chunks != null && message.chunks.length)
                                for (var i = 0; i < message.chunks.length; ++i)
                                    $root.google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.encode(message.chunks[i], writer.uint32(/* id 6, wireType 2 =*/50).fork()).ldelim();
                            if (message.done != null && Object.hasOwnProperty.call(message, "done"))
                                writer.uint32(/* id 8, wireType 0 =*/64).bool(message.done);
                            if (message.token != null && Object.hasOwnProperty.call(message, "token"))
                                writer.uint32(/* id 9, wireType 2 =*/74).string(message.token);
                            if (message.estimatedLowWatermark != null && Object.hasOwnProperty.call(message, "estimatedLowWatermark"))
                                $root.google.protobuf.Timestamp.encode(message.estimatedLowWatermark, writer.uint32(/* id 10, wireType 2 =*/82).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified DataChange message, length delimited. Does not implicitly {@link google.bigtable.v2.ReadChangeStreamResponse.DataChange.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.DataChange
                         * @static
                         * @param {google.bigtable.v2.ReadChangeStreamResponse.IDataChange} message DataChange message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        DataChange.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a DataChange message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.DataChange
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.v2.ReadChangeStreamResponse.DataChange} DataChange
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        DataChange.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.ReadChangeStreamResponse.DataChange();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.type = reader.int32();
                                        break;
                                    }
                                case 2: {
                                        message.sourceClusterId = reader.string();
                                        break;
                                    }
                                case 3: {
                                        message.rowKey = reader.bytes();
                                        break;
                                    }
                                case 4: {
                                        message.commitTimestamp = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 5: {
                                        message.tiebreaker = reader.int32();
                                        break;
                                    }
                                case 6: {
                                        if (!(message.chunks && message.chunks.length))
                                            message.chunks = [];
                                        message.chunks.push($root.google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.decode(reader, reader.uint32()));
                                        break;
                                    }
                                case 8: {
                                        message.done = reader.bool();
                                        break;
                                    }
                                case 9: {
                                        message.token = reader.string();
                                        break;
                                    }
                                case 10: {
                                        message.estimatedLowWatermark = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a DataChange message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.DataChange
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.v2.ReadChangeStreamResponse.DataChange} DataChange
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        DataChange.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a DataChange message.
                         * @function verify
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.DataChange
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        DataChange.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.type != null && message.hasOwnProperty("type"))
                                switch (message.type) {
                                default:
                                    return "type: enum value expected";
                                case 0:
                                case 1:
                                case 2:
                                case 3:
                                    break;
                                }
                            if (message.sourceClusterId != null && message.hasOwnProperty("sourceClusterId"))
                                if (!$util.isString(message.sourceClusterId))
                                    return "sourceClusterId: string expected";
                            if (message.rowKey != null && message.hasOwnProperty("rowKey"))
                                if (!(message.rowKey && typeof message.rowKey.length === "number" || $util.isString(message.rowKey)))
                                    return "rowKey: buffer expected";
                            if (message.commitTimestamp != null && message.hasOwnProperty("commitTimestamp")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.commitTimestamp);
                                if (error)
                                    return "commitTimestamp." + error;
                            }
                            if (message.tiebreaker != null && message.hasOwnProperty("tiebreaker"))
                                if (!$util.isInteger(message.tiebreaker))
                                    return "tiebreaker: integer expected";
                            if (message.chunks != null && message.hasOwnProperty("chunks")) {
                                if (!Array.isArray(message.chunks))
                                    return "chunks: array expected";
                                for (var i = 0; i < message.chunks.length; ++i) {
                                    var error = $root.google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.verify(message.chunks[i]);
                                    if (error)
                                        return "chunks." + error;
                                }
                            }
                            if (message.done != null && message.hasOwnProperty("done"))
                                if (typeof message.done !== "boolean")
                                    return "done: boolean expected";
                            if (message.token != null && message.hasOwnProperty("token"))
                                if (!$util.isString(message.token))
                                    return "token: string expected";
                            if (message.estimatedLowWatermark != null && message.hasOwnProperty("estimatedLowWatermark")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.estimatedLowWatermark);
                                if (error)
                                    return "estimatedLowWatermark." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates a DataChange message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.DataChange
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.v2.ReadChangeStreamResponse.DataChange} DataChange
                         */
                        DataChange.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.v2.ReadChangeStreamResponse.DataChange)
                                return object;
                            var message = new $root.google.bigtable.v2.ReadChangeStreamResponse.DataChange();
                            switch (object.type) {
                            default:
                                if (typeof object.type === "number") {
                                    message.type = object.type;
                                    break;
                                }
                                break;
                            case "TYPE_UNSPECIFIED":
                            case 0:
                                message.type = 0;
                                break;
                            case "USER":
                            case 1:
                                message.type = 1;
                                break;
                            case "GARBAGE_COLLECTION":
                            case 2:
                                message.type = 2;
                                break;
                            case "CONTINUATION":
                            case 3:
                                message.type = 3;
                                break;
                            }
                            if (object.sourceClusterId != null)
                                message.sourceClusterId = String(object.sourceClusterId);
                            if (object.rowKey != null)
                                if (typeof object.rowKey === "string")
                                    $util.base64.decode(object.rowKey, message.rowKey = $util.newBuffer($util.base64.length(object.rowKey)), 0);
                                else if (object.rowKey.length >= 0)
                                    message.rowKey = object.rowKey;
                            if (object.commitTimestamp != null) {
                                if (typeof object.commitTimestamp !== "object")
                                    throw TypeError(".google.bigtable.v2.ReadChangeStreamResponse.DataChange.commitTimestamp: object expected");
                                message.commitTimestamp = $root.google.protobuf.Timestamp.fromObject(object.commitTimestamp);
                            }
                            if (object.tiebreaker != null)
                                message.tiebreaker = object.tiebreaker | 0;
                            if (object.chunks) {
                                if (!Array.isArray(object.chunks))
                                    throw TypeError(".google.bigtable.v2.ReadChangeStreamResponse.DataChange.chunks: array expected");
                                message.chunks = [];
                                for (var i = 0; i < object.chunks.length; ++i) {
                                    if (typeof object.chunks[i] !== "object")
                                        throw TypeError(".google.bigtable.v2.ReadChangeStreamResponse.DataChange.chunks: object expected");
                                    message.chunks[i] = $root.google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.fromObject(object.chunks[i]);
                                }
                            }
                            if (object.done != null)
                                message.done = Boolean(object.done);
                            if (object.token != null)
                                message.token = String(object.token);
                            if (object.estimatedLowWatermark != null) {
                                if (typeof object.estimatedLowWatermark !== "object")
                                    throw TypeError(".google.bigtable.v2.ReadChangeStreamResponse.DataChange.estimatedLowWatermark: object expected");
                                message.estimatedLowWatermark = $root.google.protobuf.Timestamp.fromObject(object.estimatedLowWatermark);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a DataChange message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.DataChange
                         * @static
                         * @param {google.bigtable.v2.ReadChangeStreamResponse.DataChange} message DataChange
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        DataChange.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.arrays || options.defaults)
                                object.chunks = [];
                            if (options.defaults) {
                                object.type = options.enums === String ? "TYPE_UNSPECIFIED" : 0;
                                object.sourceClusterId = "";
                                if (options.bytes === String)
                                    object.rowKey = "";
                                else {
                                    object.rowKey = [];
                                    if (options.bytes !== Array)
                                        object.rowKey = $util.newBuffer(object.rowKey);
                                }
                                object.commitTimestamp = null;
                                object.tiebreaker = 0;
                                object.done = false;
                                object.token = "";
                                object.estimatedLowWatermark = null;
                            }
                            if (message.type != null && message.hasOwnProperty("type"))
                                object.type = options.enums === String ? $root.google.bigtable.v2.ReadChangeStreamResponse.DataChange.Type[message.type] === undefined ? message.type : $root.google.bigtable.v2.ReadChangeStreamResponse.DataChange.Type[message.type] : message.type;
                            if (message.sourceClusterId != null && message.hasOwnProperty("sourceClusterId"))
                                object.sourceClusterId = message.sourceClusterId;
                            if (message.rowKey != null && message.hasOwnProperty("rowKey"))
                                object.rowKey = options.bytes === String ? $util.base64.encode(message.rowKey, 0, message.rowKey.length) : options.bytes === Array ? Array.prototype.slice.call(message.rowKey) : message.rowKey;
                            if (message.commitTimestamp != null && message.hasOwnProperty("commitTimestamp"))
                                object.commitTimestamp = $root.google.protobuf.Timestamp.toObject(message.commitTimestamp, options);
                            if (message.tiebreaker != null && message.hasOwnProperty("tiebreaker"))
                                object.tiebreaker = message.tiebreaker;
                            if (message.chunks && message.chunks.length) {
                                object.chunks = [];
                                for (var j = 0; j < message.chunks.length; ++j)
                                    object.chunks[j] = $root.google.bigtable.v2.ReadChangeStreamResponse.MutationChunk.toObject(message.chunks[j], options);
                            }
                            if (message.done != null && message.hasOwnProperty("done"))
                                object.done = message.done;
                            if (message.token != null && message.hasOwnProperty("token"))
                                object.token = message.token;
                            if (message.estimatedLowWatermark != null && message.hasOwnProperty("estimatedLowWatermark"))
                                object.estimatedLowWatermark = $root.google.protobuf.Timestamp.toObject(message.estimatedLowWatermark, options);
                            return object;
                        };
    
                        /**
                         * Converts this DataChange to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.DataChange
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        DataChange.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for DataChange
                         * @function getTypeUrl
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.DataChange
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        DataChange.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.v2.ReadChangeStreamResponse.DataChange";
                        };
    
                        /**
                         * Type enum.
                         * @name google.bigtable.v2.ReadChangeStreamResponse.DataChange.Type
                         * @enum {number}
                         * @property {number} TYPE_UNSPECIFIED=0 TYPE_UNSPECIFIED value
                         * @property {number} USER=1 USER value
                         * @property {number} GARBAGE_COLLECTION=2 GARBAGE_COLLECTION value
                         * @property {number} CONTINUATION=3 CONTINUATION value
                         */
                        DataChange.Type = (function() {
                            var valuesById = {}, values = Object.create(valuesById);
                            values[valuesById[0] = "TYPE_UNSPECIFIED"] = 0;
                            values[valuesById[1] = "USER"] = 1;
                            values[valuesById[2] = "GARBAGE_COLLECTION"] = 2;
                            values[valuesById[3] = "CONTINUATION"] = 3;
                            return values;
                        })();
    
                        return DataChange;
                    })();
    
                    ReadChangeStreamResponse.Heartbeat = (function() {
    
                        /**
                         * Properties of a Heartbeat.
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse
                         * @interface IHeartbeat
                         * @property {google.bigtable.v2.IStreamContinuationToken|null} [continuationToken] Heartbeat continuationToken
                         * @property {google.protobuf.ITimestamp|null} [estimatedLowWatermark] Heartbeat estimatedLowWatermark
                         */
    
                        /**
                         * Constructs a new Heartbeat.
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse
                         * @classdesc Represents a Heartbeat.
                         * @implements IHeartbeat
                         * @constructor
                         * @param {google.bigtable.v2.ReadChangeStreamResponse.IHeartbeat=} [properties] Properties to set
                         */
                        function Heartbeat(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * Heartbeat continuationToken.
                         * @member {google.bigtable.v2.IStreamContinuationToken|null|undefined} continuationToken
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.Heartbeat
                         * @instance
                         */
                        Heartbeat.prototype.continuationToken = null;
    
                        /**
                         * Heartbeat estimatedLowWatermark.
                         * @member {google.protobuf.ITimestamp|null|undefined} estimatedLowWatermark
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.Heartbeat
                         * @instance
                         */
                        Heartbeat.prototype.estimatedLowWatermark = null;
    
                        /**
                         * Creates a new Heartbeat instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.Heartbeat
                         * @static
                         * @param {google.bigtable.v2.ReadChangeStreamResponse.IHeartbeat=} [properties] Properties to set
                         * @returns {google.bigtable.v2.ReadChangeStreamResponse.Heartbeat} Heartbeat instance
                         */
                        Heartbeat.create = function create(properties) {
                            return new Heartbeat(properties);
                        };
    
                        /**
                         * Encodes the specified Heartbeat message. Does not implicitly {@link google.bigtable.v2.ReadChangeStreamResponse.Heartbeat.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.Heartbeat
                         * @static
                         * @param {google.bigtable.v2.ReadChangeStreamResponse.IHeartbeat} message Heartbeat message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        Heartbeat.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.continuationToken != null && Object.hasOwnProperty.call(message, "continuationToken"))
                                $root.google.bigtable.v2.StreamContinuationToken.encode(message.continuationToken, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            if (message.estimatedLowWatermark != null && Object.hasOwnProperty.call(message, "estimatedLowWatermark"))
                                $root.google.protobuf.Timestamp.encode(message.estimatedLowWatermark, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified Heartbeat message, length delimited. Does not implicitly {@link google.bigtable.v2.ReadChangeStreamResponse.Heartbeat.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.Heartbeat
                         * @static
                         * @param {google.bigtable.v2.ReadChangeStreamResponse.IHeartbeat} message Heartbeat message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        Heartbeat.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a Heartbeat message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.Heartbeat
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.v2.ReadChangeStreamResponse.Heartbeat} Heartbeat
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        Heartbeat.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.ReadChangeStreamResponse.Heartbeat();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.continuationToken = $root.google.bigtable.v2.StreamContinuationToken.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 2: {
                                        message.estimatedLowWatermark = $root.google.protobuf.Timestamp.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a Heartbeat message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.Heartbeat
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.v2.ReadChangeStreamResponse.Heartbeat} Heartbeat
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        Heartbeat.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a Heartbeat message.
                         * @function verify
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.Heartbeat
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        Heartbeat.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.continuationToken != null && message.hasOwnProperty("continuationToken")) {
                                var error = $root.google.bigtable.v2.StreamContinuationToken.verify(message.continuationToken);
                                if (error)
                                    return "continuationToken." + error;
                            }
                            if (message.estimatedLowWatermark != null && message.hasOwnProperty("estimatedLowWatermark")) {
                                var error = $root.google.protobuf.Timestamp.verify(message.estimatedLowWatermark);
                                if (error)
                                    return "estimatedLowWatermark." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates a Heartbeat message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.Heartbeat
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.v2.ReadChangeStreamResponse.Heartbeat} Heartbeat
                         */
                        Heartbeat.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.v2.ReadChangeStreamResponse.Heartbeat)
                                return object;
                            var message = new $root.google.bigtable.v2.ReadChangeStreamResponse.Heartbeat();
                            if (object.continuationToken != null) {
                                if (typeof object.continuationToken !== "object")
                                    throw TypeError(".google.bigtable.v2.ReadChangeStreamResponse.Heartbeat.continuationToken: object expected");
                                message.continuationToken = $root.google.bigtable.v2.StreamContinuationToken.fromObject(object.continuationToken);
                            }
                            if (object.estimatedLowWatermark != null) {
                                if (typeof object.estimatedLowWatermark !== "object")
                                    throw TypeError(".google.bigtable.v2.ReadChangeStreamResponse.Heartbeat.estimatedLowWatermark: object expected");
                                message.estimatedLowWatermark = $root.google.protobuf.Timestamp.fromObject(object.estimatedLowWatermark);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a Heartbeat message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.Heartbeat
                         * @static
                         * @param {google.bigtable.v2.ReadChangeStreamResponse.Heartbeat} message Heartbeat
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        Heartbeat.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.continuationToken = null;
                                object.estimatedLowWatermark = null;
                            }
                            if (message.continuationToken != null && message.hasOwnProperty("continuationToken"))
                                object.continuationToken = $root.google.bigtable.v2.StreamContinuationToken.toObject(message.continuationToken, options);
                            if (message.estimatedLowWatermark != null && message.hasOwnProperty("estimatedLowWatermark"))
                                object.estimatedLowWatermark = $root.google.protobuf.Timestamp.toObject(message.estimatedLowWatermark, options);
                            return object;
                        };
    
                        /**
                         * Converts this Heartbeat to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.Heartbeat
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        Heartbeat.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for Heartbeat
                         * @function getTypeUrl
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.Heartbeat
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        Heartbeat.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.v2.ReadChangeStreamResponse.Heartbeat";
                        };
    
                        return Heartbeat;
                    })();
    
                    ReadChangeStreamResponse.CloseStream = (function() {
    
                        /**
                         * Properties of a CloseStream.
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse
                         * @interface ICloseStream
                         * @property {google.rpc.IStatus|null} [status] CloseStream status
                         * @property {Array.<google.bigtable.v2.IStreamContinuationToken>|null} [continuationTokens] CloseStream continuationTokens
                         * @property {Array.<google.bigtable.v2.IStreamPartition>|null} [newPartitions] CloseStream newPartitions
                         */
    
                        /**
                         * Constructs a new CloseStream.
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse
                         * @classdesc Represents a CloseStream.
                         * @implements ICloseStream
                         * @constructor
                         * @param {google.bigtable.v2.ReadChangeStreamResponse.ICloseStream=} [properties] Properties to set
                         */
                        function CloseStream(properties) {
                            this.continuationTokens = [];
                            this.newPartitions = [];
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * CloseStream status.
                         * @member {google.rpc.IStatus|null|undefined} status
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.CloseStream
                         * @instance
                         */
                        CloseStream.prototype.status = null;
    
                        /**
                         * CloseStream continuationTokens.
                         * @member {Array.<google.bigtable.v2.IStreamContinuationToken>} continuationTokens
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.CloseStream
                         * @instance
                         */
                        CloseStream.prototype.continuationTokens = $util.emptyArray;
    
                        /**
                         * CloseStream newPartitions.
                         * @member {Array.<google.bigtable.v2.IStreamPartition>} newPartitions
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.CloseStream
                         * @instance
                         */
                        CloseStream.prototype.newPartitions = $util.emptyArray;
    
                        /**
                         * Creates a new CloseStream instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.CloseStream
                         * @static
                         * @param {google.bigtable.v2.ReadChangeStreamResponse.ICloseStream=} [properties] Properties to set
                         * @returns {google.bigtable.v2.ReadChangeStreamResponse.CloseStream} CloseStream instance
                         */
                        CloseStream.create = function create(properties) {
                            return new CloseStream(properties);
                        };
    
                        /**
                         * Encodes the specified CloseStream message. Does not implicitly {@link google.bigtable.v2.ReadChangeStreamResponse.CloseStream.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.CloseStream
                         * @static
                         * @param {google.bigtable.v2.ReadChangeStreamResponse.ICloseStream} message CloseStream message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CloseStream.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.status != null && Object.hasOwnProperty.call(message, "status"))
                                $root.google.rpc.Status.encode(message.status, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            if (message.continuationTokens != null && message.continuationTokens.length)
                                for (var i = 0; i < message.continuationTokens.length; ++i)
                                    $root.google.bigtable.v2.StreamContinuationToken.encode(message.continuationTokens[i], writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            if (message.newPartitions != null && message.newPartitions.length)
                                for (var i = 0; i < message.newPartitions.length; ++i)
                                    $root.google.bigtable.v2.StreamPartition.encode(message.newPartitions[i], writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified CloseStream message, length delimited. Does not implicitly {@link google.bigtable.v2.ReadChangeStreamResponse.CloseStream.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.CloseStream
                         * @static
                         * @param {google.bigtable.v2.ReadChangeStreamResponse.ICloseStream} message CloseStream message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        CloseStream.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a CloseStream message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.CloseStream
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.v2.ReadChangeStreamResponse.CloseStream} CloseStream
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CloseStream.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.ReadChangeStreamResponse.CloseStream();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.status = $root.google.rpc.Status.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 2: {
                                        if (!(message.continuationTokens && message.continuationTokens.length))
                                            message.continuationTokens = [];
                                        message.continuationTokens.push($root.google.bigtable.v2.StreamContinuationToken.decode(reader, reader.uint32()));
                                        break;
                                    }
                                case 3: {
                                        if (!(message.newPartitions && message.newPartitions.length))
                                            message.newPartitions = [];
                                        message.newPartitions.push($root.google.bigtable.v2.StreamPartition.decode(reader, reader.uint32()));
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a CloseStream message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.CloseStream
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.v2.ReadChangeStreamResponse.CloseStream} CloseStream
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        CloseStream.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a CloseStream message.
                         * @function verify
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.CloseStream
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        CloseStream.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.status != null && message.hasOwnProperty("status")) {
                                var error = $root.google.rpc.Status.verify(message.status);
                                if (error)
                                    return "status." + error;
                            }
                            if (message.continuationTokens != null && message.hasOwnProperty("continuationTokens")) {
                                if (!Array.isArray(message.continuationTokens))
                                    return "continuationTokens: array expected";
                                for (var i = 0; i < message.continuationTokens.length; ++i) {
                                    var error = $root.google.bigtable.v2.StreamContinuationToken.verify(message.continuationTokens[i]);
                                    if (error)
                                        return "continuationTokens." + error;
                                }
                            }
                            if (message.newPartitions != null && message.hasOwnProperty("newPartitions")) {
                                if (!Array.isArray(message.newPartitions))
                                    return "newPartitions: array expected";
                                for (var i = 0; i < message.newPartitions.length; ++i) {
                                    var error = $root.google.bigtable.v2.StreamPartition.verify(message.newPartitions[i]);
                                    if (error)
                                        return "newPartitions." + error;
                                }
                            }
                            return null;
                        };
    
                        /**
                         * Creates a CloseStream message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.CloseStream
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.v2.ReadChangeStreamResponse.CloseStream} CloseStream
                         */
                        CloseStream.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.v2.ReadChangeStreamResponse.CloseStream)
                                return object;
                            var message = new $root.google.bigtable.v2.ReadChangeStreamResponse.CloseStream();
                            if (object.status != null) {
                                if (typeof object.status !== "object")
                                    throw TypeError(".google.bigtable.v2.ReadChangeStreamResponse.CloseStream.status: object expected");
                                message.status = $root.google.rpc.Status.fromObject(object.status);
                            }
                            if (object.continuationTokens) {
                                if (!Array.isArray(object.continuationTokens))
                                    throw TypeError(".google.bigtable.v2.ReadChangeStreamResponse.CloseStream.continuationTokens: array expected");
                                message.continuationTokens = [];
                                for (var i = 0; i < object.continuationTokens.length; ++i) {
                                    if (typeof object.continuationTokens[i] !== "object")
                                        throw TypeError(".google.bigtable.v2.ReadChangeStreamResponse.CloseStream.continuationTokens: object expected");
                                    message.continuationTokens[i] = $root.google.bigtable.v2.StreamContinuationToken.fromObject(object.continuationTokens[i]);
                                }
                            }
                            if (object.newPartitions) {
                                if (!Array.isArray(object.newPartitions))
                                    throw TypeError(".google.bigtable.v2.ReadChangeStreamResponse.CloseStream.newPartitions: array expected");
                                message.newPartitions = [];
                                for (var i = 0; i < object.newPartitions.length; ++i) {
                                    if (typeof object.newPartitions[i] !== "object")
                                        throw TypeError(".google.bigtable.v2.ReadChangeStreamResponse.CloseStream.newPartitions: object expected");
                                    message.newPartitions[i] = $root.google.bigtable.v2.StreamPartition.fromObject(object.newPartitions[i]);
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a CloseStream message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.CloseStream
                         * @static
                         * @param {google.bigtable.v2.ReadChangeStreamResponse.CloseStream} message CloseStream
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        CloseStream.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.arrays || options.defaults) {
                                object.continuationTokens = [];
                                object.newPartitions = [];
                            }
                            if (options.defaults)
                                object.status = null;
                            if (message.status != null && message.hasOwnProperty("status"))
                                object.status = $root.google.rpc.Status.toObject(message.status, options);
                            if (message.continuationTokens && message.continuationTokens.length) {
                                object.continuationTokens = [];
                                for (var j = 0; j < message.continuationTokens.length; ++j)
                                    object.continuationTokens[j] = $root.google.bigtable.v2.StreamContinuationToken.toObject(message.continuationTokens[j], options);
                            }
                            if (message.newPartitions && message.newPartitions.length) {
                                object.newPartitions = [];
                                for (var j = 0; j < message.newPartitions.length; ++j)
                                    object.newPartitions[j] = $root.google.bigtable.v2.StreamPartition.toObject(message.newPartitions[j], options);
                            }
                            return object;
                        };
    
                        /**
                         * Converts this CloseStream to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.CloseStream
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        CloseStream.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for CloseStream
                         * @function getTypeUrl
                         * @memberof google.bigtable.v2.ReadChangeStreamResponse.CloseStream
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        CloseStream.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.v2.ReadChangeStreamResponse.CloseStream";
                        };
    
                        return CloseStream;
                    })();
    
                    return ReadChangeStreamResponse;
                })();
    
                v2.Row = (function() {
    
                    /**
                     * Properties of a Row.
                     * @memberof google.bigtable.v2
                     * @interface IRow
                     * @property {Uint8Array|null} [key] Row key
                     * @property {Array.<google.bigtable.v2.IFamily>|null} [families] Row families
                     */
    
                    /**
                     * Constructs a new Row.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a Row.
                     * @implements IRow
                     * @constructor
                     * @param {google.bigtable.v2.IRow=} [properties] Properties to set
                     */
                    function Row(properties) {
                        this.families = [];
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * Row key.
                     * @member {Uint8Array} key
                     * @memberof google.bigtable.v2.Row
                     * @instance
                     */
                    Row.prototype.key = $util.newBuffer([]);
    
                    /**
                     * Row families.
                     * @member {Array.<google.bigtable.v2.IFamily>} families
                     * @memberof google.bigtable.v2.Row
                     * @instance
                     */
                    Row.prototype.families = $util.emptyArray;
    
                    /**
                     * Creates a new Row instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.Row
                     * @static
                     * @param {google.bigtable.v2.IRow=} [properties] Properties to set
                     * @returns {google.bigtable.v2.Row} Row instance
                     */
                    Row.create = function create(properties) {
                        return new Row(properties);
                    };
    
                    /**
                     * Encodes the specified Row message. Does not implicitly {@link google.bigtable.v2.Row.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.Row
                     * @static
                     * @param {google.bigtable.v2.IRow} message Row message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    Row.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.key != null && Object.hasOwnProperty.call(message, "key"))
                            writer.uint32(/* id 1, wireType 2 =*/10).bytes(message.key);
                        if (message.families != null && message.families.length)
                            for (var i = 0; i < message.families.length; ++i)
                                $root.google.bigtable.v2.Family.encode(message.families[i], writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                        return writer;
                    };
    
                    /**
                     * Encodes the specified Row message, length delimited. Does not implicitly {@link google.bigtable.v2.Row.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.Row
                     * @static
                     * @param {google.bigtable.v2.IRow} message Row message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    Row.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a Row message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.Row
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.Row} Row
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    Row.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.Row();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.key = reader.bytes();
                                    break;
                                }
                            case 2: {
                                    if (!(message.families && message.families.length))
                                        message.families = [];
                                    message.families.push($root.google.bigtable.v2.Family.decode(reader, reader.uint32()));
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a Row message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.Row
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.Row} Row
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    Row.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a Row message.
                     * @function verify
                     * @memberof google.bigtable.v2.Row
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    Row.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.key != null && message.hasOwnProperty("key"))
                            if (!(message.key && typeof message.key.length === "number" || $util.isString(message.key)))
                                return "key: buffer expected";
                        if (message.families != null && message.hasOwnProperty("families")) {
                            if (!Array.isArray(message.families))
                                return "families: array expected";
                            for (var i = 0; i < message.families.length; ++i) {
                                var error = $root.google.bigtable.v2.Family.verify(message.families[i]);
                                if (error)
                                    return "families." + error;
                            }
                        }
                        return null;
                    };
    
                    /**
                     * Creates a Row message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.Row
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.Row} Row
                     */
                    Row.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.Row)
                            return object;
                        var message = new $root.google.bigtable.v2.Row();
                        if (object.key != null)
                            if (typeof object.key === "string")
                                $util.base64.decode(object.key, message.key = $util.newBuffer($util.base64.length(object.key)), 0);
                            else if (object.key.length >= 0)
                                message.key = object.key;
                        if (object.families) {
                            if (!Array.isArray(object.families))
                                throw TypeError(".google.bigtable.v2.Row.families: array expected");
                            message.families = [];
                            for (var i = 0; i < object.families.length; ++i) {
                                if (typeof object.families[i] !== "object")
                                    throw TypeError(".google.bigtable.v2.Row.families: object expected");
                                message.families[i] = $root.google.bigtable.v2.Family.fromObject(object.families[i]);
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a Row message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.Row
                     * @static
                     * @param {google.bigtable.v2.Row} message Row
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    Row.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.arrays || options.defaults)
                            object.families = [];
                        if (options.defaults)
                            if (options.bytes === String)
                                object.key = "";
                            else {
                                object.key = [];
                                if (options.bytes !== Array)
                                    object.key = $util.newBuffer(object.key);
                            }
                        if (message.key != null && message.hasOwnProperty("key"))
                            object.key = options.bytes === String ? $util.base64.encode(message.key, 0, message.key.length) : options.bytes === Array ? Array.prototype.slice.call(message.key) : message.key;
                        if (message.families && message.families.length) {
                            object.families = [];
                            for (var j = 0; j < message.families.length; ++j)
                                object.families[j] = $root.google.bigtable.v2.Family.toObject(message.families[j], options);
                        }
                        return object;
                    };
    
                    /**
                     * Converts this Row to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.Row
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    Row.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for Row
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.Row
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    Row.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.Row";
                    };
    
                    return Row;
                })();
    
                v2.Family = (function() {
    
                    /**
                     * Properties of a Family.
                     * @memberof google.bigtable.v2
                     * @interface IFamily
                     * @property {string|null} [name] Family name
                     * @property {Array.<google.bigtable.v2.IColumn>|null} [columns] Family columns
                     */
    
                    /**
                     * Constructs a new Family.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a Family.
                     * @implements IFamily
                     * @constructor
                     * @param {google.bigtable.v2.IFamily=} [properties] Properties to set
                     */
                    function Family(properties) {
                        this.columns = [];
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * Family name.
                     * @member {string} name
                     * @memberof google.bigtable.v2.Family
                     * @instance
                     */
                    Family.prototype.name = "";
    
                    /**
                     * Family columns.
                     * @member {Array.<google.bigtable.v2.IColumn>} columns
                     * @memberof google.bigtable.v2.Family
                     * @instance
                     */
                    Family.prototype.columns = $util.emptyArray;
    
                    /**
                     * Creates a new Family instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.Family
                     * @static
                     * @param {google.bigtable.v2.IFamily=} [properties] Properties to set
                     * @returns {google.bigtable.v2.Family} Family instance
                     */
                    Family.create = function create(properties) {
                        return new Family(properties);
                    };
    
                    /**
                     * Encodes the specified Family message. Does not implicitly {@link google.bigtable.v2.Family.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.Family
                     * @static
                     * @param {google.bigtable.v2.IFamily} message Family message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    Family.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                            writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                        if (message.columns != null && message.columns.length)
                            for (var i = 0; i < message.columns.length; ++i)
                                $root.google.bigtable.v2.Column.encode(message.columns[i], writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                        return writer;
                    };
    
                    /**
                     * Encodes the specified Family message, length delimited. Does not implicitly {@link google.bigtable.v2.Family.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.Family
                     * @static
                     * @param {google.bigtable.v2.IFamily} message Family message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    Family.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a Family message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.Family
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.Family} Family
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    Family.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.Family();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.name = reader.string();
                                    break;
                                }
                            case 2: {
                                    if (!(message.columns && message.columns.length))
                                        message.columns = [];
                                    message.columns.push($root.google.bigtable.v2.Column.decode(reader, reader.uint32()));
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a Family message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.Family
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.Family} Family
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    Family.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a Family message.
                     * @function verify
                     * @memberof google.bigtable.v2.Family
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    Family.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.name != null && message.hasOwnProperty("name"))
                            if (!$util.isString(message.name))
                                return "name: string expected";
                        if (message.columns != null && message.hasOwnProperty("columns")) {
                            if (!Array.isArray(message.columns))
                                return "columns: array expected";
                            for (var i = 0; i < message.columns.length; ++i) {
                                var error = $root.google.bigtable.v2.Column.verify(message.columns[i]);
                                if (error)
                                    return "columns." + error;
                            }
                        }
                        return null;
                    };
    
                    /**
                     * Creates a Family message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.Family
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.Family} Family
                     */
                    Family.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.Family)
                            return object;
                        var message = new $root.google.bigtable.v2.Family();
                        if (object.name != null)
                            message.name = String(object.name);
                        if (object.columns) {
                            if (!Array.isArray(object.columns))
                                throw TypeError(".google.bigtable.v2.Family.columns: array expected");
                            message.columns = [];
                            for (var i = 0; i < object.columns.length; ++i) {
                                if (typeof object.columns[i] !== "object")
                                    throw TypeError(".google.bigtable.v2.Family.columns: object expected");
                                message.columns[i] = $root.google.bigtable.v2.Column.fromObject(object.columns[i]);
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a Family message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.Family
                     * @static
                     * @param {google.bigtable.v2.Family} message Family
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    Family.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.arrays || options.defaults)
                            object.columns = [];
                        if (options.defaults)
                            object.name = "";
                        if (message.name != null && message.hasOwnProperty("name"))
                            object.name = message.name;
                        if (message.columns && message.columns.length) {
                            object.columns = [];
                            for (var j = 0; j < message.columns.length; ++j)
                                object.columns[j] = $root.google.bigtable.v2.Column.toObject(message.columns[j], options);
                        }
                        return object;
                    };
    
                    /**
                     * Converts this Family to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.Family
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    Family.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for Family
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.Family
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    Family.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.Family";
                    };
    
                    return Family;
                })();
    
                v2.Column = (function() {
    
                    /**
                     * Properties of a Column.
                     * @memberof google.bigtable.v2
                     * @interface IColumn
                     * @property {Uint8Array|null} [qualifier] Column qualifier
                     * @property {Array.<google.bigtable.v2.ICell>|null} [cells] Column cells
                     */
    
                    /**
                     * Constructs a new Column.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a Column.
                     * @implements IColumn
                     * @constructor
                     * @param {google.bigtable.v2.IColumn=} [properties] Properties to set
                     */
                    function Column(properties) {
                        this.cells = [];
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * Column qualifier.
                     * @member {Uint8Array} qualifier
                     * @memberof google.bigtable.v2.Column
                     * @instance
                     */
                    Column.prototype.qualifier = $util.newBuffer([]);
    
                    /**
                     * Column cells.
                     * @member {Array.<google.bigtable.v2.ICell>} cells
                     * @memberof google.bigtable.v2.Column
                     * @instance
                     */
                    Column.prototype.cells = $util.emptyArray;
    
                    /**
                     * Creates a new Column instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.Column
                     * @static
                     * @param {google.bigtable.v2.IColumn=} [properties] Properties to set
                     * @returns {google.bigtable.v2.Column} Column instance
                     */
                    Column.create = function create(properties) {
                        return new Column(properties);
                    };
    
                    /**
                     * Encodes the specified Column message. Does not implicitly {@link google.bigtable.v2.Column.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.Column
                     * @static
                     * @param {google.bigtable.v2.IColumn} message Column message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    Column.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.qualifier != null && Object.hasOwnProperty.call(message, "qualifier"))
                            writer.uint32(/* id 1, wireType 2 =*/10).bytes(message.qualifier);
                        if (message.cells != null && message.cells.length)
                            for (var i = 0; i < message.cells.length; ++i)
                                $root.google.bigtable.v2.Cell.encode(message.cells[i], writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                        return writer;
                    };
    
                    /**
                     * Encodes the specified Column message, length delimited. Does not implicitly {@link google.bigtable.v2.Column.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.Column
                     * @static
                     * @param {google.bigtable.v2.IColumn} message Column message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    Column.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a Column message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.Column
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.Column} Column
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    Column.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.Column();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.qualifier = reader.bytes();
                                    break;
                                }
                            case 2: {
                                    if (!(message.cells && message.cells.length))
                                        message.cells = [];
                                    message.cells.push($root.google.bigtable.v2.Cell.decode(reader, reader.uint32()));
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a Column message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.Column
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.Column} Column
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    Column.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a Column message.
                     * @function verify
                     * @memberof google.bigtable.v2.Column
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    Column.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.qualifier != null && message.hasOwnProperty("qualifier"))
                            if (!(message.qualifier && typeof message.qualifier.length === "number" || $util.isString(message.qualifier)))
                                return "qualifier: buffer expected";
                        if (message.cells != null && message.hasOwnProperty("cells")) {
                            if (!Array.isArray(message.cells))
                                return "cells: array expected";
                            for (var i = 0; i < message.cells.length; ++i) {
                                var error = $root.google.bigtable.v2.Cell.verify(message.cells[i]);
                                if (error)
                                    return "cells." + error;
                            }
                        }
                        return null;
                    };
    
                    /**
                     * Creates a Column message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.Column
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.Column} Column
                     */
                    Column.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.Column)
                            return object;
                        var message = new $root.google.bigtable.v2.Column();
                        if (object.qualifier != null)
                            if (typeof object.qualifier === "string")
                                $util.base64.decode(object.qualifier, message.qualifier = $util.newBuffer($util.base64.length(object.qualifier)), 0);
                            else if (object.qualifier.length >= 0)
                                message.qualifier = object.qualifier;
                        if (object.cells) {
                            if (!Array.isArray(object.cells))
                                throw TypeError(".google.bigtable.v2.Column.cells: array expected");
                            message.cells = [];
                            for (var i = 0; i < object.cells.length; ++i) {
                                if (typeof object.cells[i] !== "object")
                                    throw TypeError(".google.bigtable.v2.Column.cells: object expected");
                                message.cells[i] = $root.google.bigtable.v2.Cell.fromObject(object.cells[i]);
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a Column message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.Column
                     * @static
                     * @param {google.bigtable.v2.Column} message Column
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    Column.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.arrays || options.defaults)
                            object.cells = [];
                        if (options.defaults)
                            if (options.bytes === String)
                                object.qualifier = "";
                            else {
                                object.qualifier = [];
                                if (options.bytes !== Array)
                                    object.qualifier = $util.newBuffer(object.qualifier);
                            }
                        if (message.qualifier != null && message.hasOwnProperty("qualifier"))
                            object.qualifier = options.bytes === String ? $util.base64.encode(message.qualifier, 0, message.qualifier.length) : options.bytes === Array ? Array.prototype.slice.call(message.qualifier) : message.qualifier;
                        if (message.cells && message.cells.length) {
                            object.cells = [];
                            for (var j = 0; j < message.cells.length; ++j)
                                object.cells[j] = $root.google.bigtable.v2.Cell.toObject(message.cells[j], options);
                        }
                        return object;
                    };
    
                    /**
                     * Converts this Column to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.Column
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    Column.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for Column
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.Column
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    Column.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.Column";
                    };
    
                    return Column;
                })();
    
                v2.Cell = (function() {
    
                    /**
                     * Properties of a Cell.
                     * @memberof google.bigtable.v2
                     * @interface ICell
                     * @property {number|Long|null} [timestampMicros] Cell timestampMicros
                     * @property {Uint8Array|null} [value] Cell value
                     * @property {Array.<string>|null} [labels] Cell labels
                     */
    
                    /**
                     * Constructs a new Cell.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a Cell.
                     * @implements ICell
                     * @constructor
                     * @param {google.bigtable.v2.ICell=} [properties] Properties to set
                     */
                    function Cell(properties) {
                        this.labels = [];
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * Cell timestampMicros.
                     * @member {number|Long} timestampMicros
                     * @memberof google.bigtable.v2.Cell
                     * @instance
                     */
                    Cell.prototype.timestampMicros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                    /**
                     * Cell value.
                     * @member {Uint8Array} value
                     * @memberof google.bigtable.v2.Cell
                     * @instance
                     */
                    Cell.prototype.value = $util.newBuffer([]);
    
                    /**
                     * Cell labels.
                     * @member {Array.<string>} labels
                     * @memberof google.bigtable.v2.Cell
                     * @instance
                     */
                    Cell.prototype.labels = $util.emptyArray;
    
                    /**
                     * Creates a new Cell instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.Cell
                     * @static
                     * @param {google.bigtable.v2.ICell=} [properties] Properties to set
                     * @returns {google.bigtable.v2.Cell} Cell instance
                     */
                    Cell.create = function create(properties) {
                        return new Cell(properties);
                    };
    
                    /**
                     * Encodes the specified Cell message. Does not implicitly {@link google.bigtable.v2.Cell.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.Cell
                     * @static
                     * @param {google.bigtable.v2.ICell} message Cell message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    Cell.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.timestampMicros != null && Object.hasOwnProperty.call(message, "timestampMicros"))
                            writer.uint32(/* id 1, wireType 0 =*/8).int64(message.timestampMicros);
                        if (message.value != null && Object.hasOwnProperty.call(message, "value"))
                            writer.uint32(/* id 2, wireType 2 =*/18).bytes(message.value);
                        if (message.labels != null && message.labels.length)
                            for (var i = 0; i < message.labels.length; ++i)
                                writer.uint32(/* id 3, wireType 2 =*/26).string(message.labels[i]);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified Cell message, length delimited. Does not implicitly {@link google.bigtable.v2.Cell.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.Cell
                     * @static
                     * @param {google.bigtable.v2.ICell} message Cell message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    Cell.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a Cell message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.Cell
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.Cell} Cell
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    Cell.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.Cell();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.timestampMicros = reader.int64();
                                    break;
                                }
                            case 2: {
                                    message.value = reader.bytes();
                                    break;
                                }
                            case 3: {
                                    if (!(message.labels && message.labels.length))
                                        message.labels = [];
                                    message.labels.push(reader.string());
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a Cell message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.Cell
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.Cell} Cell
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    Cell.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a Cell message.
                     * @function verify
                     * @memberof google.bigtable.v2.Cell
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    Cell.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.timestampMicros != null && message.hasOwnProperty("timestampMicros"))
                            if (!$util.isInteger(message.timestampMicros) && !(message.timestampMicros && $util.isInteger(message.timestampMicros.low) && $util.isInteger(message.timestampMicros.high)))
                                return "timestampMicros: integer|Long expected";
                        if (message.value != null && message.hasOwnProperty("value"))
                            if (!(message.value && typeof message.value.length === "number" || $util.isString(message.value)))
                                return "value: buffer expected";
                        if (message.labels != null && message.hasOwnProperty("labels")) {
                            if (!Array.isArray(message.labels))
                                return "labels: array expected";
                            for (var i = 0; i < message.labels.length; ++i)
                                if (!$util.isString(message.labels[i]))
                                    return "labels: string[] expected";
                        }
                        return null;
                    };
    
                    /**
                     * Creates a Cell message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.Cell
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.Cell} Cell
                     */
                    Cell.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.Cell)
                            return object;
                        var message = new $root.google.bigtable.v2.Cell();
                        if (object.timestampMicros != null)
                            if ($util.Long)
                                (message.timestampMicros = $util.Long.fromValue(object.timestampMicros)).unsigned = false;
                            else if (typeof object.timestampMicros === "string")
                                message.timestampMicros = parseInt(object.timestampMicros, 10);
                            else if (typeof object.timestampMicros === "number")
                                message.timestampMicros = object.timestampMicros;
                            else if (typeof object.timestampMicros === "object")
                                message.timestampMicros = new $util.LongBits(object.timestampMicros.low >>> 0, object.timestampMicros.high >>> 0).toNumber();
                        if (object.value != null)
                            if (typeof object.value === "string")
                                $util.base64.decode(object.value, message.value = $util.newBuffer($util.base64.length(object.value)), 0);
                            else if (object.value.length >= 0)
                                message.value = object.value;
                        if (object.labels) {
                            if (!Array.isArray(object.labels))
                                throw TypeError(".google.bigtable.v2.Cell.labels: array expected");
                            message.labels = [];
                            for (var i = 0; i < object.labels.length; ++i)
                                message.labels[i] = String(object.labels[i]);
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a Cell message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.Cell
                     * @static
                     * @param {google.bigtable.v2.Cell} message Cell
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    Cell.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.arrays || options.defaults)
                            object.labels = [];
                        if (options.defaults) {
                            if ($util.Long) {
                                var long = new $util.Long(0, 0, false);
                                object.timestampMicros = options.longs === String ? long.toString() : options.longs === Number ? long.toNumber() : long;
                            } else
                                object.timestampMicros = options.longs === String ? "0" : 0;
                            if (options.bytes === String)
                                object.value = "";
                            else {
                                object.value = [];
                                if (options.bytes !== Array)
                                    object.value = $util.newBuffer(object.value);
                            }
                        }
                        if (message.timestampMicros != null && message.hasOwnProperty("timestampMicros"))
                            if (typeof message.timestampMicros === "number")
                                object.timestampMicros = options.longs === String ? String(message.timestampMicros) : message.timestampMicros;
                            else
                                object.timestampMicros = options.longs === String ? $util.Long.prototype.toString.call(message.timestampMicros) : options.longs === Number ? new $util.LongBits(message.timestampMicros.low >>> 0, message.timestampMicros.high >>> 0).toNumber() : message.timestampMicros;
                        if (message.value != null && message.hasOwnProperty("value"))
                            object.value = options.bytes === String ? $util.base64.encode(message.value, 0, message.value.length) : options.bytes === Array ? Array.prototype.slice.call(message.value) : message.value;
                        if (message.labels && message.labels.length) {
                            object.labels = [];
                            for (var j = 0; j < message.labels.length; ++j)
                                object.labels[j] = message.labels[j];
                        }
                        return object;
                    };
    
                    /**
                     * Converts this Cell to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.Cell
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    Cell.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for Cell
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.Cell
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    Cell.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.Cell";
                    };
    
                    return Cell;
                })();
    
                v2.Value = (function() {
    
                    /**
                     * Properties of a Value.
                     * @memberof google.bigtable.v2
                     * @interface IValue
                     * @property {Uint8Array|null} [rawValue] Value rawValue
                     * @property {number|Long|null} [rawTimestampMicros] Value rawTimestampMicros
                     * @property {number|Long|null} [intValue] Value intValue
                     */
    
                    /**
                     * Constructs a new Value.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a Value.
                     * @implements IValue
                     * @constructor
                     * @param {google.bigtable.v2.IValue=} [properties] Properties to set
                     */
                    function Value(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * Value rawValue.
                     * @member {Uint8Array|null|undefined} rawValue
                     * @memberof google.bigtable.v2.Value
                     * @instance
                     */
                    Value.prototype.rawValue = null;
    
                    /**
                     * Value rawTimestampMicros.
                     * @member {number|Long|null|undefined} rawTimestampMicros
                     * @memberof google.bigtable.v2.Value
                     * @instance
                     */
                    Value.prototype.rawTimestampMicros = null;
    
                    /**
                     * Value intValue.
                     * @member {number|Long|null|undefined} intValue
                     * @memberof google.bigtable.v2.Value
                     * @instance
                     */
                    Value.prototype.intValue = null;
    
                    // OneOf field names bound to virtual getters and setters
                    var $oneOfFields;
    
                    /**
                     * Value kind.
                     * @member {"rawValue"|"rawTimestampMicros"|"intValue"|undefined} kind
                     * @memberof google.bigtable.v2.Value
                     * @instance
                     */
                    Object.defineProperty(Value.prototype, "kind", {
                        get: $util.oneOfGetter($oneOfFields = ["rawValue", "rawTimestampMicros", "intValue"]),
                        set: $util.oneOfSetter($oneOfFields)
                    });
    
                    /**
                     * Creates a new Value instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.Value
                     * @static
                     * @param {google.bigtable.v2.IValue=} [properties] Properties to set
                     * @returns {google.bigtable.v2.Value} Value instance
                     */
                    Value.create = function create(properties) {
                        return new Value(properties);
                    };
    
                    /**
                     * Encodes the specified Value message. Does not implicitly {@link google.bigtable.v2.Value.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.Value
                     * @static
                     * @param {google.bigtable.v2.IValue} message Value message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    Value.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.intValue != null && Object.hasOwnProperty.call(message, "intValue"))
                            writer.uint32(/* id 6, wireType 0 =*/48).int64(message.intValue);
                        if (message.rawValue != null && Object.hasOwnProperty.call(message, "rawValue"))
                            writer.uint32(/* id 8, wireType 2 =*/66).bytes(message.rawValue);
                        if (message.rawTimestampMicros != null && Object.hasOwnProperty.call(message, "rawTimestampMicros"))
                            writer.uint32(/* id 9, wireType 0 =*/72).int64(message.rawTimestampMicros);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified Value message, length delimited. Does not implicitly {@link google.bigtable.v2.Value.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.Value
                     * @static
                     * @param {google.bigtable.v2.IValue} message Value message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    Value.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a Value message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.Value
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.Value} Value
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    Value.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.Value();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 8: {
                                    message.rawValue = reader.bytes();
                                    break;
                                }
                            case 9: {
                                    message.rawTimestampMicros = reader.int64();
                                    break;
                                }
                            case 6: {
                                    message.intValue = reader.int64();
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a Value message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.Value
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.Value} Value
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    Value.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a Value message.
                     * @function verify
                     * @memberof google.bigtable.v2.Value
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    Value.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        var properties = {};
                        if (message.rawValue != null && message.hasOwnProperty("rawValue")) {
                            properties.kind = 1;
                            if (!(message.rawValue && typeof message.rawValue.length === "number" || $util.isString(message.rawValue)))
                                return "rawValue: buffer expected";
                        }
                        if (message.rawTimestampMicros != null && message.hasOwnProperty("rawTimestampMicros")) {
                            if (properties.kind === 1)
                                return "kind: multiple values";
                            properties.kind = 1;
                            if (!$util.isInteger(message.rawTimestampMicros) && !(message.rawTimestampMicros && $util.isInteger(message.rawTimestampMicros.low) && $util.isInteger(message.rawTimestampMicros.high)))
                                return "rawTimestampMicros: integer|Long expected";
                        }
                        if (message.intValue != null && message.hasOwnProperty("intValue")) {
                            if (properties.kind === 1)
                                return "kind: multiple values";
                            properties.kind = 1;
                            if (!$util.isInteger(message.intValue) && !(message.intValue && $util.isInteger(message.intValue.low) && $util.isInteger(message.intValue.high)))
                                return "intValue: integer|Long expected";
                        }
                        return null;
                    };
    
                    /**
                     * Creates a Value message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.Value
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.Value} Value
                     */
                    Value.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.Value)
                            return object;
                        var message = new $root.google.bigtable.v2.Value();
                        if (object.rawValue != null)
                            if (typeof object.rawValue === "string")
                                $util.base64.decode(object.rawValue, message.rawValue = $util.newBuffer($util.base64.length(object.rawValue)), 0);
                            else if (object.rawValue.length >= 0)
                                message.rawValue = object.rawValue;
                        if (object.rawTimestampMicros != null)
                            if ($util.Long)
                                (message.rawTimestampMicros = $util.Long.fromValue(object.rawTimestampMicros)).unsigned = false;
                            else if (typeof object.rawTimestampMicros === "string")
                                message.rawTimestampMicros = parseInt(object.rawTimestampMicros, 10);
                            else if (typeof object.rawTimestampMicros === "number")
                                message.rawTimestampMicros = object.rawTimestampMicros;
                            else if (typeof object.rawTimestampMicros === "object")
                                message.rawTimestampMicros = new $util.LongBits(object.rawTimestampMicros.low >>> 0, object.rawTimestampMicros.high >>> 0).toNumber();
                        if (object.intValue != null)
                            if ($util.Long)
                                (message.intValue = $util.Long.fromValue(object.intValue)).unsigned = false;
                            else if (typeof object.intValue === "string")
                                message.intValue = parseInt(object.intValue, 10);
                            else if (typeof object.intValue === "number")
                                message.intValue = object.intValue;
                            else if (typeof object.intValue === "object")
                                message.intValue = new $util.LongBits(object.intValue.low >>> 0, object.intValue.high >>> 0).toNumber();
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a Value message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.Value
                     * @static
                     * @param {google.bigtable.v2.Value} message Value
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    Value.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (message.intValue != null && message.hasOwnProperty("intValue")) {
                            if (typeof message.intValue === "number")
                                object.intValue = options.longs === String ? String(message.intValue) : message.intValue;
                            else
                                object.intValue = options.longs === String ? $util.Long.prototype.toString.call(message.intValue) : options.longs === Number ? new $util.LongBits(message.intValue.low >>> 0, message.intValue.high >>> 0).toNumber() : message.intValue;
                            if (options.oneofs)
                                object.kind = "intValue";
                        }
                        if (message.rawValue != null && message.hasOwnProperty("rawValue")) {
                            object.rawValue = options.bytes === String ? $util.base64.encode(message.rawValue, 0, message.rawValue.length) : options.bytes === Array ? Array.prototype.slice.call(message.rawValue) : message.rawValue;
                            if (options.oneofs)
                                object.kind = "rawValue";
                        }
                        if (message.rawTimestampMicros != null && message.hasOwnProperty("rawTimestampMicros")) {
                            if (typeof message.rawTimestampMicros === "number")
                                object.rawTimestampMicros = options.longs === String ? String(message.rawTimestampMicros) : message.rawTimestampMicros;
                            else
                                object.rawTimestampMicros = options.longs === String ? $util.Long.prototype.toString.call(message.rawTimestampMicros) : options.longs === Number ? new $util.LongBits(message.rawTimestampMicros.low >>> 0, message.rawTimestampMicros.high >>> 0).toNumber() : message.rawTimestampMicros;
                            if (options.oneofs)
                                object.kind = "rawTimestampMicros";
                        }
                        return object;
                    };
    
                    /**
                     * Converts this Value to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.Value
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    Value.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for Value
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.Value
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    Value.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.Value";
                    };
    
                    return Value;
                })();
    
                v2.RowRange = (function() {
    
                    /**
                     * Properties of a RowRange.
                     * @memberof google.bigtable.v2
                     * @interface IRowRange
                     * @property {Uint8Array|null} [startKeyClosed] RowRange startKeyClosed
                     * @property {Uint8Array|null} [startKeyOpen] RowRange startKeyOpen
                     * @property {Uint8Array|null} [endKeyOpen] RowRange endKeyOpen
                     * @property {Uint8Array|null} [endKeyClosed] RowRange endKeyClosed
                     */
    
                    /**
                     * Constructs a new RowRange.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a RowRange.
                     * @implements IRowRange
                     * @constructor
                     * @param {google.bigtable.v2.IRowRange=} [properties] Properties to set
                     */
                    function RowRange(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * RowRange startKeyClosed.
                     * @member {Uint8Array|null|undefined} startKeyClosed
                     * @memberof google.bigtable.v2.RowRange
                     * @instance
                     */
                    RowRange.prototype.startKeyClosed = null;
    
                    /**
                     * RowRange startKeyOpen.
                     * @member {Uint8Array|null|undefined} startKeyOpen
                     * @memberof google.bigtable.v2.RowRange
                     * @instance
                     */
                    RowRange.prototype.startKeyOpen = null;
    
                    /**
                     * RowRange endKeyOpen.
                     * @member {Uint8Array|null|undefined} endKeyOpen
                     * @memberof google.bigtable.v2.RowRange
                     * @instance
                     */
                    RowRange.prototype.endKeyOpen = null;
    
                    /**
                     * RowRange endKeyClosed.
                     * @member {Uint8Array|null|undefined} endKeyClosed
                     * @memberof google.bigtable.v2.RowRange
                     * @instance
                     */
                    RowRange.prototype.endKeyClosed = null;
    
                    // OneOf field names bound to virtual getters and setters
                    var $oneOfFields;
    
                    /**
                     * RowRange startKey.
                     * @member {"startKeyClosed"|"startKeyOpen"|undefined} startKey
                     * @memberof google.bigtable.v2.RowRange
                     * @instance
                     */
                    Object.defineProperty(RowRange.prototype, "startKey", {
                        get: $util.oneOfGetter($oneOfFields = ["startKeyClosed", "startKeyOpen"]),
                        set: $util.oneOfSetter($oneOfFields)
                    });
    
                    /**
                     * RowRange endKey.
                     * @member {"endKeyOpen"|"endKeyClosed"|undefined} endKey
                     * @memberof google.bigtable.v2.RowRange
                     * @instance
                     */
                    Object.defineProperty(RowRange.prototype, "endKey", {
                        get: $util.oneOfGetter($oneOfFields = ["endKeyOpen", "endKeyClosed"]),
                        set: $util.oneOfSetter($oneOfFields)
                    });
    
                    /**
                     * Creates a new RowRange instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.RowRange
                     * @static
                     * @param {google.bigtable.v2.IRowRange=} [properties] Properties to set
                     * @returns {google.bigtable.v2.RowRange} RowRange instance
                     */
                    RowRange.create = function create(properties) {
                        return new RowRange(properties);
                    };
    
                    /**
                     * Encodes the specified RowRange message. Does not implicitly {@link google.bigtable.v2.RowRange.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.RowRange
                     * @static
                     * @param {google.bigtable.v2.IRowRange} message RowRange message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    RowRange.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.startKeyClosed != null && Object.hasOwnProperty.call(message, "startKeyClosed"))
                            writer.uint32(/* id 1, wireType 2 =*/10).bytes(message.startKeyClosed);
                        if (message.startKeyOpen != null && Object.hasOwnProperty.call(message, "startKeyOpen"))
                            writer.uint32(/* id 2, wireType 2 =*/18).bytes(message.startKeyOpen);
                        if (message.endKeyOpen != null && Object.hasOwnProperty.call(message, "endKeyOpen"))
                            writer.uint32(/* id 3, wireType 2 =*/26).bytes(message.endKeyOpen);
                        if (message.endKeyClosed != null && Object.hasOwnProperty.call(message, "endKeyClosed"))
                            writer.uint32(/* id 4, wireType 2 =*/34).bytes(message.endKeyClosed);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified RowRange message, length delimited. Does not implicitly {@link google.bigtable.v2.RowRange.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.RowRange
                     * @static
                     * @param {google.bigtable.v2.IRowRange} message RowRange message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    RowRange.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a RowRange message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.RowRange
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.RowRange} RowRange
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    RowRange.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.RowRange();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.startKeyClosed = reader.bytes();
                                    break;
                                }
                            case 2: {
                                    message.startKeyOpen = reader.bytes();
                                    break;
                                }
                            case 3: {
                                    message.endKeyOpen = reader.bytes();
                                    break;
                                }
                            case 4: {
                                    message.endKeyClosed = reader.bytes();
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a RowRange message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.RowRange
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.RowRange} RowRange
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    RowRange.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a RowRange message.
                     * @function verify
                     * @memberof google.bigtable.v2.RowRange
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    RowRange.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        var properties = {};
                        if (message.startKeyClosed != null && message.hasOwnProperty("startKeyClosed")) {
                            properties.startKey = 1;
                            if (!(message.startKeyClosed && typeof message.startKeyClosed.length === "number" || $util.isString(message.startKeyClosed)))
                                return "startKeyClosed: buffer expected";
                        }
                        if (message.startKeyOpen != null && message.hasOwnProperty("startKeyOpen")) {
                            if (properties.startKey === 1)
                                return "startKey: multiple values";
                            properties.startKey = 1;
                            if (!(message.startKeyOpen && typeof message.startKeyOpen.length === "number" || $util.isString(message.startKeyOpen)))
                                return "startKeyOpen: buffer expected";
                        }
                        if (message.endKeyOpen != null && message.hasOwnProperty("endKeyOpen")) {
                            properties.endKey = 1;
                            if (!(message.endKeyOpen && typeof message.endKeyOpen.length === "number" || $util.isString(message.endKeyOpen)))
                                return "endKeyOpen: buffer expected";
                        }
                        if (message.endKeyClosed != null && message.hasOwnProperty("endKeyClosed")) {
                            if (properties.endKey === 1)
                                return "endKey: multiple values";
                            properties.endKey = 1;
                            if (!(message.endKeyClosed && typeof message.endKeyClosed.length === "number" || $util.isString(message.endKeyClosed)))
                                return "endKeyClosed: buffer expected";
                        }
                        return null;
                    };
    
                    /**
                     * Creates a RowRange message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.RowRange
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.RowRange} RowRange
                     */
                    RowRange.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.RowRange)
                            return object;
                        var message = new $root.google.bigtable.v2.RowRange();
                        if (object.startKeyClosed != null)
                            if (typeof object.startKeyClosed === "string")
                                $util.base64.decode(object.startKeyClosed, message.startKeyClosed = $util.newBuffer($util.base64.length(object.startKeyClosed)), 0);
                            else if (object.startKeyClosed.length >= 0)
                                message.startKeyClosed = object.startKeyClosed;
                        if (object.startKeyOpen != null)
                            if (typeof object.startKeyOpen === "string")
                                $util.base64.decode(object.startKeyOpen, message.startKeyOpen = $util.newBuffer($util.base64.length(object.startKeyOpen)), 0);
                            else if (object.startKeyOpen.length >= 0)
                                message.startKeyOpen = object.startKeyOpen;
                        if (object.endKeyOpen != null)
                            if (typeof object.endKeyOpen === "string")
                                $util.base64.decode(object.endKeyOpen, message.endKeyOpen = $util.newBuffer($util.base64.length(object.endKeyOpen)), 0);
                            else if (object.endKeyOpen.length >= 0)
                                message.endKeyOpen = object.endKeyOpen;
                        if (object.endKeyClosed != null)
                            if (typeof object.endKeyClosed === "string")
                                $util.base64.decode(object.endKeyClosed, message.endKeyClosed = $util.newBuffer($util.base64.length(object.endKeyClosed)), 0);
                            else if (object.endKeyClosed.length >= 0)
                                message.endKeyClosed = object.endKeyClosed;
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a RowRange message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.RowRange
                     * @static
                     * @param {google.bigtable.v2.RowRange} message RowRange
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    RowRange.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (message.startKeyClosed != null && message.hasOwnProperty("startKeyClosed")) {
                            object.startKeyClosed = options.bytes === String ? $util.base64.encode(message.startKeyClosed, 0, message.startKeyClosed.length) : options.bytes === Array ? Array.prototype.slice.call(message.startKeyClosed) : message.startKeyClosed;
                            if (options.oneofs)
                                object.startKey = "startKeyClosed";
                        }
                        if (message.startKeyOpen != null && message.hasOwnProperty("startKeyOpen")) {
                            object.startKeyOpen = options.bytes === String ? $util.base64.encode(message.startKeyOpen, 0, message.startKeyOpen.length) : options.bytes === Array ? Array.prototype.slice.call(message.startKeyOpen) : message.startKeyOpen;
                            if (options.oneofs)
                                object.startKey = "startKeyOpen";
                        }
                        if (message.endKeyOpen != null && message.hasOwnProperty("endKeyOpen")) {
                            object.endKeyOpen = options.bytes === String ? $util.base64.encode(message.endKeyOpen, 0, message.endKeyOpen.length) : options.bytes === Array ? Array.prototype.slice.call(message.endKeyOpen) : message.endKeyOpen;
                            if (options.oneofs)
                                object.endKey = "endKeyOpen";
                        }
                        if (message.endKeyClosed != null && message.hasOwnProperty("endKeyClosed")) {
                            object.endKeyClosed = options.bytes === String ? $util.base64.encode(message.endKeyClosed, 0, message.endKeyClosed.length) : options.bytes === Array ? Array.prototype.slice.call(message.endKeyClosed) : message.endKeyClosed;
                            if (options.oneofs)
                                object.endKey = "endKeyClosed";
                        }
                        return object;
                    };
    
                    /**
                     * Converts this RowRange to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.RowRange
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    RowRange.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for RowRange
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.RowRange
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    RowRange.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.RowRange";
                    };
    
                    return RowRange;
                })();
    
                v2.RowSet = (function() {
    
                    /**
                     * Properties of a RowSet.
                     * @memberof google.bigtable.v2
                     * @interface IRowSet
                     * @property {Array.<Uint8Array>|null} [rowKeys] RowSet rowKeys
                     * @property {Array.<google.bigtable.v2.IRowRange>|null} [rowRanges] RowSet rowRanges
                     */
    
                    /**
                     * Constructs a new RowSet.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a RowSet.
                     * @implements IRowSet
                     * @constructor
                     * @param {google.bigtable.v2.IRowSet=} [properties] Properties to set
                     */
                    function RowSet(properties) {
                        this.rowKeys = [];
                        this.rowRanges = [];
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * RowSet rowKeys.
                     * @member {Array.<Uint8Array>} rowKeys
                     * @memberof google.bigtable.v2.RowSet
                     * @instance
                     */
                    RowSet.prototype.rowKeys = $util.emptyArray;
    
                    /**
                     * RowSet rowRanges.
                     * @member {Array.<google.bigtable.v2.IRowRange>} rowRanges
                     * @memberof google.bigtable.v2.RowSet
                     * @instance
                     */
                    RowSet.prototype.rowRanges = $util.emptyArray;
    
                    /**
                     * Creates a new RowSet instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.RowSet
                     * @static
                     * @param {google.bigtable.v2.IRowSet=} [properties] Properties to set
                     * @returns {google.bigtable.v2.RowSet} RowSet instance
                     */
                    RowSet.create = function create(properties) {
                        return new RowSet(properties);
                    };
    
                    /**
                     * Encodes the specified RowSet message. Does not implicitly {@link google.bigtable.v2.RowSet.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.RowSet
                     * @static
                     * @param {google.bigtable.v2.IRowSet} message RowSet message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    RowSet.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.rowKeys != null && message.rowKeys.length)
                            for (var i = 0; i < message.rowKeys.length; ++i)
                                writer.uint32(/* id 1, wireType 2 =*/10).bytes(message.rowKeys[i]);
                        if (message.rowRanges != null && message.rowRanges.length)
                            for (var i = 0; i < message.rowRanges.length; ++i)
                                $root.google.bigtable.v2.RowRange.encode(message.rowRanges[i], writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                        return writer;
                    };
    
                    /**
                     * Encodes the specified RowSet message, length delimited. Does not implicitly {@link google.bigtable.v2.RowSet.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.RowSet
                     * @static
                     * @param {google.bigtable.v2.IRowSet} message RowSet message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    RowSet.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a RowSet message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.RowSet
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.RowSet} RowSet
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    RowSet.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.RowSet();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    if (!(message.rowKeys && message.rowKeys.length))
                                        message.rowKeys = [];
                                    message.rowKeys.push(reader.bytes());
                                    break;
                                }
                            case 2: {
                                    if (!(message.rowRanges && message.rowRanges.length))
                                        message.rowRanges = [];
                                    message.rowRanges.push($root.google.bigtable.v2.RowRange.decode(reader, reader.uint32()));
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a RowSet message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.RowSet
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.RowSet} RowSet
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    RowSet.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a RowSet message.
                     * @function verify
                     * @memberof google.bigtable.v2.RowSet
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    RowSet.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.rowKeys != null && message.hasOwnProperty("rowKeys")) {
                            if (!Array.isArray(message.rowKeys))
                                return "rowKeys: array expected";
                            for (var i = 0; i < message.rowKeys.length; ++i)
                                if (!(message.rowKeys[i] && typeof message.rowKeys[i].length === "number" || $util.isString(message.rowKeys[i])))
                                    return "rowKeys: buffer[] expected";
                        }
                        if (message.rowRanges != null && message.hasOwnProperty("rowRanges")) {
                            if (!Array.isArray(message.rowRanges))
                                return "rowRanges: array expected";
                            for (var i = 0; i < message.rowRanges.length; ++i) {
                                var error = $root.google.bigtable.v2.RowRange.verify(message.rowRanges[i]);
                                if (error)
                                    return "rowRanges." + error;
                            }
                        }
                        return null;
                    };
    
                    /**
                     * Creates a RowSet message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.RowSet
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.RowSet} RowSet
                     */
                    RowSet.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.RowSet)
                            return object;
                        var message = new $root.google.bigtable.v2.RowSet();
                        if (object.rowKeys) {
                            if (!Array.isArray(object.rowKeys))
                                throw TypeError(".google.bigtable.v2.RowSet.rowKeys: array expected");
                            message.rowKeys = [];
                            for (var i = 0; i < object.rowKeys.length; ++i)
                                if (typeof object.rowKeys[i] === "string")
                                    $util.base64.decode(object.rowKeys[i], message.rowKeys[i] = $util.newBuffer($util.base64.length(object.rowKeys[i])), 0);
                                else if (object.rowKeys[i].length >= 0)
                                    message.rowKeys[i] = object.rowKeys[i];
                        }
                        if (object.rowRanges) {
                            if (!Array.isArray(object.rowRanges))
                                throw TypeError(".google.bigtable.v2.RowSet.rowRanges: array expected");
                            message.rowRanges = [];
                            for (var i = 0; i < object.rowRanges.length; ++i) {
                                if (typeof object.rowRanges[i] !== "object")
                                    throw TypeError(".google.bigtable.v2.RowSet.rowRanges: object expected");
                                message.rowRanges[i] = $root.google.bigtable.v2.RowRange.fromObject(object.rowRanges[i]);
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a RowSet message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.RowSet
                     * @static
                     * @param {google.bigtable.v2.RowSet} message RowSet
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    RowSet.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.arrays || options.defaults) {
                            object.rowKeys = [];
                            object.rowRanges = [];
                        }
                        if (message.rowKeys && message.rowKeys.length) {
                            object.rowKeys = [];
                            for (var j = 0; j < message.rowKeys.length; ++j)
                                object.rowKeys[j] = options.bytes === String ? $util.base64.encode(message.rowKeys[j], 0, message.rowKeys[j].length) : options.bytes === Array ? Array.prototype.slice.call(message.rowKeys[j]) : message.rowKeys[j];
                        }
                        if (message.rowRanges && message.rowRanges.length) {
                            object.rowRanges = [];
                            for (var j = 0; j < message.rowRanges.length; ++j)
                                object.rowRanges[j] = $root.google.bigtable.v2.RowRange.toObject(message.rowRanges[j], options);
                        }
                        return object;
                    };
    
                    /**
                     * Converts this RowSet to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.RowSet
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    RowSet.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for RowSet
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.RowSet
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    RowSet.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.RowSet";
                    };
    
                    return RowSet;
                })();
    
                v2.ColumnRange = (function() {
    
                    /**
                     * Properties of a ColumnRange.
                     * @memberof google.bigtable.v2
                     * @interface IColumnRange
                     * @property {string|null} [familyName] ColumnRange familyName
                     * @property {Uint8Array|null} [startQualifierClosed] ColumnRange startQualifierClosed
                     * @property {Uint8Array|null} [startQualifierOpen] ColumnRange startQualifierOpen
                     * @property {Uint8Array|null} [endQualifierClosed] ColumnRange endQualifierClosed
                     * @property {Uint8Array|null} [endQualifierOpen] ColumnRange endQualifierOpen
                     */
    
                    /**
                     * Constructs a new ColumnRange.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a ColumnRange.
                     * @implements IColumnRange
                     * @constructor
                     * @param {google.bigtable.v2.IColumnRange=} [properties] Properties to set
                     */
                    function ColumnRange(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * ColumnRange familyName.
                     * @member {string} familyName
                     * @memberof google.bigtable.v2.ColumnRange
                     * @instance
                     */
                    ColumnRange.prototype.familyName = "";
    
                    /**
                     * ColumnRange startQualifierClosed.
                     * @member {Uint8Array|null|undefined} startQualifierClosed
                     * @memberof google.bigtable.v2.ColumnRange
                     * @instance
                     */
                    ColumnRange.prototype.startQualifierClosed = null;
    
                    /**
                     * ColumnRange startQualifierOpen.
                     * @member {Uint8Array|null|undefined} startQualifierOpen
                     * @memberof google.bigtable.v2.ColumnRange
                     * @instance
                     */
                    ColumnRange.prototype.startQualifierOpen = null;
    
                    /**
                     * ColumnRange endQualifierClosed.
                     * @member {Uint8Array|null|undefined} endQualifierClosed
                     * @memberof google.bigtable.v2.ColumnRange
                     * @instance
                     */
                    ColumnRange.prototype.endQualifierClosed = null;
    
                    /**
                     * ColumnRange endQualifierOpen.
                     * @member {Uint8Array|null|undefined} endQualifierOpen
                     * @memberof google.bigtable.v2.ColumnRange
                     * @instance
                     */
                    ColumnRange.prototype.endQualifierOpen = null;
    
                    // OneOf field names bound to virtual getters and setters
                    var $oneOfFields;
    
                    /**
                     * ColumnRange startQualifier.
                     * @member {"startQualifierClosed"|"startQualifierOpen"|undefined} startQualifier
                     * @memberof google.bigtable.v2.ColumnRange
                     * @instance
                     */
                    Object.defineProperty(ColumnRange.prototype, "startQualifier", {
                        get: $util.oneOfGetter($oneOfFields = ["startQualifierClosed", "startQualifierOpen"]),
                        set: $util.oneOfSetter($oneOfFields)
                    });
    
                    /**
                     * ColumnRange endQualifier.
                     * @member {"endQualifierClosed"|"endQualifierOpen"|undefined} endQualifier
                     * @memberof google.bigtable.v2.ColumnRange
                     * @instance
                     */
                    Object.defineProperty(ColumnRange.prototype, "endQualifier", {
                        get: $util.oneOfGetter($oneOfFields = ["endQualifierClosed", "endQualifierOpen"]),
                        set: $util.oneOfSetter($oneOfFields)
                    });
    
                    /**
                     * Creates a new ColumnRange instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.ColumnRange
                     * @static
                     * @param {google.bigtable.v2.IColumnRange=} [properties] Properties to set
                     * @returns {google.bigtable.v2.ColumnRange} ColumnRange instance
                     */
                    ColumnRange.create = function create(properties) {
                        return new ColumnRange(properties);
                    };
    
                    /**
                     * Encodes the specified ColumnRange message. Does not implicitly {@link google.bigtable.v2.ColumnRange.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.ColumnRange
                     * @static
                     * @param {google.bigtable.v2.IColumnRange} message ColumnRange message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ColumnRange.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.familyName != null && Object.hasOwnProperty.call(message, "familyName"))
                            writer.uint32(/* id 1, wireType 2 =*/10).string(message.familyName);
                        if (message.startQualifierClosed != null && Object.hasOwnProperty.call(message, "startQualifierClosed"))
                            writer.uint32(/* id 2, wireType 2 =*/18).bytes(message.startQualifierClosed);
                        if (message.startQualifierOpen != null && Object.hasOwnProperty.call(message, "startQualifierOpen"))
                            writer.uint32(/* id 3, wireType 2 =*/26).bytes(message.startQualifierOpen);
                        if (message.endQualifierClosed != null && Object.hasOwnProperty.call(message, "endQualifierClosed"))
                            writer.uint32(/* id 4, wireType 2 =*/34).bytes(message.endQualifierClosed);
                        if (message.endQualifierOpen != null && Object.hasOwnProperty.call(message, "endQualifierOpen"))
                            writer.uint32(/* id 5, wireType 2 =*/42).bytes(message.endQualifierOpen);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified ColumnRange message, length delimited. Does not implicitly {@link google.bigtable.v2.ColumnRange.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.ColumnRange
                     * @static
                     * @param {google.bigtable.v2.IColumnRange} message ColumnRange message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ColumnRange.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a ColumnRange message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.ColumnRange
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.ColumnRange} ColumnRange
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ColumnRange.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.ColumnRange();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.familyName = reader.string();
                                    break;
                                }
                            case 2: {
                                    message.startQualifierClosed = reader.bytes();
                                    break;
                                }
                            case 3: {
                                    message.startQualifierOpen = reader.bytes();
                                    break;
                                }
                            case 4: {
                                    message.endQualifierClosed = reader.bytes();
                                    break;
                                }
                            case 5: {
                                    message.endQualifierOpen = reader.bytes();
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a ColumnRange message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.ColumnRange
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.ColumnRange} ColumnRange
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ColumnRange.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a ColumnRange message.
                     * @function verify
                     * @memberof google.bigtable.v2.ColumnRange
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    ColumnRange.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        var properties = {};
                        if (message.familyName != null && message.hasOwnProperty("familyName"))
                            if (!$util.isString(message.familyName))
                                return "familyName: string expected";
                        if (message.startQualifierClosed != null && message.hasOwnProperty("startQualifierClosed")) {
                            properties.startQualifier = 1;
                            if (!(message.startQualifierClosed && typeof message.startQualifierClosed.length === "number" || $util.isString(message.startQualifierClosed)))
                                return "startQualifierClosed: buffer expected";
                        }
                        if (message.startQualifierOpen != null && message.hasOwnProperty("startQualifierOpen")) {
                            if (properties.startQualifier === 1)
                                return "startQualifier: multiple values";
                            properties.startQualifier = 1;
                            if (!(message.startQualifierOpen && typeof message.startQualifierOpen.length === "number" || $util.isString(message.startQualifierOpen)))
                                return "startQualifierOpen: buffer expected";
                        }
                        if (message.endQualifierClosed != null && message.hasOwnProperty("endQualifierClosed")) {
                            properties.endQualifier = 1;
                            if (!(message.endQualifierClosed && typeof message.endQualifierClosed.length === "number" || $util.isString(message.endQualifierClosed)))
                                return "endQualifierClosed: buffer expected";
                        }
                        if (message.endQualifierOpen != null && message.hasOwnProperty("endQualifierOpen")) {
                            if (properties.endQualifier === 1)
                                return "endQualifier: multiple values";
                            properties.endQualifier = 1;
                            if (!(message.endQualifierOpen && typeof message.endQualifierOpen.length === "number" || $util.isString(message.endQualifierOpen)))
                                return "endQualifierOpen: buffer expected";
                        }
                        return null;
                    };
    
                    /**
                     * Creates a ColumnRange message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.ColumnRange
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.ColumnRange} ColumnRange
                     */
                    ColumnRange.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.ColumnRange)
                            return object;
                        var message = new $root.google.bigtable.v2.ColumnRange();
                        if (object.familyName != null)
                            message.familyName = String(object.familyName);
                        if (object.startQualifierClosed != null)
                            if (typeof object.startQualifierClosed === "string")
                                $util.base64.decode(object.startQualifierClosed, message.startQualifierClosed = $util.newBuffer($util.base64.length(object.startQualifierClosed)), 0);
                            else if (object.startQualifierClosed.length >= 0)
                                message.startQualifierClosed = object.startQualifierClosed;
                        if (object.startQualifierOpen != null)
                            if (typeof object.startQualifierOpen === "string")
                                $util.base64.decode(object.startQualifierOpen, message.startQualifierOpen = $util.newBuffer($util.base64.length(object.startQualifierOpen)), 0);
                            else if (object.startQualifierOpen.length >= 0)
                                message.startQualifierOpen = object.startQualifierOpen;
                        if (object.endQualifierClosed != null)
                            if (typeof object.endQualifierClosed === "string")
                                $util.base64.decode(object.endQualifierClosed, message.endQualifierClosed = $util.newBuffer($util.base64.length(object.endQualifierClosed)), 0);
                            else if (object.endQualifierClosed.length >= 0)
                                message.endQualifierClosed = object.endQualifierClosed;
                        if (object.endQualifierOpen != null)
                            if (typeof object.endQualifierOpen === "string")
                                $util.base64.decode(object.endQualifierOpen, message.endQualifierOpen = $util.newBuffer($util.base64.length(object.endQualifierOpen)), 0);
                            else if (object.endQualifierOpen.length >= 0)
                                message.endQualifierOpen = object.endQualifierOpen;
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a ColumnRange message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.ColumnRange
                     * @static
                     * @param {google.bigtable.v2.ColumnRange} message ColumnRange
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    ColumnRange.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults)
                            object.familyName = "";
                        if (message.familyName != null && message.hasOwnProperty("familyName"))
                            object.familyName = message.familyName;
                        if (message.startQualifierClosed != null && message.hasOwnProperty("startQualifierClosed")) {
                            object.startQualifierClosed = options.bytes === String ? $util.base64.encode(message.startQualifierClosed, 0, message.startQualifierClosed.length) : options.bytes === Array ? Array.prototype.slice.call(message.startQualifierClosed) : message.startQualifierClosed;
                            if (options.oneofs)
                                object.startQualifier = "startQualifierClosed";
                        }
                        if (message.startQualifierOpen != null && message.hasOwnProperty("startQualifierOpen")) {
                            object.startQualifierOpen = options.bytes === String ? $util.base64.encode(message.startQualifierOpen, 0, message.startQualifierOpen.length) : options.bytes === Array ? Array.prototype.slice.call(message.startQualifierOpen) : message.startQualifierOpen;
                            if (options.oneofs)
                                object.startQualifier = "startQualifierOpen";
                        }
                        if (message.endQualifierClosed != null && message.hasOwnProperty("endQualifierClosed")) {
                            object.endQualifierClosed = options.bytes === String ? $util.base64.encode(message.endQualifierClosed, 0, message.endQualifierClosed.length) : options.bytes === Array ? Array.prototype.slice.call(message.endQualifierClosed) : message.endQualifierClosed;
                            if (options.oneofs)
                                object.endQualifier = "endQualifierClosed";
                        }
                        if (message.endQualifierOpen != null && message.hasOwnProperty("endQualifierOpen")) {
                            object.endQualifierOpen = options.bytes === String ? $util.base64.encode(message.endQualifierOpen, 0, message.endQualifierOpen.length) : options.bytes === Array ? Array.prototype.slice.call(message.endQualifierOpen) : message.endQualifierOpen;
                            if (options.oneofs)
                                object.endQualifier = "endQualifierOpen";
                        }
                        return object;
                    };
    
                    /**
                     * Converts this ColumnRange to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.ColumnRange
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    ColumnRange.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for ColumnRange
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.ColumnRange
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    ColumnRange.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.ColumnRange";
                    };
    
                    return ColumnRange;
                })();
    
                v2.TimestampRange = (function() {
    
                    /**
                     * Properties of a TimestampRange.
                     * @memberof google.bigtable.v2
                     * @interface ITimestampRange
                     * @property {number|Long|null} [startTimestampMicros] TimestampRange startTimestampMicros
                     * @property {number|Long|null} [endTimestampMicros] TimestampRange endTimestampMicros
                     */
    
                    /**
                     * Constructs a new TimestampRange.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a TimestampRange.
                     * @implements ITimestampRange
                     * @constructor
                     * @param {google.bigtable.v2.ITimestampRange=} [properties] Properties to set
                     */
                    function TimestampRange(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * TimestampRange startTimestampMicros.
                     * @member {number|Long} startTimestampMicros
                     * @memberof google.bigtable.v2.TimestampRange
                     * @instance
                     */
                    TimestampRange.prototype.startTimestampMicros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                    /**
                     * TimestampRange endTimestampMicros.
                     * @member {number|Long} endTimestampMicros
                     * @memberof google.bigtable.v2.TimestampRange
                     * @instance
                     */
                    TimestampRange.prototype.endTimestampMicros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                    /**
                     * Creates a new TimestampRange instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.TimestampRange
                     * @static
                     * @param {google.bigtable.v2.ITimestampRange=} [properties] Properties to set
                     * @returns {google.bigtable.v2.TimestampRange} TimestampRange instance
                     */
                    TimestampRange.create = function create(properties) {
                        return new TimestampRange(properties);
                    };
    
                    /**
                     * Encodes the specified TimestampRange message. Does not implicitly {@link google.bigtable.v2.TimestampRange.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.TimestampRange
                     * @static
                     * @param {google.bigtable.v2.ITimestampRange} message TimestampRange message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    TimestampRange.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.startTimestampMicros != null && Object.hasOwnProperty.call(message, "startTimestampMicros"))
                            writer.uint32(/* id 1, wireType 0 =*/8).int64(message.startTimestampMicros);
                        if (message.endTimestampMicros != null && Object.hasOwnProperty.call(message, "endTimestampMicros"))
                            writer.uint32(/* id 2, wireType 0 =*/16).int64(message.endTimestampMicros);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified TimestampRange message, length delimited. Does not implicitly {@link google.bigtable.v2.TimestampRange.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.TimestampRange
                     * @static
                     * @param {google.bigtable.v2.ITimestampRange} message TimestampRange message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    TimestampRange.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a TimestampRange message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.TimestampRange
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.TimestampRange} TimestampRange
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    TimestampRange.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.TimestampRange();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.startTimestampMicros = reader.int64();
                                    break;
                                }
                            case 2: {
                                    message.endTimestampMicros = reader.int64();
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a TimestampRange message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.TimestampRange
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.TimestampRange} TimestampRange
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    TimestampRange.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a TimestampRange message.
                     * @function verify
                     * @memberof google.bigtable.v2.TimestampRange
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    TimestampRange.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.startTimestampMicros != null && message.hasOwnProperty("startTimestampMicros"))
                            if (!$util.isInteger(message.startTimestampMicros) && !(message.startTimestampMicros && $util.isInteger(message.startTimestampMicros.low) && $util.isInteger(message.startTimestampMicros.high)))
                                return "startTimestampMicros: integer|Long expected";
                        if (message.endTimestampMicros != null && message.hasOwnProperty("endTimestampMicros"))
                            if (!$util.isInteger(message.endTimestampMicros) && !(message.endTimestampMicros && $util.isInteger(message.endTimestampMicros.low) && $util.isInteger(message.endTimestampMicros.high)))
                                return "endTimestampMicros: integer|Long expected";
                        return null;
                    };
    
                    /**
                     * Creates a TimestampRange message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.TimestampRange
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.TimestampRange} TimestampRange
                     */
                    TimestampRange.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.TimestampRange)
                            return object;
                        var message = new $root.google.bigtable.v2.TimestampRange();
                        if (object.startTimestampMicros != null)
                            if ($util.Long)
                                (message.startTimestampMicros = $util.Long.fromValue(object.startTimestampMicros)).unsigned = false;
                            else if (typeof object.startTimestampMicros === "string")
                                message.startTimestampMicros = parseInt(object.startTimestampMicros, 10);
                            else if (typeof object.startTimestampMicros === "number")
                                message.startTimestampMicros = object.startTimestampMicros;
                            else if (typeof object.startTimestampMicros === "object")
                                message.startTimestampMicros = new $util.LongBits(object.startTimestampMicros.low >>> 0, object.startTimestampMicros.high >>> 0).toNumber();
                        if (object.endTimestampMicros != null)
                            if ($util.Long)
                                (message.endTimestampMicros = $util.Long.fromValue(object.endTimestampMicros)).unsigned = false;
                            else if (typeof object.endTimestampMicros === "string")
                                message.endTimestampMicros = parseInt(object.endTimestampMicros, 10);
                            else if (typeof object.endTimestampMicros === "number")
                                message.endTimestampMicros = object.endTimestampMicros;
                            else if (typeof object.endTimestampMicros === "object")
                                message.endTimestampMicros = new $util.LongBits(object.endTimestampMicros.low >>> 0, object.endTimestampMicros.high >>> 0).toNumber();
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a TimestampRange message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.TimestampRange
                     * @static
                     * @param {google.bigtable.v2.TimestampRange} message TimestampRange
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    TimestampRange.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults) {
                            if ($util.Long) {
                                var long = new $util.Long(0, 0, false);
                                object.startTimestampMicros = options.longs === String ? long.toString() : options.longs === Number ? long.toNumber() : long;
                            } else
                                object.startTimestampMicros = options.longs === String ? "0" : 0;
                            if ($util.Long) {
                                var long = new $util.Long(0, 0, false);
                                object.endTimestampMicros = options.longs === String ? long.toString() : options.longs === Number ? long.toNumber() : long;
                            } else
                                object.endTimestampMicros = options.longs === String ? "0" : 0;
                        }
                        if (message.startTimestampMicros != null && message.hasOwnProperty("startTimestampMicros"))
                            if (typeof message.startTimestampMicros === "number")
                                object.startTimestampMicros = options.longs === String ? String(message.startTimestampMicros) : message.startTimestampMicros;
                            else
                                object.startTimestampMicros = options.longs === String ? $util.Long.prototype.toString.call(message.startTimestampMicros) : options.longs === Number ? new $util.LongBits(message.startTimestampMicros.low >>> 0, message.startTimestampMicros.high >>> 0).toNumber() : message.startTimestampMicros;
                        if (message.endTimestampMicros != null && message.hasOwnProperty("endTimestampMicros"))
                            if (typeof message.endTimestampMicros === "number")
                                object.endTimestampMicros = options.longs === String ? String(message.endTimestampMicros) : message.endTimestampMicros;
                            else
                                object.endTimestampMicros = options.longs === String ? $util.Long.prototype.toString.call(message.endTimestampMicros) : options.longs === Number ? new $util.LongBits(message.endTimestampMicros.low >>> 0, message.endTimestampMicros.high >>> 0).toNumber() : message.endTimestampMicros;
                        return object;
                    };
    
                    /**
                     * Converts this TimestampRange to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.TimestampRange
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    TimestampRange.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for TimestampRange
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.TimestampRange
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    TimestampRange.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.TimestampRange";
                    };
    
                    return TimestampRange;
                })();
    
                v2.ValueRange = (function() {
    
                    /**
                     * Properties of a ValueRange.
                     * @memberof google.bigtable.v2
                     * @interface IValueRange
                     * @property {Uint8Array|null} [startValueClosed] ValueRange startValueClosed
                     * @property {Uint8Array|null} [startValueOpen] ValueRange startValueOpen
                     * @property {Uint8Array|null} [endValueClosed] ValueRange endValueClosed
                     * @property {Uint8Array|null} [endValueOpen] ValueRange endValueOpen
                     */
    
                    /**
                     * Constructs a new ValueRange.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a ValueRange.
                     * @implements IValueRange
                     * @constructor
                     * @param {google.bigtable.v2.IValueRange=} [properties] Properties to set
                     */
                    function ValueRange(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * ValueRange startValueClosed.
                     * @member {Uint8Array|null|undefined} startValueClosed
                     * @memberof google.bigtable.v2.ValueRange
                     * @instance
                     */
                    ValueRange.prototype.startValueClosed = null;
    
                    /**
                     * ValueRange startValueOpen.
                     * @member {Uint8Array|null|undefined} startValueOpen
                     * @memberof google.bigtable.v2.ValueRange
                     * @instance
                     */
                    ValueRange.prototype.startValueOpen = null;
    
                    /**
                     * ValueRange endValueClosed.
                     * @member {Uint8Array|null|undefined} endValueClosed
                     * @memberof google.bigtable.v2.ValueRange
                     * @instance
                     */
                    ValueRange.prototype.endValueClosed = null;
    
                    /**
                     * ValueRange endValueOpen.
                     * @member {Uint8Array|null|undefined} endValueOpen
                     * @memberof google.bigtable.v2.ValueRange
                     * @instance
                     */
                    ValueRange.prototype.endValueOpen = null;
    
                    // OneOf field names bound to virtual getters and setters
                    var $oneOfFields;
    
                    /**
                     * ValueRange startValue.
                     * @member {"startValueClosed"|"startValueOpen"|undefined} startValue
                     * @memberof google.bigtable.v2.ValueRange
                     * @instance
                     */
                    Object.defineProperty(ValueRange.prototype, "startValue", {
                        get: $util.oneOfGetter($oneOfFields = ["startValueClosed", "startValueOpen"]),
                        set: $util.oneOfSetter($oneOfFields)
                    });
    
                    /**
                     * ValueRange endValue.
                     * @member {"endValueClosed"|"endValueOpen"|undefined} endValue
                     * @memberof google.bigtable.v2.ValueRange
                     * @instance
                     */
                    Object.defineProperty(ValueRange.prototype, "endValue", {
                        get: $util.oneOfGetter($oneOfFields = ["endValueClosed", "endValueOpen"]),
                        set: $util.oneOfSetter($oneOfFields)
                    });
    
                    /**
                     * Creates a new ValueRange instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.ValueRange
                     * @static
                     * @param {google.bigtable.v2.IValueRange=} [properties] Properties to set
                     * @returns {google.bigtable.v2.ValueRange} ValueRange instance
                     */
                    ValueRange.create = function create(properties) {
                        return new ValueRange(properties);
                    };
    
                    /**
                     * Encodes the specified ValueRange message. Does not implicitly {@link google.bigtable.v2.ValueRange.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.ValueRange
                     * @static
                     * @param {google.bigtable.v2.IValueRange} message ValueRange message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ValueRange.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.startValueClosed != null && Object.hasOwnProperty.call(message, "startValueClosed"))
                            writer.uint32(/* id 1, wireType 2 =*/10).bytes(message.startValueClosed);
                        if (message.startValueOpen != null && Object.hasOwnProperty.call(message, "startValueOpen"))
                            writer.uint32(/* id 2, wireType 2 =*/18).bytes(message.startValueOpen);
                        if (message.endValueClosed != null && Object.hasOwnProperty.call(message, "endValueClosed"))
                            writer.uint32(/* id 3, wireType 2 =*/26).bytes(message.endValueClosed);
                        if (message.endValueOpen != null && Object.hasOwnProperty.call(message, "endValueOpen"))
                            writer.uint32(/* id 4, wireType 2 =*/34).bytes(message.endValueOpen);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified ValueRange message, length delimited. Does not implicitly {@link google.bigtable.v2.ValueRange.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.ValueRange
                     * @static
                     * @param {google.bigtable.v2.IValueRange} message ValueRange message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ValueRange.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a ValueRange message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.ValueRange
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.ValueRange} ValueRange
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ValueRange.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.ValueRange();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.startValueClosed = reader.bytes();
                                    break;
                                }
                            case 2: {
                                    message.startValueOpen = reader.bytes();
                                    break;
                                }
                            case 3: {
                                    message.endValueClosed = reader.bytes();
                                    break;
                                }
                            case 4: {
                                    message.endValueOpen = reader.bytes();
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a ValueRange message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.ValueRange
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.ValueRange} ValueRange
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ValueRange.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a ValueRange message.
                     * @function verify
                     * @memberof google.bigtable.v2.ValueRange
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    ValueRange.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        var properties = {};
                        if (message.startValueClosed != null && message.hasOwnProperty("startValueClosed")) {
                            properties.startValue = 1;
                            if (!(message.startValueClosed && typeof message.startValueClosed.length === "number" || $util.isString(message.startValueClosed)))
                                return "startValueClosed: buffer expected";
                        }
                        if (message.startValueOpen != null && message.hasOwnProperty("startValueOpen")) {
                            if (properties.startValue === 1)
                                return "startValue: multiple values";
                            properties.startValue = 1;
                            if (!(message.startValueOpen && typeof message.startValueOpen.length === "number" || $util.isString(message.startValueOpen)))
                                return "startValueOpen: buffer expected";
                        }
                        if (message.endValueClosed != null && message.hasOwnProperty("endValueClosed")) {
                            properties.endValue = 1;
                            if (!(message.endValueClosed && typeof message.endValueClosed.length === "number" || $util.isString(message.endValueClosed)))
                                return "endValueClosed: buffer expected";
                        }
                        if (message.endValueOpen != null && message.hasOwnProperty("endValueOpen")) {
                            if (properties.endValue === 1)
                                return "endValue: multiple values";
                            properties.endValue = 1;
                            if (!(message.endValueOpen && typeof message.endValueOpen.length === "number" || $util.isString(message.endValueOpen)))
                                return "endValueOpen: buffer expected";
                        }
                        return null;
                    };
    
                    /**
                     * Creates a ValueRange message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.ValueRange
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.ValueRange} ValueRange
                     */
                    ValueRange.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.ValueRange)
                            return object;
                        var message = new $root.google.bigtable.v2.ValueRange();
                        if (object.startValueClosed != null)
                            if (typeof object.startValueClosed === "string")
                                $util.base64.decode(object.startValueClosed, message.startValueClosed = $util.newBuffer($util.base64.length(object.startValueClosed)), 0);
                            else if (object.startValueClosed.length >= 0)
                                message.startValueClosed = object.startValueClosed;
                        if (object.startValueOpen != null)
                            if (typeof object.startValueOpen === "string")
                                $util.base64.decode(object.startValueOpen, message.startValueOpen = $util.newBuffer($util.base64.length(object.startValueOpen)), 0);
                            else if (object.startValueOpen.length >= 0)
                                message.startValueOpen = object.startValueOpen;
                        if (object.endValueClosed != null)
                            if (typeof object.endValueClosed === "string")
                                $util.base64.decode(object.endValueClosed, message.endValueClosed = $util.newBuffer($util.base64.length(object.endValueClosed)), 0);
                            else if (object.endValueClosed.length >= 0)
                                message.endValueClosed = object.endValueClosed;
                        if (object.endValueOpen != null)
                            if (typeof object.endValueOpen === "string")
                                $util.base64.decode(object.endValueOpen, message.endValueOpen = $util.newBuffer($util.base64.length(object.endValueOpen)), 0);
                            else if (object.endValueOpen.length >= 0)
                                message.endValueOpen = object.endValueOpen;
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a ValueRange message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.ValueRange
                     * @static
                     * @param {google.bigtable.v2.ValueRange} message ValueRange
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    ValueRange.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (message.startValueClosed != null && message.hasOwnProperty("startValueClosed")) {
                            object.startValueClosed = options.bytes === String ? $util.base64.encode(message.startValueClosed, 0, message.startValueClosed.length) : options.bytes === Array ? Array.prototype.slice.call(message.startValueClosed) : message.startValueClosed;
                            if (options.oneofs)
                                object.startValue = "startValueClosed";
                        }
                        if (message.startValueOpen != null && message.hasOwnProperty("startValueOpen")) {
                            object.startValueOpen = options.bytes === String ? $util.base64.encode(message.startValueOpen, 0, message.startValueOpen.length) : options.bytes === Array ? Array.prototype.slice.call(message.startValueOpen) : message.startValueOpen;
                            if (options.oneofs)
                                object.startValue = "startValueOpen";
                        }
                        if (message.endValueClosed != null && message.hasOwnProperty("endValueClosed")) {
                            object.endValueClosed = options.bytes === String ? $util.base64.encode(message.endValueClosed, 0, message.endValueClosed.length) : options.bytes === Array ? Array.prototype.slice.call(message.endValueClosed) : message.endValueClosed;
                            if (options.oneofs)
                                object.endValue = "endValueClosed";
                        }
                        if (message.endValueOpen != null && message.hasOwnProperty("endValueOpen")) {
                            object.endValueOpen = options.bytes === String ? $util.base64.encode(message.endValueOpen, 0, message.endValueOpen.length) : options.bytes === Array ? Array.prototype.slice.call(message.endValueOpen) : message.endValueOpen;
                            if (options.oneofs)
                                object.endValue = "endValueOpen";
                        }
                        return object;
                    };
    
                    /**
                     * Converts this ValueRange to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.ValueRange
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    ValueRange.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for ValueRange
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.ValueRange
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    ValueRange.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.ValueRange";
                    };
    
                    return ValueRange;
                })();
    
                v2.RowFilter = (function() {
    
                    /**
                     * Properties of a RowFilter.
                     * @memberof google.bigtable.v2
                     * @interface IRowFilter
                     * @property {google.bigtable.v2.RowFilter.IChain|null} [chain] RowFilter chain
                     * @property {google.bigtable.v2.RowFilter.IInterleave|null} [interleave] RowFilter interleave
                     * @property {google.bigtable.v2.RowFilter.ICondition|null} [condition] RowFilter condition
                     * @property {boolean|null} [sink] RowFilter sink
                     * @property {boolean|null} [passAllFilter] RowFilter passAllFilter
                     * @property {boolean|null} [blockAllFilter] RowFilter blockAllFilter
                     * @property {Uint8Array|null} [rowKeyRegexFilter] RowFilter rowKeyRegexFilter
                     * @property {number|null} [rowSampleFilter] RowFilter rowSampleFilter
                     * @property {string|null} [familyNameRegexFilter] RowFilter familyNameRegexFilter
                     * @property {Uint8Array|null} [columnQualifierRegexFilter] RowFilter columnQualifierRegexFilter
                     * @property {google.bigtable.v2.IColumnRange|null} [columnRangeFilter] RowFilter columnRangeFilter
                     * @property {google.bigtable.v2.ITimestampRange|null} [timestampRangeFilter] RowFilter timestampRangeFilter
                     * @property {Uint8Array|null} [valueRegexFilter] RowFilter valueRegexFilter
                     * @property {google.bigtable.v2.IValueRange|null} [valueRangeFilter] RowFilter valueRangeFilter
                     * @property {number|null} [cellsPerRowOffsetFilter] RowFilter cellsPerRowOffsetFilter
                     * @property {number|null} [cellsPerRowLimitFilter] RowFilter cellsPerRowLimitFilter
                     * @property {number|null} [cellsPerColumnLimitFilter] RowFilter cellsPerColumnLimitFilter
                     * @property {boolean|null} [stripValueTransformer] RowFilter stripValueTransformer
                     * @property {string|null} [applyLabelTransformer] RowFilter applyLabelTransformer
                     */
    
                    /**
                     * Constructs a new RowFilter.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a RowFilter.
                     * @implements IRowFilter
                     * @constructor
                     * @param {google.bigtable.v2.IRowFilter=} [properties] Properties to set
                     */
                    function RowFilter(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * RowFilter chain.
                     * @member {google.bigtable.v2.RowFilter.IChain|null|undefined} chain
                     * @memberof google.bigtable.v2.RowFilter
                     * @instance
                     */
                    RowFilter.prototype.chain = null;
    
                    /**
                     * RowFilter interleave.
                     * @member {google.bigtable.v2.RowFilter.IInterleave|null|undefined} interleave
                     * @memberof google.bigtable.v2.RowFilter
                     * @instance
                     */
                    RowFilter.prototype.interleave = null;
    
                    /**
                     * RowFilter condition.
                     * @member {google.bigtable.v2.RowFilter.ICondition|null|undefined} condition
                     * @memberof google.bigtable.v2.RowFilter
                     * @instance
                     */
                    RowFilter.prototype.condition = null;
    
                    /**
                     * RowFilter sink.
                     * @member {boolean|null|undefined} sink
                     * @memberof google.bigtable.v2.RowFilter
                     * @instance
                     */
                    RowFilter.prototype.sink = null;
    
                    /**
                     * RowFilter passAllFilter.
                     * @member {boolean|null|undefined} passAllFilter
                     * @memberof google.bigtable.v2.RowFilter
                     * @instance
                     */
                    RowFilter.prototype.passAllFilter = null;
    
                    /**
                     * RowFilter blockAllFilter.
                     * @member {boolean|null|undefined} blockAllFilter
                     * @memberof google.bigtable.v2.RowFilter
                     * @instance
                     */
                    RowFilter.prototype.blockAllFilter = null;
    
                    /**
                     * RowFilter rowKeyRegexFilter.
                     * @member {Uint8Array|null|undefined} rowKeyRegexFilter
                     * @memberof google.bigtable.v2.RowFilter
                     * @instance
                     */
                    RowFilter.prototype.rowKeyRegexFilter = null;
    
                    /**
                     * RowFilter rowSampleFilter.
                     * @member {number|null|undefined} rowSampleFilter
                     * @memberof google.bigtable.v2.RowFilter
                     * @instance
                     */
                    RowFilter.prototype.rowSampleFilter = null;
    
                    /**
                     * RowFilter familyNameRegexFilter.
                     * @member {string|null|undefined} familyNameRegexFilter
                     * @memberof google.bigtable.v2.RowFilter
                     * @instance
                     */
                    RowFilter.prototype.familyNameRegexFilter = null;
    
                    /**
                     * RowFilter columnQualifierRegexFilter.
                     * @member {Uint8Array|null|undefined} columnQualifierRegexFilter
                     * @memberof google.bigtable.v2.RowFilter
                     * @instance
                     */
                    RowFilter.prototype.columnQualifierRegexFilter = null;
    
                    /**
                     * RowFilter columnRangeFilter.
                     * @member {google.bigtable.v2.IColumnRange|null|undefined} columnRangeFilter
                     * @memberof google.bigtable.v2.RowFilter
                     * @instance
                     */
                    RowFilter.prototype.columnRangeFilter = null;
    
                    /**
                     * RowFilter timestampRangeFilter.
                     * @member {google.bigtable.v2.ITimestampRange|null|undefined} timestampRangeFilter
                     * @memberof google.bigtable.v2.RowFilter
                     * @instance
                     */
                    RowFilter.prototype.timestampRangeFilter = null;
    
                    /**
                     * RowFilter valueRegexFilter.
                     * @member {Uint8Array|null|undefined} valueRegexFilter
                     * @memberof google.bigtable.v2.RowFilter
                     * @instance
                     */
                    RowFilter.prototype.valueRegexFilter = null;
    
                    /**
                     * RowFilter valueRangeFilter.
                     * @member {google.bigtable.v2.IValueRange|null|undefined} valueRangeFilter
                     * @memberof google.bigtable.v2.RowFilter
                     * @instance
                     */
                    RowFilter.prototype.valueRangeFilter = null;
    
                    /**
                     * RowFilter cellsPerRowOffsetFilter.
                     * @member {number|null|undefined} cellsPerRowOffsetFilter
                     * @memberof google.bigtable.v2.RowFilter
                     * @instance
                     */
                    RowFilter.prototype.cellsPerRowOffsetFilter = null;
    
                    /**
                     * RowFilter cellsPerRowLimitFilter.
                     * @member {number|null|undefined} cellsPerRowLimitFilter
                     * @memberof google.bigtable.v2.RowFilter
                     * @instance
                     */
                    RowFilter.prototype.cellsPerRowLimitFilter = null;
    
                    /**
                     * RowFilter cellsPerColumnLimitFilter.
                     * @member {number|null|undefined} cellsPerColumnLimitFilter
                     * @memberof google.bigtable.v2.RowFilter
                     * @instance
                     */
                    RowFilter.prototype.cellsPerColumnLimitFilter = null;
    
                    /**
                     * RowFilter stripValueTransformer.
                     * @member {boolean|null|undefined} stripValueTransformer
                     * @memberof google.bigtable.v2.RowFilter
                     * @instance
                     */
                    RowFilter.prototype.stripValueTransformer = null;
    
                    /**
                     * RowFilter applyLabelTransformer.
                     * @member {string|null|undefined} applyLabelTransformer
                     * @memberof google.bigtable.v2.RowFilter
                     * @instance
                     */
                    RowFilter.prototype.applyLabelTransformer = null;
    
                    // OneOf field names bound to virtual getters and setters
                    var $oneOfFields;
    
                    /**
                     * RowFilter filter.
                     * @member {"chain"|"interleave"|"condition"|"sink"|"passAllFilter"|"blockAllFilter"|"rowKeyRegexFilter"|"rowSampleFilter"|"familyNameRegexFilter"|"columnQualifierRegexFilter"|"columnRangeFilter"|"timestampRangeFilter"|"valueRegexFilter"|"valueRangeFilter"|"cellsPerRowOffsetFilter"|"cellsPerRowLimitFilter"|"cellsPerColumnLimitFilter"|"stripValueTransformer"|"applyLabelTransformer"|undefined} filter
                     * @memberof google.bigtable.v2.RowFilter
                     * @instance
                     */
                    Object.defineProperty(RowFilter.prototype, "filter", {
                        get: $util.oneOfGetter($oneOfFields = ["chain", "interleave", "condition", "sink", "passAllFilter", "blockAllFilter", "rowKeyRegexFilter", "rowSampleFilter", "familyNameRegexFilter", "columnQualifierRegexFilter", "columnRangeFilter", "timestampRangeFilter", "valueRegexFilter", "valueRangeFilter", "cellsPerRowOffsetFilter", "cellsPerRowLimitFilter", "cellsPerColumnLimitFilter", "stripValueTransformer", "applyLabelTransformer"]),
                        set: $util.oneOfSetter($oneOfFields)
                    });
    
                    /**
                     * Creates a new RowFilter instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.RowFilter
                     * @static
                     * @param {google.bigtable.v2.IRowFilter=} [properties] Properties to set
                     * @returns {google.bigtable.v2.RowFilter} RowFilter instance
                     */
                    RowFilter.create = function create(properties) {
                        return new RowFilter(properties);
                    };
    
                    /**
                     * Encodes the specified RowFilter message. Does not implicitly {@link google.bigtable.v2.RowFilter.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.RowFilter
                     * @static
                     * @param {google.bigtable.v2.IRowFilter} message RowFilter message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    RowFilter.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.chain != null && Object.hasOwnProperty.call(message, "chain"))
                            $root.google.bigtable.v2.RowFilter.Chain.encode(message.chain, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                        if (message.interleave != null && Object.hasOwnProperty.call(message, "interleave"))
                            $root.google.bigtable.v2.RowFilter.Interleave.encode(message.interleave, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                        if (message.condition != null && Object.hasOwnProperty.call(message, "condition"))
                            $root.google.bigtable.v2.RowFilter.Condition.encode(message.condition, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                        if (message.rowKeyRegexFilter != null && Object.hasOwnProperty.call(message, "rowKeyRegexFilter"))
                            writer.uint32(/* id 4, wireType 2 =*/34).bytes(message.rowKeyRegexFilter);
                        if (message.familyNameRegexFilter != null && Object.hasOwnProperty.call(message, "familyNameRegexFilter"))
                            writer.uint32(/* id 5, wireType 2 =*/42).string(message.familyNameRegexFilter);
                        if (message.columnQualifierRegexFilter != null && Object.hasOwnProperty.call(message, "columnQualifierRegexFilter"))
                            writer.uint32(/* id 6, wireType 2 =*/50).bytes(message.columnQualifierRegexFilter);
                        if (message.columnRangeFilter != null && Object.hasOwnProperty.call(message, "columnRangeFilter"))
                            $root.google.bigtable.v2.ColumnRange.encode(message.columnRangeFilter, writer.uint32(/* id 7, wireType 2 =*/58).fork()).ldelim();
                        if (message.timestampRangeFilter != null && Object.hasOwnProperty.call(message, "timestampRangeFilter"))
                            $root.google.bigtable.v2.TimestampRange.encode(message.timestampRangeFilter, writer.uint32(/* id 8, wireType 2 =*/66).fork()).ldelim();
                        if (message.valueRegexFilter != null && Object.hasOwnProperty.call(message, "valueRegexFilter"))
                            writer.uint32(/* id 9, wireType 2 =*/74).bytes(message.valueRegexFilter);
                        if (message.cellsPerRowOffsetFilter != null && Object.hasOwnProperty.call(message, "cellsPerRowOffsetFilter"))
                            writer.uint32(/* id 10, wireType 0 =*/80).int32(message.cellsPerRowOffsetFilter);
                        if (message.cellsPerRowLimitFilter != null && Object.hasOwnProperty.call(message, "cellsPerRowLimitFilter"))
                            writer.uint32(/* id 11, wireType 0 =*/88).int32(message.cellsPerRowLimitFilter);
                        if (message.cellsPerColumnLimitFilter != null && Object.hasOwnProperty.call(message, "cellsPerColumnLimitFilter"))
                            writer.uint32(/* id 12, wireType 0 =*/96).int32(message.cellsPerColumnLimitFilter);
                        if (message.stripValueTransformer != null && Object.hasOwnProperty.call(message, "stripValueTransformer"))
                            writer.uint32(/* id 13, wireType 0 =*/104).bool(message.stripValueTransformer);
                        if (message.rowSampleFilter != null && Object.hasOwnProperty.call(message, "rowSampleFilter"))
                            writer.uint32(/* id 14, wireType 1 =*/113).double(message.rowSampleFilter);
                        if (message.valueRangeFilter != null && Object.hasOwnProperty.call(message, "valueRangeFilter"))
                            $root.google.bigtable.v2.ValueRange.encode(message.valueRangeFilter, writer.uint32(/* id 15, wireType 2 =*/122).fork()).ldelim();
                        if (message.sink != null && Object.hasOwnProperty.call(message, "sink"))
                            writer.uint32(/* id 16, wireType 0 =*/128).bool(message.sink);
                        if (message.passAllFilter != null && Object.hasOwnProperty.call(message, "passAllFilter"))
                            writer.uint32(/* id 17, wireType 0 =*/136).bool(message.passAllFilter);
                        if (message.blockAllFilter != null && Object.hasOwnProperty.call(message, "blockAllFilter"))
                            writer.uint32(/* id 18, wireType 0 =*/144).bool(message.blockAllFilter);
                        if (message.applyLabelTransformer != null && Object.hasOwnProperty.call(message, "applyLabelTransformer"))
                            writer.uint32(/* id 19, wireType 2 =*/154).string(message.applyLabelTransformer);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified RowFilter message, length delimited. Does not implicitly {@link google.bigtable.v2.RowFilter.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.RowFilter
                     * @static
                     * @param {google.bigtable.v2.IRowFilter} message RowFilter message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    RowFilter.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a RowFilter message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.RowFilter
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.RowFilter} RowFilter
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    RowFilter.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.RowFilter();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.chain = $root.google.bigtable.v2.RowFilter.Chain.decode(reader, reader.uint32());
                                    break;
                                }
                            case 2: {
                                    message.interleave = $root.google.bigtable.v2.RowFilter.Interleave.decode(reader, reader.uint32());
                                    break;
                                }
                            case 3: {
                                    message.condition = $root.google.bigtable.v2.RowFilter.Condition.decode(reader, reader.uint32());
                                    break;
                                }
                            case 16: {
                                    message.sink = reader.bool();
                                    break;
                                }
                            case 17: {
                                    message.passAllFilter = reader.bool();
                                    break;
                                }
                            case 18: {
                                    message.blockAllFilter = reader.bool();
                                    break;
                                }
                            case 4: {
                                    message.rowKeyRegexFilter = reader.bytes();
                                    break;
                                }
                            case 14: {
                                    message.rowSampleFilter = reader.double();
                                    break;
                                }
                            case 5: {
                                    message.familyNameRegexFilter = reader.string();
                                    break;
                                }
                            case 6: {
                                    message.columnQualifierRegexFilter = reader.bytes();
                                    break;
                                }
                            case 7: {
                                    message.columnRangeFilter = $root.google.bigtable.v2.ColumnRange.decode(reader, reader.uint32());
                                    break;
                                }
                            case 8: {
                                    message.timestampRangeFilter = $root.google.bigtable.v2.TimestampRange.decode(reader, reader.uint32());
                                    break;
                                }
                            case 9: {
                                    message.valueRegexFilter = reader.bytes();
                                    break;
                                }
                            case 15: {
                                    message.valueRangeFilter = $root.google.bigtable.v2.ValueRange.decode(reader, reader.uint32());
                                    break;
                                }
                            case 10: {
                                    message.cellsPerRowOffsetFilter = reader.int32();
                                    break;
                                }
                            case 11: {
                                    message.cellsPerRowLimitFilter = reader.int32();
                                    break;
                                }
                            case 12: {
                                    message.cellsPerColumnLimitFilter = reader.int32();
                                    break;
                                }
                            case 13: {
                                    message.stripValueTransformer = reader.bool();
                                    break;
                                }
                            case 19: {
                                    message.applyLabelTransformer = reader.string();
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a RowFilter message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.RowFilter
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.RowFilter} RowFilter
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    RowFilter.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a RowFilter message.
                     * @function verify
                     * @memberof google.bigtable.v2.RowFilter
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    RowFilter.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        var properties = {};
                        if (message.chain != null && message.hasOwnProperty("chain")) {
                            properties.filter = 1;
                            {
                                var error = $root.google.bigtable.v2.RowFilter.Chain.verify(message.chain);
                                if (error)
                                    return "chain." + error;
                            }
                        }
                        if (message.interleave != null && message.hasOwnProperty("interleave")) {
                            if (properties.filter === 1)
                                return "filter: multiple values";
                            properties.filter = 1;
                            {
                                var error = $root.google.bigtable.v2.RowFilter.Interleave.verify(message.interleave);
                                if (error)
                                    return "interleave." + error;
                            }
                        }
                        if (message.condition != null && message.hasOwnProperty("condition")) {
                            if (properties.filter === 1)
                                return "filter: multiple values";
                            properties.filter = 1;
                            {
                                var error = $root.google.bigtable.v2.RowFilter.Condition.verify(message.condition);
                                if (error)
                                    return "condition." + error;
                            }
                        }
                        if (message.sink != null && message.hasOwnProperty("sink")) {
                            if (properties.filter === 1)
                                return "filter: multiple values";
                            properties.filter = 1;
                            if (typeof message.sink !== "boolean")
                                return "sink: boolean expected";
                        }
                        if (message.passAllFilter != null && message.hasOwnProperty("passAllFilter")) {
                            if (properties.filter === 1)
                                return "filter: multiple values";
                            properties.filter = 1;
                            if (typeof message.passAllFilter !== "boolean")
                                return "passAllFilter: boolean expected";
                        }
                        if (message.blockAllFilter != null && message.hasOwnProperty("blockAllFilter")) {
                            if (properties.filter === 1)
                                return "filter: multiple values";
                            properties.filter = 1;
                            if (typeof message.blockAllFilter !== "boolean")
                                return "blockAllFilter: boolean expected";
                        }
                        if (message.rowKeyRegexFilter != null && message.hasOwnProperty("rowKeyRegexFilter")) {
                            if (properties.filter === 1)
                                return "filter: multiple values";
                            properties.filter = 1;
                            if (!(message.rowKeyRegexFilter && typeof message.rowKeyRegexFilter.length === "number" || $util.isString(message.rowKeyRegexFilter)))
                                return "rowKeyRegexFilter: buffer expected";
                        }
                        if (message.rowSampleFilter != null && message.hasOwnProperty("rowSampleFilter")) {
                            if (properties.filter === 1)
                                return "filter: multiple values";
                            properties.filter = 1;
                            if (typeof message.rowSampleFilter !== "number")
                                return "rowSampleFilter: number expected";
                        }
                        if (message.familyNameRegexFilter != null && message.hasOwnProperty("familyNameRegexFilter")) {
                            if (properties.filter === 1)
                                return "filter: multiple values";
                            properties.filter = 1;
                            if (!$util.isString(message.familyNameRegexFilter))
                                return "familyNameRegexFilter: string expected";
                        }
                        if (message.columnQualifierRegexFilter != null && message.hasOwnProperty("columnQualifierRegexFilter")) {
                            if (properties.filter === 1)
                                return "filter: multiple values";
                            properties.filter = 1;
                            if (!(message.columnQualifierRegexFilter && typeof message.columnQualifierRegexFilter.length === "number" || $util.isString(message.columnQualifierRegexFilter)))
                                return "columnQualifierRegexFilter: buffer expected";
                        }
                        if (message.columnRangeFilter != null && message.hasOwnProperty("columnRangeFilter")) {
                            if (properties.filter === 1)
                                return "filter: multiple values";
                            properties.filter = 1;
                            {
                                var error = $root.google.bigtable.v2.ColumnRange.verify(message.columnRangeFilter);
                                if (error)
                                    return "columnRangeFilter." + error;
                            }
                        }
                        if (message.timestampRangeFilter != null && message.hasOwnProperty("timestampRangeFilter")) {
                            if (properties.filter === 1)
                                return "filter: multiple values";
                            properties.filter = 1;
                            {
                                var error = $root.google.bigtable.v2.TimestampRange.verify(message.timestampRangeFilter);
                                if (error)
                                    return "timestampRangeFilter." + error;
                            }
                        }
                        if (message.valueRegexFilter != null && message.hasOwnProperty("valueRegexFilter")) {
                            if (properties.filter === 1)
                                return "filter: multiple values";
                            properties.filter = 1;
                            if (!(message.valueRegexFilter && typeof message.valueRegexFilter.length === "number" || $util.isString(message.valueRegexFilter)))
                                return "valueRegexFilter: buffer expected";
                        }
                        if (message.valueRangeFilter != null && message.hasOwnProperty("valueRangeFilter")) {
                            if (properties.filter === 1)
                                return "filter: multiple values";
                            properties.filter = 1;
                            {
                                var error = $root.google.bigtable.v2.ValueRange.verify(message.valueRangeFilter);
                                if (error)
                                    return "valueRangeFilter." + error;
                            }
                        }
                        if (message.cellsPerRowOffsetFilter != null && message.hasOwnProperty("cellsPerRowOffsetFilter")) {
                            if (properties.filter === 1)
                                return "filter: multiple values";
                            properties.filter = 1;
                            if (!$util.isInteger(message.cellsPerRowOffsetFilter))
                                return "cellsPerRowOffsetFilter: integer expected";
                        }
                        if (message.cellsPerRowLimitFilter != null && message.hasOwnProperty("cellsPerRowLimitFilter")) {
                            if (properties.filter === 1)
                                return "filter: multiple values";
                            properties.filter = 1;
                            if (!$util.isInteger(message.cellsPerRowLimitFilter))
                                return "cellsPerRowLimitFilter: integer expected";
                        }
                        if (message.cellsPerColumnLimitFilter != null && message.hasOwnProperty("cellsPerColumnLimitFilter")) {
                            if (properties.filter === 1)
                                return "filter: multiple values";
                            properties.filter = 1;
                            if (!$util.isInteger(message.cellsPerColumnLimitFilter))
                                return "cellsPerColumnLimitFilter: integer expected";
                        }
                        if (message.stripValueTransformer != null && message.hasOwnProperty("stripValueTransformer")) {
                            if (properties.filter === 1)
                                return "filter: multiple values";
                            properties.filter = 1;
                            if (typeof message.stripValueTransformer !== "boolean")
                                return "stripValueTransformer: boolean expected";
                        }
                        if (message.applyLabelTransformer != null && message.hasOwnProperty("applyLabelTransformer")) {
                            if (properties.filter === 1)
                                return "filter: multiple values";
                            properties.filter = 1;
                            if (!$util.isString(message.applyLabelTransformer))
                                return "applyLabelTransformer: string expected";
                        }
                        return null;
                    };
    
                    /**
                     * Creates a RowFilter message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.RowFilter
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.RowFilter} RowFilter
                     */
                    RowFilter.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.RowFilter)
                            return object;
                        var message = new $root.google.bigtable.v2.RowFilter();
                        if (object.chain != null) {
                            if (typeof object.chain !== "object")
                                throw TypeError(".google.bigtable.v2.RowFilter.chain: object expected");
                            message.chain = $root.google.bigtable.v2.RowFilter.Chain.fromObject(object.chain);
                        }
                        if (object.interleave != null) {
                            if (typeof object.interleave !== "object")
                                throw TypeError(".google.bigtable.v2.RowFilter.interleave: object expected");
                            message.interleave = $root.google.bigtable.v2.RowFilter.Interleave.fromObject(object.interleave);
                        }
                        if (object.condition != null) {
                            if (typeof object.condition !== "object")
                                throw TypeError(".google.bigtable.v2.RowFilter.condition: object expected");
                            message.condition = $root.google.bigtable.v2.RowFilter.Condition.fromObject(object.condition);
                        }
                        if (object.sink != null)
                            message.sink = Boolean(object.sink);
                        if (object.passAllFilter != null)
                            message.passAllFilter = Boolean(object.passAllFilter);
                        if (object.blockAllFilter != null)
                            message.blockAllFilter = Boolean(object.blockAllFilter);
                        if (object.rowKeyRegexFilter != null)
                            if (typeof object.rowKeyRegexFilter === "string")
                                $util.base64.decode(object.rowKeyRegexFilter, message.rowKeyRegexFilter = $util.newBuffer($util.base64.length(object.rowKeyRegexFilter)), 0);
                            else if (object.rowKeyRegexFilter.length >= 0)
                                message.rowKeyRegexFilter = object.rowKeyRegexFilter;
                        if (object.rowSampleFilter != null)
                            message.rowSampleFilter = Number(object.rowSampleFilter);
                        if (object.familyNameRegexFilter != null)
                            message.familyNameRegexFilter = String(object.familyNameRegexFilter);
                        if (object.columnQualifierRegexFilter != null)
                            if (typeof object.columnQualifierRegexFilter === "string")
                                $util.base64.decode(object.columnQualifierRegexFilter, message.columnQualifierRegexFilter = $util.newBuffer($util.base64.length(object.columnQualifierRegexFilter)), 0);
                            else if (object.columnQualifierRegexFilter.length >= 0)
                                message.columnQualifierRegexFilter = object.columnQualifierRegexFilter;
                        if (object.columnRangeFilter != null) {
                            if (typeof object.columnRangeFilter !== "object")
                                throw TypeError(".google.bigtable.v2.RowFilter.columnRangeFilter: object expected");
                            message.columnRangeFilter = $root.google.bigtable.v2.ColumnRange.fromObject(object.columnRangeFilter);
                        }
                        if (object.timestampRangeFilter != null) {
                            if (typeof object.timestampRangeFilter !== "object")
                                throw TypeError(".google.bigtable.v2.RowFilter.timestampRangeFilter: object expected");
                            message.timestampRangeFilter = $root.google.bigtable.v2.TimestampRange.fromObject(object.timestampRangeFilter);
                        }
                        if (object.valueRegexFilter != null)
                            if (typeof object.valueRegexFilter === "string")
                                $util.base64.decode(object.valueRegexFilter, message.valueRegexFilter = $util.newBuffer($util.base64.length(object.valueRegexFilter)), 0);
                            else if (object.valueRegexFilter.length >= 0)
                                message.valueRegexFilter = object.valueRegexFilter;
                        if (object.valueRangeFilter != null) {
                            if (typeof object.valueRangeFilter !== "object")
                                throw TypeError(".google.bigtable.v2.RowFilter.valueRangeFilter: object expected");
                            message.valueRangeFilter = $root.google.bigtable.v2.ValueRange.fromObject(object.valueRangeFilter);
                        }
                        if (object.cellsPerRowOffsetFilter != null)
                            message.cellsPerRowOffsetFilter = object.cellsPerRowOffsetFilter | 0;
                        if (object.cellsPerRowLimitFilter != null)
                            message.cellsPerRowLimitFilter = object.cellsPerRowLimitFilter | 0;
                        if (object.cellsPerColumnLimitFilter != null)
                            message.cellsPerColumnLimitFilter = object.cellsPerColumnLimitFilter | 0;
                        if (object.stripValueTransformer != null)
                            message.stripValueTransformer = Boolean(object.stripValueTransformer);
                        if (object.applyLabelTransformer != null)
                            message.applyLabelTransformer = String(object.applyLabelTransformer);
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a RowFilter message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.RowFilter
                     * @static
                     * @param {google.bigtable.v2.RowFilter} message RowFilter
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    RowFilter.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (message.chain != null && message.hasOwnProperty("chain")) {
                            object.chain = $root.google.bigtable.v2.RowFilter.Chain.toObject(message.chain, options);
                            if (options.oneofs)
                                object.filter = "chain";
                        }
                        if (message.interleave != null && message.hasOwnProperty("interleave")) {
                            object.interleave = $root.google.bigtable.v2.RowFilter.Interleave.toObject(message.interleave, options);
                            if (options.oneofs)
                                object.filter = "interleave";
                        }
                        if (message.condition != null && message.hasOwnProperty("condition")) {
                            object.condition = $root.google.bigtable.v2.RowFilter.Condition.toObject(message.condition, options);
                            if (options.oneofs)
                                object.filter = "condition";
                        }
                        if (message.rowKeyRegexFilter != null && message.hasOwnProperty("rowKeyRegexFilter")) {
                            object.rowKeyRegexFilter = options.bytes === String ? $util.base64.encode(message.rowKeyRegexFilter, 0, message.rowKeyRegexFilter.length) : options.bytes === Array ? Array.prototype.slice.call(message.rowKeyRegexFilter) : message.rowKeyRegexFilter;
                            if (options.oneofs)
                                object.filter = "rowKeyRegexFilter";
                        }
                        if (message.familyNameRegexFilter != null && message.hasOwnProperty("familyNameRegexFilter")) {
                            object.familyNameRegexFilter = message.familyNameRegexFilter;
                            if (options.oneofs)
                                object.filter = "familyNameRegexFilter";
                        }
                        if (message.columnQualifierRegexFilter != null && message.hasOwnProperty("columnQualifierRegexFilter")) {
                            object.columnQualifierRegexFilter = options.bytes === String ? $util.base64.encode(message.columnQualifierRegexFilter, 0, message.columnQualifierRegexFilter.length) : options.bytes === Array ? Array.prototype.slice.call(message.columnQualifierRegexFilter) : message.columnQualifierRegexFilter;
                            if (options.oneofs)
                                object.filter = "columnQualifierRegexFilter";
                        }
                        if (message.columnRangeFilter != null && message.hasOwnProperty("columnRangeFilter")) {
                            object.columnRangeFilter = $root.google.bigtable.v2.ColumnRange.toObject(message.columnRangeFilter, options);
                            if (options.oneofs)
                                object.filter = "columnRangeFilter";
                        }
                        if (message.timestampRangeFilter != null && message.hasOwnProperty("timestampRangeFilter")) {
                            object.timestampRangeFilter = $root.google.bigtable.v2.TimestampRange.toObject(message.timestampRangeFilter, options);
                            if (options.oneofs)
                                object.filter = "timestampRangeFilter";
                        }
                        if (message.valueRegexFilter != null && message.hasOwnProperty("valueRegexFilter")) {
                            object.valueRegexFilter = options.bytes === String ? $util.base64.encode(message.valueRegexFilter, 0, message.valueRegexFilter.length) : options.bytes === Array ? Array.prototype.slice.call(message.valueRegexFilter) : message.valueRegexFilter;
                            if (options.oneofs)
                                object.filter = "valueRegexFilter";
                        }
                        if (message.cellsPerRowOffsetFilter != null && message.hasOwnProperty("cellsPerRowOffsetFilter")) {
                            object.cellsPerRowOffsetFilter = message.cellsPerRowOffsetFilter;
                            if (options.oneofs)
                                object.filter = "cellsPerRowOffsetFilter";
                        }
                        if (message.cellsPerRowLimitFilter != null && message.hasOwnProperty("cellsPerRowLimitFilter")) {
                            object.cellsPerRowLimitFilter = message.cellsPerRowLimitFilter;
                            if (options.oneofs)
                                object.filter = "cellsPerRowLimitFilter";
                        }
                        if (message.cellsPerColumnLimitFilter != null && message.hasOwnProperty("cellsPerColumnLimitFilter")) {
                            object.cellsPerColumnLimitFilter = message.cellsPerColumnLimitFilter;
                            if (options.oneofs)
                                object.filter = "cellsPerColumnLimitFilter";
                        }
                        if (message.stripValueTransformer != null && message.hasOwnProperty("stripValueTransformer")) {
                            object.stripValueTransformer = message.stripValueTransformer;
                            if (options.oneofs)
                                object.filter = "stripValueTransformer";
                        }
                        if (message.rowSampleFilter != null && message.hasOwnProperty("rowSampleFilter")) {
                            object.rowSampleFilter = options.json && !isFinite(message.rowSampleFilter) ? String(message.rowSampleFilter) : message.rowSampleFilter;
                            if (options.oneofs)
                                object.filter = "rowSampleFilter";
                        }
                        if (message.valueRangeFilter != null && message.hasOwnProperty("valueRangeFilter")) {
                            object.valueRangeFilter = $root.google.bigtable.v2.ValueRange.toObject(message.valueRangeFilter, options);
                            if (options.oneofs)
                                object.filter = "valueRangeFilter";
                        }
                        if (message.sink != null && message.hasOwnProperty("sink")) {
                            object.sink = message.sink;
                            if (options.oneofs)
                                object.filter = "sink";
                        }
                        if (message.passAllFilter != null && message.hasOwnProperty("passAllFilter")) {
                            object.passAllFilter = message.passAllFilter;
                            if (options.oneofs)
                                object.filter = "passAllFilter";
                        }
                        if (message.blockAllFilter != null && message.hasOwnProperty("blockAllFilter")) {
                            object.blockAllFilter = message.blockAllFilter;
                            if (options.oneofs)
                                object.filter = "blockAllFilter";
                        }
                        if (message.applyLabelTransformer != null && message.hasOwnProperty("applyLabelTransformer")) {
                            object.applyLabelTransformer = message.applyLabelTransformer;
                            if (options.oneofs)
                                object.filter = "applyLabelTransformer";
                        }
                        return object;
                    };
    
                    /**
                     * Converts this RowFilter to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.RowFilter
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    RowFilter.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for RowFilter
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.RowFilter
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    RowFilter.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.RowFilter";
                    };
    
                    RowFilter.Chain = (function() {
    
                        /**
                         * Properties of a Chain.
                         * @memberof google.bigtable.v2.RowFilter
                         * @interface IChain
                         * @property {Array.<google.bigtable.v2.IRowFilter>|null} [filters] Chain filters
                         */
    
                        /**
                         * Constructs a new Chain.
                         * @memberof google.bigtable.v2.RowFilter
                         * @classdesc Represents a Chain.
                         * @implements IChain
                         * @constructor
                         * @param {google.bigtable.v2.RowFilter.IChain=} [properties] Properties to set
                         */
                        function Chain(properties) {
                            this.filters = [];
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * Chain filters.
                         * @member {Array.<google.bigtable.v2.IRowFilter>} filters
                         * @memberof google.bigtable.v2.RowFilter.Chain
                         * @instance
                         */
                        Chain.prototype.filters = $util.emptyArray;
    
                        /**
                         * Creates a new Chain instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.v2.RowFilter.Chain
                         * @static
                         * @param {google.bigtable.v2.RowFilter.IChain=} [properties] Properties to set
                         * @returns {google.bigtable.v2.RowFilter.Chain} Chain instance
                         */
                        Chain.create = function create(properties) {
                            return new Chain(properties);
                        };
    
                        /**
                         * Encodes the specified Chain message. Does not implicitly {@link google.bigtable.v2.RowFilter.Chain.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.v2.RowFilter.Chain
                         * @static
                         * @param {google.bigtable.v2.RowFilter.IChain} message Chain message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        Chain.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.filters != null && message.filters.length)
                                for (var i = 0; i < message.filters.length; ++i)
                                    $root.google.bigtable.v2.RowFilter.encode(message.filters[i], writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified Chain message, length delimited. Does not implicitly {@link google.bigtable.v2.RowFilter.Chain.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.v2.RowFilter.Chain
                         * @static
                         * @param {google.bigtable.v2.RowFilter.IChain} message Chain message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        Chain.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a Chain message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.v2.RowFilter.Chain
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.v2.RowFilter.Chain} Chain
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        Chain.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.RowFilter.Chain();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        if (!(message.filters && message.filters.length))
                                            message.filters = [];
                                        message.filters.push($root.google.bigtable.v2.RowFilter.decode(reader, reader.uint32()));
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a Chain message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.v2.RowFilter.Chain
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.v2.RowFilter.Chain} Chain
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        Chain.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a Chain message.
                         * @function verify
                         * @memberof google.bigtable.v2.RowFilter.Chain
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        Chain.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.filters != null && message.hasOwnProperty("filters")) {
                                if (!Array.isArray(message.filters))
                                    return "filters: array expected";
                                for (var i = 0; i < message.filters.length; ++i) {
                                    var error = $root.google.bigtable.v2.RowFilter.verify(message.filters[i]);
                                    if (error)
                                        return "filters." + error;
                                }
                            }
                            return null;
                        };
    
                        /**
                         * Creates a Chain message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.v2.RowFilter.Chain
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.v2.RowFilter.Chain} Chain
                         */
                        Chain.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.v2.RowFilter.Chain)
                                return object;
                            var message = new $root.google.bigtable.v2.RowFilter.Chain();
                            if (object.filters) {
                                if (!Array.isArray(object.filters))
                                    throw TypeError(".google.bigtable.v2.RowFilter.Chain.filters: array expected");
                                message.filters = [];
                                for (var i = 0; i < object.filters.length; ++i) {
                                    if (typeof object.filters[i] !== "object")
                                        throw TypeError(".google.bigtable.v2.RowFilter.Chain.filters: object expected");
                                    message.filters[i] = $root.google.bigtable.v2.RowFilter.fromObject(object.filters[i]);
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a Chain message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.v2.RowFilter.Chain
                         * @static
                         * @param {google.bigtable.v2.RowFilter.Chain} message Chain
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        Chain.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.arrays || options.defaults)
                                object.filters = [];
                            if (message.filters && message.filters.length) {
                                object.filters = [];
                                for (var j = 0; j < message.filters.length; ++j)
                                    object.filters[j] = $root.google.bigtable.v2.RowFilter.toObject(message.filters[j], options);
                            }
                            return object;
                        };
    
                        /**
                         * Converts this Chain to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.v2.RowFilter.Chain
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        Chain.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for Chain
                         * @function getTypeUrl
                         * @memberof google.bigtable.v2.RowFilter.Chain
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        Chain.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.v2.RowFilter.Chain";
                        };
    
                        return Chain;
                    })();
    
                    RowFilter.Interleave = (function() {
    
                        /**
                         * Properties of an Interleave.
                         * @memberof google.bigtable.v2.RowFilter
                         * @interface IInterleave
                         * @property {Array.<google.bigtable.v2.IRowFilter>|null} [filters] Interleave filters
                         */
    
                        /**
                         * Constructs a new Interleave.
                         * @memberof google.bigtable.v2.RowFilter
                         * @classdesc Represents an Interleave.
                         * @implements IInterleave
                         * @constructor
                         * @param {google.bigtable.v2.RowFilter.IInterleave=} [properties] Properties to set
                         */
                        function Interleave(properties) {
                            this.filters = [];
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * Interleave filters.
                         * @member {Array.<google.bigtable.v2.IRowFilter>} filters
                         * @memberof google.bigtable.v2.RowFilter.Interleave
                         * @instance
                         */
                        Interleave.prototype.filters = $util.emptyArray;
    
                        /**
                         * Creates a new Interleave instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.v2.RowFilter.Interleave
                         * @static
                         * @param {google.bigtable.v2.RowFilter.IInterleave=} [properties] Properties to set
                         * @returns {google.bigtable.v2.RowFilter.Interleave} Interleave instance
                         */
                        Interleave.create = function create(properties) {
                            return new Interleave(properties);
                        };
    
                        /**
                         * Encodes the specified Interleave message. Does not implicitly {@link google.bigtable.v2.RowFilter.Interleave.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.v2.RowFilter.Interleave
                         * @static
                         * @param {google.bigtable.v2.RowFilter.IInterleave} message Interleave message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        Interleave.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.filters != null && message.filters.length)
                                for (var i = 0; i < message.filters.length; ++i)
                                    $root.google.bigtable.v2.RowFilter.encode(message.filters[i], writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified Interleave message, length delimited. Does not implicitly {@link google.bigtable.v2.RowFilter.Interleave.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.v2.RowFilter.Interleave
                         * @static
                         * @param {google.bigtable.v2.RowFilter.IInterleave} message Interleave message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        Interleave.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes an Interleave message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.v2.RowFilter.Interleave
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.v2.RowFilter.Interleave} Interleave
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        Interleave.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.RowFilter.Interleave();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        if (!(message.filters && message.filters.length))
                                            message.filters = [];
                                        message.filters.push($root.google.bigtable.v2.RowFilter.decode(reader, reader.uint32()));
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes an Interleave message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.v2.RowFilter.Interleave
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.v2.RowFilter.Interleave} Interleave
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        Interleave.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies an Interleave message.
                         * @function verify
                         * @memberof google.bigtable.v2.RowFilter.Interleave
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        Interleave.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.filters != null && message.hasOwnProperty("filters")) {
                                if (!Array.isArray(message.filters))
                                    return "filters: array expected";
                                for (var i = 0; i < message.filters.length; ++i) {
                                    var error = $root.google.bigtable.v2.RowFilter.verify(message.filters[i]);
                                    if (error)
                                        return "filters." + error;
                                }
                            }
                            return null;
                        };
    
                        /**
                         * Creates an Interleave message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.v2.RowFilter.Interleave
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.v2.RowFilter.Interleave} Interleave
                         */
                        Interleave.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.v2.RowFilter.Interleave)
                                return object;
                            var message = new $root.google.bigtable.v2.RowFilter.Interleave();
                            if (object.filters) {
                                if (!Array.isArray(object.filters))
                                    throw TypeError(".google.bigtable.v2.RowFilter.Interleave.filters: array expected");
                                message.filters = [];
                                for (var i = 0; i < object.filters.length; ++i) {
                                    if (typeof object.filters[i] !== "object")
                                        throw TypeError(".google.bigtable.v2.RowFilter.Interleave.filters: object expected");
                                    message.filters[i] = $root.google.bigtable.v2.RowFilter.fromObject(object.filters[i]);
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from an Interleave message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.v2.RowFilter.Interleave
                         * @static
                         * @param {google.bigtable.v2.RowFilter.Interleave} message Interleave
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        Interleave.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.arrays || options.defaults)
                                object.filters = [];
                            if (message.filters && message.filters.length) {
                                object.filters = [];
                                for (var j = 0; j < message.filters.length; ++j)
                                    object.filters[j] = $root.google.bigtable.v2.RowFilter.toObject(message.filters[j], options);
                            }
                            return object;
                        };
    
                        /**
                         * Converts this Interleave to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.v2.RowFilter.Interleave
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        Interleave.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for Interleave
                         * @function getTypeUrl
                         * @memberof google.bigtable.v2.RowFilter.Interleave
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        Interleave.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.v2.RowFilter.Interleave";
                        };
    
                        return Interleave;
                    })();
    
                    RowFilter.Condition = (function() {
    
                        /**
                         * Properties of a Condition.
                         * @memberof google.bigtable.v2.RowFilter
                         * @interface ICondition
                         * @property {google.bigtable.v2.IRowFilter|null} [predicateFilter] Condition predicateFilter
                         * @property {google.bigtable.v2.IRowFilter|null} [trueFilter] Condition trueFilter
                         * @property {google.bigtable.v2.IRowFilter|null} [falseFilter] Condition falseFilter
                         */
    
                        /**
                         * Constructs a new Condition.
                         * @memberof google.bigtable.v2.RowFilter
                         * @classdesc Represents a Condition.
                         * @implements ICondition
                         * @constructor
                         * @param {google.bigtable.v2.RowFilter.ICondition=} [properties] Properties to set
                         */
                        function Condition(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * Condition predicateFilter.
                         * @member {google.bigtable.v2.IRowFilter|null|undefined} predicateFilter
                         * @memberof google.bigtable.v2.RowFilter.Condition
                         * @instance
                         */
                        Condition.prototype.predicateFilter = null;
    
                        /**
                         * Condition trueFilter.
                         * @member {google.bigtable.v2.IRowFilter|null|undefined} trueFilter
                         * @memberof google.bigtable.v2.RowFilter.Condition
                         * @instance
                         */
                        Condition.prototype.trueFilter = null;
    
                        /**
                         * Condition falseFilter.
                         * @member {google.bigtable.v2.IRowFilter|null|undefined} falseFilter
                         * @memberof google.bigtable.v2.RowFilter.Condition
                         * @instance
                         */
                        Condition.prototype.falseFilter = null;
    
                        /**
                         * Creates a new Condition instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.v2.RowFilter.Condition
                         * @static
                         * @param {google.bigtable.v2.RowFilter.ICondition=} [properties] Properties to set
                         * @returns {google.bigtable.v2.RowFilter.Condition} Condition instance
                         */
                        Condition.create = function create(properties) {
                            return new Condition(properties);
                        };
    
                        /**
                         * Encodes the specified Condition message. Does not implicitly {@link google.bigtable.v2.RowFilter.Condition.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.v2.RowFilter.Condition
                         * @static
                         * @param {google.bigtable.v2.RowFilter.ICondition} message Condition message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        Condition.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.predicateFilter != null && Object.hasOwnProperty.call(message, "predicateFilter"))
                                $root.google.bigtable.v2.RowFilter.encode(message.predicateFilter, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                            if (message.trueFilter != null && Object.hasOwnProperty.call(message, "trueFilter"))
                                $root.google.bigtable.v2.RowFilter.encode(message.trueFilter, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            if (message.falseFilter != null && Object.hasOwnProperty.call(message, "falseFilter"))
                                $root.google.bigtable.v2.RowFilter.encode(message.falseFilter, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified Condition message, length delimited. Does not implicitly {@link google.bigtable.v2.RowFilter.Condition.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.v2.RowFilter.Condition
                         * @static
                         * @param {google.bigtable.v2.RowFilter.ICondition} message Condition message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        Condition.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a Condition message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.v2.RowFilter.Condition
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.v2.RowFilter.Condition} Condition
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        Condition.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.RowFilter.Condition();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.predicateFilter = $root.google.bigtable.v2.RowFilter.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 2: {
                                        message.trueFilter = $root.google.bigtable.v2.RowFilter.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 3: {
                                        message.falseFilter = $root.google.bigtable.v2.RowFilter.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a Condition message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.v2.RowFilter.Condition
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.v2.RowFilter.Condition} Condition
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        Condition.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a Condition message.
                         * @function verify
                         * @memberof google.bigtable.v2.RowFilter.Condition
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        Condition.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.predicateFilter != null && message.hasOwnProperty("predicateFilter")) {
                                var error = $root.google.bigtable.v2.RowFilter.verify(message.predicateFilter);
                                if (error)
                                    return "predicateFilter." + error;
                            }
                            if (message.trueFilter != null && message.hasOwnProperty("trueFilter")) {
                                var error = $root.google.bigtable.v2.RowFilter.verify(message.trueFilter);
                                if (error)
                                    return "trueFilter." + error;
                            }
                            if (message.falseFilter != null && message.hasOwnProperty("falseFilter")) {
                                var error = $root.google.bigtable.v2.RowFilter.verify(message.falseFilter);
                                if (error)
                                    return "falseFilter." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates a Condition message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.v2.RowFilter.Condition
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.v2.RowFilter.Condition} Condition
                         */
                        Condition.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.v2.RowFilter.Condition)
                                return object;
                            var message = new $root.google.bigtable.v2.RowFilter.Condition();
                            if (object.predicateFilter != null) {
                                if (typeof object.predicateFilter !== "object")
                                    throw TypeError(".google.bigtable.v2.RowFilter.Condition.predicateFilter: object expected");
                                message.predicateFilter = $root.google.bigtable.v2.RowFilter.fromObject(object.predicateFilter);
                            }
                            if (object.trueFilter != null) {
                                if (typeof object.trueFilter !== "object")
                                    throw TypeError(".google.bigtable.v2.RowFilter.Condition.trueFilter: object expected");
                                message.trueFilter = $root.google.bigtable.v2.RowFilter.fromObject(object.trueFilter);
                            }
                            if (object.falseFilter != null) {
                                if (typeof object.falseFilter !== "object")
                                    throw TypeError(".google.bigtable.v2.RowFilter.Condition.falseFilter: object expected");
                                message.falseFilter = $root.google.bigtable.v2.RowFilter.fromObject(object.falseFilter);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a Condition message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.v2.RowFilter.Condition
                         * @static
                         * @param {google.bigtable.v2.RowFilter.Condition} message Condition
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        Condition.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.predicateFilter = null;
                                object.trueFilter = null;
                                object.falseFilter = null;
                            }
                            if (message.predicateFilter != null && message.hasOwnProperty("predicateFilter"))
                                object.predicateFilter = $root.google.bigtable.v2.RowFilter.toObject(message.predicateFilter, options);
                            if (message.trueFilter != null && message.hasOwnProperty("trueFilter"))
                                object.trueFilter = $root.google.bigtable.v2.RowFilter.toObject(message.trueFilter, options);
                            if (message.falseFilter != null && message.hasOwnProperty("falseFilter"))
                                object.falseFilter = $root.google.bigtable.v2.RowFilter.toObject(message.falseFilter, options);
                            return object;
                        };
    
                        /**
                         * Converts this Condition to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.v2.RowFilter.Condition
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        Condition.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for Condition
                         * @function getTypeUrl
                         * @memberof google.bigtable.v2.RowFilter.Condition
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        Condition.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.v2.RowFilter.Condition";
                        };
    
                        return Condition;
                    })();
    
                    return RowFilter;
                })();
    
                v2.Mutation = (function() {
    
                    /**
                     * Properties of a Mutation.
                     * @memberof google.bigtable.v2
                     * @interface IMutation
                     * @property {google.bigtable.v2.Mutation.ISetCell|null} [setCell] Mutation setCell
                     * @property {google.bigtable.v2.Mutation.IAddToCell|null} [addToCell] Mutation addToCell
                     * @property {google.bigtable.v2.Mutation.IDeleteFromColumn|null} [deleteFromColumn] Mutation deleteFromColumn
                     * @property {google.bigtable.v2.Mutation.IDeleteFromFamily|null} [deleteFromFamily] Mutation deleteFromFamily
                     * @property {google.bigtable.v2.Mutation.IDeleteFromRow|null} [deleteFromRow] Mutation deleteFromRow
                     */
    
                    /**
                     * Constructs a new Mutation.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a Mutation.
                     * @implements IMutation
                     * @constructor
                     * @param {google.bigtable.v2.IMutation=} [properties] Properties to set
                     */
                    function Mutation(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * Mutation setCell.
                     * @member {google.bigtable.v2.Mutation.ISetCell|null|undefined} setCell
                     * @memberof google.bigtable.v2.Mutation
                     * @instance
                     */
                    Mutation.prototype.setCell = null;
    
                    /**
                     * Mutation addToCell.
                     * @member {google.bigtable.v2.Mutation.IAddToCell|null|undefined} addToCell
                     * @memberof google.bigtable.v2.Mutation
                     * @instance
                     */
                    Mutation.prototype.addToCell = null;
    
                    /**
                     * Mutation deleteFromColumn.
                     * @member {google.bigtable.v2.Mutation.IDeleteFromColumn|null|undefined} deleteFromColumn
                     * @memberof google.bigtable.v2.Mutation
                     * @instance
                     */
                    Mutation.prototype.deleteFromColumn = null;
    
                    /**
                     * Mutation deleteFromFamily.
                     * @member {google.bigtable.v2.Mutation.IDeleteFromFamily|null|undefined} deleteFromFamily
                     * @memberof google.bigtable.v2.Mutation
                     * @instance
                     */
                    Mutation.prototype.deleteFromFamily = null;
    
                    /**
                     * Mutation deleteFromRow.
                     * @member {google.bigtable.v2.Mutation.IDeleteFromRow|null|undefined} deleteFromRow
                     * @memberof google.bigtable.v2.Mutation
                     * @instance
                     */
                    Mutation.prototype.deleteFromRow = null;
    
                    // OneOf field names bound to virtual getters and setters
                    var $oneOfFields;
    
                    /**
                     * Mutation mutation.
                     * @member {"setCell"|"addToCell"|"deleteFromColumn"|"deleteFromFamily"|"deleteFromRow"|undefined} mutation
                     * @memberof google.bigtable.v2.Mutation
                     * @instance
                     */
                    Object.defineProperty(Mutation.prototype, "mutation", {
                        get: $util.oneOfGetter($oneOfFields = ["setCell", "addToCell", "deleteFromColumn", "deleteFromFamily", "deleteFromRow"]),
                        set: $util.oneOfSetter($oneOfFields)
                    });
    
                    /**
                     * Creates a new Mutation instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.Mutation
                     * @static
                     * @param {google.bigtable.v2.IMutation=} [properties] Properties to set
                     * @returns {google.bigtable.v2.Mutation} Mutation instance
                     */
                    Mutation.create = function create(properties) {
                        return new Mutation(properties);
                    };
    
                    /**
                     * Encodes the specified Mutation message. Does not implicitly {@link google.bigtable.v2.Mutation.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.Mutation
                     * @static
                     * @param {google.bigtable.v2.IMutation} message Mutation message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    Mutation.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.setCell != null && Object.hasOwnProperty.call(message, "setCell"))
                            $root.google.bigtable.v2.Mutation.SetCell.encode(message.setCell, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                        if (message.deleteFromColumn != null && Object.hasOwnProperty.call(message, "deleteFromColumn"))
                            $root.google.bigtable.v2.Mutation.DeleteFromColumn.encode(message.deleteFromColumn, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                        if (message.deleteFromFamily != null && Object.hasOwnProperty.call(message, "deleteFromFamily"))
                            $root.google.bigtable.v2.Mutation.DeleteFromFamily.encode(message.deleteFromFamily, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                        if (message.deleteFromRow != null && Object.hasOwnProperty.call(message, "deleteFromRow"))
                            $root.google.bigtable.v2.Mutation.DeleteFromRow.encode(message.deleteFromRow, writer.uint32(/* id 4, wireType 2 =*/34).fork()).ldelim();
                        if (message.addToCell != null && Object.hasOwnProperty.call(message, "addToCell"))
                            $root.google.bigtable.v2.Mutation.AddToCell.encode(message.addToCell, writer.uint32(/* id 5, wireType 2 =*/42).fork()).ldelim();
                        return writer;
                    };
    
                    /**
                     * Encodes the specified Mutation message, length delimited. Does not implicitly {@link google.bigtable.v2.Mutation.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.Mutation
                     * @static
                     * @param {google.bigtable.v2.IMutation} message Mutation message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    Mutation.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a Mutation message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.Mutation
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.Mutation} Mutation
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    Mutation.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.Mutation();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.setCell = $root.google.bigtable.v2.Mutation.SetCell.decode(reader, reader.uint32());
                                    break;
                                }
                            case 5: {
                                    message.addToCell = $root.google.bigtable.v2.Mutation.AddToCell.decode(reader, reader.uint32());
                                    break;
                                }
                            case 2: {
                                    message.deleteFromColumn = $root.google.bigtable.v2.Mutation.DeleteFromColumn.decode(reader, reader.uint32());
                                    break;
                                }
                            case 3: {
                                    message.deleteFromFamily = $root.google.bigtable.v2.Mutation.DeleteFromFamily.decode(reader, reader.uint32());
                                    break;
                                }
                            case 4: {
                                    message.deleteFromRow = $root.google.bigtable.v2.Mutation.DeleteFromRow.decode(reader, reader.uint32());
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a Mutation message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.Mutation
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.Mutation} Mutation
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    Mutation.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a Mutation message.
                     * @function verify
                     * @memberof google.bigtable.v2.Mutation
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    Mutation.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        var properties = {};
                        if (message.setCell != null && message.hasOwnProperty("setCell")) {
                            properties.mutation = 1;
                            {
                                var error = $root.google.bigtable.v2.Mutation.SetCell.verify(message.setCell);
                                if (error)
                                    return "setCell." + error;
                            }
                        }
                        if (message.addToCell != null && message.hasOwnProperty("addToCell")) {
                            if (properties.mutation === 1)
                                return "mutation: multiple values";
                            properties.mutation = 1;
                            {
                                var error = $root.google.bigtable.v2.Mutation.AddToCell.verify(message.addToCell);
                                if (error)
                                    return "addToCell." + error;
                            }
                        }
                        if (message.deleteFromColumn != null && message.hasOwnProperty("deleteFromColumn")) {
                            if (properties.mutation === 1)
                                return "mutation: multiple values";
                            properties.mutation = 1;
                            {
                                var error = $root.google.bigtable.v2.Mutation.DeleteFromColumn.verify(message.deleteFromColumn);
                                if (error)
                                    return "deleteFromColumn." + error;
                            }
                        }
                        if (message.deleteFromFamily != null && message.hasOwnProperty("deleteFromFamily")) {
                            if (properties.mutation === 1)
                                return "mutation: multiple values";
                            properties.mutation = 1;
                            {
                                var error = $root.google.bigtable.v2.Mutation.DeleteFromFamily.verify(message.deleteFromFamily);
                                if (error)
                                    return "deleteFromFamily." + error;
                            }
                        }
                        if (message.deleteFromRow != null && message.hasOwnProperty("deleteFromRow")) {
                            if (properties.mutation === 1)
                                return "mutation: multiple values";
                            properties.mutation = 1;
                            {
                                var error = $root.google.bigtable.v2.Mutation.DeleteFromRow.verify(message.deleteFromRow);
                                if (error)
                                    return "deleteFromRow." + error;
                            }
                        }
                        return null;
                    };
    
                    /**
                     * Creates a Mutation message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.Mutation
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.Mutation} Mutation
                     */
                    Mutation.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.Mutation)
                            return object;
                        var message = new $root.google.bigtable.v2.Mutation();
                        if (object.setCell != null) {
                            if (typeof object.setCell !== "object")
                                throw TypeError(".google.bigtable.v2.Mutation.setCell: object expected");
                            message.setCell = $root.google.bigtable.v2.Mutation.SetCell.fromObject(object.setCell);
                        }
                        if (object.addToCell != null) {
                            if (typeof object.addToCell !== "object")
                                throw TypeError(".google.bigtable.v2.Mutation.addToCell: object expected");
                            message.addToCell = $root.google.bigtable.v2.Mutation.AddToCell.fromObject(object.addToCell);
                        }
                        if (object.deleteFromColumn != null) {
                            if (typeof object.deleteFromColumn !== "object")
                                throw TypeError(".google.bigtable.v2.Mutation.deleteFromColumn: object expected");
                            message.deleteFromColumn = $root.google.bigtable.v2.Mutation.DeleteFromColumn.fromObject(object.deleteFromColumn);
                        }
                        if (object.deleteFromFamily != null) {
                            if (typeof object.deleteFromFamily !== "object")
                                throw TypeError(".google.bigtable.v2.Mutation.deleteFromFamily: object expected");
                            message.deleteFromFamily = $root.google.bigtable.v2.Mutation.DeleteFromFamily.fromObject(object.deleteFromFamily);
                        }
                        if (object.deleteFromRow != null) {
                            if (typeof object.deleteFromRow !== "object")
                                throw TypeError(".google.bigtable.v2.Mutation.deleteFromRow: object expected");
                            message.deleteFromRow = $root.google.bigtable.v2.Mutation.DeleteFromRow.fromObject(object.deleteFromRow);
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a Mutation message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.Mutation
                     * @static
                     * @param {google.bigtable.v2.Mutation} message Mutation
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    Mutation.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (message.setCell != null && message.hasOwnProperty("setCell")) {
                            object.setCell = $root.google.bigtable.v2.Mutation.SetCell.toObject(message.setCell, options);
                            if (options.oneofs)
                                object.mutation = "setCell";
                        }
                        if (message.deleteFromColumn != null && message.hasOwnProperty("deleteFromColumn")) {
                            object.deleteFromColumn = $root.google.bigtable.v2.Mutation.DeleteFromColumn.toObject(message.deleteFromColumn, options);
                            if (options.oneofs)
                                object.mutation = "deleteFromColumn";
                        }
                        if (message.deleteFromFamily != null && message.hasOwnProperty("deleteFromFamily")) {
                            object.deleteFromFamily = $root.google.bigtable.v2.Mutation.DeleteFromFamily.toObject(message.deleteFromFamily, options);
                            if (options.oneofs)
                                object.mutation = "deleteFromFamily";
                        }
                        if (message.deleteFromRow != null && message.hasOwnProperty("deleteFromRow")) {
                            object.deleteFromRow = $root.google.bigtable.v2.Mutation.DeleteFromRow.toObject(message.deleteFromRow, options);
                            if (options.oneofs)
                                object.mutation = "deleteFromRow";
                        }
                        if (message.addToCell != null && message.hasOwnProperty("addToCell")) {
                            object.addToCell = $root.google.bigtable.v2.Mutation.AddToCell.toObject(message.addToCell, options);
                            if (options.oneofs)
                                object.mutation = "addToCell";
                        }
                        return object;
                    };
    
                    /**
                     * Converts this Mutation to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.Mutation
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    Mutation.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for Mutation
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.Mutation
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    Mutation.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.Mutation";
                    };
    
                    Mutation.SetCell = (function() {
    
                        /**
                         * Properties of a SetCell.
                         * @memberof google.bigtable.v2.Mutation
                         * @interface ISetCell
                         * @property {string|null} [familyName] SetCell familyName
                         * @property {Uint8Array|null} [columnQualifier] SetCell columnQualifier
                         * @property {number|Long|null} [timestampMicros] SetCell timestampMicros
                         * @property {Uint8Array|null} [value] SetCell value
                         */
    
                        /**
                         * Constructs a new SetCell.
                         * @memberof google.bigtable.v2.Mutation
                         * @classdesc Represents a SetCell.
                         * @implements ISetCell
                         * @constructor
                         * @param {google.bigtable.v2.Mutation.ISetCell=} [properties] Properties to set
                         */
                        function SetCell(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * SetCell familyName.
                         * @member {string} familyName
                         * @memberof google.bigtable.v2.Mutation.SetCell
                         * @instance
                         */
                        SetCell.prototype.familyName = "";
    
                        /**
                         * SetCell columnQualifier.
                         * @member {Uint8Array} columnQualifier
                         * @memberof google.bigtable.v2.Mutation.SetCell
                         * @instance
                         */
                        SetCell.prototype.columnQualifier = $util.newBuffer([]);
    
                        /**
                         * SetCell timestampMicros.
                         * @member {number|Long} timestampMicros
                         * @memberof google.bigtable.v2.Mutation.SetCell
                         * @instance
                         */
                        SetCell.prototype.timestampMicros = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                        /**
                         * SetCell value.
                         * @member {Uint8Array} value
                         * @memberof google.bigtable.v2.Mutation.SetCell
                         * @instance
                         */
                        SetCell.prototype.value = $util.newBuffer([]);
    
                        /**
                         * Creates a new SetCell instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.v2.Mutation.SetCell
                         * @static
                         * @param {google.bigtable.v2.Mutation.ISetCell=} [properties] Properties to set
                         * @returns {google.bigtable.v2.Mutation.SetCell} SetCell instance
                         */
                        SetCell.create = function create(properties) {
                            return new SetCell(properties);
                        };
    
                        /**
                         * Encodes the specified SetCell message. Does not implicitly {@link google.bigtable.v2.Mutation.SetCell.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.v2.Mutation.SetCell
                         * @static
                         * @param {google.bigtable.v2.Mutation.ISetCell} message SetCell message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        SetCell.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.familyName != null && Object.hasOwnProperty.call(message, "familyName"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.familyName);
                            if (message.columnQualifier != null && Object.hasOwnProperty.call(message, "columnQualifier"))
                                writer.uint32(/* id 2, wireType 2 =*/18).bytes(message.columnQualifier);
                            if (message.timestampMicros != null && Object.hasOwnProperty.call(message, "timestampMicros"))
                                writer.uint32(/* id 3, wireType 0 =*/24).int64(message.timestampMicros);
                            if (message.value != null && Object.hasOwnProperty.call(message, "value"))
                                writer.uint32(/* id 4, wireType 2 =*/34).bytes(message.value);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified SetCell message, length delimited. Does not implicitly {@link google.bigtable.v2.Mutation.SetCell.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.v2.Mutation.SetCell
                         * @static
                         * @param {google.bigtable.v2.Mutation.ISetCell} message SetCell message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        SetCell.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a SetCell message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.v2.Mutation.SetCell
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.v2.Mutation.SetCell} SetCell
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        SetCell.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.Mutation.SetCell();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.familyName = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.columnQualifier = reader.bytes();
                                        break;
                                    }
                                case 3: {
                                        message.timestampMicros = reader.int64();
                                        break;
                                    }
                                case 4: {
                                        message.value = reader.bytes();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a SetCell message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.v2.Mutation.SetCell
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.v2.Mutation.SetCell} SetCell
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        SetCell.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a SetCell message.
                         * @function verify
                         * @memberof google.bigtable.v2.Mutation.SetCell
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        SetCell.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.familyName != null && message.hasOwnProperty("familyName"))
                                if (!$util.isString(message.familyName))
                                    return "familyName: string expected";
                            if (message.columnQualifier != null && message.hasOwnProperty("columnQualifier"))
                                if (!(message.columnQualifier && typeof message.columnQualifier.length === "number" || $util.isString(message.columnQualifier)))
                                    return "columnQualifier: buffer expected";
                            if (message.timestampMicros != null && message.hasOwnProperty("timestampMicros"))
                                if (!$util.isInteger(message.timestampMicros) && !(message.timestampMicros && $util.isInteger(message.timestampMicros.low) && $util.isInteger(message.timestampMicros.high)))
                                    return "timestampMicros: integer|Long expected";
                            if (message.value != null && message.hasOwnProperty("value"))
                                if (!(message.value && typeof message.value.length === "number" || $util.isString(message.value)))
                                    return "value: buffer expected";
                            return null;
                        };
    
                        /**
                         * Creates a SetCell message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.v2.Mutation.SetCell
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.v2.Mutation.SetCell} SetCell
                         */
                        SetCell.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.v2.Mutation.SetCell)
                                return object;
                            var message = new $root.google.bigtable.v2.Mutation.SetCell();
                            if (object.familyName != null)
                                message.familyName = String(object.familyName);
                            if (object.columnQualifier != null)
                                if (typeof object.columnQualifier === "string")
                                    $util.base64.decode(object.columnQualifier, message.columnQualifier = $util.newBuffer($util.base64.length(object.columnQualifier)), 0);
                                else if (object.columnQualifier.length >= 0)
                                    message.columnQualifier = object.columnQualifier;
                            if (object.timestampMicros != null)
                                if ($util.Long)
                                    (message.timestampMicros = $util.Long.fromValue(object.timestampMicros)).unsigned = false;
                                else if (typeof object.timestampMicros === "string")
                                    message.timestampMicros = parseInt(object.timestampMicros, 10);
                                else if (typeof object.timestampMicros === "number")
                                    message.timestampMicros = object.timestampMicros;
                                else if (typeof object.timestampMicros === "object")
                                    message.timestampMicros = new $util.LongBits(object.timestampMicros.low >>> 0, object.timestampMicros.high >>> 0).toNumber();
                            if (object.value != null)
                                if (typeof object.value === "string")
                                    $util.base64.decode(object.value, message.value = $util.newBuffer($util.base64.length(object.value)), 0);
                                else if (object.value.length >= 0)
                                    message.value = object.value;
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a SetCell message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.v2.Mutation.SetCell
                         * @static
                         * @param {google.bigtable.v2.Mutation.SetCell} message SetCell
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        SetCell.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.familyName = "";
                                if (options.bytes === String)
                                    object.columnQualifier = "";
                                else {
                                    object.columnQualifier = [];
                                    if (options.bytes !== Array)
                                        object.columnQualifier = $util.newBuffer(object.columnQualifier);
                                }
                                if ($util.Long) {
                                    var long = new $util.Long(0, 0, false);
                                    object.timestampMicros = options.longs === String ? long.toString() : options.longs === Number ? long.toNumber() : long;
                                } else
                                    object.timestampMicros = options.longs === String ? "0" : 0;
                                if (options.bytes === String)
                                    object.value = "";
                                else {
                                    object.value = [];
                                    if (options.bytes !== Array)
                                        object.value = $util.newBuffer(object.value);
                                }
                            }
                            if (message.familyName != null && message.hasOwnProperty("familyName"))
                                object.familyName = message.familyName;
                            if (message.columnQualifier != null && message.hasOwnProperty("columnQualifier"))
                                object.columnQualifier = options.bytes === String ? $util.base64.encode(message.columnQualifier, 0, message.columnQualifier.length) : options.bytes === Array ? Array.prototype.slice.call(message.columnQualifier) : message.columnQualifier;
                            if (message.timestampMicros != null && message.hasOwnProperty("timestampMicros"))
                                if (typeof message.timestampMicros === "number")
                                    object.timestampMicros = options.longs === String ? String(message.timestampMicros) : message.timestampMicros;
                                else
                                    object.timestampMicros = options.longs === String ? $util.Long.prototype.toString.call(message.timestampMicros) : options.longs === Number ? new $util.LongBits(message.timestampMicros.low >>> 0, message.timestampMicros.high >>> 0).toNumber() : message.timestampMicros;
                            if (message.value != null && message.hasOwnProperty("value"))
                                object.value = options.bytes === String ? $util.base64.encode(message.value, 0, message.value.length) : options.bytes === Array ? Array.prototype.slice.call(message.value) : message.value;
                            return object;
                        };
    
                        /**
                         * Converts this SetCell to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.v2.Mutation.SetCell
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        SetCell.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for SetCell
                         * @function getTypeUrl
                         * @memberof google.bigtable.v2.Mutation.SetCell
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        SetCell.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.v2.Mutation.SetCell";
                        };
    
                        return SetCell;
                    })();
    
                    Mutation.AddToCell = (function() {
    
                        /**
                         * Properties of an AddToCell.
                         * @memberof google.bigtable.v2.Mutation
                         * @interface IAddToCell
                         * @property {string|null} [familyName] AddToCell familyName
                         * @property {google.bigtable.v2.IValue|null} [columnQualifier] AddToCell columnQualifier
                         * @property {google.bigtable.v2.IValue|null} [timestamp] AddToCell timestamp
                         * @property {google.bigtable.v2.IValue|null} [input] AddToCell input
                         */
    
                        /**
                         * Constructs a new AddToCell.
                         * @memberof google.bigtable.v2.Mutation
                         * @classdesc Represents an AddToCell.
                         * @implements IAddToCell
                         * @constructor
                         * @param {google.bigtable.v2.Mutation.IAddToCell=} [properties] Properties to set
                         */
                        function AddToCell(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * AddToCell familyName.
                         * @member {string} familyName
                         * @memberof google.bigtable.v2.Mutation.AddToCell
                         * @instance
                         */
                        AddToCell.prototype.familyName = "";
    
                        /**
                         * AddToCell columnQualifier.
                         * @member {google.bigtable.v2.IValue|null|undefined} columnQualifier
                         * @memberof google.bigtable.v2.Mutation.AddToCell
                         * @instance
                         */
                        AddToCell.prototype.columnQualifier = null;
    
                        /**
                         * AddToCell timestamp.
                         * @member {google.bigtable.v2.IValue|null|undefined} timestamp
                         * @memberof google.bigtable.v2.Mutation.AddToCell
                         * @instance
                         */
                        AddToCell.prototype.timestamp = null;
    
                        /**
                         * AddToCell input.
                         * @member {google.bigtable.v2.IValue|null|undefined} input
                         * @memberof google.bigtable.v2.Mutation.AddToCell
                         * @instance
                         */
                        AddToCell.prototype.input = null;
    
                        /**
                         * Creates a new AddToCell instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.v2.Mutation.AddToCell
                         * @static
                         * @param {google.bigtable.v2.Mutation.IAddToCell=} [properties] Properties to set
                         * @returns {google.bigtable.v2.Mutation.AddToCell} AddToCell instance
                         */
                        AddToCell.create = function create(properties) {
                            return new AddToCell(properties);
                        };
    
                        /**
                         * Encodes the specified AddToCell message. Does not implicitly {@link google.bigtable.v2.Mutation.AddToCell.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.v2.Mutation.AddToCell
                         * @static
                         * @param {google.bigtable.v2.Mutation.IAddToCell} message AddToCell message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        AddToCell.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.familyName != null && Object.hasOwnProperty.call(message, "familyName"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.familyName);
                            if (message.columnQualifier != null && Object.hasOwnProperty.call(message, "columnQualifier"))
                                $root.google.bigtable.v2.Value.encode(message.columnQualifier, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                            if (message.timestamp != null && Object.hasOwnProperty.call(message, "timestamp"))
                                $root.google.bigtable.v2.Value.encode(message.timestamp, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            if (message.input != null && Object.hasOwnProperty.call(message, "input"))
                                $root.google.bigtable.v2.Value.encode(message.input, writer.uint32(/* id 4, wireType 2 =*/34).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified AddToCell message, length delimited. Does not implicitly {@link google.bigtable.v2.Mutation.AddToCell.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.v2.Mutation.AddToCell
                         * @static
                         * @param {google.bigtable.v2.Mutation.IAddToCell} message AddToCell message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        AddToCell.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes an AddToCell message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.v2.Mutation.AddToCell
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.v2.Mutation.AddToCell} AddToCell
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        AddToCell.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.Mutation.AddToCell();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.familyName = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.columnQualifier = $root.google.bigtable.v2.Value.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 3: {
                                        message.timestamp = $root.google.bigtable.v2.Value.decode(reader, reader.uint32());
                                        break;
                                    }
                                case 4: {
                                        message.input = $root.google.bigtable.v2.Value.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes an AddToCell message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.v2.Mutation.AddToCell
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.v2.Mutation.AddToCell} AddToCell
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        AddToCell.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies an AddToCell message.
                         * @function verify
                         * @memberof google.bigtable.v2.Mutation.AddToCell
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        AddToCell.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.familyName != null && message.hasOwnProperty("familyName"))
                                if (!$util.isString(message.familyName))
                                    return "familyName: string expected";
                            if (message.columnQualifier != null && message.hasOwnProperty("columnQualifier")) {
                                var error = $root.google.bigtable.v2.Value.verify(message.columnQualifier);
                                if (error)
                                    return "columnQualifier." + error;
                            }
                            if (message.timestamp != null && message.hasOwnProperty("timestamp")) {
                                var error = $root.google.bigtable.v2.Value.verify(message.timestamp);
                                if (error)
                                    return "timestamp." + error;
                            }
                            if (message.input != null && message.hasOwnProperty("input")) {
                                var error = $root.google.bigtable.v2.Value.verify(message.input);
                                if (error)
                                    return "input." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates an AddToCell message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.v2.Mutation.AddToCell
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.v2.Mutation.AddToCell} AddToCell
                         */
                        AddToCell.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.v2.Mutation.AddToCell)
                                return object;
                            var message = new $root.google.bigtable.v2.Mutation.AddToCell();
                            if (object.familyName != null)
                                message.familyName = String(object.familyName);
                            if (object.columnQualifier != null) {
                                if (typeof object.columnQualifier !== "object")
                                    throw TypeError(".google.bigtable.v2.Mutation.AddToCell.columnQualifier: object expected");
                                message.columnQualifier = $root.google.bigtable.v2.Value.fromObject(object.columnQualifier);
                            }
                            if (object.timestamp != null) {
                                if (typeof object.timestamp !== "object")
                                    throw TypeError(".google.bigtable.v2.Mutation.AddToCell.timestamp: object expected");
                                message.timestamp = $root.google.bigtable.v2.Value.fromObject(object.timestamp);
                            }
                            if (object.input != null) {
                                if (typeof object.input !== "object")
                                    throw TypeError(".google.bigtable.v2.Mutation.AddToCell.input: object expected");
                                message.input = $root.google.bigtable.v2.Value.fromObject(object.input);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from an AddToCell message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.v2.Mutation.AddToCell
                         * @static
                         * @param {google.bigtable.v2.Mutation.AddToCell} message AddToCell
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        AddToCell.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.familyName = "";
                                object.columnQualifier = null;
                                object.timestamp = null;
                                object.input = null;
                            }
                            if (message.familyName != null && message.hasOwnProperty("familyName"))
                                object.familyName = message.familyName;
                            if (message.columnQualifier != null && message.hasOwnProperty("columnQualifier"))
                                object.columnQualifier = $root.google.bigtable.v2.Value.toObject(message.columnQualifier, options);
                            if (message.timestamp != null && message.hasOwnProperty("timestamp"))
                                object.timestamp = $root.google.bigtable.v2.Value.toObject(message.timestamp, options);
                            if (message.input != null && message.hasOwnProperty("input"))
                                object.input = $root.google.bigtable.v2.Value.toObject(message.input, options);
                            return object;
                        };
    
                        /**
                         * Converts this AddToCell to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.v2.Mutation.AddToCell
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        AddToCell.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for AddToCell
                         * @function getTypeUrl
                         * @memberof google.bigtable.v2.Mutation.AddToCell
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        AddToCell.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.v2.Mutation.AddToCell";
                        };
    
                        return AddToCell;
                    })();
    
                    Mutation.DeleteFromColumn = (function() {
    
                        /**
                         * Properties of a DeleteFromColumn.
                         * @memberof google.bigtable.v2.Mutation
                         * @interface IDeleteFromColumn
                         * @property {string|null} [familyName] DeleteFromColumn familyName
                         * @property {Uint8Array|null} [columnQualifier] DeleteFromColumn columnQualifier
                         * @property {google.bigtable.v2.ITimestampRange|null} [timeRange] DeleteFromColumn timeRange
                         */
    
                        /**
                         * Constructs a new DeleteFromColumn.
                         * @memberof google.bigtable.v2.Mutation
                         * @classdesc Represents a DeleteFromColumn.
                         * @implements IDeleteFromColumn
                         * @constructor
                         * @param {google.bigtable.v2.Mutation.IDeleteFromColumn=} [properties] Properties to set
                         */
                        function DeleteFromColumn(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * DeleteFromColumn familyName.
                         * @member {string} familyName
                         * @memberof google.bigtable.v2.Mutation.DeleteFromColumn
                         * @instance
                         */
                        DeleteFromColumn.prototype.familyName = "";
    
                        /**
                         * DeleteFromColumn columnQualifier.
                         * @member {Uint8Array} columnQualifier
                         * @memberof google.bigtable.v2.Mutation.DeleteFromColumn
                         * @instance
                         */
                        DeleteFromColumn.prototype.columnQualifier = $util.newBuffer([]);
    
                        /**
                         * DeleteFromColumn timeRange.
                         * @member {google.bigtable.v2.ITimestampRange|null|undefined} timeRange
                         * @memberof google.bigtable.v2.Mutation.DeleteFromColumn
                         * @instance
                         */
                        DeleteFromColumn.prototype.timeRange = null;
    
                        /**
                         * Creates a new DeleteFromColumn instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.v2.Mutation.DeleteFromColumn
                         * @static
                         * @param {google.bigtable.v2.Mutation.IDeleteFromColumn=} [properties] Properties to set
                         * @returns {google.bigtable.v2.Mutation.DeleteFromColumn} DeleteFromColumn instance
                         */
                        DeleteFromColumn.create = function create(properties) {
                            return new DeleteFromColumn(properties);
                        };
    
                        /**
                         * Encodes the specified DeleteFromColumn message. Does not implicitly {@link google.bigtable.v2.Mutation.DeleteFromColumn.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.v2.Mutation.DeleteFromColumn
                         * @static
                         * @param {google.bigtable.v2.Mutation.IDeleteFromColumn} message DeleteFromColumn message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        DeleteFromColumn.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.familyName != null && Object.hasOwnProperty.call(message, "familyName"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.familyName);
                            if (message.columnQualifier != null && Object.hasOwnProperty.call(message, "columnQualifier"))
                                writer.uint32(/* id 2, wireType 2 =*/18).bytes(message.columnQualifier);
                            if (message.timeRange != null && Object.hasOwnProperty.call(message, "timeRange"))
                                $root.google.bigtable.v2.TimestampRange.encode(message.timeRange, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified DeleteFromColumn message, length delimited. Does not implicitly {@link google.bigtable.v2.Mutation.DeleteFromColumn.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.v2.Mutation.DeleteFromColumn
                         * @static
                         * @param {google.bigtable.v2.Mutation.IDeleteFromColumn} message DeleteFromColumn message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        DeleteFromColumn.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a DeleteFromColumn message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.v2.Mutation.DeleteFromColumn
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.v2.Mutation.DeleteFromColumn} DeleteFromColumn
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        DeleteFromColumn.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.Mutation.DeleteFromColumn();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.familyName = reader.string();
                                        break;
                                    }
                                case 2: {
                                        message.columnQualifier = reader.bytes();
                                        break;
                                    }
                                case 3: {
                                        message.timeRange = $root.google.bigtable.v2.TimestampRange.decode(reader, reader.uint32());
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a DeleteFromColumn message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.v2.Mutation.DeleteFromColumn
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.v2.Mutation.DeleteFromColumn} DeleteFromColumn
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        DeleteFromColumn.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a DeleteFromColumn message.
                         * @function verify
                         * @memberof google.bigtable.v2.Mutation.DeleteFromColumn
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        DeleteFromColumn.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.familyName != null && message.hasOwnProperty("familyName"))
                                if (!$util.isString(message.familyName))
                                    return "familyName: string expected";
                            if (message.columnQualifier != null && message.hasOwnProperty("columnQualifier"))
                                if (!(message.columnQualifier && typeof message.columnQualifier.length === "number" || $util.isString(message.columnQualifier)))
                                    return "columnQualifier: buffer expected";
                            if (message.timeRange != null && message.hasOwnProperty("timeRange")) {
                                var error = $root.google.bigtable.v2.TimestampRange.verify(message.timeRange);
                                if (error)
                                    return "timeRange." + error;
                            }
                            return null;
                        };
    
                        /**
                         * Creates a DeleteFromColumn message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.v2.Mutation.DeleteFromColumn
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.v2.Mutation.DeleteFromColumn} DeleteFromColumn
                         */
                        DeleteFromColumn.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.v2.Mutation.DeleteFromColumn)
                                return object;
                            var message = new $root.google.bigtable.v2.Mutation.DeleteFromColumn();
                            if (object.familyName != null)
                                message.familyName = String(object.familyName);
                            if (object.columnQualifier != null)
                                if (typeof object.columnQualifier === "string")
                                    $util.base64.decode(object.columnQualifier, message.columnQualifier = $util.newBuffer($util.base64.length(object.columnQualifier)), 0);
                                else if (object.columnQualifier.length >= 0)
                                    message.columnQualifier = object.columnQualifier;
                            if (object.timeRange != null) {
                                if (typeof object.timeRange !== "object")
                                    throw TypeError(".google.bigtable.v2.Mutation.DeleteFromColumn.timeRange: object expected");
                                message.timeRange = $root.google.bigtable.v2.TimestampRange.fromObject(object.timeRange);
                            }
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a DeleteFromColumn message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.v2.Mutation.DeleteFromColumn
                         * @static
                         * @param {google.bigtable.v2.Mutation.DeleteFromColumn} message DeleteFromColumn
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        DeleteFromColumn.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults) {
                                object.familyName = "";
                                if (options.bytes === String)
                                    object.columnQualifier = "";
                                else {
                                    object.columnQualifier = [];
                                    if (options.bytes !== Array)
                                        object.columnQualifier = $util.newBuffer(object.columnQualifier);
                                }
                                object.timeRange = null;
                            }
                            if (message.familyName != null && message.hasOwnProperty("familyName"))
                                object.familyName = message.familyName;
                            if (message.columnQualifier != null && message.hasOwnProperty("columnQualifier"))
                                object.columnQualifier = options.bytes === String ? $util.base64.encode(message.columnQualifier, 0, message.columnQualifier.length) : options.bytes === Array ? Array.prototype.slice.call(message.columnQualifier) : message.columnQualifier;
                            if (message.timeRange != null && message.hasOwnProperty("timeRange"))
                                object.timeRange = $root.google.bigtable.v2.TimestampRange.toObject(message.timeRange, options);
                            return object;
                        };
    
                        /**
                         * Converts this DeleteFromColumn to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.v2.Mutation.DeleteFromColumn
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        DeleteFromColumn.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for DeleteFromColumn
                         * @function getTypeUrl
                         * @memberof google.bigtable.v2.Mutation.DeleteFromColumn
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        DeleteFromColumn.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.v2.Mutation.DeleteFromColumn";
                        };
    
                        return DeleteFromColumn;
                    })();
    
                    Mutation.DeleteFromFamily = (function() {
    
                        /**
                         * Properties of a DeleteFromFamily.
                         * @memberof google.bigtable.v2.Mutation
                         * @interface IDeleteFromFamily
                         * @property {string|null} [familyName] DeleteFromFamily familyName
                         */
    
                        /**
                         * Constructs a new DeleteFromFamily.
                         * @memberof google.bigtable.v2.Mutation
                         * @classdesc Represents a DeleteFromFamily.
                         * @implements IDeleteFromFamily
                         * @constructor
                         * @param {google.bigtable.v2.Mutation.IDeleteFromFamily=} [properties] Properties to set
                         */
                        function DeleteFromFamily(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * DeleteFromFamily familyName.
                         * @member {string} familyName
                         * @memberof google.bigtable.v2.Mutation.DeleteFromFamily
                         * @instance
                         */
                        DeleteFromFamily.prototype.familyName = "";
    
                        /**
                         * Creates a new DeleteFromFamily instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.v2.Mutation.DeleteFromFamily
                         * @static
                         * @param {google.bigtable.v2.Mutation.IDeleteFromFamily=} [properties] Properties to set
                         * @returns {google.bigtable.v2.Mutation.DeleteFromFamily} DeleteFromFamily instance
                         */
                        DeleteFromFamily.create = function create(properties) {
                            return new DeleteFromFamily(properties);
                        };
    
                        /**
                         * Encodes the specified DeleteFromFamily message. Does not implicitly {@link google.bigtable.v2.Mutation.DeleteFromFamily.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.v2.Mutation.DeleteFromFamily
                         * @static
                         * @param {google.bigtable.v2.Mutation.IDeleteFromFamily} message DeleteFromFamily message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        DeleteFromFamily.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            if (message.familyName != null && Object.hasOwnProperty.call(message, "familyName"))
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.familyName);
                            return writer;
                        };
    
                        /**
                         * Encodes the specified DeleteFromFamily message, length delimited. Does not implicitly {@link google.bigtable.v2.Mutation.DeleteFromFamily.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.v2.Mutation.DeleteFromFamily
                         * @static
                         * @param {google.bigtable.v2.Mutation.IDeleteFromFamily} message DeleteFromFamily message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        DeleteFromFamily.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a DeleteFromFamily message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.v2.Mutation.DeleteFromFamily
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.v2.Mutation.DeleteFromFamily} DeleteFromFamily
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        DeleteFromFamily.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.Mutation.DeleteFromFamily();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                case 1: {
                                        message.familyName = reader.string();
                                        break;
                                    }
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a DeleteFromFamily message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.v2.Mutation.DeleteFromFamily
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.v2.Mutation.DeleteFromFamily} DeleteFromFamily
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        DeleteFromFamily.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a DeleteFromFamily message.
                         * @function verify
                         * @memberof google.bigtable.v2.Mutation.DeleteFromFamily
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        DeleteFromFamily.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            if (message.familyName != null && message.hasOwnProperty("familyName"))
                                if (!$util.isString(message.familyName))
                                    return "familyName: string expected";
                            return null;
                        };
    
                        /**
                         * Creates a DeleteFromFamily message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.v2.Mutation.DeleteFromFamily
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.v2.Mutation.DeleteFromFamily} DeleteFromFamily
                         */
                        DeleteFromFamily.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.v2.Mutation.DeleteFromFamily)
                                return object;
                            var message = new $root.google.bigtable.v2.Mutation.DeleteFromFamily();
                            if (object.familyName != null)
                                message.familyName = String(object.familyName);
                            return message;
                        };
    
                        /**
                         * Creates a plain object from a DeleteFromFamily message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.v2.Mutation.DeleteFromFamily
                         * @static
                         * @param {google.bigtable.v2.Mutation.DeleteFromFamily} message DeleteFromFamily
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        DeleteFromFamily.toObject = function toObject(message, options) {
                            if (!options)
                                options = {};
                            var object = {};
                            if (options.defaults)
                                object.familyName = "";
                            if (message.familyName != null && message.hasOwnProperty("familyName"))
                                object.familyName = message.familyName;
                            return object;
                        };
    
                        /**
                         * Converts this DeleteFromFamily to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.v2.Mutation.DeleteFromFamily
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        DeleteFromFamily.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for DeleteFromFamily
                         * @function getTypeUrl
                         * @memberof google.bigtable.v2.Mutation.DeleteFromFamily
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        DeleteFromFamily.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.v2.Mutation.DeleteFromFamily";
                        };
    
                        return DeleteFromFamily;
                    })();
    
                    Mutation.DeleteFromRow = (function() {
    
                        /**
                         * Properties of a DeleteFromRow.
                         * @memberof google.bigtable.v2.Mutation
                         * @interface IDeleteFromRow
                         */
    
                        /**
                         * Constructs a new DeleteFromRow.
                         * @memberof google.bigtable.v2.Mutation
                         * @classdesc Represents a DeleteFromRow.
                         * @implements IDeleteFromRow
                         * @constructor
                         * @param {google.bigtable.v2.Mutation.IDeleteFromRow=} [properties] Properties to set
                         */
                        function DeleteFromRow(properties) {
                            if (properties)
                                for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                    if (properties[keys[i]] != null)
                                        this[keys[i]] = properties[keys[i]];
                        }
    
                        /**
                         * Creates a new DeleteFromRow instance using the specified properties.
                         * @function create
                         * @memberof google.bigtable.v2.Mutation.DeleteFromRow
                         * @static
                         * @param {google.bigtable.v2.Mutation.IDeleteFromRow=} [properties] Properties to set
                         * @returns {google.bigtable.v2.Mutation.DeleteFromRow} DeleteFromRow instance
                         */
                        DeleteFromRow.create = function create(properties) {
                            return new DeleteFromRow(properties);
                        };
    
                        /**
                         * Encodes the specified DeleteFromRow message. Does not implicitly {@link google.bigtable.v2.Mutation.DeleteFromRow.verify|verify} messages.
                         * @function encode
                         * @memberof google.bigtable.v2.Mutation.DeleteFromRow
                         * @static
                         * @param {google.bigtable.v2.Mutation.IDeleteFromRow} message DeleteFromRow message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        DeleteFromRow.encode = function encode(message, writer) {
                            if (!writer)
                                writer = $Writer.create();
                            return writer;
                        };
    
                        /**
                         * Encodes the specified DeleteFromRow message, length delimited. Does not implicitly {@link google.bigtable.v2.Mutation.DeleteFromRow.verify|verify} messages.
                         * @function encodeDelimited
                         * @memberof google.bigtable.v2.Mutation.DeleteFromRow
                         * @static
                         * @param {google.bigtable.v2.Mutation.IDeleteFromRow} message DeleteFromRow message or plain object to encode
                         * @param {$protobuf.Writer} [writer] Writer to encode to
                         * @returns {$protobuf.Writer} Writer
                         */
                        DeleteFromRow.encodeDelimited = function encodeDelimited(message, writer) {
                            return this.encode(message, writer).ldelim();
                        };
    
                        /**
                         * Decodes a DeleteFromRow message from the specified reader or buffer.
                         * @function decode
                         * @memberof google.bigtable.v2.Mutation.DeleteFromRow
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @param {number} [length] Message length if known beforehand
                         * @returns {google.bigtable.v2.Mutation.DeleteFromRow} DeleteFromRow
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        DeleteFromRow.decode = function decode(reader, length) {
                            if (!(reader instanceof $Reader))
                                reader = $Reader.create(reader);
                            var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.Mutation.DeleteFromRow();
                            while (reader.pos < end) {
                                var tag = reader.uint32();
                                switch (tag >>> 3) {
                                default:
                                    reader.skipType(tag & 7);
                                    break;
                                }
                            }
                            return message;
                        };
    
                        /**
                         * Decodes a DeleteFromRow message from the specified reader or buffer, length delimited.
                         * @function decodeDelimited
                         * @memberof google.bigtable.v2.Mutation.DeleteFromRow
                         * @static
                         * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                         * @returns {google.bigtable.v2.Mutation.DeleteFromRow} DeleteFromRow
                         * @throws {Error} If the payload is not a reader or valid buffer
                         * @throws {$protobuf.util.ProtocolError} If required fields are missing
                         */
                        DeleteFromRow.decodeDelimited = function decodeDelimited(reader) {
                            if (!(reader instanceof $Reader))
                                reader = new $Reader(reader);
                            return this.decode(reader, reader.uint32());
                        };
    
                        /**
                         * Verifies a DeleteFromRow message.
                         * @function verify
                         * @memberof google.bigtable.v2.Mutation.DeleteFromRow
                         * @static
                         * @param {Object.<string,*>} message Plain object to verify
                         * @returns {string|null} `null` if valid, otherwise the reason why it is not
                         */
                        DeleteFromRow.verify = function verify(message) {
                            if (typeof message !== "object" || message === null)
                                return "object expected";
                            return null;
                        };
    
                        /**
                         * Creates a DeleteFromRow message from a plain object. Also converts values to their respective internal types.
                         * @function fromObject
                         * @memberof google.bigtable.v2.Mutation.DeleteFromRow
                         * @static
                         * @param {Object.<string,*>} object Plain object
                         * @returns {google.bigtable.v2.Mutation.DeleteFromRow} DeleteFromRow
                         */
                        DeleteFromRow.fromObject = function fromObject(object) {
                            if (object instanceof $root.google.bigtable.v2.Mutation.DeleteFromRow)
                                return object;
                            return new $root.google.bigtable.v2.Mutation.DeleteFromRow();
                        };
    
                        /**
                         * Creates a plain object from a DeleteFromRow message. Also converts values to other types if specified.
                         * @function toObject
                         * @memberof google.bigtable.v2.Mutation.DeleteFromRow
                         * @static
                         * @param {google.bigtable.v2.Mutation.DeleteFromRow} message DeleteFromRow
                         * @param {$protobuf.IConversionOptions} [options] Conversion options
                         * @returns {Object.<string,*>} Plain object
                         */
                        DeleteFromRow.toObject = function toObject() {
                            return {};
                        };
    
                        /**
                         * Converts this DeleteFromRow to JSON.
                         * @function toJSON
                         * @memberof google.bigtable.v2.Mutation.DeleteFromRow
                         * @instance
                         * @returns {Object.<string,*>} JSON object
                         */
                        DeleteFromRow.prototype.toJSON = function toJSON() {
                            return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                        };
    
                        /**
                         * Gets the default type url for DeleteFromRow
                         * @function getTypeUrl
                         * @memberof google.bigtable.v2.Mutation.DeleteFromRow
                         * @static
                         * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                         * @returns {string} The default type url
                         */
                        DeleteFromRow.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                            if (typeUrlPrefix === undefined) {
                                typeUrlPrefix = "type.googleapis.com";
                            }
                            return typeUrlPrefix + "/google.bigtable.v2.Mutation.DeleteFromRow";
                        };
    
                        return DeleteFromRow;
                    })();
    
                    return Mutation;
                })();
    
                v2.ReadModifyWriteRule = (function() {
    
                    /**
                     * Properties of a ReadModifyWriteRule.
                     * @memberof google.bigtable.v2
                     * @interface IReadModifyWriteRule
                     * @property {string|null} [familyName] ReadModifyWriteRule familyName
                     * @property {Uint8Array|null} [columnQualifier] ReadModifyWriteRule columnQualifier
                     * @property {Uint8Array|null} [appendValue] ReadModifyWriteRule appendValue
                     * @property {number|Long|null} [incrementAmount] ReadModifyWriteRule incrementAmount
                     */
    
                    /**
                     * Constructs a new ReadModifyWriteRule.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a ReadModifyWriteRule.
                     * @implements IReadModifyWriteRule
                     * @constructor
                     * @param {google.bigtable.v2.IReadModifyWriteRule=} [properties] Properties to set
                     */
                    function ReadModifyWriteRule(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * ReadModifyWriteRule familyName.
                     * @member {string} familyName
                     * @memberof google.bigtable.v2.ReadModifyWriteRule
                     * @instance
                     */
                    ReadModifyWriteRule.prototype.familyName = "";
    
                    /**
                     * ReadModifyWriteRule columnQualifier.
                     * @member {Uint8Array} columnQualifier
                     * @memberof google.bigtable.v2.ReadModifyWriteRule
                     * @instance
                     */
                    ReadModifyWriteRule.prototype.columnQualifier = $util.newBuffer([]);
    
                    /**
                     * ReadModifyWriteRule appendValue.
                     * @member {Uint8Array|null|undefined} appendValue
                     * @memberof google.bigtable.v2.ReadModifyWriteRule
                     * @instance
                     */
                    ReadModifyWriteRule.prototype.appendValue = null;
    
                    /**
                     * ReadModifyWriteRule incrementAmount.
                     * @member {number|Long|null|undefined} incrementAmount
                     * @memberof google.bigtable.v2.ReadModifyWriteRule
                     * @instance
                     */
                    ReadModifyWriteRule.prototype.incrementAmount = null;
    
                    // OneOf field names bound to virtual getters and setters
                    var $oneOfFields;
    
                    /**
                     * ReadModifyWriteRule rule.
                     * @member {"appendValue"|"incrementAmount"|undefined} rule
                     * @memberof google.bigtable.v2.ReadModifyWriteRule
                     * @instance
                     */
                    Object.defineProperty(ReadModifyWriteRule.prototype, "rule", {
                        get: $util.oneOfGetter($oneOfFields = ["appendValue", "incrementAmount"]),
                        set: $util.oneOfSetter($oneOfFields)
                    });
    
                    /**
                     * Creates a new ReadModifyWriteRule instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.ReadModifyWriteRule
                     * @static
                     * @param {google.bigtable.v2.IReadModifyWriteRule=} [properties] Properties to set
                     * @returns {google.bigtable.v2.ReadModifyWriteRule} ReadModifyWriteRule instance
                     */
                    ReadModifyWriteRule.create = function create(properties) {
                        return new ReadModifyWriteRule(properties);
                    };
    
                    /**
                     * Encodes the specified ReadModifyWriteRule message. Does not implicitly {@link google.bigtable.v2.ReadModifyWriteRule.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.ReadModifyWriteRule
                     * @static
                     * @param {google.bigtable.v2.IReadModifyWriteRule} message ReadModifyWriteRule message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ReadModifyWriteRule.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.familyName != null && Object.hasOwnProperty.call(message, "familyName"))
                            writer.uint32(/* id 1, wireType 2 =*/10).string(message.familyName);
                        if (message.columnQualifier != null && Object.hasOwnProperty.call(message, "columnQualifier"))
                            writer.uint32(/* id 2, wireType 2 =*/18).bytes(message.columnQualifier);
                        if (message.appendValue != null && Object.hasOwnProperty.call(message, "appendValue"))
                            writer.uint32(/* id 3, wireType 2 =*/26).bytes(message.appendValue);
                        if (message.incrementAmount != null && Object.hasOwnProperty.call(message, "incrementAmount"))
                            writer.uint32(/* id 4, wireType 0 =*/32).int64(message.incrementAmount);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified ReadModifyWriteRule message, length delimited. Does not implicitly {@link google.bigtable.v2.ReadModifyWriteRule.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.ReadModifyWriteRule
                     * @static
                     * @param {google.bigtable.v2.IReadModifyWriteRule} message ReadModifyWriteRule message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ReadModifyWriteRule.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a ReadModifyWriteRule message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.ReadModifyWriteRule
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.ReadModifyWriteRule} ReadModifyWriteRule
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ReadModifyWriteRule.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.ReadModifyWriteRule();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.familyName = reader.string();
                                    break;
                                }
                            case 2: {
                                    message.columnQualifier = reader.bytes();
                                    break;
                                }
                            case 3: {
                                    message.appendValue = reader.bytes();
                                    break;
                                }
                            case 4: {
                                    message.incrementAmount = reader.int64();
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a ReadModifyWriteRule message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.ReadModifyWriteRule
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.ReadModifyWriteRule} ReadModifyWriteRule
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ReadModifyWriteRule.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a ReadModifyWriteRule message.
                     * @function verify
                     * @memberof google.bigtable.v2.ReadModifyWriteRule
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    ReadModifyWriteRule.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        var properties = {};
                        if (message.familyName != null && message.hasOwnProperty("familyName"))
                            if (!$util.isString(message.familyName))
                                return "familyName: string expected";
                        if (message.columnQualifier != null && message.hasOwnProperty("columnQualifier"))
                            if (!(message.columnQualifier && typeof message.columnQualifier.length === "number" || $util.isString(message.columnQualifier)))
                                return "columnQualifier: buffer expected";
                        if (message.appendValue != null && message.hasOwnProperty("appendValue")) {
                            properties.rule = 1;
                            if (!(message.appendValue && typeof message.appendValue.length === "number" || $util.isString(message.appendValue)))
                                return "appendValue: buffer expected";
                        }
                        if (message.incrementAmount != null && message.hasOwnProperty("incrementAmount")) {
                            if (properties.rule === 1)
                                return "rule: multiple values";
                            properties.rule = 1;
                            if (!$util.isInteger(message.incrementAmount) && !(message.incrementAmount && $util.isInteger(message.incrementAmount.low) && $util.isInteger(message.incrementAmount.high)))
                                return "incrementAmount: integer|Long expected";
                        }
                        return null;
                    };
    
                    /**
                     * Creates a ReadModifyWriteRule message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.ReadModifyWriteRule
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.ReadModifyWriteRule} ReadModifyWriteRule
                     */
                    ReadModifyWriteRule.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.ReadModifyWriteRule)
                            return object;
                        var message = new $root.google.bigtable.v2.ReadModifyWriteRule();
                        if (object.familyName != null)
                            message.familyName = String(object.familyName);
                        if (object.columnQualifier != null)
                            if (typeof object.columnQualifier === "string")
                                $util.base64.decode(object.columnQualifier, message.columnQualifier = $util.newBuffer($util.base64.length(object.columnQualifier)), 0);
                            else if (object.columnQualifier.length >= 0)
                                message.columnQualifier = object.columnQualifier;
                        if (object.appendValue != null)
                            if (typeof object.appendValue === "string")
                                $util.base64.decode(object.appendValue, message.appendValue = $util.newBuffer($util.base64.length(object.appendValue)), 0);
                            else if (object.appendValue.length >= 0)
                                message.appendValue = object.appendValue;
                        if (object.incrementAmount != null)
                            if ($util.Long)
                                (message.incrementAmount = $util.Long.fromValue(object.incrementAmount)).unsigned = false;
                            else if (typeof object.incrementAmount === "string")
                                message.incrementAmount = parseInt(object.incrementAmount, 10);
                            else if (typeof object.incrementAmount === "number")
                                message.incrementAmount = object.incrementAmount;
                            else if (typeof object.incrementAmount === "object")
                                message.incrementAmount = new $util.LongBits(object.incrementAmount.low >>> 0, object.incrementAmount.high >>> 0).toNumber();
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a ReadModifyWriteRule message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.ReadModifyWriteRule
                     * @static
                     * @param {google.bigtable.v2.ReadModifyWriteRule} message ReadModifyWriteRule
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    ReadModifyWriteRule.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults) {
                            object.familyName = "";
                            if (options.bytes === String)
                                object.columnQualifier = "";
                            else {
                                object.columnQualifier = [];
                                if (options.bytes !== Array)
                                    object.columnQualifier = $util.newBuffer(object.columnQualifier);
                            }
                        }
                        if (message.familyName != null && message.hasOwnProperty("familyName"))
                            object.familyName = message.familyName;
                        if (message.columnQualifier != null && message.hasOwnProperty("columnQualifier"))
                            object.columnQualifier = options.bytes === String ? $util.base64.encode(message.columnQualifier, 0, message.columnQualifier.length) : options.bytes === Array ? Array.prototype.slice.call(message.columnQualifier) : message.columnQualifier;
                        if (message.appendValue != null && message.hasOwnProperty("appendValue")) {
                            object.appendValue = options.bytes === String ? $util.base64.encode(message.appendValue, 0, message.appendValue.length) : options.bytes === Array ? Array.prototype.slice.call(message.appendValue) : message.appendValue;
                            if (options.oneofs)
                                object.rule = "appendValue";
                        }
                        if (message.incrementAmount != null && message.hasOwnProperty("incrementAmount")) {
                            if (typeof message.incrementAmount === "number")
                                object.incrementAmount = options.longs === String ? String(message.incrementAmount) : message.incrementAmount;
                            else
                                object.incrementAmount = options.longs === String ? $util.Long.prototype.toString.call(message.incrementAmount) : options.longs === Number ? new $util.LongBits(message.incrementAmount.low >>> 0, message.incrementAmount.high >>> 0).toNumber() : message.incrementAmount;
                            if (options.oneofs)
                                object.rule = "incrementAmount";
                        }
                        return object;
                    };
    
                    /**
                     * Converts this ReadModifyWriteRule to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.ReadModifyWriteRule
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    ReadModifyWriteRule.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for ReadModifyWriteRule
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.ReadModifyWriteRule
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    ReadModifyWriteRule.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.ReadModifyWriteRule";
                    };
    
                    return ReadModifyWriteRule;
                })();
    
                v2.StreamPartition = (function() {
    
                    /**
                     * Properties of a StreamPartition.
                     * @memberof google.bigtable.v2
                     * @interface IStreamPartition
                     * @property {google.bigtable.v2.IRowRange|null} [rowRange] StreamPartition rowRange
                     */
    
                    /**
                     * Constructs a new StreamPartition.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a StreamPartition.
                     * @implements IStreamPartition
                     * @constructor
                     * @param {google.bigtable.v2.IStreamPartition=} [properties] Properties to set
                     */
                    function StreamPartition(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * StreamPartition rowRange.
                     * @member {google.bigtable.v2.IRowRange|null|undefined} rowRange
                     * @memberof google.bigtable.v2.StreamPartition
                     * @instance
                     */
                    StreamPartition.prototype.rowRange = null;
    
                    /**
                     * Creates a new StreamPartition instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.StreamPartition
                     * @static
                     * @param {google.bigtable.v2.IStreamPartition=} [properties] Properties to set
                     * @returns {google.bigtable.v2.StreamPartition} StreamPartition instance
                     */
                    StreamPartition.create = function create(properties) {
                        return new StreamPartition(properties);
                    };
    
                    /**
                     * Encodes the specified StreamPartition message. Does not implicitly {@link google.bigtable.v2.StreamPartition.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.StreamPartition
                     * @static
                     * @param {google.bigtable.v2.IStreamPartition} message StreamPartition message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    StreamPartition.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.rowRange != null && Object.hasOwnProperty.call(message, "rowRange"))
                            $root.google.bigtable.v2.RowRange.encode(message.rowRange, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                        return writer;
                    };
    
                    /**
                     * Encodes the specified StreamPartition message, length delimited. Does not implicitly {@link google.bigtable.v2.StreamPartition.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.StreamPartition
                     * @static
                     * @param {google.bigtable.v2.IStreamPartition} message StreamPartition message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    StreamPartition.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a StreamPartition message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.StreamPartition
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.StreamPartition} StreamPartition
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    StreamPartition.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.StreamPartition();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.rowRange = $root.google.bigtable.v2.RowRange.decode(reader, reader.uint32());
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a StreamPartition message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.StreamPartition
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.StreamPartition} StreamPartition
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    StreamPartition.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a StreamPartition message.
                     * @function verify
                     * @memberof google.bigtable.v2.StreamPartition
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    StreamPartition.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.rowRange != null && message.hasOwnProperty("rowRange")) {
                            var error = $root.google.bigtable.v2.RowRange.verify(message.rowRange);
                            if (error)
                                return "rowRange." + error;
                        }
                        return null;
                    };
    
                    /**
                     * Creates a StreamPartition message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.StreamPartition
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.StreamPartition} StreamPartition
                     */
                    StreamPartition.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.StreamPartition)
                            return object;
                        var message = new $root.google.bigtable.v2.StreamPartition();
                        if (object.rowRange != null) {
                            if (typeof object.rowRange !== "object")
                                throw TypeError(".google.bigtable.v2.StreamPartition.rowRange: object expected");
                            message.rowRange = $root.google.bigtable.v2.RowRange.fromObject(object.rowRange);
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a StreamPartition message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.StreamPartition
                     * @static
                     * @param {google.bigtable.v2.StreamPartition} message StreamPartition
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    StreamPartition.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults)
                            object.rowRange = null;
                        if (message.rowRange != null && message.hasOwnProperty("rowRange"))
                            object.rowRange = $root.google.bigtable.v2.RowRange.toObject(message.rowRange, options);
                        return object;
                    };
    
                    /**
                     * Converts this StreamPartition to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.StreamPartition
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    StreamPartition.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for StreamPartition
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.StreamPartition
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    StreamPartition.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.StreamPartition";
                    };
    
                    return StreamPartition;
                })();
    
                v2.StreamContinuationTokens = (function() {
    
                    /**
                     * Properties of a StreamContinuationTokens.
                     * @memberof google.bigtable.v2
                     * @interface IStreamContinuationTokens
                     * @property {Array.<google.bigtable.v2.IStreamContinuationToken>|null} [tokens] StreamContinuationTokens tokens
                     */
    
                    /**
                     * Constructs a new StreamContinuationTokens.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a StreamContinuationTokens.
                     * @implements IStreamContinuationTokens
                     * @constructor
                     * @param {google.bigtable.v2.IStreamContinuationTokens=} [properties] Properties to set
                     */
                    function StreamContinuationTokens(properties) {
                        this.tokens = [];
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * StreamContinuationTokens tokens.
                     * @member {Array.<google.bigtable.v2.IStreamContinuationToken>} tokens
                     * @memberof google.bigtable.v2.StreamContinuationTokens
                     * @instance
                     */
                    StreamContinuationTokens.prototype.tokens = $util.emptyArray;
    
                    /**
                     * Creates a new StreamContinuationTokens instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.StreamContinuationTokens
                     * @static
                     * @param {google.bigtable.v2.IStreamContinuationTokens=} [properties] Properties to set
                     * @returns {google.bigtable.v2.StreamContinuationTokens} StreamContinuationTokens instance
                     */
                    StreamContinuationTokens.create = function create(properties) {
                        return new StreamContinuationTokens(properties);
                    };
    
                    /**
                     * Encodes the specified StreamContinuationTokens message. Does not implicitly {@link google.bigtable.v2.StreamContinuationTokens.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.StreamContinuationTokens
                     * @static
                     * @param {google.bigtable.v2.IStreamContinuationTokens} message StreamContinuationTokens message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    StreamContinuationTokens.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.tokens != null && message.tokens.length)
                            for (var i = 0; i < message.tokens.length; ++i)
                                $root.google.bigtable.v2.StreamContinuationToken.encode(message.tokens[i], writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                        return writer;
                    };
    
                    /**
                     * Encodes the specified StreamContinuationTokens message, length delimited. Does not implicitly {@link google.bigtable.v2.StreamContinuationTokens.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.StreamContinuationTokens
                     * @static
                     * @param {google.bigtable.v2.IStreamContinuationTokens} message StreamContinuationTokens message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    StreamContinuationTokens.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a StreamContinuationTokens message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.StreamContinuationTokens
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.StreamContinuationTokens} StreamContinuationTokens
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    StreamContinuationTokens.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.StreamContinuationTokens();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    if (!(message.tokens && message.tokens.length))
                                        message.tokens = [];
                                    message.tokens.push($root.google.bigtable.v2.StreamContinuationToken.decode(reader, reader.uint32()));
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a StreamContinuationTokens message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.StreamContinuationTokens
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.StreamContinuationTokens} StreamContinuationTokens
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    StreamContinuationTokens.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a StreamContinuationTokens message.
                     * @function verify
                     * @memberof google.bigtable.v2.StreamContinuationTokens
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    StreamContinuationTokens.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.tokens != null && message.hasOwnProperty("tokens")) {
                            if (!Array.isArray(message.tokens))
                                return "tokens: array expected";
                            for (var i = 0; i < message.tokens.length; ++i) {
                                var error = $root.google.bigtable.v2.StreamContinuationToken.verify(message.tokens[i]);
                                if (error)
                                    return "tokens." + error;
                            }
                        }
                        return null;
                    };
    
                    /**
                     * Creates a StreamContinuationTokens message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.StreamContinuationTokens
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.StreamContinuationTokens} StreamContinuationTokens
                     */
                    StreamContinuationTokens.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.StreamContinuationTokens)
                            return object;
                        var message = new $root.google.bigtable.v2.StreamContinuationTokens();
                        if (object.tokens) {
                            if (!Array.isArray(object.tokens))
                                throw TypeError(".google.bigtable.v2.StreamContinuationTokens.tokens: array expected");
                            message.tokens = [];
                            for (var i = 0; i < object.tokens.length; ++i) {
                                if (typeof object.tokens[i] !== "object")
                                    throw TypeError(".google.bigtable.v2.StreamContinuationTokens.tokens: object expected");
                                message.tokens[i] = $root.google.bigtable.v2.StreamContinuationToken.fromObject(object.tokens[i]);
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a StreamContinuationTokens message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.StreamContinuationTokens
                     * @static
                     * @param {google.bigtable.v2.StreamContinuationTokens} message StreamContinuationTokens
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    StreamContinuationTokens.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.arrays || options.defaults)
                            object.tokens = [];
                        if (message.tokens && message.tokens.length) {
                            object.tokens = [];
                            for (var j = 0; j < message.tokens.length; ++j)
                                object.tokens[j] = $root.google.bigtable.v2.StreamContinuationToken.toObject(message.tokens[j], options);
                        }
                        return object;
                    };
    
                    /**
                     * Converts this StreamContinuationTokens to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.StreamContinuationTokens
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    StreamContinuationTokens.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for StreamContinuationTokens
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.StreamContinuationTokens
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    StreamContinuationTokens.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.StreamContinuationTokens";
                    };
    
                    return StreamContinuationTokens;
                })();
    
                v2.StreamContinuationToken = (function() {
    
                    /**
                     * Properties of a StreamContinuationToken.
                     * @memberof google.bigtable.v2
                     * @interface IStreamContinuationToken
                     * @property {google.bigtable.v2.IStreamPartition|null} [partition] StreamContinuationToken partition
                     * @property {string|null} [token] StreamContinuationToken token
                     */
    
                    /**
                     * Constructs a new StreamContinuationToken.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a StreamContinuationToken.
                     * @implements IStreamContinuationToken
                     * @constructor
                     * @param {google.bigtable.v2.IStreamContinuationToken=} [properties] Properties to set
                     */
                    function StreamContinuationToken(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * StreamContinuationToken partition.
                     * @member {google.bigtable.v2.IStreamPartition|null|undefined} partition
                     * @memberof google.bigtable.v2.StreamContinuationToken
                     * @instance
                     */
                    StreamContinuationToken.prototype.partition = null;
    
                    /**
                     * StreamContinuationToken token.
                     * @member {string} token
                     * @memberof google.bigtable.v2.StreamContinuationToken
                     * @instance
                     */
                    StreamContinuationToken.prototype.token = "";
    
                    /**
                     * Creates a new StreamContinuationToken instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.StreamContinuationToken
                     * @static
                     * @param {google.bigtable.v2.IStreamContinuationToken=} [properties] Properties to set
                     * @returns {google.bigtable.v2.StreamContinuationToken} StreamContinuationToken instance
                     */
                    StreamContinuationToken.create = function create(properties) {
                        return new StreamContinuationToken(properties);
                    };
    
                    /**
                     * Encodes the specified StreamContinuationToken message. Does not implicitly {@link google.bigtable.v2.StreamContinuationToken.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.StreamContinuationToken
                     * @static
                     * @param {google.bigtable.v2.IStreamContinuationToken} message StreamContinuationToken message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    StreamContinuationToken.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.partition != null && Object.hasOwnProperty.call(message, "partition"))
                            $root.google.bigtable.v2.StreamPartition.encode(message.partition, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                        if (message.token != null && Object.hasOwnProperty.call(message, "token"))
                            writer.uint32(/* id 2, wireType 2 =*/18).string(message.token);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified StreamContinuationToken message, length delimited. Does not implicitly {@link google.bigtable.v2.StreamContinuationToken.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.StreamContinuationToken
                     * @static
                     * @param {google.bigtable.v2.IStreamContinuationToken} message StreamContinuationToken message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    StreamContinuationToken.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a StreamContinuationToken message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.StreamContinuationToken
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.StreamContinuationToken} StreamContinuationToken
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    StreamContinuationToken.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.StreamContinuationToken();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.partition = $root.google.bigtable.v2.StreamPartition.decode(reader, reader.uint32());
                                    break;
                                }
                            case 2: {
                                    message.token = reader.string();
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a StreamContinuationToken message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.StreamContinuationToken
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.StreamContinuationToken} StreamContinuationToken
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    StreamContinuationToken.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a StreamContinuationToken message.
                     * @function verify
                     * @memberof google.bigtable.v2.StreamContinuationToken
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    StreamContinuationToken.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.partition != null && message.hasOwnProperty("partition")) {
                            var error = $root.google.bigtable.v2.StreamPartition.verify(message.partition);
                            if (error)
                                return "partition." + error;
                        }
                        if (message.token != null && message.hasOwnProperty("token"))
                            if (!$util.isString(message.token))
                                return "token: string expected";
                        return null;
                    };
    
                    /**
                     * Creates a StreamContinuationToken message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.StreamContinuationToken
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.StreamContinuationToken} StreamContinuationToken
                     */
                    StreamContinuationToken.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.StreamContinuationToken)
                            return object;
                        var message = new $root.google.bigtable.v2.StreamContinuationToken();
                        if (object.partition != null) {
                            if (typeof object.partition !== "object")
                                throw TypeError(".google.bigtable.v2.StreamContinuationToken.partition: object expected");
                            message.partition = $root.google.bigtable.v2.StreamPartition.fromObject(object.partition);
                        }
                        if (object.token != null)
                            message.token = String(object.token);
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a StreamContinuationToken message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.StreamContinuationToken
                     * @static
                     * @param {google.bigtable.v2.StreamContinuationToken} message StreamContinuationToken
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    StreamContinuationToken.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults) {
                            object.partition = null;
                            object.token = "";
                        }
                        if (message.partition != null && message.hasOwnProperty("partition"))
                            object.partition = $root.google.bigtable.v2.StreamPartition.toObject(message.partition, options);
                        if (message.token != null && message.hasOwnProperty("token"))
                            object.token = message.token;
                        return object;
                    };
    
                    /**
                     * Converts this StreamContinuationToken to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.StreamContinuationToken
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    StreamContinuationToken.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for StreamContinuationToken
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.StreamContinuationToken
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    StreamContinuationToken.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.StreamContinuationToken";
                    };
    
                    return StreamContinuationToken;
                })();
    
                v2.ReadIterationStats = (function() {
    
                    /**
                     * Properties of a ReadIterationStats.
                     * @memberof google.bigtable.v2
                     * @interface IReadIterationStats
                     * @property {number|Long|null} [rowsSeenCount] ReadIterationStats rowsSeenCount
                     * @property {number|Long|null} [rowsReturnedCount] ReadIterationStats rowsReturnedCount
                     * @property {number|Long|null} [cellsSeenCount] ReadIterationStats cellsSeenCount
                     * @property {number|Long|null} [cellsReturnedCount] ReadIterationStats cellsReturnedCount
                     */
    
                    /**
                     * Constructs a new ReadIterationStats.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a ReadIterationStats.
                     * @implements IReadIterationStats
                     * @constructor
                     * @param {google.bigtable.v2.IReadIterationStats=} [properties] Properties to set
                     */
                    function ReadIterationStats(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * ReadIterationStats rowsSeenCount.
                     * @member {number|Long} rowsSeenCount
                     * @memberof google.bigtable.v2.ReadIterationStats
                     * @instance
                     */
                    ReadIterationStats.prototype.rowsSeenCount = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                    /**
                     * ReadIterationStats rowsReturnedCount.
                     * @member {number|Long} rowsReturnedCount
                     * @memberof google.bigtable.v2.ReadIterationStats
                     * @instance
                     */
                    ReadIterationStats.prototype.rowsReturnedCount = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                    /**
                     * ReadIterationStats cellsSeenCount.
                     * @member {number|Long} cellsSeenCount
                     * @memberof google.bigtable.v2.ReadIterationStats
                     * @instance
                     */
                    ReadIterationStats.prototype.cellsSeenCount = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                    /**
                     * ReadIterationStats cellsReturnedCount.
                     * @member {number|Long} cellsReturnedCount
                     * @memberof google.bigtable.v2.ReadIterationStats
                     * @instance
                     */
                    ReadIterationStats.prototype.cellsReturnedCount = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                    /**
                     * Creates a new ReadIterationStats instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.ReadIterationStats
                     * @static
                     * @param {google.bigtable.v2.IReadIterationStats=} [properties] Properties to set
                     * @returns {google.bigtable.v2.ReadIterationStats} ReadIterationStats instance
                     */
                    ReadIterationStats.create = function create(properties) {
                        return new ReadIterationStats(properties);
                    };
    
                    /**
                     * Encodes the specified ReadIterationStats message. Does not implicitly {@link google.bigtable.v2.ReadIterationStats.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.ReadIterationStats
                     * @static
                     * @param {google.bigtable.v2.IReadIterationStats} message ReadIterationStats message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ReadIterationStats.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.rowsSeenCount != null && Object.hasOwnProperty.call(message, "rowsSeenCount"))
                            writer.uint32(/* id 1, wireType 0 =*/8).int64(message.rowsSeenCount);
                        if (message.rowsReturnedCount != null && Object.hasOwnProperty.call(message, "rowsReturnedCount"))
                            writer.uint32(/* id 2, wireType 0 =*/16).int64(message.rowsReturnedCount);
                        if (message.cellsSeenCount != null && Object.hasOwnProperty.call(message, "cellsSeenCount"))
                            writer.uint32(/* id 3, wireType 0 =*/24).int64(message.cellsSeenCount);
                        if (message.cellsReturnedCount != null && Object.hasOwnProperty.call(message, "cellsReturnedCount"))
                            writer.uint32(/* id 4, wireType 0 =*/32).int64(message.cellsReturnedCount);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified ReadIterationStats message, length delimited. Does not implicitly {@link google.bigtable.v2.ReadIterationStats.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.ReadIterationStats
                     * @static
                     * @param {google.bigtable.v2.IReadIterationStats} message ReadIterationStats message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ReadIterationStats.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a ReadIterationStats message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.ReadIterationStats
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.ReadIterationStats} ReadIterationStats
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ReadIterationStats.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.ReadIterationStats();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.rowsSeenCount = reader.int64();
                                    break;
                                }
                            case 2: {
                                    message.rowsReturnedCount = reader.int64();
                                    break;
                                }
                            case 3: {
                                    message.cellsSeenCount = reader.int64();
                                    break;
                                }
                            case 4: {
                                    message.cellsReturnedCount = reader.int64();
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a ReadIterationStats message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.ReadIterationStats
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.ReadIterationStats} ReadIterationStats
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ReadIterationStats.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a ReadIterationStats message.
                     * @function verify
                     * @memberof google.bigtable.v2.ReadIterationStats
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    ReadIterationStats.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.rowsSeenCount != null && message.hasOwnProperty("rowsSeenCount"))
                            if (!$util.isInteger(message.rowsSeenCount) && !(message.rowsSeenCount && $util.isInteger(message.rowsSeenCount.low) && $util.isInteger(message.rowsSeenCount.high)))
                                return "rowsSeenCount: integer|Long expected";
                        if (message.rowsReturnedCount != null && message.hasOwnProperty("rowsReturnedCount"))
                            if (!$util.isInteger(message.rowsReturnedCount) && !(message.rowsReturnedCount && $util.isInteger(message.rowsReturnedCount.low) && $util.isInteger(message.rowsReturnedCount.high)))
                                return "rowsReturnedCount: integer|Long expected";
                        if (message.cellsSeenCount != null && message.hasOwnProperty("cellsSeenCount"))
                            if (!$util.isInteger(message.cellsSeenCount) && !(message.cellsSeenCount && $util.isInteger(message.cellsSeenCount.low) && $util.isInteger(message.cellsSeenCount.high)))
                                return "cellsSeenCount: integer|Long expected";
                        if (message.cellsReturnedCount != null && message.hasOwnProperty("cellsReturnedCount"))
                            if (!$util.isInteger(message.cellsReturnedCount) && !(message.cellsReturnedCount && $util.isInteger(message.cellsReturnedCount.low) && $util.isInteger(message.cellsReturnedCount.high)))
                                return "cellsReturnedCount: integer|Long expected";
                        return null;
                    };
    
                    /**
                     * Creates a ReadIterationStats message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.ReadIterationStats
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.ReadIterationStats} ReadIterationStats
                     */
                    ReadIterationStats.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.ReadIterationStats)
                            return object;
                        var message = new $root.google.bigtable.v2.ReadIterationStats();
                        if (object.rowsSeenCount != null)
                            if ($util.Long)
                                (message.rowsSeenCount = $util.Long.fromValue(object.rowsSeenCount)).unsigned = false;
                            else if (typeof object.rowsSeenCount === "string")
                                message.rowsSeenCount = parseInt(object.rowsSeenCount, 10);
                            else if (typeof object.rowsSeenCount === "number")
                                message.rowsSeenCount = object.rowsSeenCount;
                            else if (typeof object.rowsSeenCount === "object")
                                message.rowsSeenCount = new $util.LongBits(object.rowsSeenCount.low >>> 0, object.rowsSeenCount.high >>> 0).toNumber();
                        if (object.rowsReturnedCount != null)
                            if ($util.Long)
                                (message.rowsReturnedCount = $util.Long.fromValue(object.rowsReturnedCount)).unsigned = false;
                            else if (typeof object.rowsReturnedCount === "string")
                                message.rowsReturnedCount = parseInt(object.rowsReturnedCount, 10);
                            else if (typeof object.rowsReturnedCount === "number")
                                message.rowsReturnedCount = object.rowsReturnedCount;
                            else if (typeof object.rowsReturnedCount === "object")
                                message.rowsReturnedCount = new $util.LongBits(object.rowsReturnedCount.low >>> 0, object.rowsReturnedCount.high >>> 0).toNumber();
                        if (object.cellsSeenCount != null)
                            if ($util.Long)
                                (message.cellsSeenCount = $util.Long.fromValue(object.cellsSeenCount)).unsigned = false;
                            else if (typeof object.cellsSeenCount === "string")
                                message.cellsSeenCount = parseInt(object.cellsSeenCount, 10);
                            else if (typeof object.cellsSeenCount === "number")
                                message.cellsSeenCount = object.cellsSeenCount;
                            else if (typeof object.cellsSeenCount === "object")
                                message.cellsSeenCount = new $util.LongBits(object.cellsSeenCount.low >>> 0, object.cellsSeenCount.high >>> 0).toNumber();
                        if (object.cellsReturnedCount != null)
                            if ($util.Long)
                                (message.cellsReturnedCount = $util.Long.fromValue(object.cellsReturnedCount)).unsigned = false;
                            else if (typeof object.cellsReturnedCount === "string")
                                message.cellsReturnedCount = parseInt(object.cellsReturnedCount, 10);
                            else if (typeof object.cellsReturnedCount === "number")
                                message.cellsReturnedCount = object.cellsReturnedCount;
                            else if (typeof object.cellsReturnedCount === "object")
                                message.cellsReturnedCount = new $util.LongBits(object.cellsReturnedCount.low >>> 0, object.cellsReturnedCount.high >>> 0).toNumber();
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a ReadIterationStats message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.ReadIterationStats
                     * @static
                     * @param {google.bigtable.v2.ReadIterationStats} message ReadIterationStats
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    ReadIterationStats.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults) {
                            if ($util.Long) {
                                var long = new $util.Long(0, 0, false);
                                object.rowsSeenCount = options.longs === String ? long.toString() : options.longs === Number ? long.toNumber() : long;
                            } else
                                object.rowsSeenCount = options.longs === String ? "0" : 0;
                            if ($util.Long) {
                                var long = new $util.Long(0, 0, false);
                                object.rowsReturnedCount = options.longs === String ? long.toString() : options.longs === Number ? long.toNumber() : long;
                            } else
                                object.rowsReturnedCount = options.longs === String ? "0" : 0;
                            if ($util.Long) {
                                var long = new $util.Long(0, 0, false);
                                object.cellsSeenCount = options.longs === String ? long.toString() : options.longs === Number ? long.toNumber() : long;
                            } else
                                object.cellsSeenCount = options.longs === String ? "0" : 0;
                            if ($util.Long) {
                                var long = new $util.Long(0, 0, false);
                                object.cellsReturnedCount = options.longs === String ? long.toString() : options.longs === Number ? long.toNumber() : long;
                            } else
                                object.cellsReturnedCount = options.longs === String ? "0" : 0;
                        }
                        if (message.rowsSeenCount != null && message.hasOwnProperty("rowsSeenCount"))
                            if (typeof message.rowsSeenCount === "number")
                                object.rowsSeenCount = options.longs === String ? String(message.rowsSeenCount) : message.rowsSeenCount;
                            else
                                object.rowsSeenCount = options.longs === String ? $util.Long.prototype.toString.call(message.rowsSeenCount) : options.longs === Number ? new $util.LongBits(message.rowsSeenCount.low >>> 0, message.rowsSeenCount.high >>> 0).toNumber() : message.rowsSeenCount;
                        if (message.rowsReturnedCount != null && message.hasOwnProperty("rowsReturnedCount"))
                            if (typeof message.rowsReturnedCount === "number")
                                object.rowsReturnedCount = options.longs === String ? String(message.rowsReturnedCount) : message.rowsReturnedCount;
                            else
                                object.rowsReturnedCount = options.longs === String ? $util.Long.prototype.toString.call(message.rowsReturnedCount) : options.longs === Number ? new $util.LongBits(message.rowsReturnedCount.low >>> 0, message.rowsReturnedCount.high >>> 0).toNumber() : message.rowsReturnedCount;
                        if (message.cellsSeenCount != null && message.hasOwnProperty("cellsSeenCount"))
                            if (typeof message.cellsSeenCount === "number")
                                object.cellsSeenCount = options.longs === String ? String(message.cellsSeenCount) : message.cellsSeenCount;
                            else
                                object.cellsSeenCount = options.longs === String ? $util.Long.prototype.toString.call(message.cellsSeenCount) : options.longs === Number ? new $util.LongBits(message.cellsSeenCount.low >>> 0, message.cellsSeenCount.high >>> 0).toNumber() : message.cellsSeenCount;
                        if (message.cellsReturnedCount != null && message.hasOwnProperty("cellsReturnedCount"))
                            if (typeof message.cellsReturnedCount === "number")
                                object.cellsReturnedCount = options.longs === String ? String(message.cellsReturnedCount) : message.cellsReturnedCount;
                            else
                                object.cellsReturnedCount = options.longs === String ? $util.Long.prototype.toString.call(message.cellsReturnedCount) : options.longs === Number ? new $util.LongBits(message.cellsReturnedCount.low >>> 0, message.cellsReturnedCount.high >>> 0).toNumber() : message.cellsReturnedCount;
                        return object;
                    };
    
                    /**
                     * Converts this ReadIterationStats to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.ReadIterationStats
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    ReadIterationStats.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for ReadIterationStats
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.ReadIterationStats
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    ReadIterationStats.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.ReadIterationStats";
                    };
    
                    return ReadIterationStats;
                })();
    
                v2.RequestLatencyStats = (function() {
    
                    /**
                     * Properties of a RequestLatencyStats.
                     * @memberof google.bigtable.v2
                     * @interface IRequestLatencyStats
                     * @property {google.protobuf.IDuration|null} [frontendServerLatency] RequestLatencyStats frontendServerLatency
                     */
    
                    /**
                     * Constructs a new RequestLatencyStats.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a RequestLatencyStats.
                     * @implements IRequestLatencyStats
                     * @constructor
                     * @param {google.bigtable.v2.IRequestLatencyStats=} [properties] Properties to set
                     */
                    function RequestLatencyStats(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * RequestLatencyStats frontendServerLatency.
                     * @member {google.protobuf.IDuration|null|undefined} frontendServerLatency
                     * @memberof google.bigtable.v2.RequestLatencyStats
                     * @instance
                     */
                    RequestLatencyStats.prototype.frontendServerLatency = null;
    
                    /**
                     * Creates a new RequestLatencyStats instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.RequestLatencyStats
                     * @static
                     * @param {google.bigtable.v2.IRequestLatencyStats=} [properties] Properties to set
                     * @returns {google.bigtable.v2.RequestLatencyStats} RequestLatencyStats instance
                     */
                    RequestLatencyStats.create = function create(properties) {
                        return new RequestLatencyStats(properties);
                    };
    
                    /**
                     * Encodes the specified RequestLatencyStats message. Does not implicitly {@link google.bigtable.v2.RequestLatencyStats.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.RequestLatencyStats
                     * @static
                     * @param {google.bigtable.v2.IRequestLatencyStats} message RequestLatencyStats message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    RequestLatencyStats.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.frontendServerLatency != null && Object.hasOwnProperty.call(message, "frontendServerLatency"))
                            $root.google.protobuf.Duration.encode(message.frontendServerLatency, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                        return writer;
                    };
    
                    /**
                     * Encodes the specified RequestLatencyStats message, length delimited. Does not implicitly {@link google.bigtable.v2.RequestLatencyStats.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.RequestLatencyStats
                     * @static
                     * @param {google.bigtable.v2.IRequestLatencyStats} message RequestLatencyStats message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    RequestLatencyStats.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a RequestLatencyStats message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.RequestLatencyStats
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.RequestLatencyStats} RequestLatencyStats
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    RequestLatencyStats.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.RequestLatencyStats();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.frontendServerLatency = $root.google.protobuf.Duration.decode(reader, reader.uint32());
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a RequestLatencyStats message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.RequestLatencyStats
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.RequestLatencyStats} RequestLatencyStats
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    RequestLatencyStats.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a RequestLatencyStats message.
                     * @function verify
                     * @memberof google.bigtable.v2.RequestLatencyStats
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    RequestLatencyStats.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.frontendServerLatency != null && message.hasOwnProperty("frontendServerLatency")) {
                            var error = $root.google.protobuf.Duration.verify(message.frontendServerLatency);
                            if (error)
                                return "frontendServerLatency." + error;
                        }
                        return null;
                    };
    
                    /**
                     * Creates a RequestLatencyStats message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.RequestLatencyStats
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.RequestLatencyStats} RequestLatencyStats
                     */
                    RequestLatencyStats.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.RequestLatencyStats)
                            return object;
                        var message = new $root.google.bigtable.v2.RequestLatencyStats();
                        if (object.frontendServerLatency != null) {
                            if (typeof object.frontendServerLatency !== "object")
                                throw TypeError(".google.bigtable.v2.RequestLatencyStats.frontendServerLatency: object expected");
                            message.frontendServerLatency = $root.google.protobuf.Duration.fromObject(object.frontendServerLatency);
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a RequestLatencyStats message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.RequestLatencyStats
                     * @static
                     * @param {google.bigtable.v2.RequestLatencyStats} message RequestLatencyStats
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    RequestLatencyStats.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults)
                            object.frontendServerLatency = null;
                        if (message.frontendServerLatency != null && message.hasOwnProperty("frontendServerLatency"))
                            object.frontendServerLatency = $root.google.protobuf.Duration.toObject(message.frontendServerLatency, options);
                        return object;
                    };
    
                    /**
                     * Converts this RequestLatencyStats to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.RequestLatencyStats
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    RequestLatencyStats.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for RequestLatencyStats
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.RequestLatencyStats
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    RequestLatencyStats.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.RequestLatencyStats";
                    };
    
                    return RequestLatencyStats;
                })();
    
                v2.FullReadStatsView = (function() {
    
                    /**
                     * Properties of a FullReadStatsView.
                     * @memberof google.bigtable.v2
                     * @interface IFullReadStatsView
                     * @property {google.bigtable.v2.IReadIterationStats|null} [readIterationStats] FullReadStatsView readIterationStats
                     * @property {google.bigtable.v2.IRequestLatencyStats|null} [requestLatencyStats] FullReadStatsView requestLatencyStats
                     */
    
                    /**
                     * Constructs a new FullReadStatsView.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a FullReadStatsView.
                     * @implements IFullReadStatsView
                     * @constructor
                     * @param {google.bigtable.v2.IFullReadStatsView=} [properties] Properties to set
                     */
                    function FullReadStatsView(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * FullReadStatsView readIterationStats.
                     * @member {google.bigtable.v2.IReadIterationStats|null|undefined} readIterationStats
                     * @memberof google.bigtable.v2.FullReadStatsView
                     * @instance
                     */
                    FullReadStatsView.prototype.readIterationStats = null;
    
                    /**
                     * FullReadStatsView requestLatencyStats.
                     * @member {google.bigtable.v2.IRequestLatencyStats|null|undefined} requestLatencyStats
                     * @memberof google.bigtable.v2.FullReadStatsView
                     * @instance
                     */
                    FullReadStatsView.prototype.requestLatencyStats = null;
    
                    /**
                     * Creates a new FullReadStatsView instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.FullReadStatsView
                     * @static
                     * @param {google.bigtable.v2.IFullReadStatsView=} [properties] Properties to set
                     * @returns {google.bigtable.v2.FullReadStatsView} FullReadStatsView instance
                     */
                    FullReadStatsView.create = function create(properties) {
                        return new FullReadStatsView(properties);
                    };
    
                    /**
                     * Encodes the specified FullReadStatsView message. Does not implicitly {@link google.bigtable.v2.FullReadStatsView.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.FullReadStatsView
                     * @static
                     * @param {google.bigtable.v2.IFullReadStatsView} message FullReadStatsView message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    FullReadStatsView.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.readIterationStats != null && Object.hasOwnProperty.call(message, "readIterationStats"))
                            $root.google.bigtable.v2.ReadIterationStats.encode(message.readIterationStats, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                        if (message.requestLatencyStats != null && Object.hasOwnProperty.call(message, "requestLatencyStats"))
                            $root.google.bigtable.v2.RequestLatencyStats.encode(message.requestLatencyStats, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                        return writer;
                    };
    
                    /**
                     * Encodes the specified FullReadStatsView message, length delimited. Does not implicitly {@link google.bigtable.v2.FullReadStatsView.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.FullReadStatsView
                     * @static
                     * @param {google.bigtable.v2.IFullReadStatsView} message FullReadStatsView message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    FullReadStatsView.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a FullReadStatsView message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.FullReadStatsView
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.FullReadStatsView} FullReadStatsView
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    FullReadStatsView.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.FullReadStatsView();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.readIterationStats = $root.google.bigtable.v2.ReadIterationStats.decode(reader, reader.uint32());
                                    break;
                                }
                            case 2: {
                                    message.requestLatencyStats = $root.google.bigtable.v2.RequestLatencyStats.decode(reader, reader.uint32());
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a FullReadStatsView message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.FullReadStatsView
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.FullReadStatsView} FullReadStatsView
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    FullReadStatsView.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a FullReadStatsView message.
                     * @function verify
                     * @memberof google.bigtable.v2.FullReadStatsView
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    FullReadStatsView.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.readIterationStats != null && message.hasOwnProperty("readIterationStats")) {
                            var error = $root.google.bigtable.v2.ReadIterationStats.verify(message.readIterationStats);
                            if (error)
                                return "readIterationStats." + error;
                        }
                        if (message.requestLatencyStats != null && message.hasOwnProperty("requestLatencyStats")) {
                            var error = $root.google.bigtable.v2.RequestLatencyStats.verify(message.requestLatencyStats);
                            if (error)
                                return "requestLatencyStats." + error;
                        }
                        return null;
                    };
    
                    /**
                     * Creates a FullReadStatsView message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.FullReadStatsView
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.FullReadStatsView} FullReadStatsView
                     */
                    FullReadStatsView.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.FullReadStatsView)
                            return object;
                        var message = new $root.google.bigtable.v2.FullReadStatsView();
                        if (object.readIterationStats != null) {
                            if (typeof object.readIterationStats !== "object")
                                throw TypeError(".google.bigtable.v2.FullReadStatsView.readIterationStats: object expected");
                            message.readIterationStats = $root.google.bigtable.v2.ReadIterationStats.fromObject(object.readIterationStats);
                        }
                        if (object.requestLatencyStats != null) {
                            if (typeof object.requestLatencyStats !== "object")
                                throw TypeError(".google.bigtable.v2.FullReadStatsView.requestLatencyStats: object expected");
                            message.requestLatencyStats = $root.google.bigtable.v2.RequestLatencyStats.fromObject(object.requestLatencyStats);
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a FullReadStatsView message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.FullReadStatsView
                     * @static
                     * @param {google.bigtable.v2.FullReadStatsView} message FullReadStatsView
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    FullReadStatsView.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults) {
                            object.readIterationStats = null;
                            object.requestLatencyStats = null;
                        }
                        if (message.readIterationStats != null && message.hasOwnProperty("readIterationStats"))
                            object.readIterationStats = $root.google.bigtable.v2.ReadIterationStats.toObject(message.readIterationStats, options);
                        if (message.requestLatencyStats != null && message.hasOwnProperty("requestLatencyStats"))
                            object.requestLatencyStats = $root.google.bigtable.v2.RequestLatencyStats.toObject(message.requestLatencyStats, options);
                        return object;
                    };
    
                    /**
                     * Converts this FullReadStatsView to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.FullReadStatsView
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    FullReadStatsView.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for FullReadStatsView
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.FullReadStatsView
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    FullReadStatsView.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.FullReadStatsView";
                    };
    
                    return FullReadStatsView;
                })();
    
                v2.RequestStats = (function() {
    
                    /**
                     * Properties of a RequestStats.
                     * @memberof google.bigtable.v2
                     * @interface IRequestStats
                     * @property {google.bigtable.v2.IFullReadStatsView|null} [fullReadStatsView] RequestStats fullReadStatsView
                     */
    
                    /**
                     * Constructs a new RequestStats.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a RequestStats.
                     * @implements IRequestStats
                     * @constructor
                     * @param {google.bigtable.v2.IRequestStats=} [properties] Properties to set
                     */
                    function RequestStats(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * RequestStats fullReadStatsView.
                     * @member {google.bigtable.v2.IFullReadStatsView|null|undefined} fullReadStatsView
                     * @memberof google.bigtable.v2.RequestStats
                     * @instance
                     */
                    RequestStats.prototype.fullReadStatsView = null;
    
                    // OneOf field names bound to virtual getters and setters
                    var $oneOfFields;
    
                    /**
                     * RequestStats statsView.
                     * @member {"fullReadStatsView"|undefined} statsView
                     * @memberof google.bigtable.v2.RequestStats
                     * @instance
                     */
                    Object.defineProperty(RequestStats.prototype, "statsView", {
                        get: $util.oneOfGetter($oneOfFields = ["fullReadStatsView"]),
                        set: $util.oneOfSetter($oneOfFields)
                    });
    
                    /**
                     * Creates a new RequestStats instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.RequestStats
                     * @static
                     * @param {google.bigtable.v2.IRequestStats=} [properties] Properties to set
                     * @returns {google.bigtable.v2.RequestStats} RequestStats instance
                     */
                    RequestStats.create = function create(properties) {
                        return new RequestStats(properties);
                    };
    
                    /**
                     * Encodes the specified RequestStats message. Does not implicitly {@link google.bigtable.v2.RequestStats.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.RequestStats
                     * @static
                     * @param {google.bigtable.v2.IRequestStats} message RequestStats message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    RequestStats.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.fullReadStatsView != null && Object.hasOwnProperty.call(message, "fullReadStatsView"))
                            $root.google.bigtable.v2.FullReadStatsView.encode(message.fullReadStatsView, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                        return writer;
                    };
    
                    /**
                     * Encodes the specified RequestStats message, length delimited. Does not implicitly {@link google.bigtable.v2.RequestStats.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.RequestStats
                     * @static
                     * @param {google.bigtable.v2.IRequestStats} message RequestStats message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    RequestStats.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a RequestStats message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.RequestStats
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.RequestStats} RequestStats
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    RequestStats.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.RequestStats();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.fullReadStatsView = $root.google.bigtable.v2.FullReadStatsView.decode(reader, reader.uint32());
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a RequestStats message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.RequestStats
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.RequestStats} RequestStats
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    RequestStats.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a RequestStats message.
                     * @function verify
                     * @memberof google.bigtable.v2.RequestStats
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    RequestStats.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        var properties = {};
                        if (message.fullReadStatsView != null && message.hasOwnProperty("fullReadStatsView")) {
                            properties.statsView = 1;
                            {
                                var error = $root.google.bigtable.v2.FullReadStatsView.verify(message.fullReadStatsView);
                                if (error)
                                    return "fullReadStatsView." + error;
                            }
                        }
                        return null;
                    };
    
                    /**
                     * Creates a RequestStats message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.RequestStats
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.RequestStats} RequestStats
                     */
                    RequestStats.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.RequestStats)
                            return object;
                        var message = new $root.google.bigtable.v2.RequestStats();
                        if (object.fullReadStatsView != null) {
                            if (typeof object.fullReadStatsView !== "object")
                                throw TypeError(".google.bigtable.v2.RequestStats.fullReadStatsView: object expected");
                            message.fullReadStatsView = $root.google.bigtable.v2.FullReadStatsView.fromObject(object.fullReadStatsView);
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a RequestStats message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.RequestStats
                     * @static
                     * @param {google.bigtable.v2.RequestStats} message RequestStats
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    RequestStats.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (message.fullReadStatsView != null && message.hasOwnProperty("fullReadStatsView")) {
                            object.fullReadStatsView = $root.google.bigtable.v2.FullReadStatsView.toObject(message.fullReadStatsView, options);
                            if (options.oneofs)
                                object.statsView = "fullReadStatsView";
                        }
                        return object;
                    };
    
                    /**
                     * Converts this RequestStats to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.RequestStats
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    RequestStats.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for RequestStats
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.RequestStats
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    RequestStats.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.RequestStats";
                    };
    
                    return RequestStats;
                })();
    
                v2.FeatureFlags = (function() {
    
                    /**
                     * Properties of a FeatureFlags.
                     * @memberof google.bigtable.v2
                     * @interface IFeatureFlags
                     * @property {boolean|null} [reverseScans] FeatureFlags reverseScans
                     * @property {boolean|null} [mutateRowsRateLimit] FeatureFlags mutateRowsRateLimit
                     * @property {boolean|null} [mutateRowsRateLimit2] FeatureFlags mutateRowsRateLimit2
                     * @property {boolean|null} [lastScannedRowResponses] FeatureFlags lastScannedRowResponses
                     * @property {boolean|null} [routingCookie] FeatureFlags routingCookie
                     * @property {boolean|null} [retryInfo] FeatureFlags retryInfo
                     * @property {boolean|null} [clientSideMetricsEnabled] FeatureFlags clientSideMetricsEnabled
                     */
    
                    /**
                     * Constructs a new FeatureFlags.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a FeatureFlags.
                     * @implements IFeatureFlags
                     * @constructor
                     * @param {google.bigtable.v2.IFeatureFlags=} [properties] Properties to set
                     */
                    function FeatureFlags(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * FeatureFlags reverseScans.
                     * @member {boolean} reverseScans
                     * @memberof google.bigtable.v2.FeatureFlags
                     * @instance
                     */
                    FeatureFlags.prototype.reverseScans = false;
    
                    /**
                     * FeatureFlags mutateRowsRateLimit.
                     * @member {boolean} mutateRowsRateLimit
                     * @memberof google.bigtable.v2.FeatureFlags
                     * @instance
                     */
                    FeatureFlags.prototype.mutateRowsRateLimit = false;
    
                    /**
                     * FeatureFlags mutateRowsRateLimit2.
                     * @member {boolean} mutateRowsRateLimit2
                     * @memberof google.bigtable.v2.FeatureFlags
                     * @instance
                     */
                    FeatureFlags.prototype.mutateRowsRateLimit2 = false;
    
                    /**
                     * FeatureFlags lastScannedRowResponses.
                     * @member {boolean} lastScannedRowResponses
                     * @memberof google.bigtable.v2.FeatureFlags
                     * @instance
                     */
                    FeatureFlags.prototype.lastScannedRowResponses = false;
    
                    /**
                     * FeatureFlags routingCookie.
                     * @member {boolean} routingCookie
                     * @memberof google.bigtable.v2.FeatureFlags
                     * @instance
                     */
                    FeatureFlags.prototype.routingCookie = false;
    
                    /**
                     * FeatureFlags retryInfo.
                     * @member {boolean} retryInfo
                     * @memberof google.bigtable.v2.FeatureFlags
                     * @instance
                     */
                    FeatureFlags.prototype.retryInfo = false;
    
                    /**
                     * FeatureFlags clientSideMetricsEnabled.
                     * @member {boolean} clientSideMetricsEnabled
                     * @memberof google.bigtable.v2.FeatureFlags
                     * @instance
                     */
                    FeatureFlags.prototype.clientSideMetricsEnabled = false;
    
                    /**
                     * Creates a new FeatureFlags instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.FeatureFlags
                     * @static
                     * @param {google.bigtable.v2.IFeatureFlags=} [properties] Properties to set
                     * @returns {google.bigtable.v2.FeatureFlags} FeatureFlags instance
                     */
                    FeatureFlags.create = function create(properties) {
                        return new FeatureFlags(properties);
                    };
    
                    /**
                     * Encodes the specified FeatureFlags message. Does not implicitly {@link google.bigtable.v2.FeatureFlags.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.FeatureFlags
                     * @static
                     * @param {google.bigtable.v2.IFeatureFlags} message FeatureFlags message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    FeatureFlags.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.reverseScans != null && Object.hasOwnProperty.call(message, "reverseScans"))
                            writer.uint32(/* id 1, wireType 0 =*/8).bool(message.reverseScans);
                        if (message.mutateRowsRateLimit != null && Object.hasOwnProperty.call(message, "mutateRowsRateLimit"))
                            writer.uint32(/* id 3, wireType 0 =*/24).bool(message.mutateRowsRateLimit);
                        if (message.lastScannedRowResponses != null && Object.hasOwnProperty.call(message, "lastScannedRowResponses"))
                            writer.uint32(/* id 4, wireType 0 =*/32).bool(message.lastScannedRowResponses);
                        if (message.mutateRowsRateLimit2 != null && Object.hasOwnProperty.call(message, "mutateRowsRateLimit2"))
                            writer.uint32(/* id 5, wireType 0 =*/40).bool(message.mutateRowsRateLimit2);
                        if (message.routingCookie != null && Object.hasOwnProperty.call(message, "routingCookie"))
                            writer.uint32(/* id 6, wireType 0 =*/48).bool(message.routingCookie);
                        if (message.retryInfo != null && Object.hasOwnProperty.call(message, "retryInfo"))
                            writer.uint32(/* id 7, wireType 0 =*/56).bool(message.retryInfo);
                        if (message.clientSideMetricsEnabled != null && Object.hasOwnProperty.call(message, "clientSideMetricsEnabled"))
                            writer.uint32(/* id 8, wireType 0 =*/64).bool(message.clientSideMetricsEnabled);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified FeatureFlags message, length delimited. Does not implicitly {@link google.bigtable.v2.FeatureFlags.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.FeatureFlags
                     * @static
                     * @param {google.bigtable.v2.IFeatureFlags} message FeatureFlags message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    FeatureFlags.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a FeatureFlags message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.FeatureFlags
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.FeatureFlags} FeatureFlags
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    FeatureFlags.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.FeatureFlags();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.reverseScans = reader.bool();
                                    break;
                                }
                            case 3: {
                                    message.mutateRowsRateLimit = reader.bool();
                                    break;
                                }
                            case 5: {
                                    message.mutateRowsRateLimit2 = reader.bool();
                                    break;
                                }
                            case 4: {
                                    message.lastScannedRowResponses = reader.bool();
                                    break;
                                }
                            case 6: {
                                    message.routingCookie = reader.bool();
                                    break;
                                }
                            case 7: {
                                    message.retryInfo = reader.bool();
                                    break;
                                }
                            case 8: {
                                    message.clientSideMetricsEnabled = reader.bool();
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a FeatureFlags message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.FeatureFlags
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.FeatureFlags} FeatureFlags
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    FeatureFlags.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a FeatureFlags message.
                     * @function verify
                     * @memberof google.bigtable.v2.FeatureFlags
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    FeatureFlags.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.reverseScans != null && message.hasOwnProperty("reverseScans"))
                            if (typeof message.reverseScans !== "boolean")
                                return "reverseScans: boolean expected";
                        if (message.mutateRowsRateLimit != null && message.hasOwnProperty("mutateRowsRateLimit"))
                            if (typeof message.mutateRowsRateLimit !== "boolean")
                                return "mutateRowsRateLimit: boolean expected";
                        if (message.mutateRowsRateLimit2 != null && message.hasOwnProperty("mutateRowsRateLimit2"))
                            if (typeof message.mutateRowsRateLimit2 !== "boolean")
                                return "mutateRowsRateLimit2: boolean expected";
                        if (message.lastScannedRowResponses != null && message.hasOwnProperty("lastScannedRowResponses"))
                            if (typeof message.lastScannedRowResponses !== "boolean")
                                return "lastScannedRowResponses: boolean expected";
                        if (message.routingCookie != null && message.hasOwnProperty("routingCookie"))
                            if (typeof message.routingCookie !== "boolean")
                                return "routingCookie: boolean expected";
                        if (message.retryInfo != null && message.hasOwnProperty("retryInfo"))
                            if (typeof message.retryInfo !== "boolean")
                                return "retryInfo: boolean expected";
                        if (message.clientSideMetricsEnabled != null && message.hasOwnProperty("clientSideMetricsEnabled"))
                            if (typeof message.clientSideMetricsEnabled !== "boolean")
                                return "clientSideMetricsEnabled: boolean expected";
                        return null;
                    };
    
                    /**
                     * Creates a FeatureFlags message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.FeatureFlags
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.FeatureFlags} FeatureFlags
                     */
                    FeatureFlags.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.FeatureFlags)
                            return object;
                        var message = new $root.google.bigtable.v2.FeatureFlags();
                        if (object.reverseScans != null)
                            message.reverseScans = Boolean(object.reverseScans);
                        if (object.mutateRowsRateLimit != null)
                            message.mutateRowsRateLimit = Boolean(object.mutateRowsRateLimit);
                        if (object.mutateRowsRateLimit2 != null)
                            message.mutateRowsRateLimit2 = Boolean(object.mutateRowsRateLimit2);
                        if (object.lastScannedRowResponses != null)
                            message.lastScannedRowResponses = Boolean(object.lastScannedRowResponses);
                        if (object.routingCookie != null)
                            message.routingCookie = Boolean(object.routingCookie);
                        if (object.retryInfo != null)
                            message.retryInfo = Boolean(object.retryInfo);
                        if (object.clientSideMetricsEnabled != null)
                            message.clientSideMetricsEnabled = Boolean(object.clientSideMetricsEnabled);
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a FeatureFlags message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.FeatureFlags
                     * @static
                     * @param {google.bigtable.v2.FeatureFlags} message FeatureFlags
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    FeatureFlags.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults) {
                            object.reverseScans = false;
                            object.mutateRowsRateLimit = false;
                            object.lastScannedRowResponses = false;
                            object.mutateRowsRateLimit2 = false;
                            object.routingCookie = false;
                            object.retryInfo = false;
                            object.clientSideMetricsEnabled = false;
                        }
                        if (message.reverseScans != null && message.hasOwnProperty("reverseScans"))
                            object.reverseScans = message.reverseScans;
                        if (message.mutateRowsRateLimit != null && message.hasOwnProperty("mutateRowsRateLimit"))
                            object.mutateRowsRateLimit = message.mutateRowsRateLimit;
                        if (message.lastScannedRowResponses != null && message.hasOwnProperty("lastScannedRowResponses"))
                            object.lastScannedRowResponses = message.lastScannedRowResponses;
                        if (message.mutateRowsRateLimit2 != null && message.hasOwnProperty("mutateRowsRateLimit2"))
                            object.mutateRowsRateLimit2 = message.mutateRowsRateLimit2;
                        if (message.routingCookie != null && message.hasOwnProperty("routingCookie"))
                            object.routingCookie = message.routingCookie;
                        if (message.retryInfo != null && message.hasOwnProperty("retryInfo"))
                            object.retryInfo = message.retryInfo;
                        if (message.clientSideMetricsEnabled != null && message.hasOwnProperty("clientSideMetricsEnabled"))
                            object.clientSideMetricsEnabled = message.clientSideMetricsEnabled;
                        return object;
                    };
    
                    /**
                     * Converts this FeatureFlags to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.FeatureFlags
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    FeatureFlags.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for FeatureFlags
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.FeatureFlags
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    FeatureFlags.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.FeatureFlags";
                    };
    
                    return FeatureFlags;
                })();
    
                v2.ResponseParams = (function() {
    
                    /**
                     * Properties of a ResponseParams.
                     * @memberof google.bigtable.v2
                     * @interface IResponseParams
                     * @property {string|null} [zoneId] ResponseParams zoneId
                     * @property {string|null} [clusterId] ResponseParams clusterId
                     */
    
                    /**
                     * Constructs a new ResponseParams.
                     * @memberof google.bigtable.v2
                     * @classdesc Represents a ResponseParams.
                     * @implements IResponseParams
                     * @constructor
                     * @param {google.bigtable.v2.IResponseParams=} [properties] Properties to set
                     */
                    function ResponseParams(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * ResponseParams zoneId.
                     * @member {string|null|undefined} zoneId
                     * @memberof google.bigtable.v2.ResponseParams
                     * @instance
                     */
                    ResponseParams.prototype.zoneId = null;
    
                    /**
                     * ResponseParams clusterId.
                     * @member {string|null|undefined} clusterId
                     * @memberof google.bigtable.v2.ResponseParams
                     * @instance
                     */
                    ResponseParams.prototype.clusterId = null;
    
                    // OneOf field names bound to virtual getters and setters
                    var $oneOfFields;
    
                    /**
                     * ResponseParams _zoneId.
                     * @member {"zoneId"|undefined} _zoneId
                     * @memberof google.bigtable.v2.ResponseParams
                     * @instance
                     */
                    Object.defineProperty(ResponseParams.prototype, "_zoneId", {
                        get: $util.oneOfGetter($oneOfFields = ["zoneId"]),
                        set: $util.oneOfSetter($oneOfFields)
                    });
    
                    /**
                     * ResponseParams _clusterId.
                     * @member {"clusterId"|undefined} _clusterId
                     * @memberof google.bigtable.v2.ResponseParams
                     * @instance
                     */
                    Object.defineProperty(ResponseParams.prototype, "_clusterId", {
                        get: $util.oneOfGetter($oneOfFields = ["clusterId"]),
                        set: $util.oneOfSetter($oneOfFields)
                    });
    
                    /**
                     * Creates a new ResponseParams instance using the specified properties.
                     * @function create
                     * @memberof google.bigtable.v2.ResponseParams
                     * @static
                     * @param {google.bigtable.v2.IResponseParams=} [properties] Properties to set
                     * @returns {google.bigtable.v2.ResponseParams} ResponseParams instance
                     */
                    ResponseParams.create = function create(properties) {
                        return new ResponseParams(properties);
                    };
    
                    /**
                     * Encodes the specified ResponseParams message. Does not implicitly {@link google.bigtable.v2.ResponseParams.verify|verify} messages.
                     * @function encode
                     * @memberof google.bigtable.v2.ResponseParams
                     * @static
                     * @param {google.bigtable.v2.IResponseParams} message ResponseParams message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ResponseParams.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.zoneId != null && Object.hasOwnProperty.call(message, "zoneId"))
                            writer.uint32(/* id 1, wireType 2 =*/10).string(message.zoneId);
                        if (message.clusterId != null && Object.hasOwnProperty.call(message, "clusterId"))
                            writer.uint32(/* id 2, wireType 2 =*/18).string(message.clusterId);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified ResponseParams message, length delimited. Does not implicitly {@link google.bigtable.v2.ResponseParams.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.bigtable.v2.ResponseParams
                     * @static
                     * @param {google.bigtable.v2.IResponseParams} message ResponseParams message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ResponseParams.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a ResponseParams message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.bigtable.v2.ResponseParams
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.bigtable.v2.ResponseParams} ResponseParams
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ResponseParams.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.bigtable.v2.ResponseParams();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.zoneId = reader.string();
                                    break;
                                }
                            case 2: {
                                    message.clusterId = reader.string();
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a ResponseParams message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.bigtable.v2.ResponseParams
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.bigtable.v2.ResponseParams} ResponseParams
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ResponseParams.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a ResponseParams message.
                     * @function verify
                     * @memberof google.bigtable.v2.ResponseParams
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    ResponseParams.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        var properties = {};
                        if (message.zoneId != null && message.hasOwnProperty("zoneId")) {
                            properties._zoneId = 1;
                            if (!$util.isString(message.zoneId))
                                return "zoneId: string expected";
                        }
                        if (message.clusterId != null && message.hasOwnProperty("clusterId")) {
                            properties._clusterId = 1;
                            if (!$util.isString(message.clusterId))
                                return "clusterId: string expected";
                        }
                        return null;
                    };
    
                    /**
                     * Creates a ResponseParams message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.bigtable.v2.ResponseParams
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.bigtable.v2.ResponseParams} ResponseParams
                     */
                    ResponseParams.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.bigtable.v2.ResponseParams)
                            return object;
                        var message = new $root.google.bigtable.v2.ResponseParams();
                        if (object.zoneId != null)
                            message.zoneId = String(object.zoneId);
                        if (object.clusterId != null)
                            message.clusterId = String(object.clusterId);
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a ResponseParams message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.bigtable.v2.ResponseParams
                     * @static
                     * @param {google.bigtable.v2.ResponseParams} message ResponseParams
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    ResponseParams.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (message.zoneId != null && message.hasOwnProperty("zoneId")) {
                            object.zoneId = message.zoneId;
                            if (options.oneofs)
                                object._zoneId = "zoneId";
                        }
                        if (message.clusterId != null && message.hasOwnProperty("clusterId")) {
                            object.clusterId = message.clusterId;
                            if (options.oneofs)
                                object._clusterId = "clusterId";
                        }
                        return object;
                    };
    
                    /**
                     * Converts this ResponseParams to JSON.
                     * @function toJSON
                     * @memberof google.bigtable.v2.ResponseParams
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    ResponseParams.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for ResponseParams
                     * @function getTypeUrl
                     * @memberof google.bigtable.v2.ResponseParams
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    ResponseParams.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.bigtable.v2.ResponseParams";
                    };
    
                    return ResponseParams;
                })();
    
                return v2;
            })();
    
            return bigtable;
        })();
    
        google.api = (function() {
    
            /**
             * Namespace api.
             * @memberof google
             * @namespace
             */
            var api = {};
    
            api.Http = (function() {
    
                /**
                 * Properties of a Http.
                 * @memberof google.api
                 * @interface IHttp
                 * @property {Array.<google.api.IHttpRule>|null} [rules] Http rules
                 * @property {boolean|null} [fullyDecodeReservedExpansion] Http fullyDecodeReservedExpansion
                 */
    
                /**
                 * Constructs a new Http.
                 * @memberof google.api
                 * @classdesc Represents a Http.
                 * @implements IHttp
                 * @constructor
                 * @param {google.api.IHttp=} [properties] Properties to set
                 */
                function Http(properties) {
                    this.rules = [];
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * Http rules.
                 * @member {Array.<google.api.IHttpRule>} rules
                 * @memberof google.api.Http
                 * @instance
                 */
                Http.prototype.rules = $util.emptyArray;
    
                /**
                 * Http fullyDecodeReservedExpansion.
                 * @member {boolean} fullyDecodeReservedExpansion
                 * @memberof google.api.Http
                 * @instance
                 */
                Http.prototype.fullyDecodeReservedExpansion = false;
    
                /**
                 * Creates a new Http instance using the specified properties.
                 * @function create
                 * @memberof google.api.Http
                 * @static
                 * @param {google.api.IHttp=} [properties] Properties to set
                 * @returns {google.api.Http} Http instance
                 */
                Http.create = function create(properties) {
                    return new Http(properties);
                };
    
                /**
                 * Encodes the specified Http message. Does not implicitly {@link google.api.Http.verify|verify} messages.
                 * @function encode
                 * @memberof google.api.Http
                 * @static
                 * @param {google.api.IHttp} message Http message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Http.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.rules != null && message.rules.length)
                        for (var i = 0; i < message.rules.length; ++i)
                            $root.google.api.HttpRule.encode(message.rules[i], writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                    if (message.fullyDecodeReservedExpansion != null && Object.hasOwnProperty.call(message, "fullyDecodeReservedExpansion"))
                        writer.uint32(/* id 2, wireType 0 =*/16).bool(message.fullyDecodeReservedExpansion);
                    return writer;
                };
    
                /**
                 * Encodes the specified Http message, length delimited. Does not implicitly {@link google.api.Http.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.api.Http
                 * @static
                 * @param {google.api.IHttp} message Http message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Http.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a Http message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.api.Http
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.api.Http} Http
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Http.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.api.Http();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                if (!(message.rules && message.rules.length))
                                    message.rules = [];
                                message.rules.push($root.google.api.HttpRule.decode(reader, reader.uint32()));
                                break;
                            }
                        case 2: {
                                message.fullyDecodeReservedExpansion = reader.bool();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a Http message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.api.Http
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.api.Http} Http
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Http.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a Http message.
                 * @function verify
                 * @memberof google.api.Http
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                Http.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.rules != null && message.hasOwnProperty("rules")) {
                        if (!Array.isArray(message.rules))
                            return "rules: array expected";
                        for (var i = 0; i < message.rules.length; ++i) {
                            var error = $root.google.api.HttpRule.verify(message.rules[i]);
                            if (error)
                                return "rules." + error;
                        }
                    }
                    if (message.fullyDecodeReservedExpansion != null && message.hasOwnProperty("fullyDecodeReservedExpansion"))
                        if (typeof message.fullyDecodeReservedExpansion !== "boolean")
                            return "fullyDecodeReservedExpansion: boolean expected";
                    return null;
                };
    
                /**
                 * Creates a Http message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.api.Http
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.api.Http} Http
                 */
                Http.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.api.Http)
                        return object;
                    var message = new $root.google.api.Http();
                    if (object.rules) {
                        if (!Array.isArray(object.rules))
                            throw TypeError(".google.api.Http.rules: array expected");
                        message.rules = [];
                        for (var i = 0; i < object.rules.length; ++i) {
                            if (typeof object.rules[i] !== "object")
                                throw TypeError(".google.api.Http.rules: object expected");
                            message.rules[i] = $root.google.api.HttpRule.fromObject(object.rules[i]);
                        }
                    }
                    if (object.fullyDecodeReservedExpansion != null)
                        message.fullyDecodeReservedExpansion = Boolean(object.fullyDecodeReservedExpansion);
                    return message;
                };
    
                /**
                 * Creates a plain object from a Http message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.api.Http
                 * @static
                 * @param {google.api.Http} message Http
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                Http.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.arrays || options.defaults)
                        object.rules = [];
                    if (options.defaults)
                        object.fullyDecodeReservedExpansion = false;
                    if (message.rules && message.rules.length) {
                        object.rules = [];
                        for (var j = 0; j < message.rules.length; ++j)
                            object.rules[j] = $root.google.api.HttpRule.toObject(message.rules[j], options);
                    }
                    if (message.fullyDecodeReservedExpansion != null && message.hasOwnProperty("fullyDecodeReservedExpansion"))
                        object.fullyDecodeReservedExpansion = message.fullyDecodeReservedExpansion;
                    return object;
                };
    
                /**
                 * Converts this Http to JSON.
                 * @function toJSON
                 * @memberof google.api.Http
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                Http.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for Http
                 * @function getTypeUrl
                 * @memberof google.api.Http
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                Http.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.api.Http";
                };
    
                return Http;
            })();
    
            api.HttpRule = (function() {
    
                /**
                 * Properties of a HttpRule.
                 * @memberof google.api
                 * @interface IHttpRule
                 * @property {string|null} [selector] HttpRule selector
                 * @property {string|null} [get] HttpRule get
                 * @property {string|null} [put] HttpRule put
                 * @property {string|null} [post] HttpRule post
                 * @property {string|null} ["delete"] HttpRule delete
                 * @property {string|null} [patch] HttpRule patch
                 * @property {google.api.ICustomHttpPattern|null} [custom] HttpRule custom
                 * @property {string|null} [body] HttpRule body
                 * @property {string|null} [responseBody] HttpRule responseBody
                 * @property {Array.<google.api.IHttpRule>|null} [additionalBindings] HttpRule additionalBindings
                 */
    
                /**
                 * Constructs a new HttpRule.
                 * @memberof google.api
                 * @classdesc Represents a HttpRule.
                 * @implements IHttpRule
                 * @constructor
                 * @param {google.api.IHttpRule=} [properties] Properties to set
                 */
                function HttpRule(properties) {
                    this.additionalBindings = [];
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * HttpRule selector.
                 * @member {string} selector
                 * @memberof google.api.HttpRule
                 * @instance
                 */
                HttpRule.prototype.selector = "";
    
                /**
                 * HttpRule get.
                 * @member {string|null|undefined} get
                 * @memberof google.api.HttpRule
                 * @instance
                 */
                HttpRule.prototype.get = null;
    
                /**
                 * HttpRule put.
                 * @member {string|null|undefined} put
                 * @memberof google.api.HttpRule
                 * @instance
                 */
                HttpRule.prototype.put = null;
    
                /**
                 * HttpRule post.
                 * @member {string|null|undefined} post
                 * @memberof google.api.HttpRule
                 * @instance
                 */
                HttpRule.prototype.post = null;
    
                /**
                 * HttpRule delete.
                 * @member {string|null|undefined} delete
                 * @memberof google.api.HttpRule
                 * @instance
                 */
                HttpRule.prototype["delete"] = null;
    
                /**
                 * HttpRule patch.
                 * @member {string|null|undefined} patch
                 * @memberof google.api.HttpRule
                 * @instance
                 */
                HttpRule.prototype.patch = null;
    
                /**
                 * HttpRule custom.
                 * @member {google.api.ICustomHttpPattern|null|undefined} custom
                 * @memberof google.api.HttpRule
                 * @instance
                 */
                HttpRule.prototype.custom = null;
    
                /**
                 * HttpRule body.
                 * @member {string} body
                 * @memberof google.api.HttpRule
                 * @instance
                 */
                HttpRule.prototype.body = "";
    
                /**
                 * HttpRule responseBody.
                 * @member {string} responseBody
                 * @memberof google.api.HttpRule
                 * @instance
                 */
                HttpRule.prototype.responseBody = "";
    
                /**
                 * HttpRule additionalBindings.
                 * @member {Array.<google.api.IHttpRule>} additionalBindings
                 * @memberof google.api.HttpRule
                 * @instance
                 */
                HttpRule.prototype.additionalBindings = $util.emptyArray;
    
                // OneOf field names bound to virtual getters and setters
                var $oneOfFields;
    
                /**
                 * HttpRule pattern.
                 * @member {"get"|"put"|"post"|"delete"|"patch"|"custom"|undefined} pattern
                 * @memberof google.api.HttpRule
                 * @instance
                 */
                Object.defineProperty(HttpRule.prototype, "pattern", {
                    get: $util.oneOfGetter($oneOfFields = ["get", "put", "post", "delete", "patch", "custom"]),
                    set: $util.oneOfSetter($oneOfFields)
                });
    
                /**
                 * Creates a new HttpRule instance using the specified properties.
                 * @function create
                 * @memberof google.api.HttpRule
                 * @static
                 * @param {google.api.IHttpRule=} [properties] Properties to set
                 * @returns {google.api.HttpRule} HttpRule instance
                 */
                HttpRule.create = function create(properties) {
                    return new HttpRule(properties);
                };
    
                /**
                 * Encodes the specified HttpRule message. Does not implicitly {@link google.api.HttpRule.verify|verify} messages.
                 * @function encode
                 * @memberof google.api.HttpRule
                 * @static
                 * @param {google.api.IHttpRule} message HttpRule message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                HttpRule.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.selector != null && Object.hasOwnProperty.call(message, "selector"))
                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.selector);
                    if (message.get != null && Object.hasOwnProperty.call(message, "get"))
                        writer.uint32(/* id 2, wireType 2 =*/18).string(message.get);
                    if (message.put != null && Object.hasOwnProperty.call(message, "put"))
                        writer.uint32(/* id 3, wireType 2 =*/26).string(message.put);
                    if (message.post != null && Object.hasOwnProperty.call(message, "post"))
                        writer.uint32(/* id 4, wireType 2 =*/34).string(message.post);
                    if (message["delete"] != null && Object.hasOwnProperty.call(message, "delete"))
                        writer.uint32(/* id 5, wireType 2 =*/42).string(message["delete"]);
                    if (message.patch != null && Object.hasOwnProperty.call(message, "patch"))
                        writer.uint32(/* id 6, wireType 2 =*/50).string(message.patch);
                    if (message.body != null && Object.hasOwnProperty.call(message, "body"))
                        writer.uint32(/* id 7, wireType 2 =*/58).string(message.body);
                    if (message.custom != null && Object.hasOwnProperty.call(message, "custom"))
                        $root.google.api.CustomHttpPattern.encode(message.custom, writer.uint32(/* id 8, wireType 2 =*/66).fork()).ldelim();
                    if (message.additionalBindings != null && message.additionalBindings.length)
                        for (var i = 0; i < message.additionalBindings.length; ++i)
                            $root.google.api.HttpRule.encode(message.additionalBindings[i], writer.uint32(/* id 11, wireType 2 =*/90).fork()).ldelim();
                    if (message.responseBody != null && Object.hasOwnProperty.call(message, "responseBody"))
                        writer.uint32(/* id 12, wireType 2 =*/98).string(message.responseBody);
                    return writer;
                };
    
                /**
                 * Encodes the specified HttpRule message, length delimited. Does not implicitly {@link google.api.HttpRule.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.api.HttpRule
                 * @static
                 * @param {google.api.IHttpRule} message HttpRule message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                HttpRule.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a HttpRule message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.api.HttpRule
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.api.HttpRule} HttpRule
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                HttpRule.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.api.HttpRule();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.selector = reader.string();
                                break;
                            }
                        case 2: {
                                message.get = reader.string();
                                break;
                            }
                        case 3: {
                                message.put = reader.string();
                                break;
                            }
                        case 4: {
                                message.post = reader.string();
                                break;
                            }
                        case 5: {
                                message["delete"] = reader.string();
                                break;
                            }
                        case 6: {
                                message.patch = reader.string();
                                break;
                            }
                        case 8: {
                                message.custom = $root.google.api.CustomHttpPattern.decode(reader, reader.uint32());
                                break;
                            }
                        case 7: {
                                message.body = reader.string();
                                break;
                            }
                        case 12: {
                                message.responseBody = reader.string();
                                break;
                            }
                        case 11: {
                                if (!(message.additionalBindings && message.additionalBindings.length))
                                    message.additionalBindings = [];
                                message.additionalBindings.push($root.google.api.HttpRule.decode(reader, reader.uint32()));
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a HttpRule message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.api.HttpRule
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.api.HttpRule} HttpRule
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                HttpRule.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a HttpRule message.
                 * @function verify
                 * @memberof google.api.HttpRule
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                HttpRule.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    var properties = {};
                    if (message.selector != null && message.hasOwnProperty("selector"))
                        if (!$util.isString(message.selector))
                            return "selector: string expected";
                    if (message.get != null && message.hasOwnProperty("get")) {
                        properties.pattern = 1;
                        if (!$util.isString(message.get))
                            return "get: string expected";
                    }
                    if (message.put != null && message.hasOwnProperty("put")) {
                        if (properties.pattern === 1)
                            return "pattern: multiple values";
                        properties.pattern = 1;
                        if (!$util.isString(message.put))
                            return "put: string expected";
                    }
                    if (message.post != null && message.hasOwnProperty("post")) {
                        if (properties.pattern === 1)
                            return "pattern: multiple values";
                        properties.pattern = 1;
                        if (!$util.isString(message.post))
                            return "post: string expected";
                    }
                    if (message["delete"] != null && message.hasOwnProperty("delete")) {
                        if (properties.pattern === 1)
                            return "pattern: multiple values";
                        properties.pattern = 1;
                        if (!$util.isString(message["delete"]))
                            return "delete: string expected";
                    }
                    if (message.patch != null && message.hasOwnProperty("patch")) {
                        if (properties.pattern === 1)
                            return "pattern: multiple values";
                        properties.pattern = 1;
                        if (!$util.isString(message.patch))
                            return "patch: string expected";
                    }
                    if (message.custom != null && message.hasOwnProperty("custom")) {
                        if (properties.pattern === 1)
                            return "pattern: multiple values";
                        properties.pattern = 1;
                        {
                            var error = $root.google.api.CustomHttpPattern.verify(message.custom);
                            if (error)
                                return "custom." + error;
                        }
                    }
                    if (message.body != null && message.hasOwnProperty("body"))
                        if (!$util.isString(message.body))
                            return "body: string expected";
                    if (message.responseBody != null && message.hasOwnProperty("responseBody"))
                        if (!$util.isString(message.responseBody))
                            return "responseBody: string expected";
                    if (message.additionalBindings != null && message.hasOwnProperty("additionalBindings")) {
                        if (!Array.isArray(message.additionalBindings))
                            return "additionalBindings: array expected";
                        for (var i = 0; i < message.additionalBindings.length; ++i) {
                            var error = $root.google.api.HttpRule.verify(message.additionalBindings[i]);
                            if (error)
                                return "additionalBindings." + error;
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates a HttpRule message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.api.HttpRule
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.api.HttpRule} HttpRule
                 */
                HttpRule.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.api.HttpRule)
                        return object;
                    var message = new $root.google.api.HttpRule();
                    if (object.selector != null)
                        message.selector = String(object.selector);
                    if (object.get != null)
                        message.get = String(object.get);
                    if (object.put != null)
                        message.put = String(object.put);
                    if (object.post != null)
                        message.post = String(object.post);
                    if (object["delete"] != null)
                        message["delete"] = String(object["delete"]);
                    if (object.patch != null)
                        message.patch = String(object.patch);
                    if (object.custom != null) {
                        if (typeof object.custom !== "object")
                            throw TypeError(".google.api.HttpRule.custom: object expected");
                        message.custom = $root.google.api.CustomHttpPattern.fromObject(object.custom);
                    }
                    if (object.body != null)
                        message.body = String(object.body);
                    if (object.responseBody != null)
                        message.responseBody = String(object.responseBody);
                    if (object.additionalBindings) {
                        if (!Array.isArray(object.additionalBindings))
                            throw TypeError(".google.api.HttpRule.additionalBindings: array expected");
                        message.additionalBindings = [];
                        for (var i = 0; i < object.additionalBindings.length; ++i) {
                            if (typeof object.additionalBindings[i] !== "object")
                                throw TypeError(".google.api.HttpRule.additionalBindings: object expected");
                            message.additionalBindings[i] = $root.google.api.HttpRule.fromObject(object.additionalBindings[i]);
                        }
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from a HttpRule message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.api.HttpRule
                 * @static
                 * @param {google.api.HttpRule} message HttpRule
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                HttpRule.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.arrays || options.defaults)
                        object.additionalBindings = [];
                    if (options.defaults) {
                        object.selector = "";
                        object.body = "";
                        object.responseBody = "";
                    }
                    if (message.selector != null && message.hasOwnProperty("selector"))
                        object.selector = message.selector;
                    if (message.get != null && message.hasOwnProperty("get")) {
                        object.get = message.get;
                        if (options.oneofs)
                            object.pattern = "get";
                    }
                    if (message.put != null && message.hasOwnProperty("put")) {
                        object.put = message.put;
                        if (options.oneofs)
                            object.pattern = "put";
                    }
                    if (message.post != null && message.hasOwnProperty("post")) {
                        object.post = message.post;
                        if (options.oneofs)
                            object.pattern = "post";
                    }
                    if (message["delete"] != null && message.hasOwnProperty("delete")) {
                        object["delete"] = message["delete"];
                        if (options.oneofs)
                            object.pattern = "delete";
                    }
                    if (message.patch != null && message.hasOwnProperty("patch")) {
                        object.patch = message.patch;
                        if (options.oneofs)
                            object.pattern = "patch";
                    }
                    if (message.body != null && message.hasOwnProperty("body"))
                        object.body = message.body;
                    if (message.custom != null && message.hasOwnProperty("custom")) {
                        object.custom = $root.google.api.CustomHttpPattern.toObject(message.custom, options);
                        if (options.oneofs)
                            object.pattern = "custom";
                    }
                    if (message.additionalBindings && message.additionalBindings.length) {
                        object.additionalBindings = [];
                        for (var j = 0; j < message.additionalBindings.length; ++j)
                            object.additionalBindings[j] = $root.google.api.HttpRule.toObject(message.additionalBindings[j], options);
                    }
                    if (message.responseBody != null && message.hasOwnProperty("responseBody"))
                        object.responseBody = message.responseBody;
                    return object;
                };
    
                /**
                 * Converts this HttpRule to JSON.
                 * @function toJSON
                 * @memberof google.api.HttpRule
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                HttpRule.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for HttpRule
                 * @function getTypeUrl
                 * @memberof google.api.HttpRule
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                HttpRule.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.api.HttpRule";
                };
    
                return HttpRule;
            })();
    
            api.CustomHttpPattern = (function() {
    
                /**
                 * Properties of a CustomHttpPattern.
                 * @memberof google.api
                 * @interface ICustomHttpPattern
                 * @property {string|null} [kind] CustomHttpPattern kind
                 * @property {string|null} [path] CustomHttpPattern path
                 */
    
                /**
                 * Constructs a new CustomHttpPattern.
                 * @memberof google.api
                 * @classdesc Represents a CustomHttpPattern.
                 * @implements ICustomHttpPattern
                 * @constructor
                 * @param {google.api.ICustomHttpPattern=} [properties] Properties to set
                 */
                function CustomHttpPattern(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * CustomHttpPattern kind.
                 * @member {string} kind
                 * @memberof google.api.CustomHttpPattern
                 * @instance
                 */
                CustomHttpPattern.prototype.kind = "";
    
                /**
                 * CustomHttpPattern path.
                 * @member {string} path
                 * @memberof google.api.CustomHttpPattern
                 * @instance
                 */
                CustomHttpPattern.prototype.path = "";
    
                /**
                 * Creates a new CustomHttpPattern instance using the specified properties.
                 * @function create
                 * @memberof google.api.CustomHttpPattern
                 * @static
                 * @param {google.api.ICustomHttpPattern=} [properties] Properties to set
                 * @returns {google.api.CustomHttpPattern} CustomHttpPattern instance
                 */
                CustomHttpPattern.create = function create(properties) {
                    return new CustomHttpPattern(properties);
                };
    
                /**
                 * Encodes the specified CustomHttpPattern message. Does not implicitly {@link google.api.CustomHttpPattern.verify|verify} messages.
                 * @function encode
                 * @memberof google.api.CustomHttpPattern
                 * @static
                 * @param {google.api.ICustomHttpPattern} message CustomHttpPattern message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                CustomHttpPattern.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.kind != null && Object.hasOwnProperty.call(message, "kind"))
                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.kind);
                    if (message.path != null && Object.hasOwnProperty.call(message, "path"))
                        writer.uint32(/* id 2, wireType 2 =*/18).string(message.path);
                    return writer;
                };
    
                /**
                 * Encodes the specified CustomHttpPattern message, length delimited. Does not implicitly {@link google.api.CustomHttpPattern.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.api.CustomHttpPattern
                 * @static
                 * @param {google.api.ICustomHttpPattern} message CustomHttpPattern message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                CustomHttpPattern.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a CustomHttpPattern message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.api.CustomHttpPattern
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.api.CustomHttpPattern} CustomHttpPattern
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                CustomHttpPattern.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.api.CustomHttpPattern();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.kind = reader.string();
                                break;
                            }
                        case 2: {
                                message.path = reader.string();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a CustomHttpPattern message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.api.CustomHttpPattern
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.api.CustomHttpPattern} CustomHttpPattern
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                CustomHttpPattern.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a CustomHttpPattern message.
                 * @function verify
                 * @memberof google.api.CustomHttpPattern
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                CustomHttpPattern.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.kind != null && message.hasOwnProperty("kind"))
                        if (!$util.isString(message.kind))
                            return "kind: string expected";
                    if (message.path != null && message.hasOwnProperty("path"))
                        if (!$util.isString(message.path))
                            return "path: string expected";
                    return null;
                };
    
                /**
                 * Creates a CustomHttpPattern message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.api.CustomHttpPattern
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.api.CustomHttpPattern} CustomHttpPattern
                 */
                CustomHttpPattern.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.api.CustomHttpPattern)
                        return object;
                    var message = new $root.google.api.CustomHttpPattern();
                    if (object.kind != null)
                        message.kind = String(object.kind);
                    if (object.path != null)
                        message.path = String(object.path);
                    return message;
                };
    
                /**
                 * Creates a plain object from a CustomHttpPattern message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.api.CustomHttpPattern
                 * @static
                 * @param {google.api.CustomHttpPattern} message CustomHttpPattern
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                CustomHttpPattern.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults) {
                        object.kind = "";
                        object.path = "";
                    }
                    if (message.kind != null && message.hasOwnProperty("kind"))
                        object.kind = message.kind;
                    if (message.path != null && message.hasOwnProperty("path"))
                        object.path = message.path;
                    return object;
                };
    
                /**
                 * Converts this CustomHttpPattern to JSON.
                 * @function toJSON
                 * @memberof google.api.CustomHttpPattern
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                CustomHttpPattern.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for CustomHttpPattern
                 * @function getTypeUrl
                 * @memberof google.api.CustomHttpPattern
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                CustomHttpPattern.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.api.CustomHttpPattern";
                };
    
                return CustomHttpPattern;
            })();
    
            api.CommonLanguageSettings = (function() {
    
                /**
                 * Properties of a CommonLanguageSettings.
                 * @memberof google.api
                 * @interface ICommonLanguageSettings
                 * @property {string|null} [referenceDocsUri] CommonLanguageSettings referenceDocsUri
                 * @property {Array.<google.api.ClientLibraryDestination>|null} [destinations] CommonLanguageSettings destinations
                 */
    
                /**
                 * Constructs a new CommonLanguageSettings.
                 * @memberof google.api
                 * @classdesc Represents a CommonLanguageSettings.
                 * @implements ICommonLanguageSettings
                 * @constructor
                 * @param {google.api.ICommonLanguageSettings=} [properties] Properties to set
                 */
                function CommonLanguageSettings(properties) {
                    this.destinations = [];
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * CommonLanguageSettings referenceDocsUri.
                 * @member {string} referenceDocsUri
                 * @memberof google.api.CommonLanguageSettings
                 * @instance
                 */
                CommonLanguageSettings.prototype.referenceDocsUri = "";
    
                /**
                 * CommonLanguageSettings destinations.
                 * @member {Array.<google.api.ClientLibraryDestination>} destinations
                 * @memberof google.api.CommonLanguageSettings
                 * @instance
                 */
                CommonLanguageSettings.prototype.destinations = $util.emptyArray;
    
                /**
                 * Creates a new CommonLanguageSettings instance using the specified properties.
                 * @function create
                 * @memberof google.api.CommonLanguageSettings
                 * @static
                 * @param {google.api.ICommonLanguageSettings=} [properties] Properties to set
                 * @returns {google.api.CommonLanguageSettings} CommonLanguageSettings instance
                 */
                CommonLanguageSettings.create = function create(properties) {
                    return new CommonLanguageSettings(properties);
                };
    
                /**
                 * Encodes the specified CommonLanguageSettings message. Does not implicitly {@link google.api.CommonLanguageSettings.verify|verify} messages.
                 * @function encode
                 * @memberof google.api.CommonLanguageSettings
                 * @static
                 * @param {google.api.ICommonLanguageSettings} message CommonLanguageSettings message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                CommonLanguageSettings.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.referenceDocsUri != null && Object.hasOwnProperty.call(message, "referenceDocsUri"))
                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.referenceDocsUri);
                    if (message.destinations != null && message.destinations.length) {
                        writer.uint32(/* id 2, wireType 2 =*/18).fork();
                        for (var i = 0; i < message.destinations.length; ++i)
                            writer.int32(message.destinations[i]);
                        writer.ldelim();
                    }
                    return writer;
                };
    
                /**
                 * Encodes the specified CommonLanguageSettings message, length delimited. Does not implicitly {@link google.api.CommonLanguageSettings.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.api.CommonLanguageSettings
                 * @static
                 * @param {google.api.ICommonLanguageSettings} message CommonLanguageSettings message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                CommonLanguageSettings.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a CommonLanguageSettings message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.api.CommonLanguageSettings
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.api.CommonLanguageSettings} CommonLanguageSettings
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                CommonLanguageSettings.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.api.CommonLanguageSettings();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.referenceDocsUri = reader.string();
                                break;
                            }
                        case 2: {
                                if (!(message.destinations && message.destinations.length))
                                    message.destinations = [];
                                if ((tag & 7) === 2) {
                                    var end2 = reader.uint32() + reader.pos;
                                    while (reader.pos < end2)
                                        message.destinations.push(reader.int32());
                                } else
                                    message.destinations.push(reader.int32());
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a CommonLanguageSettings message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.api.CommonLanguageSettings
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.api.CommonLanguageSettings} CommonLanguageSettings
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                CommonLanguageSettings.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a CommonLanguageSettings message.
                 * @function verify
                 * @memberof google.api.CommonLanguageSettings
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                CommonLanguageSettings.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.referenceDocsUri != null && message.hasOwnProperty("referenceDocsUri"))
                        if (!$util.isString(message.referenceDocsUri))
                            return "referenceDocsUri: string expected";
                    if (message.destinations != null && message.hasOwnProperty("destinations")) {
                        if (!Array.isArray(message.destinations))
                            return "destinations: array expected";
                        for (var i = 0; i < message.destinations.length; ++i)
                            switch (message.destinations[i]) {
                            default:
                                return "destinations: enum value[] expected";
                            case 0:
                            case 10:
                            case 20:
                                break;
                            }
                    }
                    return null;
                };
    
                /**
                 * Creates a CommonLanguageSettings message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.api.CommonLanguageSettings
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.api.CommonLanguageSettings} CommonLanguageSettings
                 */
                CommonLanguageSettings.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.api.CommonLanguageSettings)
                        return object;
                    var message = new $root.google.api.CommonLanguageSettings();
                    if (object.referenceDocsUri != null)
                        message.referenceDocsUri = String(object.referenceDocsUri);
                    if (object.destinations) {
                        if (!Array.isArray(object.destinations))
                            throw TypeError(".google.api.CommonLanguageSettings.destinations: array expected");
                        message.destinations = [];
                        for (var i = 0; i < object.destinations.length; ++i)
                            switch (object.destinations[i]) {
                            default:
                                if (typeof object.destinations[i] === "number") {
                                    message.destinations[i] = object.destinations[i];
                                    break;
                                }
                            case "CLIENT_LIBRARY_DESTINATION_UNSPECIFIED":
                            case 0:
                                message.destinations[i] = 0;
                                break;
                            case "GITHUB":
                            case 10:
                                message.destinations[i] = 10;
                                break;
                            case "PACKAGE_MANAGER":
                            case 20:
                                message.destinations[i] = 20;
                                break;
                            }
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from a CommonLanguageSettings message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.api.CommonLanguageSettings
                 * @static
                 * @param {google.api.CommonLanguageSettings} message CommonLanguageSettings
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                CommonLanguageSettings.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.arrays || options.defaults)
                        object.destinations = [];
                    if (options.defaults)
                        object.referenceDocsUri = "";
                    if (message.referenceDocsUri != null && message.hasOwnProperty("referenceDocsUri"))
                        object.referenceDocsUri = message.referenceDocsUri;
                    if (message.destinations && message.destinations.length) {
                        object.destinations = [];
                        for (var j = 0; j < message.destinations.length; ++j)
                            object.destinations[j] = options.enums === String ? $root.google.api.ClientLibraryDestination[message.destinations[j]] === undefined ? message.destinations[j] : $root.google.api.ClientLibraryDestination[message.destinations[j]] : message.destinations[j];
                    }
                    return object;
                };
    
                /**
                 * Converts this CommonLanguageSettings to JSON.
                 * @function toJSON
                 * @memberof google.api.CommonLanguageSettings
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                CommonLanguageSettings.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for CommonLanguageSettings
                 * @function getTypeUrl
                 * @memberof google.api.CommonLanguageSettings
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                CommonLanguageSettings.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.api.CommonLanguageSettings";
                };
    
                return CommonLanguageSettings;
            })();
    
            api.ClientLibrarySettings = (function() {
    
                /**
                 * Properties of a ClientLibrarySettings.
                 * @memberof google.api
                 * @interface IClientLibrarySettings
                 * @property {string|null} [version] ClientLibrarySettings version
                 * @property {google.api.LaunchStage|null} [launchStage] ClientLibrarySettings launchStage
                 * @property {boolean|null} [restNumericEnums] ClientLibrarySettings restNumericEnums
                 * @property {google.api.IJavaSettings|null} [javaSettings] ClientLibrarySettings javaSettings
                 * @property {google.api.ICppSettings|null} [cppSettings] ClientLibrarySettings cppSettings
                 * @property {google.api.IPhpSettings|null} [phpSettings] ClientLibrarySettings phpSettings
                 * @property {google.api.IPythonSettings|null} [pythonSettings] ClientLibrarySettings pythonSettings
                 * @property {google.api.INodeSettings|null} [nodeSettings] ClientLibrarySettings nodeSettings
                 * @property {google.api.IDotnetSettings|null} [dotnetSettings] ClientLibrarySettings dotnetSettings
                 * @property {google.api.IRubySettings|null} [rubySettings] ClientLibrarySettings rubySettings
                 * @property {google.api.IGoSettings|null} [goSettings] ClientLibrarySettings goSettings
                 */
    
                /**
                 * Constructs a new ClientLibrarySettings.
                 * @memberof google.api
                 * @classdesc Represents a ClientLibrarySettings.
                 * @implements IClientLibrarySettings
                 * @constructor
                 * @param {google.api.IClientLibrarySettings=} [properties] Properties to set
                 */
                function ClientLibrarySettings(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * ClientLibrarySettings version.
                 * @member {string} version
                 * @memberof google.api.ClientLibrarySettings
                 * @instance
                 */
                ClientLibrarySettings.prototype.version = "";
    
                /**
                 * ClientLibrarySettings launchStage.
                 * @member {google.api.LaunchStage} launchStage
                 * @memberof google.api.ClientLibrarySettings
                 * @instance
                 */
                ClientLibrarySettings.prototype.launchStage = 0;
    
                /**
                 * ClientLibrarySettings restNumericEnums.
                 * @member {boolean} restNumericEnums
                 * @memberof google.api.ClientLibrarySettings
                 * @instance
                 */
                ClientLibrarySettings.prototype.restNumericEnums = false;
    
                /**
                 * ClientLibrarySettings javaSettings.
                 * @member {google.api.IJavaSettings|null|undefined} javaSettings
                 * @memberof google.api.ClientLibrarySettings
                 * @instance
                 */
                ClientLibrarySettings.prototype.javaSettings = null;
    
                /**
                 * ClientLibrarySettings cppSettings.
                 * @member {google.api.ICppSettings|null|undefined} cppSettings
                 * @memberof google.api.ClientLibrarySettings
                 * @instance
                 */
                ClientLibrarySettings.prototype.cppSettings = null;
    
                /**
                 * ClientLibrarySettings phpSettings.
                 * @member {google.api.IPhpSettings|null|undefined} phpSettings
                 * @memberof google.api.ClientLibrarySettings
                 * @instance
                 */
                ClientLibrarySettings.prototype.phpSettings = null;
    
                /**
                 * ClientLibrarySettings pythonSettings.
                 * @member {google.api.IPythonSettings|null|undefined} pythonSettings
                 * @memberof google.api.ClientLibrarySettings
                 * @instance
                 */
                ClientLibrarySettings.prototype.pythonSettings = null;
    
                /**
                 * ClientLibrarySettings nodeSettings.
                 * @member {google.api.INodeSettings|null|undefined} nodeSettings
                 * @memberof google.api.ClientLibrarySettings
                 * @instance
                 */
                ClientLibrarySettings.prototype.nodeSettings = null;
    
                /**
                 * ClientLibrarySettings dotnetSettings.
                 * @member {google.api.IDotnetSettings|null|undefined} dotnetSettings
                 * @memberof google.api.ClientLibrarySettings
                 * @instance
                 */
                ClientLibrarySettings.prototype.dotnetSettings = null;
    
                /**
                 * ClientLibrarySettings rubySettings.
                 * @member {google.api.IRubySettings|null|undefined} rubySettings
                 * @memberof google.api.ClientLibrarySettings
                 * @instance
                 */
                ClientLibrarySettings.prototype.rubySettings = null;
    
                /**
                 * ClientLibrarySettings goSettings.
                 * @member {google.api.IGoSettings|null|undefined} goSettings
                 * @memberof google.api.ClientLibrarySettings
                 * @instance
                 */
                ClientLibrarySettings.prototype.goSettings = null;
    
                /**
                 * Creates a new ClientLibrarySettings instance using the specified properties.
                 * @function create
                 * @memberof google.api.ClientLibrarySettings
                 * @static
                 * @param {google.api.IClientLibrarySettings=} [properties] Properties to set
                 * @returns {google.api.ClientLibrarySettings} ClientLibrarySettings instance
                 */
                ClientLibrarySettings.create = function create(properties) {
                    return new ClientLibrarySettings(properties);
                };
    
                /**
                 * Encodes the specified ClientLibrarySettings message. Does not implicitly {@link google.api.ClientLibrarySettings.verify|verify} messages.
                 * @function encode
                 * @memberof google.api.ClientLibrarySettings
                 * @static
                 * @param {google.api.IClientLibrarySettings} message ClientLibrarySettings message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ClientLibrarySettings.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.version != null && Object.hasOwnProperty.call(message, "version"))
                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.version);
                    if (message.launchStage != null && Object.hasOwnProperty.call(message, "launchStage"))
                        writer.uint32(/* id 2, wireType 0 =*/16).int32(message.launchStage);
                    if (message.restNumericEnums != null && Object.hasOwnProperty.call(message, "restNumericEnums"))
                        writer.uint32(/* id 3, wireType 0 =*/24).bool(message.restNumericEnums);
                    if (message.javaSettings != null && Object.hasOwnProperty.call(message, "javaSettings"))
                        $root.google.api.JavaSettings.encode(message.javaSettings, writer.uint32(/* id 21, wireType 2 =*/170).fork()).ldelim();
                    if (message.cppSettings != null && Object.hasOwnProperty.call(message, "cppSettings"))
                        $root.google.api.CppSettings.encode(message.cppSettings, writer.uint32(/* id 22, wireType 2 =*/178).fork()).ldelim();
                    if (message.phpSettings != null && Object.hasOwnProperty.call(message, "phpSettings"))
                        $root.google.api.PhpSettings.encode(message.phpSettings, writer.uint32(/* id 23, wireType 2 =*/186).fork()).ldelim();
                    if (message.pythonSettings != null && Object.hasOwnProperty.call(message, "pythonSettings"))
                        $root.google.api.PythonSettings.encode(message.pythonSettings, writer.uint32(/* id 24, wireType 2 =*/194).fork()).ldelim();
                    if (message.nodeSettings != null && Object.hasOwnProperty.call(message, "nodeSettings"))
                        $root.google.api.NodeSettings.encode(message.nodeSettings, writer.uint32(/* id 25, wireType 2 =*/202).fork()).ldelim();
                    if (message.dotnetSettings != null && Object.hasOwnProperty.call(message, "dotnetSettings"))
                        $root.google.api.DotnetSettings.encode(message.dotnetSettings, writer.uint32(/* id 26, wireType 2 =*/210).fork()).ldelim();
                    if (message.rubySettings != null && Object.hasOwnProperty.call(message, "rubySettings"))
                        $root.google.api.RubySettings.encode(message.rubySettings, writer.uint32(/* id 27, wireType 2 =*/218).fork()).ldelim();
                    if (message.goSettings != null && Object.hasOwnProperty.call(message, "goSettings"))
                        $root.google.api.GoSettings.encode(message.goSettings, writer.uint32(/* id 28, wireType 2 =*/226).fork()).ldelim();
                    return writer;
                };
    
                /**
                 * Encodes the specified ClientLibrarySettings message, length delimited. Does not implicitly {@link google.api.ClientLibrarySettings.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.api.ClientLibrarySettings
                 * @static
                 * @param {google.api.IClientLibrarySettings} message ClientLibrarySettings message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ClientLibrarySettings.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a ClientLibrarySettings message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.api.ClientLibrarySettings
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.api.ClientLibrarySettings} ClientLibrarySettings
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ClientLibrarySettings.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.api.ClientLibrarySettings();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.version = reader.string();
                                break;
                            }
                        case 2: {
                                message.launchStage = reader.int32();
                                break;
                            }
                        case 3: {
                                message.restNumericEnums = reader.bool();
                                break;
                            }
                        case 21: {
                                message.javaSettings = $root.google.api.JavaSettings.decode(reader, reader.uint32());
                                break;
                            }
                        case 22: {
                                message.cppSettings = $root.google.api.CppSettings.decode(reader, reader.uint32());
                                break;
                            }
                        case 23: {
                                message.phpSettings = $root.google.api.PhpSettings.decode(reader, reader.uint32());
                                break;
                            }
                        case 24: {
                                message.pythonSettings = $root.google.api.PythonSettings.decode(reader, reader.uint32());
                                break;
                            }
                        case 25: {
                                message.nodeSettings = $root.google.api.NodeSettings.decode(reader, reader.uint32());
                                break;
                            }
                        case 26: {
                                message.dotnetSettings = $root.google.api.DotnetSettings.decode(reader, reader.uint32());
                                break;
                            }
                        case 27: {
                                message.rubySettings = $root.google.api.RubySettings.decode(reader, reader.uint32());
                                break;
                            }
                        case 28: {
                                message.goSettings = $root.google.api.GoSettings.decode(reader, reader.uint32());
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a ClientLibrarySettings message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.api.ClientLibrarySettings
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.api.ClientLibrarySettings} ClientLibrarySettings
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ClientLibrarySettings.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a ClientLibrarySettings message.
                 * @function verify
                 * @memberof google.api.ClientLibrarySettings
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                ClientLibrarySettings.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.version != null && message.hasOwnProperty("version"))
                        if (!$util.isString(message.version))
                            return "version: string expected";
                    if (message.launchStage != null && message.hasOwnProperty("launchStage"))
                        switch (message.launchStage) {
                        default:
                            return "launchStage: enum value expected";
                        case 0:
                        case 6:
                        case 7:
                        case 1:
                        case 2:
                        case 3:
                        case 4:
                        case 5:
                            break;
                        }
                    if (message.restNumericEnums != null && message.hasOwnProperty("restNumericEnums"))
                        if (typeof message.restNumericEnums !== "boolean")
                            return "restNumericEnums: boolean expected";
                    if (message.javaSettings != null && message.hasOwnProperty("javaSettings")) {
                        var error = $root.google.api.JavaSettings.verify(message.javaSettings);
                        if (error)
                            return "javaSettings." + error;
                    }
                    if (message.cppSettings != null && message.hasOwnProperty("cppSettings")) {
                        var error = $root.google.api.CppSettings.verify(message.cppSettings);
                        if (error)
                            return "cppSettings." + error;
                    }
                    if (message.phpSettings != null && message.hasOwnProperty("phpSettings")) {
                        var error = $root.google.api.PhpSettings.verify(message.phpSettings);
                        if (error)
                            return "phpSettings." + error;
                    }
                    if (message.pythonSettings != null && message.hasOwnProperty("pythonSettings")) {
                        var error = $root.google.api.PythonSettings.verify(message.pythonSettings);
                        if (error)
                            return "pythonSettings." + error;
                    }
                    if (message.nodeSettings != null && message.hasOwnProperty("nodeSettings")) {
                        var error = $root.google.api.NodeSettings.verify(message.nodeSettings);
                        if (error)
                            return "nodeSettings." + error;
                    }
                    if (message.dotnetSettings != null && message.hasOwnProperty("dotnetSettings")) {
                        var error = $root.google.api.DotnetSettings.verify(message.dotnetSettings);
                        if (error)
                            return "dotnetSettings." + error;
                    }
                    if (message.rubySettings != null && message.hasOwnProperty("rubySettings")) {
                        var error = $root.google.api.RubySettings.verify(message.rubySettings);
                        if (error)
                            return "rubySettings." + error;
                    }
                    if (message.goSettings != null && message.hasOwnProperty("goSettings")) {
                        var error = $root.google.api.GoSettings.verify(message.goSettings);
                        if (error)
                            return "goSettings." + error;
                    }
                    return null;
                };
    
                /**
                 * Creates a ClientLibrarySettings message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.api.ClientLibrarySettings
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.api.ClientLibrarySettings} ClientLibrarySettings
                 */
                ClientLibrarySettings.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.api.ClientLibrarySettings)
                        return object;
                    var message = new $root.google.api.ClientLibrarySettings();
                    if (object.version != null)
                        message.version = String(object.version);
                    switch (object.launchStage) {
                    default:
                        if (typeof object.launchStage === "number") {
                            message.launchStage = object.launchStage;
                            break;
                        }
                        break;
                    case "LAUNCH_STAGE_UNSPECIFIED":
                    case 0:
                        message.launchStage = 0;
                        break;
                    case "UNIMPLEMENTED":
                    case 6:
                        message.launchStage = 6;
                        break;
                    case "PRELAUNCH":
                    case 7:
                        message.launchStage = 7;
                        break;
                    case "EARLY_ACCESS":
                    case 1:
                        message.launchStage = 1;
                        break;
                    case "ALPHA":
                    case 2:
                        message.launchStage = 2;
                        break;
                    case "BETA":
                    case 3:
                        message.launchStage = 3;
                        break;
                    case "GA":
                    case 4:
                        message.launchStage = 4;
                        break;
                    case "DEPRECATED":
                    case 5:
                        message.launchStage = 5;
                        break;
                    }
                    if (object.restNumericEnums != null)
                        message.restNumericEnums = Boolean(object.restNumericEnums);
                    if (object.javaSettings != null) {
                        if (typeof object.javaSettings !== "object")
                            throw TypeError(".google.api.ClientLibrarySettings.javaSettings: object expected");
                        message.javaSettings = $root.google.api.JavaSettings.fromObject(object.javaSettings);
                    }
                    if (object.cppSettings != null) {
                        if (typeof object.cppSettings !== "object")
                            throw TypeError(".google.api.ClientLibrarySettings.cppSettings: object expected");
                        message.cppSettings = $root.google.api.CppSettings.fromObject(object.cppSettings);
                    }
                    if (object.phpSettings != null) {
                        if (typeof object.phpSettings !== "object")
                            throw TypeError(".google.api.ClientLibrarySettings.phpSettings: object expected");
                        message.phpSettings = $root.google.api.PhpSettings.fromObject(object.phpSettings);
                    }
                    if (object.pythonSettings != null) {
                        if (typeof object.pythonSettings !== "object")
                            throw TypeError(".google.api.ClientLibrarySettings.pythonSettings: object expected");
                        message.pythonSettings = $root.google.api.PythonSettings.fromObject(object.pythonSettings);
                    }
                    if (object.nodeSettings != null) {
                        if (typeof object.nodeSettings !== "object")
                            throw TypeError(".google.api.ClientLibrarySettings.nodeSettings: object expected");
                        message.nodeSettings = $root.google.api.NodeSettings.fromObject(object.nodeSettings);
                    }
                    if (object.dotnetSettings != null) {
                        if (typeof object.dotnetSettings !== "object")
                            throw TypeError(".google.api.ClientLibrarySettings.dotnetSettings: object expected");
                        message.dotnetSettings = $root.google.api.DotnetSettings.fromObject(object.dotnetSettings);
                    }
                    if (object.rubySettings != null) {
                        if (typeof object.rubySettings !== "object")
                            throw TypeError(".google.api.ClientLibrarySettings.rubySettings: object expected");
                        message.rubySettings = $root.google.api.RubySettings.fromObject(object.rubySettings);
                    }
                    if (object.goSettings != null) {
                        if (typeof object.goSettings !== "object")
                            throw TypeError(".google.api.ClientLibrarySettings.goSettings: object expected");
                        message.goSettings = $root.google.api.GoSettings.fromObject(object.goSettings);
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from a ClientLibrarySettings message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.api.ClientLibrarySettings
                 * @static
                 * @param {google.api.ClientLibrarySettings} message ClientLibrarySettings
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                ClientLibrarySettings.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults) {
                        object.version = "";
                        object.launchStage = options.enums === String ? "LAUNCH_STAGE_UNSPECIFIED" : 0;
                        object.restNumericEnums = false;
                        object.javaSettings = null;
                        object.cppSettings = null;
                        object.phpSettings = null;
                        object.pythonSettings = null;
                        object.nodeSettings = null;
                        object.dotnetSettings = null;
                        object.rubySettings = null;
                        object.goSettings = null;
                    }
                    if (message.version != null && message.hasOwnProperty("version"))
                        object.version = message.version;
                    if (message.launchStage != null && message.hasOwnProperty("launchStage"))
                        object.launchStage = options.enums === String ? $root.google.api.LaunchStage[message.launchStage] === undefined ? message.launchStage : $root.google.api.LaunchStage[message.launchStage] : message.launchStage;
                    if (message.restNumericEnums != null && message.hasOwnProperty("restNumericEnums"))
                        object.restNumericEnums = message.restNumericEnums;
                    if (message.javaSettings != null && message.hasOwnProperty("javaSettings"))
                        object.javaSettings = $root.google.api.JavaSettings.toObject(message.javaSettings, options);
                    if (message.cppSettings != null && message.hasOwnProperty("cppSettings"))
                        object.cppSettings = $root.google.api.CppSettings.toObject(message.cppSettings, options);
                    if (message.phpSettings != null && message.hasOwnProperty("phpSettings"))
                        object.phpSettings = $root.google.api.PhpSettings.toObject(message.phpSettings, options);
                    if (message.pythonSettings != null && message.hasOwnProperty("pythonSettings"))
                        object.pythonSettings = $root.google.api.PythonSettings.toObject(message.pythonSettings, options);
                    if (message.nodeSettings != null && message.hasOwnProperty("nodeSettings"))
                        object.nodeSettings = $root.google.api.NodeSettings.toObject(message.nodeSettings, options);
                    if (message.dotnetSettings != null && message.hasOwnProperty("dotnetSettings"))
                        object.dotnetSettings = $root.google.api.DotnetSettings.toObject(message.dotnetSettings, options);
                    if (message.rubySettings != null && message.hasOwnProperty("rubySettings"))
                        object.rubySettings = $root.google.api.RubySettings.toObject(message.rubySettings, options);
                    if (message.goSettings != null && message.hasOwnProperty("goSettings"))
                        object.goSettings = $root.google.api.GoSettings.toObject(message.goSettings, options);
                    return object;
                };
    
                /**
                 * Converts this ClientLibrarySettings to JSON.
                 * @function toJSON
                 * @memberof google.api.ClientLibrarySettings
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                ClientLibrarySettings.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for ClientLibrarySettings
                 * @function getTypeUrl
                 * @memberof google.api.ClientLibrarySettings
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                ClientLibrarySettings.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.api.ClientLibrarySettings";
                };
    
                return ClientLibrarySettings;
            })();
    
            api.Publishing = (function() {
    
                /**
                 * Properties of a Publishing.
                 * @memberof google.api
                 * @interface IPublishing
                 * @property {Array.<google.api.IMethodSettings>|null} [methodSettings] Publishing methodSettings
                 * @property {string|null} [newIssueUri] Publishing newIssueUri
                 * @property {string|null} [documentationUri] Publishing documentationUri
                 * @property {string|null} [apiShortName] Publishing apiShortName
                 * @property {string|null} [githubLabel] Publishing githubLabel
                 * @property {Array.<string>|null} [codeownerGithubTeams] Publishing codeownerGithubTeams
                 * @property {string|null} [docTagPrefix] Publishing docTagPrefix
                 * @property {google.api.ClientLibraryOrganization|null} [organization] Publishing organization
                 * @property {Array.<google.api.IClientLibrarySettings>|null} [librarySettings] Publishing librarySettings
                 * @property {string|null} [protoReferenceDocumentationUri] Publishing protoReferenceDocumentationUri
                 * @property {string|null} [restReferenceDocumentationUri] Publishing restReferenceDocumentationUri
                 */
    
                /**
                 * Constructs a new Publishing.
                 * @memberof google.api
                 * @classdesc Represents a Publishing.
                 * @implements IPublishing
                 * @constructor
                 * @param {google.api.IPublishing=} [properties] Properties to set
                 */
                function Publishing(properties) {
                    this.methodSettings = [];
                    this.codeownerGithubTeams = [];
                    this.librarySettings = [];
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * Publishing methodSettings.
                 * @member {Array.<google.api.IMethodSettings>} methodSettings
                 * @memberof google.api.Publishing
                 * @instance
                 */
                Publishing.prototype.methodSettings = $util.emptyArray;
    
                /**
                 * Publishing newIssueUri.
                 * @member {string} newIssueUri
                 * @memberof google.api.Publishing
                 * @instance
                 */
                Publishing.prototype.newIssueUri = "";
    
                /**
                 * Publishing documentationUri.
                 * @member {string} documentationUri
                 * @memberof google.api.Publishing
                 * @instance
                 */
                Publishing.prototype.documentationUri = "";
    
                /**
                 * Publishing apiShortName.
                 * @member {string} apiShortName
                 * @memberof google.api.Publishing
                 * @instance
                 */
                Publishing.prototype.apiShortName = "";
    
                /**
                 * Publishing githubLabel.
                 * @member {string} githubLabel
                 * @memberof google.api.Publishing
                 * @instance
                 */
                Publishing.prototype.githubLabel = "";
    
                /**
                 * Publishing codeownerGithubTeams.
                 * @member {Array.<string>} codeownerGithubTeams
                 * @memberof google.api.Publishing
                 * @instance
                 */
                Publishing.prototype.codeownerGithubTeams = $util.emptyArray;
    
                /**
                 * Publishing docTagPrefix.
                 * @member {string} docTagPrefix
                 * @memberof google.api.Publishing
                 * @instance
                 */
                Publishing.prototype.docTagPrefix = "";
    
                /**
                 * Publishing organization.
                 * @member {google.api.ClientLibraryOrganization} organization
                 * @memberof google.api.Publishing
                 * @instance
                 */
                Publishing.prototype.organization = 0;
    
                /**
                 * Publishing librarySettings.
                 * @member {Array.<google.api.IClientLibrarySettings>} librarySettings
                 * @memberof google.api.Publishing
                 * @instance
                 */
                Publishing.prototype.librarySettings = $util.emptyArray;
    
                /**
                 * Publishing protoReferenceDocumentationUri.
                 * @member {string} protoReferenceDocumentationUri
                 * @memberof google.api.Publishing
                 * @instance
                 */
                Publishing.prototype.protoReferenceDocumentationUri = "";
    
                /**
                 * Publishing restReferenceDocumentationUri.
                 * @member {string} restReferenceDocumentationUri
                 * @memberof google.api.Publishing
                 * @instance
                 */
                Publishing.prototype.restReferenceDocumentationUri = "";
    
                /**
                 * Creates a new Publishing instance using the specified properties.
                 * @function create
                 * @memberof google.api.Publishing
                 * @static
                 * @param {google.api.IPublishing=} [properties] Properties to set
                 * @returns {google.api.Publishing} Publishing instance
                 */
                Publishing.create = function create(properties) {
                    return new Publishing(properties);
                };
    
                /**
                 * Encodes the specified Publishing message. Does not implicitly {@link google.api.Publishing.verify|verify} messages.
                 * @function encode
                 * @memberof google.api.Publishing
                 * @static
                 * @param {google.api.IPublishing} message Publishing message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Publishing.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.methodSettings != null && message.methodSettings.length)
                        for (var i = 0; i < message.methodSettings.length; ++i)
                            $root.google.api.MethodSettings.encode(message.methodSettings[i], writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                    if (message.newIssueUri != null && Object.hasOwnProperty.call(message, "newIssueUri"))
                        writer.uint32(/* id 101, wireType 2 =*/810).string(message.newIssueUri);
                    if (message.documentationUri != null && Object.hasOwnProperty.call(message, "documentationUri"))
                        writer.uint32(/* id 102, wireType 2 =*/818).string(message.documentationUri);
                    if (message.apiShortName != null && Object.hasOwnProperty.call(message, "apiShortName"))
                        writer.uint32(/* id 103, wireType 2 =*/826).string(message.apiShortName);
                    if (message.githubLabel != null && Object.hasOwnProperty.call(message, "githubLabel"))
                        writer.uint32(/* id 104, wireType 2 =*/834).string(message.githubLabel);
                    if (message.codeownerGithubTeams != null && message.codeownerGithubTeams.length)
                        for (var i = 0; i < message.codeownerGithubTeams.length; ++i)
                            writer.uint32(/* id 105, wireType 2 =*/842).string(message.codeownerGithubTeams[i]);
                    if (message.docTagPrefix != null && Object.hasOwnProperty.call(message, "docTagPrefix"))
                        writer.uint32(/* id 106, wireType 2 =*/850).string(message.docTagPrefix);
                    if (message.organization != null && Object.hasOwnProperty.call(message, "organization"))
                        writer.uint32(/* id 107, wireType 0 =*/856).int32(message.organization);
                    if (message.librarySettings != null && message.librarySettings.length)
                        for (var i = 0; i < message.librarySettings.length; ++i)
                            $root.google.api.ClientLibrarySettings.encode(message.librarySettings[i], writer.uint32(/* id 109, wireType 2 =*/874).fork()).ldelim();
                    if (message.protoReferenceDocumentationUri != null && Object.hasOwnProperty.call(message, "protoReferenceDocumentationUri"))
                        writer.uint32(/* id 110, wireType 2 =*/882).string(message.protoReferenceDocumentationUri);
                    if (message.restReferenceDocumentationUri != null && Object.hasOwnProperty.call(message, "restReferenceDocumentationUri"))
                        writer.uint32(/* id 111, wireType 2 =*/890).string(message.restReferenceDocumentationUri);
                    return writer;
                };
    
                /**
                 * Encodes the specified Publishing message, length delimited. Does not implicitly {@link google.api.Publishing.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.api.Publishing
                 * @static
                 * @param {google.api.IPublishing} message Publishing message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Publishing.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a Publishing message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.api.Publishing
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.api.Publishing} Publishing
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Publishing.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.api.Publishing();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 2: {
                                if (!(message.methodSettings && message.methodSettings.length))
                                    message.methodSettings = [];
                                message.methodSettings.push($root.google.api.MethodSettings.decode(reader, reader.uint32()));
                                break;
                            }
                        case 101: {
                                message.newIssueUri = reader.string();
                                break;
                            }
                        case 102: {
                                message.documentationUri = reader.string();
                                break;
                            }
                        case 103: {
                                message.apiShortName = reader.string();
                                break;
                            }
                        case 104: {
                                message.githubLabel = reader.string();
                                break;
                            }
                        case 105: {
                                if (!(message.codeownerGithubTeams && message.codeownerGithubTeams.length))
                                    message.codeownerGithubTeams = [];
                                message.codeownerGithubTeams.push(reader.string());
                                break;
                            }
                        case 106: {
                                message.docTagPrefix = reader.string();
                                break;
                            }
                        case 107: {
                                message.organization = reader.int32();
                                break;
                            }
                        case 109: {
                                if (!(message.librarySettings && message.librarySettings.length))
                                    message.librarySettings = [];
                                message.librarySettings.push($root.google.api.ClientLibrarySettings.decode(reader, reader.uint32()));
                                break;
                            }
                        case 110: {
                                message.protoReferenceDocumentationUri = reader.string();
                                break;
                            }
                        case 111: {
                                message.restReferenceDocumentationUri = reader.string();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a Publishing message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.api.Publishing
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.api.Publishing} Publishing
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Publishing.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a Publishing message.
                 * @function verify
                 * @memberof google.api.Publishing
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                Publishing.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.methodSettings != null && message.hasOwnProperty("methodSettings")) {
                        if (!Array.isArray(message.methodSettings))
                            return "methodSettings: array expected";
                        for (var i = 0; i < message.methodSettings.length; ++i) {
                            var error = $root.google.api.MethodSettings.verify(message.methodSettings[i]);
                            if (error)
                                return "methodSettings." + error;
                        }
                    }
                    if (message.newIssueUri != null && message.hasOwnProperty("newIssueUri"))
                        if (!$util.isString(message.newIssueUri))
                            return "newIssueUri: string expected";
                    if (message.documentationUri != null && message.hasOwnProperty("documentationUri"))
                        if (!$util.isString(message.documentationUri))
                            return "documentationUri: string expected";
                    if (message.apiShortName != null && message.hasOwnProperty("apiShortName"))
                        if (!$util.isString(message.apiShortName))
                            return "apiShortName: string expected";
                    if (message.githubLabel != null && message.hasOwnProperty("githubLabel"))
                        if (!$util.isString(message.githubLabel))
                            return "githubLabel: string expected";
                    if (message.codeownerGithubTeams != null && message.hasOwnProperty("codeownerGithubTeams")) {
                        if (!Array.isArray(message.codeownerGithubTeams))
                            return "codeownerGithubTeams: array expected";
                        for (var i = 0; i < message.codeownerGithubTeams.length; ++i)
                            if (!$util.isString(message.codeownerGithubTeams[i]))
                                return "codeownerGithubTeams: string[] expected";
                    }
                    if (message.docTagPrefix != null && message.hasOwnProperty("docTagPrefix"))
                        if (!$util.isString(message.docTagPrefix))
                            return "docTagPrefix: string expected";
                    if (message.organization != null && message.hasOwnProperty("organization"))
                        switch (message.organization) {
                        default:
                            return "organization: enum value expected";
                        case 0:
                        case 1:
                        case 2:
                        case 3:
                        case 4:
                        case 5:
                        case 6:
                        case 7:
                            break;
                        }
                    if (message.librarySettings != null && message.hasOwnProperty("librarySettings")) {
                        if (!Array.isArray(message.librarySettings))
                            return "librarySettings: array expected";
                        for (var i = 0; i < message.librarySettings.length; ++i) {
                            var error = $root.google.api.ClientLibrarySettings.verify(message.librarySettings[i]);
                            if (error)
                                return "librarySettings." + error;
                        }
                    }
                    if (message.protoReferenceDocumentationUri != null && message.hasOwnProperty("protoReferenceDocumentationUri"))
                        if (!$util.isString(message.protoReferenceDocumentationUri))
                            return "protoReferenceDocumentationUri: string expected";
                    if (message.restReferenceDocumentationUri != null && message.hasOwnProperty("restReferenceDocumentationUri"))
                        if (!$util.isString(message.restReferenceDocumentationUri))
                            return "restReferenceDocumentationUri: string expected";
                    return null;
                };
    
                /**
                 * Creates a Publishing message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.api.Publishing
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.api.Publishing} Publishing
                 */
                Publishing.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.api.Publishing)
                        return object;
                    var message = new $root.google.api.Publishing();
                    if (object.methodSettings) {
                        if (!Array.isArray(object.methodSettings))
                            throw TypeError(".google.api.Publishing.methodSettings: array expected");
                        message.methodSettings = [];
                        for (var i = 0; i < object.methodSettings.length; ++i) {
                            if (typeof object.methodSettings[i] !== "object")
                                throw TypeError(".google.api.Publishing.methodSettings: object expected");
                            message.methodSettings[i] = $root.google.api.MethodSettings.fromObject(object.methodSettings[i]);
                        }
                    }
                    if (object.newIssueUri != null)
                        message.newIssueUri = String(object.newIssueUri);
                    if (object.documentationUri != null)
                        message.documentationUri = String(object.documentationUri);
                    if (object.apiShortName != null)
                        message.apiShortName = String(object.apiShortName);
                    if (object.githubLabel != null)
                        message.githubLabel = String(object.githubLabel);
                    if (object.codeownerGithubTeams) {
                        if (!Array.isArray(object.codeownerGithubTeams))
                            throw TypeError(".google.api.Publishing.codeownerGithubTeams: array expected");
                        message.codeownerGithubTeams = [];
                        for (var i = 0; i < object.codeownerGithubTeams.length; ++i)
                            message.codeownerGithubTeams[i] = String(object.codeownerGithubTeams[i]);
                    }
                    if (object.docTagPrefix != null)
                        message.docTagPrefix = String(object.docTagPrefix);
                    switch (object.organization) {
                    default:
                        if (typeof object.organization === "number") {
                            message.organization = object.organization;
                            break;
                        }
                        break;
                    case "CLIENT_LIBRARY_ORGANIZATION_UNSPECIFIED":
                    case 0:
                        message.organization = 0;
                        break;
                    case "CLOUD":
                    case 1:
                        message.organization = 1;
                        break;
                    case "ADS":
                    case 2:
                        message.organization = 2;
                        break;
                    case "PHOTOS":
                    case 3:
                        message.organization = 3;
                        break;
                    case "STREET_VIEW":
                    case 4:
                        message.organization = 4;
                        break;
                    case "SHOPPING":
                    case 5:
                        message.organization = 5;
                        break;
                    case "GEO":
                    case 6:
                        message.organization = 6;
                        break;
                    case "GENERATIVE_AI":
                    case 7:
                        message.organization = 7;
                        break;
                    }
                    if (object.librarySettings) {
                        if (!Array.isArray(object.librarySettings))
                            throw TypeError(".google.api.Publishing.librarySettings: array expected");
                        message.librarySettings = [];
                        for (var i = 0; i < object.librarySettings.length; ++i) {
                            if (typeof object.librarySettings[i] !== "object")
                                throw TypeError(".google.api.Publishing.librarySettings: object expected");
                            message.librarySettings[i] = $root.google.api.ClientLibrarySettings.fromObject(object.librarySettings[i]);
                        }
                    }
                    if (object.protoReferenceDocumentationUri != null)
                        message.protoReferenceDocumentationUri = String(object.protoReferenceDocumentationUri);
                    if (object.restReferenceDocumentationUri != null)
                        message.restReferenceDocumentationUri = String(object.restReferenceDocumentationUri);
                    return message;
                };
    
                /**
                 * Creates a plain object from a Publishing message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.api.Publishing
                 * @static
                 * @param {google.api.Publishing} message Publishing
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                Publishing.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.arrays || options.defaults) {
                        object.methodSettings = [];
                        object.codeownerGithubTeams = [];
                        object.librarySettings = [];
                    }
                    if (options.defaults) {
                        object.newIssueUri = "";
                        object.documentationUri = "";
                        object.apiShortName = "";
                        object.githubLabel = "";
                        object.docTagPrefix = "";
                        object.organization = options.enums === String ? "CLIENT_LIBRARY_ORGANIZATION_UNSPECIFIED" : 0;
                        object.protoReferenceDocumentationUri = "";
                        object.restReferenceDocumentationUri = "";
                    }
                    if (message.methodSettings && message.methodSettings.length) {
                        object.methodSettings = [];
                        for (var j = 0; j < message.methodSettings.length; ++j)
                            object.methodSettings[j] = $root.google.api.MethodSettings.toObject(message.methodSettings[j], options);
                    }
                    if (message.newIssueUri != null && message.hasOwnProperty("newIssueUri"))
                        object.newIssueUri = message.newIssueUri;
                    if (message.documentationUri != null && message.hasOwnProperty("documentationUri"))
                        object.documentationUri = message.documentationUri;
                    if (message.apiShortName != null && message.hasOwnProperty("apiShortName"))
                        object.apiShortName = message.apiShortName;
                    if (message.githubLabel != null && message.hasOwnProperty("githubLabel"))
                        object.githubLabel = message.githubLabel;
                    if (message.codeownerGithubTeams && message.codeownerGithubTeams.length) {
                        object.codeownerGithubTeams = [];
                        for (var j = 0; j < message.codeownerGithubTeams.length; ++j)
                            object.codeownerGithubTeams[j] = message.codeownerGithubTeams[j];
                    }
                    if (message.docTagPrefix != null && message.hasOwnProperty("docTagPrefix"))
                        object.docTagPrefix = message.docTagPrefix;
                    if (message.organization != null && message.hasOwnProperty("organization"))
                        object.organization = options.enums === String ? $root.google.api.ClientLibraryOrganization[message.organization] === undefined ? message.organization : $root.google.api.ClientLibraryOrganization[message.organization] : message.organization;
                    if (message.librarySettings && message.librarySettings.length) {
                        object.librarySettings = [];
                        for (var j = 0; j < message.librarySettings.length; ++j)
                            object.librarySettings[j] = $root.google.api.ClientLibrarySettings.toObject(message.librarySettings[j], options);
                    }
                    if (message.protoReferenceDocumentationUri != null && message.hasOwnProperty("protoReferenceDocumentationUri"))
                        object.protoReferenceDocumentationUri = message.protoReferenceDocumentationUri;
                    if (message.restReferenceDocumentationUri != null && message.hasOwnProperty("restReferenceDocumentationUri"))
                        object.restReferenceDocumentationUri = message.restReferenceDocumentationUri;
                    return object;
                };
    
                /**
                 * Converts this Publishing to JSON.
                 * @function toJSON
                 * @memberof google.api.Publishing
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                Publishing.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for Publishing
                 * @function getTypeUrl
                 * @memberof google.api.Publishing
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                Publishing.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.api.Publishing";
                };
    
                return Publishing;
            })();
    
            api.JavaSettings = (function() {
    
                /**
                 * Properties of a JavaSettings.
                 * @memberof google.api
                 * @interface IJavaSettings
                 * @property {string|null} [libraryPackage] JavaSettings libraryPackage
                 * @property {Object.<string,string>|null} [serviceClassNames] JavaSettings serviceClassNames
                 * @property {google.api.ICommonLanguageSettings|null} [common] JavaSettings common
                 */
    
                /**
                 * Constructs a new JavaSettings.
                 * @memberof google.api
                 * @classdesc Represents a JavaSettings.
                 * @implements IJavaSettings
                 * @constructor
                 * @param {google.api.IJavaSettings=} [properties] Properties to set
                 */
                function JavaSettings(properties) {
                    this.serviceClassNames = {};
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * JavaSettings libraryPackage.
                 * @member {string} libraryPackage
                 * @memberof google.api.JavaSettings
                 * @instance
                 */
                JavaSettings.prototype.libraryPackage = "";
    
                /**
                 * JavaSettings serviceClassNames.
                 * @member {Object.<string,string>} serviceClassNames
                 * @memberof google.api.JavaSettings
                 * @instance
                 */
                JavaSettings.prototype.serviceClassNames = $util.emptyObject;
    
                /**
                 * JavaSettings common.
                 * @member {google.api.ICommonLanguageSettings|null|undefined} common
                 * @memberof google.api.JavaSettings
                 * @instance
                 */
                JavaSettings.prototype.common = null;
    
                /**
                 * Creates a new JavaSettings instance using the specified properties.
                 * @function create
                 * @memberof google.api.JavaSettings
                 * @static
                 * @param {google.api.IJavaSettings=} [properties] Properties to set
                 * @returns {google.api.JavaSettings} JavaSettings instance
                 */
                JavaSettings.create = function create(properties) {
                    return new JavaSettings(properties);
                };
    
                /**
                 * Encodes the specified JavaSettings message. Does not implicitly {@link google.api.JavaSettings.verify|verify} messages.
                 * @function encode
                 * @memberof google.api.JavaSettings
                 * @static
                 * @param {google.api.IJavaSettings} message JavaSettings message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                JavaSettings.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.libraryPackage != null && Object.hasOwnProperty.call(message, "libraryPackage"))
                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.libraryPackage);
                    if (message.serviceClassNames != null && Object.hasOwnProperty.call(message, "serviceClassNames"))
                        for (var keys = Object.keys(message.serviceClassNames), i = 0; i < keys.length; ++i)
                            writer.uint32(/* id 2, wireType 2 =*/18).fork().uint32(/* id 1, wireType 2 =*/10).string(keys[i]).uint32(/* id 2, wireType 2 =*/18).string(message.serviceClassNames[keys[i]]).ldelim();
                    if (message.common != null && Object.hasOwnProperty.call(message, "common"))
                        $root.google.api.CommonLanguageSettings.encode(message.common, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                    return writer;
                };
    
                /**
                 * Encodes the specified JavaSettings message, length delimited. Does not implicitly {@link google.api.JavaSettings.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.api.JavaSettings
                 * @static
                 * @param {google.api.IJavaSettings} message JavaSettings message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                JavaSettings.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a JavaSettings message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.api.JavaSettings
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.api.JavaSettings} JavaSettings
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                JavaSettings.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.api.JavaSettings(), key, value;
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.libraryPackage = reader.string();
                                break;
                            }
                        case 2: {
                                if (message.serviceClassNames === $util.emptyObject)
                                    message.serviceClassNames = {};
                                var end2 = reader.uint32() + reader.pos;
                                key = "";
                                value = "";
                                while (reader.pos < end2) {
                                    var tag2 = reader.uint32();
                                    switch (tag2 >>> 3) {
                                    case 1:
                                        key = reader.string();
                                        break;
                                    case 2:
                                        value = reader.string();
                                        break;
                                    default:
                                        reader.skipType(tag2 & 7);
                                        break;
                                    }
                                }
                                message.serviceClassNames[key] = value;
                                break;
                            }
                        case 3: {
                                message.common = $root.google.api.CommonLanguageSettings.decode(reader, reader.uint32());
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a JavaSettings message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.api.JavaSettings
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.api.JavaSettings} JavaSettings
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                JavaSettings.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a JavaSettings message.
                 * @function verify
                 * @memberof google.api.JavaSettings
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                JavaSettings.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.libraryPackage != null && message.hasOwnProperty("libraryPackage"))
                        if (!$util.isString(message.libraryPackage))
                            return "libraryPackage: string expected";
                    if (message.serviceClassNames != null && message.hasOwnProperty("serviceClassNames")) {
                        if (!$util.isObject(message.serviceClassNames))
                            return "serviceClassNames: object expected";
                        var key = Object.keys(message.serviceClassNames);
                        for (var i = 0; i < key.length; ++i)
                            if (!$util.isString(message.serviceClassNames[key[i]]))
                                return "serviceClassNames: string{k:string} expected";
                    }
                    if (message.common != null && message.hasOwnProperty("common")) {
                        var error = $root.google.api.CommonLanguageSettings.verify(message.common);
                        if (error)
                            return "common." + error;
                    }
                    return null;
                };
    
                /**
                 * Creates a JavaSettings message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.api.JavaSettings
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.api.JavaSettings} JavaSettings
                 */
                JavaSettings.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.api.JavaSettings)
                        return object;
                    var message = new $root.google.api.JavaSettings();
                    if (object.libraryPackage != null)
                        message.libraryPackage = String(object.libraryPackage);
                    if (object.serviceClassNames) {
                        if (typeof object.serviceClassNames !== "object")
                            throw TypeError(".google.api.JavaSettings.serviceClassNames: object expected");
                        message.serviceClassNames = {};
                        for (var keys = Object.keys(object.serviceClassNames), i = 0; i < keys.length; ++i)
                            message.serviceClassNames[keys[i]] = String(object.serviceClassNames[keys[i]]);
                    }
                    if (object.common != null) {
                        if (typeof object.common !== "object")
                            throw TypeError(".google.api.JavaSettings.common: object expected");
                        message.common = $root.google.api.CommonLanguageSettings.fromObject(object.common);
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from a JavaSettings message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.api.JavaSettings
                 * @static
                 * @param {google.api.JavaSettings} message JavaSettings
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                JavaSettings.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.objects || options.defaults)
                        object.serviceClassNames = {};
                    if (options.defaults) {
                        object.libraryPackage = "";
                        object.common = null;
                    }
                    if (message.libraryPackage != null && message.hasOwnProperty("libraryPackage"))
                        object.libraryPackage = message.libraryPackage;
                    var keys2;
                    if (message.serviceClassNames && (keys2 = Object.keys(message.serviceClassNames)).length) {
                        object.serviceClassNames = {};
                        for (var j = 0; j < keys2.length; ++j)
                            object.serviceClassNames[keys2[j]] = message.serviceClassNames[keys2[j]];
                    }
                    if (message.common != null && message.hasOwnProperty("common"))
                        object.common = $root.google.api.CommonLanguageSettings.toObject(message.common, options);
                    return object;
                };
    
                /**
                 * Converts this JavaSettings to JSON.
                 * @function toJSON
                 * @memberof google.api.JavaSettings
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                JavaSettings.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for JavaSettings
                 * @function getTypeUrl
                 * @memberof google.api.JavaSettings
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                JavaSettings.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.api.JavaSettings";
                };
    
                return JavaSettings;
            })();
    
            api.CppSettings = (function() {
    
                /**
                 * Properties of a CppSettings.
                 * @memberof google.api
                 * @interface ICppSettings
                 * @property {google.api.ICommonLanguageSettings|null} [common] CppSettings common
                 */
    
                /**
                 * Constructs a new CppSettings.
                 * @memberof google.api
                 * @classdesc Represents a CppSettings.
                 * @implements ICppSettings
                 * @constructor
                 * @param {google.api.ICppSettings=} [properties] Properties to set
                 */
                function CppSettings(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * CppSettings common.
                 * @member {google.api.ICommonLanguageSettings|null|undefined} common
                 * @memberof google.api.CppSettings
                 * @instance
                 */
                CppSettings.prototype.common = null;
    
                /**
                 * Creates a new CppSettings instance using the specified properties.
                 * @function create
                 * @memberof google.api.CppSettings
                 * @static
                 * @param {google.api.ICppSettings=} [properties] Properties to set
                 * @returns {google.api.CppSettings} CppSettings instance
                 */
                CppSettings.create = function create(properties) {
                    return new CppSettings(properties);
                };
    
                /**
                 * Encodes the specified CppSettings message. Does not implicitly {@link google.api.CppSettings.verify|verify} messages.
                 * @function encode
                 * @memberof google.api.CppSettings
                 * @static
                 * @param {google.api.ICppSettings} message CppSettings message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                CppSettings.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.common != null && Object.hasOwnProperty.call(message, "common"))
                        $root.google.api.CommonLanguageSettings.encode(message.common, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                    return writer;
                };
    
                /**
                 * Encodes the specified CppSettings message, length delimited. Does not implicitly {@link google.api.CppSettings.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.api.CppSettings
                 * @static
                 * @param {google.api.ICppSettings} message CppSettings message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                CppSettings.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a CppSettings message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.api.CppSettings
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.api.CppSettings} CppSettings
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                CppSettings.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.api.CppSettings();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.common = $root.google.api.CommonLanguageSettings.decode(reader, reader.uint32());
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a CppSettings message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.api.CppSettings
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.api.CppSettings} CppSettings
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                CppSettings.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a CppSettings message.
                 * @function verify
                 * @memberof google.api.CppSettings
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                CppSettings.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.common != null && message.hasOwnProperty("common")) {
                        var error = $root.google.api.CommonLanguageSettings.verify(message.common);
                        if (error)
                            return "common." + error;
                    }
                    return null;
                };
    
                /**
                 * Creates a CppSettings message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.api.CppSettings
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.api.CppSettings} CppSettings
                 */
                CppSettings.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.api.CppSettings)
                        return object;
                    var message = new $root.google.api.CppSettings();
                    if (object.common != null) {
                        if (typeof object.common !== "object")
                            throw TypeError(".google.api.CppSettings.common: object expected");
                        message.common = $root.google.api.CommonLanguageSettings.fromObject(object.common);
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from a CppSettings message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.api.CppSettings
                 * @static
                 * @param {google.api.CppSettings} message CppSettings
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                CppSettings.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults)
                        object.common = null;
                    if (message.common != null && message.hasOwnProperty("common"))
                        object.common = $root.google.api.CommonLanguageSettings.toObject(message.common, options);
                    return object;
                };
    
                /**
                 * Converts this CppSettings to JSON.
                 * @function toJSON
                 * @memberof google.api.CppSettings
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                CppSettings.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for CppSettings
                 * @function getTypeUrl
                 * @memberof google.api.CppSettings
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                CppSettings.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.api.CppSettings";
                };
    
                return CppSettings;
            })();
    
            api.PhpSettings = (function() {
    
                /**
                 * Properties of a PhpSettings.
                 * @memberof google.api
                 * @interface IPhpSettings
                 * @property {google.api.ICommonLanguageSettings|null} [common] PhpSettings common
                 */
    
                /**
                 * Constructs a new PhpSettings.
                 * @memberof google.api
                 * @classdesc Represents a PhpSettings.
                 * @implements IPhpSettings
                 * @constructor
                 * @param {google.api.IPhpSettings=} [properties] Properties to set
                 */
                function PhpSettings(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * PhpSettings common.
                 * @member {google.api.ICommonLanguageSettings|null|undefined} common
                 * @memberof google.api.PhpSettings
                 * @instance
                 */
                PhpSettings.prototype.common = null;
    
                /**
                 * Creates a new PhpSettings instance using the specified properties.
                 * @function create
                 * @memberof google.api.PhpSettings
                 * @static
                 * @param {google.api.IPhpSettings=} [properties] Properties to set
                 * @returns {google.api.PhpSettings} PhpSettings instance
                 */
                PhpSettings.create = function create(properties) {
                    return new PhpSettings(properties);
                };
    
                /**
                 * Encodes the specified PhpSettings message. Does not implicitly {@link google.api.PhpSettings.verify|verify} messages.
                 * @function encode
                 * @memberof google.api.PhpSettings
                 * @static
                 * @param {google.api.IPhpSettings} message PhpSettings message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                PhpSettings.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.common != null && Object.hasOwnProperty.call(message, "common"))
                        $root.google.api.CommonLanguageSettings.encode(message.common, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                    return writer;
                };
    
                /**
                 * Encodes the specified PhpSettings message, length delimited. Does not implicitly {@link google.api.PhpSettings.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.api.PhpSettings
                 * @static
                 * @param {google.api.IPhpSettings} message PhpSettings message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                PhpSettings.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a PhpSettings message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.api.PhpSettings
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.api.PhpSettings} PhpSettings
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                PhpSettings.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.api.PhpSettings();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.common = $root.google.api.CommonLanguageSettings.decode(reader, reader.uint32());
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a PhpSettings message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.api.PhpSettings
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.api.PhpSettings} PhpSettings
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                PhpSettings.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a PhpSettings message.
                 * @function verify
                 * @memberof google.api.PhpSettings
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                PhpSettings.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.common != null && message.hasOwnProperty("common")) {
                        var error = $root.google.api.CommonLanguageSettings.verify(message.common);
                        if (error)
                            return "common." + error;
                    }
                    return null;
                };
    
                /**
                 * Creates a PhpSettings message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.api.PhpSettings
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.api.PhpSettings} PhpSettings
                 */
                PhpSettings.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.api.PhpSettings)
                        return object;
                    var message = new $root.google.api.PhpSettings();
                    if (object.common != null) {
                        if (typeof object.common !== "object")
                            throw TypeError(".google.api.PhpSettings.common: object expected");
                        message.common = $root.google.api.CommonLanguageSettings.fromObject(object.common);
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from a PhpSettings message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.api.PhpSettings
                 * @static
                 * @param {google.api.PhpSettings} message PhpSettings
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                PhpSettings.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults)
                        object.common = null;
                    if (message.common != null && message.hasOwnProperty("common"))
                        object.common = $root.google.api.CommonLanguageSettings.toObject(message.common, options);
                    return object;
                };
    
                /**
                 * Converts this PhpSettings to JSON.
                 * @function toJSON
                 * @memberof google.api.PhpSettings
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                PhpSettings.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for PhpSettings
                 * @function getTypeUrl
                 * @memberof google.api.PhpSettings
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                PhpSettings.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.api.PhpSettings";
                };
    
                return PhpSettings;
            })();
    
            api.PythonSettings = (function() {
    
                /**
                 * Properties of a PythonSettings.
                 * @memberof google.api
                 * @interface IPythonSettings
                 * @property {google.api.ICommonLanguageSettings|null} [common] PythonSettings common
                 */
    
                /**
                 * Constructs a new PythonSettings.
                 * @memberof google.api
                 * @classdesc Represents a PythonSettings.
                 * @implements IPythonSettings
                 * @constructor
                 * @param {google.api.IPythonSettings=} [properties] Properties to set
                 */
                function PythonSettings(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * PythonSettings common.
                 * @member {google.api.ICommonLanguageSettings|null|undefined} common
                 * @memberof google.api.PythonSettings
                 * @instance
                 */
                PythonSettings.prototype.common = null;
    
                /**
                 * Creates a new PythonSettings instance using the specified properties.
                 * @function create
                 * @memberof google.api.PythonSettings
                 * @static
                 * @param {google.api.IPythonSettings=} [properties] Properties to set
                 * @returns {google.api.PythonSettings} PythonSettings instance
                 */
                PythonSettings.create = function create(properties) {
                    return new PythonSettings(properties);
                };
    
                /**
                 * Encodes the specified PythonSettings message. Does not implicitly {@link google.api.PythonSettings.verify|verify} messages.
                 * @function encode
                 * @memberof google.api.PythonSettings
                 * @static
                 * @param {google.api.IPythonSettings} message PythonSettings message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                PythonSettings.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.common != null && Object.hasOwnProperty.call(message, "common"))
                        $root.google.api.CommonLanguageSettings.encode(message.common, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                    return writer;
                };
    
                /**
                 * Encodes the specified PythonSettings message, length delimited. Does not implicitly {@link google.api.PythonSettings.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.api.PythonSettings
                 * @static
                 * @param {google.api.IPythonSettings} message PythonSettings message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                PythonSettings.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a PythonSettings message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.api.PythonSettings
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.api.PythonSettings} PythonSettings
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                PythonSettings.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.api.PythonSettings();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.common = $root.google.api.CommonLanguageSettings.decode(reader, reader.uint32());
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a PythonSettings message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.api.PythonSettings
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.api.PythonSettings} PythonSettings
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                PythonSettings.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a PythonSettings message.
                 * @function verify
                 * @memberof google.api.PythonSettings
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                PythonSettings.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.common != null && message.hasOwnProperty("common")) {
                        var error = $root.google.api.CommonLanguageSettings.verify(message.common);
                        if (error)
                            return "common." + error;
                    }
                    return null;
                };
    
                /**
                 * Creates a PythonSettings message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.api.PythonSettings
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.api.PythonSettings} PythonSettings
                 */
                PythonSettings.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.api.PythonSettings)
                        return object;
                    var message = new $root.google.api.PythonSettings();
                    if (object.common != null) {
                        if (typeof object.common !== "object")
                            throw TypeError(".google.api.PythonSettings.common: object expected");
                        message.common = $root.google.api.CommonLanguageSettings.fromObject(object.common);
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from a PythonSettings message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.api.PythonSettings
                 * @static
                 * @param {google.api.PythonSettings} message PythonSettings
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                PythonSettings.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults)
                        object.common = null;
                    if (message.common != null && message.hasOwnProperty("common"))
                        object.common = $root.google.api.CommonLanguageSettings.toObject(message.common, options);
                    return object;
                };
    
                /**
                 * Converts this PythonSettings to JSON.
                 * @function toJSON
                 * @memberof google.api.PythonSettings
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                PythonSettings.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for PythonSettings
                 * @function getTypeUrl
                 * @memberof google.api.PythonSettings
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                PythonSettings.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.api.PythonSettings";
                };
    
                return PythonSettings;
            })();
    
            api.NodeSettings = (function() {
    
                /**
                 * Properties of a NodeSettings.
                 * @memberof google.api
                 * @interface INodeSettings
                 * @property {google.api.ICommonLanguageSettings|null} [common] NodeSettings common
                 */
    
                /**
                 * Constructs a new NodeSettings.
                 * @memberof google.api
                 * @classdesc Represents a NodeSettings.
                 * @implements INodeSettings
                 * @constructor
                 * @param {google.api.INodeSettings=} [properties] Properties to set
                 */
                function NodeSettings(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * NodeSettings common.
                 * @member {google.api.ICommonLanguageSettings|null|undefined} common
                 * @memberof google.api.NodeSettings
                 * @instance
                 */
                NodeSettings.prototype.common = null;
    
                /**
                 * Creates a new NodeSettings instance using the specified properties.
                 * @function create
                 * @memberof google.api.NodeSettings
                 * @static
                 * @param {google.api.INodeSettings=} [properties] Properties to set
                 * @returns {google.api.NodeSettings} NodeSettings instance
                 */
                NodeSettings.create = function create(properties) {
                    return new NodeSettings(properties);
                };
    
                /**
                 * Encodes the specified NodeSettings message. Does not implicitly {@link google.api.NodeSettings.verify|verify} messages.
                 * @function encode
                 * @memberof google.api.NodeSettings
                 * @static
                 * @param {google.api.INodeSettings} message NodeSettings message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                NodeSettings.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.common != null && Object.hasOwnProperty.call(message, "common"))
                        $root.google.api.CommonLanguageSettings.encode(message.common, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                    return writer;
                };
    
                /**
                 * Encodes the specified NodeSettings message, length delimited. Does not implicitly {@link google.api.NodeSettings.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.api.NodeSettings
                 * @static
                 * @param {google.api.INodeSettings} message NodeSettings message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                NodeSettings.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a NodeSettings message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.api.NodeSettings
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.api.NodeSettings} NodeSettings
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                NodeSettings.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.api.NodeSettings();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.common = $root.google.api.CommonLanguageSettings.decode(reader, reader.uint32());
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a NodeSettings message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.api.NodeSettings
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.api.NodeSettings} NodeSettings
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                NodeSettings.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a NodeSettings message.
                 * @function verify
                 * @memberof google.api.NodeSettings
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                NodeSettings.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.common != null && message.hasOwnProperty("common")) {
                        var error = $root.google.api.CommonLanguageSettings.verify(message.common);
                        if (error)
                            return "common." + error;
                    }
                    return null;
                };
    
                /**
                 * Creates a NodeSettings message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.api.NodeSettings
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.api.NodeSettings} NodeSettings
                 */
                NodeSettings.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.api.NodeSettings)
                        return object;
                    var message = new $root.google.api.NodeSettings();
                    if (object.common != null) {
                        if (typeof object.common !== "object")
                            throw TypeError(".google.api.NodeSettings.common: object expected");
                        message.common = $root.google.api.CommonLanguageSettings.fromObject(object.common);
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from a NodeSettings message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.api.NodeSettings
                 * @static
                 * @param {google.api.NodeSettings} message NodeSettings
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                NodeSettings.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults)
                        object.common = null;
                    if (message.common != null && message.hasOwnProperty("common"))
                        object.common = $root.google.api.CommonLanguageSettings.toObject(message.common, options);
                    return object;
                };
    
                /**
                 * Converts this NodeSettings to JSON.
                 * @function toJSON
                 * @memberof google.api.NodeSettings
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                NodeSettings.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for NodeSettings
                 * @function getTypeUrl
                 * @memberof google.api.NodeSettings
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                NodeSettings.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.api.NodeSettings";
                };
    
                return NodeSettings;
            })();
    
            api.DotnetSettings = (function() {
    
                /**
                 * Properties of a DotnetSettings.
                 * @memberof google.api
                 * @interface IDotnetSettings
                 * @property {google.api.ICommonLanguageSettings|null} [common] DotnetSettings common
                 * @property {Object.<string,string>|null} [renamedServices] DotnetSettings renamedServices
                 * @property {Object.<string,string>|null} [renamedResources] DotnetSettings renamedResources
                 * @property {Array.<string>|null} [ignoredResources] DotnetSettings ignoredResources
                 * @property {Array.<string>|null} [forcedNamespaceAliases] DotnetSettings forcedNamespaceAliases
                 * @property {Array.<string>|null} [handwrittenSignatures] DotnetSettings handwrittenSignatures
                 */
    
                /**
                 * Constructs a new DotnetSettings.
                 * @memberof google.api
                 * @classdesc Represents a DotnetSettings.
                 * @implements IDotnetSettings
                 * @constructor
                 * @param {google.api.IDotnetSettings=} [properties] Properties to set
                 */
                function DotnetSettings(properties) {
                    this.renamedServices = {};
                    this.renamedResources = {};
                    this.ignoredResources = [];
                    this.forcedNamespaceAliases = [];
                    this.handwrittenSignatures = [];
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * DotnetSettings common.
                 * @member {google.api.ICommonLanguageSettings|null|undefined} common
                 * @memberof google.api.DotnetSettings
                 * @instance
                 */
                DotnetSettings.prototype.common = null;
    
                /**
                 * DotnetSettings renamedServices.
                 * @member {Object.<string,string>} renamedServices
                 * @memberof google.api.DotnetSettings
                 * @instance
                 */
                DotnetSettings.prototype.renamedServices = $util.emptyObject;
    
                /**
                 * DotnetSettings renamedResources.
                 * @member {Object.<string,string>} renamedResources
                 * @memberof google.api.DotnetSettings
                 * @instance
                 */
                DotnetSettings.prototype.renamedResources = $util.emptyObject;
    
                /**
                 * DotnetSettings ignoredResources.
                 * @member {Array.<string>} ignoredResources
                 * @memberof google.api.DotnetSettings
                 * @instance
                 */
                DotnetSettings.prototype.ignoredResources = $util.emptyArray;
    
                /**
                 * DotnetSettings forcedNamespaceAliases.
                 * @member {Array.<string>} forcedNamespaceAliases
                 * @memberof google.api.DotnetSettings
                 * @instance
                 */
                DotnetSettings.prototype.forcedNamespaceAliases = $util.emptyArray;
    
                /**
                 * DotnetSettings handwrittenSignatures.
                 * @member {Array.<string>} handwrittenSignatures
                 * @memberof google.api.DotnetSettings
                 * @instance
                 */
                DotnetSettings.prototype.handwrittenSignatures = $util.emptyArray;
    
                /**
                 * Creates a new DotnetSettings instance using the specified properties.
                 * @function create
                 * @memberof google.api.DotnetSettings
                 * @static
                 * @param {google.api.IDotnetSettings=} [properties] Properties to set
                 * @returns {google.api.DotnetSettings} DotnetSettings instance
                 */
                DotnetSettings.create = function create(properties) {
                    return new DotnetSettings(properties);
                };
    
                /**
                 * Encodes the specified DotnetSettings message. Does not implicitly {@link google.api.DotnetSettings.verify|verify} messages.
                 * @function encode
                 * @memberof google.api.DotnetSettings
                 * @static
                 * @param {google.api.IDotnetSettings} message DotnetSettings message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                DotnetSettings.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.common != null && Object.hasOwnProperty.call(message, "common"))
                        $root.google.api.CommonLanguageSettings.encode(message.common, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                    if (message.renamedServices != null && Object.hasOwnProperty.call(message, "renamedServices"))
                        for (var keys = Object.keys(message.renamedServices), i = 0; i < keys.length; ++i)
                            writer.uint32(/* id 2, wireType 2 =*/18).fork().uint32(/* id 1, wireType 2 =*/10).string(keys[i]).uint32(/* id 2, wireType 2 =*/18).string(message.renamedServices[keys[i]]).ldelim();
                    if (message.renamedResources != null && Object.hasOwnProperty.call(message, "renamedResources"))
                        for (var keys = Object.keys(message.renamedResources), i = 0; i < keys.length; ++i)
                            writer.uint32(/* id 3, wireType 2 =*/26).fork().uint32(/* id 1, wireType 2 =*/10).string(keys[i]).uint32(/* id 2, wireType 2 =*/18).string(message.renamedResources[keys[i]]).ldelim();
                    if (message.ignoredResources != null && message.ignoredResources.length)
                        for (var i = 0; i < message.ignoredResources.length; ++i)
                            writer.uint32(/* id 4, wireType 2 =*/34).string(message.ignoredResources[i]);
                    if (message.forcedNamespaceAliases != null && message.forcedNamespaceAliases.length)
                        for (var i = 0; i < message.forcedNamespaceAliases.length; ++i)
                            writer.uint32(/* id 5, wireType 2 =*/42).string(message.forcedNamespaceAliases[i]);
                    if (message.handwrittenSignatures != null && message.handwrittenSignatures.length)
                        for (var i = 0; i < message.handwrittenSignatures.length; ++i)
                            writer.uint32(/* id 6, wireType 2 =*/50).string(message.handwrittenSignatures[i]);
                    return writer;
                };
    
                /**
                 * Encodes the specified DotnetSettings message, length delimited. Does not implicitly {@link google.api.DotnetSettings.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.api.DotnetSettings
                 * @static
                 * @param {google.api.IDotnetSettings} message DotnetSettings message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                DotnetSettings.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a DotnetSettings message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.api.DotnetSettings
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.api.DotnetSettings} DotnetSettings
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                DotnetSettings.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.api.DotnetSettings(), key, value;
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.common = $root.google.api.CommonLanguageSettings.decode(reader, reader.uint32());
                                break;
                            }
                        case 2: {
                                if (message.renamedServices === $util.emptyObject)
                                    message.renamedServices = {};
                                var end2 = reader.uint32() + reader.pos;
                                key = "";
                                value = "";
                                while (reader.pos < end2) {
                                    var tag2 = reader.uint32();
                                    switch (tag2 >>> 3) {
                                    case 1:
                                        key = reader.string();
                                        break;
                                    case 2:
                                        value = reader.string();
                                        break;
                                    default:
                                        reader.skipType(tag2 & 7);
                                        break;
                                    }
                                }
                                message.renamedServices[key] = value;
                                break;
                            }
                        case 3: {
                                if (message.renamedResources === $util.emptyObject)
                                    message.renamedResources = {};
                                var end2 = reader.uint32() + reader.pos;
                                key = "";
                                value = "";
                                while (reader.pos < end2) {
                                    var tag2 = reader.uint32();
                                    switch (tag2 >>> 3) {
                                    case 1:
                                        key = reader.string();
                                        break;
                                    case 2:
                                        value = reader.string();
                                        break;
                                    default:
                                        reader.skipType(tag2 & 7);
                                        break;
                                    }
                                }
                                message.renamedResources[key] = value;
                                break;
                            }
                        case 4: {
                                if (!(message.ignoredResources && message.ignoredResources.length))
                                    message.ignoredResources = [];
                                message.ignoredResources.push(reader.string());
                                break;
                            }
                        case 5: {
                                if (!(message.forcedNamespaceAliases && message.forcedNamespaceAliases.length))
                                    message.forcedNamespaceAliases = [];
                                message.forcedNamespaceAliases.push(reader.string());
                                break;
                            }
                        case 6: {
                                if (!(message.handwrittenSignatures && message.handwrittenSignatures.length))
                                    message.handwrittenSignatures = [];
                                message.handwrittenSignatures.push(reader.string());
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a DotnetSettings message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.api.DotnetSettings
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.api.DotnetSettings} DotnetSettings
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                DotnetSettings.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a DotnetSettings message.
                 * @function verify
                 * @memberof google.api.DotnetSettings
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                DotnetSettings.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.common != null && message.hasOwnProperty("common")) {
                        var error = $root.google.api.CommonLanguageSettings.verify(message.common);
                        if (error)
                            return "common." + error;
                    }
                    if (message.renamedServices != null && message.hasOwnProperty("renamedServices")) {
                        if (!$util.isObject(message.renamedServices))
                            return "renamedServices: object expected";
                        var key = Object.keys(message.renamedServices);
                        for (var i = 0; i < key.length; ++i)
                            if (!$util.isString(message.renamedServices[key[i]]))
                                return "renamedServices: string{k:string} expected";
                    }
                    if (message.renamedResources != null && message.hasOwnProperty("renamedResources")) {
                        if (!$util.isObject(message.renamedResources))
                            return "renamedResources: object expected";
                        var key = Object.keys(message.renamedResources);
                        for (var i = 0; i < key.length; ++i)
                            if (!$util.isString(message.renamedResources[key[i]]))
                                return "renamedResources: string{k:string} expected";
                    }
                    if (message.ignoredResources != null && message.hasOwnProperty("ignoredResources")) {
                        if (!Array.isArray(message.ignoredResources))
                            return "ignoredResources: array expected";
                        for (var i = 0; i < message.ignoredResources.length; ++i)
                            if (!$util.isString(message.ignoredResources[i]))
                                return "ignoredResources: string[] expected";
                    }
                    if (message.forcedNamespaceAliases != null && message.hasOwnProperty("forcedNamespaceAliases")) {
                        if (!Array.isArray(message.forcedNamespaceAliases))
                            return "forcedNamespaceAliases: array expected";
                        for (var i = 0; i < message.forcedNamespaceAliases.length; ++i)
                            if (!$util.isString(message.forcedNamespaceAliases[i]))
                                return "forcedNamespaceAliases: string[] expected";
                    }
                    if (message.handwrittenSignatures != null && message.hasOwnProperty("handwrittenSignatures")) {
                        if (!Array.isArray(message.handwrittenSignatures))
                            return "handwrittenSignatures: array expected";
                        for (var i = 0; i < message.handwrittenSignatures.length; ++i)
                            if (!$util.isString(message.handwrittenSignatures[i]))
                                return "handwrittenSignatures: string[] expected";
                    }
                    return null;
                };
    
                /**
                 * Creates a DotnetSettings message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.api.DotnetSettings
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.api.DotnetSettings} DotnetSettings
                 */
                DotnetSettings.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.api.DotnetSettings)
                        return object;
                    var message = new $root.google.api.DotnetSettings();
                    if (object.common != null) {
                        if (typeof object.common !== "object")
                            throw TypeError(".google.api.DotnetSettings.common: object expected");
                        message.common = $root.google.api.CommonLanguageSettings.fromObject(object.common);
                    }
                    if (object.renamedServices) {
                        if (typeof object.renamedServices !== "object")
                            throw TypeError(".google.api.DotnetSettings.renamedServices: object expected");
                        message.renamedServices = {};
                        for (var keys = Object.keys(object.renamedServices), i = 0; i < keys.length; ++i)
                            message.renamedServices[keys[i]] = String(object.renamedServices[keys[i]]);
                    }
                    if (object.renamedResources) {
                        if (typeof object.renamedResources !== "object")
                            throw TypeError(".google.api.DotnetSettings.renamedResources: object expected");
                        message.renamedResources = {};
                        for (var keys = Object.keys(object.renamedResources), i = 0; i < keys.length; ++i)
                            message.renamedResources[keys[i]] = String(object.renamedResources[keys[i]]);
                    }
                    if (object.ignoredResources) {
                        if (!Array.isArray(object.ignoredResources))
                            throw TypeError(".google.api.DotnetSettings.ignoredResources: array expected");
                        message.ignoredResources = [];
                        for (var i = 0; i < object.ignoredResources.length; ++i)
                            message.ignoredResources[i] = String(object.ignoredResources[i]);
                    }
                    if (object.forcedNamespaceAliases) {
                        if (!Array.isArray(object.forcedNamespaceAliases))
                            throw TypeError(".google.api.DotnetSettings.forcedNamespaceAliases: array expected");
                        message.forcedNamespaceAliases = [];
                        for (var i = 0; i < object.forcedNamespaceAliases.length; ++i)
                            message.forcedNamespaceAliases[i] = String(object.forcedNamespaceAliases[i]);
                    }
                    if (object.handwrittenSignatures) {
                        if (!Array.isArray(object.handwrittenSignatures))
                            throw TypeError(".google.api.DotnetSettings.handwrittenSignatures: array expected");
                        message.handwrittenSignatures = [];
                        for (var i = 0; i < object.handwrittenSignatures.length; ++i)
                            message.handwrittenSignatures[i] = String(object.handwrittenSignatures[i]);
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from a DotnetSettings message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.api.DotnetSettings
                 * @static
                 * @param {google.api.DotnetSettings} message DotnetSettings
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                DotnetSettings.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.arrays || options.defaults) {
                        object.ignoredResources = [];
                        object.forcedNamespaceAliases = [];
                        object.handwrittenSignatures = [];
                    }
                    if (options.objects || options.defaults) {
                        object.renamedServices = {};
                        object.renamedResources = {};
                    }
                    if (options.defaults)
                        object.common = null;
                    if (message.common != null && message.hasOwnProperty("common"))
                        object.common = $root.google.api.CommonLanguageSettings.toObject(message.common, options);
                    var keys2;
                    if (message.renamedServices && (keys2 = Object.keys(message.renamedServices)).length) {
                        object.renamedServices = {};
                        for (var j = 0; j < keys2.length; ++j)
                            object.renamedServices[keys2[j]] = message.renamedServices[keys2[j]];
                    }
                    if (message.renamedResources && (keys2 = Object.keys(message.renamedResources)).length) {
                        object.renamedResources = {};
                        for (var j = 0; j < keys2.length; ++j)
                            object.renamedResources[keys2[j]] = message.renamedResources[keys2[j]];
                    }
                    if (message.ignoredResources && message.ignoredResources.length) {
                        object.ignoredResources = [];
                        for (var j = 0; j < message.ignoredResources.length; ++j)
                            object.ignoredResources[j] = message.ignoredResources[j];
                    }
                    if (message.forcedNamespaceAliases && message.forcedNamespaceAliases.length) {
                        object.forcedNamespaceAliases = [];
                        for (var j = 0; j < message.forcedNamespaceAliases.length; ++j)
                            object.forcedNamespaceAliases[j] = message.forcedNamespaceAliases[j];
                    }
                    if (message.handwrittenSignatures && message.handwrittenSignatures.length) {
                        object.handwrittenSignatures = [];
                        for (var j = 0; j < message.handwrittenSignatures.length; ++j)
                            object.handwrittenSignatures[j] = message.handwrittenSignatures[j];
                    }
                    return object;
                };
    
                /**
                 * Converts this DotnetSettings to JSON.
                 * @function toJSON
                 * @memberof google.api.DotnetSettings
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                DotnetSettings.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for DotnetSettings
                 * @function getTypeUrl
                 * @memberof google.api.DotnetSettings
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                DotnetSettings.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.api.DotnetSettings";
                };
    
                return DotnetSettings;
            })();
    
            api.RubySettings = (function() {
    
                /**
                 * Properties of a RubySettings.
                 * @memberof google.api
                 * @interface IRubySettings
                 * @property {google.api.ICommonLanguageSettings|null} [common] RubySettings common
                 */
    
                /**
                 * Constructs a new RubySettings.
                 * @memberof google.api
                 * @classdesc Represents a RubySettings.
                 * @implements IRubySettings
                 * @constructor
                 * @param {google.api.IRubySettings=} [properties] Properties to set
                 */
                function RubySettings(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * RubySettings common.
                 * @member {google.api.ICommonLanguageSettings|null|undefined} common
                 * @memberof google.api.RubySettings
                 * @instance
                 */
                RubySettings.prototype.common = null;
    
                /**
                 * Creates a new RubySettings instance using the specified properties.
                 * @function create
                 * @memberof google.api.RubySettings
                 * @static
                 * @param {google.api.IRubySettings=} [properties] Properties to set
                 * @returns {google.api.RubySettings} RubySettings instance
                 */
                RubySettings.create = function create(properties) {
                    return new RubySettings(properties);
                };
    
                /**
                 * Encodes the specified RubySettings message. Does not implicitly {@link google.api.RubySettings.verify|verify} messages.
                 * @function encode
                 * @memberof google.api.RubySettings
                 * @static
                 * @param {google.api.IRubySettings} message RubySettings message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                RubySettings.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.common != null && Object.hasOwnProperty.call(message, "common"))
                        $root.google.api.CommonLanguageSettings.encode(message.common, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                    return writer;
                };
    
                /**
                 * Encodes the specified RubySettings message, length delimited. Does not implicitly {@link google.api.RubySettings.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.api.RubySettings
                 * @static
                 * @param {google.api.IRubySettings} message RubySettings message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                RubySettings.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a RubySettings message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.api.RubySettings
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.api.RubySettings} RubySettings
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                RubySettings.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.api.RubySettings();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.common = $root.google.api.CommonLanguageSettings.decode(reader, reader.uint32());
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a RubySettings message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.api.RubySettings
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.api.RubySettings} RubySettings
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                RubySettings.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a RubySettings message.
                 * @function verify
                 * @memberof google.api.RubySettings
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                RubySettings.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.common != null && message.hasOwnProperty("common")) {
                        var error = $root.google.api.CommonLanguageSettings.verify(message.common);
                        if (error)
                            return "common." + error;
                    }
                    return null;
                };
    
                /**
                 * Creates a RubySettings message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.api.RubySettings
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.api.RubySettings} RubySettings
                 */
                RubySettings.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.api.RubySettings)
                        return object;
                    var message = new $root.google.api.RubySettings();
                    if (object.common != null) {
                        if (typeof object.common !== "object")
                            throw TypeError(".google.api.RubySettings.common: object expected");
                        message.common = $root.google.api.CommonLanguageSettings.fromObject(object.common);
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from a RubySettings message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.api.RubySettings
                 * @static
                 * @param {google.api.RubySettings} message RubySettings
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                RubySettings.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults)
                        object.common = null;
                    if (message.common != null && message.hasOwnProperty("common"))
                        object.common = $root.google.api.CommonLanguageSettings.toObject(message.common, options);
                    return object;
                };
    
                /**
                 * Converts this RubySettings to JSON.
                 * @function toJSON
                 * @memberof google.api.RubySettings
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                RubySettings.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for RubySettings
                 * @function getTypeUrl
                 * @memberof google.api.RubySettings
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                RubySettings.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.api.RubySettings";
                };
    
                return RubySettings;
            })();
    
            api.GoSettings = (function() {
    
                /**
                 * Properties of a GoSettings.
                 * @memberof google.api
                 * @interface IGoSettings
                 * @property {google.api.ICommonLanguageSettings|null} [common] GoSettings common
                 */
    
                /**
                 * Constructs a new GoSettings.
                 * @memberof google.api
                 * @classdesc Represents a GoSettings.
                 * @implements IGoSettings
                 * @constructor
                 * @param {google.api.IGoSettings=} [properties] Properties to set
                 */
                function GoSettings(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * GoSettings common.
                 * @member {google.api.ICommonLanguageSettings|null|undefined} common
                 * @memberof google.api.GoSettings
                 * @instance
                 */
                GoSettings.prototype.common = null;
    
                /**
                 * Creates a new GoSettings instance using the specified properties.
                 * @function create
                 * @memberof google.api.GoSettings
                 * @static
                 * @param {google.api.IGoSettings=} [properties] Properties to set
                 * @returns {google.api.GoSettings} GoSettings instance
                 */
                GoSettings.create = function create(properties) {
                    return new GoSettings(properties);
                };
    
                /**
                 * Encodes the specified GoSettings message. Does not implicitly {@link google.api.GoSettings.verify|verify} messages.
                 * @function encode
                 * @memberof google.api.GoSettings
                 * @static
                 * @param {google.api.IGoSettings} message GoSettings message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                GoSettings.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.common != null && Object.hasOwnProperty.call(message, "common"))
                        $root.google.api.CommonLanguageSettings.encode(message.common, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                    return writer;
                };
    
                /**
                 * Encodes the specified GoSettings message, length delimited. Does not implicitly {@link google.api.GoSettings.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.api.GoSettings
                 * @static
                 * @param {google.api.IGoSettings} message GoSettings message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                GoSettings.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a GoSettings message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.api.GoSettings
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.api.GoSettings} GoSettings
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                GoSettings.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.api.GoSettings();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.common = $root.google.api.CommonLanguageSettings.decode(reader, reader.uint32());
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a GoSettings message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.api.GoSettings
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.api.GoSettings} GoSettings
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                GoSettings.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a GoSettings message.
                 * @function verify
                 * @memberof google.api.GoSettings
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                GoSettings.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.common != null && message.hasOwnProperty("common")) {
                        var error = $root.google.api.CommonLanguageSettings.verify(message.common);
                        if (error)
                            return "common." + error;
                    }
                    return null;
                };
    
                /**
                 * Creates a GoSettings message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.api.GoSettings
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.api.GoSettings} GoSettings
                 */
                GoSettings.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.api.GoSettings)
                        return object;
                    var message = new $root.google.api.GoSettings();
                    if (object.common != null) {
                        if (typeof object.common !== "object")
                            throw TypeError(".google.api.GoSettings.common: object expected");
                        message.common = $root.google.api.CommonLanguageSettings.fromObject(object.common);
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from a GoSettings message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.api.GoSettings
                 * @static
                 * @param {google.api.GoSettings} message GoSettings
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                GoSettings.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults)
                        object.common = null;
                    if (message.common != null && message.hasOwnProperty("common"))
                        object.common = $root.google.api.CommonLanguageSettings.toObject(message.common, options);
                    return object;
                };
    
                /**
                 * Converts this GoSettings to JSON.
                 * @function toJSON
                 * @memberof google.api.GoSettings
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                GoSettings.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for GoSettings
                 * @function getTypeUrl
                 * @memberof google.api.GoSettings
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                GoSettings.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.api.GoSettings";
                };
    
                return GoSettings;
            })();
    
            api.MethodSettings = (function() {
    
                /**
                 * Properties of a MethodSettings.
                 * @memberof google.api
                 * @interface IMethodSettings
                 * @property {string|null} [selector] MethodSettings selector
                 * @property {google.api.MethodSettings.ILongRunning|null} [longRunning] MethodSettings longRunning
                 * @property {Array.<string>|null} [autoPopulatedFields] MethodSettings autoPopulatedFields
                 */
    
                /**
                 * Constructs a new MethodSettings.
                 * @memberof google.api
                 * @classdesc Represents a MethodSettings.
                 * @implements IMethodSettings
                 * @constructor
                 * @param {google.api.IMethodSettings=} [properties] Properties to set
                 */
                function MethodSettings(properties) {
                    this.autoPopulatedFields = [];
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * MethodSettings selector.
                 * @member {string} selector
                 * @memberof google.api.MethodSettings
                 * @instance
                 */
                MethodSettings.prototype.selector = "";
    
                /**
                 * MethodSettings longRunning.
                 * @member {google.api.MethodSettings.ILongRunning|null|undefined} longRunning
                 * @memberof google.api.MethodSettings
                 * @instance
                 */
                MethodSettings.prototype.longRunning = null;
    
                /**
                 * MethodSettings autoPopulatedFields.
                 * @member {Array.<string>} autoPopulatedFields
                 * @memberof google.api.MethodSettings
                 * @instance
                 */
                MethodSettings.prototype.autoPopulatedFields = $util.emptyArray;
    
                /**
                 * Creates a new MethodSettings instance using the specified properties.
                 * @function create
                 * @memberof google.api.MethodSettings
                 * @static
                 * @param {google.api.IMethodSettings=} [properties] Properties to set
                 * @returns {google.api.MethodSettings} MethodSettings instance
                 */
                MethodSettings.create = function create(properties) {
                    return new MethodSettings(properties);
                };
    
                /**
                 * Encodes the specified MethodSettings message. Does not implicitly {@link google.api.MethodSettings.verify|verify} messages.
                 * @function encode
                 * @memberof google.api.MethodSettings
                 * @static
                 * @param {google.api.IMethodSettings} message MethodSettings message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                MethodSettings.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.selector != null && Object.hasOwnProperty.call(message, "selector"))
                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.selector);
                    if (message.longRunning != null && Object.hasOwnProperty.call(message, "longRunning"))
                        $root.google.api.MethodSettings.LongRunning.encode(message.longRunning, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                    if (message.autoPopulatedFields != null && message.autoPopulatedFields.length)
                        for (var i = 0; i < message.autoPopulatedFields.length; ++i)
                            writer.uint32(/* id 3, wireType 2 =*/26).string(message.autoPopulatedFields[i]);
                    return writer;
                };
    
                /**
                 * Encodes the specified MethodSettings message, length delimited. Does not implicitly {@link google.api.MethodSettings.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.api.MethodSettings
                 * @static
                 * @param {google.api.IMethodSettings} message MethodSettings message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                MethodSettings.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a MethodSettings message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.api.MethodSettings
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.api.MethodSettings} MethodSettings
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                MethodSettings.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.api.MethodSettings();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.selector = reader.string();
                                break;
                            }
                        case 2: {
                                message.longRunning = $root.google.api.MethodSettings.LongRunning.decode(reader, reader.uint32());
                                break;
                            }
                        case 3: {
                                if (!(message.autoPopulatedFields && message.autoPopulatedFields.length))
                                    message.autoPopulatedFields = [];
                                message.autoPopulatedFields.push(reader.string());
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a MethodSettings message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.api.MethodSettings
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.api.MethodSettings} MethodSettings
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                MethodSettings.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a MethodSettings message.
                 * @function verify
                 * @memberof google.api.MethodSettings
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                MethodSettings.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.selector != null && message.hasOwnProperty("selector"))
                        if (!$util.isString(message.selector))
                            return "selector: string expected";
                    if (message.longRunning != null && message.hasOwnProperty("longRunning")) {
                        var error = $root.google.api.MethodSettings.LongRunning.verify(message.longRunning);
                        if (error)
                            return "longRunning." + error;
                    }
                    if (message.autoPopulatedFields != null && message.hasOwnProperty("autoPopulatedFields")) {
                        if (!Array.isArray(message.autoPopulatedFields))
                            return "autoPopulatedFields: array expected";
                        for (var i = 0; i < message.autoPopulatedFields.length; ++i)
                            if (!$util.isString(message.autoPopulatedFields[i]))
                                return "autoPopulatedFields: string[] expected";
                    }
                    return null;
                };
    
                /**
                 * Creates a MethodSettings message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.api.MethodSettings
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.api.MethodSettings} MethodSettings
                 */
                MethodSettings.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.api.MethodSettings)
                        return object;
                    var message = new $root.google.api.MethodSettings();
                    if (object.selector != null)
                        message.selector = String(object.selector);
                    if (object.longRunning != null) {
                        if (typeof object.longRunning !== "object")
                            throw TypeError(".google.api.MethodSettings.longRunning: object expected");
                        message.longRunning = $root.google.api.MethodSettings.LongRunning.fromObject(object.longRunning);
                    }
                    if (object.autoPopulatedFields) {
                        if (!Array.isArray(object.autoPopulatedFields))
                            throw TypeError(".google.api.MethodSettings.autoPopulatedFields: array expected");
                        message.autoPopulatedFields = [];
                        for (var i = 0; i < object.autoPopulatedFields.length; ++i)
                            message.autoPopulatedFields[i] = String(object.autoPopulatedFields[i]);
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from a MethodSettings message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.api.MethodSettings
                 * @static
                 * @param {google.api.MethodSettings} message MethodSettings
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                MethodSettings.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.arrays || options.defaults)
                        object.autoPopulatedFields = [];
                    if (options.defaults) {
                        object.selector = "";
                        object.longRunning = null;
                    }
                    if (message.selector != null && message.hasOwnProperty("selector"))
                        object.selector = message.selector;
                    if (message.longRunning != null && message.hasOwnProperty("longRunning"))
                        object.longRunning = $root.google.api.MethodSettings.LongRunning.toObject(message.longRunning, options);
                    if (message.autoPopulatedFields && message.autoPopulatedFields.length) {
                        object.autoPopulatedFields = [];
                        for (var j = 0; j < message.autoPopulatedFields.length; ++j)
                            object.autoPopulatedFields[j] = message.autoPopulatedFields[j];
                    }
                    return object;
                };
    
                /**
                 * Converts this MethodSettings to JSON.
                 * @function toJSON
                 * @memberof google.api.MethodSettings
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                MethodSettings.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for MethodSettings
                 * @function getTypeUrl
                 * @memberof google.api.MethodSettings
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                MethodSettings.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.api.MethodSettings";
                };
    
                MethodSettings.LongRunning = (function() {
    
                    /**
                     * Properties of a LongRunning.
                     * @memberof google.api.MethodSettings
                     * @interface ILongRunning
                     * @property {google.protobuf.IDuration|null} [initialPollDelay] LongRunning initialPollDelay
                     * @property {number|null} [pollDelayMultiplier] LongRunning pollDelayMultiplier
                     * @property {google.protobuf.IDuration|null} [maxPollDelay] LongRunning maxPollDelay
                     * @property {google.protobuf.IDuration|null} [totalPollTimeout] LongRunning totalPollTimeout
                     */
    
                    /**
                     * Constructs a new LongRunning.
                     * @memberof google.api.MethodSettings
                     * @classdesc Represents a LongRunning.
                     * @implements ILongRunning
                     * @constructor
                     * @param {google.api.MethodSettings.ILongRunning=} [properties] Properties to set
                     */
                    function LongRunning(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * LongRunning initialPollDelay.
                     * @member {google.protobuf.IDuration|null|undefined} initialPollDelay
                     * @memberof google.api.MethodSettings.LongRunning
                     * @instance
                     */
                    LongRunning.prototype.initialPollDelay = null;
    
                    /**
                     * LongRunning pollDelayMultiplier.
                     * @member {number} pollDelayMultiplier
                     * @memberof google.api.MethodSettings.LongRunning
                     * @instance
                     */
                    LongRunning.prototype.pollDelayMultiplier = 0;
    
                    /**
                     * LongRunning maxPollDelay.
                     * @member {google.protobuf.IDuration|null|undefined} maxPollDelay
                     * @memberof google.api.MethodSettings.LongRunning
                     * @instance
                     */
                    LongRunning.prototype.maxPollDelay = null;
    
                    /**
                     * LongRunning totalPollTimeout.
                     * @member {google.protobuf.IDuration|null|undefined} totalPollTimeout
                     * @memberof google.api.MethodSettings.LongRunning
                     * @instance
                     */
                    LongRunning.prototype.totalPollTimeout = null;
    
                    /**
                     * Creates a new LongRunning instance using the specified properties.
                     * @function create
                     * @memberof google.api.MethodSettings.LongRunning
                     * @static
                     * @param {google.api.MethodSettings.ILongRunning=} [properties] Properties to set
                     * @returns {google.api.MethodSettings.LongRunning} LongRunning instance
                     */
                    LongRunning.create = function create(properties) {
                        return new LongRunning(properties);
                    };
    
                    /**
                     * Encodes the specified LongRunning message. Does not implicitly {@link google.api.MethodSettings.LongRunning.verify|verify} messages.
                     * @function encode
                     * @memberof google.api.MethodSettings.LongRunning
                     * @static
                     * @param {google.api.MethodSettings.ILongRunning} message LongRunning message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    LongRunning.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.initialPollDelay != null && Object.hasOwnProperty.call(message, "initialPollDelay"))
                            $root.google.protobuf.Duration.encode(message.initialPollDelay, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                        if (message.pollDelayMultiplier != null && Object.hasOwnProperty.call(message, "pollDelayMultiplier"))
                            writer.uint32(/* id 2, wireType 5 =*/21).float(message.pollDelayMultiplier);
                        if (message.maxPollDelay != null && Object.hasOwnProperty.call(message, "maxPollDelay"))
                            $root.google.protobuf.Duration.encode(message.maxPollDelay, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                        if (message.totalPollTimeout != null && Object.hasOwnProperty.call(message, "totalPollTimeout"))
                            $root.google.protobuf.Duration.encode(message.totalPollTimeout, writer.uint32(/* id 4, wireType 2 =*/34).fork()).ldelim();
                        return writer;
                    };
    
                    /**
                     * Encodes the specified LongRunning message, length delimited. Does not implicitly {@link google.api.MethodSettings.LongRunning.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.api.MethodSettings.LongRunning
                     * @static
                     * @param {google.api.MethodSettings.ILongRunning} message LongRunning message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    LongRunning.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a LongRunning message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.api.MethodSettings.LongRunning
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.api.MethodSettings.LongRunning} LongRunning
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    LongRunning.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.api.MethodSettings.LongRunning();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.initialPollDelay = $root.google.protobuf.Duration.decode(reader, reader.uint32());
                                    break;
                                }
                            case 2: {
                                    message.pollDelayMultiplier = reader.float();
                                    break;
                                }
                            case 3: {
                                    message.maxPollDelay = $root.google.protobuf.Duration.decode(reader, reader.uint32());
                                    break;
                                }
                            case 4: {
                                    message.totalPollTimeout = $root.google.protobuf.Duration.decode(reader, reader.uint32());
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a LongRunning message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.api.MethodSettings.LongRunning
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.api.MethodSettings.LongRunning} LongRunning
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    LongRunning.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a LongRunning message.
                     * @function verify
                     * @memberof google.api.MethodSettings.LongRunning
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    LongRunning.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.initialPollDelay != null && message.hasOwnProperty("initialPollDelay")) {
                            var error = $root.google.protobuf.Duration.verify(message.initialPollDelay);
                            if (error)
                                return "initialPollDelay." + error;
                        }
                        if (message.pollDelayMultiplier != null && message.hasOwnProperty("pollDelayMultiplier"))
                            if (typeof message.pollDelayMultiplier !== "number")
                                return "pollDelayMultiplier: number expected";
                        if (message.maxPollDelay != null && message.hasOwnProperty("maxPollDelay")) {
                            var error = $root.google.protobuf.Duration.verify(message.maxPollDelay);
                            if (error)
                                return "maxPollDelay." + error;
                        }
                        if (message.totalPollTimeout != null && message.hasOwnProperty("totalPollTimeout")) {
                            var error = $root.google.protobuf.Duration.verify(message.totalPollTimeout);
                            if (error)
                                return "totalPollTimeout." + error;
                        }
                        return null;
                    };
    
                    /**
                     * Creates a LongRunning message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.api.MethodSettings.LongRunning
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.api.MethodSettings.LongRunning} LongRunning
                     */
                    LongRunning.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.api.MethodSettings.LongRunning)
                            return object;
                        var message = new $root.google.api.MethodSettings.LongRunning();
                        if (object.initialPollDelay != null) {
                            if (typeof object.initialPollDelay !== "object")
                                throw TypeError(".google.api.MethodSettings.LongRunning.initialPollDelay: object expected");
                            message.initialPollDelay = $root.google.protobuf.Duration.fromObject(object.initialPollDelay);
                        }
                        if (object.pollDelayMultiplier != null)
                            message.pollDelayMultiplier = Number(object.pollDelayMultiplier);
                        if (object.maxPollDelay != null) {
                            if (typeof object.maxPollDelay !== "object")
                                throw TypeError(".google.api.MethodSettings.LongRunning.maxPollDelay: object expected");
                            message.maxPollDelay = $root.google.protobuf.Duration.fromObject(object.maxPollDelay);
                        }
                        if (object.totalPollTimeout != null) {
                            if (typeof object.totalPollTimeout !== "object")
                                throw TypeError(".google.api.MethodSettings.LongRunning.totalPollTimeout: object expected");
                            message.totalPollTimeout = $root.google.protobuf.Duration.fromObject(object.totalPollTimeout);
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a LongRunning message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.api.MethodSettings.LongRunning
                     * @static
                     * @param {google.api.MethodSettings.LongRunning} message LongRunning
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    LongRunning.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults) {
                            object.initialPollDelay = null;
                            object.pollDelayMultiplier = 0;
                            object.maxPollDelay = null;
                            object.totalPollTimeout = null;
                        }
                        if (message.initialPollDelay != null && message.hasOwnProperty("initialPollDelay"))
                            object.initialPollDelay = $root.google.protobuf.Duration.toObject(message.initialPollDelay, options);
                        if (message.pollDelayMultiplier != null && message.hasOwnProperty("pollDelayMultiplier"))
                            object.pollDelayMultiplier = options.json && !isFinite(message.pollDelayMultiplier) ? String(message.pollDelayMultiplier) : message.pollDelayMultiplier;
                        if (message.maxPollDelay != null && message.hasOwnProperty("maxPollDelay"))
                            object.maxPollDelay = $root.google.protobuf.Duration.toObject(message.maxPollDelay, options);
                        if (message.totalPollTimeout != null && message.hasOwnProperty("totalPollTimeout"))
                            object.totalPollTimeout = $root.google.protobuf.Duration.toObject(message.totalPollTimeout, options);
                        return object;
                    };
    
                    /**
                     * Converts this LongRunning to JSON.
                     * @function toJSON
                     * @memberof google.api.MethodSettings.LongRunning
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    LongRunning.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for LongRunning
                     * @function getTypeUrl
                     * @memberof google.api.MethodSettings.LongRunning
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    LongRunning.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.api.MethodSettings.LongRunning";
                    };
    
                    return LongRunning;
                })();
    
                return MethodSettings;
            })();
    
            /**
             * ClientLibraryOrganization enum.
             * @name google.api.ClientLibraryOrganization
             * @enum {number}
             * @property {number} CLIENT_LIBRARY_ORGANIZATION_UNSPECIFIED=0 CLIENT_LIBRARY_ORGANIZATION_UNSPECIFIED value
             * @property {number} CLOUD=1 CLOUD value
             * @property {number} ADS=2 ADS value
             * @property {number} PHOTOS=3 PHOTOS value
             * @property {number} STREET_VIEW=4 STREET_VIEW value
             * @property {number} SHOPPING=5 SHOPPING value
             * @property {number} GEO=6 GEO value
             * @property {number} GENERATIVE_AI=7 GENERATIVE_AI value
             */
            api.ClientLibraryOrganization = (function() {
                var valuesById = {}, values = Object.create(valuesById);
                values[valuesById[0] = "CLIENT_LIBRARY_ORGANIZATION_UNSPECIFIED"] = 0;
                values[valuesById[1] = "CLOUD"] = 1;
                values[valuesById[2] = "ADS"] = 2;
                values[valuesById[3] = "PHOTOS"] = 3;
                values[valuesById[4] = "STREET_VIEW"] = 4;
                values[valuesById[5] = "SHOPPING"] = 5;
                values[valuesById[6] = "GEO"] = 6;
                values[valuesById[7] = "GENERATIVE_AI"] = 7;
                return values;
            })();
    
            /**
             * ClientLibraryDestination enum.
             * @name google.api.ClientLibraryDestination
             * @enum {number}
             * @property {number} CLIENT_LIBRARY_DESTINATION_UNSPECIFIED=0 CLIENT_LIBRARY_DESTINATION_UNSPECIFIED value
             * @property {number} GITHUB=10 GITHUB value
             * @property {number} PACKAGE_MANAGER=20 PACKAGE_MANAGER value
             */
            api.ClientLibraryDestination = (function() {
                var valuesById = {}, values = Object.create(valuesById);
                values[valuesById[0] = "CLIENT_LIBRARY_DESTINATION_UNSPECIFIED"] = 0;
                values[valuesById[10] = "GITHUB"] = 10;
                values[valuesById[20] = "PACKAGE_MANAGER"] = 20;
                return values;
            })();
    
            /**
             * LaunchStage enum.
             * @name google.api.LaunchStage
             * @enum {number}
             * @property {number} LAUNCH_STAGE_UNSPECIFIED=0 LAUNCH_STAGE_UNSPECIFIED value
             * @property {number} UNIMPLEMENTED=6 UNIMPLEMENTED value
             * @property {number} PRELAUNCH=7 PRELAUNCH value
             * @property {number} EARLY_ACCESS=1 EARLY_ACCESS value
             * @property {number} ALPHA=2 ALPHA value
             * @property {number} BETA=3 BETA value
             * @property {number} GA=4 GA value
             * @property {number} DEPRECATED=5 DEPRECATED value
             */
            api.LaunchStage = (function() {
                var valuesById = {}, values = Object.create(valuesById);
                values[valuesById[0] = "LAUNCH_STAGE_UNSPECIFIED"] = 0;
                values[valuesById[6] = "UNIMPLEMENTED"] = 6;
                values[valuesById[7] = "PRELAUNCH"] = 7;
                values[valuesById[1] = "EARLY_ACCESS"] = 1;
                values[valuesById[2] = "ALPHA"] = 2;
                values[valuesById[3] = "BETA"] = 3;
                values[valuesById[4] = "GA"] = 4;
                values[valuesById[5] = "DEPRECATED"] = 5;
                return values;
            })();
    
            /**
             * FieldBehavior enum.
             * @name google.api.FieldBehavior
             * @enum {number}
             * @property {number} FIELD_BEHAVIOR_UNSPECIFIED=0 FIELD_BEHAVIOR_UNSPECIFIED value
             * @property {number} OPTIONAL=1 OPTIONAL value
             * @property {number} REQUIRED=2 REQUIRED value
             * @property {number} OUTPUT_ONLY=3 OUTPUT_ONLY value
             * @property {number} INPUT_ONLY=4 INPUT_ONLY value
             * @property {number} IMMUTABLE=5 IMMUTABLE value
             * @property {number} UNORDERED_LIST=6 UNORDERED_LIST value
             * @property {number} NON_EMPTY_DEFAULT=7 NON_EMPTY_DEFAULT value
             * @property {number} IDENTIFIER=8 IDENTIFIER value
             */
            api.FieldBehavior = (function() {
                var valuesById = {}, values = Object.create(valuesById);
                values[valuesById[0] = "FIELD_BEHAVIOR_UNSPECIFIED"] = 0;
                values[valuesById[1] = "OPTIONAL"] = 1;
                values[valuesById[2] = "REQUIRED"] = 2;
                values[valuesById[3] = "OUTPUT_ONLY"] = 3;
                values[valuesById[4] = "INPUT_ONLY"] = 4;
                values[valuesById[5] = "IMMUTABLE"] = 5;
                values[valuesById[6] = "UNORDERED_LIST"] = 6;
                values[valuesById[7] = "NON_EMPTY_DEFAULT"] = 7;
                values[valuesById[8] = "IDENTIFIER"] = 8;
                return values;
            })();
    
            api.ResourceDescriptor = (function() {
    
                /**
                 * Properties of a ResourceDescriptor.
                 * @memberof google.api
                 * @interface IResourceDescriptor
                 * @property {string|null} [type] ResourceDescriptor type
                 * @property {Array.<string>|null} [pattern] ResourceDescriptor pattern
                 * @property {string|null} [nameField] ResourceDescriptor nameField
                 * @property {google.api.ResourceDescriptor.History|null} [history] ResourceDescriptor history
                 * @property {string|null} [plural] ResourceDescriptor plural
                 * @property {string|null} [singular] ResourceDescriptor singular
                 * @property {Array.<google.api.ResourceDescriptor.Style>|null} [style] ResourceDescriptor style
                 */
    
                /**
                 * Constructs a new ResourceDescriptor.
                 * @memberof google.api
                 * @classdesc Represents a ResourceDescriptor.
                 * @implements IResourceDescriptor
                 * @constructor
                 * @param {google.api.IResourceDescriptor=} [properties] Properties to set
                 */
                function ResourceDescriptor(properties) {
                    this.pattern = [];
                    this.style = [];
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * ResourceDescriptor type.
                 * @member {string} type
                 * @memberof google.api.ResourceDescriptor
                 * @instance
                 */
                ResourceDescriptor.prototype.type = "";
    
                /**
                 * ResourceDescriptor pattern.
                 * @member {Array.<string>} pattern
                 * @memberof google.api.ResourceDescriptor
                 * @instance
                 */
                ResourceDescriptor.prototype.pattern = $util.emptyArray;
    
                /**
                 * ResourceDescriptor nameField.
                 * @member {string} nameField
                 * @memberof google.api.ResourceDescriptor
                 * @instance
                 */
                ResourceDescriptor.prototype.nameField = "";
    
                /**
                 * ResourceDescriptor history.
                 * @member {google.api.ResourceDescriptor.History} history
                 * @memberof google.api.ResourceDescriptor
                 * @instance
                 */
                ResourceDescriptor.prototype.history = 0;
    
                /**
                 * ResourceDescriptor plural.
                 * @member {string} plural
                 * @memberof google.api.ResourceDescriptor
                 * @instance
                 */
                ResourceDescriptor.prototype.plural = "";
    
                /**
                 * ResourceDescriptor singular.
                 * @member {string} singular
                 * @memberof google.api.ResourceDescriptor
                 * @instance
                 */
                ResourceDescriptor.prototype.singular = "";
    
                /**
                 * ResourceDescriptor style.
                 * @member {Array.<google.api.ResourceDescriptor.Style>} style
                 * @memberof google.api.ResourceDescriptor
                 * @instance
                 */
                ResourceDescriptor.prototype.style = $util.emptyArray;
    
                /**
                 * Creates a new ResourceDescriptor instance using the specified properties.
                 * @function create
                 * @memberof google.api.ResourceDescriptor
                 * @static
                 * @param {google.api.IResourceDescriptor=} [properties] Properties to set
                 * @returns {google.api.ResourceDescriptor} ResourceDescriptor instance
                 */
                ResourceDescriptor.create = function create(properties) {
                    return new ResourceDescriptor(properties);
                };
    
                /**
                 * Encodes the specified ResourceDescriptor message. Does not implicitly {@link google.api.ResourceDescriptor.verify|verify} messages.
                 * @function encode
                 * @memberof google.api.ResourceDescriptor
                 * @static
                 * @param {google.api.IResourceDescriptor} message ResourceDescriptor message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ResourceDescriptor.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.type != null && Object.hasOwnProperty.call(message, "type"))
                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.type);
                    if (message.pattern != null && message.pattern.length)
                        for (var i = 0; i < message.pattern.length; ++i)
                            writer.uint32(/* id 2, wireType 2 =*/18).string(message.pattern[i]);
                    if (message.nameField != null && Object.hasOwnProperty.call(message, "nameField"))
                        writer.uint32(/* id 3, wireType 2 =*/26).string(message.nameField);
                    if (message.history != null && Object.hasOwnProperty.call(message, "history"))
                        writer.uint32(/* id 4, wireType 0 =*/32).int32(message.history);
                    if (message.plural != null && Object.hasOwnProperty.call(message, "plural"))
                        writer.uint32(/* id 5, wireType 2 =*/42).string(message.plural);
                    if (message.singular != null && Object.hasOwnProperty.call(message, "singular"))
                        writer.uint32(/* id 6, wireType 2 =*/50).string(message.singular);
                    if (message.style != null && message.style.length) {
                        writer.uint32(/* id 10, wireType 2 =*/82).fork();
                        for (var i = 0; i < message.style.length; ++i)
                            writer.int32(message.style[i]);
                        writer.ldelim();
                    }
                    return writer;
                };
    
                /**
                 * Encodes the specified ResourceDescriptor message, length delimited. Does not implicitly {@link google.api.ResourceDescriptor.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.api.ResourceDescriptor
                 * @static
                 * @param {google.api.IResourceDescriptor} message ResourceDescriptor message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ResourceDescriptor.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a ResourceDescriptor message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.api.ResourceDescriptor
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.api.ResourceDescriptor} ResourceDescriptor
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ResourceDescriptor.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.api.ResourceDescriptor();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.type = reader.string();
                                break;
                            }
                        case 2: {
                                if (!(message.pattern && message.pattern.length))
                                    message.pattern = [];
                                message.pattern.push(reader.string());
                                break;
                            }
                        case 3: {
                                message.nameField = reader.string();
                                break;
                            }
                        case 4: {
                                message.history = reader.int32();
                                break;
                            }
                        case 5: {
                                message.plural = reader.string();
                                break;
                            }
                        case 6: {
                                message.singular = reader.string();
                                break;
                            }
                        case 10: {
                                if (!(message.style && message.style.length))
                                    message.style = [];
                                if ((tag & 7) === 2) {
                                    var end2 = reader.uint32() + reader.pos;
                                    while (reader.pos < end2)
                                        message.style.push(reader.int32());
                                } else
                                    message.style.push(reader.int32());
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a ResourceDescriptor message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.api.ResourceDescriptor
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.api.ResourceDescriptor} ResourceDescriptor
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ResourceDescriptor.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a ResourceDescriptor message.
                 * @function verify
                 * @memberof google.api.ResourceDescriptor
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                ResourceDescriptor.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.type != null && message.hasOwnProperty("type"))
                        if (!$util.isString(message.type))
                            return "type: string expected";
                    if (message.pattern != null && message.hasOwnProperty("pattern")) {
                        if (!Array.isArray(message.pattern))
                            return "pattern: array expected";
                        for (var i = 0; i < message.pattern.length; ++i)
                            if (!$util.isString(message.pattern[i]))
                                return "pattern: string[] expected";
                    }
                    if (message.nameField != null && message.hasOwnProperty("nameField"))
                        if (!$util.isString(message.nameField))
                            return "nameField: string expected";
                    if (message.history != null && message.hasOwnProperty("history"))
                        switch (message.history) {
                        default:
                            return "history: enum value expected";
                        case 0:
                        case 1:
                        case 2:
                            break;
                        }
                    if (message.plural != null && message.hasOwnProperty("plural"))
                        if (!$util.isString(message.plural))
                            return "plural: string expected";
                    if (message.singular != null && message.hasOwnProperty("singular"))
                        if (!$util.isString(message.singular))
                            return "singular: string expected";
                    if (message.style != null && message.hasOwnProperty("style")) {
                        if (!Array.isArray(message.style))
                            return "style: array expected";
                        for (var i = 0; i < message.style.length; ++i)
                            switch (message.style[i]) {
                            default:
                                return "style: enum value[] expected";
                            case 0:
                            case 1:
                                break;
                            }
                    }
                    return null;
                };
    
                /**
                 * Creates a ResourceDescriptor message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.api.ResourceDescriptor
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.api.ResourceDescriptor} ResourceDescriptor
                 */
                ResourceDescriptor.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.api.ResourceDescriptor)
                        return object;
                    var message = new $root.google.api.ResourceDescriptor();
                    if (object.type != null)
                        message.type = String(object.type);
                    if (object.pattern) {
                        if (!Array.isArray(object.pattern))
                            throw TypeError(".google.api.ResourceDescriptor.pattern: array expected");
                        message.pattern = [];
                        for (var i = 0; i < object.pattern.length; ++i)
                            message.pattern[i] = String(object.pattern[i]);
                    }
                    if (object.nameField != null)
                        message.nameField = String(object.nameField);
                    switch (object.history) {
                    default:
                        if (typeof object.history === "number") {
                            message.history = object.history;
                            break;
                        }
                        break;
                    case "HISTORY_UNSPECIFIED":
                    case 0:
                        message.history = 0;
                        break;
                    case "ORIGINALLY_SINGLE_PATTERN":
                    case 1:
                        message.history = 1;
                        break;
                    case "FUTURE_MULTI_PATTERN":
                    case 2:
                        message.history = 2;
                        break;
                    }
                    if (object.plural != null)
                        message.plural = String(object.plural);
                    if (object.singular != null)
                        message.singular = String(object.singular);
                    if (object.style) {
                        if (!Array.isArray(object.style))
                            throw TypeError(".google.api.ResourceDescriptor.style: array expected");
                        message.style = [];
                        for (var i = 0; i < object.style.length; ++i)
                            switch (object.style[i]) {
                            default:
                                if (typeof object.style[i] === "number") {
                                    message.style[i] = object.style[i];
                                    break;
                                }
                            case "STYLE_UNSPECIFIED":
                            case 0:
                                message.style[i] = 0;
                                break;
                            case "DECLARATIVE_FRIENDLY":
                            case 1:
                                message.style[i] = 1;
                                break;
                            }
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from a ResourceDescriptor message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.api.ResourceDescriptor
                 * @static
                 * @param {google.api.ResourceDescriptor} message ResourceDescriptor
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                ResourceDescriptor.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.arrays || options.defaults) {
                        object.pattern = [];
                        object.style = [];
                    }
                    if (options.defaults) {
                        object.type = "";
                        object.nameField = "";
                        object.history = options.enums === String ? "HISTORY_UNSPECIFIED" : 0;
                        object.plural = "";
                        object.singular = "";
                    }
                    if (message.type != null && message.hasOwnProperty("type"))
                        object.type = message.type;
                    if (message.pattern && message.pattern.length) {
                        object.pattern = [];
                        for (var j = 0; j < message.pattern.length; ++j)
                            object.pattern[j] = message.pattern[j];
                    }
                    if (message.nameField != null && message.hasOwnProperty("nameField"))
                        object.nameField = message.nameField;
                    if (message.history != null && message.hasOwnProperty("history"))
                        object.history = options.enums === String ? $root.google.api.ResourceDescriptor.History[message.history] === undefined ? message.history : $root.google.api.ResourceDescriptor.History[message.history] : message.history;
                    if (message.plural != null && message.hasOwnProperty("plural"))
                        object.plural = message.plural;
                    if (message.singular != null && message.hasOwnProperty("singular"))
                        object.singular = message.singular;
                    if (message.style && message.style.length) {
                        object.style = [];
                        for (var j = 0; j < message.style.length; ++j)
                            object.style[j] = options.enums === String ? $root.google.api.ResourceDescriptor.Style[message.style[j]] === undefined ? message.style[j] : $root.google.api.ResourceDescriptor.Style[message.style[j]] : message.style[j];
                    }
                    return object;
                };
    
                /**
                 * Converts this ResourceDescriptor to JSON.
                 * @function toJSON
                 * @memberof google.api.ResourceDescriptor
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                ResourceDescriptor.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for ResourceDescriptor
                 * @function getTypeUrl
                 * @memberof google.api.ResourceDescriptor
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                ResourceDescriptor.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.api.ResourceDescriptor";
                };
    
                /**
                 * History enum.
                 * @name google.api.ResourceDescriptor.History
                 * @enum {number}
                 * @property {number} HISTORY_UNSPECIFIED=0 HISTORY_UNSPECIFIED value
                 * @property {number} ORIGINALLY_SINGLE_PATTERN=1 ORIGINALLY_SINGLE_PATTERN value
                 * @property {number} FUTURE_MULTI_PATTERN=2 FUTURE_MULTI_PATTERN value
                 */
                ResourceDescriptor.History = (function() {
                    var valuesById = {}, values = Object.create(valuesById);
                    values[valuesById[0] = "HISTORY_UNSPECIFIED"] = 0;
                    values[valuesById[1] = "ORIGINALLY_SINGLE_PATTERN"] = 1;
                    values[valuesById[2] = "FUTURE_MULTI_PATTERN"] = 2;
                    return values;
                })();
    
                /**
                 * Style enum.
                 * @name google.api.ResourceDescriptor.Style
                 * @enum {number}
                 * @property {number} STYLE_UNSPECIFIED=0 STYLE_UNSPECIFIED value
                 * @property {number} DECLARATIVE_FRIENDLY=1 DECLARATIVE_FRIENDLY value
                 */
                ResourceDescriptor.Style = (function() {
                    var valuesById = {}, values = Object.create(valuesById);
                    values[valuesById[0] = "STYLE_UNSPECIFIED"] = 0;
                    values[valuesById[1] = "DECLARATIVE_FRIENDLY"] = 1;
                    return values;
                })();
    
                return ResourceDescriptor;
            })();
    
            api.ResourceReference = (function() {
    
                /**
                 * Properties of a ResourceReference.
                 * @memberof google.api
                 * @interface IResourceReference
                 * @property {string|null} [type] ResourceReference type
                 * @property {string|null} [childType] ResourceReference childType
                 */
    
                /**
                 * Constructs a new ResourceReference.
                 * @memberof google.api
                 * @classdesc Represents a ResourceReference.
                 * @implements IResourceReference
                 * @constructor
                 * @param {google.api.IResourceReference=} [properties] Properties to set
                 */
                function ResourceReference(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * ResourceReference type.
                 * @member {string} type
                 * @memberof google.api.ResourceReference
                 * @instance
                 */
                ResourceReference.prototype.type = "";
    
                /**
                 * ResourceReference childType.
                 * @member {string} childType
                 * @memberof google.api.ResourceReference
                 * @instance
                 */
                ResourceReference.prototype.childType = "";
    
                /**
                 * Creates a new ResourceReference instance using the specified properties.
                 * @function create
                 * @memberof google.api.ResourceReference
                 * @static
                 * @param {google.api.IResourceReference=} [properties] Properties to set
                 * @returns {google.api.ResourceReference} ResourceReference instance
                 */
                ResourceReference.create = function create(properties) {
                    return new ResourceReference(properties);
                };
    
                /**
                 * Encodes the specified ResourceReference message. Does not implicitly {@link google.api.ResourceReference.verify|verify} messages.
                 * @function encode
                 * @memberof google.api.ResourceReference
                 * @static
                 * @param {google.api.IResourceReference} message ResourceReference message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ResourceReference.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.type != null && Object.hasOwnProperty.call(message, "type"))
                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.type);
                    if (message.childType != null && Object.hasOwnProperty.call(message, "childType"))
                        writer.uint32(/* id 2, wireType 2 =*/18).string(message.childType);
                    return writer;
                };
    
                /**
                 * Encodes the specified ResourceReference message, length delimited. Does not implicitly {@link google.api.ResourceReference.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.api.ResourceReference
                 * @static
                 * @param {google.api.IResourceReference} message ResourceReference message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ResourceReference.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a ResourceReference message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.api.ResourceReference
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.api.ResourceReference} ResourceReference
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ResourceReference.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.api.ResourceReference();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.type = reader.string();
                                break;
                            }
                        case 2: {
                                message.childType = reader.string();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a ResourceReference message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.api.ResourceReference
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.api.ResourceReference} ResourceReference
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ResourceReference.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a ResourceReference message.
                 * @function verify
                 * @memberof google.api.ResourceReference
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                ResourceReference.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.type != null && message.hasOwnProperty("type"))
                        if (!$util.isString(message.type))
                            return "type: string expected";
                    if (message.childType != null && message.hasOwnProperty("childType"))
                        if (!$util.isString(message.childType))
                            return "childType: string expected";
                    return null;
                };
    
                /**
                 * Creates a ResourceReference message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.api.ResourceReference
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.api.ResourceReference} ResourceReference
                 */
                ResourceReference.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.api.ResourceReference)
                        return object;
                    var message = new $root.google.api.ResourceReference();
                    if (object.type != null)
                        message.type = String(object.type);
                    if (object.childType != null)
                        message.childType = String(object.childType);
                    return message;
                };
    
                /**
                 * Creates a plain object from a ResourceReference message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.api.ResourceReference
                 * @static
                 * @param {google.api.ResourceReference} message ResourceReference
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                ResourceReference.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults) {
                        object.type = "";
                        object.childType = "";
                    }
                    if (message.type != null && message.hasOwnProperty("type"))
                        object.type = message.type;
                    if (message.childType != null && message.hasOwnProperty("childType"))
                        object.childType = message.childType;
                    return object;
                };
    
                /**
                 * Converts this ResourceReference to JSON.
                 * @function toJSON
                 * @memberof google.api.ResourceReference
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                ResourceReference.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for ResourceReference
                 * @function getTypeUrl
                 * @memberof google.api.ResourceReference
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                ResourceReference.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.api.ResourceReference";
                };
    
                return ResourceReference;
            })();
    
            api.RoutingRule = (function() {
    
                /**
                 * Properties of a RoutingRule.
                 * @memberof google.api
                 * @interface IRoutingRule
                 * @property {Array.<google.api.IRoutingParameter>|null} [routingParameters] RoutingRule routingParameters
                 */
    
                /**
                 * Constructs a new RoutingRule.
                 * @memberof google.api
                 * @classdesc Represents a RoutingRule.
                 * @implements IRoutingRule
                 * @constructor
                 * @param {google.api.IRoutingRule=} [properties] Properties to set
                 */
                function RoutingRule(properties) {
                    this.routingParameters = [];
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * RoutingRule routingParameters.
                 * @member {Array.<google.api.IRoutingParameter>} routingParameters
                 * @memberof google.api.RoutingRule
                 * @instance
                 */
                RoutingRule.prototype.routingParameters = $util.emptyArray;
    
                /**
                 * Creates a new RoutingRule instance using the specified properties.
                 * @function create
                 * @memberof google.api.RoutingRule
                 * @static
                 * @param {google.api.IRoutingRule=} [properties] Properties to set
                 * @returns {google.api.RoutingRule} RoutingRule instance
                 */
                RoutingRule.create = function create(properties) {
                    return new RoutingRule(properties);
                };
    
                /**
                 * Encodes the specified RoutingRule message. Does not implicitly {@link google.api.RoutingRule.verify|verify} messages.
                 * @function encode
                 * @memberof google.api.RoutingRule
                 * @static
                 * @param {google.api.IRoutingRule} message RoutingRule message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                RoutingRule.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.routingParameters != null && message.routingParameters.length)
                        for (var i = 0; i < message.routingParameters.length; ++i)
                            $root.google.api.RoutingParameter.encode(message.routingParameters[i], writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                    return writer;
                };
    
                /**
                 * Encodes the specified RoutingRule message, length delimited. Does not implicitly {@link google.api.RoutingRule.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.api.RoutingRule
                 * @static
                 * @param {google.api.IRoutingRule} message RoutingRule message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                RoutingRule.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a RoutingRule message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.api.RoutingRule
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.api.RoutingRule} RoutingRule
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                RoutingRule.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.api.RoutingRule();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 2: {
                                if (!(message.routingParameters && message.routingParameters.length))
                                    message.routingParameters = [];
                                message.routingParameters.push($root.google.api.RoutingParameter.decode(reader, reader.uint32()));
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a RoutingRule message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.api.RoutingRule
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.api.RoutingRule} RoutingRule
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                RoutingRule.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a RoutingRule message.
                 * @function verify
                 * @memberof google.api.RoutingRule
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                RoutingRule.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.routingParameters != null && message.hasOwnProperty("routingParameters")) {
                        if (!Array.isArray(message.routingParameters))
                            return "routingParameters: array expected";
                        for (var i = 0; i < message.routingParameters.length; ++i) {
                            var error = $root.google.api.RoutingParameter.verify(message.routingParameters[i]);
                            if (error)
                                return "routingParameters." + error;
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates a RoutingRule message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.api.RoutingRule
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.api.RoutingRule} RoutingRule
                 */
                RoutingRule.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.api.RoutingRule)
                        return object;
                    var message = new $root.google.api.RoutingRule();
                    if (object.routingParameters) {
                        if (!Array.isArray(object.routingParameters))
                            throw TypeError(".google.api.RoutingRule.routingParameters: array expected");
                        message.routingParameters = [];
                        for (var i = 0; i < object.routingParameters.length; ++i) {
                            if (typeof object.routingParameters[i] !== "object")
                                throw TypeError(".google.api.RoutingRule.routingParameters: object expected");
                            message.routingParameters[i] = $root.google.api.RoutingParameter.fromObject(object.routingParameters[i]);
                        }
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from a RoutingRule message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.api.RoutingRule
                 * @static
                 * @param {google.api.RoutingRule} message RoutingRule
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                RoutingRule.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.arrays || options.defaults)
                        object.routingParameters = [];
                    if (message.routingParameters && message.routingParameters.length) {
                        object.routingParameters = [];
                        for (var j = 0; j < message.routingParameters.length; ++j)
                            object.routingParameters[j] = $root.google.api.RoutingParameter.toObject(message.routingParameters[j], options);
                    }
                    return object;
                };
    
                /**
                 * Converts this RoutingRule to JSON.
                 * @function toJSON
                 * @memberof google.api.RoutingRule
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                RoutingRule.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for RoutingRule
                 * @function getTypeUrl
                 * @memberof google.api.RoutingRule
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                RoutingRule.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.api.RoutingRule";
                };
    
                return RoutingRule;
            })();
    
            api.RoutingParameter = (function() {
    
                /**
                 * Properties of a RoutingParameter.
                 * @memberof google.api
                 * @interface IRoutingParameter
                 * @property {string|null} [field] RoutingParameter field
                 * @property {string|null} [pathTemplate] RoutingParameter pathTemplate
                 */
    
                /**
                 * Constructs a new RoutingParameter.
                 * @memberof google.api
                 * @classdesc Represents a RoutingParameter.
                 * @implements IRoutingParameter
                 * @constructor
                 * @param {google.api.IRoutingParameter=} [properties] Properties to set
                 */
                function RoutingParameter(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * RoutingParameter field.
                 * @member {string} field
                 * @memberof google.api.RoutingParameter
                 * @instance
                 */
                RoutingParameter.prototype.field = "";
    
                /**
                 * RoutingParameter pathTemplate.
                 * @member {string} pathTemplate
                 * @memberof google.api.RoutingParameter
                 * @instance
                 */
                RoutingParameter.prototype.pathTemplate = "";
    
                /**
                 * Creates a new RoutingParameter instance using the specified properties.
                 * @function create
                 * @memberof google.api.RoutingParameter
                 * @static
                 * @param {google.api.IRoutingParameter=} [properties] Properties to set
                 * @returns {google.api.RoutingParameter} RoutingParameter instance
                 */
                RoutingParameter.create = function create(properties) {
                    return new RoutingParameter(properties);
                };
    
                /**
                 * Encodes the specified RoutingParameter message. Does not implicitly {@link google.api.RoutingParameter.verify|verify} messages.
                 * @function encode
                 * @memberof google.api.RoutingParameter
                 * @static
                 * @param {google.api.IRoutingParameter} message RoutingParameter message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                RoutingParameter.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.field != null && Object.hasOwnProperty.call(message, "field"))
                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.field);
                    if (message.pathTemplate != null && Object.hasOwnProperty.call(message, "pathTemplate"))
                        writer.uint32(/* id 2, wireType 2 =*/18).string(message.pathTemplate);
                    return writer;
                };
    
                /**
                 * Encodes the specified RoutingParameter message, length delimited. Does not implicitly {@link google.api.RoutingParameter.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.api.RoutingParameter
                 * @static
                 * @param {google.api.IRoutingParameter} message RoutingParameter message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                RoutingParameter.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a RoutingParameter message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.api.RoutingParameter
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.api.RoutingParameter} RoutingParameter
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                RoutingParameter.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.api.RoutingParameter();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.field = reader.string();
                                break;
                            }
                        case 2: {
                                message.pathTemplate = reader.string();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a RoutingParameter message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.api.RoutingParameter
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.api.RoutingParameter} RoutingParameter
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                RoutingParameter.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a RoutingParameter message.
                 * @function verify
                 * @memberof google.api.RoutingParameter
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                RoutingParameter.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.field != null && message.hasOwnProperty("field"))
                        if (!$util.isString(message.field))
                            return "field: string expected";
                    if (message.pathTemplate != null && message.hasOwnProperty("pathTemplate"))
                        if (!$util.isString(message.pathTemplate))
                            return "pathTemplate: string expected";
                    return null;
                };
    
                /**
                 * Creates a RoutingParameter message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.api.RoutingParameter
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.api.RoutingParameter} RoutingParameter
                 */
                RoutingParameter.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.api.RoutingParameter)
                        return object;
                    var message = new $root.google.api.RoutingParameter();
                    if (object.field != null)
                        message.field = String(object.field);
                    if (object.pathTemplate != null)
                        message.pathTemplate = String(object.pathTemplate);
                    return message;
                };
    
                /**
                 * Creates a plain object from a RoutingParameter message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.api.RoutingParameter
                 * @static
                 * @param {google.api.RoutingParameter} message RoutingParameter
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                RoutingParameter.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults) {
                        object.field = "";
                        object.pathTemplate = "";
                    }
                    if (message.field != null && message.hasOwnProperty("field"))
                        object.field = message.field;
                    if (message.pathTemplate != null && message.hasOwnProperty("pathTemplate"))
                        object.pathTemplate = message.pathTemplate;
                    return object;
                };
    
                /**
                 * Converts this RoutingParameter to JSON.
                 * @function toJSON
                 * @memberof google.api.RoutingParameter
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                RoutingParameter.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for RoutingParameter
                 * @function getTypeUrl
                 * @memberof google.api.RoutingParameter
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                RoutingParameter.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.api.RoutingParameter";
                };
    
                return RoutingParameter;
            })();
    
            return api;
        })();
    
        google.protobuf = (function() {
    
            /**
             * Namespace protobuf.
             * @memberof google
             * @namespace
             */
            var protobuf = {};
    
            protobuf.FileDescriptorSet = (function() {
    
                /**
                 * Properties of a FileDescriptorSet.
                 * @memberof google.protobuf
                 * @interface IFileDescriptorSet
                 * @property {Array.<google.protobuf.IFileDescriptorProto>|null} [file] FileDescriptorSet file
                 */
    
                /**
                 * Constructs a new FileDescriptorSet.
                 * @memberof google.protobuf
                 * @classdesc Represents a FileDescriptorSet.
                 * @implements IFileDescriptorSet
                 * @constructor
                 * @param {google.protobuf.IFileDescriptorSet=} [properties] Properties to set
                 */
                function FileDescriptorSet(properties) {
                    this.file = [];
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * FileDescriptorSet file.
                 * @member {Array.<google.protobuf.IFileDescriptorProto>} file
                 * @memberof google.protobuf.FileDescriptorSet
                 * @instance
                 */
                FileDescriptorSet.prototype.file = $util.emptyArray;
    
                /**
                 * Creates a new FileDescriptorSet instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.FileDescriptorSet
                 * @static
                 * @param {google.protobuf.IFileDescriptorSet=} [properties] Properties to set
                 * @returns {google.protobuf.FileDescriptorSet} FileDescriptorSet instance
                 */
                FileDescriptorSet.create = function create(properties) {
                    return new FileDescriptorSet(properties);
                };
    
                /**
                 * Encodes the specified FileDescriptorSet message. Does not implicitly {@link google.protobuf.FileDescriptorSet.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.FileDescriptorSet
                 * @static
                 * @param {google.protobuf.IFileDescriptorSet} message FileDescriptorSet message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FileDescriptorSet.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.file != null && message.file.length)
                        for (var i = 0; i < message.file.length; ++i)
                            $root.google.protobuf.FileDescriptorProto.encode(message.file[i], writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                    return writer;
                };
    
                /**
                 * Encodes the specified FileDescriptorSet message, length delimited. Does not implicitly {@link google.protobuf.FileDescriptorSet.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.FileDescriptorSet
                 * @static
                 * @param {google.protobuf.IFileDescriptorSet} message FileDescriptorSet message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FileDescriptorSet.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a FileDescriptorSet message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.FileDescriptorSet
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.FileDescriptorSet} FileDescriptorSet
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FileDescriptorSet.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.FileDescriptorSet();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                if (!(message.file && message.file.length))
                                    message.file = [];
                                message.file.push($root.google.protobuf.FileDescriptorProto.decode(reader, reader.uint32()));
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a FileDescriptorSet message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.FileDescriptorSet
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.FileDescriptorSet} FileDescriptorSet
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FileDescriptorSet.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a FileDescriptorSet message.
                 * @function verify
                 * @memberof google.protobuf.FileDescriptorSet
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                FileDescriptorSet.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.file != null && message.hasOwnProperty("file")) {
                        if (!Array.isArray(message.file))
                            return "file: array expected";
                        for (var i = 0; i < message.file.length; ++i) {
                            var error = $root.google.protobuf.FileDescriptorProto.verify(message.file[i]);
                            if (error)
                                return "file." + error;
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates a FileDescriptorSet message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.FileDescriptorSet
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.FileDescriptorSet} FileDescriptorSet
                 */
                FileDescriptorSet.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.FileDescriptorSet)
                        return object;
                    var message = new $root.google.protobuf.FileDescriptorSet();
                    if (object.file) {
                        if (!Array.isArray(object.file))
                            throw TypeError(".google.protobuf.FileDescriptorSet.file: array expected");
                        message.file = [];
                        for (var i = 0; i < object.file.length; ++i) {
                            if (typeof object.file[i] !== "object")
                                throw TypeError(".google.protobuf.FileDescriptorSet.file: object expected");
                            message.file[i] = $root.google.protobuf.FileDescriptorProto.fromObject(object.file[i]);
                        }
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from a FileDescriptorSet message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.FileDescriptorSet
                 * @static
                 * @param {google.protobuf.FileDescriptorSet} message FileDescriptorSet
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                FileDescriptorSet.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.arrays || options.defaults)
                        object.file = [];
                    if (message.file && message.file.length) {
                        object.file = [];
                        for (var j = 0; j < message.file.length; ++j)
                            object.file[j] = $root.google.protobuf.FileDescriptorProto.toObject(message.file[j], options);
                    }
                    return object;
                };
    
                /**
                 * Converts this FileDescriptorSet to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.FileDescriptorSet
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                FileDescriptorSet.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for FileDescriptorSet
                 * @function getTypeUrl
                 * @memberof google.protobuf.FileDescriptorSet
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                FileDescriptorSet.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.FileDescriptorSet";
                };
    
                return FileDescriptorSet;
            })();
    
            /**
             * Edition enum.
             * @name google.protobuf.Edition
             * @enum {number}
             * @property {number} EDITION_UNKNOWN=0 EDITION_UNKNOWN value
             * @property {number} EDITION_PROTO2=998 EDITION_PROTO2 value
             * @property {number} EDITION_PROTO3=999 EDITION_PROTO3 value
             * @property {number} EDITION_2023=1000 EDITION_2023 value
             * @property {number} EDITION_2024=1001 EDITION_2024 value
             * @property {number} EDITION_1_TEST_ONLY=1 EDITION_1_TEST_ONLY value
             * @property {number} EDITION_2_TEST_ONLY=2 EDITION_2_TEST_ONLY value
             * @property {number} EDITION_99997_TEST_ONLY=99997 EDITION_99997_TEST_ONLY value
             * @property {number} EDITION_99998_TEST_ONLY=99998 EDITION_99998_TEST_ONLY value
             * @property {number} EDITION_99999_TEST_ONLY=99999 EDITION_99999_TEST_ONLY value
             * @property {number} EDITION_MAX=2147483647 EDITION_MAX value
             */
            protobuf.Edition = (function() {
                var valuesById = {}, values = Object.create(valuesById);
                values[valuesById[0] = "EDITION_UNKNOWN"] = 0;
                values[valuesById[998] = "EDITION_PROTO2"] = 998;
                values[valuesById[999] = "EDITION_PROTO3"] = 999;
                values[valuesById[1000] = "EDITION_2023"] = 1000;
                values[valuesById[1001] = "EDITION_2024"] = 1001;
                values[valuesById[1] = "EDITION_1_TEST_ONLY"] = 1;
                values[valuesById[2] = "EDITION_2_TEST_ONLY"] = 2;
                values[valuesById[99997] = "EDITION_99997_TEST_ONLY"] = 99997;
                values[valuesById[99998] = "EDITION_99998_TEST_ONLY"] = 99998;
                values[valuesById[99999] = "EDITION_99999_TEST_ONLY"] = 99999;
                values[valuesById[2147483647] = "EDITION_MAX"] = 2147483647;
                return values;
            })();
    
            protobuf.FileDescriptorProto = (function() {
    
                /**
                 * Properties of a FileDescriptorProto.
                 * @memberof google.protobuf
                 * @interface IFileDescriptorProto
                 * @property {string|null} [name] FileDescriptorProto name
                 * @property {string|null} ["package"] FileDescriptorProto package
                 * @property {Array.<string>|null} [dependency] FileDescriptorProto dependency
                 * @property {Array.<number>|null} [publicDependency] FileDescriptorProto publicDependency
                 * @property {Array.<number>|null} [weakDependency] FileDescriptorProto weakDependency
                 * @property {Array.<google.protobuf.IDescriptorProto>|null} [messageType] FileDescriptorProto messageType
                 * @property {Array.<google.protobuf.IEnumDescriptorProto>|null} [enumType] FileDescriptorProto enumType
                 * @property {Array.<google.protobuf.IServiceDescriptorProto>|null} [service] FileDescriptorProto service
                 * @property {Array.<google.protobuf.IFieldDescriptorProto>|null} [extension] FileDescriptorProto extension
                 * @property {google.protobuf.IFileOptions|null} [options] FileDescriptorProto options
                 * @property {google.protobuf.ISourceCodeInfo|null} [sourceCodeInfo] FileDescriptorProto sourceCodeInfo
                 * @property {string|null} [syntax] FileDescriptorProto syntax
                 * @property {google.protobuf.Edition|null} [edition] FileDescriptorProto edition
                 */
    
                /**
                 * Constructs a new FileDescriptorProto.
                 * @memberof google.protobuf
                 * @classdesc Represents a FileDescriptorProto.
                 * @implements IFileDescriptorProto
                 * @constructor
                 * @param {google.protobuf.IFileDescriptorProto=} [properties] Properties to set
                 */
                function FileDescriptorProto(properties) {
                    this.dependency = [];
                    this.publicDependency = [];
                    this.weakDependency = [];
                    this.messageType = [];
                    this.enumType = [];
                    this.service = [];
                    this.extension = [];
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * FileDescriptorProto name.
                 * @member {string} name
                 * @memberof google.protobuf.FileDescriptorProto
                 * @instance
                 */
                FileDescriptorProto.prototype.name = "";
    
                /**
                 * FileDescriptorProto package.
                 * @member {string} package
                 * @memberof google.protobuf.FileDescriptorProto
                 * @instance
                 */
                FileDescriptorProto.prototype["package"] = "";
    
                /**
                 * FileDescriptorProto dependency.
                 * @member {Array.<string>} dependency
                 * @memberof google.protobuf.FileDescriptorProto
                 * @instance
                 */
                FileDescriptorProto.prototype.dependency = $util.emptyArray;
    
                /**
                 * FileDescriptorProto publicDependency.
                 * @member {Array.<number>} publicDependency
                 * @memberof google.protobuf.FileDescriptorProto
                 * @instance
                 */
                FileDescriptorProto.prototype.publicDependency = $util.emptyArray;
    
                /**
                 * FileDescriptorProto weakDependency.
                 * @member {Array.<number>} weakDependency
                 * @memberof google.protobuf.FileDescriptorProto
                 * @instance
                 */
                FileDescriptorProto.prototype.weakDependency = $util.emptyArray;
    
                /**
                 * FileDescriptorProto messageType.
                 * @member {Array.<google.protobuf.IDescriptorProto>} messageType
                 * @memberof google.protobuf.FileDescriptorProto
                 * @instance
                 */
                FileDescriptorProto.prototype.messageType = $util.emptyArray;
    
                /**
                 * FileDescriptorProto enumType.
                 * @member {Array.<google.protobuf.IEnumDescriptorProto>} enumType
                 * @memberof google.protobuf.FileDescriptorProto
                 * @instance
                 */
                FileDescriptorProto.prototype.enumType = $util.emptyArray;
    
                /**
                 * FileDescriptorProto service.
                 * @member {Array.<google.protobuf.IServiceDescriptorProto>} service
                 * @memberof google.protobuf.FileDescriptorProto
                 * @instance
                 */
                FileDescriptorProto.prototype.service = $util.emptyArray;
    
                /**
                 * FileDescriptorProto extension.
                 * @member {Array.<google.protobuf.IFieldDescriptorProto>} extension
                 * @memberof google.protobuf.FileDescriptorProto
                 * @instance
                 */
                FileDescriptorProto.prototype.extension = $util.emptyArray;
    
                /**
                 * FileDescriptorProto options.
                 * @member {google.protobuf.IFileOptions|null|undefined} options
                 * @memberof google.protobuf.FileDescriptorProto
                 * @instance
                 */
                FileDescriptorProto.prototype.options = null;
    
                /**
                 * FileDescriptorProto sourceCodeInfo.
                 * @member {google.protobuf.ISourceCodeInfo|null|undefined} sourceCodeInfo
                 * @memberof google.protobuf.FileDescriptorProto
                 * @instance
                 */
                FileDescriptorProto.prototype.sourceCodeInfo = null;
    
                /**
                 * FileDescriptorProto syntax.
                 * @member {string} syntax
                 * @memberof google.protobuf.FileDescriptorProto
                 * @instance
                 */
                FileDescriptorProto.prototype.syntax = "";
    
                /**
                 * FileDescriptorProto edition.
                 * @member {google.protobuf.Edition} edition
                 * @memberof google.protobuf.FileDescriptorProto
                 * @instance
                 */
                FileDescriptorProto.prototype.edition = 0;
    
                /**
                 * Creates a new FileDescriptorProto instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.FileDescriptorProto
                 * @static
                 * @param {google.protobuf.IFileDescriptorProto=} [properties] Properties to set
                 * @returns {google.protobuf.FileDescriptorProto} FileDescriptorProto instance
                 */
                FileDescriptorProto.create = function create(properties) {
                    return new FileDescriptorProto(properties);
                };
    
                /**
                 * Encodes the specified FileDescriptorProto message. Does not implicitly {@link google.protobuf.FileDescriptorProto.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.FileDescriptorProto
                 * @static
                 * @param {google.protobuf.IFileDescriptorProto} message FileDescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FileDescriptorProto.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                    if (message["package"] != null && Object.hasOwnProperty.call(message, "package"))
                        writer.uint32(/* id 2, wireType 2 =*/18).string(message["package"]);
                    if (message.dependency != null && message.dependency.length)
                        for (var i = 0; i < message.dependency.length; ++i)
                            writer.uint32(/* id 3, wireType 2 =*/26).string(message.dependency[i]);
                    if (message.messageType != null && message.messageType.length)
                        for (var i = 0; i < message.messageType.length; ++i)
                            $root.google.protobuf.DescriptorProto.encode(message.messageType[i], writer.uint32(/* id 4, wireType 2 =*/34).fork()).ldelim();
                    if (message.enumType != null && message.enumType.length)
                        for (var i = 0; i < message.enumType.length; ++i)
                            $root.google.protobuf.EnumDescriptorProto.encode(message.enumType[i], writer.uint32(/* id 5, wireType 2 =*/42).fork()).ldelim();
                    if (message.service != null && message.service.length)
                        for (var i = 0; i < message.service.length; ++i)
                            $root.google.protobuf.ServiceDescriptorProto.encode(message.service[i], writer.uint32(/* id 6, wireType 2 =*/50).fork()).ldelim();
                    if (message.extension != null && message.extension.length)
                        for (var i = 0; i < message.extension.length; ++i)
                            $root.google.protobuf.FieldDescriptorProto.encode(message.extension[i], writer.uint32(/* id 7, wireType 2 =*/58).fork()).ldelim();
                    if (message.options != null && Object.hasOwnProperty.call(message, "options"))
                        $root.google.protobuf.FileOptions.encode(message.options, writer.uint32(/* id 8, wireType 2 =*/66).fork()).ldelim();
                    if (message.sourceCodeInfo != null && Object.hasOwnProperty.call(message, "sourceCodeInfo"))
                        $root.google.protobuf.SourceCodeInfo.encode(message.sourceCodeInfo, writer.uint32(/* id 9, wireType 2 =*/74).fork()).ldelim();
                    if (message.publicDependency != null && message.publicDependency.length)
                        for (var i = 0; i < message.publicDependency.length; ++i)
                            writer.uint32(/* id 10, wireType 0 =*/80).int32(message.publicDependency[i]);
                    if (message.weakDependency != null && message.weakDependency.length)
                        for (var i = 0; i < message.weakDependency.length; ++i)
                            writer.uint32(/* id 11, wireType 0 =*/88).int32(message.weakDependency[i]);
                    if (message.syntax != null && Object.hasOwnProperty.call(message, "syntax"))
                        writer.uint32(/* id 12, wireType 2 =*/98).string(message.syntax);
                    if (message.edition != null && Object.hasOwnProperty.call(message, "edition"))
                        writer.uint32(/* id 14, wireType 0 =*/112).int32(message.edition);
                    return writer;
                };
    
                /**
                 * Encodes the specified FileDescriptorProto message, length delimited. Does not implicitly {@link google.protobuf.FileDescriptorProto.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.FileDescriptorProto
                 * @static
                 * @param {google.protobuf.IFileDescriptorProto} message FileDescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FileDescriptorProto.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a FileDescriptorProto message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.FileDescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.FileDescriptorProto} FileDescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FileDescriptorProto.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.FileDescriptorProto();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.name = reader.string();
                                break;
                            }
                        case 2: {
                                message["package"] = reader.string();
                                break;
                            }
                        case 3: {
                                if (!(message.dependency && message.dependency.length))
                                    message.dependency = [];
                                message.dependency.push(reader.string());
                                break;
                            }
                        case 10: {
                                if (!(message.publicDependency && message.publicDependency.length))
                                    message.publicDependency = [];
                                if ((tag & 7) === 2) {
                                    var end2 = reader.uint32() + reader.pos;
                                    while (reader.pos < end2)
                                        message.publicDependency.push(reader.int32());
                                } else
                                    message.publicDependency.push(reader.int32());
                                break;
                            }
                        case 11: {
                                if (!(message.weakDependency && message.weakDependency.length))
                                    message.weakDependency = [];
                                if ((tag & 7) === 2) {
                                    var end2 = reader.uint32() + reader.pos;
                                    while (reader.pos < end2)
                                        message.weakDependency.push(reader.int32());
                                } else
                                    message.weakDependency.push(reader.int32());
                                break;
                            }
                        case 4: {
                                if (!(message.messageType && message.messageType.length))
                                    message.messageType = [];
                                message.messageType.push($root.google.protobuf.DescriptorProto.decode(reader, reader.uint32()));
                                break;
                            }
                        case 5: {
                                if (!(message.enumType && message.enumType.length))
                                    message.enumType = [];
                                message.enumType.push($root.google.protobuf.EnumDescriptorProto.decode(reader, reader.uint32()));
                                break;
                            }
                        case 6: {
                                if (!(message.service && message.service.length))
                                    message.service = [];
                                message.service.push($root.google.protobuf.ServiceDescriptorProto.decode(reader, reader.uint32()));
                                break;
                            }
                        case 7: {
                                if (!(message.extension && message.extension.length))
                                    message.extension = [];
                                message.extension.push($root.google.protobuf.FieldDescriptorProto.decode(reader, reader.uint32()));
                                break;
                            }
                        case 8: {
                                message.options = $root.google.protobuf.FileOptions.decode(reader, reader.uint32());
                                break;
                            }
                        case 9: {
                                message.sourceCodeInfo = $root.google.protobuf.SourceCodeInfo.decode(reader, reader.uint32());
                                break;
                            }
                        case 12: {
                                message.syntax = reader.string();
                                break;
                            }
                        case 14: {
                                message.edition = reader.int32();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a FileDescriptorProto message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.FileDescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.FileDescriptorProto} FileDescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FileDescriptorProto.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a FileDescriptorProto message.
                 * @function verify
                 * @memberof google.protobuf.FileDescriptorProto
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                FileDescriptorProto.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.name != null && message.hasOwnProperty("name"))
                        if (!$util.isString(message.name))
                            return "name: string expected";
                    if (message["package"] != null && message.hasOwnProperty("package"))
                        if (!$util.isString(message["package"]))
                            return "package: string expected";
                    if (message.dependency != null && message.hasOwnProperty("dependency")) {
                        if (!Array.isArray(message.dependency))
                            return "dependency: array expected";
                        for (var i = 0; i < message.dependency.length; ++i)
                            if (!$util.isString(message.dependency[i]))
                                return "dependency: string[] expected";
                    }
                    if (message.publicDependency != null && message.hasOwnProperty("publicDependency")) {
                        if (!Array.isArray(message.publicDependency))
                            return "publicDependency: array expected";
                        for (var i = 0; i < message.publicDependency.length; ++i)
                            if (!$util.isInteger(message.publicDependency[i]))
                                return "publicDependency: integer[] expected";
                    }
                    if (message.weakDependency != null && message.hasOwnProperty("weakDependency")) {
                        if (!Array.isArray(message.weakDependency))
                            return "weakDependency: array expected";
                        for (var i = 0; i < message.weakDependency.length; ++i)
                            if (!$util.isInteger(message.weakDependency[i]))
                                return "weakDependency: integer[] expected";
                    }
                    if (message.messageType != null && message.hasOwnProperty("messageType")) {
                        if (!Array.isArray(message.messageType))
                            return "messageType: array expected";
                        for (var i = 0; i < message.messageType.length; ++i) {
                            var error = $root.google.protobuf.DescriptorProto.verify(message.messageType[i]);
                            if (error)
                                return "messageType." + error;
                        }
                    }
                    if (message.enumType != null && message.hasOwnProperty("enumType")) {
                        if (!Array.isArray(message.enumType))
                            return "enumType: array expected";
                        for (var i = 0; i < message.enumType.length; ++i) {
                            var error = $root.google.protobuf.EnumDescriptorProto.verify(message.enumType[i]);
                            if (error)
                                return "enumType." + error;
                        }
                    }
                    if (message.service != null && message.hasOwnProperty("service")) {
                        if (!Array.isArray(message.service))
                            return "service: array expected";
                        for (var i = 0; i < message.service.length; ++i) {
                            var error = $root.google.protobuf.ServiceDescriptorProto.verify(message.service[i]);
                            if (error)
                                return "service." + error;
                        }
                    }
                    if (message.extension != null && message.hasOwnProperty("extension")) {
                        if (!Array.isArray(message.extension))
                            return "extension: array expected";
                        for (var i = 0; i < message.extension.length; ++i) {
                            var error = $root.google.protobuf.FieldDescriptorProto.verify(message.extension[i]);
                            if (error)
                                return "extension." + error;
                        }
                    }
                    if (message.options != null && message.hasOwnProperty("options")) {
                        var error = $root.google.protobuf.FileOptions.verify(message.options);
                        if (error)
                            return "options." + error;
                    }
                    if (message.sourceCodeInfo != null && message.hasOwnProperty("sourceCodeInfo")) {
                        var error = $root.google.protobuf.SourceCodeInfo.verify(message.sourceCodeInfo);
                        if (error)
                            return "sourceCodeInfo." + error;
                    }
                    if (message.syntax != null && message.hasOwnProperty("syntax"))
                        if (!$util.isString(message.syntax))
                            return "syntax: string expected";
                    if (message.edition != null && message.hasOwnProperty("edition"))
                        switch (message.edition) {
                        default:
                            return "edition: enum value expected";
                        case 0:
                        case 998:
                        case 999:
                        case 1000:
                        case 1001:
                        case 1:
                        case 2:
                        case 99997:
                        case 99998:
                        case 99999:
                        case 2147483647:
                            break;
                        }
                    return null;
                };
    
                /**
                 * Creates a FileDescriptorProto message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.FileDescriptorProto
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.FileDescriptorProto} FileDescriptorProto
                 */
                FileDescriptorProto.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.FileDescriptorProto)
                        return object;
                    var message = new $root.google.protobuf.FileDescriptorProto();
                    if (object.name != null)
                        message.name = String(object.name);
                    if (object["package"] != null)
                        message["package"] = String(object["package"]);
                    if (object.dependency) {
                        if (!Array.isArray(object.dependency))
                            throw TypeError(".google.protobuf.FileDescriptorProto.dependency: array expected");
                        message.dependency = [];
                        for (var i = 0; i < object.dependency.length; ++i)
                            message.dependency[i] = String(object.dependency[i]);
                    }
                    if (object.publicDependency) {
                        if (!Array.isArray(object.publicDependency))
                            throw TypeError(".google.protobuf.FileDescriptorProto.publicDependency: array expected");
                        message.publicDependency = [];
                        for (var i = 0; i < object.publicDependency.length; ++i)
                            message.publicDependency[i] = object.publicDependency[i] | 0;
                    }
                    if (object.weakDependency) {
                        if (!Array.isArray(object.weakDependency))
                            throw TypeError(".google.protobuf.FileDescriptorProto.weakDependency: array expected");
                        message.weakDependency = [];
                        for (var i = 0; i < object.weakDependency.length; ++i)
                            message.weakDependency[i] = object.weakDependency[i] | 0;
                    }
                    if (object.messageType) {
                        if (!Array.isArray(object.messageType))
                            throw TypeError(".google.protobuf.FileDescriptorProto.messageType: array expected");
                        message.messageType = [];
                        for (var i = 0; i < object.messageType.length; ++i) {
                            if (typeof object.messageType[i] !== "object")
                                throw TypeError(".google.protobuf.FileDescriptorProto.messageType: object expected");
                            message.messageType[i] = $root.google.protobuf.DescriptorProto.fromObject(object.messageType[i]);
                        }
                    }
                    if (object.enumType) {
                        if (!Array.isArray(object.enumType))
                            throw TypeError(".google.protobuf.FileDescriptorProto.enumType: array expected");
                        message.enumType = [];
                        for (var i = 0; i < object.enumType.length; ++i) {
                            if (typeof object.enumType[i] !== "object")
                                throw TypeError(".google.protobuf.FileDescriptorProto.enumType: object expected");
                            message.enumType[i] = $root.google.protobuf.EnumDescriptorProto.fromObject(object.enumType[i]);
                        }
                    }
                    if (object.service) {
                        if (!Array.isArray(object.service))
                            throw TypeError(".google.protobuf.FileDescriptorProto.service: array expected");
                        message.service = [];
                        for (var i = 0; i < object.service.length; ++i) {
                            if (typeof object.service[i] !== "object")
                                throw TypeError(".google.protobuf.FileDescriptorProto.service: object expected");
                            message.service[i] = $root.google.protobuf.ServiceDescriptorProto.fromObject(object.service[i]);
                        }
                    }
                    if (object.extension) {
                        if (!Array.isArray(object.extension))
                            throw TypeError(".google.protobuf.FileDescriptorProto.extension: array expected");
                        message.extension = [];
                        for (var i = 0; i < object.extension.length; ++i) {
                            if (typeof object.extension[i] !== "object")
                                throw TypeError(".google.protobuf.FileDescriptorProto.extension: object expected");
                            message.extension[i] = $root.google.protobuf.FieldDescriptorProto.fromObject(object.extension[i]);
                        }
                    }
                    if (object.options != null) {
                        if (typeof object.options !== "object")
                            throw TypeError(".google.protobuf.FileDescriptorProto.options: object expected");
                        message.options = $root.google.protobuf.FileOptions.fromObject(object.options);
                    }
                    if (object.sourceCodeInfo != null) {
                        if (typeof object.sourceCodeInfo !== "object")
                            throw TypeError(".google.protobuf.FileDescriptorProto.sourceCodeInfo: object expected");
                        message.sourceCodeInfo = $root.google.protobuf.SourceCodeInfo.fromObject(object.sourceCodeInfo);
                    }
                    if (object.syntax != null)
                        message.syntax = String(object.syntax);
                    switch (object.edition) {
                    default:
                        if (typeof object.edition === "number") {
                            message.edition = object.edition;
                            break;
                        }
                        break;
                    case "EDITION_UNKNOWN":
                    case 0:
                        message.edition = 0;
                        break;
                    case "EDITION_PROTO2":
                    case 998:
                        message.edition = 998;
                        break;
                    case "EDITION_PROTO3":
                    case 999:
                        message.edition = 999;
                        break;
                    case "EDITION_2023":
                    case 1000:
                        message.edition = 1000;
                        break;
                    case "EDITION_2024":
                    case 1001:
                        message.edition = 1001;
                        break;
                    case "EDITION_1_TEST_ONLY":
                    case 1:
                        message.edition = 1;
                        break;
                    case "EDITION_2_TEST_ONLY":
                    case 2:
                        message.edition = 2;
                        break;
                    case "EDITION_99997_TEST_ONLY":
                    case 99997:
                        message.edition = 99997;
                        break;
                    case "EDITION_99998_TEST_ONLY":
                    case 99998:
                        message.edition = 99998;
                        break;
                    case "EDITION_99999_TEST_ONLY":
                    case 99999:
                        message.edition = 99999;
                        break;
                    case "EDITION_MAX":
                    case 2147483647:
                        message.edition = 2147483647;
                        break;
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from a FileDescriptorProto message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.FileDescriptorProto
                 * @static
                 * @param {google.protobuf.FileDescriptorProto} message FileDescriptorProto
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                FileDescriptorProto.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.arrays || options.defaults) {
                        object.dependency = [];
                        object.messageType = [];
                        object.enumType = [];
                        object.service = [];
                        object.extension = [];
                        object.publicDependency = [];
                        object.weakDependency = [];
                    }
                    if (options.defaults) {
                        object.name = "";
                        object["package"] = "";
                        object.options = null;
                        object.sourceCodeInfo = null;
                        object.syntax = "";
                        object.edition = options.enums === String ? "EDITION_UNKNOWN" : 0;
                    }
                    if (message.name != null && message.hasOwnProperty("name"))
                        object.name = message.name;
                    if (message["package"] != null && message.hasOwnProperty("package"))
                        object["package"] = message["package"];
                    if (message.dependency && message.dependency.length) {
                        object.dependency = [];
                        for (var j = 0; j < message.dependency.length; ++j)
                            object.dependency[j] = message.dependency[j];
                    }
                    if (message.messageType && message.messageType.length) {
                        object.messageType = [];
                        for (var j = 0; j < message.messageType.length; ++j)
                            object.messageType[j] = $root.google.protobuf.DescriptorProto.toObject(message.messageType[j], options);
                    }
                    if (message.enumType && message.enumType.length) {
                        object.enumType = [];
                        for (var j = 0; j < message.enumType.length; ++j)
                            object.enumType[j] = $root.google.protobuf.EnumDescriptorProto.toObject(message.enumType[j], options);
                    }
                    if (message.service && message.service.length) {
                        object.service = [];
                        for (var j = 0; j < message.service.length; ++j)
                            object.service[j] = $root.google.protobuf.ServiceDescriptorProto.toObject(message.service[j], options);
                    }
                    if (message.extension && message.extension.length) {
                        object.extension = [];
                        for (var j = 0; j < message.extension.length; ++j)
                            object.extension[j] = $root.google.protobuf.FieldDescriptorProto.toObject(message.extension[j], options);
                    }
                    if (message.options != null && message.hasOwnProperty("options"))
                        object.options = $root.google.protobuf.FileOptions.toObject(message.options, options);
                    if (message.sourceCodeInfo != null && message.hasOwnProperty("sourceCodeInfo"))
                        object.sourceCodeInfo = $root.google.protobuf.SourceCodeInfo.toObject(message.sourceCodeInfo, options);
                    if (message.publicDependency && message.publicDependency.length) {
                        object.publicDependency = [];
                        for (var j = 0; j < message.publicDependency.length; ++j)
                            object.publicDependency[j] = message.publicDependency[j];
                    }
                    if (message.weakDependency && message.weakDependency.length) {
                        object.weakDependency = [];
                        for (var j = 0; j < message.weakDependency.length; ++j)
                            object.weakDependency[j] = message.weakDependency[j];
                    }
                    if (message.syntax != null && message.hasOwnProperty("syntax"))
                        object.syntax = message.syntax;
                    if (message.edition != null && message.hasOwnProperty("edition"))
                        object.edition = options.enums === String ? $root.google.protobuf.Edition[message.edition] === undefined ? message.edition : $root.google.protobuf.Edition[message.edition] : message.edition;
                    return object;
                };
    
                /**
                 * Converts this FileDescriptorProto to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.FileDescriptorProto
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                FileDescriptorProto.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for FileDescriptorProto
                 * @function getTypeUrl
                 * @memberof google.protobuf.FileDescriptorProto
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                FileDescriptorProto.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.FileDescriptorProto";
                };
    
                return FileDescriptorProto;
            })();
    
            protobuf.DescriptorProto = (function() {
    
                /**
                 * Properties of a DescriptorProto.
                 * @memberof google.protobuf
                 * @interface IDescriptorProto
                 * @property {string|null} [name] DescriptorProto name
                 * @property {Array.<google.protobuf.IFieldDescriptorProto>|null} [field] DescriptorProto field
                 * @property {Array.<google.protobuf.IFieldDescriptorProto>|null} [extension] DescriptorProto extension
                 * @property {Array.<google.protobuf.IDescriptorProto>|null} [nestedType] DescriptorProto nestedType
                 * @property {Array.<google.protobuf.IEnumDescriptorProto>|null} [enumType] DescriptorProto enumType
                 * @property {Array.<google.protobuf.DescriptorProto.IExtensionRange>|null} [extensionRange] DescriptorProto extensionRange
                 * @property {Array.<google.protobuf.IOneofDescriptorProto>|null} [oneofDecl] DescriptorProto oneofDecl
                 * @property {google.protobuf.IMessageOptions|null} [options] DescriptorProto options
                 * @property {Array.<google.protobuf.DescriptorProto.IReservedRange>|null} [reservedRange] DescriptorProto reservedRange
                 * @property {Array.<string>|null} [reservedName] DescriptorProto reservedName
                 */
    
                /**
                 * Constructs a new DescriptorProto.
                 * @memberof google.protobuf
                 * @classdesc Represents a DescriptorProto.
                 * @implements IDescriptorProto
                 * @constructor
                 * @param {google.protobuf.IDescriptorProto=} [properties] Properties to set
                 */
                function DescriptorProto(properties) {
                    this.field = [];
                    this.extension = [];
                    this.nestedType = [];
                    this.enumType = [];
                    this.extensionRange = [];
                    this.oneofDecl = [];
                    this.reservedRange = [];
                    this.reservedName = [];
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * DescriptorProto name.
                 * @member {string} name
                 * @memberof google.protobuf.DescriptorProto
                 * @instance
                 */
                DescriptorProto.prototype.name = "";
    
                /**
                 * DescriptorProto field.
                 * @member {Array.<google.protobuf.IFieldDescriptorProto>} field
                 * @memberof google.protobuf.DescriptorProto
                 * @instance
                 */
                DescriptorProto.prototype.field = $util.emptyArray;
    
                /**
                 * DescriptorProto extension.
                 * @member {Array.<google.protobuf.IFieldDescriptorProto>} extension
                 * @memberof google.protobuf.DescriptorProto
                 * @instance
                 */
                DescriptorProto.prototype.extension = $util.emptyArray;
    
                /**
                 * DescriptorProto nestedType.
                 * @member {Array.<google.protobuf.IDescriptorProto>} nestedType
                 * @memberof google.protobuf.DescriptorProto
                 * @instance
                 */
                DescriptorProto.prototype.nestedType = $util.emptyArray;
    
                /**
                 * DescriptorProto enumType.
                 * @member {Array.<google.protobuf.IEnumDescriptorProto>} enumType
                 * @memberof google.protobuf.DescriptorProto
                 * @instance
                 */
                DescriptorProto.prototype.enumType = $util.emptyArray;
    
                /**
                 * DescriptorProto extensionRange.
                 * @member {Array.<google.protobuf.DescriptorProto.IExtensionRange>} extensionRange
                 * @memberof google.protobuf.DescriptorProto
                 * @instance
                 */
                DescriptorProto.prototype.extensionRange = $util.emptyArray;
    
                /**
                 * DescriptorProto oneofDecl.
                 * @member {Array.<google.protobuf.IOneofDescriptorProto>} oneofDecl
                 * @memberof google.protobuf.DescriptorProto
                 * @instance
                 */
                DescriptorProto.prototype.oneofDecl = $util.emptyArray;
    
                /**
                 * DescriptorProto options.
                 * @member {google.protobuf.IMessageOptions|null|undefined} options
                 * @memberof google.protobuf.DescriptorProto
                 * @instance
                 */
                DescriptorProto.prototype.options = null;
    
                /**
                 * DescriptorProto reservedRange.
                 * @member {Array.<google.protobuf.DescriptorProto.IReservedRange>} reservedRange
                 * @memberof google.protobuf.DescriptorProto
                 * @instance
                 */
                DescriptorProto.prototype.reservedRange = $util.emptyArray;
    
                /**
                 * DescriptorProto reservedName.
                 * @member {Array.<string>} reservedName
                 * @memberof google.protobuf.DescriptorProto
                 * @instance
                 */
                DescriptorProto.prototype.reservedName = $util.emptyArray;
    
                /**
                 * Creates a new DescriptorProto instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.DescriptorProto
                 * @static
                 * @param {google.protobuf.IDescriptorProto=} [properties] Properties to set
                 * @returns {google.protobuf.DescriptorProto} DescriptorProto instance
                 */
                DescriptorProto.create = function create(properties) {
                    return new DescriptorProto(properties);
                };
    
                /**
                 * Encodes the specified DescriptorProto message. Does not implicitly {@link google.protobuf.DescriptorProto.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.DescriptorProto
                 * @static
                 * @param {google.protobuf.IDescriptorProto} message DescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                DescriptorProto.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                    if (message.field != null && message.field.length)
                        for (var i = 0; i < message.field.length; ++i)
                            $root.google.protobuf.FieldDescriptorProto.encode(message.field[i], writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                    if (message.nestedType != null && message.nestedType.length)
                        for (var i = 0; i < message.nestedType.length; ++i)
                            $root.google.protobuf.DescriptorProto.encode(message.nestedType[i], writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                    if (message.enumType != null && message.enumType.length)
                        for (var i = 0; i < message.enumType.length; ++i)
                            $root.google.protobuf.EnumDescriptorProto.encode(message.enumType[i], writer.uint32(/* id 4, wireType 2 =*/34).fork()).ldelim();
                    if (message.extensionRange != null && message.extensionRange.length)
                        for (var i = 0; i < message.extensionRange.length; ++i)
                            $root.google.protobuf.DescriptorProto.ExtensionRange.encode(message.extensionRange[i], writer.uint32(/* id 5, wireType 2 =*/42).fork()).ldelim();
                    if (message.extension != null && message.extension.length)
                        for (var i = 0; i < message.extension.length; ++i)
                            $root.google.protobuf.FieldDescriptorProto.encode(message.extension[i], writer.uint32(/* id 6, wireType 2 =*/50).fork()).ldelim();
                    if (message.options != null && Object.hasOwnProperty.call(message, "options"))
                        $root.google.protobuf.MessageOptions.encode(message.options, writer.uint32(/* id 7, wireType 2 =*/58).fork()).ldelim();
                    if (message.oneofDecl != null && message.oneofDecl.length)
                        for (var i = 0; i < message.oneofDecl.length; ++i)
                            $root.google.protobuf.OneofDescriptorProto.encode(message.oneofDecl[i], writer.uint32(/* id 8, wireType 2 =*/66).fork()).ldelim();
                    if (message.reservedRange != null && message.reservedRange.length)
                        for (var i = 0; i < message.reservedRange.length; ++i)
                            $root.google.protobuf.DescriptorProto.ReservedRange.encode(message.reservedRange[i], writer.uint32(/* id 9, wireType 2 =*/74).fork()).ldelim();
                    if (message.reservedName != null && message.reservedName.length)
                        for (var i = 0; i < message.reservedName.length; ++i)
                            writer.uint32(/* id 10, wireType 2 =*/82).string(message.reservedName[i]);
                    return writer;
                };
    
                /**
                 * Encodes the specified DescriptorProto message, length delimited. Does not implicitly {@link google.protobuf.DescriptorProto.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.DescriptorProto
                 * @static
                 * @param {google.protobuf.IDescriptorProto} message DescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                DescriptorProto.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a DescriptorProto message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.DescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.DescriptorProto} DescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                DescriptorProto.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.DescriptorProto();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.name = reader.string();
                                break;
                            }
                        case 2: {
                                if (!(message.field && message.field.length))
                                    message.field = [];
                                message.field.push($root.google.protobuf.FieldDescriptorProto.decode(reader, reader.uint32()));
                                break;
                            }
                        case 6: {
                                if (!(message.extension && message.extension.length))
                                    message.extension = [];
                                message.extension.push($root.google.protobuf.FieldDescriptorProto.decode(reader, reader.uint32()));
                                break;
                            }
                        case 3: {
                                if (!(message.nestedType && message.nestedType.length))
                                    message.nestedType = [];
                                message.nestedType.push($root.google.protobuf.DescriptorProto.decode(reader, reader.uint32()));
                                break;
                            }
                        case 4: {
                                if (!(message.enumType && message.enumType.length))
                                    message.enumType = [];
                                message.enumType.push($root.google.protobuf.EnumDescriptorProto.decode(reader, reader.uint32()));
                                break;
                            }
                        case 5: {
                                if (!(message.extensionRange && message.extensionRange.length))
                                    message.extensionRange = [];
                                message.extensionRange.push($root.google.protobuf.DescriptorProto.ExtensionRange.decode(reader, reader.uint32()));
                                break;
                            }
                        case 8: {
                                if (!(message.oneofDecl && message.oneofDecl.length))
                                    message.oneofDecl = [];
                                message.oneofDecl.push($root.google.protobuf.OneofDescriptorProto.decode(reader, reader.uint32()));
                                break;
                            }
                        case 7: {
                                message.options = $root.google.protobuf.MessageOptions.decode(reader, reader.uint32());
                                break;
                            }
                        case 9: {
                                if (!(message.reservedRange && message.reservedRange.length))
                                    message.reservedRange = [];
                                message.reservedRange.push($root.google.protobuf.DescriptorProto.ReservedRange.decode(reader, reader.uint32()));
                                break;
                            }
                        case 10: {
                                if (!(message.reservedName && message.reservedName.length))
                                    message.reservedName = [];
                                message.reservedName.push(reader.string());
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a DescriptorProto message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.DescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.DescriptorProto} DescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                DescriptorProto.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a DescriptorProto message.
                 * @function verify
                 * @memberof google.protobuf.DescriptorProto
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                DescriptorProto.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.name != null && message.hasOwnProperty("name"))
                        if (!$util.isString(message.name))
                            return "name: string expected";
                    if (message.field != null && message.hasOwnProperty("field")) {
                        if (!Array.isArray(message.field))
                            return "field: array expected";
                        for (var i = 0; i < message.field.length; ++i) {
                            var error = $root.google.protobuf.FieldDescriptorProto.verify(message.field[i]);
                            if (error)
                                return "field." + error;
                        }
                    }
                    if (message.extension != null && message.hasOwnProperty("extension")) {
                        if (!Array.isArray(message.extension))
                            return "extension: array expected";
                        for (var i = 0; i < message.extension.length; ++i) {
                            var error = $root.google.protobuf.FieldDescriptorProto.verify(message.extension[i]);
                            if (error)
                                return "extension." + error;
                        }
                    }
                    if (message.nestedType != null && message.hasOwnProperty("nestedType")) {
                        if (!Array.isArray(message.nestedType))
                            return "nestedType: array expected";
                        for (var i = 0; i < message.nestedType.length; ++i) {
                            var error = $root.google.protobuf.DescriptorProto.verify(message.nestedType[i]);
                            if (error)
                                return "nestedType." + error;
                        }
                    }
                    if (message.enumType != null && message.hasOwnProperty("enumType")) {
                        if (!Array.isArray(message.enumType))
                            return "enumType: array expected";
                        for (var i = 0; i < message.enumType.length; ++i) {
                            var error = $root.google.protobuf.EnumDescriptorProto.verify(message.enumType[i]);
                            if (error)
                                return "enumType." + error;
                        }
                    }
                    if (message.extensionRange != null && message.hasOwnProperty("extensionRange")) {
                        if (!Array.isArray(message.extensionRange))
                            return "extensionRange: array expected";
                        for (var i = 0; i < message.extensionRange.length; ++i) {
                            var error = $root.google.protobuf.DescriptorProto.ExtensionRange.verify(message.extensionRange[i]);
                            if (error)
                                return "extensionRange." + error;
                        }
                    }
                    if (message.oneofDecl != null && message.hasOwnProperty("oneofDecl")) {
                        if (!Array.isArray(message.oneofDecl))
                            return "oneofDecl: array expected";
                        for (var i = 0; i < message.oneofDecl.length; ++i) {
                            var error = $root.google.protobuf.OneofDescriptorProto.verify(message.oneofDecl[i]);
                            if (error)
                                return "oneofDecl." + error;
                        }
                    }
                    if (message.options != null && message.hasOwnProperty("options")) {
                        var error = $root.google.protobuf.MessageOptions.verify(message.options);
                        if (error)
                            return "options." + error;
                    }
                    if (message.reservedRange != null && message.hasOwnProperty("reservedRange")) {
                        if (!Array.isArray(message.reservedRange))
                            return "reservedRange: array expected";
                        for (var i = 0; i < message.reservedRange.length; ++i) {
                            var error = $root.google.protobuf.DescriptorProto.ReservedRange.verify(message.reservedRange[i]);
                            if (error)
                                return "reservedRange." + error;
                        }
                    }
                    if (message.reservedName != null && message.hasOwnProperty("reservedName")) {
                        if (!Array.isArray(message.reservedName))
                            return "reservedName: array expected";
                        for (var i = 0; i < message.reservedName.length; ++i)
                            if (!$util.isString(message.reservedName[i]))
                                return "reservedName: string[] expected";
                    }
                    return null;
                };
    
                /**
                 * Creates a DescriptorProto message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.DescriptorProto
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.DescriptorProto} DescriptorProto
                 */
                DescriptorProto.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.DescriptorProto)
                        return object;
                    var message = new $root.google.protobuf.DescriptorProto();
                    if (object.name != null)
                        message.name = String(object.name);
                    if (object.field) {
                        if (!Array.isArray(object.field))
                            throw TypeError(".google.protobuf.DescriptorProto.field: array expected");
                        message.field = [];
                        for (var i = 0; i < object.field.length; ++i) {
                            if (typeof object.field[i] !== "object")
                                throw TypeError(".google.protobuf.DescriptorProto.field: object expected");
                            message.field[i] = $root.google.protobuf.FieldDescriptorProto.fromObject(object.field[i]);
                        }
                    }
                    if (object.extension) {
                        if (!Array.isArray(object.extension))
                            throw TypeError(".google.protobuf.DescriptorProto.extension: array expected");
                        message.extension = [];
                        for (var i = 0; i < object.extension.length; ++i) {
                            if (typeof object.extension[i] !== "object")
                                throw TypeError(".google.protobuf.DescriptorProto.extension: object expected");
                            message.extension[i] = $root.google.protobuf.FieldDescriptorProto.fromObject(object.extension[i]);
                        }
                    }
                    if (object.nestedType) {
                        if (!Array.isArray(object.nestedType))
                            throw TypeError(".google.protobuf.DescriptorProto.nestedType: array expected");
                        message.nestedType = [];
                        for (var i = 0; i < object.nestedType.length; ++i) {
                            if (typeof object.nestedType[i] !== "object")
                                throw TypeError(".google.protobuf.DescriptorProto.nestedType: object expected");
                            message.nestedType[i] = $root.google.protobuf.DescriptorProto.fromObject(object.nestedType[i]);
                        }
                    }
                    if (object.enumType) {
                        if (!Array.isArray(object.enumType))
                            throw TypeError(".google.protobuf.DescriptorProto.enumType: array expected");
                        message.enumType = [];
                        for (var i = 0; i < object.enumType.length; ++i) {
                            if (typeof object.enumType[i] !== "object")
                                throw TypeError(".google.protobuf.DescriptorProto.enumType: object expected");
                            message.enumType[i] = $root.google.protobuf.EnumDescriptorProto.fromObject(object.enumType[i]);
                        }
                    }
                    if (object.extensionRange) {
                        if (!Array.isArray(object.extensionRange))
                            throw TypeError(".google.protobuf.DescriptorProto.extensionRange: array expected");
                        message.extensionRange = [];
                        for (var i = 0; i < object.extensionRange.length; ++i) {
                            if (typeof object.extensionRange[i] !== "object")
                                throw TypeError(".google.protobuf.DescriptorProto.extensionRange: object expected");
                            message.extensionRange[i] = $root.google.protobuf.DescriptorProto.ExtensionRange.fromObject(object.extensionRange[i]);
                        }
                    }
                    if (object.oneofDecl) {
                        if (!Array.isArray(object.oneofDecl))
                            throw TypeError(".google.protobuf.DescriptorProto.oneofDecl: array expected");
                        message.oneofDecl = [];
                        for (var i = 0; i < object.oneofDecl.length; ++i) {
                            if (typeof object.oneofDecl[i] !== "object")
                                throw TypeError(".google.protobuf.DescriptorProto.oneofDecl: object expected");
                            message.oneofDecl[i] = $root.google.protobuf.OneofDescriptorProto.fromObject(object.oneofDecl[i]);
                        }
                    }
                    if (object.options != null) {
                        if (typeof object.options !== "object")
                            throw TypeError(".google.protobuf.DescriptorProto.options: object expected");
                        message.options = $root.google.protobuf.MessageOptions.fromObject(object.options);
                    }
                    if (object.reservedRange) {
                        if (!Array.isArray(object.reservedRange))
                            throw TypeError(".google.protobuf.DescriptorProto.reservedRange: array expected");
                        message.reservedRange = [];
                        for (var i = 0; i < object.reservedRange.length; ++i) {
                            if (typeof object.reservedRange[i] !== "object")
                                throw TypeError(".google.protobuf.DescriptorProto.reservedRange: object expected");
                            message.reservedRange[i] = $root.google.protobuf.DescriptorProto.ReservedRange.fromObject(object.reservedRange[i]);
                        }
                    }
                    if (object.reservedName) {
                        if (!Array.isArray(object.reservedName))
                            throw TypeError(".google.protobuf.DescriptorProto.reservedName: array expected");
                        message.reservedName = [];
                        for (var i = 0; i < object.reservedName.length; ++i)
                            message.reservedName[i] = String(object.reservedName[i]);
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from a DescriptorProto message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.DescriptorProto
                 * @static
                 * @param {google.protobuf.DescriptorProto} message DescriptorProto
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                DescriptorProto.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.arrays || options.defaults) {
                        object.field = [];
                        object.nestedType = [];
                        object.enumType = [];
                        object.extensionRange = [];
                        object.extension = [];
                        object.oneofDecl = [];
                        object.reservedRange = [];
                        object.reservedName = [];
                    }
                    if (options.defaults) {
                        object.name = "";
                        object.options = null;
                    }
                    if (message.name != null && message.hasOwnProperty("name"))
                        object.name = message.name;
                    if (message.field && message.field.length) {
                        object.field = [];
                        for (var j = 0; j < message.field.length; ++j)
                            object.field[j] = $root.google.protobuf.FieldDescriptorProto.toObject(message.field[j], options);
                    }
                    if (message.nestedType && message.nestedType.length) {
                        object.nestedType = [];
                        for (var j = 0; j < message.nestedType.length; ++j)
                            object.nestedType[j] = $root.google.protobuf.DescriptorProto.toObject(message.nestedType[j], options);
                    }
                    if (message.enumType && message.enumType.length) {
                        object.enumType = [];
                        for (var j = 0; j < message.enumType.length; ++j)
                            object.enumType[j] = $root.google.protobuf.EnumDescriptorProto.toObject(message.enumType[j], options);
                    }
                    if (message.extensionRange && message.extensionRange.length) {
                        object.extensionRange = [];
                        for (var j = 0; j < message.extensionRange.length; ++j)
                            object.extensionRange[j] = $root.google.protobuf.DescriptorProto.ExtensionRange.toObject(message.extensionRange[j], options);
                    }
                    if (message.extension && message.extension.length) {
                        object.extension = [];
                        for (var j = 0; j < message.extension.length; ++j)
                            object.extension[j] = $root.google.protobuf.FieldDescriptorProto.toObject(message.extension[j], options);
                    }
                    if (message.options != null && message.hasOwnProperty("options"))
                        object.options = $root.google.protobuf.MessageOptions.toObject(message.options, options);
                    if (message.oneofDecl && message.oneofDecl.length) {
                        object.oneofDecl = [];
                        for (var j = 0; j < message.oneofDecl.length; ++j)
                            object.oneofDecl[j] = $root.google.protobuf.OneofDescriptorProto.toObject(message.oneofDecl[j], options);
                    }
                    if (message.reservedRange && message.reservedRange.length) {
                        object.reservedRange = [];
                        for (var j = 0; j < message.reservedRange.length; ++j)
                            object.reservedRange[j] = $root.google.protobuf.DescriptorProto.ReservedRange.toObject(message.reservedRange[j], options);
                    }
                    if (message.reservedName && message.reservedName.length) {
                        object.reservedName = [];
                        for (var j = 0; j < message.reservedName.length; ++j)
                            object.reservedName[j] = message.reservedName[j];
                    }
                    return object;
                };
    
                /**
                 * Converts this DescriptorProto to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.DescriptorProto
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                DescriptorProto.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for DescriptorProto
                 * @function getTypeUrl
                 * @memberof google.protobuf.DescriptorProto
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                DescriptorProto.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.DescriptorProto";
                };
    
                DescriptorProto.ExtensionRange = (function() {
    
                    /**
                     * Properties of an ExtensionRange.
                     * @memberof google.protobuf.DescriptorProto
                     * @interface IExtensionRange
                     * @property {number|null} [start] ExtensionRange start
                     * @property {number|null} [end] ExtensionRange end
                     * @property {google.protobuf.IExtensionRangeOptions|null} [options] ExtensionRange options
                     */
    
                    /**
                     * Constructs a new ExtensionRange.
                     * @memberof google.protobuf.DescriptorProto
                     * @classdesc Represents an ExtensionRange.
                     * @implements IExtensionRange
                     * @constructor
                     * @param {google.protobuf.DescriptorProto.IExtensionRange=} [properties] Properties to set
                     */
                    function ExtensionRange(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * ExtensionRange start.
                     * @member {number} start
                     * @memberof google.protobuf.DescriptorProto.ExtensionRange
                     * @instance
                     */
                    ExtensionRange.prototype.start = 0;
    
                    /**
                     * ExtensionRange end.
                     * @member {number} end
                     * @memberof google.protobuf.DescriptorProto.ExtensionRange
                     * @instance
                     */
                    ExtensionRange.prototype.end = 0;
    
                    /**
                     * ExtensionRange options.
                     * @member {google.protobuf.IExtensionRangeOptions|null|undefined} options
                     * @memberof google.protobuf.DescriptorProto.ExtensionRange
                     * @instance
                     */
                    ExtensionRange.prototype.options = null;
    
                    /**
                     * Creates a new ExtensionRange instance using the specified properties.
                     * @function create
                     * @memberof google.protobuf.DescriptorProto.ExtensionRange
                     * @static
                     * @param {google.protobuf.DescriptorProto.IExtensionRange=} [properties] Properties to set
                     * @returns {google.protobuf.DescriptorProto.ExtensionRange} ExtensionRange instance
                     */
                    ExtensionRange.create = function create(properties) {
                        return new ExtensionRange(properties);
                    };
    
                    /**
                     * Encodes the specified ExtensionRange message. Does not implicitly {@link google.protobuf.DescriptorProto.ExtensionRange.verify|verify} messages.
                     * @function encode
                     * @memberof google.protobuf.DescriptorProto.ExtensionRange
                     * @static
                     * @param {google.protobuf.DescriptorProto.IExtensionRange} message ExtensionRange message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ExtensionRange.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.start != null && Object.hasOwnProperty.call(message, "start"))
                            writer.uint32(/* id 1, wireType 0 =*/8).int32(message.start);
                        if (message.end != null && Object.hasOwnProperty.call(message, "end"))
                            writer.uint32(/* id 2, wireType 0 =*/16).int32(message.end);
                        if (message.options != null && Object.hasOwnProperty.call(message, "options"))
                            $root.google.protobuf.ExtensionRangeOptions.encode(message.options, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                        return writer;
                    };
    
                    /**
                     * Encodes the specified ExtensionRange message, length delimited. Does not implicitly {@link google.protobuf.DescriptorProto.ExtensionRange.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.protobuf.DescriptorProto.ExtensionRange
                     * @static
                     * @param {google.protobuf.DescriptorProto.IExtensionRange} message ExtensionRange message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ExtensionRange.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes an ExtensionRange message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.protobuf.DescriptorProto.ExtensionRange
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.protobuf.DescriptorProto.ExtensionRange} ExtensionRange
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ExtensionRange.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.DescriptorProto.ExtensionRange();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.start = reader.int32();
                                    break;
                                }
                            case 2: {
                                    message.end = reader.int32();
                                    break;
                                }
                            case 3: {
                                    message.options = $root.google.protobuf.ExtensionRangeOptions.decode(reader, reader.uint32());
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes an ExtensionRange message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.protobuf.DescriptorProto.ExtensionRange
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.protobuf.DescriptorProto.ExtensionRange} ExtensionRange
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ExtensionRange.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies an ExtensionRange message.
                     * @function verify
                     * @memberof google.protobuf.DescriptorProto.ExtensionRange
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    ExtensionRange.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.start != null && message.hasOwnProperty("start"))
                            if (!$util.isInteger(message.start))
                                return "start: integer expected";
                        if (message.end != null && message.hasOwnProperty("end"))
                            if (!$util.isInteger(message.end))
                                return "end: integer expected";
                        if (message.options != null && message.hasOwnProperty("options")) {
                            var error = $root.google.protobuf.ExtensionRangeOptions.verify(message.options);
                            if (error)
                                return "options." + error;
                        }
                        return null;
                    };
    
                    /**
                     * Creates an ExtensionRange message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.protobuf.DescriptorProto.ExtensionRange
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.protobuf.DescriptorProto.ExtensionRange} ExtensionRange
                     */
                    ExtensionRange.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.protobuf.DescriptorProto.ExtensionRange)
                            return object;
                        var message = new $root.google.protobuf.DescriptorProto.ExtensionRange();
                        if (object.start != null)
                            message.start = object.start | 0;
                        if (object.end != null)
                            message.end = object.end | 0;
                        if (object.options != null) {
                            if (typeof object.options !== "object")
                                throw TypeError(".google.protobuf.DescriptorProto.ExtensionRange.options: object expected");
                            message.options = $root.google.protobuf.ExtensionRangeOptions.fromObject(object.options);
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from an ExtensionRange message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.protobuf.DescriptorProto.ExtensionRange
                     * @static
                     * @param {google.protobuf.DescriptorProto.ExtensionRange} message ExtensionRange
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    ExtensionRange.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults) {
                            object.start = 0;
                            object.end = 0;
                            object.options = null;
                        }
                        if (message.start != null && message.hasOwnProperty("start"))
                            object.start = message.start;
                        if (message.end != null && message.hasOwnProperty("end"))
                            object.end = message.end;
                        if (message.options != null && message.hasOwnProperty("options"))
                            object.options = $root.google.protobuf.ExtensionRangeOptions.toObject(message.options, options);
                        return object;
                    };
    
                    /**
                     * Converts this ExtensionRange to JSON.
                     * @function toJSON
                     * @memberof google.protobuf.DescriptorProto.ExtensionRange
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    ExtensionRange.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for ExtensionRange
                     * @function getTypeUrl
                     * @memberof google.protobuf.DescriptorProto.ExtensionRange
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    ExtensionRange.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.protobuf.DescriptorProto.ExtensionRange";
                    };
    
                    return ExtensionRange;
                })();
    
                DescriptorProto.ReservedRange = (function() {
    
                    /**
                     * Properties of a ReservedRange.
                     * @memberof google.protobuf.DescriptorProto
                     * @interface IReservedRange
                     * @property {number|null} [start] ReservedRange start
                     * @property {number|null} [end] ReservedRange end
                     */
    
                    /**
                     * Constructs a new ReservedRange.
                     * @memberof google.protobuf.DescriptorProto
                     * @classdesc Represents a ReservedRange.
                     * @implements IReservedRange
                     * @constructor
                     * @param {google.protobuf.DescriptorProto.IReservedRange=} [properties] Properties to set
                     */
                    function ReservedRange(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * ReservedRange start.
                     * @member {number} start
                     * @memberof google.protobuf.DescriptorProto.ReservedRange
                     * @instance
                     */
                    ReservedRange.prototype.start = 0;
    
                    /**
                     * ReservedRange end.
                     * @member {number} end
                     * @memberof google.protobuf.DescriptorProto.ReservedRange
                     * @instance
                     */
                    ReservedRange.prototype.end = 0;
    
                    /**
                     * Creates a new ReservedRange instance using the specified properties.
                     * @function create
                     * @memberof google.protobuf.DescriptorProto.ReservedRange
                     * @static
                     * @param {google.protobuf.DescriptorProto.IReservedRange=} [properties] Properties to set
                     * @returns {google.protobuf.DescriptorProto.ReservedRange} ReservedRange instance
                     */
                    ReservedRange.create = function create(properties) {
                        return new ReservedRange(properties);
                    };
    
                    /**
                     * Encodes the specified ReservedRange message. Does not implicitly {@link google.protobuf.DescriptorProto.ReservedRange.verify|verify} messages.
                     * @function encode
                     * @memberof google.protobuf.DescriptorProto.ReservedRange
                     * @static
                     * @param {google.protobuf.DescriptorProto.IReservedRange} message ReservedRange message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ReservedRange.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.start != null && Object.hasOwnProperty.call(message, "start"))
                            writer.uint32(/* id 1, wireType 0 =*/8).int32(message.start);
                        if (message.end != null && Object.hasOwnProperty.call(message, "end"))
                            writer.uint32(/* id 2, wireType 0 =*/16).int32(message.end);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified ReservedRange message, length delimited. Does not implicitly {@link google.protobuf.DescriptorProto.ReservedRange.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.protobuf.DescriptorProto.ReservedRange
                     * @static
                     * @param {google.protobuf.DescriptorProto.IReservedRange} message ReservedRange message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    ReservedRange.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a ReservedRange message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.protobuf.DescriptorProto.ReservedRange
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.protobuf.DescriptorProto.ReservedRange} ReservedRange
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ReservedRange.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.DescriptorProto.ReservedRange();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.start = reader.int32();
                                    break;
                                }
                            case 2: {
                                    message.end = reader.int32();
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a ReservedRange message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.protobuf.DescriptorProto.ReservedRange
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.protobuf.DescriptorProto.ReservedRange} ReservedRange
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    ReservedRange.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a ReservedRange message.
                     * @function verify
                     * @memberof google.protobuf.DescriptorProto.ReservedRange
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    ReservedRange.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.start != null && message.hasOwnProperty("start"))
                            if (!$util.isInteger(message.start))
                                return "start: integer expected";
                        if (message.end != null && message.hasOwnProperty("end"))
                            if (!$util.isInteger(message.end))
                                return "end: integer expected";
                        return null;
                    };
    
                    /**
                     * Creates a ReservedRange message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.protobuf.DescriptorProto.ReservedRange
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.protobuf.DescriptorProto.ReservedRange} ReservedRange
                     */
                    ReservedRange.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.protobuf.DescriptorProto.ReservedRange)
                            return object;
                        var message = new $root.google.protobuf.DescriptorProto.ReservedRange();
                        if (object.start != null)
                            message.start = object.start | 0;
                        if (object.end != null)
                            message.end = object.end | 0;
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a ReservedRange message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.protobuf.DescriptorProto.ReservedRange
                     * @static
                     * @param {google.protobuf.DescriptorProto.ReservedRange} message ReservedRange
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    ReservedRange.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults) {
                            object.start = 0;
                            object.end = 0;
                        }
                        if (message.start != null && message.hasOwnProperty("start"))
                            object.start = message.start;
                        if (message.end != null && message.hasOwnProperty("end"))
                            object.end = message.end;
                        return object;
                    };
    
                    /**
                     * Converts this ReservedRange to JSON.
                     * @function toJSON
                     * @memberof google.protobuf.DescriptorProto.ReservedRange
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    ReservedRange.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for ReservedRange
                     * @function getTypeUrl
                     * @memberof google.protobuf.DescriptorProto.ReservedRange
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    ReservedRange.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.protobuf.DescriptorProto.ReservedRange";
                    };
    
                    return ReservedRange;
                })();
    
                return DescriptorProto;
            })();
    
            protobuf.ExtensionRangeOptions = (function() {
    
                /**
                 * Properties of an ExtensionRangeOptions.
                 * @memberof google.protobuf
                 * @interface IExtensionRangeOptions
                 * @property {Array.<google.protobuf.IUninterpretedOption>|null} [uninterpretedOption] ExtensionRangeOptions uninterpretedOption
                 * @property {Array.<google.protobuf.ExtensionRangeOptions.IDeclaration>|null} [declaration] ExtensionRangeOptions declaration
                 * @property {google.protobuf.IFeatureSet|null} [features] ExtensionRangeOptions features
                 * @property {google.protobuf.ExtensionRangeOptions.VerificationState|null} [verification] ExtensionRangeOptions verification
                 */
    
                /**
                 * Constructs a new ExtensionRangeOptions.
                 * @memberof google.protobuf
                 * @classdesc Represents an ExtensionRangeOptions.
                 * @implements IExtensionRangeOptions
                 * @constructor
                 * @param {google.protobuf.IExtensionRangeOptions=} [properties] Properties to set
                 */
                function ExtensionRangeOptions(properties) {
                    this.uninterpretedOption = [];
                    this.declaration = [];
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * ExtensionRangeOptions uninterpretedOption.
                 * @member {Array.<google.protobuf.IUninterpretedOption>} uninterpretedOption
                 * @memberof google.protobuf.ExtensionRangeOptions
                 * @instance
                 */
                ExtensionRangeOptions.prototype.uninterpretedOption = $util.emptyArray;
    
                /**
                 * ExtensionRangeOptions declaration.
                 * @member {Array.<google.protobuf.ExtensionRangeOptions.IDeclaration>} declaration
                 * @memberof google.protobuf.ExtensionRangeOptions
                 * @instance
                 */
                ExtensionRangeOptions.prototype.declaration = $util.emptyArray;
    
                /**
                 * ExtensionRangeOptions features.
                 * @member {google.protobuf.IFeatureSet|null|undefined} features
                 * @memberof google.protobuf.ExtensionRangeOptions
                 * @instance
                 */
                ExtensionRangeOptions.prototype.features = null;
    
                /**
                 * ExtensionRangeOptions verification.
                 * @member {google.protobuf.ExtensionRangeOptions.VerificationState} verification
                 * @memberof google.protobuf.ExtensionRangeOptions
                 * @instance
                 */
                ExtensionRangeOptions.prototype.verification = 1;
    
                /**
                 * Creates a new ExtensionRangeOptions instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.ExtensionRangeOptions
                 * @static
                 * @param {google.protobuf.IExtensionRangeOptions=} [properties] Properties to set
                 * @returns {google.protobuf.ExtensionRangeOptions} ExtensionRangeOptions instance
                 */
                ExtensionRangeOptions.create = function create(properties) {
                    return new ExtensionRangeOptions(properties);
                };
    
                /**
                 * Encodes the specified ExtensionRangeOptions message. Does not implicitly {@link google.protobuf.ExtensionRangeOptions.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.ExtensionRangeOptions
                 * @static
                 * @param {google.protobuf.IExtensionRangeOptions} message ExtensionRangeOptions message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ExtensionRangeOptions.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.declaration != null && message.declaration.length)
                        for (var i = 0; i < message.declaration.length; ++i)
                            $root.google.protobuf.ExtensionRangeOptions.Declaration.encode(message.declaration[i], writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                    if (message.verification != null && Object.hasOwnProperty.call(message, "verification"))
                        writer.uint32(/* id 3, wireType 0 =*/24).int32(message.verification);
                    if (message.features != null && Object.hasOwnProperty.call(message, "features"))
                        $root.google.protobuf.FeatureSet.encode(message.features, writer.uint32(/* id 50, wireType 2 =*/402).fork()).ldelim();
                    if (message.uninterpretedOption != null && message.uninterpretedOption.length)
                        for (var i = 0; i < message.uninterpretedOption.length; ++i)
                            $root.google.protobuf.UninterpretedOption.encode(message.uninterpretedOption[i], writer.uint32(/* id 999, wireType 2 =*/7994).fork()).ldelim();
                    return writer;
                };
    
                /**
                 * Encodes the specified ExtensionRangeOptions message, length delimited. Does not implicitly {@link google.protobuf.ExtensionRangeOptions.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.ExtensionRangeOptions
                 * @static
                 * @param {google.protobuf.IExtensionRangeOptions} message ExtensionRangeOptions message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ExtensionRangeOptions.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes an ExtensionRangeOptions message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.ExtensionRangeOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.ExtensionRangeOptions} ExtensionRangeOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ExtensionRangeOptions.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.ExtensionRangeOptions();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 999: {
                                if (!(message.uninterpretedOption && message.uninterpretedOption.length))
                                    message.uninterpretedOption = [];
                                message.uninterpretedOption.push($root.google.protobuf.UninterpretedOption.decode(reader, reader.uint32()));
                                break;
                            }
                        case 2: {
                                if (!(message.declaration && message.declaration.length))
                                    message.declaration = [];
                                message.declaration.push($root.google.protobuf.ExtensionRangeOptions.Declaration.decode(reader, reader.uint32()));
                                break;
                            }
                        case 50: {
                                message.features = $root.google.protobuf.FeatureSet.decode(reader, reader.uint32());
                                break;
                            }
                        case 3: {
                                message.verification = reader.int32();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes an ExtensionRangeOptions message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.ExtensionRangeOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.ExtensionRangeOptions} ExtensionRangeOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ExtensionRangeOptions.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies an ExtensionRangeOptions message.
                 * @function verify
                 * @memberof google.protobuf.ExtensionRangeOptions
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                ExtensionRangeOptions.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.uninterpretedOption != null && message.hasOwnProperty("uninterpretedOption")) {
                        if (!Array.isArray(message.uninterpretedOption))
                            return "uninterpretedOption: array expected";
                        for (var i = 0; i < message.uninterpretedOption.length; ++i) {
                            var error = $root.google.protobuf.UninterpretedOption.verify(message.uninterpretedOption[i]);
                            if (error)
                                return "uninterpretedOption." + error;
                        }
                    }
                    if (message.declaration != null && message.hasOwnProperty("declaration")) {
                        if (!Array.isArray(message.declaration))
                            return "declaration: array expected";
                        for (var i = 0; i < message.declaration.length; ++i) {
                            var error = $root.google.protobuf.ExtensionRangeOptions.Declaration.verify(message.declaration[i]);
                            if (error)
                                return "declaration." + error;
                        }
                    }
                    if (message.features != null && message.hasOwnProperty("features")) {
                        var error = $root.google.protobuf.FeatureSet.verify(message.features);
                        if (error)
                            return "features." + error;
                    }
                    if (message.verification != null && message.hasOwnProperty("verification"))
                        switch (message.verification) {
                        default:
                            return "verification: enum value expected";
                        case 0:
                        case 1:
                            break;
                        }
                    return null;
                };
    
                /**
                 * Creates an ExtensionRangeOptions message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.ExtensionRangeOptions
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.ExtensionRangeOptions} ExtensionRangeOptions
                 */
                ExtensionRangeOptions.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.ExtensionRangeOptions)
                        return object;
                    var message = new $root.google.protobuf.ExtensionRangeOptions();
                    if (object.uninterpretedOption) {
                        if (!Array.isArray(object.uninterpretedOption))
                            throw TypeError(".google.protobuf.ExtensionRangeOptions.uninterpretedOption: array expected");
                        message.uninterpretedOption = [];
                        for (var i = 0; i < object.uninterpretedOption.length; ++i) {
                            if (typeof object.uninterpretedOption[i] !== "object")
                                throw TypeError(".google.protobuf.ExtensionRangeOptions.uninterpretedOption: object expected");
                            message.uninterpretedOption[i] = $root.google.protobuf.UninterpretedOption.fromObject(object.uninterpretedOption[i]);
                        }
                    }
                    if (object.declaration) {
                        if (!Array.isArray(object.declaration))
                            throw TypeError(".google.protobuf.ExtensionRangeOptions.declaration: array expected");
                        message.declaration = [];
                        for (var i = 0; i < object.declaration.length; ++i) {
                            if (typeof object.declaration[i] !== "object")
                                throw TypeError(".google.protobuf.ExtensionRangeOptions.declaration: object expected");
                            message.declaration[i] = $root.google.protobuf.ExtensionRangeOptions.Declaration.fromObject(object.declaration[i]);
                        }
                    }
                    if (object.features != null) {
                        if (typeof object.features !== "object")
                            throw TypeError(".google.protobuf.ExtensionRangeOptions.features: object expected");
                        message.features = $root.google.protobuf.FeatureSet.fromObject(object.features);
                    }
                    switch (object.verification) {
                    case "DECLARATION":
                    case 0:
                        message.verification = 0;
                        break;
                    default:
                        if (typeof object.verification === "number") {
                            message.verification = object.verification;
                            break;
                        }
                        break;
                    case "UNVERIFIED":
                    case 1:
                        message.verification = 1;
                        break;
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from an ExtensionRangeOptions message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.ExtensionRangeOptions
                 * @static
                 * @param {google.protobuf.ExtensionRangeOptions} message ExtensionRangeOptions
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                ExtensionRangeOptions.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.arrays || options.defaults) {
                        object.declaration = [];
                        object.uninterpretedOption = [];
                    }
                    if (options.defaults) {
                        object.verification = options.enums === String ? "UNVERIFIED" : 1;
                        object.features = null;
                    }
                    if (message.declaration && message.declaration.length) {
                        object.declaration = [];
                        for (var j = 0; j < message.declaration.length; ++j)
                            object.declaration[j] = $root.google.protobuf.ExtensionRangeOptions.Declaration.toObject(message.declaration[j], options);
                    }
                    if (message.verification != null && message.hasOwnProperty("verification"))
                        object.verification = options.enums === String ? $root.google.protobuf.ExtensionRangeOptions.VerificationState[message.verification] === undefined ? message.verification : $root.google.protobuf.ExtensionRangeOptions.VerificationState[message.verification] : message.verification;
                    if (message.features != null && message.hasOwnProperty("features"))
                        object.features = $root.google.protobuf.FeatureSet.toObject(message.features, options);
                    if (message.uninterpretedOption && message.uninterpretedOption.length) {
                        object.uninterpretedOption = [];
                        for (var j = 0; j < message.uninterpretedOption.length; ++j)
                            object.uninterpretedOption[j] = $root.google.protobuf.UninterpretedOption.toObject(message.uninterpretedOption[j], options);
                    }
                    return object;
                };
    
                /**
                 * Converts this ExtensionRangeOptions to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.ExtensionRangeOptions
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                ExtensionRangeOptions.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for ExtensionRangeOptions
                 * @function getTypeUrl
                 * @memberof google.protobuf.ExtensionRangeOptions
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                ExtensionRangeOptions.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.ExtensionRangeOptions";
                };
    
                ExtensionRangeOptions.Declaration = (function() {
    
                    /**
                     * Properties of a Declaration.
                     * @memberof google.protobuf.ExtensionRangeOptions
                     * @interface IDeclaration
                     * @property {number|null} [number] Declaration number
                     * @property {string|null} [fullName] Declaration fullName
                     * @property {string|null} [type] Declaration type
                     * @property {boolean|null} [reserved] Declaration reserved
                     * @property {boolean|null} [repeated] Declaration repeated
                     */
    
                    /**
                     * Constructs a new Declaration.
                     * @memberof google.protobuf.ExtensionRangeOptions
                     * @classdesc Represents a Declaration.
                     * @implements IDeclaration
                     * @constructor
                     * @param {google.protobuf.ExtensionRangeOptions.IDeclaration=} [properties] Properties to set
                     */
                    function Declaration(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * Declaration number.
                     * @member {number} number
                     * @memberof google.protobuf.ExtensionRangeOptions.Declaration
                     * @instance
                     */
                    Declaration.prototype.number = 0;
    
                    /**
                     * Declaration fullName.
                     * @member {string} fullName
                     * @memberof google.protobuf.ExtensionRangeOptions.Declaration
                     * @instance
                     */
                    Declaration.prototype.fullName = "";
    
                    /**
                     * Declaration type.
                     * @member {string} type
                     * @memberof google.protobuf.ExtensionRangeOptions.Declaration
                     * @instance
                     */
                    Declaration.prototype.type = "";
    
                    /**
                     * Declaration reserved.
                     * @member {boolean} reserved
                     * @memberof google.protobuf.ExtensionRangeOptions.Declaration
                     * @instance
                     */
                    Declaration.prototype.reserved = false;
    
                    /**
                     * Declaration repeated.
                     * @member {boolean} repeated
                     * @memberof google.protobuf.ExtensionRangeOptions.Declaration
                     * @instance
                     */
                    Declaration.prototype.repeated = false;
    
                    /**
                     * Creates a new Declaration instance using the specified properties.
                     * @function create
                     * @memberof google.protobuf.ExtensionRangeOptions.Declaration
                     * @static
                     * @param {google.protobuf.ExtensionRangeOptions.IDeclaration=} [properties] Properties to set
                     * @returns {google.protobuf.ExtensionRangeOptions.Declaration} Declaration instance
                     */
                    Declaration.create = function create(properties) {
                        return new Declaration(properties);
                    };
    
                    /**
                     * Encodes the specified Declaration message. Does not implicitly {@link google.protobuf.ExtensionRangeOptions.Declaration.verify|verify} messages.
                     * @function encode
                     * @memberof google.protobuf.ExtensionRangeOptions.Declaration
                     * @static
                     * @param {google.protobuf.ExtensionRangeOptions.IDeclaration} message Declaration message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    Declaration.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.number != null && Object.hasOwnProperty.call(message, "number"))
                            writer.uint32(/* id 1, wireType 0 =*/8).int32(message.number);
                        if (message.fullName != null && Object.hasOwnProperty.call(message, "fullName"))
                            writer.uint32(/* id 2, wireType 2 =*/18).string(message.fullName);
                        if (message.type != null && Object.hasOwnProperty.call(message, "type"))
                            writer.uint32(/* id 3, wireType 2 =*/26).string(message.type);
                        if (message.reserved != null && Object.hasOwnProperty.call(message, "reserved"))
                            writer.uint32(/* id 5, wireType 0 =*/40).bool(message.reserved);
                        if (message.repeated != null && Object.hasOwnProperty.call(message, "repeated"))
                            writer.uint32(/* id 6, wireType 0 =*/48).bool(message.repeated);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified Declaration message, length delimited. Does not implicitly {@link google.protobuf.ExtensionRangeOptions.Declaration.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.protobuf.ExtensionRangeOptions.Declaration
                     * @static
                     * @param {google.protobuf.ExtensionRangeOptions.IDeclaration} message Declaration message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    Declaration.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a Declaration message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.protobuf.ExtensionRangeOptions.Declaration
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.protobuf.ExtensionRangeOptions.Declaration} Declaration
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    Declaration.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.ExtensionRangeOptions.Declaration();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.number = reader.int32();
                                    break;
                                }
                            case 2: {
                                    message.fullName = reader.string();
                                    break;
                                }
                            case 3: {
                                    message.type = reader.string();
                                    break;
                                }
                            case 5: {
                                    message.reserved = reader.bool();
                                    break;
                                }
                            case 6: {
                                    message.repeated = reader.bool();
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a Declaration message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.protobuf.ExtensionRangeOptions.Declaration
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.protobuf.ExtensionRangeOptions.Declaration} Declaration
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    Declaration.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a Declaration message.
                     * @function verify
                     * @memberof google.protobuf.ExtensionRangeOptions.Declaration
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    Declaration.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.number != null && message.hasOwnProperty("number"))
                            if (!$util.isInteger(message.number))
                                return "number: integer expected";
                        if (message.fullName != null && message.hasOwnProperty("fullName"))
                            if (!$util.isString(message.fullName))
                                return "fullName: string expected";
                        if (message.type != null && message.hasOwnProperty("type"))
                            if (!$util.isString(message.type))
                                return "type: string expected";
                        if (message.reserved != null && message.hasOwnProperty("reserved"))
                            if (typeof message.reserved !== "boolean")
                                return "reserved: boolean expected";
                        if (message.repeated != null && message.hasOwnProperty("repeated"))
                            if (typeof message.repeated !== "boolean")
                                return "repeated: boolean expected";
                        return null;
                    };
    
                    /**
                     * Creates a Declaration message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.protobuf.ExtensionRangeOptions.Declaration
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.protobuf.ExtensionRangeOptions.Declaration} Declaration
                     */
                    Declaration.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.protobuf.ExtensionRangeOptions.Declaration)
                            return object;
                        var message = new $root.google.protobuf.ExtensionRangeOptions.Declaration();
                        if (object.number != null)
                            message.number = object.number | 0;
                        if (object.fullName != null)
                            message.fullName = String(object.fullName);
                        if (object.type != null)
                            message.type = String(object.type);
                        if (object.reserved != null)
                            message.reserved = Boolean(object.reserved);
                        if (object.repeated != null)
                            message.repeated = Boolean(object.repeated);
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a Declaration message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.protobuf.ExtensionRangeOptions.Declaration
                     * @static
                     * @param {google.protobuf.ExtensionRangeOptions.Declaration} message Declaration
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    Declaration.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults) {
                            object.number = 0;
                            object.fullName = "";
                            object.type = "";
                            object.reserved = false;
                            object.repeated = false;
                        }
                        if (message.number != null && message.hasOwnProperty("number"))
                            object.number = message.number;
                        if (message.fullName != null && message.hasOwnProperty("fullName"))
                            object.fullName = message.fullName;
                        if (message.type != null && message.hasOwnProperty("type"))
                            object.type = message.type;
                        if (message.reserved != null && message.hasOwnProperty("reserved"))
                            object.reserved = message.reserved;
                        if (message.repeated != null && message.hasOwnProperty("repeated"))
                            object.repeated = message.repeated;
                        return object;
                    };
    
                    /**
                     * Converts this Declaration to JSON.
                     * @function toJSON
                     * @memberof google.protobuf.ExtensionRangeOptions.Declaration
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    Declaration.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for Declaration
                     * @function getTypeUrl
                     * @memberof google.protobuf.ExtensionRangeOptions.Declaration
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    Declaration.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.protobuf.ExtensionRangeOptions.Declaration";
                    };
    
                    return Declaration;
                })();
    
                /**
                 * VerificationState enum.
                 * @name google.protobuf.ExtensionRangeOptions.VerificationState
                 * @enum {number}
                 * @property {number} DECLARATION=0 DECLARATION value
                 * @property {number} UNVERIFIED=1 UNVERIFIED value
                 */
                ExtensionRangeOptions.VerificationState = (function() {
                    var valuesById = {}, values = Object.create(valuesById);
                    values[valuesById[0] = "DECLARATION"] = 0;
                    values[valuesById[1] = "UNVERIFIED"] = 1;
                    return values;
                })();
    
                return ExtensionRangeOptions;
            })();
    
            protobuf.FieldDescriptorProto = (function() {
    
                /**
                 * Properties of a FieldDescriptorProto.
                 * @memberof google.protobuf
                 * @interface IFieldDescriptorProto
                 * @property {string|null} [name] FieldDescriptorProto name
                 * @property {number|null} [number] FieldDescriptorProto number
                 * @property {google.protobuf.FieldDescriptorProto.Label|null} [label] FieldDescriptorProto label
                 * @property {google.protobuf.FieldDescriptorProto.Type|null} [type] FieldDescriptorProto type
                 * @property {string|null} [typeName] FieldDescriptorProto typeName
                 * @property {string|null} [extendee] FieldDescriptorProto extendee
                 * @property {string|null} [defaultValue] FieldDescriptorProto defaultValue
                 * @property {number|null} [oneofIndex] FieldDescriptorProto oneofIndex
                 * @property {string|null} [jsonName] FieldDescriptorProto jsonName
                 * @property {google.protobuf.IFieldOptions|null} [options] FieldDescriptorProto options
                 * @property {boolean|null} [proto3Optional] FieldDescriptorProto proto3Optional
                 */
    
                /**
                 * Constructs a new FieldDescriptorProto.
                 * @memberof google.protobuf
                 * @classdesc Represents a FieldDescriptorProto.
                 * @implements IFieldDescriptorProto
                 * @constructor
                 * @param {google.protobuf.IFieldDescriptorProto=} [properties] Properties to set
                 */
                function FieldDescriptorProto(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * FieldDescriptorProto name.
                 * @member {string} name
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @instance
                 */
                FieldDescriptorProto.prototype.name = "";
    
                /**
                 * FieldDescriptorProto number.
                 * @member {number} number
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @instance
                 */
                FieldDescriptorProto.prototype.number = 0;
    
                /**
                 * FieldDescriptorProto label.
                 * @member {google.protobuf.FieldDescriptorProto.Label} label
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @instance
                 */
                FieldDescriptorProto.prototype.label = 1;
    
                /**
                 * FieldDescriptorProto type.
                 * @member {google.protobuf.FieldDescriptorProto.Type} type
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @instance
                 */
                FieldDescriptorProto.prototype.type = 1;
    
                /**
                 * FieldDescriptorProto typeName.
                 * @member {string} typeName
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @instance
                 */
                FieldDescriptorProto.prototype.typeName = "";
    
                /**
                 * FieldDescriptorProto extendee.
                 * @member {string} extendee
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @instance
                 */
                FieldDescriptorProto.prototype.extendee = "";
    
                /**
                 * FieldDescriptorProto defaultValue.
                 * @member {string} defaultValue
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @instance
                 */
                FieldDescriptorProto.prototype.defaultValue = "";
    
                /**
                 * FieldDescriptorProto oneofIndex.
                 * @member {number} oneofIndex
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @instance
                 */
                FieldDescriptorProto.prototype.oneofIndex = 0;
    
                /**
                 * FieldDescriptorProto jsonName.
                 * @member {string} jsonName
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @instance
                 */
                FieldDescriptorProto.prototype.jsonName = "";
    
                /**
                 * FieldDescriptorProto options.
                 * @member {google.protobuf.IFieldOptions|null|undefined} options
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @instance
                 */
                FieldDescriptorProto.prototype.options = null;
    
                /**
                 * FieldDescriptorProto proto3Optional.
                 * @member {boolean} proto3Optional
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @instance
                 */
                FieldDescriptorProto.prototype.proto3Optional = false;
    
                /**
                 * Creates a new FieldDescriptorProto instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @static
                 * @param {google.protobuf.IFieldDescriptorProto=} [properties] Properties to set
                 * @returns {google.protobuf.FieldDescriptorProto} FieldDescriptorProto instance
                 */
                FieldDescriptorProto.create = function create(properties) {
                    return new FieldDescriptorProto(properties);
                };
    
                /**
                 * Encodes the specified FieldDescriptorProto message. Does not implicitly {@link google.protobuf.FieldDescriptorProto.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @static
                 * @param {google.protobuf.IFieldDescriptorProto} message FieldDescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FieldDescriptorProto.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                    if (message.extendee != null && Object.hasOwnProperty.call(message, "extendee"))
                        writer.uint32(/* id 2, wireType 2 =*/18).string(message.extendee);
                    if (message.number != null && Object.hasOwnProperty.call(message, "number"))
                        writer.uint32(/* id 3, wireType 0 =*/24).int32(message.number);
                    if (message.label != null && Object.hasOwnProperty.call(message, "label"))
                        writer.uint32(/* id 4, wireType 0 =*/32).int32(message.label);
                    if (message.type != null && Object.hasOwnProperty.call(message, "type"))
                        writer.uint32(/* id 5, wireType 0 =*/40).int32(message.type);
                    if (message.typeName != null && Object.hasOwnProperty.call(message, "typeName"))
                        writer.uint32(/* id 6, wireType 2 =*/50).string(message.typeName);
                    if (message.defaultValue != null && Object.hasOwnProperty.call(message, "defaultValue"))
                        writer.uint32(/* id 7, wireType 2 =*/58).string(message.defaultValue);
                    if (message.options != null && Object.hasOwnProperty.call(message, "options"))
                        $root.google.protobuf.FieldOptions.encode(message.options, writer.uint32(/* id 8, wireType 2 =*/66).fork()).ldelim();
                    if (message.oneofIndex != null && Object.hasOwnProperty.call(message, "oneofIndex"))
                        writer.uint32(/* id 9, wireType 0 =*/72).int32(message.oneofIndex);
                    if (message.jsonName != null && Object.hasOwnProperty.call(message, "jsonName"))
                        writer.uint32(/* id 10, wireType 2 =*/82).string(message.jsonName);
                    if (message.proto3Optional != null && Object.hasOwnProperty.call(message, "proto3Optional"))
                        writer.uint32(/* id 17, wireType 0 =*/136).bool(message.proto3Optional);
                    return writer;
                };
    
                /**
                 * Encodes the specified FieldDescriptorProto message, length delimited. Does not implicitly {@link google.protobuf.FieldDescriptorProto.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @static
                 * @param {google.protobuf.IFieldDescriptorProto} message FieldDescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FieldDescriptorProto.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a FieldDescriptorProto message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.FieldDescriptorProto} FieldDescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FieldDescriptorProto.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.FieldDescriptorProto();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.name = reader.string();
                                break;
                            }
                        case 3: {
                                message.number = reader.int32();
                                break;
                            }
                        case 4: {
                                message.label = reader.int32();
                                break;
                            }
                        case 5: {
                                message.type = reader.int32();
                                break;
                            }
                        case 6: {
                                message.typeName = reader.string();
                                break;
                            }
                        case 2: {
                                message.extendee = reader.string();
                                break;
                            }
                        case 7: {
                                message.defaultValue = reader.string();
                                break;
                            }
                        case 9: {
                                message.oneofIndex = reader.int32();
                                break;
                            }
                        case 10: {
                                message.jsonName = reader.string();
                                break;
                            }
                        case 8: {
                                message.options = $root.google.protobuf.FieldOptions.decode(reader, reader.uint32());
                                break;
                            }
                        case 17: {
                                message.proto3Optional = reader.bool();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a FieldDescriptorProto message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.FieldDescriptorProto} FieldDescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FieldDescriptorProto.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a FieldDescriptorProto message.
                 * @function verify
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                FieldDescriptorProto.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.name != null && message.hasOwnProperty("name"))
                        if (!$util.isString(message.name))
                            return "name: string expected";
                    if (message.number != null && message.hasOwnProperty("number"))
                        if (!$util.isInteger(message.number))
                            return "number: integer expected";
                    if (message.label != null && message.hasOwnProperty("label"))
                        switch (message.label) {
                        default:
                            return "label: enum value expected";
                        case 1:
                        case 3:
                        case 2:
                            break;
                        }
                    if (message.type != null && message.hasOwnProperty("type"))
                        switch (message.type) {
                        default:
                            return "type: enum value expected";
                        case 1:
                        case 2:
                        case 3:
                        case 4:
                        case 5:
                        case 6:
                        case 7:
                        case 8:
                        case 9:
                        case 10:
                        case 11:
                        case 12:
                        case 13:
                        case 14:
                        case 15:
                        case 16:
                        case 17:
                        case 18:
                            break;
                        }
                    if (message.typeName != null && message.hasOwnProperty("typeName"))
                        if (!$util.isString(message.typeName))
                            return "typeName: string expected";
                    if (message.extendee != null && message.hasOwnProperty("extendee"))
                        if (!$util.isString(message.extendee))
                            return "extendee: string expected";
                    if (message.defaultValue != null && message.hasOwnProperty("defaultValue"))
                        if (!$util.isString(message.defaultValue))
                            return "defaultValue: string expected";
                    if (message.oneofIndex != null && message.hasOwnProperty("oneofIndex"))
                        if (!$util.isInteger(message.oneofIndex))
                            return "oneofIndex: integer expected";
                    if (message.jsonName != null && message.hasOwnProperty("jsonName"))
                        if (!$util.isString(message.jsonName))
                            return "jsonName: string expected";
                    if (message.options != null && message.hasOwnProperty("options")) {
                        var error = $root.google.protobuf.FieldOptions.verify(message.options);
                        if (error)
                            return "options." + error;
                    }
                    if (message.proto3Optional != null && message.hasOwnProperty("proto3Optional"))
                        if (typeof message.proto3Optional !== "boolean")
                            return "proto3Optional: boolean expected";
                    return null;
                };
    
                /**
                 * Creates a FieldDescriptorProto message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.FieldDescriptorProto} FieldDescriptorProto
                 */
                FieldDescriptorProto.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.FieldDescriptorProto)
                        return object;
                    var message = new $root.google.protobuf.FieldDescriptorProto();
                    if (object.name != null)
                        message.name = String(object.name);
                    if (object.number != null)
                        message.number = object.number | 0;
                    switch (object.label) {
                    default:
                        if (typeof object.label === "number") {
                            message.label = object.label;
                            break;
                        }
                        break;
                    case "LABEL_OPTIONAL":
                    case 1:
                        message.label = 1;
                        break;
                    case "LABEL_REPEATED":
                    case 3:
                        message.label = 3;
                        break;
                    case "LABEL_REQUIRED":
                    case 2:
                        message.label = 2;
                        break;
                    }
                    switch (object.type) {
                    default:
                        if (typeof object.type === "number") {
                            message.type = object.type;
                            break;
                        }
                        break;
                    case "TYPE_DOUBLE":
                    case 1:
                        message.type = 1;
                        break;
                    case "TYPE_FLOAT":
                    case 2:
                        message.type = 2;
                        break;
                    case "TYPE_INT64":
                    case 3:
                        message.type = 3;
                        break;
                    case "TYPE_UINT64":
                    case 4:
                        message.type = 4;
                        break;
                    case "TYPE_INT32":
                    case 5:
                        message.type = 5;
                        break;
                    case "TYPE_FIXED64":
                    case 6:
                        message.type = 6;
                        break;
                    case "TYPE_FIXED32":
                    case 7:
                        message.type = 7;
                        break;
                    case "TYPE_BOOL":
                    case 8:
                        message.type = 8;
                        break;
                    case "TYPE_STRING":
                    case 9:
                        message.type = 9;
                        break;
                    case "TYPE_GROUP":
                    case 10:
                        message.type = 10;
                        break;
                    case "TYPE_MESSAGE":
                    case 11:
                        message.type = 11;
                        break;
                    case "TYPE_BYTES":
                    case 12:
                        message.type = 12;
                        break;
                    case "TYPE_UINT32":
                    case 13:
                        message.type = 13;
                        break;
                    case "TYPE_ENUM":
                    case 14:
                        message.type = 14;
                        break;
                    case "TYPE_SFIXED32":
                    case 15:
                        message.type = 15;
                        break;
                    case "TYPE_SFIXED64":
                    case 16:
                        message.type = 16;
                        break;
                    case "TYPE_SINT32":
                    case 17:
                        message.type = 17;
                        break;
                    case "TYPE_SINT64":
                    case 18:
                        message.type = 18;
                        break;
                    }
                    if (object.typeName != null)
                        message.typeName = String(object.typeName);
                    if (object.extendee != null)
                        message.extendee = String(object.extendee);
                    if (object.defaultValue != null)
                        message.defaultValue = String(object.defaultValue);
                    if (object.oneofIndex != null)
                        message.oneofIndex = object.oneofIndex | 0;
                    if (object.jsonName != null)
                        message.jsonName = String(object.jsonName);
                    if (object.options != null) {
                        if (typeof object.options !== "object")
                            throw TypeError(".google.protobuf.FieldDescriptorProto.options: object expected");
                        message.options = $root.google.protobuf.FieldOptions.fromObject(object.options);
                    }
                    if (object.proto3Optional != null)
                        message.proto3Optional = Boolean(object.proto3Optional);
                    return message;
                };
    
                /**
                 * Creates a plain object from a FieldDescriptorProto message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @static
                 * @param {google.protobuf.FieldDescriptorProto} message FieldDescriptorProto
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                FieldDescriptorProto.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults) {
                        object.name = "";
                        object.extendee = "";
                        object.number = 0;
                        object.label = options.enums === String ? "LABEL_OPTIONAL" : 1;
                        object.type = options.enums === String ? "TYPE_DOUBLE" : 1;
                        object.typeName = "";
                        object.defaultValue = "";
                        object.options = null;
                        object.oneofIndex = 0;
                        object.jsonName = "";
                        object.proto3Optional = false;
                    }
                    if (message.name != null && message.hasOwnProperty("name"))
                        object.name = message.name;
                    if (message.extendee != null && message.hasOwnProperty("extendee"))
                        object.extendee = message.extendee;
                    if (message.number != null && message.hasOwnProperty("number"))
                        object.number = message.number;
                    if (message.label != null && message.hasOwnProperty("label"))
                        object.label = options.enums === String ? $root.google.protobuf.FieldDescriptorProto.Label[message.label] === undefined ? message.label : $root.google.protobuf.FieldDescriptorProto.Label[message.label] : message.label;
                    if (message.type != null && message.hasOwnProperty("type"))
                        object.type = options.enums === String ? $root.google.protobuf.FieldDescriptorProto.Type[message.type] === undefined ? message.type : $root.google.protobuf.FieldDescriptorProto.Type[message.type] : message.type;
                    if (message.typeName != null && message.hasOwnProperty("typeName"))
                        object.typeName = message.typeName;
                    if (message.defaultValue != null && message.hasOwnProperty("defaultValue"))
                        object.defaultValue = message.defaultValue;
                    if (message.options != null && message.hasOwnProperty("options"))
                        object.options = $root.google.protobuf.FieldOptions.toObject(message.options, options);
                    if (message.oneofIndex != null && message.hasOwnProperty("oneofIndex"))
                        object.oneofIndex = message.oneofIndex;
                    if (message.jsonName != null && message.hasOwnProperty("jsonName"))
                        object.jsonName = message.jsonName;
                    if (message.proto3Optional != null && message.hasOwnProperty("proto3Optional"))
                        object.proto3Optional = message.proto3Optional;
                    return object;
                };
    
                /**
                 * Converts this FieldDescriptorProto to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                FieldDescriptorProto.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for FieldDescriptorProto
                 * @function getTypeUrl
                 * @memberof google.protobuf.FieldDescriptorProto
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                FieldDescriptorProto.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.FieldDescriptorProto";
                };
    
                /**
                 * Type enum.
                 * @name google.protobuf.FieldDescriptorProto.Type
                 * @enum {number}
                 * @property {number} TYPE_DOUBLE=1 TYPE_DOUBLE value
                 * @property {number} TYPE_FLOAT=2 TYPE_FLOAT value
                 * @property {number} TYPE_INT64=3 TYPE_INT64 value
                 * @property {number} TYPE_UINT64=4 TYPE_UINT64 value
                 * @property {number} TYPE_INT32=5 TYPE_INT32 value
                 * @property {number} TYPE_FIXED64=6 TYPE_FIXED64 value
                 * @property {number} TYPE_FIXED32=7 TYPE_FIXED32 value
                 * @property {number} TYPE_BOOL=8 TYPE_BOOL value
                 * @property {number} TYPE_STRING=9 TYPE_STRING value
                 * @property {number} TYPE_GROUP=10 TYPE_GROUP value
                 * @property {number} TYPE_MESSAGE=11 TYPE_MESSAGE value
                 * @property {number} TYPE_BYTES=12 TYPE_BYTES value
                 * @property {number} TYPE_UINT32=13 TYPE_UINT32 value
                 * @property {number} TYPE_ENUM=14 TYPE_ENUM value
                 * @property {number} TYPE_SFIXED32=15 TYPE_SFIXED32 value
                 * @property {number} TYPE_SFIXED64=16 TYPE_SFIXED64 value
                 * @property {number} TYPE_SINT32=17 TYPE_SINT32 value
                 * @property {number} TYPE_SINT64=18 TYPE_SINT64 value
                 */
                FieldDescriptorProto.Type = (function() {
                    var valuesById = {}, values = Object.create(valuesById);
                    values[valuesById[1] = "TYPE_DOUBLE"] = 1;
                    values[valuesById[2] = "TYPE_FLOAT"] = 2;
                    values[valuesById[3] = "TYPE_INT64"] = 3;
                    values[valuesById[4] = "TYPE_UINT64"] = 4;
                    values[valuesById[5] = "TYPE_INT32"] = 5;
                    values[valuesById[6] = "TYPE_FIXED64"] = 6;
                    values[valuesById[7] = "TYPE_FIXED32"] = 7;
                    values[valuesById[8] = "TYPE_BOOL"] = 8;
                    values[valuesById[9] = "TYPE_STRING"] = 9;
                    values[valuesById[10] = "TYPE_GROUP"] = 10;
                    values[valuesById[11] = "TYPE_MESSAGE"] = 11;
                    values[valuesById[12] = "TYPE_BYTES"] = 12;
                    values[valuesById[13] = "TYPE_UINT32"] = 13;
                    values[valuesById[14] = "TYPE_ENUM"] = 14;
                    values[valuesById[15] = "TYPE_SFIXED32"] = 15;
                    values[valuesById[16] = "TYPE_SFIXED64"] = 16;
                    values[valuesById[17] = "TYPE_SINT32"] = 17;
                    values[valuesById[18] = "TYPE_SINT64"] = 18;
                    return values;
                })();
    
                /**
                 * Label enum.
                 * @name google.protobuf.FieldDescriptorProto.Label
                 * @enum {number}
                 * @property {number} LABEL_OPTIONAL=1 LABEL_OPTIONAL value
                 * @property {number} LABEL_REPEATED=3 LABEL_REPEATED value
                 * @property {number} LABEL_REQUIRED=2 LABEL_REQUIRED value
                 */
                FieldDescriptorProto.Label = (function() {
                    var valuesById = {}, values = Object.create(valuesById);
                    values[valuesById[1] = "LABEL_OPTIONAL"] = 1;
                    values[valuesById[3] = "LABEL_REPEATED"] = 3;
                    values[valuesById[2] = "LABEL_REQUIRED"] = 2;
                    return values;
                })();
    
                return FieldDescriptorProto;
            })();
    
            protobuf.OneofDescriptorProto = (function() {
    
                /**
                 * Properties of an OneofDescriptorProto.
                 * @memberof google.protobuf
                 * @interface IOneofDescriptorProto
                 * @property {string|null} [name] OneofDescriptorProto name
                 * @property {google.protobuf.IOneofOptions|null} [options] OneofDescriptorProto options
                 */
    
                /**
                 * Constructs a new OneofDescriptorProto.
                 * @memberof google.protobuf
                 * @classdesc Represents an OneofDescriptorProto.
                 * @implements IOneofDescriptorProto
                 * @constructor
                 * @param {google.protobuf.IOneofDescriptorProto=} [properties] Properties to set
                 */
                function OneofDescriptorProto(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * OneofDescriptorProto name.
                 * @member {string} name
                 * @memberof google.protobuf.OneofDescriptorProto
                 * @instance
                 */
                OneofDescriptorProto.prototype.name = "";
    
                /**
                 * OneofDescriptorProto options.
                 * @member {google.protobuf.IOneofOptions|null|undefined} options
                 * @memberof google.protobuf.OneofDescriptorProto
                 * @instance
                 */
                OneofDescriptorProto.prototype.options = null;
    
                /**
                 * Creates a new OneofDescriptorProto instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.OneofDescriptorProto
                 * @static
                 * @param {google.protobuf.IOneofDescriptorProto=} [properties] Properties to set
                 * @returns {google.protobuf.OneofDescriptorProto} OneofDescriptorProto instance
                 */
                OneofDescriptorProto.create = function create(properties) {
                    return new OneofDescriptorProto(properties);
                };
    
                /**
                 * Encodes the specified OneofDescriptorProto message. Does not implicitly {@link google.protobuf.OneofDescriptorProto.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.OneofDescriptorProto
                 * @static
                 * @param {google.protobuf.IOneofDescriptorProto} message OneofDescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                OneofDescriptorProto.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                    if (message.options != null && Object.hasOwnProperty.call(message, "options"))
                        $root.google.protobuf.OneofOptions.encode(message.options, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                    return writer;
                };
    
                /**
                 * Encodes the specified OneofDescriptorProto message, length delimited. Does not implicitly {@link google.protobuf.OneofDescriptorProto.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.OneofDescriptorProto
                 * @static
                 * @param {google.protobuf.IOneofDescriptorProto} message OneofDescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                OneofDescriptorProto.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes an OneofDescriptorProto message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.OneofDescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.OneofDescriptorProto} OneofDescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                OneofDescriptorProto.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.OneofDescriptorProto();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.name = reader.string();
                                break;
                            }
                        case 2: {
                                message.options = $root.google.protobuf.OneofOptions.decode(reader, reader.uint32());
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes an OneofDescriptorProto message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.OneofDescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.OneofDescriptorProto} OneofDescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                OneofDescriptorProto.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies an OneofDescriptorProto message.
                 * @function verify
                 * @memberof google.protobuf.OneofDescriptorProto
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                OneofDescriptorProto.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.name != null && message.hasOwnProperty("name"))
                        if (!$util.isString(message.name))
                            return "name: string expected";
                    if (message.options != null && message.hasOwnProperty("options")) {
                        var error = $root.google.protobuf.OneofOptions.verify(message.options);
                        if (error)
                            return "options." + error;
                    }
                    return null;
                };
    
                /**
                 * Creates an OneofDescriptorProto message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.OneofDescriptorProto
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.OneofDescriptorProto} OneofDescriptorProto
                 */
                OneofDescriptorProto.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.OneofDescriptorProto)
                        return object;
                    var message = new $root.google.protobuf.OneofDescriptorProto();
                    if (object.name != null)
                        message.name = String(object.name);
                    if (object.options != null) {
                        if (typeof object.options !== "object")
                            throw TypeError(".google.protobuf.OneofDescriptorProto.options: object expected");
                        message.options = $root.google.protobuf.OneofOptions.fromObject(object.options);
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from an OneofDescriptorProto message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.OneofDescriptorProto
                 * @static
                 * @param {google.protobuf.OneofDescriptorProto} message OneofDescriptorProto
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                OneofDescriptorProto.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults) {
                        object.name = "";
                        object.options = null;
                    }
                    if (message.name != null && message.hasOwnProperty("name"))
                        object.name = message.name;
                    if (message.options != null && message.hasOwnProperty("options"))
                        object.options = $root.google.protobuf.OneofOptions.toObject(message.options, options);
                    return object;
                };
    
                /**
                 * Converts this OneofDescriptorProto to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.OneofDescriptorProto
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                OneofDescriptorProto.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for OneofDescriptorProto
                 * @function getTypeUrl
                 * @memberof google.protobuf.OneofDescriptorProto
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                OneofDescriptorProto.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.OneofDescriptorProto";
                };
    
                return OneofDescriptorProto;
            })();
    
            protobuf.EnumDescriptorProto = (function() {
    
                /**
                 * Properties of an EnumDescriptorProto.
                 * @memberof google.protobuf
                 * @interface IEnumDescriptorProto
                 * @property {string|null} [name] EnumDescriptorProto name
                 * @property {Array.<google.protobuf.IEnumValueDescriptorProto>|null} [value] EnumDescriptorProto value
                 * @property {google.protobuf.IEnumOptions|null} [options] EnumDescriptorProto options
                 * @property {Array.<google.protobuf.EnumDescriptorProto.IEnumReservedRange>|null} [reservedRange] EnumDescriptorProto reservedRange
                 * @property {Array.<string>|null} [reservedName] EnumDescriptorProto reservedName
                 */
    
                /**
                 * Constructs a new EnumDescriptorProto.
                 * @memberof google.protobuf
                 * @classdesc Represents an EnumDescriptorProto.
                 * @implements IEnumDescriptorProto
                 * @constructor
                 * @param {google.protobuf.IEnumDescriptorProto=} [properties] Properties to set
                 */
                function EnumDescriptorProto(properties) {
                    this.value = [];
                    this.reservedRange = [];
                    this.reservedName = [];
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * EnumDescriptorProto name.
                 * @member {string} name
                 * @memberof google.protobuf.EnumDescriptorProto
                 * @instance
                 */
                EnumDescriptorProto.prototype.name = "";
    
                /**
                 * EnumDescriptorProto value.
                 * @member {Array.<google.protobuf.IEnumValueDescriptorProto>} value
                 * @memberof google.protobuf.EnumDescriptorProto
                 * @instance
                 */
                EnumDescriptorProto.prototype.value = $util.emptyArray;
    
                /**
                 * EnumDescriptorProto options.
                 * @member {google.protobuf.IEnumOptions|null|undefined} options
                 * @memberof google.protobuf.EnumDescriptorProto
                 * @instance
                 */
                EnumDescriptorProto.prototype.options = null;
    
                /**
                 * EnumDescriptorProto reservedRange.
                 * @member {Array.<google.protobuf.EnumDescriptorProto.IEnumReservedRange>} reservedRange
                 * @memberof google.protobuf.EnumDescriptorProto
                 * @instance
                 */
                EnumDescriptorProto.prototype.reservedRange = $util.emptyArray;
    
                /**
                 * EnumDescriptorProto reservedName.
                 * @member {Array.<string>} reservedName
                 * @memberof google.protobuf.EnumDescriptorProto
                 * @instance
                 */
                EnumDescriptorProto.prototype.reservedName = $util.emptyArray;
    
                /**
                 * Creates a new EnumDescriptorProto instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.EnumDescriptorProto
                 * @static
                 * @param {google.protobuf.IEnumDescriptorProto=} [properties] Properties to set
                 * @returns {google.protobuf.EnumDescriptorProto} EnumDescriptorProto instance
                 */
                EnumDescriptorProto.create = function create(properties) {
                    return new EnumDescriptorProto(properties);
                };
    
                /**
                 * Encodes the specified EnumDescriptorProto message. Does not implicitly {@link google.protobuf.EnumDescriptorProto.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.EnumDescriptorProto
                 * @static
                 * @param {google.protobuf.IEnumDescriptorProto} message EnumDescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                EnumDescriptorProto.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                    if (message.value != null && message.value.length)
                        for (var i = 0; i < message.value.length; ++i)
                            $root.google.protobuf.EnumValueDescriptorProto.encode(message.value[i], writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                    if (message.options != null && Object.hasOwnProperty.call(message, "options"))
                        $root.google.protobuf.EnumOptions.encode(message.options, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                    if (message.reservedRange != null && message.reservedRange.length)
                        for (var i = 0; i < message.reservedRange.length; ++i)
                            $root.google.protobuf.EnumDescriptorProto.EnumReservedRange.encode(message.reservedRange[i], writer.uint32(/* id 4, wireType 2 =*/34).fork()).ldelim();
                    if (message.reservedName != null && message.reservedName.length)
                        for (var i = 0; i < message.reservedName.length; ++i)
                            writer.uint32(/* id 5, wireType 2 =*/42).string(message.reservedName[i]);
                    return writer;
                };
    
                /**
                 * Encodes the specified EnumDescriptorProto message, length delimited. Does not implicitly {@link google.protobuf.EnumDescriptorProto.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.EnumDescriptorProto
                 * @static
                 * @param {google.protobuf.IEnumDescriptorProto} message EnumDescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                EnumDescriptorProto.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes an EnumDescriptorProto message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.EnumDescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.EnumDescriptorProto} EnumDescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                EnumDescriptorProto.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.EnumDescriptorProto();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.name = reader.string();
                                break;
                            }
                        case 2: {
                                if (!(message.value && message.value.length))
                                    message.value = [];
                                message.value.push($root.google.protobuf.EnumValueDescriptorProto.decode(reader, reader.uint32()));
                                break;
                            }
                        case 3: {
                                message.options = $root.google.protobuf.EnumOptions.decode(reader, reader.uint32());
                                break;
                            }
                        case 4: {
                                if (!(message.reservedRange && message.reservedRange.length))
                                    message.reservedRange = [];
                                message.reservedRange.push($root.google.protobuf.EnumDescriptorProto.EnumReservedRange.decode(reader, reader.uint32()));
                                break;
                            }
                        case 5: {
                                if (!(message.reservedName && message.reservedName.length))
                                    message.reservedName = [];
                                message.reservedName.push(reader.string());
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes an EnumDescriptorProto message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.EnumDescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.EnumDescriptorProto} EnumDescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                EnumDescriptorProto.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies an EnumDescriptorProto message.
                 * @function verify
                 * @memberof google.protobuf.EnumDescriptorProto
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                EnumDescriptorProto.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.name != null && message.hasOwnProperty("name"))
                        if (!$util.isString(message.name))
                            return "name: string expected";
                    if (message.value != null && message.hasOwnProperty("value")) {
                        if (!Array.isArray(message.value))
                            return "value: array expected";
                        for (var i = 0; i < message.value.length; ++i) {
                            var error = $root.google.protobuf.EnumValueDescriptorProto.verify(message.value[i]);
                            if (error)
                                return "value." + error;
                        }
                    }
                    if (message.options != null && message.hasOwnProperty("options")) {
                        var error = $root.google.protobuf.EnumOptions.verify(message.options);
                        if (error)
                            return "options." + error;
                    }
                    if (message.reservedRange != null && message.hasOwnProperty("reservedRange")) {
                        if (!Array.isArray(message.reservedRange))
                            return "reservedRange: array expected";
                        for (var i = 0; i < message.reservedRange.length; ++i) {
                            var error = $root.google.protobuf.EnumDescriptorProto.EnumReservedRange.verify(message.reservedRange[i]);
                            if (error)
                                return "reservedRange." + error;
                        }
                    }
                    if (message.reservedName != null && message.hasOwnProperty("reservedName")) {
                        if (!Array.isArray(message.reservedName))
                            return "reservedName: array expected";
                        for (var i = 0; i < message.reservedName.length; ++i)
                            if (!$util.isString(message.reservedName[i]))
                                return "reservedName: string[] expected";
                    }
                    return null;
                };
    
                /**
                 * Creates an EnumDescriptorProto message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.EnumDescriptorProto
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.EnumDescriptorProto} EnumDescriptorProto
                 */
                EnumDescriptorProto.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.EnumDescriptorProto)
                        return object;
                    var message = new $root.google.protobuf.EnumDescriptorProto();
                    if (object.name != null)
                        message.name = String(object.name);
                    if (object.value) {
                        if (!Array.isArray(object.value))
                            throw TypeError(".google.protobuf.EnumDescriptorProto.value: array expected");
                        message.value = [];
                        for (var i = 0; i < object.value.length; ++i) {
                            if (typeof object.value[i] !== "object")
                                throw TypeError(".google.protobuf.EnumDescriptorProto.value: object expected");
                            message.value[i] = $root.google.protobuf.EnumValueDescriptorProto.fromObject(object.value[i]);
                        }
                    }
                    if (object.options != null) {
                        if (typeof object.options !== "object")
                            throw TypeError(".google.protobuf.EnumDescriptorProto.options: object expected");
                        message.options = $root.google.protobuf.EnumOptions.fromObject(object.options);
                    }
                    if (object.reservedRange) {
                        if (!Array.isArray(object.reservedRange))
                            throw TypeError(".google.protobuf.EnumDescriptorProto.reservedRange: array expected");
                        message.reservedRange = [];
                        for (var i = 0; i < object.reservedRange.length; ++i) {
                            if (typeof object.reservedRange[i] !== "object")
                                throw TypeError(".google.protobuf.EnumDescriptorProto.reservedRange: object expected");
                            message.reservedRange[i] = $root.google.protobuf.EnumDescriptorProto.EnumReservedRange.fromObject(object.reservedRange[i]);
                        }
                    }
                    if (object.reservedName) {
                        if (!Array.isArray(object.reservedName))
                            throw TypeError(".google.protobuf.EnumDescriptorProto.reservedName: array expected");
                        message.reservedName = [];
                        for (var i = 0; i < object.reservedName.length; ++i)
                            message.reservedName[i] = String(object.reservedName[i]);
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from an EnumDescriptorProto message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.EnumDescriptorProto
                 * @static
                 * @param {google.protobuf.EnumDescriptorProto} message EnumDescriptorProto
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                EnumDescriptorProto.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.arrays || options.defaults) {
                        object.value = [];
                        object.reservedRange = [];
                        object.reservedName = [];
                    }
                    if (options.defaults) {
                        object.name = "";
                        object.options = null;
                    }
                    if (message.name != null && message.hasOwnProperty("name"))
                        object.name = message.name;
                    if (message.value && message.value.length) {
                        object.value = [];
                        for (var j = 0; j < message.value.length; ++j)
                            object.value[j] = $root.google.protobuf.EnumValueDescriptorProto.toObject(message.value[j], options);
                    }
                    if (message.options != null && message.hasOwnProperty("options"))
                        object.options = $root.google.protobuf.EnumOptions.toObject(message.options, options);
                    if (message.reservedRange && message.reservedRange.length) {
                        object.reservedRange = [];
                        for (var j = 0; j < message.reservedRange.length; ++j)
                            object.reservedRange[j] = $root.google.protobuf.EnumDescriptorProto.EnumReservedRange.toObject(message.reservedRange[j], options);
                    }
                    if (message.reservedName && message.reservedName.length) {
                        object.reservedName = [];
                        for (var j = 0; j < message.reservedName.length; ++j)
                            object.reservedName[j] = message.reservedName[j];
                    }
                    return object;
                };
    
                /**
                 * Converts this EnumDescriptorProto to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.EnumDescriptorProto
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                EnumDescriptorProto.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for EnumDescriptorProto
                 * @function getTypeUrl
                 * @memberof google.protobuf.EnumDescriptorProto
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                EnumDescriptorProto.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.EnumDescriptorProto";
                };
    
                EnumDescriptorProto.EnumReservedRange = (function() {
    
                    /**
                     * Properties of an EnumReservedRange.
                     * @memberof google.protobuf.EnumDescriptorProto
                     * @interface IEnumReservedRange
                     * @property {number|null} [start] EnumReservedRange start
                     * @property {number|null} [end] EnumReservedRange end
                     */
    
                    /**
                     * Constructs a new EnumReservedRange.
                     * @memberof google.protobuf.EnumDescriptorProto
                     * @classdesc Represents an EnumReservedRange.
                     * @implements IEnumReservedRange
                     * @constructor
                     * @param {google.protobuf.EnumDescriptorProto.IEnumReservedRange=} [properties] Properties to set
                     */
                    function EnumReservedRange(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * EnumReservedRange start.
                     * @member {number} start
                     * @memberof google.protobuf.EnumDescriptorProto.EnumReservedRange
                     * @instance
                     */
                    EnumReservedRange.prototype.start = 0;
    
                    /**
                     * EnumReservedRange end.
                     * @member {number} end
                     * @memberof google.protobuf.EnumDescriptorProto.EnumReservedRange
                     * @instance
                     */
                    EnumReservedRange.prototype.end = 0;
    
                    /**
                     * Creates a new EnumReservedRange instance using the specified properties.
                     * @function create
                     * @memberof google.protobuf.EnumDescriptorProto.EnumReservedRange
                     * @static
                     * @param {google.protobuf.EnumDescriptorProto.IEnumReservedRange=} [properties] Properties to set
                     * @returns {google.protobuf.EnumDescriptorProto.EnumReservedRange} EnumReservedRange instance
                     */
                    EnumReservedRange.create = function create(properties) {
                        return new EnumReservedRange(properties);
                    };
    
                    /**
                     * Encodes the specified EnumReservedRange message. Does not implicitly {@link google.protobuf.EnumDescriptorProto.EnumReservedRange.verify|verify} messages.
                     * @function encode
                     * @memberof google.protobuf.EnumDescriptorProto.EnumReservedRange
                     * @static
                     * @param {google.protobuf.EnumDescriptorProto.IEnumReservedRange} message EnumReservedRange message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    EnumReservedRange.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.start != null && Object.hasOwnProperty.call(message, "start"))
                            writer.uint32(/* id 1, wireType 0 =*/8).int32(message.start);
                        if (message.end != null && Object.hasOwnProperty.call(message, "end"))
                            writer.uint32(/* id 2, wireType 0 =*/16).int32(message.end);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified EnumReservedRange message, length delimited. Does not implicitly {@link google.protobuf.EnumDescriptorProto.EnumReservedRange.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.protobuf.EnumDescriptorProto.EnumReservedRange
                     * @static
                     * @param {google.protobuf.EnumDescriptorProto.IEnumReservedRange} message EnumReservedRange message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    EnumReservedRange.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes an EnumReservedRange message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.protobuf.EnumDescriptorProto.EnumReservedRange
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.protobuf.EnumDescriptorProto.EnumReservedRange} EnumReservedRange
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    EnumReservedRange.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.EnumDescriptorProto.EnumReservedRange();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.start = reader.int32();
                                    break;
                                }
                            case 2: {
                                    message.end = reader.int32();
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes an EnumReservedRange message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.protobuf.EnumDescriptorProto.EnumReservedRange
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.protobuf.EnumDescriptorProto.EnumReservedRange} EnumReservedRange
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    EnumReservedRange.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies an EnumReservedRange message.
                     * @function verify
                     * @memberof google.protobuf.EnumDescriptorProto.EnumReservedRange
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    EnumReservedRange.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.start != null && message.hasOwnProperty("start"))
                            if (!$util.isInteger(message.start))
                                return "start: integer expected";
                        if (message.end != null && message.hasOwnProperty("end"))
                            if (!$util.isInteger(message.end))
                                return "end: integer expected";
                        return null;
                    };
    
                    /**
                     * Creates an EnumReservedRange message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.protobuf.EnumDescriptorProto.EnumReservedRange
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.protobuf.EnumDescriptorProto.EnumReservedRange} EnumReservedRange
                     */
                    EnumReservedRange.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.protobuf.EnumDescriptorProto.EnumReservedRange)
                            return object;
                        var message = new $root.google.protobuf.EnumDescriptorProto.EnumReservedRange();
                        if (object.start != null)
                            message.start = object.start | 0;
                        if (object.end != null)
                            message.end = object.end | 0;
                        return message;
                    };
    
                    /**
                     * Creates a plain object from an EnumReservedRange message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.protobuf.EnumDescriptorProto.EnumReservedRange
                     * @static
                     * @param {google.protobuf.EnumDescriptorProto.EnumReservedRange} message EnumReservedRange
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    EnumReservedRange.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults) {
                            object.start = 0;
                            object.end = 0;
                        }
                        if (message.start != null && message.hasOwnProperty("start"))
                            object.start = message.start;
                        if (message.end != null && message.hasOwnProperty("end"))
                            object.end = message.end;
                        return object;
                    };
    
                    /**
                     * Converts this EnumReservedRange to JSON.
                     * @function toJSON
                     * @memberof google.protobuf.EnumDescriptorProto.EnumReservedRange
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    EnumReservedRange.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for EnumReservedRange
                     * @function getTypeUrl
                     * @memberof google.protobuf.EnumDescriptorProto.EnumReservedRange
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    EnumReservedRange.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.protobuf.EnumDescriptorProto.EnumReservedRange";
                    };
    
                    return EnumReservedRange;
                })();
    
                return EnumDescriptorProto;
            })();
    
            protobuf.EnumValueDescriptorProto = (function() {
    
                /**
                 * Properties of an EnumValueDescriptorProto.
                 * @memberof google.protobuf
                 * @interface IEnumValueDescriptorProto
                 * @property {string|null} [name] EnumValueDescriptorProto name
                 * @property {number|null} [number] EnumValueDescriptorProto number
                 * @property {google.protobuf.IEnumValueOptions|null} [options] EnumValueDescriptorProto options
                 */
    
                /**
                 * Constructs a new EnumValueDescriptorProto.
                 * @memberof google.protobuf
                 * @classdesc Represents an EnumValueDescriptorProto.
                 * @implements IEnumValueDescriptorProto
                 * @constructor
                 * @param {google.protobuf.IEnumValueDescriptorProto=} [properties] Properties to set
                 */
                function EnumValueDescriptorProto(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * EnumValueDescriptorProto name.
                 * @member {string} name
                 * @memberof google.protobuf.EnumValueDescriptorProto
                 * @instance
                 */
                EnumValueDescriptorProto.prototype.name = "";
    
                /**
                 * EnumValueDescriptorProto number.
                 * @member {number} number
                 * @memberof google.protobuf.EnumValueDescriptorProto
                 * @instance
                 */
                EnumValueDescriptorProto.prototype.number = 0;
    
                /**
                 * EnumValueDescriptorProto options.
                 * @member {google.protobuf.IEnumValueOptions|null|undefined} options
                 * @memberof google.protobuf.EnumValueDescriptorProto
                 * @instance
                 */
                EnumValueDescriptorProto.prototype.options = null;
    
                /**
                 * Creates a new EnumValueDescriptorProto instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.EnumValueDescriptorProto
                 * @static
                 * @param {google.protobuf.IEnumValueDescriptorProto=} [properties] Properties to set
                 * @returns {google.protobuf.EnumValueDescriptorProto} EnumValueDescriptorProto instance
                 */
                EnumValueDescriptorProto.create = function create(properties) {
                    return new EnumValueDescriptorProto(properties);
                };
    
                /**
                 * Encodes the specified EnumValueDescriptorProto message. Does not implicitly {@link google.protobuf.EnumValueDescriptorProto.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.EnumValueDescriptorProto
                 * @static
                 * @param {google.protobuf.IEnumValueDescriptorProto} message EnumValueDescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                EnumValueDescriptorProto.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                    if (message.number != null && Object.hasOwnProperty.call(message, "number"))
                        writer.uint32(/* id 2, wireType 0 =*/16).int32(message.number);
                    if (message.options != null && Object.hasOwnProperty.call(message, "options"))
                        $root.google.protobuf.EnumValueOptions.encode(message.options, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                    return writer;
                };
    
                /**
                 * Encodes the specified EnumValueDescriptorProto message, length delimited. Does not implicitly {@link google.protobuf.EnumValueDescriptorProto.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.EnumValueDescriptorProto
                 * @static
                 * @param {google.protobuf.IEnumValueDescriptorProto} message EnumValueDescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                EnumValueDescriptorProto.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes an EnumValueDescriptorProto message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.EnumValueDescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.EnumValueDescriptorProto} EnumValueDescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                EnumValueDescriptorProto.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.EnumValueDescriptorProto();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.name = reader.string();
                                break;
                            }
                        case 2: {
                                message.number = reader.int32();
                                break;
                            }
                        case 3: {
                                message.options = $root.google.protobuf.EnumValueOptions.decode(reader, reader.uint32());
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes an EnumValueDescriptorProto message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.EnumValueDescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.EnumValueDescriptorProto} EnumValueDescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                EnumValueDescriptorProto.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies an EnumValueDescriptorProto message.
                 * @function verify
                 * @memberof google.protobuf.EnumValueDescriptorProto
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                EnumValueDescriptorProto.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.name != null && message.hasOwnProperty("name"))
                        if (!$util.isString(message.name))
                            return "name: string expected";
                    if (message.number != null && message.hasOwnProperty("number"))
                        if (!$util.isInteger(message.number))
                            return "number: integer expected";
                    if (message.options != null && message.hasOwnProperty("options")) {
                        var error = $root.google.protobuf.EnumValueOptions.verify(message.options);
                        if (error)
                            return "options." + error;
                    }
                    return null;
                };
    
                /**
                 * Creates an EnumValueDescriptorProto message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.EnumValueDescriptorProto
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.EnumValueDescriptorProto} EnumValueDescriptorProto
                 */
                EnumValueDescriptorProto.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.EnumValueDescriptorProto)
                        return object;
                    var message = new $root.google.protobuf.EnumValueDescriptorProto();
                    if (object.name != null)
                        message.name = String(object.name);
                    if (object.number != null)
                        message.number = object.number | 0;
                    if (object.options != null) {
                        if (typeof object.options !== "object")
                            throw TypeError(".google.protobuf.EnumValueDescriptorProto.options: object expected");
                        message.options = $root.google.protobuf.EnumValueOptions.fromObject(object.options);
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from an EnumValueDescriptorProto message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.EnumValueDescriptorProto
                 * @static
                 * @param {google.protobuf.EnumValueDescriptorProto} message EnumValueDescriptorProto
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                EnumValueDescriptorProto.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults) {
                        object.name = "";
                        object.number = 0;
                        object.options = null;
                    }
                    if (message.name != null && message.hasOwnProperty("name"))
                        object.name = message.name;
                    if (message.number != null && message.hasOwnProperty("number"))
                        object.number = message.number;
                    if (message.options != null && message.hasOwnProperty("options"))
                        object.options = $root.google.protobuf.EnumValueOptions.toObject(message.options, options);
                    return object;
                };
    
                /**
                 * Converts this EnumValueDescriptorProto to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.EnumValueDescriptorProto
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                EnumValueDescriptorProto.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for EnumValueDescriptorProto
                 * @function getTypeUrl
                 * @memberof google.protobuf.EnumValueDescriptorProto
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                EnumValueDescriptorProto.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.EnumValueDescriptorProto";
                };
    
                return EnumValueDescriptorProto;
            })();
    
            protobuf.ServiceDescriptorProto = (function() {
    
                /**
                 * Properties of a ServiceDescriptorProto.
                 * @memberof google.protobuf
                 * @interface IServiceDescriptorProto
                 * @property {string|null} [name] ServiceDescriptorProto name
                 * @property {Array.<google.protobuf.IMethodDescriptorProto>|null} [method] ServiceDescriptorProto method
                 * @property {google.protobuf.IServiceOptions|null} [options] ServiceDescriptorProto options
                 */
    
                /**
                 * Constructs a new ServiceDescriptorProto.
                 * @memberof google.protobuf
                 * @classdesc Represents a ServiceDescriptorProto.
                 * @implements IServiceDescriptorProto
                 * @constructor
                 * @param {google.protobuf.IServiceDescriptorProto=} [properties] Properties to set
                 */
                function ServiceDescriptorProto(properties) {
                    this.method = [];
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * ServiceDescriptorProto name.
                 * @member {string} name
                 * @memberof google.protobuf.ServiceDescriptorProto
                 * @instance
                 */
                ServiceDescriptorProto.prototype.name = "";
    
                /**
                 * ServiceDescriptorProto method.
                 * @member {Array.<google.protobuf.IMethodDescriptorProto>} method
                 * @memberof google.protobuf.ServiceDescriptorProto
                 * @instance
                 */
                ServiceDescriptorProto.prototype.method = $util.emptyArray;
    
                /**
                 * ServiceDescriptorProto options.
                 * @member {google.protobuf.IServiceOptions|null|undefined} options
                 * @memberof google.protobuf.ServiceDescriptorProto
                 * @instance
                 */
                ServiceDescriptorProto.prototype.options = null;
    
                /**
                 * Creates a new ServiceDescriptorProto instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.ServiceDescriptorProto
                 * @static
                 * @param {google.protobuf.IServiceDescriptorProto=} [properties] Properties to set
                 * @returns {google.protobuf.ServiceDescriptorProto} ServiceDescriptorProto instance
                 */
                ServiceDescriptorProto.create = function create(properties) {
                    return new ServiceDescriptorProto(properties);
                };
    
                /**
                 * Encodes the specified ServiceDescriptorProto message. Does not implicitly {@link google.protobuf.ServiceDescriptorProto.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.ServiceDescriptorProto
                 * @static
                 * @param {google.protobuf.IServiceDescriptorProto} message ServiceDescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ServiceDescriptorProto.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                    if (message.method != null && message.method.length)
                        for (var i = 0; i < message.method.length; ++i)
                            $root.google.protobuf.MethodDescriptorProto.encode(message.method[i], writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                    if (message.options != null && Object.hasOwnProperty.call(message, "options"))
                        $root.google.protobuf.ServiceOptions.encode(message.options, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                    return writer;
                };
    
                /**
                 * Encodes the specified ServiceDescriptorProto message, length delimited. Does not implicitly {@link google.protobuf.ServiceDescriptorProto.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.ServiceDescriptorProto
                 * @static
                 * @param {google.protobuf.IServiceDescriptorProto} message ServiceDescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ServiceDescriptorProto.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a ServiceDescriptorProto message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.ServiceDescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.ServiceDescriptorProto} ServiceDescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ServiceDescriptorProto.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.ServiceDescriptorProto();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.name = reader.string();
                                break;
                            }
                        case 2: {
                                if (!(message.method && message.method.length))
                                    message.method = [];
                                message.method.push($root.google.protobuf.MethodDescriptorProto.decode(reader, reader.uint32()));
                                break;
                            }
                        case 3: {
                                message.options = $root.google.protobuf.ServiceOptions.decode(reader, reader.uint32());
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a ServiceDescriptorProto message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.ServiceDescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.ServiceDescriptorProto} ServiceDescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ServiceDescriptorProto.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a ServiceDescriptorProto message.
                 * @function verify
                 * @memberof google.protobuf.ServiceDescriptorProto
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                ServiceDescriptorProto.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.name != null && message.hasOwnProperty("name"))
                        if (!$util.isString(message.name))
                            return "name: string expected";
                    if (message.method != null && message.hasOwnProperty("method")) {
                        if (!Array.isArray(message.method))
                            return "method: array expected";
                        for (var i = 0; i < message.method.length; ++i) {
                            var error = $root.google.protobuf.MethodDescriptorProto.verify(message.method[i]);
                            if (error)
                                return "method." + error;
                        }
                    }
                    if (message.options != null && message.hasOwnProperty("options")) {
                        var error = $root.google.protobuf.ServiceOptions.verify(message.options);
                        if (error)
                            return "options." + error;
                    }
                    return null;
                };
    
                /**
                 * Creates a ServiceDescriptorProto message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.ServiceDescriptorProto
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.ServiceDescriptorProto} ServiceDescriptorProto
                 */
                ServiceDescriptorProto.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.ServiceDescriptorProto)
                        return object;
                    var message = new $root.google.protobuf.ServiceDescriptorProto();
                    if (object.name != null)
                        message.name = String(object.name);
                    if (object.method) {
                        if (!Array.isArray(object.method))
                            throw TypeError(".google.protobuf.ServiceDescriptorProto.method: array expected");
                        message.method = [];
                        for (var i = 0; i < object.method.length; ++i) {
                            if (typeof object.method[i] !== "object")
                                throw TypeError(".google.protobuf.ServiceDescriptorProto.method: object expected");
                            message.method[i] = $root.google.protobuf.MethodDescriptorProto.fromObject(object.method[i]);
                        }
                    }
                    if (object.options != null) {
                        if (typeof object.options !== "object")
                            throw TypeError(".google.protobuf.ServiceDescriptorProto.options: object expected");
                        message.options = $root.google.protobuf.ServiceOptions.fromObject(object.options);
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from a ServiceDescriptorProto message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.ServiceDescriptorProto
                 * @static
                 * @param {google.protobuf.ServiceDescriptorProto} message ServiceDescriptorProto
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                ServiceDescriptorProto.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.arrays || options.defaults)
                        object.method = [];
                    if (options.defaults) {
                        object.name = "";
                        object.options = null;
                    }
                    if (message.name != null && message.hasOwnProperty("name"))
                        object.name = message.name;
                    if (message.method && message.method.length) {
                        object.method = [];
                        for (var j = 0; j < message.method.length; ++j)
                            object.method[j] = $root.google.protobuf.MethodDescriptorProto.toObject(message.method[j], options);
                    }
                    if (message.options != null && message.hasOwnProperty("options"))
                        object.options = $root.google.protobuf.ServiceOptions.toObject(message.options, options);
                    return object;
                };
    
                /**
                 * Converts this ServiceDescriptorProto to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.ServiceDescriptorProto
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                ServiceDescriptorProto.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for ServiceDescriptorProto
                 * @function getTypeUrl
                 * @memberof google.protobuf.ServiceDescriptorProto
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                ServiceDescriptorProto.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.ServiceDescriptorProto";
                };
    
                return ServiceDescriptorProto;
            })();
    
            protobuf.MethodDescriptorProto = (function() {
    
                /**
                 * Properties of a MethodDescriptorProto.
                 * @memberof google.protobuf
                 * @interface IMethodDescriptorProto
                 * @property {string|null} [name] MethodDescriptorProto name
                 * @property {string|null} [inputType] MethodDescriptorProto inputType
                 * @property {string|null} [outputType] MethodDescriptorProto outputType
                 * @property {google.protobuf.IMethodOptions|null} [options] MethodDescriptorProto options
                 * @property {boolean|null} [clientStreaming] MethodDescriptorProto clientStreaming
                 * @property {boolean|null} [serverStreaming] MethodDescriptorProto serverStreaming
                 */
    
                /**
                 * Constructs a new MethodDescriptorProto.
                 * @memberof google.protobuf
                 * @classdesc Represents a MethodDescriptorProto.
                 * @implements IMethodDescriptorProto
                 * @constructor
                 * @param {google.protobuf.IMethodDescriptorProto=} [properties] Properties to set
                 */
                function MethodDescriptorProto(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * MethodDescriptorProto name.
                 * @member {string} name
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @instance
                 */
                MethodDescriptorProto.prototype.name = "";
    
                /**
                 * MethodDescriptorProto inputType.
                 * @member {string} inputType
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @instance
                 */
                MethodDescriptorProto.prototype.inputType = "";
    
                /**
                 * MethodDescriptorProto outputType.
                 * @member {string} outputType
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @instance
                 */
                MethodDescriptorProto.prototype.outputType = "";
    
                /**
                 * MethodDescriptorProto options.
                 * @member {google.protobuf.IMethodOptions|null|undefined} options
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @instance
                 */
                MethodDescriptorProto.prototype.options = null;
    
                /**
                 * MethodDescriptorProto clientStreaming.
                 * @member {boolean} clientStreaming
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @instance
                 */
                MethodDescriptorProto.prototype.clientStreaming = false;
    
                /**
                 * MethodDescriptorProto serverStreaming.
                 * @member {boolean} serverStreaming
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @instance
                 */
                MethodDescriptorProto.prototype.serverStreaming = false;
    
                /**
                 * Creates a new MethodDescriptorProto instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @static
                 * @param {google.protobuf.IMethodDescriptorProto=} [properties] Properties to set
                 * @returns {google.protobuf.MethodDescriptorProto} MethodDescriptorProto instance
                 */
                MethodDescriptorProto.create = function create(properties) {
                    return new MethodDescriptorProto(properties);
                };
    
                /**
                 * Encodes the specified MethodDescriptorProto message. Does not implicitly {@link google.protobuf.MethodDescriptorProto.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @static
                 * @param {google.protobuf.IMethodDescriptorProto} message MethodDescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                MethodDescriptorProto.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                    if (message.inputType != null && Object.hasOwnProperty.call(message, "inputType"))
                        writer.uint32(/* id 2, wireType 2 =*/18).string(message.inputType);
                    if (message.outputType != null && Object.hasOwnProperty.call(message, "outputType"))
                        writer.uint32(/* id 3, wireType 2 =*/26).string(message.outputType);
                    if (message.options != null && Object.hasOwnProperty.call(message, "options"))
                        $root.google.protobuf.MethodOptions.encode(message.options, writer.uint32(/* id 4, wireType 2 =*/34).fork()).ldelim();
                    if (message.clientStreaming != null && Object.hasOwnProperty.call(message, "clientStreaming"))
                        writer.uint32(/* id 5, wireType 0 =*/40).bool(message.clientStreaming);
                    if (message.serverStreaming != null && Object.hasOwnProperty.call(message, "serverStreaming"))
                        writer.uint32(/* id 6, wireType 0 =*/48).bool(message.serverStreaming);
                    return writer;
                };
    
                /**
                 * Encodes the specified MethodDescriptorProto message, length delimited. Does not implicitly {@link google.protobuf.MethodDescriptorProto.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @static
                 * @param {google.protobuf.IMethodDescriptorProto} message MethodDescriptorProto message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                MethodDescriptorProto.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a MethodDescriptorProto message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.MethodDescriptorProto} MethodDescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                MethodDescriptorProto.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.MethodDescriptorProto();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.name = reader.string();
                                break;
                            }
                        case 2: {
                                message.inputType = reader.string();
                                break;
                            }
                        case 3: {
                                message.outputType = reader.string();
                                break;
                            }
                        case 4: {
                                message.options = $root.google.protobuf.MethodOptions.decode(reader, reader.uint32());
                                break;
                            }
                        case 5: {
                                message.clientStreaming = reader.bool();
                                break;
                            }
                        case 6: {
                                message.serverStreaming = reader.bool();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a MethodDescriptorProto message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.MethodDescriptorProto} MethodDescriptorProto
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                MethodDescriptorProto.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a MethodDescriptorProto message.
                 * @function verify
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                MethodDescriptorProto.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.name != null && message.hasOwnProperty("name"))
                        if (!$util.isString(message.name))
                            return "name: string expected";
                    if (message.inputType != null && message.hasOwnProperty("inputType"))
                        if (!$util.isString(message.inputType))
                            return "inputType: string expected";
                    if (message.outputType != null && message.hasOwnProperty("outputType"))
                        if (!$util.isString(message.outputType))
                            return "outputType: string expected";
                    if (message.options != null && message.hasOwnProperty("options")) {
                        var error = $root.google.protobuf.MethodOptions.verify(message.options);
                        if (error)
                            return "options." + error;
                    }
                    if (message.clientStreaming != null && message.hasOwnProperty("clientStreaming"))
                        if (typeof message.clientStreaming !== "boolean")
                            return "clientStreaming: boolean expected";
                    if (message.serverStreaming != null && message.hasOwnProperty("serverStreaming"))
                        if (typeof message.serverStreaming !== "boolean")
                            return "serverStreaming: boolean expected";
                    return null;
                };
    
                /**
                 * Creates a MethodDescriptorProto message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.MethodDescriptorProto} MethodDescriptorProto
                 */
                MethodDescriptorProto.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.MethodDescriptorProto)
                        return object;
                    var message = new $root.google.protobuf.MethodDescriptorProto();
                    if (object.name != null)
                        message.name = String(object.name);
                    if (object.inputType != null)
                        message.inputType = String(object.inputType);
                    if (object.outputType != null)
                        message.outputType = String(object.outputType);
                    if (object.options != null) {
                        if (typeof object.options !== "object")
                            throw TypeError(".google.protobuf.MethodDescriptorProto.options: object expected");
                        message.options = $root.google.protobuf.MethodOptions.fromObject(object.options);
                    }
                    if (object.clientStreaming != null)
                        message.clientStreaming = Boolean(object.clientStreaming);
                    if (object.serverStreaming != null)
                        message.serverStreaming = Boolean(object.serverStreaming);
                    return message;
                };
    
                /**
                 * Creates a plain object from a MethodDescriptorProto message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @static
                 * @param {google.protobuf.MethodDescriptorProto} message MethodDescriptorProto
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                MethodDescriptorProto.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults) {
                        object.name = "";
                        object.inputType = "";
                        object.outputType = "";
                        object.options = null;
                        object.clientStreaming = false;
                        object.serverStreaming = false;
                    }
                    if (message.name != null && message.hasOwnProperty("name"))
                        object.name = message.name;
                    if (message.inputType != null && message.hasOwnProperty("inputType"))
                        object.inputType = message.inputType;
                    if (message.outputType != null && message.hasOwnProperty("outputType"))
                        object.outputType = message.outputType;
                    if (message.options != null && message.hasOwnProperty("options"))
                        object.options = $root.google.protobuf.MethodOptions.toObject(message.options, options);
                    if (message.clientStreaming != null && message.hasOwnProperty("clientStreaming"))
                        object.clientStreaming = message.clientStreaming;
                    if (message.serverStreaming != null && message.hasOwnProperty("serverStreaming"))
                        object.serverStreaming = message.serverStreaming;
                    return object;
                };
    
                /**
                 * Converts this MethodDescriptorProto to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                MethodDescriptorProto.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for MethodDescriptorProto
                 * @function getTypeUrl
                 * @memberof google.protobuf.MethodDescriptorProto
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                MethodDescriptorProto.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.MethodDescriptorProto";
                };
    
                return MethodDescriptorProto;
            })();
    
            protobuf.FileOptions = (function() {
    
                /**
                 * Properties of a FileOptions.
                 * @memberof google.protobuf
                 * @interface IFileOptions
                 * @property {string|null} [javaPackage] FileOptions javaPackage
                 * @property {string|null} [javaOuterClassname] FileOptions javaOuterClassname
                 * @property {boolean|null} [javaMultipleFiles] FileOptions javaMultipleFiles
                 * @property {boolean|null} [javaGenerateEqualsAndHash] FileOptions javaGenerateEqualsAndHash
                 * @property {boolean|null} [javaStringCheckUtf8] FileOptions javaStringCheckUtf8
                 * @property {google.protobuf.FileOptions.OptimizeMode|null} [optimizeFor] FileOptions optimizeFor
                 * @property {string|null} [goPackage] FileOptions goPackage
                 * @property {boolean|null} [ccGenericServices] FileOptions ccGenericServices
                 * @property {boolean|null} [javaGenericServices] FileOptions javaGenericServices
                 * @property {boolean|null} [pyGenericServices] FileOptions pyGenericServices
                 * @property {boolean|null} [deprecated] FileOptions deprecated
                 * @property {boolean|null} [ccEnableArenas] FileOptions ccEnableArenas
                 * @property {string|null} [objcClassPrefix] FileOptions objcClassPrefix
                 * @property {string|null} [csharpNamespace] FileOptions csharpNamespace
                 * @property {string|null} [swiftPrefix] FileOptions swiftPrefix
                 * @property {string|null} [phpClassPrefix] FileOptions phpClassPrefix
                 * @property {string|null} [phpNamespace] FileOptions phpNamespace
                 * @property {string|null} [phpMetadataNamespace] FileOptions phpMetadataNamespace
                 * @property {string|null} [rubyPackage] FileOptions rubyPackage
                 * @property {google.protobuf.IFeatureSet|null} [features] FileOptions features
                 * @property {Array.<google.protobuf.IUninterpretedOption>|null} [uninterpretedOption] FileOptions uninterpretedOption
                 * @property {Array.<google.api.IResourceDescriptor>|null} [".google.api.resourceDefinition"] FileOptions .google.api.resourceDefinition
                 */
    
                /**
                 * Constructs a new FileOptions.
                 * @memberof google.protobuf
                 * @classdesc Represents a FileOptions.
                 * @implements IFileOptions
                 * @constructor
                 * @param {google.protobuf.IFileOptions=} [properties] Properties to set
                 */
                function FileOptions(properties) {
                    this.uninterpretedOption = [];
                    this[".google.api.resourceDefinition"] = [];
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * FileOptions javaPackage.
                 * @member {string} javaPackage
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.javaPackage = "";
    
                /**
                 * FileOptions javaOuterClassname.
                 * @member {string} javaOuterClassname
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.javaOuterClassname = "";
    
                /**
                 * FileOptions javaMultipleFiles.
                 * @member {boolean} javaMultipleFiles
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.javaMultipleFiles = false;
    
                /**
                 * FileOptions javaGenerateEqualsAndHash.
                 * @member {boolean} javaGenerateEqualsAndHash
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.javaGenerateEqualsAndHash = false;
    
                /**
                 * FileOptions javaStringCheckUtf8.
                 * @member {boolean} javaStringCheckUtf8
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.javaStringCheckUtf8 = false;
    
                /**
                 * FileOptions optimizeFor.
                 * @member {google.protobuf.FileOptions.OptimizeMode} optimizeFor
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.optimizeFor = 1;
    
                /**
                 * FileOptions goPackage.
                 * @member {string} goPackage
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.goPackage = "";
    
                /**
                 * FileOptions ccGenericServices.
                 * @member {boolean} ccGenericServices
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.ccGenericServices = false;
    
                /**
                 * FileOptions javaGenericServices.
                 * @member {boolean} javaGenericServices
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.javaGenericServices = false;
    
                /**
                 * FileOptions pyGenericServices.
                 * @member {boolean} pyGenericServices
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.pyGenericServices = false;
    
                /**
                 * FileOptions deprecated.
                 * @member {boolean} deprecated
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.deprecated = false;
    
                /**
                 * FileOptions ccEnableArenas.
                 * @member {boolean} ccEnableArenas
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.ccEnableArenas = true;
    
                /**
                 * FileOptions objcClassPrefix.
                 * @member {string} objcClassPrefix
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.objcClassPrefix = "";
    
                /**
                 * FileOptions csharpNamespace.
                 * @member {string} csharpNamespace
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.csharpNamespace = "";
    
                /**
                 * FileOptions swiftPrefix.
                 * @member {string} swiftPrefix
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.swiftPrefix = "";
    
                /**
                 * FileOptions phpClassPrefix.
                 * @member {string} phpClassPrefix
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.phpClassPrefix = "";
    
                /**
                 * FileOptions phpNamespace.
                 * @member {string} phpNamespace
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.phpNamespace = "";
    
                /**
                 * FileOptions phpMetadataNamespace.
                 * @member {string} phpMetadataNamespace
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.phpMetadataNamespace = "";
    
                /**
                 * FileOptions rubyPackage.
                 * @member {string} rubyPackage
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.rubyPackage = "";
    
                /**
                 * FileOptions features.
                 * @member {google.protobuf.IFeatureSet|null|undefined} features
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.features = null;
    
                /**
                 * FileOptions uninterpretedOption.
                 * @member {Array.<google.protobuf.IUninterpretedOption>} uninterpretedOption
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype.uninterpretedOption = $util.emptyArray;
    
                /**
                 * FileOptions .google.api.resourceDefinition.
                 * @member {Array.<google.api.IResourceDescriptor>} .google.api.resourceDefinition
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 */
                FileOptions.prototype[".google.api.resourceDefinition"] = $util.emptyArray;
    
                /**
                 * Creates a new FileOptions instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.FileOptions
                 * @static
                 * @param {google.protobuf.IFileOptions=} [properties] Properties to set
                 * @returns {google.protobuf.FileOptions} FileOptions instance
                 */
                FileOptions.create = function create(properties) {
                    return new FileOptions(properties);
                };
    
                /**
                 * Encodes the specified FileOptions message. Does not implicitly {@link google.protobuf.FileOptions.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.FileOptions
                 * @static
                 * @param {google.protobuf.IFileOptions} message FileOptions message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FileOptions.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.javaPackage != null && Object.hasOwnProperty.call(message, "javaPackage"))
                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.javaPackage);
                    if (message.javaOuterClassname != null && Object.hasOwnProperty.call(message, "javaOuterClassname"))
                        writer.uint32(/* id 8, wireType 2 =*/66).string(message.javaOuterClassname);
                    if (message.optimizeFor != null && Object.hasOwnProperty.call(message, "optimizeFor"))
                        writer.uint32(/* id 9, wireType 0 =*/72).int32(message.optimizeFor);
                    if (message.javaMultipleFiles != null && Object.hasOwnProperty.call(message, "javaMultipleFiles"))
                        writer.uint32(/* id 10, wireType 0 =*/80).bool(message.javaMultipleFiles);
                    if (message.goPackage != null && Object.hasOwnProperty.call(message, "goPackage"))
                        writer.uint32(/* id 11, wireType 2 =*/90).string(message.goPackage);
                    if (message.ccGenericServices != null && Object.hasOwnProperty.call(message, "ccGenericServices"))
                        writer.uint32(/* id 16, wireType 0 =*/128).bool(message.ccGenericServices);
                    if (message.javaGenericServices != null && Object.hasOwnProperty.call(message, "javaGenericServices"))
                        writer.uint32(/* id 17, wireType 0 =*/136).bool(message.javaGenericServices);
                    if (message.pyGenericServices != null && Object.hasOwnProperty.call(message, "pyGenericServices"))
                        writer.uint32(/* id 18, wireType 0 =*/144).bool(message.pyGenericServices);
                    if (message.javaGenerateEqualsAndHash != null && Object.hasOwnProperty.call(message, "javaGenerateEqualsAndHash"))
                        writer.uint32(/* id 20, wireType 0 =*/160).bool(message.javaGenerateEqualsAndHash);
                    if (message.deprecated != null && Object.hasOwnProperty.call(message, "deprecated"))
                        writer.uint32(/* id 23, wireType 0 =*/184).bool(message.deprecated);
                    if (message.javaStringCheckUtf8 != null && Object.hasOwnProperty.call(message, "javaStringCheckUtf8"))
                        writer.uint32(/* id 27, wireType 0 =*/216).bool(message.javaStringCheckUtf8);
                    if (message.ccEnableArenas != null && Object.hasOwnProperty.call(message, "ccEnableArenas"))
                        writer.uint32(/* id 31, wireType 0 =*/248).bool(message.ccEnableArenas);
                    if (message.objcClassPrefix != null && Object.hasOwnProperty.call(message, "objcClassPrefix"))
                        writer.uint32(/* id 36, wireType 2 =*/290).string(message.objcClassPrefix);
                    if (message.csharpNamespace != null && Object.hasOwnProperty.call(message, "csharpNamespace"))
                        writer.uint32(/* id 37, wireType 2 =*/298).string(message.csharpNamespace);
                    if (message.swiftPrefix != null && Object.hasOwnProperty.call(message, "swiftPrefix"))
                        writer.uint32(/* id 39, wireType 2 =*/314).string(message.swiftPrefix);
                    if (message.phpClassPrefix != null && Object.hasOwnProperty.call(message, "phpClassPrefix"))
                        writer.uint32(/* id 40, wireType 2 =*/322).string(message.phpClassPrefix);
                    if (message.phpNamespace != null && Object.hasOwnProperty.call(message, "phpNamespace"))
                        writer.uint32(/* id 41, wireType 2 =*/330).string(message.phpNamespace);
                    if (message.phpMetadataNamespace != null && Object.hasOwnProperty.call(message, "phpMetadataNamespace"))
                        writer.uint32(/* id 44, wireType 2 =*/354).string(message.phpMetadataNamespace);
                    if (message.rubyPackage != null && Object.hasOwnProperty.call(message, "rubyPackage"))
                        writer.uint32(/* id 45, wireType 2 =*/362).string(message.rubyPackage);
                    if (message.features != null && Object.hasOwnProperty.call(message, "features"))
                        $root.google.protobuf.FeatureSet.encode(message.features, writer.uint32(/* id 50, wireType 2 =*/402).fork()).ldelim();
                    if (message.uninterpretedOption != null && message.uninterpretedOption.length)
                        for (var i = 0; i < message.uninterpretedOption.length; ++i)
                            $root.google.protobuf.UninterpretedOption.encode(message.uninterpretedOption[i], writer.uint32(/* id 999, wireType 2 =*/7994).fork()).ldelim();
                    if (message[".google.api.resourceDefinition"] != null && message[".google.api.resourceDefinition"].length)
                        for (var i = 0; i < message[".google.api.resourceDefinition"].length; ++i)
                            $root.google.api.ResourceDescriptor.encode(message[".google.api.resourceDefinition"][i], writer.uint32(/* id 1053, wireType 2 =*/8426).fork()).ldelim();
                    return writer;
                };
    
                /**
                 * Encodes the specified FileOptions message, length delimited. Does not implicitly {@link google.protobuf.FileOptions.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.FileOptions
                 * @static
                 * @param {google.protobuf.IFileOptions} message FileOptions message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FileOptions.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a FileOptions message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.FileOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.FileOptions} FileOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FileOptions.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.FileOptions();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.javaPackage = reader.string();
                                break;
                            }
                        case 8: {
                                message.javaOuterClassname = reader.string();
                                break;
                            }
                        case 10: {
                                message.javaMultipleFiles = reader.bool();
                                break;
                            }
                        case 20: {
                                message.javaGenerateEqualsAndHash = reader.bool();
                                break;
                            }
                        case 27: {
                                message.javaStringCheckUtf8 = reader.bool();
                                break;
                            }
                        case 9: {
                                message.optimizeFor = reader.int32();
                                break;
                            }
                        case 11: {
                                message.goPackage = reader.string();
                                break;
                            }
                        case 16: {
                                message.ccGenericServices = reader.bool();
                                break;
                            }
                        case 17: {
                                message.javaGenericServices = reader.bool();
                                break;
                            }
                        case 18: {
                                message.pyGenericServices = reader.bool();
                                break;
                            }
                        case 23: {
                                message.deprecated = reader.bool();
                                break;
                            }
                        case 31: {
                                message.ccEnableArenas = reader.bool();
                                break;
                            }
                        case 36: {
                                message.objcClassPrefix = reader.string();
                                break;
                            }
                        case 37: {
                                message.csharpNamespace = reader.string();
                                break;
                            }
                        case 39: {
                                message.swiftPrefix = reader.string();
                                break;
                            }
                        case 40: {
                                message.phpClassPrefix = reader.string();
                                break;
                            }
                        case 41: {
                                message.phpNamespace = reader.string();
                                break;
                            }
                        case 44: {
                                message.phpMetadataNamespace = reader.string();
                                break;
                            }
                        case 45: {
                                message.rubyPackage = reader.string();
                                break;
                            }
                        case 50: {
                                message.features = $root.google.protobuf.FeatureSet.decode(reader, reader.uint32());
                                break;
                            }
                        case 999: {
                                if (!(message.uninterpretedOption && message.uninterpretedOption.length))
                                    message.uninterpretedOption = [];
                                message.uninterpretedOption.push($root.google.protobuf.UninterpretedOption.decode(reader, reader.uint32()));
                                break;
                            }
                        case 1053: {
                                if (!(message[".google.api.resourceDefinition"] && message[".google.api.resourceDefinition"].length))
                                    message[".google.api.resourceDefinition"] = [];
                                message[".google.api.resourceDefinition"].push($root.google.api.ResourceDescriptor.decode(reader, reader.uint32()));
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a FileOptions message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.FileOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.FileOptions} FileOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FileOptions.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a FileOptions message.
                 * @function verify
                 * @memberof google.protobuf.FileOptions
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                FileOptions.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.javaPackage != null && message.hasOwnProperty("javaPackage"))
                        if (!$util.isString(message.javaPackage))
                            return "javaPackage: string expected";
                    if (message.javaOuterClassname != null && message.hasOwnProperty("javaOuterClassname"))
                        if (!$util.isString(message.javaOuterClassname))
                            return "javaOuterClassname: string expected";
                    if (message.javaMultipleFiles != null && message.hasOwnProperty("javaMultipleFiles"))
                        if (typeof message.javaMultipleFiles !== "boolean")
                            return "javaMultipleFiles: boolean expected";
                    if (message.javaGenerateEqualsAndHash != null && message.hasOwnProperty("javaGenerateEqualsAndHash"))
                        if (typeof message.javaGenerateEqualsAndHash !== "boolean")
                            return "javaGenerateEqualsAndHash: boolean expected";
                    if (message.javaStringCheckUtf8 != null && message.hasOwnProperty("javaStringCheckUtf8"))
                        if (typeof message.javaStringCheckUtf8 !== "boolean")
                            return "javaStringCheckUtf8: boolean expected";
                    if (message.optimizeFor != null && message.hasOwnProperty("optimizeFor"))
                        switch (message.optimizeFor) {
                        default:
                            return "optimizeFor: enum value expected";
                        case 1:
                        case 2:
                        case 3:
                            break;
                        }
                    if (message.goPackage != null && message.hasOwnProperty("goPackage"))
                        if (!$util.isString(message.goPackage))
                            return "goPackage: string expected";
                    if (message.ccGenericServices != null && message.hasOwnProperty("ccGenericServices"))
                        if (typeof message.ccGenericServices !== "boolean")
                            return "ccGenericServices: boolean expected";
                    if (message.javaGenericServices != null && message.hasOwnProperty("javaGenericServices"))
                        if (typeof message.javaGenericServices !== "boolean")
                            return "javaGenericServices: boolean expected";
                    if (message.pyGenericServices != null && message.hasOwnProperty("pyGenericServices"))
                        if (typeof message.pyGenericServices !== "boolean")
                            return "pyGenericServices: boolean expected";
                    if (message.deprecated != null && message.hasOwnProperty("deprecated"))
                        if (typeof message.deprecated !== "boolean")
                            return "deprecated: boolean expected";
                    if (message.ccEnableArenas != null && message.hasOwnProperty("ccEnableArenas"))
                        if (typeof message.ccEnableArenas !== "boolean")
                            return "ccEnableArenas: boolean expected";
                    if (message.objcClassPrefix != null && message.hasOwnProperty("objcClassPrefix"))
                        if (!$util.isString(message.objcClassPrefix))
                            return "objcClassPrefix: string expected";
                    if (message.csharpNamespace != null && message.hasOwnProperty("csharpNamespace"))
                        if (!$util.isString(message.csharpNamespace))
                            return "csharpNamespace: string expected";
                    if (message.swiftPrefix != null && message.hasOwnProperty("swiftPrefix"))
                        if (!$util.isString(message.swiftPrefix))
                            return "swiftPrefix: string expected";
                    if (message.phpClassPrefix != null && message.hasOwnProperty("phpClassPrefix"))
                        if (!$util.isString(message.phpClassPrefix))
                            return "phpClassPrefix: string expected";
                    if (message.phpNamespace != null && message.hasOwnProperty("phpNamespace"))
                        if (!$util.isString(message.phpNamespace))
                            return "phpNamespace: string expected";
                    if (message.phpMetadataNamespace != null && message.hasOwnProperty("phpMetadataNamespace"))
                        if (!$util.isString(message.phpMetadataNamespace))
                            return "phpMetadataNamespace: string expected";
                    if (message.rubyPackage != null && message.hasOwnProperty("rubyPackage"))
                        if (!$util.isString(message.rubyPackage))
                            return "rubyPackage: string expected";
                    if (message.features != null && message.hasOwnProperty("features")) {
                        var error = $root.google.protobuf.FeatureSet.verify(message.features);
                        if (error)
                            return "features." + error;
                    }
                    if (message.uninterpretedOption != null && message.hasOwnProperty("uninterpretedOption")) {
                        if (!Array.isArray(message.uninterpretedOption))
                            return "uninterpretedOption: array expected";
                        for (var i = 0; i < message.uninterpretedOption.length; ++i) {
                            var error = $root.google.protobuf.UninterpretedOption.verify(message.uninterpretedOption[i]);
                            if (error)
                                return "uninterpretedOption." + error;
                        }
                    }
                    if (message[".google.api.resourceDefinition"] != null && message.hasOwnProperty(".google.api.resourceDefinition")) {
                        if (!Array.isArray(message[".google.api.resourceDefinition"]))
                            return ".google.api.resourceDefinition: array expected";
                        for (var i = 0; i < message[".google.api.resourceDefinition"].length; ++i) {
                            var error = $root.google.api.ResourceDescriptor.verify(message[".google.api.resourceDefinition"][i]);
                            if (error)
                                return ".google.api.resourceDefinition." + error;
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates a FileOptions message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.FileOptions
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.FileOptions} FileOptions
                 */
                FileOptions.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.FileOptions)
                        return object;
                    var message = new $root.google.protobuf.FileOptions();
                    if (object.javaPackage != null)
                        message.javaPackage = String(object.javaPackage);
                    if (object.javaOuterClassname != null)
                        message.javaOuterClassname = String(object.javaOuterClassname);
                    if (object.javaMultipleFiles != null)
                        message.javaMultipleFiles = Boolean(object.javaMultipleFiles);
                    if (object.javaGenerateEqualsAndHash != null)
                        message.javaGenerateEqualsAndHash = Boolean(object.javaGenerateEqualsAndHash);
                    if (object.javaStringCheckUtf8 != null)
                        message.javaStringCheckUtf8 = Boolean(object.javaStringCheckUtf8);
                    switch (object.optimizeFor) {
                    default:
                        if (typeof object.optimizeFor === "number") {
                            message.optimizeFor = object.optimizeFor;
                            break;
                        }
                        break;
                    case "SPEED":
                    case 1:
                        message.optimizeFor = 1;
                        break;
                    case "CODE_SIZE":
                    case 2:
                        message.optimizeFor = 2;
                        break;
                    case "LITE_RUNTIME":
                    case 3:
                        message.optimizeFor = 3;
                        break;
                    }
                    if (object.goPackage != null)
                        message.goPackage = String(object.goPackage);
                    if (object.ccGenericServices != null)
                        message.ccGenericServices = Boolean(object.ccGenericServices);
                    if (object.javaGenericServices != null)
                        message.javaGenericServices = Boolean(object.javaGenericServices);
                    if (object.pyGenericServices != null)
                        message.pyGenericServices = Boolean(object.pyGenericServices);
                    if (object.deprecated != null)
                        message.deprecated = Boolean(object.deprecated);
                    if (object.ccEnableArenas != null)
                        message.ccEnableArenas = Boolean(object.ccEnableArenas);
                    if (object.objcClassPrefix != null)
                        message.objcClassPrefix = String(object.objcClassPrefix);
                    if (object.csharpNamespace != null)
                        message.csharpNamespace = String(object.csharpNamespace);
                    if (object.swiftPrefix != null)
                        message.swiftPrefix = String(object.swiftPrefix);
                    if (object.phpClassPrefix != null)
                        message.phpClassPrefix = String(object.phpClassPrefix);
                    if (object.phpNamespace != null)
                        message.phpNamespace = String(object.phpNamespace);
                    if (object.phpMetadataNamespace != null)
                        message.phpMetadataNamespace = String(object.phpMetadataNamespace);
                    if (object.rubyPackage != null)
                        message.rubyPackage = String(object.rubyPackage);
                    if (object.features != null) {
                        if (typeof object.features !== "object")
                            throw TypeError(".google.protobuf.FileOptions.features: object expected");
                        message.features = $root.google.protobuf.FeatureSet.fromObject(object.features);
                    }
                    if (object.uninterpretedOption) {
                        if (!Array.isArray(object.uninterpretedOption))
                            throw TypeError(".google.protobuf.FileOptions.uninterpretedOption: array expected");
                        message.uninterpretedOption = [];
                        for (var i = 0; i < object.uninterpretedOption.length; ++i) {
                            if (typeof object.uninterpretedOption[i] !== "object")
                                throw TypeError(".google.protobuf.FileOptions.uninterpretedOption: object expected");
                            message.uninterpretedOption[i] = $root.google.protobuf.UninterpretedOption.fromObject(object.uninterpretedOption[i]);
                        }
                    }
                    if (object[".google.api.resourceDefinition"]) {
                        if (!Array.isArray(object[".google.api.resourceDefinition"]))
                            throw TypeError(".google.protobuf.FileOptions..google.api.resourceDefinition: array expected");
                        message[".google.api.resourceDefinition"] = [];
                        for (var i = 0; i < object[".google.api.resourceDefinition"].length; ++i) {
                            if (typeof object[".google.api.resourceDefinition"][i] !== "object")
                                throw TypeError(".google.protobuf.FileOptions..google.api.resourceDefinition: object expected");
                            message[".google.api.resourceDefinition"][i] = $root.google.api.ResourceDescriptor.fromObject(object[".google.api.resourceDefinition"][i]);
                        }
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from a FileOptions message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.FileOptions
                 * @static
                 * @param {google.protobuf.FileOptions} message FileOptions
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                FileOptions.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.arrays || options.defaults) {
                        object.uninterpretedOption = [];
                        object[".google.api.resourceDefinition"] = [];
                    }
                    if (options.defaults) {
                        object.javaPackage = "";
                        object.javaOuterClassname = "";
                        object.optimizeFor = options.enums === String ? "SPEED" : 1;
                        object.javaMultipleFiles = false;
                        object.goPackage = "";
                        object.ccGenericServices = false;
                        object.javaGenericServices = false;
                        object.pyGenericServices = false;
                        object.javaGenerateEqualsAndHash = false;
                        object.deprecated = false;
                        object.javaStringCheckUtf8 = false;
                        object.ccEnableArenas = true;
                        object.objcClassPrefix = "";
                        object.csharpNamespace = "";
                        object.swiftPrefix = "";
                        object.phpClassPrefix = "";
                        object.phpNamespace = "";
                        object.phpMetadataNamespace = "";
                        object.rubyPackage = "";
                        object.features = null;
                    }
                    if (message.javaPackage != null && message.hasOwnProperty("javaPackage"))
                        object.javaPackage = message.javaPackage;
                    if (message.javaOuterClassname != null && message.hasOwnProperty("javaOuterClassname"))
                        object.javaOuterClassname = message.javaOuterClassname;
                    if (message.optimizeFor != null && message.hasOwnProperty("optimizeFor"))
                        object.optimizeFor = options.enums === String ? $root.google.protobuf.FileOptions.OptimizeMode[message.optimizeFor] === undefined ? message.optimizeFor : $root.google.protobuf.FileOptions.OptimizeMode[message.optimizeFor] : message.optimizeFor;
                    if (message.javaMultipleFiles != null && message.hasOwnProperty("javaMultipleFiles"))
                        object.javaMultipleFiles = message.javaMultipleFiles;
                    if (message.goPackage != null && message.hasOwnProperty("goPackage"))
                        object.goPackage = message.goPackage;
                    if (message.ccGenericServices != null && message.hasOwnProperty("ccGenericServices"))
                        object.ccGenericServices = message.ccGenericServices;
                    if (message.javaGenericServices != null && message.hasOwnProperty("javaGenericServices"))
                        object.javaGenericServices = message.javaGenericServices;
                    if (message.pyGenericServices != null && message.hasOwnProperty("pyGenericServices"))
                        object.pyGenericServices = message.pyGenericServices;
                    if (message.javaGenerateEqualsAndHash != null && message.hasOwnProperty("javaGenerateEqualsAndHash"))
                        object.javaGenerateEqualsAndHash = message.javaGenerateEqualsAndHash;
                    if (message.deprecated != null && message.hasOwnProperty("deprecated"))
                        object.deprecated = message.deprecated;
                    if (message.javaStringCheckUtf8 != null && message.hasOwnProperty("javaStringCheckUtf8"))
                        object.javaStringCheckUtf8 = message.javaStringCheckUtf8;
                    if (message.ccEnableArenas != null && message.hasOwnProperty("ccEnableArenas"))
                        object.ccEnableArenas = message.ccEnableArenas;
                    if (message.objcClassPrefix != null && message.hasOwnProperty("objcClassPrefix"))
                        object.objcClassPrefix = message.objcClassPrefix;
                    if (message.csharpNamespace != null && message.hasOwnProperty("csharpNamespace"))
                        object.csharpNamespace = message.csharpNamespace;
                    if (message.swiftPrefix != null && message.hasOwnProperty("swiftPrefix"))
                        object.swiftPrefix = message.swiftPrefix;
                    if (message.phpClassPrefix != null && message.hasOwnProperty("phpClassPrefix"))
                        object.phpClassPrefix = message.phpClassPrefix;
                    if (message.phpNamespace != null && message.hasOwnProperty("phpNamespace"))
                        object.phpNamespace = message.phpNamespace;
                    if (message.phpMetadataNamespace != null && message.hasOwnProperty("phpMetadataNamespace"))
                        object.phpMetadataNamespace = message.phpMetadataNamespace;
                    if (message.rubyPackage != null && message.hasOwnProperty("rubyPackage"))
                        object.rubyPackage = message.rubyPackage;
                    if (message.features != null && message.hasOwnProperty("features"))
                        object.features = $root.google.protobuf.FeatureSet.toObject(message.features, options);
                    if (message.uninterpretedOption && message.uninterpretedOption.length) {
                        object.uninterpretedOption = [];
                        for (var j = 0; j < message.uninterpretedOption.length; ++j)
                            object.uninterpretedOption[j] = $root.google.protobuf.UninterpretedOption.toObject(message.uninterpretedOption[j], options);
                    }
                    if (message[".google.api.resourceDefinition"] && message[".google.api.resourceDefinition"].length) {
                        object[".google.api.resourceDefinition"] = [];
                        for (var j = 0; j < message[".google.api.resourceDefinition"].length; ++j)
                            object[".google.api.resourceDefinition"][j] = $root.google.api.ResourceDescriptor.toObject(message[".google.api.resourceDefinition"][j], options);
                    }
                    return object;
                };
    
                /**
                 * Converts this FileOptions to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.FileOptions
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                FileOptions.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for FileOptions
                 * @function getTypeUrl
                 * @memberof google.protobuf.FileOptions
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                FileOptions.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.FileOptions";
                };
    
                /**
                 * OptimizeMode enum.
                 * @name google.protobuf.FileOptions.OptimizeMode
                 * @enum {number}
                 * @property {number} SPEED=1 SPEED value
                 * @property {number} CODE_SIZE=2 CODE_SIZE value
                 * @property {number} LITE_RUNTIME=3 LITE_RUNTIME value
                 */
                FileOptions.OptimizeMode = (function() {
                    var valuesById = {}, values = Object.create(valuesById);
                    values[valuesById[1] = "SPEED"] = 1;
                    values[valuesById[2] = "CODE_SIZE"] = 2;
                    values[valuesById[3] = "LITE_RUNTIME"] = 3;
                    return values;
                })();
    
                return FileOptions;
            })();
    
            protobuf.MessageOptions = (function() {
    
                /**
                 * Properties of a MessageOptions.
                 * @memberof google.protobuf
                 * @interface IMessageOptions
                 * @property {boolean|null} [messageSetWireFormat] MessageOptions messageSetWireFormat
                 * @property {boolean|null} [noStandardDescriptorAccessor] MessageOptions noStandardDescriptorAccessor
                 * @property {boolean|null} [deprecated] MessageOptions deprecated
                 * @property {boolean|null} [mapEntry] MessageOptions mapEntry
                 * @property {boolean|null} [deprecatedLegacyJsonFieldConflicts] MessageOptions deprecatedLegacyJsonFieldConflicts
                 * @property {google.protobuf.IFeatureSet|null} [features] MessageOptions features
                 * @property {Array.<google.protobuf.IUninterpretedOption>|null} [uninterpretedOption] MessageOptions uninterpretedOption
                 * @property {google.api.IResourceDescriptor|null} [".google.api.resource"] MessageOptions .google.api.resource
                 */
    
                /**
                 * Constructs a new MessageOptions.
                 * @memberof google.protobuf
                 * @classdesc Represents a MessageOptions.
                 * @implements IMessageOptions
                 * @constructor
                 * @param {google.protobuf.IMessageOptions=} [properties] Properties to set
                 */
                function MessageOptions(properties) {
                    this.uninterpretedOption = [];
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * MessageOptions messageSetWireFormat.
                 * @member {boolean} messageSetWireFormat
                 * @memberof google.protobuf.MessageOptions
                 * @instance
                 */
                MessageOptions.prototype.messageSetWireFormat = false;
    
                /**
                 * MessageOptions noStandardDescriptorAccessor.
                 * @member {boolean} noStandardDescriptorAccessor
                 * @memberof google.protobuf.MessageOptions
                 * @instance
                 */
                MessageOptions.prototype.noStandardDescriptorAccessor = false;
    
                /**
                 * MessageOptions deprecated.
                 * @member {boolean} deprecated
                 * @memberof google.protobuf.MessageOptions
                 * @instance
                 */
                MessageOptions.prototype.deprecated = false;
    
                /**
                 * MessageOptions mapEntry.
                 * @member {boolean} mapEntry
                 * @memberof google.protobuf.MessageOptions
                 * @instance
                 */
                MessageOptions.prototype.mapEntry = false;
    
                /**
                 * MessageOptions deprecatedLegacyJsonFieldConflicts.
                 * @member {boolean} deprecatedLegacyJsonFieldConflicts
                 * @memberof google.protobuf.MessageOptions
                 * @instance
                 */
                MessageOptions.prototype.deprecatedLegacyJsonFieldConflicts = false;
    
                /**
                 * MessageOptions features.
                 * @member {google.protobuf.IFeatureSet|null|undefined} features
                 * @memberof google.protobuf.MessageOptions
                 * @instance
                 */
                MessageOptions.prototype.features = null;
    
                /**
                 * MessageOptions uninterpretedOption.
                 * @member {Array.<google.protobuf.IUninterpretedOption>} uninterpretedOption
                 * @memberof google.protobuf.MessageOptions
                 * @instance
                 */
                MessageOptions.prototype.uninterpretedOption = $util.emptyArray;
    
                /**
                 * MessageOptions .google.api.resource.
                 * @member {google.api.IResourceDescriptor|null|undefined} .google.api.resource
                 * @memberof google.protobuf.MessageOptions
                 * @instance
                 */
                MessageOptions.prototype[".google.api.resource"] = null;
    
                /**
                 * Creates a new MessageOptions instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.MessageOptions
                 * @static
                 * @param {google.protobuf.IMessageOptions=} [properties] Properties to set
                 * @returns {google.protobuf.MessageOptions} MessageOptions instance
                 */
                MessageOptions.create = function create(properties) {
                    return new MessageOptions(properties);
                };
    
                /**
                 * Encodes the specified MessageOptions message. Does not implicitly {@link google.protobuf.MessageOptions.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.MessageOptions
                 * @static
                 * @param {google.protobuf.IMessageOptions} message MessageOptions message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                MessageOptions.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.messageSetWireFormat != null && Object.hasOwnProperty.call(message, "messageSetWireFormat"))
                        writer.uint32(/* id 1, wireType 0 =*/8).bool(message.messageSetWireFormat);
                    if (message.noStandardDescriptorAccessor != null && Object.hasOwnProperty.call(message, "noStandardDescriptorAccessor"))
                        writer.uint32(/* id 2, wireType 0 =*/16).bool(message.noStandardDescriptorAccessor);
                    if (message.deprecated != null && Object.hasOwnProperty.call(message, "deprecated"))
                        writer.uint32(/* id 3, wireType 0 =*/24).bool(message.deprecated);
                    if (message.mapEntry != null && Object.hasOwnProperty.call(message, "mapEntry"))
                        writer.uint32(/* id 7, wireType 0 =*/56).bool(message.mapEntry);
                    if (message.deprecatedLegacyJsonFieldConflicts != null && Object.hasOwnProperty.call(message, "deprecatedLegacyJsonFieldConflicts"))
                        writer.uint32(/* id 11, wireType 0 =*/88).bool(message.deprecatedLegacyJsonFieldConflicts);
                    if (message.features != null && Object.hasOwnProperty.call(message, "features"))
                        $root.google.protobuf.FeatureSet.encode(message.features, writer.uint32(/* id 12, wireType 2 =*/98).fork()).ldelim();
                    if (message.uninterpretedOption != null && message.uninterpretedOption.length)
                        for (var i = 0; i < message.uninterpretedOption.length; ++i)
                            $root.google.protobuf.UninterpretedOption.encode(message.uninterpretedOption[i], writer.uint32(/* id 999, wireType 2 =*/7994).fork()).ldelim();
                    if (message[".google.api.resource"] != null && Object.hasOwnProperty.call(message, ".google.api.resource"))
                        $root.google.api.ResourceDescriptor.encode(message[".google.api.resource"], writer.uint32(/* id 1053, wireType 2 =*/8426).fork()).ldelim();
                    return writer;
                };
    
                /**
                 * Encodes the specified MessageOptions message, length delimited. Does not implicitly {@link google.protobuf.MessageOptions.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.MessageOptions
                 * @static
                 * @param {google.protobuf.IMessageOptions} message MessageOptions message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                MessageOptions.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a MessageOptions message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.MessageOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.MessageOptions} MessageOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                MessageOptions.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.MessageOptions();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.messageSetWireFormat = reader.bool();
                                break;
                            }
                        case 2: {
                                message.noStandardDescriptorAccessor = reader.bool();
                                break;
                            }
                        case 3: {
                                message.deprecated = reader.bool();
                                break;
                            }
                        case 7: {
                                message.mapEntry = reader.bool();
                                break;
                            }
                        case 11: {
                                message.deprecatedLegacyJsonFieldConflicts = reader.bool();
                                break;
                            }
                        case 12: {
                                message.features = $root.google.protobuf.FeatureSet.decode(reader, reader.uint32());
                                break;
                            }
                        case 999: {
                                if (!(message.uninterpretedOption && message.uninterpretedOption.length))
                                    message.uninterpretedOption = [];
                                message.uninterpretedOption.push($root.google.protobuf.UninterpretedOption.decode(reader, reader.uint32()));
                                break;
                            }
                        case 1053: {
                                message[".google.api.resource"] = $root.google.api.ResourceDescriptor.decode(reader, reader.uint32());
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a MessageOptions message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.MessageOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.MessageOptions} MessageOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                MessageOptions.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a MessageOptions message.
                 * @function verify
                 * @memberof google.protobuf.MessageOptions
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                MessageOptions.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.messageSetWireFormat != null && message.hasOwnProperty("messageSetWireFormat"))
                        if (typeof message.messageSetWireFormat !== "boolean")
                            return "messageSetWireFormat: boolean expected";
                    if (message.noStandardDescriptorAccessor != null && message.hasOwnProperty("noStandardDescriptorAccessor"))
                        if (typeof message.noStandardDescriptorAccessor !== "boolean")
                            return "noStandardDescriptorAccessor: boolean expected";
                    if (message.deprecated != null && message.hasOwnProperty("deprecated"))
                        if (typeof message.deprecated !== "boolean")
                            return "deprecated: boolean expected";
                    if (message.mapEntry != null && message.hasOwnProperty("mapEntry"))
                        if (typeof message.mapEntry !== "boolean")
                            return "mapEntry: boolean expected";
                    if (message.deprecatedLegacyJsonFieldConflicts != null && message.hasOwnProperty("deprecatedLegacyJsonFieldConflicts"))
                        if (typeof message.deprecatedLegacyJsonFieldConflicts !== "boolean")
                            return "deprecatedLegacyJsonFieldConflicts: boolean expected";
                    if (message.features != null && message.hasOwnProperty("features")) {
                        var error = $root.google.protobuf.FeatureSet.verify(message.features);
                        if (error)
                            return "features." + error;
                    }
                    if (message.uninterpretedOption != null && message.hasOwnProperty("uninterpretedOption")) {
                        if (!Array.isArray(message.uninterpretedOption))
                            return "uninterpretedOption: array expected";
                        for (var i = 0; i < message.uninterpretedOption.length; ++i) {
                            var error = $root.google.protobuf.UninterpretedOption.verify(message.uninterpretedOption[i]);
                            if (error)
                                return "uninterpretedOption." + error;
                        }
                    }
                    if (message[".google.api.resource"] != null && message.hasOwnProperty(".google.api.resource")) {
                        var error = $root.google.api.ResourceDescriptor.verify(message[".google.api.resource"]);
                        if (error)
                            return ".google.api.resource." + error;
                    }
                    return null;
                };
    
                /**
                 * Creates a MessageOptions message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.MessageOptions
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.MessageOptions} MessageOptions
                 */
                MessageOptions.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.MessageOptions)
                        return object;
                    var message = new $root.google.protobuf.MessageOptions();
                    if (object.messageSetWireFormat != null)
                        message.messageSetWireFormat = Boolean(object.messageSetWireFormat);
                    if (object.noStandardDescriptorAccessor != null)
                        message.noStandardDescriptorAccessor = Boolean(object.noStandardDescriptorAccessor);
                    if (object.deprecated != null)
                        message.deprecated = Boolean(object.deprecated);
                    if (object.mapEntry != null)
                        message.mapEntry = Boolean(object.mapEntry);
                    if (object.deprecatedLegacyJsonFieldConflicts != null)
                        message.deprecatedLegacyJsonFieldConflicts = Boolean(object.deprecatedLegacyJsonFieldConflicts);
                    if (object.features != null) {
                        if (typeof object.features !== "object")
                            throw TypeError(".google.protobuf.MessageOptions.features: object expected");
                        message.features = $root.google.protobuf.FeatureSet.fromObject(object.features);
                    }
                    if (object.uninterpretedOption) {
                        if (!Array.isArray(object.uninterpretedOption))
                            throw TypeError(".google.protobuf.MessageOptions.uninterpretedOption: array expected");
                        message.uninterpretedOption = [];
                        for (var i = 0; i < object.uninterpretedOption.length; ++i) {
                            if (typeof object.uninterpretedOption[i] !== "object")
                                throw TypeError(".google.protobuf.MessageOptions.uninterpretedOption: object expected");
                            message.uninterpretedOption[i] = $root.google.protobuf.UninterpretedOption.fromObject(object.uninterpretedOption[i]);
                        }
                    }
                    if (object[".google.api.resource"] != null) {
                        if (typeof object[".google.api.resource"] !== "object")
                            throw TypeError(".google.protobuf.MessageOptions..google.api.resource: object expected");
                        message[".google.api.resource"] = $root.google.api.ResourceDescriptor.fromObject(object[".google.api.resource"]);
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from a MessageOptions message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.MessageOptions
                 * @static
                 * @param {google.protobuf.MessageOptions} message MessageOptions
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                MessageOptions.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.arrays || options.defaults)
                        object.uninterpretedOption = [];
                    if (options.defaults) {
                        object.messageSetWireFormat = false;
                        object.noStandardDescriptorAccessor = false;
                        object.deprecated = false;
                        object.mapEntry = false;
                        object.deprecatedLegacyJsonFieldConflicts = false;
                        object.features = null;
                        object[".google.api.resource"] = null;
                    }
                    if (message.messageSetWireFormat != null && message.hasOwnProperty("messageSetWireFormat"))
                        object.messageSetWireFormat = message.messageSetWireFormat;
                    if (message.noStandardDescriptorAccessor != null && message.hasOwnProperty("noStandardDescriptorAccessor"))
                        object.noStandardDescriptorAccessor = message.noStandardDescriptorAccessor;
                    if (message.deprecated != null && message.hasOwnProperty("deprecated"))
                        object.deprecated = message.deprecated;
                    if (message.mapEntry != null && message.hasOwnProperty("mapEntry"))
                        object.mapEntry = message.mapEntry;
                    if (message.deprecatedLegacyJsonFieldConflicts != null && message.hasOwnProperty("deprecatedLegacyJsonFieldConflicts"))
                        object.deprecatedLegacyJsonFieldConflicts = message.deprecatedLegacyJsonFieldConflicts;
                    if (message.features != null && message.hasOwnProperty("features"))
                        object.features = $root.google.protobuf.FeatureSet.toObject(message.features, options);
                    if (message.uninterpretedOption && message.uninterpretedOption.length) {
                        object.uninterpretedOption = [];
                        for (var j = 0; j < message.uninterpretedOption.length; ++j)
                            object.uninterpretedOption[j] = $root.google.protobuf.UninterpretedOption.toObject(message.uninterpretedOption[j], options);
                    }
                    if (message[".google.api.resource"] != null && message.hasOwnProperty(".google.api.resource"))
                        object[".google.api.resource"] = $root.google.api.ResourceDescriptor.toObject(message[".google.api.resource"], options);
                    return object;
                };
    
                /**
                 * Converts this MessageOptions to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.MessageOptions
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                MessageOptions.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for MessageOptions
                 * @function getTypeUrl
                 * @memberof google.protobuf.MessageOptions
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                MessageOptions.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.MessageOptions";
                };
    
                return MessageOptions;
            })();
    
            protobuf.FieldOptions = (function() {
    
                /**
                 * Properties of a FieldOptions.
                 * @memberof google.protobuf
                 * @interface IFieldOptions
                 * @property {google.protobuf.FieldOptions.CType|null} [ctype] FieldOptions ctype
                 * @property {boolean|null} [packed] FieldOptions packed
                 * @property {google.protobuf.FieldOptions.JSType|null} [jstype] FieldOptions jstype
                 * @property {boolean|null} [lazy] FieldOptions lazy
                 * @property {boolean|null} [unverifiedLazy] FieldOptions unverifiedLazy
                 * @property {boolean|null} [deprecated] FieldOptions deprecated
                 * @property {boolean|null} [weak] FieldOptions weak
                 * @property {boolean|null} [debugRedact] FieldOptions debugRedact
                 * @property {google.protobuf.FieldOptions.OptionRetention|null} [retention] FieldOptions retention
                 * @property {Array.<google.protobuf.FieldOptions.OptionTargetType>|null} [targets] FieldOptions targets
                 * @property {Array.<google.protobuf.FieldOptions.IEditionDefault>|null} [editionDefaults] FieldOptions editionDefaults
                 * @property {google.protobuf.IFeatureSet|null} [features] FieldOptions features
                 * @property {Array.<google.protobuf.IUninterpretedOption>|null} [uninterpretedOption] FieldOptions uninterpretedOption
                 * @property {Array.<google.api.FieldBehavior>|null} [".google.api.fieldBehavior"] FieldOptions .google.api.fieldBehavior
                 * @property {google.api.IResourceReference|null} [".google.api.resourceReference"] FieldOptions .google.api.resourceReference
                 */
    
                /**
                 * Constructs a new FieldOptions.
                 * @memberof google.protobuf
                 * @classdesc Represents a FieldOptions.
                 * @implements IFieldOptions
                 * @constructor
                 * @param {google.protobuf.IFieldOptions=} [properties] Properties to set
                 */
                function FieldOptions(properties) {
                    this.targets = [];
                    this.editionDefaults = [];
                    this.uninterpretedOption = [];
                    this[".google.api.fieldBehavior"] = [];
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * FieldOptions ctype.
                 * @member {google.protobuf.FieldOptions.CType} ctype
                 * @memberof google.protobuf.FieldOptions
                 * @instance
                 */
                FieldOptions.prototype.ctype = 0;
    
                /**
                 * FieldOptions packed.
                 * @member {boolean} packed
                 * @memberof google.protobuf.FieldOptions
                 * @instance
                 */
                FieldOptions.prototype.packed = false;
    
                /**
                 * FieldOptions jstype.
                 * @member {google.protobuf.FieldOptions.JSType} jstype
                 * @memberof google.protobuf.FieldOptions
                 * @instance
                 */
                FieldOptions.prototype.jstype = 0;
    
                /**
                 * FieldOptions lazy.
                 * @member {boolean} lazy
                 * @memberof google.protobuf.FieldOptions
                 * @instance
                 */
                FieldOptions.prototype.lazy = false;
    
                /**
                 * FieldOptions unverifiedLazy.
                 * @member {boolean} unverifiedLazy
                 * @memberof google.protobuf.FieldOptions
                 * @instance
                 */
                FieldOptions.prototype.unverifiedLazy = false;
    
                /**
                 * FieldOptions deprecated.
                 * @member {boolean} deprecated
                 * @memberof google.protobuf.FieldOptions
                 * @instance
                 */
                FieldOptions.prototype.deprecated = false;
    
                /**
                 * FieldOptions weak.
                 * @member {boolean} weak
                 * @memberof google.protobuf.FieldOptions
                 * @instance
                 */
                FieldOptions.prototype.weak = false;
    
                /**
                 * FieldOptions debugRedact.
                 * @member {boolean} debugRedact
                 * @memberof google.protobuf.FieldOptions
                 * @instance
                 */
                FieldOptions.prototype.debugRedact = false;
    
                /**
                 * FieldOptions retention.
                 * @member {google.protobuf.FieldOptions.OptionRetention} retention
                 * @memberof google.protobuf.FieldOptions
                 * @instance
                 */
                FieldOptions.prototype.retention = 0;
    
                /**
                 * FieldOptions targets.
                 * @member {Array.<google.protobuf.FieldOptions.OptionTargetType>} targets
                 * @memberof google.protobuf.FieldOptions
                 * @instance
                 */
                FieldOptions.prototype.targets = $util.emptyArray;
    
                /**
                 * FieldOptions editionDefaults.
                 * @member {Array.<google.protobuf.FieldOptions.IEditionDefault>} editionDefaults
                 * @memberof google.protobuf.FieldOptions
                 * @instance
                 */
                FieldOptions.prototype.editionDefaults = $util.emptyArray;
    
                /**
                 * FieldOptions features.
                 * @member {google.protobuf.IFeatureSet|null|undefined} features
                 * @memberof google.protobuf.FieldOptions
                 * @instance
                 */
                FieldOptions.prototype.features = null;
    
                /**
                 * FieldOptions uninterpretedOption.
                 * @member {Array.<google.protobuf.IUninterpretedOption>} uninterpretedOption
                 * @memberof google.protobuf.FieldOptions
                 * @instance
                 */
                FieldOptions.prototype.uninterpretedOption = $util.emptyArray;
    
                /**
                 * FieldOptions .google.api.fieldBehavior.
                 * @member {Array.<google.api.FieldBehavior>} .google.api.fieldBehavior
                 * @memberof google.protobuf.FieldOptions
                 * @instance
                 */
                FieldOptions.prototype[".google.api.fieldBehavior"] = $util.emptyArray;
    
                /**
                 * FieldOptions .google.api.resourceReference.
                 * @member {google.api.IResourceReference|null|undefined} .google.api.resourceReference
                 * @memberof google.protobuf.FieldOptions
                 * @instance
                 */
                FieldOptions.prototype[".google.api.resourceReference"] = null;
    
                /**
                 * Creates a new FieldOptions instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.FieldOptions
                 * @static
                 * @param {google.protobuf.IFieldOptions=} [properties] Properties to set
                 * @returns {google.protobuf.FieldOptions} FieldOptions instance
                 */
                FieldOptions.create = function create(properties) {
                    return new FieldOptions(properties);
                };
    
                /**
                 * Encodes the specified FieldOptions message. Does not implicitly {@link google.protobuf.FieldOptions.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.FieldOptions
                 * @static
                 * @param {google.protobuf.IFieldOptions} message FieldOptions message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FieldOptions.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.ctype != null && Object.hasOwnProperty.call(message, "ctype"))
                        writer.uint32(/* id 1, wireType 0 =*/8).int32(message.ctype);
                    if (message.packed != null && Object.hasOwnProperty.call(message, "packed"))
                        writer.uint32(/* id 2, wireType 0 =*/16).bool(message.packed);
                    if (message.deprecated != null && Object.hasOwnProperty.call(message, "deprecated"))
                        writer.uint32(/* id 3, wireType 0 =*/24).bool(message.deprecated);
                    if (message.lazy != null && Object.hasOwnProperty.call(message, "lazy"))
                        writer.uint32(/* id 5, wireType 0 =*/40).bool(message.lazy);
                    if (message.jstype != null && Object.hasOwnProperty.call(message, "jstype"))
                        writer.uint32(/* id 6, wireType 0 =*/48).int32(message.jstype);
                    if (message.weak != null && Object.hasOwnProperty.call(message, "weak"))
                        writer.uint32(/* id 10, wireType 0 =*/80).bool(message.weak);
                    if (message.unverifiedLazy != null && Object.hasOwnProperty.call(message, "unverifiedLazy"))
                        writer.uint32(/* id 15, wireType 0 =*/120).bool(message.unverifiedLazy);
                    if (message.debugRedact != null && Object.hasOwnProperty.call(message, "debugRedact"))
                        writer.uint32(/* id 16, wireType 0 =*/128).bool(message.debugRedact);
                    if (message.retention != null && Object.hasOwnProperty.call(message, "retention"))
                        writer.uint32(/* id 17, wireType 0 =*/136).int32(message.retention);
                    if (message.targets != null && message.targets.length)
                        for (var i = 0; i < message.targets.length; ++i)
                            writer.uint32(/* id 19, wireType 0 =*/152).int32(message.targets[i]);
                    if (message.editionDefaults != null && message.editionDefaults.length)
                        for (var i = 0; i < message.editionDefaults.length; ++i)
                            $root.google.protobuf.FieldOptions.EditionDefault.encode(message.editionDefaults[i], writer.uint32(/* id 20, wireType 2 =*/162).fork()).ldelim();
                    if (message.features != null && Object.hasOwnProperty.call(message, "features"))
                        $root.google.protobuf.FeatureSet.encode(message.features, writer.uint32(/* id 21, wireType 2 =*/170).fork()).ldelim();
                    if (message.uninterpretedOption != null && message.uninterpretedOption.length)
                        for (var i = 0; i < message.uninterpretedOption.length; ++i)
                            $root.google.protobuf.UninterpretedOption.encode(message.uninterpretedOption[i], writer.uint32(/* id 999, wireType 2 =*/7994).fork()).ldelim();
                    if (message[".google.api.fieldBehavior"] != null && message[".google.api.fieldBehavior"].length)
                        for (var i = 0; i < message[".google.api.fieldBehavior"].length; ++i)
                            writer.uint32(/* id 1052, wireType 0 =*/8416).int32(message[".google.api.fieldBehavior"][i]);
                    if (message[".google.api.resourceReference"] != null && Object.hasOwnProperty.call(message, ".google.api.resourceReference"))
                        $root.google.api.ResourceReference.encode(message[".google.api.resourceReference"], writer.uint32(/* id 1055, wireType 2 =*/8442).fork()).ldelim();
                    return writer;
                };
    
                /**
                 * Encodes the specified FieldOptions message, length delimited. Does not implicitly {@link google.protobuf.FieldOptions.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.FieldOptions
                 * @static
                 * @param {google.protobuf.IFieldOptions} message FieldOptions message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FieldOptions.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a FieldOptions message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.FieldOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.FieldOptions} FieldOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FieldOptions.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.FieldOptions();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.ctype = reader.int32();
                                break;
                            }
                        case 2: {
                                message.packed = reader.bool();
                                break;
                            }
                        case 6: {
                                message.jstype = reader.int32();
                                break;
                            }
                        case 5: {
                                message.lazy = reader.bool();
                                break;
                            }
                        case 15: {
                                message.unverifiedLazy = reader.bool();
                                break;
                            }
                        case 3: {
                                message.deprecated = reader.bool();
                                break;
                            }
                        case 10: {
                                message.weak = reader.bool();
                                break;
                            }
                        case 16: {
                                message.debugRedact = reader.bool();
                                break;
                            }
                        case 17: {
                                message.retention = reader.int32();
                                break;
                            }
                        case 19: {
                                if (!(message.targets && message.targets.length))
                                    message.targets = [];
                                if ((tag & 7) === 2) {
                                    var end2 = reader.uint32() + reader.pos;
                                    while (reader.pos < end2)
                                        message.targets.push(reader.int32());
                                } else
                                    message.targets.push(reader.int32());
                                break;
                            }
                        case 20: {
                                if (!(message.editionDefaults && message.editionDefaults.length))
                                    message.editionDefaults = [];
                                message.editionDefaults.push($root.google.protobuf.FieldOptions.EditionDefault.decode(reader, reader.uint32()));
                                break;
                            }
                        case 21: {
                                message.features = $root.google.protobuf.FeatureSet.decode(reader, reader.uint32());
                                break;
                            }
                        case 999: {
                                if (!(message.uninterpretedOption && message.uninterpretedOption.length))
                                    message.uninterpretedOption = [];
                                message.uninterpretedOption.push($root.google.protobuf.UninterpretedOption.decode(reader, reader.uint32()));
                                break;
                            }
                        case 1052: {
                                if (!(message[".google.api.fieldBehavior"] && message[".google.api.fieldBehavior"].length))
                                    message[".google.api.fieldBehavior"] = [];
                                if ((tag & 7) === 2) {
                                    var end2 = reader.uint32() + reader.pos;
                                    while (reader.pos < end2)
                                        message[".google.api.fieldBehavior"].push(reader.int32());
                                } else
                                    message[".google.api.fieldBehavior"].push(reader.int32());
                                break;
                            }
                        case 1055: {
                                message[".google.api.resourceReference"] = $root.google.api.ResourceReference.decode(reader, reader.uint32());
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a FieldOptions message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.FieldOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.FieldOptions} FieldOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FieldOptions.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a FieldOptions message.
                 * @function verify
                 * @memberof google.protobuf.FieldOptions
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                FieldOptions.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.ctype != null && message.hasOwnProperty("ctype"))
                        switch (message.ctype) {
                        default:
                            return "ctype: enum value expected";
                        case 0:
                        case 1:
                        case 2:
                            break;
                        }
                    if (message.packed != null && message.hasOwnProperty("packed"))
                        if (typeof message.packed !== "boolean")
                            return "packed: boolean expected";
                    if (message.jstype != null && message.hasOwnProperty("jstype"))
                        switch (message.jstype) {
                        default:
                            return "jstype: enum value expected";
                        case 0:
                        case 1:
                        case 2:
                            break;
                        }
                    if (message.lazy != null && message.hasOwnProperty("lazy"))
                        if (typeof message.lazy !== "boolean")
                            return "lazy: boolean expected";
                    if (message.unverifiedLazy != null && message.hasOwnProperty("unverifiedLazy"))
                        if (typeof message.unverifiedLazy !== "boolean")
                            return "unverifiedLazy: boolean expected";
                    if (message.deprecated != null && message.hasOwnProperty("deprecated"))
                        if (typeof message.deprecated !== "boolean")
                            return "deprecated: boolean expected";
                    if (message.weak != null && message.hasOwnProperty("weak"))
                        if (typeof message.weak !== "boolean")
                            return "weak: boolean expected";
                    if (message.debugRedact != null && message.hasOwnProperty("debugRedact"))
                        if (typeof message.debugRedact !== "boolean")
                            return "debugRedact: boolean expected";
                    if (message.retention != null && message.hasOwnProperty("retention"))
                        switch (message.retention) {
                        default:
                            return "retention: enum value expected";
                        case 0:
                        case 1:
                        case 2:
                            break;
                        }
                    if (message.targets != null && message.hasOwnProperty("targets")) {
                        if (!Array.isArray(message.targets))
                            return "targets: array expected";
                        for (var i = 0; i < message.targets.length; ++i)
                            switch (message.targets[i]) {
                            default:
                                return "targets: enum value[] expected";
                            case 0:
                            case 1:
                            case 2:
                            case 3:
                            case 4:
                            case 5:
                            case 6:
                            case 7:
                            case 8:
                            case 9:
                                break;
                            }
                    }
                    if (message.editionDefaults != null && message.hasOwnProperty("editionDefaults")) {
                        if (!Array.isArray(message.editionDefaults))
                            return "editionDefaults: array expected";
                        for (var i = 0; i < message.editionDefaults.length; ++i) {
                            var error = $root.google.protobuf.FieldOptions.EditionDefault.verify(message.editionDefaults[i]);
                            if (error)
                                return "editionDefaults." + error;
                        }
                    }
                    if (message.features != null && message.hasOwnProperty("features")) {
                        var error = $root.google.protobuf.FeatureSet.verify(message.features);
                        if (error)
                            return "features." + error;
                    }
                    if (message.uninterpretedOption != null && message.hasOwnProperty("uninterpretedOption")) {
                        if (!Array.isArray(message.uninterpretedOption))
                            return "uninterpretedOption: array expected";
                        for (var i = 0; i < message.uninterpretedOption.length; ++i) {
                            var error = $root.google.protobuf.UninterpretedOption.verify(message.uninterpretedOption[i]);
                            if (error)
                                return "uninterpretedOption." + error;
                        }
                    }
                    if (message[".google.api.fieldBehavior"] != null && message.hasOwnProperty(".google.api.fieldBehavior")) {
                        if (!Array.isArray(message[".google.api.fieldBehavior"]))
                            return ".google.api.fieldBehavior: array expected";
                        for (var i = 0; i < message[".google.api.fieldBehavior"].length; ++i)
                            switch (message[".google.api.fieldBehavior"][i]) {
                            default:
                                return ".google.api.fieldBehavior: enum value[] expected";
                            case 0:
                            case 1:
                            case 2:
                            case 3:
                            case 4:
                            case 5:
                            case 6:
                            case 7:
                            case 8:
                                break;
                            }
                    }
                    if (message[".google.api.resourceReference"] != null && message.hasOwnProperty(".google.api.resourceReference")) {
                        var error = $root.google.api.ResourceReference.verify(message[".google.api.resourceReference"]);
                        if (error)
                            return ".google.api.resourceReference." + error;
                    }
                    return null;
                };
    
                /**
                 * Creates a FieldOptions message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.FieldOptions
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.FieldOptions} FieldOptions
                 */
                FieldOptions.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.FieldOptions)
                        return object;
                    var message = new $root.google.protobuf.FieldOptions();
                    switch (object.ctype) {
                    default:
                        if (typeof object.ctype === "number") {
                            message.ctype = object.ctype;
                            break;
                        }
                        break;
                    case "STRING":
                    case 0:
                        message.ctype = 0;
                        break;
                    case "CORD":
                    case 1:
                        message.ctype = 1;
                        break;
                    case "STRING_PIECE":
                    case 2:
                        message.ctype = 2;
                        break;
                    }
                    if (object.packed != null)
                        message.packed = Boolean(object.packed);
                    switch (object.jstype) {
                    default:
                        if (typeof object.jstype === "number") {
                            message.jstype = object.jstype;
                            break;
                        }
                        break;
                    case "JS_NORMAL":
                    case 0:
                        message.jstype = 0;
                        break;
                    case "JS_STRING":
                    case 1:
                        message.jstype = 1;
                        break;
                    case "JS_NUMBER":
                    case 2:
                        message.jstype = 2;
                        break;
                    }
                    if (object.lazy != null)
                        message.lazy = Boolean(object.lazy);
                    if (object.unverifiedLazy != null)
                        message.unverifiedLazy = Boolean(object.unverifiedLazy);
                    if (object.deprecated != null)
                        message.deprecated = Boolean(object.deprecated);
                    if (object.weak != null)
                        message.weak = Boolean(object.weak);
                    if (object.debugRedact != null)
                        message.debugRedact = Boolean(object.debugRedact);
                    switch (object.retention) {
                    default:
                        if (typeof object.retention === "number") {
                            message.retention = object.retention;
                            break;
                        }
                        break;
                    case "RETENTION_UNKNOWN":
                    case 0:
                        message.retention = 0;
                        break;
                    case "RETENTION_RUNTIME":
                    case 1:
                        message.retention = 1;
                        break;
                    case "RETENTION_SOURCE":
                    case 2:
                        message.retention = 2;
                        break;
                    }
                    if (object.targets) {
                        if (!Array.isArray(object.targets))
                            throw TypeError(".google.protobuf.FieldOptions.targets: array expected");
                        message.targets = [];
                        for (var i = 0; i < object.targets.length; ++i)
                            switch (object.targets[i]) {
                            default:
                                if (typeof object.targets[i] === "number") {
                                    message.targets[i] = object.targets[i];
                                    break;
                                }
                            case "TARGET_TYPE_UNKNOWN":
                            case 0:
                                message.targets[i] = 0;
                                break;
                            case "TARGET_TYPE_FILE":
                            case 1:
                                message.targets[i] = 1;
                                break;
                            case "TARGET_TYPE_EXTENSION_RANGE":
                            case 2:
                                message.targets[i] = 2;
                                break;
                            case "TARGET_TYPE_MESSAGE":
                            case 3:
                                message.targets[i] = 3;
                                break;
                            case "TARGET_TYPE_FIELD":
                            case 4:
                                message.targets[i] = 4;
                                break;
                            case "TARGET_TYPE_ONEOF":
                            case 5:
                                message.targets[i] = 5;
                                break;
                            case "TARGET_TYPE_ENUM":
                            case 6:
                                message.targets[i] = 6;
                                break;
                            case "TARGET_TYPE_ENUM_ENTRY":
                            case 7:
                                message.targets[i] = 7;
                                break;
                            case "TARGET_TYPE_SERVICE":
                            case 8:
                                message.targets[i] = 8;
                                break;
                            case "TARGET_TYPE_METHOD":
                            case 9:
                                message.targets[i] = 9;
                                break;
                            }
                    }
                    if (object.editionDefaults) {
                        if (!Array.isArray(object.editionDefaults))
                            throw TypeError(".google.protobuf.FieldOptions.editionDefaults: array expected");
                        message.editionDefaults = [];
                        for (var i = 0; i < object.editionDefaults.length; ++i) {
                            if (typeof object.editionDefaults[i] !== "object")
                                throw TypeError(".google.protobuf.FieldOptions.editionDefaults: object expected");
                            message.editionDefaults[i] = $root.google.protobuf.FieldOptions.EditionDefault.fromObject(object.editionDefaults[i]);
                        }
                    }
                    if (object.features != null) {
                        if (typeof object.features !== "object")
                            throw TypeError(".google.protobuf.FieldOptions.features: object expected");
                        message.features = $root.google.protobuf.FeatureSet.fromObject(object.features);
                    }
                    if (object.uninterpretedOption) {
                        if (!Array.isArray(object.uninterpretedOption))
                            throw TypeError(".google.protobuf.FieldOptions.uninterpretedOption: array expected");
                        message.uninterpretedOption = [];
                        for (var i = 0; i < object.uninterpretedOption.length; ++i) {
                            if (typeof object.uninterpretedOption[i] !== "object")
                                throw TypeError(".google.protobuf.FieldOptions.uninterpretedOption: object expected");
                            message.uninterpretedOption[i] = $root.google.protobuf.UninterpretedOption.fromObject(object.uninterpretedOption[i]);
                        }
                    }
                    if (object[".google.api.fieldBehavior"]) {
                        if (!Array.isArray(object[".google.api.fieldBehavior"]))
                            throw TypeError(".google.protobuf.FieldOptions..google.api.fieldBehavior: array expected");
                        message[".google.api.fieldBehavior"] = [];
                        for (var i = 0; i < object[".google.api.fieldBehavior"].length; ++i)
                            switch (object[".google.api.fieldBehavior"][i]) {
                            default:
                                if (typeof object[".google.api.fieldBehavior"][i] === "number") {
                                    message[".google.api.fieldBehavior"][i] = object[".google.api.fieldBehavior"][i];
                                    break;
                                }
                            case "FIELD_BEHAVIOR_UNSPECIFIED":
                            case 0:
                                message[".google.api.fieldBehavior"][i] = 0;
                                break;
                            case "OPTIONAL":
                            case 1:
                                message[".google.api.fieldBehavior"][i] = 1;
                                break;
                            case "REQUIRED":
                            case 2:
                                message[".google.api.fieldBehavior"][i] = 2;
                                break;
                            case "OUTPUT_ONLY":
                            case 3:
                                message[".google.api.fieldBehavior"][i] = 3;
                                break;
                            case "INPUT_ONLY":
                            case 4:
                                message[".google.api.fieldBehavior"][i] = 4;
                                break;
                            case "IMMUTABLE":
                            case 5:
                                message[".google.api.fieldBehavior"][i] = 5;
                                break;
                            case "UNORDERED_LIST":
                            case 6:
                                message[".google.api.fieldBehavior"][i] = 6;
                                break;
                            case "NON_EMPTY_DEFAULT":
                            case 7:
                                message[".google.api.fieldBehavior"][i] = 7;
                                break;
                            case "IDENTIFIER":
                            case 8:
                                message[".google.api.fieldBehavior"][i] = 8;
                                break;
                            }
                    }
                    if (object[".google.api.resourceReference"] != null) {
                        if (typeof object[".google.api.resourceReference"] !== "object")
                            throw TypeError(".google.protobuf.FieldOptions..google.api.resourceReference: object expected");
                        message[".google.api.resourceReference"] = $root.google.api.ResourceReference.fromObject(object[".google.api.resourceReference"]);
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from a FieldOptions message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.FieldOptions
                 * @static
                 * @param {google.protobuf.FieldOptions} message FieldOptions
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                FieldOptions.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.arrays || options.defaults) {
                        object.targets = [];
                        object.editionDefaults = [];
                        object.uninterpretedOption = [];
                        object[".google.api.fieldBehavior"] = [];
                    }
                    if (options.defaults) {
                        object.ctype = options.enums === String ? "STRING" : 0;
                        object.packed = false;
                        object.deprecated = false;
                        object.lazy = false;
                        object.jstype = options.enums === String ? "JS_NORMAL" : 0;
                        object.weak = false;
                        object.unverifiedLazy = false;
                        object.debugRedact = false;
                        object.retention = options.enums === String ? "RETENTION_UNKNOWN" : 0;
                        object.features = null;
                        object[".google.api.resourceReference"] = null;
                    }
                    if (message.ctype != null && message.hasOwnProperty("ctype"))
                        object.ctype = options.enums === String ? $root.google.protobuf.FieldOptions.CType[message.ctype] === undefined ? message.ctype : $root.google.protobuf.FieldOptions.CType[message.ctype] : message.ctype;
                    if (message.packed != null && message.hasOwnProperty("packed"))
                        object.packed = message.packed;
                    if (message.deprecated != null && message.hasOwnProperty("deprecated"))
                        object.deprecated = message.deprecated;
                    if (message.lazy != null && message.hasOwnProperty("lazy"))
                        object.lazy = message.lazy;
                    if (message.jstype != null && message.hasOwnProperty("jstype"))
                        object.jstype = options.enums === String ? $root.google.protobuf.FieldOptions.JSType[message.jstype] === undefined ? message.jstype : $root.google.protobuf.FieldOptions.JSType[message.jstype] : message.jstype;
                    if (message.weak != null && message.hasOwnProperty("weak"))
                        object.weak = message.weak;
                    if (message.unverifiedLazy != null && message.hasOwnProperty("unverifiedLazy"))
                        object.unverifiedLazy = message.unverifiedLazy;
                    if (message.debugRedact != null && message.hasOwnProperty("debugRedact"))
                        object.debugRedact = message.debugRedact;
                    if (message.retention != null && message.hasOwnProperty("retention"))
                        object.retention = options.enums === String ? $root.google.protobuf.FieldOptions.OptionRetention[message.retention] === undefined ? message.retention : $root.google.protobuf.FieldOptions.OptionRetention[message.retention] : message.retention;
                    if (message.targets && message.targets.length) {
                        object.targets = [];
                        for (var j = 0; j < message.targets.length; ++j)
                            object.targets[j] = options.enums === String ? $root.google.protobuf.FieldOptions.OptionTargetType[message.targets[j]] === undefined ? message.targets[j] : $root.google.protobuf.FieldOptions.OptionTargetType[message.targets[j]] : message.targets[j];
                    }
                    if (message.editionDefaults && message.editionDefaults.length) {
                        object.editionDefaults = [];
                        for (var j = 0; j < message.editionDefaults.length; ++j)
                            object.editionDefaults[j] = $root.google.protobuf.FieldOptions.EditionDefault.toObject(message.editionDefaults[j], options);
                    }
                    if (message.features != null && message.hasOwnProperty("features"))
                        object.features = $root.google.protobuf.FeatureSet.toObject(message.features, options);
                    if (message.uninterpretedOption && message.uninterpretedOption.length) {
                        object.uninterpretedOption = [];
                        for (var j = 0; j < message.uninterpretedOption.length; ++j)
                            object.uninterpretedOption[j] = $root.google.protobuf.UninterpretedOption.toObject(message.uninterpretedOption[j], options);
                    }
                    if (message[".google.api.fieldBehavior"] && message[".google.api.fieldBehavior"].length) {
                        object[".google.api.fieldBehavior"] = [];
                        for (var j = 0; j < message[".google.api.fieldBehavior"].length; ++j)
                            object[".google.api.fieldBehavior"][j] = options.enums === String ? $root.google.api.FieldBehavior[message[".google.api.fieldBehavior"][j]] === undefined ? message[".google.api.fieldBehavior"][j] : $root.google.api.FieldBehavior[message[".google.api.fieldBehavior"][j]] : message[".google.api.fieldBehavior"][j];
                    }
                    if (message[".google.api.resourceReference"] != null && message.hasOwnProperty(".google.api.resourceReference"))
                        object[".google.api.resourceReference"] = $root.google.api.ResourceReference.toObject(message[".google.api.resourceReference"], options);
                    return object;
                };
    
                /**
                 * Converts this FieldOptions to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.FieldOptions
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                FieldOptions.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for FieldOptions
                 * @function getTypeUrl
                 * @memberof google.protobuf.FieldOptions
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                FieldOptions.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.FieldOptions";
                };
    
                /**
                 * CType enum.
                 * @name google.protobuf.FieldOptions.CType
                 * @enum {number}
                 * @property {number} STRING=0 STRING value
                 * @property {number} CORD=1 CORD value
                 * @property {number} STRING_PIECE=2 STRING_PIECE value
                 */
                FieldOptions.CType = (function() {
                    var valuesById = {}, values = Object.create(valuesById);
                    values[valuesById[0] = "STRING"] = 0;
                    values[valuesById[1] = "CORD"] = 1;
                    values[valuesById[2] = "STRING_PIECE"] = 2;
                    return values;
                })();
    
                /**
                 * JSType enum.
                 * @name google.protobuf.FieldOptions.JSType
                 * @enum {number}
                 * @property {number} JS_NORMAL=0 JS_NORMAL value
                 * @property {number} JS_STRING=1 JS_STRING value
                 * @property {number} JS_NUMBER=2 JS_NUMBER value
                 */
                FieldOptions.JSType = (function() {
                    var valuesById = {}, values = Object.create(valuesById);
                    values[valuesById[0] = "JS_NORMAL"] = 0;
                    values[valuesById[1] = "JS_STRING"] = 1;
                    values[valuesById[2] = "JS_NUMBER"] = 2;
                    return values;
                })();
    
                /**
                 * OptionRetention enum.
                 * @name google.protobuf.FieldOptions.OptionRetention
                 * @enum {number}
                 * @property {number} RETENTION_UNKNOWN=0 RETENTION_UNKNOWN value
                 * @property {number} RETENTION_RUNTIME=1 RETENTION_RUNTIME value
                 * @property {number} RETENTION_SOURCE=2 RETENTION_SOURCE value
                 */
                FieldOptions.OptionRetention = (function() {
                    var valuesById = {}, values = Object.create(valuesById);
                    values[valuesById[0] = "RETENTION_UNKNOWN"] = 0;
                    values[valuesById[1] = "RETENTION_RUNTIME"] = 1;
                    values[valuesById[2] = "RETENTION_SOURCE"] = 2;
                    return values;
                })();
    
                /**
                 * OptionTargetType enum.
                 * @name google.protobuf.FieldOptions.OptionTargetType
                 * @enum {number}
                 * @property {number} TARGET_TYPE_UNKNOWN=0 TARGET_TYPE_UNKNOWN value
                 * @property {number} TARGET_TYPE_FILE=1 TARGET_TYPE_FILE value
                 * @property {number} TARGET_TYPE_EXTENSION_RANGE=2 TARGET_TYPE_EXTENSION_RANGE value
                 * @property {number} TARGET_TYPE_MESSAGE=3 TARGET_TYPE_MESSAGE value
                 * @property {number} TARGET_TYPE_FIELD=4 TARGET_TYPE_FIELD value
                 * @property {number} TARGET_TYPE_ONEOF=5 TARGET_TYPE_ONEOF value
                 * @property {number} TARGET_TYPE_ENUM=6 TARGET_TYPE_ENUM value
                 * @property {number} TARGET_TYPE_ENUM_ENTRY=7 TARGET_TYPE_ENUM_ENTRY value
                 * @property {number} TARGET_TYPE_SERVICE=8 TARGET_TYPE_SERVICE value
                 * @property {number} TARGET_TYPE_METHOD=9 TARGET_TYPE_METHOD value
                 */
                FieldOptions.OptionTargetType = (function() {
                    var valuesById = {}, values = Object.create(valuesById);
                    values[valuesById[0] = "TARGET_TYPE_UNKNOWN"] = 0;
                    values[valuesById[1] = "TARGET_TYPE_FILE"] = 1;
                    values[valuesById[2] = "TARGET_TYPE_EXTENSION_RANGE"] = 2;
                    values[valuesById[3] = "TARGET_TYPE_MESSAGE"] = 3;
                    values[valuesById[4] = "TARGET_TYPE_FIELD"] = 4;
                    values[valuesById[5] = "TARGET_TYPE_ONEOF"] = 5;
                    values[valuesById[6] = "TARGET_TYPE_ENUM"] = 6;
                    values[valuesById[7] = "TARGET_TYPE_ENUM_ENTRY"] = 7;
                    values[valuesById[8] = "TARGET_TYPE_SERVICE"] = 8;
                    values[valuesById[9] = "TARGET_TYPE_METHOD"] = 9;
                    return values;
                })();
    
                FieldOptions.EditionDefault = (function() {
    
                    /**
                     * Properties of an EditionDefault.
                     * @memberof google.protobuf.FieldOptions
                     * @interface IEditionDefault
                     * @property {google.protobuf.Edition|null} [edition] EditionDefault edition
                     * @property {string|null} [value] EditionDefault value
                     */
    
                    /**
                     * Constructs a new EditionDefault.
                     * @memberof google.protobuf.FieldOptions
                     * @classdesc Represents an EditionDefault.
                     * @implements IEditionDefault
                     * @constructor
                     * @param {google.protobuf.FieldOptions.IEditionDefault=} [properties] Properties to set
                     */
                    function EditionDefault(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * EditionDefault edition.
                     * @member {google.protobuf.Edition} edition
                     * @memberof google.protobuf.FieldOptions.EditionDefault
                     * @instance
                     */
                    EditionDefault.prototype.edition = 0;
    
                    /**
                     * EditionDefault value.
                     * @member {string} value
                     * @memberof google.protobuf.FieldOptions.EditionDefault
                     * @instance
                     */
                    EditionDefault.prototype.value = "";
    
                    /**
                     * Creates a new EditionDefault instance using the specified properties.
                     * @function create
                     * @memberof google.protobuf.FieldOptions.EditionDefault
                     * @static
                     * @param {google.protobuf.FieldOptions.IEditionDefault=} [properties] Properties to set
                     * @returns {google.protobuf.FieldOptions.EditionDefault} EditionDefault instance
                     */
                    EditionDefault.create = function create(properties) {
                        return new EditionDefault(properties);
                    };
    
                    /**
                     * Encodes the specified EditionDefault message. Does not implicitly {@link google.protobuf.FieldOptions.EditionDefault.verify|verify} messages.
                     * @function encode
                     * @memberof google.protobuf.FieldOptions.EditionDefault
                     * @static
                     * @param {google.protobuf.FieldOptions.IEditionDefault} message EditionDefault message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    EditionDefault.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.value != null && Object.hasOwnProperty.call(message, "value"))
                            writer.uint32(/* id 2, wireType 2 =*/18).string(message.value);
                        if (message.edition != null && Object.hasOwnProperty.call(message, "edition"))
                            writer.uint32(/* id 3, wireType 0 =*/24).int32(message.edition);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified EditionDefault message, length delimited. Does not implicitly {@link google.protobuf.FieldOptions.EditionDefault.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.protobuf.FieldOptions.EditionDefault
                     * @static
                     * @param {google.protobuf.FieldOptions.IEditionDefault} message EditionDefault message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    EditionDefault.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes an EditionDefault message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.protobuf.FieldOptions.EditionDefault
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.protobuf.FieldOptions.EditionDefault} EditionDefault
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    EditionDefault.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.FieldOptions.EditionDefault();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 3: {
                                    message.edition = reader.int32();
                                    break;
                                }
                            case 2: {
                                    message.value = reader.string();
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes an EditionDefault message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.protobuf.FieldOptions.EditionDefault
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.protobuf.FieldOptions.EditionDefault} EditionDefault
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    EditionDefault.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies an EditionDefault message.
                     * @function verify
                     * @memberof google.protobuf.FieldOptions.EditionDefault
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    EditionDefault.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.edition != null && message.hasOwnProperty("edition"))
                            switch (message.edition) {
                            default:
                                return "edition: enum value expected";
                            case 0:
                            case 998:
                            case 999:
                            case 1000:
                            case 1001:
                            case 1:
                            case 2:
                            case 99997:
                            case 99998:
                            case 99999:
                            case 2147483647:
                                break;
                            }
                        if (message.value != null && message.hasOwnProperty("value"))
                            if (!$util.isString(message.value))
                                return "value: string expected";
                        return null;
                    };
    
                    /**
                     * Creates an EditionDefault message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.protobuf.FieldOptions.EditionDefault
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.protobuf.FieldOptions.EditionDefault} EditionDefault
                     */
                    EditionDefault.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.protobuf.FieldOptions.EditionDefault)
                            return object;
                        var message = new $root.google.protobuf.FieldOptions.EditionDefault();
                        switch (object.edition) {
                        default:
                            if (typeof object.edition === "number") {
                                message.edition = object.edition;
                                break;
                            }
                            break;
                        case "EDITION_UNKNOWN":
                        case 0:
                            message.edition = 0;
                            break;
                        case "EDITION_PROTO2":
                        case 998:
                            message.edition = 998;
                            break;
                        case "EDITION_PROTO3":
                        case 999:
                            message.edition = 999;
                            break;
                        case "EDITION_2023":
                        case 1000:
                            message.edition = 1000;
                            break;
                        case "EDITION_2024":
                        case 1001:
                            message.edition = 1001;
                            break;
                        case "EDITION_1_TEST_ONLY":
                        case 1:
                            message.edition = 1;
                            break;
                        case "EDITION_2_TEST_ONLY":
                        case 2:
                            message.edition = 2;
                            break;
                        case "EDITION_99997_TEST_ONLY":
                        case 99997:
                            message.edition = 99997;
                            break;
                        case "EDITION_99998_TEST_ONLY":
                        case 99998:
                            message.edition = 99998;
                            break;
                        case "EDITION_99999_TEST_ONLY":
                        case 99999:
                            message.edition = 99999;
                            break;
                        case "EDITION_MAX":
                        case 2147483647:
                            message.edition = 2147483647;
                            break;
                        }
                        if (object.value != null)
                            message.value = String(object.value);
                        return message;
                    };
    
                    /**
                     * Creates a plain object from an EditionDefault message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.protobuf.FieldOptions.EditionDefault
                     * @static
                     * @param {google.protobuf.FieldOptions.EditionDefault} message EditionDefault
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    EditionDefault.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults) {
                            object.value = "";
                            object.edition = options.enums === String ? "EDITION_UNKNOWN" : 0;
                        }
                        if (message.value != null && message.hasOwnProperty("value"))
                            object.value = message.value;
                        if (message.edition != null && message.hasOwnProperty("edition"))
                            object.edition = options.enums === String ? $root.google.protobuf.Edition[message.edition] === undefined ? message.edition : $root.google.protobuf.Edition[message.edition] : message.edition;
                        return object;
                    };
    
                    /**
                     * Converts this EditionDefault to JSON.
                     * @function toJSON
                     * @memberof google.protobuf.FieldOptions.EditionDefault
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    EditionDefault.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for EditionDefault
                     * @function getTypeUrl
                     * @memberof google.protobuf.FieldOptions.EditionDefault
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    EditionDefault.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.protobuf.FieldOptions.EditionDefault";
                    };
    
                    return EditionDefault;
                })();
    
                return FieldOptions;
            })();
    
            protobuf.OneofOptions = (function() {
    
                /**
                 * Properties of an OneofOptions.
                 * @memberof google.protobuf
                 * @interface IOneofOptions
                 * @property {google.protobuf.IFeatureSet|null} [features] OneofOptions features
                 * @property {Array.<google.protobuf.IUninterpretedOption>|null} [uninterpretedOption] OneofOptions uninterpretedOption
                 */
    
                /**
                 * Constructs a new OneofOptions.
                 * @memberof google.protobuf
                 * @classdesc Represents an OneofOptions.
                 * @implements IOneofOptions
                 * @constructor
                 * @param {google.protobuf.IOneofOptions=} [properties] Properties to set
                 */
                function OneofOptions(properties) {
                    this.uninterpretedOption = [];
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * OneofOptions features.
                 * @member {google.protobuf.IFeatureSet|null|undefined} features
                 * @memberof google.protobuf.OneofOptions
                 * @instance
                 */
                OneofOptions.prototype.features = null;
    
                /**
                 * OneofOptions uninterpretedOption.
                 * @member {Array.<google.protobuf.IUninterpretedOption>} uninterpretedOption
                 * @memberof google.protobuf.OneofOptions
                 * @instance
                 */
                OneofOptions.prototype.uninterpretedOption = $util.emptyArray;
    
                /**
                 * Creates a new OneofOptions instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.OneofOptions
                 * @static
                 * @param {google.protobuf.IOneofOptions=} [properties] Properties to set
                 * @returns {google.protobuf.OneofOptions} OneofOptions instance
                 */
                OneofOptions.create = function create(properties) {
                    return new OneofOptions(properties);
                };
    
                /**
                 * Encodes the specified OneofOptions message. Does not implicitly {@link google.protobuf.OneofOptions.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.OneofOptions
                 * @static
                 * @param {google.protobuf.IOneofOptions} message OneofOptions message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                OneofOptions.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.features != null && Object.hasOwnProperty.call(message, "features"))
                        $root.google.protobuf.FeatureSet.encode(message.features, writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                    if (message.uninterpretedOption != null && message.uninterpretedOption.length)
                        for (var i = 0; i < message.uninterpretedOption.length; ++i)
                            $root.google.protobuf.UninterpretedOption.encode(message.uninterpretedOption[i], writer.uint32(/* id 999, wireType 2 =*/7994).fork()).ldelim();
                    return writer;
                };
    
                /**
                 * Encodes the specified OneofOptions message, length delimited. Does not implicitly {@link google.protobuf.OneofOptions.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.OneofOptions
                 * @static
                 * @param {google.protobuf.IOneofOptions} message OneofOptions message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                OneofOptions.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes an OneofOptions message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.OneofOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.OneofOptions} OneofOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                OneofOptions.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.OneofOptions();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.features = $root.google.protobuf.FeatureSet.decode(reader, reader.uint32());
                                break;
                            }
                        case 999: {
                                if (!(message.uninterpretedOption && message.uninterpretedOption.length))
                                    message.uninterpretedOption = [];
                                message.uninterpretedOption.push($root.google.protobuf.UninterpretedOption.decode(reader, reader.uint32()));
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes an OneofOptions message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.OneofOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.OneofOptions} OneofOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                OneofOptions.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies an OneofOptions message.
                 * @function verify
                 * @memberof google.protobuf.OneofOptions
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                OneofOptions.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.features != null && message.hasOwnProperty("features")) {
                        var error = $root.google.protobuf.FeatureSet.verify(message.features);
                        if (error)
                            return "features." + error;
                    }
                    if (message.uninterpretedOption != null && message.hasOwnProperty("uninterpretedOption")) {
                        if (!Array.isArray(message.uninterpretedOption))
                            return "uninterpretedOption: array expected";
                        for (var i = 0; i < message.uninterpretedOption.length; ++i) {
                            var error = $root.google.protobuf.UninterpretedOption.verify(message.uninterpretedOption[i]);
                            if (error)
                                return "uninterpretedOption." + error;
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates an OneofOptions message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.OneofOptions
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.OneofOptions} OneofOptions
                 */
                OneofOptions.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.OneofOptions)
                        return object;
                    var message = new $root.google.protobuf.OneofOptions();
                    if (object.features != null) {
                        if (typeof object.features !== "object")
                            throw TypeError(".google.protobuf.OneofOptions.features: object expected");
                        message.features = $root.google.protobuf.FeatureSet.fromObject(object.features);
                    }
                    if (object.uninterpretedOption) {
                        if (!Array.isArray(object.uninterpretedOption))
                            throw TypeError(".google.protobuf.OneofOptions.uninterpretedOption: array expected");
                        message.uninterpretedOption = [];
                        for (var i = 0; i < object.uninterpretedOption.length; ++i) {
                            if (typeof object.uninterpretedOption[i] !== "object")
                                throw TypeError(".google.protobuf.OneofOptions.uninterpretedOption: object expected");
                            message.uninterpretedOption[i] = $root.google.protobuf.UninterpretedOption.fromObject(object.uninterpretedOption[i]);
                        }
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from an OneofOptions message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.OneofOptions
                 * @static
                 * @param {google.protobuf.OneofOptions} message OneofOptions
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                OneofOptions.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.arrays || options.defaults)
                        object.uninterpretedOption = [];
                    if (options.defaults)
                        object.features = null;
                    if (message.features != null && message.hasOwnProperty("features"))
                        object.features = $root.google.protobuf.FeatureSet.toObject(message.features, options);
                    if (message.uninterpretedOption && message.uninterpretedOption.length) {
                        object.uninterpretedOption = [];
                        for (var j = 0; j < message.uninterpretedOption.length; ++j)
                            object.uninterpretedOption[j] = $root.google.protobuf.UninterpretedOption.toObject(message.uninterpretedOption[j], options);
                    }
                    return object;
                };
    
                /**
                 * Converts this OneofOptions to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.OneofOptions
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                OneofOptions.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for OneofOptions
                 * @function getTypeUrl
                 * @memberof google.protobuf.OneofOptions
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                OneofOptions.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.OneofOptions";
                };
    
                return OneofOptions;
            })();
    
            protobuf.EnumOptions = (function() {
    
                /**
                 * Properties of an EnumOptions.
                 * @memberof google.protobuf
                 * @interface IEnumOptions
                 * @property {boolean|null} [allowAlias] EnumOptions allowAlias
                 * @property {boolean|null} [deprecated] EnumOptions deprecated
                 * @property {boolean|null} [deprecatedLegacyJsonFieldConflicts] EnumOptions deprecatedLegacyJsonFieldConflicts
                 * @property {google.protobuf.IFeatureSet|null} [features] EnumOptions features
                 * @property {Array.<google.protobuf.IUninterpretedOption>|null} [uninterpretedOption] EnumOptions uninterpretedOption
                 */
    
                /**
                 * Constructs a new EnumOptions.
                 * @memberof google.protobuf
                 * @classdesc Represents an EnumOptions.
                 * @implements IEnumOptions
                 * @constructor
                 * @param {google.protobuf.IEnumOptions=} [properties] Properties to set
                 */
                function EnumOptions(properties) {
                    this.uninterpretedOption = [];
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * EnumOptions allowAlias.
                 * @member {boolean} allowAlias
                 * @memberof google.protobuf.EnumOptions
                 * @instance
                 */
                EnumOptions.prototype.allowAlias = false;
    
                /**
                 * EnumOptions deprecated.
                 * @member {boolean} deprecated
                 * @memberof google.protobuf.EnumOptions
                 * @instance
                 */
                EnumOptions.prototype.deprecated = false;
    
                /**
                 * EnumOptions deprecatedLegacyJsonFieldConflicts.
                 * @member {boolean} deprecatedLegacyJsonFieldConflicts
                 * @memberof google.protobuf.EnumOptions
                 * @instance
                 */
                EnumOptions.prototype.deprecatedLegacyJsonFieldConflicts = false;
    
                /**
                 * EnumOptions features.
                 * @member {google.protobuf.IFeatureSet|null|undefined} features
                 * @memberof google.protobuf.EnumOptions
                 * @instance
                 */
                EnumOptions.prototype.features = null;
    
                /**
                 * EnumOptions uninterpretedOption.
                 * @member {Array.<google.protobuf.IUninterpretedOption>} uninterpretedOption
                 * @memberof google.protobuf.EnumOptions
                 * @instance
                 */
                EnumOptions.prototype.uninterpretedOption = $util.emptyArray;
    
                /**
                 * Creates a new EnumOptions instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.EnumOptions
                 * @static
                 * @param {google.protobuf.IEnumOptions=} [properties] Properties to set
                 * @returns {google.protobuf.EnumOptions} EnumOptions instance
                 */
                EnumOptions.create = function create(properties) {
                    return new EnumOptions(properties);
                };
    
                /**
                 * Encodes the specified EnumOptions message. Does not implicitly {@link google.protobuf.EnumOptions.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.EnumOptions
                 * @static
                 * @param {google.protobuf.IEnumOptions} message EnumOptions message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                EnumOptions.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.allowAlias != null && Object.hasOwnProperty.call(message, "allowAlias"))
                        writer.uint32(/* id 2, wireType 0 =*/16).bool(message.allowAlias);
                    if (message.deprecated != null && Object.hasOwnProperty.call(message, "deprecated"))
                        writer.uint32(/* id 3, wireType 0 =*/24).bool(message.deprecated);
                    if (message.deprecatedLegacyJsonFieldConflicts != null && Object.hasOwnProperty.call(message, "deprecatedLegacyJsonFieldConflicts"))
                        writer.uint32(/* id 6, wireType 0 =*/48).bool(message.deprecatedLegacyJsonFieldConflicts);
                    if (message.features != null && Object.hasOwnProperty.call(message, "features"))
                        $root.google.protobuf.FeatureSet.encode(message.features, writer.uint32(/* id 7, wireType 2 =*/58).fork()).ldelim();
                    if (message.uninterpretedOption != null && message.uninterpretedOption.length)
                        for (var i = 0; i < message.uninterpretedOption.length; ++i)
                            $root.google.protobuf.UninterpretedOption.encode(message.uninterpretedOption[i], writer.uint32(/* id 999, wireType 2 =*/7994).fork()).ldelim();
                    return writer;
                };
    
                /**
                 * Encodes the specified EnumOptions message, length delimited. Does not implicitly {@link google.protobuf.EnumOptions.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.EnumOptions
                 * @static
                 * @param {google.protobuf.IEnumOptions} message EnumOptions message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                EnumOptions.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes an EnumOptions message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.EnumOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.EnumOptions} EnumOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                EnumOptions.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.EnumOptions();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 2: {
                                message.allowAlias = reader.bool();
                                break;
                            }
                        case 3: {
                                message.deprecated = reader.bool();
                                break;
                            }
                        case 6: {
                                message.deprecatedLegacyJsonFieldConflicts = reader.bool();
                                break;
                            }
                        case 7: {
                                message.features = $root.google.protobuf.FeatureSet.decode(reader, reader.uint32());
                                break;
                            }
                        case 999: {
                                if (!(message.uninterpretedOption && message.uninterpretedOption.length))
                                    message.uninterpretedOption = [];
                                message.uninterpretedOption.push($root.google.protobuf.UninterpretedOption.decode(reader, reader.uint32()));
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes an EnumOptions message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.EnumOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.EnumOptions} EnumOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                EnumOptions.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies an EnumOptions message.
                 * @function verify
                 * @memberof google.protobuf.EnumOptions
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                EnumOptions.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.allowAlias != null && message.hasOwnProperty("allowAlias"))
                        if (typeof message.allowAlias !== "boolean")
                            return "allowAlias: boolean expected";
                    if (message.deprecated != null && message.hasOwnProperty("deprecated"))
                        if (typeof message.deprecated !== "boolean")
                            return "deprecated: boolean expected";
                    if (message.deprecatedLegacyJsonFieldConflicts != null && message.hasOwnProperty("deprecatedLegacyJsonFieldConflicts"))
                        if (typeof message.deprecatedLegacyJsonFieldConflicts !== "boolean")
                            return "deprecatedLegacyJsonFieldConflicts: boolean expected";
                    if (message.features != null && message.hasOwnProperty("features")) {
                        var error = $root.google.protobuf.FeatureSet.verify(message.features);
                        if (error)
                            return "features." + error;
                    }
                    if (message.uninterpretedOption != null && message.hasOwnProperty("uninterpretedOption")) {
                        if (!Array.isArray(message.uninterpretedOption))
                            return "uninterpretedOption: array expected";
                        for (var i = 0; i < message.uninterpretedOption.length; ++i) {
                            var error = $root.google.protobuf.UninterpretedOption.verify(message.uninterpretedOption[i]);
                            if (error)
                                return "uninterpretedOption." + error;
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates an EnumOptions message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.EnumOptions
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.EnumOptions} EnumOptions
                 */
                EnumOptions.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.EnumOptions)
                        return object;
                    var message = new $root.google.protobuf.EnumOptions();
                    if (object.allowAlias != null)
                        message.allowAlias = Boolean(object.allowAlias);
                    if (object.deprecated != null)
                        message.deprecated = Boolean(object.deprecated);
                    if (object.deprecatedLegacyJsonFieldConflicts != null)
                        message.deprecatedLegacyJsonFieldConflicts = Boolean(object.deprecatedLegacyJsonFieldConflicts);
                    if (object.features != null) {
                        if (typeof object.features !== "object")
                            throw TypeError(".google.protobuf.EnumOptions.features: object expected");
                        message.features = $root.google.protobuf.FeatureSet.fromObject(object.features);
                    }
                    if (object.uninterpretedOption) {
                        if (!Array.isArray(object.uninterpretedOption))
                            throw TypeError(".google.protobuf.EnumOptions.uninterpretedOption: array expected");
                        message.uninterpretedOption = [];
                        for (var i = 0; i < object.uninterpretedOption.length; ++i) {
                            if (typeof object.uninterpretedOption[i] !== "object")
                                throw TypeError(".google.protobuf.EnumOptions.uninterpretedOption: object expected");
                            message.uninterpretedOption[i] = $root.google.protobuf.UninterpretedOption.fromObject(object.uninterpretedOption[i]);
                        }
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from an EnumOptions message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.EnumOptions
                 * @static
                 * @param {google.protobuf.EnumOptions} message EnumOptions
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                EnumOptions.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.arrays || options.defaults)
                        object.uninterpretedOption = [];
                    if (options.defaults) {
                        object.allowAlias = false;
                        object.deprecated = false;
                        object.deprecatedLegacyJsonFieldConflicts = false;
                        object.features = null;
                    }
                    if (message.allowAlias != null && message.hasOwnProperty("allowAlias"))
                        object.allowAlias = message.allowAlias;
                    if (message.deprecated != null && message.hasOwnProperty("deprecated"))
                        object.deprecated = message.deprecated;
                    if (message.deprecatedLegacyJsonFieldConflicts != null && message.hasOwnProperty("deprecatedLegacyJsonFieldConflicts"))
                        object.deprecatedLegacyJsonFieldConflicts = message.deprecatedLegacyJsonFieldConflicts;
                    if (message.features != null && message.hasOwnProperty("features"))
                        object.features = $root.google.protobuf.FeatureSet.toObject(message.features, options);
                    if (message.uninterpretedOption && message.uninterpretedOption.length) {
                        object.uninterpretedOption = [];
                        for (var j = 0; j < message.uninterpretedOption.length; ++j)
                            object.uninterpretedOption[j] = $root.google.protobuf.UninterpretedOption.toObject(message.uninterpretedOption[j], options);
                    }
                    return object;
                };
    
                /**
                 * Converts this EnumOptions to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.EnumOptions
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                EnumOptions.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for EnumOptions
                 * @function getTypeUrl
                 * @memberof google.protobuf.EnumOptions
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                EnumOptions.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.EnumOptions";
                };
    
                return EnumOptions;
            })();
    
            protobuf.EnumValueOptions = (function() {
    
                /**
                 * Properties of an EnumValueOptions.
                 * @memberof google.protobuf
                 * @interface IEnumValueOptions
                 * @property {boolean|null} [deprecated] EnumValueOptions deprecated
                 * @property {google.protobuf.IFeatureSet|null} [features] EnumValueOptions features
                 * @property {boolean|null} [debugRedact] EnumValueOptions debugRedact
                 * @property {Array.<google.protobuf.IUninterpretedOption>|null} [uninterpretedOption] EnumValueOptions uninterpretedOption
                 */
    
                /**
                 * Constructs a new EnumValueOptions.
                 * @memberof google.protobuf
                 * @classdesc Represents an EnumValueOptions.
                 * @implements IEnumValueOptions
                 * @constructor
                 * @param {google.protobuf.IEnumValueOptions=} [properties] Properties to set
                 */
                function EnumValueOptions(properties) {
                    this.uninterpretedOption = [];
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * EnumValueOptions deprecated.
                 * @member {boolean} deprecated
                 * @memberof google.protobuf.EnumValueOptions
                 * @instance
                 */
                EnumValueOptions.prototype.deprecated = false;
    
                /**
                 * EnumValueOptions features.
                 * @member {google.protobuf.IFeatureSet|null|undefined} features
                 * @memberof google.protobuf.EnumValueOptions
                 * @instance
                 */
                EnumValueOptions.prototype.features = null;
    
                /**
                 * EnumValueOptions debugRedact.
                 * @member {boolean} debugRedact
                 * @memberof google.protobuf.EnumValueOptions
                 * @instance
                 */
                EnumValueOptions.prototype.debugRedact = false;
    
                /**
                 * EnumValueOptions uninterpretedOption.
                 * @member {Array.<google.protobuf.IUninterpretedOption>} uninterpretedOption
                 * @memberof google.protobuf.EnumValueOptions
                 * @instance
                 */
                EnumValueOptions.prototype.uninterpretedOption = $util.emptyArray;
    
                /**
                 * Creates a new EnumValueOptions instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.EnumValueOptions
                 * @static
                 * @param {google.protobuf.IEnumValueOptions=} [properties] Properties to set
                 * @returns {google.protobuf.EnumValueOptions} EnumValueOptions instance
                 */
                EnumValueOptions.create = function create(properties) {
                    return new EnumValueOptions(properties);
                };
    
                /**
                 * Encodes the specified EnumValueOptions message. Does not implicitly {@link google.protobuf.EnumValueOptions.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.EnumValueOptions
                 * @static
                 * @param {google.protobuf.IEnumValueOptions} message EnumValueOptions message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                EnumValueOptions.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.deprecated != null && Object.hasOwnProperty.call(message, "deprecated"))
                        writer.uint32(/* id 1, wireType 0 =*/8).bool(message.deprecated);
                    if (message.features != null && Object.hasOwnProperty.call(message, "features"))
                        $root.google.protobuf.FeatureSet.encode(message.features, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                    if (message.debugRedact != null && Object.hasOwnProperty.call(message, "debugRedact"))
                        writer.uint32(/* id 3, wireType 0 =*/24).bool(message.debugRedact);
                    if (message.uninterpretedOption != null && message.uninterpretedOption.length)
                        for (var i = 0; i < message.uninterpretedOption.length; ++i)
                            $root.google.protobuf.UninterpretedOption.encode(message.uninterpretedOption[i], writer.uint32(/* id 999, wireType 2 =*/7994).fork()).ldelim();
                    return writer;
                };
    
                /**
                 * Encodes the specified EnumValueOptions message, length delimited. Does not implicitly {@link google.protobuf.EnumValueOptions.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.EnumValueOptions
                 * @static
                 * @param {google.protobuf.IEnumValueOptions} message EnumValueOptions message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                EnumValueOptions.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes an EnumValueOptions message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.EnumValueOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.EnumValueOptions} EnumValueOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                EnumValueOptions.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.EnumValueOptions();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.deprecated = reader.bool();
                                break;
                            }
                        case 2: {
                                message.features = $root.google.protobuf.FeatureSet.decode(reader, reader.uint32());
                                break;
                            }
                        case 3: {
                                message.debugRedact = reader.bool();
                                break;
                            }
                        case 999: {
                                if (!(message.uninterpretedOption && message.uninterpretedOption.length))
                                    message.uninterpretedOption = [];
                                message.uninterpretedOption.push($root.google.protobuf.UninterpretedOption.decode(reader, reader.uint32()));
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes an EnumValueOptions message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.EnumValueOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.EnumValueOptions} EnumValueOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                EnumValueOptions.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies an EnumValueOptions message.
                 * @function verify
                 * @memberof google.protobuf.EnumValueOptions
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                EnumValueOptions.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.deprecated != null && message.hasOwnProperty("deprecated"))
                        if (typeof message.deprecated !== "boolean")
                            return "deprecated: boolean expected";
                    if (message.features != null && message.hasOwnProperty("features")) {
                        var error = $root.google.protobuf.FeatureSet.verify(message.features);
                        if (error)
                            return "features." + error;
                    }
                    if (message.debugRedact != null && message.hasOwnProperty("debugRedact"))
                        if (typeof message.debugRedact !== "boolean")
                            return "debugRedact: boolean expected";
                    if (message.uninterpretedOption != null && message.hasOwnProperty("uninterpretedOption")) {
                        if (!Array.isArray(message.uninterpretedOption))
                            return "uninterpretedOption: array expected";
                        for (var i = 0; i < message.uninterpretedOption.length; ++i) {
                            var error = $root.google.protobuf.UninterpretedOption.verify(message.uninterpretedOption[i]);
                            if (error)
                                return "uninterpretedOption." + error;
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates an EnumValueOptions message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.EnumValueOptions
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.EnumValueOptions} EnumValueOptions
                 */
                EnumValueOptions.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.EnumValueOptions)
                        return object;
                    var message = new $root.google.protobuf.EnumValueOptions();
                    if (object.deprecated != null)
                        message.deprecated = Boolean(object.deprecated);
                    if (object.features != null) {
                        if (typeof object.features !== "object")
                            throw TypeError(".google.protobuf.EnumValueOptions.features: object expected");
                        message.features = $root.google.protobuf.FeatureSet.fromObject(object.features);
                    }
                    if (object.debugRedact != null)
                        message.debugRedact = Boolean(object.debugRedact);
                    if (object.uninterpretedOption) {
                        if (!Array.isArray(object.uninterpretedOption))
                            throw TypeError(".google.protobuf.EnumValueOptions.uninterpretedOption: array expected");
                        message.uninterpretedOption = [];
                        for (var i = 0; i < object.uninterpretedOption.length; ++i) {
                            if (typeof object.uninterpretedOption[i] !== "object")
                                throw TypeError(".google.protobuf.EnumValueOptions.uninterpretedOption: object expected");
                            message.uninterpretedOption[i] = $root.google.protobuf.UninterpretedOption.fromObject(object.uninterpretedOption[i]);
                        }
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from an EnumValueOptions message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.EnumValueOptions
                 * @static
                 * @param {google.protobuf.EnumValueOptions} message EnumValueOptions
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                EnumValueOptions.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.arrays || options.defaults)
                        object.uninterpretedOption = [];
                    if (options.defaults) {
                        object.deprecated = false;
                        object.features = null;
                        object.debugRedact = false;
                    }
                    if (message.deprecated != null && message.hasOwnProperty("deprecated"))
                        object.deprecated = message.deprecated;
                    if (message.features != null && message.hasOwnProperty("features"))
                        object.features = $root.google.protobuf.FeatureSet.toObject(message.features, options);
                    if (message.debugRedact != null && message.hasOwnProperty("debugRedact"))
                        object.debugRedact = message.debugRedact;
                    if (message.uninterpretedOption && message.uninterpretedOption.length) {
                        object.uninterpretedOption = [];
                        for (var j = 0; j < message.uninterpretedOption.length; ++j)
                            object.uninterpretedOption[j] = $root.google.protobuf.UninterpretedOption.toObject(message.uninterpretedOption[j], options);
                    }
                    return object;
                };
    
                /**
                 * Converts this EnumValueOptions to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.EnumValueOptions
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                EnumValueOptions.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for EnumValueOptions
                 * @function getTypeUrl
                 * @memberof google.protobuf.EnumValueOptions
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                EnumValueOptions.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.EnumValueOptions";
                };
    
                return EnumValueOptions;
            })();
    
            protobuf.ServiceOptions = (function() {
    
                /**
                 * Properties of a ServiceOptions.
                 * @memberof google.protobuf
                 * @interface IServiceOptions
                 * @property {google.protobuf.IFeatureSet|null} [features] ServiceOptions features
                 * @property {boolean|null} [deprecated] ServiceOptions deprecated
                 * @property {Array.<google.protobuf.IUninterpretedOption>|null} [uninterpretedOption] ServiceOptions uninterpretedOption
                 * @property {string|null} [".google.api.defaultHost"] ServiceOptions .google.api.defaultHost
                 * @property {string|null} [".google.api.oauthScopes"] ServiceOptions .google.api.oauthScopes
                 * @property {string|null} [".google.api.apiVersion"] ServiceOptions .google.api.apiVersion
                 */
    
                /**
                 * Constructs a new ServiceOptions.
                 * @memberof google.protobuf
                 * @classdesc Represents a ServiceOptions.
                 * @implements IServiceOptions
                 * @constructor
                 * @param {google.protobuf.IServiceOptions=} [properties] Properties to set
                 */
                function ServiceOptions(properties) {
                    this.uninterpretedOption = [];
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * ServiceOptions features.
                 * @member {google.protobuf.IFeatureSet|null|undefined} features
                 * @memberof google.protobuf.ServiceOptions
                 * @instance
                 */
                ServiceOptions.prototype.features = null;
    
                /**
                 * ServiceOptions deprecated.
                 * @member {boolean} deprecated
                 * @memberof google.protobuf.ServiceOptions
                 * @instance
                 */
                ServiceOptions.prototype.deprecated = false;
    
                /**
                 * ServiceOptions uninterpretedOption.
                 * @member {Array.<google.protobuf.IUninterpretedOption>} uninterpretedOption
                 * @memberof google.protobuf.ServiceOptions
                 * @instance
                 */
                ServiceOptions.prototype.uninterpretedOption = $util.emptyArray;
    
                /**
                 * ServiceOptions .google.api.defaultHost.
                 * @member {string} .google.api.defaultHost
                 * @memberof google.protobuf.ServiceOptions
                 * @instance
                 */
                ServiceOptions.prototype[".google.api.defaultHost"] = "";
    
                /**
                 * ServiceOptions .google.api.oauthScopes.
                 * @member {string} .google.api.oauthScopes
                 * @memberof google.protobuf.ServiceOptions
                 * @instance
                 */
                ServiceOptions.prototype[".google.api.oauthScopes"] = "";
    
                /**
                 * ServiceOptions .google.api.apiVersion.
                 * @member {string} .google.api.apiVersion
                 * @memberof google.protobuf.ServiceOptions
                 * @instance
                 */
                ServiceOptions.prototype[".google.api.apiVersion"] = "";
    
                /**
                 * Creates a new ServiceOptions instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.ServiceOptions
                 * @static
                 * @param {google.protobuf.IServiceOptions=} [properties] Properties to set
                 * @returns {google.protobuf.ServiceOptions} ServiceOptions instance
                 */
                ServiceOptions.create = function create(properties) {
                    return new ServiceOptions(properties);
                };
    
                /**
                 * Encodes the specified ServiceOptions message. Does not implicitly {@link google.protobuf.ServiceOptions.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.ServiceOptions
                 * @static
                 * @param {google.protobuf.IServiceOptions} message ServiceOptions message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ServiceOptions.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.deprecated != null && Object.hasOwnProperty.call(message, "deprecated"))
                        writer.uint32(/* id 33, wireType 0 =*/264).bool(message.deprecated);
                    if (message.features != null && Object.hasOwnProperty.call(message, "features"))
                        $root.google.protobuf.FeatureSet.encode(message.features, writer.uint32(/* id 34, wireType 2 =*/274).fork()).ldelim();
                    if (message.uninterpretedOption != null && message.uninterpretedOption.length)
                        for (var i = 0; i < message.uninterpretedOption.length; ++i)
                            $root.google.protobuf.UninterpretedOption.encode(message.uninterpretedOption[i], writer.uint32(/* id 999, wireType 2 =*/7994).fork()).ldelim();
                    if (message[".google.api.defaultHost"] != null && Object.hasOwnProperty.call(message, ".google.api.defaultHost"))
                        writer.uint32(/* id 1049, wireType 2 =*/8394).string(message[".google.api.defaultHost"]);
                    if (message[".google.api.oauthScopes"] != null && Object.hasOwnProperty.call(message, ".google.api.oauthScopes"))
                        writer.uint32(/* id 1050, wireType 2 =*/8402).string(message[".google.api.oauthScopes"]);
                    if (message[".google.api.apiVersion"] != null && Object.hasOwnProperty.call(message, ".google.api.apiVersion"))
                        writer.uint32(/* id 525000001, wireType 2 =*/4200000010).string(message[".google.api.apiVersion"]);
                    return writer;
                };
    
                /**
                 * Encodes the specified ServiceOptions message, length delimited. Does not implicitly {@link google.protobuf.ServiceOptions.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.ServiceOptions
                 * @static
                 * @param {google.protobuf.IServiceOptions} message ServiceOptions message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ServiceOptions.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a ServiceOptions message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.ServiceOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.ServiceOptions} ServiceOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ServiceOptions.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.ServiceOptions();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 34: {
                                message.features = $root.google.protobuf.FeatureSet.decode(reader, reader.uint32());
                                break;
                            }
                        case 33: {
                                message.deprecated = reader.bool();
                                break;
                            }
                        case 999: {
                                if (!(message.uninterpretedOption && message.uninterpretedOption.length))
                                    message.uninterpretedOption = [];
                                message.uninterpretedOption.push($root.google.protobuf.UninterpretedOption.decode(reader, reader.uint32()));
                                break;
                            }
                        case 1049: {
                                message[".google.api.defaultHost"] = reader.string();
                                break;
                            }
                        case 1050: {
                                message[".google.api.oauthScopes"] = reader.string();
                                break;
                            }
                        case 525000001: {
                                message[".google.api.apiVersion"] = reader.string();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a ServiceOptions message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.ServiceOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.ServiceOptions} ServiceOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ServiceOptions.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a ServiceOptions message.
                 * @function verify
                 * @memberof google.protobuf.ServiceOptions
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                ServiceOptions.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.features != null && message.hasOwnProperty("features")) {
                        var error = $root.google.protobuf.FeatureSet.verify(message.features);
                        if (error)
                            return "features." + error;
                    }
                    if (message.deprecated != null && message.hasOwnProperty("deprecated"))
                        if (typeof message.deprecated !== "boolean")
                            return "deprecated: boolean expected";
                    if (message.uninterpretedOption != null && message.hasOwnProperty("uninterpretedOption")) {
                        if (!Array.isArray(message.uninterpretedOption))
                            return "uninterpretedOption: array expected";
                        for (var i = 0; i < message.uninterpretedOption.length; ++i) {
                            var error = $root.google.protobuf.UninterpretedOption.verify(message.uninterpretedOption[i]);
                            if (error)
                                return "uninterpretedOption." + error;
                        }
                    }
                    if (message[".google.api.defaultHost"] != null && message.hasOwnProperty(".google.api.defaultHost"))
                        if (!$util.isString(message[".google.api.defaultHost"]))
                            return ".google.api.defaultHost: string expected";
                    if (message[".google.api.oauthScopes"] != null && message.hasOwnProperty(".google.api.oauthScopes"))
                        if (!$util.isString(message[".google.api.oauthScopes"]))
                            return ".google.api.oauthScopes: string expected";
                    if (message[".google.api.apiVersion"] != null && message.hasOwnProperty(".google.api.apiVersion"))
                        if (!$util.isString(message[".google.api.apiVersion"]))
                            return ".google.api.apiVersion: string expected";
                    return null;
                };
    
                /**
                 * Creates a ServiceOptions message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.ServiceOptions
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.ServiceOptions} ServiceOptions
                 */
                ServiceOptions.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.ServiceOptions)
                        return object;
                    var message = new $root.google.protobuf.ServiceOptions();
                    if (object.features != null) {
                        if (typeof object.features !== "object")
                            throw TypeError(".google.protobuf.ServiceOptions.features: object expected");
                        message.features = $root.google.protobuf.FeatureSet.fromObject(object.features);
                    }
                    if (object.deprecated != null)
                        message.deprecated = Boolean(object.deprecated);
                    if (object.uninterpretedOption) {
                        if (!Array.isArray(object.uninterpretedOption))
                            throw TypeError(".google.protobuf.ServiceOptions.uninterpretedOption: array expected");
                        message.uninterpretedOption = [];
                        for (var i = 0; i < object.uninterpretedOption.length; ++i) {
                            if (typeof object.uninterpretedOption[i] !== "object")
                                throw TypeError(".google.protobuf.ServiceOptions.uninterpretedOption: object expected");
                            message.uninterpretedOption[i] = $root.google.protobuf.UninterpretedOption.fromObject(object.uninterpretedOption[i]);
                        }
                    }
                    if (object[".google.api.defaultHost"] != null)
                        message[".google.api.defaultHost"] = String(object[".google.api.defaultHost"]);
                    if (object[".google.api.oauthScopes"] != null)
                        message[".google.api.oauthScopes"] = String(object[".google.api.oauthScopes"]);
                    if (object[".google.api.apiVersion"] != null)
                        message[".google.api.apiVersion"] = String(object[".google.api.apiVersion"]);
                    return message;
                };
    
                /**
                 * Creates a plain object from a ServiceOptions message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.ServiceOptions
                 * @static
                 * @param {google.protobuf.ServiceOptions} message ServiceOptions
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                ServiceOptions.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.arrays || options.defaults)
                        object.uninterpretedOption = [];
                    if (options.defaults) {
                        object.deprecated = false;
                        object.features = null;
                        object[".google.api.defaultHost"] = "";
                        object[".google.api.oauthScopes"] = "";
                        object[".google.api.apiVersion"] = "";
                    }
                    if (message.deprecated != null && message.hasOwnProperty("deprecated"))
                        object.deprecated = message.deprecated;
                    if (message.features != null && message.hasOwnProperty("features"))
                        object.features = $root.google.protobuf.FeatureSet.toObject(message.features, options);
                    if (message.uninterpretedOption && message.uninterpretedOption.length) {
                        object.uninterpretedOption = [];
                        for (var j = 0; j < message.uninterpretedOption.length; ++j)
                            object.uninterpretedOption[j] = $root.google.protobuf.UninterpretedOption.toObject(message.uninterpretedOption[j], options);
                    }
                    if (message[".google.api.defaultHost"] != null && message.hasOwnProperty(".google.api.defaultHost"))
                        object[".google.api.defaultHost"] = message[".google.api.defaultHost"];
                    if (message[".google.api.oauthScopes"] != null && message.hasOwnProperty(".google.api.oauthScopes"))
                        object[".google.api.oauthScopes"] = message[".google.api.oauthScopes"];
                    if (message[".google.api.apiVersion"] != null && message.hasOwnProperty(".google.api.apiVersion"))
                        object[".google.api.apiVersion"] = message[".google.api.apiVersion"];
                    return object;
                };
    
                /**
                 * Converts this ServiceOptions to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.ServiceOptions
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                ServiceOptions.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for ServiceOptions
                 * @function getTypeUrl
                 * @memberof google.protobuf.ServiceOptions
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                ServiceOptions.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.ServiceOptions";
                };
    
                return ServiceOptions;
            })();
    
            protobuf.MethodOptions = (function() {
    
                /**
                 * Properties of a MethodOptions.
                 * @memberof google.protobuf
                 * @interface IMethodOptions
                 * @property {boolean|null} [deprecated] MethodOptions deprecated
                 * @property {google.protobuf.MethodOptions.IdempotencyLevel|null} [idempotencyLevel] MethodOptions idempotencyLevel
                 * @property {google.protobuf.IFeatureSet|null} [features] MethodOptions features
                 * @property {Array.<google.protobuf.IUninterpretedOption>|null} [uninterpretedOption] MethodOptions uninterpretedOption
                 * @property {google.api.IHttpRule|null} [".google.api.http"] MethodOptions .google.api.http
                 * @property {Array.<string>|null} [".google.api.methodSignature"] MethodOptions .google.api.methodSignature
                 * @property {google.api.IRoutingRule|null} [".google.api.routing"] MethodOptions .google.api.routing
                 * @property {google.longrunning.IOperationInfo|null} [".google.longrunning.operationInfo"] MethodOptions .google.longrunning.operationInfo
                 */
    
                /**
                 * Constructs a new MethodOptions.
                 * @memberof google.protobuf
                 * @classdesc Represents a MethodOptions.
                 * @implements IMethodOptions
                 * @constructor
                 * @param {google.protobuf.IMethodOptions=} [properties] Properties to set
                 */
                function MethodOptions(properties) {
                    this.uninterpretedOption = [];
                    this[".google.api.methodSignature"] = [];
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * MethodOptions deprecated.
                 * @member {boolean} deprecated
                 * @memberof google.protobuf.MethodOptions
                 * @instance
                 */
                MethodOptions.prototype.deprecated = false;
    
                /**
                 * MethodOptions idempotencyLevel.
                 * @member {google.protobuf.MethodOptions.IdempotencyLevel} idempotencyLevel
                 * @memberof google.protobuf.MethodOptions
                 * @instance
                 */
                MethodOptions.prototype.idempotencyLevel = 0;
    
                /**
                 * MethodOptions features.
                 * @member {google.protobuf.IFeatureSet|null|undefined} features
                 * @memberof google.protobuf.MethodOptions
                 * @instance
                 */
                MethodOptions.prototype.features = null;
    
                /**
                 * MethodOptions uninterpretedOption.
                 * @member {Array.<google.protobuf.IUninterpretedOption>} uninterpretedOption
                 * @memberof google.protobuf.MethodOptions
                 * @instance
                 */
                MethodOptions.prototype.uninterpretedOption = $util.emptyArray;
    
                /**
                 * MethodOptions .google.api.http.
                 * @member {google.api.IHttpRule|null|undefined} .google.api.http
                 * @memberof google.protobuf.MethodOptions
                 * @instance
                 */
                MethodOptions.prototype[".google.api.http"] = null;
    
                /**
                 * MethodOptions .google.api.methodSignature.
                 * @member {Array.<string>} .google.api.methodSignature
                 * @memberof google.protobuf.MethodOptions
                 * @instance
                 */
                MethodOptions.prototype[".google.api.methodSignature"] = $util.emptyArray;
    
                /**
                 * MethodOptions .google.api.routing.
                 * @member {google.api.IRoutingRule|null|undefined} .google.api.routing
                 * @memberof google.protobuf.MethodOptions
                 * @instance
                 */
                MethodOptions.prototype[".google.api.routing"] = null;
    
                /**
                 * MethodOptions .google.longrunning.operationInfo.
                 * @member {google.longrunning.IOperationInfo|null|undefined} .google.longrunning.operationInfo
                 * @memberof google.protobuf.MethodOptions
                 * @instance
                 */
                MethodOptions.prototype[".google.longrunning.operationInfo"] = null;
    
                /**
                 * Creates a new MethodOptions instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.MethodOptions
                 * @static
                 * @param {google.protobuf.IMethodOptions=} [properties] Properties to set
                 * @returns {google.protobuf.MethodOptions} MethodOptions instance
                 */
                MethodOptions.create = function create(properties) {
                    return new MethodOptions(properties);
                };
    
                /**
                 * Encodes the specified MethodOptions message. Does not implicitly {@link google.protobuf.MethodOptions.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.MethodOptions
                 * @static
                 * @param {google.protobuf.IMethodOptions} message MethodOptions message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                MethodOptions.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.deprecated != null && Object.hasOwnProperty.call(message, "deprecated"))
                        writer.uint32(/* id 33, wireType 0 =*/264).bool(message.deprecated);
                    if (message.idempotencyLevel != null && Object.hasOwnProperty.call(message, "idempotencyLevel"))
                        writer.uint32(/* id 34, wireType 0 =*/272).int32(message.idempotencyLevel);
                    if (message.features != null && Object.hasOwnProperty.call(message, "features"))
                        $root.google.protobuf.FeatureSet.encode(message.features, writer.uint32(/* id 35, wireType 2 =*/282).fork()).ldelim();
                    if (message.uninterpretedOption != null && message.uninterpretedOption.length)
                        for (var i = 0; i < message.uninterpretedOption.length; ++i)
                            $root.google.protobuf.UninterpretedOption.encode(message.uninterpretedOption[i], writer.uint32(/* id 999, wireType 2 =*/7994).fork()).ldelim();
                    if (message[".google.longrunning.operationInfo"] != null && Object.hasOwnProperty.call(message, ".google.longrunning.operationInfo"))
                        $root.google.longrunning.OperationInfo.encode(message[".google.longrunning.operationInfo"], writer.uint32(/* id 1049, wireType 2 =*/8394).fork()).ldelim();
                    if (message[".google.api.methodSignature"] != null && message[".google.api.methodSignature"].length)
                        for (var i = 0; i < message[".google.api.methodSignature"].length; ++i)
                            writer.uint32(/* id 1051, wireType 2 =*/8410).string(message[".google.api.methodSignature"][i]);
                    if (message[".google.api.http"] != null && Object.hasOwnProperty.call(message, ".google.api.http"))
                        $root.google.api.HttpRule.encode(message[".google.api.http"], writer.uint32(/* id 72295728, wireType 2 =*/578365826).fork()).ldelim();
                    if (message[".google.api.routing"] != null && Object.hasOwnProperty.call(message, ".google.api.routing"))
                        $root.google.api.RoutingRule.encode(message[".google.api.routing"], writer.uint32(/* id 72295729, wireType 2 =*/578365834).fork()).ldelim();
                    return writer;
                };
    
                /**
                 * Encodes the specified MethodOptions message, length delimited. Does not implicitly {@link google.protobuf.MethodOptions.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.MethodOptions
                 * @static
                 * @param {google.protobuf.IMethodOptions} message MethodOptions message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                MethodOptions.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a MethodOptions message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.MethodOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.MethodOptions} MethodOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                MethodOptions.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.MethodOptions();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 33: {
                                message.deprecated = reader.bool();
                                break;
                            }
                        case 34: {
                                message.idempotencyLevel = reader.int32();
                                break;
                            }
                        case 35: {
                                message.features = $root.google.protobuf.FeatureSet.decode(reader, reader.uint32());
                                break;
                            }
                        case 999: {
                                if (!(message.uninterpretedOption && message.uninterpretedOption.length))
                                    message.uninterpretedOption = [];
                                message.uninterpretedOption.push($root.google.protobuf.UninterpretedOption.decode(reader, reader.uint32()));
                                break;
                            }
                        case 72295728: {
                                message[".google.api.http"] = $root.google.api.HttpRule.decode(reader, reader.uint32());
                                break;
                            }
                        case 1051: {
                                if (!(message[".google.api.methodSignature"] && message[".google.api.methodSignature"].length))
                                    message[".google.api.methodSignature"] = [];
                                message[".google.api.methodSignature"].push(reader.string());
                                break;
                            }
                        case 72295729: {
                                message[".google.api.routing"] = $root.google.api.RoutingRule.decode(reader, reader.uint32());
                                break;
                            }
                        case 1049: {
                                message[".google.longrunning.operationInfo"] = $root.google.longrunning.OperationInfo.decode(reader, reader.uint32());
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a MethodOptions message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.MethodOptions
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.MethodOptions} MethodOptions
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                MethodOptions.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a MethodOptions message.
                 * @function verify
                 * @memberof google.protobuf.MethodOptions
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                MethodOptions.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.deprecated != null && message.hasOwnProperty("deprecated"))
                        if (typeof message.deprecated !== "boolean")
                            return "deprecated: boolean expected";
                    if (message.idempotencyLevel != null && message.hasOwnProperty("idempotencyLevel"))
                        switch (message.idempotencyLevel) {
                        default:
                            return "idempotencyLevel: enum value expected";
                        case 0:
                        case 1:
                        case 2:
                            break;
                        }
                    if (message.features != null && message.hasOwnProperty("features")) {
                        var error = $root.google.protobuf.FeatureSet.verify(message.features);
                        if (error)
                            return "features." + error;
                    }
                    if (message.uninterpretedOption != null && message.hasOwnProperty("uninterpretedOption")) {
                        if (!Array.isArray(message.uninterpretedOption))
                            return "uninterpretedOption: array expected";
                        for (var i = 0; i < message.uninterpretedOption.length; ++i) {
                            var error = $root.google.protobuf.UninterpretedOption.verify(message.uninterpretedOption[i]);
                            if (error)
                                return "uninterpretedOption." + error;
                        }
                    }
                    if (message[".google.api.http"] != null && message.hasOwnProperty(".google.api.http")) {
                        var error = $root.google.api.HttpRule.verify(message[".google.api.http"]);
                        if (error)
                            return ".google.api.http." + error;
                    }
                    if (message[".google.api.methodSignature"] != null && message.hasOwnProperty(".google.api.methodSignature")) {
                        if (!Array.isArray(message[".google.api.methodSignature"]))
                            return ".google.api.methodSignature: array expected";
                        for (var i = 0; i < message[".google.api.methodSignature"].length; ++i)
                            if (!$util.isString(message[".google.api.methodSignature"][i]))
                                return ".google.api.methodSignature: string[] expected";
                    }
                    if (message[".google.api.routing"] != null && message.hasOwnProperty(".google.api.routing")) {
                        var error = $root.google.api.RoutingRule.verify(message[".google.api.routing"]);
                        if (error)
                            return ".google.api.routing." + error;
                    }
                    if (message[".google.longrunning.operationInfo"] != null && message.hasOwnProperty(".google.longrunning.operationInfo")) {
                        var error = $root.google.longrunning.OperationInfo.verify(message[".google.longrunning.operationInfo"]);
                        if (error)
                            return ".google.longrunning.operationInfo." + error;
                    }
                    return null;
                };
    
                /**
                 * Creates a MethodOptions message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.MethodOptions
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.MethodOptions} MethodOptions
                 */
                MethodOptions.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.MethodOptions)
                        return object;
                    var message = new $root.google.protobuf.MethodOptions();
                    if (object.deprecated != null)
                        message.deprecated = Boolean(object.deprecated);
                    switch (object.idempotencyLevel) {
                    default:
                        if (typeof object.idempotencyLevel === "number") {
                            message.idempotencyLevel = object.idempotencyLevel;
                            break;
                        }
                        break;
                    case "IDEMPOTENCY_UNKNOWN":
                    case 0:
                        message.idempotencyLevel = 0;
                        break;
                    case "NO_SIDE_EFFECTS":
                    case 1:
                        message.idempotencyLevel = 1;
                        break;
                    case "IDEMPOTENT":
                    case 2:
                        message.idempotencyLevel = 2;
                        break;
                    }
                    if (object.features != null) {
                        if (typeof object.features !== "object")
                            throw TypeError(".google.protobuf.MethodOptions.features: object expected");
                        message.features = $root.google.protobuf.FeatureSet.fromObject(object.features);
                    }
                    if (object.uninterpretedOption) {
                        if (!Array.isArray(object.uninterpretedOption))
                            throw TypeError(".google.protobuf.MethodOptions.uninterpretedOption: array expected");
                        message.uninterpretedOption = [];
                        for (var i = 0; i < object.uninterpretedOption.length; ++i) {
                            if (typeof object.uninterpretedOption[i] !== "object")
                                throw TypeError(".google.protobuf.MethodOptions.uninterpretedOption: object expected");
                            message.uninterpretedOption[i] = $root.google.protobuf.UninterpretedOption.fromObject(object.uninterpretedOption[i]);
                        }
                    }
                    if (object[".google.api.http"] != null) {
                        if (typeof object[".google.api.http"] !== "object")
                            throw TypeError(".google.protobuf.MethodOptions..google.api.http: object expected");
                        message[".google.api.http"] = $root.google.api.HttpRule.fromObject(object[".google.api.http"]);
                    }
                    if (object[".google.api.methodSignature"]) {
                        if (!Array.isArray(object[".google.api.methodSignature"]))
                            throw TypeError(".google.protobuf.MethodOptions..google.api.methodSignature: array expected");
                        message[".google.api.methodSignature"] = [];
                        for (var i = 0; i < object[".google.api.methodSignature"].length; ++i)
                            message[".google.api.methodSignature"][i] = String(object[".google.api.methodSignature"][i]);
                    }
                    if (object[".google.api.routing"] != null) {
                        if (typeof object[".google.api.routing"] !== "object")
                            throw TypeError(".google.protobuf.MethodOptions..google.api.routing: object expected");
                        message[".google.api.routing"] = $root.google.api.RoutingRule.fromObject(object[".google.api.routing"]);
                    }
                    if (object[".google.longrunning.operationInfo"] != null) {
                        if (typeof object[".google.longrunning.operationInfo"] !== "object")
                            throw TypeError(".google.protobuf.MethodOptions..google.longrunning.operationInfo: object expected");
                        message[".google.longrunning.operationInfo"] = $root.google.longrunning.OperationInfo.fromObject(object[".google.longrunning.operationInfo"]);
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from a MethodOptions message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.MethodOptions
                 * @static
                 * @param {google.protobuf.MethodOptions} message MethodOptions
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                MethodOptions.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.arrays || options.defaults) {
                        object.uninterpretedOption = [];
                        object[".google.api.methodSignature"] = [];
                    }
                    if (options.defaults) {
                        object.deprecated = false;
                        object.idempotencyLevel = options.enums === String ? "IDEMPOTENCY_UNKNOWN" : 0;
                        object.features = null;
                        object[".google.longrunning.operationInfo"] = null;
                        object[".google.api.http"] = null;
                        object[".google.api.routing"] = null;
                    }
                    if (message.deprecated != null && message.hasOwnProperty("deprecated"))
                        object.deprecated = message.deprecated;
                    if (message.idempotencyLevel != null && message.hasOwnProperty("idempotencyLevel"))
                        object.idempotencyLevel = options.enums === String ? $root.google.protobuf.MethodOptions.IdempotencyLevel[message.idempotencyLevel] === undefined ? message.idempotencyLevel : $root.google.protobuf.MethodOptions.IdempotencyLevel[message.idempotencyLevel] : message.idempotencyLevel;
                    if (message.features != null && message.hasOwnProperty("features"))
                        object.features = $root.google.protobuf.FeatureSet.toObject(message.features, options);
                    if (message.uninterpretedOption && message.uninterpretedOption.length) {
                        object.uninterpretedOption = [];
                        for (var j = 0; j < message.uninterpretedOption.length; ++j)
                            object.uninterpretedOption[j] = $root.google.protobuf.UninterpretedOption.toObject(message.uninterpretedOption[j], options);
                    }
                    if (message[".google.longrunning.operationInfo"] != null && message.hasOwnProperty(".google.longrunning.operationInfo"))
                        object[".google.longrunning.operationInfo"] = $root.google.longrunning.OperationInfo.toObject(message[".google.longrunning.operationInfo"], options);
                    if (message[".google.api.methodSignature"] && message[".google.api.methodSignature"].length) {
                        object[".google.api.methodSignature"] = [];
                        for (var j = 0; j < message[".google.api.methodSignature"].length; ++j)
                            object[".google.api.methodSignature"][j] = message[".google.api.methodSignature"][j];
                    }
                    if (message[".google.api.http"] != null && message.hasOwnProperty(".google.api.http"))
                        object[".google.api.http"] = $root.google.api.HttpRule.toObject(message[".google.api.http"], options);
                    if (message[".google.api.routing"] != null && message.hasOwnProperty(".google.api.routing"))
                        object[".google.api.routing"] = $root.google.api.RoutingRule.toObject(message[".google.api.routing"], options);
                    return object;
                };
    
                /**
                 * Converts this MethodOptions to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.MethodOptions
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                MethodOptions.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for MethodOptions
                 * @function getTypeUrl
                 * @memberof google.protobuf.MethodOptions
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                MethodOptions.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.MethodOptions";
                };
    
                /**
                 * IdempotencyLevel enum.
                 * @name google.protobuf.MethodOptions.IdempotencyLevel
                 * @enum {number}
                 * @property {number} IDEMPOTENCY_UNKNOWN=0 IDEMPOTENCY_UNKNOWN value
                 * @property {number} NO_SIDE_EFFECTS=1 NO_SIDE_EFFECTS value
                 * @property {number} IDEMPOTENT=2 IDEMPOTENT value
                 */
                MethodOptions.IdempotencyLevel = (function() {
                    var valuesById = {}, values = Object.create(valuesById);
                    values[valuesById[0] = "IDEMPOTENCY_UNKNOWN"] = 0;
                    values[valuesById[1] = "NO_SIDE_EFFECTS"] = 1;
                    values[valuesById[2] = "IDEMPOTENT"] = 2;
                    return values;
                })();
    
                return MethodOptions;
            })();
    
            protobuf.UninterpretedOption = (function() {
    
                /**
                 * Properties of an UninterpretedOption.
                 * @memberof google.protobuf
                 * @interface IUninterpretedOption
                 * @property {Array.<google.protobuf.UninterpretedOption.INamePart>|null} [name] UninterpretedOption name
                 * @property {string|null} [identifierValue] UninterpretedOption identifierValue
                 * @property {number|Long|null} [positiveIntValue] UninterpretedOption positiveIntValue
                 * @property {number|Long|null} [negativeIntValue] UninterpretedOption negativeIntValue
                 * @property {number|null} [doubleValue] UninterpretedOption doubleValue
                 * @property {Uint8Array|null} [stringValue] UninterpretedOption stringValue
                 * @property {string|null} [aggregateValue] UninterpretedOption aggregateValue
                 */
    
                /**
                 * Constructs a new UninterpretedOption.
                 * @memberof google.protobuf
                 * @classdesc Represents an UninterpretedOption.
                 * @implements IUninterpretedOption
                 * @constructor
                 * @param {google.protobuf.IUninterpretedOption=} [properties] Properties to set
                 */
                function UninterpretedOption(properties) {
                    this.name = [];
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * UninterpretedOption name.
                 * @member {Array.<google.protobuf.UninterpretedOption.INamePart>} name
                 * @memberof google.protobuf.UninterpretedOption
                 * @instance
                 */
                UninterpretedOption.prototype.name = $util.emptyArray;
    
                /**
                 * UninterpretedOption identifierValue.
                 * @member {string} identifierValue
                 * @memberof google.protobuf.UninterpretedOption
                 * @instance
                 */
                UninterpretedOption.prototype.identifierValue = "";
    
                /**
                 * UninterpretedOption positiveIntValue.
                 * @member {number|Long} positiveIntValue
                 * @memberof google.protobuf.UninterpretedOption
                 * @instance
                 */
                UninterpretedOption.prototype.positiveIntValue = $util.Long ? $util.Long.fromBits(0,0,true) : 0;
    
                /**
                 * UninterpretedOption negativeIntValue.
                 * @member {number|Long} negativeIntValue
                 * @memberof google.protobuf.UninterpretedOption
                 * @instance
                 */
                UninterpretedOption.prototype.negativeIntValue = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                /**
                 * UninterpretedOption doubleValue.
                 * @member {number} doubleValue
                 * @memberof google.protobuf.UninterpretedOption
                 * @instance
                 */
                UninterpretedOption.prototype.doubleValue = 0;
    
                /**
                 * UninterpretedOption stringValue.
                 * @member {Uint8Array} stringValue
                 * @memberof google.protobuf.UninterpretedOption
                 * @instance
                 */
                UninterpretedOption.prototype.stringValue = $util.newBuffer([]);
    
                /**
                 * UninterpretedOption aggregateValue.
                 * @member {string} aggregateValue
                 * @memberof google.protobuf.UninterpretedOption
                 * @instance
                 */
                UninterpretedOption.prototype.aggregateValue = "";
    
                /**
                 * Creates a new UninterpretedOption instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.UninterpretedOption
                 * @static
                 * @param {google.protobuf.IUninterpretedOption=} [properties] Properties to set
                 * @returns {google.protobuf.UninterpretedOption} UninterpretedOption instance
                 */
                UninterpretedOption.create = function create(properties) {
                    return new UninterpretedOption(properties);
                };
    
                /**
                 * Encodes the specified UninterpretedOption message. Does not implicitly {@link google.protobuf.UninterpretedOption.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.UninterpretedOption
                 * @static
                 * @param {google.protobuf.IUninterpretedOption} message UninterpretedOption message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                UninterpretedOption.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.name != null && message.name.length)
                        for (var i = 0; i < message.name.length; ++i)
                            $root.google.protobuf.UninterpretedOption.NamePart.encode(message.name[i], writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                    if (message.identifierValue != null && Object.hasOwnProperty.call(message, "identifierValue"))
                        writer.uint32(/* id 3, wireType 2 =*/26).string(message.identifierValue);
                    if (message.positiveIntValue != null && Object.hasOwnProperty.call(message, "positiveIntValue"))
                        writer.uint32(/* id 4, wireType 0 =*/32).uint64(message.positiveIntValue);
                    if (message.negativeIntValue != null && Object.hasOwnProperty.call(message, "negativeIntValue"))
                        writer.uint32(/* id 5, wireType 0 =*/40).int64(message.negativeIntValue);
                    if (message.doubleValue != null && Object.hasOwnProperty.call(message, "doubleValue"))
                        writer.uint32(/* id 6, wireType 1 =*/49).double(message.doubleValue);
                    if (message.stringValue != null && Object.hasOwnProperty.call(message, "stringValue"))
                        writer.uint32(/* id 7, wireType 2 =*/58).bytes(message.stringValue);
                    if (message.aggregateValue != null && Object.hasOwnProperty.call(message, "aggregateValue"))
                        writer.uint32(/* id 8, wireType 2 =*/66).string(message.aggregateValue);
                    return writer;
                };
    
                /**
                 * Encodes the specified UninterpretedOption message, length delimited. Does not implicitly {@link google.protobuf.UninterpretedOption.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.UninterpretedOption
                 * @static
                 * @param {google.protobuf.IUninterpretedOption} message UninterpretedOption message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                UninterpretedOption.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes an UninterpretedOption message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.UninterpretedOption
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.UninterpretedOption} UninterpretedOption
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                UninterpretedOption.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.UninterpretedOption();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 2: {
                                if (!(message.name && message.name.length))
                                    message.name = [];
                                message.name.push($root.google.protobuf.UninterpretedOption.NamePart.decode(reader, reader.uint32()));
                                break;
                            }
                        case 3: {
                                message.identifierValue = reader.string();
                                break;
                            }
                        case 4: {
                                message.positiveIntValue = reader.uint64();
                                break;
                            }
                        case 5: {
                                message.negativeIntValue = reader.int64();
                                break;
                            }
                        case 6: {
                                message.doubleValue = reader.double();
                                break;
                            }
                        case 7: {
                                message.stringValue = reader.bytes();
                                break;
                            }
                        case 8: {
                                message.aggregateValue = reader.string();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes an UninterpretedOption message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.UninterpretedOption
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.UninterpretedOption} UninterpretedOption
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                UninterpretedOption.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies an UninterpretedOption message.
                 * @function verify
                 * @memberof google.protobuf.UninterpretedOption
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                UninterpretedOption.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.name != null && message.hasOwnProperty("name")) {
                        if (!Array.isArray(message.name))
                            return "name: array expected";
                        for (var i = 0; i < message.name.length; ++i) {
                            var error = $root.google.protobuf.UninterpretedOption.NamePart.verify(message.name[i]);
                            if (error)
                                return "name." + error;
                        }
                    }
                    if (message.identifierValue != null && message.hasOwnProperty("identifierValue"))
                        if (!$util.isString(message.identifierValue))
                            return "identifierValue: string expected";
                    if (message.positiveIntValue != null && message.hasOwnProperty("positiveIntValue"))
                        if (!$util.isInteger(message.positiveIntValue) && !(message.positiveIntValue && $util.isInteger(message.positiveIntValue.low) && $util.isInteger(message.positiveIntValue.high)))
                            return "positiveIntValue: integer|Long expected";
                    if (message.negativeIntValue != null && message.hasOwnProperty("negativeIntValue"))
                        if (!$util.isInteger(message.negativeIntValue) && !(message.negativeIntValue && $util.isInteger(message.negativeIntValue.low) && $util.isInteger(message.negativeIntValue.high)))
                            return "negativeIntValue: integer|Long expected";
                    if (message.doubleValue != null && message.hasOwnProperty("doubleValue"))
                        if (typeof message.doubleValue !== "number")
                            return "doubleValue: number expected";
                    if (message.stringValue != null && message.hasOwnProperty("stringValue"))
                        if (!(message.stringValue && typeof message.stringValue.length === "number" || $util.isString(message.stringValue)))
                            return "stringValue: buffer expected";
                    if (message.aggregateValue != null && message.hasOwnProperty("aggregateValue"))
                        if (!$util.isString(message.aggregateValue))
                            return "aggregateValue: string expected";
                    return null;
                };
    
                /**
                 * Creates an UninterpretedOption message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.UninterpretedOption
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.UninterpretedOption} UninterpretedOption
                 */
                UninterpretedOption.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.UninterpretedOption)
                        return object;
                    var message = new $root.google.protobuf.UninterpretedOption();
                    if (object.name) {
                        if (!Array.isArray(object.name))
                            throw TypeError(".google.protobuf.UninterpretedOption.name: array expected");
                        message.name = [];
                        for (var i = 0; i < object.name.length; ++i) {
                            if (typeof object.name[i] !== "object")
                                throw TypeError(".google.protobuf.UninterpretedOption.name: object expected");
                            message.name[i] = $root.google.protobuf.UninterpretedOption.NamePart.fromObject(object.name[i]);
                        }
                    }
                    if (object.identifierValue != null)
                        message.identifierValue = String(object.identifierValue);
                    if (object.positiveIntValue != null)
                        if ($util.Long)
                            (message.positiveIntValue = $util.Long.fromValue(object.positiveIntValue)).unsigned = true;
                        else if (typeof object.positiveIntValue === "string")
                            message.positiveIntValue = parseInt(object.positiveIntValue, 10);
                        else if (typeof object.positiveIntValue === "number")
                            message.positiveIntValue = object.positiveIntValue;
                        else if (typeof object.positiveIntValue === "object")
                            message.positiveIntValue = new $util.LongBits(object.positiveIntValue.low >>> 0, object.positiveIntValue.high >>> 0).toNumber(true);
                    if (object.negativeIntValue != null)
                        if ($util.Long)
                            (message.negativeIntValue = $util.Long.fromValue(object.negativeIntValue)).unsigned = false;
                        else if (typeof object.negativeIntValue === "string")
                            message.negativeIntValue = parseInt(object.negativeIntValue, 10);
                        else if (typeof object.negativeIntValue === "number")
                            message.negativeIntValue = object.negativeIntValue;
                        else if (typeof object.negativeIntValue === "object")
                            message.negativeIntValue = new $util.LongBits(object.negativeIntValue.low >>> 0, object.negativeIntValue.high >>> 0).toNumber();
                    if (object.doubleValue != null)
                        message.doubleValue = Number(object.doubleValue);
                    if (object.stringValue != null)
                        if (typeof object.stringValue === "string")
                            $util.base64.decode(object.stringValue, message.stringValue = $util.newBuffer($util.base64.length(object.stringValue)), 0);
                        else if (object.stringValue.length >= 0)
                            message.stringValue = object.stringValue;
                    if (object.aggregateValue != null)
                        message.aggregateValue = String(object.aggregateValue);
                    return message;
                };
    
                /**
                 * Creates a plain object from an UninterpretedOption message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.UninterpretedOption
                 * @static
                 * @param {google.protobuf.UninterpretedOption} message UninterpretedOption
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                UninterpretedOption.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.arrays || options.defaults)
                        object.name = [];
                    if (options.defaults) {
                        object.identifierValue = "";
                        if ($util.Long) {
                            var long = new $util.Long(0, 0, true);
                            object.positiveIntValue = options.longs === String ? long.toString() : options.longs === Number ? long.toNumber() : long;
                        } else
                            object.positiveIntValue = options.longs === String ? "0" : 0;
                        if ($util.Long) {
                            var long = new $util.Long(0, 0, false);
                            object.negativeIntValue = options.longs === String ? long.toString() : options.longs === Number ? long.toNumber() : long;
                        } else
                            object.negativeIntValue = options.longs === String ? "0" : 0;
                        object.doubleValue = 0;
                        if (options.bytes === String)
                            object.stringValue = "";
                        else {
                            object.stringValue = [];
                            if (options.bytes !== Array)
                                object.stringValue = $util.newBuffer(object.stringValue);
                        }
                        object.aggregateValue = "";
                    }
                    if (message.name && message.name.length) {
                        object.name = [];
                        for (var j = 0; j < message.name.length; ++j)
                            object.name[j] = $root.google.protobuf.UninterpretedOption.NamePart.toObject(message.name[j], options);
                    }
                    if (message.identifierValue != null && message.hasOwnProperty("identifierValue"))
                        object.identifierValue = message.identifierValue;
                    if (message.positiveIntValue != null && message.hasOwnProperty("positiveIntValue"))
                        if (typeof message.positiveIntValue === "number")
                            object.positiveIntValue = options.longs === String ? String(message.positiveIntValue) : message.positiveIntValue;
                        else
                            object.positiveIntValue = options.longs === String ? $util.Long.prototype.toString.call(message.positiveIntValue) : options.longs === Number ? new $util.LongBits(message.positiveIntValue.low >>> 0, message.positiveIntValue.high >>> 0).toNumber(true) : message.positiveIntValue;
                    if (message.negativeIntValue != null && message.hasOwnProperty("negativeIntValue"))
                        if (typeof message.negativeIntValue === "number")
                            object.negativeIntValue = options.longs === String ? String(message.negativeIntValue) : message.negativeIntValue;
                        else
                            object.negativeIntValue = options.longs === String ? $util.Long.prototype.toString.call(message.negativeIntValue) : options.longs === Number ? new $util.LongBits(message.negativeIntValue.low >>> 0, message.negativeIntValue.high >>> 0).toNumber() : message.negativeIntValue;
                    if (message.doubleValue != null && message.hasOwnProperty("doubleValue"))
                        object.doubleValue = options.json && !isFinite(message.doubleValue) ? String(message.doubleValue) : message.doubleValue;
                    if (message.stringValue != null && message.hasOwnProperty("stringValue"))
                        object.stringValue = options.bytes === String ? $util.base64.encode(message.stringValue, 0, message.stringValue.length) : options.bytes === Array ? Array.prototype.slice.call(message.stringValue) : message.stringValue;
                    if (message.aggregateValue != null && message.hasOwnProperty("aggregateValue"))
                        object.aggregateValue = message.aggregateValue;
                    return object;
                };
    
                /**
                 * Converts this UninterpretedOption to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.UninterpretedOption
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                UninterpretedOption.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for UninterpretedOption
                 * @function getTypeUrl
                 * @memberof google.protobuf.UninterpretedOption
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                UninterpretedOption.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.UninterpretedOption";
                };
    
                UninterpretedOption.NamePart = (function() {
    
                    /**
                     * Properties of a NamePart.
                     * @memberof google.protobuf.UninterpretedOption
                     * @interface INamePart
                     * @property {string} namePart NamePart namePart
                     * @property {boolean} isExtension NamePart isExtension
                     */
    
                    /**
                     * Constructs a new NamePart.
                     * @memberof google.protobuf.UninterpretedOption
                     * @classdesc Represents a NamePart.
                     * @implements INamePart
                     * @constructor
                     * @param {google.protobuf.UninterpretedOption.INamePart=} [properties] Properties to set
                     */
                    function NamePart(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * NamePart namePart.
                     * @member {string} namePart
                     * @memberof google.protobuf.UninterpretedOption.NamePart
                     * @instance
                     */
                    NamePart.prototype.namePart = "";
    
                    /**
                     * NamePart isExtension.
                     * @member {boolean} isExtension
                     * @memberof google.protobuf.UninterpretedOption.NamePart
                     * @instance
                     */
                    NamePart.prototype.isExtension = false;
    
                    /**
                     * Creates a new NamePart instance using the specified properties.
                     * @function create
                     * @memberof google.protobuf.UninterpretedOption.NamePart
                     * @static
                     * @param {google.protobuf.UninterpretedOption.INamePart=} [properties] Properties to set
                     * @returns {google.protobuf.UninterpretedOption.NamePart} NamePart instance
                     */
                    NamePart.create = function create(properties) {
                        return new NamePart(properties);
                    };
    
                    /**
                     * Encodes the specified NamePart message. Does not implicitly {@link google.protobuf.UninterpretedOption.NamePart.verify|verify} messages.
                     * @function encode
                     * @memberof google.protobuf.UninterpretedOption.NamePart
                     * @static
                     * @param {google.protobuf.UninterpretedOption.INamePart} message NamePart message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    NamePart.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.namePart);
                        writer.uint32(/* id 2, wireType 0 =*/16).bool(message.isExtension);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified NamePart message, length delimited. Does not implicitly {@link google.protobuf.UninterpretedOption.NamePart.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.protobuf.UninterpretedOption.NamePart
                     * @static
                     * @param {google.protobuf.UninterpretedOption.INamePart} message NamePart message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    NamePart.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a NamePart message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.protobuf.UninterpretedOption.NamePart
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.protobuf.UninterpretedOption.NamePart} NamePart
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    NamePart.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.UninterpretedOption.NamePart();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.namePart = reader.string();
                                    break;
                                }
                            case 2: {
                                    message.isExtension = reader.bool();
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        if (!message.hasOwnProperty("namePart"))
                            throw $util.ProtocolError("missing required 'namePart'", { instance: message });
                        if (!message.hasOwnProperty("isExtension"))
                            throw $util.ProtocolError("missing required 'isExtension'", { instance: message });
                        return message;
                    };
    
                    /**
                     * Decodes a NamePart message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.protobuf.UninterpretedOption.NamePart
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.protobuf.UninterpretedOption.NamePart} NamePart
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    NamePart.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a NamePart message.
                     * @function verify
                     * @memberof google.protobuf.UninterpretedOption.NamePart
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    NamePart.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (!$util.isString(message.namePart))
                            return "namePart: string expected";
                        if (typeof message.isExtension !== "boolean")
                            return "isExtension: boolean expected";
                        return null;
                    };
    
                    /**
                     * Creates a NamePart message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.protobuf.UninterpretedOption.NamePart
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.protobuf.UninterpretedOption.NamePart} NamePart
                     */
                    NamePart.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.protobuf.UninterpretedOption.NamePart)
                            return object;
                        var message = new $root.google.protobuf.UninterpretedOption.NamePart();
                        if (object.namePart != null)
                            message.namePart = String(object.namePart);
                        if (object.isExtension != null)
                            message.isExtension = Boolean(object.isExtension);
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a NamePart message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.protobuf.UninterpretedOption.NamePart
                     * @static
                     * @param {google.protobuf.UninterpretedOption.NamePart} message NamePart
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    NamePart.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults) {
                            object.namePart = "";
                            object.isExtension = false;
                        }
                        if (message.namePart != null && message.hasOwnProperty("namePart"))
                            object.namePart = message.namePart;
                        if (message.isExtension != null && message.hasOwnProperty("isExtension"))
                            object.isExtension = message.isExtension;
                        return object;
                    };
    
                    /**
                     * Converts this NamePart to JSON.
                     * @function toJSON
                     * @memberof google.protobuf.UninterpretedOption.NamePart
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    NamePart.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for NamePart
                     * @function getTypeUrl
                     * @memberof google.protobuf.UninterpretedOption.NamePart
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    NamePart.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.protobuf.UninterpretedOption.NamePart";
                    };
    
                    return NamePart;
                })();
    
                return UninterpretedOption;
            })();
    
            protobuf.FeatureSet = (function() {
    
                /**
                 * Properties of a FeatureSet.
                 * @memberof google.protobuf
                 * @interface IFeatureSet
                 * @property {google.protobuf.FeatureSet.FieldPresence|null} [fieldPresence] FeatureSet fieldPresence
                 * @property {google.protobuf.FeatureSet.EnumType|null} [enumType] FeatureSet enumType
                 * @property {google.protobuf.FeatureSet.RepeatedFieldEncoding|null} [repeatedFieldEncoding] FeatureSet repeatedFieldEncoding
                 * @property {google.protobuf.FeatureSet.Utf8Validation|null} [utf8Validation] FeatureSet utf8Validation
                 * @property {google.protobuf.FeatureSet.MessageEncoding|null} [messageEncoding] FeatureSet messageEncoding
                 * @property {google.protobuf.FeatureSet.JsonFormat|null} [jsonFormat] FeatureSet jsonFormat
                 */
    
                /**
                 * Constructs a new FeatureSet.
                 * @memberof google.protobuf
                 * @classdesc Represents a FeatureSet.
                 * @implements IFeatureSet
                 * @constructor
                 * @param {google.protobuf.IFeatureSet=} [properties] Properties to set
                 */
                function FeatureSet(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * FeatureSet fieldPresence.
                 * @member {google.protobuf.FeatureSet.FieldPresence} fieldPresence
                 * @memberof google.protobuf.FeatureSet
                 * @instance
                 */
                FeatureSet.prototype.fieldPresence = 0;
    
                /**
                 * FeatureSet enumType.
                 * @member {google.protobuf.FeatureSet.EnumType} enumType
                 * @memberof google.protobuf.FeatureSet
                 * @instance
                 */
                FeatureSet.prototype.enumType = 0;
    
                /**
                 * FeatureSet repeatedFieldEncoding.
                 * @member {google.protobuf.FeatureSet.RepeatedFieldEncoding} repeatedFieldEncoding
                 * @memberof google.protobuf.FeatureSet
                 * @instance
                 */
                FeatureSet.prototype.repeatedFieldEncoding = 0;
    
                /**
                 * FeatureSet utf8Validation.
                 * @member {google.protobuf.FeatureSet.Utf8Validation} utf8Validation
                 * @memberof google.protobuf.FeatureSet
                 * @instance
                 */
                FeatureSet.prototype.utf8Validation = 0;
    
                /**
                 * FeatureSet messageEncoding.
                 * @member {google.protobuf.FeatureSet.MessageEncoding} messageEncoding
                 * @memberof google.protobuf.FeatureSet
                 * @instance
                 */
                FeatureSet.prototype.messageEncoding = 0;
    
                /**
                 * FeatureSet jsonFormat.
                 * @member {google.protobuf.FeatureSet.JsonFormat} jsonFormat
                 * @memberof google.protobuf.FeatureSet
                 * @instance
                 */
                FeatureSet.prototype.jsonFormat = 0;
    
                /**
                 * Creates a new FeatureSet instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.FeatureSet
                 * @static
                 * @param {google.protobuf.IFeatureSet=} [properties] Properties to set
                 * @returns {google.protobuf.FeatureSet} FeatureSet instance
                 */
                FeatureSet.create = function create(properties) {
                    return new FeatureSet(properties);
                };
    
                /**
                 * Encodes the specified FeatureSet message. Does not implicitly {@link google.protobuf.FeatureSet.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.FeatureSet
                 * @static
                 * @param {google.protobuf.IFeatureSet} message FeatureSet message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FeatureSet.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.fieldPresence != null && Object.hasOwnProperty.call(message, "fieldPresence"))
                        writer.uint32(/* id 1, wireType 0 =*/8).int32(message.fieldPresence);
                    if (message.enumType != null && Object.hasOwnProperty.call(message, "enumType"))
                        writer.uint32(/* id 2, wireType 0 =*/16).int32(message.enumType);
                    if (message.repeatedFieldEncoding != null && Object.hasOwnProperty.call(message, "repeatedFieldEncoding"))
                        writer.uint32(/* id 3, wireType 0 =*/24).int32(message.repeatedFieldEncoding);
                    if (message.utf8Validation != null && Object.hasOwnProperty.call(message, "utf8Validation"))
                        writer.uint32(/* id 4, wireType 0 =*/32).int32(message.utf8Validation);
                    if (message.messageEncoding != null && Object.hasOwnProperty.call(message, "messageEncoding"))
                        writer.uint32(/* id 5, wireType 0 =*/40).int32(message.messageEncoding);
                    if (message.jsonFormat != null && Object.hasOwnProperty.call(message, "jsonFormat"))
                        writer.uint32(/* id 6, wireType 0 =*/48).int32(message.jsonFormat);
                    return writer;
                };
    
                /**
                 * Encodes the specified FeatureSet message, length delimited. Does not implicitly {@link google.protobuf.FeatureSet.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.FeatureSet
                 * @static
                 * @param {google.protobuf.IFeatureSet} message FeatureSet message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FeatureSet.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a FeatureSet message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.FeatureSet
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.FeatureSet} FeatureSet
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FeatureSet.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.FeatureSet();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.fieldPresence = reader.int32();
                                break;
                            }
                        case 2: {
                                message.enumType = reader.int32();
                                break;
                            }
                        case 3: {
                                message.repeatedFieldEncoding = reader.int32();
                                break;
                            }
                        case 4: {
                                message.utf8Validation = reader.int32();
                                break;
                            }
                        case 5: {
                                message.messageEncoding = reader.int32();
                                break;
                            }
                        case 6: {
                                message.jsonFormat = reader.int32();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a FeatureSet message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.FeatureSet
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.FeatureSet} FeatureSet
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FeatureSet.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a FeatureSet message.
                 * @function verify
                 * @memberof google.protobuf.FeatureSet
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                FeatureSet.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.fieldPresence != null && message.hasOwnProperty("fieldPresence"))
                        switch (message.fieldPresence) {
                        default:
                            return "fieldPresence: enum value expected";
                        case 0:
                        case 1:
                        case 2:
                        case 3:
                            break;
                        }
                    if (message.enumType != null && message.hasOwnProperty("enumType"))
                        switch (message.enumType) {
                        default:
                            return "enumType: enum value expected";
                        case 0:
                        case 1:
                        case 2:
                            break;
                        }
                    if (message.repeatedFieldEncoding != null && message.hasOwnProperty("repeatedFieldEncoding"))
                        switch (message.repeatedFieldEncoding) {
                        default:
                            return "repeatedFieldEncoding: enum value expected";
                        case 0:
                        case 1:
                        case 2:
                            break;
                        }
                    if (message.utf8Validation != null && message.hasOwnProperty("utf8Validation"))
                        switch (message.utf8Validation) {
                        default:
                            return "utf8Validation: enum value expected";
                        case 0:
                        case 2:
                        case 3:
                            break;
                        }
                    if (message.messageEncoding != null && message.hasOwnProperty("messageEncoding"))
                        switch (message.messageEncoding) {
                        default:
                            return "messageEncoding: enum value expected";
                        case 0:
                        case 1:
                        case 2:
                            break;
                        }
                    if (message.jsonFormat != null && message.hasOwnProperty("jsonFormat"))
                        switch (message.jsonFormat) {
                        default:
                            return "jsonFormat: enum value expected";
                        case 0:
                        case 1:
                        case 2:
                            break;
                        }
                    return null;
                };
    
                /**
                 * Creates a FeatureSet message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.FeatureSet
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.FeatureSet} FeatureSet
                 */
                FeatureSet.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.FeatureSet)
                        return object;
                    var message = new $root.google.protobuf.FeatureSet();
                    switch (object.fieldPresence) {
                    default:
                        if (typeof object.fieldPresence === "number") {
                            message.fieldPresence = object.fieldPresence;
                            break;
                        }
                        break;
                    case "FIELD_PRESENCE_UNKNOWN":
                    case 0:
                        message.fieldPresence = 0;
                        break;
                    case "EXPLICIT":
                    case 1:
                        message.fieldPresence = 1;
                        break;
                    case "IMPLICIT":
                    case 2:
                        message.fieldPresence = 2;
                        break;
                    case "LEGACY_REQUIRED":
                    case 3:
                        message.fieldPresence = 3;
                        break;
                    }
                    switch (object.enumType) {
                    default:
                        if (typeof object.enumType === "number") {
                            message.enumType = object.enumType;
                            break;
                        }
                        break;
                    case "ENUM_TYPE_UNKNOWN":
                    case 0:
                        message.enumType = 0;
                        break;
                    case "OPEN":
                    case 1:
                        message.enumType = 1;
                        break;
                    case "CLOSED":
                    case 2:
                        message.enumType = 2;
                        break;
                    }
                    switch (object.repeatedFieldEncoding) {
                    default:
                        if (typeof object.repeatedFieldEncoding === "number") {
                            message.repeatedFieldEncoding = object.repeatedFieldEncoding;
                            break;
                        }
                        break;
                    case "REPEATED_FIELD_ENCODING_UNKNOWN":
                    case 0:
                        message.repeatedFieldEncoding = 0;
                        break;
                    case "PACKED":
                    case 1:
                        message.repeatedFieldEncoding = 1;
                        break;
                    case "EXPANDED":
                    case 2:
                        message.repeatedFieldEncoding = 2;
                        break;
                    }
                    switch (object.utf8Validation) {
                    default:
                        if (typeof object.utf8Validation === "number") {
                            message.utf8Validation = object.utf8Validation;
                            break;
                        }
                        break;
                    case "UTF8_VALIDATION_UNKNOWN":
                    case 0:
                        message.utf8Validation = 0;
                        break;
                    case "VERIFY":
                    case 2:
                        message.utf8Validation = 2;
                        break;
                    case "NONE":
                    case 3:
                        message.utf8Validation = 3;
                        break;
                    }
                    switch (object.messageEncoding) {
                    default:
                        if (typeof object.messageEncoding === "number") {
                            message.messageEncoding = object.messageEncoding;
                            break;
                        }
                        break;
                    case "MESSAGE_ENCODING_UNKNOWN":
                    case 0:
                        message.messageEncoding = 0;
                        break;
                    case "LENGTH_PREFIXED":
                    case 1:
                        message.messageEncoding = 1;
                        break;
                    case "DELIMITED":
                    case 2:
                        message.messageEncoding = 2;
                        break;
                    }
                    switch (object.jsonFormat) {
                    default:
                        if (typeof object.jsonFormat === "number") {
                            message.jsonFormat = object.jsonFormat;
                            break;
                        }
                        break;
                    case "JSON_FORMAT_UNKNOWN":
                    case 0:
                        message.jsonFormat = 0;
                        break;
                    case "ALLOW":
                    case 1:
                        message.jsonFormat = 1;
                        break;
                    case "LEGACY_BEST_EFFORT":
                    case 2:
                        message.jsonFormat = 2;
                        break;
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from a FeatureSet message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.FeatureSet
                 * @static
                 * @param {google.protobuf.FeatureSet} message FeatureSet
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                FeatureSet.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults) {
                        object.fieldPresence = options.enums === String ? "FIELD_PRESENCE_UNKNOWN" : 0;
                        object.enumType = options.enums === String ? "ENUM_TYPE_UNKNOWN" : 0;
                        object.repeatedFieldEncoding = options.enums === String ? "REPEATED_FIELD_ENCODING_UNKNOWN" : 0;
                        object.utf8Validation = options.enums === String ? "UTF8_VALIDATION_UNKNOWN" : 0;
                        object.messageEncoding = options.enums === String ? "MESSAGE_ENCODING_UNKNOWN" : 0;
                        object.jsonFormat = options.enums === String ? "JSON_FORMAT_UNKNOWN" : 0;
                    }
                    if (message.fieldPresence != null && message.hasOwnProperty("fieldPresence"))
                        object.fieldPresence = options.enums === String ? $root.google.protobuf.FeatureSet.FieldPresence[message.fieldPresence] === undefined ? message.fieldPresence : $root.google.protobuf.FeatureSet.FieldPresence[message.fieldPresence] : message.fieldPresence;
                    if (message.enumType != null && message.hasOwnProperty("enumType"))
                        object.enumType = options.enums === String ? $root.google.protobuf.FeatureSet.EnumType[message.enumType] === undefined ? message.enumType : $root.google.protobuf.FeatureSet.EnumType[message.enumType] : message.enumType;
                    if (message.repeatedFieldEncoding != null && message.hasOwnProperty("repeatedFieldEncoding"))
                        object.repeatedFieldEncoding = options.enums === String ? $root.google.protobuf.FeatureSet.RepeatedFieldEncoding[message.repeatedFieldEncoding] === undefined ? message.repeatedFieldEncoding : $root.google.protobuf.FeatureSet.RepeatedFieldEncoding[message.repeatedFieldEncoding] : message.repeatedFieldEncoding;
                    if (message.utf8Validation != null && message.hasOwnProperty("utf8Validation"))
                        object.utf8Validation = options.enums === String ? $root.google.protobuf.FeatureSet.Utf8Validation[message.utf8Validation] === undefined ? message.utf8Validation : $root.google.protobuf.FeatureSet.Utf8Validation[message.utf8Validation] : message.utf8Validation;
                    if (message.messageEncoding != null && message.hasOwnProperty("messageEncoding"))
                        object.messageEncoding = options.enums === String ? $root.google.protobuf.FeatureSet.MessageEncoding[message.messageEncoding] === undefined ? message.messageEncoding : $root.google.protobuf.FeatureSet.MessageEncoding[message.messageEncoding] : message.messageEncoding;
                    if (message.jsonFormat != null && message.hasOwnProperty("jsonFormat"))
                        object.jsonFormat = options.enums === String ? $root.google.protobuf.FeatureSet.JsonFormat[message.jsonFormat] === undefined ? message.jsonFormat : $root.google.protobuf.FeatureSet.JsonFormat[message.jsonFormat] : message.jsonFormat;
                    return object;
                };
    
                /**
                 * Converts this FeatureSet to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.FeatureSet
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                FeatureSet.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for FeatureSet
                 * @function getTypeUrl
                 * @memberof google.protobuf.FeatureSet
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                FeatureSet.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.FeatureSet";
                };
    
                /**
                 * FieldPresence enum.
                 * @name google.protobuf.FeatureSet.FieldPresence
                 * @enum {number}
                 * @property {number} FIELD_PRESENCE_UNKNOWN=0 FIELD_PRESENCE_UNKNOWN value
                 * @property {number} EXPLICIT=1 EXPLICIT value
                 * @property {number} IMPLICIT=2 IMPLICIT value
                 * @property {number} LEGACY_REQUIRED=3 LEGACY_REQUIRED value
                 */
                FeatureSet.FieldPresence = (function() {
                    var valuesById = {}, values = Object.create(valuesById);
                    values[valuesById[0] = "FIELD_PRESENCE_UNKNOWN"] = 0;
                    values[valuesById[1] = "EXPLICIT"] = 1;
                    values[valuesById[2] = "IMPLICIT"] = 2;
                    values[valuesById[3] = "LEGACY_REQUIRED"] = 3;
                    return values;
                })();
    
                /**
                 * EnumType enum.
                 * @name google.protobuf.FeatureSet.EnumType
                 * @enum {number}
                 * @property {number} ENUM_TYPE_UNKNOWN=0 ENUM_TYPE_UNKNOWN value
                 * @property {number} OPEN=1 OPEN value
                 * @property {number} CLOSED=2 CLOSED value
                 */
                FeatureSet.EnumType = (function() {
                    var valuesById = {}, values = Object.create(valuesById);
                    values[valuesById[0] = "ENUM_TYPE_UNKNOWN"] = 0;
                    values[valuesById[1] = "OPEN"] = 1;
                    values[valuesById[2] = "CLOSED"] = 2;
                    return values;
                })();
    
                /**
                 * RepeatedFieldEncoding enum.
                 * @name google.protobuf.FeatureSet.RepeatedFieldEncoding
                 * @enum {number}
                 * @property {number} REPEATED_FIELD_ENCODING_UNKNOWN=0 REPEATED_FIELD_ENCODING_UNKNOWN value
                 * @property {number} PACKED=1 PACKED value
                 * @property {number} EXPANDED=2 EXPANDED value
                 */
                FeatureSet.RepeatedFieldEncoding = (function() {
                    var valuesById = {}, values = Object.create(valuesById);
                    values[valuesById[0] = "REPEATED_FIELD_ENCODING_UNKNOWN"] = 0;
                    values[valuesById[1] = "PACKED"] = 1;
                    values[valuesById[2] = "EXPANDED"] = 2;
                    return values;
                })();
    
                /**
                 * Utf8Validation enum.
                 * @name google.protobuf.FeatureSet.Utf8Validation
                 * @enum {number}
                 * @property {number} UTF8_VALIDATION_UNKNOWN=0 UTF8_VALIDATION_UNKNOWN value
                 * @property {number} VERIFY=2 VERIFY value
                 * @property {number} NONE=3 NONE value
                 */
                FeatureSet.Utf8Validation = (function() {
                    var valuesById = {}, values = Object.create(valuesById);
                    values[valuesById[0] = "UTF8_VALIDATION_UNKNOWN"] = 0;
                    values[valuesById[2] = "VERIFY"] = 2;
                    values[valuesById[3] = "NONE"] = 3;
                    return values;
                })();
    
                /**
                 * MessageEncoding enum.
                 * @name google.protobuf.FeatureSet.MessageEncoding
                 * @enum {number}
                 * @property {number} MESSAGE_ENCODING_UNKNOWN=0 MESSAGE_ENCODING_UNKNOWN value
                 * @property {number} LENGTH_PREFIXED=1 LENGTH_PREFIXED value
                 * @property {number} DELIMITED=2 DELIMITED value
                 */
                FeatureSet.MessageEncoding = (function() {
                    var valuesById = {}, values = Object.create(valuesById);
                    values[valuesById[0] = "MESSAGE_ENCODING_UNKNOWN"] = 0;
                    values[valuesById[1] = "LENGTH_PREFIXED"] = 1;
                    values[valuesById[2] = "DELIMITED"] = 2;
                    return values;
                })();
    
                /**
                 * JsonFormat enum.
                 * @name google.protobuf.FeatureSet.JsonFormat
                 * @enum {number}
                 * @property {number} JSON_FORMAT_UNKNOWN=0 JSON_FORMAT_UNKNOWN value
                 * @property {number} ALLOW=1 ALLOW value
                 * @property {number} LEGACY_BEST_EFFORT=2 LEGACY_BEST_EFFORT value
                 */
                FeatureSet.JsonFormat = (function() {
                    var valuesById = {}, values = Object.create(valuesById);
                    values[valuesById[0] = "JSON_FORMAT_UNKNOWN"] = 0;
                    values[valuesById[1] = "ALLOW"] = 1;
                    values[valuesById[2] = "LEGACY_BEST_EFFORT"] = 2;
                    return values;
                })();
    
                return FeatureSet;
            })();
    
            protobuf.FeatureSetDefaults = (function() {
    
                /**
                 * Properties of a FeatureSetDefaults.
                 * @memberof google.protobuf
                 * @interface IFeatureSetDefaults
                 * @property {Array.<google.protobuf.FeatureSetDefaults.IFeatureSetEditionDefault>|null} [defaults] FeatureSetDefaults defaults
                 * @property {google.protobuf.Edition|null} [minimumEdition] FeatureSetDefaults minimumEdition
                 * @property {google.protobuf.Edition|null} [maximumEdition] FeatureSetDefaults maximumEdition
                 */
    
                /**
                 * Constructs a new FeatureSetDefaults.
                 * @memberof google.protobuf
                 * @classdesc Represents a FeatureSetDefaults.
                 * @implements IFeatureSetDefaults
                 * @constructor
                 * @param {google.protobuf.IFeatureSetDefaults=} [properties] Properties to set
                 */
                function FeatureSetDefaults(properties) {
                    this.defaults = [];
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * FeatureSetDefaults defaults.
                 * @member {Array.<google.protobuf.FeatureSetDefaults.IFeatureSetEditionDefault>} defaults
                 * @memberof google.protobuf.FeatureSetDefaults
                 * @instance
                 */
                FeatureSetDefaults.prototype.defaults = $util.emptyArray;
    
                /**
                 * FeatureSetDefaults minimumEdition.
                 * @member {google.protobuf.Edition} minimumEdition
                 * @memberof google.protobuf.FeatureSetDefaults
                 * @instance
                 */
                FeatureSetDefaults.prototype.minimumEdition = 0;
    
                /**
                 * FeatureSetDefaults maximumEdition.
                 * @member {google.protobuf.Edition} maximumEdition
                 * @memberof google.protobuf.FeatureSetDefaults
                 * @instance
                 */
                FeatureSetDefaults.prototype.maximumEdition = 0;
    
                /**
                 * Creates a new FeatureSetDefaults instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.FeatureSetDefaults
                 * @static
                 * @param {google.protobuf.IFeatureSetDefaults=} [properties] Properties to set
                 * @returns {google.protobuf.FeatureSetDefaults} FeatureSetDefaults instance
                 */
                FeatureSetDefaults.create = function create(properties) {
                    return new FeatureSetDefaults(properties);
                };
    
                /**
                 * Encodes the specified FeatureSetDefaults message. Does not implicitly {@link google.protobuf.FeatureSetDefaults.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.FeatureSetDefaults
                 * @static
                 * @param {google.protobuf.IFeatureSetDefaults} message FeatureSetDefaults message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FeatureSetDefaults.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.defaults != null && message.defaults.length)
                        for (var i = 0; i < message.defaults.length; ++i)
                            $root.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.encode(message.defaults[i], writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                    if (message.minimumEdition != null && Object.hasOwnProperty.call(message, "minimumEdition"))
                        writer.uint32(/* id 4, wireType 0 =*/32).int32(message.minimumEdition);
                    if (message.maximumEdition != null && Object.hasOwnProperty.call(message, "maximumEdition"))
                        writer.uint32(/* id 5, wireType 0 =*/40).int32(message.maximumEdition);
                    return writer;
                };
    
                /**
                 * Encodes the specified FeatureSetDefaults message, length delimited. Does not implicitly {@link google.protobuf.FeatureSetDefaults.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.FeatureSetDefaults
                 * @static
                 * @param {google.protobuf.IFeatureSetDefaults} message FeatureSetDefaults message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FeatureSetDefaults.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a FeatureSetDefaults message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.FeatureSetDefaults
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.FeatureSetDefaults} FeatureSetDefaults
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FeatureSetDefaults.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.FeatureSetDefaults();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                if (!(message.defaults && message.defaults.length))
                                    message.defaults = [];
                                message.defaults.push($root.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.decode(reader, reader.uint32()));
                                break;
                            }
                        case 4: {
                                message.minimumEdition = reader.int32();
                                break;
                            }
                        case 5: {
                                message.maximumEdition = reader.int32();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a FeatureSetDefaults message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.FeatureSetDefaults
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.FeatureSetDefaults} FeatureSetDefaults
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FeatureSetDefaults.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a FeatureSetDefaults message.
                 * @function verify
                 * @memberof google.protobuf.FeatureSetDefaults
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                FeatureSetDefaults.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.defaults != null && message.hasOwnProperty("defaults")) {
                        if (!Array.isArray(message.defaults))
                            return "defaults: array expected";
                        for (var i = 0; i < message.defaults.length; ++i) {
                            var error = $root.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.verify(message.defaults[i]);
                            if (error)
                                return "defaults." + error;
                        }
                    }
                    if (message.minimumEdition != null && message.hasOwnProperty("minimumEdition"))
                        switch (message.minimumEdition) {
                        default:
                            return "minimumEdition: enum value expected";
                        case 0:
                        case 998:
                        case 999:
                        case 1000:
                        case 1001:
                        case 1:
                        case 2:
                        case 99997:
                        case 99998:
                        case 99999:
                        case 2147483647:
                            break;
                        }
                    if (message.maximumEdition != null && message.hasOwnProperty("maximumEdition"))
                        switch (message.maximumEdition) {
                        default:
                            return "maximumEdition: enum value expected";
                        case 0:
                        case 998:
                        case 999:
                        case 1000:
                        case 1001:
                        case 1:
                        case 2:
                        case 99997:
                        case 99998:
                        case 99999:
                        case 2147483647:
                            break;
                        }
                    return null;
                };
    
                /**
                 * Creates a FeatureSetDefaults message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.FeatureSetDefaults
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.FeatureSetDefaults} FeatureSetDefaults
                 */
                FeatureSetDefaults.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.FeatureSetDefaults)
                        return object;
                    var message = new $root.google.protobuf.FeatureSetDefaults();
                    if (object.defaults) {
                        if (!Array.isArray(object.defaults))
                            throw TypeError(".google.protobuf.FeatureSetDefaults.defaults: array expected");
                        message.defaults = [];
                        for (var i = 0; i < object.defaults.length; ++i) {
                            if (typeof object.defaults[i] !== "object")
                                throw TypeError(".google.protobuf.FeatureSetDefaults.defaults: object expected");
                            message.defaults[i] = $root.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.fromObject(object.defaults[i]);
                        }
                    }
                    switch (object.minimumEdition) {
                    default:
                        if (typeof object.minimumEdition === "number") {
                            message.minimumEdition = object.minimumEdition;
                            break;
                        }
                        break;
                    case "EDITION_UNKNOWN":
                    case 0:
                        message.minimumEdition = 0;
                        break;
                    case "EDITION_PROTO2":
                    case 998:
                        message.minimumEdition = 998;
                        break;
                    case "EDITION_PROTO3":
                    case 999:
                        message.minimumEdition = 999;
                        break;
                    case "EDITION_2023":
                    case 1000:
                        message.minimumEdition = 1000;
                        break;
                    case "EDITION_2024":
                    case 1001:
                        message.minimumEdition = 1001;
                        break;
                    case "EDITION_1_TEST_ONLY":
                    case 1:
                        message.minimumEdition = 1;
                        break;
                    case "EDITION_2_TEST_ONLY":
                    case 2:
                        message.minimumEdition = 2;
                        break;
                    case "EDITION_99997_TEST_ONLY":
                    case 99997:
                        message.minimumEdition = 99997;
                        break;
                    case "EDITION_99998_TEST_ONLY":
                    case 99998:
                        message.minimumEdition = 99998;
                        break;
                    case "EDITION_99999_TEST_ONLY":
                    case 99999:
                        message.minimumEdition = 99999;
                        break;
                    case "EDITION_MAX":
                    case 2147483647:
                        message.minimumEdition = 2147483647;
                        break;
                    }
                    switch (object.maximumEdition) {
                    default:
                        if (typeof object.maximumEdition === "number") {
                            message.maximumEdition = object.maximumEdition;
                            break;
                        }
                        break;
                    case "EDITION_UNKNOWN":
                    case 0:
                        message.maximumEdition = 0;
                        break;
                    case "EDITION_PROTO2":
                    case 998:
                        message.maximumEdition = 998;
                        break;
                    case "EDITION_PROTO3":
                    case 999:
                        message.maximumEdition = 999;
                        break;
                    case "EDITION_2023":
                    case 1000:
                        message.maximumEdition = 1000;
                        break;
                    case "EDITION_2024":
                    case 1001:
                        message.maximumEdition = 1001;
                        break;
                    case "EDITION_1_TEST_ONLY":
                    case 1:
                        message.maximumEdition = 1;
                        break;
                    case "EDITION_2_TEST_ONLY":
                    case 2:
                        message.maximumEdition = 2;
                        break;
                    case "EDITION_99997_TEST_ONLY":
                    case 99997:
                        message.maximumEdition = 99997;
                        break;
                    case "EDITION_99998_TEST_ONLY":
                    case 99998:
                        message.maximumEdition = 99998;
                        break;
                    case "EDITION_99999_TEST_ONLY":
                    case 99999:
                        message.maximumEdition = 99999;
                        break;
                    case "EDITION_MAX":
                    case 2147483647:
                        message.maximumEdition = 2147483647;
                        break;
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from a FeatureSetDefaults message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.FeatureSetDefaults
                 * @static
                 * @param {google.protobuf.FeatureSetDefaults} message FeatureSetDefaults
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                FeatureSetDefaults.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.arrays || options.defaults)
                        object.defaults = [];
                    if (options.defaults) {
                        object.minimumEdition = options.enums === String ? "EDITION_UNKNOWN" : 0;
                        object.maximumEdition = options.enums === String ? "EDITION_UNKNOWN" : 0;
                    }
                    if (message.defaults && message.defaults.length) {
                        object.defaults = [];
                        for (var j = 0; j < message.defaults.length; ++j)
                            object.defaults[j] = $root.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.toObject(message.defaults[j], options);
                    }
                    if (message.minimumEdition != null && message.hasOwnProperty("minimumEdition"))
                        object.minimumEdition = options.enums === String ? $root.google.protobuf.Edition[message.minimumEdition] === undefined ? message.minimumEdition : $root.google.protobuf.Edition[message.minimumEdition] : message.minimumEdition;
                    if (message.maximumEdition != null && message.hasOwnProperty("maximumEdition"))
                        object.maximumEdition = options.enums === String ? $root.google.protobuf.Edition[message.maximumEdition] === undefined ? message.maximumEdition : $root.google.protobuf.Edition[message.maximumEdition] : message.maximumEdition;
                    return object;
                };
    
                /**
                 * Converts this FeatureSetDefaults to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.FeatureSetDefaults
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                FeatureSetDefaults.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for FeatureSetDefaults
                 * @function getTypeUrl
                 * @memberof google.protobuf.FeatureSetDefaults
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                FeatureSetDefaults.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.FeatureSetDefaults";
                };
    
                FeatureSetDefaults.FeatureSetEditionDefault = (function() {
    
                    /**
                     * Properties of a FeatureSetEditionDefault.
                     * @memberof google.protobuf.FeatureSetDefaults
                     * @interface IFeatureSetEditionDefault
                     * @property {google.protobuf.Edition|null} [edition] FeatureSetEditionDefault edition
                     * @property {google.protobuf.IFeatureSet|null} [features] FeatureSetEditionDefault features
                     */
    
                    /**
                     * Constructs a new FeatureSetEditionDefault.
                     * @memberof google.protobuf.FeatureSetDefaults
                     * @classdesc Represents a FeatureSetEditionDefault.
                     * @implements IFeatureSetEditionDefault
                     * @constructor
                     * @param {google.protobuf.FeatureSetDefaults.IFeatureSetEditionDefault=} [properties] Properties to set
                     */
                    function FeatureSetEditionDefault(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * FeatureSetEditionDefault edition.
                     * @member {google.protobuf.Edition} edition
                     * @memberof google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault
                     * @instance
                     */
                    FeatureSetEditionDefault.prototype.edition = 0;
    
                    /**
                     * FeatureSetEditionDefault features.
                     * @member {google.protobuf.IFeatureSet|null|undefined} features
                     * @memberof google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault
                     * @instance
                     */
                    FeatureSetEditionDefault.prototype.features = null;
    
                    /**
                     * Creates a new FeatureSetEditionDefault instance using the specified properties.
                     * @function create
                     * @memberof google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault
                     * @static
                     * @param {google.protobuf.FeatureSetDefaults.IFeatureSetEditionDefault=} [properties] Properties to set
                     * @returns {google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault} FeatureSetEditionDefault instance
                     */
                    FeatureSetEditionDefault.create = function create(properties) {
                        return new FeatureSetEditionDefault(properties);
                    };
    
                    /**
                     * Encodes the specified FeatureSetEditionDefault message. Does not implicitly {@link google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.verify|verify} messages.
                     * @function encode
                     * @memberof google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault
                     * @static
                     * @param {google.protobuf.FeatureSetDefaults.IFeatureSetEditionDefault} message FeatureSetEditionDefault message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    FeatureSetEditionDefault.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.features != null && Object.hasOwnProperty.call(message, "features"))
                            $root.google.protobuf.FeatureSet.encode(message.features, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                        if (message.edition != null && Object.hasOwnProperty.call(message, "edition"))
                            writer.uint32(/* id 3, wireType 0 =*/24).int32(message.edition);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified FeatureSetEditionDefault message, length delimited. Does not implicitly {@link google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault
                     * @static
                     * @param {google.protobuf.FeatureSetDefaults.IFeatureSetEditionDefault} message FeatureSetEditionDefault message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    FeatureSetEditionDefault.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a FeatureSetEditionDefault message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault} FeatureSetEditionDefault
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    FeatureSetEditionDefault.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 3: {
                                    message.edition = reader.int32();
                                    break;
                                }
                            case 2: {
                                    message.features = $root.google.protobuf.FeatureSet.decode(reader, reader.uint32());
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a FeatureSetEditionDefault message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault} FeatureSetEditionDefault
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    FeatureSetEditionDefault.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a FeatureSetEditionDefault message.
                     * @function verify
                     * @memberof google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    FeatureSetEditionDefault.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.edition != null && message.hasOwnProperty("edition"))
                            switch (message.edition) {
                            default:
                                return "edition: enum value expected";
                            case 0:
                            case 998:
                            case 999:
                            case 1000:
                            case 1001:
                            case 1:
                            case 2:
                            case 99997:
                            case 99998:
                            case 99999:
                            case 2147483647:
                                break;
                            }
                        if (message.features != null && message.hasOwnProperty("features")) {
                            var error = $root.google.protobuf.FeatureSet.verify(message.features);
                            if (error)
                                return "features." + error;
                        }
                        return null;
                    };
    
                    /**
                     * Creates a FeatureSetEditionDefault message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault} FeatureSetEditionDefault
                     */
                    FeatureSetEditionDefault.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault)
                            return object;
                        var message = new $root.google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault();
                        switch (object.edition) {
                        default:
                            if (typeof object.edition === "number") {
                                message.edition = object.edition;
                                break;
                            }
                            break;
                        case "EDITION_UNKNOWN":
                        case 0:
                            message.edition = 0;
                            break;
                        case "EDITION_PROTO2":
                        case 998:
                            message.edition = 998;
                            break;
                        case "EDITION_PROTO3":
                        case 999:
                            message.edition = 999;
                            break;
                        case "EDITION_2023":
                        case 1000:
                            message.edition = 1000;
                            break;
                        case "EDITION_2024":
                        case 1001:
                            message.edition = 1001;
                            break;
                        case "EDITION_1_TEST_ONLY":
                        case 1:
                            message.edition = 1;
                            break;
                        case "EDITION_2_TEST_ONLY":
                        case 2:
                            message.edition = 2;
                            break;
                        case "EDITION_99997_TEST_ONLY":
                        case 99997:
                            message.edition = 99997;
                            break;
                        case "EDITION_99998_TEST_ONLY":
                        case 99998:
                            message.edition = 99998;
                            break;
                        case "EDITION_99999_TEST_ONLY":
                        case 99999:
                            message.edition = 99999;
                            break;
                        case "EDITION_MAX":
                        case 2147483647:
                            message.edition = 2147483647;
                            break;
                        }
                        if (object.features != null) {
                            if (typeof object.features !== "object")
                                throw TypeError(".google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault.features: object expected");
                            message.features = $root.google.protobuf.FeatureSet.fromObject(object.features);
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a FeatureSetEditionDefault message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault
                     * @static
                     * @param {google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault} message FeatureSetEditionDefault
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    FeatureSetEditionDefault.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults) {
                            object.features = null;
                            object.edition = options.enums === String ? "EDITION_UNKNOWN" : 0;
                        }
                        if (message.features != null && message.hasOwnProperty("features"))
                            object.features = $root.google.protobuf.FeatureSet.toObject(message.features, options);
                        if (message.edition != null && message.hasOwnProperty("edition"))
                            object.edition = options.enums === String ? $root.google.protobuf.Edition[message.edition] === undefined ? message.edition : $root.google.protobuf.Edition[message.edition] : message.edition;
                        return object;
                    };
    
                    /**
                     * Converts this FeatureSetEditionDefault to JSON.
                     * @function toJSON
                     * @memberof google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    FeatureSetEditionDefault.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for FeatureSetEditionDefault
                     * @function getTypeUrl
                     * @memberof google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    FeatureSetEditionDefault.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.protobuf.FeatureSetDefaults.FeatureSetEditionDefault";
                    };
    
                    return FeatureSetEditionDefault;
                })();
    
                return FeatureSetDefaults;
            })();
    
            protobuf.SourceCodeInfo = (function() {
    
                /**
                 * Properties of a SourceCodeInfo.
                 * @memberof google.protobuf
                 * @interface ISourceCodeInfo
                 * @property {Array.<google.protobuf.SourceCodeInfo.ILocation>|null} [location] SourceCodeInfo location
                 */
    
                /**
                 * Constructs a new SourceCodeInfo.
                 * @memberof google.protobuf
                 * @classdesc Represents a SourceCodeInfo.
                 * @implements ISourceCodeInfo
                 * @constructor
                 * @param {google.protobuf.ISourceCodeInfo=} [properties] Properties to set
                 */
                function SourceCodeInfo(properties) {
                    this.location = [];
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * SourceCodeInfo location.
                 * @member {Array.<google.protobuf.SourceCodeInfo.ILocation>} location
                 * @memberof google.protobuf.SourceCodeInfo
                 * @instance
                 */
                SourceCodeInfo.prototype.location = $util.emptyArray;
    
                /**
                 * Creates a new SourceCodeInfo instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.SourceCodeInfo
                 * @static
                 * @param {google.protobuf.ISourceCodeInfo=} [properties] Properties to set
                 * @returns {google.protobuf.SourceCodeInfo} SourceCodeInfo instance
                 */
                SourceCodeInfo.create = function create(properties) {
                    return new SourceCodeInfo(properties);
                };
    
                /**
                 * Encodes the specified SourceCodeInfo message. Does not implicitly {@link google.protobuf.SourceCodeInfo.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.SourceCodeInfo
                 * @static
                 * @param {google.protobuf.ISourceCodeInfo} message SourceCodeInfo message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                SourceCodeInfo.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.location != null && message.location.length)
                        for (var i = 0; i < message.location.length; ++i)
                            $root.google.protobuf.SourceCodeInfo.Location.encode(message.location[i], writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                    return writer;
                };
    
                /**
                 * Encodes the specified SourceCodeInfo message, length delimited. Does not implicitly {@link google.protobuf.SourceCodeInfo.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.SourceCodeInfo
                 * @static
                 * @param {google.protobuf.ISourceCodeInfo} message SourceCodeInfo message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                SourceCodeInfo.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a SourceCodeInfo message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.SourceCodeInfo
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.SourceCodeInfo} SourceCodeInfo
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                SourceCodeInfo.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.SourceCodeInfo();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                if (!(message.location && message.location.length))
                                    message.location = [];
                                message.location.push($root.google.protobuf.SourceCodeInfo.Location.decode(reader, reader.uint32()));
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a SourceCodeInfo message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.SourceCodeInfo
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.SourceCodeInfo} SourceCodeInfo
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                SourceCodeInfo.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a SourceCodeInfo message.
                 * @function verify
                 * @memberof google.protobuf.SourceCodeInfo
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                SourceCodeInfo.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.location != null && message.hasOwnProperty("location")) {
                        if (!Array.isArray(message.location))
                            return "location: array expected";
                        for (var i = 0; i < message.location.length; ++i) {
                            var error = $root.google.protobuf.SourceCodeInfo.Location.verify(message.location[i]);
                            if (error)
                                return "location." + error;
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates a SourceCodeInfo message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.SourceCodeInfo
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.SourceCodeInfo} SourceCodeInfo
                 */
                SourceCodeInfo.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.SourceCodeInfo)
                        return object;
                    var message = new $root.google.protobuf.SourceCodeInfo();
                    if (object.location) {
                        if (!Array.isArray(object.location))
                            throw TypeError(".google.protobuf.SourceCodeInfo.location: array expected");
                        message.location = [];
                        for (var i = 0; i < object.location.length; ++i) {
                            if (typeof object.location[i] !== "object")
                                throw TypeError(".google.protobuf.SourceCodeInfo.location: object expected");
                            message.location[i] = $root.google.protobuf.SourceCodeInfo.Location.fromObject(object.location[i]);
                        }
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from a SourceCodeInfo message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.SourceCodeInfo
                 * @static
                 * @param {google.protobuf.SourceCodeInfo} message SourceCodeInfo
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                SourceCodeInfo.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.arrays || options.defaults)
                        object.location = [];
                    if (message.location && message.location.length) {
                        object.location = [];
                        for (var j = 0; j < message.location.length; ++j)
                            object.location[j] = $root.google.protobuf.SourceCodeInfo.Location.toObject(message.location[j], options);
                    }
                    return object;
                };
    
                /**
                 * Converts this SourceCodeInfo to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.SourceCodeInfo
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                SourceCodeInfo.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for SourceCodeInfo
                 * @function getTypeUrl
                 * @memberof google.protobuf.SourceCodeInfo
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                SourceCodeInfo.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.SourceCodeInfo";
                };
    
                SourceCodeInfo.Location = (function() {
    
                    /**
                     * Properties of a Location.
                     * @memberof google.protobuf.SourceCodeInfo
                     * @interface ILocation
                     * @property {Array.<number>|null} [path] Location path
                     * @property {Array.<number>|null} [span] Location span
                     * @property {string|null} [leadingComments] Location leadingComments
                     * @property {string|null} [trailingComments] Location trailingComments
                     * @property {Array.<string>|null} [leadingDetachedComments] Location leadingDetachedComments
                     */
    
                    /**
                     * Constructs a new Location.
                     * @memberof google.protobuf.SourceCodeInfo
                     * @classdesc Represents a Location.
                     * @implements ILocation
                     * @constructor
                     * @param {google.protobuf.SourceCodeInfo.ILocation=} [properties] Properties to set
                     */
                    function Location(properties) {
                        this.path = [];
                        this.span = [];
                        this.leadingDetachedComments = [];
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * Location path.
                     * @member {Array.<number>} path
                     * @memberof google.protobuf.SourceCodeInfo.Location
                     * @instance
                     */
                    Location.prototype.path = $util.emptyArray;
    
                    /**
                     * Location span.
                     * @member {Array.<number>} span
                     * @memberof google.protobuf.SourceCodeInfo.Location
                     * @instance
                     */
                    Location.prototype.span = $util.emptyArray;
    
                    /**
                     * Location leadingComments.
                     * @member {string} leadingComments
                     * @memberof google.protobuf.SourceCodeInfo.Location
                     * @instance
                     */
                    Location.prototype.leadingComments = "";
    
                    /**
                     * Location trailingComments.
                     * @member {string} trailingComments
                     * @memberof google.protobuf.SourceCodeInfo.Location
                     * @instance
                     */
                    Location.prototype.trailingComments = "";
    
                    /**
                     * Location leadingDetachedComments.
                     * @member {Array.<string>} leadingDetachedComments
                     * @memberof google.protobuf.SourceCodeInfo.Location
                     * @instance
                     */
                    Location.prototype.leadingDetachedComments = $util.emptyArray;
    
                    /**
                     * Creates a new Location instance using the specified properties.
                     * @function create
                     * @memberof google.protobuf.SourceCodeInfo.Location
                     * @static
                     * @param {google.protobuf.SourceCodeInfo.ILocation=} [properties] Properties to set
                     * @returns {google.protobuf.SourceCodeInfo.Location} Location instance
                     */
                    Location.create = function create(properties) {
                        return new Location(properties);
                    };
    
                    /**
                     * Encodes the specified Location message. Does not implicitly {@link google.protobuf.SourceCodeInfo.Location.verify|verify} messages.
                     * @function encode
                     * @memberof google.protobuf.SourceCodeInfo.Location
                     * @static
                     * @param {google.protobuf.SourceCodeInfo.ILocation} message Location message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    Location.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.path != null && message.path.length) {
                            writer.uint32(/* id 1, wireType 2 =*/10).fork();
                            for (var i = 0; i < message.path.length; ++i)
                                writer.int32(message.path[i]);
                            writer.ldelim();
                        }
                        if (message.span != null && message.span.length) {
                            writer.uint32(/* id 2, wireType 2 =*/18).fork();
                            for (var i = 0; i < message.span.length; ++i)
                                writer.int32(message.span[i]);
                            writer.ldelim();
                        }
                        if (message.leadingComments != null && Object.hasOwnProperty.call(message, "leadingComments"))
                            writer.uint32(/* id 3, wireType 2 =*/26).string(message.leadingComments);
                        if (message.trailingComments != null && Object.hasOwnProperty.call(message, "trailingComments"))
                            writer.uint32(/* id 4, wireType 2 =*/34).string(message.trailingComments);
                        if (message.leadingDetachedComments != null && message.leadingDetachedComments.length)
                            for (var i = 0; i < message.leadingDetachedComments.length; ++i)
                                writer.uint32(/* id 6, wireType 2 =*/50).string(message.leadingDetachedComments[i]);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified Location message, length delimited. Does not implicitly {@link google.protobuf.SourceCodeInfo.Location.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.protobuf.SourceCodeInfo.Location
                     * @static
                     * @param {google.protobuf.SourceCodeInfo.ILocation} message Location message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    Location.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a Location message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.protobuf.SourceCodeInfo.Location
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.protobuf.SourceCodeInfo.Location} Location
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    Location.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.SourceCodeInfo.Location();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    if (!(message.path && message.path.length))
                                        message.path = [];
                                    if ((tag & 7) === 2) {
                                        var end2 = reader.uint32() + reader.pos;
                                        while (reader.pos < end2)
                                            message.path.push(reader.int32());
                                    } else
                                        message.path.push(reader.int32());
                                    break;
                                }
                            case 2: {
                                    if (!(message.span && message.span.length))
                                        message.span = [];
                                    if ((tag & 7) === 2) {
                                        var end2 = reader.uint32() + reader.pos;
                                        while (reader.pos < end2)
                                            message.span.push(reader.int32());
                                    } else
                                        message.span.push(reader.int32());
                                    break;
                                }
                            case 3: {
                                    message.leadingComments = reader.string();
                                    break;
                                }
                            case 4: {
                                    message.trailingComments = reader.string();
                                    break;
                                }
                            case 6: {
                                    if (!(message.leadingDetachedComments && message.leadingDetachedComments.length))
                                        message.leadingDetachedComments = [];
                                    message.leadingDetachedComments.push(reader.string());
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a Location message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.protobuf.SourceCodeInfo.Location
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.protobuf.SourceCodeInfo.Location} Location
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    Location.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a Location message.
                     * @function verify
                     * @memberof google.protobuf.SourceCodeInfo.Location
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    Location.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.path != null && message.hasOwnProperty("path")) {
                            if (!Array.isArray(message.path))
                                return "path: array expected";
                            for (var i = 0; i < message.path.length; ++i)
                                if (!$util.isInteger(message.path[i]))
                                    return "path: integer[] expected";
                        }
                        if (message.span != null && message.hasOwnProperty("span")) {
                            if (!Array.isArray(message.span))
                                return "span: array expected";
                            for (var i = 0; i < message.span.length; ++i)
                                if (!$util.isInteger(message.span[i]))
                                    return "span: integer[] expected";
                        }
                        if (message.leadingComments != null && message.hasOwnProperty("leadingComments"))
                            if (!$util.isString(message.leadingComments))
                                return "leadingComments: string expected";
                        if (message.trailingComments != null && message.hasOwnProperty("trailingComments"))
                            if (!$util.isString(message.trailingComments))
                                return "trailingComments: string expected";
                        if (message.leadingDetachedComments != null && message.hasOwnProperty("leadingDetachedComments")) {
                            if (!Array.isArray(message.leadingDetachedComments))
                                return "leadingDetachedComments: array expected";
                            for (var i = 0; i < message.leadingDetachedComments.length; ++i)
                                if (!$util.isString(message.leadingDetachedComments[i]))
                                    return "leadingDetachedComments: string[] expected";
                        }
                        return null;
                    };
    
                    /**
                     * Creates a Location message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.protobuf.SourceCodeInfo.Location
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.protobuf.SourceCodeInfo.Location} Location
                     */
                    Location.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.protobuf.SourceCodeInfo.Location)
                            return object;
                        var message = new $root.google.protobuf.SourceCodeInfo.Location();
                        if (object.path) {
                            if (!Array.isArray(object.path))
                                throw TypeError(".google.protobuf.SourceCodeInfo.Location.path: array expected");
                            message.path = [];
                            for (var i = 0; i < object.path.length; ++i)
                                message.path[i] = object.path[i] | 0;
                        }
                        if (object.span) {
                            if (!Array.isArray(object.span))
                                throw TypeError(".google.protobuf.SourceCodeInfo.Location.span: array expected");
                            message.span = [];
                            for (var i = 0; i < object.span.length; ++i)
                                message.span[i] = object.span[i] | 0;
                        }
                        if (object.leadingComments != null)
                            message.leadingComments = String(object.leadingComments);
                        if (object.trailingComments != null)
                            message.trailingComments = String(object.trailingComments);
                        if (object.leadingDetachedComments) {
                            if (!Array.isArray(object.leadingDetachedComments))
                                throw TypeError(".google.protobuf.SourceCodeInfo.Location.leadingDetachedComments: array expected");
                            message.leadingDetachedComments = [];
                            for (var i = 0; i < object.leadingDetachedComments.length; ++i)
                                message.leadingDetachedComments[i] = String(object.leadingDetachedComments[i]);
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a Location message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.protobuf.SourceCodeInfo.Location
                     * @static
                     * @param {google.protobuf.SourceCodeInfo.Location} message Location
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    Location.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.arrays || options.defaults) {
                            object.path = [];
                            object.span = [];
                            object.leadingDetachedComments = [];
                        }
                        if (options.defaults) {
                            object.leadingComments = "";
                            object.trailingComments = "";
                        }
                        if (message.path && message.path.length) {
                            object.path = [];
                            for (var j = 0; j < message.path.length; ++j)
                                object.path[j] = message.path[j];
                        }
                        if (message.span && message.span.length) {
                            object.span = [];
                            for (var j = 0; j < message.span.length; ++j)
                                object.span[j] = message.span[j];
                        }
                        if (message.leadingComments != null && message.hasOwnProperty("leadingComments"))
                            object.leadingComments = message.leadingComments;
                        if (message.trailingComments != null && message.hasOwnProperty("trailingComments"))
                            object.trailingComments = message.trailingComments;
                        if (message.leadingDetachedComments && message.leadingDetachedComments.length) {
                            object.leadingDetachedComments = [];
                            for (var j = 0; j < message.leadingDetachedComments.length; ++j)
                                object.leadingDetachedComments[j] = message.leadingDetachedComments[j];
                        }
                        return object;
                    };
    
                    /**
                     * Converts this Location to JSON.
                     * @function toJSON
                     * @memberof google.protobuf.SourceCodeInfo.Location
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    Location.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for Location
                     * @function getTypeUrl
                     * @memberof google.protobuf.SourceCodeInfo.Location
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    Location.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.protobuf.SourceCodeInfo.Location";
                    };
    
                    return Location;
                })();
    
                return SourceCodeInfo;
            })();
    
            protobuf.GeneratedCodeInfo = (function() {
    
                /**
                 * Properties of a GeneratedCodeInfo.
                 * @memberof google.protobuf
                 * @interface IGeneratedCodeInfo
                 * @property {Array.<google.protobuf.GeneratedCodeInfo.IAnnotation>|null} [annotation] GeneratedCodeInfo annotation
                 */
    
                /**
                 * Constructs a new GeneratedCodeInfo.
                 * @memberof google.protobuf
                 * @classdesc Represents a GeneratedCodeInfo.
                 * @implements IGeneratedCodeInfo
                 * @constructor
                 * @param {google.protobuf.IGeneratedCodeInfo=} [properties] Properties to set
                 */
                function GeneratedCodeInfo(properties) {
                    this.annotation = [];
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * GeneratedCodeInfo annotation.
                 * @member {Array.<google.protobuf.GeneratedCodeInfo.IAnnotation>} annotation
                 * @memberof google.protobuf.GeneratedCodeInfo
                 * @instance
                 */
                GeneratedCodeInfo.prototype.annotation = $util.emptyArray;
    
                /**
                 * Creates a new GeneratedCodeInfo instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.GeneratedCodeInfo
                 * @static
                 * @param {google.protobuf.IGeneratedCodeInfo=} [properties] Properties to set
                 * @returns {google.protobuf.GeneratedCodeInfo} GeneratedCodeInfo instance
                 */
                GeneratedCodeInfo.create = function create(properties) {
                    return new GeneratedCodeInfo(properties);
                };
    
                /**
                 * Encodes the specified GeneratedCodeInfo message. Does not implicitly {@link google.protobuf.GeneratedCodeInfo.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.GeneratedCodeInfo
                 * @static
                 * @param {google.protobuf.IGeneratedCodeInfo} message GeneratedCodeInfo message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                GeneratedCodeInfo.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.annotation != null && message.annotation.length)
                        for (var i = 0; i < message.annotation.length; ++i)
                            $root.google.protobuf.GeneratedCodeInfo.Annotation.encode(message.annotation[i], writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                    return writer;
                };
    
                /**
                 * Encodes the specified GeneratedCodeInfo message, length delimited. Does not implicitly {@link google.protobuf.GeneratedCodeInfo.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.GeneratedCodeInfo
                 * @static
                 * @param {google.protobuf.IGeneratedCodeInfo} message GeneratedCodeInfo message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                GeneratedCodeInfo.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a GeneratedCodeInfo message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.GeneratedCodeInfo
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.GeneratedCodeInfo} GeneratedCodeInfo
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                GeneratedCodeInfo.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.GeneratedCodeInfo();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                if (!(message.annotation && message.annotation.length))
                                    message.annotation = [];
                                message.annotation.push($root.google.protobuf.GeneratedCodeInfo.Annotation.decode(reader, reader.uint32()));
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a GeneratedCodeInfo message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.GeneratedCodeInfo
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.GeneratedCodeInfo} GeneratedCodeInfo
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                GeneratedCodeInfo.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a GeneratedCodeInfo message.
                 * @function verify
                 * @memberof google.protobuf.GeneratedCodeInfo
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                GeneratedCodeInfo.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.annotation != null && message.hasOwnProperty("annotation")) {
                        if (!Array.isArray(message.annotation))
                            return "annotation: array expected";
                        for (var i = 0; i < message.annotation.length; ++i) {
                            var error = $root.google.protobuf.GeneratedCodeInfo.Annotation.verify(message.annotation[i]);
                            if (error)
                                return "annotation." + error;
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates a GeneratedCodeInfo message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.GeneratedCodeInfo
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.GeneratedCodeInfo} GeneratedCodeInfo
                 */
                GeneratedCodeInfo.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.GeneratedCodeInfo)
                        return object;
                    var message = new $root.google.protobuf.GeneratedCodeInfo();
                    if (object.annotation) {
                        if (!Array.isArray(object.annotation))
                            throw TypeError(".google.protobuf.GeneratedCodeInfo.annotation: array expected");
                        message.annotation = [];
                        for (var i = 0; i < object.annotation.length; ++i) {
                            if (typeof object.annotation[i] !== "object")
                                throw TypeError(".google.protobuf.GeneratedCodeInfo.annotation: object expected");
                            message.annotation[i] = $root.google.protobuf.GeneratedCodeInfo.Annotation.fromObject(object.annotation[i]);
                        }
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from a GeneratedCodeInfo message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.GeneratedCodeInfo
                 * @static
                 * @param {google.protobuf.GeneratedCodeInfo} message GeneratedCodeInfo
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                GeneratedCodeInfo.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.arrays || options.defaults)
                        object.annotation = [];
                    if (message.annotation && message.annotation.length) {
                        object.annotation = [];
                        for (var j = 0; j < message.annotation.length; ++j)
                            object.annotation[j] = $root.google.protobuf.GeneratedCodeInfo.Annotation.toObject(message.annotation[j], options);
                    }
                    return object;
                };
    
                /**
                 * Converts this GeneratedCodeInfo to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.GeneratedCodeInfo
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                GeneratedCodeInfo.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for GeneratedCodeInfo
                 * @function getTypeUrl
                 * @memberof google.protobuf.GeneratedCodeInfo
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                GeneratedCodeInfo.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.GeneratedCodeInfo";
                };
    
                GeneratedCodeInfo.Annotation = (function() {
    
                    /**
                     * Properties of an Annotation.
                     * @memberof google.protobuf.GeneratedCodeInfo
                     * @interface IAnnotation
                     * @property {Array.<number>|null} [path] Annotation path
                     * @property {string|null} [sourceFile] Annotation sourceFile
                     * @property {number|null} [begin] Annotation begin
                     * @property {number|null} [end] Annotation end
                     * @property {google.protobuf.GeneratedCodeInfo.Annotation.Semantic|null} [semantic] Annotation semantic
                     */
    
                    /**
                     * Constructs a new Annotation.
                     * @memberof google.protobuf.GeneratedCodeInfo
                     * @classdesc Represents an Annotation.
                     * @implements IAnnotation
                     * @constructor
                     * @param {google.protobuf.GeneratedCodeInfo.IAnnotation=} [properties] Properties to set
                     */
                    function Annotation(properties) {
                        this.path = [];
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * Annotation path.
                     * @member {Array.<number>} path
                     * @memberof google.protobuf.GeneratedCodeInfo.Annotation
                     * @instance
                     */
                    Annotation.prototype.path = $util.emptyArray;
    
                    /**
                     * Annotation sourceFile.
                     * @member {string} sourceFile
                     * @memberof google.protobuf.GeneratedCodeInfo.Annotation
                     * @instance
                     */
                    Annotation.prototype.sourceFile = "";
    
                    /**
                     * Annotation begin.
                     * @member {number} begin
                     * @memberof google.protobuf.GeneratedCodeInfo.Annotation
                     * @instance
                     */
                    Annotation.prototype.begin = 0;
    
                    /**
                     * Annotation end.
                     * @member {number} end
                     * @memberof google.protobuf.GeneratedCodeInfo.Annotation
                     * @instance
                     */
                    Annotation.prototype.end = 0;
    
                    /**
                     * Annotation semantic.
                     * @member {google.protobuf.GeneratedCodeInfo.Annotation.Semantic} semantic
                     * @memberof google.protobuf.GeneratedCodeInfo.Annotation
                     * @instance
                     */
                    Annotation.prototype.semantic = 0;
    
                    /**
                     * Creates a new Annotation instance using the specified properties.
                     * @function create
                     * @memberof google.protobuf.GeneratedCodeInfo.Annotation
                     * @static
                     * @param {google.protobuf.GeneratedCodeInfo.IAnnotation=} [properties] Properties to set
                     * @returns {google.protobuf.GeneratedCodeInfo.Annotation} Annotation instance
                     */
                    Annotation.create = function create(properties) {
                        return new Annotation(properties);
                    };
    
                    /**
                     * Encodes the specified Annotation message. Does not implicitly {@link google.protobuf.GeneratedCodeInfo.Annotation.verify|verify} messages.
                     * @function encode
                     * @memberof google.protobuf.GeneratedCodeInfo.Annotation
                     * @static
                     * @param {google.protobuf.GeneratedCodeInfo.IAnnotation} message Annotation message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    Annotation.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.path != null && message.path.length) {
                            writer.uint32(/* id 1, wireType 2 =*/10).fork();
                            for (var i = 0; i < message.path.length; ++i)
                                writer.int32(message.path[i]);
                            writer.ldelim();
                        }
                        if (message.sourceFile != null && Object.hasOwnProperty.call(message, "sourceFile"))
                            writer.uint32(/* id 2, wireType 2 =*/18).string(message.sourceFile);
                        if (message.begin != null && Object.hasOwnProperty.call(message, "begin"))
                            writer.uint32(/* id 3, wireType 0 =*/24).int32(message.begin);
                        if (message.end != null && Object.hasOwnProperty.call(message, "end"))
                            writer.uint32(/* id 4, wireType 0 =*/32).int32(message.end);
                        if (message.semantic != null && Object.hasOwnProperty.call(message, "semantic"))
                            writer.uint32(/* id 5, wireType 0 =*/40).int32(message.semantic);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified Annotation message, length delimited. Does not implicitly {@link google.protobuf.GeneratedCodeInfo.Annotation.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.protobuf.GeneratedCodeInfo.Annotation
                     * @static
                     * @param {google.protobuf.GeneratedCodeInfo.IAnnotation} message Annotation message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    Annotation.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes an Annotation message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.protobuf.GeneratedCodeInfo.Annotation
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.protobuf.GeneratedCodeInfo.Annotation} Annotation
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    Annotation.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.GeneratedCodeInfo.Annotation();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    if (!(message.path && message.path.length))
                                        message.path = [];
                                    if ((tag & 7) === 2) {
                                        var end2 = reader.uint32() + reader.pos;
                                        while (reader.pos < end2)
                                            message.path.push(reader.int32());
                                    } else
                                        message.path.push(reader.int32());
                                    break;
                                }
                            case 2: {
                                    message.sourceFile = reader.string();
                                    break;
                                }
                            case 3: {
                                    message.begin = reader.int32();
                                    break;
                                }
                            case 4: {
                                    message.end = reader.int32();
                                    break;
                                }
                            case 5: {
                                    message.semantic = reader.int32();
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes an Annotation message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.protobuf.GeneratedCodeInfo.Annotation
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.protobuf.GeneratedCodeInfo.Annotation} Annotation
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    Annotation.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies an Annotation message.
                     * @function verify
                     * @memberof google.protobuf.GeneratedCodeInfo.Annotation
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    Annotation.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.path != null && message.hasOwnProperty("path")) {
                            if (!Array.isArray(message.path))
                                return "path: array expected";
                            for (var i = 0; i < message.path.length; ++i)
                                if (!$util.isInteger(message.path[i]))
                                    return "path: integer[] expected";
                        }
                        if (message.sourceFile != null && message.hasOwnProperty("sourceFile"))
                            if (!$util.isString(message.sourceFile))
                                return "sourceFile: string expected";
                        if (message.begin != null && message.hasOwnProperty("begin"))
                            if (!$util.isInteger(message.begin))
                                return "begin: integer expected";
                        if (message.end != null && message.hasOwnProperty("end"))
                            if (!$util.isInteger(message.end))
                                return "end: integer expected";
                        if (message.semantic != null && message.hasOwnProperty("semantic"))
                            switch (message.semantic) {
                            default:
                                return "semantic: enum value expected";
                            case 0:
                            case 1:
                            case 2:
                                break;
                            }
                        return null;
                    };
    
                    /**
                     * Creates an Annotation message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.protobuf.GeneratedCodeInfo.Annotation
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.protobuf.GeneratedCodeInfo.Annotation} Annotation
                     */
                    Annotation.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.protobuf.GeneratedCodeInfo.Annotation)
                            return object;
                        var message = new $root.google.protobuf.GeneratedCodeInfo.Annotation();
                        if (object.path) {
                            if (!Array.isArray(object.path))
                                throw TypeError(".google.protobuf.GeneratedCodeInfo.Annotation.path: array expected");
                            message.path = [];
                            for (var i = 0; i < object.path.length; ++i)
                                message.path[i] = object.path[i] | 0;
                        }
                        if (object.sourceFile != null)
                            message.sourceFile = String(object.sourceFile);
                        if (object.begin != null)
                            message.begin = object.begin | 0;
                        if (object.end != null)
                            message.end = object.end | 0;
                        switch (object.semantic) {
                        default:
                            if (typeof object.semantic === "number") {
                                message.semantic = object.semantic;
                                break;
                            }
                            break;
                        case "NONE":
                        case 0:
                            message.semantic = 0;
                            break;
                        case "SET":
                        case 1:
                            message.semantic = 1;
                            break;
                        case "ALIAS":
                        case 2:
                            message.semantic = 2;
                            break;
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from an Annotation message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.protobuf.GeneratedCodeInfo.Annotation
                     * @static
                     * @param {google.protobuf.GeneratedCodeInfo.Annotation} message Annotation
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    Annotation.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.arrays || options.defaults)
                            object.path = [];
                        if (options.defaults) {
                            object.sourceFile = "";
                            object.begin = 0;
                            object.end = 0;
                            object.semantic = options.enums === String ? "NONE" : 0;
                        }
                        if (message.path && message.path.length) {
                            object.path = [];
                            for (var j = 0; j < message.path.length; ++j)
                                object.path[j] = message.path[j];
                        }
                        if (message.sourceFile != null && message.hasOwnProperty("sourceFile"))
                            object.sourceFile = message.sourceFile;
                        if (message.begin != null && message.hasOwnProperty("begin"))
                            object.begin = message.begin;
                        if (message.end != null && message.hasOwnProperty("end"))
                            object.end = message.end;
                        if (message.semantic != null && message.hasOwnProperty("semantic"))
                            object.semantic = options.enums === String ? $root.google.protobuf.GeneratedCodeInfo.Annotation.Semantic[message.semantic] === undefined ? message.semantic : $root.google.protobuf.GeneratedCodeInfo.Annotation.Semantic[message.semantic] : message.semantic;
                        return object;
                    };
    
                    /**
                     * Converts this Annotation to JSON.
                     * @function toJSON
                     * @memberof google.protobuf.GeneratedCodeInfo.Annotation
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    Annotation.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for Annotation
                     * @function getTypeUrl
                     * @memberof google.protobuf.GeneratedCodeInfo.Annotation
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    Annotation.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.protobuf.GeneratedCodeInfo.Annotation";
                    };
    
                    /**
                     * Semantic enum.
                     * @name google.protobuf.GeneratedCodeInfo.Annotation.Semantic
                     * @enum {number}
                     * @property {number} NONE=0 NONE value
                     * @property {number} SET=1 SET value
                     * @property {number} ALIAS=2 ALIAS value
                     */
                    Annotation.Semantic = (function() {
                        var valuesById = {}, values = Object.create(valuesById);
                        values[valuesById[0] = "NONE"] = 0;
                        values[valuesById[1] = "SET"] = 1;
                        values[valuesById[2] = "ALIAS"] = 2;
                        return values;
                    })();
    
                    return Annotation;
                })();
    
                return GeneratedCodeInfo;
            })();
    
            protobuf.Duration = (function() {
    
                /**
                 * Properties of a Duration.
                 * @memberof google.protobuf
                 * @interface IDuration
                 * @property {number|Long|null} [seconds] Duration seconds
                 * @property {number|null} [nanos] Duration nanos
                 */
    
                /**
                 * Constructs a new Duration.
                 * @memberof google.protobuf
                 * @classdesc Represents a Duration.
                 * @implements IDuration
                 * @constructor
                 * @param {google.protobuf.IDuration=} [properties] Properties to set
                 */
                function Duration(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * Duration seconds.
                 * @member {number|Long} seconds
                 * @memberof google.protobuf.Duration
                 * @instance
                 */
                Duration.prototype.seconds = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                /**
                 * Duration nanos.
                 * @member {number} nanos
                 * @memberof google.protobuf.Duration
                 * @instance
                 */
                Duration.prototype.nanos = 0;
    
                /**
                 * Creates a new Duration instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.Duration
                 * @static
                 * @param {google.protobuf.IDuration=} [properties] Properties to set
                 * @returns {google.protobuf.Duration} Duration instance
                 */
                Duration.create = function create(properties) {
                    return new Duration(properties);
                };
    
                /**
                 * Encodes the specified Duration message. Does not implicitly {@link google.protobuf.Duration.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.Duration
                 * @static
                 * @param {google.protobuf.IDuration} message Duration message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Duration.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.seconds != null && Object.hasOwnProperty.call(message, "seconds"))
                        writer.uint32(/* id 1, wireType 0 =*/8).int64(message.seconds);
                    if (message.nanos != null && Object.hasOwnProperty.call(message, "nanos"))
                        writer.uint32(/* id 2, wireType 0 =*/16).int32(message.nanos);
                    return writer;
                };
    
                /**
                 * Encodes the specified Duration message, length delimited. Does not implicitly {@link google.protobuf.Duration.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.Duration
                 * @static
                 * @param {google.protobuf.IDuration} message Duration message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Duration.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a Duration message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.Duration
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.Duration} Duration
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Duration.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.Duration();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.seconds = reader.int64();
                                break;
                            }
                        case 2: {
                                message.nanos = reader.int32();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a Duration message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.Duration
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.Duration} Duration
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Duration.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a Duration message.
                 * @function verify
                 * @memberof google.protobuf.Duration
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                Duration.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.seconds != null && message.hasOwnProperty("seconds"))
                        if (!$util.isInteger(message.seconds) && !(message.seconds && $util.isInteger(message.seconds.low) && $util.isInteger(message.seconds.high)))
                            return "seconds: integer|Long expected";
                    if (message.nanos != null && message.hasOwnProperty("nanos"))
                        if (!$util.isInteger(message.nanos))
                            return "nanos: integer expected";
                    return null;
                };
    
                /**
                 * Creates a Duration message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.Duration
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.Duration} Duration
                 */
                Duration.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.Duration)
                        return object;
                    var message = new $root.google.protobuf.Duration();
                    if (object.seconds != null)
                        if ($util.Long)
                            (message.seconds = $util.Long.fromValue(object.seconds)).unsigned = false;
                        else if (typeof object.seconds === "string")
                            message.seconds = parseInt(object.seconds, 10);
                        else if (typeof object.seconds === "number")
                            message.seconds = object.seconds;
                        else if (typeof object.seconds === "object")
                            message.seconds = new $util.LongBits(object.seconds.low >>> 0, object.seconds.high >>> 0).toNumber();
                    if (object.nanos != null)
                        message.nanos = object.nanos | 0;
                    return message;
                };
    
                /**
                 * Creates a plain object from a Duration message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.Duration
                 * @static
                 * @param {google.protobuf.Duration} message Duration
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                Duration.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults) {
                        if ($util.Long) {
                            var long = new $util.Long(0, 0, false);
                            object.seconds = options.longs === String ? long.toString() : options.longs === Number ? long.toNumber() : long;
                        } else
                            object.seconds = options.longs === String ? "0" : 0;
                        object.nanos = 0;
                    }
                    if (message.seconds != null && message.hasOwnProperty("seconds"))
                        if (typeof message.seconds === "number")
                            object.seconds = options.longs === String ? String(message.seconds) : message.seconds;
                        else
                            object.seconds = options.longs === String ? $util.Long.prototype.toString.call(message.seconds) : options.longs === Number ? new $util.LongBits(message.seconds.low >>> 0, message.seconds.high >>> 0).toNumber() : message.seconds;
                    if (message.nanos != null && message.hasOwnProperty("nanos"))
                        object.nanos = message.nanos;
                    return object;
                };
    
                /**
                 * Converts this Duration to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.Duration
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                Duration.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for Duration
                 * @function getTypeUrl
                 * @memberof google.protobuf.Duration
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                Duration.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.Duration";
                };
    
                return Duration;
            })();
    
            protobuf.Timestamp = (function() {
    
                /**
                 * Properties of a Timestamp.
                 * @memberof google.protobuf
                 * @interface ITimestamp
                 * @property {number|Long|null} [seconds] Timestamp seconds
                 * @property {number|null} [nanos] Timestamp nanos
                 */
    
                /**
                 * Constructs a new Timestamp.
                 * @memberof google.protobuf
                 * @classdesc Represents a Timestamp.
                 * @implements ITimestamp
                 * @constructor
                 * @param {google.protobuf.ITimestamp=} [properties] Properties to set
                 */
                function Timestamp(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * Timestamp seconds.
                 * @member {number|Long} seconds
                 * @memberof google.protobuf.Timestamp
                 * @instance
                 */
                Timestamp.prototype.seconds = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                /**
                 * Timestamp nanos.
                 * @member {number} nanos
                 * @memberof google.protobuf.Timestamp
                 * @instance
                 */
                Timestamp.prototype.nanos = 0;
    
                /**
                 * Creates a new Timestamp instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.Timestamp
                 * @static
                 * @param {google.protobuf.ITimestamp=} [properties] Properties to set
                 * @returns {google.protobuf.Timestamp} Timestamp instance
                 */
                Timestamp.create = function create(properties) {
                    return new Timestamp(properties);
                };
    
                /**
                 * Encodes the specified Timestamp message. Does not implicitly {@link google.protobuf.Timestamp.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.Timestamp
                 * @static
                 * @param {google.protobuf.ITimestamp} message Timestamp message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Timestamp.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.seconds != null && Object.hasOwnProperty.call(message, "seconds"))
                        writer.uint32(/* id 1, wireType 0 =*/8).int64(message.seconds);
                    if (message.nanos != null && Object.hasOwnProperty.call(message, "nanos"))
                        writer.uint32(/* id 2, wireType 0 =*/16).int32(message.nanos);
                    return writer;
                };
    
                /**
                 * Encodes the specified Timestamp message, length delimited. Does not implicitly {@link google.protobuf.Timestamp.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.Timestamp
                 * @static
                 * @param {google.protobuf.ITimestamp} message Timestamp message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Timestamp.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a Timestamp message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.Timestamp
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.Timestamp} Timestamp
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Timestamp.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.Timestamp();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.seconds = reader.int64();
                                break;
                            }
                        case 2: {
                                message.nanos = reader.int32();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a Timestamp message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.Timestamp
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.Timestamp} Timestamp
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Timestamp.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a Timestamp message.
                 * @function verify
                 * @memberof google.protobuf.Timestamp
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                Timestamp.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.seconds != null && message.hasOwnProperty("seconds"))
                        if (!$util.isInteger(message.seconds) && !(message.seconds && $util.isInteger(message.seconds.low) && $util.isInteger(message.seconds.high)))
                            return "seconds: integer|Long expected";
                    if (message.nanos != null && message.hasOwnProperty("nanos"))
                        if (!$util.isInteger(message.nanos))
                            return "nanos: integer expected";
                    return null;
                };
    
                /**
                 * Creates a Timestamp message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.Timestamp
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.Timestamp} Timestamp
                 */
                Timestamp.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.Timestamp)
                        return object;
                    var message = new $root.google.protobuf.Timestamp();
                    if (object.seconds != null)
                        if ($util.Long)
                            (message.seconds = $util.Long.fromValue(object.seconds)).unsigned = false;
                        else if (typeof object.seconds === "string")
                            message.seconds = parseInt(object.seconds, 10);
                        else if (typeof object.seconds === "number")
                            message.seconds = object.seconds;
                        else if (typeof object.seconds === "object")
                            message.seconds = new $util.LongBits(object.seconds.low >>> 0, object.seconds.high >>> 0).toNumber();
                    if (object.nanos != null)
                        message.nanos = object.nanos | 0;
                    return message;
                };
    
                /**
                 * Creates a plain object from a Timestamp message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.Timestamp
                 * @static
                 * @param {google.protobuf.Timestamp} message Timestamp
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                Timestamp.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults) {
                        if ($util.Long) {
                            var long = new $util.Long(0, 0, false);
                            object.seconds = options.longs === String ? long.toString() : options.longs === Number ? long.toNumber() : long;
                        } else
                            object.seconds = options.longs === String ? "0" : 0;
                        object.nanos = 0;
                    }
                    if (message.seconds != null && message.hasOwnProperty("seconds"))
                        if (typeof message.seconds === "number")
                            object.seconds = options.longs === String ? String(message.seconds) : message.seconds;
                        else
                            object.seconds = options.longs === String ? $util.Long.prototype.toString.call(message.seconds) : options.longs === Number ? new $util.LongBits(message.seconds.low >>> 0, message.seconds.high >>> 0).toNumber() : message.seconds;
                    if (message.nanos != null && message.hasOwnProperty("nanos"))
                        object.nanos = message.nanos;
                    return object;
                };
    
                /**
                 * Converts this Timestamp to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.Timestamp
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                Timestamp.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for Timestamp
                 * @function getTypeUrl
                 * @memberof google.protobuf.Timestamp
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                Timestamp.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.Timestamp";
                };
    
                return Timestamp;
            })();
    
            protobuf.FieldMask = (function() {
    
                /**
                 * Properties of a FieldMask.
                 * @memberof google.protobuf
                 * @interface IFieldMask
                 * @property {Array.<string>|null} [paths] FieldMask paths
                 */
    
                /**
                 * Constructs a new FieldMask.
                 * @memberof google.protobuf
                 * @classdesc Represents a FieldMask.
                 * @implements IFieldMask
                 * @constructor
                 * @param {google.protobuf.IFieldMask=} [properties] Properties to set
                 */
                function FieldMask(properties) {
                    this.paths = [];
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * FieldMask paths.
                 * @member {Array.<string>} paths
                 * @memberof google.protobuf.FieldMask
                 * @instance
                 */
                FieldMask.prototype.paths = $util.emptyArray;
    
                /**
                 * Creates a new FieldMask instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.FieldMask
                 * @static
                 * @param {google.protobuf.IFieldMask=} [properties] Properties to set
                 * @returns {google.protobuf.FieldMask} FieldMask instance
                 */
                FieldMask.create = function create(properties) {
                    return new FieldMask(properties);
                };
    
                /**
                 * Encodes the specified FieldMask message. Does not implicitly {@link google.protobuf.FieldMask.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.FieldMask
                 * @static
                 * @param {google.protobuf.IFieldMask} message FieldMask message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FieldMask.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.paths != null && message.paths.length)
                        for (var i = 0; i < message.paths.length; ++i)
                            writer.uint32(/* id 1, wireType 2 =*/10).string(message.paths[i]);
                    return writer;
                };
    
                /**
                 * Encodes the specified FieldMask message, length delimited. Does not implicitly {@link google.protobuf.FieldMask.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.FieldMask
                 * @static
                 * @param {google.protobuf.IFieldMask} message FieldMask message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FieldMask.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a FieldMask message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.FieldMask
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.FieldMask} FieldMask
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FieldMask.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.FieldMask();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                if (!(message.paths && message.paths.length))
                                    message.paths = [];
                                message.paths.push(reader.string());
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a FieldMask message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.FieldMask
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.FieldMask} FieldMask
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FieldMask.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a FieldMask message.
                 * @function verify
                 * @memberof google.protobuf.FieldMask
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                FieldMask.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.paths != null && message.hasOwnProperty("paths")) {
                        if (!Array.isArray(message.paths))
                            return "paths: array expected";
                        for (var i = 0; i < message.paths.length; ++i)
                            if (!$util.isString(message.paths[i]))
                                return "paths: string[] expected";
                    }
                    return null;
                };
    
                /**
                 * Creates a FieldMask message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.FieldMask
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.FieldMask} FieldMask
                 */
                FieldMask.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.FieldMask)
                        return object;
                    var message = new $root.google.protobuf.FieldMask();
                    if (object.paths) {
                        if (!Array.isArray(object.paths))
                            throw TypeError(".google.protobuf.FieldMask.paths: array expected");
                        message.paths = [];
                        for (var i = 0; i < object.paths.length; ++i)
                            message.paths[i] = String(object.paths[i]);
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from a FieldMask message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.FieldMask
                 * @static
                 * @param {google.protobuf.FieldMask} message FieldMask
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                FieldMask.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.arrays || options.defaults)
                        object.paths = [];
                    if (message.paths && message.paths.length) {
                        object.paths = [];
                        for (var j = 0; j < message.paths.length; ++j)
                            object.paths[j] = message.paths[j];
                    }
                    return object;
                };
    
                /**
                 * Converts this FieldMask to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.FieldMask
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                FieldMask.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for FieldMask
                 * @function getTypeUrl
                 * @memberof google.protobuf.FieldMask
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                FieldMask.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.FieldMask";
                };
    
                return FieldMask;
            })();
    
            protobuf.Any = (function() {
    
                /**
                 * Properties of an Any.
                 * @memberof google.protobuf
                 * @interface IAny
                 * @property {string|null} [type_url] Any type_url
                 * @property {Uint8Array|null} [value] Any value
                 */
    
                /**
                 * Constructs a new Any.
                 * @memberof google.protobuf
                 * @classdesc Represents an Any.
                 * @implements IAny
                 * @constructor
                 * @param {google.protobuf.IAny=} [properties] Properties to set
                 */
                function Any(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * Any type_url.
                 * @member {string} type_url
                 * @memberof google.protobuf.Any
                 * @instance
                 */
                Any.prototype.type_url = "";
    
                /**
                 * Any value.
                 * @member {Uint8Array} value
                 * @memberof google.protobuf.Any
                 * @instance
                 */
                Any.prototype.value = $util.newBuffer([]);
    
                /**
                 * Creates a new Any instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.Any
                 * @static
                 * @param {google.protobuf.IAny=} [properties] Properties to set
                 * @returns {google.protobuf.Any} Any instance
                 */
                Any.create = function create(properties) {
                    return new Any(properties);
                };
    
                /**
                 * Encodes the specified Any message. Does not implicitly {@link google.protobuf.Any.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.Any
                 * @static
                 * @param {google.protobuf.IAny} message Any message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Any.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.type_url != null && Object.hasOwnProperty.call(message, "type_url"))
                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.type_url);
                    if (message.value != null && Object.hasOwnProperty.call(message, "value"))
                        writer.uint32(/* id 2, wireType 2 =*/18).bytes(message.value);
                    return writer;
                };
    
                /**
                 * Encodes the specified Any message, length delimited. Does not implicitly {@link google.protobuf.Any.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.Any
                 * @static
                 * @param {google.protobuf.IAny} message Any message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Any.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes an Any message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.Any
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.Any} Any
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Any.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.Any();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.type_url = reader.string();
                                break;
                            }
                        case 2: {
                                message.value = reader.bytes();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes an Any message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.Any
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.Any} Any
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Any.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies an Any message.
                 * @function verify
                 * @memberof google.protobuf.Any
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                Any.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.type_url != null && message.hasOwnProperty("type_url"))
                        if (!$util.isString(message.type_url))
                            return "type_url: string expected";
                    if (message.value != null && message.hasOwnProperty("value"))
                        if (!(message.value && typeof message.value.length === "number" || $util.isString(message.value)))
                            return "value: buffer expected";
                    return null;
                };
    
                /**
                 * Creates an Any message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.Any
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.Any} Any
                 */
                Any.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.Any)
                        return object;
                    var message = new $root.google.protobuf.Any();
                    if (object.type_url != null)
                        message.type_url = String(object.type_url);
                    if (object.value != null)
                        if (typeof object.value === "string")
                            $util.base64.decode(object.value, message.value = $util.newBuffer($util.base64.length(object.value)), 0);
                        else if (object.value.length >= 0)
                            message.value = object.value;
                    return message;
                };
    
                /**
                 * Creates a plain object from an Any message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.Any
                 * @static
                 * @param {google.protobuf.Any} message Any
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                Any.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults) {
                        object.type_url = "";
                        if (options.bytes === String)
                            object.value = "";
                        else {
                            object.value = [];
                            if (options.bytes !== Array)
                                object.value = $util.newBuffer(object.value);
                        }
                    }
                    if (message.type_url != null && message.hasOwnProperty("type_url"))
                        object.type_url = message.type_url;
                    if (message.value != null && message.hasOwnProperty("value"))
                        object.value = options.bytes === String ? $util.base64.encode(message.value, 0, message.value.length) : options.bytes === Array ? Array.prototype.slice.call(message.value) : message.value;
                    return object;
                };
    
                /**
                 * Converts this Any to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.Any
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                Any.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for Any
                 * @function getTypeUrl
                 * @memberof google.protobuf.Any
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                Any.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.Any";
                };
    
                return Any;
            })();
    
            protobuf.Empty = (function() {
    
                /**
                 * Properties of an Empty.
                 * @memberof google.protobuf
                 * @interface IEmpty
                 */
    
                /**
                 * Constructs a new Empty.
                 * @memberof google.protobuf
                 * @classdesc Represents an Empty.
                 * @implements IEmpty
                 * @constructor
                 * @param {google.protobuf.IEmpty=} [properties] Properties to set
                 */
                function Empty(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * Creates a new Empty instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.Empty
                 * @static
                 * @param {google.protobuf.IEmpty=} [properties] Properties to set
                 * @returns {google.protobuf.Empty} Empty instance
                 */
                Empty.create = function create(properties) {
                    return new Empty(properties);
                };
    
                /**
                 * Encodes the specified Empty message. Does not implicitly {@link google.protobuf.Empty.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.Empty
                 * @static
                 * @param {google.protobuf.IEmpty} message Empty message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Empty.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    return writer;
                };
    
                /**
                 * Encodes the specified Empty message, length delimited. Does not implicitly {@link google.protobuf.Empty.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.Empty
                 * @static
                 * @param {google.protobuf.IEmpty} message Empty message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Empty.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes an Empty message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.Empty
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.Empty} Empty
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Empty.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.Empty();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes an Empty message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.Empty
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.Empty} Empty
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Empty.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies an Empty message.
                 * @function verify
                 * @memberof google.protobuf.Empty
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                Empty.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    return null;
                };
    
                /**
                 * Creates an Empty message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.Empty
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.Empty} Empty
                 */
                Empty.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.Empty)
                        return object;
                    return new $root.google.protobuf.Empty();
                };
    
                /**
                 * Creates a plain object from an Empty message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.Empty
                 * @static
                 * @param {google.protobuf.Empty} message Empty
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                Empty.toObject = function toObject() {
                    return {};
                };
    
                /**
                 * Converts this Empty to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.Empty
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                Empty.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for Empty
                 * @function getTypeUrl
                 * @memberof google.protobuf.Empty
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                Empty.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.Empty";
                };
    
                return Empty;
            })();
    
            protobuf.DoubleValue = (function() {
    
                /**
                 * Properties of a DoubleValue.
                 * @memberof google.protobuf
                 * @interface IDoubleValue
                 * @property {number|null} [value] DoubleValue value
                 */
    
                /**
                 * Constructs a new DoubleValue.
                 * @memberof google.protobuf
                 * @classdesc Represents a DoubleValue.
                 * @implements IDoubleValue
                 * @constructor
                 * @param {google.protobuf.IDoubleValue=} [properties] Properties to set
                 */
                function DoubleValue(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * DoubleValue value.
                 * @member {number} value
                 * @memberof google.protobuf.DoubleValue
                 * @instance
                 */
                DoubleValue.prototype.value = 0;
    
                /**
                 * Creates a new DoubleValue instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.DoubleValue
                 * @static
                 * @param {google.protobuf.IDoubleValue=} [properties] Properties to set
                 * @returns {google.protobuf.DoubleValue} DoubleValue instance
                 */
                DoubleValue.create = function create(properties) {
                    return new DoubleValue(properties);
                };
    
                /**
                 * Encodes the specified DoubleValue message. Does not implicitly {@link google.protobuf.DoubleValue.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.DoubleValue
                 * @static
                 * @param {google.protobuf.IDoubleValue} message DoubleValue message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                DoubleValue.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.value != null && Object.hasOwnProperty.call(message, "value"))
                        writer.uint32(/* id 1, wireType 1 =*/9).double(message.value);
                    return writer;
                };
    
                /**
                 * Encodes the specified DoubleValue message, length delimited. Does not implicitly {@link google.protobuf.DoubleValue.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.DoubleValue
                 * @static
                 * @param {google.protobuf.IDoubleValue} message DoubleValue message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                DoubleValue.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a DoubleValue message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.DoubleValue
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.DoubleValue} DoubleValue
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                DoubleValue.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.DoubleValue();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.value = reader.double();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a DoubleValue message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.DoubleValue
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.DoubleValue} DoubleValue
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                DoubleValue.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a DoubleValue message.
                 * @function verify
                 * @memberof google.protobuf.DoubleValue
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                DoubleValue.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.value != null && message.hasOwnProperty("value"))
                        if (typeof message.value !== "number")
                            return "value: number expected";
                    return null;
                };
    
                /**
                 * Creates a DoubleValue message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.DoubleValue
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.DoubleValue} DoubleValue
                 */
                DoubleValue.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.DoubleValue)
                        return object;
                    var message = new $root.google.protobuf.DoubleValue();
                    if (object.value != null)
                        message.value = Number(object.value);
                    return message;
                };
    
                /**
                 * Creates a plain object from a DoubleValue message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.DoubleValue
                 * @static
                 * @param {google.protobuf.DoubleValue} message DoubleValue
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                DoubleValue.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults)
                        object.value = 0;
                    if (message.value != null && message.hasOwnProperty("value"))
                        object.value = options.json && !isFinite(message.value) ? String(message.value) : message.value;
                    return object;
                };
    
                /**
                 * Converts this DoubleValue to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.DoubleValue
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                DoubleValue.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for DoubleValue
                 * @function getTypeUrl
                 * @memberof google.protobuf.DoubleValue
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                DoubleValue.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.DoubleValue";
                };
    
                return DoubleValue;
            })();
    
            protobuf.FloatValue = (function() {
    
                /**
                 * Properties of a FloatValue.
                 * @memberof google.protobuf
                 * @interface IFloatValue
                 * @property {number|null} [value] FloatValue value
                 */
    
                /**
                 * Constructs a new FloatValue.
                 * @memberof google.protobuf
                 * @classdesc Represents a FloatValue.
                 * @implements IFloatValue
                 * @constructor
                 * @param {google.protobuf.IFloatValue=} [properties] Properties to set
                 */
                function FloatValue(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * FloatValue value.
                 * @member {number} value
                 * @memberof google.protobuf.FloatValue
                 * @instance
                 */
                FloatValue.prototype.value = 0;
    
                /**
                 * Creates a new FloatValue instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.FloatValue
                 * @static
                 * @param {google.protobuf.IFloatValue=} [properties] Properties to set
                 * @returns {google.protobuf.FloatValue} FloatValue instance
                 */
                FloatValue.create = function create(properties) {
                    return new FloatValue(properties);
                };
    
                /**
                 * Encodes the specified FloatValue message. Does not implicitly {@link google.protobuf.FloatValue.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.FloatValue
                 * @static
                 * @param {google.protobuf.IFloatValue} message FloatValue message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FloatValue.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.value != null && Object.hasOwnProperty.call(message, "value"))
                        writer.uint32(/* id 1, wireType 5 =*/13).float(message.value);
                    return writer;
                };
    
                /**
                 * Encodes the specified FloatValue message, length delimited. Does not implicitly {@link google.protobuf.FloatValue.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.FloatValue
                 * @static
                 * @param {google.protobuf.IFloatValue} message FloatValue message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                FloatValue.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a FloatValue message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.FloatValue
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.FloatValue} FloatValue
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FloatValue.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.FloatValue();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.value = reader.float();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a FloatValue message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.FloatValue
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.FloatValue} FloatValue
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                FloatValue.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a FloatValue message.
                 * @function verify
                 * @memberof google.protobuf.FloatValue
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                FloatValue.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.value != null && message.hasOwnProperty("value"))
                        if (typeof message.value !== "number")
                            return "value: number expected";
                    return null;
                };
    
                /**
                 * Creates a FloatValue message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.FloatValue
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.FloatValue} FloatValue
                 */
                FloatValue.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.FloatValue)
                        return object;
                    var message = new $root.google.protobuf.FloatValue();
                    if (object.value != null)
                        message.value = Number(object.value);
                    return message;
                };
    
                /**
                 * Creates a plain object from a FloatValue message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.FloatValue
                 * @static
                 * @param {google.protobuf.FloatValue} message FloatValue
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                FloatValue.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults)
                        object.value = 0;
                    if (message.value != null && message.hasOwnProperty("value"))
                        object.value = options.json && !isFinite(message.value) ? String(message.value) : message.value;
                    return object;
                };
    
                /**
                 * Converts this FloatValue to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.FloatValue
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                FloatValue.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for FloatValue
                 * @function getTypeUrl
                 * @memberof google.protobuf.FloatValue
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                FloatValue.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.FloatValue";
                };
    
                return FloatValue;
            })();
    
            protobuf.Int64Value = (function() {
    
                /**
                 * Properties of an Int64Value.
                 * @memberof google.protobuf
                 * @interface IInt64Value
                 * @property {number|Long|null} [value] Int64Value value
                 */
    
                /**
                 * Constructs a new Int64Value.
                 * @memberof google.protobuf
                 * @classdesc Represents an Int64Value.
                 * @implements IInt64Value
                 * @constructor
                 * @param {google.protobuf.IInt64Value=} [properties] Properties to set
                 */
                function Int64Value(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * Int64Value value.
                 * @member {number|Long} value
                 * @memberof google.protobuf.Int64Value
                 * @instance
                 */
                Int64Value.prototype.value = $util.Long ? $util.Long.fromBits(0,0,false) : 0;
    
                /**
                 * Creates a new Int64Value instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.Int64Value
                 * @static
                 * @param {google.protobuf.IInt64Value=} [properties] Properties to set
                 * @returns {google.protobuf.Int64Value} Int64Value instance
                 */
                Int64Value.create = function create(properties) {
                    return new Int64Value(properties);
                };
    
                /**
                 * Encodes the specified Int64Value message. Does not implicitly {@link google.protobuf.Int64Value.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.Int64Value
                 * @static
                 * @param {google.protobuf.IInt64Value} message Int64Value message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Int64Value.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.value != null && Object.hasOwnProperty.call(message, "value"))
                        writer.uint32(/* id 1, wireType 0 =*/8).int64(message.value);
                    return writer;
                };
    
                /**
                 * Encodes the specified Int64Value message, length delimited. Does not implicitly {@link google.protobuf.Int64Value.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.Int64Value
                 * @static
                 * @param {google.protobuf.IInt64Value} message Int64Value message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Int64Value.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes an Int64Value message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.Int64Value
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.Int64Value} Int64Value
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Int64Value.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.Int64Value();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.value = reader.int64();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes an Int64Value message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.Int64Value
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.Int64Value} Int64Value
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Int64Value.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies an Int64Value message.
                 * @function verify
                 * @memberof google.protobuf.Int64Value
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                Int64Value.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.value != null && message.hasOwnProperty("value"))
                        if (!$util.isInteger(message.value) && !(message.value && $util.isInteger(message.value.low) && $util.isInteger(message.value.high)))
                            return "value: integer|Long expected";
                    return null;
                };
    
                /**
                 * Creates an Int64Value message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.Int64Value
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.Int64Value} Int64Value
                 */
                Int64Value.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.Int64Value)
                        return object;
                    var message = new $root.google.protobuf.Int64Value();
                    if (object.value != null)
                        if ($util.Long)
                            (message.value = $util.Long.fromValue(object.value)).unsigned = false;
                        else if (typeof object.value === "string")
                            message.value = parseInt(object.value, 10);
                        else if (typeof object.value === "number")
                            message.value = object.value;
                        else if (typeof object.value === "object")
                            message.value = new $util.LongBits(object.value.low >>> 0, object.value.high >>> 0).toNumber();
                    return message;
                };
    
                /**
                 * Creates a plain object from an Int64Value message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.Int64Value
                 * @static
                 * @param {google.protobuf.Int64Value} message Int64Value
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                Int64Value.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults)
                        if ($util.Long) {
                            var long = new $util.Long(0, 0, false);
                            object.value = options.longs === String ? long.toString() : options.longs === Number ? long.toNumber() : long;
                        } else
                            object.value = options.longs === String ? "0" : 0;
                    if (message.value != null && message.hasOwnProperty("value"))
                        if (typeof message.value === "number")
                            object.value = options.longs === String ? String(message.value) : message.value;
                        else
                            object.value = options.longs === String ? $util.Long.prototype.toString.call(message.value) : options.longs === Number ? new $util.LongBits(message.value.low >>> 0, message.value.high >>> 0).toNumber() : message.value;
                    return object;
                };
    
                /**
                 * Converts this Int64Value to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.Int64Value
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                Int64Value.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for Int64Value
                 * @function getTypeUrl
                 * @memberof google.protobuf.Int64Value
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                Int64Value.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.Int64Value";
                };
    
                return Int64Value;
            })();
    
            protobuf.UInt64Value = (function() {
    
                /**
                 * Properties of a UInt64Value.
                 * @memberof google.protobuf
                 * @interface IUInt64Value
                 * @property {number|Long|null} [value] UInt64Value value
                 */
    
                /**
                 * Constructs a new UInt64Value.
                 * @memberof google.protobuf
                 * @classdesc Represents a UInt64Value.
                 * @implements IUInt64Value
                 * @constructor
                 * @param {google.protobuf.IUInt64Value=} [properties] Properties to set
                 */
                function UInt64Value(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * UInt64Value value.
                 * @member {number|Long} value
                 * @memberof google.protobuf.UInt64Value
                 * @instance
                 */
                UInt64Value.prototype.value = $util.Long ? $util.Long.fromBits(0,0,true) : 0;
    
                /**
                 * Creates a new UInt64Value instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.UInt64Value
                 * @static
                 * @param {google.protobuf.IUInt64Value=} [properties] Properties to set
                 * @returns {google.protobuf.UInt64Value} UInt64Value instance
                 */
                UInt64Value.create = function create(properties) {
                    return new UInt64Value(properties);
                };
    
                /**
                 * Encodes the specified UInt64Value message. Does not implicitly {@link google.protobuf.UInt64Value.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.UInt64Value
                 * @static
                 * @param {google.protobuf.IUInt64Value} message UInt64Value message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                UInt64Value.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.value != null && Object.hasOwnProperty.call(message, "value"))
                        writer.uint32(/* id 1, wireType 0 =*/8).uint64(message.value);
                    return writer;
                };
    
                /**
                 * Encodes the specified UInt64Value message, length delimited. Does not implicitly {@link google.protobuf.UInt64Value.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.UInt64Value
                 * @static
                 * @param {google.protobuf.IUInt64Value} message UInt64Value message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                UInt64Value.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a UInt64Value message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.UInt64Value
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.UInt64Value} UInt64Value
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                UInt64Value.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.UInt64Value();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.value = reader.uint64();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a UInt64Value message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.UInt64Value
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.UInt64Value} UInt64Value
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                UInt64Value.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a UInt64Value message.
                 * @function verify
                 * @memberof google.protobuf.UInt64Value
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                UInt64Value.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.value != null && message.hasOwnProperty("value"))
                        if (!$util.isInteger(message.value) && !(message.value && $util.isInteger(message.value.low) && $util.isInteger(message.value.high)))
                            return "value: integer|Long expected";
                    return null;
                };
    
                /**
                 * Creates a UInt64Value message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.UInt64Value
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.UInt64Value} UInt64Value
                 */
                UInt64Value.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.UInt64Value)
                        return object;
                    var message = new $root.google.protobuf.UInt64Value();
                    if (object.value != null)
                        if ($util.Long)
                            (message.value = $util.Long.fromValue(object.value)).unsigned = true;
                        else if (typeof object.value === "string")
                            message.value = parseInt(object.value, 10);
                        else if (typeof object.value === "number")
                            message.value = object.value;
                        else if (typeof object.value === "object")
                            message.value = new $util.LongBits(object.value.low >>> 0, object.value.high >>> 0).toNumber(true);
                    return message;
                };
    
                /**
                 * Creates a plain object from a UInt64Value message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.UInt64Value
                 * @static
                 * @param {google.protobuf.UInt64Value} message UInt64Value
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                UInt64Value.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults)
                        if ($util.Long) {
                            var long = new $util.Long(0, 0, true);
                            object.value = options.longs === String ? long.toString() : options.longs === Number ? long.toNumber() : long;
                        } else
                            object.value = options.longs === String ? "0" : 0;
                    if (message.value != null && message.hasOwnProperty("value"))
                        if (typeof message.value === "number")
                            object.value = options.longs === String ? String(message.value) : message.value;
                        else
                            object.value = options.longs === String ? $util.Long.prototype.toString.call(message.value) : options.longs === Number ? new $util.LongBits(message.value.low >>> 0, message.value.high >>> 0).toNumber(true) : message.value;
                    return object;
                };
    
                /**
                 * Converts this UInt64Value to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.UInt64Value
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                UInt64Value.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for UInt64Value
                 * @function getTypeUrl
                 * @memberof google.protobuf.UInt64Value
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                UInt64Value.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.UInt64Value";
                };
    
                return UInt64Value;
            })();
    
            protobuf.Int32Value = (function() {
    
                /**
                 * Properties of an Int32Value.
                 * @memberof google.protobuf
                 * @interface IInt32Value
                 * @property {number|null} [value] Int32Value value
                 */
    
                /**
                 * Constructs a new Int32Value.
                 * @memberof google.protobuf
                 * @classdesc Represents an Int32Value.
                 * @implements IInt32Value
                 * @constructor
                 * @param {google.protobuf.IInt32Value=} [properties] Properties to set
                 */
                function Int32Value(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * Int32Value value.
                 * @member {number} value
                 * @memberof google.protobuf.Int32Value
                 * @instance
                 */
                Int32Value.prototype.value = 0;
    
                /**
                 * Creates a new Int32Value instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.Int32Value
                 * @static
                 * @param {google.protobuf.IInt32Value=} [properties] Properties to set
                 * @returns {google.protobuf.Int32Value} Int32Value instance
                 */
                Int32Value.create = function create(properties) {
                    return new Int32Value(properties);
                };
    
                /**
                 * Encodes the specified Int32Value message. Does not implicitly {@link google.protobuf.Int32Value.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.Int32Value
                 * @static
                 * @param {google.protobuf.IInt32Value} message Int32Value message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Int32Value.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.value != null && Object.hasOwnProperty.call(message, "value"))
                        writer.uint32(/* id 1, wireType 0 =*/8).int32(message.value);
                    return writer;
                };
    
                /**
                 * Encodes the specified Int32Value message, length delimited. Does not implicitly {@link google.protobuf.Int32Value.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.Int32Value
                 * @static
                 * @param {google.protobuf.IInt32Value} message Int32Value message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Int32Value.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes an Int32Value message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.Int32Value
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.Int32Value} Int32Value
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Int32Value.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.Int32Value();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.value = reader.int32();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes an Int32Value message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.Int32Value
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.Int32Value} Int32Value
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Int32Value.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies an Int32Value message.
                 * @function verify
                 * @memberof google.protobuf.Int32Value
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                Int32Value.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.value != null && message.hasOwnProperty("value"))
                        if (!$util.isInteger(message.value))
                            return "value: integer expected";
                    return null;
                };
    
                /**
                 * Creates an Int32Value message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.Int32Value
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.Int32Value} Int32Value
                 */
                Int32Value.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.Int32Value)
                        return object;
                    var message = new $root.google.protobuf.Int32Value();
                    if (object.value != null)
                        message.value = object.value | 0;
                    return message;
                };
    
                /**
                 * Creates a plain object from an Int32Value message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.Int32Value
                 * @static
                 * @param {google.protobuf.Int32Value} message Int32Value
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                Int32Value.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults)
                        object.value = 0;
                    if (message.value != null && message.hasOwnProperty("value"))
                        object.value = message.value;
                    return object;
                };
    
                /**
                 * Converts this Int32Value to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.Int32Value
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                Int32Value.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for Int32Value
                 * @function getTypeUrl
                 * @memberof google.protobuf.Int32Value
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                Int32Value.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.Int32Value";
                };
    
                return Int32Value;
            })();
    
            protobuf.UInt32Value = (function() {
    
                /**
                 * Properties of a UInt32Value.
                 * @memberof google.protobuf
                 * @interface IUInt32Value
                 * @property {number|null} [value] UInt32Value value
                 */
    
                /**
                 * Constructs a new UInt32Value.
                 * @memberof google.protobuf
                 * @classdesc Represents a UInt32Value.
                 * @implements IUInt32Value
                 * @constructor
                 * @param {google.protobuf.IUInt32Value=} [properties] Properties to set
                 */
                function UInt32Value(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * UInt32Value value.
                 * @member {number} value
                 * @memberof google.protobuf.UInt32Value
                 * @instance
                 */
                UInt32Value.prototype.value = 0;
    
                /**
                 * Creates a new UInt32Value instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.UInt32Value
                 * @static
                 * @param {google.protobuf.IUInt32Value=} [properties] Properties to set
                 * @returns {google.protobuf.UInt32Value} UInt32Value instance
                 */
                UInt32Value.create = function create(properties) {
                    return new UInt32Value(properties);
                };
    
                /**
                 * Encodes the specified UInt32Value message. Does not implicitly {@link google.protobuf.UInt32Value.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.UInt32Value
                 * @static
                 * @param {google.protobuf.IUInt32Value} message UInt32Value message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                UInt32Value.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.value != null && Object.hasOwnProperty.call(message, "value"))
                        writer.uint32(/* id 1, wireType 0 =*/8).uint32(message.value);
                    return writer;
                };
    
                /**
                 * Encodes the specified UInt32Value message, length delimited. Does not implicitly {@link google.protobuf.UInt32Value.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.UInt32Value
                 * @static
                 * @param {google.protobuf.IUInt32Value} message UInt32Value message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                UInt32Value.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a UInt32Value message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.UInt32Value
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.UInt32Value} UInt32Value
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                UInt32Value.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.UInt32Value();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.value = reader.uint32();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a UInt32Value message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.UInt32Value
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.UInt32Value} UInt32Value
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                UInt32Value.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a UInt32Value message.
                 * @function verify
                 * @memberof google.protobuf.UInt32Value
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                UInt32Value.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.value != null && message.hasOwnProperty("value"))
                        if (!$util.isInteger(message.value))
                            return "value: integer expected";
                    return null;
                };
    
                /**
                 * Creates a UInt32Value message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.UInt32Value
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.UInt32Value} UInt32Value
                 */
                UInt32Value.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.UInt32Value)
                        return object;
                    var message = new $root.google.protobuf.UInt32Value();
                    if (object.value != null)
                        message.value = object.value >>> 0;
                    return message;
                };
    
                /**
                 * Creates a plain object from a UInt32Value message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.UInt32Value
                 * @static
                 * @param {google.protobuf.UInt32Value} message UInt32Value
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                UInt32Value.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults)
                        object.value = 0;
                    if (message.value != null && message.hasOwnProperty("value"))
                        object.value = message.value;
                    return object;
                };
    
                /**
                 * Converts this UInt32Value to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.UInt32Value
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                UInt32Value.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for UInt32Value
                 * @function getTypeUrl
                 * @memberof google.protobuf.UInt32Value
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                UInt32Value.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.UInt32Value";
                };
    
                return UInt32Value;
            })();
    
            protobuf.BoolValue = (function() {
    
                /**
                 * Properties of a BoolValue.
                 * @memberof google.protobuf
                 * @interface IBoolValue
                 * @property {boolean|null} [value] BoolValue value
                 */
    
                /**
                 * Constructs a new BoolValue.
                 * @memberof google.protobuf
                 * @classdesc Represents a BoolValue.
                 * @implements IBoolValue
                 * @constructor
                 * @param {google.protobuf.IBoolValue=} [properties] Properties to set
                 */
                function BoolValue(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * BoolValue value.
                 * @member {boolean} value
                 * @memberof google.protobuf.BoolValue
                 * @instance
                 */
                BoolValue.prototype.value = false;
    
                /**
                 * Creates a new BoolValue instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.BoolValue
                 * @static
                 * @param {google.protobuf.IBoolValue=} [properties] Properties to set
                 * @returns {google.protobuf.BoolValue} BoolValue instance
                 */
                BoolValue.create = function create(properties) {
                    return new BoolValue(properties);
                };
    
                /**
                 * Encodes the specified BoolValue message. Does not implicitly {@link google.protobuf.BoolValue.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.BoolValue
                 * @static
                 * @param {google.protobuf.IBoolValue} message BoolValue message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                BoolValue.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.value != null && Object.hasOwnProperty.call(message, "value"))
                        writer.uint32(/* id 1, wireType 0 =*/8).bool(message.value);
                    return writer;
                };
    
                /**
                 * Encodes the specified BoolValue message, length delimited. Does not implicitly {@link google.protobuf.BoolValue.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.BoolValue
                 * @static
                 * @param {google.protobuf.IBoolValue} message BoolValue message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                BoolValue.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a BoolValue message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.BoolValue
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.BoolValue} BoolValue
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                BoolValue.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.BoolValue();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.value = reader.bool();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a BoolValue message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.BoolValue
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.BoolValue} BoolValue
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                BoolValue.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a BoolValue message.
                 * @function verify
                 * @memberof google.protobuf.BoolValue
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                BoolValue.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.value != null && message.hasOwnProperty("value"))
                        if (typeof message.value !== "boolean")
                            return "value: boolean expected";
                    return null;
                };
    
                /**
                 * Creates a BoolValue message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.BoolValue
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.BoolValue} BoolValue
                 */
                BoolValue.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.BoolValue)
                        return object;
                    var message = new $root.google.protobuf.BoolValue();
                    if (object.value != null)
                        message.value = Boolean(object.value);
                    return message;
                };
    
                /**
                 * Creates a plain object from a BoolValue message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.BoolValue
                 * @static
                 * @param {google.protobuf.BoolValue} message BoolValue
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                BoolValue.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults)
                        object.value = false;
                    if (message.value != null && message.hasOwnProperty("value"))
                        object.value = message.value;
                    return object;
                };
    
                /**
                 * Converts this BoolValue to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.BoolValue
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                BoolValue.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for BoolValue
                 * @function getTypeUrl
                 * @memberof google.protobuf.BoolValue
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                BoolValue.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.BoolValue";
                };
    
                return BoolValue;
            })();
    
            protobuf.StringValue = (function() {
    
                /**
                 * Properties of a StringValue.
                 * @memberof google.protobuf
                 * @interface IStringValue
                 * @property {string|null} [value] StringValue value
                 */
    
                /**
                 * Constructs a new StringValue.
                 * @memberof google.protobuf
                 * @classdesc Represents a StringValue.
                 * @implements IStringValue
                 * @constructor
                 * @param {google.protobuf.IStringValue=} [properties] Properties to set
                 */
                function StringValue(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * StringValue value.
                 * @member {string} value
                 * @memberof google.protobuf.StringValue
                 * @instance
                 */
                StringValue.prototype.value = "";
    
                /**
                 * Creates a new StringValue instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.StringValue
                 * @static
                 * @param {google.protobuf.IStringValue=} [properties] Properties to set
                 * @returns {google.protobuf.StringValue} StringValue instance
                 */
                StringValue.create = function create(properties) {
                    return new StringValue(properties);
                };
    
                /**
                 * Encodes the specified StringValue message. Does not implicitly {@link google.protobuf.StringValue.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.StringValue
                 * @static
                 * @param {google.protobuf.IStringValue} message StringValue message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                StringValue.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.value != null && Object.hasOwnProperty.call(message, "value"))
                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.value);
                    return writer;
                };
    
                /**
                 * Encodes the specified StringValue message, length delimited. Does not implicitly {@link google.protobuf.StringValue.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.StringValue
                 * @static
                 * @param {google.protobuf.IStringValue} message StringValue message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                StringValue.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a StringValue message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.StringValue
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.StringValue} StringValue
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                StringValue.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.StringValue();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.value = reader.string();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a StringValue message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.StringValue
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.StringValue} StringValue
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                StringValue.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a StringValue message.
                 * @function verify
                 * @memberof google.protobuf.StringValue
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                StringValue.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.value != null && message.hasOwnProperty("value"))
                        if (!$util.isString(message.value))
                            return "value: string expected";
                    return null;
                };
    
                /**
                 * Creates a StringValue message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.StringValue
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.StringValue} StringValue
                 */
                StringValue.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.StringValue)
                        return object;
                    var message = new $root.google.protobuf.StringValue();
                    if (object.value != null)
                        message.value = String(object.value);
                    return message;
                };
    
                /**
                 * Creates a plain object from a StringValue message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.StringValue
                 * @static
                 * @param {google.protobuf.StringValue} message StringValue
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                StringValue.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults)
                        object.value = "";
                    if (message.value != null && message.hasOwnProperty("value"))
                        object.value = message.value;
                    return object;
                };
    
                /**
                 * Converts this StringValue to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.StringValue
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                StringValue.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for StringValue
                 * @function getTypeUrl
                 * @memberof google.protobuf.StringValue
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                StringValue.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.StringValue";
                };
    
                return StringValue;
            })();
    
            protobuf.BytesValue = (function() {
    
                /**
                 * Properties of a BytesValue.
                 * @memberof google.protobuf
                 * @interface IBytesValue
                 * @property {Uint8Array|null} [value] BytesValue value
                 */
    
                /**
                 * Constructs a new BytesValue.
                 * @memberof google.protobuf
                 * @classdesc Represents a BytesValue.
                 * @implements IBytesValue
                 * @constructor
                 * @param {google.protobuf.IBytesValue=} [properties] Properties to set
                 */
                function BytesValue(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * BytesValue value.
                 * @member {Uint8Array} value
                 * @memberof google.protobuf.BytesValue
                 * @instance
                 */
                BytesValue.prototype.value = $util.newBuffer([]);
    
                /**
                 * Creates a new BytesValue instance using the specified properties.
                 * @function create
                 * @memberof google.protobuf.BytesValue
                 * @static
                 * @param {google.protobuf.IBytesValue=} [properties] Properties to set
                 * @returns {google.protobuf.BytesValue} BytesValue instance
                 */
                BytesValue.create = function create(properties) {
                    return new BytesValue(properties);
                };
    
                /**
                 * Encodes the specified BytesValue message. Does not implicitly {@link google.protobuf.BytesValue.verify|verify} messages.
                 * @function encode
                 * @memberof google.protobuf.BytesValue
                 * @static
                 * @param {google.protobuf.IBytesValue} message BytesValue message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                BytesValue.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.value != null && Object.hasOwnProperty.call(message, "value"))
                        writer.uint32(/* id 1, wireType 2 =*/10).bytes(message.value);
                    return writer;
                };
    
                /**
                 * Encodes the specified BytesValue message, length delimited. Does not implicitly {@link google.protobuf.BytesValue.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.protobuf.BytesValue
                 * @static
                 * @param {google.protobuf.IBytesValue} message BytesValue message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                BytesValue.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a BytesValue message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.protobuf.BytesValue
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.protobuf.BytesValue} BytesValue
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                BytesValue.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.protobuf.BytesValue();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.value = reader.bytes();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a BytesValue message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.protobuf.BytesValue
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.protobuf.BytesValue} BytesValue
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                BytesValue.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a BytesValue message.
                 * @function verify
                 * @memberof google.protobuf.BytesValue
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                BytesValue.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.value != null && message.hasOwnProperty("value"))
                        if (!(message.value && typeof message.value.length === "number" || $util.isString(message.value)))
                            return "value: buffer expected";
                    return null;
                };
    
                /**
                 * Creates a BytesValue message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.protobuf.BytesValue
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.protobuf.BytesValue} BytesValue
                 */
                BytesValue.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.protobuf.BytesValue)
                        return object;
                    var message = new $root.google.protobuf.BytesValue();
                    if (object.value != null)
                        if (typeof object.value === "string")
                            $util.base64.decode(object.value, message.value = $util.newBuffer($util.base64.length(object.value)), 0);
                        else if (object.value.length >= 0)
                            message.value = object.value;
                    return message;
                };
    
                /**
                 * Creates a plain object from a BytesValue message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.protobuf.BytesValue
                 * @static
                 * @param {google.protobuf.BytesValue} message BytesValue
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                BytesValue.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults)
                        if (options.bytes === String)
                            object.value = "";
                        else {
                            object.value = [];
                            if (options.bytes !== Array)
                                object.value = $util.newBuffer(object.value);
                        }
                    if (message.value != null && message.hasOwnProperty("value"))
                        object.value = options.bytes === String ? $util.base64.encode(message.value, 0, message.value.length) : options.bytes === Array ? Array.prototype.slice.call(message.value) : message.value;
                    return object;
                };
    
                /**
                 * Converts this BytesValue to JSON.
                 * @function toJSON
                 * @memberof google.protobuf.BytesValue
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                BytesValue.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for BytesValue
                 * @function getTypeUrl
                 * @memberof google.protobuf.BytesValue
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                BytesValue.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.protobuf.BytesValue";
                };
    
                return BytesValue;
            })();
    
            return protobuf;
        })();
    
        google.iam = (function() {
    
            /**
             * Namespace iam.
             * @memberof google
             * @namespace
             */
            var iam = {};
    
            iam.v1 = (function() {
    
                /**
                 * Namespace v1.
                 * @memberof google.iam
                 * @namespace
                 */
                var v1 = {};
    
                v1.IAMPolicy = (function() {
    
                    /**
                     * Constructs a new IAMPolicy service.
                     * @memberof google.iam.v1
                     * @classdesc Represents a IAMPolicy
                     * @extends $protobuf.rpc.Service
                     * @constructor
                     * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                     * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                     * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                     */
                    function IAMPolicy(rpcImpl, requestDelimited, responseDelimited) {
                        $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                    }
    
                    (IAMPolicy.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = IAMPolicy;
    
                    /**
                     * Creates new IAMPolicy service using the specified rpc implementation.
                     * @function create
                     * @memberof google.iam.v1.IAMPolicy
                     * @static
                     * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                     * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                     * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                     * @returns {IAMPolicy} RPC service. Useful where requests and/or responses are streamed.
                     */
                    IAMPolicy.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                        return new this(rpcImpl, requestDelimited, responseDelimited);
                    };
    
                    /**
                     * Callback as used by {@link google.iam.v1.IAMPolicy|setIamPolicy}.
                     * @memberof google.iam.v1.IAMPolicy
                     * @typedef SetIamPolicyCallback
                     * @type {function}
                     * @param {Error|null} error Error, if any
                     * @param {google.iam.v1.Policy} [response] Policy
                     */
    
                    /**
                     * Calls SetIamPolicy.
                     * @function setIamPolicy
                     * @memberof google.iam.v1.IAMPolicy
                     * @instance
                     * @param {google.iam.v1.ISetIamPolicyRequest} request SetIamPolicyRequest message or plain object
                     * @param {google.iam.v1.IAMPolicy.SetIamPolicyCallback} callback Node-style callback called with the error, if any, and Policy
                     * @returns {undefined}
                     * @variation 1
                     */
                    Object.defineProperty(IAMPolicy.prototype.setIamPolicy = function setIamPolicy(request, callback) {
                        return this.rpcCall(setIamPolicy, $root.google.iam.v1.SetIamPolicyRequest, $root.google.iam.v1.Policy, request, callback);
                    }, "name", { value: "SetIamPolicy" });
    
                    /**
                     * Calls SetIamPolicy.
                     * @function setIamPolicy
                     * @memberof google.iam.v1.IAMPolicy
                     * @instance
                     * @param {google.iam.v1.ISetIamPolicyRequest} request SetIamPolicyRequest message or plain object
                     * @returns {Promise<google.iam.v1.Policy>} Promise
                     * @variation 2
                     */
    
                    /**
                     * Callback as used by {@link google.iam.v1.IAMPolicy|getIamPolicy}.
                     * @memberof google.iam.v1.IAMPolicy
                     * @typedef GetIamPolicyCallback
                     * @type {function}
                     * @param {Error|null} error Error, if any
                     * @param {google.iam.v1.Policy} [response] Policy
                     */
    
                    /**
                     * Calls GetIamPolicy.
                     * @function getIamPolicy
                     * @memberof google.iam.v1.IAMPolicy
                     * @instance
                     * @param {google.iam.v1.IGetIamPolicyRequest} request GetIamPolicyRequest message or plain object
                     * @param {google.iam.v1.IAMPolicy.GetIamPolicyCallback} callback Node-style callback called with the error, if any, and Policy
                     * @returns {undefined}
                     * @variation 1
                     */
                    Object.defineProperty(IAMPolicy.prototype.getIamPolicy = function getIamPolicy(request, callback) {
                        return this.rpcCall(getIamPolicy, $root.google.iam.v1.GetIamPolicyRequest, $root.google.iam.v1.Policy, request, callback);
                    }, "name", { value: "GetIamPolicy" });
    
                    /**
                     * Calls GetIamPolicy.
                     * @function getIamPolicy
                     * @memberof google.iam.v1.IAMPolicy
                     * @instance
                     * @param {google.iam.v1.IGetIamPolicyRequest} request GetIamPolicyRequest message or plain object
                     * @returns {Promise<google.iam.v1.Policy>} Promise
                     * @variation 2
                     */
    
                    /**
                     * Callback as used by {@link google.iam.v1.IAMPolicy|testIamPermissions}.
                     * @memberof google.iam.v1.IAMPolicy
                     * @typedef TestIamPermissionsCallback
                     * @type {function}
                     * @param {Error|null} error Error, if any
                     * @param {google.iam.v1.TestIamPermissionsResponse} [response] TestIamPermissionsResponse
                     */
    
                    /**
                     * Calls TestIamPermissions.
                     * @function testIamPermissions
                     * @memberof google.iam.v1.IAMPolicy
                     * @instance
                     * @param {google.iam.v1.ITestIamPermissionsRequest} request TestIamPermissionsRequest message or plain object
                     * @param {google.iam.v1.IAMPolicy.TestIamPermissionsCallback} callback Node-style callback called with the error, if any, and TestIamPermissionsResponse
                     * @returns {undefined}
                     * @variation 1
                     */
                    Object.defineProperty(IAMPolicy.prototype.testIamPermissions = function testIamPermissions(request, callback) {
                        return this.rpcCall(testIamPermissions, $root.google.iam.v1.TestIamPermissionsRequest, $root.google.iam.v1.TestIamPermissionsResponse, request, callback);
                    }, "name", { value: "TestIamPermissions" });
    
                    /**
                     * Calls TestIamPermissions.
                     * @function testIamPermissions
                     * @memberof google.iam.v1.IAMPolicy
                     * @instance
                     * @param {google.iam.v1.ITestIamPermissionsRequest} request TestIamPermissionsRequest message or plain object
                     * @returns {Promise<google.iam.v1.TestIamPermissionsResponse>} Promise
                     * @variation 2
                     */
    
                    return IAMPolicy;
                })();
    
                v1.SetIamPolicyRequest = (function() {
    
                    /**
                     * Properties of a SetIamPolicyRequest.
                     * @memberof google.iam.v1
                     * @interface ISetIamPolicyRequest
                     * @property {string|null} [resource] SetIamPolicyRequest resource
                     * @property {google.iam.v1.IPolicy|null} [policy] SetIamPolicyRequest policy
                     * @property {google.protobuf.IFieldMask|null} [updateMask] SetIamPolicyRequest updateMask
                     */
    
                    /**
                     * Constructs a new SetIamPolicyRequest.
                     * @memberof google.iam.v1
                     * @classdesc Represents a SetIamPolicyRequest.
                     * @implements ISetIamPolicyRequest
                     * @constructor
                     * @param {google.iam.v1.ISetIamPolicyRequest=} [properties] Properties to set
                     */
                    function SetIamPolicyRequest(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * SetIamPolicyRequest resource.
                     * @member {string} resource
                     * @memberof google.iam.v1.SetIamPolicyRequest
                     * @instance
                     */
                    SetIamPolicyRequest.prototype.resource = "";
    
                    /**
                     * SetIamPolicyRequest policy.
                     * @member {google.iam.v1.IPolicy|null|undefined} policy
                     * @memberof google.iam.v1.SetIamPolicyRequest
                     * @instance
                     */
                    SetIamPolicyRequest.prototype.policy = null;
    
                    /**
                     * SetIamPolicyRequest updateMask.
                     * @member {google.protobuf.IFieldMask|null|undefined} updateMask
                     * @memberof google.iam.v1.SetIamPolicyRequest
                     * @instance
                     */
                    SetIamPolicyRequest.prototype.updateMask = null;
    
                    /**
                     * Creates a new SetIamPolicyRequest instance using the specified properties.
                     * @function create
                     * @memberof google.iam.v1.SetIamPolicyRequest
                     * @static
                     * @param {google.iam.v1.ISetIamPolicyRequest=} [properties] Properties to set
                     * @returns {google.iam.v1.SetIamPolicyRequest} SetIamPolicyRequest instance
                     */
                    SetIamPolicyRequest.create = function create(properties) {
                        return new SetIamPolicyRequest(properties);
                    };
    
                    /**
                     * Encodes the specified SetIamPolicyRequest message. Does not implicitly {@link google.iam.v1.SetIamPolicyRequest.verify|verify} messages.
                     * @function encode
                     * @memberof google.iam.v1.SetIamPolicyRequest
                     * @static
                     * @param {google.iam.v1.ISetIamPolicyRequest} message SetIamPolicyRequest message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    SetIamPolicyRequest.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.resource != null && Object.hasOwnProperty.call(message, "resource"))
                            writer.uint32(/* id 1, wireType 2 =*/10).string(message.resource);
                        if (message.policy != null && Object.hasOwnProperty.call(message, "policy"))
                            $root.google.iam.v1.Policy.encode(message.policy, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                        if (message.updateMask != null && Object.hasOwnProperty.call(message, "updateMask"))
                            $root.google.protobuf.FieldMask.encode(message.updateMask, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                        return writer;
                    };
    
                    /**
                     * Encodes the specified SetIamPolicyRequest message, length delimited. Does not implicitly {@link google.iam.v1.SetIamPolicyRequest.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.iam.v1.SetIamPolicyRequest
                     * @static
                     * @param {google.iam.v1.ISetIamPolicyRequest} message SetIamPolicyRequest message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    SetIamPolicyRequest.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a SetIamPolicyRequest message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.iam.v1.SetIamPolicyRequest
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.iam.v1.SetIamPolicyRequest} SetIamPolicyRequest
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    SetIamPolicyRequest.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.iam.v1.SetIamPolicyRequest();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.resource = reader.string();
                                    break;
                                }
                            case 2: {
                                    message.policy = $root.google.iam.v1.Policy.decode(reader, reader.uint32());
                                    break;
                                }
                            case 3: {
                                    message.updateMask = $root.google.protobuf.FieldMask.decode(reader, reader.uint32());
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a SetIamPolicyRequest message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.iam.v1.SetIamPolicyRequest
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.iam.v1.SetIamPolicyRequest} SetIamPolicyRequest
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    SetIamPolicyRequest.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a SetIamPolicyRequest message.
                     * @function verify
                     * @memberof google.iam.v1.SetIamPolicyRequest
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    SetIamPolicyRequest.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.resource != null && message.hasOwnProperty("resource"))
                            if (!$util.isString(message.resource))
                                return "resource: string expected";
                        if (message.policy != null && message.hasOwnProperty("policy")) {
                            var error = $root.google.iam.v1.Policy.verify(message.policy);
                            if (error)
                                return "policy." + error;
                        }
                        if (message.updateMask != null && message.hasOwnProperty("updateMask")) {
                            var error = $root.google.protobuf.FieldMask.verify(message.updateMask);
                            if (error)
                                return "updateMask." + error;
                        }
                        return null;
                    };
    
                    /**
                     * Creates a SetIamPolicyRequest message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.iam.v1.SetIamPolicyRequest
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.iam.v1.SetIamPolicyRequest} SetIamPolicyRequest
                     */
                    SetIamPolicyRequest.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.iam.v1.SetIamPolicyRequest)
                            return object;
                        var message = new $root.google.iam.v1.SetIamPolicyRequest();
                        if (object.resource != null)
                            message.resource = String(object.resource);
                        if (object.policy != null) {
                            if (typeof object.policy !== "object")
                                throw TypeError(".google.iam.v1.SetIamPolicyRequest.policy: object expected");
                            message.policy = $root.google.iam.v1.Policy.fromObject(object.policy);
                        }
                        if (object.updateMask != null) {
                            if (typeof object.updateMask !== "object")
                                throw TypeError(".google.iam.v1.SetIamPolicyRequest.updateMask: object expected");
                            message.updateMask = $root.google.protobuf.FieldMask.fromObject(object.updateMask);
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a SetIamPolicyRequest message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.iam.v1.SetIamPolicyRequest
                     * @static
                     * @param {google.iam.v1.SetIamPolicyRequest} message SetIamPolicyRequest
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    SetIamPolicyRequest.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults) {
                            object.resource = "";
                            object.policy = null;
                            object.updateMask = null;
                        }
                        if (message.resource != null && message.hasOwnProperty("resource"))
                            object.resource = message.resource;
                        if (message.policy != null && message.hasOwnProperty("policy"))
                            object.policy = $root.google.iam.v1.Policy.toObject(message.policy, options);
                        if (message.updateMask != null && message.hasOwnProperty("updateMask"))
                            object.updateMask = $root.google.protobuf.FieldMask.toObject(message.updateMask, options);
                        return object;
                    };
    
                    /**
                     * Converts this SetIamPolicyRequest to JSON.
                     * @function toJSON
                     * @memberof google.iam.v1.SetIamPolicyRequest
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    SetIamPolicyRequest.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for SetIamPolicyRequest
                     * @function getTypeUrl
                     * @memberof google.iam.v1.SetIamPolicyRequest
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    SetIamPolicyRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.iam.v1.SetIamPolicyRequest";
                    };
    
                    return SetIamPolicyRequest;
                })();
    
                v1.GetIamPolicyRequest = (function() {
    
                    /**
                     * Properties of a GetIamPolicyRequest.
                     * @memberof google.iam.v1
                     * @interface IGetIamPolicyRequest
                     * @property {string|null} [resource] GetIamPolicyRequest resource
                     * @property {google.iam.v1.IGetPolicyOptions|null} [options] GetIamPolicyRequest options
                     */
    
                    /**
                     * Constructs a new GetIamPolicyRequest.
                     * @memberof google.iam.v1
                     * @classdesc Represents a GetIamPolicyRequest.
                     * @implements IGetIamPolicyRequest
                     * @constructor
                     * @param {google.iam.v1.IGetIamPolicyRequest=} [properties] Properties to set
                     */
                    function GetIamPolicyRequest(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * GetIamPolicyRequest resource.
                     * @member {string} resource
                     * @memberof google.iam.v1.GetIamPolicyRequest
                     * @instance
                     */
                    GetIamPolicyRequest.prototype.resource = "";
    
                    /**
                     * GetIamPolicyRequest options.
                     * @member {google.iam.v1.IGetPolicyOptions|null|undefined} options
                     * @memberof google.iam.v1.GetIamPolicyRequest
                     * @instance
                     */
                    GetIamPolicyRequest.prototype.options = null;
    
                    /**
                     * Creates a new GetIamPolicyRequest instance using the specified properties.
                     * @function create
                     * @memberof google.iam.v1.GetIamPolicyRequest
                     * @static
                     * @param {google.iam.v1.IGetIamPolicyRequest=} [properties] Properties to set
                     * @returns {google.iam.v1.GetIamPolicyRequest} GetIamPolicyRequest instance
                     */
                    GetIamPolicyRequest.create = function create(properties) {
                        return new GetIamPolicyRequest(properties);
                    };
    
                    /**
                     * Encodes the specified GetIamPolicyRequest message. Does not implicitly {@link google.iam.v1.GetIamPolicyRequest.verify|verify} messages.
                     * @function encode
                     * @memberof google.iam.v1.GetIamPolicyRequest
                     * @static
                     * @param {google.iam.v1.IGetIamPolicyRequest} message GetIamPolicyRequest message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    GetIamPolicyRequest.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.resource != null && Object.hasOwnProperty.call(message, "resource"))
                            writer.uint32(/* id 1, wireType 2 =*/10).string(message.resource);
                        if (message.options != null && Object.hasOwnProperty.call(message, "options"))
                            $root.google.iam.v1.GetPolicyOptions.encode(message.options, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                        return writer;
                    };
    
                    /**
                     * Encodes the specified GetIamPolicyRequest message, length delimited. Does not implicitly {@link google.iam.v1.GetIamPolicyRequest.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.iam.v1.GetIamPolicyRequest
                     * @static
                     * @param {google.iam.v1.IGetIamPolicyRequest} message GetIamPolicyRequest message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    GetIamPolicyRequest.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a GetIamPolicyRequest message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.iam.v1.GetIamPolicyRequest
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.iam.v1.GetIamPolicyRequest} GetIamPolicyRequest
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    GetIamPolicyRequest.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.iam.v1.GetIamPolicyRequest();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.resource = reader.string();
                                    break;
                                }
                            case 2: {
                                    message.options = $root.google.iam.v1.GetPolicyOptions.decode(reader, reader.uint32());
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a GetIamPolicyRequest message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.iam.v1.GetIamPolicyRequest
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.iam.v1.GetIamPolicyRequest} GetIamPolicyRequest
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    GetIamPolicyRequest.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a GetIamPolicyRequest message.
                     * @function verify
                     * @memberof google.iam.v1.GetIamPolicyRequest
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    GetIamPolicyRequest.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.resource != null && message.hasOwnProperty("resource"))
                            if (!$util.isString(message.resource))
                                return "resource: string expected";
                        if (message.options != null && message.hasOwnProperty("options")) {
                            var error = $root.google.iam.v1.GetPolicyOptions.verify(message.options);
                            if (error)
                                return "options." + error;
                        }
                        return null;
                    };
    
                    /**
                     * Creates a GetIamPolicyRequest message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.iam.v1.GetIamPolicyRequest
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.iam.v1.GetIamPolicyRequest} GetIamPolicyRequest
                     */
                    GetIamPolicyRequest.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.iam.v1.GetIamPolicyRequest)
                            return object;
                        var message = new $root.google.iam.v1.GetIamPolicyRequest();
                        if (object.resource != null)
                            message.resource = String(object.resource);
                        if (object.options != null) {
                            if (typeof object.options !== "object")
                                throw TypeError(".google.iam.v1.GetIamPolicyRequest.options: object expected");
                            message.options = $root.google.iam.v1.GetPolicyOptions.fromObject(object.options);
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a GetIamPolicyRequest message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.iam.v1.GetIamPolicyRequest
                     * @static
                     * @param {google.iam.v1.GetIamPolicyRequest} message GetIamPolicyRequest
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    GetIamPolicyRequest.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults) {
                            object.resource = "";
                            object.options = null;
                        }
                        if (message.resource != null && message.hasOwnProperty("resource"))
                            object.resource = message.resource;
                        if (message.options != null && message.hasOwnProperty("options"))
                            object.options = $root.google.iam.v1.GetPolicyOptions.toObject(message.options, options);
                        return object;
                    };
    
                    /**
                     * Converts this GetIamPolicyRequest to JSON.
                     * @function toJSON
                     * @memberof google.iam.v1.GetIamPolicyRequest
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    GetIamPolicyRequest.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for GetIamPolicyRequest
                     * @function getTypeUrl
                     * @memberof google.iam.v1.GetIamPolicyRequest
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    GetIamPolicyRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.iam.v1.GetIamPolicyRequest";
                    };
    
                    return GetIamPolicyRequest;
                })();
    
                v1.TestIamPermissionsRequest = (function() {
    
                    /**
                     * Properties of a TestIamPermissionsRequest.
                     * @memberof google.iam.v1
                     * @interface ITestIamPermissionsRequest
                     * @property {string|null} [resource] TestIamPermissionsRequest resource
                     * @property {Array.<string>|null} [permissions] TestIamPermissionsRequest permissions
                     */
    
                    /**
                     * Constructs a new TestIamPermissionsRequest.
                     * @memberof google.iam.v1
                     * @classdesc Represents a TestIamPermissionsRequest.
                     * @implements ITestIamPermissionsRequest
                     * @constructor
                     * @param {google.iam.v1.ITestIamPermissionsRequest=} [properties] Properties to set
                     */
                    function TestIamPermissionsRequest(properties) {
                        this.permissions = [];
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * TestIamPermissionsRequest resource.
                     * @member {string} resource
                     * @memberof google.iam.v1.TestIamPermissionsRequest
                     * @instance
                     */
                    TestIamPermissionsRequest.prototype.resource = "";
    
                    /**
                     * TestIamPermissionsRequest permissions.
                     * @member {Array.<string>} permissions
                     * @memberof google.iam.v1.TestIamPermissionsRequest
                     * @instance
                     */
                    TestIamPermissionsRequest.prototype.permissions = $util.emptyArray;
    
                    /**
                     * Creates a new TestIamPermissionsRequest instance using the specified properties.
                     * @function create
                     * @memberof google.iam.v1.TestIamPermissionsRequest
                     * @static
                     * @param {google.iam.v1.ITestIamPermissionsRequest=} [properties] Properties to set
                     * @returns {google.iam.v1.TestIamPermissionsRequest} TestIamPermissionsRequest instance
                     */
                    TestIamPermissionsRequest.create = function create(properties) {
                        return new TestIamPermissionsRequest(properties);
                    };
    
                    /**
                     * Encodes the specified TestIamPermissionsRequest message. Does not implicitly {@link google.iam.v1.TestIamPermissionsRequest.verify|verify} messages.
                     * @function encode
                     * @memberof google.iam.v1.TestIamPermissionsRequest
                     * @static
                     * @param {google.iam.v1.ITestIamPermissionsRequest} message TestIamPermissionsRequest message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    TestIamPermissionsRequest.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.resource != null && Object.hasOwnProperty.call(message, "resource"))
                            writer.uint32(/* id 1, wireType 2 =*/10).string(message.resource);
                        if (message.permissions != null && message.permissions.length)
                            for (var i = 0; i < message.permissions.length; ++i)
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.permissions[i]);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified TestIamPermissionsRequest message, length delimited. Does not implicitly {@link google.iam.v1.TestIamPermissionsRequest.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.iam.v1.TestIamPermissionsRequest
                     * @static
                     * @param {google.iam.v1.ITestIamPermissionsRequest} message TestIamPermissionsRequest message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    TestIamPermissionsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a TestIamPermissionsRequest message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.iam.v1.TestIamPermissionsRequest
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.iam.v1.TestIamPermissionsRequest} TestIamPermissionsRequest
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    TestIamPermissionsRequest.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.iam.v1.TestIamPermissionsRequest();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.resource = reader.string();
                                    break;
                                }
                            case 2: {
                                    if (!(message.permissions && message.permissions.length))
                                        message.permissions = [];
                                    message.permissions.push(reader.string());
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a TestIamPermissionsRequest message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.iam.v1.TestIamPermissionsRequest
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.iam.v1.TestIamPermissionsRequest} TestIamPermissionsRequest
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    TestIamPermissionsRequest.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a TestIamPermissionsRequest message.
                     * @function verify
                     * @memberof google.iam.v1.TestIamPermissionsRequest
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    TestIamPermissionsRequest.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.resource != null && message.hasOwnProperty("resource"))
                            if (!$util.isString(message.resource))
                                return "resource: string expected";
                        if (message.permissions != null && message.hasOwnProperty("permissions")) {
                            if (!Array.isArray(message.permissions))
                                return "permissions: array expected";
                            for (var i = 0; i < message.permissions.length; ++i)
                                if (!$util.isString(message.permissions[i]))
                                    return "permissions: string[] expected";
                        }
                        return null;
                    };
    
                    /**
                     * Creates a TestIamPermissionsRequest message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.iam.v1.TestIamPermissionsRequest
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.iam.v1.TestIamPermissionsRequest} TestIamPermissionsRequest
                     */
                    TestIamPermissionsRequest.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.iam.v1.TestIamPermissionsRequest)
                            return object;
                        var message = new $root.google.iam.v1.TestIamPermissionsRequest();
                        if (object.resource != null)
                            message.resource = String(object.resource);
                        if (object.permissions) {
                            if (!Array.isArray(object.permissions))
                                throw TypeError(".google.iam.v1.TestIamPermissionsRequest.permissions: array expected");
                            message.permissions = [];
                            for (var i = 0; i < object.permissions.length; ++i)
                                message.permissions[i] = String(object.permissions[i]);
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a TestIamPermissionsRequest message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.iam.v1.TestIamPermissionsRequest
                     * @static
                     * @param {google.iam.v1.TestIamPermissionsRequest} message TestIamPermissionsRequest
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    TestIamPermissionsRequest.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.arrays || options.defaults)
                            object.permissions = [];
                        if (options.defaults)
                            object.resource = "";
                        if (message.resource != null && message.hasOwnProperty("resource"))
                            object.resource = message.resource;
                        if (message.permissions && message.permissions.length) {
                            object.permissions = [];
                            for (var j = 0; j < message.permissions.length; ++j)
                                object.permissions[j] = message.permissions[j];
                        }
                        return object;
                    };
    
                    /**
                     * Converts this TestIamPermissionsRequest to JSON.
                     * @function toJSON
                     * @memberof google.iam.v1.TestIamPermissionsRequest
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    TestIamPermissionsRequest.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for TestIamPermissionsRequest
                     * @function getTypeUrl
                     * @memberof google.iam.v1.TestIamPermissionsRequest
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    TestIamPermissionsRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.iam.v1.TestIamPermissionsRequest";
                    };
    
                    return TestIamPermissionsRequest;
                })();
    
                v1.TestIamPermissionsResponse = (function() {
    
                    /**
                     * Properties of a TestIamPermissionsResponse.
                     * @memberof google.iam.v1
                     * @interface ITestIamPermissionsResponse
                     * @property {Array.<string>|null} [permissions] TestIamPermissionsResponse permissions
                     */
    
                    /**
                     * Constructs a new TestIamPermissionsResponse.
                     * @memberof google.iam.v1
                     * @classdesc Represents a TestIamPermissionsResponse.
                     * @implements ITestIamPermissionsResponse
                     * @constructor
                     * @param {google.iam.v1.ITestIamPermissionsResponse=} [properties] Properties to set
                     */
                    function TestIamPermissionsResponse(properties) {
                        this.permissions = [];
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * TestIamPermissionsResponse permissions.
                     * @member {Array.<string>} permissions
                     * @memberof google.iam.v1.TestIamPermissionsResponse
                     * @instance
                     */
                    TestIamPermissionsResponse.prototype.permissions = $util.emptyArray;
    
                    /**
                     * Creates a new TestIamPermissionsResponse instance using the specified properties.
                     * @function create
                     * @memberof google.iam.v1.TestIamPermissionsResponse
                     * @static
                     * @param {google.iam.v1.ITestIamPermissionsResponse=} [properties] Properties to set
                     * @returns {google.iam.v1.TestIamPermissionsResponse} TestIamPermissionsResponse instance
                     */
                    TestIamPermissionsResponse.create = function create(properties) {
                        return new TestIamPermissionsResponse(properties);
                    };
    
                    /**
                     * Encodes the specified TestIamPermissionsResponse message. Does not implicitly {@link google.iam.v1.TestIamPermissionsResponse.verify|verify} messages.
                     * @function encode
                     * @memberof google.iam.v1.TestIamPermissionsResponse
                     * @static
                     * @param {google.iam.v1.ITestIamPermissionsResponse} message TestIamPermissionsResponse message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    TestIamPermissionsResponse.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.permissions != null && message.permissions.length)
                            for (var i = 0; i < message.permissions.length; ++i)
                                writer.uint32(/* id 1, wireType 2 =*/10).string(message.permissions[i]);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified TestIamPermissionsResponse message, length delimited. Does not implicitly {@link google.iam.v1.TestIamPermissionsResponse.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.iam.v1.TestIamPermissionsResponse
                     * @static
                     * @param {google.iam.v1.ITestIamPermissionsResponse} message TestIamPermissionsResponse message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    TestIamPermissionsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a TestIamPermissionsResponse message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.iam.v1.TestIamPermissionsResponse
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.iam.v1.TestIamPermissionsResponse} TestIamPermissionsResponse
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    TestIamPermissionsResponse.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.iam.v1.TestIamPermissionsResponse();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    if (!(message.permissions && message.permissions.length))
                                        message.permissions = [];
                                    message.permissions.push(reader.string());
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a TestIamPermissionsResponse message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.iam.v1.TestIamPermissionsResponse
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.iam.v1.TestIamPermissionsResponse} TestIamPermissionsResponse
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    TestIamPermissionsResponse.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a TestIamPermissionsResponse message.
                     * @function verify
                     * @memberof google.iam.v1.TestIamPermissionsResponse
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    TestIamPermissionsResponse.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.permissions != null && message.hasOwnProperty("permissions")) {
                            if (!Array.isArray(message.permissions))
                                return "permissions: array expected";
                            for (var i = 0; i < message.permissions.length; ++i)
                                if (!$util.isString(message.permissions[i]))
                                    return "permissions: string[] expected";
                        }
                        return null;
                    };
    
                    /**
                     * Creates a TestIamPermissionsResponse message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.iam.v1.TestIamPermissionsResponse
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.iam.v1.TestIamPermissionsResponse} TestIamPermissionsResponse
                     */
                    TestIamPermissionsResponse.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.iam.v1.TestIamPermissionsResponse)
                            return object;
                        var message = new $root.google.iam.v1.TestIamPermissionsResponse();
                        if (object.permissions) {
                            if (!Array.isArray(object.permissions))
                                throw TypeError(".google.iam.v1.TestIamPermissionsResponse.permissions: array expected");
                            message.permissions = [];
                            for (var i = 0; i < object.permissions.length; ++i)
                                message.permissions[i] = String(object.permissions[i]);
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a TestIamPermissionsResponse message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.iam.v1.TestIamPermissionsResponse
                     * @static
                     * @param {google.iam.v1.TestIamPermissionsResponse} message TestIamPermissionsResponse
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    TestIamPermissionsResponse.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.arrays || options.defaults)
                            object.permissions = [];
                        if (message.permissions && message.permissions.length) {
                            object.permissions = [];
                            for (var j = 0; j < message.permissions.length; ++j)
                                object.permissions[j] = message.permissions[j];
                        }
                        return object;
                    };
    
                    /**
                     * Converts this TestIamPermissionsResponse to JSON.
                     * @function toJSON
                     * @memberof google.iam.v1.TestIamPermissionsResponse
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    TestIamPermissionsResponse.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for TestIamPermissionsResponse
                     * @function getTypeUrl
                     * @memberof google.iam.v1.TestIamPermissionsResponse
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    TestIamPermissionsResponse.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.iam.v1.TestIamPermissionsResponse";
                    };
    
                    return TestIamPermissionsResponse;
                })();
    
                v1.GetPolicyOptions = (function() {
    
                    /**
                     * Properties of a GetPolicyOptions.
                     * @memberof google.iam.v1
                     * @interface IGetPolicyOptions
                     * @property {number|null} [requestedPolicyVersion] GetPolicyOptions requestedPolicyVersion
                     */
    
                    /**
                     * Constructs a new GetPolicyOptions.
                     * @memberof google.iam.v1
                     * @classdesc Represents a GetPolicyOptions.
                     * @implements IGetPolicyOptions
                     * @constructor
                     * @param {google.iam.v1.IGetPolicyOptions=} [properties] Properties to set
                     */
                    function GetPolicyOptions(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * GetPolicyOptions requestedPolicyVersion.
                     * @member {number} requestedPolicyVersion
                     * @memberof google.iam.v1.GetPolicyOptions
                     * @instance
                     */
                    GetPolicyOptions.prototype.requestedPolicyVersion = 0;
    
                    /**
                     * Creates a new GetPolicyOptions instance using the specified properties.
                     * @function create
                     * @memberof google.iam.v1.GetPolicyOptions
                     * @static
                     * @param {google.iam.v1.IGetPolicyOptions=} [properties] Properties to set
                     * @returns {google.iam.v1.GetPolicyOptions} GetPolicyOptions instance
                     */
                    GetPolicyOptions.create = function create(properties) {
                        return new GetPolicyOptions(properties);
                    };
    
                    /**
                     * Encodes the specified GetPolicyOptions message. Does not implicitly {@link google.iam.v1.GetPolicyOptions.verify|verify} messages.
                     * @function encode
                     * @memberof google.iam.v1.GetPolicyOptions
                     * @static
                     * @param {google.iam.v1.IGetPolicyOptions} message GetPolicyOptions message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    GetPolicyOptions.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.requestedPolicyVersion != null && Object.hasOwnProperty.call(message, "requestedPolicyVersion"))
                            writer.uint32(/* id 1, wireType 0 =*/8).int32(message.requestedPolicyVersion);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified GetPolicyOptions message, length delimited. Does not implicitly {@link google.iam.v1.GetPolicyOptions.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.iam.v1.GetPolicyOptions
                     * @static
                     * @param {google.iam.v1.IGetPolicyOptions} message GetPolicyOptions message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    GetPolicyOptions.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a GetPolicyOptions message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.iam.v1.GetPolicyOptions
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.iam.v1.GetPolicyOptions} GetPolicyOptions
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    GetPolicyOptions.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.iam.v1.GetPolicyOptions();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.requestedPolicyVersion = reader.int32();
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a GetPolicyOptions message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.iam.v1.GetPolicyOptions
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.iam.v1.GetPolicyOptions} GetPolicyOptions
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    GetPolicyOptions.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a GetPolicyOptions message.
                     * @function verify
                     * @memberof google.iam.v1.GetPolicyOptions
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    GetPolicyOptions.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.requestedPolicyVersion != null && message.hasOwnProperty("requestedPolicyVersion"))
                            if (!$util.isInteger(message.requestedPolicyVersion))
                                return "requestedPolicyVersion: integer expected";
                        return null;
                    };
    
                    /**
                     * Creates a GetPolicyOptions message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.iam.v1.GetPolicyOptions
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.iam.v1.GetPolicyOptions} GetPolicyOptions
                     */
                    GetPolicyOptions.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.iam.v1.GetPolicyOptions)
                            return object;
                        var message = new $root.google.iam.v1.GetPolicyOptions();
                        if (object.requestedPolicyVersion != null)
                            message.requestedPolicyVersion = object.requestedPolicyVersion | 0;
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a GetPolicyOptions message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.iam.v1.GetPolicyOptions
                     * @static
                     * @param {google.iam.v1.GetPolicyOptions} message GetPolicyOptions
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    GetPolicyOptions.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults)
                            object.requestedPolicyVersion = 0;
                        if (message.requestedPolicyVersion != null && message.hasOwnProperty("requestedPolicyVersion"))
                            object.requestedPolicyVersion = message.requestedPolicyVersion;
                        return object;
                    };
    
                    /**
                     * Converts this GetPolicyOptions to JSON.
                     * @function toJSON
                     * @memberof google.iam.v1.GetPolicyOptions
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    GetPolicyOptions.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for GetPolicyOptions
                     * @function getTypeUrl
                     * @memberof google.iam.v1.GetPolicyOptions
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    GetPolicyOptions.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.iam.v1.GetPolicyOptions";
                    };
    
                    return GetPolicyOptions;
                })();
    
                v1.Policy = (function() {
    
                    /**
                     * Properties of a Policy.
                     * @memberof google.iam.v1
                     * @interface IPolicy
                     * @property {number|null} [version] Policy version
                     * @property {Array.<google.iam.v1.IBinding>|null} [bindings] Policy bindings
                     * @property {Array.<google.iam.v1.IAuditConfig>|null} [auditConfigs] Policy auditConfigs
                     * @property {Uint8Array|null} [etag] Policy etag
                     */
    
                    /**
                     * Constructs a new Policy.
                     * @memberof google.iam.v1
                     * @classdesc Represents a Policy.
                     * @implements IPolicy
                     * @constructor
                     * @param {google.iam.v1.IPolicy=} [properties] Properties to set
                     */
                    function Policy(properties) {
                        this.bindings = [];
                        this.auditConfigs = [];
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * Policy version.
                     * @member {number} version
                     * @memberof google.iam.v1.Policy
                     * @instance
                     */
                    Policy.prototype.version = 0;
    
                    /**
                     * Policy bindings.
                     * @member {Array.<google.iam.v1.IBinding>} bindings
                     * @memberof google.iam.v1.Policy
                     * @instance
                     */
                    Policy.prototype.bindings = $util.emptyArray;
    
                    /**
                     * Policy auditConfigs.
                     * @member {Array.<google.iam.v1.IAuditConfig>} auditConfigs
                     * @memberof google.iam.v1.Policy
                     * @instance
                     */
                    Policy.prototype.auditConfigs = $util.emptyArray;
    
                    /**
                     * Policy etag.
                     * @member {Uint8Array} etag
                     * @memberof google.iam.v1.Policy
                     * @instance
                     */
                    Policy.prototype.etag = $util.newBuffer([]);
    
                    /**
                     * Creates a new Policy instance using the specified properties.
                     * @function create
                     * @memberof google.iam.v1.Policy
                     * @static
                     * @param {google.iam.v1.IPolicy=} [properties] Properties to set
                     * @returns {google.iam.v1.Policy} Policy instance
                     */
                    Policy.create = function create(properties) {
                        return new Policy(properties);
                    };
    
                    /**
                     * Encodes the specified Policy message. Does not implicitly {@link google.iam.v1.Policy.verify|verify} messages.
                     * @function encode
                     * @memberof google.iam.v1.Policy
                     * @static
                     * @param {google.iam.v1.IPolicy} message Policy message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    Policy.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.version != null && Object.hasOwnProperty.call(message, "version"))
                            writer.uint32(/* id 1, wireType 0 =*/8).int32(message.version);
                        if (message.etag != null && Object.hasOwnProperty.call(message, "etag"))
                            writer.uint32(/* id 3, wireType 2 =*/26).bytes(message.etag);
                        if (message.bindings != null && message.bindings.length)
                            for (var i = 0; i < message.bindings.length; ++i)
                                $root.google.iam.v1.Binding.encode(message.bindings[i], writer.uint32(/* id 4, wireType 2 =*/34).fork()).ldelim();
                        if (message.auditConfigs != null && message.auditConfigs.length)
                            for (var i = 0; i < message.auditConfigs.length; ++i)
                                $root.google.iam.v1.AuditConfig.encode(message.auditConfigs[i], writer.uint32(/* id 6, wireType 2 =*/50).fork()).ldelim();
                        return writer;
                    };
    
                    /**
                     * Encodes the specified Policy message, length delimited. Does not implicitly {@link google.iam.v1.Policy.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.iam.v1.Policy
                     * @static
                     * @param {google.iam.v1.IPolicy} message Policy message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    Policy.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a Policy message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.iam.v1.Policy
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.iam.v1.Policy} Policy
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    Policy.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.iam.v1.Policy();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.version = reader.int32();
                                    break;
                                }
                            case 4: {
                                    if (!(message.bindings && message.bindings.length))
                                        message.bindings = [];
                                    message.bindings.push($root.google.iam.v1.Binding.decode(reader, reader.uint32()));
                                    break;
                                }
                            case 6: {
                                    if (!(message.auditConfigs && message.auditConfigs.length))
                                        message.auditConfigs = [];
                                    message.auditConfigs.push($root.google.iam.v1.AuditConfig.decode(reader, reader.uint32()));
                                    break;
                                }
                            case 3: {
                                    message.etag = reader.bytes();
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a Policy message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.iam.v1.Policy
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.iam.v1.Policy} Policy
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    Policy.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a Policy message.
                     * @function verify
                     * @memberof google.iam.v1.Policy
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    Policy.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.version != null && message.hasOwnProperty("version"))
                            if (!$util.isInteger(message.version))
                                return "version: integer expected";
                        if (message.bindings != null && message.hasOwnProperty("bindings")) {
                            if (!Array.isArray(message.bindings))
                                return "bindings: array expected";
                            for (var i = 0; i < message.bindings.length; ++i) {
                                var error = $root.google.iam.v1.Binding.verify(message.bindings[i]);
                                if (error)
                                    return "bindings." + error;
                            }
                        }
                        if (message.auditConfigs != null && message.hasOwnProperty("auditConfigs")) {
                            if (!Array.isArray(message.auditConfigs))
                                return "auditConfigs: array expected";
                            for (var i = 0; i < message.auditConfigs.length; ++i) {
                                var error = $root.google.iam.v1.AuditConfig.verify(message.auditConfigs[i]);
                                if (error)
                                    return "auditConfigs." + error;
                            }
                        }
                        if (message.etag != null && message.hasOwnProperty("etag"))
                            if (!(message.etag && typeof message.etag.length === "number" || $util.isString(message.etag)))
                                return "etag: buffer expected";
                        return null;
                    };
    
                    /**
                     * Creates a Policy message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.iam.v1.Policy
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.iam.v1.Policy} Policy
                     */
                    Policy.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.iam.v1.Policy)
                            return object;
                        var message = new $root.google.iam.v1.Policy();
                        if (object.version != null)
                            message.version = object.version | 0;
                        if (object.bindings) {
                            if (!Array.isArray(object.bindings))
                                throw TypeError(".google.iam.v1.Policy.bindings: array expected");
                            message.bindings = [];
                            for (var i = 0; i < object.bindings.length; ++i) {
                                if (typeof object.bindings[i] !== "object")
                                    throw TypeError(".google.iam.v1.Policy.bindings: object expected");
                                message.bindings[i] = $root.google.iam.v1.Binding.fromObject(object.bindings[i]);
                            }
                        }
                        if (object.auditConfigs) {
                            if (!Array.isArray(object.auditConfigs))
                                throw TypeError(".google.iam.v1.Policy.auditConfigs: array expected");
                            message.auditConfigs = [];
                            for (var i = 0; i < object.auditConfigs.length; ++i) {
                                if (typeof object.auditConfigs[i] !== "object")
                                    throw TypeError(".google.iam.v1.Policy.auditConfigs: object expected");
                                message.auditConfigs[i] = $root.google.iam.v1.AuditConfig.fromObject(object.auditConfigs[i]);
                            }
                        }
                        if (object.etag != null)
                            if (typeof object.etag === "string")
                                $util.base64.decode(object.etag, message.etag = $util.newBuffer($util.base64.length(object.etag)), 0);
                            else if (object.etag.length >= 0)
                                message.etag = object.etag;
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a Policy message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.iam.v1.Policy
                     * @static
                     * @param {google.iam.v1.Policy} message Policy
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    Policy.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.arrays || options.defaults) {
                            object.bindings = [];
                            object.auditConfigs = [];
                        }
                        if (options.defaults) {
                            object.version = 0;
                            if (options.bytes === String)
                                object.etag = "";
                            else {
                                object.etag = [];
                                if (options.bytes !== Array)
                                    object.etag = $util.newBuffer(object.etag);
                            }
                        }
                        if (message.version != null && message.hasOwnProperty("version"))
                            object.version = message.version;
                        if (message.etag != null && message.hasOwnProperty("etag"))
                            object.etag = options.bytes === String ? $util.base64.encode(message.etag, 0, message.etag.length) : options.bytes === Array ? Array.prototype.slice.call(message.etag) : message.etag;
                        if (message.bindings && message.bindings.length) {
                            object.bindings = [];
                            for (var j = 0; j < message.bindings.length; ++j)
                                object.bindings[j] = $root.google.iam.v1.Binding.toObject(message.bindings[j], options);
                        }
                        if (message.auditConfigs && message.auditConfigs.length) {
                            object.auditConfigs = [];
                            for (var j = 0; j < message.auditConfigs.length; ++j)
                                object.auditConfigs[j] = $root.google.iam.v1.AuditConfig.toObject(message.auditConfigs[j], options);
                        }
                        return object;
                    };
    
                    /**
                     * Converts this Policy to JSON.
                     * @function toJSON
                     * @memberof google.iam.v1.Policy
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    Policy.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for Policy
                     * @function getTypeUrl
                     * @memberof google.iam.v1.Policy
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    Policy.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.iam.v1.Policy";
                    };
    
                    return Policy;
                })();
    
                v1.Binding = (function() {
    
                    /**
                     * Properties of a Binding.
                     * @memberof google.iam.v1
                     * @interface IBinding
                     * @property {string|null} [role] Binding role
                     * @property {Array.<string>|null} [members] Binding members
                     * @property {google.type.IExpr|null} [condition] Binding condition
                     */
    
                    /**
                     * Constructs a new Binding.
                     * @memberof google.iam.v1
                     * @classdesc Represents a Binding.
                     * @implements IBinding
                     * @constructor
                     * @param {google.iam.v1.IBinding=} [properties] Properties to set
                     */
                    function Binding(properties) {
                        this.members = [];
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * Binding role.
                     * @member {string} role
                     * @memberof google.iam.v1.Binding
                     * @instance
                     */
                    Binding.prototype.role = "";
    
                    /**
                     * Binding members.
                     * @member {Array.<string>} members
                     * @memberof google.iam.v1.Binding
                     * @instance
                     */
                    Binding.prototype.members = $util.emptyArray;
    
                    /**
                     * Binding condition.
                     * @member {google.type.IExpr|null|undefined} condition
                     * @memberof google.iam.v1.Binding
                     * @instance
                     */
                    Binding.prototype.condition = null;
    
                    /**
                     * Creates a new Binding instance using the specified properties.
                     * @function create
                     * @memberof google.iam.v1.Binding
                     * @static
                     * @param {google.iam.v1.IBinding=} [properties] Properties to set
                     * @returns {google.iam.v1.Binding} Binding instance
                     */
                    Binding.create = function create(properties) {
                        return new Binding(properties);
                    };
    
                    /**
                     * Encodes the specified Binding message. Does not implicitly {@link google.iam.v1.Binding.verify|verify} messages.
                     * @function encode
                     * @memberof google.iam.v1.Binding
                     * @static
                     * @param {google.iam.v1.IBinding} message Binding message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    Binding.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.role != null && Object.hasOwnProperty.call(message, "role"))
                            writer.uint32(/* id 1, wireType 2 =*/10).string(message.role);
                        if (message.members != null && message.members.length)
                            for (var i = 0; i < message.members.length; ++i)
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.members[i]);
                        if (message.condition != null && Object.hasOwnProperty.call(message, "condition"))
                            $root.google.type.Expr.encode(message.condition, writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                        return writer;
                    };
    
                    /**
                     * Encodes the specified Binding message, length delimited. Does not implicitly {@link google.iam.v1.Binding.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.iam.v1.Binding
                     * @static
                     * @param {google.iam.v1.IBinding} message Binding message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    Binding.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a Binding message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.iam.v1.Binding
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.iam.v1.Binding} Binding
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    Binding.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.iam.v1.Binding();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.role = reader.string();
                                    break;
                                }
                            case 2: {
                                    if (!(message.members && message.members.length))
                                        message.members = [];
                                    message.members.push(reader.string());
                                    break;
                                }
                            case 3: {
                                    message.condition = $root.google.type.Expr.decode(reader, reader.uint32());
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a Binding message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.iam.v1.Binding
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.iam.v1.Binding} Binding
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    Binding.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a Binding message.
                     * @function verify
                     * @memberof google.iam.v1.Binding
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    Binding.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.role != null && message.hasOwnProperty("role"))
                            if (!$util.isString(message.role))
                                return "role: string expected";
                        if (message.members != null && message.hasOwnProperty("members")) {
                            if (!Array.isArray(message.members))
                                return "members: array expected";
                            for (var i = 0; i < message.members.length; ++i)
                                if (!$util.isString(message.members[i]))
                                    return "members: string[] expected";
                        }
                        if (message.condition != null && message.hasOwnProperty("condition")) {
                            var error = $root.google.type.Expr.verify(message.condition);
                            if (error)
                                return "condition." + error;
                        }
                        return null;
                    };
    
                    /**
                     * Creates a Binding message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.iam.v1.Binding
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.iam.v1.Binding} Binding
                     */
                    Binding.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.iam.v1.Binding)
                            return object;
                        var message = new $root.google.iam.v1.Binding();
                        if (object.role != null)
                            message.role = String(object.role);
                        if (object.members) {
                            if (!Array.isArray(object.members))
                                throw TypeError(".google.iam.v1.Binding.members: array expected");
                            message.members = [];
                            for (var i = 0; i < object.members.length; ++i)
                                message.members[i] = String(object.members[i]);
                        }
                        if (object.condition != null) {
                            if (typeof object.condition !== "object")
                                throw TypeError(".google.iam.v1.Binding.condition: object expected");
                            message.condition = $root.google.type.Expr.fromObject(object.condition);
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a Binding message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.iam.v1.Binding
                     * @static
                     * @param {google.iam.v1.Binding} message Binding
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    Binding.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.arrays || options.defaults)
                            object.members = [];
                        if (options.defaults) {
                            object.role = "";
                            object.condition = null;
                        }
                        if (message.role != null && message.hasOwnProperty("role"))
                            object.role = message.role;
                        if (message.members && message.members.length) {
                            object.members = [];
                            for (var j = 0; j < message.members.length; ++j)
                                object.members[j] = message.members[j];
                        }
                        if (message.condition != null && message.hasOwnProperty("condition"))
                            object.condition = $root.google.type.Expr.toObject(message.condition, options);
                        return object;
                    };
    
                    /**
                     * Converts this Binding to JSON.
                     * @function toJSON
                     * @memberof google.iam.v1.Binding
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    Binding.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for Binding
                     * @function getTypeUrl
                     * @memberof google.iam.v1.Binding
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    Binding.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.iam.v1.Binding";
                    };
    
                    return Binding;
                })();
    
                v1.AuditConfig = (function() {
    
                    /**
                     * Properties of an AuditConfig.
                     * @memberof google.iam.v1
                     * @interface IAuditConfig
                     * @property {string|null} [service] AuditConfig service
                     * @property {Array.<google.iam.v1.IAuditLogConfig>|null} [auditLogConfigs] AuditConfig auditLogConfigs
                     */
    
                    /**
                     * Constructs a new AuditConfig.
                     * @memberof google.iam.v1
                     * @classdesc Represents an AuditConfig.
                     * @implements IAuditConfig
                     * @constructor
                     * @param {google.iam.v1.IAuditConfig=} [properties] Properties to set
                     */
                    function AuditConfig(properties) {
                        this.auditLogConfigs = [];
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * AuditConfig service.
                     * @member {string} service
                     * @memberof google.iam.v1.AuditConfig
                     * @instance
                     */
                    AuditConfig.prototype.service = "";
    
                    /**
                     * AuditConfig auditLogConfigs.
                     * @member {Array.<google.iam.v1.IAuditLogConfig>} auditLogConfigs
                     * @memberof google.iam.v1.AuditConfig
                     * @instance
                     */
                    AuditConfig.prototype.auditLogConfigs = $util.emptyArray;
    
                    /**
                     * Creates a new AuditConfig instance using the specified properties.
                     * @function create
                     * @memberof google.iam.v1.AuditConfig
                     * @static
                     * @param {google.iam.v1.IAuditConfig=} [properties] Properties to set
                     * @returns {google.iam.v1.AuditConfig} AuditConfig instance
                     */
                    AuditConfig.create = function create(properties) {
                        return new AuditConfig(properties);
                    };
    
                    /**
                     * Encodes the specified AuditConfig message. Does not implicitly {@link google.iam.v1.AuditConfig.verify|verify} messages.
                     * @function encode
                     * @memberof google.iam.v1.AuditConfig
                     * @static
                     * @param {google.iam.v1.IAuditConfig} message AuditConfig message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    AuditConfig.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.service != null && Object.hasOwnProperty.call(message, "service"))
                            writer.uint32(/* id 1, wireType 2 =*/10).string(message.service);
                        if (message.auditLogConfigs != null && message.auditLogConfigs.length)
                            for (var i = 0; i < message.auditLogConfigs.length; ++i)
                                $root.google.iam.v1.AuditLogConfig.encode(message.auditLogConfigs[i], writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                        return writer;
                    };
    
                    /**
                     * Encodes the specified AuditConfig message, length delimited. Does not implicitly {@link google.iam.v1.AuditConfig.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.iam.v1.AuditConfig
                     * @static
                     * @param {google.iam.v1.IAuditConfig} message AuditConfig message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    AuditConfig.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes an AuditConfig message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.iam.v1.AuditConfig
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.iam.v1.AuditConfig} AuditConfig
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    AuditConfig.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.iam.v1.AuditConfig();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.service = reader.string();
                                    break;
                                }
                            case 3: {
                                    if (!(message.auditLogConfigs && message.auditLogConfigs.length))
                                        message.auditLogConfigs = [];
                                    message.auditLogConfigs.push($root.google.iam.v1.AuditLogConfig.decode(reader, reader.uint32()));
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes an AuditConfig message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.iam.v1.AuditConfig
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.iam.v1.AuditConfig} AuditConfig
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    AuditConfig.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies an AuditConfig message.
                     * @function verify
                     * @memberof google.iam.v1.AuditConfig
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    AuditConfig.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.service != null && message.hasOwnProperty("service"))
                            if (!$util.isString(message.service))
                                return "service: string expected";
                        if (message.auditLogConfigs != null && message.hasOwnProperty("auditLogConfigs")) {
                            if (!Array.isArray(message.auditLogConfigs))
                                return "auditLogConfigs: array expected";
                            for (var i = 0; i < message.auditLogConfigs.length; ++i) {
                                var error = $root.google.iam.v1.AuditLogConfig.verify(message.auditLogConfigs[i]);
                                if (error)
                                    return "auditLogConfigs." + error;
                            }
                        }
                        return null;
                    };
    
                    /**
                     * Creates an AuditConfig message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.iam.v1.AuditConfig
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.iam.v1.AuditConfig} AuditConfig
                     */
                    AuditConfig.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.iam.v1.AuditConfig)
                            return object;
                        var message = new $root.google.iam.v1.AuditConfig();
                        if (object.service != null)
                            message.service = String(object.service);
                        if (object.auditLogConfigs) {
                            if (!Array.isArray(object.auditLogConfigs))
                                throw TypeError(".google.iam.v1.AuditConfig.auditLogConfigs: array expected");
                            message.auditLogConfigs = [];
                            for (var i = 0; i < object.auditLogConfigs.length; ++i) {
                                if (typeof object.auditLogConfigs[i] !== "object")
                                    throw TypeError(".google.iam.v1.AuditConfig.auditLogConfigs: object expected");
                                message.auditLogConfigs[i] = $root.google.iam.v1.AuditLogConfig.fromObject(object.auditLogConfigs[i]);
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from an AuditConfig message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.iam.v1.AuditConfig
                     * @static
                     * @param {google.iam.v1.AuditConfig} message AuditConfig
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    AuditConfig.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.arrays || options.defaults)
                            object.auditLogConfigs = [];
                        if (options.defaults)
                            object.service = "";
                        if (message.service != null && message.hasOwnProperty("service"))
                            object.service = message.service;
                        if (message.auditLogConfigs && message.auditLogConfigs.length) {
                            object.auditLogConfigs = [];
                            for (var j = 0; j < message.auditLogConfigs.length; ++j)
                                object.auditLogConfigs[j] = $root.google.iam.v1.AuditLogConfig.toObject(message.auditLogConfigs[j], options);
                        }
                        return object;
                    };
    
                    /**
                     * Converts this AuditConfig to JSON.
                     * @function toJSON
                     * @memberof google.iam.v1.AuditConfig
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    AuditConfig.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for AuditConfig
                     * @function getTypeUrl
                     * @memberof google.iam.v1.AuditConfig
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    AuditConfig.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.iam.v1.AuditConfig";
                    };
    
                    return AuditConfig;
                })();
    
                v1.AuditLogConfig = (function() {
    
                    /**
                     * Properties of an AuditLogConfig.
                     * @memberof google.iam.v1
                     * @interface IAuditLogConfig
                     * @property {google.iam.v1.AuditLogConfig.LogType|null} [logType] AuditLogConfig logType
                     * @property {Array.<string>|null} [exemptedMembers] AuditLogConfig exemptedMembers
                     */
    
                    /**
                     * Constructs a new AuditLogConfig.
                     * @memberof google.iam.v1
                     * @classdesc Represents an AuditLogConfig.
                     * @implements IAuditLogConfig
                     * @constructor
                     * @param {google.iam.v1.IAuditLogConfig=} [properties] Properties to set
                     */
                    function AuditLogConfig(properties) {
                        this.exemptedMembers = [];
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * AuditLogConfig logType.
                     * @member {google.iam.v1.AuditLogConfig.LogType} logType
                     * @memberof google.iam.v1.AuditLogConfig
                     * @instance
                     */
                    AuditLogConfig.prototype.logType = 0;
    
                    /**
                     * AuditLogConfig exemptedMembers.
                     * @member {Array.<string>} exemptedMembers
                     * @memberof google.iam.v1.AuditLogConfig
                     * @instance
                     */
                    AuditLogConfig.prototype.exemptedMembers = $util.emptyArray;
    
                    /**
                     * Creates a new AuditLogConfig instance using the specified properties.
                     * @function create
                     * @memberof google.iam.v1.AuditLogConfig
                     * @static
                     * @param {google.iam.v1.IAuditLogConfig=} [properties] Properties to set
                     * @returns {google.iam.v1.AuditLogConfig} AuditLogConfig instance
                     */
                    AuditLogConfig.create = function create(properties) {
                        return new AuditLogConfig(properties);
                    };
    
                    /**
                     * Encodes the specified AuditLogConfig message. Does not implicitly {@link google.iam.v1.AuditLogConfig.verify|verify} messages.
                     * @function encode
                     * @memberof google.iam.v1.AuditLogConfig
                     * @static
                     * @param {google.iam.v1.IAuditLogConfig} message AuditLogConfig message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    AuditLogConfig.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.logType != null && Object.hasOwnProperty.call(message, "logType"))
                            writer.uint32(/* id 1, wireType 0 =*/8).int32(message.logType);
                        if (message.exemptedMembers != null && message.exemptedMembers.length)
                            for (var i = 0; i < message.exemptedMembers.length; ++i)
                                writer.uint32(/* id 2, wireType 2 =*/18).string(message.exemptedMembers[i]);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified AuditLogConfig message, length delimited. Does not implicitly {@link google.iam.v1.AuditLogConfig.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.iam.v1.AuditLogConfig
                     * @static
                     * @param {google.iam.v1.IAuditLogConfig} message AuditLogConfig message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    AuditLogConfig.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes an AuditLogConfig message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.iam.v1.AuditLogConfig
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.iam.v1.AuditLogConfig} AuditLogConfig
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    AuditLogConfig.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.iam.v1.AuditLogConfig();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.logType = reader.int32();
                                    break;
                                }
                            case 2: {
                                    if (!(message.exemptedMembers && message.exemptedMembers.length))
                                        message.exemptedMembers = [];
                                    message.exemptedMembers.push(reader.string());
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes an AuditLogConfig message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.iam.v1.AuditLogConfig
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.iam.v1.AuditLogConfig} AuditLogConfig
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    AuditLogConfig.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies an AuditLogConfig message.
                     * @function verify
                     * @memberof google.iam.v1.AuditLogConfig
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    AuditLogConfig.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.logType != null && message.hasOwnProperty("logType"))
                            switch (message.logType) {
                            default:
                                return "logType: enum value expected";
                            case 0:
                            case 1:
                            case 2:
                            case 3:
                                break;
                            }
                        if (message.exemptedMembers != null && message.hasOwnProperty("exemptedMembers")) {
                            if (!Array.isArray(message.exemptedMembers))
                                return "exemptedMembers: array expected";
                            for (var i = 0; i < message.exemptedMembers.length; ++i)
                                if (!$util.isString(message.exemptedMembers[i]))
                                    return "exemptedMembers: string[] expected";
                        }
                        return null;
                    };
    
                    /**
                     * Creates an AuditLogConfig message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.iam.v1.AuditLogConfig
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.iam.v1.AuditLogConfig} AuditLogConfig
                     */
                    AuditLogConfig.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.iam.v1.AuditLogConfig)
                            return object;
                        var message = new $root.google.iam.v1.AuditLogConfig();
                        switch (object.logType) {
                        default:
                            if (typeof object.logType === "number") {
                                message.logType = object.logType;
                                break;
                            }
                            break;
                        case "LOG_TYPE_UNSPECIFIED":
                        case 0:
                            message.logType = 0;
                            break;
                        case "ADMIN_READ":
                        case 1:
                            message.logType = 1;
                            break;
                        case "DATA_WRITE":
                        case 2:
                            message.logType = 2;
                            break;
                        case "DATA_READ":
                        case 3:
                            message.logType = 3;
                            break;
                        }
                        if (object.exemptedMembers) {
                            if (!Array.isArray(object.exemptedMembers))
                                throw TypeError(".google.iam.v1.AuditLogConfig.exemptedMembers: array expected");
                            message.exemptedMembers = [];
                            for (var i = 0; i < object.exemptedMembers.length; ++i)
                                message.exemptedMembers[i] = String(object.exemptedMembers[i]);
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from an AuditLogConfig message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.iam.v1.AuditLogConfig
                     * @static
                     * @param {google.iam.v1.AuditLogConfig} message AuditLogConfig
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    AuditLogConfig.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.arrays || options.defaults)
                            object.exemptedMembers = [];
                        if (options.defaults)
                            object.logType = options.enums === String ? "LOG_TYPE_UNSPECIFIED" : 0;
                        if (message.logType != null && message.hasOwnProperty("logType"))
                            object.logType = options.enums === String ? $root.google.iam.v1.AuditLogConfig.LogType[message.logType] === undefined ? message.logType : $root.google.iam.v1.AuditLogConfig.LogType[message.logType] : message.logType;
                        if (message.exemptedMembers && message.exemptedMembers.length) {
                            object.exemptedMembers = [];
                            for (var j = 0; j < message.exemptedMembers.length; ++j)
                                object.exemptedMembers[j] = message.exemptedMembers[j];
                        }
                        return object;
                    };
    
                    /**
                     * Converts this AuditLogConfig to JSON.
                     * @function toJSON
                     * @memberof google.iam.v1.AuditLogConfig
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    AuditLogConfig.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for AuditLogConfig
                     * @function getTypeUrl
                     * @memberof google.iam.v1.AuditLogConfig
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    AuditLogConfig.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.iam.v1.AuditLogConfig";
                    };
    
                    /**
                     * LogType enum.
                     * @name google.iam.v1.AuditLogConfig.LogType
                     * @enum {number}
                     * @property {number} LOG_TYPE_UNSPECIFIED=0 LOG_TYPE_UNSPECIFIED value
                     * @property {number} ADMIN_READ=1 ADMIN_READ value
                     * @property {number} DATA_WRITE=2 DATA_WRITE value
                     * @property {number} DATA_READ=3 DATA_READ value
                     */
                    AuditLogConfig.LogType = (function() {
                        var valuesById = {}, values = Object.create(valuesById);
                        values[valuesById[0] = "LOG_TYPE_UNSPECIFIED"] = 0;
                        values[valuesById[1] = "ADMIN_READ"] = 1;
                        values[valuesById[2] = "DATA_WRITE"] = 2;
                        values[valuesById[3] = "DATA_READ"] = 3;
                        return values;
                    })();
    
                    return AuditLogConfig;
                })();
    
                v1.PolicyDelta = (function() {
    
                    /**
                     * Properties of a PolicyDelta.
                     * @memberof google.iam.v1
                     * @interface IPolicyDelta
                     * @property {Array.<google.iam.v1.IBindingDelta>|null} [bindingDeltas] PolicyDelta bindingDeltas
                     * @property {Array.<google.iam.v1.IAuditConfigDelta>|null} [auditConfigDeltas] PolicyDelta auditConfigDeltas
                     */
    
                    /**
                     * Constructs a new PolicyDelta.
                     * @memberof google.iam.v1
                     * @classdesc Represents a PolicyDelta.
                     * @implements IPolicyDelta
                     * @constructor
                     * @param {google.iam.v1.IPolicyDelta=} [properties] Properties to set
                     */
                    function PolicyDelta(properties) {
                        this.bindingDeltas = [];
                        this.auditConfigDeltas = [];
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * PolicyDelta bindingDeltas.
                     * @member {Array.<google.iam.v1.IBindingDelta>} bindingDeltas
                     * @memberof google.iam.v1.PolicyDelta
                     * @instance
                     */
                    PolicyDelta.prototype.bindingDeltas = $util.emptyArray;
    
                    /**
                     * PolicyDelta auditConfigDeltas.
                     * @member {Array.<google.iam.v1.IAuditConfigDelta>} auditConfigDeltas
                     * @memberof google.iam.v1.PolicyDelta
                     * @instance
                     */
                    PolicyDelta.prototype.auditConfigDeltas = $util.emptyArray;
    
                    /**
                     * Creates a new PolicyDelta instance using the specified properties.
                     * @function create
                     * @memberof google.iam.v1.PolicyDelta
                     * @static
                     * @param {google.iam.v1.IPolicyDelta=} [properties] Properties to set
                     * @returns {google.iam.v1.PolicyDelta} PolicyDelta instance
                     */
                    PolicyDelta.create = function create(properties) {
                        return new PolicyDelta(properties);
                    };
    
                    /**
                     * Encodes the specified PolicyDelta message. Does not implicitly {@link google.iam.v1.PolicyDelta.verify|verify} messages.
                     * @function encode
                     * @memberof google.iam.v1.PolicyDelta
                     * @static
                     * @param {google.iam.v1.IPolicyDelta} message PolicyDelta message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    PolicyDelta.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.bindingDeltas != null && message.bindingDeltas.length)
                            for (var i = 0; i < message.bindingDeltas.length; ++i)
                                $root.google.iam.v1.BindingDelta.encode(message.bindingDeltas[i], writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                        if (message.auditConfigDeltas != null && message.auditConfigDeltas.length)
                            for (var i = 0; i < message.auditConfigDeltas.length; ++i)
                                $root.google.iam.v1.AuditConfigDelta.encode(message.auditConfigDeltas[i], writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                        return writer;
                    };
    
                    /**
                     * Encodes the specified PolicyDelta message, length delimited. Does not implicitly {@link google.iam.v1.PolicyDelta.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.iam.v1.PolicyDelta
                     * @static
                     * @param {google.iam.v1.IPolicyDelta} message PolicyDelta message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    PolicyDelta.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a PolicyDelta message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.iam.v1.PolicyDelta
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.iam.v1.PolicyDelta} PolicyDelta
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    PolicyDelta.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.iam.v1.PolicyDelta();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    if (!(message.bindingDeltas && message.bindingDeltas.length))
                                        message.bindingDeltas = [];
                                    message.bindingDeltas.push($root.google.iam.v1.BindingDelta.decode(reader, reader.uint32()));
                                    break;
                                }
                            case 2: {
                                    if (!(message.auditConfigDeltas && message.auditConfigDeltas.length))
                                        message.auditConfigDeltas = [];
                                    message.auditConfigDeltas.push($root.google.iam.v1.AuditConfigDelta.decode(reader, reader.uint32()));
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a PolicyDelta message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.iam.v1.PolicyDelta
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.iam.v1.PolicyDelta} PolicyDelta
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    PolicyDelta.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a PolicyDelta message.
                     * @function verify
                     * @memberof google.iam.v1.PolicyDelta
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    PolicyDelta.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.bindingDeltas != null && message.hasOwnProperty("bindingDeltas")) {
                            if (!Array.isArray(message.bindingDeltas))
                                return "bindingDeltas: array expected";
                            for (var i = 0; i < message.bindingDeltas.length; ++i) {
                                var error = $root.google.iam.v1.BindingDelta.verify(message.bindingDeltas[i]);
                                if (error)
                                    return "bindingDeltas." + error;
                            }
                        }
                        if (message.auditConfigDeltas != null && message.hasOwnProperty("auditConfigDeltas")) {
                            if (!Array.isArray(message.auditConfigDeltas))
                                return "auditConfigDeltas: array expected";
                            for (var i = 0; i < message.auditConfigDeltas.length; ++i) {
                                var error = $root.google.iam.v1.AuditConfigDelta.verify(message.auditConfigDeltas[i]);
                                if (error)
                                    return "auditConfigDeltas." + error;
                            }
                        }
                        return null;
                    };
    
                    /**
                     * Creates a PolicyDelta message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.iam.v1.PolicyDelta
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.iam.v1.PolicyDelta} PolicyDelta
                     */
                    PolicyDelta.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.iam.v1.PolicyDelta)
                            return object;
                        var message = new $root.google.iam.v1.PolicyDelta();
                        if (object.bindingDeltas) {
                            if (!Array.isArray(object.bindingDeltas))
                                throw TypeError(".google.iam.v1.PolicyDelta.bindingDeltas: array expected");
                            message.bindingDeltas = [];
                            for (var i = 0; i < object.bindingDeltas.length; ++i) {
                                if (typeof object.bindingDeltas[i] !== "object")
                                    throw TypeError(".google.iam.v1.PolicyDelta.bindingDeltas: object expected");
                                message.bindingDeltas[i] = $root.google.iam.v1.BindingDelta.fromObject(object.bindingDeltas[i]);
                            }
                        }
                        if (object.auditConfigDeltas) {
                            if (!Array.isArray(object.auditConfigDeltas))
                                throw TypeError(".google.iam.v1.PolicyDelta.auditConfigDeltas: array expected");
                            message.auditConfigDeltas = [];
                            for (var i = 0; i < object.auditConfigDeltas.length; ++i) {
                                if (typeof object.auditConfigDeltas[i] !== "object")
                                    throw TypeError(".google.iam.v1.PolicyDelta.auditConfigDeltas: object expected");
                                message.auditConfigDeltas[i] = $root.google.iam.v1.AuditConfigDelta.fromObject(object.auditConfigDeltas[i]);
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a PolicyDelta message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.iam.v1.PolicyDelta
                     * @static
                     * @param {google.iam.v1.PolicyDelta} message PolicyDelta
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    PolicyDelta.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.arrays || options.defaults) {
                            object.bindingDeltas = [];
                            object.auditConfigDeltas = [];
                        }
                        if (message.bindingDeltas && message.bindingDeltas.length) {
                            object.bindingDeltas = [];
                            for (var j = 0; j < message.bindingDeltas.length; ++j)
                                object.bindingDeltas[j] = $root.google.iam.v1.BindingDelta.toObject(message.bindingDeltas[j], options);
                        }
                        if (message.auditConfigDeltas && message.auditConfigDeltas.length) {
                            object.auditConfigDeltas = [];
                            for (var j = 0; j < message.auditConfigDeltas.length; ++j)
                                object.auditConfigDeltas[j] = $root.google.iam.v1.AuditConfigDelta.toObject(message.auditConfigDeltas[j], options);
                        }
                        return object;
                    };
    
                    /**
                     * Converts this PolicyDelta to JSON.
                     * @function toJSON
                     * @memberof google.iam.v1.PolicyDelta
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    PolicyDelta.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for PolicyDelta
                     * @function getTypeUrl
                     * @memberof google.iam.v1.PolicyDelta
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    PolicyDelta.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.iam.v1.PolicyDelta";
                    };
    
                    return PolicyDelta;
                })();
    
                v1.BindingDelta = (function() {
    
                    /**
                     * Properties of a BindingDelta.
                     * @memberof google.iam.v1
                     * @interface IBindingDelta
                     * @property {google.iam.v1.BindingDelta.Action|null} [action] BindingDelta action
                     * @property {string|null} [role] BindingDelta role
                     * @property {string|null} [member] BindingDelta member
                     * @property {google.type.IExpr|null} [condition] BindingDelta condition
                     */
    
                    /**
                     * Constructs a new BindingDelta.
                     * @memberof google.iam.v1
                     * @classdesc Represents a BindingDelta.
                     * @implements IBindingDelta
                     * @constructor
                     * @param {google.iam.v1.IBindingDelta=} [properties] Properties to set
                     */
                    function BindingDelta(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * BindingDelta action.
                     * @member {google.iam.v1.BindingDelta.Action} action
                     * @memberof google.iam.v1.BindingDelta
                     * @instance
                     */
                    BindingDelta.prototype.action = 0;
    
                    /**
                     * BindingDelta role.
                     * @member {string} role
                     * @memberof google.iam.v1.BindingDelta
                     * @instance
                     */
                    BindingDelta.prototype.role = "";
    
                    /**
                     * BindingDelta member.
                     * @member {string} member
                     * @memberof google.iam.v1.BindingDelta
                     * @instance
                     */
                    BindingDelta.prototype.member = "";
    
                    /**
                     * BindingDelta condition.
                     * @member {google.type.IExpr|null|undefined} condition
                     * @memberof google.iam.v1.BindingDelta
                     * @instance
                     */
                    BindingDelta.prototype.condition = null;
    
                    /**
                     * Creates a new BindingDelta instance using the specified properties.
                     * @function create
                     * @memberof google.iam.v1.BindingDelta
                     * @static
                     * @param {google.iam.v1.IBindingDelta=} [properties] Properties to set
                     * @returns {google.iam.v1.BindingDelta} BindingDelta instance
                     */
                    BindingDelta.create = function create(properties) {
                        return new BindingDelta(properties);
                    };
    
                    /**
                     * Encodes the specified BindingDelta message. Does not implicitly {@link google.iam.v1.BindingDelta.verify|verify} messages.
                     * @function encode
                     * @memberof google.iam.v1.BindingDelta
                     * @static
                     * @param {google.iam.v1.IBindingDelta} message BindingDelta message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    BindingDelta.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.action != null && Object.hasOwnProperty.call(message, "action"))
                            writer.uint32(/* id 1, wireType 0 =*/8).int32(message.action);
                        if (message.role != null && Object.hasOwnProperty.call(message, "role"))
                            writer.uint32(/* id 2, wireType 2 =*/18).string(message.role);
                        if (message.member != null && Object.hasOwnProperty.call(message, "member"))
                            writer.uint32(/* id 3, wireType 2 =*/26).string(message.member);
                        if (message.condition != null && Object.hasOwnProperty.call(message, "condition"))
                            $root.google.type.Expr.encode(message.condition, writer.uint32(/* id 4, wireType 2 =*/34).fork()).ldelim();
                        return writer;
                    };
    
                    /**
                     * Encodes the specified BindingDelta message, length delimited. Does not implicitly {@link google.iam.v1.BindingDelta.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.iam.v1.BindingDelta
                     * @static
                     * @param {google.iam.v1.IBindingDelta} message BindingDelta message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    BindingDelta.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes a BindingDelta message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.iam.v1.BindingDelta
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.iam.v1.BindingDelta} BindingDelta
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    BindingDelta.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.iam.v1.BindingDelta();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.action = reader.int32();
                                    break;
                                }
                            case 2: {
                                    message.role = reader.string();
                                    break;
                                }
                            case 3: {
                                    message.member = reader.string();
                                    break;
                                }
                            case 4: {
                                    message.condition = $root.google.type.Expr.decode(reader, reader.uint32());
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes a BindingDelta message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.iam.v1.BindingDelta
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.iam.v1.BindingDelta} BindingDelta
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    BindingDelta.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies a BindingDelta message.
                     * @function verify
                     * @memberof google.iam.v1.BindingDelta
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    BindingDelta.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.action != null && message.hasOwnProperty("action"))
                            switch (message.action) {
                            default:
                                return "action: enum value expected";
                            case 0:
                            case 1:
                            case 2:
                                break;
                            }
                        if (message.role != null && message.hasOwnProperty("role"))
                            if (!$util.isString(message.role))
                                return "role: string expected";
                        if (message.member != null && message.hasOwnProperty("member"))
                            if (!$util.isString(message.member))
                                return "member: string expected";
                        if (message.condition != null && message.hasOwnProperty("condition")) {
                            var error = $root.google.type.Expr.verify(message.condition);
                            if (error)
                                return "condition." + error;
                        }
                        return null;
                    };
    
                    /**
                     * Creates a BindingDelta message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.iam.v1.BindingDelta
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.iam.v1.BindingDelta} BindingDelta
                     */
                    BindingDelta.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.iam.v1.BindingDelta)
                            return object;
                        var message = new $root.google.iam.v1.BindingDelta();
                        switch (object.action) {
                        default:
                            if (typeof object.action === "number") {
                                message.action = object.action;
                                break;
                            }
                            break;
                        case "ACTION_UNSPECIFIED":
                        case 0:
                            message.action = 0;
                            break;
                        case "ADD":
                        case 1:
                            message.action = 1;
                            break;
                        case "REMOVE":
                        case 2:
                            message.action = 2;
                            break;
                        }
                        if (object.role != null)
                            message.role = String(object.role);
                        if (object.member != null)
                            message.member = String(object.member);
                        if (object.condition != null) {
                            if (typeof object.condition !== "object")
                                throw TypeError(".google.iam.v1.BindingDelta.condition: object expected");
                            message.condition = $root.google.type.Expr.fromObject(object.condition);
                        }
                        return message;
                    };
    
                    /**
                     * Creates a plain object from a BindingDelta message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.iam.v1.BindingDelta
                     * @static
                     * @param {google.iam.v1.BindingDelta} message BindingDelta
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    BindingDelta.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults) {
                            object.action = options.enums === String ? "ACTION_UNSPECIFIED" : 0;
                            object.role = "";
                            object.member = "";
                            object.condition = null;
                        }
                        if (message.action != null && message.hasOwnProperty("action"))
                            object.action = options.enums === String ? $root.google.iam.v1.BindingDelta.Action[message.action] === undefined ? message.action : $root.google.iam.v1.BindingDelta.Action[message.action] : message.action;
                        if (message.role != null && message.hasOwnProperty("role"))
                            object.role = message.role;
                        if (message.member != null && message.hasOwnProperty("member"))
                            object.member = message.member;
                        if (message.condition != null && message.hasOwnProperty("condition"))
                            object.condition = $root.google.type.Expr.toObject(message.condition, options);
                        return object;
                    };
    
                    /**
                     * Converts this BindingDelta to JSON.
                     * @function toJSON
                     * @memberof google.iam.v1.BindingDelta
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    BindingDelta.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for BindingDelta
                     * @function getTypeUrl
                     * @memberof google.iam.v1.BindingDelta
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    BindingDelta.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.iam.v1.BindingDelta";
                    };
    
                    /**
                     * Action enum.
                     * @name google.iam.v1.BindingDelta.Action
                     * @enum {number}
                     * @property {number} ACTION_UNSPECIFIED=0 ACTION_UNSPECIFIED value
                     * @property {number} ADD=1 ADD value
                     * @property {number} REMOVE=2 REMOVE value
                     */
                    BindingDelta.Action = (function() {
                        var valuesById = {}, values = Object.create(valuesById);
                        values[valuesById[0] = "ACTION_UNSPECIFIED"] = 0;
                        values[valuesById[1] = "ADD"] = 1;
                        values[valuesById[2] = "REMOVE"] = 2;
                        return values;
                    })();
    
                    return BindingDelta;
                })();
    
                v1.AuditConfigDelta = (function() {
    
                    /**
                     * Properties of an AuditConfigDelta.
                     * @memberof google.iam.v1
                     * @interface IAuditConfigDelta
                     * @property {google.iam.v1.AuditConfigDelta.Action|null} [action] AuditConfigDelta action
                     * @property {string|null} [service] AuditConfigDelta service
                     * @property {string|null} [exemptedMember] AuditConfigDelta exemptedMember
                     * @property {string|null} [logType] AuditConfigDelta logType
                     */
    
                    /**
                     * Constructs a new AuditConfigDelta.
                     * @memberof google.iam.v1
                     * @classdesc Represents an AuditConfigDelta.
                     * @implements IAuditConfigDelta
                     * @constructor
                     * @param {google.iam.v1.IAuditConfigDelta=} [properties] Properties to set
                     */
                    function AuditConfigDelta(properties) {
                        if (properties)
                            for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                                if (properties[keys[i]] != null)
                                    this[keys[i]] = properties[keys[i]];
                    }
    
                    /**
                     * AuditConfigDelta action.
                     * @member {google.iam.v1.AuditConfigDelta.Action} action
                     * @memberof google.iam.v1.AuditConfigDelta
                     * @instance
                     */
                    AuditConfigDelta.prototype.action = 0;
    
                    /**
                     * AuditConfigDelta service.
                     * @member {string} service
                     * @memberof google.iam.v1.AuditConfigDelta
                     * @instance
                     */
                    AuditConfigDelta.prototype.service = "";
    
                    /**
                     * AuditConfigDelta exemptedMember.
                     * @member {string} exemptedMember
                     * @memberof google.iam.v1.AuditConfigDelta
                     * @instance
                     */
                    AuditConfigDelta.prototype.exemptedMember = "";
    
                    /**
                     * AuditConfigDelta logType.
                     * @member {string} logType
                     * @memberof google.iam.v1.AuditConfigDelta
                     * @instance
                     */
                    AuditConfigDelta.prototype.logType = "";
    
                    /**
                     * Creates a new AuditConfigDelta instance using the specified properties.
                     * @function create
                     * @memberof google.iam.v1.AuditConfigDelta
                     * @static
                     * @param {google.iam.v1.IAuditConfigDelta=} [properties] Properties to set
                     * @returns {google.iam.v1.AuditConfigDelta} AuditConfigDelta instance
                     */
                    AuditConfigDelta.create = function create(properties) {
                        return new AuditConfigDelta(properties);
                    };
    
                    /**
                     * Encodes the specified AuditConfigDelta message. Does not implicitly {@link google.iam.v1.AuditConfigDelta.verify|verify} messages.
                     * @function encode
                     * @memberof google.iam.v1.AuditConfigDelta
                     * @static
                     * @param {google.iam.v1.IAuditConfigDelta} message AuditConfigDelta message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    AuditConfigDelta.encode = function encode(message, writer) {
                        if (!writer)
                            writer = $Writer.create();
                        if (message.action != null && Object.hasOwnProperty.call(message, "action"))
                            writer.uint32(/* id 1, wireType 0 =*/8).int32(message.action);
                        if (message.service != null && Object.hasOwnProperty.call(message, "service"))
                            writer.uint32(/* id 2, wireType 2 =*/18).string(message.service);
                        if (message.exemptedMember != null && Object.hasOwnProperty.call(message, "exemptedMember"))
                            writer.uint32(/* id 3, wireType 2 =*/26).string(message.exemptedMember);
                        if (message.logType != null && Object.hasOwnProperty.call(message, "logType"))
                            writer.uint32(/* id 4, wireType 2 =*/34).string(message.logType);
                        return writer;
                    };
    
                    /**
                     * Encodes the specified AuditConfigDelta message, length delimited. Does not implicitly {@link google.iam.v1.AuditConfigDelta.verify|verify} messages.
                     * @function encodeDelimited
                     * @memberof google.iam.v1.AuditConfigDelta
                     * @static
                     * @param {google.iam.v1.IAuditConfigDelta} message AuditConfigDelta message or plain object to encode
                     * @param {$protobuf.Writer} [writer] Writer to encode to
                     * @returns {$protobuf.Writer} Writer
                     */
                    AuditConfigDelta.encodeDelimited = function encodeDelimited(message, writer) {
                        return this.encode(message, writer).ldelim();
                    };
    
                    /**
                     * Decodes an AuditConfigDelta message from the specified reader or buffer.
                     * @function decode
                     * @memberof google.iam.v1.AuditConfigDelta
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @param {number} [length] Message length if known beforehand
                     * @returns {google.iam.v1.AuditConfigDelta} AuditConfigDelta
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    AuditConfigDelta.decode = function decode(reader, length) {
                        if (!(reader instanceof $Reader))
                            reader = $Reader.create(reader);
                        var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.iam.v1.AuditConfigDelta();
                        while (reader.pos < end) {
                            var tag = reader.uint32();
                            switch (tag >>> 3) {
                            case 1: {
                                    message.action = reader.int32();
                                    break;
                                }
                            case 2: {
                                    message.service = reader.string();
                                    break;
                                }
                            case 3: {
                                    message.exemptedMember = reader.string();
                                    break;
                                }
                            case 4: {
                                    message.logType = reader.string();
                                    break;
                                }
                            default:
                                reader.skipType(tag & 7);
                                break;
                            }
                        }
                        return message;
                    };
    
                    /**
                     * Decodes an AuditConfigDelta message from the specified reader or buffer, length delimited.
                     * @function decodeDelimited
                     * @memberof google.iam.v1.AuditConfigDelta
                     * @static
                     * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                     * @returns {google.iam.v1.AuditConfigDelta} AuditConfigDelta
                     * @throws {Error} If the payload is not a reader or valid buffer
                     * @throws {$protobuf.util.ProtocolError} If required fields are missing
                     */
                    AuditConfigDelta.decodeDelimited = function decodeDelimited(reader) {
                        if (!(reader instanceof $Reader))
                            reader = new $Reader(reader);
                        return this.decode(reader, reader.uint32());
                    };
    
                    /**
                     * Verifies an AuditConfigDelta message.
                     * @function verify
                     * @memberof google.iam.v1.AuditConfigDelta
                     * @static
                     * @param {Object.<string,*>} message Plain object to verify
                     * @returns {string|null} `null` if valid, otherwise the reason why it is not
                     */
                    AuditConfigDelta.verify = function verify(message) {
                        if (typeof message !== "object" || message === null)
                            return "object expected";
                        if (message.action != null && message.hasOwnProperty("action"))
                            switch (message.action) {
                            default:
                                return "action: enum value expected";
                            case 0:
                            case 1:
                            case 2:
                                break;
                            }
                        if (message.service != null && message.hasOwnProperty("service"))
                            if (!$util.isString(message.service))
                                return "service: string expected";
                        if (message.exemptedMember != null && message.hasOwnProperty("exemptedMember"))
                            if (!$util.isString(message.exemptedMember))
                                return "exemptedMember: string expected";
                        if (message.logType != null && message.hasOwnProperty("logType"))
                            if (!$util.isString(message.logType))
                                return "logType: string expected";
                        return null;
                    };
    
                    /**
                     * Creates an AuditConfigDelta message from a plain object. Also converts values to their respective internal types.
                     * @function fromObject
                     * @memberof google.iam.v1.AuditConfigDelta
                     * @static
                     * @param {Object.<string,*>} object Plain object
                     * @returns {google.iam.v1.AuditConfigDelta} AuditConfigDelta
                     */
                    AuditConfigDelta.fromObject = function fromObject(object) {
                        if (object instanceof $root.google.iam.v1.AuditConfigDelta)
                            return object;
                        var message = new $root.google.iam.v1.AuditConfigDelta();
                        switch (object.action) {
                        default:
                            if (typeof object.action === "number") {
                                message.action = object.action;
                                break;
                            }
                            break;
                        case "ACTION_UNSPECIFIED":
                        case 0:
                            message.action = 0;
                            break;
                        case "ADD":
                        case 1:
                            message.action = 1;
                            break;
                        case "REMOVE":
                        case 2:
                            message.action = 2;
                            break;
                        }
                        if (object.service != null)
                            message.service = String(object.service);
                        if (object.exemptedMember != null)
                            message.exemptedMember = String(object.exemptedMember);
                        if (object.logType != null)
                            message.logType = String(object.logType);
                        return message;
                    };
    
                    /**
                     * Creates a plain object from an AuditConfigDelta message. Also converts values to other types if specified.
                     * @function toObject
                     * @memberof google.iam.v1.AuditConfigDelta
                     * @static
                     * @param {google.iam.v1.AuditConfigDelta} message AuditConfigDelta
                     * @param {$protobuf.IConversionOptions} [options] Conversion options
                     * @returns {Object.<string,*>} Plain object
                     */
                    AuditConfigDelta.toObject = function toObject(message, options) {
                        if (!options)
                            options = {};
                        var object = {};
                        if (options.defaults) {
                            object.action = options.enums === String ? "ACTION_UNSPECIFIED" : 0;
                            object.service = "";
                            object.exemptedMember = "";
                            object.logType = "";
                        }
                        if (message.action != null && message.hasOwnProperty("action"))
                            object.action = options.enums === String ? $root.google.iam.v1.AuditConfigDelta.Action[message.action] === undefined ? message.action : $root.google.iam.v1.AuditConfigDelta.Action[message.action] : message.action;
                        if (message.service != null && message.hasOwnProperty("service"))
                            object.service = message.service;
                        if (message.exemptedMember != null && message.hasOwnProperty("exemptedMember"))
                            object.exemptedMember = message.exemptedMember;
                        if (message.logType != null && message.hasOwnProperty("logType"))
                            object.logType = message.logType;
                        return object;
                    };
    
                    /**
                     * Converts this AuditConfigDelta to JSON.
                     * @function toJSON
                     * @memberof google.iam.v1.AuditConfigDelta
                     * @instance
                     * @returns {Object.<string,*>} JSON object
                     */
                    AuditConfigDelta.prototype.toJSON = function toJSON() {
                        return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                    };
    
                    /**
                     * Gets the default type url for AuditConfigDelta
                     * @function getTypeUrl
                     * @memberof google.iam.v1.AuditConfigDelta
                     * @static
                     * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                     * @returns {string} The default type url
                     */
                    AuditConfigDelta.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                        if (typeUrlPrefix === undefined) {
                            typeUrlPrefix = "type.googleapis.com";
                        }
                        return typeUrlPrefix + "/google.iam.v1.AuditConfigDelta";
                    };
    
                    /**
                     * Action enum.
                     * @name google.iam.v1.AuditConfigDelta.Action
                     * @enum {number}
                     * @property {number} ACTION_UNSPECIFIED=0 ACTION_UNSPECIFIED value
                     * @property {number} ADD=1 ADD value
                     * @property {number} REMOVE=2 REMOVE value
                     */
                    AuditConfigDelta.Action = (function() {
                        var valuesById = {}, values = Object.create(valuesById);
                        values[valuesById[0] = "ACTION_UNSPECIFIED"] = 0;
                        values[valuesById[1] = "ADD"] = 1;
                        values[valuesById[2] = "REMOVE"] = 2;
                        return values;
                    })();
    
                    return AuditConfigDelta;
                })();
    
                return v1;
            })();
    
            return iam;
        })();
    
        google.type = (function() {
    
            /**
             * Namespace type.
             * @memberof google
             * @namespace
             */
            var type = {};
    
            type.Expr = (function() {
    
                /**
                 * Properties of an Expr.
                 * @memberof google.type
                 * @interface IExpr
                 * @property {string|null} [expression] Expr expression
                 * @property {string|null} [title] Expr title
                 * @property {string|null} [description] Expr description
                 * @property {string|null} [location] Expr location
                 */
    
                /**
                 * Constructs a new Expr.
                 * @memberof google.type
                 * @classdesc Represents an Expr.
                 * @implements IExpr
                 * @constructor
                 * @param {google.type.IExpr=} [properties] Properties to set
                 */
                function Expr(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * Expr expression.
                 * @member {string} expression
                 * @memberof google.type.Expr
                 * @instance
                 */
                Expr.prototype.expression = "";
    
                /**
                 * Expr title.
                 * @member {string} title
                 * @memberof google.type.Expr
                 * @instance
                 */
                Expr.prototype.title = "";
    
                /**
                 * Expr description.
                 * @member {string} description
                 * @memberof google.type.Expr
                 * @instance
                 */
                Expr.prototype.description = "";
    
                /**
                 * Expr location.
                 * @member {string} location
                 * @memberof google.type.Expr
                 * @instance
                 */
                Expr.prototype.location = "";
    
                /**
                 * Creates a new Expr instance using the specified properties.
                 * @function create
                 * @memberof google.type.Expr
                 * @static
                 * @param {google.type.IExpr=} [properties] Properties to set
                 * @returns {google.type.Expr} Expr instance
                 */
                Expr.create = function create(properties) {
                    return new Expr(properties);
                };
    
                /**
                 * Encodes the specified Expr message. Does not implicitly {@link google.type.Expr.verify|verify} messages.
                 * @function encode
                 * @memberof google.type.Expr
                 * @static
                 * @param {google.type.IExpr} message Expr message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Expr.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.expression != null && Object.hasOwnProperty.call(message, "expression"))
                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.expression);
                    if (message.title != null && Object.hasOwnProperty.call(message, "title"))
                        writer.uint32(/* id 2, wireType 2 =*/18).string(message.title);
                    if (message.description != null && Object.hasOwnProperty.call(message, "description"))
                        writer.uint32(/* id 3, wireType 2 =*/26).string(message.description);
                    if (message.location != null && Object.hasOwnProperty.call(message, "location"))
                        writer.uint32(/* id 4, wireType 2 =*/34).string(message.location);
                    return writer;
                };
    
                /**
                 * Encodes the specified Expr message, length delimited. Does not implicitly {@link google.type.Expr.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.type.Expr
                 * @static
                 * @param {google.type.IExpr} message Expr message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Expr.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes an Expr message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.type.Expr
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.type.Expr} Expr
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Expr.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.type.Expr();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.expression = reader.string();
                                break;
                            }
                        case 2: {
                                message.title = reader.string();
                                break;
                            }
                        case 3: {
                                message.description = reader.string();
                                break;
                            }
                        case 4: {
                                message.location = reader.string();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes an Expr message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.type.Expr
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.type.Expr} Expr
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Expr.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies an Expr message.
                 * @function verify
                 * @memberof google.type.Expr
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                Expr.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.expression != null && message.hasOwnProperty("expression"))
                        if (!$util.isString(message.expression))
                            return "expression: string expected";
                    if (message.title != null && message.hasOwnProperty("title"))
                        if (!$util.isString(message.title))
                            return "title: string expected";
                    if (message.description != null && message.hasOwnProperty("description"))
                        if (!$util.isString(message.description))
                            return "description: string expected";
                    if (message.location != null && message.hasOwnProperty("location"))
                        if (!$util.isString(message.location))
                            return "location: string expected";
                    return null;
                };
    
                /**
                 * Creates an Expr message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.type.Expr
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.type.Expr} Expr
                 */
                Expr.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.type.Expr)
                        return object;
                    var message = new $root.google.type.Expr();
                    if (object.expression != null)
                        message.expression = String(object.expression);
                    if (object.title != null)
                        message.title = String(object.title);
                    if (object.description != null)
                        message.description = String(object.description);
                    if (object.location != null)
                        message.location = String(object.location);
                    return message;
                };
    
                /**
                 * Creates a plain object from an Expr message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.type.Expr
                 * @static
                 * @param {google.type.Expr} message Expr
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                Expr.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults) {
                        object.expression = "";
                        object.title = "";
                        object.description = "";
                        object.location = "";
                    }
                    if (message.expression != null && message.hasOwnProperty("expression"))
                        object.expression = message.expression;
                    if (message.title != null && message.hasOwnProperty("title"))
                        object.title = message.title;
                    if (message.description != null && message.hasOwnProperty("description"))
                        object.description = message.description;
                    if (message.location != null && message.hasOwnProperty("location"))
                        object.location = message.location;
                    return object;
                };
    
                /**
                 * Converts this Expr to JSON.
                 * @function toJSON
                 * @memberof google.type.Expr
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                Expr.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for Expr
                 * @function getTypeUrl
                 * @memberof google.type.Expr
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                Expr.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.type.Expr";
                };
    
                return Expr;
            })();
    
            return type;
        })();
    
        google.longrunning = (function() {
    
            /**
             * Namespace longrunning.
             * @memberof google
             * @namespace
             */
            var longrunning = {};
    
            longrunning.Operations = (function() {
    
                /**
                 * Constructs a new Operations service.
                 * @memberof google.longrunning
                 * @classdesc Represents an Operations
                 * @extends $protobuf.rpc.Service
                 * @constructor
                 * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                 * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                 * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                 */
                function Operations(rpcImpl, requestDelimited, responseDelimited) {
                    $protobuf.rpc.Service.call(this, rpcImpl, requestDelimited, responseDelimited);
                }
    
                (Operations.prototype = Object.create($protobuf.rpc.Service.prototype)).constructor = Operations;
    
                /**
                 * Creates new Operations service using the specified rpc implementation.
                 * @function create
                 * @memberof google.longrunning.Operations
                 * @static
                 * @param {$protobuf.RPCImpl} rpcImpl RPC implementation
                 * @param {boolean} [requestDelimited=false] Whether requests are length-delimited
                 * @param {boolean} [responseDelimited=false] Whether responses are length-delimited
                 * @returns {Operations} RPC service. Useful where requests and/or responses are streamed.
                 */
                Operations.create = function create(rpcImpl, requestDelimited, responseDelimited) {
                    return new this(rpcImpl, requestDelimited, responseDelimited);
                };
    
                /**
                 * Callback as used by {@link google.longrunning.Operations|listOperations}.
                 * @memberof google.longrunning.Operations
                 * @typedef ListOperationsCallback
                 * @type {function}
                 * @param {Error|null} error Error, if any
                 * @param {google.longrunning.ListOperationsResponse} [response] ListOperationsResponse
                 */
    
                /**
                 * Calls ListOperations.
                 * @function listOperations
                 * @memberof google.longrunning.Operations
                 * @instance
                 * @param {google.longrunning.IListOperationsRequest} request ListOperationsRequest message or plain object
                 * @param {google.longrunning.Operations.ListOperationsCallback} callback Node-style callback called with the error, if any, and ListOperationsResponse
                 * @returns {undefined}
                 * @variation 1
                 */
                Object.defineProperty(Operations.prototype.listOperations = function listOperations(request, callback) {
                    return this.rpcCall(listOperations, $root.google.longrunning.ListOperationsRequest, $root.google.longrunning.ListOperationsResponse, request, callback);
                }, "name", { value: "ListOperations" });
    
                /**
                 * Calls ListOperations.
                 * @function listOperations
                 * @memberof google.longrunning.Operations
                 * @instance
                 * @param {google.longrunning.IListOperationsRequest} request ListOperationsRequest message or plain object
                 * @returns {Promise<google.longrunning.ListOperationsResponse>} Promise
                 * @variation 2
                 */
    
                /**
                 * Callback as used by {@link google.longrunning.Operations|getOperation}.
                 * @memberof google.longrunning.Operations
                 * @typedef GetOperationCallback
                 * @type {function}
                 * @param {Error|null} error Error, if any
                 * @param {google.longrunning.Operation} [response] Operation
                 */
    
                /**
                 * Calls GetOperation.
                 * @function getOperation
                 * @memberof google.longrunning.Operations
                 * @instance
                 * @param {google.longrunning.IGetOperationRequest} request GetOperationRequest message or plain object
                 * @param {google.longrunning.Operations.GetOperationCallback} callback Node-style callback called with the error, if any, and Operation
                 * @returns {undefined}
                 * @variation 1
                 */
                Object.defineProperty(Operations.prototype.getOperation = function getOperation(request, callback) {
                    return this.rpcCall(getOperation, $root.google.longrunning.GetOperationRequest, $root.google.longrunning.Operation, request, callback);
                }, "name", { value: "GetOperation" });
    
                /**
                 * Calls GetOperation.
                 * @function getOperation
                 * @memberof google.longrunning.Operations
                 * @instance
                 * @param {google.longrunning.IGetOperationRequest} request GetOperationRequest message or plain object
                 * @returns {Promise<google.longrunning.Operation>} Promise
                 * @variation 2
                 */
    
                /**
                 * Callback as used by {@link google.longrunning.Operations|deleteOperation}.
                 * @memberof google.longrunning.Operations
                 * @typedef DeleteOperationCallback
                 * @type {function}
                 * @param {Error|null} error Error, if any
                 * @param {google.protobuf.Empty} [response] Empty
                 */
    
                /**
                 * Calls DeleteOperation.
                 * @function deleteOperation
                 * @memberof google.longrunning.Operations
                 * @instance
                 * @param {google.longrunning.IDeleteOperationRequest} request DeleteOperationRequest message or plain object
                 * @param {google.longrunning.Operations.DeleteOperationCallback} callback Node-style callback called with the error, if any, and Empty
                 * @returns {undefined}
                 * @variation 1
                 */
                Object.defineProperty(Operations.prototype.deleteOperation = function deleteOperation(request, callback) {
                    return this.rpcCall(deleteOperation, $root.google.longrunning.DeleteOperationRequest, $root.google.protobuf.Empty, request, callback);
                }, "name", { value: "DeleteOperation" });
    
                /**
                 * Calls DeleteOperation.
                 * @function deleteOperation
                 * @memberof google.longrunning.Operations
                 * @instance
                 * @param {google.longrunning.IDeleteOperationRequest} request DeleteOperationRequest message or plain object
                 * @returns {Promise<google.protobuf.Empty>} Promise
                 * @variation 2
                 */
    
                /**
                 * Callback as used by {@link google.longrunning.Operations|cancelOperation}.
                 * @memberof google.longrunning.Operations
                 * @typedef CancelOperationCallback
                 * @type {function}
                 * @param {Error|null} error Error, if any
                 * @param {google.protobuf.Empty} [response] Empty
                 */
    
                /**
                 * Calls CancelOperation.
                 * @function cancelOperation
                 * @memberof google.longrunning.Operations
                 * @instance
                 * @param {google.longrunning.ICancelOperationRequest} request CancelOperationRequest message or plain object
                 * @param {google.longrunning.Operations.CancelOperationCallback} callback Node-style callback called with the error, if any, and Empty
                 * @returns {undefined}
                 * @variation 1
                 */
                Object.defineProperty(Operations.prototype.cancelOperation = function cancelOperation(request, callback) {
                    return this.rpcCall(cancelOperation, $root.google.longrunning.CancelOperationRequest, $root.google.protobuf.Empty, request, callback);
                }, "name", { value: "CancelOperation" });
    
                /**
                 * Calls CancelOperation.
                 * @function cancelOperation
                 * @memberof google.longrunning.Operations
                 * @instance
                 * @param {google.longrunning.ICancelOperationRequest} request CancelOperationRequest message or plain object
                 * @returns {Promise<google.protobuf.Empty>} Promise
                 * @variation 2
                 */
    
                /**
                 * Callback as used by {@link google.longrunning.Operations|waitOperation}.
                 * @memberof google.longrunning.Operations
                 * @typedef WaitOperationCallback
                 * @type {function}
                 * @param {Error|null} error Error, if any
                 * @param {google.longrunning.Operation} [response] Operation
                 */
    
                /**
                 * Calls WaitOperation.
                 * @function waitOperation
                 * @memberof google.longrunning.Operations
                 * @instance
                 * @param {google.longrunning.IWaitOperationRequest} request WaitOperationRequest message or plain object
                 * @param {google.longrunning.Operations.WaitOperationCallback} callback Node-style callback called with the error, if any, and Operation
                 * @returns {undefined}
                 * @variation 1
                 */
                Object.defineProperty(Operations.prototype.waitOperation = function waitOperation(request, callback) {
                    return this.rpcCall(waitOperation, $root.google.longrunning.WaitOperationRequest, $root.google.longrunning.Operation, request, callback);
                }, "name", { value: "WaitOperation" });
    
                /**
                 * Calls WaitOperation.
                 * @function waitOperation
                 * @memberof google.longrunning.Operations
                 * @instance
                 * @param {google.longrunning.IWaitOperationRequest} request WaitOperationRequest message or plain object
                 * @returns {Promise<google.longrunning.Operation>} Promise
                 * @variation 2
                 */
    
                return Operations;
            })();
    
            longrunning.Operation = (function() {
    
                /**
                 * Properties of an Operation.
                 * @memberof google.longrunning
                 * @interface IOperation
                 * @property {string|null} [name] Operation name
                 * @property {google.protobuf.IAny|null} [metadata] Operation metadata
                 * @property {boolean|null} [done] Operation done
                 * @property {google.rpc.IStatus|null} [error] Operation error
                 * @property {google.protobuf.IAny|null} [response] Operation response
                 */
    
                /**
                 * Constructs a new Operation.
                 * @memberof google.longrunning
                 * @classdesc Represents an Operation.
                 * @implements IOperation
                 * @constructor
                 * @param {google.longrunning.IOperation=} [properties] Properties to set
                 */
                function Operation(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * Operation name.
                 * @member {string} name
                 * @memberof google.longrunning.Operation
                 * @instance
                 */
                Operation.prototype.name = "";
    
                /**
                 * Operation metadata.
                 * @member {google.protobuf.IAny|null|undefined} metadata
                 * @memberof google.longrunning.Operation
                 * @instance
                 */
                Operation.prototype.metadata = null;
    
                /**
                 * Operation done.
                 * @member {boolean} done
                 * @memberof google.longrunning.Operation
                 * @instance
                 */
                Operation.prototype.done = false;
    
                /**
                 * Operation error.
                 * @member {google.rpc.IStatus|null|undefined} error
                 * @memberof google.longrunning.Operation
                 * @instance
                 */
                Operation.prototype.error = null;
    
                /**
                 * Operation response.
                 * @member {google.protobuf.IAny|null|undefined} response
                 * @memberof google.longrunning.Operation
                 * @instance
                 */
                Operation.prototype.response = null;
    
                // OneOf field names bound to virtual getters and setters
                var $oneOfFields;
    
                /**
                 * Operation result.
                 * @member {"error"|"response"|undefined} result
                 * @memberof google.longrunning.Operation
                 * @instance
                 */
                Object.defineProperty(Operation.prototype, "result", {
                    get: $util.oneOfGetter($oneOfFields = ["error", "response"]),
                    set: $util.oneOfSetter($oneOfFields)
                });
    
                /**
                 * Creates a new Operation instance using the specified properties.
                 * @function create
                 * @memberof google.longrunning.Operation
                 * @static
                 * @param {google.longrunning.IOperation=} [properties] Properties to set
                 * @returns {google.longrunning.Operation} Operation instance
                 */
                Operation.create = function create(properties) {
                    return new Operation(properties);
                };
    
                /**
                 * Encodes the specified Operation message. Does not implicitly {@link google.longrunning.Operation.verify|verify} messages.
                 * @function encode
                 * @memberof google.longrunning.Operation
                 * @static
                 * @param {google.longrunning.IOperation} message Operation message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Operation.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                    if (message.metadata != null && Object.hasOwnProperty.call(message, "metadata"))
                        $root.google.protobuf.Any.encode(message.metadata, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                    if (message.done != null && Object.hasOwnProperty.call(message, "done"))
                        writer.uint32(/* id 3, wireType 0 =*/24).bool(message.done);
                    if (message.error != null && Object.hasOwnProperty.call(message, "error"))
                        $root.google.rpc.Status.encode(message.error, writer.uint32(/* id 4, wireType 2 =*/34).fork()).ldelim();
                    if (message.response != null && Object.hasOwnProperty.call(message, "response"))
                        $root.google.protobuf.Any.encode(message.response, writer.uint32(/* id 5, wireType 2 =*/42).fork()).ldelim();
                    return writer;
                };
    
                /**
                 * Encodes the specified Operation message, length delimited. Does not implicitly {@link google.longrunning.Operation.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.longrunning.Operation
                 * @static
                 * @param {google.longrunning.IOperation} message Operation message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Operation.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes an Operation message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.longrunning.Operation
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.longrunning.Operation} Operation
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Operation.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.longrunning.Operation();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.name = reader.string();
                                break;
                            }
                        case 2: {
                                message.metadata = $root.google.protobuf.Any.decode(reader, reader.uint32());
                                break;
                            }
                        case 3: {
                                message.done = reader.bool();
                                break;
                            }
                        case 4: {
                                message.error = $root.google.rpc.Status.decode(reader, reader.uint32());
                                break;
                            }
                        case 5: {
                                message.response = $root.google.protobuf.Any.decode(reader, reader.uint32());
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes an Operation message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.longrunning.Operation
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.longrunning.Operation} Operation
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Operation.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies an Operation message.
                 * @function verify
                 * @memberof google.longrunning.Operation
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                Operation.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    var properties = {};
                    if (message.name != null && message.hasOwnProperty("name"))
                        if (!$util.isString(message.name))
                            return "name: string expected";
                    if (message.metadata != null && message.hasOwnProperty("metadata")) {
                        var error = $root.google.protobuf.Any.verify(message.metadata);
                        if (error)
                            return "metadata." + error;
                    }
                    if (message.done != null && message.hasOwnProperty("done"))
                        if (typeof message.done !== "boolean")
                            return "done: boolean expected";
                    if (message.error != null && message.hasOwnProperty("error")) {
                        properties.result = 1;
                        {
                            var error = $root.google.rpc.Status.verify(message.error);
                            if (error)
                                return "error." + error;
                        }
                    }
                    if (message.response != null && message.hasOwnProperty("response")) {
                        if (properties.result === 1)
                            return "result: multiple values";
                        properties.result = 1;
                        {
                            var error = $root.google.protobuf.Any.verify(message.response);
                            if (error)
                                return "response." + error;
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates an Operation message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.longrunning.Operation
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.longrunning.Operation} Operation
                 */
                Operation.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.longrunning.Operation)
                        return object;
                    var message = new $root.google.longrunning.Operation();
                    if (object.name != null)
                        message.name = String(object.name);
                    if (object.metadata != null) {
                        if (typeof object.metadata !== "object")
                            throw TypeError(".google.longrunning.Operation.metadata: object expected");
                        message.metadata = $root.google.protobuf.Any.fromObject(object.metadata);
                    }
                    if (object.done != null)
                        message.done = Boolean(object.done);
                    if (object.error != null) {
                        if (typeof object.error !== "object")
                            throw TypeError(".google.longrunning.Operation.error: object expected");
                        message.error = $root.google.rpc.Status.fromObject(object.error);
                    }
                    if (object.response != null) {
                        if (typeof object.response !== "object")
                            throw TypeError(".google.longrunning.Operation.response: object expected");
                        message.response = $root.google.protobuf.Any.fromObject(object.response);
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from an Operation message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.longrunning.Operation
                 * @static
                 * @param {google.longrunning.Operation} message Operation
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                Operation.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults) {
                        object.name = "";
                        object.metadata = null;
                        object.done = false;
                    }
                    if (message.name != null && message.hasOwnProperty("name"))
                        object.name = message.name;
                    if (message.metadata != null && message.hasOwnProperty("metadata"))
                        object.metadata = $root.google.protobuf.Any.toObject(message.metadata, options);
                    if (message.done != null && message.hasOwnProperty("done"))
                        object.done = message.done;
                    if (message.error != null && message.hasOwnProperty("error")) {
                        object.error = $root.google.rpc.Status.toObject(message.error, options);
                        if (options.oneofs)
                            object.result = "error";
                    }
                    if (message.response != null && message.hasOwnProperty("response")) {
                        object.response = $root.google.protobuf.Any.toObject(message.response, options);
                        if (options.oneofs)
                            object.result = "response";
                    }
                    return object;
                };
    
                /**
                 * Converts this Operation to JSON.
                 * @function toJSON
                 * @memberof google.longrunning.Operation
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                Operation.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for Operation
                 * @function getTypeUrl
                 * @memberof google.longrunning.Operation
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                Operation.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.longrunning.Operation";
                };
    
                return Operation;
            })();
    
            longrunning.GetOperationRequest = (function() {
    
                /**
                 * Properties of a GetOperationRequest.
                 * @memberof google.longrunning
                 * @interface IGetOperationRequest
                 * @property {string|null} [name] GetOperationRequest name
                 */
    
                /**
                 * Constructs a new GetOperationRequest.
                 * @memberof google.longrunning
                 * @classdesc Represents a GetOperationRequest.
                 * @implements IGetOperationRequest
                 * @constructor
                 * @param {google.longrunning.IGetOperationRequest=} [properties] Properties to set
                 */
                function GetOperationRequest(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * GetOperationRequest name.
                 * @member {string} name
                 * @memberof google.longrunning.GetOperationRequest
                 * @instance
                 */
                GetOperationRequest.prototype.name = "";
    
                /**
                 * Creates a new GetOperationRequest instance using the specified properties.
                 * @function create
                 * @memberof google.longrunning.GetOperationRequest
                 * @static
                 * @param {google.longrunning.IGetOperationRequest=} [properties] Properties to set
                 * @returns {google.longrunning.GetOperationRequest} GetOperationRequest instance
                 */
                GetOperationRequest.create = function create(properties) {
                    return new GetOperationRequest(properties);
                };
    
                /**
                 * Encodes the specified GetOperationRequest message. Does not implicitly {@link google.longrunning.GetOperationRequest.verify|verify} messages.
                 * @function encode
                 * @memberof google.longrunning.GetOperationRequest
                 * @static
                 * @param {google.longrunning.IGetOperationRequest} message GetOperationRequest message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                GetOperationRequest.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                    return writer;
                };
    
                /**
                 * Encodes the specified GetOperationRequest message, length delimited. Does not implicitly {@link google.longrunning.GetOperationRequest.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.longrunning.GetOperationRequest
                 * @static
                 * @param {google.longrunning.IGetOperationRequest} message GetOperationRequest message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                GetOperationRequest.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a GetOperationRequest message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.longrunning.GetOperationRequest
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.longrunning.GetOperationRequest} GetOperationRequest
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                GetOperationRequest.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.longrunning.GetOperationRequest();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.name = reader.string();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a GetOperationRequest message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.longrunning.GetOperationRequest
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.longrunning.GetOperationRequest} GetOperationRequest
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                GetOperationRequest.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a GetOperationRequest message.
                 * @function verify
                 * @memberof google.longrunning.GetOperationRequest
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                GetOperationRequest.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.name != null && message.hasOwnProperty("name"))
                        if (!$util.isString(message.name))
                            return "name: string expected";
                    return null;
                };
    
                /**
                 * Creates a GetOperationRequest message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.longrunning.GetOperationRequest
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.longrunning.GetOperationRequest} GetOperationRequest
                 */
                GetOperationRequest.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.longrunning.GetOperationRequest)
                        return object;
                    var message = new $root.google.longrunning.GetOperationRequest();
                    if (object.name != null)
                        message.name = String(object.name);
                    return message;
                };
    
                /**
                 * Creates a plain object from a GetOperationRequest message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.longrunning.GetOperationRequest
                 * @static
                 * @param {google.longrunning.GetOperationRequest} message GetOperationRequest
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                GetOperationRequest.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults)
                        object.name = "";
                    if (message.name != null && message.hasOwnProperty("name"))
                        object.name = message.name;
                    return object;
                };
    
                /**
                 * Converts this GetOperationRequest to JSON.
                 * @function toJSON
                 * @memberof google.longrunning.GetOperationRequest
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                GetOperationRequest.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for GetOperationRequest
                 * @function getTypeUrl
                 * @memberof google.longrunning.GetOperationRequest
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                GetOperationRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.longrunning.GetOperationRequest";
                };
    
                return GetOperationRequest;
            })();
    
            longrunning.ListOperationsRequest = (function() {
    
                /**
                 * Properties of a ListOperationsRequest.
                 * @memberof google.longrunning
                 * @interface IListOperationsRequest
                 * @property {string|null} [name] ListOperationsRequest name
                 * @property {string|null} [filter] ListOperationsRequest filter
                 * @property {number|null} [pageSize] ListOperationsRequest pageSize
                 * @property {string|null} [pageToken] ListOperationsRequest pageToken
                 */
    
                /**
                 * Constructs a new ListOperationsRequest.
                 * @memberof google.longrunning
                 * @classdesc Represents a ListOperationsRequest.
                 * @implements IListOperationsRequest
                 * @constructor
                 * @param {google.longrunning.IListOperationsRequest=} [properties] Properties to set
                 */
                function ListOperationsRequest(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * ListOperationsRequest name.
                 * @member {string} name
                 * @memberof google.longrunning.ListOperationsRequest
                 * @instance
                 */
                ListOperationsRequest.prototype.name = "";
    
                /**
                 * ListOperationsRequest filter.
                 * @member {string} filter
                 * @memberof google.longrunning.ListOperationsRequest
                 * @instance
                 */
                ListOperationsRequest.prototype.filter = "";
    
                /**
                 * ListOperationsRequest pageSize.
                 * @member {number} pageSize
                 * @memberof google.longrunning.ListOperationsRequest
                 * @instance
                 */
                ListOperationsRequest.prototype.pageSize = 0;
    
                /**
                 * ListOperationsRequest pageToken.
                 * @member {string} pageToken
                 * @memberof google.longrunning.ListOperationsRequest
                 * @instance
                 */
                ListOperationsRequest.prototype.pageToken = "";
    
                /**
                 * Creates a new ListOperationsRequest instance using the specified properties.
                 * @function create
                 * @memberof google.longrunning.ListOperationsRequest
                 * @static
                 * @param {google.longrunning.IListOperationsRequest=} [properties] Properties to set
                 * @returns {google.longrunning.ListOperationsRequest} ListOperationsRequest instance
                 */
                ListOperationsRequest.create = function create(properties) {
                    return new ListOperationsRequest(properties);
                };
    
                /**
                 * Encodes the specified ListOperationsRequest message. Does not implicitly {@link google.longrunning.ListOperationsRequest.verify|verify} messages.
                 * @function encode
                 * @memberof google.longrunning.ListOperationsRequest
                 * @static
                 * @param {google.longrunning.IListOperationsRequest} message ListOperationsRequest message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ListOperationsRequest.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.filter != null && Object.hasOwnProperty.call(message, "filter"))
                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.filter);
                    if (message.pageSize != null && Object.hasOwnProperty.call(message, "pageSize"))
                        writer.uint32(/* id 2, wireType 0 =*/16).int32(message.pageSize);
                    if (message.pageToken != null && Object.hasOwnProperty.call(message, "pageToken"))
                        writer.uint32(/* id 3, wireType 2 =*/26).string(message.pageToken);
                    if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                        writer.uint32(/* id 4, wireType 2 =*/34).string(message.name);
                    return writer;
                };
    
                /**
                 * Encodes the specified ListOperationsRequest message, length delimited. Does not implicitly {@link google.longrunning.ListOperationsRequest.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.longrunning.ListOperationsRequest
                 * @static
                 * @param {google.longrunning.IListOperationsRequest} message ListOperationsRequest message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ListOperationsRequest.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a ListOperationsRequest message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.longrunning.ListOperationsRequest
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.longrunning.ListOperationsRequest} ListOperationsRequest
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ListOperationsRequest.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.longrunning.ListOperationsRequest();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 4: {
                                message.name = reader.string();
                                break;
                            }
                        case 1: {
                                message.filter = reader.string();
                                break;
                            }
                        case 2: {
                                message.pageSize = reader.int32();
                                break;
                            }
                        case 3: {
                                message.pageToken = reader.string();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a ListOperationsRequest message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.longrunning.ListOperationsRequest
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.longrunning.ListOperationsRequest} ListOperationsRequest
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ListOperationsRequest.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a ListOperationsRequest message.
                 * @function verify
                 * @memberof google.longrunning.ListOperationsRequest
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                ListOperationsRequest.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.name != null && message.hasOwnProperty("name"))
                        if (!$util.isString(message.name))
                            return "name: string expected";
                    if (message.filter != null && message.hasOwnProperty("filter"))
                        if (!$util.isString(message.filter))
                            return "filter: string expected";
                    if (message.pageSize != null && message.hasOwnProperty("pageSize"))
                        if (!$util.isInteger(message.pageSize))
                            return "pageSize: integer expected";
                    if (message.pageToken != null && message.hasOwnProperty("pageToken"))
                        if (!$util.isString(message.pageToken))
                            return "pageToken: string expected";
                    return null;
                };
    
                /**
                 * Creates a ListOperationsRequest message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.longrunning.ListOperationsRequest
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.longrunning.ListOperationsRequest} ListOperationsRequest
                 */
                ListOperationsRequest.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.longrunning.ListOperationsRequest)
                        return object;
                    var message = new $root.google.longrunning.ListOperationsRequest();
                    if (object.name != null)
                        message.name = String(object.name);
                    if (object.filter != null)
                        message.filter = String(object.filter);
                    if (object.pageSize != null)
                        message.pageSize = object.pageSize | 0;
                    if (object.pageToken != null)
                        message.pageToken = String(object.pageToken);
                    return message;
                };
    
                /**
                 * Creates a plain object from a ListOperationsRequest message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.longrunning.ListOperationsRequest
                 * @static
                 * @param {google.longrunning.ListOperationsRequest} message ListOperationsRequest
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                ListOperationsRequest.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults) {
                        object.filter = "";
                        object.pageSize = 0;
                        object.pageToken = "";
                        object.name = "";
                    }
                    if (message.filter != null && message.hasOwnProperty("filter"))
                        object.filter = message.filter;
                    if (message.pageSize != null && message.hasOwnProperty("pageSize"))
                        object.pageSize = message.pageSize;
                    if (message.pageToken != null && message.hasOwnProperty("pageToken"))
                        object.pageToken = message.pageToken;
                    if (message.name != null && message.hasOwnProperty("name"))
                        object.name = message.name;
                    return object;
                };
    
                /**
                 * Converts this ListOperationsRequest to JSON.
                 * @function toJSON
                 * @memberof google.longrunning.ListOperationsRequest
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                ListOperationsRequest.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for ListOperationsRequest
                 * @function getTypeUrl
                 * @memberof google.longrunning.ListOperationsRequest
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                ListOperationsRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.longrunning.ListOperationsRequest";
                };
    
                return ListOperationsRequest;
            })();
    
            longrunning.ListOperationsResponse = (function() {
    
                /**
                 * Properties of a ListOperationsResponse.
                 * @memberof google.longrunning
                 * @interface IListOperationsResponse
                 * @property {Array.<google.longrunning.IOperation>|null} [operations] ListOperationsResponse operations
                 * @property {string|null} [nextPageToken] ListOperationsResponse nextPageToken
                 */
    
                /**
                 * Constructs a new ListOperationsResponse.
                 * @memberof google.longrunning
                 * @classdesc Represents a ListOperationsResponse.
                 * @implements IListOperationsResponse
                 * @constructor
                 * @param {google.longrunning.IListOperationsResponse=} [properties] Properties to set
                 */
                function ListOperationsResponse(properties) {
                    this.operations = [];
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * ListOperationsResponse operations.
                 * @member {Array.<google.longrunning.IOperation>} operations
                 * @memberof google.longrunning.ListOperationsResponse
                 * @instance
                 */
                ListOperationsResponse.prototype.operations = $util.emptyArray;
    
                /**
                 * ListOperationsResponse nextPageToken.
                 * @member {string} nextPageToken
                 * @memberof google.longrunning.ListOperationsResponse
                 * @instance
                 */
                ListOperationsResponse.prototype.nextPageToken = "";
    
                /**
                 * Creates a new ListOperationsResponse instance using the specified properties.
                 * @function create
                 * @memberof google.longrunning.ListOperationsResponse
                 * @static
                 * @param {google.longrunning.IListOperationsResponse=} [properties] Properties to set
                 * @returns {google.longrunning.ListOperationsResponse} ListOperationsResponse instance
                 */
                ListOperationsResponse.create = function create(properties) {
                    return new ListOperationsResponse(properties);
                };
    
                /**
                 * Encodes the specified ListOperationsResponse message. Does not implicitly {@link google.longrunning.ListOperationsResponse.verify|verify} messages.
                 * @function encode
                 * @memberof google.longrunning.ListOperationsResponse
                 * @static
                 * @param {google.longrunning.IListOperationsResponse} message ListOperationsResponse message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ListOperationsResponse.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.operations != null && message.operations.length)
                        for (var i = 0; i < message.operations.length; ++i)
                            $root.google.longrunning.Operation.encode(message.operations[i], writer.uint32(/* id 1, wireType 2 =*/10).fork()).ldelim();
                    if (message.nextPageToken != null && Object.hasOwnProperty.call(message, "nextPageToken"))
                        writer.uint32(/* id 2, wireType 2 =*/18).string(message.nextPageToken);
                    return writer;
                };
    
                /**
                 * Encodes the specified ListOperationsResponse message, length delimited. Does not implicitly {@link google.longrunning.ListOperationsResponse.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.longrunning.ListOperationsResponse
                 * @static
                 * @param {google.longrunning.IListOperationsResponse} message ListOperationsResponse message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                ListOperationsResponse.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a ListOperationsResponse message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.longrunning.ListOperationsResponse
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.longrunning.ListOperationsResponse} ListOperationsResponse
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ListOperationsResponse.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.longrunning.ListOperationsResponse();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                if (!(message.operations && message.operations.length))
                                    message.operations = [];
                                message.operations.push($root.google.longrunning.Operation.decode(reader, reader.uint32()));
                                break;
                            }
                        case 2: {
                                message.nextPageToken = reader.string();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a ListOperationsResponse message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.longrunning.ListOperationsResponse
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.longrunning.ListOperationsResponse} ListOperationsResponse
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                ListOperationsResponse.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a ListOperationsResponse message.
                 * @function verify
                 * @memberof google.longrunning.ListOperationsResponse
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                ListOperationsResponse.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.operations != null && message.hasOwnProperty("operations")) {
                        if (!Array.isArray(message.operations))
                            return "operations: array expected";
                        for (var i = 0; i < message.operations.length; ++i) {
                            var error = $root.google.longrunning.Operation.verify(message.operations[i]);
                            if (error)
                                return "operations." + error;
                        }
                    }
                    if (message.nextPageToken != null && message.hasOwnProperty("nextPageToken"))
                        if (!$util.isString(message.nextPageToken))
                            return "nextPageToken: string expected";
                    return null;
                };
    
                /**
                 * Creates a ListOperationsResponse message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.longrunning.ListOperationsResponse
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.longrunning.ListOperationsResponse} ListOperationsResponse
                 */
                ListOperationsResponse.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.longrunning.ListOperationsResponse)
                        return object;
                    var message = new $root.google.longrunning.ListOperationsResponse();
                    if (object.operations) {
                        if (!Array.isArray(object.operations))
                            throw TypeError(".google.longrunning.ListOperationsResponse.operations: array expected");
                        message.operations = [];
                        for (var i = 0; i < object.operations.length; ++i) {
                            if (typeof object.operations[i] !== "object")
                                throw TypeError(".google.longrunning.ListOperationsResponse.operations: object expected");
                            message.operations[i] = $root.google.longrunning.Operation.fromObject(object.operations[i]);
                        }
                    }
                    if (object.nextPageToken != null)
                        message.nextPageToken = String(object.nextPageToken);
                    return message;
                };
    
                /**
                 * Creates a plain object from a ListOperationsResponse message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.longrunning.ListOperationsResponse
                 * @static
                 * @param {google.longrunning.ListOperationsResponse} message ListOperationsResponse
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                ListOperationsResponse.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.arrays || options.defaults)
                        object.operations = [];
                    if (options.defaults)
                        object.nextPageToken = "";
                    if (message.operations && message.operations.length) {
                        object.operations = [];
                        for (var j = 0; j < message.operations.length; ++j)
                            object.operations[j] = $root.google.longrunning.Operation.toObject(message.operations[j], options);
                    }
                    if (message.nextPageToken != null && message.hasOwnProperty("nextPageToken"))
                        object.nextPageToken = message.nextPageToken;
                    return object;
                };
    
                /**
                 * Converts this ListOperationsResponse to JSON.
                 * @function toJSON
                 * @memberof google.longrunning.ListOperationsResponse
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                ListOperationsResponse.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for ListOperationsResponse
                 * @function getTypeUrl
                 * @memberof google.longrunning.ListOperationsResponse
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                ListOperationsResponse.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.longrunning.ListOperationsResponse";
                };
    
                return ListOperationsResponse;
            })();
    
            longrunning.CancelOperationRequest = (function() {
    
                /**
                 * Properties of a CancelOperationRequest.
                 * @memberof google.longrunning
                 * @interface ICancelOperationRequest
                 * @property {string|null} [name] CancelOperationRequest name
                 */
    
                /**
                 * Constructs a new CancelOperationRequest.
                 * @memberof google.longrunning
                 * @classdesc Represents a CancelOperationRequest.
                 * @implements ICancelOperationRequest
                 * @constructor
                 * @param {google.longrunning.ICancelOperationRequest=} [properties] Properties to set
                 */
                function CancelOperationRequest(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * CancelOperationRequest name.
                 * @member {string} name
                 * @memberof google.longrunning.CancelOperationRequest
                 * @instance
                 */
                CancelOperationRequest.prototype.name = "";
    
                /**
                 * Creates a new CancelOperationRequest instance using the specified properties.
                 * @function create
                 * @memberof google.longrunning.CancelOperationRequest
                 * @static
                 * @param {google.longrunning.ICancelOperationRequest=} [properties] Properties to set
                 * @returns {google.longrunning.CancelOperationRequest} CancelOperationRequest instance
                 */
                CancelOperationRequest.create = function create(properties) {
                    return new CancelOperationRequest(properties);
                };
    
                /**
                 * Encodes the specified CancelOperationRequest message. Does not implicitly {@link google.longrunning.CancelOperationRequest.verify|verify} messages.
                 * @function encode
                 * @memberof google.longrunning.CancelOperationRequest
                 * @static
                 * @param {google.longrunning.ICancelOperationRequest} message CancelOperationRequest message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                CancelOperationRequest.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                    return writer;
                };
    
                /**
                 * Encodes the specified CancelOperationRequest message, length delimited. Does not implicitly {@link google.longrunning.CancelOperationRequest.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.longrunning.CancelOperationRequest
                 * @static
                 * @param {google.longrunning.ICancelOperationRequest} message CancelOperationRequest message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                CancelOperationRequest.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a CancelOperationRequest message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.longrunning.CancelOperationRequest
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.longrunning.CancelOperationRequest} CancelOperationRequest
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                CancelOperationRequest.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.longrunning.CancelOperationRequest();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.name = reader.string();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a CancelOperationRequest message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.longrunning.CancelOperationRequest
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.longrunning.CancelOperationRequest} CancelOperationRequest
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                CancelOperationRequest.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a CancelOperationRequest message.
                 * @function verify
                 * @memberof google.longrunning.CancelOperationRequest
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                CancelOperationRequest.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.name != null && message.hasOwnProperty("name"))
                        if (!$util.isString(message.name))
                            return "name: string expected";
                    return null;
                };
    
                /**
                 * Creates a CancelOperationRequest message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.longrunning.CancelOperationRequest
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.longrunning.CancelOperationRequest} CancelOperationRequest
                 */
                CancelOperationRequest.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.longrunning.CancelOperationRequest)
                        return object;
                    var message = new $root.google.longrunning.CancelOperationRequest();
                    if (object.name != null)
                        message.name = String(object.name);
                    return message;
                };
    
                /**
                 * Creates a plain object from a CancelOperationRequest message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.longrunning.CancelOperationRequest
                 * @static
                 * @param {google.longrunning.CancelOperationRequest} message CancelOperationRequest
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                CancelOperationRequest.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults)
                        object.name = "";
                    if (message.name != null && message.hasOwnProperty("name"))
                        object.name = message.name;
                    return object;
                };
    
                /**
                 * Converts this CancelOperationRequest to JSON.
                 * @function toJSON
                 * @memberof google.longrunning.CancelOperationRequest
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                CancelOperationRequest.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for CancelOperationRequest
                 * @function getTypeUrl
                 * @memberof google.longrunning.CancelOperationRequest
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                CancelOperationRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.longrunning.CancelOperationRequest";
                };
    
                return CancelOperationRequest;
            })();
    
            longrunning.DeleteOperationRequest = (function() {
    
                /**
                 * Properties of a DeleteOperationRequest.
                 * @memberof google.longrunning
                 * @interface IDeleteOperationRequest
                 * @property {string|null} [name] DeleteOperationRequest name
                 */
    
                /**
                 * Constructs a new DeleteOperationRequest.
                 * @memberof google.longrunning
                 * @classdesc Represents a DeleteOperationRequest.
                 * @implements IDeleteOperationRequest
                 * @constructor
                 * @param {google.longrunning.IDeleteOperationRequest=} [properties] Properties to set
                 */
                function DeleteOperationRequest(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * DeleteOperationRequest name.
                 * @member {string} name
                 * @memberof google.longrunning.DeleteOperationRequest
                 * @instance
                 */
                DeleteOperationRequest.prototype.name = "";
    
                /**
                 * Creates a new DeleteOperationRequest instance using the specified properties.
                 * @function create
                 * @memberof google.longrunning.DeleteOperationRequest
                 * @static
                 * @param {google.longrunning.IDeleteOperationRequest=} [properties] Properties to set
                 * @returns {google.longrunning.DeleteOperationRequest} DeleteOperationRequest instance
                 */
                DeleteOperationRequest.create = function create(properties) {
                    return new DeleteOperationRequest(properties);
                };
    
                /**
                 * Encodes the specified DeleteOperationRequest message. Does not implicitly {@link google.longrunning.DeleteOperationRequest.verify|verify} messages.
                 * @function encode
                 * @memberof google.longrunning.DeleteOperationRequest
                 * @static
                 * @param {google.longrunning.IDeleteOperationRequest} message DeleteOperationRequest message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                DeleteOperationRequest.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                    return writer;
                };
    
                /**
                 * Encodes the specified DeleteOperationRequest message, length delimited. Does not implicitly {@link google.longrunning.DeleteOperationRequest.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.longrunning.DeleteOperationRequest
                 * @static
                 * @param {google.longrunning.IDeleteOperationRequest} message DeleteOperationRequest message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                DeleteOperationRequest.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a DeleteOperationRequest message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.longrunning.DeleteOperationRequest
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.longrunning.DeleteOperationRequest} DeleteOperationRequest
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                DeleteOperationRequest.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.longrunning.DeleteOperationRequest();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.name = reader.string();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a DeleteOperationRequest message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.longrunning.DeleteOperationRequest
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.longrunning.DeleteOperationRequest} DeleteOperationRequest
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                DeleteOperationRequest.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a DeleteOperationRequest message.
                 * @function verify
                 * @memberof google.longrunning.DeleteOperationRequest
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                DeleteOperationRequest.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.name != null && message.hasOwnProperty("name"))
                        if (!$util.isString(message.name))
                            return "name: string expected";
                    return null;
                };
    
                /**
                 * Creates a DeleteOperationRequest message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.longrunning.DeleteOperationRequest
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.longrunning.DeleteOperationRequest} DeleteOperationRequest
                 */
                DeleteOperationRequest.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.longrunning.DeleteOperationRequest)
                        return object;
                    var message = new $root.google.longrunning.DeleteOperationRequest();
                    if (object.name != null)
                        message.name = String(object.name);
                    return message;
                };
    
                /**
                 * Creates a plain object from a DeleteOperationRequest message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.longrunning.DeleteOperationRequest
                 * @static
                 * @param {google.longrunning.DeleteOperationRequest} message DeleteOperationRequest
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                DeleteOperationRequest.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults)
                        object.name = "";
                    if (message.name != null && message.hasOwnProperty("name"))
                        object.name = message.name;
                    return object;
                };
    
                /**
                 * Converts this DeleteOperationRequest to JSON.
                 * @function toJSON
                 * @memberof google.longrunning.DeleteOperationRequest
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                DeleteOperationRequest.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for DeleteOperationRequest
                 * @function getTypeUrl
                 * @memberof google.longrunning.DeleteOperationRequest
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                DeleteOperationRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.longrunning.DeleteOperationRequest";
                };
    
                return DeleteOperationRequest;
            })();
    
            longrunning.WaitOperationRequest = (function() {
    
                /**
                 * Properties of a WaitOperationRequest.
                 * @memberof google.longrunning
                 * @interface IWaitOperationRequest
                 * @property {string|null} [name] WaitOperationRequest name
                 * @property {google.protobuf.IDuration|null} [timeout] WaitOperationRequest timeout
                 */
    
                /**
                 * Constructs a new WaitOperationRequest.
                 * @memberof google.longrunning
                 * @classdesc Represents a WaitOperationRequest.
                 * @implements IWaitOperationRequest
                 * @constructor
                 * @param {google.longrunning.IWaitOperationRequest=} [properties] Properties to set
                 */
                function WaitOperationRequest(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * WaitOperationRequest name.
                 * @member {string} name
                 * @memberof google.longrunning.WaitOperationRequest
                 * @instance
                 */
                WaitOperationRequest.prototype.name = "";
    
                /**
                 * WaitOperationRequest timeout.
                 * @member {google.protobuf.IDuration|null|undefined} timeout
                 * @memberof google.longrunning.WaitOperationRequest
                 * @instance
                 */
                WaitOperationRequest.prototype.timeout = null;
    
                /**
                 * Creates a new WaitOperationRequest instance using the specified properties.
                 * @function create
                 * @memberof google.longrunning.WaitOperationRequest
                 * @static
                 * @param {google.longrunning.IWaitOperationRequest=} [properties] Properties to set
                 * @returns {google.longrunning.WaitOperationRequest} WaitOperationRequest instance
                 */
                WaitOperationRequest.create = function create(properties) {
                    return new WaitOperationRequest(properties);
                };
    
                /**
                 * Encodes the specified WaitOperationRequest message. Does not implicitly {@link google.longrunning.WaitOperationRequest.verify|verify} messages.
                 * @function encode
                 * @memberof google.longrunning.WaitOperationRequest
                 * @static
                 * @param {google.longrunning.IWaitOperationRequest} message WaitOperationRequest message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                WaitOperationRequest.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.name != null && Object.hasOwnProperty.call(message, "name"))
                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.name);
                    if (message.timeout != null && Object.hasOwnProperty.call(message, "timeout"))
                        $root.google.protobuf.Duration.encode(message.timeout, writer.uint32(/* id 2, wireType 2 =*/18).fork()).ldelim();
                    return writer;
                };
    
                /**
                 * Encodes the specified WaitOperationRequest message, length delimited. Does not implicitly {@link google.longrunning.WaitOperationRequest.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.longrunning.WaitOperationRequest
                 * @static
                 * @param {google.longrunning.IWaitOperationRequest} message WaitOperationRequest message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                WaitOperationRequest.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a WaitOperationRequest message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.longrunning.WaitOperationRequest
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.longrunning.WaitOperationRequest} WaitOperationRequest
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                WaitOperationRequest.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.longrunning.WaitOperationRequest();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.name = reader.string();
                                break;
                            }
                        case 2: {
                                message.timeout = $root.google.protobuf.Duration.decode(reader, reader.uint32());
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a WaitOperationRequest message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.longrunning.WaitOperationRequest
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.longrunning.WaitOperationRequest} WaitOperationRequest
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                WaitOperationRequest.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a WaitOperationRequest message.
                 * @function verify
                 * @memberof google.longrunning.WaitOperationRequest
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                WaitOperationRequest.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.name != null && message.hasOwnProperty("name"))
                        if (!$util.isString(message.name))
                            return "name: string expected";
                    if (message.timeout != null && message.hasOwnProperty("timeout")) {
                        var error = $root.google.protobuf.Duration.verify(message.timeout);
                        if (error)
                            return "timeout." + error;
                    }
                    return null;
                };
    
                /**
                 * Creates a WaitOperationRequest message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.longrunning.WaitOperationRequest
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.longrunning.WaitOperationRequest} WaitOperationRequest
                 */
                WaitOperationRequest.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.longrunning.WaitOperationRequest)
                        return object;
                    var message = new $root.google.longrunning.WaitOperationRequest();
                    if (object.name != null)
                        message.name = String(object.name);
                    if (object.timeout != null) {
                        if (typeof object.timeout !== "object")
                            throw TypeError(".google.longrunning.WaitOperationRequest.timeout: object expected");
                        message.timeout = $root.google.protobuf.Duration.fromObject(object.timeout);
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from a WaitOperationRequest message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.longrunning.WaitOperationRequest
                 * @static
                 * @param {google.longrunning.WaitOperationRequest} message WaitOperationRequest
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                WaitOperationRequest.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults) {
                        object.name = "";
                        object.timeout = null;
                    }
                    if (message.name != null && message.hasOwnProperty("name"))
                        object.name = message.name;
                    if (message.timeout != null && message.hasOwnProperty("timeout"))
                        object.timeout = $root.google.protobuf.Duration.toObject(message.timeout, options);
                    return object;
                };
    
                /**
                 * Converts this WaitOperationRequest to JSON.
                 * @function toJSON
                 * @memberof google.longrunning.WaitOperationRequest
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                WaitOperationRequest.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for WaitOperationRequest
                 * @function getTypeUrl
                 * @memberof google.longrunning.WaitOperationRequest
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                WaitOperationRequest.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.longrunning.WaitOperationRequest";
                };
    
                return WaitOperationRequest;
            })();
    
            longrunning.OperationInfo = (function() {
    
                /**
                 * Properties of an OperationInfo.
                 * @memberof google.longrunning
                 * @interface IOperationInfo
                 * @property {string|null} [responseType] OperationInfo responseType
                 * @property {string|null} [metadataType] OperationInfo metadataType
                 */
    
                /**
                 * Constructs a new OperationInfo.
                 * @memberof google.longrunning
                 * @classdesc Represents an OperationInfo.
                 * @implements IOperationInfo
                 * @constructor
                 * @param {google.longrunning.IOperationInfo=} [properties] Properties to set
                 */
                function OperationInfo(properties) {
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * OperationInfo responseType.
                 * @member {string} responseType
                 * @memberof google.longrunning.OperationInfo
                 * @instance
                 */
                OperationInfo.prototype.responseType = "";
    
                /**
                 * OperationInfo metadataType.
                 * @member {string} metadataType
                 * @memberof google.longrunning.OperationInfo
                 * @instance
                 */
                OperationInfo.prototype.metadataType = "";
    
                /**
                 * Creates a new OperationInfo instance using the specified properties.
                 * @function create
                 * @memberof google.longrunning.OperationInfo
                 * @static
                 * @param {google.longrunning.IOperationInfo=} [properties] Properties to set
                 * @returns {google.longrunning.OperationInfo} OperationInfo instance
                 */
                OperationInfo.create = function create(properties) {
                    return new OperationInfo(properties);
                };
    
                /**
                 * Encodes the specified OperationInfo message. Does not implicitly {@link google.longrunning.OperationInfo.verify|verify} messages.
                 * @function encode
                 * @memberof google.longrunning.OperationInfo
                 * @static
                 * @param {google.longrunning.IOperationInfo} message OperationInfo message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                OperationInfo.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.responseType != null && Object.hasOwnProperty.call(message, "responseType"))
                        writer.uint32(/* id 1, wireType 2 =*/10).string(message.responseType);
                    if (message.metadataType != null && Object.hasOwnProperty.call(message, "metadataType"))
                        writer.uint32(/* id 2, wireType 2 =*/18).string(message.metadataType);
                    return writer;
                };
    
                /**
                 * Encodes the specified OperationInfo message, length delimited. Does not implicitly {@link google.longrunning.OperationInfo.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.longrunning.OperationInfo
                 * @static
                 * @param {google.longrunning.IOperationInfo} message OperationInfo message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                OperationInfo.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes an OperationInfo message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.longrunning.OperationInfo
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.longrunning.OperationInfo} OperationInfo
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                OperationInfo.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.longrunning.OperationInfo();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.responseType = reader.string();
                                break;
                            }
                        case 2: {
                                message.metadataType = reader.string();
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes an OperationInfo message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.longrunning.OperationInfo
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.longrunning.OperationInfo} OperationInfo
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                OperationInfo.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies an OperationInfo message.
                 * @function verify
                 * @memberof google.longrunning.OperationInfo
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                OperationInfo.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.responseType != null && message.hasOwnProperty("responseType"))
                        if (!$util.isString(message.responseType))
                            return "responseType: string expected";
                    if (message.metadataType != null && message.hasOwnProperty("metadataType"))
                        if (!$util.isString(message.metadataType))
                            return "metadataType: string expected";
                    return null;
                };
    
                /**
                 * Creates an OperationInfo message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.longrunning.OperationInfo
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.longrunning.OperationInfo} OperationInfo
                 */
                OperationInfo.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.longrunning.OperationInfo)
                        return object;
                    var message = new $root.google.longrunning.OperationInfo();
                    if (object.responseType != null)
                        message.responseType = String(object.responseType);
                    if (object.metadataType != null)
                        message.metadataType = String(object.metadataType);
                    return message;
                };
    
                /**
                 * Creates a plain object from an OperationInfo message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.longrunning.OperationInfo
                 * @static
                 * @param {google.longrunning.OperationInfo} message OperationInfo
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                OperationInfo.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.defaults) {
                        object.responseType = "";
                        object.metadataType = "";
                    }
                    if (message.responseType != null && message.hasOwnProperty("responseType"))
                        object.responseType = message.responseType;
                    if (message.metadataType != null && message.hasOwnProperty("metadataType"))
                        object.metadataType = message.metadataType;
                    return object;
                };
    
                /**
                 * Converts this OperationInfo to JSON.
                 * @function toJSON
                 * @memberof google.longrunning.OperationInfo
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                OperationInfo.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for OperationInfo
                 * @function getTypeUrl
                 * @memberof google.longrunning.OperationInfo
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                OperationInfo.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.longrunning.OperationInfo";
                };
    
                return OperationInfo;
            })();
    
            return longrunning;
        })();
    
        google.rpc = (function() {
    
            /**
             * Namespace rpc.
             * @memberof google
             * @namespace
             */
            var rpc = {};
    
            rpc.Status = (function() {
    
                /**
                 * Properties of a Status.
                 * @memberof google.rpc
                 * @interface IStatus
                 * @property {number|null} [code] Status code
                 * @property {string|null} [message] Status message
                 * @property {Array.<google.protobuf.IAny>|null} [details] Status details
                 */
    
                /**
                 * Constructs a new Status.
                 * @memberof google.rpc
                 * @classdesc Represents a Status.
                 * @implements IStatus
                 * @constructor
                 * @param {google.rpc.IStatus=} [properties] Properties to set
                 */
                function Status(properties) {
                    this.details = [];
                    if (properties)
                        for (var keys = Object.keys(properties), i = 0; i < keys.length; ++i)
                            if (properties[keys[i]] != null)
                                this[keys[i]] = properties[keys[i]];
                }
    
                /**
                 * Status code.
                 * @member {number} code
                 * @memberof google.rpc.Status
                 * @instance
                 */
                Status.prototype.code = 0;
    
                /**
                 * Status message.
                 * @member {string} message
                 * @memberof google.rpc.Status
                 * @instance
                 */
                Status.prototype.message = "";
    
                /**
                 * Status details.
                 * @member {Array.<google.protobuf.IAny>} details
                 * @memberof google.rpc.Status
                 * @instance
                 */
                Status.prototype.details = $util.emptyArray;
    
                /**
                 * Creates a new Status instance using the specified properties.
                 * @function create
                 * @memberof google.rpc.Status
                 * @static
                 * @param {google.rpc.IStatus=} [properties] Properties to set
                 * @returns {google.rpc.Status} Status instance
                 */
                Status.create = function create(properties) {
                    return new Status(properties);
                };
    
                /**
                 * Encodes the specified Status message. Does not implicitly {@link google.rpc.Status.verify|verify} messages.
                 * @function encode
                 * @memberof google.rpc.Status
                 * @static
                 * @param {google.rpc.IStatus} message Status message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Status.encode = function encode(message, writer) {
                    if (!writer)
                        writer = $Writer.create();
                    if (message.code != null && Object.hasOwnProperty.call(message, "code"))
                        writer.uint32(/* id 1, wireType 0 =*/8).int32(message.code);
                    if (message.message != null && Object.hasOwnProperty.call(message, "message"))
                        writer.uint32(/* id 2, wireType 2 =*/18).string(message.message);
                    if (message.details != null && message.details.length)
                        for (var i = 0; i < message.details.length; ++i)
                            $root.google.protobuf.Any.encode(message.details[i], writer.uint32(/* id 3, wireType 2 =*/26).fork()).ldelim();
                    return writer;
                };
    
                /**
                 * Encodes the specified Status message, length delimited. Does not implicitly {@link google.rpc.Status.verify|verify} messages.
                 * @function encodeDelimited
                 * @memberof google.rpc.Status
                 * @static
                 * @param {google.rpc.IStatus} message Status message or plain object to encode
                 * @param {$protobuf.Writer} [writer] Writer to encode to
                 * @returns {$protobuf.Writer} Writer
                 */
                Status.encodeDelimited = function encodeDelimited(message, writer) {
                    return this.encode(message, writer).ldelim();
                };
    
                /**
                 * Decodes a Status message from the specified reader or buffer.
                 * @function decode
                 * @memberof google.rpc.Status
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @param {number} [length] Message length if known beforehand
                 * @returns {google.rpc.Status} Status
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Status.decode = function decode(reader, length) {
                    if (!(reader instanceof $Reader))
                        reader = $Reader.create(reader);
                    var end = length === undefined ? reader.len : reader.pos + length, message = new $root.google.rpc.Status();
                    while (reader.pos < end) {
                        var tag = reader.uint32();
                        switch (tag >>> 3) {
                        case 1: {
                                message.code = reader.int32();
                                break;
                            }
                        case 2: {
                                message.message = reader.string();
                                break;
                            }
                        case 3: {
                                if (!(message.details && message.details.length))
                                    message.details = [];
                                message.details.push($root.google.protobuf.Any.decode(reader, reader.uint32()));
                                break;
                            }
                        default:
                            reader.skipType(tag & 7);
                            break;
                        }
                    }
                    return message;
                };
    
                /**
                 * Decodes a Status message from the specified reader or buffer, length delimited.
                 * @function decodeDelimited
                 * @memberof google.rpc.Status
                 * @static
                 * @param {$protobuf.Reader|Uint8Array} reader Reader or buffer to decode from
                 * @returns {google.rpc.Status} Status
                 * @throws {Error} If the payload is not a reader or valid buffer
                 * @throws {$protobuf.util.ProtocolError} If required fields are missing
                 */
                Status.decodeDelimited = function decodeDelimited(reader) {
                    if (!(reader instanceof $Reader))
                        reader = new $Reader(reader);
                    return this.decode(reader, reader.uint32());
                };
    
                /**
                 * Verifies a Status message.
                 * @function verify
                 * @memberof google.rpc.Status
                 * @static
                 * @param {Object.<string,*>} message Plain object to verify
                 * @returns {string|null} `null` if valid, otherwise the reason why it is not
                 */
                Status.verify = function verify(message) {
                    if (typeof message !== "object" || message === null)
                        return "object expected";
                    if (message.code != null && message.hasOwnProperty("code"))
                        if (!$util.isInteger(message.code))
                            return "code: integer expected";
                    if (message.message != null && message.hasOwnProperty("message"))
                        if (!$util.isString(message.message))
                            return "message: string expected";
                    if (message.details != null && message.hasOwnProperty("details")) {
                        if (!Array.isArray(message.details))
                            return "details: array expected";
                        for (var i = 0; i < message.details.length; ++i) {
                            var error = $root.google.protobuf.Any.verify(message.details[i]);
                            if (error)
                                return "details." + error;
                        }
                    }
                    return null;
                };
    
                /**
                 * Creates a Status message from a plain object. Also converts values to their respective internal types.
                 * @function fromObject
                 * @memberof google.rpc.Status
                 * @static
                 * @param {Object.<string,*>} object Plain object
                 * @returns {google.rpc.Status} Status
                 */
                Status.fromObject = function fromObject(object) {
                    if (object instanceof $root.google.rpc.Status)
                        return object;
                    var message = new $root.google.rpc.Status();
                    if (object.code != null)
                        message.code = object.code | 0;
                    if (object.message != null)
                        message.message = String(object.message);
                    if (object.details) {
                        if (!Array.isArray(object.details))
                            throw TypeError(".google.rpc.Status.details: array expected");
                        message.details = [];
                        for (var i = 0; i < object.details.length; ++i) {
                            if (typeof object.details[i] !== "object")
                                throw TypeError(".google.rpc.Status.details: object expected");
                            message.details[i] = $root.google.protobuf.Any.fromObject(object.details[i]);
                        }
                    }
                    return message;
                };
    
                /**
                 * Creates a plain object from a Status message. Also converts values to other types if specified.
                 * @function toObject
                 * @memberof google.rpc.Status
                 * @static
                 * @param {google.rpc.Status} message Status
                 * @param {$protobuf.IConversionOptions} [options] Conversion options
                 * @returns {Object.<string,*>} Plain object
                 */
                Status.toObject = function toObject(message, options) {
                    if (!options)
                        options = {};
                    var object = {};
                    if (options.arrays || options.defaults)
                        object.details = [];
                    if (options.defaults) {
                        object.code = 0;
                        object.message = "";
                    }
                    if (message.code != null && message.hasOwnProperty("code"))
                        object.code = message.code;
                    if (message.message != null && message.hasOwnProperty("message"))
                        object.message = message.message;
                    if (message.details && message.details.length) {
                        object.details = [];
                        for (var j = 0; j < message.details.length; ++j)
                            object.details[j] = $root.google.protobuf.Any.toObject(message.details[j], options);
                    }
                    return object;
                };
    
                /**
                 * Converts this Status to JSON.
                 * @function toJSON
                 * @memberof google.rpc.Status
                 * @instance
                 * @returns {Object.<string,*>} JSON object
                 */
                Status.prototype.toJSON = function toJSON() {
                    return this.constructor.toObject(this, $protobuf.util.toJSONOptions);
                };
    
                /**
                 * Gets the default type url for Status
                 * @function getTypeUrl
                 * @memberof google.rpc.Status
                 * @static
                 * @param {string} [typeUrlPrefix] your custom typeUrlPrefix(default "type.googleapis.com")
                 * @returns {string} The default type url
                 */
                Status.getTypeUrl = function getTypeUrl(typeUrlPrefix) {
                    if (typeUrlPrefix === undefined) {
                        typeUrlPrefix = "type.googleapis.com";
                    }
                    return typeUrlPrefix + "/google.rpc.Status";
                };
    
                return Status;
            })();
    
            return rpc;
        })();
    
        return google;
    })();

    return $root;
});
